!=====================================================                  
! IGCM3_1: FULL PHYSICS + MIXED LAYER OCEAN (Parallel)                  
!                                                                       
! DEPARTMENT OF METEOROLOGY, UNIVERSITY OF READING                      
!=====================================================                  
        PROGRAM IGCM3                                                   
        IMPLICIT NONE                                                   
! IGCM Intermediate Global Circulation Model                            
!                                                                       
! Department of Meteorology, University of Reading, UK                  
!                                                                       
! This code is maintained at Reading using the Cray NUPDATE utility,    
! which has been ported to SUN and SGI.  Each parameter statement or    
! common block is maintained in a separate COMDECK and each             
! subroutine is in a separate DECK.                                     
!                                                                       
! Parallel version of code (Marc November 2010)                         
! =============================================                         
! This code forms the parallel version of the IGCM for versions 1,      
! 2.2 and 3 of the code. Where differences occur between any of the     
! versions this has been dealt with by using the nupdate directives     
! *IF, *ELSE and *ENDIF.                                                
!   The code has been parallised with the Message Passing Interface     
! (MPI) which consists of a number of routines beginning MPI_. They     
! allow information to be passed from one Processor Element (PE) to     
! another. Generally data is stored in a buffer space on a PE until     
! the receiving PE is ready to receive it.                              
!   The code has employed a 2D decomposition across PEs. There is       
! said to be columns and rows of PEs, as shown in the example below     
! for 8 PEs with 4 columns of PEs (Px) and two rows of PEs (Py).        
!         -----------------------------------                           
!        |  PE 4  |  PE 5  |  PE 6  |  PE 7  |                          
! 2 rows  -----------------------------------                           
!        |  PE 0  |  PE 1  |  PE 2  |  PE 3  |                          
!         -----------------------------------                           
!                     4 columns                                         
!                                                                       
! The code is expected to be most efficient when the number of          
! columns and rows are of a similar size.                               
!   This means that at any stage only one of the three dimensions is    
! not divided across PEs. A lot of transposing is required to ensure    
! that the one complete dimension is the correct one for a given        
! stage in the code.                                                    
!   Mostly the complete dimension is the levels. In spectral space      
! this means that the zonal wavenumber dimension is split across the    
! columns of PEs, while the meridional wavenumber dimension is split    
! across the rows of PEs. And in grid space the longitude dimension     
! is split across the columns of PEs, while the latitude dimension is   
! split across the rows of PEs.                                         
!   But for example, before calling the FFT on the way to creating      
! grid space variables the meridional wavenumber dimension needs to be  
! the complete dimension. Going from spectral space to grid space and   
! back to spectral space requires six transpositions (the TRANS*        
! subroutines) the complete dimension changes from: levels -> zonal     
! wavenumber (before LTI/HEXPV) -> meridional wavenumber (before FFTs)  
! -> levels (before MGRMLT/DGRMLT/ICTGRMLT) -> longitude (before        
! LTD/HANALV) -> latitude (before FFTs) -> levels.                      
!   The advantage of the 2D decomposition over a 1D decomposition       
! (only one dimension is divided across PEs, and so two dimensions      
! are complete) is that each transposition only requires data to be     
! passed between PEs all on either the same row or column. For a 1D     
! decomposition any transpositiion requires all the PEs to comunicate   
! with one another, and generally more data needs to be sent.           
! Literature, such as `Development of Fully Parallelized Regional       
! Spectral Model at NCEP' by Han and Juang, show that the 2D            
! decomposition shows significant improvement in speed-up and memory    
! usage.                                                                
!   A further issue with parallel code is that in nearly all cases we   
! only want one PE to read in and write out the data. Routines -        
! who's names begin READ - read input on PE 0 and pass it to the        
! other PEs, while several routines - their names beginning WRITE -     
! allow PE 0 to collect the data from the other PEs and write it to     
! file. The SHARE* routines have been written to help with both the     
! reading and writing process.                                          
!                                                                       
! Extra comdeck                                                         
! =============                                                         
! Most of the comdeck have been altered but there is only one new       
! comdeck, MPIDECK, which ncludes the MPI header file and and defines   
! many of the variables needed for the MPI calls.                       
!                                                                       
! Extra subroutines                                                     
! =================                                                     
! For all versions:                                                     
!   - subroutine INIPARA has been added to initialise a MPI session     
!     and set many of the variables needed for a parallel session       
!   - when not running a parallel session, several subroutines have     
!     been added in the NOMPI deck- by including *DEF NOMPI as a        
!     nupdate directive. These routines are just Fortran stubs that     
!     allow the code to compile, becuase the linker can find routines   
!     called MPI_INIT etc, but they should never be called in the       
!     process of running the code.                                      
!   - the subroutines TRANS1, TRANS2, TRANS3, TRANS4, TRAN5, and        
!     TRANS6 have been added to transpose variables between PEs.        
!   - the subroutine HANALVICT is similar to HANALV but only does       
!     calculations for the tracer variable                              
!   - the subroutines READCOEF, SHARECOEF and WRITECOEF have been       
!     added so that one PE can read and write data.                     
!                                                                       
! For versions 2.2 and 3                                                
!   - the subroutines TRANS2A, TRANS3A, TRANS4A, TRANS5A, TRANS6A       
!     have been added for transposing variables in the spectral-to-     
!     grid-space-to-spectal loop involving ICTRAC.                      
!   - the subroutines TRANS4B, TRANS5B and TRANS6B have been added      
!     for transposing variables in the spectral-to-grid-space-to-       
!     spectal loop involving DGRMLT.                                    
!   - the subroutines SHAREGRID, SHAREGRID3 and SHAREGRID4              
!     (SHAREGRID2 is only used in IGCM3), WRITEDIAG and WRITEGRID       
!     have been added so that one PE can read and write data.           
!                                                                       
! For version 3 only                                                    
!   - the subroutines SHAREGRID2 and READCLIM have been added so        
!     that one PE can read and write data.                              
!                                                                       
! Removed subroutines                                                   
! ===================                                                   
! The subroutines HANAL, HANAL1, HEXP and HEXP1, along with logical     
! LTVEC - have been removed for simplicity and because their job was    
! already being done by HANALV and HEXPV.                               
!                                                                       
! Old documentation giving a history of code development                
! ======================================================                
! IGCM1 - first portable version derived from the Reading Simple        
!         Global Circulation model. Created in February 1997. Tested    
!         on SUN, SGI and Cray                                          
!                                                                       
! IGCM2 - first portable version with simple parametrizations,          
!         moisture and additional optional tracers. The                 
!         parametrizations closely follow those in the previous         
!         Cray-specific moist model, BGCM5. Moisture/tracer advection   
!         can be in advective or flux form. Piers Forster, April 1997.  
!                                                                       
!         New common block/decks included in this version:              
!         COMGRM,GRIDPP,PHYSCA,PTENDZ,XTRAP                             
!                                                                       
!         New routines/decks included in this version:                  
!         BLAYER,VDIFF,CONVEC,DRYADJ         }    physical              
!         CUDIF,CUBM,CBCON,CBADJ,LSCRN,RADN  } parametrizations         
!         ICTRAC,INIPHYS       -  initialisation                        
!         DLSGCR,PVCR          -  initialise a PV tracer                
!         DGRMLT,LTIDT,LTDDIA  -  grid-point calculations & transforms  
!                                                                       
!         NUPDATE modsets included in this version:                     
!         TRAC - multiple tracer advection                              
!         PHYS - infrastructure for parametrizations                    
!                                                                       
! IGCM 2.1 - updated version of IGCM2.                                  
!            Created May 2000 using modsets from several contributors.  
!                                                                       
!         New routines/decks included in this version:                  
!         DANALV     -  direct transforms for the diabatic timestep     
!         MASCOR     -  global mass correction to enforce conservation  
!                                                                       
! IGCM3   IGCM2 with orography and modified physics. Knows about day of 
!         Year for solar insolation, surface temperature and humidity,  
!         Land/ Sea contrast in surface fields.                         
! IGCM3_1   IGCM3 tidied and with bug fixes. New long wave radiation    
!           code with more gasses.                                      
!           SH 2000                                                     
!                                                                       
! COMDECK     COMMON BLOCK DESCRIPTION                                  
! ===================================================================== 
!  PARAM1     (PARAMETERs) Model resolution parameters                  
!  PARAM2     (PARAMETERs) Basic constants derived from PARAM1,         
!                          mainly for array dimensions                  
!  BLANK                   Basic planetary constants plus information   
!                          on the vertical grid and spectral domain     
!  BATS       BATS         Constant arrays and variables associated     
!                          mainly with time and vertical differencing   
!  LEGAU      LEGAU        Legendre polynomials and information about   
!                          gaussian latitudes                           
!  OUTCON     OUTCON       Switches, counters and constants controlling 
!                          the type and frequency of model output       
!  COMFFT     COMFFT       Arrays and constants for the FFT             
!  POLYNO     POLYNO       Polynomial to aid vectorisation of the       
!                          Legendre transforms                          
!  RESTIJ     RESTIJ       Restoration temperature field and constants  
!                          which determine it, plus timescales          
!  RESTOR     RESTOR       Restoration fields and timescale             
!  BALAN      BALAN        Constants and arrays for balancing           
!  GRIDPA     GRIDPA       The core half transforms. The levels are     
!                          split across the columns of PEs, the zonal   
!                          wavenumbers are split across the rows of     
!                          PEs and the latitudes are complete.          
!  GRIDPB     GRIDPB       Like GRIDPA but the other half transforms    
!                          needed in the MGRMLT spectral to grid space  
!                          loop                                         
!  GRIDPA2    GRIDPA2      Can be either half transforms or grid space  
!                          variables for the same variables in GRIDPA.  
!                          The levels are always split across the       
!                          columns of PEs and the latitudes are always  
!                          split across the rows of PEs.                
!  GRIDPB2    GRIDPB2      Like GRIDA2 for variables in GRIDB           
!  GRIDPA3    GRIDPA3      Grid space variables for the same variables  
!                          found in GRIDPA1 and GRIDPA2. The levels are 
!                          complete with the longitudes splits across   
!                          the columns of PEs and the latitudes splits  
!                          across the rows of PEs.                      
!  GRIDPB3    GRIDPB3      Like GRIDA3 for variables in GRIDB and       
!                          GRIDB2.                                      
!  GRIDPC     GRIDPC       Radiation variables                          
!  SPECTR     SPECTR       Spectral arrays. The meriodonal wavenumbers  
!                          are split across the columns of PEs, the     
!                          the zonal wavenumbers are split across the   
!                          rows of PEs, and the levels are complete.    
!  SPECTR2    SPECTR2      Same as SPECTR, except the levels are split  
!                          across the columns of PEs instead of the     
!                          the meriodonal wavenumbers.                  
!  COMGRM     COMGRM       Vertical derivative arrays for interpolation 
!  PHYSCA     PHYS         Variables/arrays for the parametrizations    
!  PTENDZ     PTENDZ       Zonally averaged parametrization diagnostics 
!  GRIDPP     GRIDP        Grid point arrays, defined as REAL, for the  
!                          diabatic timestep.  This equivalences the    
!                          parametrization forcings to fields in GRIDP  
!  CPIERS     CPIERS       Radiation variables                          
!  OROG_G     GSG          Orography variables                          
!  QSTABS     QSTABS       Used for PQSAT function                      
!  RADHT      RADHT        More radiation variables                     
!  PARRAY                  Sets some parameters                         
!  MPIDECK    MPIDECK      Includes the MPI header file and defines the 
!                          variables that will be needed with the MPI   
!                          routines.                                    
!                                                                       
! DECK        SUBROUTINE   DESCRIPTION                                  
! ===================================================================== 
!  MLTRI      MLTRI        Main program.  Calls subroutines to perform  
!                          the spectral transforms, balancing and time- 
!                          stepping.  Organises the counters and output 
!  INITAL     INITAL       Calls the initialisation subroutines.        
!  INIPARA    INIPARA      Initializes the MPI session and decides      
!                          which latitude and zonal wavenumber will be  
!                          run on each PE.                              
!  INISET     INISET       Sets the default namelist values, reads the  
!                          namelists INPPL, INPRN, INPOP, then checks   
!                          and writes the namelist values.  Sets        
!                          certain run-specific common-block values.    
!  INIGAU     INIGAU       Initialises the gaussian latitudes & weights 
!  INISI      INISI        Initialises arrays and variables for the     
!                          vertical structure and semi-implicit scheme. 
!  INIRESIJ   INIRESIJ     Sets up restoration variables and arrays, by 
!                          setting the default values for namelist      
!                          INPRSIJ and then reading the namelist.       
!  INIRES     INIRES       Sets up restoration variables and arrays by  
!                          setting the default values for namelist      
!                          INPRS and then reading the namelist.         
!  INISTR     INISTR       Reads spectral data for a start/restart run. 
!  INIBAL     INIBAL       Reads the namelist INPBL and computes arrays 
!                          needed for balancing.                        
!  INISP      INISP        Initialises the spectral arrays.             
!  ICTRAC     ICTRAC       Initialises moisture, optional tracers       
!                          and a zonally symmetric surface state.       
!  INIPHYS    INIPHYS      Initialises parametrizations.                
!  INIQS      INIQS        Calculates saturation specific humidity.     
!  INISURF    INISURF      Initialises surface model.                   
!  BALANC     BALANC       Performs balancing from wind (vorticity)     
!                          to mass (temperature, surface pressure).     
!  BLAYER     BLAYER       Surface turbulent fluxes of momentum, heat   
!                          and moisture, affecting the lowest level only
!  BLSURF     BLSURF       Time-splitting integration of surface and    
!                          boundary layer schemes.                      
!  CALNDR     CALNDR       Calculated month and day from Julian day.    
!  CBADJ      CBADJ        Deep convective adjustment scheme,           
!                          based on the Betts-Miller scheme.            
!  CBCON      CBCON        Deep convection. Old routine using a crude   
!                          non-entraining mass-flux scheme.             
!  CLDTRN     CLDTRN       Cloud parameterisation for radiation scheme. 
!  CONVEC     CONVEC       Main convection routine. Finds unstable layer
!                          based on parcel instability and              
!                          calls relevant routines.                     
!  CUBM       CUBM         Shallow convective adjustment scheme,        
!                          based on the Betts-Miller scheme.            
!  CUDIF      CUDIF        Shallow convection. Old routine based on     
!                          enhanced vertical diffusion. Default-not used
!  DANALV     DANALV       Direct transforms for the diabatic           
!                          timestep.  Optimised for vector machines.    
!  DEDD       DEDD         Computes cloud reflectivity for SW.          
!  DIFUSE     DIFUSE       Calculates spectral tendencies from restor-  
!                          ation (if included) and hyper-diffusion.     
!  DGRMLT     DGMRLT       Grid-point calculations in diabatic timestep 
!                          and calls to the parametrization routines.   
!  DLSGCR     DLSGCR       Calculates vertical derivative.              
!  DRYADJ     DRYADJ       Dry convective adjustment (heat and moisture)
!  DSTEP      DSTEP        Performs the diabatic part of the timestep,  
!                          including completion of the time-filter.     
!  ENERGY     ENERGY       Calculates & writes various global           
!                          diagnostic quantities.                       
!  FFT        FFT          Calculates either the fast Fourier transform 
!                          or the inverse fast Fourier transform.       
!  HANALV     HANALV       Direct Legendre transform from Fourier to    
!                          spectral space.  Black-box routine for one   
!                          transform-type for a multi-level field.      
!                          fields/transform-types in the adiabatic step 
!  HANALVICT  HANALVICT    Same as HANALV but just for tracer variable  
!  HEXPV      HEXPV        Inverse Legendre transform from spectral to  
!                          Fourier space.  Black-box routine for one    
!                          transform-type for a multi-level field.      
!  INTERP     INTERPF      Interpolates from climatology to model vertic
!                          grid. Not used for O3.                       
!  IRRAD      IRRAD        LW radiation scheme.                         
!  LGNDRE     LGNDRE       Calculates the Legendre polynomials and      
!                          their derivatives at a single latitude.      
!  LSCRN      LSCRN        Stratiform precipitation, removing super-    
!                          saturation without re-evaporation.           
!  LTD        LTD          Direct Legendre transform for the adiabatic  
!                          part of the timestep.  Control routine.      
!  LTDDIA     LTDDIA       Inverse Legendre transforms for the diabatic 
!                          timestep.                                    
!  LTI        LTI          Inverse Legendre transform for the adiabatic 
!                          part of the timestep.  Control routine.      
!  LTIDT      LTIDT        Inverse Legendre transform required to       
!                          initialise moisture and tracers.             
!  MASCOR     MASCOR       Global mass correction.                      
!  MATINV     MATINV       Inverts a matrix.                            
!  MGRMLT     MGRMLT       Computes grid point values of the non-linear 
!                          contributions to the tendencies.             
!  MORC       RADN         Initialises radiation scheme.                
!  NIKOS      NIKOSRAD     Calls LW and SW radiation schemes.           
!  NOISE      NOISE        Uses a random number generator to add a      
!                          white noise perturbation to (ln) surface     
!                          pressure.                                    
!  NOMPI      NOMPI        This is used when the code is not being run  
!                          in parallel and the MPI routines aren't      
!                          being used. It just provides FORTRAN stubs   
!                          that allow the code to compile without       
!                          searching for the MPI routines. It only has  
!                          content if *DEF,NOMPI is added to update     
!                          file.                                        
!  O3INTERP   O3INTERP     INTERPF for O3.                              
!  PVCR       PVCR         Calculates PV to initialise a tracer.        
!  RADSW      RADSW        Initialises SW radiation scheme.             
!  READCOEF   READCOEF     Reads in spectral coefficients on PE 0 and   
!                          passed the spectral coefficients to the      
!                          other PEs                                    
!  SETRES     SETRES       Sets up and saves a restoration state from   
!                          the initial zonally averaged state.          
!  SETTEE     SETTEE       Modifies the restoration temperature field   
!                          to include a seasonal cycle.                 
!  SETZT      SETZT        Sets up a restoration temperature field,     
!                          using lapse-rate and tropopause parameters.  
!  SHARECOEF  SHARECOEF    Either shares spectral coefficients from     
!                          PE 0 to the other PEs or gets the spectral   
!                          coefficients from the other PEs onto PE 0.   
!  SHAREGRID  SHAREGRID    Grid variable is either shared from PE 0     
!                          to the other PEs or other PEs send all       
!                          the data to PE 0.                            
!  SHAREGRID2 SHAREGRID2   Same as SHAREGRID2 but dimensions are in a   
!                          a different order.                           
!  SHAREGRID3 SHAREGRID3   Like SHAREGRID but it only retreive data     
!                          and the only dimension separated across      
!                          PEs for the variables here is the latitude   
!                          dimension.                                   
!  SPDEL2     SPDEL2       Compute del-squared or its inverse for a     
!                          spectral field.                              
!  SPOP       SPOP         Controls diagnostic output of the spectral   
!                          coefficients.                                
!  SURFM      SURFM        Performs soil moisture/ snow depth update.   
!  SW         SW           Computes short wave radiation fluxes.        
!  SWTT       SWTT         Computes transmission functions for SW.      
!  TBAL       TBAL         Performs balancing from mass (temperature,   
!                          surface pressure) to wind (vorticity).       
!  TRANS1     TRANS1       Before coming to LTI and HANALV the          
!                          meridional wave dimension must be complete   
!                          on each PE. This routine achieves this by    
!                          splitting the levels across the columns of   
!                          PEs for the spectral coefficients.           
!  TRANS2     TRANS2       Before coming to FFT991 the zonal            
!                          wavenumber dimension must be complete on     
!                          each PE. The routine achieves this by        
!                          splitting the latitudes across the rows      
!                          of PEs for the half transforms.              
!  TRANS2A    TRANS2A      Similar to TRANS2 but only for CHIG and SFG. 
!  TRANS3     TRANS3       Before coming to MGRMLT the levels must      
!                          be complete. This is done by splitting       
!                          the longitudes across the columns of PEs     
!                          for these grid space variables.              
!  TRANS3A    TRANS3A      Similar to TRANS2 but for CHIG2 and SFG2.    
!  TRANS4     TRANS4       Before coming to FFT991 the longitude        
!                          dimension must be complete. This is done     
!                          by splitting the levels across the           
!                          columns of PEs for these grid space          
!                          variables.                                   
!  TRANS4A    TRANS4A      Similar to TRANS4 but only for TRANLG2       
!  TRANS4B    TRANS4B      Similar to TRANS4 but only for TNLG2, FUG2,  
!                          FVG2 and TRANLG2.                            
!  TRANS5     TRANS5       Before coming to LTD and HANALV the          
!                          latitudes dimension be complete. This is     
!                          is done by splitting the zonal wavenumbers   
!                          across the rows of PEs for these half        
!                          transforms.                                  
!  TRANS5A    TRANS5A      Similar to TRANS5 but only for TRANLG        
!  TRANS5B    TRANS5B      Similar TRANS5 but only for TNLG, FUG,       
!                          FVG and TRANLG.                              
!  TRANS6     TRANS6       Most of the program requires the levels      
!                          for the source terms for the spectral        
!                          coefficients, ZT etc, to be complete. This   
!                          is done by splitting the meriodonal          
!                          wavenumbers across the columns of PEs.       
!  TRANS6A    TRANS6A      Similar to TRANS6 but only for TRAT          
!  TRANS6B    TRANS6B      Similar to TRANS6 but only for ZT, DT, TT    
!                          and TRAT.                                    
!  TSTEP      TSTEP        Performs an adiabatic timestep in spectral   
!                          space, including the semi-implicit treatment 
!                          of gravity wave propagation.  Performs the   
!                          explicit part of the time-filter.            
!  VDIFF      VDIFF        Vertical diffusion of momentum, heat         
!                          and moisture throughout the atmospheric depth
!                          with constant exchange coefficients.         
!  WRITECOEF  WRITECOEF    Gets spectal coefficients onto PE 0 and      
!                          writes them to file.                         
!  WRITEDIAG  WRITEDIAG    Uses SHAREGRID3 to collect all the data      
!                          onto PE 0 and write to file.                 
!  WRITEGRID  WRITEGRID    Uses SHAREGRID to collect all the data       
!                          onto PE 0 and write to file.                 
!  WRSPS      WRSPS        Writes spectral coefficients.                
!  XSECT      XSECT        Writes sigma-latitude cross-sections for     
!                          quick-look diagnostic output.                
!                                                                       
! DECK        FUNCTION     DESCRIPTION                                  
! ===================================================================== 
!  POLAR      POLAR        Return the magnitude of a complex number     
!                          and it's arctangent.                         
!  PQSAT      PQSAT        Exponential function involving tendency of   
!                          temperature                                  
! ===================================================================== 
! Note: DECKs that begin Z_ are compiled at a lower optimisation,       
! because they are written in such a way that compiling at a higher     
! precison produces different results.                                  
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Constants and arrays needed for balancing                             
!-----------------------------------------------------                  
      COMMON/BALAN/BFILT,RGT0,RG,TMEAN                                  
     +            ,EP1,EP2,KBAL,SRGT0,LTBAL                             
     &            ,PMNRE                                                
! Maximum zonal wavenumbers used for balancing                          
      INTEGER, PARAMETER :: MFTBAL=9                                    
! Number of zonal wavenumbers used for balancing                        
      INTEGER, PARAMETER :: NFTBAL=1+MFTBAL/MOCT                        
      REAL :: BFILT(NL)                                                 
      REAL :: RGT0(NL)                                                  
      REAL :: RG(NL,NL)                                                 
      REAL :: TMEAN(NL)                                                 
      REAL :: EP1(IDM,NFTBAL,NHEM)                                      
      REAL :: EP2(IDM,NFTBAL,NHEM)                                      
      INTEGER :: KBAL                                                   
      REAL :: SRGT0                                                     
      LOGICAL :: LTBAL                                                  
      REAL :: PMNRE(IDM,IDM,3)                                          
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Constants and arrays needed for the fast Fourier transforms           
!-----------------------------------------------------                  
      COMMON/COMFFT/TRIG,WORK                                           
      COMMON/COMFFTINT/IFAX                                             
      REAL :: TRIG((MG+MG+MG)/2+1)                                      
      REAL :: WORK(NCRAY*(MG+1))                                        
      INTEGER :: IFAX(10)                                               
!-----------------------------------------------------                  
! Switches counters and constants controlling type and frequency of     
! model output                                                          
!-----------------------------------------------------                  
      COMMON/OUTCON/RNTAPE,DAY,SQR2,RSQR2,EAM1,EAM2,TOUT1               
     &     ,TOUT2,LSPO,LGPO                                             
     &     ,RNTAPO                                                      
     &     ,LSHIST,LMINIH                                               
      COMMON/OUTCONINT/NCOEFF,NLAT,INLAT,INSPC,KOUNTP                   
     &                 ,KOUNTE,KOUNTH,KOUNTR,KOUTP                      
     &                 ,KOUTE,KOUTH,KOUTR                               
! Identyfying run number                                                
      REAL :: RNTAPE                                                    
! Maximum wavenumber of printed spectral coeffecients                   
      INTEGER :: NCOEFF                                                 
! Number of latitudes in printed grid point fields                      
      INTEGER :: NLAT                                                   
      INTEGER :: INLAT                                                  
!                                                                       
      INTEGER :: INSPC                                                  
! Number of timsteps between successive writing of spectral             
! and gridded fields                                                    
      INTEGER :: KOUNTP                                                 
      INTEGER :: KOUNTE                                                 
! Number of timsteps between successive writing of history              
! records, containing spectral fields at the present and                
! previous time levels.                                                 
      INTEGER :: KOUNTH                                                 
! Number of timsteps between successive writing of restart              
! records, containing the spectral fields at the present and            
! previous time levels                                                  
      INTEGER :: KOUNTR                                                 
      INTEGER :: KOUTP                                                  
      INTEGER :: KOUTE                                                  
      INTEGER :: KOUTH                                                  
      INTEGER :: KOUTR                                                  
      REAL :: DAY                                                       
      REAL :: SQR2                                                      
      REAL :: RSQR2                                                     
      REAL :: EAM1                                                      
      REAL :: EAM2                                                      
      REAL :: TOUT1                                                     
      REAL :: TOUT2                                                     
      LOGICAL :: LSPO(NL)                                               
      LOGICAL :: LGPO(NL)                                               
      REAL :: RNTAPO                                                    
      LOGICAL :: LSHIST                                                 
! Reduce the history file to single precision if true                   
      LOGICAL :: LMINIH                                                 
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Stores diagnostic data                                                
!-----------------------------------------------------                  
      COMMON/PTENDZ/UTVDZ,VTVDZ,TTVDZ,QTVDZ,TTCRZ,QTCRZ                 
     &     ,TTLRZ,QTLRZ,TTRDZ,CTCRZ,CTLRZ                               
     &     ,UTBLZ,VTBLZ,TTBLZ,QTBLZ                                     
     &     ,AUTVDZ,AVTVDZ,ATTVDZ,AQTVDZ,ATTCRZ,AQTCRZ                   
     &     ,ATTLRZ,AQTLRZ,ATTRDZ,ACTCRZ,ACTLRZ                          
     &     ,AUTBLZ,AVTBLZ,ATTBLZ,AQTBLZ                                 
      REAL :: UTVDZ(JGy,NHEM,NL)                                        
      REAL :: VTVDZ(JGy,NHEM,NL)                                        
      REAL :: TTVDZ(JGy,NHEM,NL)                                        
      REAL :: QTVDZ(JGy,NHEM,NL)                                        
      REAL :: TTCRZ(JGy,NHEM,NL)                                        
      REAL :: QTCRZ(JGy,NHEM,NL)                                        
      REAL :: TTLRZ(JGy,NHEM,NL)                                        
      REAL :: QTLRZ(JGy,NHEM,NL)                                        
      REAL :: TTRDZ(JGy,NHEM,NL)                                        
      REAL :: CTCRZ(JGy,NHEM,NL)                                        
      REAL :: CTLRZ(JGy,NHEM,NL)                                        
      REAL :: UTBLZ(JGy,NHEM)                                           
      REAL :: VTBLZ(JGy,NHEM)                                           
      REAL :: TTBLZ(JGy,NHEM)                                           
      REAL :: QTBLZ(JGy,NHEM)                                           
! Variables below are for storing averages of the variables above       
      REAL :: AUTVDZ(JGy,NHEM,NL)                                       
      REAL :: AVTVDZ(JGy,NHEM,NL)                                       
      REAL :: ATTVDZ(JGy,NHEM,NL)                                       
      REAL :: AQTVDZ(JGy,NHEM,NL)                                       
      REAL :: ATTCRZ(JGy,NHEM,NL)                                       
      REAL :: AQTCRZ(JGy,NHEM,NL)                                       
      REAL :: ATTLRZ(JGy,NHEM,NL)                                       
      REAL :: AQTLRZ(JGy,NHEM,NL)                                       
      REAL :: ATTRDZ(JGy,NHEM,NL)                                       
      REAL :: ACTCRZ(JGy,NHEM,NL)                                       
      REAL :: ACTLRZ(JGy,NHEM,NL)                                       
      REAL :: AUTBLZ(JGy,NHEM)                                          
      REAL :: AVTBLZ(JGy,NHEM)                                          
      REAL :: ATTBLZ(JGy,NHEM)                                          
      REAL :: AQTBLZ(JGy,NHEM)                                          
!-----------------------------------------------------                  
! Restoration temperature field and constants which determine it,       
! also contains timescales                                              
!-----------------------------------------------------                  
      COMMON/RESTIJ/TTRES                                               
     + ,DTNS,DTEP,DTTRP,FAC,DDAMP,TFRC,YRLEN,TRS                        
     +  ,ALR,ZTROP,TGR                                                  
      COMPLEX :: TTRES(NNTx,MMTy,NHEM,NL)                               
      REAL :: DTNS                                                      
      REAL :: DTEP                                                      
      REAL :: DTTRP                                                     
      REAL :: FAC(NL)                                                   
      REAL :: DDAMP(NL)                                                 
      REAL :: TFRC(NL)                                                  
      REAL :: YRLEN                                                     
      REAL :: TRS(NL)                                                   
      REAL :: ALR                                                       
      REAL :: ZTROP                                                     
      REAL :: TGR                                                       
!-----------------------------------------------------                  
! Restoration fields and timescale                                      
!-----------------------------------------------------                  
      COMMON/RESTOR/ZRES,DRES,TRES,SPRES,DAMP                           
      REAL :: ZRES(IDM,NHEM,NL)                                         
      REAL :: DRES(IDM,NHEM,NL)                                         
      REAL :: TRES(IDM,NHEM,NL)                                         
      REAL :: SPRES(IDM,NHEM)                                           
      REAL :: DAMP                                                      
!-----------------------------------------------------                  
! Radiation variables                                                   
!-----------------------------------------------------                  
      COMMON/CPIERS/ICFLAG,CFRAC,PNET,SNET,RRFLUX,TTSW,TTLW             
      INTEGER :: ICFLAG(MGx,NHEM,5,2)                                   
      REAL :: CFRAC(MGx,NHEM,5)                                         
      REAL :: PNET(MGx,NHEM,JGy)                                        
      REAL :: SNET(MGx,NHEM,JGy)                                        
      REAL :: RRFLUX(MGx,NHEM,JGy,6)                                    
      REAL :: TTSW(MGx,NHEM,NL)                                         
      REAL :: TTLW(MGx,NHEM,NL)                                         
!-----------------------------------------------------                  
!                                                                       
!-----------------------------------------------------                  
      COMMON/STATS/GMSP0,GMSPMI,LMASCOR,LMASOLD,LMASPRT                 
      LOGICAL :: LMASCOR,LMASOLD,LMASPRT                                
      REAL :: GMSP0,GMSPMI                                              
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Half transforms. The levels are divided across the columns of PEs;    
! the zonal wavenumbers are divided across the row of PEs; and on       
! ech PE the latitude are complete.                                     
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA/UG,FUG,VG,FVG,TG,TNLG,PLG                           
     &     ,TRAG,TRANLG                                                 
! U_psi, then U (rotational wind)                                       
      COMPLEX :: UG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FUG(2*MMTy,NHEM,JG,NLx)                                
! V_chi, then V (divergent wind)                                        
      COMPLEX :: VG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FVG(2*MMTy,NHEM,JG,NLx)                                
! Temperature anomaly (T')                                              
      COMPLEX :: TG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: TNLG(2*MMTy,NHEM,JG,NLx)                               
! ln p* or p*                                                           
      COMPLEX :: PLG(2*MMTy,NHEM,JG)                                    
! Tracer term                                                           
      COMPLEX :: TRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                         
!                                                                       
      COMPLEX :: TRANLG(2*MMTy,NHEM,JG,NLx,NTRAC)                       
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB/ZG,DG,PJG,PMG,SPG,VPG,EG,UTG,VTG                    
     &     ,UTRAG,VTRAG                                                 
     &     ,CHIG,SFG,FVGT,FUGT                                          
! Vorticity (zeta)                                                      
      COMPLEX :: ZG(2*MMTy,NHEM,JG,NLx)                                 
! Divergence (d)                                                        
      COMPLEX :: DG(2*MMTy,NHEM,JG,NLx)                                 
! (1-mu^2) (d(ln p*)/d mu)                                              
      COMPLEX :: PJG(2*MMTy,NHEM,JG)                                    
! (d(ln p*)/d lambda)                                                   
      COMPLEX :: PMG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: SPG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: VPG(2*MMTy,NHEM,JG)                                    
! (U^2 + V^2)                                                           
      COMPLEX :: EG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: UTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: VTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: UTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: VTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: CHIG(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: SFG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: FVGT(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: FUGT(2*MMTy,NHEM,JG,NLx)                               
!-----------------------------------------------------                  
! Like the variables in SPECTR but the meridional wavenumbers           
! are complete instead of the levels.                                   
!-----------------------------------------------------                  
      COMMON/SPECTR2/Z2,D2,T2,SP2,SPA2,VP2,DTE2,TT2,DT2,ZT2             
     &     ,TRA2,TRAT2                                                  
! Vorticity coefficient                                                 
      COMPLEX :: Z2(NNT,MMTy,NHEM,NLx)                                  
! Divergence coefficient                                                
      COMPLEX :: D2(NNT,MMTy,NHEM,NLx)                                  
! Temperature coefficient                                               
      COMPLEX :: T2(NNT,MMTy,NHEM,NLx)                                  
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP2(NNT,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA2(NNT,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP2(NNT,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE2(NNT,MMTy,NHEM,NLx)                                
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT2(NNT,MMTy,NHEM,NLx)                                 
! Tracer coefficient                                                    
      COMPLEX :: TRA2(NNT,MMTy,NHEM,NLx,NTRAC)                          
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT2(NNT,MMTy,NHEM,NLx,NTRAC)                         
!-----------------------------------------------------                  
! Similar to variables in GRIDP but in grid space, and without          
! equivalent variables for CHIG, SFG, FVGT and FUGT. The levels         
! are divided across the columns of PEs; the latitudes are divided      
! across the row of PEs; and the zonal wavenumbers/longitudes are       
! complete.                                                             
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA2/UG2,FUG2,VG2,FVG2,TG2,TNLG2,PLG2                   
     &     ,TRAG2,TRANLG2                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FUG2(MGPP,NHEM,JGy,NLx)                                   
! V_chi, then V (divergent wind)                                        
      REAL :: VG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FVG2(MGPP,NHEM,JGy,NLx)                                   
! Temperature anomaly (T')                                              
      REAL :: TG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: TNLG2(MGPP,NHEM,JGy,NLx)                                  
! ln p* or p*                                                           
      REAL :: PLG2(MGPP,NHEM,JGy)                                       
! Tracer term                                                           
      REAL :: TRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                            
      REAL :: TRANLG2(MGPP,NHEM,JGy,NLx,NTRAC)                          
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB2/ZG2,DG2,PJG2,PMG2,SPG2,VPG2,EG2,UTG2,VTG2          
     &     ,UTRAG2,VTRAG2                                               
     &     ,CHIG2,SFG2                                                  
! Vorticity (zeta)                                                      
      REAL :: ZG2(MGPP,NHEM,JGy,NLx)                                    
! Divergence (d)                                                        
      REAL :: DG2(MGPP,NHEM,JGy,NLx)                                    
! (1-mu^2) (d(ln p*)/d mu)                                              
      REAL :: PJG2(MGPP,NHEM,JGy)                                       
! (d(ln p*)/d lambda)                                                   
      REAL :: PMG2(MGPP,NHEM,JGy)                                       
      REAL :: SPG2(MGPP,NHEM,JGy)                                       
      REAL :: VPG2(MGPP,NHEM,JGy)                                       
! (U^2 + V^2)                                                           
      REAL :: EG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: UTG2(MGPP,NHEM,JGy,NLx)                                   
      REAL :: VTG2(MGPP,NHEM,JGy,NLx)                                   
      REAL :: UTRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                           
      REAL :: VTRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                           
      REAL :: CHIG2(MGPP,NHEM,JGy,NLx)                                  
      REAL :: SFG2(MGPP,NHEM,JGy,NLx)                                   
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Radiation variables                                                   
!-----------------------------------------------------                  
      COMMON /RADHT/HTNET,TAVE                                          
      REAL :: HTNET(NHEM,JGy,MGx,NL)                                    
      COMPLEX :: TAVE(NNTx,MMTy,NHEM,NL)                                
!-----------------------------------------------------                  
! Orogrophy                                                             
!-----------------------------------------------------                  
      COMMON/GSG/GSG,TOPVAR,RLEVBZMO,FXZMO,ofrac,runoff,iareanum        
! Topog variance, GW diags for Orog, Non orog waves                     
      REAL :: TOPVAR(MGx,NHEM,JGy)                                      
      REAL :: RLEVBZMO(JGy,NHEM,NL)                                     
      REAL :: FXZMO(JGy,NHEM,NL)                                        
         REAL :: GSG(MGx,NHEM,JGy)                                      
         REAL :: ofrac(MGx,NHEM,JGy)                                    
         INTEGER :: runoff(50,2,33)                                     
         INTEGER :: iareanum                                            
C OASIS size parameters                                                 
      INTEGER, PARAMETER :: IMB = 128                                   
      INTEGER, PARAMETER :: JM = 64                                     
      INTEGER, PARAMETER :: IMJM = IMB*JM                               
      common/oasiso2a/zsst(imjm),zscur(imjm),mscur(imjm)                
     $   ,zsst_2d(MGx,NHEM,JGy)                                         
     $   ,zscur_2d(MGx,NHEM,JGy),mscur_2d(MGx,NHEM,JGy)                 
      real zsst,zsst_2d,zscur,zscur_2d,mscur,mscur_2d                   
      common/oasisa2o/swlw(imjm),heatlat(imjm),water(imjm)              
     $   ,sensible(imjm),taux(imjm),tauy(imjm),exwater(imjm)            
     $   ,exwat(MGx,NHEM,JGy),sensible2(imjm),sensible3(imjm)           
     $   ,oceaniceflux(MGx,NHEM),exiwat(MGx,NHEM,JGy),exiwater(imjm)    
     $   ,exiwatn(MGx,NHEM,JGy),exiwatern(imjm)                         
     $   ,tstaroo(MGx,NHEM,JGy),tstarf(MGx,NHEM,JGy)                    
     $   ,qstarf(MGx,NHEM,JGy),sqstarf(MGx,NHEM,JGy)                    
     $   ,swlw2d(MGx,NHEM,JGy),heatlat2d(MGx,NHEM,JGy)                  
     $   ,water2d(MGx,NHEM,JGy)                                         
     $   ,sensible2d(MGx,NHEM,JGy),taux2d(MGx,NHEM,JGy)                 
     $   ,tauy2d(MGx,NHEM,JGy)                                          
     $   ,SHBLC(MGx,NHEM,JGy),SLBLC(MGx,NHEM,JGy)                       
     $   ,exwater2d(MGx,NHEM,JGy),sensible2_2d(MGx,NHEM,JGy)            
     $   ,sensible3_2d(MGx,NHEM,JGy)                                    
     $   ,exiwater2d(MGx,NHEM,JGy),exiwatern2d(MGx,NHEM,JGy)            
      real swlw,heatlat,water,sensible,taux,tauy,shblc,slblc,exwater    
     $,exwat,sensible2,sensible3,oceaniceflux,exiwat,exiwater           
     $,exiwatn,exiwatern,tstaroo,tstarf,qstarf,sqstarf                  
     $,swlw2d,heatlat2d,water2d,sensible2d,taux2d,tauy2d                
     $,exwater2d,sensible2_2d,sensible3_2d,exiwater2d,exiwatern2d       
                                                                        
      common/wat/water2(imjm), water2_2d(MGx,NHEM,JGy)                  
C Arrays wathbl, watlsl, watsbl are used for balancing water in IGCM4   
C ashbl, aslbl, assbl were used in IGCM3 FORTE but these are already    
C in use in elsewhere in IGCM4 for surface flux diagnostics             
     $   ,wathbl(MGx,NHEM,JGy),watlbl(MGx,NHEM,JGy),watsbl(MGx,NHEM,JGy)
     $   ,sicecov(MGx,NHEM,JGy)                                         
      real water2,water2_2d,wathbl,watlbl,watsbl,sicecov                
                                                                        
                                                                        
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
! RKOUNT is an integer, but output expects a REAL                       
      REAL :: RKOUNT                                                    
! Array needed by SPDEL2                                                
      REAL :: FILT(0:NN)                                                
      INTEGER :: KKOUT                                                  
      INTEGER :: KountStart                                             
      REAL :: RKP                                                       
      REAL :: RREC                                                      
      REAL :: GSFAC                                                     
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, IN, J, L, KOUNT2                          
      INTEGER npas      ! number of timesteps for the simulation        
      INTEGER nexca     ! coupling interval (in timesteps).             
      INTEGER itimestep ! timestep in seconds                           
      INTEGER itap, ji, jf, jpos                                        
      INTEGER counter,ifirstday,start,bs_av1,bs_av2,ibabs               
      LOGICAL last,archts                                               
                                                                        
      real gcouple,restorice,tspd                                       
                                                                        
c                                                                       
      EXTERNAL inicma           ! initialize the coupling channels      
      EXTERNAL intocpl          ! sends the fields to Oasis             
      EXTERNAL fromcpl          ! receives the fields from Oasis        
                                                                        
! Full 2D variables for OASIS collected from and sent to PEs            
                                                                        
      REAL, ALLOCATABLE :: SWLW2DFULL(:,:,:)                            
      REAL, ALLOCATABLE :: HEATLAT2DFULL(:,:,:)                         
      REAL, ALLOCATABLE :: WATER2DFULL(:,:,:)                           
      REAL, ALLOCATABLE :: SENSIBLE2DFULL(:,:,:)                        
      REAL, ALLOCATABLE :: TAUX2DFULL(:,:,:)                            
      REAL, ALLOCATABLE :: TAUY2DFULL(:,:,:)                            
      REAL, ALLOCATABLE :: ZSST_2DFULL(:,:,:)                           
      REAL, ALLOCATABLE :: ZSCUR_2DFULL(:,:,:)                          
      REAL, ALLOCATABLE :: MSCUR_2DFULL(:,:,:)                          
C Water2dfull allocated in OAS_TEST update                              
      REAL, ALLOCATABLE :: exwater2dfull(:,:,:)                         
      REAL, ALLOCATABLE :: exiwater2dfull(:,:,:)                        
      REAL, ALLOCATABLE :: exiwatern2dfull(:,:,:)                       
      REAL, ALLOCATABLE :: water2_2dfull(:,:,:)                         
!-----------------------------------------------------                  
! Format statements                                                     
!-----------------------------------------------------                  
 2000 FORMAT(/' RESTART RECORD WRITTEN TO CHANNEL ',I3,/                
     &        ' RKOUNT  RNTAPE  DAY  DOY  =',4F12.3)                    
 2010 FORMAT(/' HISTORY RECORD WRITTEN TO CHANNEL ',I3,/                
     &        ' RKOUNT  RNTAPE  DAY  DOY  =',4F12.3)                    
 2020 FORMAT(/' RESTORATION RECORD WRITTEN TO CHANNEL ',I3,/            
     &        ' RKOUNT  RNTAPE  DAY  DOY  =',4F12.3)                    
!-----------------------------------------------------                  
! Initialize variables and physical state                               
!-----------------------------------------------------                  
      CALL INITAL                                                       
c                                                                       
      counter=1                                                         
      gcouple=2                                                         
                                                                        
      if(npe.gt.1)CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                
      IF(MYPE.EQ.0)THEN                                                 
                                                                        
        nexca=1                                                         
        npas=KRUN/ITSPD                                                 
        itimestep=1                                                     
        ifirstday=day                                                   
        start=start+ifirstday                                           
                                                                        
        PRINT * , 'Toyclim: the atmospheric grid is  imb  = ', imb      
        PRINT * , '                               *  jm  = ', jm        
        PRINT * ,'                            so  imjm  = ', imjm       
        PRINT *                                                         
        PRINT * ,'npas, nexca, itimestep=', npas, nexca, itimestep      
                                                                        
c Initialize coupling                                                   
        CALL inicma(npas, nexca, itimestep)                             
      ENDIF   ! End MYPE if                                             
      if(npe.gt.1)CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                
                                                                        
c Set oasis exchange arrays to zero only at start of run                
C      IF (mod(kount,ITSPD).eq.0) then                                  
      IF (kount.eq.0) then                                              
                                                                        
      IF(MYPE.EQ.0)THEN                                                 
C     write(6,*)'clearing exchange arrays'                              
      DO i = 1, imjm                                                    
        swlw(i) = 0.                                                    
        heatlat(i) = 0.                                                 
        water(i) = 0.                                                   
        water2(i) = 0.                                                  
        sensible(i) = 0.                                                
        taux(i) = 0.                                                    
        tauy(i) = 0.                                                    
        exwater(i) = 0.                                                 
        exiwater(i) = 0.                                                
        exiwatern(i) = 0.                                               
      ENDDO                                                             
      ENDIF   ! End MYPE if                                             
                                                                        
      do j=1,JGyy(iRow)                                                 
      do ihem=1,nhem                                                    
      do i=1,MGxx(iCol)                                                 
        swlw2d(i,ihem,j) = 0.                                           
        heatlat2d(i,ihem,j) = 0.                                        
        water2d(i,ihem,j) = 0.                                          
        water2_2d(i,ihem,j) = 0.                                        
        sensible2d(i,ihem,j) = 0.                                       
        taux2d(i,ihem,j) = 0.                                           
        tauy2d(i,ihem,j) = 0.                                           
        exwater2d(i,ihem,j) = 0.                                        
        exiwater2d(i,ihem,j) = 0.                                       
        exiwatern2d(i,ihem,j) = 0.                                      
      end do                                                            
      end do                                                            
      end do                                                            
                                                                        
C                                                                       
                                                                        
      ENDIF                                                             
      if(npe.gt.1)CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                
C                                                                       
!-----------------------------------------------------                  
! Loop across timesteps                                                 
!-----------------------------------------------------                  
      KountStart=KOUNT                                                  
      DO KOUNT2=KountStart,KTOTAL                                       
c IF coupling timestep, get oceanic fields from Oasis                   
C At timesteps 0,1,2,3*ITSPD + 1 etc collect SST from MOMA via OASIS    
C This happens in the timestep after calling ALTERWATER and INTOCPL     
                                                                        
      IF (MOD( (KOUNT-1),ITSPD).eq.0) THEN                              
        if(npe.gt.1)CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)              
        IF(MYPE.EQ.0)THEN                                               
                                                                        
          CALL fromcpl(nint(DAY-ifirstday)*nexca, imjm, zsst,           
     $                 zscur,mscur)                                     
                                                                        
          ALLOCATE(zsst_2dfull(MG,NHEM,JG))                             
          ALLOCATE(zscur_2dfull(MG,NHEM,JG))                            
          ALLOCATE(mscur_2dfull(MG,NHEM,JG))                            
                                                                        
          do j=1,jg                                                     
            do i=1,mg                                                   
              zsst_2dfull(i,1,j)=zsst((j-1)*imb+i)                      
              zsst_2dfull(i,2,jg-j+1)=zsst((jg+j-1)*imb+i)              
              zscur_2dfull(i,1,j)=zscur((j-1)*imb+i)                    
              zscur_2dfull(i,2,jg-j+1)=zscur((jg+j-1)*imb+i)            
              mscur_2dfull(i,1,j)=mscur((j-1)*imb+i)                    
              mscur_2dfull(i,2,jg-j+1)=mscur((jg+j-1)*imb+i)            
            end do                                                      
          end do                                                        
                                                                        
        endif ! End MYPE if                                             
        if(npe.gt.1)CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)              
                                                                        
C     Send ZSST from full arrays to PEs                                 
        CALL SHAREGRID(.TRUE.,MG,1,zsst_2dfull,zsst_2d)                 
        CALL SHAREGRID(.TRUE.,MG,1,zscur_2dfull,zscur_2d)               
        CALL SHAREGRID(.TRUE.,MG,1,mscur_2dfull,mscur_2d)               
                                                                        
        IF(MYPE.EQ.0)DEALLOCATE(zsst_2dfull)                            
        IF(MYPE.EQ.0)DEALLOCATE(zscur_2dfull)                           
        IF(MYPE.EQ.0)DEALLOCATE(mscur_2dfull)                           
        if(npe.gt.1)CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)              
                                                                        
      ENDIF    ! End MOD(KOUNT-1,ITSPD) if                              
                                                                        
C At timesteps 0,1,2,3*ITSPD etc collect water & runoff quantities      
C and call alterwater which alters WATER2D                              
                                                                        
      IF (MOD(KOUNT,ITSPD).eq.0) THEN                                   
                                                                        
        IF(MYPE.EQ.0)THEN                                               
          ALLOCATE(water2dfull(MG,NHEM,JG))                             
          ALLOCATE(exwater2dfull(MG,NHEM,JG))                           
          ALLOCATE(exiwater2dfull(MG,NHEM,JG))                          
          ALLOCATE(exiwatern2dfull(MG,NHEM,JG))                         
          ALLOCATE(water2_2dfull(MG,NHEM,JG))                           
        ENDIF                                                           
                                                                        
C collect variables from processes to arrays water2dfull,               
C exwater2dfull,exiwater2dfull,exiwater2dfull,water2_2dfull             
C MPI NOTE: WATER2_2DFULL is not used in ALTERWATER so                  
C use this array to pass the full OFRAC array to ALTERWATER.            
C This is because ALTERWATER uses OFRAC annoyingly enough               
C which is split amongst the different processes.                       
                                                                        
        CALL SHAREGRID(.FALSE.,MG,1,water2dfull,water2d)                
        CALL SHAREGRID(.FALSE.,MG,1,exwater2dfull,exwater2d)            
        CALL SHAREGRID(.FALSE.,MG,1,exiwater2dfull,exiwater2d)          
        CALL SHAREGRID(.FALSE.,MG,1,exiwatern2dfull,exiwatern2d)        
        CALL SHAREGRID(.FALSE.,MG,1,water2_2dfull,ofrac)                
                                                                        
        if(npe.gt.1)CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)              
        if(mype.eq.0)then                                               
          CALL alterwater(exwater2dfull,exiwater2dfull,                 
     1    exiwatern2dfull,water2dfull,water2_2dfull)                    
        endif   ! End MYPE if                                           
        if(npe.gt.1)CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)              
                                                                        
C send variables from 'full' arrays back to different processors        
C in analagous manner to GSGfull & GSG in INISTR                        
                                                                        
        CALL SHAREGRID(.TRUE.,MG,1,water2dfull,water2d)                 
        CALL SHAREGRID(.TRUE.,MG,1,exwater2dfull,exwater2d)             
        CALL SHAREGRID(.TRUE.,MG,1,exiwater2dfull,exiwater2d)           
        CALL SHAREGRID(.TRUE.,MG,1,exiwatern2dfull,exiwatern2d)         
                                                                        
        IF(MYPE.EQ.0)THEN                                               
          DEALLOCATE(water2dfull)                                       
          DEALLOCATE(exwater2dfull)                                     
          DEALLOCATE(exiwater2dfull)                                    
          DEALLOCATE(exiwatern2dfull)                                   
          DEALLOCATE(water2_2dfull)                                     
        endif   ! End MYPE if                                           
                                                                        
        if(npe.gt.1)CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)              
                                                                        
      ENDIF    ! End MOD(KOUNT,ITSPD) IF                                
C                                                                       
         !-----------------------------------------------------         
                                                                        
C At timesteps 0,1,2,3*ITSPD etc collect coupling quantities            
C and call INTOCPL which sends variables to MOMA via OASIS              
                                                                        
      IF (MOD(KOUNT,ITSPD).eq.0) THEN                                   
        if(npe.gt.1)CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)              
        IF(MYPE.EQ.0)THEN                                               
                                                                        
c       For last time step, keep files just before coupling             
          last=.FALSE.                                                  
C          IF (nint(day-ifirstday).eq.npas) last=.TRUE.                 
          IF (KOUNT2.eq.KTOTAL) last=.TRUE.                             
          IF (koutr.eq.KOUNTR) archts=.TRUE.                            
                                                                        
          ALLOCATE(water2dfull(MG,NHEM,JG))                             
          ALLOCATE(swlw2dfull(MG,NHEM,JG))                              
          ALLOCATE(heatlat2dfull(MG,NHEM,JG))                           
          ALLOCATE(sensible2dfull(MG,NHEM,JG))                          
          ALLOCATE(taux2dfull(MG,NHEM,JG))                              
          ALLOCATE(tauy2dfull(MG,NHEM,JG))                              
                                                                        
        ENDIF   ! End mype if                                           
                                                                        
C     Collect data from PEs and send to full arrays                     
        CALL SHAREGRID(.FALSE.,MG,1,water2dfull,water2d)                
        CALL SHAREGRID(.FALSE.,MG,1,swlw2dfull,swlw2d)                  
        CALL SHAREGRID(.FALSE.,MG,1,heatlat2dfull,heatlat2d)            
        CALL SHAREGRID(.FALSE.,MG,1,sensible2dfull,sensible2d)          
        CALL SHAREGRID(.FALSE.,MG,1,taux2dfull,taux2d)                  
        CALL SHAREGRID(.FALSE.,MG,1,tauy2dfull,tauy2d)                  
c                                                                       
c       Give atmospheric fields to Oasis                                
c                                                                       
        if(npe.gt.1)CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)              
        if(mype.eq.0)then                                               
          do j=1,jg                                                     
          do i=1,mg                                                     
          swlw((j-1)*imb+i)=swlw2dfull(i,1,j)                           
          swlw((jg+j-1)*imb+i)=swlw2dfull(i,2,jg-j+1)                   
          heatlat((j-1)*imb+i)=heatlat2dfull(i,1,j)                     
          heatlat((jg+j-1)*imb+i)=heatlat2dfull(i,2,jg-j+1)             
          water((j-1)*imb+i)=water2dfull(i,1,j)                         
          water((jg+j-1)*imb+i)=water2dfull(i,2,jg-j+1)                 
          sensible((j-1)*imb+i)=sensible2dfull(i,1,j)                   
          sensible((jg+j-1)*imb+i)=sensible2dfull(i,2,jg-j+1)           
          taux((j-1)*imb+i)=taux2dfull(i,1,j)                           
          taux((jg+j-1)*imb+i)=taux2dfull(i,2,jg-j+1)                   
          tauy((j-1)*imb+i)=tauy2dfull(i,1,j)                           
          tauy((jg+j-1)*imb+i)=tauy2dfull(i,2,jg-j+1)                   
          end do                                                        
          end do                                                        
                                                                        
          DEALLOCATE(swlw2dfull)                                        
          DEALLOCATE(heatlat2dfull)                                     
          DEALLOCATE(sensible2dfull)                                    
          DEALLOCATE(water2dfull)                                       
          DEALLOCATE(taux2dfull)                                        
          DEALLOCATE(tauy2dfull)                                        
                                                                        
          CALL intocpl(nint(DAY-ifirstday)*nexca, imjm, swlw, heatlat   
     $              , water, sensible, taux, tauy, archts, last)        
          archts=.false.                                                
        ENDIF    ! End MYPE if                                          
      IF(MYPE.EQ.0)THEN                                                 
C     write(6,*)'clearing exchange arrays'                              
      DO i = 1, imjm                                                    
        swlw(i) = 0.                                                    
        heatlat(i) = 0.                                                 
        water(i) = 0.                                                   
        water2(i) = 0.                                                  
        sensible(i) = 0.                                                
        taux(i) = 0.                                                    
        tauy(i) = 0.                                                    
        exwater(i) = 0.                                                 
        exiwater(i) = 0.                                                
        exiwatern(i) = 0.                                               
      ENDDO                                                             
      ENDIF   ! End MYPE if                                             
                                                                        
      do j=1,JGyy(iRow)                                                 
      do ihem=1,nhem                                                    
      do i=1,MGxx(iCol)                                                 
        swlw2d(i,ihem,j) = 0.                                           
        heatlat2d(i,ihem,j) = 0.                                        
        water2d(i,ihem,j) = 0.                                          
        water2_2d(i,ihem,j) = 0.                                        
        sensible2d(i,ihem,j) = 0.                                       
        taux2d(i,ihem,j) = 0.                                           
        tauy2d(i,ihem,j) = 0.                                           
        exwater2d(i,ihem,j) = 0.                                        
        exiwater2d(i,ihem,j) = 0.                                       
        exiwatern2d(i,ihem,j) = 0.                                      
      end do                                                            
      end do                                                            
      end do                                                            
                                                                        
C                                                                       
                                                                        
        if(npe.gt.1)CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)              
      ENDIF  ! End MOD(KOUNT,ITPSD) if                                  
         ! Adiabatic part of timestep.                                  
         ! Preset tendencies to zero.                                   
         !-----------------------------------------------------         
         VP2(:,:,:) =0.0                                                
         SPA2(:,:,:)=0.0                                                
         ZT2(:,:,:,:)=0.0                                               
         DT2(:,:,:,:)=0.0                                               
         TT2(:,:,:,:)=0.0                                               
         DTE2(:,:,:,:)=0.0                                              
         TRAT2(:,:,:,:,:)=0.0                                           
         !-----------------------------------------------------         
         ! Actions for initial timestep                                 
         !-----------------------------------------------------         
         IF (KOUNT.EQ.0) THEN                                           
            !-----------------------------------------------------      
            ! Initialise new tracer fields.                             
            !-----------------------------------------------------      
            IF (.NOT. LRSTRT) CALL ICTRAC                               
            !-----------------------------------------------------      
            ! Add white noise perturbation                              
            !-----------------------------------------------------      
            IF (LNOISE.AND..NOT.LRSTRT) CALL NOISE                      
            !-----------------------------------------------------      
            ! Calculate restoration arrays                              
            !-----------------------------------------------------      
            IF (.NOT. LRESTIJ) CALL SETRES                              
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Set surface pressure to 976mb                                
         !-----------------------------------------------------         
         IF ( (MyPe.EQ.0).AND.(DAY.LT.0.1) ) THEN                       
            print *,' setting surface pressure to 976mb'                
            SP(1,1,1)=CMPLX((sqr2*(0.976-1.0)),0.0)                     
         ENDIF                                                          
      IF (MyPe.EQ.0.and.mod(kount,720).EQ.0)print*,day,                 
     :sp(1,1,1),sp(1,1,2),T(1,1,1,20),D(1,1,2,1)                        
         KKOUT=KOUNT*(KOUNTP-KOUTP)                                     
            IF (LNOISE.AND..NOT.LRSTRT) CALL NOISEold                   
         ! Transpose the arrays so that the meridional wavenumber       
         ! dimension is the only dimension that isn't decomposed        
         ! over PEs                                                     
         !-----------------------------------------------------         
         CALL TRANS1                                                    
         !-----------------------------------------------------         
         ! Convert the meridional wavenumber dimension to a             
         ! latitude dimension                                           
         !-----------------------------------------------------         
         CALL LTI                                                       
         !-----------------------------------------------------         
         ! Transpose the arrays so that the zonal wavenumber            
         ! dimension is the only dimension that isn't decomposed        
         ! over PEs                                                     
         !-----------------------------------------------------         
         CALL TRANS2(UG,VG,ZG,DG,TG,TRAG,PLG,PJG,PMG)                   
         !-----------------------------------------------------         
         ! Convert zonal wavenumber dimension to longitude              
         ! dimension.                                                   
         !-----------------------------------------------------         
         CALL FFT(1,NHEM*JGy*NLx,UG2)                                   
         CALL FFT(1,NHEM*JGy*NLx,VG2)                                   
         CALL FFT(1,NHEM*JGy*NLx,ZG2)                                   
         CALL FFT(1,NHEM*JGy*NLx,DG2)                                   
         CALL FFT(1,NHEM*JGy*NLx,TG2)                                   
         CALL FFT(1,NHEM*JGy*NLx*NTRAC,TRAG2)                           
         IF (iCol.EQ.1) THEN                                            
            CALL FFT(1,NHEM*JGy,PLG2)                                   
            CALL FFT(1,NHEM*JGy,PJG2)                                   
            CALL FFT(1,NHEM*JGy,PMG2)                                   
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Transpose the arrays so that layer dimension is the          
         ! only dimension that isn't decomposed over PEs                
         !-----------------------------------------------------         
         CALL TRANS3                                                    
         !-----------------------------------------------------         
         ! Calculate nonlinear terms                                    
         !-----------------------------------------------------         
         CALL MGRMLT                                                    
         !-----------------------------------------------------         
         ! Transpose the arrays so that longitude dimension is          
         ! the only dimension that isn't decomposed over PEs            
         !-----------------------------------------------------         
         CALL TRANS4                                                    
         !-----------------------------------------------------         
         ! Convert the longitude dimension to a zonal wavenumber        
         ! dimension                                                    
         !-----------------------------------------------------         
         IF (iCol.EQ.1) THEN                                            
            CALL FFT(-1,NHEM*JGy,SPG2)                                  
            CALL FFT(-1,NHEM*JGy,VPG2)                                  
         ENDIF                                                          
         CALL FFT(-1,NHEM*JGy*NLx,EG2)                                  
         CALL FFT(-1,NHEM*JGy*NLx,TNLG2)                                
         CALL FFT(-1,NHEM*JGy*NLx,FUG2)                                 
         CALL FFT(-1,NHEM*JGy*NLx,FVG2)                                 
         CALL FFT(-1,NHEM*JGy*NLx,UTG2)                                 
         CALL FFT(-1,NHEM*JGy*NLx,VTG2)                                 
         CALL FFT(-1,NHEM*JGy*NLx*NTRAC,TRANLG2)                        
         CALL FFT(-1,NHEM*JGy*NLx*NTRAC,UTRAG2)                         
         CALL FFT(-1,NHEM*JGy*NLx*NTRAC,VTRAG2)                         
         !-----------------------------------------------------         
         ! Transpose the arrays so that latitude dimension is           
         ! the only dimension that isn't decomposed over PEs            
         !-----------------------------------------------------         
         CALL TRANS5(SPG,VPG,EG,TNLG,TRANLG,FUG,FVG,                    
     &        UTG,UTRAG,VTG,VTRAG)                                      
         !-----------------------------------------------------         
         ! Convert the zonal wavenumber dimension to a longitude        
         ! dimension                                                    
         !-----------------------------------------------------         
         CALL LTD                                                       
         !-----------------------------------------------------         
         ! Transpose the arrays so that the layer dimension is the      
         ! only dimension that isn't decomposed over PEs                
         !-----------------------------------------------------         
         CALL TRANS6                                                    
         !-----------------------------------------------------         
         ! Take del**2 of the energy term and add to the                
         ! divergence tendency.                                         
         !-----------------------------------------------------         
         CALL SPDEL2(DTE,FILT,MMTy,NNTx,NN,MM,NHEM,NL,0,2,              
     &        MMTyy(iRow),NNTxx(iCol),nWave(1,1,iCol))                  
         DT(:,:,:,:)=DT(:,:,:,:)+DTE(:,:,:,:)                           
         !-----------------------------------------------------         
         ! See if balancing is required                                 
         !-----------------------------------------------------         
         IF ((LBALAN).AND.(KOUNT.LT.0)) THEN                            
            !-----------------------------------------------------      
            ! Balance spectral fields during initialisation.            
            !-----------------------------------------------------      
            KOUNT=KOUNT+1                                               
            IF (.NOT.LTBAL) CALL BALANC                                 
            IF (     LTBAL) CALL TBAL                                   
            IF (KOUTP.EQ.0) CALL ENERGY                                 
         ELSE                                                           
            !-----------------------------------------------------      
            ! First timestep - output history and diagnostics           
            !-----------------------------------------------------      
            IF (KOUNT.EQ.0) THEN                                        
               RKOUNT=KOUNT                                             
               !-----------------------------------------------------   
               ! Write Z, D, T, TRA and SP to file                      
               !-----------------------------------------------------   
               CALL WRITECOEF(9, 1, RKOUNT, DOY)                        
               IF (MYPE.EQ.0) WRITE(2,2010)9,RKOUNT,RNTAPE,DAY,DOY      
               IF (LRESTIJ) THEN                                        
                  !---------------------------------------------------  
                  ! Write TTRES to file                                 
                  !---------------------------------------------------  
                  CALL WRITECOEF(13, 4, RKOUNT, DOY)                    
                  IF (MYPE.EQ.0) WRITE(2,2020)13,RKOUNT,RNTAPE,DAY,DOY  
               ENDIF                                                    
               !-----------------------------------------------------   
               ! Write sigma-latitude x-sections                        
               !-----------------------------------------------------   
               CALL XSECT(INLAT)                                        
               !-----------------------------------------------------   
               ! Write out the spectral coefficients                    
               !-----------------------------------------------------   
               CALL SPOP                                                
               !-----------------------------------------------------   
               ! Work out RMS diagnostics in ENERGY routine             
               !-----------------------------------------------------   
               IF (KOUTP.EQ.0) CALL ENERGY                              
            ELSE IF (KOUNT.EQ.KSTART) THEN                              
               !-----------------------------------------------------   
               ! Work out RMS diagnostics in ENERGY routine             
               !-----------------------------------------------------   
               IF (KOUTP.EQ.0) CALL ENERGY                              
            ENDIF                                                       
            IF (LRESTIJ) THEN                                           
               !-----------------------------------------------------   
               ! Write a restoration record                             
               !-----------------------------------------------------   
               IF (KOUTH.EQ.KOUNTH.OR.KOUTR.EQ.KOUNTR) THEN             
                  RKOUNT=KOUNT                                          
                  !---------------------------------------------------  
                  ! Write TTRES to file                                 
                  !---------------------------------------------------  
                  CALL WRITECOEF(13, 4, RKOUNT, DOY)                    
                  IF (MYPE.EQ.0) WRITE(2,2020)13,RKOUNT,RNTAPE,DAY,DOY  
               ENDIF                                                    
            ENDIF                                                       
            !-----------------------------------------------------      
            ! Write a restart record                                    
            !-----------------------------------------------------      
            IF (KOUTR.EQ.KOUNTR) THEN                                   
               RKOUNT=KOUNT                                             
               !-----------------------------------------------------   
               ! Write Z, D, T, TRA, SP, ZMI, DMI, TMI, TRAMI SPMI and  
               ! HTNET to file.                                         
               !-----------------------------------------------------   
               CALL WRITECOEF(11, 35, RKOUNT, DOY)                      
               IF ((LMASCOR).AND.(MyPe.EQ.0)) THEN                      
                  RREC=2.0                                              
                  WRITE(11) RKOUNT,RNTAPE,DAY,RREC,GMSP0,GMSPMI,RNTAPE  
               ENDIF                                                    
               !-----------------------------------------------------   
               ! Write grid surface variables to file                   
               !-----------------------------------------------------   
               CALL WRITEGRID(19, 1, RKOUNT, RNTAPE, DAY, DOY)          
               IF (MYPE.EQ.0) WRITE(2,2000)11,RKOUNT,RNTAPE,DAY,DOY     
               KOUTR=0                                                  
         call flush(11)                                                 
         call flush(19)                                                 
                                                                        
            ENDIF                                                       
            !-----------------------------------------------------      
            ! Write a history record                                    
            !-----------------------------------------------------      
            IF (KOUTH.EQ.KOUNTH) THEN                                   
               RKOUNT=KOUNT                                             
               IF (LMINIH) THEN                                         
                  !---------------------------------------------------  
                  ! Take average of temperatures                        
                  !---------------------------------------------------  
                  TAVE(:,:,:,:)=TAVE(:,:,:,:)/REAL(KOUNTH)              
                  !---------------------------------------------------  
                  ! Write Z, D, TAVE, TRA and SP to file                
                  !---------------------------------------------------  
                  CALL WRITECOEF(9, 9, RKOUNT, DOY)                     
                  !---------------------------------------------------  
                  ! Zero TAVE for next set of averaging                 
                  !---------------------------------------------------  
                  TAVE(:,:,:,:)=0.0                                     
               ELSE                                                     
                  !---------------------------------------------------  
                  ! Write Z, D, T, TRA and SP to file                   
                  !---------------------------------------------------  
                  CALL WRITECOEF(9, 1, RKOUNT, DOY)                     
               ENDIF                                                    
               IF (MYPE.EQ.0) WRITE(2,2010)9,RKOUNT,RNTAPE,DAY,DOY      
               KOUTH=0                                                  
            ENDIF                                                       
            !-----------------------------------------------------      
            ! Output diagnostics                                        
            !-----------------------------------------------------      
            IF (KOUTP.EQ.KOUNTP) THEN                                   
               !-----------------------------------------------------   
               ! Write sigma-latitude x-sections                        
               !-----------------------------------------------------   
               CALL XSECT(INLAT)                                        
               !-----------------------------------------------------   
               ! Write out the spectral coefficients                    
               !-----------------------------------------------------   
               CALL SPOP                                                
               KOUTP=0                                                  
            ENDIF                                                       
            IF (KOUTE.EQ.KOUNTE) THEN                                   
               IF (KOUTP.EQ.0) CALL ENERGY                              
               KOUTE=0                                                  
            ENDIF                                                       
            !-----------------------------------------------------      
            ! Move on a timestep                                        
            !-----------------------------------------------------      
            IF (KOUNT.LT.KTOTAL) THEN                                   
               KOUTP=KOUTP+1                                            
               KOUTE=KOUTE+1                                            
               KOUTH=KOUTH+1                                            
               KOUTR=KOUTR+1                                            
               KOUNT=KOUNT+1                                            
               IF(KOUNT.EQ.1.AND.KITS.GT.0) DAY=DAY+DELT/PI2            
               DAY=DAY+DELT/PI2                                         
               IF (.NOT. LPERPET) THEN                                  
                  IF(KOUNT.EQ.1.AND.KITS.GT.0) DOY=DOY+DELT/PI2         
                  DOY=DOY+DELT/PI2                                      
                  IF (DOY.GE.361) DOY = DOY-360.0                       
               ENDIF                                                    
               !-----------------------------------------------------   
               ! Adiabatic part of timestep                             
               !-----------------------------------------------------   
               CALL TSTEP                                               
               !-----------------------------------------------------   
               ! Mass correction.                                       
               !-----------------------------------------------------   
               IF (MyPe.EQ.0) THEN                                      
                  IF (LMASCOR) THEN                                     
                     CALL MASCOR                                        
                  ELSE IF (LRESTIJ) THEN                                
                     !------------------------------------------------  
                     ! Fix to maintain surface pressure                 
                     !------------------------------------------------  
                     SP(1,1,1)=CMPLX(0.,0.)                             
                  ENDIF                                                 
               ENDIF                                                    
               !-----------------------------------------------------   
               ! Annual cycle of TRES                                   
               !-----------------------------------------------------   
               IF (LRESTIJ) CALL SETTEE                                 
               !-----------------------------------------------------   
               ! Preset accumulated diagnostics.                        
               !-----------------------------------------------------   
               IF (KOUTH.EQ.1) THEN                                     
                  AUTVDZ(:,:,:)=0.0                                     
                  AVTVDZ(:,:,:)=0.0                                     
                  ATTVDZ(:,:,:)=0.0                                     
                  AQTVDZ(:,:,:)=0.0                                     
                  ATTCRZ(:,:,:)=0.0                                     
                  AQTCRZ(:,:,:)=0.0                                     
                  ATTLRZ(:,:,:)=0.0                                     
                  AQTLRZ(:,:,:)=0.0                                     
                  ATTRDZ(:,:,:)=0.0                                     
                  ACTCRZ(:,:,:)=0.0                                     
                  ACTLRZ(:,:,:)=0.0                                     
                  AUTBLZ(:,:)=0.0                                       
                  AVTBLZ(:,:)=0.0                                       
                  ATTBLZ(:,:)=0.0                                       
                  AQTBLZ(:,:)=0.0                                       
               ENDIF                                                    
               !-----------------------------------------------------   
               ! Diabatic part of timestep. Preset tendencies to zero.  
               !-----------------------------------------------------   
               ZT2(:,:,:,:)=0.0                                         
               DT2(:,:,:,:)=0.0                                         
               TT2(:,:,:,:)=0.0                                         
               TRAT2(:,:,:,:,:)=0.0                                     
               !-----------------------------------------------------   
               ! Transpose the arrays so that the meridional wavenumber 
               ! dimension is the only dimension that isn't decomposed  
               ! over PEs                                               
               !-----------------------------------------------------   
               CALL TRANS1                                              
               !-----------------------------------------------------   
               ! Convert the meridional wavenumber dimension to a       
               ! latitude dimension                                     
               !-----------------------------------------------------   
               CALL LTIDIA                                              
               !-----------------------------------------------------   
               ! Transpose the arrays so that the zonal wavenumber      
               ! dimension is the only dimension that isn't decomposed  
               ! over PEs                                               
               !-----------------------------------------------------   
               CALL TRANS2B(UG,VG,TG,TRAG,PLG)                          
               !-----------------------------------------------------   
               ! Convert zonal wavenumber dimension to longitude        
               ! dimension.                                             
               !-----------------------------------------------------   
               CALL FFT(1,NHEM*JGy*NLx,UG2)                             
               CALL FFT(1,NHEM*JGy*NLx,VG2)                             
               CALL FFT(1,NHEM*JGy*NLx,TG2)                             
               CALL FFT(1,NHEM*JGy*NLx*NTRAC,TRAG2)                     
               IF (iCol.EQ.1) THEN                                      
                  CALL FFT(1,NHEM*JGy,PLG2)                             
               ENDIF                                                    
               !-----------------------------------------------------   
               ! Transpose the arrays so that layer dimension is the    
               ! only dimension that isn't decomposed over PEs          
               !-----------------------------------------------------   
               CALL TRANS3B                                             
               !-----------------------------------------------------   
               ! Calculate diabatic terms                               
               !-----------------------------------------------------   
               CALL DGRMLT                                              
               !-----------------------------------------------------   
               ! Write grid variables to history file                   
               !-----------------------------------------------------   
               RKOUNT=KOUNT                                             
               IF(KOUTH.EQ.KOUNTH) THEN                                 
                  IF (LSHIST) THEN                                      
                     !------------------------------------------------- 
                     ! Write ASSBL, ..., ACLD and TNLG to file.         
                     !------------------------------------------------- 
                     CALL WRITEGRID(9, 10, RKOUNT, RNTAPE, DAY, DOY)    
                  ELSE                                                  
                     !------------------------------------------------- 
                     ! Write ASSBL, ..., ACLD, SSBL, ..., CLD and TNLG  
                     ! to file.                                         
                     !------------------------------------------------- 
                     CALL WRITEGRID(9, 14, RKOUNT, RNTAPE, DAY, DOY)    
                  ENDIF                                                 
               ENDIF                                                    
               !-----------------------------------------------------   
               ! Transpose the arrays so that longitude dimension is    
               ! the only dimension that isn't decomposed over PEs      
               !-----------------------------------------------------   
               CALL TRANS4B                                             
               !-----------------------------------------------------   
               ! Convert the longitude dimension to a zonal wavenumber  
               ! dimension                                              
               !-----------------------------------------------------   
               CALL FFT(-1,NHEM*JGy*NLx,TNLG2)                          
               CALL FFT(-1,NHEM*JGy*NLx,FUG2)                           
               CALL FFT(-1,NHEM*JGy*NLx,FVG2)                           
               CALL FFT(-1,NHEM*JGy*NLx*NTRAC,TRANLG2)                  
               !-----------------------------------------------------   
               ! Transpose the arrays so that latitude dimension is     
               ! the only dimension that isn't decomposed over PEs      
               !-----------------------------------------------------   
               CALL TRANS5B(TNLG,TRANLG,FUG,FVG)                        
               !-----------------------------------------------------   
               ! Convert the zonal wavenumber dimension to a longitude  
               ! dimension                                              
               !-----------------------------------------------------   
               CALL LTDDIA                                              
               !-----------------------------------------------------   
               ! Transpose the arrays so that the layer dimension is the
               ! only dimension that isn't decomposed over PEs          
               !-----------------------------------------------------   
               CALL TRANS6B                                             
               !-----------------------------------------------------   
               ! Write zonally averaged diagnostics and spectral heating
               ! to history file.                                       
               !-----------------------------------------------------   
               IF(KOUTH.GE.1.AND.KOUTH.LE.KOUNTH) THEN                  
                  !---------------------------------------------------  
                  ! Sum the grid-space diagnostics, ready for           
                  ! averaging later.                                    
                  !---------------------------------------------------  
                  AUTVDZ(:,:,:)=AUTVDZ(:,:,:)+UTVDZ(:,:,:)              
                  AVTVDZ(:,:,:)=AVTVDZ(:,:,:)+VTVDZ(:,:,:)              
                  ATTVDZ(:,:,:)=ATTVDZ(:,:,:)+TTVDZ(:,:,:)              
                  AQTVDZ(:,:,:)=AQTVDZ(:,:,:)+QTVDZ(:,:,:)              
                  ATTCRZ(:,:,:)=ATTCRZ(:,:,:)+TTCRZ(:,:,:)              
                  AQTCRZ(:,:,:)=AQTCRZ(:,:,:)+QTCRZ(:,:,:)              
                  ATTLRZ(:,:,:)=ATTLRZ(:,:,:)+TTLRZ(:,:,:)              
                  AQTLRZ(:,:,:)=AQTLRZ(:,:,:)+QTLRZ(:,:,:)              
                  ATTRDZ(:,:,:)=ATTRDZ(:,:,:)+TTRDZ(:,:,:)              
                  ACTCRZ(:,:,:)=ACTCRZ(:,:,:)+CTCRZ(:,:,:)              
                  ACTLRZ(:,:,:)=ACTLRZ(:,:,:)+CTLRZ(:,:,:)              
                  AUTBLZ(:,:)=AUTBLZ(:,:)+UTBLZ(:,:)                    
                  AVTBLZ(:,:)=AVTBLZ(:,:)+VTBLZ(:,:)                    
                  ATTBLZ(:,:)=ATTBLZ(:,:)+TTBLZ(:,:)                    
                  AQTBLZ(:,:)=AQTBLZ(:,:)+QTBLZ(:,:)                    
               ENDIF                                                    
               IF(KOUTH.EQ.KOUNTH) THEN                                 
                  !---------------------------------------------------  
                  ! Average the grid-space diagnostics                  
                  !---------------------------------------------------  
                  RKP=1.0/REAL(KOUNTH)                                  
                  AUTVDZ(:,:,:)=AUTVDZ(:,:,:)*RKP                       
                  AVTVDZ(:,:,:)=AVTVDZ(:,:,:)*RKP                       
                  ATTVDZ(:,:,:)=ATTVDZ(:,:,:)*RKP                       
                  AQTVDZ(:,:,:)=AQTVDZ(:,:,:)*RKP                       
                  ATTCRZ(:,:,:)=ATTCRZ(:,:,:)*RKP                       
                  AQTCRZ(:,:,:)=AQTCRZ(:,:,:)*RKP                       
                  ATTLRZ(:,:,:)=ATTLRZ(:,:,:)*RKP                       
                  AQTLRZ(:,:,:)=AQTLRZ(:,:,:)*RKP                       
                  ATTRDZ(:,:,:)=ATTRDZ(:,:,:)*RKP                       
                  ACTCRZ(:,:,:)=ACTCRZ(:,:,:)*RKP                       
                  ACTLRZ(:,:,:)=ACTLRZ(:,:,:)*RKP                       
                  AUTBLZ(:,:)=AUTBLZ(:,:)*RKP                           
                  AVTBLZ(:,:)=AVTBLZ(:,:)*RKP                           
                  ATTBLZ(:,:)=ATTBLZ(:,:)*RKP                           
                  AQTBLZ(:,:)=AQTBLZ(:,:)*RKP                           
                  !---------------------------------------------------  
                  ! Write grid-space data to file                       
                  !---------------------------------------------------  
                  RKOUNT=KOUNT                                          
                  CALL WRITEDIAG(9, RKOUNT, DOY)                        
                  !---------------------------------------------------  
                  ! Write TT to file                                    
                  !---------------------------------------------------  
                  CALL WRITECOEF(9, 16, RKOUNT, DOY)                    
               ENDIF                                                    
               !-----------------------------------------------------   
               ! Apply dissipation and optional linear restoration.     
               !-----------------------------------------------------   
               CALL DIFUSE                                              
               !-----------------------------------------------------   
               ! Update spectral fields in the diabatic timestep.       
               !-----------------------------------------------------   
               CALL DSTEP                                               
               !-----------------------------------------------------   
               ! Sum temperature over a number of timesteps, ready      
               ! for averaging later.                                   
               !-----------------------------------------------------   
               DO L=1,NL                                                
                  DO IHEM=1,NHEM                                        
                     DO IM=1,MMTyy(iRow)                                
                        DO IN=1,NNTxx(iCol)                             
                           TAVE(IN,IM,IHEM,L)=TAVE(IN,IM,IHEM,L)+       
     &                          T(IN,IM,IHEM,L)                         
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDIF                                                       
            !-----------------------------------------------------      
            ! End of timestep                                           
            !-----------------------------------------------------      
         ENDIF   ! closes ELSE for IF ((LBALAN).AND.(KOUNT.LT.0))       
      ENDDO      ! closes DO KOUNT=-KITS,KTOTAL                         
!-----------------------------------------------------                  
! Write the final restart record                                        
!-----------------------------------------------------                  
      RKOUNT=KOUNT                                                      
      IF (MyPe.EQ.0) OPEN(UNIT=12,FILE='LastRestart.dat',               
     &     FORM='unformatted')                                          
      CALL WRITECOEF(12, 35, RKOUNT, DOY)                               
      IF (MyPe.EQ.0) THEN                                               
         IF (LMASCOR) THEN                                              
            RREC=2.0                                                    
            WRITE(12) RKOUNT,RNTAPE,DAY,RREC,GMSP0,GMSPMI,RNTAPE        
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Write surface quantities to file                             
         !-----------------------------------------------------         
         OPEN(UNIT=17,FILE='LastSurfRestart.dat',FORM='unformatted')    
      ENDIF                                                             
      CALL WRITEGRID(17, 1, RKOUNT, RNTAPE, DAY, DOY)                   
      IF (MYPE.EQ.0) THEN                                               
         CLOSE(17)                                                      
         WRITE(2,2000)12,RKOUNT,RNTAPE,DAY,DOY                          
         CLOSE(12)                                                      
      ENDIF                                                             
      IF (LRESTIJ) THEN                                                 
         CALL WRITECOEF(13, 3, RKOUNT, DOY)                             
         IF (MYPE.EQ.0) THEN                                            
            WRITE(2,2020)13,RKOUNT,RNTAPE,DAY, DOY                      
            !-----------------------------------------------------      
            ! Close the restoration file                                
            !-----------------------------------------------------      
            CLOSE(13)                                                   
         ENDIF                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Close files                                                           
!-----------------------------------------------------                  
      IF (MYPE.EQ.0) THEN                                               
         CLOSE(2)                                                       
         CLOSE(9)                                                       
         CLOSE(11)                                                      
         CLOSE(19)                                                      
      ENDIF                                                             
!-----------------------------------------------------                  
! End MPI session                                                       
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         CALL MPI_BARRIER(MPI_COMM_WORLD,ierr)                          
         CALL MPI_FINALIZE(ierr)                                        
      ENDIF                                                             
      STOP                                                              
      END                                                               
!=====================================================                  
! INITAL calls other initialisation routines.                           
!=====================================================                  
      SUBROUTINE INITAL                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Intialisation of MPI session, assign latitudes,                       
! zonal wavenumbers etc. to PEs                                         
!-----------------------------------------------------                  
      CALL INIPARA                                                      
!-----------------------------------------------------                  
! Open input and output files                                           
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         !-----------------------------------------------------         
         ! Input files                                                  
         !-----------------------------------------------------         
         OPEN(UNIT=7,FILE='data',FORM='formatted')                      
         !-----------------------------------------------------         
         ! Output files                                                 
         !-----------------------------------------------------         
         OPEN(UNIT=2,FILE='output.dat',FORM='formatted')                
         OPEN(UNIT=9,FILE='history.dat',FORM='unformatted')             
         OPEN(UNIT=11,FILE='restart.dat',FORM='unformatted')            
         OPEN(UNIT=19,FILE='SurfRestart.dat',FORM='unformatted')        
      ENDIF                                                             
 !-----------------------------------------------------                 
 ! Display parallel PE information                                      
 !-----------------------------------------------------                 
      IF (MyPe.eq.0) THEN                                               
         WRITE(2,*) 'Number of PEs:        ',NPE                        
         WRITE(2,*) 'PE along longitudues: ',Px                         
         WRITE(2,*) 'PE along latitudues:  ',Py                         
         WRITE(2,*) 'MMT:',MMT                                          
         WRITE(2,*) 'NNT:',NNT                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Carry out the other initialisation routines                           
!-----------------------------------------------------                  
      CALL INISET                                                       
      CALL INIGAU                                                       
      CALL INISI                                                        
      CALL INIPHYS                                                      
      IF (LRESTIJ) THEN                                                 
        CALL INIRESIJ                                                   
      ELSE                                                              
        CALL INIRES                                                     
      ENDIF                                                             
      CALL INISTR                                                       
      CALL INISURF                                                      
      CALL INIQS                                                        
      END                                                               
!=====================================================                  
! Initialises the MPI session, divides up both the latitudes            
! and the zonal wavenumber between processors.                          
!=====================================================                  
      SUBROUTINE INIPARA                                                
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Define mpi variables needed for initialization                        
!-----------------------------------------------------                  
      INTEGER(KIND=Type) :: InitMype                                    
      INTEGER(KIND=Type) :: size                                        
      INTEGER(KIND=Type), PARAMETER :: BufferSize =                     
     &     4000000                                                      
      REAL, SAVE :: BsendBuffer(BufferSize)                             
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
! TWIN(1,:) is number of Ms (=1 or 2); TWIN(2:) is 1st M; TWIN(3:)      
! is 2nd M; TWIN(4,:) is number of Ns for 1st M.                        
      INTEGER :: TWIN(4,MMT)                                            
      INTEGER :: iPe                                                    
      INTEGER :: IM                                                     
      INTEGER :: iLength                                                
      INTEGER :: NNT2                                                   
      INTEGER :: nFull(NNT)                                             
      INTEGER :: N                                                      
      INTEGER :: MP                                                     
      INTEGER :: Coefconvfull(NNT,MMT)                                  
      INTEGER :: iCol2, iRow2                                           
! Loop variables                                                        
      INTEGER :: I, IN, IT, iZon, J, JM                                 
!-----------------------------------------------------                  
! Check that there are enough zonal wavenumbers for this run            
!-----------------------------------------------------                  
      IF ( (2*Py).GT.(1+MM/MOCT) ) THEN                                 
         WRITE(6,*) "ERROR: expect there to be at least twice the ",    
     &        "number of zonal wavenumbers as rows of PEs"              
         CALL ABORT                                                     
      ENDIF                                                             
!-----------------------------------------------------                  
! Initialise MPI session (if wanted)                                    
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         !-----------------------------------------------------         
         ! Start MPI session                                            
         !-----------------------------------------------------         
         CALL MPI_INIT(ierr)                                            
         CALL MPI_COMM_SIZE(MPI_COMM_WORLD, size, ierr)                 
         CALL MPI_COMM_RANK(MPI_COMM_WORLD, InitMyPe, ierr)             
         MyPe=InitMyPe                                                  
         !-----------------------------------------------------         
         ! Check we have the right number of processors                 
         !-----------------------------------------------------         
         IF (size.NE.NPE) THEN                                          
            WRITE(*,*) 'ERROR: set NPE to ',size                        
            CALL ABORT                                                  
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Set the buffer size                                          
         !-----------------------------------------------------         
         CALL MPI_BUFFER_ATTACH(BsendBuffer, 8*BufferSize, ierr)        
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
      ELSE                                                              
         !-----------------------------------------------------         
         ! One processor run                                            
         !-----------------------------------------------------         
         MyPe=0                                                         
      ENDIF                                                             
!-----------------------------------------------------                  
! Work out position of PE in grid                                       
!-----------------------------------------------------                  
      DO J=1,Py                                                         
         DO I=1,Px                                                      
            PeGrid(I,J)=(I-1)+(J-1)*Px                                  
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Store how dimensions are divided up                                   
!-----------------------------------------------------                  
      DO iPe=0,NPE-1                                                    
         !-----------------------------------------------------         
         ! Store grid position                                          
         !-----------------------------------------------------         
         iCol2=MOD(iPe,Px)+1                                            
         iRow2=iPe/Px+1                                                 
         GRID(1,iPe)=iCol2                                              
         GRID(2,iPe)=iRow2                                              
      ENDDO                                                             
!-----------------------------------------------------                  
! Separating dimensions across the PE columns                           
!-----------------------------------------------------                  
      DO iCol2=1,Px                                                     
         !-----------------------------------------------------         
         ! Divide up longitudes                                         
         !-----------------------------------------------------         
         MGxx(iCol2)=(MG+Px-iCol2)/Px                                   
         !-----------------------------------------------------         
         ! Divide up layers                                             
         !-----------------------------------------------------         
         NLxx(iCol2)=(NL+Px-iCol2)/Px                                   
         !-----------------------------------------------------         
         ! Divide up twined meridional wavenumbers                      
         !-----------------------------------------------------         
         NNTxx(iCol2)=(NNT+Px-iCol2)/Px                                 
      ENDDO                                                             
!-----------------------------------------------------                  
! Reference position for the PE column dimensions                       
!-----------------------------------------------------                  
      II1(1)=0                                                          
      KK1(1)=0                                                          
      NNT1(1)=0                                                         
      DO iCol2=2,Px                                                     
         II1(iCol2)=II1(iCol2-1)+MGxx(iCol2-1)                          
         KK1(iCol2)=KK1(iCol2-1)+NLxx(iCol2-1)                          
         NNT1(iCol2)=NNT1(iCol2-1)+NNTxx(iCol2-1)                       
      ENDDO                                                             
!-----------------------------------------------------                  
! Separating dimensions across the PE rows                              
!-----------------------------------------------------                  
      DO iRow2=1,Py                                                     
         !-----------------------------------------------------         
         ! Divide up latitudes                                          
         !-----------------------------------------------------         
         JGyy(iRow2)=(JG+Py-iRow2)/Py                                   
         !-----------------------------------------------------         
         ! Divide up twinned zonal wavenumbers                          
         !-----------------------------------------------------         
         MMTyy(iRow2)=(MMT+Py-iRow2)/Py                                 
      ENDDO                                                             
!-----------------------------------------------------                  
! Reference position for the PE row dimensions                          
!-----------------------------------------------------                  
      JJ1(1)=0                                                          
      MMT1(1)=0                                                         
      DO iRow2=2,Py                                                     
         JJ1(iRow2)=JJ1(iRow2-1)+JGyy(iRow2-1)                          
         MMT1(iRow2)=MMT1(iRow2-1)+MMTyy(iRow2-1)                       
      ENDDO                                                             
!-----------------------------------------------------                  
! Twin zonal wavenumbers. Start the zonal wavenumbers with              
! the most meridional wavenumbers.                                      
!-----------------------------------------------------                  
      TWIN(1,:)=1                                                       
      I=0                                                               
      DO IM=1,MMT                                                       
         !-----------------------------------------------------         
         ! Store value of wavenumber, MP                                
         !-----------------------------------------------------         
         MP=(IM-1)*MOCT                                                 
         TWIN(2,IM)=MP                                                  
         !-----------------------------------------------------         
         ! Determine the number of meridional wavenumber with this      
         !-----------------------------------------------------         
         TWIN(4,IM)=(MM-MP+1)/2                                         
         !-----------------------------------------------------         
         ! Store the order the coefficients are read in and             
         ! stored in NNT by MMT array.                                  
         !-----------------------------------------------------         
         DO J=1,TWIN(4,IM)                                              
            CoefConvFull(J,IM)=I+J                                      
         ENDDO                                                          
         I=I+TWIN(4,IM)                                                 
      ENDDO                                                             
!-----------------------------------------------------                  
! Add the 2nd set of zonal wavenumber to the 1st.                       
!-----------------------------------------------------                  
      DO JM=MMT,(MM-1)/MOCT                                             
         IM=2*MMT-JM                                                    
         !-----------------------------------------------------         
         ! Store value of wavenumber, MP                                
         !-----------------------------------------------------         
         MP=JM*MOCT                                                     
         TWIN(3,IM)=MP                                                  
         TWIN(1,IM)=2                                                   
         !-----------------------------------------------------         
         ! Store how the order the coefficients are read in are         
         ! stored in NNT by MMT array.                                  
         !-----------------------------------------------------         
         DO J=1,NNT-TWIN(4,IM)                                          
            CoefConvFull(J+TWIN(4,IM),IM)=I+J                           
         ENDDO                                                          
         I=I+NNT-TWIN(4,IM)                                             
      ENDDO                                                             
!-----------------------------------------------------                  
! Check the number of meriodonal wavenumber for a pair of               
! zonal wavenumber is correct.                                          
!-----------------------------------------------------                  
      NNT2=( (MM-TWIN(2,1)+1)/2 )                                       
      IF (TWIN(1,1).EQ.2) THEN                                          
         NNT2=NNT2+( (MM-TWIN(3,1)+1)/2 )                               
      ENDIF                                                             
      IF ((MyPe.EQ.0).AND.(NNT2.NE.NNT)) THEN                           
         IF (NNT2.NE.NNT) THEN                                          
            WRITE(6,*) 'ERROR: NNT needs changing from ',NNT,           
     &           ' to ',NNT2                                            
            CALL ABORT                                                  
         ENDIF                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Work out properties of measures on this PE                            
!-----------------------------------------------------                  
      iCol=GRID(1,MyPe)                                                 
      iRow=GRID(2,MyPe)                                                 
!-----------------------------------------------------                  
! Work out many Ns for each M on this PE                                
!-----------------------------------------------------                  
      DO IM=1,MMTyy(iRow)                                               
         mTwin2(0,IM)=TWIN(1,IM+MMT1(iRow))                             
         mTwin2(1,IM)=0                                                 
         mTwin2(2,IM)=TWIN(4,IM+MMT1(iRow))                             
         mTwin2(3,IM)=NNT                                               
         !-----------------------------------------------------         
         ! Values for this PE                                           
         !-----------------------------------------------------         
         mTwin1(0,IM)=mTwin2(0,IM)                                      
         mTwin1(1,IM)=0                                                 
         mTwin1(3,IM)=NNTxx(iCol)                                       
         IF ( mTwin2(2,IM).GE.(NNT1(iCol)+NNTxx(iCol)) ) THEN           
            mTwin1(2,IM)=NNTxx(iCol)                                    
         ELSE IF (mTwin2(2,IM).LE.NNT1(iCol)) THEN                      
            mTwin1(2,IM)=0                                              
         ELSE                                                           
            mTwin1(2,IM)=mTwin2(2,IM)-NNT1(iCol)                        
         ENDIF                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Store the values of N                                                 
!-----------------------------------------------------                  
      DO IM=1,MMTyy(iRow)                                               
         !-----------------------------------------------------         
         ! Create a complete list of N for this M pair                  
         !-----------------------------------------------------         
         mWave(1,IM)=TWIN(2,IM+MMT1(iRow))                              
         N=TWIN(2,IM+MMT1(iRow))                                        
         I=1                                                            
         DO N=TWIN(2,IM+MMT1(iRow)),NN-1,2                              
            nFull(I)=N                                                  
            I=I+1                                                       
         ENDDO                                                          
         IF (TWIN(1,IM+MMT1(iRow)).EQ.2) THEN                           
            mWave(2,IM)=TWIN(3,IM+MMT1(iRow))                           
            DO N=mWave(2,IM),NN-1,2                                     
               nFull(I)=N                                               
               I=I+1                                                    
            ENDDO                                                       
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Keep part of N for this PE                                   
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            DO IN=1,NNTxx(iCol2)                                        
               nWave(IN,IM,iCol2)=nFull(NNT1(iCol2)+IN)                 
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Create an array, CoefConv, for converting between how the             
! the coefficients are stored for reading in and writing out            
! and how they are stored in the code                                   
!-----------------------------------------------------                  
      DO iPe=0,NPE-1                                                    
         iCol2=GRID(1,iPe)                                              
         iRow2=GRID(2,iPe)                                              
         DO IM=1,MMTyy(iRow2)                                           
            DO IN=1,NNTxx(iCol2)                                        
               CoefConv(IN,IM,iPe)=                                     
     &              CoefConvFull(IN+NNT1(iCol2),IM+MMT1(iRow2))         
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
      END                                                               
!=====================================================                  
! Sets up various variables and arrays. Sets NAMELIST variables         
! to their default settings, then reads NAMELIST                        
!=====================================================                  
      SUBROUTINE INISET                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Switches counters and constants controlling type and frequency of     
! model output                                                          
!-----------------------------------------------------                  
      COMMON/OUTCON/RNTAPE,DAY,SQR2,RSQR2,EAM1,EAM2,TOUT1               
     &     ,TOUT2,LSPO,LGPO                                             
     &     ,RNTAPO                                                      
     &     ,LSHIST,LMINIH                                               
      COMMON/OUTCONINT/NCOEFF,NLAT,INLAT,INSPC,KOUNTP                   
     &                 ,KOUNTE,KOUNTH,KOUNTR,KOUTP                      
     &                 ,KOUTE,KOUTH,KOUTR                               
! Identyfying run number                                                
      REAL :: RNTAPE                                                    
! Maximum wavenumber of printed spectral coeffecients                   
      INTEGER :: NCOEFF                                                 
! Number of latitudes in printed grid point fields                      
      INTEGER :: NLAT                                                   
      INTEGER :: INLAT                                                  
!                                                                       
      INTEGER :: INSPC                                                  
! Number of timsteps between successive writing of spectral             
! and gridded fields                                                    
      INTEGER :: KOUNTP                                                 
      INTEGER :: KOUNTE                                                 
! Number of timsteps between successive writing of history              
! records, containing spectral fields at the present and                
! previous time levels.                                                 
      INTEGER :: KOUNTH                                                 
! Number of timsteps between successive writing of restart              
! records, containing the spectral fields at the present and            
! previous time levels                                                  
      INTEGER :: KOUNTR                                                 
      INTEGER :: KOUTP                                                  
      INTEGER :: KOUTE                                                  
      INTEGER :: KOUTH                                                  
      INTEGER :: KOUTR                                                  
      REAL :: DAY                                                       
      REAL :: SQR2                                                      
      REAL :: RSQR2                                                     
      REAL :: EAM1                                                      
      REAL :: EAM2                                                      
      REAL :: TOUT1                                                     
      REAL :: TOUT2                                                     
      LOGICAL :: LSPO(NL)                                               
      LOGICAL :: LGPO(NL)                                               
      REAL :: RNTAPO                                                    
      LOGICAL :: LSHIST                                                 
! Reduce the history file to single precision if true                   
      LOGICAL :: LMINIH                                                 
!-----------------------------------------------------                  
! Constants and arrays needed for the fast Fourier transforms           
!-----------------------------------------------------                  
      COMMON/COMFFT/TRIG,WORK                                           
      COMMON/COMFFTINT/IFAX                                             
      REAL :: TRIG((MG+MG+MG)/2+1)                                      
      REAL :: WORK(NCRAY*(MG+1))                                        
      INTEGER :: IFAX(10)                                               
!-----------------------------------------------------                  
! Polynomial used to aid vectorization of Legendre transforms           
!-----------------------------------------------------                  
      COMMON/POLYNO/POLY,CMPA                                           
      REAL :: POLY(NNT,MMTy,2,2)                                        
      COMPLEX :: CMPA(2*MMTy,NHEM)                                      
!-----------------------------------------------------                  
! Restoration fields and timescale                                      
!-----------------------------------------------------                  
      COMMON/RESTOR/ZRES,DRES,TRES,SPRES,DAMP                           
      REAL :: ZRES(IDM,NHEM,NL)                                         
      REAL :: DRES(IDM,NHEM,NL)                                         
      REAL :: TRES(IDM,NHEM,NL)                                         
      REAL :: SPRES(IDM,NHEM)                                           
      REAL :: DAMP                                                      
!-----------------------------------------------------                  
!                                                                       
!-----------------------------------------------------                  
      COMMON/STATS/GMSP0,GMSPMI,LMASCOR,LMASOLD,LMASPRT                 
      LOGICAL :: LMASCOR,LMASOLD,LMASPRT                                
      REAL :: GMSP0,GMSPMI                                              
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: TDISS, TSPD                                               
      REAL :: DAYS,DAYH,DAYR,DAYP                                       
      INTEGER :: NCKNTIN,NCKNT                                          
      INTEGER :: NDEL, NTRWG, JP                                        
      INTEGER :: NWJCH, NWJCH2                                          
      INTEGER :: NDELH                                                  
      REAL :: AKK                                                       
      CHARACTER*(80) :: CDUM                                            
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, KK, L, MP, NP, nT                         
!-----------------------------------------------------                  
! Namelists                                                             
!-----------------------------------------------------                  
      NAMELIST/INPPL/ GA,GASCON,RADEA,AKAP,WW                           
     &     ,P0,RV,CPD,CLATNT                                            
      NAMELIST/INPRN/ DAYS,BEGDAY,TSPD,KITS,PNU,TDISS                   
c    $   ,ACCELRUN,START,SYNC,ASYNC,OLDACCEL                            
     &     ,NDEL,T0,LRSTRT,LSTRETCH,LSHORT                              
     &     ,LBALAN,LRESTIJ,LNOISE                                       
     &     ,LMASCOR,LMASOLD,LMASPRT                                     
     &     ,LFLUX,KOLOUR                                                
     &     ,BEGDOY,LCLIM,LPERPET,L22L,LOROG,LCSFCT                      
      NAMELIST/INPOP/RNTAPE,NCKNTIN,DAYH,DAYR,DAYP,KOUNTE               
     &      ,NCOEFF,NLAT,LGPO,LSPO                                      
     &      ,RNTAPO                                                     
     &     ,NTRACO,LSHIST,LMINIH                                        
!-----------------------------------------------------                  
! Format statements                                                     
!-----------------------------------------------------                  
  205 FORMAT(/' *****RNTAPE*****',F12.3)                                
  207 FORMAT(' PRINTED OUTPUT EVERY ',I3,' TIMESTEPS'/                  
     +       ' RMS QUANTITIES OUTPUT EVERY ',I3,' TIMESTEPS'/           
     +       ' HISTORY RECORD WRITTEN EVERY ',I3,' TIMESTEPS'/          
     +       ' RESTART RECORD WRITTEN EVERY ',I3,' TIMESTEPS')          
  209 FORMAT(' INTEGRATION WITH',I3,                                    
     +' LEVELS IN THE VERTICAL (NL=',I3,')'/                            
     +' JAGGED TRIANGULAR/TRAPEZOIDAL TRUNCATION AT TOTAL WAVENO. ',I3/ 
     +' AND ZONAL WAVENO. ',I3,' (NN=',I3,' MM=',I3,')')                
  221 FORMAT(' NO LATERAL DISSIPATION')                                 
  222 FORMAT(' DEL',I2,' LATERAL DISSIPATION ON VORTICITY, DIVERGENCE'/ 
     + ' AND TEMPERATURE WITH DIFFUSION COEFFICIENT  ',E10.4,           
     + ' m**',I1,'/s'/' THE E-FOLDING TIME FOR SMALLEST RESOLVED',      
     + ' SCALE IS ',F5.3,' DAYS')                                       
  223 FORMAT(' NO TIME FILTER')                                         
  224 FORMAT(' ROBERT TIME FILTER WITH PARAMETER PNU ',F5.2)            
  280 FORMAT(' GLOBAL DOMAIN: BOTH EVEN AND ODD COEFFICIENTS INCLUDED', 
     +' (NHEM=',I1,')')                                                 
  281 FORMAT(' HEMISPHERIC DOMAIN: ONLY EVEN DIVERGENCE TEMPERATURE'/   
     +' SURFACE PRESSURE AND ODD VORTICITY COEFFICIENTS INCLUDED',      
     +' (NHEM=',I1,')')                                                 
  210 FORMAT(' ',I2,'-FOLD SYMMETRY IN LONGITUDE IMPOSED AND ONLY'/     
     +' 1 /',I2,' OF THE DOMAIN USED (MOCT=',I2,')')                    
  211 FORMAT(' NON LINEAR TERMS EVALUATED ON GRID OF ',I3,              
     +' GAUSSIAN LATITUDES '/' AND ',I3,                                
     +' EVENLY SPACED LONGITUDES (JG=',I3,' MG=',I3,')')                
  212 FORMAT(' ECMWF ANGULAR MOMENTUM CONSERVING VERTICAL SCHEME')      
  213 FORMAT(' GLOBAL MASS CORRECTION SWITCHED ON')                     
  214 FORMAT(' GLOBAL MASS CORRECTION SWITCHED OFF')                    
  225 FORMAT(/' ***ABORT*** CORRECT VALUE OF NWJ2',I5,' VALUE GIVEN',I5)
  232 FORMAT(/' ***ABORT***  NLAT IS GREATER THAN JG*NHEM')             
  233 FORMAT(/' ***ABORT***NCOEFF IS GREATER THAN NN')                  
  240 FORMAT(/' ***ABORT*** NTRACO IS GREATER THAN NTRAC')              
!-----------------------------------------------------                  
! Variables are set on PE 0 and then passed to the other PEs            
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         !-----------------------------------------------------         
         ! Set default values and override as desired through           
         ! NAMELIST input                                               
         !-----------------------------------------------------         
         GA=9.80665                                                     
         GASCON=287.0                                                   
         RADEA=6371000.0                                                
         AKAP=0.286                                                     
         WW=7.292E-5                                                    
C                                                                       
         KRUN=0                                                         
         BEGDAY=0.0                                                     
         TSPD=24.0                                                      
         KITS=3                                                         
         PNU=0.02                                                       
         TDISS=0.25                                                     
         NDEL=6                                                         
         DO L=1,NL                                                      
            T0(L)=250.0                                                 
         ENDDO                                                          
         LRSTRT=.FALSE.                                                 
         LSTRETCH =.FALSE.                                              
                                                                        
         LSHORT=.FALSE.                                                 
         LBALAN=.FALSE.                                                 
         LRESTIJ=.FALSE.                                                
         LNOISE=.FALSE.                                                 
C                                                                       
         RNTAPE=0.0                                                     
         KOUNTH=0                                                       
         KOUNTR=0                                                       
         KOUNTP=0                                                       
         KOUNTE=0                                                       
         NCOEFF=0                                                       
         NLAT=MIN(16,JG*NHEM)                                           
                                                                        
         DO I=1,NL                                                      
            LSPO(I)=.FALSE.                                             
            LGPO(I)=.FALSE.                                             
         ENDDO                                                          
         ! mass correction switched on                                  
         LMASCOR=.TRUE.                                                 
         ! masses read from restart file                                
         LMASOLD=.TRUE.                                                 
         ! mass correction printed every KOUNTP steps                   
         LMASPRT=.FALSE.                                                
         LFLUX=.TRUE.                                                   
                                                                        
         P0=100000.0                                                    
         RV=461.51                                                      
         CLATNT=2.5E6                                                   
         CPD=GASCON/AKAP                                                
                                                                        
         RNTAPO=-999.0                                                  
         NTRACO=0                                                       
                                                                        
         BEGDOY=0.0                                                     
                                                                        
         LPERPET=.TRUE.                                                 
         LCLIM=.FALSE.                                                  
         L22L=.TRUE.                                                    
         LOROG=.FALSE.                                                  
         LCSFCT=.TRUE.                                                  
         LSHIST=.TRUE.                                                  
         LMINIH=.TRUE.                                                  
                                                                        
         IF (RNTAPO.EQ.-999.0) RNTAPO=RNTAPE                            
         !-----------------------------------------------------         
         ! Read NAMELISTs, overwrite defaults and write them out        
         !-----------------------------------------------------         
         READ(7,INPPL)                                                  
         WRITE(2,INPPL)                                                 
      READ(7,INPRN)                                                     
      KRUN=DAYS*TSPD                                                    
      write(6,*) " KRUN = ",KRUN                                        
         WRITE(2,INPRN)                                                 
      READ(7,INPOP)                                                     
      NCKNT=NCKNTIN                                                     
      KOUNTP=DAYP*TSPD                                                  
      KOUNTH=DAYH*TSPD                                                  
      KOUNTR=DAYR*TSPD                                                  
         WRITE(2,INPOP)                                                 
      ENDIF       ! ends IF (MYPE.EQ.0) THEN                            
!-----------------------------------------------------                  
! Pass the information to the other PEs                                 
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         iSend=0                                                        
         nBuffSize=1                                                    
         !-----------------------------------------------------         
         ! Send most of contents of INPPL namelist                      
         !-----------------------------------------------------         
         CALL MPI_BARRIER(MPI_COMM_WORLD,ierr)                          
         CALL MPI_BCAST(GA,nBuffSize,ParaReal,iSend,                    
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(GASCON,nBuffSize,ParaReal,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(RADEA,nBuffSize,ParaReal,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(AKAP,nBuffSize,ParaReal,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(WW,nBuffSize,ParaReal,iSend,                    
     &        MPI_COMM_WORLD,ierr)                                      
         !-----------------------------------------------------         
         ! Send most of contents of INPRN namelist                      
         !-----------------------------------------------------         
         CALL MPI_BCAST(KRUN,nBuffSize,ParaInt,iSend,                   
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(BEGDAY,nBuffSize,ParaReal,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(TSPD,nBuffSize,ParaReal,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(KITS,nBuffSize,ParaInt,iSend,                   
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(PNU,nBuffSize,ParaReal,iSend,                   
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(TDISS,nBuffSize,ParaReal,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(NDEL,nBuffSize,ParaInt,iSend,                   
     &        MPI_COMM_WORLD,ierr)                                      
         nBuffSize=NL                                                   
         CALL MPI_BCAST(T0,nBuffSize,ParaReal,iSend,                    
     &        MPI_COMM_WORLD,ierr)                                      
         nBuffSize=1                                                    
         CALL MPI_BCAST(LRSTRT,nBuffSize,ParaLog,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LSTRETCH,nBuffSize,ParaLog,iSend,               
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LSHORT,nBuffSize,ParaLog,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LBALAN,nBuffSize,ParaLog,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LRESTIJ,nBuffSize,ParaLog,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LNOISE,nBuffSize,ParaLog,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         !-----------------------------------------------------         
         ! Send most of contents of INPOP namelist                      
         !-----------------------------------------------------         
         CALL MPI_BCAST(RNTAPE,nBuffSize,ParaReal,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(KOUNTH,nBuffSize,ParaInt,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(KOUNTR,nBuffSize,ParaInt,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(KOUNTP,nBuffSize,ParaInt,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(KOUNTE,nBuffSize,ParaInt,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(NCOEFF,nBuffSize,ParaInt,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(NLAT,nBuffSize,ParaInt,iSend,                   
     &        MPI_COMM_WORLD,ierr)                                      
         nBuffSize=NL                                                   
         CALL MPI_BCAST(LSPO,nBuffSize,ParaLog,iSend,                   
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LGPO,nBuffSize,ParaLog,iSend,                   
     &        MPI_COMM_WORLD,ierr)                                      
         nBuffSize=1                                                    
         CALL MPI_BCAST(LMASCOR,nBuffSize,ParaLog,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LMASOLD,nBuffSize,ParaLog,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LMASPRT,nBuffSize,ParaLog,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LFLUX,nBuffSize,ParaLog,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         !-----------------------------------------------------         
         !                                                              
         !-----------------------------------------------------         
         CALL MPI_BCAST(P0,nBuffSize,ParaReal,iSend,                    
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(RV,nBuffSize,ParaReal,iSend,                    
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(CLATNT,nBuffSize,ParaReal,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(CPD,nBuffSize,ParaReal,iSend,                   
     &        MPI_COMM_WORLD,ierr)                                      
         !-----------------------------------------------------         
         !                                                              
         !-----------------------------------------------------         
         CALL MPI_BCAST(RNTAPO,nBuffSize,ParaReal,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(NTRACO,nBuffSize,ParaReal,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(BEGDOY,nBuffSize,ParaReal,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         !-----------------------------------------------------         
         !                                                              
         !-----------------------------------------------------         
         CALL MPI_BCAST(LPERPET,nBuffSize,ParaLog,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LCLIM,nBuffSize,ParaLog,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(L22L,nBuffSize,ParaLog,iSend,                   
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LOROG,nBuffSize,ParaLog,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LCSFCT,nBuffSize,ParaLog,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         !-----------------------------------------------------         
         !                                                              
         !-----------------------------------------------------         
         CALL MPI_BCAST(LSHIST,nBuffSize,ParaLog,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LMINIH,nBuffSize,ParaLog,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
      ENDIF                                                             
!-----------------------------------------------------                  
! Write dimension to main output file                                   
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         !-----------------------------------------------------         
         ! Write out details of model run                               
         !-----------------------------------------------------         
         WRITE(2,205)RNTAPE                                             
         WRITE(2,209)NL,NL,NN,MM,NN,MM                                  
         IF(NHEM.EQ.2) WRITE(2,280) NHEM                                
         IF(NHEM.EQ.1) WRITE(2,281) NHEM                                
         WRITE(2,210) MOCT,MOCT,MOCT                                    
         WRITE(2,211)JG,MG,JG,MG                                        
         WRITE(2,212)                                                   
         IF (LMASCOR) THEN                                              
            WRITE(2,213)                                                
         ELSE                                                           
            WRITE(2,214)                                                
         ENDIF                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Set remaining physical constants and resolution dependent             
! quantities                                                            
!-----------------------------------------------------                  
      AIOCT=(0.,1.)*MOCT                                                
      RD=GASCON                                                         
!-----------------------------------------------------                  
! Set SQ(n+1)=n(n+1) and RSQ(n+1)=1/(n(n+1))                            
!-----------------------------------------------------                  
      DO NP=1,NN+1                                                      
         SQ(NP)=NP*(NP-1)                                               
         IF (NP.GT.1) THEN                                              
            RSQ(NP)=1./SQ(NP)                                           
         ELSE                                                           
            RSQ(1)=0.                                                   
         ENDIF                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Compute internal diffusion parameter                                  
!-----------------------------------------------------                  
      IF(TDISS.EQ.0.0) THEN                                             
         AKK=0.0                                                        
      ELSE                                                              
         AKK=WW*(RADEA**NDEL)/                                          
     &        (2.0*PI*TDISS*((NN*(NN+1))**REAL(NDEL/2)))                
      END IF                                                            
      IF (MYPE.EQ.0) THEN                                               
         IF(AKK.EQ.0.0) WRITE(2,221)                                    
         IF(AKK.NE.0.0) WRITE(2,222) NDEL,AKK,NDEL,TDISS                
      ENDIF                                                             
      AKK=AKK/(WW*(RADEA**NDEL))                                        
      NDELH=NDEL/2                                                      
      DO NP=1,NNP                                                       
         AK(NP)=AKK*(SQ(NP)**NDELH)                                     
      ENDDO                                                             
!-----------------------------------------------------                  
! Set time variables and counters                                       
!-----------------------------------------------------                  
      IF (MYPE.EQ.0) THEN                                               
         IF(PNU.EQ.0.0)WRITE(2,223)                                     
         IF(PNU.NE.0.0)WRITE(2,224)PNU                                  
         WRITE(2,207)KOUNTP,KOUNTE,KOUNTH,KOUNTR                        
      ENDIF                                                             
                                                                        
      IF(KOUNTP.EQ.0) KOUNTP=-999                                       
      IF(KOUNTE.EQ.0) KOUNTE=-999                                       
      IF(KOUNTH.EQ.0) KOUNTH=-999                                       
      IF(KOUNTR.EQ.0) KOUNTR=-999                                       
      DELT=PI2/TSPD                                                     
      PNU2=PNU+PNU                                                      
      PNU21=1.0-PNU2                                                    
      ITSPD=NINT(TSPD)                                                  
!-----------------------------------------------------                  
! Check variables make sense                                            
!-----------------------------------------------------                  
      IF (MYPE.EQ.0) THEN                                               
         IF (NLAT.GT.(JG*NHEM)) THEN                                    
            WRITE(2,232)                                                
            CLOSE(2)                                                    
            CALL ABORT                                                  
         ENDIF                                                          
         IF (NCOEFF.GT.NN) THEN                                         
            WRITE(2,233)                                                
            CLOSE(2)                                                    
            CALL ABORT                                                  
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Work out how many spectral coefficients need retaining       
         !-----------------------------------------------------         
         NWJCH=0                                                        
         DO MP=0,MM-1,MOCT                                              
            DO JP=MP,NN-1,MH                                            
               NWJCH=NWJCH+1                                            
            ENDDO                                                       
         ENDDO                                                          
         IF(NWJ2.NE.NWJCH) THEN                                         
            WRITE(2,225) NWJCH,NWJ2                                     
            CLOSE(2)                                                    
            CALL ABORT                                                  
         ENDIF                                                          
         IF (NTRACO.GT.NTRAC) THEN                                      
            WRITE(2,240)                                                
            CLOSE(2)                                                    
            CALL ABORT                                                  
         ENDIF                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Set dimensionalising factors                                          
!-----------------------------------------------------                  
      EZ=1.0/SQRT(.375)                                                 
      CV=RADEA*WW                                                       
      CG=CV*CV                                                          
      CT=CG/GASCON                                                      
      CQ=1000.0                                                         
      PFAC=0.5*CV*CV*1.0E5/GA                                           
      SQR2=SQRT(2.0)                                                    
      RSQR2=1.0/SQR2                                                    
      EAM1=SQR2/3.                                                      
      EAM2=SQRT(2./45.)                                                 
!-----------------------------------------------------                  
! Make T0 dimensionless                                                 
!-----------------------------------------------------                  
      DO L=1,NL                                                         
        T0(L)=T0(L)/CT                                                  
      ENDDO                                                             
!-----------------------------------------------------                  
! Hard wires water to Tracer Number 1 water has "colour" of 3.          
!-----------------------------------------------------                  
      IF (NTRAC.GE.1) THEN                                              
         KOLOUR(1)=3                                                    
         !-----------------------------------------------------         
         ! convert from g/Kg                                            
         !-----------------------------------------------------         
         CTRA(1)=CQ                                                     
      ENDIF                                                             
!-----------------------------------------------------                  
! Calculate auxiliary values required by FFT991                         
!-----------------------------------------------------                  
      CALL SET99(TRIG,IFAX,MG)                                          
!-----------------------------------------------------                  
! Set output control variables and initialise WRSPS                     
!-----------------------------------------------------                  
      INSPC=0                                                           
      DO MP=1,NCOEFF,MOCT                                               
         DO JP=MP,NCOEFF,MH                                             
            INSPC=INSPC+1                                               
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Calling WRSPS the first time sets variables that need to be set       
! once before spectral coefficients can be written out                  
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) CALL WRSPS(CDUM,Z,1,NL)                            
      IF (NLAT.NE.0) THEN                                               
         INLAT=(JG*NHEM)/NLAT                                           
      ELSE                                                              
         INLAT=0                                                        
      ENDIF                                                             
!-----------------------------------------------------                  
! Set up CMPA array to calculate x-derivative of half transforms        
!-----------------------------------------------------                  
      CMPA(:,:)=0.0                                                     
      DO IHEM=1,NHEM                                                    
         DO nT=1,2                                                      
            DO IM=1,MMTyy(iRow)                                         
               MP=(nT-1)*(2*(MMTyy(iRow)-IM)+1)+IM                      
               CMPA(MP,IHEM)=CMPLX(0.,REAL(mWave(nT,IM)))               
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
      END                                                               
!=====================================================                  
! This subroutine calculates gaussian weights and latitudes             
!=====================================================                  
      SUBROUTINE INIGAU                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
! Legendre polynomials                                                  
      REAL :: ALPJ(2,NWJ2)                                              
! Derivatives of Legendre polynomials                                   
      REAL :: DALPJ(2,NWJ2)                                             
! Gaussian weight for current latitude                                  
      REAL :: WEIGHT                                                    
      INTEGER :: INFO                                                   
      REAL :: SISQ                                                      
      INTEGER :: I                                                      
      REAL :: ALAT2(JG)                                                 
      LOGICAL :: lDum                                                   
      INTEGER :: NP                                                     
      INTEGER :: iPe                                                    
! Loop variables                                                        
      INTEGER :: IHEM, IM, IN, J, JJ, K, M, N                           
!-----------------------------------------------------                  
! Format statments                                                      
!-----------------------------------------------------                  
  202 FORMAT(' GAUSSIAN LATITUDES')                                     
  201 FORMAT(10F7.2)                                                    
!-----------------------------------------------------                  
! Loop over the latitudes                                               
!-----------------------------------------------------                  
      DO J=1,JG                                                         
         !-----------------------------------------------------         
         ! Calculate the sine of latitude and Gaussian weight           
         ! (GWTLT is external subroutine found in libutil)              
         !-----------------------------------------------------         
         CALL GWTLT(SI(J,1),WEIGHT,J,JG,INFO)                           
         IF ((MyPe.EQ.0).AND.(INFO.NE.0)) THEN                          
            WRITE(2,'(15a)') ' NO CONVERGENCE'                          
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Calculate other coefficient connected with latitude          
         !-----------------------------------------------------         
         SISQ=SI(J,1)*SI(J,1)                                           
         CSSQ(J,1)=1.-SISQ                                              
         SECSQ(J,1)=1./CSSQ(J,1)                                        
         CS(J,1)=SQRT(CSSQ(J,1))                                        
         AW(J,1)=WEIGHT*2.0*SECSQ(J,1)                                  
         ALAT(J,1)=ATAN(SI(J,1)/CS(J,1))*180.0/PI                       
         !-----------------------------------------------------         
         ! Compute Legendre functions at the current latitude.          
         !-----------------------------------------------------         
         CALL LGNDRE(NN,MM,MOCT,ALPJ,DALPJ,NWJ2,SI(J,1),CS(J,1))        
         !-----------------------------------------------------         
         ! Reorder Legendre functions, separating even/odd functions.   
         !-----------------------------------------------------         
         DO K=1,2                                                       
            DO IM=1,MMTyy(iRow)                                         
               DO I=1,Px                                                
                  DO IN=1,NNTxx(I)                                      
                     NP=nWave(IN,IM,I)                                  
                     iPe=PeGrid(I,iRow)                                 
                     ALP(IN+NNT1(I),IM,K,J)=                            
     &                    ALPJ(K,CoefConv(IN,IM,iPe))                   
                     DALP(IN+NNT1(I),IM,K,J)=                           
     &                    DALPJ(K,CoefConv(IN,IM,iPe))                  
                     RLP(IN+NNT1(I),IM,K,J)=                            
     &                    -RSQ(NP+K)*ALP(IN+NNT1(I),IM,K,J)             
                     RDLP(IN+NNT1(I),IM,K,J)=                           
     &                    -RSQ(NP+K)*DALP(IN+NNT1(I),IM,K,J)            
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Set values for the 2nd hemisphere                                     
!-----------------------------------------------------                  
      IF (NHEM.EQ.2) THEN                                               
CDIR$    IVDEP                                                          
         DO J=1,JG                                                      
            SI(J,NHEM)   =-SI(J,1)                                      
            CS(J,NHEM)   =CS(J,1)                                       
            CSSQ(J,NHEM) =CSSQ(J,1)                                     
            SECSQ(J,NHEM)=SECSQ(J,1)                                    
            AW(J,NHEM)   =AW(J,1)                                       
            ALAT(J,NHEM)=-ALAT(J,1)                                     
         ENDDO                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Output the Gaussian latitudes                                         
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         WRITE(2,202)                                                   
         WRITE(2,201)((ALAT(J+(IHEM-1)*(JG-2*J+1),IHEM),J=1,JG),        
     &        IHEM=1,NHEM)                                              
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Sets up arrays and variables for the vertical structure               
! and the semi-implicit scheme                                          
!=====================================================                  
      SUBROUTINE INISI                                                  
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Switches counters and constants controlling type and frequency of     
! model output                                                          
!-----------------------------------------------------                  
      COMMON/OUTCON/RNTAPE,DAY,SQR2,RSQR2,EAM1,EAM2,TOUT1               
     &     ,TOUT2,LSPO,LGPO                                             
     &     ,RNTAPO                                                      
     &     ,LSHIST,LMINIH                                               
      COMMON/OUTCONINT/NCOEFF,NLAT,INLAT,INSPC,KOUNTP                   
     &                 ,KOUNTE,KOUNTH,KOUNTR,KOUTP                      
     &                 ,KOUTE,KOUTH,KOUTR                               
! Identyfying run number                                                
      REAL :: RNTAPE                                                    
! Maximum wavenumber of printed spectral coeffecients                   
      INTEGER :: NCOEFF                                                 
! Number of latitudes in printed grid point fields                      
      INTEGER :: NLAT                                                   
      INTEGER :: INLAT                                                  
!                                                                       
      INTEGER :: INSPC                                                  
! Number of timsteps between successive writing of spectral             
! and gridded fields                                                    
      INTEGER :: KOUNTP                                                 
      INTEGER :: KOUNTE                                                 
! Number of timsteps between successive writing of history              
! records, containing spectral fields at the present and                
! previous time levels.                                                 
      INTEGER :: KOUNTH                                                 
! Number of timsteps between successive writing of restart              
! records, containing the spectral fields at the present and            
! previous time levels                                                  
      INTEGER :: KOUNTR                                                 
      INTEGER :: KOUTP                                                  
      INTEGER :: KOUTE                                                  
      INTEGER :: KOUTH                                                  
      INTEGER :: KOUTR                                                  
      REAL :: DAY                                                       
      REAL :: SQR2                                                      
      REAL :: RSQR2                                                     
      REAL :: EAM1                                                      
      REAL :: EAM2                                                      
      REAL :: TOUT1                                                     
      REAL :: TOUT2                                                     
      LOGICAL :: LSPO(NL)                                               
      LOGICAL :: LGPO(NL)                                               
      REAL :: RNTAPO                                                    
      LOGICAL :: LSHIST                                                 
! Reduce the history file to single precision if true                   
      LOGICAL :: LMINIH                                                 
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: H(NL),CR(NL),CI(NL),TBM1(NL,NL),WA(NL)                    
      INTEGER :: IWA(NL)                                                
      REAL :: STP                                                       
      REAL :: T1, T2                                                    
      REAL :: P                                                         
      REAL :: S1, S2                                                    
      REAL :: T0M                                                       
      REAL :: T0P                                                       
      REAL :: ALJ                                                       
      REAL :: HL                                                        
      REAL :: TAQ                                                       
      REAL :: RDSIGL                                                    
      REAL :: SIG                                                       
      REAL :: DSIG                                                      
      REAL :: SIGM                                                      
      REAL :: TM, TMM, TTM                                              
      REAL :: TTAU                                                      
      REAL :: TTKP                                                      
      REAL :: FAC                                                       
      REAL :: TT01S2                                                    
      REAL :: DELTSQ                                                    
      REAL :: T0L                                                       
      REAL :: RCN                                                       
      REAL :: SFAC                                                      
      REAL :: RGVAL                                                     
      REAL :: DA, DB, DC, DD                                            
! Loop variables                                                        
      INTEGER :: I, IN, J, L, M, N                                      
!-----------------------------------------------------                  
! Format statements                                                     
!-----------------------------------------------------                  
  203 FORMAT(' STANDARD HEIGHTS IN KM')                                 
  206 FORMAT(' GRAVITY WAVE SPEEDS IN M/SEC')                           
  208 FORMAT(' VALUES OF SIGMA AT HALF LEVELS')                         
  213 FORMAT(1X,8F8.4)                                                  
  214 FORMAT(' VALUES OF SIGMA AT FULL LEVELS')                         
  215 FORMAT(' BASIC STATE TEMPERATURES (NON-DIMENSIONAL)')             
  216 FORMAT(1X,8F8.3)                                                  
  217 FORMAT(5X,8F8.3)                                                  
!-----------------------------------------------------                  
! Set the sigma levels                                                  
!-----------------------------------------------------                  
      STP=1.0/NL                                                        
      IF (.NOT. LSTRETCH) THEN                                          
         DO L=1,NL-1                                                    
            SIGMAH(L)=L*STP                                             
         ENDDO                                                          
      ELSE                                                              
         P=0.0                                                          
         T1=(.9375/.94-1.25)/(.9375*(SQRT(.9375)-1.0))                  
         T2=4.0+T1                                                      
         DO L=1,NL-1                                                    
            P=P+STP                                                     
            SIGMAH(L)=P*(2.0-P)*(1.0+0.25*SIN(PI2*(P**.6)))/            
     &                (5.0-T2*P+T1*(P**1.5))                            
         ENDDO                                                          
      ENDIF                                                             
      IF (L22L.AND.NL.EQ.22) THEN                                       
         SIGMAH(1)=0.002                                                
         SIGMAH(2)=0.009                                                
         SIGMAH(3)=0.019                                                
         SIGMAH(4)=0.037                                                
         SIGMAH(5)=0.063                                                
         SIGMAH(6)=0.100                                                
         SIGMAH(7)=0.136                                                
         SIGMAH(8)=0.172                                                
         SIGMAH(9)=0.207                                                
         SIGMAH(10)=0.242                                               
         SIGMAH(11)=0.277                                               
         SIGMAH(12)=0.316                                               
         SIGMAH(13)=0.360                                               
         SIGMAH(14)=0.411                                               
         SIGMAH(15)=0.472                                               
         SIGMAH(16)=0.544                                               
         SIGMAH(17)=0.624                                               
         SIGMAH(18)=0.712                                               
         SIGMAH(19)=0.800                                               
         SIGMAH(20)=0.882                                               
         SIGMAH(21)=0.951                                               
      ENDIF                                                             
      S1=0.                                                             
      DO L=1,NL-1                                                       
         S2=SIGMAH(L)                                                   
         DSIGMA(L)=S2-S1                                                
         SIGMA(L)=0.5*(S2+S1)                                           
         RDSIG(L)=0.5/DSIGMA(L)                                         
         S1=S2                                                          
      ENDDO                                                             
      DSIGMA(NL)=1.-SIGMAH(NL-1)                                        
      RDSIG(NL)=0.5/DSIGMA(NL)                                          
      SIGMA(NL)=0.5*(1.+SIGMAH(NL-1))                                   
!-----------------------------------------------------                  
! This value, used in setting ALPHA(1), is irrelevant in the            
! angular momentum conserving ECMWF scheme                              
!-----------------------------------------------------                  
      S1=LOG(SIGMA(1)*SIGMA(1)/SIGMAH(1))                               
      T0M=T0(1)                                                         
      DO L=1,NL-1                                                       
         S2=LOG(SIGMAH(L))                                              
         T0P=T0(L+1)                                                    
         G(1,L)=0.                                                      
         T01S2(L)=T0P-T0M                                               
         ALPHA(L)=S2-S1                                                 
         TKP(L)=AKAP*T0M                                                
         T0M=T0P                                                        
         S1=S2                                                          
      ENDDO                                                             
      ALPHA(NL)=-S1                                                     
      TKP(NL)=AKAP*T0M                                                  
      G(1,1)=1.0                                                        
      DO J=2,NL                                                         
         ALJ=ALPHA(J)                                                   
         DO I=1,J-1                                                     
            G(J,I)=ALJ                                                  
         ENDDO                                                          
         G(J,J)=1.0-ALJ*SIGMAH(J-1)/DSIGMA(J)                           
         IF (J.LT.NL) THEN                                              
            DO I=J+1,NL                                                 
               G(J,I)=0.                                                
            ENDDO                                                       
         ENDIF                                                          
      ENDDO                                                             
      DO I=1,NL                                                         
         DO J=I,NL                                                      
            C(I,J)=G(J,I)*DSIGMA(I)/DSIGMA(J)                           
         ENDDO                                                          
      ENDDO                                                             
      TT01S2=T01S2(1)                                                   
      TAU(1,1)=0.5*TT01S2*(SIGMAH(1)-1.0)+TKP(1)*C(1,1)                 
      DO L=2,NL                                                         
         TAU(L,1)=0.5*TT01S2*DSIGMA(L)                                  
      ENDDO                                                             
      SIG=SIGMAH(1)                                                     
      DO L=2,NL                                                         
         TTKP=TKP(L)                                                    
         TTM=TT01S2                                                     
         SIGM=SIG                                                       
         IF (L.LT.NL) THEN                                              
            TT01S2=T01S2(L)                                             
            SIG=SIGMAH(L)                                               
         ENDIF                                                          
         RDSIGL=RDSIG(L)                                                
         DO M=1,NL                                                      
            IF( M.LT.L) THEN                                            
               TM=1.                                                    
               TMM=1.                                                   
            ELSEIF (M.EQ.L) THEN                                        
               TM=1.                                                    
               TMM=0.                                                   
            ELSE                                                        
               TM=0.                                                    
               TMM=0.                                                   
            ENDIF                                                       
            TTAU=TTM*(SIGM-TMM)                                         
            IF (L.LT.NL) TTAU=TTAU+TT01S2*(SIG-TM)                      
            TTAU=TTAU*RDSIGL*DSIGMA(M)                                  
            IF (M.LE.L) TTAU=TTAU+TTKP*C(M,L)                           
            TAU(M,L)=TTAU                                               
         ENDDO                                                          
      ENDDO                                                             
      FAC=0.001*CG/GA                                                   
      DO L=1,NL                                                         
         HL=0.                                                          
         DO M=1,NL                                                      
            HL=HL+G(M,L)*T0(M)                                          
         ENDDO                                                          
         H(L)=HL*FAC                                                    
      ENDDO                                                             
      DO L=1,NL                                                         
         DO M=1,NL                                                      
            TAQ=T0(L)*DSIGMA(M)                                         
            DO N=1,NL                                                   
               TAQ=TAQ+G(N,L)*TAU(M,N)                                  
            ENDDO                                                       
            AQ(M,L)=TAQ                                                 
            TBM1(M,L)=TAQ                                               
         ENDDO                                                          
      ENDDO                                                             
      CALL QREIG(TBM1,NL,NL,NL,CR,CI)                                   
      DO L=1,NL                                                         
         CR(L)=CV*SQRT(CR(L))                                           
      ENDDO                                                             
!-----------------------------------------------------                  
! Write out vertical information                                        
!-----------------------------------------------------                  
      IF (MYPE.EQ.0) THEN                                               
         WRITE(2,208)                                                   
         WRITE(2,217)(SIGMAH(L),L=1,NL-1)                               
         WRITE(2,214)                                                   
         WRITE(2,213)(SIGMA(L),L=1,NL)                                  
         WRITE(2,215)                                                   
         WRITE(2,216)(T0(L),L=1,NL)                                     
         WRITE(2,203)                                                   
         WRITE(2,216)(H(L),L=1,NL)                                      
         WRITE(2,206)                                                   
         WRITE(2,216)(CR(L),L=1,NL)                                     
         WRITE(2,*)                                                     
      ENDIF                                                             
!-----------------------------------------------------                  
! RGG matrix for vertical derivatives:                                  
! d()/dln(sigma) = (sigma)d()/d(sigma).                                 
!-----------------------------------------------------                  
      DO L=1,NL                                                         
         RGVAL=SIGMA(L)*RDSIG(L)                                        
         DO M=1,NL                                                      
            RGG(M,L)=0.                                                 
            IF (M.EQ.L-1) RGG(M,L)=-RGVAL                               
            IF (M.EQ.L+1) RGG(M,L)=RGVAL                                
         ENDDO                                                          
      ENDDO                                                             
      DA=SIGMA(1)                                                       
      DB=SIGMA(2)-DA                                                    
      DC=SIGMA(3)-SIGMA(2)                                              
      DD=DC+DB                                                          
      RGG(1,1)=-DA*(DD+DB)/(DB*DD)                                      
      RGG(2,1)=DA*DD/(DB*DC)                                            
      RGG(3,1)=-DA*DB/(DC*DD)                                           
      DA=SIGMA(NL)                                                      
      DD=SIGMA(NL-1)                                                    
      DB=DA-DD                                                          
      DC=DD-SIGMA(NL-2)                                                 
      DD=DC+DB                                                          
      RGG(NL,NL)=DA*(DD+DB)/(DB*DD)                                     
      RGG(NL-1,NL)=-DA*DD/(DB*DC)                                       
      RGG(NL-2,NL)=DA*DB/(DC*DD)                                        
!-----------------------------------------------------                  
! Setup arrays for semi-implicit scheme                                 
!-----------------------------------------------------                  
      DELTSQ=DELT*DELT                                                  
      DO IN=1,NN                                                        
         RCN=RSQ(IN+1)                                                  
         DO L=1,NL                                                      
            DO M=1,NL                                                   
               TBM1(M,L)=AQ(M,L)*DELTSQ                                 
               IF(M.EQ.L)TBM1(M,L)=TBM1(M,L)+RCN                        
            ENDDO                                                       
         ENDDO                                                          
         CALL MATINV(TBM1,NL,NL,IWA,WA)                                 
         !-----------------------------------------------------         
         ! Copy contents of TBM1 into BM1 at wavenumber IN              
         !-----------------------------------------------------         
         DO L=1,NL                                                      
            DO M=1,NL                                                   
               BM1(M,L,IN)=TBM1(M,L)                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
      SFAC=0.5**KITS                                                    
      IF(LRSTRT.AND..NOT.LSHORT)SFAC=1.0                                
      DELT=DELT*SFAC                                                    
      DELT2=DELT+DELT                                                   
      DELTSQ=DELT*DELT                                                  
      DO L=1,NL                                                         
         DO M=1,NL                                                      
            AQ(M,L)=AQ(M,L)*DELTSQ                                      
         ENDDO                                                          
      ENDDO                                                             
      TOUT1=0.                                                          
      TOUT2=0.                                                          
      DO L=1,NL                                                         
         T0L=T0(L)                                                      
         DSIG=DSIGMA(L)*T0(L)                                           
         TOUT1=TOUT1+DSIG                                               
         TOUT2=TOUT2+DSIG*T0L                                           
      ENDDO                                                             
      END                                                               
!=====================================================                  
! Sets up PHYSICS variables and arrays. Sets NAMELIST variables         
! to their default settings, then reads NAMELIST                        
! (Piers Forster, 23/10/96)                                             
!=====================================================                  
      SUBROUTINE INIPHYS                                                
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER :: NCNT                                                   
      REAL :: FW                                                        
      REAL :: FRADC                                                     
      REAL :: CLATNTI                                                   
      REAL :: SFAC                                                      
      REAL :: CDIF                                                      
      REAL :: FC                                                        
      REAL :: ZAIRMWG, ZCO2MWG, ZCH4MWG                                 
      REAL :: ZN2OMWG, ZC11MWG, ZC12MWG                                 
      REAL :: ZH2OMWG                                                   
!-----------------------------------------------------                  
! Common block for gas abundances                                       
!-----------------------------------------------------                  
      COMMON/MORCGAS/VMRCO2,ZCARDI,VMRCH4,RCH4,VMRN2O,RN2O,             
     &               VMRCFC11,RCFC11,VMRCFC12,RCFC12,                   
     &               VMRH2OS,RH2OS,NEXOTIC                              
      LOGICAL :: NEXOTIC                                                
      REAL :: ZCARDI,RCH4,RN2O,RCFC11,RCFC12,RH2OS                      
      REAL :: VMRCO2,VMRCH4,VMRN2O,VMRCFC11,VMRCFC12,VMRH2OS            
! Loop variables                                                        
      INTEGER :: IHEM, J, L                                             
!-----------------------------------------------------                  
! Data values                                                           
!-----------------------------------------------------                  
      DATA ZAIRMWG/28.970/                                              
      DATA ZCO2MWG/44.011/                                              
      DATA ZCH4MWG/16.043/                                              
      DATA ZN2OMWG/44.013/                                              
      DATA ZC11MWG/137.3686/                                            
      DATA ZC12MWG/120.9140/                                            
      DATA ZH2OMWG/18.015/                                              
!-----------------------------------------------------                  
! Physics namelist                                                      
!-----------------------------------------------------                  
      NAMELIST/INPHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                  
     &     ,CD,BLRH,AKVV,AKTV,AKQV,AKTC,AKQC,NLCR,NCUTOP,FW,FRADC       
     &     ,CURHM,CUBMT,CBADJT,CBADJP                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK,ITSLL,ITSLO               
      NAMELIST/INMORCGAS/VMRCO2,VMRCH4,VMRN2O,                          
     &              VMRCFC11,VMRCFC12,VMRH2OS,NEXOTIC                   
!-----------------------------------------------------                  
! Format statements                                                     
!-----------------------------------------------------                  
  240 FORMAT(' PHYSICAL PROCESSES INCLUDED ARE AS FOLLOWS:')            
  241 FORMAT(' SURFACE FLUXES : CD, RH-FACTOR =',F10.4,F10.1,'%')       
  242 FORMAT(' CONVECTION THROUGH',I4,' LAYERS FROM LOWEST LEVEL'/      
     :' DRY CONVECTION   : ADJUSTMENT TO NEUTRALITY IN SINGLE TIMESTEP')
  246 FORMAT(' PRECIPITATING CONVECTION : NON-ENTRAINING CLOUD MODEL'   
     :,' WITH ENVIRONMENTAL SDOT =',E7.1)                               
  247 FORMAT(' PRECIPITATING CONVECTION : ADJUSTMENT TO SUBSATURATED'   
     :,' MOIST ADIABAT WITH SCRIPT-P(MB) TAU(HOURS) =',2F6.1)           
  248 FORMAT(' NON-PRECIPITATING DIFFUSIVE MOIST CONVECTION WITH'       
     :,' BACKGROUND KT KQ =',2F6.1,' M2/S')                             
  249 FORMAT(' NON-PRECIPITATING BETTS-MILLER MOIST CONVECTION WITH'    
     :,' BACKGROUND TIMESCALE ',F6.1,' HOURS')                          
  250 FORMAT(' FORMALLY NON-PRECIPITATING CONVECTION INCLUDES RAINOUT'  
     :,' EXCESS OVER',F6.1,'% RH')                                      
  243 FORMAT(' LARGE-SCALE CONDENSATION TO SATURATION IN TIMESTEP')     
  244 FORMAT(' RADIATION: MORCRETTE CODE(SMR+PMF), DIAGNOSTIC CLOUD     
     : VERSION 2.0')                                                    
  245 FORMAT(' VERTICAL DIFFUSION OF MOMENTUM HEAT AND MOISTURE WITH',  
     :       ' CONSTANT COEFFS AKVV AKTV AKQV =',3F8.2)                 
!-----------------------------------------------------                  
! Preset namelist values.                                               
!-----------------------------------------------------                  
      LBL=.FALSE.                                                       
      LVD=.FALSE.                                                       
      LCR=.FALSE.                                                       
      LLR=.FALSE.                                                       
      LRD=.FALSE.                                                       
      LCUBM=.FALSE.                                                     
      LCBADJ=.FALSE.                                                    
      CD=0.001                                                          
      BLRH=100.0                                                        
      AKVV=1.0                                                          
      AKTV=1.0                                                          
      AKQV=1.0                                                          
      AKTC=10.0                                                         
      AKQC=10.0                                                         
      FW=1.0E-6                                                         
      FRADC=1.25                                                        
      CURHM=200.0                                                       
      CUBMT=3.0                                                         
      CBADJT=3.0                                                        
      CBADJP=-30.0                                                      
                                                                        
      LSL=.TRUE.                                                        
      LOC=.TRUE.                                                        
      LNOICE=.FALSE.                                                    
      LOLDBL=.FALSE.                                                    
      LCOND=.TRUE.                                                      
      LNNSK=.TRUE.                                                      
      ITSLL=12.                                                         
      ITSLO=12.                                                         
                                                                        
!-----------------------------------------------------                  
! PRESET DEFAULT GAS Vol. Mix. Ratios                                   
!                                                                       
! IPCC/SACC VALUES FOR 1990:                                            
!  VMRCO2: 353.93E-6, VMRCH4: 1.717E-6, VMRN2O: 309.68E-6               
!  VMRCFC11: 280.0E-12, VMRCFC12: 484.4E-12                             
!  VMRH2OS = stratospheric water vapour abundance                       
!          3-6E-6 (IPCC/TACC value for 1998)                            
!-----------------------------------------------------                  
      VMRCO2=353.9E-6                                                   
      VMRCH4=1.717E-06                                                  
      VMRN2O=309.7E-09                                                  
      VMRCFC11=280.0E-12                                                
      VMRCFC12=484.4E-12                                                
      VMRH2OS=3.0E-6                                                    
      NEXOTIC=.FALSE.                                                   
!-----------------------------------------------------                  
! Initialise parameters for physical processes                          
!-----------------------------------------------------                  
      NLCR=NL-1                                                         
      NCNT=0                                                            
      DO L=1,NL                                                         
         IF(SIGMA(L).GE.0.75) NCNT=NCNT+1                               
      ENDDO                                                             
      NCUTOP=NL+1-NCNT                                                  
!-----------------------------------------------------                  
! Read in namelist data and write out information                       
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         READ (7,INPHYS)                                                
         WRITE(2,INPHYS)                                                
         WRITE(2,240)                                                   
         IF(LBL) WRITE(2,241)CD,BLRH                                    
         IF(LCR) WRITE(2,242)NLCR                                       
         IF(LCR.AND..NOT.LCBADJ)WRITE(2,246)FW                          
         IF(LCR.AND.LCBADJ)     WRITE(2,247)CBADJP,CBADJT               
         IF(LCR.AND..NOT.LCUBM) WRITE(2,248)AKTC,AKQC                   
         IF(LCR.AND.LCUBM)      WRITE(2,249)CUBMT                       
         IF(LCR.AND.(CURHM.LT.100.01))WRITE(2,250)CURHM                 
         IF(LLR) WRITE(2,243)                                           
         IF(LRD) WRITE(2,244)                                           
         IF(LVD) WRITE(2,245)AKVV,AKTV,AKQV                             
      ENDIF                                                             
      IF (NPE.GT.1) THEN                                                
         !-----------------------------------------------------         
         ! Need to pass the data read in by PE 0 to the                 
         ! other PEs.                                                   
         !-----------------------------------------------------         
         iSend=0                                                        
         nBuffSize=1                                                    
         CALL MPI_BCAST(LBL,nBuffSize,ParaLog,iSend,                    
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LVD,nBuffSize,ParaLog,iSend,                    
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LCR,nBuffSize,ParaLog,iSend,                    
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LLR,nBuffSize,ParaLog,iSend,                    
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LRD,nBuffSize,ParaLog,iSend,                    
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LCUBM,nBuffSize,ParaLog,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LCBADJ,nBuffSize,ParaLog,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(CD,nBuffSize,ParaReal,iSend,                    
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(BLRH,nBuffSize,ParaReal,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(AKVV,nBuffSize,ParaReal,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(AKTV,nBuffSize,ParaReal,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(AKQV,nBuffSize,ParaReal,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(AKTC,nBuffSize,ParaReal,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(AKQC,nBuffSize,ParaReal,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(NLCR,nBuffSize,ParaInt,iSend,                   
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(NCUTOP,nBuffSize,ParaInt,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(FW,nBuffSize,ParaReal,iSend,                    
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(FRADC,nBuffSize,ParaReal,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(CURHM,nBuffSize,ParaReal,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(CUBMT,nBuffSize,ParaReal,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(CBADJT,nBuffSize,ParaReal,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(CBADJP,nBuffSize,ParaReal,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LSL,nBuffSize,ParaLog,iSend,                    
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LOC,nBuffSize,ParaLog,iSend,                    
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LNOICE,nBuffSize,ParaLog,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LOLDBL,nBuffSize,ParaLog,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LCOND,nBuffSize,ParaLog,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LNNSK,nBuffSize,ParaLog,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(ITSLL,nBuffSize,ParaInt,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(ITSLO,nBuffSize,ParaInt,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
      ENDIF                                                             
!-----------------------------------------------------                  
!                                                                       
!-----------------------------------------------------                  
      CLATNTI=2.834E6                                                   
      SFAC=0.5**KITS                                                    
      IF(LRSTRT.AND..NOT.LSHORT) SFAC=1.0                               
      CTQ=CLATNT/(CPD*CT)                                               
      CTQI=CLATNTI/(CPD*CT)                                             
      if (LNOICE) CTQI=CTQ                                              
      CCC=CLATNT*CLATNT/(RV*CPD*CT*CT)                                  
      ESCONB=CLATNT/RV                                                  
      ESCONA=RD*EXP(ESCONB/273.15)*610.7/(RV*P0)                        
      ESCONB=ESCONB/CT                                                  
      EPSIQ=0.01/CQ                                                     
! Used to be multiplied by old (constant) CD. Now done in BLAYER        
      DRAG=GA*RADEA/(RD*CT*DSIGMA(NL))                                  
      BLVAD=3.0/CV                                                      
      BLA=500.0/CV ! A=500m/s                                           
      BLRH=BLRH/100.0                                                   
      FW=FW/WW                                                          
      DTBUOY=1.0E-5/CT                                                  
      TSLA=55.0/CT                                                      
      TSLB=2840.0/CT                                                    
      TSLC=3.5                                                          
      TSLD=3.5*LOG(CT)-LOG(P0)-0.67485                                  
      CBADJT=CBADJT*3600.0*WW                                           
      CBADJP=CBADJP*100.0/P0                                            
      CURHM=CURHM/100.0                                                 
      CUBMT=CUBMT*3600.0*WW                                             
C     Used to be divided by old CD, now done in CUDIF and CUBM          
      CUT1=1.0E4*DSIGMA(NL)/(RADEA*0.2)                                 
      CUT2=1.0E4*DSIGMA(NL-1)/RADEA                                     
      CUT2=CUT2*CUT2*0.5                                                
      AKTC=AKTC/(RADEA*CV)                                              
      AKQC=AKQC/(RADEA*CV)                                              
      CDIF=GA/(CV*WW)                                                   
      CDIF=CDIF*CDIF                                                    
      RCON=P0*DELT/(SFAC*GA)                                            
      CCR=RCON*FW                                                       
!-----------------------------------------------------                  
! Initialise values for the soil model                                  
!-----------------------------------------------------                  
      sdsnd=300.             ! density of snow (kg/m3) dimensional      
      sdsn=sdsnd*(RD*CT)/P0  ! density of snow dedimensionalised        
      sdw=1000.*(RD*CT)/P0   ! density of water (kg/m3)                 
      shcs=2085.0E3*CT/P0    ! rho*c for 50% saturated soil             
      shcsp=15.312E6*CT/P0   ! equ. heat cap. for freezing of soil water
      shcsn=627.0E3*CT/P0    ! rho*c for snow                           
      skse=1.072818*CT/(RADEA*CV*P0)  ! effective soil conductivity     
      sksn=0.24*CT/(RADEA*CV*P0)  ! thermal conductivity for snow       
      slhf=3.5E5/(RD*CT)     ! latent heat of fusion of ice             
      sd1=0.06/RADEA         ! depth of upper soil level                
      sd2=2.2999/RADEA       ! depth of lower soil level                
      ssmc=0.5/RADEA         ! depth equivalent to capacity of model    
!-----------------------------------------------------                  
! Albedos for snow and sea ice, depth for snow albedo function          
!-----------------------------------------------------                  
      sasnow=0.80  ! albedo of snow over ice (land snow set in SNOLOOK) 
      saice=0.60  ! ice albedo                                          
      if(mype.eq.0)then                                                 
      print*,'***********************************'                      
      print*,'*                                 *'                      
      print*,'*  DOUBLE CHECK SEA ICE IS NOT    *'                      
      print*,'*          OVERWRITTEN            *'                      
      print*,'*          IN JOB DECK            *'                      
      print*,'*                                 *'                      
      print*,'*                                 *'                      
      print*,'***********************************'                      
      endif                                                             
      shsstar=0.30/RADEA  ! depth of snow for albedo transfer function  
!-----------------------------------------------------                  
! max depth of snow for heat capacity of top soil layer                 
! Note - equal to e-folding depth/sqrt(2)                               
!-----------------------------------------------------                  
      SHSMAX=4.0/RADEA                                                  
!-----------------------------------------------------                  
! heat capacity of ocean mixed layer. Mixed layer depth 25m.            
!-----------------------------------------------------                  
C Thickness and heat capacity for MOMA ocean top layer                  
      shco=30.*1000.*4190.*CT/P0/RADEA                                  
C Thickness and heat capacity for ice at -2C; double this at TICESNOW   
      shci=1.0*900.*2100.*CT/P0/RADEA                                   
      DO L=1,NL-1                                                       
        CLR(L)=RCON*DSIGMA(L)                                           
        FWS(L) =-FW*RDSIG(L)                                            
        FB(L)=CDIF*4.0*SIGMAH(L)*SIGMAH(L)/(SIGMA(L+1)-SIGMA(L))        
        SKAP(L)=SIGMA(L)**AKAP                                          
        SK(L)  =(SIGMA(L)/SIGMA(L+1))**AKAP                             
      ENDDO                                                             
      CLR(NL)=RCON*DSIGMA(NL)                                           
      FWS(NL)=-FW*RDSIG(NL)                                             
      SKAP(NL)=SIGMA(NL)**AKAP                                          
!-----------------------------------------------------                  
! Cooling rate set to be identical at all latitudes.                    
!-----------------------------------------------------                  
      FC=FRADC/(PI2*CT)                                                 
      DO IHEM=1,NHEM                                                    
         DO J=1,JGyy(iRow)                                              
            FRAD(J,IHEM)=-FC                                            
         ENDDO                                                          
      ENDDO                                                             
      AKVV=AKVV/(RADEA*CV)                                              
      AKTV=AKTV/(RADEA*CV)                                              
      AKQV=AKQV/(RADEA*CV)                                              
      NAVRD=20                                                          
      NAVWT=21                                                          
!-----------------------------------------------------                  
! READ NAMELIST INMORCGAS AND CONVERT VMR->MMR                          
! M. Bourqui Nov. 2001                                                  
! Read in namelist                                                      
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         READ (7,INMORCGAS)                                             
         WRITE(2,INMORCGAS)                                             
      ENDIF                                                             
      IF (NPE.GT.1) THEN                                                
         !-----------------------------------------------------         
         ! Pass informations from PE 0 to other PEs                     
         !-----------------------------------------------------         
         iSend=0                                                        
         nBuffSize=1                                                    
         CALL MPI_BCAST(VMRCO2,nBuffSize,ParaReal,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(VMRCH4,nBuffSize,ParaReal,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(VMRN2O,nBuffSize,ParaReal,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(VMRCFC11,nBuffSize,ParaReal,iSend,              
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(VMRCFC12,nBuffSize,ParaReal,iSend,              
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(VMRH2OS,nBuffSize,ParaReal,iSend,               
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(NEXOTIC,nBuffSize,ParaInt,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
      ENDIF                                                             
                                                                        
      ZCARDI  = VMRCO2*ZCO2MWG/ZAIRMWG                                  
      RCH4    = VMRCH4*ZCH4MWG/ZAIRMWG                                  
      RN2O    = VMRN2O*ZN2OMWG/ZAIRMWG                                  
      RCFC11  = VMRCFC11*ZC11MWG/ZAIRMWG                                
      RCFC12  = VMRCFC12*ZC12MWG/ZAIRMWG                                
      RH2OS   = VMRH2OS*ZH2OMWG/ZAIRMWG                                 
      RETURN                                                            
      END                                                               
!=====================================================                  
! Sets up restoration variables and arrays. Sets NAMELIST               
! variables to their default settings, then reads NAMELIST.             
! Sets up Rayleigh friction coefficients.                               
!=====================================================                  
      SUBROUTINE INIRESIJ                                               
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Restoration temperature field and constants which determine it,       
! also contains timescales                                              
!-----------------------------------------------------                  
      COMMON/RESTIJ/TTRES                                               
     + ,DTNS,DTEP,DTTRP,FAC,DDAMP,TFRC,YRLEN,TRS                        
     +  ,ALR,ZTROP,TGR                                                  
      COMPLEX :: TTRES(NNTx,MMTy,NHEM,NL)                               
      REAL :: DTNS                                                      
      REAL :: DTEP                                                      
      REAL :: DTTRP                                                     
      REAL :: FAC(NL)                                                   
      REAL :: DDAMP(NL)                                                 
      REAL :: TFRC(NL)                                                  
      REAL :: YRLEN                                                     
      REAL :: TRS(NL)                                                   
      REAL :: ALR                                                       
      REAL :: ZTROP                                                     
      REAL :: TGR                                                       
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: RESTIM(NL)                                                
      REAL :: TTROP                                                     
      REAL :: THING                                                     
      REAL :: STPS                                                      
! Loop variable                                                         
      INTEGER :: L                                                      
!-----------------------------------------------------                  
! Namelist                                                              
!-----------------------------------------------------                  
      NAMELIST/INPRSIJ/ TFRC,RESTIM,DTNS,DTEP,ALR,                      
     &                  DTTRP,ZTROP,TGR,YRLEN                           
                                                                        
      TFRC(NL)=1.                                                       
      DO L=1,NL-1                                                       
         TFRC(L) = 0.                                                   
      ENDDO                                                             
      DO L=1,NL                                                         
         RESTIM(L) = 15.0                                               
      ENDDO                                                             
      DTNS=0.                                                           
      DTEP=60.                                                          
      ALR=6.5E-03                                                       
      DTTRP=2.                                                          
      ZTROP=12.0E03                                                     
      TGR=288.                                                          
      YRLEN=0.                                                          
!-----------------------------------------------------                  
! Read in namelist on PE 0                                              
!-----------------------------------------------------                  
      IF (MYPE.EQ.0) THEN                                               
         READ(7,INPRSIJ)                                                
         WRITE(2,INPRSIJ)                                               
      ENDIF                                                             
!-----------------------------------------------------                  
! Pass information from namelist to other PEs                           
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         CALL MPI_BARRIER(MPI_COMM_WORLD,IERR)                          
         nBuffSize=NL                                                   
         iSend=0                                                        
         CALL MPI_BCAST(TFRC,nBuffSize,ParaReal,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(RESTIM,nBuffSize,ParaReal,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         nBuffSize=1                                                    
         CALL MPI_BCAST(DTNS,nBuffSize,ParaReal,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(DTEP,nBuffSize,ParaReal,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(ALR,nBuffSize,ParaReal,iSend,                   
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(DTTRP,nBuffSize,ParaReal,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(ZTROP,nBuffSize,ParaReal,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(TGR,nBuffSize,ParaReal,iSend,                   
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(YRLEN,nBuffSize,ParaReal,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
      ENDIF                                                             
!-----------------------------------------------------                  
! Dimensionless coefficient for Newtonian cooling friction              
! and timestep. A day is 2*pi in non dimensional                        
! units using omega as the unit of frquency.                            
!-----------------------------------------------------                  
      DO L=1,NL                                                         
         IF (RESTIM(L).GT.0.0) THEN                                     
            DDAMP(L)=1.0/(PI2*RESTIM(L))                                
         ELSE                                                           
            DDAMP(L)=0.0                                                
         ENDIF                                                          
         IF (TFRC(L).GT.0.0) THEN                                       
            TFRC(L)=1.0/(PI2*TFRC(L))                                   
         ELSE                                                           
            TFRC(L)=0.0                                                 
         ENDIF                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Make temperatures dimensionless                                       
!-----------------------------------------------------                  
      DTNS=DTNS/CT                                                      
      DTEP=DTEP/CT                                                      
      DTTRP=DTTRP/CT                                                    
!-----------------------------------------------------                  
! Loop to set array FAC - this controls temperature gradients           
! as a function of SIGMA in TTRES. It is a sine wave from one           
! at SIGMA = 1 to zero at STPS (SIGMA at the tropopause).               
!                                                                       
! First find SIGMA at ZTROP                                             
!-----------------------------------------------------                  
      TTROP = TGR - ZTROP*ALR                                           
      STPS = (TTROP/TGR)**(GA/(ALR*GASCON))                             
      DO L=1,NL                                                         
         THING=SIN(0.5*PI*(SIGMA(L)-STPS)/(1.-STPS))                    
         IF (THING.LT.0.) THEN                                          
            FAC(L)=0.                                                   
         ELSE                                                           
            FAC(L)=THING                                                
         ENDIF                                                          
      ENDDO                                                             
      END                                                               
!=====================================================                  
! Sets up restoration variables and arrays. Sets NAMELIST               
! variables to their default settings, then reads NAMELIST              
!=====================================================                  
      SUBROUTINE INIRES                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Restoration fields and timescale                                      
!-----------------------------------------------------                  
      COMMON/RESTOR/ZRES,DRES,TRES,SPRES,DAMP                           
      REAL :: ZRES(IDM,NHEM,NL)                                         
      REAL :: DRES(IDM,NHEM,NL)                                         
      REAL :: TRES(IDM,NHEM,NL)                                         
      REAL :: SPRES(IDM,NHEM)                                           
      REAL :: DAMP                                                      
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: RESTIM                                                    
!-----------------------------------------------------                  
! Namelist                                                              
!-----------------------------------------------------                  
      NAMELIST/INPRS/ RESTIM                                            
                                                                        
      RESTIM=0.0                                                        
                                                                        
      IF (MYPE.EQ.0) READ(7,INPRS)                                      
      IF (NPE.GT.1) THEN                                                
         CALL MPI_BARRIER(MPI_COMM_WORLD,IERR)                          
         nBuffSize=1                                                    
         iSend=0                                                        
         CALL MPI_BCAST(RESTIM,nBuffSize,ParaReal,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
      ENDIF                                                             
      IF (MYPE.EQ.0) WRITE(2,INPRS)                                     
!-----------------------------------------------------                  
! Dimensionless coefficient for Newtonian cooling friction              
! and timestep. A day is 2*pi in non dimensional                        
! units using omega as the unit of frequency.                           
!-----------------------------------------------------                  
      IF (RESTIM.GT.0.0) THEN                                           
         DAMP=1.0/(PI2*RESTIM)                                          
      ELSE                                                              
         DAMP=0.0                                                       
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Reads in data for a start/restart run                                 
!=====================================================                  
      SUBROUTINE INISTR                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Switches counters and constants controlling type and frequency of     
! model output                                                          
!-----------------------------------------------------                  
      COMMON/OUTCON/RNTAPE,DAY,SQR2,RSQR2,EAM1,EAM2,TOUT1               
     &     ,TOUT2,LSPO,LGPO                                             
     &     ,RNTAPO                                                      
     &     ,LSHIST,LMINIH                                               
      COMMON/OUTCONINT/NCOEFF,NLAT,INLAT,INSPC,KOUNTP                   
     &                 ,KOUNTE,KOUNTH,KOUNTR,KOUTP                      
     &                 ,KOUTE,KOUTH,KOUTR                               
! Identyfying run number                                                
      REAL :: RNTAPE                                                    
! Maximum wavenumber of printed spectral coeffecients                   
      INTEGER :: NCOEFF                                                 
! Number of latitudes in printed grid point fields                      
      INTEGER :: NLAT                                                   
      INTEGER :: INLAT                                                  
!                                                                       
      INTEGER :: INSPC                                                  
! Number of timsteps between successive writing of spectral             
! and gridded fields                                                    
      INTEGER :: KOUNTP                                                 
      INTEGER :: KOUNTE                                                 
! Number of timsteps between successive writing of history              
! records, containing spectral fields at the present and                
! previous time levels.                                                 
      INTEGER :: KOUNTH                                                 
! Number of timsteps between successive writing of restart              
! records, containing the spectral fields at the present and            
! previous time levels                                                  
      INTEGER :: KOUNTR                                                 
      INTEGER :: KOUTP                                                  
      INTEGER :: KOUTE                                                  
      INTEGER :: KOUTH                                                  
      INTEGER :: KOUTR                                                  
      REAL :: DAY                                                       
      REAL :: SQR2                                                      
      REAL :: RSQR2                                                     
      REAL :: EAM1                                                      
      REAL :: EAM2                                                      
      REAL :: TOUT1                                                     
      REAL :: TOUT2                                                     
      LOGICAL :: LSPO(NL)                                               
      LOGICAL :: LGPO(NL)                                               
      REAL :: RNTAPO                                                    
      LOGICAL :: LSHIST                                                 
! Reduce the history file to single precision if true                   
      LOGICAL :: LMINIH                                                 
!-----------------------------------------------------                  
! Restoration fields and timescale                                      
!-----------------------------------------------------                  
      COMMON/RESTOR/ZRES,DRES,TRES,SPRES,DAMP                           
      REAL :: ZRES(IDM,NHEM,NL)                                         
      REAL :: DRES(IDM,NHEM,NL)                                         
      REAL :: TRES(IDM,NHEM,NL)                                         
      REAL :: SPRES(IDM,NHEM)                                           
      REAL :: DAMP                                                      
!-----------------------------------------------------                  
! Restoration temperature field and constants which determine it,       
! also contains timescales                                              
!-----------------------------------------------------                  
      COMMON/RESTIJ/TTRES                                               
     + ,DTNS,DTEP,DTTRP,FAC,DDAMP,TFRC,YRLEN,TRS                        
     +  ,ALR,ZTROP,TGR                                                  
      COMPLEX :: TTRES(NNTx,MMTy,NHEM,NL)                               
      REAL :: DTNS                                                      
      REAL :: DTEP                                                      
      REAL :: DTTRP                                                     
      REAL :: FAC(NL)                                                   
      REAL :: DDAMP(NL)                                                 
      REAL :: TFRC(NL)                                                  
      REAL :: YRLEN                                                     
      REAL :: TRS(NL)                                                   
      REAL :: ALR                                                       
      REAL :: ZTROP                                                     
      REAL :: TGR                                                       
!-----------------------------------------------------                  
!                                                                       
!-----------------------------------------------------                  
      COMMON/STATS/GMSP0,GMSPMI,LMASCOR,LMASOLD,LMASPRT                 
      LOGICAL :: LMASCOR,LMASOLD,LMASPRT                                
      REAL :: GMSP0,GMSPMI                                              
!-----------------------------------------------------                  
! Constants and arrays needed for balancing                             
!-----------------------------------------------------                  
      COMMON/BALAN/BFILT,RGT0,RG,TMEAN                                  
     +            ,EP1,EP2,KBAL,SRGT0,LTBAL                             
     &            ,PMNRE                                                
! Maximum zonal wavenumbers used for balancing                          
      INTEGER, PARAMETER :: MFTBAL=9                                    
! Number of zonal wavenumbers used for balancing                        
      INTEGER, PARAMETER :: NFTBAL=1+MFTBAL/MOCT                        
      REAL :: BFILT(NL)                                                 
      REAL :: RGT0(NL)                                                  
      REAL :: RG(NL,NL)                                                 
      REAL :: TMEAN(NL)                                                 
      REAL :: EP1(IDM,NFTBAL,NHEM)                                      
      REAL :: EP2(IDM,NFTBAL,NHEM)                                      
      INTEGER :: KBAL                                                   
      REAL :: SRGT0                                                     
      LOGICAL :: LTBAL                                                  
      REAL :: PMNRE(IDM,IDM,3)                                          
!-----------------------------------------------------                  
! Orogrophy                                                             
!-----------------------------------------------------                  
      COMMON/GSG/GSG,TOPVAR,RLEVBZMO,FXZMO,ofrac,runoff,iareanum        
! Topog variance, GW diags for Orog, Non orog waves                     
      REAL :: TOPVAR(MGx,NHEM,JGy)                                      
      REAL :: RLEVBZMO(JGy,NHEM,NL)                                     
      REAL :: FXZMO(JGy,NHEM,NL)                                        
         REAL :: GSG(MGx,NHEM,JGy)                                      
         REAL :: ofrac(MGx,NHEM,JGy)                                    
         INTEGER :: runoff(50,2,33)                                     
         INTEGER :: iareanum                                            
!-----------------------------------------------------                  
! Radiation variables                                                   
!-----------------------------------------------------                  
      COMMON /RADHT/HTNET,TAVE                                          
      REAL :: HTNET(NHEM,JGy,MGx,NL)                                    
      COMPLEX :: TAVE(NNTx,MMTy,NHEM,NL)                                
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
! File unit for writing data for a later restart run                    
      INTEGER :: ID                                                     
      REAL :: DAYNEAR                                                   
      REAL :: RM1TAPE, RM2TAPE, RM3TAPE                                 
! RKOUNT is an integer, but output expects a REAL                       
      REAL :: RKOUNT                                                    
      INTEGER :: KTEMP                                                  
      REAL :: HoldVar(NNTx,NHEM,3*NL+1)                                 
      REAL, ALLOCATABLE :: AllRes(:,:,:)                                
      INTEGER, PARAMETER :: TagRef=100                                  
      INTEGER :: iCol2                                                  
      REAL :: ADUM                                                      
      COMPLEX, ALLOCATABLE :: GSFull(:,:)                               
      REAL, ALLOCATABLE :: OFRACFULL(:,:,:)                             
      REAL, ALLOCATABLE :: GSGFull(:,:,:)                               
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, IN, J, KK, L                              
      REAL :: GSFAC                                                     
      REAL :: TMEANJ(NL)                                                
       DATA TMEANJ /220.0,213.1,209.0,209.2,212.8,217.6,                
     $ 222.2,226.8,231.8,237.3,243.5,249.9,256.3,                       
     $ 262.3,268.2,273.9,278.8,282.5,285.2,287.5/                       
      INTEGER :: IAREA, NCOAST                                          
!-----------------------------------------------------                  
! Format statments                                                      
!-----------------------------------------------------                  
 2000 FORMAT(' ***WARNING*** KITS < 1 FOR AN INITIAL RUN.'/)            
 2030 FORMAT(/' ***ABORT*** CANNOT FIND THE CORRECT HISTORY RECORD.'/   
     & ' LOOKING FOR DAY',F8.2/,' BUT THE NEAREST RECORD FOUND',        
     & ' IS FOR DAY',F8.2)                                              
 2040 FORMAT(/' HISTORY RECORD READ FROM CHANNEL ',I3,/                 
     & ' KOUNT  RMTAPE  DAY =',I8,2F12.3)                               
 2011 FORMAT(/' ***ABORT*** THE RESTART RECORDS READ FROM CHANNEL '     
     & ,I3,/' ARE NOT IN CORRECT FORMAT ')                              
 2021 FORMAT(/' ***ABORT*** THE RUN NUMBER IN THE RESTART RECORD ',     
     &     F10.3 /'DIFFERS FROM RNTAPO IN THE NAMELIST ',F10.3)         
 2041 FORMAT(/' RESTART RECORD READ FROM CHANNEL ',I3,/                 
     & ' KOUNT  RMTAPE  DAY =',I8,2F12.3)                               
 2012 FORMAT(/' ***ABORT*** THE RESTORATION RECORDS READ FROM CHANNEL ' 
     & ,I3,/' ARE NOT IN CORRECT FORMAT ')                              
 2022 FORMAT(/' ***ABORT*** THE RUN NUMBER IN THE RESTORATION RECORD'/  
     & ' IS NOT THE SAME AS RNTAPE ENTERED IN NAMELIST')                
 2042 FORMAT(' RESTORATION RECORD READ FROM CHANNEL ',I3,/              
     & ' KOUNT  RMTAPE  DAY =',I8,2F12.3)                               
 2050 FORMAT(/' SPECTRAL ARRAYS ARE SET TO ZERO ')                      
 2100 FORMAT(/' MASS INFORMATION READ AT RESTART:'                      
     &       /'    INITIAL (REFERENCE) MASS     = ',1PE20.12,' (PA)'    
     &       /'    TIME-LAGGED MASS             = ',1PE20.12,' (PA)')   
 2130 FORMAT(/' ***ABORT: EOF READING MASS RESTART RECORD AT DAY ',F8.2)
!-----------------------------------------------------                  
! Initialise spectral arrays to zero and overwrite as desired           
!-----------------------------------------------------                  
      SP(:,:,:)=(0.0,0.0)                                               
      SPMI(:,:,:)=(0.,0.)                                               
      GS(:,:,:)=(0.0,0.0)                                               
      Z(:,:,:,:)=(0.0,0.0)                                              
      D(:,:,:,:)=(0.0,0.0)                                              
      T(:,:,:,:)=(0.0,0.0)                                              
      ZMI(:,:,:,:)=(0.0,0.0)                                            
      DMI(:,:,:,:)=(0.0,0.0)                                            
      TMI(:,:,:,:)=(0.0,0.0)                                            
!-----------------------------------------------------                  
! Zero HTNET array                                                      
!-----------------------------------------------------                  
      HTNET(:,:,:,:)=0.0                                                
!-----------------------------------------------------                  
! Initialise current and global reference mass to zero (unset).         
!-----------------------------------------------------                  
      GMSP0=0.0                                                         
      GMSPMI=0.0                                                        
!-----------------------------------------------------                  
! Determine if this is a restart run                                    
!-----------------------------------------------------                  
      IF (.NOT.LRSTRT) THEN                                             
         !-----------------------------------------------------         
         ! Initial run                                                  
         !-----------------------------------------------------         
         IF ( (KITS.LT.1).AND.(MYPE.EQ.0) ) WRITE(2,2000)               
         DAY=0.0                                                        
         IF (KITS.EQ.0) THEN                                            
            KTOTAL=KRUN                                                 
            KOUTP=0                                                     
            KOUTE=0                                                     
            KOUTH=0                                                     
            KOUTR=0                                                     
         ELSE                                                           
            KTOTAL=KRUN+KITS-1                                          
            KOUTP=1-KITS                                                
            KOUTE=1-KITS                                                
            KOUTH=1-KITS                                                
            KOUTR=1-KITS                                                
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Initialise restoration array                                 
         !-----------------------------------------------------         
         IF (LRESTIJ) CALL SETZT                                        
         !-----------------------------------------------------         
         ! Initialise spectral arrays                                   
         !-----------------------------------------------------         
         IF (LBALAN) THEN                                               
            CALL INIBAL                                                 
            KOUNT=-KBAL                                                 
            KSTART=0                                                    
         ELSE                                                           
            IF (LRESTIJ) THEN                                           
               CALL INISP                                               
            ELSE                                                        
               IF (MYPE.EQ.0) WRITE (2,2050)                            
            ENDIF                                                       
            KOUNT=0                                                     
            KSTART=KOUNT                                                
         ENDIF                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! Code for restart and normal mode perturbation runs.          
         ! assume spectral data is set up non-dimensionalised           
         ! on a history (LSHORT) or restart (.NOT.LSHORT) record.       
         !-----------------------------------------------------         
         ID=10                                                          
         DAYNEAR=-9999.5 ! allows desired day no. = 0                   
         IF (LSHORT) THEN                                               
            !-----------------------------------------------------      
            ! Only read in data through PE 0                            
            !-----------------------------------------------------      
            IF (ABS(BEGDAY) .LT. 1.0E-02) THEN                          
               IF (Mype.EQ.0) OPEN(UNIT=ID,FILE='input.dat',            
     &              FORM='formatted')                                   
               CALL READCOEF(ID, 1, .FALSE., DAYNEAR,                   
     &              RKOUNT, RM1TAPE, DAY)                               
            ELSE                                                        
               IF (Mype.EQ.0) OPEN(UNIT=ID,FILE='input.dat',            
     &              FORM='unformatted')                                 
               CALL READCOEF(ID, 1, .TRUE., DAYNEAR,                    
     &              RKOUNT, RM1TAPE, DAY)                               
            ENDIF                                                       
            !-----------------------------------------------------      
            ! Set the count values                                      
            !-----------------------------------------------------      
            KOUNT=NINT(RKOUNT)                                          
            IF (MYPE.EQ.0) WRITE(2,2040) ID,KOUNT,RM1TAPE,BEGDAY        
            KOUNT=0                                                     
            IF (KITS.EQ.0) THEN                                         
               KTOTAL=KRUN                                              
               KOUTP=0                                                  
               KOUTE=0                                                  
               KOUTH=0                                                  
               KOUTR=0                                                  
            ELSE                                                        
               KTOTAL=KRUN+KITS-1                                       
               KOUTP=1-KITS                                             
               KOUTE=1-KITS                                             
               KOUTH=1-KITS                                             
               KOUTR=1-KITS                                             
            ENDIF                                                       
            KSTART=KOUNT                                                
            !-----------------------------------------------------      
            ! Set the old value of vorticty, surface pressure etc       
            ! to the current values                                     
            !-----------------------------------------------------      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMTyy(iRow)                                      
                  DO IN=1,NNTxx(iCol)                                   
                     SPMI(IN,IM,IHEM)=SP(IN,IM,IHEM)                    
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
            DO L=1,NL                                                   
               DO IHEM=1,NHEM                                           
                  DO IM=1,MMTyy(iRow)                                   
                     DO IN=1,NNTxx(iCol)                                
                        ZMI(IN,IM,IHEM,L)=Z(IN,IM,IHEM,L)               
                        DMI(IN,IM,IHEM,L)=D(IN,IM,IHEM,L)               
                        TMI(IN,IM,IHEM,L)=T(IN,IM,IHEM,L)               
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
            DO KK=1,NTRAC                                               
               DO L=1,NL                                                
                  DO IHEM=1,NHEM                                        
                     DO IM=1,MMTyy(iRow)                                
                        DO IN=1,NNTxx(iCol)                             
                           TRAMI(IN,IM,IHEM,L,KK)=TRA(IN,IM,IHEM,L,KK)  
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ELSE                                                           
            !-----------------------------------------------------      
            ! Only read in data through PE 0                            
            !-----------------------------------------------------      
            IF (ABS(BEGDAY) .LT. 1.0E-02) THEN                          
               IF (Mype.EQ.0) OPEN(UNIT=ID,FILE='input.dat',            
     &              FORM='formatted')                                   
               CALL READCOEF(ID, 35, .FALSE., DAYNEAR,                  
     &              RKOUNT, RM1TAPE, DAY)                               
            ELSE                                                        
               IF (Mype.EQ.0) OPEN(UNIT=ID,FILE='input.dat',            
     &              FORM='unformatted')                                 
               CALL READCOEF(ID, 35, .TRUE., DAYNEAR,                   
     &              RKOUNT, RM1TAPE, DAY)                               
            ENDIF                                                       
            !-----------------------------------------------------      
            ! Set the count values                                      
            !-----------------------------------------------------      
            KOUNT=NINT(RKOUNT)                                          
            IF (MYPE.EQ.0) WRITE(2,2041) ID,KOUNT,RM1TAPE,BEGDAY        
            KTOTAL=KOUNT+KRUN                                           
            KSTART=KOUNT                                                
            KTEMP=KOUNT                                                 
            IF(KITS.GT.0) KTEMP=KOUNT+1-KITS                            
            KOUTP=KTEMP-KOUNTP*(KTEMP/KOUNTP)                           
            KOUTE=KTEMP-KOUNTE*(KTEMP/KOUNTE)                           
            KOUTH=KTEMP-KOUNTH*(KTEMP/KOUNTH)                           
            KOUTR=KTEMP-KOUNTR*(KTEMP/KOUNTR)                           
            IF ((KTEMP.GT.0).AND.(KOUTH.EQ.0)) THEN                     
               KOUTH=KOUNTH                                             
            ENDIF                                                       
            !-----------------------------------------------------      
            ! Write values to output file                               
            !-----------------------------------------------------      
            IF (LMASOLD) THEN                                           
               IF (MYPE.EQ.0) WRITE(2,2100) GMSP0*P0,GMSPMI*P0          
            ENDIF                                                       
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Close the input file                                         
         !-----------------------------------------------------         
         IF (MyPe.EQ.0) CLOSE(ID)                                       
         !-----------------------------------------------------         
         ! Read in restoration state from separate file                 
         !-----------------------------------------------------         
         IF (LRESTIJ) THEN                                              
            OPEN(UNIT=13,FILE='restoration.dat',FORM='formatted')       
            CALL READCOEF(13, 4, .FALSE., DAYNEAR,                      
     &           RKOUNT, RM1TAPE, DAY)                                  
            CLOSE(13)                                                   
         ELSE                                                           
            IF(DAMP.GT.0.0) THEN                                        
               IF (iRow.EQ.1) THEN                                      
                  IF (MYPE.EQ.0) THEN                                   
                     !------------------------------------------------  
                     ! Read in restoration data                         
                     !------------------------------------------------  
                     ALLOCATE(AllRes(NNT,NHEM,3*NL+1))                  
                     OPEN(UNIT=13,FILE='restoration.dat',               
     &                 FORM='unformatted')                              
                     READ(13) AllRes                                    
                     CLOSE(13)                                          
                     !------------------------------------------------  
                     ! Pass data to the other PEs on row 1              
                     !------------------------------------------------  
                     DO iCol2=1,Px                                      
                        DO L=1,3*NL+1                                   
                           DO IHEM=1,NHEM                               
                              DO J=mTwin1(1,1)+1,mTwin1(2,1)            
                                 HoldVar(J,IHEM,L)=                     
     &                                AllRes(NNT1(iCol2)+J,IHEM,L)      
                              ENDDO                                     
                           ENDDO                                        
                        ENDDO                                           
                        !---------------------------------------------  
                        ! Send data need by other PEs                   
                        !---------------------------------------------  
                        IF (iCol2.GT.1) THEN                            
                           iRecv=PeGrid(iCol2,1)                        
                           nBuffSize=NNTx*NHEM*(3*NL+1)                 
                           iTag=TagRef+iRecv                            
                           CALL MPI_BSEND(HoldVar,nBuffSize,ParaReal,   
     &                          iRecv,iTag,MPI_COMM_WORLD,ierr)         
                        ENDIF                                           
                     ENDDO                                              
                  ELSE                                                  
                     !------------------------------------------------  
                     ! Receive the data                                 
                     !------------------------------------------------  
                     iSend=0                                            
                     nBuffSize=NNTx*NHEM*(3*NL+1)                       
                     iTag=TagRef+MyPe                                   
                     CALL MPI_RECV(HoldVar,nBuffSize,ParaReal,iSend,    
     &                    iTag,MPI_COMM_WORLD,istat,ierr)               
                  ENDIF                                                 
                  !---------------------------------------------------  
                  ! Write all the data in ZRES etc                      
                  !---------------------------------------------------  
                  DO L=1,NL                                             
                     DO IHEM=1,NHEM                                     
                        DO J=mTwin1(1,1)+1,mTwin1(2,1)                  
                           ZRES(J,IHEM,L)=HoldVar(J,IHEM,L)             
                           DRES(J,IHEM,L)=HoldVar(J,IHEM,NL+L)          
                           TRES(J,IHEM,L)=HoldVar(J,IHEM,2*NL+L)        
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
                  DO IHEM=1,NHEM                                        
                     DO J=mTwin1(1,1)+1,mTwin1(2,1)                     
                        SPRES(J,IHEM)=HoldVar(J,IHEM,3*NL+1)            
                     ENDDO                                              
                  ENDDO                                                 
               ENDIF                                                    
            ENDIF                                                       
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Initialise any new tracer fields and surface state.          
         !-----------------------------------------------------         
         CALL ICTRAC                                                    
      ENDIF                                                             
!-----------------------------------------------------                  
C Read ocean fraction file                                              
C MUST BE READ AS SAME AS OROG                                          
                                                                        
      if(mype .eq. 0)then                                               
                                                                        
      ALLOCATE(ofracfull(MG,NHEM,JG))                                   
      open(50,file='orogdata/ofrac.igcm3',status='old')                 
      do j=1,jg                                                         
      read(50,*)(ofracfull(i,1,j),i=1,mg)                               
      end do                                                            
      do j=jg,1,-1                                                      
      read(50,*)(ofracfull(i,2,j),i=1,mg)                               
      end do                                                            
      close(50)                                                         
      write(2,*) 'fractional land mask looks like:'                     
      do j=1,jg                                                         
         write (2,'(128I1)')(nint( 10.*ofracfull(i,1,j)  ) ,i=1,mg)     
      enddo                                                             
      do j=jg,1,-1                                                      
         write (2,'(128I1)')(nint( 10.*ofracfull(i,2,j)),i=1,mg)        
      enddo                                                             
      endif     ! end MYPE if                                           
                                                                        
      ! Share OFRAC across PEs                                          
      CALL SHAREGRID(.TRUE.,MG,1,OFRACFULL,OFRAC)                       
                                                                        
      if(mype .eq. 0) DEALLOCATE(ofracfull)                             
      if(npe.gt.1)CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                
! READS in T21 orography  part                                          
!-----------------------------------------------------                  
      IF (LOROG) THEN                                                   
         !-----------------------------------------------------         
         ! Read in geopotential spectral field                          
         !-----------------------------------------------------         
         IF (MyPe.EQ.0) THEN                                            
            ALLOCATE(GSFull(NWJ2,NHEM))                                 
            open(50,file='orogdata/t42.50',status='old')                
            read(50,*) GSFull                                           
            close(50)                                                   
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Share geopotential spectral field across PEs                 
         !-----------------------------------------------------         
         CALL SHARECOEF(.TRUE.,NHEM,GSFull,GS)                          
         !-----------------------------------------------------         
         ! Read in gridpoint field                                      
         ! Skipping extra line on equator                               
         ! Tstar goes from 1 at pole to jg at equator, gsg made         
         ! the same                                                     
         !-----------------------------------------------------         
         IF (MyPe.EQ.0) THEN                                            
            DEALLOCATE(GSFull)                                          
            ALLOCATE(GSGFull(MG,NHEM,JG))                               
            open(50,file='orogdata/t42.59',status='old')                
            READ(50,*)((GSGFull(i,1,j),i=1,mg),j=1,jg),(adum,i=1,mg),   
     &           ((GSGFull(i,2,j),i=1,mg),j=jg,1,-1)                    
            CLOSE(50)                                                   
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Share GSG across PEs                                         
         !-----------------------------------------------------         
         CALL SHAREGRID(.TRUE.,MG,1,GSGFull,GSG)                        
         !-----------------------------------------------------         
         ! Print out land mask                                          
         !-----------------------------------------------------         
         IF (MyPe.EQ.0) THEN                                            
            write(2,*) 'land mask looks like:'                          
            do j=1,jg                                                   
               write (2,'(128I1)')                                      
     &              (nint((GSGFull(i,1,j)/(GSGFull(i,1,j)+.1))),i=1,mg) 
            enddo                                                       
            do j=jg,1,-1                                                
               write (2,'(128I1)')                                      
     &              (nint((GSGFull(i,2,j)/(GSGFull(i,2,j)+.1))),i=1,mg) 
            enddo                                                       
            DEALLOCATE(GSGFull)                                         
         ENDIF                                                          
      ENDIF                                                             
      IF (MyPe.EQ.0) THEN                                               
c     read in the preprepared runoff grid (non-polar)                   
      open(50,file='orogdata/T42.runoff-real.ForteIGCM4.grid'           
     $,status='old')                                                    
      read (50,*) iareanum                                              
      do iarea=1,iareanum                                               
c      read in the land box perimeter                                   
       read (50,*)                                                      
         read (50,*) runoff(1,1,iarea),runoff(1,2,iarea)                
     $             ,runoff(2,1,iarea),runoff(2,2,iarea)                 
         read (50,*) runoff(3,1,iarea)                                  
         do ncoast=1, runoff(3,1,iarea)                                 
         read (50,*) runoff(3+ncoast,1,iarea),runoff(3+ncoast,2,iarea)  
         end do                                                         
      end do                                                            
      close(50)                                                         
      ENDIF   ! End Mype if                                             
      if(npe.gt.1)CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                
!-----------------------------------------------------                  
! If BEGDOY=0 sets DOY from restart record - else                       
! sets it from namelist                                                 
!-----------------------------------------------------                  
      IF (BEGDOY.GT.0.9) DOY=BEGDOY                                     
      IF (MyPe.EQ.0) THEN                                               
         WRITE(2,*) 'STARTING FROM ',DOY,' DAY OF YEAR '                
         if (lshort) then                                               
            print *,'resetting DAY to 0.0'                              
            DAY=0.0                                                     
         else                                                           
            print*,' begdoy day doy ',begdoy,day,doy                    
         endif                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Open restoration file                                                 
!-----------------------------------------------------                  
      IF ((MyPe.EQ.0).AND.(LRESTIJ))                                    
     &     OPEN(UNIT=13,FILE='restoration.dat',FORM='unformatted')      
      END                                                               
!=====================================================                  
! *********PMF Version 1.0 (8.4.97)*******************                  
! Subroutine which initialises the tracer fields.                       
! Tracers can be started as any function of the usual grid              
! point fields.                                                         
! Mass mixing ratios converted to volume mixing ratios for              
! flux form of advection.                                               
!                                    John Methven, 10.04.95             
! Slighly modified to add more tracers and add array GWORK              
! Piers Forster 22.01.97                                                
!                                                                       
! Changed again to combine with phys2 update                            
! to include surface water and temperature                              
! initilization.                                                        
! Also lets water vapour profile be cacluated by setting                
! surface RH     Piers Forster  18.02.97                                
!                                                                       
! Flag LFLUX added for FLUX form of advection                           
!                                                                       
! Marc 2 November 2010                                                  
! Signficant change to this routine. In the non-parallel                
! version the transformation in this routines was:                      
! TRA (spectral space) -> TRAG (grid space) -> Grid space               
! calculations -> TRAG (grid space) -> TRA.                             
! To be consistent with the other spectral to grid space                
! to spectral loops (the ones involving MGRMLT and DGRMLT)              
! and for ease in writing other routines, here the                      
! transformation is:                                                    
! TRA (spectral space) -> TRAG (grid space) -> Grid space               
! calculations -> TRANLG (grid space) -> TRAT,                          
! and where necessary the values of TRAT are read into                  
! TRA.                                                                  
!=====================================================                  
      SUBROUTINE ICTRAC                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Half transforms. The levels are divided across the columns of PEs;    
! the zonal wavenumbers are divided across the row of PEs; and on       
! ech PE the latitude are complete.                                     
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA/UG,FUG,VG,FVG,TG,TNLG,PLG                           
     &     ,TRAG,TRANLG                                                 
! U_psi, then U (rotational wind)                                       
      COMPLEX :: UG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FUG(2*MMTy,NHEM,JG,NLx)                                
! V_chi, then V (divergent wind)                                        
      COMPLEX :: VG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FVG(2*MMTy,NHEM,JG,NLx)                                
! Temperature anomaly (T')                                              
      COMPLEX :: TG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: TNLG(2*MMTy,NHEM,JG,NLx)                               
! ln p* or p*                                                           
      COMPLEX :: PLG(2*MMTy,NHEM,JG)                                    
! Tracer term                                                           
      COMPLEX :: TRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                         
!                                                                       
      COMPLEX :: TRANLG(2*MMTy,NHEM,JG,NLx,NTRAC)                       
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB/ZG,DG,PJG,PMG,SPG,VPG,EG,UTG,VTG                    
     &     ,UTRAG,VTRAG                                                 
     &     ,CHIG,SFG,FVGT,FUGT                                          
! Vorticity (zeta)                                                      
      COMPLEX :: ZG(2*MMTy,NHEM,JG,NLx)                                 
! Divergence (d)                                                        
      COMPLEX :: DG(2*MMTy,NHEM,JG,NLx)                                 
! (1-mu^2) (d(ln p*)/d mu)                                              
      COMPLEX :: PJG(2*MMTy,NHEM,JG)                                    
! (d(ln p*)/d lambda)                                                   
      COMPLEX :: PMG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: SPG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: VPG(2*MMTy,NHEM,JG)                                    
! (U^2 + V^2)                                                           
      COMPLEX :: EG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: UTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: VTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: UTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: VTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: CHIG(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: SFG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: FVGT(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: FUGT(2*MMTy,NHEM,JG,NLx)                               
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Similar to variables in GRIDP but in grid space, and without          
! equivalent variables for CHIG, SFG, FVGT and FUGT. The levels         
! are divided across the columns of PEs; the latitudes are divided      
! across the row of PEs; and the zonal wavenumbers/longitudes are       
! complete.                                                             
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA2/UG2,FUG2,VG2,FVG2,TG2,TNLG2,PLG2                   
     &     ,TRAG2,TRANLG2                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FUG2(MGPP,NHEM,JGy,NLx)                                   
! V_chi, then V (divergent wind)                                        
      REAL :: VG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FVG2(MGPP,NHEM,JGy,NLx)                                   
! Temperature anomaly (T')                                              
      REAL :: TG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: TNLG2(MGPP,NHEM,JGy,NLx)                                  
! ln p* or p*                                                           
      REAL :: PLG2(MGPP,NHEM,JGy)                                       
! Tracer term                                                           
      REAL :: TRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                            
      REAL :: TRANLG2(MGPP,NHEM,JGy,NLx,NTRAC)                          
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB2/ZG2,DG2,PJG2,PMG2,SPG2,VPG2,EG2,UTG2,VTG2          
     &     ,UTRAG2,VTRAG2                                               
     &     ,CHIG2,SFG2                                                  
! Vorticity (zeta)                                                      
      REAL :: ZG2(MGPP,NHEM,JGy,NLx)                                    
! Divergence (d)                                                        
      REAL :: DG2(MGPP,NHEM,JGy,NLx)                                    
! (1-mu^2) (d(ln p*)/d mu)                                              
      REAL :: PJG2(MGPP,NHEM,JGy)                                       
! (d(ln p*)/d lambda)                                                   
      REAL :: PMG2(MGPP,NHEM,JGy)                                       
      REAL :: SPG2(MGPP,NHEM,JGy)                                       
      REAL :: VPG2(MGPP,NHEM,JGy)                                       
! (U^2 + V^2)                                                           
      REAL :: EG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: UTG2(MGPP,NHEM,JGy,NLx)                                   
      REAL :: VTG2(MGPP,NHEM,JGy,NLx)                                   
      REAL :: UTRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                           
      REAL :: VTRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                           
      REAL :: CHIG2(MGPP,NHEM,JGy,NLx)                                  
      REAL :: SFG2(MGPP,NHEM,JGy,NLx)                                   
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      LOGICAL :: LNSURF                                                 
      LOGICAL :: LRH                                                    
      REAL :: RH(NL)                                                    
      CHARACTER :: LHEM(2)*25                                           
      REAL, ALLOCATABLE :: TstarFull(:,:,:)                             
      REAL, ALLOCATABLE :: QstarFull(:,:,:)                             
      REAL :: TDUM(JG),QDUM(JG)                                         
! Loop variables                                                        
      INTEGER :: I, iCol2, IHEM, IM, IN, J, KK, L                       
!-----------------------------------------------------                  
! Set data in LHEM                                                      
!-----------------------------------------------------                  
      DATA LHEM/'  N HEM  EQUATOR.....POLE'                             
     :         ,'  S HEM  EQUATOR.....POLE'/                            
!-----------------------------------------------------                  
! Format statements                                                     
!-----------------------------------------------------                  
 2010 FORMAT('0SURFACE TEMPERATURE IN DEGC',A25/20(1X,F8.2))            
 2020 FORMAT('0SURFACE SPECIFIC HUMIDITY IN G/KG',A25/20(1X,f8.4))      
 2030 FORMAT(3X,20(1X,F5.1))                                            
!-----------------------------------------------------                  
! Namelist                                                              
!-----------------------------------------------------                  
      NAMELIST/INQ/LRH,RH,LNSURF                                        
!-----------------------------------------------------                  
! Preset namelist options                                               
!-----------------------------------------------------                  
      LRH=.FALSE.                                                       
      LNSURF=.FALSE.                                                    
      DO L=1,NL                                                         
        RH(L)=90.0*SIGMA(L)                                             
      ENDDO                                                             
!-----------------------------------------------------                  
! Read in namelist data                                                 
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         READ(7,INQ)                                                    
         WRITE(2,INQ)                                                   
      ENDIF                                                             
      IF (NPE.GT.1) THEN                                                
         iSend=0                                                        
         nBuffSize=1                                                    
         CALL MPI_BCAST(LRH,nBuffSize,ParaLog,iSend,                    
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LNSURF,nBuffSize,ParaLog,iSend,                 
     &        MPI_COMM_WORLD,ierr)                                      
         nBuffSize=NL                                                   
         CALL MPI_BCAST(RH,nBuffSize,ParaReal,iSend,                    
     &        MPI_COMM_WORLD,ierr)                                      
      ENDIF                                                             
!-----------------------------------------------------                  
! Check that flags make sense.                                          
!-----------------------------------------------------                  
      IF (.NOT.LRH.AND.NTRACO.LT.NTRAC) THEN                            
         WRITE(2,'(A25)') 'No water Vapour. Need to set LRH to TRUE'    
         CALL ABORT                                                     
      ENDIF                                                             
!-----------------------------------------------------                  
! Convert RH from % to fraction                                         
!-----------------------------------------------------                  
      DO L=1,NL                                                         
        RH(L)=RH(L)/100.0                                               
      ENDDO                                                             
!-----------------------------------------------------                  
! Transpose the arrays so that the meridional wavenumber                
! dimension is the only dimension that isn't decomposed                 
! over PEs                                                              
!-----------------------------------------------------                  
      CALL TRANS1                                                       
!-----------------------------------------------------                  
! Convert the meridional wavenumber dimension to a                      
! latitude dimension                                                    
!-----------------------------------------------------                  
      CALL LTI                                                          
      CALL LTIDT                                                        
!-----------------------------------------------------                  
! Transpose the arrays so that the zonal wavenumber                     
! dimension is the only dimension that isn't decomposed                 
! over PEs                                                              
!-----------------------------------------------------                  
      CALL TRANS2(UG,VG,ZG,DG,TG,TRAG,PLG,PJG,PMG)                      
      CALL TRANS2A(CHIG,SFG)                                            
!-----------------------------------------------------                  
! Convert zonal wavenumber dimension to longitude                       
! dimension.                                                            
!-----------------------------------------------------                  
      CALL FFT(1,NHEM*JGy*NLx,CHIG2)                                    
      CALL FFT(1,NHEM*JGy*NLx,SFG2)                                     
      CALL FFT(1,NHEM*JGy*NLx,UG2)                                      
      CALL FFT(1,NHEM*JGy*NLx,VG2)                                      
      CALL FFT(1,NHEM*JGy*NLx,ZG2)                                      
      CALL FFT(1,NHEM*JGy*NLx,TG2)                                      
      IF (iCol.EQ.1) THEN                                               
         CALL FFT(1,NHEM*JGy,PLG2)                                      
         CALL FFT(1,NHEM*JGy,PJG2)                                      
         CALL FFT(1,NHEM*JGy,PMG2)                                      
      ENDIF                                                             
!-----------------------------------------------------                  
! Transpose the arrays so that layer dimension is the                   
! only dimension that isn't decomposed over PEs                         
!-----------------------------------------------------                  
      CALL TRANS3                                                       
      CALL TRANS3A                                                      
!-----------------------------------------------------                  
! Carry out the grid-point calculations                                 
!-----------------------------------------------------                  
      CALL ICTGRMLT(LNSURF, LRH, RH)                                    
      CALL TRANS4A                                                      
!-----------------------------------------------------                  
! Transform all tracer fields to spectral space at each latitude.       
! First do inverse FFT to give half-transforms.                         
!-----------------------------------------------------                  
      CALL FFT(-1,NHEM*JGy*NLx*NTRAC,TRANLG2)                           
      CALL TRANS5A(TRANLG)                                              
!-----------------------------------------------------                  
! Call HANAL for direct Legendre transform.                             
!-----------------------------------------------------                  
      CALL HANALVICT                                                    
!-----------------------------------------------------                  
! Transpose the arrays so that the layer dimension is the               
! only dimension that isn't decomposed over PEs                         
!-----------------------------------------------------                  
      CALL TRANS6A                                                      
!-----------------------------------------------------                  
! Return old tracers from the restart record into array TRA             
! and set the t-1 record for the new tracers.                           
!-----------------------------------------------------                  
      DO KK=NTRACO+1,NTRAC                                              
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMTyy(iRow)                                      
                  DO IN=1,NNTxx(iCol)                                   
                     TRA(IN,IM,IHEM,L,KK)=TRAT(IN,IM,IHEM,L,KK)         
                     TRAMI(IN,IM,IHEM,L,KK)=TRA(IN,IM,IHEM,L,KK)        
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Writes out surface temp and humdidity to channel 14                   
! Continue to write out restart in old format....                       
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         ALLOCATE(TstarFull(MGPP,NHEM,JG))                              
         ALLOCATE(QstarFull(MGPP,NHEM,JG))                              
      ENDIF                                                             
      CALL SHAREGRID(.FALSE.,MGPP,1,TstarFull,tstar)                    
      CALL SHAREGRID(.FALSE.,MGPP,1,QstarFull,qstar)                    
      IF (MyPe.eq.0) THEN                                               
         !-----------------------------------------------------         
         ! Write the TSTAR and QSTAR values to unit 14                  
         !-----------------------------------------------------         
         IF (LNSURF) THEN                                               
            IF (NHEM.EQ.1) THEN                                         
               WRITE(14,*) TstarFull,QstarFull                          
            ELSE                                                        
               WRITE(14,*) ( ((TstarFull(i,1,j),j=1,jg),                
     &              (TstarFull(i,2,j),j=1,jg)),i=1,MGPP),               
     &              ( ((QstarFull(i,1,j),j=1,jg),                       
     &              (QstarFull(i,2,j),j=1,jg)),i=1,MGPP)                
            ENDIF                                                       
         ENDIF                                                          
         WRITE(2,'(A40)') 'O DEG LONG X-SECTS SHOWN BELOW'              
         DO IHEM=1,NHEM                                                 
            DO J=1,JG                                                   
               TDUM(J)=CT*TstarFull(1,IHEM,J)-273.15                    
               QDUM(J)=CQ*QstarFull(1,IHEM,J)                           
            ENDDO                                                       
            IF(JG.LE.20) THEN                                           
               WRITE(2,2010) LHEM(IHEM),(TDUM(J),J=JG,1,-1)             
               WRITE(2,2020) LHEM(IHEM),(QDUM(J),J=JG,1,-1)             
            ELSE                                                        
               WRITE(2,2010) LHEM(IHEM),(TDUM(J),J=JG,2,-2)             
               WRITE(2,2030)            (TDUM(J),J=JG-1,1,-2)           
               WRITE(2,2020) LHEM(IHEM),(QDUM(J),J=JG,2,-2)             
               WRITE(2,2030)            (QDUM(J),J=JG-1,1,-2)           
            ENDIF                                                       
         ENDDO                                                          
         DEALLOCATE(TstarFull)                                          
         DEALLOCATE(QstarFull)                                          
      ENDIF                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! Grid-space calculations used in initialisation                        
!=====================================================                  
      SUBROUTINE ICTGRMLT(LNSURF, LRH, RH)                              
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Vertical derivative arrays for interpolation                          
!-----------------------------------------------------                  
      COMMON/COMGRM/DUDLSG,DVDLSG,DTDLSG                                
      REAL :: DUDLSG(MGx,NHEM,NL)                                       
      REAL :: DVDLSG(MGx,NHEM,NL)                                       
      REAL :: DTDLSG(MGx,NHEM,NL)                                       
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB3/ZG3,DG3,PJG3,PMG3,SPG3,VPG3,EG3,UTG3,VTG3          
     &     ,UTRAG3,VTRAG3                                               
     &     ,CHIG3,SFG3                                                  
! Vorticity (zeta)                                                      
      REAL :: ZG3(MGx,NHEM,JGy,NL)                                      
! Divergence (d)                                                        
      REAL :: DG3(MGx,NHEM,JGy,NL)                                      
! (1-mu^2) (d(ln p*)/d mu)                                              
      REAL :: PJG3(MGx,NHEM,JGy)                                        
! (d(ln p*)/d lambda)                                                   
      REAL :: PMG3(MGx,NHEM,JGy)                                        
      REAL :: SPG3(MGx,NHEM,JGy)                                        
      REAL :: VPG3(MGx,NHEM,JGy)                                        
! (U^2 + V^2)                                                           
      REAL :: EG3(MGx,NHEM,JGy,NL)                                      
      REAL :: UTG3(MGx,NHEM,JGy,NL)                                     
      REAL :: VTG3(MGx,NHEM,JGy,NL)                                     
      REAL :: UTRAG3(MGx,NHEM,JGy,NL,NTRAC)                             
      REAL :: VTRAG3(MGx,NHEM,JGy,NL,NTRAC)                             
      REAL :: CHIG3(MGx,NHEM,JGy,NL)                                    
      REAL :: SFG3(MGx,NHEM,JGy,NL)                                     
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed arguments                                                      
!-----------------------------------------------------                  
      LOGICAL :: LNSURF                                                 
      LOGICAL :: LRH                                                    
      REAL :: RH(NL)                                                    
!-----------------------------------------------------                  
! External function                                                     
!-----------------------------------------------------                  
      REAL, EXTERNAL :: PQSAT                                           
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=300                                  
      REAL :: TEMPTR(MGx,NHEM,NL)                                       
      REAL :: SPZ(NHEM,JGy)                                             
      REAL :: PlgHold(MGx,NHEM,JGy)                                     
      REAL :: HoldVar(NHEM,JGy)                                         
      REAL :: ESCON                                                     
! Loop variables                                                        
      INTEGER :: I, iCol2, IHEM, J, KK, L                               
!-----------------------------------------------------                  
! Grid point calculations for surface pressure,                         
! temperature T (=TO+T') and vertical derivatives of U,V and T.         
!-----------------------------------------------------                  
      DO J=1,JGyy(iRow)                                                 
         DO IHEM=1,NHEM                                                 
            DO I=1,MGxx(iCol)                                           
               PLG3(I,IHEM,J)=EXP(PLG3(I,IHEM,J))                       
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! If LNSURF is true, we'll need a sum of PLG over latitudes             
! later                                                                 
!-----------------------------------------------------                  
      IF (LNSURF) THEN                                                  
         IF (iCol.GT.1) THEN                                            
            !-----------------------------------------------------      
            ! Send data to first PE on this row                         
            !-----------------------------------------------------      
            iRecv=PeGrid(1,iRow)                                        
            nBuffSize=MGx*NHEM*JGy                                      
            iTag=TagRef+2*MyPe                                          
            CALL MPI_BSEND(PLG3,nBuffSize,ParaReal,                     
     &           iRecv,iTag,MPI_COMM_WORLD,ierr)                        
         ENDIF                                                          
         IF (Px.GT.1) CALL MPI_BARRIER(MPI_COMM_WORLD,ierr)             
         !-----------------------------------------------------         
         ! Need an average of PLG3 over latitudes                       
         !-----------------------------------------------------         
         SPZ(:,:)=0.0                                                   
         IF (iCol.EQ.1) THEN                                            
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,MGxx(iCol)                                     
                     SPZ(IHEM,J)=SPZ(IHEM,J)+PLG3(I,IHEM,J)             
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Loop through the other PEs on this row                    
            !-----------------------------------------------------      
            IF (Px.GT.1) THEN                                           
               DO iCol2=2,Px                                            
                  !---------------------------------------------------  
                  ! Receive data from the other PEs on this row         
                  !---------------------------------------------------  
                  iSend=PeGrid(iCol2, iRow)                             
                  nBuffSize=MGx*NHEM*JGy                                
                  iTag=TagRef+2*iSend                                   
                  CALL MPI_RECV(PlgHold,nBuffSize,ParaReal,             
     &                 iSend,iTag,MPI_COMM_WORLD,istat,ierr)            
                  !---------------------------------------------------  
                  ! Add the data together                               
                  !---------------------------------------------------  
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,MGxx(iCol2)                              
                           SPZ(IHEM,J)=SPZ(IHEM,J)+PlgHold(I,IHEM,J)    
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Loop through the other PEs on this row again (this time
               ! sending data).                                         
               !-----------------------------------------------------   
               DO iCol2=2,Px                                            
                  !---------------------------------------------------  
                  ! Send the data to the other PEs                      
                  !---------------------------------------------------  
                  iRecv=PeGrid(iCol2,iRow)                              
                  nBuffSize=JGy*NHEM                                    
                  iTag=TagRef+2*iRecv+1                                 
                  CALL MPI_BSEND(SPZ,nBuffSize,ParaReal,                
     &                 iRecv,iTag,MPI_COMM_WORLD,ierr)                  
               ENDDO                                                    
            ENDIF                                                       
         ENDIF                                                          
         IF (Px.GT.1) CALL MPI_BARRIER(MPI_COMM_WORLD,ierr)             
         IF (iCol.GT.1) THEN                                            
            !-----------------------------------------------------      
            ! Receive summed data from first PE on this row             
            !-----------------------------------------------------      
            iSend=PeGrid(1,iRow)                                        
            nBuffSize=JGy*NHEM                                          
            iTag=TagRef+2*MyPe+1                                        
            CALL MPI_RECV(SPZ,nBuffSize,ParaReal,                       
     &           iSend,iTag,MPI_COMM_WORLD,istat,ierr)                  
         ENDIF                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Loop over latitudes                                                   
!-----------------------------------------------------                  
      DO J=1,JGyy(iRow)                                                 
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO I=1,MGxx(iCol)                                        
                  TG3(I,IHEM,J,L)=TG3(I,IHEM,J,L)+T0(L)                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Calculate TSTAR                                              
         !-----------------------------------------------------         
         IF (LNSURF) THEN                                               
            DO IHEM=1,NHEM                                              
               DO I=1,MGxx(iCol)                                        
                  TSTAR(I,IHEM,J)=TG3(I,IHEM,J,NL)/SKAP(NL)             
               ENDDO                                                    
            ENDDO                                                       
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Compute non-linear diabatic forcing                          
         !-----------------------------------------------------         
         CALL DLSGCR(MGx,MGx,NHEM,JGy,NL,J,UG3,RGG,DUDLSG)              
         CALL DLSGCR(MGx,MGx,NHEM,JGy,NL,J,VG3,RGG,DVDLSG)              
         CALL DLSGCR(MGx,MGx,NHEM,JGy,NL,J,TG3,RGG,DTDLSG)              
         !-----------------------------------------------------         
         ! Grid point calculations for new tracer fields at one         
         ! latitude (JH).                                               
         ! Initilise water vapour if LRH set  depends on RH             
         ! goes into tracer No. 1                                       
         !-----------------------------------------------------         
         IF (LRH) THEN                                                  
            DO IHEM=1,NHEM                                              
               DO I=1,MGxx(iCol)                                        
                  ESCON=1./PLG3(I,IHEM,J)                               
                  DO L=1,NL                                             
                     TRANLG3(I,IHEM,J,L,1)=ESCON*RH(L)*                 
     &                    PQSAT(TG3(I,IHEM,J,L))/SIGMA(L)               
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Optionally initialise surface humidity as saturated          
         ! at the zonal mean surface temperature and pressure.          
         !-----------------------------------------------------         
         IF (LNSURF) THEN                                               
            DO IHEM=1,NHEM                                              
               ESCON=REAL(MG)/SPZ(IHEM,J)                               
               DO I=1,MGxx(iCol)                                        
                  QSTAR(I,IHEM,J)=ESCON*PQSAT(TSTAR(I,IHEM,J))          
               ENDDO                                                    
               PRINT '(i2,x,10(F5.1,x))',J,                             
     &              (ct*tstar(I,IHEM,J)-273.15,I=1,10)                  
            ENDDO                                                       
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Initialise any additional tracers.                           
         !-----------------------------------------------------         
         DO KK=NTRACO+1,NTRAC                                           
            DO L=1,NL                                                   
               DO IHEM=1,NHEM                                           
                  DO I=1,MGxx(iCol)                                     
                     TEMPTR(I,IHEM,L)=TRANLG3(I,IHEM,J,L,KK)            
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
            IF(KOLOUR(KK).EQ.1) THEN                                    
               CALL PVCR(J,TEMPTR,1)                                    
            ENDIF                                                       
            IF(KOLOUR(KK).EQ.2) THEN                                    
               CALL PVCR(J,TEMPTR,2)                                    
            ENDIF                                                       
            DO L=1,NL                                                   
               DO IHEM=1,NHEM                                           
                  DO I=1,MGxx(iCol)                                     
                     TRANLG3(I,IHEM,J,L,KK)=TEMPTR(I,IHEM,L)            
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Convert mass mixing ratios to volume mixing ratios.       
            !-----------------------------------------------------      
            IF (LFLUX) THEN                                             
               DO L=1,NL                                                
                  DO IHEM=1,NHEM                                        
                     DO I=1,MGxx(iCol)                                  
                        TRANLG3(I,IHEM,J,L,KK)=PLG3(I,IHEM,J)*          
     &                    TRANLG3(I,IHEM,J,L,KK)                        
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDIF                                                       
         ENDDO                                                          
      ENDDO                                                             
                                                                        
      END                                                               
!=====================================================                  
! This subroutine reads data used for balancing and                     
! calculates arrays needed for balancing.                               
!=====================================================                  
      SUBROUTINE INIBAL                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Switches counters and constants controlling type and frequency of     
! model output                                                          
!-----------------------------------------------------                  
      COMMON/OUTCON/RNTAPE,DAY,SQR2,RSQR2,EAM1,EAM2,TOUT1               
     &     ,TOUT2,LSPO,LGPO                                             
     &     ,RNTAPO                                                      
     &     ,LSHIST,LMINIH                                               
      COMMON/OUTCONINT/NCOEFF,NLAT,INLAT,INSPC,KOUNTP                   
     &                 ,KOUNTE,KOUNTH,KOUNTR,KOUTP                      
     &                 ,KOUTE,KOUTH,KOUTR                               
! Identyfying run number                                                
      REAL :: RNTAPE                                                    
! Maximum wavenumber of printed spectral coeffecients                   
      INTEGER :: NCOEFF                                                 
! Number of latitudes in printed grid point fields                      
      INTEGER :: NLAT                                                   
      INTEGER :: INLAT                                                  
!                                                                       
      INTEGER :: INSPC                                                  
! Number of timsteps between successive writing of spectral             
! and gridded fields                                                    
      INTEGER :: KOUNTP                                                 
      INTEGER :: KOUNTE                                                 
! Number of timsteps between successive writing of history              
! records, containing spectral fields at the present and                
! previous time levels.                                                 
      INTEGER :: KOUNTH                                                 
! Number of timsteps between successive writing of restart              
! records, containing the spectral fields at the present and            
! previous time levels                                                  
      INTEGER :: KOUNTR                                                 
      INTEGER :: KOUTP                                                  
      INTEGER :: KOUTE                                                  
      INTEGER :: KOUTH                                                  
      INTEGER :: KOUTR                                                  
      REAL :: DAY                                                       
      REAL :: SQR2                                                      
      REAL :: RSQR2                                                     
      REAL :: EAM1                                                      
      REAL :: EAM2                                                      
      REAL :: TOUT1                                                     
      REAL :: TOUT2                                                     
      LOGICAL :: LSPO(NL)                                               
      LOGICAL :: LGPO(NL)                                               
      REAL :: RNTAPO                                                    
      LOGICAL :: LSHIST                                                 
! Reduce the history file to single precision if true                   
      LOGICAL :: LMINIH                                                 
!-----------------------------------------------------                  
! Constants and arrays needed for balancing                             
!-----------------------------------------------------                  
      COMMON/BALAN/BFILT,RGT0,RG,TMEAN                                  
     +            ,EP1,EP2,KBAL,SRGT0,LTBAL                             
     &            ,PMNRE                                                
! Maximum zonal wavenumbers used for balancing                          
      INTEGER, PARAMETER :: MFTBAL=9                                    
! Number of zonal wavenumbers used for balancing                        
      INTEGER, PARAMETER :: NFTBAL=1+MFTBAL/MOCT                        
      REAL :: BFILT(NL)                                                 
      REAL :: RGT0(NL)                                                  
      REAL :: RG(NL,NL)                                                 
      REAL :: TMEAN(NL)                                                 
      REAL :: EP1(IDM,NFTBAL,NHEM)                                      
      REAL :: EP2(IDM,NFTBAL,NHEM)                                      
      INTEGER :: KBAL                                                   
      REAL :: SRGT0                                                     
      LOGICAL :: LTBAL                                                  
      REAL :: PMNRE(IDM,IDM,3)                                          
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Intrinic routine                                                      
!-----------------------------------------------------                  
      INTEGER, EXTERNAL :: ICAMAX                                       
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: TBM1(NL,NL)                                               
      REAL :: SM(IDM,IDM)                                               
      REAL :: WA2(NL)                                                   
      INTEGER :: IWA2(NL)                                               
      REAL, ALLOCATABLE :: WA1(:)                                       
      INTEGER, ALLOCATABLE :: IWA1(:)                                   
      REAL, ALLOCATABLE :: AU(:,:)                                      
      REAL :: ZDATN(IDM,NHEM,NL),ZDAT1(IDM,NHEM)                        
! File unit for writing data for a later restart run                    
      INTEGER :: ID                                                     
      INTEGER :: IG                                                     
      REAL :: TRGT0                                                     
      INTEGER :: MAXIND                                                 
      REAL :: GSMAX                                                     
      REAL :: GSMAX2                                                    
      REAL :: TEMP, TEMPP                                               
      REAL :: FACT                                                      
      REAL :: RM1TAPE, RM2TAPE                                          
      REAL :: RDAY                                                      
      INTEGER :: NWJ2IN                                                 
! RKOUNT is an integer, but output expects a REAL                       
      REAL :: RKOUNT                                                    
      INTEGER :: IE                                                     
      INTEGER :: J2                                                     
      REAL :: TAL                                                       
      INTEGER :: IM                                                     
      INTEGER :: iPe                                                    
      REAL :: HoldVar(NNTx,NHEM,NL+1)                                   
      INTEGER, PARAMETER :: TagRef=500                                  
      INTEGER :: IDM2                                                   
! Loop variableS                                                        
      INTEGER :: I, iCol2, IHEM, IN, J, K, L, M, MP, NP                 
!-----------------------------------------------------                  
! Namelist                                                              
!-----------------------------------------------------                  
      NAMELIST/INPBL/ KBAL,LTBAL,TMEAN                                  
      NAMELIST/TMPSP/ ZDATN,ZDAT1                                       
      NAMELIST/WVORT/ ZDATN                                             
!-----------------------------------------------------                  
! Format statements                                                     
!-----------------------------------------------------                  
  219 FORMAT(/' BALANCING FROM TEMPERATURE AND SURFACE PRESSURE TO',    
     +       ' OBTAIN VORTICITY')                                       
  220 FORMAT(/' BALANCING FROM VORTICITY TO OBTAIN TEMPERATURE AND',    
     +       ' SURFACE PRESSURE')                                       
  234 FORMAT(/' ***ABORT*** BALANCING ATTEMPTED WITH OROGRAPHY'/        
     +       ' TEMPERATURE FIELD WOULD CONTAIN 2-GRID WAVE')            
!-----------------------------------------------------                  
! Set default values and override as desired through NAMELIST input     
!-----------------------------------------------------                  
      KBAL=0                                                            
      LTBAL=.FALSE.                                                     
      DO L=1,NL                                                         
         TMEAN(L)=250.0                                                 
      ENDDO                                                             
                                                                        
      IF (MYPE.EQ.0) READ(7,INPBL)                                      
      IF (NPE.GT.1) THEN                                                
         CALL MPI_BARRIER(MPI_COMM_WORLD,IERR)                          
         nBuffSize=1                                                    
         iSend=0                                                        
         CALL MPI_BCAST(KBAL,nBuffSize,ParaInt,iSend,                   
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(LTBAL,nBuffSize,ParaLog,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
         nBuffSize=NL                                                   
         CALL MPI_BCAST(TMEAN,nBuffSize,ParaLog,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
      ENDIF                                                             
      IF (MYPE.EQ.0) THEN                                               
         WRITE(2,INPBL)                                                 
         IF (     LTBAL) WRITE(2,219)                                   
         IF (.NOT.LTBAL) WRITE(2,220)                                   
      ENDIF                                                             
!-----------------------------------------------------                  
! Make TMEAN dimensionless                                              
!-----------------------------------------------------                  
      DO L=1,NL                                                         
        TMEAN(L)=TMEAN(L)/CT                                            
      ENDDO                                                             
!-----------------------------------------------------                  
! Read in ZDATN and maybe ZDAT2 from namelist                           
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         IF(.NOT.LTBAL) THEN                                            
            READ (7,WVORT)                                              
            WRITE(2,WVORT)                                              
         ELSE                                                           
            READ (7,TMPSP)                                              
            WRITE(2,TMPSP)                                              
         ENDIF                                                          
      ENDIF                                                             
      IF (NPE.GT.1) THEN                                                
         IF ( (Px.GT.1).AND.(MyPe.EQ.0) ) THEN                          
            !-----------------------------------------------------      
            ! Pass parts of ZDATN to other the PEs contained            
            ! meridional wavenumber for 1st zonal wavenumber.           
            !-----------------------------------------------------      
            DO iCol2=2,Px                                               
               iPe=PeGrid(iCol2,1)                                      
               !---------------------------------------------------     
               ! See if any of ZDATN or ZDAT1 need passing              
               !---------------------------------------------------     
               IF (IDM.GT.NNT1(iCol2)) THEN                             
                  IDM2=MIN(IDM-NNT1(iCol2),NNTx)                        
                  !------------------------------------------------     
                  ! Store data in ZDATN for passing                     
                  !------------------------------------------------     
                  DO L=1,NL                                             
                     DO IHEM=1,NHEM                                     
                        DO IN=1,IDM2                                    
                           HoldVar(IN,IHEM,L)=                          
     &                          ZDATN(NNT1(iCol2)+IN,IHEM,L)            
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
                  IF (LTBAL) THEN                                       
                     DO IHEM=1,NHEM                                     
                        DO IN=1,IDM2                                    
                           HoldVar(IN,IHEM,NL+1)=                       
     &                          ZDAT1(NNT1(iCol2)+IN,IHEM)              
                        ENDDO                                           
                     ENDDO                                              
                  ENDIF                                                 
                  iRecv=iPe                                             
                  nBuffSize=NNTx*NHEM*(NL+1)                            
                  iTag=TagRef+iPe                                       
                  CALL MPI_BSEND(HoldVar, nBuffSize, ParaCom, iRecv,    
     &                 iTag, MPI_COMM_WORLD, ierr)                      
               ENDIF                                                    
            ENDDO                                                       
         ENDIF                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD,IERR)                          
         IF ( (Px.GT.1).AND.(iRow.EQ.1).AND.(MyPe.GT.0) ) THEN          
            !----------------------------------------------------       
            ! Check that at least part of 1st zonal wavenumber          
            ! is stored on this PE                                      
            !----------------------------------------------------       
            IF (mTwin1(2,1).GT.0) THEN                                  
               !---------------------------------------------------     
               ! Receive part of ZDATN from PE 0                        
               !---------------------------------------------------     
               iSend=0                                                  
               nBuffSize=NNTx*NHEM*(NL+1)                               
               iTag=TagRef+MyPe                                         
               CALL MPI_RECV(HoldVar, nBuffSize, ParaCom, iSend,        
     &              iTag, MPI_COMM_WORLD, istat, ierr)                  
               DO L=1,NL                                                
                  DO IHEM=1,NHEM                                        
                     DO IN=mTwin1(1,1)+1,mTwin1(2,1)                    
                        ZDATN(IN,IHEM,L)=HoldVar(IN,IHEM,L)             
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               IF (LTBAL) THEN                                          
                  DO IHEM=1,NHEM                                        
                     DO IN=mTwin1(1,1)+1,mTwin1(2,1)                    
                        ZDAT1(IN,IHEM)=                                 
     &                       HoldVar(IN,IHEM,NL+1)                      
                     ENDDO                                              
                  ENDDO                                                 
               ENDIF                                                    
            ENDIF                                                       
         ENDIF                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Read zonally averaged state from NAMELIST WVORT or TMPSP.             
! Assumes data is non-dimensionalised spectral coefficients             
! and that temperature (if used) includes layer mean.                   
!-----------------------------------------------------                  
      IF(.NOT.LTBAL) THEN                                               
         IF (iRow.EQ.1) THEN                                            
            DO L=1,NL                                                   
               DO IHEM=1,NHEM                                           
                  DO I=1,mTwin1(2,1)                                    
                     Z(I,1,IHEM,L)=ZDATN(I,IHEM,L)                      
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
            IF (MyPe.EQ.0) THEN                                         
               DO L=1,NL                                                
                  T(1,1,1,L)=T(1,1,1,L)+SQR2*(TMEAN(L)-T0(L))           
               ENDDO                                                    
            ENDIF                                                       
         ENDIF                                                          
      ELSE                                                              
         IF (iRow.EQ.1) THEN                                            
            DO IHEM=1,NHEM                                              
               DO L=1,NL                                                
                  DO I=1,mTwin1(2,1)                                    
                     T(I,1,IHEM,L)=ZDATN(I,IHEM,L)                      
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
            DO IHEM=1,NHEM                                              
               DO I=1,mTwin1(2,1)                                       
                  SP(I,1,IHEM)=ZDAT1(I,IHEM)                            
               ENDDO                                                    
            ENDDO                                                       
            DO L=1,NL                                                   
               T(1,1,1,L)=T(1,1,1,L)-SQR2*T0(L)                         
            ENDDO                                                       
         ENDIF                                                          
         IF (NPE.GT.1) THEN                                             
            CALL MPI_BARRIER(MPI_COMM_WORLD,ierr)                       
            iSend=0                                                     
            nBuffSize=1                                                 
            CALL MPI_BCAST(RKOUNT,nBuffSize,ParaReal,iSend,             
     &           MPI_COMM_WORLD,ierr)                                   
            CALL MPI_BCAST(RDAY,nBuffSize,ParaReal,iSend,               
     &           MPI_COMM_WORLD,ierr)                                   
         ENDIF                                                          
      ENDIF                                                             
      IF (MyPe.EQ.0) THEN                                               
         DO L=1,NL                                                      
            Z(1,1,1,L)=Z(1,1,1,L)+EZ                                    
         ENDDO                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! If the KBAL=0 leave this routine here                                 
!-----------------------------------------------------                  
      IF (KBAL.EQ.0) RETURN                                             
!-----------------------------------------------------                  
! Determine which choice of balancing is needed                         
!-----------------------------------------------------                  
      IF(.NOT.LTBAL) THEN                                               
         !-----------------------------------------------------         
         ! Set values required in BALANC.                               
         ! Find maximum value in GS                                     
         !-----------------------------------------------------         
         MAXIND=ICAMAX((NNTx*MMTy*NHEM),GS,1)                           
         IHEM=((MAXIND-1)/(NNTx*MMTy))+1                                
         IM=(( MAXIND-((IHEM-1)*NNTx*MMTy)-1 )/MMTy)+1                  
         IN=MOD(MAXIND,NNTx)                                            
         GSMAX=ABS(GS(IN,IM,IHEM))                                      
         IF (NPE.GT.1) THEN                                             
            !-----------------------------------------------------      
            ! Find maximum value of GS across PEs                       
            !-----------------------------------------------------      
            CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                      
            nBuffSize=1                                                 
            CALL MPI_ALLREDUCE(GSMAX, GSMAX2, nBuffSize, ParaReal,      
     &           ParaMax, MPI_COMM_WORLD, ierr)                         
            GSMAX=GSMAX2                                                
         ENDIF                                                          
         !-----------------------------------------------------         
         ! With orography the balanced temperature field contains a     
         ! 2-grid wave in the vertical. ABORT if this is attempted.     
         !-----------------------------------------------------         
         IF(GSMAX.GT.1.0E-10) THEN                                      
            IF (MYPE.EQ.0) WRITE(2,234)                                 
            CALL ABORT                                                  
         ENDIF                                                          
         DO L=1,NL                                                      
            DO M=1,NL                                                   
               TBM1(M,L)=G(M,L)                                         
            ENDDO                                                       
         ENDDO                                                          
         CALL MATINV(TBM1,NL,NL,IWA2,WA2)                               
         DO L=1,NL                                                      
            DO M=1,NL                                                   
               RG(M,L)=TBM1(M,L)                                        
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NL                                                      
            BFILT(L)=0.                                                 
         ENDDO                                                          
         BFILT(1)=1.                                                    
         BFILT(2)=1.0                                                   
         IF (NL.GE.3) THEN                                              
            TEMPP=1.0                                                   
            DO I=2,NL-1                                                 
               DO J=2,I                                                 
                  TEMP=BFILT(J)                                         
                  BFILT(J)=TEMP+TEMPP                                   
                  TEMPP=TEMP                                            
               ENDDO                                                    
               BFILT(I+1)=1.0                                           
            ENDDO                                                       
         ENDIF                                                          
         FACT=-1.0                                                      
         DO I=2,NL                                                      
            BFILT(I)=BFILT(I)*FACT                                      
            FACT=-FACT                                                  
         ENDDO                                                          
         SRGT0=0.                                                       
         IG=0                                                           
         DO L=1,NL                                                      
            TRGT0=0.                                                    
            DO M=1,NL                                                   
               TRGT0=TRGT0+RG(M,L)*T0(M)                                
            ENDDO                                                       
            RGT0(L)=TRGT0                                               
            SRGT0=SRGT0+TRGT0*BFILT(L)                                  
         ENDDO                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! Carry all calculation of EP1, EP2 and PMNRE only on PE 0     
         !-----------------------------------------------------         
         IF (MyPe.EQ.0) THEN                                            
            !-----------------------------------------------------      
            ! Calculate EP1 and EP2 for zonal wavenumbers below         
            ! MFTBAL                                                    
            !-----------------------------------------------------      
            DO MP=0,MFTBAL,MOCT                                         
               IM=1+(MP/MOCT)                                           
               IN=1                                                     
               DO NP=MP+1,MM,2                                          
                  EP2(IN,IM,1)=2.0*SQRT( (NP*NP-MP*MP)/                 
     &                 (4.0*NP*NP-1.0) )*(1.0-1.0/NP)                   
                  EP1(IN,IM,1)=2.0*SQRT( ( (NP+1)*(NP+1)-MP*MP)/        
     &                 (4.0*(NP+1)*(NP+1)-1.0) )*(1.0+1.0/(NP+1))       
                  IN=IN+1                                               
               ENDDO                                                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Loop over all the zonal wavenumbers great than 0, but     
            ! less than MFTBAL                                          
            !-----------------------------------------------------      
            DO MP=MOCT,MFTBAL,MOCT                                      
               IM=1+(MP/MOCT)                                           
               J2=(NN-MP-1)/2+1                                         
               !-----------------------------------------------------   
               ! Zero arrays                                            
               !-----------------------------------------------------   
               ALLOCATE(AU(J2,J2),IWA1(J2),WA1(J2))                     
               AU(:,:)=0.0                                              
               IWA1(:)=0                                                
               WA1(:)=0.0                                               
               SM(:,:)=0.0                                              
               !-----------------------------------------------------   
               ! Calculate SM and AU.                                   
               !-----------------------------------------------------   
               DO J=1,J2                                                
                  SM(J,J)=EP2(J,IM,1)                                   
                  AU(J,J)=EP2(J,IM,1)*EP2(J,IM,1)+                      
     &                    EP1(J,IM,1)*EP1(J,IM,1)                       
                  IF(J.GT.1)AU(J-1,J)=EP1(J-1,IM,1)*EP2(J,IM,1)         
                  IF (J.NE.J2) THEN                                     
                     SM(J+1,J)=EP1(J,IM,1)                              
                     AU(J+1,J)=EP1(J,IM,1)*EP2(J+1,IM,1)                
                  ENDIF                                                 
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Invert an array AU which is of size J2 by J2           
               !-----------------------------------------------------   
               CALL MATINV(AU,J2,J2,IWA1,WA1)                           
               DO I=1,J2                                                
                  DO J=1,J2                                             
                     TAL=0.0                                            
                     DO K=1,J2                                          
                        TAL=TAL+AU(K,I)*SM(J,K)                         
                     ENDDO                                              
                     PMNRE(J,I,IM)=TAL                                  
                  ENDDO                                                 
               ENDDO                                                    
               DEALLOCATE(AU,IWA1,WA1)                                  
            ENDDO                                                       
                                                                        
            IF(NHEM.EQ.2) THEN                                          
               !-----------------------------------------------------   
               ! Calculations for zonal wavenumber 0                    
               !-----------------------------------------------------   
               DO IN=1,IDM                                              
                  NP=2*(IN-1)                                           
                  IF (NP.GT.0) THEN                                     
                     EP2(IN,1,NHEM)=2.0*SQRT(NP*NP/(4.0*NP*NP-1.0))*    
     &                    (1.0-1.0/NP)                                  
                     EP1(IN,1,NHEM)=2.0*SQRT( (NP+1.0)*(NP+1.0)/        
     &                    (4.0*(NP+1.0)*(NP+1.0)-1.0) )*                
     &                    (1.0+1.0/(NP+1.0))                            
                  ENDIF                                                 
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Calculations for zonal wavenumber > 0                  
               !-----------------------------------------------------   
               DO MP=MOCT,MFTBAL,MOCT                                   
                  IM=1+(MP/MOCT)                                        
                  IN=1                                                  
                  DO NP=MP+1,MM,2                                       
                     EP1(IN,IM,NHEM)=2.0*SQRT((NP*NP-MP*MP)/            
     &                    (4.0*NP*NP-1.0))*(1.0+1.0/NP)                 
                     EP2(IN,IM,NHEM)=2.0*SQRT(                          
     &                    ((NP+1.0)*(NP+1.0)-MP*MP)/(4.0*(NP+1.0)*      
     &                    (NP+1.0)-1.0) )*(1.0-1.0/(NP+1.0))            
                     IN=IN+1                                            
                  ENDDO                                                 
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Calculations for zonal wavenumber 0                    
               !-----------------------------------------------------   
               ALLOCATE(AU(IDM-1,IDM-1),IWA1(IDM-1),WA1(IDM-1))         
               DO J=1,IDM-1                                             
                  DO I=1,IDM-1                                          
                     AU(I,J)=0.0                                        
                     SM(I,J)=0.0                                        
                  ENDDO                                                 
               ENDDO                                                    
               DO J=1,IDM-1                                             
                  SM(J,J)=EP2(J+1,1,NHEM)                               
                  AU(J,J)=EP1(J+1,1,NHEM)*EP1(J+1,1,NHEM) +             
     &                 EP2(J+1,1,NHEM)*EP2(J+1,1,NHEM)                  
                  IF(J.GT.1) AU(J-1,J)=EP1(J,1,NHEM)*EP2(J+1,1,NHEM)    
                  SM(J+1,J)=EP1(J+1,1,NHEM)                             
                  IF(J.LT.IDM-1) AU(J+1,J)=EP1(J+1,1,NHEM)*             
     &                 EP2(J+2,1,NHEM)                                  
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Invert matrix                                          
               !-----------------------------------------------------   
               CALL MATINV(AU,(IDM-1),(IDM-1),IWA1,WA1)                 
               !-----------------------------------------------------   
               ! Calculate PMNRE                                        
               !-----------------------------------------------------   
               DO I=1,IDM-1                                             
                  DO J=1,IDM                                            
                     TAL=0.0                                            
                     DO K=1,IDM-1                                       
                        TAL=TAL + AU(K,I)*SM(J,K)                       
                     ENDDO                                              
                     PMNRE(J,I,1)=TAL                                   
                  ENDDO                                                 
               ENDDO                                                    
               DEALLOCATE(AU)                                           
            ENDIF                                                       
         ENDIF                                                          
      ENDIF                                                             
                                                                        
      END                                                               
!=====================================================                  
! Initialise spectral arrays                                            
!=====================================================                  
      SUBROUTINE INISP                                                  
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Switches counters and constants controlling type and frequency of     
! model output                                                          
!-----------------------------------------------------                  
      COMMON/OUTCON/RNTAPE,DAY,SQR2,RSQR2,EAM1,EAM2,TOUT1               
     &     ,TOUT2,LSPO,LGPO                                             
     &     ,RNTAPO                                                      
     &     ,LSHIST,LMINIH                                               
      COMMON/OUTCONINT/NCOEFF,NLAT,INLAT,INSPC,KOUNTP                   
     &                 ,KOUNTE,KOUNTH,KOUNTR,KOUTP                      
     &                 ,KOUTE,KOUTH,KOUTR                               
! Identyfying run number                                                
      REAL :: RNTAPE                                                    
! Maximum wavenumber of printed spectral coeffecients                   
      INTEGER :: NCOEFF                                                 
! Number of latitudes in printed grid point fields                      
      INTEGER :: NLAT                                                   
      INTEGER :: INLAT                                                  
!                                                                       
      INTEGER :: INSPC                                                  
! Number of timsteps between successive writing of spectral             
! and gridded fields                                                    
      INTEGER :: KOUNTP                                                 
      INTEGER :: KOUNTE                                                 
! Number of timsteps between successive writing of history              
! records, containing spectral fields at the present and                
! previous time levels.                                                 
      INTEGER :: KOUNTH                                                 
! Number of timsteps between successive writing of restart              
! records, containing the spectral fields at the present and            
! previous time levels                                                  
      INTEGER :: KOUNTR                                                 
      INTEGER :: KOUTP                                                  
      INTEGER :: KOUTE                                                  
      INTEGER :: KOUTH                                                  
      INTEGER :: KOUTR                                                  
      REAL :: DAY                                                       
      REAL :: SQR2                                                      
      REAL :: RSQR2                                                     
      REAL :: EAM1                                                      
      REAL :: EAM2                                                      
      REAL :: TOUT1                                                     
      REAL :: TOUT2                                                     
      LOGICAL :: LSPO(NL)                                               
      LOGICAL :: LGPO(NL)                                               
      REAL :: RNTAPO                                                    
      LOGICAL :: LSHIST                                                 
! Reduce the history file to single precision if true                   
      LOGICAL :: LMINIH                                                 
!-----------------------------------------------------                  
! Restoration temperature field and constants which determine it,       
! also contains timescales                                              
!-----------------------------------------------------                  
      COMMON/RESTIJ/TTRES                                               
     + ,DTNS,DTEP,DTTRP,FAC,DDAMP,TFRC,YRLEN,TRS                        
     +  ,ALR,ZTROP,TGR                                                  
      COMPLEX :: TTRES(NNTx,MMTy,NHEM,NL)                               
      REAL :: DTNS                                                      
      REAL :: DTEP                                                      
      REAL :: DTTRP                                                     
      REAL :: FAC(NL)                                                   
      REAL :: DDAMP(NL)                                                 
      REAL :: TFRC(NL)                                                  
      REAL :: YRLEN                                                     
      REAL :: TRS(NL)                                                   
      REAL :: ALR                                                       
      REAL :: ZTROP                                                     
      REAL :: TGR                                                       
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
! Loop variables                                                        
      INTEGER :: L                                                      
!-----------------------------------------------------                  
! Initialise temperature and vorticity                                  
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         DO L=1,NL                                                      
            T(1,1,1,L)=T(1,1,1,L)+SQR2*(TRS(L)-T0(L))                   
            TMI(1,1,1,L)=TMI(1,1,1,L)+SQR2*(TRS(L)-T0(L))               
         ENDDO                                                          
         DO L=1,NL                                                      
            Z(1,1,1,L)=Z(1,1,1,L)+EZ                                    
            ZMI(1,1,1,L)=ZMI(1,1,1,L)+EZ                                
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Calculates a value for PQSVAL.                                        
!=====================================================                  
       SUBROUTINE INIQS                                                 
       IMPLICIT NONE                                                    
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Used for PQSAT function                                               
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: NQSTAB=50000                                
      COMMON /QSTABS/ PQSVAL                                            
      REAL :: PQSVAL(NQSTAB)                                            
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
       REAL :: t                                                        
! Loop variable                                                         
       INTEGER :: I                                                     
!-----------------------------------------------------                  
! Calculate  PQSVAL                                                     
!-----------------------------------------------------                  
      do i=1,nqstab                                                     
        t=REAL(i)/1e5                                                   
        PQSVAL(i)=ESCONA*EXP(-ESCONB/T)                                 
      enddo                                                             
      END                                                               
!=====================================================                  
! Subroutine to initialise the surface model from history on channel    
! 18.                                                                   
!=====================================================                  
      SUBROUTINE INISURF                                                
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Grid variables for DGRMLT loop                                        
!-----------------------------------------------------                  
      COMMON/GRIDPC/ASSBL,ASHBL,ASLBL,ARRCR,ARRLR                       
     &     ,TTVD,QTVD,TYBL,TXBL,TTRD,TTLR,TTCR,UTVD,VTVD                
     &     ,QTLR,QTCR,SSBL,SHBL,SLBL,RRCR,RRLR                          
     &     ,ARFLUX,ASFLD,ACLD,RFLUX,SFLD,CLD                            
      REAL :: ASSBL(MGx,NHEM,JGy)                                       
      REAL :: ASHBL(MGx,NHEM,JGy)                                       
      REAL :: ASLBL(MGx,NHEM,JGy)                                       
      REAL :: ARRCR(MGx,NHEM,JGy)                                       
      REAL :: ARRLR(MGx,NHEM,JGy)                                       
      REAL :: TTVD(MGx,NHEM,NL)                                         
      REAL :: QTVD(MGx,NHEM,NL)                                         
      REAL :: TYBL(MGx,NHEM)                                            
      REAL :: TXBL(MGx,NHEM)                                            
      REAL :: TTRD(MGx,NHEM,NL)                                         
      REAL :: TTLR(MGx,NHEM,NL)                                         
      REAL :: TTCR(MGx,NHEM,NL)                                         
      REAL :: UTVD(MGx,NHEM,NL)                                         
      REAL :: VTVD(MGx,NHEM,NL)                                         
      REAL :: QTLR(MGx,NHEM,NL)                                         
      REAL :: QTCR(MGx,NHEM,NL)                                         
      REAL :: SSBL(MGx,NHEM,JGy)                                        
      REAL :: SHBL(MGx,NHEM,JGy)                                        
      REAL :: SLBL(MGx,NHEM,JGy)                                        
      REAL :: RRCR(MGx,NHEM,JGy)                                        
      REAL :: RRLR(MGx,NHEM,JGy)                                        
      REAL :: arflux(MGx,NHEM,JGy,6)                                    
      REAL :: asfld(MGx,NHEM,JGy,6)                                     
      REAL :: acld(MGx,NHEM,JGy,4)                                      
      REAL :: rflux(MGx,NHEM,JGy,6)                                     
      REAL :: sfld(MGx,NHEM,JGy,6)                                      
      REAL :: cld(MGx,NHEM,JGy,4)                                       
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Radiation variables                                                   
!-----------------------------------------------------                  
      COMMON/CPIERS/ICFLAG,CFRAC,PNET,SNET,RRFLUX,TTSW,TTLW             
      INTEGER :: ICFLAG(MGx,NHEM,5,2)                                   
      REAL :: CFRAC(MGx,NHEM,5)                                         
      REAL :: PNET(MGx,NHEM,JGy)                                        
      REAL :: SNET(MGx,NHEM,JGy)                                        
      REAL :: RRFLUX(MGx,NHEM,JGy,6)                                    
      REAL :: TTSW(MGx,NHEM,NL)                                         
      REAL :: TTLW(MGx,NHEM,NL)                                         
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Orogrophy                                                             
!-----------------------------------------------------                  
      COMMON/GSG/GSG,TOPVAR,RLEVBZMO,FXZMO,ofrac,runoff,iareanum        
! Topog variance, GW diags for Orog, Non orog waves                     
      REAL :: TOPVAR(MGx,NHEM,JGy)                                      
      REAL :: RLEVBZMO(JGy,NHEM,NL)                                     
      REAL :: FXZMO(JGy,NHEM,NL)                                        
         REAL :: GSG(MGx,NHEM,JGy)                                      
         REAL :: ofrac(MGx,NHEM,JGy)                                    
         INTEGER :: runoff(50,2,33)                                     
         INTEGER :: iareanum                                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
C OASIS size parameters                                                 
      INTEGER, PARAMETER :: IMB = 128                                   
      INTEGER, PARAMETER :: JM = 64                                     
      INTEGER, PARAMETER :: IMJM = IMB*JM                               
      common/oasiso2a/zsst(imjm),zscur(imjm),mscur(imjm)                
     $   ,zsst_2d(MGx,NHEM,JGy)                                         
     $   ,zscur_2d(MGx,NHEM,JGy),mscur_2d(MGx,NHEM,JGy)                 
      real zsst,zsst_2d,zscur,zscur_2d,mscur,mscur_2d                   
      common/oasisa2o/swlw(imjm),heatlat(imjm),water(imjm)              
     $   ,sensible(imjm),taux(imjm),tauy(imjm),exwater(imjm)            
     $   ,exwat(MGx,NHEM,JGy),sensible2(imjm),sensible3(imjm)           
     $   ,oceaniceflux(MGx,NHEM),exiwat(MGx,NHEM,JGy),exiwater(imjm)    
     $   ,exiwatn(MGx,NHEM,JGy),exiwatern(imjm)                         
     $   ,tstaroo(MGx,NHEM,JGy),tstarf(MGx,NHEM,JGy)                    
     $   ,qstarf(MGx,NHEM,JGy),sqstarf(MGx,NHEM,JGy)                    
     $   ,swlw2d(MGx,NHEM,JGy),heatlat2d(MGx,NHEM,JGy)                  
     $   ,water2d(MGx,NHEM,JGy)                                         
     $   ,sensible2d(MGx,NHEM,JGy),taux2d(MGx,NHEM,JGy)                 
     $   ,tauy2d(MGx,NHEM,JGy)                                          
     $   ,SHBLC(MGx,NHEM,JGy),SLBLC(MGx,NHEM,JGy)                       
     $   ,exwater2d(MGx,NHEM,JGy),sensible2_2d(MGx,NHEM,JGy)            
     $   ,sensible3_2d(MGx,NHEM,JGy)                                    
     $   ,exiwater2d(MGx,NHEM,JGy),exiwatern2d(MGx,NHEM,JGy)            
      real swlw,heatlat,water,sensible,taux,tauy,shblc,slblc,exwater    
     $,exwat,sensible2,sensible3,oceaniceflux,exiwat,exiwater           
     $,exiwatn,exiwatern,tstaroo,tstarf,qstarf,sqstarf                  
     $,swlw2d,heatlat2d,water2d,sensible2d,taux2d,tauy2d                
     $,exwater2d,sensible2_2d,sensible3_2d,exiwater2d,exiwatern2d       
                                                                        
      common/wat/water2(imjm), water2_2d(MGx,NHEM,JGy)                  
C Arrays wathbl, watlsl, watsbl are used for balancing water in IGCM4   
C ashbl, aslbl, assbl were used in IGCM3 FORTE but these are already    
C in use in elsewhere in IGCM4 for surface flux diagnostics             
     $   ,wathbl(MGx,NHEM,JGy),watlbl(MGx,NHEM,JGy),watsbl(MGx,NHEM,JGy)
     $   ,sicecov(MGx,NHEM,JGy)                                         
      real water2,water2_2d,wathbl,watlbl,watsbl,sicecov                
                                                                        
                                                                        
!-----------------------------------------------------                  
! External function                                                     
!-----------------------------------------------------                  
      REAL, EXTERNAL :: PQSAT                                           
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL, ALLOCATABLE :: SvegeFull(:,:,:)                             
      REAL, ALLOCATABLE :: SurfVarsFull(:,:,:,:)                        
      REAL, ALLOCATABLE :: SurfVars(:,:,:,:)                            
      REAL :: DAYNEAR                                                   
      REAL :: RCHECK                                                    
      REAL :: RRKOUNT                                                   
      REAL :: RM1TAPE                                                   
      REAL :: RDAY                                                      
      REAL :: CDOY                                                      
      REAL :: RM2TAPE                                                   
      REAL :: SVLAT                                                     
      REAL :: SALOOK(24)                                                
! Loop variables                                                        
      INTEGER :: I, IHEM, J                                             
!-----------------------------------------------------                  
! Lookup table for vegetation to albedo: from Unified Model Doc. no. 70 
! MMJ April 2000                                                        
! Snow free lookup table                                                
!-----------------------------------------------------                  
                                                                        
C Vegetation-albedo tuned to observations and lack of aerosol           
                                                                        
       DATA SALOOK /0.10, 0.80, 0.20,                                   
     & 0.25, 0.25, 0.25, 0.25, 0.25, 0.25, 0.25,                        
     & 0.25, 0.25, 0.25, 0.25,                                          
     & 0.25, 0.25, 0.25, 0.25, 0.25, 0.25, 0.25,                        
     & 0.50, 0.20, 0.30/                                                
                                                                        
!-----------------------------------------------------                  
! Read in data for RCHECK                                               
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         DAYNEAR=0.                                                     
         OPEN(UNIT=18,FILE='RestartSurface.dat',FORM='UNFORMATTED')     
 101     read(18,end=200) RCHECK                                        
         if (abs(RCHECK+999.999).gt.0.0001) then                        
            write (2,*) 'Wrong type of surface restart ',               
     &           rcheck,abs(rcheck+999.999)                             
            call abort                                                  
         endif                                                          
         !-----------------------------------------------------         
         ! Read the remaining data from unit 18                         
         !-----------------------------------------------------         
       if(daynear.lt.1.0E-5)ALLOCATE(SurfVarsFull(MGPP,NHEM,JG,17))     
         READ(18) rrkount,rm1tape,rday,CDOY,SurfVarsFull,RM2TAPE        
         !-----------------------------------------------------         
         ! Check that RM1TAPE and RM2TAPE are consistent                
         !-----------------------------------------------------         
         if (RM1TAPE.ne.RM2TAPE) then                                   
            write (2,*) 'Surface (SOIL) restart record wrong.'          
            write (2,*) 'RM1TAPE != RM2TAPE',RM1TAPE,RM2TAPE            
            call abort                                                  
         endif                                                          
         !-----------------------------------------------------         
         ! Check that RDAY and BEGDAY agree                             
         !-----------------------------------------------------         
         IF (ABS(RDAY-BEGDAY).GT.1.0E-2)THEN                            
            IF (ABS(RDAY-BEGDAY).LT.(ABS(DAYNEAR-BEGDAY)))THEN          
               DAYNEAR=RDAY                                             
            ENDIF                                                       
            GOTO 101                                                    
         ENDIF                                                          
         CLOSE(18)                                                      
         !-----------------------------------------------------         
         ! Check that CDOY and DOY agree                                
         !-----------------------------------------------------         
         if (CDOY.NE.DOY) then                                          
            write (2,*) 'Surface DOY not equal to dynamical DOY.'       
            write (2,*) cdoy,doy                                        
            write (2,*) 'Value from dynamics used.'                     
         endif                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Pass RDAY to the other PEs                                            
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         CALL MPI_BARRIER(MPI_COMM_WORLD,IERR)                          
         nBuffSize=1                                                    
         iSend=0                                                        
         CALL MPI_BCAST(RDAY,nBuffSize,ParaReal,iSend,                  
     &        MPI_COMM_WORLD,ierr)                                      
      ENDIF                                                             
!-----------------------------------------------------                  
! Share the surface variables across PEs                                
!-----------------------------------------------------                  
CCC      ALLOCATE(SurfVars(MGx,NHEM,JGy,11))                            
CCC      CALL SHAREGRID(.TRUE.,MGPP,11,SurfVarsFull,SurfVars)           
      ALLOCATE(SurfVars(MGx,NHEM,JGy,17))                               
      CALL SHAREGRID(.TRUE.,MGPP,17,SurfVarsFull,SurfVars)              
      IF (MyPe.EQ.0) DEALLOCATE(SurfVarsFull)                           
!-----------------------------------------------------                  
! Write to the surface variables                                        
!-----------------------------------------------------                  
      TSTAR(:,:,:)=SurfVars(:,:,:,1)                                    
      TDEEP(:,:,:)=SurfVars(:,:,:,2)                                    
      SMSTAR(:,:,:)=SurfVars(:,:,:,3)                                   
      QSTAR(:,:,:)=SurfVars(:,:,:,4)                                    
      HSNOW(:,:,:)=SurfVars(:,:,:,5)                                    
      SQSTAR(:,:,:)=SurfVars(:,:,:,6)                                   
      SALB(:,:,:)=SurfVars(:,:,:,7)                                     
      SBAL(:,:,:)=SurfVars(:,:,:,8)                                     
      TSTARO(:,:,:)=SurfVars(:,:,:,9)                                   
      TDEEPO(:,:,:)=SurfVars(:,:,:,10)                                  
      SNET(:,:,:)=SurfVars(:,:,:,11)                                    
      TSTAROO(:,:,:)=SurfVars(:,:,:,12)                                 
      TSTARF(:,:,:)=SurfVars(:,:,:,13)                                  
      zsst_2d(:,:,:)=SurfVars(:,:,:,14)                                 
      SICECOV(:,:,:)=SurfVars(:,:,:,15)                                 
      zscur_2d(:,:,:)=SurfVars(:,:,:,16)                                
      mscur_2d(:,:,:)=SurfVars(:,:,:,17)                                
C                                                                       
      DEALLOCATE(SurfVars)                                              
!-----------------------------------------------------                  
! Read in vegetation data                                               
!-----------------------------------------------------                  
      if(mype.eq.0)then                                                 
      ALLOCATE(SvegeFull(MGPP,NHEM,JG))                                 
      OPEN(UNIT=31,FILE='vegetation.dat',FORM='FORMATTED')              
      READ(31,*) SvegeFull                                              
      CLOSE(31)                                                         
      endif                                                             
      CALL SHAREGRID(.TRUE.,MGPP,1,SvegeFull,SVEGE)                     
      if(mype.eq.0)DEALLOCATE(SvegeFull)                                
      if(npe.gt.1)CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                
                                                                        
! Albedo field initialised from vegetation index                        
!-----------------------------------------------------                  
! Leave this routine                                                    
!-----------------------------------------------------                  
      RETURN                                                            
!-----------------------------------------------------                  
! Only get here if the file has been read beyond                        
!-----------------------------------------------------                  
 200  write (2,*) 'EOF on surface restart file'                         
      write (2,*) 'Looking for day ',begday,' but closest was ',        
     $     daynear                                                      
      CALL ABORT                                                        
      END                                                               
!=====================================================                  
! Temperature and surface pressure is obtained from relative vorticity  
!=====================================================                  
      SUBROUTINE BALANC                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Constants and arrays needed for balancing                             
!-----------------------------------------------------                  
      COMMON/BALAN/BFILT,RGT0,RG,TMEAN                                  
     +            ,EP1,EP2,KBAL,SRGT0,LTBAL                             
     &            ,PMNRE                                                
! Maximum zonal wavenumbers used for balancing                          
      INTEGER, PARAMETER :: MFTBAL=9                                    
! Number of zonal wavenumbers used for balancing                        
      INTEGER, PARAMETER :: NFTBAL=1+MFTBAL/MOCT                        
      REAL :: BFILT(NL)                                                 
      REAL :: RGT0(NL)                                                  
      REAL :: RG(NL,NL)                                                 
      REAL :: TMEAN(NL)                                                 
      REAL :: EP1(IDM,NFTBAL,NHEM)                                      
      REAL :: EP2(IDM,NFTBAL,NHEM)                                      
      INTEGER :: KBAL                                                   
      REAL :: SRGT0                                                     
      LOGICAL :: LTBAL                                                  
      REAL :: PMNRE(IDM,IDM,3)                                          
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      COMPLEX :: GP(NL)                                                 
      COMPLEX :: TA,GSI1,VPS,TK,SRGT                                    
      INTEGER :: IN                                                     
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, J, L, M, MP, nT                           
!-----------------------------------------------------                  
! Note: 2-grid vertical temperature wave is not removed if orography    
!       is included. Program aborts in INITAL if attempted.             
!-----------------------------------------------------                  
      DO IHEM=1,NHEM                                                    
         DO IM=1,MMTyy(iRow)                                            
            DO I=1,NNTxx(iCol)                                          
               IN=nWave(I,IM,iCol)                                      
               !---------------------------------------------------     
               ! Skip the code below for 1st zonal wavenum' & ihem=1    
               !---------------------------------------------------     
               IF((IN+IHEM).GT.1) THEN                                  
                  VPS=VP(I,IM,IHEM)                                     
                  GSI1=GS(I,IM,IHEM)                                    
                  DO L=1,NL                                             
                     TA=(0.,0.)                                         
                     DO M=1,NL                                          
                        TA=TA+G(M,L)*TT(I,IM,IHEM,M)                    
                     ENDDO                                              
                     TA=(T0(L)*VPS-TA)*DELT -                           
     &                    RSQ(IN+IHEM)*DT(I,IM,IHEM,L)                  
                     GP(L)=TA-GSI1                                      
                  ENDDO                                                 
                  SRGT=(0.,0.)                                          
                  DO L=1,NL                                             
                     TK=(0.,0.)                                         
                     DO M=1,NL                                          
                        TK=TK+RG(M,L)*GP(M)                             
                     ENDDO                                              
                     T(I,IM,IHEM,L)=TK                                  
                     SRGT=SRGT+BFILT(L)*TK                              
                  ENDDO                                                 
                  SRGT=SRGT/SRGT0                                       
                  SP(I,IM,IHEM)=SRGT                                    
                  DO L=1,NL                                             
                     T(I,IM,IHEM,L)=T(I,IM,IHEM,L)-RGT0(L)*SRGT         
                  ENDDO                                                 
               ENDIF                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
                                                                        
      IF (KOUNT.EQ.0) THEN                                              
         DO IHEM=1,NHEM                                                 
            DO IM=1,MMTyy(iRow)                                         
               DO IN=1,NNTxx(iCol)                                      
                  SPMI(IN,IM,IHEM)=SP(IN,IM,IHEM)                       
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMTyy(iRow)                                      
                  DO IN=1,NNTxx(iCol)                                   
                     ZMI(IN,IM,IHEM,L)=Z(IN,IM,IHEM,L)                  
                     DMI(IN,IM,IHEM,L)=D(IN,IM,IHEM,L)                  
                     TMI(IN,IM,IHEM,L)=T(IN,IM,IHEM,L)                  
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! Surface layer fluxes for present latitude.                            
! Bulk aerodynamice formulation for lowest level tendencies.            
!=====================================================                  
      SUBROUTINE BLAYER(JH)                                             
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Grid variables for DGRMLT loop                                        
!-----------------------------------------------------                  
      COMMON/GRIDPC/ASSBL,ASHBL,ASLBL,ARRCR,ARRLR                       
     &     ,TTVD,QTVD,TYBL,TXBL,TTRD,TTLR,TTCR,UTVD,VTVD                
     &     ,QTLR,QTCR,SSBL,SHBL,SLBL,RRCR,RRLR                          
     &     ,ARFLUX,ASFLD,ACLD,RFLUX,SFLD,CLD                            
      REAL :: ASSBL(MGx,NHEM,JGy)                                       
      REAL :: ASHBL(MGx,NHEM,JGy)                                       
      REAL :: ASLBL(MGx,NHEM,JGy)                                       
      REAL :: ARRCR(MGx,NHEM,JGy)                                       
      REAL :: ARRLR(MGx,NHEM,JGy)                                       
      REAL :: TTVD(MGx,NHEM,NL)                                         
      REAL :: QTVD(MGx,NHEM,NL)                                         
      REAL :: TYBL(MGx,NHEM)                                            
      REAL :: TXBL(MGx,NHEM)                                            
      REAL :: TTRD(MGx,NHEM,NL)                                         
      REAL :: TTLR(MGx,NHEM,NL)                                         
      REAL :: TTCR(MGx,NHEM,NL)                                         
      REAL :: UTVD(MGx,NHEM,NL)                                         
      REAL :: VTVD(MGx,NHEM,NL)                                         
      REAL :: QTLR(MGx,NHEM,NL)                                         
      REAL :: QTCR(MGx,NHEM,NL)                                         
      REAL :: SSBL(MGx,NHEM,JGy)                                        
      REAL :: SHBL(MGx,NHEM,JGy)                                        
      REAL :: SLBL(MGx,NHEM,JGy)                                        
      REAL :: RRCR(MGx,NHEM,JGy)                                        
      REAL :: RRLR(MGx,NHEM,JGy)                                        
      REAL :: arflux(MGx,NHEM,JGy,6)                                    
      REAL :: asfld(MGx,NHEM,JGy,6)                                     
      REAL :: acld(MGx,NHEM,JGy,4)                                      
      REAL :: rflux(MGx,NHEM,JGy,6)                                     
      REAL :: sfld(MGx,NHEM,JGy,6)                                      
      REAL :: cld(MGx,NHEM,JGy,4)                                       
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed argument                                                       
!-----------------------------------------------------                  
! Latitude index                                                        
      INTEGER, INTENT(IN) :: JH                                         
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: RCSJ, SQRC                                                
      REAL :: CTAU, CSH                                                 
      REAL :: DRAGJ                                                     
      REAL :: VM, VMP                                                   
      REAL :: CVM                                                       
      REAL :: THNL                                                      
      REAL :: QSURF                                                     
      REAL :: DTH                                                       
      REAL :: DQ                                                        
! Loop variables                                                        
      INTEGER :: I, IHEM                                                
!-----------------------------------------------------                  
!                                                                       
!-----------------------------------------------------                  
      RCSJ=SECSQ(JJ1(iRow)+JH,1)                                        
      SQRC=SQRT(RCSJ)                                                   
      DO IHEM=1,NHEM                                                    
         DO I=1,MGxx(iCol)                                              
            BLCD(I,IHEM)=CD                                             
            CTAU=CD/TSTAR(I,IHEM,JH)                                    
            CSH =CTAU/AKAP                                              
            DRAGJ=CD*DRAG/TSTAR(I,IHEM,JH)                              
                                                                        
            VM=SQRT(RCSJ*(UG3(I,IHEM,JH,NL)*UG3(I,IHEM,JH,NL)+          
     &           VG3(I,IHEM,JH,NL)*VG3(I,IHEM,JH,NL)))                  
            VMP=VM+BLVAD                                                
            CVM=DRAGJ*VMP                                               
            FUG3(I,IHEM,JH,NL)=-CVM*UG3(I,IHEM,JH,NL)                   
            FVG3(I,IHEM,JH,NL)=-CVM*VG3(I,IHEM,JH,NL)                   
            CVM=CTAU*PLG3(I,IHEM,JH)*VMP*SQRC                           
            TXBL(I,IHEM)=CVM*UG3(I,IHEM,JH,NL)                          
            TYBL(I,IHEM)=CVM*VG3(I,IHEM,JH,NL)                          
            THNL=TG3(I,IHEM,JH,NL)/SKAP(NL)                             
            !-----------------------------------------------------      
            ! The next line prevents condensation onto a warm surface,  
            ! which doesn't happen in reality due to the surface        
            ! effectively becoming saturated after a very small amount  
            ! of condensation has occurred                              
            !-----------------------------------------------------      
            QSURF=QSTAR(I,IHEM,JH)                                      
            IF (.NOT.LCOND) THEN                                        
               QSURF=MIN(SQSTAR(I,IHEM,JH),                             
     &              MAX(QSTAR(I,IHEM,JH),TRAG3(I,IHEM,JH,NL,1)))        
            ENDIF                                                       
            DTH=TSTAR(I,IHEM,JH)-THNL                                   
            DQ=BLRH*(QSURF-TRAG3(I,IHEM,JH,NL,1))                       
            BLVB(I,IHEM)=BLA*SQRT(ABS(DTH)/                             
     &           (0.5*(THNL+TSTAR(I,IHEM,JH))))                         
            IF (DTH.LT.0.0) THEN                                        
               BLVB(I,IHEM)=0.0                                         
            END IF                                                      
            VMP=VM+BLVB(I,IHEM)                                         
            CVM=0.2*DRAGJ*VMP                                           
            TNLG3(I,IHEM,JH,NL)=CVM*DTH                                 
            TRANLG3(I,IHEM,JH,NL,1)=CVM*DQ                              
            CVM=0.2*CSH*PLG3(I,IHEM,JH)*VMP                             
            SHBL(I,IHEM,JH)=CVM*DTH                                     
            !-----------------------------------------------------      
            ! Choose appropriate value of latent heat based on          
            ! ground T, ie, we assume it snows if the ground is         
            ! colder than 273.15K                                       
            !-----------------------------------------------------      
            IF (tstar(I,IHEM,JH).gt.0.363224029) THEN                   
               slbl(I,IHEM,JH)=cvm*ctq*dq                               
            ELSE                                                        
               slbl(I,IHEM,JH)=cvm*ctqi*dq                              
            ENDIF                                                       
            UG3(I,IHEM,JH,NL)=UG3(I,IHEM,JH,NL)+                        
     &           DELT2C*FUG3(I,IHEM,JH,NL)                              
            VG3(I,IHEM,JH,NL)=VG3(I,IHEM,JH,NL)+                        
     &           DELT2C*FVG3(I,IHEM,JH,NL)                              
            TG3(I,IHEM,JH,NL)=TG3(I,IHEM,JH,NL)+                        
     &           DELT2C*TNLG3(I,IHEM,JH,NL)                             
            TRAG3(I,IHEM,JH,NL,1)=TRAG3(I,IHEM,JH,NL,1)+                
     &           DELT2C*TRANLG3(I,IHEM,JH,NL,1)                         
         ENDDO                                                          
      ENDDO                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! Perform time-splitting integration of the boundary layer and          
! Surface scheme together for a whole timestep                          
! In the case of LSL or LOC being false, should forget the              
! timesplitting and just do the BL stuff (leave the surface alone)      
!=====================================================                  
      SUBROUTINE BLSURF(JH)                                             
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Grid variables for DGRMLT loop                                        
!-----------------------------------------------------                  
      COMMON/GRIDPC/ASSBL,ASHBL,ASLBL,ARRCR,ARRLR                       
     &     ,TTVD,QTVD,TYBL,TXBL,TTRD,TTLR,TTCR,UTVD,VTVD                
     &     ,QTLR,QTCR,SSBL,SHBL,SLBL,RRCR,RRLR                          
     &     ,ARFLUX,ASFLD,ACLD,RFLUX,SFLD,CLD                            
      REAL :: ASSBL(MGx,NHEM,JGy)                                       
      REAL :: ASHBL(MGx,NHEM,JGy)                                       
      REAL :: ASLBL(MGx,NHEM,JGy)                                       
      REAL :: ARRCR(MGx,NHEM,JGy)                                       
      REAL :: ARRLR(MGx,NHEM,JGy)                                       
      REAL :: TTVD(MGx,NHEM,NL)                                         
      REAL :: QTVD(MGx,NHEM,NL)                                         
      REAL :: TYBL(MGx,NHEM)                                            
      REAL :: TXBL(MGx,NHEM)                                            
      REAL :: TTRD(MGx,NHEM,NL)                                         
      REAL :: TTLR(MGx,NHEM,NL)                                         
      REAL :: TTCR(MGx,NHEM,NL)                                         
      REAL :: UTVD(MGx,NHEM,NL)                                         
      REAL :: VTVD(MGx,NHEM,NL)                                         
      REAL :: QTLR(MGx,NHEM,NL)                                         
      REAL :: QTCR(MGx,NHEM,NL)                                         
      REAL :: SSBL(MGx,NHEM,JGy)                                        
      REAL :: SHBL(MGx,NHEM,JGy)                                        
      REAL :: SLBL(MGx,NHEM,JGy)                                        
      REAL :: RRCR(MGx,NHEM,JGy)                                        
      REAL :: RRLR(MGx,NHEM,JGy)                                        
      REAL :: arflux(MGx,NHEM,JGy,6)                                    
      REAL :: asfld(MGx,NHEM,JGy,6)                                     
      REAL :: acld(MGx,NHEM,JGy,4)                                      
      REAL :: rflux(MGx,NHEM,JGy,6)                                     
      REAL :: sfld(MGx,NHEM,JGy,6)                                      
      REAL :: cld(MGx,NHEM,JGy,4)                                       
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Radiation variables                                                   
!-----------------------------------------------------                  
      COMMON/CPIERS/ICFLAG,CFRAC,PNET,SNET,RRFLUX,TTSW,TTLW             
      INTEGER :: ICFLAG(MGx,NHEM,5,2)                                   
      REAL :: CFRAC(MGx,NHEM,5)                                         
      REAL :: PNET(MGx,NHEM,JGy)                                        
      REAL :: SNET(MGx,NHEM,JGy)                                        
      REAL :: RRFLUX(MGx,NHEM,JGy,6)                                    
      REAL :: TTSW(MGx,NHEM,NL)                                         
      REAL :: TTLW(MGx,NHEM,NL)                                         
!-----------------------------------------------------                  
! Orogrophy                                                             
!-----------------------------------------------------                  
      COMMON/GSG/GSG,TOPVAR,RLEVBZMO,FXZMO,ofrac,runoff,iareanum        
! Topog variance, GW diags for Orog, Non orog waves                     
      REAL :: TOPVAR(MGx,NHEM,JGy)                                      
      REAL :: RLEVBZMO(JGy,NHEM,NL)                                     
      REAL :: FXZMO(JGy,NHEM,NL)                                        
         REAL :: GSG(MGx,NHEM,JGy)                                      
         REAL :: ofrac(MGx,NHEM,JGy)                                    
         INTEGER :: runoff(50,2,33)                                     
         INTEGER :: iareanum                                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
C OASIS size parameters                                                 
      INTEGER, PARAMETER :: IMB = 128                                   
      INTEGER, PARAMETER :: JM = 64                                     
      INTEGER, PARAMETER :: IMJM = IMB*JM                               
      common/oasisa2o/swlw(imjm),heatlat(imjm),water(imjm)              
     $   ,sensible(imjm),taux(imjm),tauy(imjm),exwater(imjm)            
     $   ,exwat(MGx,NHEM,JGy),sensible2(imjm),sensible3(imjm)           
     $   ,oceaniceflux(MGx,NHEM),exiwat(MGx,NHEM,JGy),exiwater(imjm)    
     $   ,exiwatn(MGx,NHEM,JGy),exiwatern(imjm)                         
     $   ,tstaroo(MGx,NHEM,JGy),tstarf(MGx,NHEM,JGy)                    
     $   ,qstarf(MGx,NHEM,JGy),sqstarf(MGx,NHEM,JGy)                    
     $   ,swlw2d(MGx,NHEM,JGy),heatlat2d(MGx,NHEM,JGy)                  
     $   ,water2d(MGx,NHEM,JGy)                                         
     $   ,sensible2d(MGx,NHEM,JGy),taux2d(MGx,NHEM,JGy)                 
     $   ,tauy2d(MGx,NHEM,JGy)                                          
     $   ,SHBLC(MGx,NHEM,JGy),SLBLC(MGx,NHEM,JGy)                       
     $   ,exwater2d(MGx,NHEM,JGy),sensible2_2d(MGx,NHEM,JGy)            
     $   ,sensible3_2d(MGx,NHEM,JGy)                                    
     $   ,exiwater2d(MGx,NHEM,JGy),exiwatern2d(MGx,NHEM,JGy)            
      real swlw,heatlat,water,sensible,taux,tauy,shblc,slblc,exwater    
     $,exwat,sensible2,sensible3,oceaniceflux,exiwat,exiwater           
     $,exiwatn,exiwatern,tstaroo,tstarf,qstarf,sqstarf                  
     $,swlw2d,heatlat2d,water2d,sensible2d,taux2d,tauy2d                
     $,exwater2d,sensible2_2d,sensible3_2d,exiwater2d,exiwatern2d       
                                                                        
      common/wat/water2(imjm), water2_2d(MGx,NHEM,JGy)                  
C Arrays wathbl, watlsl, watsbl are used for balancing water in IGCM4   
C ashbl, aslbl, assbl were used in IGCM3 FORTE but these are already    
C in use in elsewhere in IGCM4 for surface flux diagnostics             
     $   ,wathbl(MGx,NHEM,JGy),watlbl(MGx,NHEM,JGy),watsbl(MGx,NHEM,JGy)
     $   ,sicecov(MGx,NHEM,JGy)                                         
      real water2,water2_2d,wathbl,watlbl,watsbl,sicecov                
                                                                        
      common/oasiso2a/zsst(imjm),zscur(imjm),mscur(imjm)                
     $   ,zsst_2d(MGx,NHEM,JGy)                                         
     $   ,zscur_2d(MGx,NHEM,JGy),mscur_2d(MGx,NHEM,JGy)                 
      real zsst,zsst_2d,zscur,zscur_2d,mscur,mscur_2d                   
       REAL deltasnow                                                   
                                                                        
!-----------------------------------------------------                  
! Passed argument                                                       
!-----------------------------------------------------                  
! Latitude index                                                        
      INTEGER, INTENT(IN) :: JH                                         
!-----------------------------------------------------                  
! External function                                                     
!-----------------------------------------------------                  
      REAL, EXTERNAL :: PQSAT                                           
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: SZLOOK(24)                                                
! Set up values for reference pressure and various temperatures         
! in non-dimensionalised form                                           
      REAL, PARAMETER :: TZC=273.15                                     
      REAL :: dum                                                       
      LOGICAL :: LSLH,LOCH                                              
      INTEGER :: IFIRST                                                 
      DATA IFIRST/1/                                                    
      INTEGER :: Mpth1                                                  
      REAL :: ApFRAC                                                    
      REAL :: tp1, tz, tm2, tm3                                         
      REAL :: DELTL, DELTO                                              
      REAL :: ssmc23, ssmc13, srsum                                     
      REAL :: RCSJ, SQRC                                                
      REAL :: RSIGF                                                     
C      REAL :: RGZ0L, RGZ0O                                             
      REAL :: RGZ0L, RGZ0O, RGZ0IW, RGZ0IC                              
      INTEGER :: MAXITER                                                
      REAL :: DELTU                                                     
                                                                        
      INTEGER :: MPth2                                                  
      REAL :: RGZ0                                                      
      REAL :: TSCUR, TDCUR                                              
      REAL :: QCUR, SQCUR                                               
      REAL :: QNLGS, TNLGS                                              
      REAL :: UNLGS, VNLGS                                              
      REAL :: SHBLS, SLBLS                                              
      REAL :: BLCDS, BLVBS                                              
      REAL :: TXBLS, TYBLS                                              
      REAL :: SRH                                                       
      REAL :: THNL                                                      
      REAL :: THBAR                                                     
      REAL :: RZZ0, RLZZ02                                              
      REAL :: CTAU, CSH                                                 
      REAL :: DRAGJ                                                     
      REAL :: VM, VMP                                                   
      REAL :: CVM                                                       
      REAL :: DTH, DQ                                                   
      REAL :: SFC                                                       
      REAL :: rkappa                                                    
      REAL :: pc1, pc2                                                  
      REAL :: hc1, hc2                                                  
      REAL :: trans                                                     
      REAL :: rct1, rct2                                                
      REAL :: OCF                                                       
      REAL :: hc                                                        
      REAL :: dts                                                       
      real :: TGBSOC,QGBSOC,QGBSOC2,UGBSOC,VGBSOC                       
      real :: UNLGJNL,VNLGJNL,TNLGJNL,QNLGJNL                           
      real :: SHBLJ,SLBLJ,TXBLJ,TYBLJ,BLVBJ                             
      real :: stempbs0,stempbs3,tscurw,escon,snowexcess                 
      real :: restorice,tspd                                            
      real :: UG3CUR, VG3CUR,UGBSOCUR,VGBSOCUR                          
      integer :: ifsstj                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, iMon, ITER, L                                 
       REAL VMPCD                                                       
       REAL TICEFORM  ! Temperature at which ice forms                  
       REAL TICESNOW  ! Temperature at which ice covered with snow      
!-----------------------------------------------------                  
! Only need to read in flux data once                                   
!-----------------------------------------------------                  
      SAVE IFIRST, MPth1, MPth2                                         
!-----------------------------------------------------                  
! Lookup table for vegetation to Z0: from Unified Model Doc. no. 70     
! Snow free lookup table                            MMJ April 2000      
!-----------------------------------------------------                  
                                                                        
C Roughness length as a function of land surface type or ocean          
                                                                        
       DATA SZLOOK /0.0001,0.03,0.001,                                  
     & 0.1,0.1,0.1,0.1,0.1,0.1,0.1,                                     
     & 0.05,0.05,0.05,0.05,                                             
     & 0.05,0.05,0.05,0.05,0.05,0.05,0.05,                              
     & 0.03,0.03,0.03/                                                  
!-----------------------------------------------------                  
! THIS BIT READS IN RUN 1.0 (10 year ocean flux)                        
! 30-3-98  PMF                                                          
!-----------------------------------------------------                  
      IF (IFIRST.EQ.1.AND.LOC) THEN                                     
         IFIRST=0                                                       
      ENDIF                                                             
!-----------------------------------------------------                  
! SORT OUT TIMINGS                                                      
!-----------------------------------------------------                  
      IF (JH.EQ.1.AND.LOC) THEN                                         
         CALL CALNDR(DOY,Mpth1,ApFRAC)                                  
         MPth2=mpth1+1                                                  
         if (mpth2.eq.13) mpth2=1                                       
      ENDIF                                                             
      tp1=(tzc+1.)/ct                                                   
      tz=tzc/ct                                                         
      tm2=(tzc-1.9)/ct    ! Also done in T42.accum                      
C Temperature at which ice forms > -1.9C (brine freezing point)         
C because this is the average temperature of a 30m thick layer of ocean 
C Polar ocean is sensitive to this: keep between -1.5 and -0.5C.        
      TICEFORM=(tzc-1.5)/ct                                             
C Temperature at which albedo of ice is sasnow, also temperature        
C at which ice thickness is SHCI (2m) + thickness > -2C (1m)            
C which is maximum thickness observed in Arctic                         
      TICESNOW=(tzc-7.0)/ct                                             
C Timescale in days of restoring topmost ocean layer to -2C             
C 2 days appropriate if ocean cooled by intense mixing                  
      restorice=10.                                                     
      tspd=real(itspd)                                                  
C Warm ice roughness height > -2C < 0C                                  
      RGZ0IW=1.0E-2/2.2E4                                               
C Cold ice roughness height =< -2C                                      
      RGZ0IC=1.0E-2/2.2E4                                               
!-----------------------------------------------------                  
! Set up the number of timesplitting levels for land and ocean,         
! also the timesteps.                                                   
!-----------------------------------------------------                  
      DELTL=DELT2C/REAL(ITSLL)                                          
      DELTO=DELT2C/REAL(ITSLO)                                          
!-----------------------------------------------------                  
! Set up some more constants for the soil scheme                        
!-----------------------------------------------------                  
      ssmc23=ssmc*(2./3.)                                               
      ssmc13=ssmc*(1./3.)                                               
      srsum=sd1/skse+sd2/skse                                           
!-----------------------------------------------------                  
! And some for the BL scheme                                            
!-----------------------------------------------------                  
      RCSJ=SECSQ(JJ1(iRow)+JH,1)                                        
      SQRC=SQRT(RCSJ)                                                   
      RSIGF=2.*(1.0-SIGMA(NL))/(1.0+SIGMA(NL))                          
!-----------------------------------------------------                  
! Now begin looping over gridpoints                                     
!-----------------------------------------------------                  
      DO IHEM=1,NHEM                                                    
         DO I=1,MGxx(iCol)                                              
        TGBSOC=TG3(I,IHEM,JH,NL)                                        
        QGBSOC=TRAG3(I,IHEM,JH,NL,1)                                    
        QGBSOC2=TRAG3(I,IHEM,JH,NL,1)                                   
        UGBSOC=UG3(I,IHEM,JH,NL)                                        
        VGBSOC=VG3(I,IHEM,JH,NL)                                        
c not used                                                              
c      stempbs0=0.                                                      
c      do l=1,nl                                                        
c      stempbs0=stempbs0+TRAG3(I,IHEM,JH,L,1)*clr(l)                    
c      end do                                                           
            oceaniceflux(i,ihem)=0.0                                    
            !-----------------------------------------------------      
            ! Get land roughness lengths from vegetation index          
            !-----------------------------------------------------      
            RGZ0L=SZLOOK(NINT(SVEGE(I,IHEM,JH)))/2.2E4                  
            !-----------------------------------------------------      
            ! Now do a single gridpoint                                 
            !-----------------------------------------------------      
            LSLH=(LSL.and.(GSG(I,IHEM,jh).gt.0.))                       
            LOCH=(LOC.and.(GSG(I,IHEM,jh).le.0.))                       
            if (GSG(I,IHEM,jh).gt.0.) then                              
               MAXITER=ITSLL                                            
               DELTU=DELTL                                              
               RGZ0=RGZ0L                                               
               if (LSL) then                                            
                  TSCUR=TSTARO(I,IHEM,JH)                               
                  TDCUR=TDEEPO(I,IHEM,JH)                               
               else                                                     
                  TSCUR=TSTAR(I,IHEM,JH)                                
                  QCUR=QSTAR(I,IHEM,JH)                                 
                  SQCUR=SQSTAR(I,IHEM,JH)                               
               endif                                                    
            else                                                        
                                                                        
c put ssts in here now, Arctic special case first: polewards of 86.5N   
C which is one gridpoint at T42                                         
                                                                        
               if(SI(JJ1(iRow)+JH,IHEM) .gt. 0.9981)then                
C                 if(kount.gt.kstart+1)then                             
                 if(kount.gt.1)then                                     
                   if (LOCH) then                                       
C Updating of surface temperature depends on sea-ice cover              
                     if(sicecov(i,ihem,jh).lt.0.5)then                  
                       tstaro(I,IHEM,JH)=zsst_2d(I,IHEM,JH)*GASCON/CG   
                     else                                               
                       tscurw=zsst_2d(I,IHEM,jh)*GASCON/CG              
C Relaxation to sea-ice base temp at -2C                                
           oceaniceflux(I,IHEM)=(tscurw-max(TSTARO(I,IHEM,JH),tm2))*    
     $                           shco/restorice/tspd/DELT2C             
                     endif                                              
                  else                                                  
                    tstar(I,IHEM,jh)=zsst_2d(I,IHEM,jh)*GASCON/CG       
                  end if                                                
                endif                                                   
              endif                                                     
                                                                        
c put ssts in here now, moved from SFCT                                 
                                                                        
              if(ofrac(I,IHEM,jh).gt.0)then                             
C                if(kount.gt.kstart+1)then                              
                if(kount.gt.1)then                                      
                  if (LOCH) then                                        
C Updating of surface temperature depends on sea-ice cover              
                    if(sicecov(i,ihem,jh).lt.0.5)then                   
                      tstaro(I,IHEM,JH)=zsst_2d(I,IHEM,JH)*GASCON/CG    
                    else                                                
                      tscurw=zsst_2d(I,IHEM,jh)*GASCON/CG               
C Relaxation to sea-ice base temp at -2C                                
              oceaniceflux(I,IHEM)=(tscurw-max(TSTARO(I,IHEM,JH),tm2))* 
     $                           shco/restorice/tspd/DELT2C             
                    endif                                               
                  else                                                  
                    tstar(I,IHEM,jh)=zsst_2d(I,IHEM,jh)*GASCON/CG       
                  end if                                                
                endif                                                   
              endif                                                     
               MAXITER=ITSLO                                            
               DELTU=DELTO                                              
               if (LOC) then                                            
                  TSCUR=TSTARO(I,IHEM,JH)                               
               else                                                     
                                                                        
c again, moved from SFCT now that sst assignment moved                  
                  escon=1./PLG3(I,IHEM,JH)                              
                  sqstar(I,IHEM,JH)=escon*pqsat(tstar(I,IHEM,JH))       
                  qstar(I,IHEM,JH)=sqstar(I,IHEM,JH)                    
                  TSCUR=TSTAR(I,IHEM,JH)                                
                  QCUR=QSTAR(I,IHEM,JH)                                 
                  SQCUR=SQSTAR(I,IHEM,JH)                               
               endif                                                    
C Used with VMPCD code: Cd increases with windspeed                     
               RGZ0O=SZLOOK(NINT(SVEGE(I,IHEM,JH)))/2.2E4               
C Ice Roughness length depends on Tsurf                                 
               RGZ0=RGZ0O                                               
               if(tscur.le.tz)RGZ0=RGZ0IW                               
               if(tscur.le.tm2)RGZ0=RGZ0IC                              
            endif                                                       
                                                                        
            QNLGS=0.                                                    
            TNLGS=0.                                                    
            UNLGS=0.                                                    
            VNLGS=0.                                                    
            SHBLS=0.                                                    
            SLBLS=0.                                                    
            BLCDS=0.                                                    
            BLVBS=0.                                                    
            TXBLS=0.                                                    
            TYBLS=0.                                                    
                                                                        
c------get runoff here                                                  
       exwat(I,IHEM,jh)=0.                                              
       exiwat(I,IHEM,jh)=0.                                             
       exiwatn(I,IHEM,jh)=0.                                            
            do iter=1,MAXITER                                           
               !-----------------------------------------------------   
               ! If we have interactive soil, calculate surface         
               ! humidities                                             
               !-----------------------------------------------------   
               if (LSLH) then                                           
C Soil moisture conservation done in SURFM                              
                  if ((smstar(I,IHEM,jh).gt.ssmc23)                     
     &                    .or.(hsnow(I,IHEM,jh).gt.0.)) then            
                     srh=1.                                             
                  else if (smstar(I,IHEM,jh).gt.ssmc13) then            
                     srh=(smstar(I,IHEM,jh)/ssmc13)-1.                  
                  else                                                  
                     srh=0.                                             
                  endif                                                 
                  sqcur=pqsat(tscur)/PLG3(I,IHEM,JH)                    
                  qcur=srh*sqcur                                        
               endif                                                    
               !-----------------------------------------------------   
               ! Ditto for interactive ocean                            
               !-----------------------------------------------------   
               if (LOCH) then                                           
                  sqcur=pqsat(tscur)/PLG3(I,IHEM,JH)                    
                  qcur=sqcur                                            
               endif                                                    
                                                                        
C Surface winds used by BL routine UG3CUR,VG3CUR are                    
C atmospheric winds minus nondimensionalised surface current            
C Updated here since UG3 and VG3 are updated during ITER loop           
                                                                        
               UG3CUR=UG3(I,IHEM,JH,NL)-zscur_2d(I,IHEM,JH)/SQRC/CV     
               VG3CUR=VG3(I,IHEM,JH,NL)-mscur_2d(I,IHEM,JH)/SQRC/CV     
               THNL=TG3(I,IHEM,JH,NL)/SKAP(NL)                          
               THBAR=0.5*(THNL+TSCUR)                                   
               RZZ0=RSIGF*THBAR/RGZ0                                    
               RLZZ02=(LOG(RZZ0))**2                                    
               !-----------------------------------------------------   
               ! CALCULATE WIND SPEED, INCLUDING MINIMUM                
               !-----------------------------------------------------   
C Surface winds used by BL routine UG3CUR,VG3CUR are                    
C atmospheric winds minus nondimensionalised surface current            
               VM=SQRT(RCSJ*(UG3CUR*UG3CUR+                             
     &              VG3CUR*VG3CUR))                                     
               VMP=VM+BLVAD                                             
               VMPCD=MIN(VMP*CV,40.0)                                   
               !-----------------------------------------------------   
               ! CD=(K/LN(Z/Z0))**2, K=0.41                             
               !-----------------------------------------------------   
               CD=0.1681/RLZZ02                                         
C Ocean Cd=f(Z0)+f(VMP); Max Cd=0.003 at VMP=40m/s without ice cover    
C From Wu (1980) JPO v10 p727; assumes V(NL)~=(4/3)**V(10m)             
               if(GSG(I,IHEM,JH).le.0.0.and.sicecov(i,ihem,jh).lt.0.5)  
     &         CD=CD+5.6E-5*VMPCD                                       
C increase southern ocean Cd: continuous swell means more surface area  
C experimental and commented out                                        
C              CD=CD*(1.0+                                              
C     &         0.2*exp(-((SI(JJ1(iRow)+J,IHEM)+0.87)/0.15)**2))        
               blcds=blcds+cd/REAL(maxiter)                             
               CTAU=CD/TSCUR                                            
               CSH=CTAU/AKAP                                            
               DRAGJ=CD*DRAG/TSCUR                                      
               !-----------------------------------------------------   
               ! CALCULATE SURFACE STRESS AND LOWEST LEVEL FRICTION     
               !-----------------------------------------------------   
               CVM=DRAGJ*VMP                                            
               FUG3(I,IHEM,JH,NL)=-CVM*UG3CUR                           
               UNLGS=UNLGS+FUG3(I,IHEM,JH,NL)/REAL(MAXITER)             
               FVG3(I,IHEM,JH,NL)=-CVM*VG3CUR                           
               VNLGS=VNLGS+FVG3(I,IHEM,JH,NL)/REAL(MAXITER)             
               CVM=CTAU*PLG3(I,IHEM,JH)*VMP*SQRC                        
               TXBL(I,IHEM)=CVM*UG3CUR                                  
               TXBLS=TXBLS+TXBL(I,IHEM)/REAL(MAXITER)                   
               TYBL(I,IHEM)=CVM*VG3CUR                                  
               TYBLS=TYBLS+TYBL(I,IHEM)/REAL(MAXITER)                   
               !-----------------------------------------------------   
               ! CALCULATE SENSIBLE AND LATENT HEAT FLUXES              
               !-----------------------------------------------------   
               QCUR=MIN(SQCUR,MAX(QCUR,TRAG3(I,IHEM,JH,NL,1)))          
               DTH=TSCUR-THNL                                           
               DQ=BLRH*(QCUR-TRAG3(I,IHEM,JH,NL,1))                     
               BLVB(I,IHEM)=0.                                          
C Enhancement of surface-air transfer when unstable                     
C VMP=VM+5*BLVB should be VMP=VM+VM*BLVB from Forster et al 2000        
C 5.95*RLZZO2 = 1/Cd; RGZ0 includes a factor of g                       
C Limit BLVB to 4*VM gustiness factor                                   
                                                                        
               IF (DTH.GT.0.)                                           
     $         BLVB(I,IHEM)=5.95*RLZZ02*SQRT(RGZ0*DTH/THBAR)            
               VMP=VM+VM*MIN(BLVB(I,IHEM)*CV,4.0)                       
C Bug fix: add calculation of BLVBS                                     
               blvbs=blvbs+BLVB(I,IHEM)/real(maxiter)                   
               CVM=0.2*DRAGJ*VMP                                        
               TNLG3(I,IHEM,JH,NL)=CVM*DTH                              
               TNLGS=TNLGS+TNLG3(I,IHEM,JH,NL)/REAL(MAXITER)            
               TRANLG3(I,IHEM,JH,NL,1)=CVM*DQ                           
               QNLGS=QNLGS+TRANLG3(I,IHEM,JH,NL,1)/REAL(MAXITER)        
               CVM=0.2*CSH*PLG3(I,IHEM,JH)*VMP                          
               SHBL(I,IHEM,JH)=CVM*DTH                                  
               SHBLS=SHBLS+SHBL(I,IHEM,JH)/REAL(MAXITER)                
               !-----------------------------------------------------   
               ! CHOOSE APPROPRIATE LATENT HEAT                         
               !-----------------------------------------------------   
               IF (TSCUR.GT.TZ) THEN                                    
                  SLBL(I,IHEM,JH)=CVM*CTQ*DQ                            
               ELSE                                                     
                  SLBL(I,IHEM,JH)=CVM*CTQI*DQ                           
               ENDIF                                                    
               SLBLS=SLBLS+SLBL(I,IHEM,JH)/REAL(MAXITER)                
               UG3(I,IHEM,JH,NL)=UG3(I,IHEM,JH,NL)+                     
     &              DELTU*FUG3(I,IHEM,JH,NL)                            
               VG3(I,IHEM,JH,NL)=VG3(I,IHEM,JH,NL)+                     
     &              DELTU*FVG3(I,IHEM,JH,NL)                            
               TG3(I,IHEM,JH,NL)=TG3(I,IHEM,JH,NL)+                     
     &              DELTU*TNLG3(I,IHEM,JH,NL)                           
               TRAG3(I,IHEM,JH,NL,1)=TRAG3(I,IHEM,JH,NL,1)+             
     &              DELTU*TRANLG3(I,IHEM,JH,NL,1)                       
                                                                        
               if (LSLH) then                                           
                  !---------------------------------------------------  
                  ! CHECK SIGNS of shbl,slbl,snet                       
                  !---------------------------------------------------  
                  SFC=-shbl(I,IHEM,JH)-slbl(I,IHEM,JH)+                 
     &                 snet(I,IHEM,jh)/(CV*P0)                          
                  rkappa=2./(srsum+hsnow(I,IHEM,jh)/sksn)               
                  pc1=shcs                                              
                  pc2=shcs                                              
                  if ((tscur.gt.tm3).and.(tscur.lt.tp1))                
     &                 pc1=pc1+shcsp                                    
                  if ((tdcur.gt.tm3).and.(tdcur.lt.tp1))                
     &                 pc2=pc2+shcsp                                    
                  hc1=pc1*sd1+min(hsnow(I,IHEM,jh),shsmax)*shcsn        
                  hc2=pc2*sd2                                           
                                                                        
                  trans=rkappa*(tscur-tdcur)                            
                  rct1=(sfc-trans)/hc1                                  
                  rct2=trans/hc2                                        
                                                                        
                  tscur=tscur+rct1*deltu                                
                  tdcur=tdcur+rct2*deltu                                
                                                                        
                  if (tscur.gt.tz) then                                 
                     if (hsnow(I,IHEM,jh).gt.0.) then                   
                        !---------------------------------------------  
                        ! The factor of 2 is *REALLY* meant to be here, 
                        ! since we are dealing with a forward time      
                        ! single level variable (hsnow) within an       
                        ! (effectively) centre time timestep - thus     
                        ! each segment of time is covered twice in this 
                        ! routine...                                    
                        !---------------------------------------------  
      deltasnow=0.0                                                     
C No snowmelt over icecaps: snowdepth > SHSMAX handled later            
      IF(SVEGE(I,IHEM,JH).NE.2)THEN                                     
        deltasnow=-1.0*(tscur-tz)*hc1/(slhf*sdsn)/2.                    
        if (hsnow(i,ihem,jh)+deltasnow.lt.0) then                       
           deltasnow=-1*hsnow(i,ihem,jh)                                
        endif                                                           
        hsnow(i,ihem,jh)=hsnow(i,ihem,jh)+deltasnow                     
      ENDIF                                                             
C I've changed SHSMAX in the IGCM to 4m to match FORTE                  
                                                                        
c     Put the melted snow into runoff after first recharging soil       
      if (deltasnow.lt.0.) then                                         
C Enough capacity in soil to take melted snow                           
         if(ssmc-smstar(i,ihem,jh).gt.-deltasnow*sdsn/sdw) then         
           smstar(i,ihem,jh)=smstar(i,ihem,jh)-deltasnow*sdsn/sdw       
         else                                                           
C Soil is saturated, excess snow into runoff                            
           deltasnow=deltasnow+(ssmc-smstar(i,ihem,jh))*sdw/sdsn        
           smstar(i,ihem,jh)=ssmc                                       
           exwat(i,ihem,jh)=exwat(i,ihem,jh)-                           
     :     deltasnow*(sdsn/sdw)*1000.*RADEA                             
         endif                                                          
      endif                                                             
                                                                        
c  alterwater called once both rain and snow are included               
                                                                        
C Soil moisture calculation now handled in runoff code above            
C                        smstar(i,ihem,jh) =                            
C     &                  max(smstar(i,ihem,jh),ssmc23)                  
C                                                                       
C                        smstar(I,IHEM,jh) = smstar(I,IHEM,jh)          
C     &                  +(tscur-tz)*hc1/(slhf*sdw)/2.                  
                     endif                                              
                  endif                                                 
c     Set a max snow depth of 4m to prevent runaway accumulation        
      if (hsnow(i,ihem,jh).gt.SHSMAX) then                              
C Excess southwards of 55S added to iceberg flux for ocean              
        if (SI(JJ1(iRow)+JH,IHEM) .lt. -0.831)then                      
          snowexcess=hsnow(i,ihem,jh)-SHSMAX                            
          hsnow(i,ihem,jh)=SHSMAX                                       
          exiwat(i,ihem,jh)=exiwat(i,ihem,jh)+                          
     :    snowexcess*(sdsn/sdw)*1000.*RADEA                             
                                                                        
C Excess northwards of 66N added to iceberg flux for ocean              
C 66N chosen because otherwise large IGCM snowfall over                 
C S Alaska/BC region goes into Arctic instead of Pacific                
C        elseif(SI(JJ1(iRow)+JH,IHEM) .ge. 0.914)then                   
C                                                                       
C          snowexcess=hsnow(i,ihem,jh)-SHSMAX                           
C          hsnow(i,ihem,jh)=SHSMAX                                      
C          exiwatn(i,ihem,jh)=exiwatn(i,ihem,jh)+                       
C     :    snowexcess*(sdsn/sdw)*1000.*RADEA                            
                                                                        
        else                                                            
C Excess at other latitudes added to coastal runoff                     
        snowexcess=hsnow(i,ihem,jh)-SHSMAX                              
        hsnow(i,ihem,jh)=SHSMAX                                         
        exwat(i,ihem,jh)=exwat(i,ihem,jh)+                              
     :  snowexcess*(sdsn/sdw)*1000.*RADEA                               
        endif                                                           
      endif                                                             
                                                                        
               endif                                                    
                                                                        
               if (LOCH) then                                           
                  !---------------------------------------------------  
                  ! INTEROP between months                              
                  !---------------------------------------------------  
c     OCF=0.                                                            
               if (sicecov(i,ihem,jh).ge.0.5)then                       
                                                                        
C SHCI constant: snow conductivity very low                             
                 hc=shci                                                
C     $              +log(tm2-tscur+1.0)*shci/log(tm2-ticesnow+1)       
C                                                                       
               elseif(tscur.gt.tm2.and.sicecov(i,ihem,jh).ge.0.5)then   
                 hc=shci                                                
               else                                                     
C Without ice heat capacity is that of ocean                            
                 hc=shco                                                
               endif                                                    
                                                                        
                  !---------------------------------------------------  
                  ! Add OCF term to ocean heat flux                     
                  !---------------------------------------------------  
c  actually, /don't/ add in extra fluxes - for coupled operation        
                SFC=-shbl(I,IHEM,JH)-slbl(I,IHEM,JH)+                   
     &          (snet(I,IHEM,JH))/(CV*P0)                               
                                                                        
                dts = (sfc +oceaniceflux(I,IHEM))/hc*deltu              
                                                                        
C Start brine rejection bit- turned off                                 
C Below -2C ice thickness increases as Ts decreases.                    
C Freshwater flux to ocean is calculated similarly to snowmelt:         
C F = F + water depth change ~= +0.1m per +1C in Tsurf change           
C Over a season this should average to zero and conserve water          
c                if(sicecov(I,IHEM,JH).gt.0.5)                          
c     &          water2d(I,IHEM,JH)=water2d(I,IHEM,JH)+                 
c     &          dts*CT*0.1*1000.0/RADEA                                
C End Brine rejection bit                                               
                                                                        
c                                                                       
                  tscur=tscur+dts                                       
                                                                        
               endif                                                    
            enddo                                                       
                                                                        
            if(GSG(I,IHEM,jh).le.0.)then                                
                                                                        
C At end of iteration add to sea-ice fractional cover if Ts<TICEFORM    
C over a timescale of a day- avoids 2dT waves but synoptically short    
C Formulation allows for hysteresis                                     
              if(tscur.le.TICEFORM)                                     
     :        sicecov(I,IHEM,jh)=min(sicecov(I,IHEM,jh)+1.0/TSPD,1.0)   
C If Ts>0C remove ice over one day                                      
              if(tscur.gt.tz)                                           
     :        sicecov(I,IHEM,jh)=max(sicecov(I,IHEM,jh)-1.0/TSPD,0.0)   
C Above 0C albedo based on ice cover                                    
              if(tscur.gt.TZ)then                                       
                salb(I,IHEM,jh)=sbal(i,ihem,jh)+                        
     :          (salb(I,IHEM,jh)-sbal(i,ihem,jh))*sicecov(i,ihem,jh)    
C Below 0C, Albedo is MAX(present SALB,diagnosed SALB from sea-ice)     
C representing ice staying covered with snow until final melting        
              elseif(tscur.gt.TICEFORM.and.tscur.le.tz)then             
                salb(I,IHEM,jh)=max(sbal(i,ihem,jh)+                    
     :          (saice-sbal(i,ihem,jh))*sicecov(i,ihem,jh),             
     :                          salb(I,IHEM,jh))                        
C Below TICEFORM, Albedo is MAX(present SALB,diagnosed SALB from snow)  
C representing ice staying covered with snow until final melting        
              elseif(tscur.ge.TICESNOW.and.tscur.le.TICEFORM)then       
                salb(I,IHEM,jh)=max(sasnow+                             
     :          (saice-sasnow)*(tscur-TICESNOW)/(TICEFORM-TICESNOW),    
     :                          salb(I,IHEM,jh))                        
C Below TICESNOW ice completely snow-covered                            
              else                                                      
                salb(I,IHEM,jh)=sasnow                                  
              endif                                                     
                                                                        
C ice-free wind-based whitecaps and seasalt parameterisation            
C based on (U-Threshold)^3 (similar to dust raising)                    
C EXPERIMENTAL AND TURNED OFF FOR NOW                                   
C              if(sicecov(i,ihem,jh).le.1.0e-5)salb(I,IHEM,jh)=         
C     :        min(sbal(i,ihem,jh)+max(0.0E-4*((VMPCD-5.0)**3),0.0),0.6)
                                                                        
            endif                                                       
            !-----------------------------------------------------      
            ! End of iteration process, done full double timestep.      
            !-----------------------------------------------------      
            if (LSLH) then                                              
               tdeepo(I,IHEM,jh)=tdeep(I,IHEM,jh)*(1.-2.*pnu)           
     &              +pnu*(tdeepo(I,IHEM,jh)+tdcur)                      
               tdeep(I,IHEM,jh)=tdcur                                   
               tstaro(I,IHEM,jh)=tstar(I,IHEM,jh)*(1.-2.*pnu)           
     &              +pnu*(tstaro(I,IHEM,jh)+tscur)                      
               tstar(I,IHEM,jh)=tscur                                   
               sqstar(I,IHEM,jh)=sqcur                                  
               qstar(I,IHEM,jh)=qcur                                    
            endif                                                       
                                                                        
            if (LOCH) then                                              
c    only update tstaro in IGCM if sea-ice or land                      
               if (sicecov(I,IHEM,JH).ge.0.5 .or.                       
     &         ofrac(I,IHEM,JH).eq.0.)then                              
               tstaro(I,IHEM,jh)=tstar(I,IHEM,jh)*(1.-2.*pnu)           
     &              +pnu*(tstaro(I,IHEM,jh)+tscur)                      
               tstar(I,IHEM,jh)=tscur                                   
               else                                                     
c     energy consistency, keep tstar=zsst                               
                tstar(I,IHEM,jh)=tstaro(I,IHEM,jh)                      
               end if                                                   
               sqstar(I,IHEM,jh)=sqcur                                  
               qstar(I,IHEM,jh)=qcur                                    
            endif                                                       
                                                                        
            TRANLG3(I,IHEM,JH,NL,1)=QNLGS                               
            TNLG3(I,IHEM,JH,NL)=TNLGS                                   
            FUG3(I,IHEM,JH,NL)=UNLGS                                    
            FVG3(I,IHEM,JH,NL)=VNLGS                                    
            SHBL(I,IHEM,JH)=SHBLS                                       
            SLBL(I,IHEM,JH)=SLBLS                                       
            BLCD(I,IHEM)=BLCDS                                          
            BLVB(I,IHEM)=BLVBS                                          
            TXBL(I,IHEM)=TXBLS                                          
            TYBL(I,IHEM)=TYBLS                                          
C                                                                       
C COASTAL TILING CODE STARTS HERE                                       
C                                                                       
      if(GSG(I,IHEM,JH).ne.0..and.ofrac(I,IHEM,JH).gt.0.                
     $  .and.ofrac(I,IHEM,JH).lt.1.)then                                
C                                                                       
C Calculate BL tendencies over ocean part of coastal tile               
C                                                                       
                                                                        
C        if(kount.gt.kstart+1)then                                      
        if(kount.gt.1)then                                              
          if(sicecov(i,ihem,jh).lt.0.5)then                             
            tstaroo(I,IHEM,JH)=zsst_2d(I,IHEM,JH)*GASCON/CG             
          else                                                          
C Ice cover- use IGCM Tsurf                                             
            tscurw=zsst_2d(I,IHEM,JH)*GASCON/CG                         
C Relaxation to sea-ice base temp                                       
            oceaniceflux(I,IHEM)=(tscurw-max(TSTAROO(I,IHEM,JH),tm2))   
     $      *shco/restorice/tspd/DELT2C                                 
          endif                                                         
C SVEGE array at coastal points is land so roughness length specified   
C from ocean points: used with VMPCD where Cd increases with windspeed  
          RGZ0O=SZLOOK(1)/2.2E4                                         
C Ice roughness length depends on ice cover, which depends on Tsurf     
          RGZ0=RGZ0O                                                    
          if(tscur.le.tz)RGZ0=RGZ0IW                                    
          if(tscur.le.tm2)RGZ0=RGZ0IC                                   
        endif                                                           
                                                                        
c Now do a single gridpoint                                             
c Put sst's in here now, moved from SFCT                                
                                                                        
C        TSCUR=TSTARO(I,IHEM,JH)                                        
        TSCUR=TSTAROO(I,IHEM,JH)                                        
                                                                        
        MAXITER=ITSLO                                                   
        DELTU=DELTO                                                     
        QNLGS=0.                                                        
        TNLGS=0.                                                        
        UNLGS=0.                                                        
        VNLGS=0.                                                        
        SHBLS=0.                                                        
        SLBLS=0.                                                        
        BLCDS=0.                                                        
        BLVBS=0.                                                        
        TXBLS=0.                                                        
        TYBLS=0.                                                        
                                                                        
        do iter=1,MAXITER                                               
                                                                        
c If we have interactive ocean, calculate surface humidities            
                                                                        
C Surface winds used by coastal BL routine UGBSOCUR,VGBSOCUR are        
C atmospheric winds minus nondimensionalised surface current            
C Updated here since UGBSOC and VGBSOC are updated during ITER loop     
                                                                        
        UGBSOCUR=UGBSOC-zscur_2d(I,IHEM,JH)/SQRC/CV                     
        VGBSOCUR=VGBSOC-mscur_2d(I,IHEM,JH)/SQRC/CV                     
                                                                        
        qcur=pqsat(tscur)/PLG3(I,IHEM,JH)                               
        sqcur=qcur                                                      
C          THNL=1.02*TGBSOC/SKAP(NL)                                    
          THNL=1.00*TGBSOC/SKAP(NL)                                     
          THBAR=0.5*(THNL+TSCUR)                                        
          RZZ0=RSIGF*THBAR/RGZ0                                         
          RLZZ02=(LOG(RZZ0))**2                                         
                                                                        
c CALCULATE WIND SPEED, INCLUDING MINIMUM                               
          VM=SQRT(RCSJ*(UGBSOCUR*UGBSOCUR+VGBSOCUR*VGBSOCUR))           
          VMP=VM+BLVAD                                                  
          VMPCD=MIN(VMP*CV,40.0)                                        
          CD=0.1681/RLZZ02                                              
                                                                        
C Ocean Cd=f(Z0)+f(VMP); Max Cd=0.003 at VMP=40m/s without ice cover    
C From Wu (1980) JPO v10 p727; assumes V(NL)~=(4/3)**V(10m)             
          if(sicecov(i,ihem,jh).lt.0.5)CD=CD+5.6E-5*VMPCD               
C                                                                       
          blcds=blcds+cd/real(maxiter)                                  
          CTAU=CD/TSCUR                                                 
          CSH=CTAU/AKAP                                                 
          DRAGJ=CD*DRAG/TSCUR                                           
c CALCULATE SURFACE STRESS AND LOWEST LEVEL FRICTION                    
          CVM=DRAGJ*VMP                                                 
          UNLGJNL=-CVM*UGBSOCUR                                         
          UNLGS=UNLGS+UNLGJNL/REAL(MAXITER)                             
          VNLGJNL=-CVM*VGBSOCUR                                         
          VNLGS=VNLGS+VNLGJNL/REAL(MAXITER)                             
          CVM=CTAU*PLG3(I,IHEM,JH)*VMP*SQRC                             
          TXBLJ=CVM*UGBSOCUR                                            
          TXBLS=TXBLS+TXBLJ/REAL(MAXITER)                               
          TYBLJ=CVM*VGBSOCUR                                            
          TYBLS=TYBLS+TYBLJ/REAL(MAXITER)                               
c CALCULATE SENSIBLE AND LATENT HEAT FLUXES                             
          QCUR=MIN(SQCUR,MAX(QCUR,QGBSOC))                              
          DTH=TSCUR-THNL                                                
          DQ=BLRH*(QCUR-QGBSOC)                                         
          BLVB(I,IHEM)=0.                                               
                                                                        
C Enhancement of surface-air transfer when unstable                     
C VMP=VM+5*BLVB should be VMP=VM+VM*BLVB from Forster et al 2000        
C 5.95*RLZZO2 = 1/Cd; RGZ0 includes a factor of g                       
C Limit BLVB to 4*VM gustiness factor                                   
                                                                        
          IF (DTH.GT.0.)                                                
     $    BLVB(I,IHEM)=5.95*RLZZ02*SQRT(RGZ0*DTH/THBAR)                 
          VMP=VM+VM*MIN(BLVB(I,IHEM)*CV,4.0)                            
C Bug fix: add calculation of BLVBS                                     
          blvbs=blvbs+BLVB(I,IHEM)/real(maxiter)                        
          CVM=0.2*DRAGJ*VMP                                             
          TNLGJNL=CVM*DTH                                               
          TNLGS=TNLGS+TNLGJNL/REAL(MAXITER)                             
          QNLGJNL=CVM*DQ                                                
          QNLGS=QNLGS+QNLGJNL/REAL(MAXITER)                             
          CVM=0.2*CSH*PLG3(I,IHEM,JH)*VMP                               
          SHBLJ=CVM*DTH                                                 
          SHBLS=SHBLS+SHBLJ/REAL(MAXITER)                               
c CHOOSE APPROPRIATE LATENT HEAT                                        
          IF (TSCUR.GT.TZ) THEN                                         
             SLBLJ=CVM*CTQ*DQ                                           
          ELSE                                                          
             SLBLJ=CVM*CTQI*DQ                                          
          ENDIF                                                         
          SLBLS=SLBLS+SLBLJ/REAL(MAXITER)                               
          UGBSOC=UGBSOC+DELTU*UNLGJNL                                   
          VGBSOC=VGBSOC+DELTU*VNLGJNL                                   
          TGBSOC=TGBSOC+DELTU*TNLGJNL                                   
          QGBSOC=QGBSOC+DELTU*QNLGJNL                                   
                                                                        
          if (sicecov(i,ihem,jh).ge.0.5)then                            
                                                                        
C SHCI constant: snow conductivity low                                  
C            hc=shci                                                    
C     $         +log(tm2-tscur+1.0)*shci/log(tm2-ticesnow+1)            
C          elseif(tscur.gt.tm2.and.sicecov(i,ihem,jh).ge.0.5)then       
            hc=shci                                                     
          else                                                          
C Without ice heat capacity is that of ocean                            
            hc=shco                                                     
          endif                                                         
                                                                        
c Do not add OCF term to ocean heat flux in coupled run                 
          SFC=-shblj-slblj+(snet(I,IHEM,JH))/(CV*P0)                    
                                                                        
C Adding in oceaniceflux to coastal points                              
          dts = (sfc +oceaniceflux(I,IHEM))/hc*deltu                    
                                                                        
C Start brine rejection bit- turned off                                 
C Below -2C ice thickness increases as Ts decreases.                    
C Freshwater flux to ocean is calculated similarly to snowmelt:         
C F = F + water depth change ~= +0.1m per +1C in Tsurf change           
C Over a season	this should average to zero and	conserve water          
C                if(sicecov(I,IHEM,JH).gt.0.5)                          
C     &          water2d(I,IHEM,JH)=water2d(I,IHEM,JH)+                 
C     &          dts*CT*0.1*1000.0/RADEA                                
C End Brine rejection bit                                               
                                                                        
        enddo                                                           
                                                                        
C At end of iteration add to sea-ice fractional cover if Ts<TICEFORM    
C over a timescale of a day- avoids 2dT waves but synoptically short    
C Formulation allows for hysteresis                                     
        if(tscur.le.TICEFORM)                                           
     :  sicecov(I,IHEM,jh)=min(sicecov(I,IHEM,jh)+1.0/TSPD,1.0)         
C If Ts>0C remove ice over one day                                      
        if(tscur.gt.tz)                                                 
     :  sicecov(I,IHEM,jh)=max(sicecov(I,IHEM,jh)-1.0/TSPD,0.0)         
C At icecap gridpoints coastal points sea-ice always present            
        if(SVEGE(I,IHEM,JH).EQ.2)                                       
     :  sicecov(I,IHEM,jh)=max(sicecov(I,IHEM,jh),0.5)                  
C Above 0C albedo based on ice cover                                    
        if(ofrac(I,IHEM,JH).gt.0.5)then                                 
          if(tscur.gt.TZ)then                                           
            salb(I,IHEM,jh)=sbal(i,ihem,jh)+                            
     :      (salb(I,IHEM,jh)-sbal(i,ihem,jh))*sicecov(i,ihem,jh)        
C Below 0C, Albedo is MAX(present SALB,diagnosed SALB from sea-ice)     
C representing ice staying covered with snow until final melting        
          elseif(tscur.gt.TICEFORM.and.tscur.le.tz)then                 
            salb(I,IHEM,jh)=max(sbal(i,ihem,jh)+                        
     :      (saice-sbal(i,ihem,jh))*sicecov(i,ihem,jh),                 
     :                      salb(I,IHEM,jh))                            
C Below TICEFORM, Albedo is MAX(present SALB,diagnosed SALB from snow)  
C representing ice staying covered with snow until final melting        
          elseif(tscur.ge.TICESNOW.and.tscur.le.TICEFORM)then           
            salb(I,IHEM,jh)=max(sasnow+                                 
     :      (saice-sasnow)*(tscur-TICESNOW)/(TICEFORM-TICESNOW),        
     :                      salb(I,IHEM,jh))                            
C Below TICESNOW ice completely snow-covered                            
          else                                                          
            salb(I,IHEM,jh)=sasnow                                      
          endif                                                         
        endif                                                           
C ice-free wind-based whitecaps and seasalt parameterisation            
C based on (U-Threshold)^3 (similar to dust raising)                    
C EXPERIMENTAL AND TURNED OFF FOR NOW                                   
C          if(sicecov(i,ihem,jh).le.1.0e-5)salb(I,IHEM,jh)=             
C     :    min(sbal(i,ihem,jh)+max(0.0E-4*((VMPCD-5.0)**3),0.0),0.6)    
                                                                        
                                                                        
c End of iteration process, done full double timestep.                  
c only update tstaro in IGCM if sea-ice                                 
        if (sicecov(I,IHEM,JH).ge.0.5) then                             
          tstaroo(I,IHEM,jh)=tstarf(I,IHEM,jh)*(1.-2.*pnu)              
     $            +pnu*(tstaroo(I,IHEM,jh)+tscur)                       
          tstarf(I,IHEM,jh)=tscur                                       
        else                                                            
          tstarf(I,IHEM,jh)=tstaroo(I,IHEM,jh)                          
        end if                                                          
                                                                        
        sqstarf(I,IHEM,jh)=sqcur                                        
        qstarf(I,IHEM,jh)=qcur                                          
                                                                        
      ENDIF                                                             
C                                                                       
C Calculate moisture converegnce in atmosphere to close water budget:   
C needs to account for pre and post coastal tiling                      
C Balance before coastal tiling:for ocean, land, and land part of coast 
                                                                        
      stempbs3=0.                                                       
      do l=1,nl                                                         
      stempbs3=stempbs3+TRANLG3(I,IHEM,JH,L,1)*clr(l)                   
      end do                                                            
      wathbl(I,IHEM,JH)=stempbs3*PLG3(I,IHEM,JH)                        
                                                                        
C Balance after coastal tiling weighted by ocean fraction               
C used for average gridpoint water balance                              
      stempbs3=0.                                                       
      do l=1,nl-1                                                       
      stempbs3=stempbs3+TRANLG3(I,IHEM,JH,L,1)*clr(l)                   
      end do                                                            
      stempbs3=stempbs3+TRANLG3(I,IHEM,JH,NL,1)*                        
     :   (1-ofrac(I,IHEM,JH))*clr(nl)                                   
     :  +QNLGS*clr(nl)*ofrac(I,IHEM,JH)                                 
      watlbl(I,IHEM,JH)=stempbs3*PLG3(I,IHEM,JH)                        
                                                                        
C Balance after coastal tiling over fractional ocean points             
C is used for water balance over ocean parts of coastal points          
      stempbs3=0.                                                       
      do l=1,nl-1                                                       
      stempbs3=stempbs3+TRANLG3(I,IHEM,JH,L,1)*clr(l)                   
      end do                                                            
      stempbs3=stempbs3+QNLGS*clr(nl)                                   
      watsbl(I,IHEM,JH)=stempbs3*PLG3(I,IHEM,JH)                        
                                                                        
      if(GSG(I,IHEM,JH).ne.0..and.ofrac(I,IHEM,JH).gt.0.                
     $  .and.ofrac(I,IHEM,JH).lt.1.)then                                
C        SHBL(I,IHEM,JH)=SHBLS                                          
C        SLBL(I,IHEM,JH)=SLBLS                                          
C Diagnosed gridpoint SHBL and SLBL weighted by OFRAC                   
        SHBL(I,IHEM,JH)=(1.-ofrac(I,IHEM,JH))*                          
     $ SHBL(I,IHEM,JH)+ofrac(I,IHEM,JH)*SHBLS                           
        SLBL(I,IHEM,JH)=(1.-ofrac(I,IHEM,JH))*                          
     $ SLBL(I,IHEM,JH)+ofrac(I,IHEM,JH)*SLBLS                           
C SHBLS and SLBLS equivalent for ocean part of gridpoint                
        SHBLC(I,IHEM,JH)=SHBLS                                          
        SLBLC(I,IHEM,JH)=SLBLS                                          
        TXBL(I,IHEM)=TXBLS                                              
        TYBL(I,IHEM)=TYBLS                                              
        TRANLG3(I,IHEM,JH,NL,1)=(1.-ofrac(I,IHEM,JH))*                  
     $TRANLG3(I,IHEM,JH,NL,1)+ofrac(I,IHEM,JH)*QNLGS                    
        TRAG3(I,IHEM,JH,NL,1)=(1.-ofrac(I,IHEM,JH))*                    
     $TRAG3(I,IHEM,JH,NL,1)+ofrac(I,IHEM,JH)*QGBSOC                     
        TNLG3(I,IHEM,JH,NL)=(1.-ofrac(I,IHEM,JH))*                      
     $TNLG3(I,IHEM,JH,NL)+ofrac(I,IHEM,JH)*TNLGS                        
        TG3(I,IHEM,JH,NL)=(1.-ofrac(I,IHEM,JH))*                        
     $TG3(I,IHEM,JH,NL)+ofrac(I,IHEM,JH)*TGBSOC                         
        FUG3(I,IHEM,JH,NL)=(1.-ofrac(I,IHEM,JH))*                       
     $FUG3(I,IHEM,JH,NL)+ofrac(I,IHEM,JH)*UNLGS                         
        UG3(I,IHEM,JH,NL)=(1.-ofrac(I,IHEM,JH))*                        
     $UG3(I,IHEM,JH,NL)+ofrac(I,IHEM,JH)*UGBSOC                         
        FVG3(I,IHEM,JH,NL)=(1.-ofrac(I,IHEM,JH))*                       
     $FVG3(I,IHEM,JH,NL)+ofrac(I,IHEM,JH)*VNLGS                         
        VG3(I,IHEM,JH,NL)=(1.-ofrac(I,IHEM,JH))*                        
     $VG3(I,IHEM,JH,NL)+ofrac(I,IHEM,JH)*VGBSOC                         
      endif                                                             
                                                                        
         enddo                                                          
      enddo                                                             
      end                                                               
!=====================================================                  
! Given the julian day, routine returns month and day as                
! character*7 variable. Leap years are ignored.                         
!=====================================================                  
      SUBROUTINE CALNDR(DOY,MDUM,amfrac)                                
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Passed arguments                                                      
!-----------------------------------------------------                  
      REAL :: DOY                                                       
      INTEGER :: MDUM                                                   
      REAL :: AMFRAC                                                    
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER :: JDAY                                                   
      REAL :: DAY, GAP                                                  
      CHARACTER*4 :: MONTH                                              
      CHARACTER*4 :: MNTH(12)                                           
      INTEGER :: MDAY                                                   
      INTEGER :: NDAY(13)                                               
! Loop variables                                                        
      INTEGER :: MM                                                     
!-----------------------------------------------------                  
! Days in each month and month names                                    
!-----------------------------------------------------                  
C      DATA NDAY/0,31,59,90,120,151,181,212,243,273,304,334,365/        
      DATA NDAY/0,30,60,90,120,150,180,210,240,270,300,330,360/         
     :    ,MNTH/'JAN.','FEB.','MAR.','APR.','MAY ','JUNE','JULY',       
     :           'AUG.','SEPT','OCT.','NOV.','DEC'/                     
!-----------------------------------------------------                  
! Perform operation                                                     
!-----------------------------------------------------                  
      DAY=DOY-1.0                                                       
      DO MM=12,1,-1                                                     
          IF (DAY.GE.(REAL(NDAY(MM+1)+NDAY(MM))/2.0)) THEN              
           MDUM=MM                                                      
           GAP=30.0  ! const month length                               
           AMFRAC=(DAY-0.5*REAL(NDAY(MM+1)+NDAY(MM)))/GAP               
           GOTO 200                                                     
          ENDIF                                                         
       ENDDO                                                            
c this means day is between 1-15                                        
      MDUM=12                                                           
      GAP=30.0                                                          
      AMFRAC=(DAY+15.)/GAP                                              
                                                                        
  200 CONTINUE                                                          
                                                                        
      END                                                               
!=====================================================                  
! PRECIPITATING DEEP CONVECTION SCHEME.                                 
! ADJUSTMENT TOWARDS A SUBSATURATED MOIST ADIABAT.                      
!=====================================================                  
      SUBROUTINE CBADJ(NCRB,NCRT,I,IHEM,JH)                             
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Grid variables for DGRMLT loop                                        
!-----------------------------------------------------                  
      COMMON/GRIDPC/ASSBL,ASHBL,ASLBL,ARRCR,ARRLR                       
     &     ,TTVD,QTVD,TYBL,TXBL,TTRD,TTLR,TTCR,UTVD,VTVD                
     &     ,QTLR,QTCR,SSBL,SHBL,SLBL,RRCR,RRLR                          
     &     ,ARFLUX,ASFLD,ACLD,RFLUX,SFLD,CLD                            
      REAL :: ASSBL(MGx,NHEM,JGy)                                       
      REAL :: ASHBL(MGx,NHEM,JGy)                                       
      REAL :: ASLBL(MGx,NHEM,JGy)                                       
      REAL :: ARRCR(MGx,NHEM,JGy)                                       
      REAL :: ARRLR(MGx,NHEM,JGy)                                       
      REAL :: TTVD(MGx,NHEM,NL)                                         
      REAL :: QTVD(MGx,NHEM,NL)                                         
      REAL :: TYBL(MGx,NHEM)                                            
      REAL :: TXBL(MGx,NHEM)                                            
      REAL :: TTRD(MGx,NHEM,NL)                                         
      REAL :: TTLR(MGx,NHEM,NL)                                         
      REAL :: TTCR(MGx,NHEM,NL)                                         
      REAL :: UTVD(MGx,NHEM,NL)                                         
      REAL :: VTVD(MGx,NHEM,NL)                                         
      REAL :: QTLR(MGx,NHEM,NL)                                         
      REAL :: QTCR(MGx,NHEM,NL)                                         
      REAL :: SSBL(MGx,NHEM,JGy)                                        
      REAL :: SHBL(MGx,NHEM,JGy)                                        
      REAL :: SLBL(MGx,NHEM,JGy)                                        
      REAL :: RRCR(MGx,NHEM,JGy)                                        
      REAL :: RRLR(MGx,NHEM,JGy)                                        
      REAL :: arflux(MGx,NHEM,JGy,6)                                    
      REAL :: asfld(MGx,NHEM,JGy,6)                                     
      REAL :: acld(MGx,NHEM,JGy,4)                                      
      REAL :: rflux(MGx,NHEM,JGy,6)                                     
      REAL :: sfld(MGx,NHEM,JGy,6)                                      
      REAL :: cld(MGx,NHEM,JGy,4)                                       
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Radiation variables                                                   
!-----------------------------------------------------                  
      COMMON/CPIERS/ICFLAG,CFRAC,PNET,SNET,RRFLUX,TTSW,TTLW             
      INTEGER :: ICFLAG(MGx,NHEM,5,2)                                   
      REAL :: CFRAC(MGx,NHEM,5)                                         
      REAL :: PNET(MGx,NHEM,JGy)                                        
      REAL :: SNET(MGx,NHEM,JGy)                                        
      REAL :: RRFLUX(MGx,NHEM,JGy,6)                                    
      REAL :: TTSW(MGx,NHEM,NL)                                         
      REAL :: TTLW(MGx,NHEM,NL)                                         
!-----------------------------------------------------                  
! Passed arguments                                                      
!-----------------------------------------------------                  
      INTEGER :: NCRB                                                   
      INTEGER :: NCRT                                                   
      INTEGER :: I                                                      
      INTEGER :: IHEM                                                   
! Latitude index                                                        
      INTEGER :: JH                                                     
!-----------------------------------------------------                  
! External function                                                     
!-----------------------------------------------------                  
      REAL, EXTERNAL :: PQSAT                                           
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: CTQUSE                                                    
      INTEGER :: NLEV                                                   
      REAL :: EPSIT                                                     
      REAL :: PL(NL)                                                    
      REAL :: PSL(NL)                                                   
      REAL :: PSK(NL)                                                   
      REAL :: CBC(NL)                                                   
      REAL :: TSL(NL)                                                   
      REAL :: QR(NL)                                                    
      REAL :: SINT                                                      
      REAL :: TGINT                                                     
      REAL :: QGINT                                                     
      REAL :: TRINT                                                     
      REAL :: QRINT                                                     
      REAL :: HGINT                                                     
      REAL :: DHL                                                       
      REAL :: P                                                         
      REAL :: GH1(NL)                                                   
      INTEGER :: NIT                                                    
      REAL :: DTMAX                                                     
      REAL :: QN                                                        
      REAL :: DTSL                                                      
      REAL :: TR(NL)                                                    
      REAL :: TTEND                                                     
      REAL :: QTEND                                                     
! Loop variables                                                        
      INTEGER :: L                                                      
!-----------------------------------------------------                  
! Choose appropriate value of latent heat based on ground T             
! ie, we assume it snows if the ground is colder than 273.15K           
!-----------------------------------------------------                  
      if (tstar(I,IHEM,jh).gt.0.363224029) then                         
         ctquse=ctq                                                     
      else                                                              
         ctquse=ctqi                                                    
      endif                                                             
      NLEV=1+NCRB-NCRT                                                  
      EPSIT=CTQUSE*EPSIQ                                                
C                                                                       
      DO L=NCRT,NCRB                                                    
        PL(L)=SIGMA(L)*PLG3(I,IHEM,JH)                                  
        PSL(L)=PL(L)+CBADJP                                             
        PSK(L)=(PSL(L)/PL(L))**AKAP                                     
        CBC(L)=CTQUSE*PSK(L)/PSL(L)                                     
        TSL(L)=TC(L)*PSK(L)                                             
        QR(L)=PQSAT(TSL(L))/PSL(L)                                      
      ENDDO                                                             
!-----------------------------------------------------                  
! Sum up DSIGMA                                                         
!-----------------------------------------------------                  
      SINT=0.0                                                          
      DO L=NCRT,NCRB                                                    
         SINT=SINT+DSIGMA(L)                                            
      ENDDO                                                             
!-----------------------------------------------------                  
!                                                                       
!-----------------------------------------------------                  
      TGINT=0.0                                                         
      QGINT=0.0                                                         
      TRINT=0.0                                                         
      QRINT=0.0                                                         
      DO L=NCRT,NCRB                                                    
         TGINT=TGINT+TG3(I,IHEM,JH,L)*DSIGMA(L)                         
         QGINT=QGINT+TRAG3(I,IHEM,JH,L,1)*DSIGMA(L)                     
         TRINT=TRINT+TC(L)*DSIGMA(L)                                    
         QRINT=QRINT+QR(L)*DSIGMA(L)                                    
      ENDDO                                                             
!-----------------------------------------------------                  
! Calculate GH1                                                         
!-----------------------------------------------------                  
      HGINT=TGINT+CTQUSE*QGINT                                          
      DHL=(HGINT-TRINT-CTQUSE*QRINT)/SINT                               
      DO L=NCRT,NCRB                                                    
         GH1(L)=(TC(L)+CTQUSE*QR(L)+DHL)*PSK(L)                         
      ENDDO                                                             
!-----------------------------------------------------                  
! Iteration                                                             
!-----------------------------------------------------                  
      NIT=0                                                             
      DTMAX=99999.99                                                    
      DO WHILE ((DTMAX.GE.EPSIT).AND.(NIT.LE.10))                       
         DTMAX=0.0                                                      
         DO L=NCRT,NCRB                                                 
            QN=CBC(L)*PQSAT(TSL(L))                                     
            DTSL=(TSL(L)+QN-GH1(L))/(1.0+ESCONB*QN/(TSL(L)*TSL(L)))     
            DTMAX=MAX(ABS(DTSL),DTMAX)                                  
            TSL(L)=TSL(L)-DTSL                                          
         ENDDO                                                          
         NIT=NIT+1                                                      
      ENDDO                                                             
!-----------------------------------------------------                  
! Calculate TR and new value for QR                                     
!-----------------------------------------------------                  
      DO L=NCRT,NCRB                                                    
        TR(L)=TSL(L)/PSK(L)                                             
        QR(L)=PQSAT(TSL(L))/PSL(L)                                      
      ENDDO                                                             
!-----------------------------------------------------                  
! Calculate dot product of QR and DSIGMA                                
!-----------------------------------------------------                  
      QRINT=0.0                                                         
      DO L=NCRT,NCRB                                                    
         QRINT=QRINT+QR(L)*DSIGMA(L)                                    
      ENDDO                                                             
!-----------------------------------------------------                  
! Determine if QRINT is greater than QGINT                              
!-----------------------------------------------------                  
      IF(QRINT.GT.QGINT) THEN                                           
C*      WRITE(2,2001) J,JH,NCRB,NCRT,CQ*QRINT/SINT,CQ*QGINT/SINT        
        IF(LCUBM)      CALL CUBM (NCRB,NCRT,I,IHEM,JH)                  
        IF(.NOT.LCUBM) CALL CUDIF(NCRB,NCRT,I,IHEM,JH)                  
      ELSE                                                              
         !-----------------------------------------------------         
         !                                                              
         !-----------------------------------------------------         
         TRINT=0.0                                                      
         DO L=NCRT,NCRB                                                 
            TRINT=TRINT+TR(L)*DSIGMA(L)                                 
         ENDDO                                                          
         DHL=(HGINT-TRINT-CTQUSE*QRINT)/SINT                            
         !-----------------------------------------------------         
         ! Loop across levels                                           
         !-----------------------------------------------------         
         DO L=NCRT,NCRB                                                 
            TR(L)=TR(L)+DHL                                             
            TTEND=(TR(L)-TG3(I,IHEM,JH,L))/CBADJT                       
            QTEND=(QR(L)-TRAG3(I,IHEM,JH,L,1))/CBADJT                   
            TTMC(L)=TTMC(L)+TTEND                                       
            QTMC(L)=QTMC(L)+QTEND                                       
            TG3(I,IHEM,JH,L)=TG3(I,IHEM,JH,L)+DELT2C*TTEND              
            TRAG3(I,IHEM,JH,L,1)=TRAG3(I,IHEM,JH,L,1)+DELT2C*QTEND      
            CTCR(L,IHEM)=CTCR(L,IHEM)+1.0                               
         ENDDO                                                          
         CTCR(NCRB,IHEM)=CTCR(NCRB,IHEM)-1.0                            
         RRCR(I,IHEM,JH)=RRCR(I,IHEM,JH)-RCON*PLG3(I,IHEM,JH)*          
     &        (QRINT-QGINT)/CBADJT                                      
         !-----------------------------------------------------         
         ! Deep convection parameterised by precip.:  P is mm/day       
         !-----------------------------------------------------         
         P=-REAL(ITSPD)*RCON*PLG3(I,IHEM,JH)*(QRINT-QGINT)/CBADJT       
         IF (P.GT.0.15) THEN                                            
C Deep conv cloud vs precip based on Slingo 1987 now includes           
C 75% reduction ABOVE CLOUD BASE (see later) as detailed in that paper  
C            CFRAC(I,IHEM,5)=0.245+0.125*LOG(P)                         
C            CFRAC(I,IHEM,5)=MIN(1.0,CFRAC(I,IHEM,5))                   
            CFRAC(I,IHEM,5)=0.25*(0.245+0.125*LOG(P))                   
            CFRAC(I,IHEM,5)=MIN(0.25,CFRAC(I,IHEM,5))                   
C Large-scale cloud sceheme not designed for conditions                 
C when convective cloud present and RH > 0.8.                           
C Remove middle or low cloud when convective cloud present              
C High level anvil cirrus is allowed                                    
            ICFLAG(I,IHEM,5,1)=NCRB                                     
            ICFLAG(I,IHEM,5,2)=NCRT                                     
         ENDIF                                                          
      ENDIF                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! DEEP CONVECTION SCHEME. NON-ENTRAINING CLOUD MODEL                    
!=====================================================                  
      SUBROUTINE CBCON(NCRB,NCRT,I,IHEM,JH)                             
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Grid variables for DGRMLT loop                                        
!-----------------------------------------------------                  
      COMMON/GRIDPC/ASSBL,ASHBL,ASLBL,ARRCR,ARRLR                       
     &     ,TTVD,QTVD,TYBL,TXBL,TTRD,TTLR,TTCR,UTVD,VTVD                
     &     ,QTLR,QTCR,SSBL,SHBL,SLBL,RRCR,RRLR                          
     &     ,ARFLUX,ASFLD,ACLD,RFLUX,SFLD,CLD                            
      REAL :: ASSBL(MGx,NHEM,JGy)                                       
      REAL :: ASHBL(MGx,NHEM,JGy)                                       
      REAL :: ASLBL(MGx,NHEM,JGy)                                       
      REAL :: ARRCR(MGx,NHEM,JGy)                                       
      REAL :: ARRLR(MGx,NHEM,JGy)                                       
      REAL :: TTVD(MGx,NHEM,NL)                                         
      REAL :: QTVD(MGx,NHEM,NL)                                         
      REAL :: TYBL(MGx,NHEM)                                            
      REAL :: TXBL(MGx,NHEM)                                            
      REAL :: TTRD(MGx,NHEM,NL)                                         
      REAL :: TTLR(MGx,NHEM,NL)                                         
      REAL :: TTCR(MGx,NHEM,NL)                                         
      REAL :: UTVD(MGx,NHEM,NL)                                         
      REAL :: VTVD(MGx,NHEM,NL)                                         
      REAL :: QTLR(MGx,NHEM,NL)                                         
      REAL :: QTCR(MGx,NHEM,NL)                                         
      REAL :: SSBL(MGx,NHEM,JGy)                                        
      REAL :: SHBL(MGx,NHEM,JGy)                                        
      REAL :: SLBL(MGx,NHEM,JGy)                                        
      REAL :: RRCR(MGx,NHEM,JGy)                                        
      REAL :: RRLR(MGx,NHEM,JGy)                                        
      REAL :: arflux(MGx,NHEM,JGy,6)                                    
      REAL :: asfld(MGx,NHEM,JGy,6)                                     
      REAL :: acld(MGx,NHEM,JGy,4)                                      
      REAL :: rflux(MGx,NHEM,JGy,6)                                     
      REAL :: sfld(MGx,NHEM,JGy,6)                                      
      REAL :: cld(MGx,NHEM,JGy,4)                                       
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Passed arguments                                                      
!-----------------------------------------------------                  
      INTEGER :: NCRB                                                   
      INTEGER :: NCRT                                                   
      INTEGER :: I                                                      
      INTEGER :: IHEM                                                   
! Latitude index                                                        
      INTEGER :: JH                                                     
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER :: NCRBM                                                  
      INTEGER :: NCRTP                                                  
      REAL :: DSELP                                                     
      REAL :: QEL                                                       
      INTEGER :: LM                                                     
      REAL :: DSELM                                                     
      REAL :: TTEND                                                     
      REAL :: QTEND                                                     
      REAL :: DSEL                                                      
      REAL :: QELP                                                      
      REAL :: QELM                                                      
! Loop variables                                                        
      INTEGER :: L                                                      
!-----------------------------------------------------                  
!                                                                       
!-----------------------------------------------------                  
      NCRBM=NCRB-1                                                      
      NCRTP=NCRT+1                                                      
      RRCR(I,IHEM,JH)=RRCR(I,IHEM,JH)+PLG3(I,IHEM,JH)*CCR*              
     &     (TRAG3(I,IHEM,JH,NCRB,1)-QC(NCRT))                           
      DSELP=-CTQ*TRAG3(I,IHEM,JH,NCRB,1)                                
      QELP=TRAG3(I,IHEM,JH,NCRB,1)                                      
      DSEL=DSELP                                                        
      QEL=QELP                                                          
      DO L=NCRB,NCRTP,-1                                                
         LM=L-1                                                         
         DSELM=TG3(I,IHEM,JH,LM)-TC(LM)-CTQ*QC(LM)                      
         TTEND=FWS(L)*(DSELP-DSELM)                                     
         QTEND=FWS(L)*(QELP-TRAG3(I,IHEM,JH,LM,1))                      
         TTMC(L)=TTMC(L)+TTEND                                          
         QTMC(L)=QTMC(L)+QTEND                                          
         TG3(I,IHEM,JH,L)=TG3(I,IHEM,JH,L)+DELT2C*TTEND                 
         TRAG3(I,IHEM,JH,L,1)=TRAG3(I,IHEM,JH,L,1)+DELT2C*QTEND         
         DSELP=DSEL                                                     
         DSEL=DSELM                                                     
         QELP=QEL                                                       
         QEL=TRAG3(I,IHEM,JH,LM,1)                                      
      ENDDO                                                             
      DSELM=TC(NCRT)-TG3(I,IHEM,JH,NCRT)-CTQ*QC(NCRT)                   
      QELM=QC(NCRT)+QC(NCRT)-TRAG3(I,IHEM,JH,NCRT,1)                    
      TTEND=FWS(NCRT)*(DSELP-DSELM)                                     
      QTEND=FWS(NCRT)*(QELP-QELM)                                       
      TTMC(NCRT)=TTMC(NCRT)+TTEND                                       
      QTMC(NCRT)=QTMC(NCRT)+QTEND                                       
      TG3(I,IHEM,JH,NCRT)=TG3(I,IHEM,JH,NCRT)+DELT2C*TTEND              
      TRAG3(I,IHEM,JH,NCRT,1)=TRAG3(I,IHEM,JH,NCRT,1)+DELT2C*QTEND      
C                                                                       
      DO L=NCRT,NCRBM                                                   
         CTCR(L,IHEM)=CTCR(L,IHEM)+1.0                                  
      ENDDO                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! CONVECTION SCHEME. THIS ROUTINE FINDS UNSTABLE LAYERS AND             
! CALLS RELEVANT ROUTINES TO CALCULATE TENDENCIES ETC.                  
!=====================================================                  
      SUBROUTINE CONVEC(JH)                                             
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Grid variables for DGRMLT loop                                        
!-----------------------------------------------------                  
      COMMON/GRIDPC/ASSBL,ASHBL,ASLBL,ARRCR,ARRLR                       
     &     ,TTVD,QTVD,TYBL,TXBL,TTRD,TTLR,TTCR,UTVD,VTVD                
     &     ,QTLR,QTCR,SSBL,SHBL,SLBL,RRCR,RRLR                          
     &     ,ARFLUX,ASFLD,ACLD,RFLUX,SFLD,CLD                            
      REAL :: ASSBL(MGx,NHEM,JGy)                                       
      REAL :: ASHBL(MGx,NHEM,JGy)                                       
      REAL :: ASLBL(MGx,NHEM,JGy)                                       
      REAL :: ARRCR(MGx,NHEM,JGy)                                       
      REAL :: ARRLR(MGx,NHEM,JGy)                                       
      REAL :: TTVD(MGx,NHEM,NL)                                         
      REAL :: QTVD(MGx,NHEM,NL)                                         
      REAL :: TYBL(MGx,NHEM)                                            
      REAL :: TXBL(MGx,NHEM)                                            
      REAL :: TTRD(MGx,NHEM,NL)                                         
      REAL :: TTLR(MGx,NHEM,NL)                                         
      REAL :: TTCR(MGx,NHEM,NL)                                         
      REAL :: UTVD(MGx,NHEM,NL)                                         
      REAL :: VTVD(MGx,NHEM,NL)                                         
      REAL :: QTLR(MGx,NHEM,NL)                                         
      REAL :: QTCR(MGx,NHEM,NL)                                         
      REAL :: SSBL(MGx,NHEM,JGy)                                        
      REAL :: SHBL(MGx,NHEM,JGy)                                        
      REAL :: SLBL(MGx,NHEM,JGy)                                        
      REAL :: RRCR(MGx,NHEM,JGy)                                        
      REAL :: RRLR(MGx,NHEM,JGy)                                        
      REAL :: arflux(MGx,NHEM,JGy,6)                                    
      REAL :: asfld(MGx,NHEM,JGy,6)                                     
      REAL :: acld(MGx,NHEM,JGy,4)                                      
      REAL :: rflux(MGx,NHEM,JGy,6)                                     
      REAL :: sfld(MGx,NHEM,JGy,6)                                      
      REAL :: cld(MGx,NHEM,JGy,4)                                       
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed argument                                                       
!-----------------------------------------------------                  
! Latitude index                                                        
      INTEGER :: JH                                                     
!-----------------------------------------------------                  
! External function                                                     
!-----------------------------------------------------                  
      REAL, EXTERNAL :: PQSAT                                           
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      LOGICAL :: LCDRY,LCWET                                            
      REAL :: ESCON(NL),SIGMIN                                          
      INTEGER :: LTOP                                                   
      INTEGER :: LP                                                     
      INTEGER :: NCRB                                                   
      REAL :: TCLP                                                      
      REAL :: QCL                                                       
      REAL :: TCL                                                       
      REAL :: QSL                                                       
      INTEGER :: NCRT                                                   
      LOGICAL :: lCont                                                  
      INTEGER :: NIT                                                    
      REAL :: DQN                                                       
      REAL :: DQ                                                        
      INTEGER :: NCT                                                    
      REAL :: PRESSMIN                                                  
      REAL :: TMIN                                                      
      REAL :: QGO(NL)                                                   
      REAL :: QEXS                                                      
      REAL :: QEX                                                       
! Loop variables                                                        
      INTEGER :: I, IHEM, L                                             
!-----------------------------------------------------                  
! 28-05-97 PMF Water vapour fix to stop too low values                  
! takes water vapour from level below                                   
! this produces a slight vertical advection of Q                        
! assumes no -ve water vapour in bottom layer                           
!-----------------------------------------------------                  
      PRESSMIN=0.20000  !  Minimum parcel pressure for                  
                        !  which moist convection is attempted.         
      TMIN=0.31000418   !  Minimum parcel temperature (dedim.) for      
                        !  which moist convection is attempted.         
      LTOP=NL-NLCR      !  NLCR gives highest level for which           
                        !  convection is attempted.                     
!-----------------------------------------------------                  
! Loop through grid points on this latitude                             
!-----------------------------------------------------                  
      DO IHEM=1,NHEM                                                    
         DO I=1,MGxx(iCol)                                              
C Allow some shallow convection rainout over ocean                      
         IF(SVEGE(I,IHEM,JH).eq.1)THEN                                  
           CURHM=80.0/100.0                                             
           CUBMT=6.0*3600.0*WW                                          
         ELSE                                                           
           CURHM=200.0/100.0                                            
           CUBMT=6.0*3600.0*WW                                          
         ENDIF                                                          
            DO L=1,NL                                                   
               TTDC(L)=0.0                                              
               QTDC(L)=0.0                                              
               TTMC(L)=0.0                                              
               QTMC(L)=0.0                                              
               ESCON(L)=1./(PLG3(I,IHEM,JH)*SIGMA(L))                   
               QGO(L)=TRAG3(I,IHEM,JH,L,1)                              
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Water vapour fix                                          
            !-----------------------------------------------------      
            DO L=1,NL-1                                                 
               IF (TRAG3(I,IHEM,JH,L,1).LT.1.0E-6) THEN                 
                  QEXS=(1.0E-6)-TRAG3(I,IHEM,JH,L,1)                    
                  IF (L.EQ.1) THEN                                      
                     QEXS=QEXS*SIGMAH(1)                                
                  ELSE                                                  
                     QEXS=QEXS*(SIGMAH(L)-SIGMAH(L-1))                  
                  ENDIF                                                 
                  IF (L.NE.NL-1) THEN                                   
                     QEX=QEXS/(SIGMAH(L+1)-SIGMAH(L))                   
                  ELSE                                                  
                     QEX=QEXS/(1.0-SIGMAH(L))                           
                  ENDIF                                                 
                  !---------------------------------------------------  
                  ! Assumes no large scale rain here in layer           
                  !---------------------------------------------------  
                  QTLR(I,IHEM,L)=((1.0E-6)-QGO(L))/DELT2C               
                  QTLR(I,IHEM,L+1)=QTLR(I,IHEM,L+1)-                    
     &                 QEX/DELT2C                                       
                  TRAG3(I,IHEM,JH,L,1)=1.0E-6                           
                  TRAG3(I,IHEM,JH,L+1,1)=TRAG3(I,IHEM,JH,L+1,1)-QEX     
               ENDIF                                                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! DRY ADJUSTMENT                                            
            !-----------------------------------------------------      
            L=NL                                                        
            !-----------------------------------------------------      
            ! Begin new parcel curve                                    
            !-----------------------------------------------------      
 20         NCRB=L                                                      
            LCDRY=.FALSE.                                               
            TCLP=TG3(I,IHEM,JH,L)                                       
            QCL=TRAG3(I,IHEM,JH,L,1)                                    
C     DRY ADIABAT TO NEXT LEVEL                                         
 30         L=L-1                                                       
            IF(L.LT.LTOP) GOTO 40                                       
            TCL=TCLP*SK(L)                                              
            IF(TCL.LE.TG3(I,IHEM,JH,L)) GOTO 40 !If stable layer        
C     BUOYANT - IF SATURATED - MOIST CONVECTION - IGNORE                
            QSL=ESCON(L)*PQSAT(TCL)                                     
            IF ( QCL.GE.QSL ) THEN ! Is parcel supersaturated?          
               IF ( ( (SIGMA(NCRB)*PLG3(I,IHEM,JH)).GE.PRESSMIN )       
     :              .AND.( TG3(I,IHEM,JH,NCRB).GE.TMIN ) ) GOTO 40      
               !-----------------------------------------------------   
               ! If parcel is in stratosphere, do dry rather than       
               ! moist convection as the dry and moist adiabats are     
               ! approximately parallel and dry convection is a         
               ! lot cheaper.                                           
               !-----------------------------------------------------   
            ENDIF                                                       
            !-----------------------------------------------------      
            ! Dry convection, continue parcel curve up.                 
            !-----------------------------------------------------      
 35         LCDRY=.TRUE.                                                
            TCLP=TCL                                                    
            GOTO 30                                                     
 40         CONTINUE                                                    
            !-----------------------------------------------------      
            ! STABLE LAYER OR MODEL TOP - ADJUST ANY UNSTABLE           
            ! LAYER BELOW                                               
            !-----------------------------------------------------      
            IF(LCDRY) THEN                                              
               NCRT=L+1                                                 
               CALL DRYADJ(NCRB,NCRT,I,IHEM,JH)                         
            ENDIF                                                       
            IF(L.GT.LTOP) GOTO 20                                       
            !-----------------------------------------------------      
            ! MOIST CONVECTION                                          
            !-----------------------------------------------------      
            L=NL                                                        
C         BEGIN NEW PARCEL CURVE                                        
 100        NCRB=L                                                      
            LCWET=.FALSE.                                               
            IF ( (SIGMA(NCRB)*PLG3(I,IHEM,JH)).LT.PRESSMIN ) THEN       
               L=LTOP                                                   
               GOTO 150                                                 
            ELSEIF ( TG3(I,IHEM,JH,NCRB).LT.TMIN ) THEN                 
               L=L-1                                                    
               GOTO 100                                                 
            ENDIF                                                       
            TC(L)=TG3(I,IHEM,JH,L)                                      
            QC(L)=TRAG3(I,IHEM,JH,L,1)                                  
C     UP ONE LEVEL - DRY ADIABAT AS FIRST GUESS                         
 110        LP=L                                                        
            L=L-1                                                       
            IF(L.LT.LTOP) GOTO 150                                      
            TC(L)=TC(LP)*SK(L)                                          
            QC(L)=QC(LP)                                                
            QSL=ESCON(L)*PQSAT(TC(L))                                   
C     IF SATURATED MAY BE MOIST CONVECTION                              
            IF(QC(L).GE.QSL) GOTO 120                                   
C     TEST FOR DRY INSTABILITY (SHOULD BE STABLE)                       
            GOTO 150            !No moist convection                    
C     POSSIBLE MOIST CONVECTION - ITERATE FOR THETAE (T,Q)              
 120        NIT=0                                                       
 130        NIT=NIT+1                                                   
            DQN=QC(L)-QSL                                               
            DQ=DQN/(1.0+CCC*QSL/(TC(L)*TC(L)))                          
            QC(L)=QC(L)-DQ                                              
            TC(L)=TC(L)+CTQ*DQ                                          
            IF(ABS(DQN).LT.EPSIQ) GOTO 140                              
            IF(NIT.GT.10) GOTO 140                                      
            QSL=ESCON(L)*PQSAT(TC(L))                                   
            GOTO 130                                                    
C     BUOYANCY TEST - IF STABLE GO TO 150                               
 140        IF(TC(L).LT.TG3(I,IHEM,JH,L)) GOTO 150                      
C     MOIST CONVECTION - CONTINUE PARCEL CURVE UP                       
            LCWET=.TRUE.                                                
            GOTO 110                                                    
 150        CONTINUE                                                    
            !-----------------------------------------------------      
            ! STABLE LAYER OR MODEL TOP - ADJUST ANY UNSTABLE           
            ! LAYER BELOW                                               
            !-----------------------------------------------------      
            IF(LCWET) THEN                                              
               NCRT=LP                                                  
               IF(NCRT.LT.NCUTOP) THEN                                  
                  IF(.NOT.LCBADJ) CALL CBCON(NCRB,NCRT,I,IHEM,JH)       
                  IF(LCBADJ)      CALL CBADJ(NCRB,NCRT,I,IHEM,JH)       
               ELSE                                                     
                  NCT=NCRT                                              
                  IF(.NOT.LCUBM)  CALL CUDIF(NCRB,NCT,I,IHEM,JH)        
                  IF(LCUBM)       CALL CUBM (NCRB,NCT,I,IHEM,JH)        
               ENDIF                                                    
            ENDIF                                                       
            IF(L.GT.LTOP) GOTO 100                                      
            !-----------------------------------------------------      
            ! COLLECT TENDENCIES AND INCREMENT T,Q FOR MOIST            
            ! CONVECTION                                                
            !-----------------------------------------------------      
            DO L=1,NL                                                   
               TTCR(I,IHEM,L)=TTDC(L)+TTMC(L)                           
               QTCR(I,IHEM,L)=QTDC(L)+QTMC(L)+QTLR(I,IHEM,L)            
               QTLR(I,IHEM,L)=0.0                                       
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! BETTS-MILLER NON-PRECIPITATING CONVECTION. FOLLOWS ECMWF TECH REP     
! NO 43. VARIABLE TIMESCALE AND CLOUD-TOP FLUXES WHEN SHALLOW.          
! OPTIONAL RAINOUT OVER A SPECIFIED REL-HUM.                            
!=====================================================                  
      SUBROUTINE CUBM(NCRB,NCT,I,IHEM,JH)                               
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Grid variables for DGRMLT loop                                        
!-----------------------------------------------------                  
      COMMON/GRIDPC/ASSBL,ASHBL,ASLBL,ARRCR,ARRLR                       
     &     ,TTVD,QTVD,TYBL,TXBL,TTRD,TTLR,TTCR,UTVD,VTVD                
     &     ,QTLR,QTCR,SSBL,SHBL,SLBL,RRCR,RRLR                          
     &     ,ARFLUX,ASFLD,ACLD,RFLUX,SFLD,CLD                            
      REAL :: ASSBL(MGx,NHEM,JGy)                                       
      REAL :: ASHBL(MGx,NHEM,JGy)                                       
      REAL :: ASLBL(MGx,NHEM,JGy)                                       
      REAL :: ARRCR(MGx,NHEM,JGy)                                       
      REAL :: ARRLR(MGx,NHEM,JGy)                                       
      REAL :: TTVD(MGx,NHEM,NL)                                         
      REAL :: QTVD(MGx,NHEM,NL)                                         
      REAL :: TYBL(MGx,NHEM)                                            
      REAL :: TXBL(MGx,NHEM)                                            
      REAL :: TTRD(MGx,NHEM,NL)                                         
      REAL :: TTLR(MGx,NHEM,NL)                                         
      REAL :: TTCR(MGx,NHEM,NL)                                         
      REAL :: UTVD(MGx,NHEM,NL)                                         
      REAL :: VTVD(MGx,NHEM,NL)                                         
      REAL :: QTLR(MGx,NHEM,NL)                                         
      REAL :: QTCR(MGx,NHEM,NL)                                         
      REAL :: SSBL(MGx,NHEM,JGy)                                        
      REAL :: SHBL(MGx,NHEM,JGy)                                        
      REAL :: SLBL(MGx,NHEM,JGy)                                        
      REAL :: RRCR(MGx,NHEM,JGy)                                        
      REAL :: RRLR(MGx,NHEM,JGy)                                        
      REAL :: arflux(MGx,NHEM,JGy,6)                                    
      REAL :: asfld(MGx,NHEM,JGy,6)                                     
      REAL :: acld(MGx,NHEM,JGy,4)                                      
      REAL :: rflux(MGx,NHEM,JGy,6)                                     
      REAL :: sfld(MGx,NHEM,JGy,6)                                      
      REAL :: cld(MGx,NHEM,JGy,4)                                       
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Radiation variables                                                   
!-----------------------------------------------------                  
      COMMON/CPIERS/ICFLAG,CFRAC,PNET,SNET,RRFLUX,TTSW,TTLW             
      INTEGER :: ICFLAG(MGx,NHEM,5,2)                                   
      REAL :: CFRAC(MGx,NHEM,5)                                         
      REAL :: PNET(MGx,NHEM,JGy)                                        
      REAL :: SNET(MGx,NHEM,JGy)                                        
      REAL :: RRFLUX(MGx,NHEM,JGy,6)                                    
      REAL :: TTSW(MGx,NHEM,NL)                                         
      REAL :: TTLW(MGx,NHEM,NL)                                         
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed arguments                                                      
!-----------------------------------------------------                  
      INTEGER :: NCRB                                                   
      INTEGER :: NCT                                                    
      INTEGER :: I                                                      
      INTEGER :: IHEM                                                   
! Latitude index                                                        
      INTEGER :: JH                                                     
!-----------------------------------------------------                  
! External function                                                     
!-----------------------------------------------------                  
      REAL, EXTERNAL :: FTSL                                            
      REAL, EXTERNAL :: PQSAT                                           
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: QGMIN                                                     
      INTEGER :: NCRT                                                   
      REAL :: CUTAU                                                     
      REAL :: TLIM                                                      
      INTEGER :: NLEV                                                   
      REAL :: RKAP                                                      
      REAL :: PL(NL)                                                    
      REAL :: PFK(NL)                                                   
      REAL :: TH(NL)                                                    
      REAL :: QQG                                                       
      REAL :: TSL(NL)                                                   
      REAL :: PSL(NL)                                                   
      REAL :: SCRIP(NL)                                                 
      REAL :: THM                                                       
      REAL :: QM                                                        
      REAL :: TMT                                                       
      REAL :: TSLM                                                      
      REAL :: PSLM                                                      
      REAL :: TLAPSM                                                    
      REAL :: SINT                                                      
      REAL :: SCRINT                                                    
      REAL :: THR(NL)                                                   
      REAL :: TR(NL)                                                    
      REAL :: TGINT                                                     
      REAL :: TRINT                                                     
      REAL :: DTINT                                                     
      REAL :: QR(NL)                                                    
      REAL :: QGINT                                                     
      REAL :: QRINT                                                     
      REAL :: DQINT                                                     
      REAL :: TGP                                                       
      REAL :: QGP                                                       
      REAL :: QS                                                        
      REAL :: QEXS                                                      
      REAL :: CTQUSE                                                    
      REAL :: QEX                                                       
      REAL :: QTCOR                                                     
! Loop variable                                                         
      INTEGER :: L                                                      
!-----------------------------------------------------                  
! Format statements                                                     
!-----------------------------------------------------                  
 2103 FORMAT(' CUBM LAPSE RATE LIMIT IMPOSED: I,IHEM,J,NCRB,NCRT = ',   
     &     5I5)                                                         
 2104 FORMAT(' CUBM LAPSE RATE LIMIT IMPOSED: I,IHEM,J,NCRB,NCRT,TLIM   
     +                                                     = ',5I5,F7.4)
!-----------------------------------------------------                  
! Minimum moisture for mixing line calculations.                        
!-----------------------------------------------------                  
      QGMIN=1.E-6                                                       
      IF(NCT.GE.NCUTOP) THEN                                            
         IF(BLVB(I,IHEM).EQ.0.0)RETURN                                  
         NCRT=NCT-1                                                     
         CUTAU=CUT1/(BLVB(I,IHEM)*BLCD(I,IHEM))                         
      ELSEIF (NCRB.GE.NCUTOP) THEN                                      
         !-----------------------------------------------------         
         ! Limit the convective top for swap points to the maximum      
         ! height of shallow convection.                                
         ! Note that NCRT includes the level above cloud top.           
         !-----------------------------------------------------         
         NCRT=NCUTOP-1                                                  
         CUTAU=CUBMT                                                    
      ELSE                                                              
         !-----------------------------------------------------         
         ! No adjustment for mid-level swap points.                     
         !-----------------------------------------------------         
         RETURN                                                         
      ENDIF                                                             
!-----------------------------------------------------                  
! Stable limit to mixing line slope d(theta)/dp=-10^-3 K/Pa             
!-----------------------------------------------------                  
      TLIM=-1.0E-3*P0/CT                                                
!-----------------------------------------------------                  
! Shallow convection                                                    
!-----------------------------------------------------                  
      ICFLAG(I,IHEM,4,1)=NCRB                                           
      ICFLAG(I,IHEM,4,2)=NCRT                                           
      CFRAC(I,IHEM,4)=0.35  ! cloud fraction for all shallow convection 
      NLEV=1+NCRB-NCRT                                                  
      RKAP=1.0/AKAP                                                     
      DO L=NCRT,NCRB                                                    
         PL(L)=SIGMA(L)*PLG3(I,IHEM,JH)                                 
         PFK(L)=PL(L)**AKAP                                             
         TH(L)=TG3(I,IHEM,JH,L)/PFK(L)                                  
         !-----------------------------------------------------         
         ! Apply a consistent minimum moisture criterion in the         
         ! mixing line calculations.                                    
         !-----------------------------------------------------         
         QQG=MAX(TRAG3(I,IHEM,JH,L,1),QGMIN)                            
         TSL(L)=FTSL(TG3(I,IHEM,JH,L),QQG,PL(L))                        
         PSL(L)=PL(L)*(TSL(L)/TG3(I,IHEM,JH,L))**RKAP                   
         SCRIP(L)=PSL(L)-PL(L)                                          
      ENDDO                                                             
C                                                                       
      THM=0.5*(TH(NCRB)+TH(NCRT))                                       
      QM=0.5*(TRAG3(I,IHEM,JH,NCRB,1)+TRAG3(I,IHEM,JH,NCRT,1))          
      QM=MAX(QM,QGMIN)                                                  
      TMT=THM*PFK(NCRT)                                                 
      TSLM=FTSL(TMT,QM,PL(NCRT))                                        
      PSLM=PL(NCRT)*(TSLM/TMT)**RKAP                                    
      TLAPSM=(THM-TH(NCRB))/(PSLM-PSL(NCRB))                            
      TLAPSM=MAX(TLAPSM,TLIM) ! limit stability                         
      TLAPSM=MIN(TLAPSM,0.0) ! prevents instability                     
      IF (TLAPSM.GT.0.0) THEN                                           
         WRITE(200+MyPe,2103) I,IHEM,JH,NCRB,NCRT                       
      ELSEIF (TLAPSM.LT.TLIM)THEN                                       
         WRITE(200+MyPe,2104) I,IHEM,JH,NCRB,NCRT,TLIM                  
      ENDIF                                                             
!-----------------------------------------------------                  
! Sum sigma levels                                                      
!-----------------------------------------------------                  
      SINT=0.0                                                          
      DO L=NCRT,NCRB                                                    
         SINT=SINT+DSIGMA(L)                                            
      ENDDO                                                             
!-----------------------------------------------------                  
! Dot product of SCRIP and DSIGMA                                       
!-----------------------------------------------------                  
      SCRINT=0.0                                                        
      DO L=NCRT,NCRB                                                    
         SCRINT=SCRINT+SCRIP(L)*DSIGMA(L)                               
      ENDDO                                                             
      SCRINT=SCRINT/SINT                                                
      SCRINT=MIN(0.0,SCRINT)                                            
!-----------------------------------------------------                  
! Loop across levels                                                    
!-----------------------------------------------------                  
      DO L=NCRT,NCRB                                                    
         THR(L)=TH(NCRB)+TLAPSM*(PL(L)-PL(NCRB))                        
         TR(L) =THR(L)*PFK(L)                                           
      ENDDO                                                             
!-----------------------------------------------------                  
! Dot prodcuts                                                          
!-----------------------------------------------------                  
      TGINT=0.0                                                         
      TRINT=0.0                                                         
      DO L=NCRT,NCRB                                                    
         TGINT=TGINT+TG3(I,IHEM,JH,L)*DSIGMA(L)                         
         TRINT=TRINT+TR(L)*DSIGMA(L)                                    
      ENDDO                                                             
      TGINT=TGINT/SINT                                                  
      TRINT=TRINT/SINT                                                  
      DTINT=TGINT-TRINT                                                 
!-----------------------------------------------------                  
! Loop across levels                                                    
!-----------------------------------------------------                  
      DO L=NCRT,NCRB                                                    
         TR(L)=TR(L)+DTINT                                              
         PSL(L)=SCRINT+PL(L)                                            
         TSL(L)=TR(L)*(PSL(L)/PL(L))**AKAP                              
         QR(L)=PQSAT(TSL(L))/PSL(L)                                     
      ENDDO                                                             
!-----------------------------------------------------                  
! Dot products                                                          
!-----------------------------------------------------                  
      QGINT=0.0                                                         
      QRINT=0.0                                                         
      DO L=NCRT,NCRB                                                    
         QGINT=QGINT+TRAG3(I,IHEM,JH,L,1)*DSIGMA(L)                     
         QRINT=QRINT+QR(L)*DSIGMA(L)                                    
      ENDDO                                                             
      QGINT=QGINT/SINT                                                  
      QRINT=QRINT/SINT                                                  
      DQINT=QGINT-QRINT                                                 
!-----------------------------------------------------                  
! Loop across levels                                                    
!-----------------------------------------------------                  
      DO L=NCRT,NCRB                                                    
         QR(L)=QR(L)+DQINT                                              
         TTMC(L)=(TR(L)-TG3(I,IHEM,JH,L))/CUTAU                         
         QTMC(L)=(QR(L)-TRAG3(I,IHEM,JH,L,1))/CUTAU                     
      ENDDO                                                             
C                                                                       
      IF(CURHM.LT.1.00001) THEN                                         
         DO L=NCRT,NCRB                                                 
            TGP=TG3(I,IHEM,JH,L)+DELT2C*TTMC(L)                         
            QGP=TRAG3(I,IHEM,JH,L,1)+DELT2C*QTMC(L)                     
            QS=CURHM*PQSAT(TGP)/PL(L)                                   
            QEXS=QGP-QS                                                 
            !-----------------------------------------------------      
            ! Choose appropriate value of latent heat based on          
            ! ground T ie, we assume it snows if the ground is          
            ! colder than 273.15K                                       
            !-----------------------------------------------------      
            IF (tstar(I,IHEM,JH).gt.0.363224029) THEN                   
               ctquse=ctq                                               
            ELSE                                                        
               ctquse=ctqi                                              
            ENDIF                                                       
            QEX=QEXS/(1.0+CTQ*QS*ESCONB/(TGP*TGP))                      
            IF (QEXS.LT.0.0) THEN                                       
               QEX=0.0                                                  
            END IF                                                      
            QTCOR=-QEX/DELT2C                                           
            TTMC(L)=TTMC(L)-CTQUSE*QTCOR                                
            QTMC(L)=QTMC(L)+QTCOR                                       
            RRCR(I,IHEM,JH)=RRCR(I,IHEM,JH)-QTCOR*CLR(L)*PLG3(I,IHEM,JH)
         ENDDO                                                          
      ENDIF                                                             
C                                                                       
      DO L=NCRT,NCRB                                                    
         TG3(I,IHEM,JH,L)=TG3(I,IHEM,JH,L)+DELT2C*TTMC(L)               
         TRAG3(I,IHEM,JH,L,1)=TRAG3(I,IHEM,JH,L,1)+DELT2C*QTMC(L)       
         CTCR(L,IHEM)=CTCR(L,IHEM)+1.0                                  
      ENDDO                                                             
      CTCR(NCRB,IHEM)=CTCR(NCRB,IHEM)-1.0                               
      CTCR(NCRT,IHEM)=CTCR(NCRT,IHEM)-1.0                               
C                                                                       
      RETURN                                                            
      END                                                               
!=====================================================                  
! NON-PRECIPITATING CONVECTION. DIFFUSION TOWARDS MIXING-LINE THETA     
! AND CONSTANT Q. CLOUD-TOP FLUXES AND VARIABLE K'S WHEN SHALLOW.       
! OPTIONAL RAINOUT OVER A SPECIFIED REL-HUM.                            
!=====================================================                  
      SUBROUTINE CUDIF(NCRB,NCT,I,IHEM,JH)                              
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Grid variables for DGRMLT loop                                        
!-----------------------------------------------------                  
      COMMON/GRIDPC/ASSBL,ASHBL,ASLBL,ARRCR,ARRLR                       
     &     ,TTVD,QTVD,TYBL,TXBL,TTRD,TTLR,TTCR,UTVD,VTVD                
     &     ,QTLR,QTCR,SSBL,SHBL,SLBL,RRCR,RRLR                          
     &     ,ARFLUX,ASFLD,ACLD,RFLUX,SFLD,CLD                            
      REAL :: ASSBL(MGx,NHEM,JGy)                                       
      REAL :: ASHBL(MGx,NHEM,JGy)                                       
      REAL :: ASLBL(MGx,NHEM,JGy)                                       
      REAL :: ARRCR(MGx,NHEM,JGy)                                       
      REAL :: ARRLR(MGx,NHEM,JGy)                                       
      REAL :: TTVD(MGx,NHEM,NL)                                         
      REAL :: QTVD(MGx,NHEM,NL)                                         
      REAL :: TYBL(MGx,NHEM)                                            
      REAL :: TXBL(MGx,NHEM)                                            
      REAL :: TTRD(MGx,NHEM,NL)                                         
      REAL :: TTLR(MGx,NHEM,NL)                                         
      REAL :: TTCR(MGx,NHEM,NL)                                         
      REAL :: UTVD(MGx,NHEM,NL)                                         
      REAL :: VTVD(MGx,NHEM,NL)                                         
      REAL :: QTLR(MGx,NHEM,NL)                                         
      REAL :: QTCR(MGx,NHEM,NL)                                         
      REAL :: SSBL(MGx,NHEM,JGy)                                        
      REAL :: SHBL(MGx,NHEM,JGy)                                        
      REAL :: SLBL(MGx,NHEM,JGy)                                        
      REAL :: RRCR(MGx,NHEM,JGy)                                        
      REAL :: RRLR(MGx,NHEM,JGy)                                        
      REAL :: arflux(MGx,NHEM,JGy,6)                                    
      REAL :: asfld(MGx,NHEM,JGy,6)                                     
      REAL :: acld(MGx,NHEM,JGy,4)                                      
      REAL :: rflux(MGx,NHEM,JGy,6)                                     
      REAL :: sfld(MGx,NHEM,JGy,6)                                      
      REAL :: cld(MGx,NHEM,JGy,4)                                       
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed arguments                                                      
!-----------------------------------------------------                  
      INTEGER :: NCRB                                                   
      INTEGER :: NCT                                                    
      INTEGER :: I                                                      
      INTEGER :: IHEM                                                   
! Latitude index                                                        
      INTEGER :: JH                                                     
!-----------------------------------------------------                  
! External functions                                                    
!-----------------------------------------------------                  
      REAL, EXTERNAL :: FTSL                                            
      REAL, EXTERNAL :: PQSAT                                           
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: QGMIN                                                     
      REAL :: TLIM                                                      
      REAL :: ESCON                                                     
      INTEGER :: NCRT                                                   
      REAL :: AKTCU                                                     
      REAL :: AKQCU                                                     
      INTEGER :: NLEV                                                   
      INTEGER :: NCRTP                                                  
      INTEGER :: NCRBM                                                  
      REAL :: RKAP                                                      
      REAL :: PL(NL)                                                    
      REAL :: PFK(NL)                                                   
      REAL :: THB                                                       
      REAL :: QQG                                                       
      REAL :: TSB                                                       
      REAL :: PSLB                                                      
      REAL :: THM                                                       
      REAL :: QM                                                        
      REAL :: TMT                                                       
      REAL :: TSLM                                                      
      REAL :: PSLM                                                      
      REAL :: TLAPSM                                                    
      REAL :: THRL                                                      
      REAL :: TR(NL)                                                    
      REAL :: TLPH                                                      
      REAL :: FPH                                                       
      REAL :: DGPH                                                      
      REAL :: DTPH                                                      
      REAL :: FMH                                                       
      REAL :: DQMH                                                      
      REAL :: DTMH                                                      
      REAL :: DQPH                                                      
      REAL :: SINT                                                      
      REAL :: TTCOR                                                     
      REAL :: TGP                                                       
      REAL :: QGP                                                       
      REAL :: QS                                                        
      REAL :: QEXS                                                      
      REAL :: QEX                                                       
      REAL :: QTCOR                                                     
! Loop variables                                                        
      INTEGER :: L                                                      
!-----------------------------------------------------                  
! Format statments                                                      
!-----------------------------------------------------                  
 2001 FORMAT(' CUDIF QMIN APPLIED AT BAS: I,IHEM,J,NCRB,NCRT,QG =',     
     &     6I5,F10.4)                                                   
 2002 FORMAT(' CUDIF QMIN APPLIED FOR QM: I,IHEM,J,NCRB,NCRT,QM =',     
     &     5I5,F10.4)                                                   
 2103 FORMAT(' CUBM LAPSE RATE LIMIT IMPOSED: I,IHEM,J,NCRB,NCRT = ',   
     &     5I5)                                                         
 2104 FORMAT(' CUBM LAPSE RATE LIMIT IMPOSED: I,IHEM,J,NCRB,NCRT,TLIM', 
     &       ' = ',5I5,F7.4)                                            
!-----------------------------------------------------                  
! Stable limit to mixing line slope d(theta)/dp=-10^-3 K/Pa             
!-----------------------------------------------------                  
      TLIM=-1.0E-3*P0/CT                                                
C                                                                       
      ESCON=1./PLG3(I,IHEM,JH)                                          
      IF(NCT.GE.NCUTOP) THEN                                            
         IF(BLVB(I,IHEM).EQ.0.0)RETURN                                  
         NCRT=NCT-1                                                     
         AKTCU=BLVB(I,IHEM)*BLCD(I,IHEM)*CUT2/CUT1                      
      ELSE IF (NCRB.GE.NCUTOP) THEN                                     
         !-----------------------------------------------------         
         ! Limit the convective top for swap points to the maximum      
         ! height of shallow convection.                                
         ! Note that NCRT includes the level above cloud top.           
         !-----------------------------------------------------         
         NCRT=NCUTOP-1                                                  
         AKTCU=AKTC                                                     
      ELSE                                                              
         !-----------------------------------------------------         
         ! No adjustment for mid-level swap points.                     
         !-----------------------------------------------------         
         RETURN                                                         
      ENDIF                                                             
      AKQCU=AKTCU*AKQC/AKTC                                             
      NLEV=1+NCRB-NCRT                                                  
      NCRTP=NCRT+1                                                      
      NCRBM=NCRB-1                                                      
C                                                                       
      RKAP=1.0/AKAP                                                     
      DO L=NCRT,NCRB                                                    
         PL(L)=SIGMA(L)*PLG3(I,IHEM,JH)                                 
         PFK(L)=PL(L)**AKAP                                             
      ENDDO                                                             
      THB=TG3(I,IHEM,JH,NCRB)/PFK(NCRB)                                 
      IF (TRAG3(I,IHEM,JH,NCRB,1).LT.QGMIN) THEN                        
         WRITE(200+MyPe,2001) I,IHEM,JH,L,NCRB,NCRT,                    
     &        CQ*TRAG3(I,IHEM,JH,NCRB,1)                                
      ENDIF                                                             
!-----------------------------------------------------                  
! Apply a consistent minimum moisture criterion in the mixing line      
! calculations.                                                         
!-----------------------------------------------------                  
      QQG=MAX(TRAG3(I,IHEM,JH,NCRB,1),QGMIN)                            
      TSB=FTSL(TG3(I,IHEM,JH,NCRB),QQG,PL(NCRB))                        
      PSLB=PL(NCRB)*(TSB/TG3(I,IHEM,JH,NCRB))**RKAP                     
      THM=0.5*(THB+TG3(I,IHEM,JH,NCRT)/PFK(NCRT))                       
      QM =0.5*(TRAG3(I,IHEM,JH,NCRB,1)+TRAG3(I,IHEM,JH,NCRT,1))         
      IF (QM.LT.QGMIN) THEN                                             
         WRITE(2,2002) I,IHEM,JH,NCRB,NCRT,CQ*QM                        
      ENDIF                                                             
      QM=MAX(QM,QGMIN)                                                  
      TMT=THM*PFK(NCRT)                                                 
      TSLM=FTSL(TMT,QM,PL(NCRT))                                        
      PSLM=PL(NCRT)*(TSLM/TMT)**RKAP                                    
      TLAPSM=(THM-THB)/(PSLM-PSLB)                                      
      TLAPSM=MAX(TLAPSM,TLIM) ! limit stability                         
      TLAPSM=MIN(TLAPSM,0.0) ! prevents instability                     
      IF (TLAPSM.GT.0.0) THEN                                           
         WRITE(200+MyPe,2103) I,IHEM,JH,NCRB,NCRT                       
      ELSEIF (TLAPSM.LT.TLIM)THEN                                       
         WRITE(200+MyPe,2104) I,IHEM,JH,NCRB,NCRT,TLIM                  
      ENDIF                                                             
                                                                        
      DO L=NCRT,NCRB                                                    
         THRL=THB+(PL(L)-PL(NCRB))*TLAPSM                               
         TR(L)=THRL*PFK(L)                                              
      ENDDO                                                             
C                                                                       
      TLPH=TG3(I,IHEM,JH,NCRT)+TG3(I,IHEM,JH,NCRTP)                     
      FPH=FB(NCRT)/(TLPH*TLPH)                                          
      DQPH=TRAG3(I,IHEM,JH,NCRTP,1)-TRAG3(I,IHEM,JH,NCRT,1)             
      DTPH=SK(NCRT)*(TG3(I,IHEM,JH,NCRTP)-TR(NCRTP))-                   
     &     (TG3(I,IHEM,JH,NCRT)-TR(NCRT))                               
      TTMC(NCRT)=AKTCU*FPH*DTPH/DSIGMA(NCRT)                            
      QTMC(NCRT)=AKQCU*FPH*DQPH/DSIGMA(NCRT)                            
C                                                                       
      IF(NLEV.GT.2) THEN                                                
         DO L=NCRTP,NCRBM                                               
            FMH=FPH                                                     
            DQMH=DQPH                                                   
            DTMH=DTPH/SK(L-1)                                           
            TLPH=TG3(I,IHEM,JH,L+1)+TG3(I,IHEM,JH,L)                    
            FPH=FB(L)/(TLPH*TLPH)                                       
            DQPH=TRAG3(I,IHEM,JH,L+1,1)-TRAG3(I,IHEM,JH,L,1)            
            DTPH=SK(L)*(TG3(I,IHEM,JH,L+1)-TR(L+1))-                    
     &           (TG3(I,IHEM,JH,L)-TR(L))                               
            TTMC(L)=AKTCU*(FPH*DTPH-FMH*DTMH)/DSIGMA(L)                 
            QTMC(L)=AKQCU*(FPH*DQPH-FMH*DQMH)/DSIGMA(L)                 
            CTCR(L,IHEM)=CTCR(L,IHEM)+1.0                               
         ENDDO                                                          
      ENDIF                                                             
C                                                                       
      TTMC(NCRB)=-AKTCU*FPH*DTPH/(SK(NCRBM)*DSIGMA(NCRB))               
      QTMC(NCRB)=-AKQCU*FPH*DQPH/DSIGMA(NCRB)                           
!-----------------------------------------------------                  
! Sum sigma levels                                                      
!-----------------------------------------------------                  
      SINT=0.0                                                          
      DO L=NCRT,NCRB                                                    
         SINT=SINT+DSIGMA(L)                                            
      ENDDO                                                             
!-----------------------------------------------------                  
! Dot product of TTMC and DISGMA                                        
!-----------------------------------------------------                  
      TTCOR=0.0                                                         
      DO L=NCRT,NCRB                                                    
         TTCOR=TTCOR+TTMC(L)*DSIGMA(L)                                  
      ENDDO                                                             
      TTCOR=TTCOR/SINT                                                  
!-----------------------------------------------------                  
! Loop across levels                                                    
!-----------------------------------------------------                  
      DO L=NCRT,NCRB                                                    
         TTMC(L)=TTMC(L)-TTCOR                                          
      ENDDO                                                             
C                                                                       
      IF(CURHM.LT.1.00001) THEN                                         
         DO L=NCRT,NCRB                                                 
            TGP=TG3(I,IHEM,JH,L)+DELT2C*TTMC(L)                         
            QGP=TRAG3(I,IHEM,JH,L,1)+DELT2C*QTMC(L)                     
            QS=CURHM*ESCON*PQSAT(TGP)/SIGMA(L)                          
            QEXS=QGP-QS                                                 
            QEX=QEXS/(1.0+CTQ*QS*ESCONB/(TGP*TGP))                      
            IF (QEXS.LT.0.0) THEN                                       
               QEX=0.0                                                  
            END IF                                                      
            QTCOR=-QEX/DELT2C                                           
            TTMC(L)=TTMC(L)-CTQ*QTCOR                                   
            QTMC(L)=QTMC(L)+QTCOR                                       
            RRCR(I,IHEM,JH)=RRCR(I,IHEM,JH)-QTCOR*CLR(L)*PLG3(I,IHEM,JH)
         ENDDO                                                          
      ENDIF                                                             
C                                                                       
      DO L=NCRT,NCRB                                                    
         TG3(I,IHEM,JH,L)=TG3(I,IHEM,JH,L)+DELT2C*TTMC(L)               
         TRAG3(I,IHEM,JH,L,1)=TRAG3(I,IHEM,JH,L,1)+DELT2C*QTMC(L)       
      ENDDO                                                             
C                                                                       
      RETURN                                                            
      END                                                               
!=====================================================                  
! Perform all the direct Legendre transforms for the diabatic           
! part of the timestep at the current latitude (pair), in place         
! of separate calls to HANAL for individual transform types.            
!                                                                       
! An even/odd value of ITYPE denotes a spectral field of even/odd       
! symmetry.                                                             
!                                                                       
! Maximum vector efficiency is achieved by chaining all multi-level     
! transforms in one loop and by chaining all single-level transforms    
! in a second loop.                                                     
!                                                                       
! All dummy argument arrays are declared complex.                       
! All array dimensions are parameters.                                  
! Multi-level arrays are 3-dimensional.                                 
!                                                                       
! Tracers (QNLG,QT) are included conditionally inside inner loops.      
! This does *not* affect vectorisation, since the number of tracers,    
! NTRAC, is a parameter and the relevant IF constructs are removed      
! at compilation.  The same applies to code conditional on NHEM.        
! (Only tested on Cray J90 with cf77).                                  
!                                                                       
! NOTE: The y-derivative transforms use integration by parts and        
!       are valid only if the input field has zero zonal mean at        
!       both poles.                                                     
!                                                                       
! NOTE: *** THE INPUT FOURIER FIELDS ARE MODIFIED IF GLOBAL ***         
!                                                                       
! Original version for IGCM2.             Mike Blackburn,  04.09.98.    
!                                                                       
!=====================================================                  
      SUBROUTINE DANALV                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Polynomial used to aid vectorization of Legendre transforms           
!-----------------------------------------------------                  
      COMMON/POLYNO/POLY,CMPA                                           
      REAL :: POLY(NNT,MMTy,2,2)                                        
      COMPLEX :: CMPA(2*MMTy,NHEM)                                      
!-----------------------------------------------------                  
! Half transforms. The levels are divided across the columns of PEs;    
! the zonal wavenumbers are divided across the row of PEs; and on       
! ech PE the latitude are complete.                                     
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA/UG,FUG,VG,FVG,TG,TNLG,PLG                           
     &     ,TRAG,TRANLG                                                 
! U_psi, then U (rotational wind)                                       
      COMPLEX :: UG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FUG(2*MMTy,NHEM,JG,NLx)                                
! V_chi, then V (divergent wind)                                        
      COMPLEX :: VG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FVG(2*MMTy,NHEM,JG,NLx)                                
! Temperature anomaly (T')                                              
      COMPLEX :: TG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: TNLG(2*MMTy,NHEM,JG,NLx)                               
! ln p* or p*                                                           
      COMPLEX :: PLG(2*MMTy,NHEM,JG)                                    
! Tracer term                                                           
      COMPLEX :: TRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                         
!                                                                       
      COMPLEX :: TRANLG(2*MMTy,NHEM,JG,NLx,NTRAC)                       
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB/ZG,DG,PJG,PMG,SPG,VPG,EG,UTG,VTG                    
     &     ,UTRAG,VTRAG                                                 
     &     ,CHIG,SFG,FVGT,FUGT                                          
! Vorticity (zeta)                                                      
      COMPLEX :: ZG(2*MMTy,NHEM,JG,NLx)                                 
! Divergence (d)                                                        
      COMPLEX :: DG(2*MMTy,NHEM,JG,NLx)                                 
! (1-mu^2) (d(ln p*)/d mu)                                              
      COMPLEX :: PJG(2*MMTy,NHEM,JG)                                    
! (d(ln p*)/d lambda)                                                   
      COMPLEX :: PMG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: SPG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: VPG(2*MMTy,NHEM,JG)                                    
! (U^2 + V^2)                                                           
      COMPLEX :: EG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: UTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: VTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: UTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: VTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: CHIG(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: SFG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: FVGT(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: FUGT(2*MMTy,NHEM,JG,NLx)                               
!-----------------------------------------------------                  
! Like the variables in SPECTR but the meridional wavenumbers           
! are complete instead of the levels.                                   
!-----------------------------------------------------                  
      COMMON/SPECTR2/Z2,D2,T2,SP2,SPA2,VP2,DTE2,TT2,DT2,ZT2             
     &     ,TRA2,TRAT2                                                  
! Vorticity coefficient                                                 
      COMPLEX :: Z2(NNT,MMTy,NHEM,NLx)                                  
! Divergence coefficient                                                
      COMPLEX :: D2(NNT,MMTy,NHEM,NLx)                                  
! Temperature coefficient                                               
      COMPLEX :: T2(NNT,MMTy,NHEM,NLx)                                  
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP2(NNT,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA2(NNT,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP2(NNT,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE2(NNT,MMTy,NHEM,NLx)                                
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT2(NNT,MMTy,NHEM,NLx)                                 
! Tracer coefficient                                                    
      COMPLEX :: TRA2(NNT,MMTy,NHEM,NLx,NTRAC)                          
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT2(NNT,MMTy,NHEM,NLx,NTRAC)                         
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: AW1256                                                    
      REAL :: AW3478                                                    
      INTEGER :: IP, IPM, MP                                            
      COMPLEX :: TEMP                                                   
! Loop variables                                                        
      INTEGER :: IHEM, IM, ITRAC, J, L, M, N, nT                        
!-----------------------------------------------------                  
! For a global run, sum and difference the complete Fourier             
! transforms at the northern and southern latitude rows to give         
! the even and odd contributions : E=(N+S)/2, O=(N-S)/2.                
! For Fourier fields symmetric about equator  : even precedes odd.      
! For Fourier fields asymmetric about equator : odd precedes even.      
!-----------------------------------------------------                  
      IF (NHEM.EQ.2) THEN                                               
         DO J=1,JG                                                      
            DO IM=1,2*MMTyy(iRow)                                       
               DO L=1,NLxx(iCol)                                        
                  !---------------------------------------------------  
                  ! Temperature tendency : main + d/dx part : symmetric 
                  !---------------------------------------------------  
                  TEMP=TNLG(IM,1,J,L)                                   
                  TNLG(IM,1,J,L)=0.5*(TEMP+TNLG(IM,NHEM,J,L))           
                  TNLG(IM,NHEM,J,L)=0.5*(TEMP-TNLG(IM,NHEM,J,L))        
                  !---------------------------------------------------  
                  ! Divergence tendency : d/dx part : symmetric.        
                  !---------------------------------------------------  
                  TEMP=FUG(IM,1,J,L)                                    
                  FUG(IM,1,J,L)=0.5*(TEMP+FUG(IM,NHEM,J,L))             
                  FUG(IM,NHEM,J,L)=0.5*(TEMP-FUG(IM,NHEM,J,L))          
                  !---------------------------------------------------  
                  ! Vorticity tendency : d/dx part : anti-symmetric.    
                  !---------------------------------------------------  
                  TEMP=FVG(IM,1,J,L)                                    
                  FVG(IM,1,J,L)=0.5*(TEMP-FVG(IM,NHEM,J,L))             
                  FVG(IM,NHEM,J,L)=0.5*(TEMP+FVG(IM,NHEM,J,L))          
                  !---------------------------------------------------  
                  ! Divergence tendency : d/dy part : anti-symmetric.   
                  !---------------------------------------------------  
                  TEMP=FVGT(IM,1,J,L)                                   
                  FVGT(IM,1,J,L)=0.5*(TEMP-FVGT(IM,NHEM,J,L))           
                  FVGT(IM,NHEM,J,L)=0.5*(TEMP+FVGT(IM,NHEM,J,L))        
                  !---------------------------------------------------  
                  ! Vorticity tendency : d/dy part : symmetric.         
                  !---------------------------------------------------  
                  TEMP=FUGT(IM,1,J,L)                                   
                  FUGT(IM,1,J,L)=0.5*(TEMP+FUGT(IM,NHEM,J,L))           
                  FUGT(IM,NHEM,J,L)=0.5*(TEMP-FUGT(IM,NHEM,J,L))        
                  !---------------------------------------------------  
                  ! Tracer tendencies                                   
                  !---------------------------------------------------  
                  DO ITRAC=1,NTRAC                                      
                     !------------------------------------------------  
                     ! main + d/dx part : symmetric.                    
                     !------------------------------------------------  
                     TEMP=TRANLG(IM,1,J,L,ITRAC)                        
                     TRANLG(IM,1,J,L,ITRAC)=                            
     &                    0.5*(TEMP+TRANLG(IM,NHEM,J,L,ITRAC))          
                     TRANLG(IM,NHEM,J,L,ITRAC)=                         
     &                    0.5*(TEMP-TRANLG(IM,NHEM,J,L,ITRAC))          
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Loop over all latitudes                                               
!-----------------------------------------------------                  
      DO J=1,JG                                                         
         !-----------------------------------------------------         
         ! Set up the appropriate Gaussian weight for the current       
         ! latitude, dependent on transform type.                       
         !-----------------------------------------------------         
         AW1256=AW(J,1)*CSSQ(J,1)                                       
         AW3478=-AW(J,1)                                                
         !-----------------------------------------------------         
         ! Calculate POLY array in a vector loop before the main        
         ! transforms, for the required Legendre Function types.        
         ! Both even and odd functions are required, irrespective of    
         ! NHEM.                                                        
         !-----------------------------------------------------         
         DO IHEM=1,2                                                    
            DO IM=1,MMTyy(iRow)                                         
               DO IP=1,NNT                                              
                  POLY(IP,IM,IHEM,1)=AW1256*ALP(IP,IM,IHEM,J)           
                  POLY(IP,IM,IHEM,2)=AW3478*DALP(IP,IM,IHEM,J)          
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Transform multi-level fields.                                
         ! Inner loop vectorisation is over total wavenumber, to access 
         ! spectral memory sequentially, avoiding skip distances being  
         ! multiple of 8 (which causes memory bank conflicts on Cray    
         ! vector machines).                                            
         !-----------------------------------------------------         
         DO L=1,NLxx(iCol)                                              
            DO nT=1,2                                                   
               DO IM=1,MMTyy(iRow)                                      
                  MP=(nT-1)*(2*(MMTyy(iRow)-IM)+1)+IM                   
                  DO IP=mTwin2(nT,IM)+1,mTwin2(nT+1,IM)                 
                     !------------------------------------------------  
                     ! Temperature tendency :main + d/dx part : type 2. 
                     ! Temperature tendency :d/dy part        : type 4. 
                     !------------------------------------------------  
                     TT2 (IP,IM,1,L)=TT2 (IP,IM,1,L) + POLY(IP,IM,1,1)* 
     &                    TNLG(MP,1,J,L)                                
                     !------------------------------------------------  
                     ! Divergence tendency  : d/dx part       : type 2. 
                     ! Divergence tendency  : d/dy part       : type 4. 
                     !------------------------------------------------  
                     DT2 (IP,IM,1,L)=DT2 (IP,IM,1,L) + POLY(IP,IM,1,1)* 
     &                    FUG (MP,1,J,L)+POLY(IP,IM,1,2)*               
     &                    FVGT(MP,1,J,L)                                
                     !------------------------------------------------  
                     ! Vorticity tendency   : d/dx part       : type 1. 
                     ! Vorticity tendency   : d/dy part       : type 3. 
                     !------------------------------------------------  
                     ZT2 (IP,IM,1,L)=ZT2 (IP,IM,1,L) + POLY(IP,IM,2,1)* 
     &                    FVG (MP,1,J,L)+POLY(IP,IM,2,2)*               
     &                    FUGT(MP,1,J,L)                                
                     !------------------------------------------------  
                     ! Tracer tendency   : d/dy part        : type 4.   
                     !------------------------------------------------  
                     DO ITRAC=1,NTRAC                                   
                        TRAT2(IP,IM,1,L,ITRAC)=TRAT2(IP,IM,1,L,ITRAC)+  
     &                       POLY(IP,IM,1,1)*TRANLG(MP,1,J,L,ITRAC)     
                     ENDDO                                              
                     IF (NHEM.EQ.2) THEN                                
                        !---------------------------------------------  
                        ! Temperature tendency: main+d/dx part: type 2. 
                        ! Temperature tendency: d/dy part     : type 4. 
                        !---------------------------------------------  
                        TT2 (IP,IM,NHEM,L)=TT2 (IP,IM,NHEM,L) +         
     &                       POLY(IP,IM,2,1)*TNLG(MP,NHEM,J,L)          
                        !---------------------------------------------  
                        ! Divergence tendency  : d/dx part    : type 2. 
                        ! Divergence tendency  : d/dy part    : type 4. 
                        !---------------------------------------------  
                        DT2 (IP,IM,NHEM,L)=DT2 (IP,IM,NHEM,L) +         
     &                       POLY(IP,IM,2,1)*FUG (MP,NHEM,J,L) +        
     &                       POLY(IP,IM,2,2)*FVGT(MP,NHEM,J,L)          
                        !---------------------------------------------  
                        ! Vorticity tendency   : d/dx part    : type 1. 
                        ! Vorticity tendency   : d/dy part    : type 3. 
                        !---------------------------------------------  
                        ZT2 (IP,IM,NHEM,L)=ZT2 (IP,IM,NHEM,L) +         
     &                       POLY(IP,IM,1,1)*FVG (MP,NHEM,J,L) +        
     &                       POLY(IP,IM,1,2)*FUGT(MP,NHEM,J,L)          
                        !---------------------------------------------  
                        ! Tracer tendency   : d/dy part       : type 4. 
                        !---------------------------------------------  
                        DO ITRAC=1,NTRAC                                
                           TRAT2(IP,IM,NHEM,L,ITRAC)=                   
     &                          TRAT2(IP,IM,NHEM,L,ITRAC)+              
     &                          POLY(IP,IM,2,1)*                        
     &                          TRANLG(MP,NHEM,J,L,ITRAC)               
                        ENDDO                                           
                    ENDIF                                               
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! Perform all the indirect Legendre transforms for the adiabatic        
! part of the timestep at the current latitude (pair), in place         
! of separate calls to HEXP for individual transform types.             
!                                                                       
! An even/odd value of ITYPE denotes a spectral field of even/odd       
! symmetry.                                                             
!                                                                       
! Maximum vector efficiency is achieved by chaining all multi-level     
! transforms in one loop and by chaining all single-level transform     
! in a second loop.                                                     
!                                                                       
! All dummy argument arrays are declared complex.                       
! All array dimensions are parameters.                                  
! Multi-level arrays are 3-dimensional.                                 
!                                                                       
! Version for RSGUP3.                     Mike Blackburn,  12.01.95     
!=====================================================                  
      SUBROUTINE DEXPV                                                  
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Like the variables in SPECTR but the meridional wavenumbers           
! are complete instead of the levels.                                   
!-----------------------------------------------------                  
      COMMON/SPECTR2/Z2,D2,T2,SP2,SPA2,VP2,DTE2,TT2,DT2,ZT2             
     &     ,TRA2,TRAT2                                                  
! Vorticity coefficient                                                 
      COMPLEX :: Z2(NNT,MMTy,NHEM,NLx)                                  
! Divergence coefficient                                                
      COMPLEX :: D2(NNT,MMTy,NHEM,NLx)                                  
! Temperature coefficient                                               
      COMPLEX :: T2(NNT,MMTy,NHEM,NLx)                                  
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP2(NNT,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA2(NNT,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP2(NNT,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE2(NNT,MMTy,NHEM,NLx)                                
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT2(NNT,MMTy,NHEM,NLx)                                 
! Tracer coefficient                                                    
      COMPLEX :: TRA2(NNT,MMTy,NHEM,NLx,NTRAC)                          
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT2(NNT,MMTy,NHEM,NLx,NTRAC)                         
!-----------------------------------------------------                  
! Half transforms. The levels are divided across the columns of PEs;    
! the zonal wavenumbers are divided across the row of PEs; and on       
! ech PE the latitude are complete.                                     
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA/UG,FUG,VG,FVG,TG,TNLG,PLG                           
     &     ,TRAG,TRANLG                                                 
! U_psi, then U (rotational wind)                                       
      COMPLEX :: UG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FUG(2*MMTy,NHEM,JG,NLx)                                
! V_chi, then V (divergent wind)                                        
      COMPLEX :: VG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FVG(2*MMTy,NHEM,JG,NLx)                                
! Temperature anomaly (T')                                              
      COMPLEX :: TG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: TNLG(2*MMTy,NHEM,JG,NLx)                               
! ln p* or p*                                                           
      COMPLEX :: PLG(2*MMTy,NHEM,JG)                                    
! Tracer term                                                           
      COMPLEX :: TRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                         
!                                                                       
      COMPLEX :: TRANLG(2*MMTy,NHEM,JG,NLx,NTRAC)                       
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB/ZG,DG,PJG,PMG,SPG,VPG,EG,UTG,VTG                    
     &     ,UTRAG,VTRAG                                                 
     &     ,CHIG,SFG,FVGT,FUGT                                          
! Vorticity (zeta)                                                      
      COMPLEX :: ZG(2*MMTy,NHEM,JG,NLx)                                 
! Divergence (d)                                                        
      COMPLEX :: DG(2*MMTy,NHEM,JG,NLx)                                 
! (1-mu^2) (d(ln p*)/d mu)                                              
      COMPLEX :: PJG(2*MMTy,NHEM,JG)                                    
! (d(ln p*)/d lambda)                                                   
      COMPLEX :: PMG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: SPG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: VPG(2*MMTy,NHEM,JG)                                    
! (U^2 + V^2)                                                           
      COMPLEX :: EG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: UTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: VTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: UTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: VTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: CHIG(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: SFG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: FVGT(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: FUGT(2*MMTy,NHEM,JG,NLx)                               
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      COMPLEX :: TEMP                                                   
      INTEGER :: MP                                                     
! Loop variables                                                        
      INTEGER :: IM, IP, ITRAC, J, L, M, N, nT                          
!-----------------------------------------------------                  
! Transform multi-level fields to create even and odd contributions     
! to the Fourier coefficients at the northern hemisphere latitude.      
!-----------------------------------------------------                  
      DO L=1,NLxx(iCol)                                                 
         DO J=1,JG                                                      
            DO nT=1,2                                                   
               DO IM=1,MMTyy(iRow)                                      
                  MP=(nT-1)*(2*(MMTyy(iRow)-IM)+1)+IM                   
                  DO IP=mTwin2(nT,IM)+1,mTwin2(nT+1,IM)                 
                     !------------------------------------------------  
                     ! Velocity potential    : type 6.                  
                     !------------------------------------------------  
                     CHIG(MP,1,J,L)=CHIG(MP,1,J,L) +                    
     &                    RLP(IP,IM,1,J)*D2(IP,IM,1,L)                  
                     !------------------------------------------------  
                     ! Streamfunction        : type 5.                  
                     !------------------------------------------------  
                     SFG (MP,1,J,L)=SFG (MP,1,J,L) +                    
     &                    RLP(IP,IM,2,J)*Z2(IP,IM,1,L)                  
                     !------------------------------------------------  
                     ! Zonal (rot) wind      : type 7.                  
                     !------------------------------------------------  
                     UG  (MP,1,J,L)=UG  (MP,1,J,L) +                    
     &                    RDLP(IP,IM,2,J)*Z2(IP,IM,1,L)                 
                     !------------------------------------------------  
                     ! Merid (div) wind      : type 8.                  
                     !------------------------------------------------  
                     VG  (MP,1,J,L)=VG  (MP,1,J,L) +                    
     &                    RDLP(IP,IM,1,J)*D2(IP,IM,1,L)                 
                     !------------------------------------------------  
                     ! Temperature           : type 2.                  
                     !------------------------------------------------  
                     TG  (MP,1,J,L)=TG  (MP,1,J,L) +                    
     &                    ALP(IP,IM,1,J)*T2(IP,IM,1,L)                  
                     !------------------------------------------------  
                     ! Tracers               : type 2.                  
                     !------------------------------------------------  
                     DO ITRAC=1,NTRAC                                   
                        TRAG(MP,1,J,L,ITRAC)=TRAG(MP,1,J,L,ITRAC)+      
     &                       ALP(IP,IM,1,J)*TRA2(IP,IM,1,L,ITRAC)       
                     ENDDO                                              
                     IF (NHEM.EQ.2) THEN                                
                        !---------------------------------------------  
                        ! Velocity potential    : type 6.               
                        !---------------------------------------------  
                        CHIG(MP,NHEM,J,L)=CHIG(MP,NHEM,J,L) +           
     &                       RLP(IP,IM,NHEM,J)*D2(IP,IM,NHEM,L)         
                        !---------------------------------------------  
                        ! Streamfunction        : type 5.               
                        !---------------------------------------------  
                        SFG (MP,NHEM,J,L)=SFG (MP,NHEM,J,L) +           
     &                       RLP(IP,IM,1,J)*Z2(IP,IM,NHEM,L)            
                        !---------------------------------------------  
                        ! Zonal (rot) wind      : type 7.               
                        !---------------------------------------------  
                        UG  (MP,NHEM,J,L)=UG  (MP,NHEM,J,L) +           
     &                       RDLP(IP,IM,1,J)*Z2(IP,IM,NHEM,L)           
                        !---------------------------------------------  
                        ! Merid (div) wind      : type 8.               
                        !---------------------------------------------  
                        VG  (MP,NHEM,J,L)=VG  (MP,NHEM,J,L) +           
     &                       RDLP(IP,IM,NHEM,J)*D2(IP,IM,NHEM,L)        
                        !---------------------------------------------  
                        ! Temperature           : type 2.               
                        !---------------------------------------------  
                        TG  (MP,NHEM,J,L)=TG  (MP,NHEM,J,L) +           
     &                       ALP(IP,IM,NHEM,J)*T2(IP,IM,NHEM,L)         
                        !---------------------------------------------  
                        ! Tracers           : type 2.                   
                        !---------------------------------------------  
                        DO ITRAC=1,NTRAC                                
                           TRAG(MP,NHEM,J,L,ITRAC)=                     
     &                          TRAG(MP,NHEM,J,L,ITRAC)+ALP(IP,IM,2,J)  
     &                          *TRA2(IP,IM,NHEM,L,ITRAC)               
                        ENDDO                                           
                     ENDIF                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Transform single-level fields to create even and odd contribution     
! to the Fourier coefficients at the northern hemisphere latitude.      
! Vectorisation is over total wavenumber for each zonal wavenumber.     
!-----------------------------------------------------                  
      IF (iCol.EQ.1) THEN                                               
         DO J=1,JG                                                      
            DO nT=1,2                                                   
               DO IM=1,MMTyy(iRow)                                      
                  MP=(nT-1)*(2*(MMTyy(iRow)-IM)+1)+IM                   
                  DO IP=mTwin2(nT,IM)+1,mTwin2(nT+1,IM)                 
                     !------------------------------------------------  
                     ! Log (surface pressure)    : type 2.              
                     !------------------------------------------------  
                     PLG(MP,1,J)=PLG(MP,1,J) +                          
     &                    ALP(IP,IM,1,J)*SP2(IP,IM,1)                   
                     IF (NHEM.EQ.2) THEN                                
                        !---------------------------------------------  
                        ! Log (surface pressure)    : type 2.           
                        !---------------------------------------------  
                        PLG(MP,NHEM,J)=PLG(MP,NHEM,J) +                 
     &                       ALP(IP,IM,NHEM,J)*SP2(IP,IM,NHEM)          
                     ENDIF                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! For a global run, sum and difference even and odd contributions       
! to give the complete Fourier transforms at the northern and           
! southern latitude rows: N=E+O, S=E-O.                                 
! For symmetric Fourier fields, even (IM,1) precedes odd (IM,2).        
! For asymmetric Fourier fields, odd (IM,1) precedes even (IM,2).       
!-----------------------------------------------------                  
      IF (NHEM.EQ.2) THEN                                               
         DO L=1,NLxx(iCol)                                              
            DO J=1,JG                                                   
               DO MP=1,2*MMTyy(iRow)                                    
                  !---------------------------------------------------  
                  ! Velocity potential : symmetric.                     
                  !---------------------------------------------------  
                  TEMP=CHIG(MP,1,J,L)                                   
                  CHIG(MP,1,J,L)=TEMP+CHIG(MP,NHEM,J,L)                 
                  CHIG(MP,NHEM,J,L)=TEMP-CHIG(MP,NHEM,J,L)              
                  !---------------------------------------------------  
                  ! Streamfunction : anti-symmetric.                    
                  !---------------------------------------------------  
                  TEMP=SFG(MP,1,J,L)                                    
                  SFG(MP,1,J,L)=SFG(MP,NHEM,J,L)+TEMP                   
                  SFG(MP,NHEM,J,L)=SFG(MP,NHEM,J,L)-TEMP                
                  !---------------------------------------------------  
                  ! Zonal (rotational) wind : symmetric.                
                  !---------------------------------------------------  
                  TEMP=UG(MP,1,J,L)                                     
                  UG(MP,1,J,L)=TEMP+UG(MP,NHEM,J,L)                     
                  UG(MP,NHEM,J,L)=TEMP-UG(MP,NHEM,J,L)                  
                  !---------------------------------------------------  
                  ! Meridional (divergent) wind : anti-symmetric.       
                  !---------------------------------------------------  
                  TEMP=VG(MP,1,J,L)                                     
                  VG(MP,1,J,L)=VG(MP,NHEM,J,L)+TEMP                     
                  VG(MP,NHEM,J,L)=VG(MP,NHEM,J,L)-TEMP                  
                  !---------------------------------------------------  
                  ! Temperature : symmetric.                            
                  !---------------------------------------------------  
                  TEMP=TG(MP,1,J,L)                                     
                  TG(MP,1,J,L)=TEMP+TG(MP,NHEM,J,L)                     
                  TG(MP,NHEM,J,L)=TEMP-TG(MP,NHEM,J,L)                  
                  !---------------------------------------------------  
                  ! Tracers : symmetric.                                
                  !---------------------------------------------------  
                  DO ITRAC=1,NTRAC                                      
                     TEMP=TRAG(MP,1,J,L,ITRAC)                          
                     TRAG(MP,1,J,L,ITRAC)=TEMP+TRAG(MP,NHEM,J,L,ITRAC)  
                     TRAG(MP,NHEM,J,L,ITRAC)=                           
     &                    TEMP-TRAG(MP,NHEM,J,L,ITRAC)                  
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
                                                                        
         IF (iCol.EQ.1) THEN                                            
            DO J=1,JG                                                   
               DO MP=1,2*MMTyy(iRow)                                    
                  !---------------------------------------------------  
                  ! Log (surface pressure) : symmetric.                 
                  !---------------------------------------------------  
                  TEMP=PLG(MP,1,J)                                      
                  PLG(MP,1,J)=TEMP+PLG(MP,NHEM,J)                       
                  PLG(MP,NHEM,J)=TEMP-PLG(MP,NHEM,J)                    
               ENDDO                                                    
            ENDDO                                                       
         ENDIF                                                          
      ENDIF                                                             
                                                                        
      RETURN                                                            
      END                                                               
!=====================================================                  
! Compute diabatic tendencies in grid point space. Accumulate           
! time averages for printed output and history assumes output           
! and history timesteps are synchronised.                               
!                                                                       
! Full Physics                                                          
! Piers 6/2/97 Version 2.0                                              
! Water vapour (things beginning with Q have changed                    
! into tracer no 1)                                                     
! and converted from Volume mixing  ratio to Mass mixing ratio          
! As water vapour advection is now done in Flux form                    
!=====================================================                  
      SUBROUTINE DGRMLT                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Switches counters and constants controlling type and frequency of     
! model output                                                          
!-----------------------------------------------------                  
      COMMON/OUTCON/RNTAPE,DAY,SQR2,RSQR2,EAM1,EAM2,TOUT1               
     &     ,TOUT2,LSPO,LGPO                                             
     &     ,RNTAPO                                                      
     &     ,LSHIST,LMINIH                                               
      COMMON/OUTCONINT/NCOEFF,NLAT,INLAT,INSPC,KOUNTP                   
     &                 ,KOUNTE,KOUNTH,KOUNTR,KOUTP                      
     &                 ,KOUTE,KOUTH,KOUTR                               
! Identyfying run number                                                
      REAL :: RNTAPE                                                    
! Maximum wavenumber of printed spectral coeffecients                   
      INTEGER :: NCOEFF                                                 
! Number of latitudes in printed grid point fields                      
      INTEGER :: NLAT                                                   
      INTEGER :: INLAT                                                  
!                                                                       
      INTEGER :: INSPC                                                  
! Number of timsteps between successive writing of spectral             
! and gridded fields                                                    
      INTEGER :: KOUNTP                                                 
      INTEGER :: KOUNTE                                                 
! Number of timsteps between successive writing of history              
! records, containing spectral fields at the present and                
! previous time levels.                                                 
      INTEGER :: KOUNTH                                                 
! Number of timsteps between successive writing of restart              
! records, containing the spectral fields at the present and            
! previous time levels                                                  
      INTEGER :: KOUNTR                                                 
      INTEGER :: KOUTP                                                  
      INTEGER :: KOUTE                                                  
      INTEGER :: KOUTH                                                  
      INTEGER :: KOUTR                                                  
      REAL :: DAY                                                       
      REAL :: SQR2                                                      
      REAL :: RSQR2                                                     
      REAL :: EAM1                                                      
      REAL :: EAM2                                                      
      REAL :: TOUT1                                                     
      REAL :: TOUT2                                                     
      LOGICAL :: LSPO(NL)                                               
      LOGICAL :: LGPO(NL)                                               
      REAL :: RNTAPO                                                    
      LOGICAL :: LSHIST                                                 
! Reduce the history file to single precision if true                   
      LOGICAL :: LMINIH                                                 
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Grid variables for DGRMLT loop                                        
!-----------------------------------------------------                  
      COMMON/GRIDPC/ASSBL,ASHBL,ASLBL,ARRCR,ARRLR                       
     &     ,TTVD,QTVD,TYBL,TXBL,TTRD,TTLR,TTCR,UTVD,VTVD                
     &     ,QTLR,QTCR,SSBL,SHBL,SLBL,RRCR,RRLR                          
     &     ,ARFLUX,ASFLD,ACLD,RFLUX,SFLD,CLD                            
      REAL :: ASSBL(MGx,NHEM,JGy)                                       
      REAL :: ASHBL(MGx,NHEM,JGy)                                       
      REAL :: ASLBL(MGx,NHEM,JGy)                                       
      REAL :: ARRCR(MGx,NHEM,JGy)                                       
      REAL :: ARRLR(MGx,NHEM,JGy)                                       
      REAL :: TTVD(MGx,NHEM,NL)                                         
      REAL :: QTVD(MGx,NHEM,NL)                                         
      REAL :: TYBL(MGx,NHEM)                                            
      REAL :: TXBL(MGx,NHEM)                                            
      REAL :: TTRD(MGx,NHEM,NL)                                         
      REAL :: TTLR(MGx,NHEM,NL)                                         
      REAL :: TTCR(MGx,NHEM,NL)                                         
      REAL :: UTVD(MGx,NHEM,NL)                                         
      REAL :: VTVD(MGx,NHEM,NL)                                         
      REAL :: QTLR(MGx,NHEM,NL)                                         
      REAL :: QTCR(MGx,NHEM,NL)                                         
      REAL :: SSBL(MGx,NHEM,JGy)                                        
      REAL :: SHBL(MGx,NHEM,JGy)                                        
      REAL :: SLBL(MGx,NHEM,JGy)                                        
      REAL :: RRCR(MGx,NHEM,JGy)                                        
      REAL :: RRLR(MGx,NHEM,JGy)                                        
      REAL :: arflux(MGx,NHEM,JGy,6)                                    
      REAL :: asfld(MGx,NHEM,JGy,6)                                     
      REAL :: acld(MGx,NHEM,JGy,4)                                      
      REAL :: rflux(MGx,NHEM,JGy,6)                                     
      REAL :: sfld(MGx,NHEM,JGy,6)                                      
      REAL :: cld(MGx,NHEM,JGy,4)                                       
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Stores diagnostic data                                                
!-----------------------------------------------------                  
      COMMON/PTENDZ/UTVDZ,VTVDZ,TTVDZ,QTVDZ,TTCRZ,QTCRZ                 
     &     ,TTLRZ,QTLRZ,TTRDZ,CTCRZ,CTLRZ                               
     &     ,UTBLZ,VTBLZ,TTBLZ,QTBLZ                                     
     &     ,AUTVDZ,AVTVDZ,ATTVDZ,AQTVDZ,ATTCRZ,AQTCRZ                   
     &     ,ATTLRZ,AQTLRZ,ATTRDZ,ACTCRZ,ACTLRZ                          
     &     ,AUTBLZ,AVTBLZ,ATTBLZ,AQTBLZ                                 
      REAL :: UTVDZ(JGy,NHEM,NL)                                        
      REAL :: VTVDZ(JGy,NHEM,NL)                                        
      REAL :: TTVDZ(JGy,NHEM,NL)                                        
      REAL :: QTVDZ(JGy,NHEM,NL)                                        
      REAL :: TTCRZ(JGy,NHEM,NL)                                        
      REAL :: QTCRZ(JGy,NHEM,NL)                                        
      REAL :: TTLRZ(JGy,NHEM,NL)                                        
      REAL :: QTLRZ(JGy,NHEM,NL)                                        
      REAL :: TTRDZ(JGy,NHEM,NL)                                        
      REAL :: CTCRZ(JGy,NHEM,NL)                                        
      REAL :: CTLRZ(JGy,NHEM,NL)                                        
      REAL :: UTBLZ(JGy,NHEM)                                           
      REAL :: VTBLZ(JGy,NHEM)                                           
      REAL :: TTBLZ(JGy,NHEM)                                           
      REAL :: QTBLZ(JGy,NHEM)                                           
! Variables below are for storing averages of the variables above       
      REAL :: AUTVDZ(JGy,NHEM,NL)                                       
      REAL :: AVTVDZ(JGy,NHEM,NL)                                       
      REAL :: ATTVDZ(JGy,NHEM,NL)                                       
      REAL :: AQTVDZ(JGy,NHEM,NL)                                       
      REAL :: ATTCRZ(JGy,NHEM,NL)                                       
      REAL :: AQTCRZ(JGy,NHEM,NL)                                       
      REAL :: ATTLRZ(JGy,NHEM,NL)                                       
      REAL :: AQTLRZ(JGy,NHEM,NL)                                       
      REAL :: ATTRDZ(JGy,NHEM,NL)                                       
      REAL :: ACTCRZ(JGy,NHEM,NL)                                       
      REAL :: ACTLRZ(JGy,NHEM,NL)                                       
      REAL :: AUTBLZ(JGy,NHEM)                                          
      REAL :: AVTBLZ(JGy,NHEM)                                          
      REAL :: ATTBLZ(JGy,NHEM)                                          
      REAL :: AQTBLZ(JGy,NHEM)                                          
!-----------------------------------------------------                  
! Radiation variables                                                   
!-----------------------------------------------------                  
      COMMON/CPIERS/ICFLAG,CFRAC,PNET,SNET,RRFLUX,TTSW,TTLW             
      INTEGER :: ICFLAG(MGx,NHEM,5,2)                                   
      REAL :: CFRAC(MGx,NHEM,5)                                         
      REAL :: PNET(MGx,NHEM,JGy)                                        
      REAL :: SNET(MGx,NHEM,JGy)                                        
      REAL :: RRFLUX(MGx,NHEM,JGy,6)                                    
      REAL :: TTSW(MGx,NHEM,NL)                                         
      REAL :: TTLW(MGx,NHEM,NL)                                         
!-----------------------------------------------------                  
! Orogrophy                                                             
!-----------------------------------------------------                  
      COMMON/GSG/GSG,TOPVAR,RLEVBZMO,FXZMO,ofrac,runoff,iareanum        
! Topog variance, GW diags for Orog, Non orog waves                     
      REAL :: TOPVAR(MGx,NHEM,JGy)                                      
      REAL :: RLEVBZMO(JGy,NHEM,NL)                                     
      REAL :: FXZMO(JGy,NHEM,NL)                                        
         REAL :: GSG(MGx,NHEM,JGy)                                      
         REAL :: ofrac(MGx,NHEM,JGy)                                    
         INTEGER :: runoff(50,2,33)                                     
         INTEGER :: iareanum                                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
C OASIS size parameters                                                 
      INTEGER, PARAMETER :: IMB = 128                                   
      INTEGER, PARAMETER :: JM = 64                                     
      INTEGER, PARAMETER :: IMJM = IMB*JM                               
      common/oasiso2a/zsst(imjm),zscur(imjm),mscur(imjm)                
     $   ,zsst_2d(MGx,NHEM,JGy)                                         
     $   ,zscur_2d(MGx,NHEM,JGy),mscur_2d(MGx,NHEM,JGy)                 
      real zsst,zsst_2d,zscur,zscur_2d,mscur,mscur_2d                   
      common/oasisa2o/swlw(imjm),heatlat(imjm),water(imjm)              
     $   ,sensible(imjm),taux(imjm),tauy(imjm),exwater(imjm)            
     $   ,exwat(MGx,NHEM,JGy),sensible2(imjm),sensible3(imjm)           
     $   ,oceaniceflux(MGx,NHEM),exiwat(MGx,NHEM,JGy),exiwater(imjm)    
     $   ,exiwatn(MGx,NHEM,JGy),exiwatern(imjm)                         
     $   ,tstaroo(MGx,NHEM,JGy),tstarf(MGx,NHEM,JGy)                    
     $   ,qstarf(MGx,NHEM,JGy),sqstarf(MGx,NHEM,JGy)                    
     $   ,swlw2d(MGx,NHEM,JGy),heatlat2d(MGx,NHEM,JGy)                  
     $   ,water2d(MGx,NHEM,JGy)                                         
     $   ,sensible2d(MGx,NHEM,JGy),taux2d(MGx,NHEM,JGy)                 
     $   ,tauy2d(MGx,NHEM,JGy)                                          
     $   ,SHBLC(MGx,NHEM,JGy),SLBLC(MGx,NHEM,JGy)                       
     $   ,exwater2d(MGx,NHEM,JGy),sensible2_2d(MGx,NHEM,JGy)            
     $   ,sensible3_2d(MGx,NHEM,JGy)                                    
     $   ,exiwater2d(MGx,NHEM,JGy),exiwatern2d(MGx,NHEM,JGy)            
      real swlw,heatlat,water,sensible,taux,tauy,shblc,slblc,exwater    
     $,exwat,sensible2,sensible3,oceaniceflux,exiwat,exiwater           
     $,exiwatn,exiwatern,tstaroo,tstarf,qstarf,sqstarf                  
     $,swlw2d,heatlat2d,water2d,sensible2d,taux2d,tauy2d                
     $,exwater2d,sensible2_2d,sensible3_2d,exiwater2d,exiwatern2d       
                                                                        
      common/wat/water2(imjm), water2_2d(MGx,NHEM,JGy)                  
C Arrays wathbl, watlsl, watsbl are used for balancing water in IGCM4   
C ashbl, aslbl, assbl were used in IGCM3 FORTE but these are already    
C in use in elsewhere in IGCM4 for surface flux diagnostics             
     $   ,wathbl(MGx,NHEM,JGy),watlbl(MGx,NHEM,JGy),watsbl(MGx,NHEM,JGy)
     $   ,sicecov(MGx,NHEM,JGy)                                         
      real water2,water2_2d,wathbl,watlbl,watsbl,sicecov                
                                                                        
                                                                        
!-----------------------------------------------------                  
! External functions                                                    
!-----------------------------------------------------                  
      REAL, EXTERNAL :: SSUM                                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      LOGICAL :: LSUM                                                   
      INTEGER :: IFIRST                                                 
      REAL :: TROPHT(MGx,NHEM,JGy)                                      
      INTEGER :: JH                                                     
      REAL :: RKP                                                       
      REAL :: RMG                                                       
      real :: tspd,rkounth,tzc,tm2,evap,pp,evap2                        
                                                                        
! Loop variables                                                        
      INTEGER :: I, IHEM, K, L                                          
!-----------------------------------------------------                  
! Store ifirst so that subroutine SFCT is only called the               
! first time through.                                                   
!-----------------------------------------------------                  
      SAVE IFIRST                                                       
      DATA IFIRST/1/                                                    
      SAVE TROPHT                                                       
!-----------------------------------------------------                  
! Switches/counters for the current timestep.                           
!-----------------------------------------------------                  
      LSUM=(KOUTH.GE.1.AND.KOUTH.LE.KOUNTH)                             
      RKP=1.0/REAL(KOUNTH)                                              
      DELT2C=DELT2                                                      
!-----------------------------------------------------                  
! Preset accumulated diagnostics.                                       
!-----------------------------------------------------                  
      IF(LSUM) THEN                                                     
         IF(KOUTH.EQ.1) THEN                                            
            !-----------------------------------------------------      
            ! Zeroing arrays                                            
            !-----------------------------------------------------      
            ARFLUX(:,:,:,:)=0.                                          
            ASFLD(:,:,:,:)=0.                                           
            ACLD(:,:,:,:)=0.                                            
            ASSBL(:,:,:)=0.0                                            
            ASHBL(:,:,:)=0.0                                            
            ASLBL(:,:,:)=0.0                                            
            ARRCR(:,:,:)=0.0                                            
            ARRLR(:,:,:)=0.0                                            
         ENDIF                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Preset instantaneous forcing and diagnostic arrays.                   
!-----------------------------------------------------                  
      FUG3(:,:,:,:)=0.0                                                 
      FVG3(:,:,:,:)=0.0                                                 
      TNLG3(:,:,:,:)=0.0                                                
      TRANLG3(:,:,:,:,:)=0.0                                            
      SSBL(:,:,:)=0.0                                                   
      SHBL(:,:,:)=0.0                                                   
      SLBL(:,:,:)=0.0                                                   
      RRCR(:,:,:)=0.0                                                   
      RRLR(:,:,:)=0.0                                                   
!-----------------------------------------------------                  
! Add mean temperature to temperature anomaly                           
!-----------------------------------------------------                  
      DO L=1,NL                                                         
         TG3(:,:,:,L)=TG3(:,:,:,L)+T0(L)                                
      ENDDO                                                             
!-----------------------------------------------------                  
! Convert ln p* to p*                                                   
!-----------------------------------------------------                  
      PLG3(1:MGxx(iCol),:,1:JGyy(iRow))=                                
     &    EXP(PLG3(1:MGxx(iCol),:,1:JGyy(iRow)))                        
!-----------------------------------------------------                  
! Alter the tracer anomaly                                              
!-----------------------------------------------------                  
      IF (LFLUX) THEN                                                   
         DO L=1,NL                                                      
            DO JH=1,JGyy(iRow)                                          
               DO IHEM=1,NHEM                                           
                  DO I=1,MGxx(iCol)                                     
                     TRAG3(I,IHEM,JH,L,1)=TRAG3(I,IHEM,JH,L,1)/         
     &                    PLG3(I,IHEM,JH)                               
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Find new T and Q based on DOY and climatology                         
!-----------------------------------------------------                  
      IF ((LCSFCT).AND.( (.NOT.LPERPET).OR.(IFIRST.EQ.1) )) THEN        
         CALL SFCT(IFIRST, TROPHT)                                      
         IFIRST=0                                                       
      ENDIF                                                             
!-----------------------------------------------------                  
! Loop over latitudes and hemispheres                                   
!-----------------------------------------------------                  
      DO JH=1,JGyy(iRow)                                                
         !-----------------------------------------------------         
         ! Zeroing arrays                                               
         !-----------------------------------------------------         
         QTVD(:,:,:)=0.0                                                
         UTVD(:,:,:)=0.0                                                
         VTVD(:,:,:)=0.0                                                
         TTVD(:,:,:)=0.0                                                
         TTCR(:,:,:)=0.0                                                
         QTCR(:,:,:)=0.0                                                
         TTLR(:,:,:)=0.0                                                
         QTLR(:,:,:)=0.0                                                
         TTRD(:,:,:)=0.0                                                
         TXBL(:,:)=0.0                                                  
         TYBL(:,:)=0.0                                                  
         CTCR(:,:)=0.0                                                  
         CTLR(:,:)=0.0                                                  
         CFRAC(:,:,:)=0.0                                               
         ICFLAG(:,:,:,1)=2                                              
         ICFLAG(:,:,:,2)=NL+1                                           
         !-----------------------------------------------------         
         ! Call physical parametrizations.                              
         !-----------------------------------------------------         
         if (LOLDBL) then                                               
            if (LBL) call BLAYER(JH)                                    
         else                                                           
            if (LBL) call BLSURF(JH)                                    
         endif                                                          
         IF(LVD) CALL VDIFF(JH)                                         
         IF(LCR) CALL CONVEC(JH)                                        
         IF(LLR) CALL LSCRN(JH)                                         
         !-----------------------------------------------------         
         ! Radiation scheme                                             
         !-----------------------------------------------------         
         IF(LRD) CALL RADN(JH,TROPHT)                                   
         !-----------------------------------------------------         
         !                                                              
         !-----------------------------------------------------         
         if (LBL.AND.(.NOT.LOLDBL)) CALL SURFM(JH)                      
         !-----------------------------------------------------         
         ! Store data that may be written out later                     
         !-----------------------------------------------------         
         DO IHEM=1,NHEM                                                 
            DO I=1,MGxx(iCol)                                           
               sfld(I,IHEM,JH,1)=salb(I,IHEM,JH)                        
               sfld(I,IHEM,JH,2)=tstar(I,IHEM,JH)                       
               sfld(I,IHEM,JH,3)=tdeep(I,IHEM,JH)                       
               sfld(I,IHEM,JH,4)=qstar(I,IHEM,JH)                       
               sfld(I,IHEM,JH,5)=smstar(I,IHEM,JH)                      
               sfld(I,IHEM,JH,6)=hsnow(I,IHEM,JH)                       
               cld(I,IHEM,JH,1)=cfrac(I,IHEM,1)                         
               cld(I,IHEM,JH,2)=cfrac(I,IHEM,2)                         
               cld(I,IHEM,JH,3)=cfrac(I,IHEM,3)                         
               cld(I,IHEM,JH,4)=cfrac(I,IHEM,4)+cfrac(I,IHEM,5)         
               DO K=1,6                                                 
                  rflux(I,IHEM,JH,K)=rrflux(I,IHEM,JH,K)                
               ENDDO                                                    
               !-----------------------------------------------------   
               ! rflux(I,IHEM,JH,2) is +downwards surface flux Wm-2     
               !-----------------------------------------------------   
               rflux(I,IHEM,JH,2)=snet(I,IHEM,JH)-                      
     &              CV*P0*(shbl(I,IHEM,JH)+slbl(I,IHEM,JH))             
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Diagnostics: surface lat-long fields.                        
         !-----------------------------------------------------         
         IF(LSUM) THEN                                                  
            DO K=1,6                                                    
               DO IHEM=1,NHEM                                           
                  DO I=1,MGxx(iCol)                                     
                     arflux(I,IHEM,JH,K)=arflux(I,IHEM,JH,K)+           
     &                    rflux(I,IHEM,JH,K)*RKP                        
                     asfld(I,IHEM,JH,K)=asfld(I,IHEM,JH,K)+             
     &                    sfld(I,IHEM,JH,K)*RKP                         
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
            DO K=1,4                                                    
               DO IHEM=1,NHEM                                           
                  DO I=1,MGxx(iCol)                                     
                     acld(I,IHEM,JH,K)=acld(I,IHEM,JH,K)+               
     &                    cld(I,IHEM,JH,K)*RKP                          
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
            DO IHEM=1,NHEM                                              
               DO I=1,MGxx(iCol)                                        
                  SSBL(I,IHEM,JH)=SQRT(TXBL(I,IHEM)*TXBL(I,IHEM)+       
     &                 TYBL(I,IHEM)*TYBL(I,IHEM))                       
                  ASSBL(I,IHEM,JH)=ASSBL(I,IHEM,JH)+SSBL(I,IHEM,JH)*RKP 
                  ASHBL(I,IHEM,JH)=ASHBL(I,IHEM,JH)+SHBL(I,IHEM,JH)*RKP 
                  ASLBL(I,IHEM,JH)=ASLBL(I,IHEM,JH)+SLBL(I,IHEM,JH)*RKP 
                  ARRCR(I,IHEM,JH)=ARRCR(I,IHEM,JH)+RRCR(I,IHEM,JH)*RKP 
                  ARRLR(I,IHEM,JH)=ARRLR(I,IHEM,JH)+RRLR(I,IHEM,JH)*RKP 
                ENDDO                                                   
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Diagnostics: zonal averages. Sum for individual           
            ! PEs and add the information across all PE once the        
            ! statistics are needed.                                    
            !-----------------------------------------------------      
            RMG=1./REAL(MG)                                             
            DO L=1,NL                                                   
               DO IHEM=1,NHEM                                           
                  UTVDZ(JH,IHEM,L)=                                     
     &                 SSUM(MGxx(iCol),UTVD(1,IHEM,L),1)*RMG            
                  VTVDZ(JH,IHEM,L)=                                     
     &                 SSUM(MGxx(iCol),VTVD(1,IHEM,L),1)*RMG            
                  TTVDZ(JH,IHEM,L)=                                     
     &                 SSUM(MGxx(iCol),TTVD(1,IHEM,L),1)*RMG            
                  QTVDZ(JH,IHEM,L)=                                     
     &                 SSUM(MGxx(iCol),QTVD(1,IHEM,L),1)*RMG            
                  TTCRZ(JH,IHEM,L)=                                     
     &                 SSUM(MGxx(iCol),TTCR(1,IHEM,L),1)*RMG            
                  QTCRZ(JH,IHEM,L)=                                     
     &                 SSUM(MGxx(iCol),QTCR(1,IHEM,L),1)*RMG            
                  TTLRZ(JH,IHEM,L)=                                     
     &                 SSUM(MGxx(iCol),TTLR(1,IHEM,L),1)*RMG            
                  QTLRZ(JH,IHEM,L)=                                     
     &                 SSUM(MGxx(iCol),QTLR(1,IHEM,L),1)*RMG            
                  TTRDZ(JH,IHEM,L)=                                     
     &                 SSUM(MGxx(iCol),TTRD(1,IHEM,L),1)*RMG            
                  CTCRZ(JH,IHEM,L)=CTCR(L,IHEM)                         
                  CTLRZ(JH,IHEM,L)=CTLR(L,IHEM)                         
               ENDDO                                                    
            ENDDO                                                       
            DO IHEM=1,NHEM                                              
               UTBLZ(JH,IHEM)=SSUM(MGxx(iCol),FUG3(1,IHEM,JH,NL),1)*RMG 
               VTBLZ(JH,IHEM)=SSUM(MGxx(iCol),FVG3(1,IHEM,JH,NL),1)*RMG 
               TTBLZ(JH,IHEM)=SSUM(MGxx(iCol),TNLG3(1,IHEM,JH,NL),1)*RMG
               QTBLZ(JH,IHEM)=SSUM(MGxx(iCol),TRANLG3(1,IHEM,JH,NL,1),1)
     &              *RMG                                                
            ENDDO                                                       
         ENDIF                  ! IF(LSUM)                              
         !-----------------------------------------------------         
         ! Total forcing from all processes.                            
         !-----------------------------------------------------         
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO I=1,MGxx(iCol)                                        
                  FUG3(I,IHEM,JH,L)=FUG3(I,IHEM,JH,L)+UTVD(I,IHEM,L)    
                  FVG3(I,IHEM,JH,L)=FVG3(I,IHEM,JH,L)+VTVD(I,IHEM,L)    
                  TNLG3(I,IHEM,JH,L)=TNLG3(I,IHEM,JH,L)+TTVD(I,IHEM,L)+ 
     &                 TTCR(I,IHEM,L)+TTLR(I,IHEM,L)+TTRD(I,IHEM,L)     
                  TRANLG3(I,IHEM,JH,L,1)=TRANLG3(I,IHEM,JH,L,1)+        
     &                 QTVD(I,IHEM,L)+QTCR(I,IHEM,L)+QTLR(I,IHEM,L)     
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Mass weight the moisture forcing.                            
         !-----------------------------------------------------         
         IF (LFLUX) THEN                                                
            DO L=1,NL                                                   
               DO IHEM=1,NHEM                                           
                  DO I=1,MGxx(iCol)                                     
                     TRANLG3(I,IHEM,JH,L,1)=PLG3(I,IHEM,JH)*            
     &                    TRANLG3(I,IHEM,JH,L,1)                        
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDIF                                                          
C                                                                       
      tspd =    real(itspd)                                             
      rkounth = real(kounth)                                            
      tzc=273.15                                                        
      tm2=(tzc-1.9)/ct    ! Also done in T42.oasis update               
                                                                        
      do ihem=1,nhem                                                    
      do I=1,MGxx(iCol)                                                 
                                                                        
c-----daily average version                                             
c don't pass land or sea-ice heat/wind fluxes...                        
c NB E-P calculation now done HERE! MUST be used with setvbc.F from 2810
                                                                        
c-----tiling - only pass ocean values for coastal points                
c----Ocean points-----------------------                                
      if (gsg(i,ihem,jh).eq.0.0) then                                   
                                                                        
C Water Correction: evap variable is moisture convergence               
        evap=wathbl(i,ihem,jh)                                          
C atmosphere-ocean thermal fluxes dependent on ice fraction;            
C ocean isolated from atmosphere when ice cover > 0.5 consistent with   
C updating of surface temperature in rest of IGCM                       
        if(sicecov(i,ihem,jh).ge.0.5)then                               
          sensible2d(i,ihem,jh)=sensible2d(i,ihem,jh) +                 
     $    (-1*CV*P0*oceaniceflux(i,ihem)/TSPD)                          
        else                                                            
          swlw2d(i,ihem,jh)=swlw2d(i,ihem,jh)+                          
     $    snet(i,ihem,jh)/TSPD                                          
          heatlat2d(i,ihem,jh)=heatlat2d(i,ihem,jh)+                    
     $    (-1*CV*P0*slbl(i,ihem,jh)/TSPD)                               
          sensible2d(i,ihem,jh)=sensible2d(i,ihem,jh) +                 
     $    (-1*CV*P0*shbl(i,ihem,jh)/TSPD)                               
        endif                                                           
C atmosphere-ocean momentum fluxes dependent on ice thickness;          
C ocean isolated from atmosphere when covered with thick ice            
        if(tstaro(I,IHEM,jh).ge.TM2)then                                
          taux2d(i,ihem,jh)=taux2d(i,ihem,jh) +                         
     $    P0*txbl(i,ihem)/TSPD                                          
          tauy2d(i,ihem,jh)=tauy2d(i,ihem,jh) +                         
     $    P0*tybl(i,ihem)/TSPD                                          
        endif                                                           
                                                                        
        pp=(rrlr(i,ihem,jh)+rrcr(i,ihem,jh))                            
        water2d(i,ihem,jh)=water2d(i,ihem,jh)+(pp-evap)                 
                                                                        
      else                                                              
                                                                        
c----Partial points-----------------------                              
        if(ofrac(i,ihem,jh).ne.0. .and.                                 
     $  ofrac(i,ihem,jh).ne.1.)then                                     
                                                                        
C Correction for ocean parts of coastal points uses post-coastal tiling 
C evap variable is moisture convergence                                 
        evap=watsbl(i,ihem,jh)                                          
C atmosphere-ocean thermal fluxes dependent on ice fraction;            
C ocean isolated from atmosphere when ice cover > 0.5 consistent with   
C updating of surface temperature in rest of IGCM                       
        if(sicecov(i,ihem,jh).ge.0.5)then                               
          sensible2d(i,ihem,jh)=sensible2d(i,ihem,jh) +                 
     $    (-1*CV*P0*oceaniceflux(i,ihem)/TSPD)                          
        else                                                            
          swlw2d(i,ihem,jh)=swlw2d(i,ihem,jh) +                         
     $    snet(i,ihem,jh)/TSPD                                          
C Use slblc, shblc: ocean only heat fluxes                              
          heatlat2d(i,ihem,jh)=heatlat2d(i,ihem,jh)+                    
     $    (-1*CV*P0*slblc(i,ihem,jh)/TSPD)                              
          sensible2d(i,ihem,jh)=sensible2d(i,ihem,jh) +                 
     $    (-1*CV*P0*shblc(i,ihem,jh)/TSPD)                              
        endif                                                           
C atmosphere-ocean momentum fluxes dependent on ice thickness;          
C ocean isolated from atmosphere when covered with thick ice            
        if(tstaroo(I,IHEM,jh).ge.TM2)then                               
          taux2d(i,ihem,jh)=taux2d(i,ihem,jh) +                         
     $    P0*txbl(i,ihem)/TSPD                                          
          tauy2d(i,ihem,jh)=tauy2d(i,ihem,jh) +                         
     $    P0*tybl(i,ihem)/TSPD                                          
        endif                                                           
C Scale water balance by OFRAC: (1-OFRAC) done by runoff                
        pp=(rrlr(i,ihem,jh)+rrcr(i,ihem,jh))                            
        water2d(i,ihem,jh)=water2d(i,ihem,jh)+                          
     $                     (pp-evap)*ofrac(i,ihem,jh)                   
        exwater2d(i,ihem,jh)=exwater2d(i,ihem,jh)+                      
     $                 exwat(i,ihem,jh)                                 
        exiwater2d(i,ihem,jh)=exiwater2d(i,ihem,jh)+                    
     $                 exiwat(i,ihem,jh)                                
        exiwatern2d(i,ihem,jh)=exiwatern2d(i,ihem,jh)+                  
     $                 exiwatn(i,ihem,jh)                               
                                                                        
        else                                                            
c----Land points-----------------------                                 
        swlw2d(i,ihem,jh)=swlw2d(i,ihem,jh) +                           
     $  snet(i,ihem,jh)/TSPD                                            
        heatlat2d(i,ihem,jh)=heatlat2d(i,ihem,jh)+                      
     $  (-1*CV*P0*slbl(i,ihem,jh)                                       
     $                                /TSPD)                            
C Water Correction after coastal tiling: can use wathbl...              
C evap variable is moisture convergence                                 
                                                                        
        evap=watlbl(i,ihem,jh)                                          
C                                                                       
        sensible2d(i,ihem,jh)=sensible2d(i,ihem,jh) +                   
     $   (-1*CV*P0*shbl(i,ihem,jh)/TSPD)                                
        taux2d(i,ihem,jh)=taux2d(i,ihem,jh) +                           
     $  P0*txbl(i,ihem)/TSPD                                            
        tauy2d(i,ihem,jh)=tauy2d(i,ihem,jh) +                           
     $  P0*tybl(i,ihem)/TSPD                                            
        pp=(rrlr(i,ihem,jh)+rrcr(i,ihem,jh))                            
        water2d(i,ihem,jh)=water2d(i,ihem,jh)+(pp-evap)                 
        exwater2d(i,ihem,jh)=exwat(i,ihem,jh)+                          
     $  exwater2d(i,ihem,jh)                                            
        exiwater2d(i,ihem,jh)=exiwat(i,ihem,jh)+                        
     $  exiwater2d(i,ihem,jh)                                           
        exiwatern2d(i,ihem,jh)=exiwatn(i,ihem,jh)+                      
     $  exiwatern2d(i,ihem,jh)                                          
                                                                        
        end if                                                          
                                                                        
      end if                                                            
                                                                        
c-------------------------------------                                  
                                                                        
      end do                                                            
      end do                                                            
                                                                        
      ENDDO  ! ends DO JH=1,JGyy(iRow)                                  
                                                                        
      RETURN                                                            
      END                                                               
!=====================================================                  
! Calculates spectral tendencies from restoration (if included)         
! and biharmonic diffusion.                                             
!=====================================================                  
      SUBROUTINE DIFUSE                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Restoration fields and timescale                                      
!-----------------------------------------------------                  
      COMMON/RESTOR/ZRES,DRES,TRES,SPRES,DAMP                           
      REAL :: ZRES(IDM,NHEM,NL)                                         
      REAL :: DRES(IDM,NHEM,NL)                                         
      REAL :: TRES(IDM,NHEM,NL)                                         
      REAL :: SPRES(IDM,NHEM)                                           
      REAL :: DAMP                                                      
!-----------------------------------------------------                  
! Restoration temperature field and constants which determine it,       
! also contains timescales                                              
!-----------------------------------------------------                  
      COMMON/RESTIJ/TTRES                                               
     + ,DTNS,DTEP,DTTRP,FAC,DDAMP,TFRC,YRLEN,TRS                        
     +  ,ALR,ZTROP,TGR                                                  
      COMPLEX :: TTRES(NNTx,MMTy,NHEM,NL)                               
      REAL :: DTNS                                                      
      REAL :: DTEP                                                      
      REAL :: DTTRP                                                     
      REAL :: FAC(NL)                                                   
      REAL :: DDAMP(NL)                                                 
      REAL :: TFRC(NL)                                                  
      REAL :: YRLEN                                                     
      REAL :: TRS(NL)                                                   
      REAL :: ALR                                                       
      REAL :: ZTROP                                                     
      REAL :: TGR                                                       
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: AKZ                                                       
      REAL :: AKDT                                                      
      INTEGER :: I, IR                                                  
      INTEGER :: NVAL                                                   
      REAL :: RfCoeff(NLEVRF)                                           
      LOGICAL :: iFirstRf                                               
! Loop variables                                                        
      INTEGER :: IHEM, IM, IN, J, KK, L, MP                             
!-----------------------------------------------------                  
! Store Rayleigh Friction variables                                     
!-----------------------------------------------------                  
      SAVE iFirstRf, RfCoeff                                            
      DATA iFirstRf /.TRUE./                                            
!-----------------------------------------------------                  
! Add newtonian cooling and drag.                                       
!-----------------------------------------------------                  
      IF (LRESTIJ) THEN                                                 
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMTyy(iRow)                                      
                  DO IN=1,NNTxx(iCol)                                   
                     TT(IN,IM,IHEM,L)=TT(IN,IM,IHEM,L)-                 
     &                    DDAMP(L)*(T(IN,IM,IHEM,L)-TTRES(IN,IM,IHEM,L))
                     ZT(IN,IM,IHEM,L)=ZT(IN,IM,IHEM,L)-                 
     &                    TFRC(L)*Z(IN,IM,IHEM,L)                       
                     DT(IN,IM,IHEM,L)=DT(IN,IM,IHEM,L)-                 
     &                    TFRC(L)*D(IN,IM,IHEM,L)                       
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! No friction on planetary vorticity                           
         !-----------------------------------------------------         
         IF (MyPe.EQ.0) THEN                                            
            DO L=1,NL                                                   
               ZT(1,1,1,L)=ZT(1,1,1,L)+TFRC(L)*EZ                       
            ENDDO                                                       
         ENDIF                                                          
      ELSE                                                              
         IF(DAMP.GT.0.0) THEN                                           
            !-----------------------------------------------------      
            ! Apply damping for first zonal wavenumber                  
            !-----------------------------------------------------      
            IF (iRow.EQ.1) THEN                                         
               DO L=1,NL                                                
                  DO IHEM=1,NHEM                                        
                     DO J=1,mTwin1(2,1)                                 
                        ZT(J,1,IHEM,L)=ZT(J,1,IHEM,L)-DAMP*             
     &                       (Z(J,1,IHEM,L)-ZRES(J,IHEM,L))             
                        DT(J,1,IHEM,L)=DT(J,1,IHEM,L)-DAMP*             
     &                       (D(J,1,IHEM,L)-DRES(J,IHEM,L))             
                        TT(J,1,IHEM,L)=TT(J,1,IHEM,L)-DAMP*             
     &                       (T(J,1,IHEM,L)-TRES(J,IHEM,L))             
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDIF                                                       
         ENDIF                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Add in biharmonic diffusion if required                               
!-----------------------------------------------------                  
      IF (AK(2).GT.0.0) THEN                                            
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMTyy(iRow)                                      
                  DO IN=1,NNTxx(iCol)                                   
                     NVAL=nWave(IN,IM,iCol)                             
                     AKZ =AK(NVAL+3-IHEM)                               
                     AKDT=AK(NVAL+IHEM)                                 
                     ZT(IN,IM,IHEM,L)=ZT(IN,IM,IHEM,L)-                 
     &                    AKZ *Z(IN,IM,IHEM,L)                          
                     DT(IN,IM,IHEM,L)=DT(IN,IM,IHEM,L)-                 
     &                    AKDT*D(IN,IM,IHEM,L)                          
                     TT(IN,IM,IHEM,L)=TT(IN,IM,IHEM,L)-                 
     &                    AKDT*T(IN,IM,IHEM,L)                          
                  ENDDO                                                 
               ENDDO                                                    
               DO KK=1,NTRAC                                            
                  DO IM=1,MMTyy(iRow)                                   
                     DO IN=1,NNTxx(iCol)                                
                        NVAL=nWave(IN,IM,iCol)                          
                        AKDT=AK(NVAL+IHEM)                              
                        TRAT(IN,IM,IHEM,L,KK)=TRAT(IN,IM,IHEM,L,KK)-    
     &                       AKDT*TRA(IN,IM,IHEM,L,KK)                  
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! No diffusion on EZ (planetary vorticity)                     
         !-----------------------------------------------------         
         IF (MyPe.EQ.0) THEN                                            
            DO L=1,NL                                                   
               ZT(1,1,1,L)=ZT(1,1,1,L)+AK(2)*EZ                         
            ENDDO                                                       
         ENDIF                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! 11-08-97 Very crude Rayleigh Friction in the top NLEVRF levels.       
!          NLEVRF is set in the parameter line.                         
! 29-06-98 Changed to use RFCOEFF instead of TFRC.  SMR                 
! 11-05-00 NB RFCOEFF AND TFRC ARE BOTH RAYLEIGH FRICTION               
!          COEFFICIENT ARRAYS.                      SH                  
!-----------------------------------------------------                  
      if (ifirstrf) then                                                
         ifirstrf=.false.                                               
         do l=1,nlevrf                                                  
            !-----------------------------------------------------      
            ! The 0.0625 is a tunable parameter. 0.1875 recommended     
            ! for L26.                                                  
            !-----------------------------------------------------      
            rfcoeff(l)=((1.0-((l-1)*(1.0/REAL(nlevrf))))/pi2)*0.0625    
         enddo                                                          
      endif                                                             
      do ihem=1,nhem                                                    
         do l=1,nlevrf                                                  
            DO IM=1,MMTyy(iRow)                                         
               DO IN=1,NNTxx(iCol)                                      
                  ZT(IN,IM,IHEM,L)=ZT(IN,IM,IHEM,L)-                    
     &                 rfcoeff(l)*Z(IN,IM,IHEM,L)                       
                  DT(IN,IM,IHEM,L)=DT(IN,IM,IHEM,L)-                    
     &                 rfcoeff(l)*D(IN,IM,IHEM,L)                       
               ENDDO                                                    
            enddo                                                       
         enddo                                                          
      enddo                                                             
      IF (MyPe.EQ.0) THEN                                               
         DO L=1,NLEVRF                                                  
            ZT(1,1,1,L)=ZT(1,1,1,L)+RFCOEFF(L)*EZ                       
         ENDDO                                                          
      ENDIF                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! Black-box routine to take vertical derivative of a 2D array           
! in the model's vertical co-ordinates using the RGG matrix:            
!                                                                       
!         DF   =   RGG   *   F                                          
!                                                                       
! gives    DF   =   d(F)/dln(sigma)   =   (sigma)d(F)/d(sigma).         
!                                                                       
! Calculates DF at NL levels for NCOL columns from the array F.         
! Uses vector loops over NCOL at each level.  Note that for             
! NCOL <= 3 inner loops unroll to give vectorisation over levels.       
!                                                                       
! Uses explicit values of RGG matrix rather than function SDOT          
! over NL elements of RGG at each level.  Due to sparse nature          
! of RGG, this gives large speed-up for NL >> 3.                        
!                                                                       
! Reading model - sigma.                                                
!=====================================================                  
      SUBROUTINE DLSGCR(NCOL,IDIM,NHEM,JDIM,NL,JH,F,RGG,DF)             
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Passed arguments                                                      
!-----------------------------------------------------                  
      INTEGER :: NCOL                                                   
      INTEGER :: IDIM                                                   
      INTEGER :: JDIM                                                   
      INTEGER :: NHEM                                                   
      INTEGER :: NL                                                     
      INTEGER :: JH                                                     
      REAL :: F(IDIM,NHEM,JDIM,NL)                                      
      REAL :: RGG(NL,NL)                                                
      REAL :: DF(IDIM,NHEM,NL)                                          
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
! Loop variables                                                        
      INTEGER :: I, IHEM, L                                             
!-----------------------------------------------------                  
! Format statements                                                     
!-----------------------------------------------------                  
 6900 FORMAT(/' ***ABORT IN DLSGCR: MORE COLUMNS THAN ARRAY DIMENSION:' 
     :       ,' NCOL IDIM ='2I10)                                       
!-----------------------------------------------------                  
!                                                                       
!-----------------------------------------------------                  
      IF (NCOL.GT.IDIM) THEN                                            
         WRITE(2,6900) NCOL,IDIM                                        
         CALL ABORT                                                     
      ENDIF                                                             
!-----------------------------------------------------                  
! Top and bottom levels.                                                
!-----------------------------------------------------                  
      DO IHEM=1,NHEM                                                    
         DO I=1,NCOL                                                    
            DF(I,IHEM,1)=F(I,IHEM,JH,1)*RGG(1,1)+                       
     &           F(I,IHEM,JH,2)*RGG(2,1)+F(I,IHEM,JH,3)*RGG(3,1)        
            DF(I,IHEM,NL)=F(I,IHEM,JH,NL-2)*RGG(NL-2,NL)+               
     &           F(I,IHEM,JH,NL-1)*RGG(NL-1,NL)+                        
     &           F(I,IHEM,JH,NL)*RGG(NL,NL)                             
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Intermediate levels.                                                  
!-----------------------------------------------------                  
      DO L=2,NL-1                                                       
         DO IHEM=1,NHEM                                                 
            DO I=1,NCOL                                                 
               DF(I,IHEM,L)=F(I,IHEM,JH,L-1)*RGG(L-1,L)+                
     &              F(I,IHEM,JH,L+1)*RGG(L+1,L)                         
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! DRY ADIABATIC ADJUSTMENT TO NEUTRALITY IN DELT2 STEP                  
! Always mix moisture, irrespective of the parcel origin level.         
!=====================================================                  
      SUBROUTINE DRYADJ(NCRB,NCRT,I,IHEM,JH)                            
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Grid variables for DGRMLT loop                                        
!-----------------------------------------------------                  
      COMMON/GRIDPC/ASSBL,ASHBL,ASLBL,ARRCR,ARRLR                       
     &     ,TTVD,QTVD,TYBL,TXBL,TTRD,TTLR,TTCR,UTVD,VTVD                
     &     ,QTLR,QTCR,SSBL,SHBL,SLBL,RRCR,RRLR                          
     &     ,ARFLUX,ASFLD,ACLD,RFLUX,SFLD,CLD                            
      REAL :: ASSBL(MGx,NHEM,JGy)                                       
      REAL :: ASHBL(MGx,NHEM,JGy)                                       
      REAL :: ASLBL(MGx,NHEM,JGy)                                       
      REAL :: ARRCR(MGx,NHEM,JGy)                                       
      REAL :: ARRLR(MGx,NHEM,JGy)                                       
      REAL :: TTVD(MGx,NHEM,NL)                                         
      REAL :: QTVD(MGx,NHEM,NL)                                         
      REAL :: TYBL(MGx,NHEM)                                            
      REAL :: TXBL(MGx,NHEM)                                            
      REAL :: TTRD(MGx,NHEM,NL)                                         
      REAL :: TTLR(MGx,NHEM,NL)                                         
      REAL :: TTCR(MGx,NHEM,NL)                                         
      REAL :: UTVD(MGx,NHEM,NL)                                         
      REAL :: VTVD(MGx,NHEM,NL)                                         
      REAL :: QTLR(MGx,NHEM,NL)                                         
      REAL :: QTCR(MGx,NHEM,NL)                                         
      REAL :: SSBL(MGx,NHEM,JGy)                                        
      REAL :: SHBL(MGx,NHEM,JGy)                                        
      REAL :: SLBL(MGx,NHEM,JGy)                                        
      REAL :: RRCR(MGx,NHEM,JGy)                                        
      REAL :: RRLR(MGx,NHEM,JGy)                                        
      REAL :: arflux(MGx,NHEM,JGy,6)                                    
      REAL :: asfld(MGx,NHEM,JGy,6)                                     
      REAL :: acld(MGx,NHEM,JGy,4)                                      
      REAL :: rflux(MGx,NHEM,JGy,6)                                     
      REAL :: sfld(MGx,NHEM,JGy,6)                                      
      REAL :: cld(MGx,NHEM,JGy,4)                                       
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Passed arguments                                                      
!-----------------------------------------------------                  
      INTEGER :: NCRB                                                   
      INTEGER :: NCRT                                                   
      INTEGER :: I                                                      
      INTEGER :: IHEM                                                   
! Latitude index                                                        
      INTEGER :: JH                                                     
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER :: NLEV                                                   
      REAL :: SINT                                                      
      REAL :: SKINT                                                     
      REAL :: TINT                                                      
      REAL :: QINT                                                      
      REAL :: TL                                                        
      REAL :: QL                                                        
! Loop variables                                                        
      INTEGER :: L                                                      
!-----------------------------------------------------                  
! Sum up the levels                                                     
!-----------------------------------------------------                  
      NLEV=1+NCRB-NCRT                                                  
      SINT=0.0                                                          
      DO L=NCRT,NCRB                                                    
         SINT=SINT+DSIGMA(L)                                            
      ENDDO                                                             
!-----------------------------------------------------                  
! Work out the dot products                                             
!-----------------------------------------------------                  
      SKINT=0.0                                                         
      TINT=0.0                                                          
      QINT=0.0                                                          
      DO L=NCRT,NCRB                                                    
         SKINT=SKINT+SKAP(L)*DSIGMA(L)                                  
         TINT=TINT+TG3(I,IHEM,JH,L)*DSIGMA(L)                           
         QINT=QINT+TRAG3(I,IHEM,JH,L,1)*DSIGMA(L)                       
      ENDDO                                                             
      TINT=TINT/SKINT                                                   
      QINT=QINT/SINT                                                    
!-----------------------------------------------------                  
! Loop across levels                                                    
!-----------------------------------------------------                  
      DO L=NCRT,NCRB                                                    
         TL=SKAP(L)*TINT                                                
         QL=QINT                                                        
         TTDC(L)=(TL-TG3(I,IHEM,JH,L))/DELT2C                           
         QTDC(L)=(QL-TRAG3(I,IHEM,JH,L,1))/DELT2C                       
         TG3(I,IHEM,JH,L)=TL                                            
         TRAG3(I,IHEM,JH,L,1)=QL                                        
         CTCR(L,IHEM)=CTCR(L,IHEM)+2.0                                  
      ENDDO                                                             
      CTCR(NCRB,IHEM)=CTCR(NCRB,IHEM)-2.0                               
      RETURN                                                            
      END                                                               
!=====================================================                  
! Diabatic part of timestep. Completion of time-filter.                 
! Note that only Z,D,T have diabatic tendencies at present.             
!=====================================================                  
      SUBROUTINE DSTEP                                                  
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, IN, KK, L                                 
!-----------------------------------------------------                  
! Choose between centred and forward timestep                           
!-----------------------------------------------------                  
      IF(KOUNT.GT.KITS) THEN                                            
         !-----------------------------------------------------         
         ! Ordinary centred timestep                                    
         !-----------------------------------------------------         
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMTyy(iRow)                                      
                  DO IN=1,NNTxx(iCol)                                   
                     Z(IN,IM,IHEM,L)=Z(IN,IM,IHEM,L)+                   
     &                    DELT2*ZT(IN,IM,IHEM,L)                        
                     T(IN,IM,IHEM,L)=T(IN,IM,IHEM,L)+                   
     &                    DELT2*TT(IN,IM,IHEM,L)                        
                     D(IN,IM,IHEM,L)=D(IN,IM,IHEM,L)+                   
     &                    DELT2*DT(IN,IM,IHEM,L)                        
                     ZMI(IN,IM,IHEM,L)=ZMI(IN,IM,IHEM,L)+               
     &                    PNU*Z(IN,IM,IHEM,L)                           
                     TMI(IN,IM,IHEM,L)=TMI(IN,IM,IHEM,L)+               
     &                    PNU*T(IN,IM,IHEM,L)                           
                     DMI(IN,IM,IHEM,L)=DMI(IN,IM,IHEM,L)+               
     &                    PNU*D(IN,IM,IHEM,L)                           
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO KK=1,NTRAC                                                  
            DO L=1,NL                                                   
               DO IHEM=1,NHEM                                           
                  DO IM=1,MMTyy(iRow)                                   
                     DO IN=1,NNTxx(iCol)                                
                        TRA(IN,IM,IHEM,L,KK)=TRA(IN,IM,IHEM,L,KK)+      
     &                       DELT2*TRAT(IN,IM,IHEM,L,KK)                
                        TRAMI(IN,IM,IHEM,L,KK)=TRAMI(IN,IM,IHEM,L,KK)+  
     &                       PNU*TRA(IN,IM,IHEM,L,KK)                   
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO IHEM=1,NHEM                                                 
            DO IM=1,MMTyy(iRow)                                         
               DO IN=1,NNTxx(iCol)                                      
                  SPMI(IN,IM,IHEM)=SPMI(IN,IM,IHEM)+                    
     &                 PNU*SP(IN,IM,IHEM)                               
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         RETURN                                                         
      ELSE                                                              
         !-----------------------------------------------------         
         ! Short initial timestep                                       
         !-----------------------------------------------------         
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMTyy(iRow)                                      
                  DO IN=1,NNTxx(iCol)                                   
                     Z(IN,IM,IHEM,L)=Z(IN,IM,IHEM,L)+                   
     &                    DELT2*ZT(IN,IM,IHEM,L)                        
                     T(IN,IM,IHEM,L)=T(IN,IM,IHEM,L)+                   
     &                    DELT2*TT(IN,IM,IHEM,L)                        
                     D(IN,IM,IHEM,L)=D(IN,IM,IHEM,L)+                   
     &                    DELT2*DT(IN,IM,IHEM,L)                        
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO KK=1,NTRAC                                                  
            DO L=1,NL                                                   
               DO IHEM=1,NHEM                                           
                  DO IM=1,MMTyy(iRow)                                   
                     DO IN=1,NNTxx(iCol)                                
                        TRA(IN,IM,IHEM,L,KK)=TRA(IN,IM,IHEM,L,KK)+      
     &                       DELT2*TRAT(IN,IM,IHEM,L,KK)                
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DELT=DELT2                                                     
         DELT2=DELT2+DELT2                                              
         RETURN                                                         
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Calculates various global diagnostic quantities                       
! every itstp timesteps.                                                
!=====================================================                  
      SUBROUTINE ENERGY                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Switches counters and constants controlling type and frequency of     
! model output                                                          
!-----------------------------------------------------                  
      COMMON/OUTCON/RNTAPE,DAY,SQR2,RSQR2,EAM1,EAM2,TOUT1               
     &     ,TOUT2,LSPO,LGPO                                             
     &     ,RNTAPO                                                      
     &     ,LSHIST,LMINIH                                               
      COMMON/OUTCONINT/NCOEFF,NLAT,INLAT,INSPC,KOUNTP                   
     &                 ,KOUNTE,KOUNTH,KOUNTR,KOUTP                      
     &                 ,KOUTE,KOUTH,KOUTR                               
! Identyfying run number                                                
      REAL :: RNTAPE                                                    
! Maximum wavenumber of printed spectral coeffecients                   
      INTEGER :: NCOEFF                                                 
! Number of latitudes in printed grid point fields                      
      INTEGER :: NLAT                                                   
      INTEGER :: INLAT                                                  
!                                                                       
      INTEGER :: INSPC                                                  
! Number of timsteps between successive writing of spectral             
! and gridded fields                                                    
      INTEGER :: KOUNTP                                                 
      INTEGER :: KOUNTE                                                 
! Number of timsteps between successive writing of history              
! records, containing spectral fields at the present and                
! previous time levels.                                                 
      INTEGER :: KOUNTH                                                 
! Number of timsteps between successive writing of restart              
! records, containing the spectral fields at the present and            
! previous time levels                                                  
      INTEGER :: KOUNTR                                                 
      INTEGER :: KOUTP                                                  
      INTEGER :: KOUTE                                                  
      INTEGER :: KOUTH                                                  
      INTEGER :: KOUTR                                                  
      REAL :: DAY                                                       
      REAL :: SQR2                                                      
      REAL :: RSQR2                                                     
      REAL :: EAM1                                                      
      REAL :: EAM2                                                      
      REAL :: TOUT1                                                     
      REAL :: TOUT2                                                     
      LOGICAL :: LSPO(NL)                                               
      LOGICAL :: LGPO(NL)                                               
      REAL :: RNTAPO                                                    
      LOGICAL :: LSHIST                                                 
! Reduce the history file to single precision if true                   
      LOGICAL :: LMINIH                                                 
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      COMPLEX :: TIG,SPIP,CTIG,CRGS                                     
      INTEGER, PARAMETER :: TagRef=600                                  
      INTEGER :: IG                                                     
      REAL :: DSIG, DSIGH                                               
      REAL :: RTIG, RZIG, RDIG                                          
      REAL :: RSPIP                                                     
      REAL :: TOTP                                                      
      REAL :: PSITOT                                                    
      REAL :: CHITOT                                                    
      REAL :: TMPTOT                                                    
      REAL :: TOTI                                                      
      REAL :: RTL                                                       
      REAL :: ST2B                                                      
      REAL :: ST                                                        
      REAL :: AMSP                                                      
      REAL :: HOLDVAR(5+NTRAC)                                          
      REAL :: GS1                                                       
      INTEGER :: iPe                                                    
      INTEGER :: iCol2, iRow2                                           
! Total moisture diagnostic (as in BGCM5).                              
      REAL :: TOTQ                                                      
! Accounts for the p0 term excluded from SPA(1) in MGRMLT               
      REAL :: AMQ                                                       
! Loop variables                                                        
      INTEGER :: IHEM, IM, IN, L                                        
!-----------------------------------------------------                  
! Remove planetary vorticity so Z contains relative vorticity           
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         DO L=1,NL                                                      
            Z(1,1,1,L)=Z(1,1,1,L)-EZ                                    
         ENDDO                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Calculate means - PSITOT RMS vorticity                                
!                   CHITOT RMS divergence                               
!                   TMPTOT RMS temperature                              
!                   TOTP  IE+PE potential energy                        
!                   AMSP mean surface pressure                          
!-----------------------------------------------------                  
      HoldVar(:)=0.0                                                    
      ST2B=0.                                                           
      ST=0.                                                             
      AMQ=0.                                                            
      DO IM=1,MMTyy(iRow)                                               
         DO IN=1,NNTxx(iCol)                                            
            !-----------------------------------------------------      
            ! Determine if this is zonal wavenumber 0                   
            !-----------------------------------------------------      
            IF ((mWave(1,IM).EQ.0).AND.(IN.LE.mTwin1(2,1))) THEN        
               !-----------------------------------------------------   
               ! Zonal wavenumber 0                                     
               !-----------------------------------------------------   
               DO IHEM=1,NHEM                                           
                  DO L=1,NL                                             
                     DSIG=DSIGMA(L)                                     
                     DSIGH=0.5*DSIG                                     
                     RZIG=REAL(Z(IN,IM,IHEM,L))                         
                     RDIG=REAL(D(IN,IM,IHEM,L))                         
                     RTIG=REAL(T(IN,IM,IHEM,L))                         
                     HoldVar(1)=HoldVar(1)+DSIGH*RZIG*RZIG              
                     HoldVar(2)=HoldVar(2)+DSIGH*RDIG*RDIG              
                     HoldVar(3)=HoldVar(3)+DSIGH*RTIG*RTIG              
                     RSPIP=REAL(SPA(IN,IM,IHEM))                        
                     HoldVar(4)=HoldVar(4)+DSIGH*RSPIP*RTIG             
                     HoldVar(6)=HoldVar(6)+DSIGH*RSPIP*                 
     &                    REAL(TRA(IN,IM,IHEM,L,1))                     
                  ENDDO                                                 
                  HoldVar(5)=HoldVar(5)+0.5*RSPIP*REAL(GS(IN,IM,IHEM))  
               ENDDO                                                    
            ELSE                                                        
               !-----------------------------------------------------   
               ! Zonal wavenumber is greater than 0                     
               !-----------------------------------------------------   
               DO IHEM=1,NHEM                                           
                  DO L=1,NL                                             
                     DSIG=DSIGMA(L)                                     
                     TIG=T(IN,IM,IHEM,L)                                
                     CTIG=CONJG(TIG)                                    
                     SPIP=SPA(IN,IM,IHEM)                               
                     HoldVar(1)=HoldVar(1)+DSIG*                        
     &                    REAL(Z(IN,IM,IHEM,L)*CONJG(Z(IN,IM,IHEM,L)))  
                     HoldVar(2)=HoldVar(2)+DSIG*                        
     &                    REAL(D(IN,IM,IHEM,L)*CONJG(D(IN,IM,IHEM,L)))  
                     HoldVar(3)=HoldVar(3)+DSIG*REAL(TIG*CTIG)          
                     HoldVar(4)=HoldVar(4)+DSIG*REAL(SPIP*CTIG)         
                     HoldVar(6)=HoldVar(6)+DSIG*                        
     &                    REAL(SPIP*CONJG(TRA(IN,IM,IHEM,L,1)))         
                  ENDDO                                                 
                  CRGS=CONJG(GS(IN,IM,IHEM))                            
                  HoldVar(5)=HoldVar(5)+REAL(SPIP*CRGS)                 
               ENDDO                                                    
            ENDIF                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Collect all the wavenumbers together                                  
!-----------------------------------------------------                  
      PSITOT=holdvar(1)                                                 
      CHITOT=holdvar(2)                                                 
      TMPTOT=holdvar(3)                                                 
      TOTI=holdvar(4)                                                   
      TOTP=holdvar(5)                                                   
      TOTQ=holdvar(6)                                                   
      IF (MyPe.GT.0) THEN                                               
         !-----------------------------------------------------         
         ! Send all the data to PE 0                                    
         !-----------------------------------------------------         
         iRecv=0                                                        
         nBuffSize=5+NTRAC                                              
         iTag=TagRef+MyPe                                               
         CALL MPI_BSEND(HoldVar, nBuffSize, ParaReal, iRecv,            
     &        iTag, MPI_COMM_WORLD, ierr)                               
      ELSE                                                              
         DO iPe=1,NPE-1                                                 
            !-----------------------------------------------------      
            ! On PE 0, receive all the data                             
            !-----------------------------------------------------      
            iSend=iPe                                                   
            nBuffSize=5+NTRAC                                           
            iTag=TagRef+iPe                                             
            CALL MPI_RECV(HoldVar, nBuffSize, ParaReal, iSend,          
     &           iTag, MPI_COMM_WORLD, istat, ierr)                     
            !-----------------------------------------------------      
            ! Calculate means                                           
            !-----------------------------------------------------      
            PSITOT=PSITOT+HoldVar(1)                                    
            CHITOT=CHITOT+HoldVar(2)                                    
            TMPTOT=TMPTOT+HoldVar(3)                                    
            TOTI=TOTI+HoldVar(4)                                        
            TOTP=TOTP+HoldVar(5)                                        
            TOTQ=TOTQ+holdvar(6)                                        
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Calculate ST2B and ST                                        
         !-----------------------------------------------------         
         DO L=1,NL                                                      
            RTL=REAL(T(1,1,1,L))                                        
            DSIG=DSIGMA(L)                                              
            ST2B=ST2B+T0(L)*RTL*DSIG                                    
            ST=ST+RTL*DSIG                                              
            AMQ=AMQ+DSIG*REAL(TRA(1,1,1,L,1))                           
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Calculate the mean surface pressure, AMSP                    
         !-----------------------------------------------------         
         AMSP=1.0+REAL(SPA(1,1,1))*RSQR2                                
         !-----------------------------------------------------         
         ! Moisture calculations                                        
         !-----------------------------------------------------         
         AMQ=CQ*RSQR2*AMQ                                               
         TOTQ=CQ*TOTQ+AMQ                                               
         !-----------------------------------------------------         
         ! Final calculations                                           
         !-----------------------------------------------------         
         GS1=REAL(GS(1,1,1))                                            
         PSITOT=SQRT(PSITOT)                                            
         CHITOT=SQRT(CHITOT)                                            
         TMPTOT=SQRT(TMPTOT+TOUT2+ST2B*SQR2)                            
         TOTP=TOTP+RSQR2*GS1+(AMSP*TOUT1+TOTI+RSQR2*ST)/AKAP            
         !-----------------------------------------------------         
         ! Display mean values to screen                                
         !-----------------------------------------------------         
         IF (KOUNT.EQ.0) WRITE(2,40)                                    
         WRITE (2,50) KOUNT,PSITOT,CHITOT,TMPTOT,TOTP,TOTQ,AMSP         
         !-----------------------------------------------------         
         ! Restore Z to absolute vorticity                              
         !-----------------------------------------------------         
         DO L=1,NL                                                      
            Z(1,1,1,L)=Z(1,1,1,L)+EZ                                    
         ENDDO                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Format statements                                                     
!-----------------------------------------------------                  
   40 FORMAT(/3X,'KOUNT',3X,'RMSVORT',8X,'RMSDIV',8X,'RMSTEMP'          
     +       ,8X,'PE+IE',8X,'TOTQ',9X,'MSP')                            
   50 FORMAT(I8,1X,4E15.6,2F13.10)                                      
                                                                        
      RETURN                                                            
      END                                                               
!=====================================================                  
! Carry out a fast-fourier transform                                    
!=====================================================                  
      SUBROUTINE FFT(iDirection, nLevels, ARRAY)                        
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Constants and arrays needed for the fast Fourier transforms           
!-----------------------------------------------------                  
      COMMON/COMFFT/TRIG,WORK                                           
      COMMON/COMFFTINT/IFAX                                             
      REAL :: TRIG((MG+MG+MG)/2+1)                                      
      REAL :: WORK(NCRAY*(MG+1))                                        
      INTEGER :: IFAX(10)                                               
!-----------------------------------------------------                  
! Passed variable                                                       
!-----------------------------------------------------                  
      INTEGER, INTENT(IN) :: iDirection                                 
      INTEGER, INTENT(IN) :: nLevels                                    
      REAL, INTENT(INOUT) :: ARRAY(MGPP, nLevels)                       
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER :: nFullNcray                                             
      INTEGER :: nRemain                                                
      INTEGER :: I                                                      
!-----------------------------------------------------                  
! Work out how many full NCRAY levels can be fitted in                  
!-----------------------------------------------------                  
      nFullNcray=nLevels/NCRAY                                          
      nRemain=nLevels - (nFullNcray * NCRAY)                            
!-----------------------------------------------------                  
! Carry at fast fourier transform                                       
!-----------------------------------------------------                  
      DO I=1,nFullNcray                                                 
         CALL FFT991(ARRAY(1,1+(I-1)*NCRAY),WORK,TRIG,IFAX,1,           
     &        MGPP,MG,NCRAY,iDirection)                                 
      ENDDO                                                             
      IF (nRemain.GT.0) THEN                                            
         CALL FFT991(ARRAY(1,1+nFullNcray*NCRAY),WORK,TRIG,IFAX,1,      
     &        MGPP,MG,nRemain,iDirection)                               
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Perform all the direct Legendre transforms for the adiabatic          
! part of the timestep at the current latitude (pair), in place         
! of separate calls to HANAL for individual transform types.            
!                                                                       
! An even/odd value of ITYPE denotes a spectral field of even/odd       
! symmetry.                                                             
!                                                                       
! Maximum vector efficiency is achieved by chaining all multi-level     
! transforms in one loop and by chaining all single-level transform     
! in a second loop.                                                     
!                                                                       
! All dummy argument arrays are declared complex.                       
! All array dimensions are parameters.                                  
! Multi-level arrays are 3-dimensional.                                 
!                                                                       
! NOTE: The y-derivative transforms use integration by parts and        
!       are valid only if the input field has zero zonal mean at        
!       both poles.                                                     
!                                                                       
! NOTE: *** THE INPUT FOURIER FIELDS ARE MODIFIED IF GLOBAL ***         
!                                                                       
! Version for RSGUP3.                     Mike Blackburn,  12.01.95     
!=====================================================                  
      SUBROUTINE HANALV                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Polynomial used to aid vectorization of Legendre transforms           
!-----------------------------------------------------                  
      COMMON/POLYNO/POLY,CMPA                                           
      REAL :: POLY(NNT,MMTy,2,2)                                        
      COMPLEX :: CMPA(2*MMTy,NHEM)                                      
!-----------------------------------------------------                  
! Half transforms. The levels are divided across the columns of PEs;    
! the zonal wavenumbers are divided across the row of PEs; and on       
! ech PE the latitude are complete.                                     
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA/UG,FUG,VG,FVG,TG,TNLG,PLG                           
     &     ,TRAG,TRANLG                                                 
! U_psi, then U (rotational wind)                                       
      COMPLEX :: UG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FUG(2*MMTy,NHEM,JG,NLx)                                
! V_chi, then V (divergent wind)                                        
      COMPLEX :: VG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FVG(2*MMTy,NHEM,JG,NLx)                                
! Temperature anomaly (T')                                              
      COMPLEX :: TG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: TNLG(2*MMTy,NHEM,JG,NLx)                               
! ln p* or p*                                                           
      COMPLEX :: PLG(2*MMTy,NHEM,JG)                                    
! Tracer term                                                           
      COMPLEX :: TRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                         
!                                                                       
      COMPLEX :: TRANLG(2*MMTy,NHEM,JG,NLx,NTRAC)                       
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB/ZG,DG,PJG,PMG,SPG,VPG,EG,UTG,VTG                    
     &     ,UTRAG,VTRAG                                                 
     &     ,CHIG,SFG,FVGT,FUGT                                          
! Vorticity (zeta)                                                      
      COMPLEX :: ZG(2*MMTy,NHEM,JG,NLx)                                 
! Divergence (d)                                                        
      COMPLEX :: DG(2*MMTy,NHEM,JG,NLx)                                 
! (1-mu^2) (d(ln p*)/d mu)                                              
      COMPLEX :: PJG(2*MMTy,NHEM,JG)                                    
! (d(ln p*)/d lambda)                                                   
      COMPLEX :: PMG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: SPG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: VPG(2*MMTy,NHEM,JG)                                    
! (U^2 + V^2)                                                           
      COMPLEX :: EG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: UTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: VTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: UTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: VTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: CHIG(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: SFG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: FVGT(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: FUGT(2*MMTy,NHEM,JG,NLx)                               
!-----------------------------------------------------                  
! Like the variables in SPECTR but the meridional wavenumbers           
! are complete instead of the levels.                                   
!-----------------------------------------------------                  
      COMMON/SPECTR2/Z2,D2,T2,SP2,SPA2,VP2,DTE2,TT2,DT2,ZT2             
     &     ,TRA2,TRAT2                                                  
! Vorticity coefficient                                                 
      COMPLEX :: Z2(NNT,MMTy,NHEM,NLx)                                  
! Divergence coefficient                                                
      COMPLEX :: D2(NNT,MMTy,NHEM,NLx)                                  
! Temperature coefficient                                               
      COMPLEX :: T2(NNT,MMTy,NHEM,NLx)                                  
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP2(NNT,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA2(NNT,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP2(NNT,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE2(NNT,MMTy,NHEM,NLx)                                
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT2(NNT,MMTy,NHEM,NLx)                                 
! Tracer coefficient                                                    
      COMPLEX :: TRA2(NNT,MMTy,NHEM,NLx,NTRAC)                          
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT2(NNT,MMTy,NHEM,NLx,NTRAC)                         
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: AW1256                                                    
      REAL :: AW3478                                                    
      INTEGER :: IP, IPM, MP                                            
      COMPLEX :: TEMP                                                   
! Loop variables                                                        
      INTEGER :: IHEM, IM, ITRAC, J, L, M, N, nT                        
!-----------------------------------------------------                  
! For a global run, sum and difference the complete Fourier             
! transforms at the northern and southern latitude rows to give         
! the even and odd contributions : E=(N+S)/2, O=(N-S)/2.                
! For Fourier fields symmetric about equator  : even precedes odd.      
! For Fourier fields asymmetric about equator : odd precedes even.      
!-----------------------------------------------------                  
      IF (NHEM.EQ.2) THEN                                               
         DO J=1,JG                                                      
            DO IM=1,2*MMTyy(iRow)                                       
               !-----------------------------------------------------   
               ! Surface pressure : symmetric.                          
               !-----------------------------------------------------   
               TEMP=SPG(IM,1,J)                                         
               SPG(IM,1,J)=0.5*(TEMP+SPG(IM,NHEM,J))                    
               SPG(IM,NHEM,J)=0.5*(TEMP-SPG(IM,NHEM,J))                 
               !-----------------------------------------------------   
               ! Surface pressure tendency : symmetric.                 
               !-----------------------------------------------------   
               TEMP=VPG(IM,1,J)                                         
               VPG(IM,1,J)=0.5*(TEMP+VPG(IM,NHEM,J))                    
               VPG(IM,NHEM,J)=0.5*(TEMP-VPG(IM,NHEM,J))                 
            ENDDO                                                       
         ENDDO                                                          
                                                                        
         DO J=1,JG                                                      
            DO IM=1,2*MMTyy(iRow)                                       
               DO L=1,NLxx(iCol)                                        
                  !---------------------------------------------------  
                  ! Divergence tendency : energy term : symmetric.      
                  !---------------------------------------------------  
                  TEMP=EG(IM,1,J,L)                                     
                  EG(IM,1,J,L)=0.5*(TEMP+EG(IM,NHEM,J,L))               
                  EG(IM,NHEM,J,L)=0.5*(TEMP-EG(IM,NHEM,J,L))            
                  !---------------------------------------------------  
                  ! Temperature tendency : main + d/dx part : symmetric 
                  !---------------------------------------------------  
                  TEMP=TNLG(IM,1,J,L)                                   
                  TNLG(IM,1,J,L)=0.5*(TEMP+TNLG(IM,NHEM,J,L))           
                  TNLG(IM,NHEM,J,L)=0.5*(TEMP-TNLG(IM,NHEM,J,L))        
                  !---------------------------------------------------  
                  ! Divergence tendency : d/dx part : symmetric.        
                  !---------------------------------------------------  
                  TEMP=FUG(IM,1,J,L)                                    
                  FUG(IM,1,J,L)=0.5*(TEMP+FUG(IM,NHEM,J,L))             
                  FUG(IM,NHEM,J,L)=0.5*(TEMP-FUG(IM,NHEM,J,L))          
                  !---------------------------------------------------  
                  ! Vorticity tendency : d/dx part : anti-symmetric.    
                  !---------------------------------------------------  
                  TEMP=FVG(IM,1,J,L)                                    
                  FVG(IM,1,J,L)=0.5*(TEMP-FVG(IM,NHEM,J,L))             
                  FVG(IM,NHEM,J,L)=0.5*(TEMP+FVG(IM,NHEM,J,L))          
                  !---------------------------------------------------  
                  ! Temperature tendency : d/dy part : anti-symmetric.  
                  !---------------------------------------------------  
                  TEMP=VTG(IM,1,J,L)                                    
                  VTG(IM,1,J,L)=0.5*(TEMP-VTG(IM,NHEM,J,L))             
                  VTG(IM,NHEM,J,L)=0.5*(TEMP+VTG(IM,NHEM,J,L))          
                  !---------------------------------------------------  
                  ! Divergence tendency : d/dy part : anti-symmetric.   
                  !---------------------------------------------------  
                  TEMP=FVGT(IM,1,J,L)                                   
                  FVGT(IM,1,J,L)=0.5*(TEMP-FVGT(IM,NHEM,J,L))           
                  FVGT(IM,NHEM,J,L)=0.5*(TEMP+FVGT(IM,NHEM,J,L))        
                  !---------------------------------------------------  
                  ! Vorticity tendency : d/dy part : symmetric.         
                  !---------------------------------------------------  
                  TEMP=FUGT(IM,1,J,L)                                   
                  FUGT(IM,1,J,L)=0.5*(TEMP+FUGT(IM,NHEM,J,L))           
                  FUGT(IM,NHEM,J,L)=0.5*(TEMP-FUGT(IM,NHEM,J,L))        
                  !---------------------------------------------------  
                  ! Tracer tendencies                                   
                  !---------------------------------------------------  
                  DO ITRAC=1,NTRAC                                      
                     !------------------------------------------------  
                     ! main + d/dx part : symmetric.                    
                     !------------------------------------------------  
                     TEMP=TRANLG(IM,1,J,L,ITRAC)                        
                     TRANLG(IM,1,J,L,ITRAC)=                            
     &                    0.5*(TEMP+TRANLG(IM,NHEM,J,L,ITRAC))          
                     TRANLG(IM,NHEM,J,L,ITRAC)=                         
     &                    0.5*(TEMP-TRANLG(IM,NHEM,J,L,ITRAC))          
                     !------------------------------------------------  
                     ! d/dy part : anti-symmetric.                      
                     !------------------------------------------------  
                     TEMP=VTRAG(IM,1,J,L,ITRAC)                         
                     VTRAG(IM,1,J,L,ITRAC)=                             
     &                    0.5*(TEMP-VTRAG(IM,NHEM,J,L,ITRAC))           
                     VTRAG(IM,NHEM,J,L,ITRAC)=                          
     &                    0.5*(TEMP+VTRAG(IM,NHEM,J,L,ITRAC))           
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Loop over all latitudes                                               
!-----------------------------------------------------                  
      DO J=1,JG                                                         
         !-----------------------------------------------------         
         ! Set up the appropriate Gaussian weight for the current       
         ! latitude, dependent on transform type.                       
         !-----------------------------------------------------         
         AW1256=AW(J,1)*CSSQ(J,1)                                       
         AW3478=-AW(J,1)                                                
         !-----------------------------------------------------         
         ! Calculate POLY array in a vector loop before the main        
         ! transforms, for the required Legendre Function types.        
         ! Both even and odd functions are required, irrespective of    
         ! NHEM.                                                        
         !-----------------------------------------------------         
         DO IHEM=1,2                                                    
            DO IM=1,MMTyy(iRow)                                         
               DO IP=1,NNT                                              
                  POLY(IP,IM,IHEM,1)=AW1256*ALP(IP,IM,IHEM,J)           
                  POLY(IP,IM,IHEM,2)=AW3478*DALP(IP,IM,IHEM,J)          
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Transform single-level fields.                               
         ! Vectorisation is over total wavenumber for each zonal        
         ! wavenumber.                                                  
         !-----------------------------------------------------         
         IF (iCol.EQ.1) THEN                                            
            DO nT=1,2                                                   
               DO IM=1,MMTyy(iRow)                                      
                  MP=(nT-1)*(2*(MMTyy(iRow)-IM)+1)+IM                   
                  DO IP=mTwin2(nT,IM)+1,mTwin2(nT+1,IM)                 
                     !------------------------------------------------  
                     ! Surface pressure          : type 2.              
                     !------------------------------------------------  
                     SPA2(IP,IM,1)=SPA2(IP,IM,1) +                      
     &                    POLY(IP,IM,1,1)*SPG(MP,1,J)                   
                     !------------------------------------------------  
                     ! Surface pressure tendency : type 2.              
                     !------------------------------------------------  
                     VP2 (IP,IM,1)=VP2 (IP,IM,1) +                      
     &                    POLY(IP,IM,1,1)*VPG(MP,1,J)                   
                     IF (NHEM.EQ.2) THEN                                
                        !---------------------------------------------  
                        ! Surface pressure          : type 2.           
                        !---------------------------------------------  
                        SPA2(IP,IM,NHEM)=SPA2(IP,IM,NHEM) +             
     &                       POLY(IP,IM,2,1)*SPG(MP,NHEM,J)             
                        !---------------------------------------------  
                        ! Surface pressure tendency : type 2.           
                        !---------------------------------------------  
                        VP2 (IP,IM,NHEM)=VP2 (IP,IM,NHEM) +             
     &                       POLY(IP,IM,2,1)*VPG(MP,NHEM,J)             
                     ENDIF                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Transform multi-level fields.                                
         ! Inner loop vectorisation is over total wavenumber, to access 
         ! spectral memory sequentially, avoiding skip distances being  
         ! multiple of 8 (which causes memory bank conflicts on Cray    
         ! vector machines).                                            
         !-----------------------------------------------------         
         DO L=1,NLxx(iCol)                                              
            DO nT=1,2                                                   
               DO IM=1,MMTyy(iRow)                                      
                  MP=(nT-1)*(2*(MMTyy(iRow)-IM)+1)+IM                   
                  DO IP=mTwin2(nT,IM)+1,mTwin2(nT+1,IM)                 
                     !------------------------------------------------  
                     ! Divergence tendency  : energy term     : type 2. 
                     !------------------------------------------------  
                     DTE2(IP,IM,1,L)=DTE2(IP,IM,1,L) +                  
     &                    POLY(IP,IM,1,1)*EG  (MP,1,J,L)                
                     !------------------------------------------------  
                     ! Temperature tendency :main + d/dx part : type 2. 
                     ! Temperature tendency :d/dy part        : type 4. 
                     !------------------------------------------------  
                     TT2 (IP,IM,1,L)=TT2 (IP,IM,1,L) + POLY(IP,IM,1,1)* 
     &                    TNLG(MP,1,J,L)+ POLY(IP,IM,1,2)*              
     &                    VTG (MP,1,J,L)                                
                     !------------------------------------------------  
                     ! Divergence tendency  : d/dx part       : type 2. 
                     ! Divergence tendency  : d/dy part       : type 4. 
                     !------------------------------------------------  
                     DT2 (IP,IM,1,L)=DT2 (IP,IM,1,L) + POLY(IP,IM,1,1)* 
     &                    FUG (MP,1,J,L)+POLY(IP,IM,1,2)*               
     &                    FVGT(MP,1,J,L)                                
                     !------------------------------------------------  
                     ! Vorticity tendency   : d/dx part       : type 1. 
                     ! Vorticity tendency   : d/dy part       : type 3. 
                     !------------------------------------------------  
                     ZT2 (IP,IM,1,L)=ZT2 (IP,IM,1,L) + POLY(IP,IM,2,1)* 
     &                    FVG (MP,1,J,L)+POLY(IP,IM,2,2)*               
     &                    FUGT(MP,1,J,L)                                
                     !------------------------------------------------  
                     ! Tracer tendency   : d/dy part        : type 4.   
                     !------------------------------------------------  
                     DO ITRAC=1,NTRAC                                   
                        TRAT2(IP,IM,1,L,ITRAC)=TRAT2(IP,IM,1,L,ITRAC)+  
     &                       POLY(IP,IM,1,1)*TRANLG(MP,1,J,L,ITRAC)+    
     &                       POLY(IP,IM,1,2)*VTRAG(MP,1,J,L,ITRAC)      
                     ENDDO                                              
                     IF (NHEM.EQ.2) THEN                                
                        !---------------------------------------------  
                        ! Divergence tendency  : energy term  : type 2. 
                        !---------------------------------------------  
                        DTE2(IP,IM,NHEM,L)=DTE2(IP,IM,NHEM,L) +         
     &                       POLY(IP,IM,2,1)*EG(MP,NHEM,J,L)            
                        !---------------------------------------------  
                        ! Temperature tendency: main+d/dx part: type 2. 
                        ! Temperature tendency: d/dy part     : type 4. 
                        !---------------------------------------------  
                        TT2 (IP,IM,NHEM,L)=TT2 (IP,IM,NHEM,L) +         
     &                       POLY(IP,IM,2,1)*TNLG(MP,NHEM,J,L) +        
     &                       POLY(IP,IM,2,2)*VTG (MP,NHEM,J,L)          
                        !---------------------------------------------  
                        ! Divergence tendency  : d/dx part    : type 2. 
                        ! Divergence tendency  : d/dy part    : type 4. 
                        !---------------------------------------------  
                        DT2 (IP,IM,NHEM,L)=DT2 (IP,IM,NHEM,L) +         
     &                       POLY(IP,IM,2,1)*FUG (MP,NHEM,J,L) +        
     &                       POLY(IP,IM,2,2)*FVGT(MP,NHEM,J,L)          
                        !---------------------------------------------  
                        ! Vorticity tendency   : d/dx part    : type 1. 
                        ! Vorticity tendency   : d/dy part    : type 3. 
                        !---------------------------------------------  
                        ZT2 (IP,IM,NHEM,L)=ZT2 (IP,IM,NHEM,L) +         
     &                       POLY(IP,IM,1,1)*FVG (MP,NHEM,J,L) +        
     &                       POLY(IP,IM,1,2)*FUGT(MP,NHEM,J,L)          
                        !---------------------------------------------  
                        ! Tracer tendency   : d/dy part       : type 4. 
                        !---------------------------------------------  
                        DO ITRAC=1,NTRAC                                
                           TRAT2(IP,IM,NHEM,L,ITRAC)=                   
     &                          TRAT2(IP,IM,NHEM,L,ITRAC)+              
     &                          POLY(IP,IM,2,1)*                        
     &                          TRANLG(MP,NHEM,J,L,ITRAC)+              
     &                          POLY(IP,IM,2,2)*                        
     &                          VTRAG(MP,NHEM,J,L,ITRAC)                
                        ENDDO                                           
                    ENDIF                                               
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! Same as HANALV but only for tracer variable                           
!=====================================================                  
      SUBROUTINE HANALVICT                                              
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Polynomial used to aid vectorization of Legendre transforms           
!-----------------------------------------------------                  
      COMMON/POLYNO/POLY,CMPA                                           
      REAL :: POLY(NNT,MMTy,2,2)                                        
      COMPLEX :: CMPA(2*MMTy,NHEM)                                      
!-----------------------------------------------------                  
! Half transforms. The levels are divided across the columns of PEs;    
! the zonal wavenumbers are divided across the row of PEs; and on       
! ech PE the latitude are complete.                                     
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA/UG,FUG,VG,FVG,TG,TNLG,PLG                           
     &     ,TRAG,TRANLG                                                 
! U_psi, then U (rotational wind)                                       
      COMPLEX :: UG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FUG(2*MMTy,NHEM,JG,NLx)                                
! V_chi, then V (divergent wind)                                        
      COMPLEX :: VG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FVG(2*MMTy,NHEM,JG,NLx)                                
! Temperature anomaly (T')                                              
      COMPLEX :: TG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: TNLG(2*MMTy,NHEM,JG,NLx)                               
! ln p* or p*                                                           
      COMPLEX :: PLG(2*MMTy,NHEM,JG)                                    
! Tracer term                                                           
      COMPLEX :: TRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                         
!                                                                       
      COMPLEX :: TRANLG(2*MMTy,NHEM,JG,NLx,NTRAC)                       
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB/ZG,DG,PJG,PMG,SPG,VPG,EG,UTG,VTG                    
     &     ,UTRAG,VTRAG                                                 
     &     ,CHIG,SFG,FVGT,FUGT                                          
! Vorticity (zeta)                                                      
      COMPLEX :: ZG(2*MMTy,NHEM,JG,NLx)                                 
! Divergence (d)                                                        
      COMPLEX :: DG(2*MMTy,NHEM,JG,NLx)                                 
! (1-mu^2) (d(ln p*)/d mu)                                              
      COMPLEX :: PJG(2*MMTy,NHEM,JG)                                    
! (d(ln p*)/d lambda)                                                   
      COMPLEX :: PMG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: SPG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: VPG(2*MMTy,NHEM,JG)                                    
! (U^2 + V^2)                                                           
      COMPLEX :: EG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: UTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: VTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: UTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: VTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: CHIG(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: SFG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: FVGT(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: FUGT(2*MMTy,NHEM,JG,NLx)                               
!-----------------------------------------------------                  
! Like the variables in SPECTR but the meridional wavenumbers           
! are complete instead of the levels.                                   
!-----------------------------------------------------                  
      COMMON/SPECTR2/Z2,D2,T2,SP2,SPA2,VP2,DTE2,TT2,DT2,ZT2             
     &     ,TRA2,TRAT2                                                  
! Vorticity coefficient                                                 
      COMPLEX :: Z2(NNT,MMTy,NHEM,NLx)                                  
! Divergence coefficient                                                
      COMPLEX :: D2(NNT,MMTy,NHEM,NLx)                                  
! Temperature coefficient                                               
      COMPLEX :: T2(NNT,MMTy,NHEM,NLx)                                  
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP2(NNT,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA2(NNT,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP2(NNT,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE2(NNT,MMTy,NHEM,NLx)                                
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT2(NNT,MMTy,NHEM,NLx)                                 
! Tracer coefficient                                                    
      COMPLEX :: TRA2(NNT,MMTy,NHEM,NLx,NTRAC)                          
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT2(NNT,MMTy,NHEM,NLx,NTRAC)                         
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: AW1256                                                    
      REAL :: AW3478                                                    
      INTEGER :: IP, IPM, MP                                            
      COMPLEX :: TEMP                                                   
! Loop variables                                                        
      INTEGER :: IHEM, IM, ITRAC, J, L, M, N, nT                        
!-----------------------------------------------------                  
! For a global run, sum and difference the complete Fourier             
! transforms at the northern and southern latitude rows to give         
! the even and odd contributions : E=(N+S)/2, O=(N-S)/2.                
! For Fourier fields symmetric about equator  : even precedes odd.      
! For Fourier fields asymmetric about equator : odd precedes even.      
!-----------------------------------------------------                  
      IF (NHEM.EQ.2) THEN                                               
         DO J=1,JG                                                      
            DO IM=1,2*MMTyy(iRow)                                       
               DO L=1,NLxx(iCol)                                        
                  !---------------------------------------------------  
                  ! Tracer tendencies                                   
                  !---------------------------------------------------  
                  DO ITRAC=1,NTRAC                                      
                     !------------------------------------------------  
                     ! main + d/dx part : symmetric.                    
                     !------------------------------------------------  
                     TEMP=TRANLG(IM,1,J,L,ITRAC)                        
                     TRANLG(IM,1,J,L,ITRAC)=                            
     &                    0.5*(TEMP+TRANLG(IM,NHEM,J,L,ITRAC))          
                     TRANLG(IM,NHEM,J,L,ITRAC)=                         
     &                    0.5*(TEMP-TRANLG(IM,NHEM,J,L,ITRAC))          
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Loop over all latitudes                                               
!-----------------------------------------------------                  
      DO J=1,JG                                                         
         !-----------------------------------------------------         
         ! Set up the appropriate Gaussian weight for the current       
         ! latitude, dependent on transform type.                       
         !-----------------------------------------------------         
         AW1256=AW(J,1)*CSSQ(J,1)                                       
         AW3478=-AW(J,1)                                                
         !-----------------------------------------------------         
         ! Calculate POLY array in a vector loop before the main        
         ! transforms, for the required Legendre Function types.        
         ! Both even and odd functions are required, irrespective of    
         ! NHEM.                                                        
         !-----------------------------------------------------         
         DO IHEM=1,2                                                    
            DO IM=1,MMTyy(iRow)                                         
               DO IP=1,NNT                                              
                  POLY(IP,IM,IHEM,1)=AW1256*ALP(IP,IM,IHEM,J)           
                  POLY(IP,IM,IHEM,2)=AW3478*DALP(IP,IM,IHEM,J)          
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Transform multi-level fields.                                
         ! Inner loop vectorisation is over total wavenumber, to access 
         ! spectral memory sequentially, avoiding skip distances being  
         ! multiple of 8 (which causes memory bank conflicts on Cray    
         ! vector machines).                                            
         !-----------------------------------------------------         
         DO L=1,NLxx(iCol)                                              
            DO nT=1,2                                                   
               DO IM=1,MMTyy(iRow)                                      
                  MP=(nT-1)*(2*(MMTyy(iRow)-IM)+1)+IM                   
                  DO IP=mTwin2(nT,IM)+1,mTwin2(nT+1,IM)                 
                     !------------------------------------------------  
                     ! Tracer tendency   : d/dy part        : type 4.   
                     !------------------------------------------------  
                     DO ITRAC=1,NTRAC                                   
                        TRAT2(IP,IM,1,L,ITRAC)=TRAT2(IP,IM,1,L,ITRAC)+  
     &                       POLY(IP,IM,1,1)*TRANLG(MP,1,J,L,ITRAC)     
                     ENDDO                                              
                     IF (NHEM.EQ.2) THEN                                
                        !---------------------------------------------  
                        ! Tracer tendency   : d/dy part       : type 4. 
                        !---------------------------------------------  
                        DO ITRAC=1,NTRAC                                
                           TRAT2(IP,IM,NHEM,L,ITRAC)=                   
     &                          TRAT2(IP,IM,NHEM,L,ITRAC)+              
     &                          POLY(IP,IM,2,1)*                        
     &                          TRANLG(MP,NHEM,J,L,ITRAC)               
                        ENDDO                                           
                    ENDIF                                               
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
                                                                        
      RETURN                                                            
      END                                                               
!=====================================================                  
! Perform all the indirect Legendre transforms for the adiabatic        
! part of the timestep at the current latitude (pair), in place         
! of separate calls to HEXP for individual transform types.             
!                                                                       
! An even/odd value of ITYPE denotes a spectral field of even/odd       
! symmetry.                                                             
!                                                                       
! Maximum vector efficiency is achieved by chaining all multi-level     
! transforms in one loop and by chaining all single-level transform     
! in a second loop.                                                     
!                                                                       
! All dummy argument arrays are declared complex.                       
! All array dimensions are parameters.                                  
! Multi-level arrays are 3-dimensional.                                 
!                                                                       
! Version for RSGUP3.                     Mike Blackburn,  12.01.95     
!=====================================================                  
      SUBROUTINE HEXPV                                                  
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Like the variables in SPECTR but the meridional wavenumbers           
! are complete instead of the levels.                                   
!-----------------------------------------------------                  
      COMMON/SPECTR2/Z2,D2,T2,SP2,SPA2,VP2,DTE2,TT2,DT2,ZT2             
     &     ,TRA2,TRAT2                                                  
! Vorticity coefficient                                                 
      COMPLEX :: Z2(NNT,MMTy,NHEM,NLx)                                  
! Divergence coefficient                                                
      COMPLEX :: D2(NNT,MMTy,NHEM,NLx)                                  
! Temperature coefficient                                               
      COMPLEX :: T2(NNT,MMTy,NHEM,NLx)                                  
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP2(NNT,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA2(NNT,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP2(NNT,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE2(NNT,MMTy,NHEM,NLx)                                
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT2(NNT,MMTy,NHEM,NLx)                                 
! Tracer coefficient                                                    
      COMPLEX :: TRA2(NNT,MMTy,NHEM,NLx,NTRAC)                          
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT2(NNT,MMTy,NHEM,NLx,NTRAC)                         
!-----------------------------------------------------                  
! Half transforms. The levels are divided across the columns of PEs;    
! the zonal wavenumbers are divided across the row of PEs; and on       
! ech PE the latitude are complete.                                     
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA/UG,FUG,VG,FVG,TG,TNLG,PLG                           
     &     ,TRAG,TRANLG                                                 
! U_psi, then U (rotational wind)                                       
      COMPLEX :: UG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FUG(2*MMTy,NHEM,JG,NLx)                                
! V_chi, then V (divergent wind)                                        
      COMPLEX :: VG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FVG(2*MMTy,NHEM,JG,NLx)                                
! Temperature anomaly (T')                                              
      COMPLEX :: TG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: TNLG(2*MMTy,NHEM,JG,NLx)                               
! ln p* or p*                                                           
      COMPLEX :: PLG(2*MMTy,NHEM,JG)                                    
! Tracer term                                                           
      COMPLEX :: TRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                         
!                                                                       
      COMPLEX :: TRANLG(2*MMTy,NHEM,JG,NLx,NTRAC)                       
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB/ZG,DG,PJG,PMG,SPG,VPG,EG,UTG,VTG                    
     &     ,UTRAG,VTRAG                                                 
     &     ,CHIG,SFG,FVGT,FUGT                                          
! Vorticity (zeta)                                                      
      COMPLEX :: ZG(2*MMTy,NHEM,JG,NLx)                                 
! Divergence (d)                                                        
      COMPLEX :: DG(2*MMTy,NHEM,JG,NLx)                                 
! (1-mu^2) (d(ln p*)/d mu)                                              
      COMPLEX :: PJG(2*MMTy,NHEM,JG)                                    
! (d(ln p*)/d lambda)                                                   
      COMPLEX :: PMG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: SPG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: VPG(2*MMTy,NHEM,JG)                                    
! (U^2 + V^2)                                                           
      COMPLEX :: EG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: UTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: VTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: UTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: VTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: CHIG(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: SFG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: FVGT(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: FUGT(2*MMTy,NHEM,JG,NLx)                               
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      COMPLEX :: TEMP                                                   
      INTEGER :: MP                                                     
! Loop variables                                                        
      INTEGER :: IM, IP, ITRAC, J, L, M, N, nT                          
!-----------------------------------------------------                  
! Transform multi-level fields to create even and odd contributions     
! to the Fourier coefficients at the northern hemisphere latitude.      
!-----------------------------------------------------                  
      DO L=1,NLxx(iCol)                                                 
         DO J=1,JG                                                      
            DO nT=1,2                                                   
               DO IM=1,MMTyy(iRow)                                      
                  MP=(nT-1)*(2*(MMTyy(iRow)-IM)+1)+IM                   
                  DO IP=mTwin2(nT,IM)+1,mTwin2(nT+1,IM)                 
                     !------------------------------------------------  
                     ! Velocity potential    : type 6.                  
                     !------------------------------------------------  
                     CHIG(MP,1,J,L)=CHIG(MP,1,J,L) +                    
     &                    RLP(IP,IM,1,J)*D2(IP,IM,1,L)                  
                     !------------------------------------------------  
                     ! Streamfunction        : type 5.                  
                     !------------------------------------------------  
                     SFG (MP,1,J,L)=SFG (MP,1,J,L) +                    
     &                    RLP(IP,IM,2,J)*Z2(IP,IM,1,L)                  
                     !------------------------------------------------  
                     ! Zonal (rot) wind      : type 7.                  
                     !------------------------------------------------  
                     UG  (MP,1,J,L)=UG  (MP,1,J,L) +                    
     &                    RDLP(IP,IM,2,J)*Z2(IP,IM,1,L)                 
                     !------------------------------------------------  
                     ! Merid (div) wind      : type 8.                  
                     !------------------------------------------------  
                     VG  (MP,1,J,L)=VG  (MP,1,J,L) +                    
     &                    RDLP(IP,IM,1,J)*D2(IP,IM,1,L)                 
                     !------------------------------------------------  
                     ! (Relative) vorticity  : type 1.                  
                     !------------------------------------------------  
                     ZG  (MP,1,J,L)=ZG  (MP,1,J,L) +                    
     &                    ALP(IP,IM,2,J)*Z2(IP,IM,1,L)                  
                     !------------------------------------------------  
                     ! Divergence            : type 2.                  
                     !------------------------------------------------  
                     DG  (MP,1,J,L)=DG  (MP,1,J,L) +                    
     &                    ALP(IP,IM,1,J)*D2(IP,IM,1,L)                  
                     !------------------------------------------------  
                     ! Temperature           : type 2.                  
                     !------------------------------------------------  
                     TG  (MP,1,J,L)=TG  (MP,1,J,L) +                    
     &                    ALP(IP,IM,1,J)*T2(IP,IM,1,L)                  
                     !------------------------------------------------  
                     ! Tracers               : type 2.                  
                     !------------------------------------------------  
                     DO ITRAC=1,NTRAC                                   
                        TRAG(MP,1,J,L,ITRAC)=TRAG(MP,1,J,L,ITRAC)+      
     &                       ALP(IP,IM,1,J)*TRA2(IP,IM,1,L,ITRAC)       
                     ENDDO                                              
                     IF (NHEM.EQ.2) THEN                                
                        !---------------------------------------------  
                        ! Velocity potential    : type 6.               
                        !---------------------------------------------  
                        CHIG(MP,NHEM,J,L)=CHIG(MP,NHEM,J,L) +           
     &                       RLP(IP,IM,NHEM,J)*D2(IP,IM,NHEM,L)         
                        !---------------------------------------------  
                        ! Streamfunction        : type 5.               
                        !---------------------------------------------  
                        SFG (MP,NHEM,J,L)=SFG (MP,NHEM,J,L) +           
     &                       RLP(IP,IM,1,J)*Z2(IP,IM,NHEM,L)            
                        !---------------------------------------------  
                        ! Zonal (rot) wind      : type 7.               
                        !---------------------------------------------  
                        UG  (MP,NHEM,J,L)=UG  (MP,NHEM,J,L) +           
     &                       RDLP(IP,IM,1,J)*Z2(IP,IM,NHEM,L)           
                        !---------------------------------------------  
                        ! Merid (div) wind      : type 8.               
                        !---------------------------------------------  
                        VG  (MP,NHEM,J,L)=VG  (MP,NHEM,J,L) +           
     &                       RDLP(IP,IM,NHEM,J)*D2(IP,IM,NHEM,L)        
                        !---------------------------------------------  
                        ! (Relative) vorticity  : type 1.               
                        !---------------------------------------------  
                        ZG  (MP,NHEM,J,L)=ZG  (MP,NHEM,J,L) +           
     &                       ALP(IP,IM,1,J)*Z2(IP,IM,NHEM,L)            
                        !---------------------------------------------  
                        ! Divergence            : type 2.               
                        !---------------------------------------------  
                        DG  (MP,NHEM,J,L)=DG  (MP,NHEM,J,L) +           
     &                       ALP(IP,IM,NHEM,J)*D2(IP,IM,NHEM,L)         
                        !---------------------------------------------  
                        ! Temperature           : type 2.               
                        !---------------------------------------------  
                        TG  (MP,NHEM,J,L)=TG  (MP,NHEM,J,L) +           
     &                       ALP(IP,IM,NHEM,J)*T2(IP,IM,NHEM,L)         
                        !---------------------------------------------  
                        ! Tracers           : type 2.                   
                        !---------------------------------------------  
                        DO ITRAC=1,NTRAC                                
                           TRAG(MP,NHEM,J,L,ITRAC)=                     
     &                          TRAG(MP,NHEM,J,L,ITRAC)+ALP(IP,IM,2,J)  
     &                          *TRA2(IP,IM,NHEM,L,ITRAC)               
                        ENDDO                                           
                     ENDIF                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Transform single-level fields to create even and odd contribution     
! to the Fourier coefficients at the northern hemisphere latitude.      
! Vectorisation is over total wavenumber for each zonal wavenumber.     
!-----------------------------------------------------                  
      IF (iCol.EQ.1) THEN                                               
         DO J=1,JG                                                      
            DO nT=1,2                                                   
               DO IM=1,MMTyy(iRow)                                      
                  MP=(nT-1)*(2*(MMTyy(iRow)-IM)+1)+IM                   
                  DO IP=mTwin2(nT,IM)+1,mTwin2(nT+1,IM)                 
                     !------------------------------------------------  
                     ! Log (surface pressure)    : type 2.              
                     !------------------------------------------------  
                     PLG(MP,1,J)=PLG(MP,1,J) +                          
     &                    ALP(IP,IM,1,J)*SP2(IP,IM,1)                   
                     !------------------------------------------------  
                     ! Merid gradient of ln (ps) : type 4.              
                     !------------------------------------------------  
                     PJG(MP,1,J)=PJG(MP,1,J) +                          
     &                    DALP(IP,IM,1,J)*SP2(IP,IM,1)                  
                     IF (NHEM.EQ.2) THEN                                
                        !---------------------------------------------  
                        ! Log (surface pressure)    : type 2.           
                        !---------------------------------------------  
                        PLG(MP,NHEM,J)=PLG(MP,NHEM,J) +                 
     &                       ALP(IP,IM,NHEM,J)*SP2(IP,IM,NHEM)          
                        !---------------------------------------------  
                        ! Merid gradient of ln (ps) : type 4.           
                        !---------------------------------------------  
                        PJG(MP,NHEM,J)=PJG(MP,NHEM,J) +                 
     &                       DALP(IP,IM,NHEM,J)*SP2(IP,IM,NHEM)         
                     ENDIF                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! For a global run, sum and difference even and odd contributions       
! to give the complete Fourier transforms at the northern and           
! southern latitude rows: N=E+O, S=E-O.                                 
! For symmetric Fourier fields, even (IM,1) precedes odd (IM,2).        
! For asymmetric Fourier fields, odd (IM,1) precedes even (IM,2).       
!-----------------------------------------------------                  
      IF (NHEM.EQ.2) THEN                                               
         DO L=1,NLxx(iCol)                                              
            DO J=1,JG                                                   
               DO MP=1,2*MMTyy(iRow)                                    
                  !---------------------------------------------------  
                  ! Velocity potential : symmetric.                     
                  !---------------------------------------------------  
                  TEMP=CHIG(MP,1,J,L)                                   
                  CHIG(MP,1,J,L)=TEMP+CHIG(MP,NHEM,J,L)                 
                  CHIG(MP,NHEM,J,L)=TEMP-CHIG(MP,NHEM,J,L)              
                  !---------------------------------------------------  
                  ! Streamfunction : anti-symmetric.                    
                  !---------------------------------------------------  
                  TEMP=SFG(MP,1,J,L)                                    
                  SFG(MP,1,J,L)=SFG(MP,NHEM,J,L)+TEMP                   
                  SFG(MP,NHEM,J,L)=SFG(MP,NHEM,J,L)-TEMP                
                  !---------------------------------------------------  
                  ! Zonal (rotational) wind : symmetric.                
                  !---------------------------------------------------  
                  TEMP=UG(MP,1,J,L)                                     
                  UG(MP,1,J,L)=TEMP+UG(MP,NHEM,J,L)                     
                  UG(MP,NHEM,J,L)=TEMP-UG(MP,NHEM,J,L)                  
                  !---------------------------------------------------  
                  ! Meridional (divergent) wind : anti-symmetric.       
                  !---------------------------------------------------  
                  TEMP=VG(MP,1,J,L)                                     
                  VG(MP,1,J,L)=VG(MP,NHEM,J,L)+TEMP                     
                  VG(MP,NHEM,J,L)=VG(MP,NHEM,J,L)-TEMP                  
                  !---------------------------------------------------  
                  ! Vorticity : anti-symmetric.                         
                  !---------------------------------------------------  
                  TEMP=ZG(MP,1,J,L)                                     
                  ZG(MP,1,J,L)=ZG(MP,NHEM,J,L)+TEMP                     
                  ZG(MP,NHEM,J,L)=ZG(MP,NHEM,J,L)-TEMP                  
                  !---------------------------------------------------  
                  ! Divergence : symmetric.                             
                  !---------------------------------------------------  
                  TEMP=DG(MP,1,J,L)                                     
                  DG(MP,1,J,L)=TEMP+DG(MP,NHEM,J,L)                     
                  DG(MP,NHEM,J,L)=TEMP-DG(MP,NHEM,J,L)                  
                  !---------------------------------------------------  
                  ! Temperature : symmetric.                            
                  !---------------------------------------------------  
                  TEMP=TG(MP,1,J,L)                                     
                  TG(MP,1,J,L)=TEMP+TG(MP,NHEM,J,L)                     
                  TG(MP,NHEM,J,L)=TEMP-TG(MP,NHEM,J,L)                  
                  !---------------------------------------------------  
                  ! Tracers : symmetric.                                
                  !---------------------------------------------------  
                  DO ITRAC=1,NTRAC                                      
                     TEMP=TRAG(MP,1,J,L,ITRAC)                          
                     TRAG(MP,1,J,L,ITRAC)=TEMP+TRAG(MP,NHEM,J,L,ITRAC)  
                     TRAG(MP,NHEM,J,L,ITRAC)=                           
     &                    TEMP-TRAG(MP,NHEM,J,L,ITRAC)                  
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
                                                                        
         IF (iCol.EQ.1) THEN                                            
            DO J=1,JG                                                   
               DO MP=1,2*MMTyy(iRow)                                    
                  !---------------------------------------------------  
                  ! Log (surface pressure) : symmetric.                 
                  !---------------------------------------------------  
                  TEMP=PLG(MP,1,J)                                      
                  PLG(MP,1,J)=TEMP+PLG(MP,NHEM,J)                       
                  PLG(MP,NHEM,J)=TEMP-PLG(MP,NHEM,J)                    
                  !---------------------------------------------------  
                  ! Meridional gradient of ln(ps) : anti-symmetric.     
                  !---------------------------------------------------  
                  TEMP=PJG(MP,1,J)                                      
                  PJG(MP,1,J)=PJG(MP,NHEM,J)+TEMP                       
                  PJG(MP,NHEM,J)=PJG(MP,NHEM,J)-TEMP                    
               ENDDO                                                    
            ENDDO                                                       
         ENDIF                                                          
      ENDIF                                                             
                                                                        
      RETURN                                                            
      END                                                               
!=====================================================                  
! Very similar to HEXPV but used from subroutine ICTRAC                 
!=====================================================                  
      SUBROUTINE HEXPVDT                                                
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Like the variables in SPECTR but the meridional wavenumbers           
! are complete instead of the levels.                                   
!-----------------------------------------------------                  
      COMMON/SPECTR2/Z2,D2,T2,SP2,SPA2,VP2,DTE2,TT2,DT2,ZT2             
     &     ,TRA2,TRAT2                                                  
! Vorticity coefficient                                                 
      COMPLEX :: Z2(NNT,MMTy,NHEM,NLx)                                  
! Divergence coefficient                                                
      COMPLEX :: D2(NNT,MMTy,NHEM,NLx)                                  
! Temperature coefficient                                               
      COMPLEX :: T2(NNT,MMTy,NHEM,NLx)                                  
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP2(NNT,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA2(NNT,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP2(NNT,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE2(NNT,MMTy,NHEM,NLx)                                
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT2(NNT,MMTy,NHEM,NLx)                                 
! Tracer coefficient                                                    
      COMPLEX :: TRA2(NNT,MMTy,NHEM,NLx,NTRAC)                          
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT2(NNT,MMTy,NHEM,NLx,NTRAC)                         
!-----------------------------------------------------                  
! Half transforms. The levels are divided across the columns of PEs;    
! the zonal wavenumbers are divided across the row of PEs; and on       
! ech PE the latitude are complete.                                     
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA/UG,FUG,VG,FVG,TG,TNLG,PLG                           
     &     ,TRAG,TRANLG                                                 
! U_psi, then U (rotational wind)                                       
      COMPLEX :: UG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FUG(2*MMTy,NHEM,JG,NLx)                                
! V_chi, then V (divergent wind)                                        
      COMPLEX :: VG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FVG(2*MMTy,NHEM,JG,NLx)                                
! Temperature anomaly (T')                                              
      COMPLEX :: TG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: TNLG(2*MMTy,NHEM,JG,NLx)                               
! ln p* or p*                                                           
      COMPLEX :: PLG(2*MMTy,NHEM,JG)                                    
! Tracer term                                                           
      COMPLEX :: TRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                         
!                                                                       
      COMPLEX :: TRANLG(2*MMTy,NHEM,JG,NLx,NTRAC)                       
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB/ZG,DG,PJG,PMG,SPG,VPG,EG,UTG,VTG                    
     &     ,UTRAG,VTRAG                                                 
     &     ,CHIG,SFG,FVGT,FUGT                                          
! Vorticity (zeta)                                                      
      COMPLEX :: ZG(2*MMTy,NHEM,JG,NLx)                                 
! Divergence (d)                                                        
      COMPLEX :: DG(2*MMTy,NHEM,JG,NLx)                                 
! (1-mu^2) (d(ln p*)/d mu)                                              
      COMPLEX :: PJG(2*MMTy,NHEM,JG)                                    
! (d(ln p*)/d lambda)                                                   
      COMPLEX :: PMG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: SPG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: VPG(2*MMTy,NHEM,JG)                                    
! (U^2 + V^2)                                                           
      COMPLEX :: EG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: UTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: VTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: UTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: VTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: CHIG(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: SFG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: FVGT(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: FUGT(2*MMTy,NHEM,JG,NLx)                               
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      COMPLEX :: TEMP                                                   
      INTEGER :: MP                                                     
! Loop variables                                                        
      INTEGER :: IM, IP, ITRAC, J, L, M, N, nT                          
!-----------------------------------------------------                  
! Transform multi-level fields to create even and odd contributions     
! to the Fourier coefficients at the northern hemisphere latitude.      
!-----------------------------------------------------                  
      DO L=1,NLxx(iCol)                                                 
         DO J=1,JG                                                      
            DO nT=1,2                                                   
               DO IM=1,MMTyy(iRow)                                      
                  MP=(nT-1)*(2*(MMTyy(iRow)-IM)+1)+IM                   
                  DO IP=mTwin2(nT,IM)+1,mTwin2(nT+1,IM)                 
                     !------------------------------------------------  
                     ! X-derivative of temperature field: type 2.       
                     !------------------------------------------------  
                     CHIG  (MP,1,J,L)=CHIG  (MP,1,J,L) +                
     &                    ALP(IP,IM,1,J)*T2(IP,IM,1,L)                  
                     !------------------------------------------------  
                     ! Y-derivative of temperature field: type 4.       
                     !------------------------------------------------  
                     SFG  (MP,1,J,L)=SFG  (MP,1,J,L) +                  
     &                    DALP(IP,IM,1,J)*T2(IP,IM,1,L)                 
                     IF (NHEM.EQ.2) THEN                                
                        !---------------------------------------------  
                        ! X-derivative of temperature field: type 2.    
                        !---------------------------------------------  
                        CHIG  (MP,NHEM,J,L)=CHIG  (MP,NHEM,J,L) +       
     &                       ALP(IP,IM,NHEM,J)*T2(IP,IM,NHEM,L)         
                        !---------------------------------------------  
                        ! Y-derivative of temperature field: type 4.    
                        !---------------------------------------------  
                        SFG  (MP,NHEM,J,L)=SFG  (MP,NHEM,J,L) +         
     &                       DALP(IP,IM,NHEM,J)*T2(IP,IM,NHEM,L)        
                     ENDIF                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! For a global run, sum and difference even and odd contributions       
! to give the complete Fourier transforms at the northern and           
! southern latitude rows: N=E+O, S=E-O.                                 
! For symmetric Fourier fields, even (IM,1) precedes odd (IM,2).        
! For asymmetric Fourier fields, odd (IM,1) precedes even (IM,2).       
!-----------------------------------------------------                  
      IF (NHEM.EQ.2) THEN                                               
         DO L=1,NLxx(iCol)                                              
            DO J=1,JG                                                   
               DO MP=1,2*MMTyy(iRow)                                    
                  !---------------------------------------------------  
                  ! X-derivative of temperature field: symmetric        
                  !---------------------------------------------------  
                  TEMP=CHIG(MP,1,J,L)                                   
                  CHIG(MP,1,J,L)=TEMP+CHIG(MP,NHEM,J,L)                 
                  CHIG(MP,NHEM,J,L)=TEMP-CHIG(MP,NHEM,J,L)              
                  !---------------------------------------------------  
                  ! Y-derivative of temperature field: symmetric        
                  !---------------------------------------------------  
                  TEMP=SFG(MP,1,J,L)                                    
                  SFG(MP,1,J,L)=TEMP+SFG(MP,NHEM,J,L)                   
                  SFG(MP,NHEM,J,L)=TEMP-SFG(MP,NHEM,J,L)                
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
                                                                        
      RETURN                                                            
      END                                                               
!=====================================================                  
! This subroutine takes in a profile from climatology (15 levels)       
! and interpolates to model vertical grid (nl.gt.15), passing back      
! profile fmod.                                                         
! N.B. The fclim profile must correspond to the climatology pressure    
! levels given in this routine (ocliml(15)).                            
!  SMR 03-06-97                                                         
!=====================================================                  
      subroutine interpf(fclim,fmod,ps)                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Passed variables                                                      
!-----------------------------------------------------                  
! Climatology profile. Must correspond to pressure levels               
! given below!                                                          
      real :: fclim(15)                                                 
! Model (interpolated) profile.                                         
      real :: fmod(nl)                                                  
! Surface pressure.                                                     
      REAL :: ps                                                        
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
! Model level counter.                                                  
      integer :: l                                                      
! climatology level counter.                                            
      integer :: lo                                                     
! pressure in mb of current level being interpolated.                   
      real :: pressu                                                    
! ozone climatology levels in mb.                                       
      real :: ocliml(15)                                                
!-----------------------------------------------------                  
! Store data for oclim1                                                 
!-----------------------------------------------------                  
      save                                                              
      data ocliml/1.0,3.0,10.0,30.0,50.0,70.0,100.0,150.0,200.0,        
     &              250.0,300.0,400.0,500.0,700.0,850.0/                
!-----------------------------------------------------                  
! Start of interpolation.                                               
! Loop over model levels.                                               
!-----------------------------------------------------                  
      do l=1,nl                  ! Loop over model levels.              
         !-----------------------------------------------------         
         ! Pressure in mb of model level where ozone required.          
         !-----------------------------------------------------         
         pressu=sigma(l)*ps/100.0                                       
         !-----------------------------------------------------         
         ! Loop over climatology levels.                                
         !-----------------------------------------------------         
         do lo=1,14                                                     
            !-----------------------------------------------------      
            ! Top                                                       
            !-----------------------------------------------------      
            if (pressu.lt.1.0) then                                     
               fmod(l)=fclim(1)                                         
            elseif (pressu.eq.ocliml(lo)) then                          
               !-----------------------------------------------------   
               ! If on a climatology level.                             
               !-----------------------------------------------------   
               fmod(l)=fclim(lo)                                        
            elseif (pressu.gt.ocliml(lo).and.pressu.lt.ocliml(lo+1))    
     &        then                                                      
               !-----------------------------------------------------   
               ! Need to interpolate                                    
               !-----------------------------------------------------   
               fmod(l)=exp(                                             
     &              log(fclim(lo)) +                                    
     &              (log(fclim(lo+1)) -                                 
     &              log(fclim(lo)))*                                    
     &              (log(pressu) - log(ocliml(lo))) /                   
     &              (log(ocliml(lo+1)) - log(ocliml(lo))))              
            elseif (pressu.ge.850.0) then                               
               !-----------------------------------------------------   
               ! Bottom                                                 
               !-----------------------------------------------------   
               fmod(l)=fclim(15)                                        
            endif                                                       
         ENDDO                  ! End of loop over climatology.         
      ENDDO                     ! End of loop over model levels.        
c                                                                       
c --------------------------- End of interpolation                      
c                                                                       
      return                                                            
      end                                                               
!=====================================================                  
! Calculates legendre polynomials (ALPJ) and their derivatives          
! (DALPJ) at the JL'th latitude (JL is in LEGAU) using                  
! recurrence relationships.                                             
!=====================================================                  
      SUBROUTINE LGNDRE(NN,MM,MOCT,ALPJ,DALPJ,NWJ2,SIJ,CSJ)             
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Passed variables                                                      
!-----------------------------------------------------                  
      INTEGER :: NN                                                     
      INTEGER :: MM                                                     
      INTEGER :: MOCT                                                   
! Legendre polynomials                                                  
      REAL :: ALPJ(2*NWJ2)                                              
! Derivatives of Legendre polynomials                                   
      REAL :: DALPJ(2*NWJ2)                                             
      INTEGER :: NWJ2                                                   
      REAL :: SIJ                                                       
      REAL :: CSJ                                                       
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: F1M, F2M                                                  
      REAL :: E1, E2                                                    
      REAL :: ANM2                                                      
      INTEGER :: JFM, LM, K                                             
! Loop variables                                                        
      INTEGER :: I, M, N                                                
!-----------------------------------------------------                  
! Set P(0,0) and P(0,1)                                                 
!-----------------------------------------------------                  
      ALPJ(1)=SQRT(.5)                                                  
      F1M=SQRT(1.5)                                                     
      ALPJ(2)=F1M*SIJ                                                   
      DALPJ(1)=0.                                                       
!-----------------------------------------------------                  
! Loop over wavenumbers                                                 
!-----------------------------------------------------                  
      LM=0                                                              
      DO M=0,MM-1,MOCT                                                  
         LM=LM+2                                                        
         IF (M.GT.0) THEN                                               
            DO I=1-MOCT,0                                               
               F2M=-F1M*CSJ/SQRT(2.0*(M+I))                             
               F1M=F2M*SQRT(2.0*(M+I)+3.)                               
            ENDDO                                                       
            E2=SQRT(2.0*M+3.)                                           
            ALPJ(LM-1)=F2M                                              
            ALPJ(LM)=F1M*SIJ                                            
            DALPJ(LM-1)=-M*ALPJ(LM)/E2                                  
         ENDIF                                                          
         JFM=NN-MOD(NN-M+1,2)                                           
         IF (JFM.GE.(M+2)) THEN                                         
            K=LM-M-1                                                    
            !-----------------------------------------------------      
            ! Loop over degree N                                        
            !-----------------------------------------------------      
            DO N=M+2,JFM                                                
               ANM2=1.0*(N-1)*(N-1)                                     
               E1=SQRT((ANM2-1.0*M*M)/(4.*ANM2-1.))                     
               E2=SQRT((4.*N*N-1.)/(1.0*N*N-1.0*M*M))                   
               ALPJ(K+N)=E2*(SIJ*ALPJ(K+N-1)-E1*ALPJ(K+N-2))            
               DALPJ(K+N-1)=(1.-N)*ALPJ(K+N)/E2+N*E1*ALPJ(K+N-2)        
            ENDDO                                                       
            LM=LM+JFM-M-1                                               
         ENDIF                                                          
         DALPJ(LM)=-(N-1.)*SIJ*ALPJ(LM)+(N+N-1.0)*ALPJ(LM-1)/E2         
       ENDDO                                                            
      RETURN                                                            
      END                                                               
!=====================================================                  
! GRID SCALE CONDENSATION TO SATURATION FOR PRESENT LATITUDE.           
! The cloud diagnostic scheme is based on Slingo (1987).                
! Cloud fractions computed for Low (sigma>0.7),                         
! mid(0.7>sigma>0.35) and  high (0.35>sigma>0.12) cloud                 
! types. Shallow convective and deep convective cloud fractions         
! computed in CUBM and LBADJ respectively.                              
!=====================================================                  
      SUBROUTINE LSCRN(JH)                                              
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Grid variables for DGRMLT loop                                        
!-----------------------------------------------------                  
      COMMON/GRIDPC/ASSBL,ASHBL,ASLBL,ARRCR,ARRLR                       
     &     ,TTVD,QTVD,TYBL,TXBL,TTRD,TTLR,TTCR,UTVD,VTVD                
     &     ,QTLR,QTCR,SSBL,SHBL,SLBL,RRCR,RRLR                          
     &     ,ARFLUX,ASFLD,ACLD,RFLUX,SFLD,CLD                            
      REAL :: ASSBL(MGx,NHEM,JGy)                                       
      REAL :: ASHBL(MGx,NHEM,JGy)                                       
      REAL :: ASLBL(MGx,NHEM,JGy)                                       
      REAL :: ARRCR(MGx,NHEM,JGy)                                       
      REAL :: ARRLR(MGx,NHEM,JGy)                                       
      REAL :: TTVD(MGx,NHEM,NL)                                         
      REAL :: QTVD(MGx,NHEM,NL)                                         
      REAL :: TYBL(MGx,NHEM)                                            
      REAL :: TXBL(MGx,NHEM)                                            
      REAL :: TTRD(MGx,NHEM,NL)                                         
      REAL :: TTLR(MGx,NHEM,NL)                                         
      REAL :: TTCR(MGx,NHEM,NL)                                         
      REAL :: UTVD(MGx,NHEM,NL)                                         
      REAL :: VTVD(MGx,NHEM,NL)                                         
      REAL :: QTLR(MGx,NHEM,NL)                                         
      REAL :: QTCR(MGx,NHEM,NL)                                         
      REAL :: SSBL(MGx,NHEM,JGy)                                        
      REAL :: SHBL(MGx,NHEM,JGy)                                        
      REAL :: SLBL(MGx,NHEM,JGy)                                        
      REAL :: RRCR(MGx,NHEM,JGy)                                        
      REAL :: RRLR(MGx,NHEM,JGy)                                        
      REAL :: arflux(MGx,NHEM,JGy,6)                                    
      REAL :: asfld(MGx,NHEM,JGy,6)                                     
      REAL :: acld(MGx,NHEM,JGy,4)                                      
      REAL :: rflux(MGx,NHEM,JGy,6)                                     
      REAL :: sfld(MGx,NHEM,JGy,6)                                      
      REAL :: cld(MGx,NHEM,JGy,4)                                       
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Radiation variables                                                   
!-----------------------------------------------------                  
      COMMON/CPIERS/ICFLAG,CFRAC,PNET,SNET,RRFLUX,TTSW,TTLW             
      INTEGER :: ICFLAG(MGx,NHEM,5,2)                                   
      REAL :: CFRAC(MGx,NHEM,5)                                         
      REAL :: PNET(MGx,NHEM,JGy)                                        
      REAL :: SNET(MGx,NHEM,JGy)                                        
      REAL :: RRFLUX(MGx,NHEM,JGy,6)                                    
      REAL :: TTSW(MGx,NHEM,NL)                                         
      REAL :: TTLW(MGx,NHEM,NL)                                         
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed arguments                                                      
!-----------------------------------------------------                  
! Index for latitudes                                                   
      INTEGER :: JH                                                     
!-----------------------------------------------------                  
! External function                                                     
!-----------------------------------------------------                  
      REAL, EXTERNAL :: PQSAT                                           
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: ESCON                                                     
      REAL :: QS                                                        
      REAL :: RH                                                        
      REAL :: CFR                                                       
      REAL :: QEXS                                                      
      REAL :: CTQUSE                                                    
      REAL :: QEX                                                       
      REAL :: DQDT(NL)                                                  
      REAL :: CONVMULT(NL)                                              
      REAL :: SDOTP                                                     
      REAL :: DTHDP1, DTHDP2                                            
! Loop variables                                                        
      INTEGER :: I, IHEM, L                                             
!-----------------------------------------------------                  
! LAYER CLOUD , High, middle, low.                                      
! Picks level (within low, mid, high) with largest RH                   
! and puts cloud there                                                  
!-----------------------------------------------------                  
      DO IHEM=1,NHEM                                                    
         DO I=1,MGxx(iCol)                                              
            ESCON=1./PLG3(I,IHEM,JH)                                    
C Calculate d theta/dP at low levels                                    
                                                                        
            DTHDP1 = ((TG3(I,IHEM,JH,NL)/SKAP(NL))-                     
     &               (TG3(I,IHEM,JH,NL-1)/SKAP(NL-1)))/                 
     &               (P0*PLG3(I,IHEM,JH)*(SIGMA(NL)-SIGMA(NL-1)))       
                                                                        
            DTHDP2 = ((TSTAR(I,IHEM,JH))-                               
     &               (TG3(I,IHEM,JH,NL)/SKAP(NL)))/                     
     &               (P0*PLG3(I,IHEM,JH)*(1.0-SIGMA(NL)))               
C Dimensionalise by CT and turn Pa into hPa                             
                                                                        
            DTHDP1 = DTHDP1*752.015*1.0E2                               
            DTHDP2 = DTHDP2*752.015*1.0E2                               
            DO L=1,NL                                                   
              CONVMULT(L)=1.0                                           
            END DO                                                      
C Check for deep convective cloud                                       
            IF(CFRAC(I,IHEM,5).GT.1.0E-8)THEN                           
              DO L=ICFLAG(I,IHEM,5,2),ICFLAG(I,IHEM,5,1)                
                CONVMULT(L)=0.0                                         
              END DO                                                    
            ENDIF                                                       
C Check for shallow convective cloud                                    
            IF(CFRAC(I,IHEM,4).GT.1.0E-8)THEN                           
              DO L=ICFLAG(I,IHEM,4,2),ICFLAG(I,IHEM,4,1)                
                CONVMULT(L)=0.0                                         
              END DO                                                    
            ENDIF                                                       
            DO L=1,NL                                                   
               QS=ESCON*PQSAT(TG3(I,IHEM,JH,L))/SIGMA(L)                
               !-----------------------------------------------------   
               ! Relative humidity                                      
               !-----------------------------------------------------   
               RH=TRAG3(I,IHEM,JH,L,1)/QS ! relative humidity           
               IF (RH.GT.0.8) THEN                                      
                  !---------------------------------------------------  
                  ! Cloud fraction mid/low/high cloud                   
                  !---------------------------------------------------  
                  CFR=((RH-0.8)/0.2)**2                                 
                  IF (CFR.GT.1.0) CFR=1.0                               
                  !---------------------------------------------------  
                  ! High cloud                                          
                  !---------------------------------------------------  
                  IF (SIGMA(L).GT.0.12.AND.SIGMA(L).LE.0.35) THEN       
                     IF (CFR.GT.CFRAC(I,IHEM,3)) THEN                   
                        ICFLAG(I,IHEM,3,1)=L                            
                        ICFLAG(I,IHEM,3,2)=L                            
                        CFRAC(I,IHEM,3)=CFR                             
                     ENDIF                                              
                  ENDIF                                                 
                  !---------------------------------------------------  
                  ! Mid cloud                                           
                  !---------------------------------------------------  
                  IF (SIGMA(L).GT.0.35.AND.SIGMA(L).LE.0.70) THEN       
                     IF (CFR.GT.CFRAC(I,IHEM,2)) THEN                   
                        ICFLAG(I,IHEM,2,1)=L                            
                        ICFLAG(I,IHEM,2,2)=L                            
                        CFRAC(I,IHEM,2)=CFR*CONVMULT(L)                 
                     ENDIF                                              
                  ENDIF                                                 
                  !---------------------------------------------------  
                  ! Low cloud not in level nl                           
                  !---------------------------------------------------  
                  IF (SIGMA(L).GT.0.70.AND.L.LT.NL) THEN                
                     IF (CFR.GT.CFRAC(I,IHEM,1)) THEN                   
                        ICFLAG(I,IHEM,1,1)=L                            
                        ICFLAG(I,IHEM,1,2)=L                            
                        CFRAC(I,IHEM,1)=CFR*CONVMULT(L)                 
                     ENDIF                                              
                  ENDIF                                                 
               ENDIF                                                    
               QEXS=TRAG3(I,IHEM,JH,L,1)-QS                             
               !-----------------------------------------------------   
               ! Choose appropriate value of latent heat based on       
               ! ground T ie, we assume it snows if the ground is       
               ! colder than 273.15K                                    
               !-----------------------------------------------------   
               if (tstar(i,ihem,jh).gt.0.363224029) then                
                  ctquse=ctq                                            
               else                                                     
                  ctquse=ctqi                                           
               endif                                                    
               QEX=QEXS/(1.0+CTQUSE*QS*ESCONB/(TG3(I,IHEM,JH,L)*        
     &              TG3(I,IHEM,JH,L)))                                  
               IF (QEXS.LT.0.0) THEN                                    
                  QEX=0.0                                               
               END IF                                                   
               IF (QEXS.GE.0.0) THEN                                    
                  CTLR(L,IHEM)=CTLR(L,IHEM)+1.0                         
               END IF                                                   
               DQDT(L)=-QEX/DELT2C                                      
               QTLR(I,IHEM,L)=QTLR(I,IHEM,L)+DQDT(L)                    
               TTLR(I,IHEM,L)=-CTQUSE*DQDT(L)                           
               TRAG3(I,IHEM,JH,L,1)=TRAG3(I,IHEM,JH,L,1)-QEX            
               TG3(I,IHEM,JH,L)=TG3(I,IHEM,JH,L)+CTQUSE*QEX             
            ENDDO                                                       
C Marine stratocumulus conditions (for cloud in layer NL-1)             
C Cloud top dth/dp < -0.03 K/hPa, near-surface dth/dp > -0.01 K/hPa     
C no low cloud already diagnosed and ocean gridpoint                    
C From Kawai and Inoue (SOLAS) 2006 with changed threshold              
C due to relatively coarse IGCM vertical resolution                     
                                                                        
              IF ((SVEGE(I,IHEM,JH).EQ.1).AND.DTHDP2 .gt. -0.01         
     &        .AND.(CFRAC(I,IHEM,1).LT.1.0E-5))                         
     &        THEN                                                      
                ICFLAG(I,IHEM,1,1)=NL-1                                 
                ICFLAG(I,IHEM,1,2)=NL-1                                 
                CFRAC(I,IHEM,1)=MAX(MIN(-12.0*(DTHDP1+0.03),1.0),0.0)   
              ENDIF                                                     
            !-----------------------------------------------------      
            ! Dot product                                               
            !-----------------------------------------------------      
            SDOTP=0.0                                                   
            DO L=1,NL                                                   
               SDOTP=SDOTP+DQDT(L)*CLR(L)                               
            ENDDO                                                       
            RRLR(I,IHEM,JH)=-SDOTP*PLG3(I,IHEM,JH)                      
         ENDDO                                                          
      ENDDO                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! Direct Legendre transform for the adiabatic part of the timestep.     
! Transforms from Fourier to spectral space at the current latitude     
! (pair).  In a global run the input arrays are complete (even+odd)     
! Fourier coefficients at the northern & southern hemisphere rows.      
!                                                                       
! Version for RSGUP3.                     Mike Blackburn,  12.01.95     
! ANSI work arrays for HANAL,SPDEL2.      Mike Blackburn,  04.09.96     
!=====================================================                  
      SUBROUTINE LTD                                                    
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Half transforms. The levels are divided across the columns of PEs;    
! the zonal wavenumbers are divided across the row of PEs; and on       
! ech PE the latitude are complete.                                     
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA/UG,FUG,VG,FVG,TG,TNLG,PLG                           
     &     ,TRAG,TRANLG                                                 
! U_psi, then U (rotational wind)                                       
      COMPLEX :: UG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FUG(2*MMTy,NHEM,JG,NLx)                                
! V_chi, then V (divergent wind)                                        
      COMPLEX :: VG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FVG(2*MMTy,NHEM,JG,NLx)                                
! Temperature anomaly (T')                                              
      COMPLEX :: TG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: TNLG(2*MMTy,NHEM,JG,NLx)                               
! ln p* or p*                                                           
      COMPLEX :: PLG(2*MMTy,NHEM,JG)                                    
! Tracer term                                                           
      COMPLEX :: TRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                         
!                                                                       
      COMPLEX :: TRANLG(2*MMTy,NHEM,JG,NLx,NTRAC)                       
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB/ZG,DG,PJG,PMG,SPG,VPG,EG,UTG,VTG                    
     &     ,UTRAG,VTRAG                                                 
     &     ,CHIG,SFG,FVGT,FUGT                                          
! Vorticity (zeta)                                                      
      COMPLEX :: ZG(2*MMTy,NHEM,JG,NLx)                                 
! Divergence (d)                                                        
      COMPLEX :: DG(2*MMTy,NHEM,JG,NLx)                                 
! (1-mu^2) (d(ln p*)/d mu)                                              
      COMPLEX :: PJG(2*MMTy,NHEM,JG)                                    
! (d(ln p*)/d lambda)                                                   
      COMPLEX :: PMG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: SPG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: VPG(2*MMTy,NHEM,JG)                                    
! (U^2 + V^2)                                                           
      COMPLEX :: EG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: UTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: VTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: UTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: VTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: CHIG(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: SFG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: FVGT(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: FUGT(2*MMTy,NHEM,JG,NLx)                               
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Polynomial used to aid vectorization of Legendre transforms           
!-----------------------------------------------------                  
      COMMON/POLYNO/POLY,CMPA                                           
      REAL :: POLY(NNT,MMTy,2,2)                                        
      COMPLEX :: CMPA(2*MMTy,NHEM)                                      
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      COMPLEX :: GWORK(MGPP/2,NHEM,3*NL+2)                              
      REAL :: AFAC                                                      
! Loop variables                                                        
      INTEGER :: I, IHEM, ITRAC, J, L                                   
!-----------------------------------------------------                  
! Prepare Fourier arrays:                                               
!  - change sign of terms which contribute negatively to tendency,      
!  - apply (1-mu**2) weighting,                                         
!  - take zonal derivatives,                                            
!  - make copies of effective momentum tendencies.                      
!-----------------------------------------------------                  
      DO L=1,NLxx(iCol)                                                 
         DO J=1,JG                                                      
            DO IHEM=1,NHEM                                              
               DO I=1,2*MMTyy(iRow)                                     
                  EG(I,IHEM,J,L)=-0.5*EG(I,IHEM,J,L)/CSSQ(J,IHEM)       
                  VTG(I,IHEM,J,L)=-VTG(I,IHEM,J,L)                      
                  TNLG(I,IHEM,J,L)=TNLG(I,IHEM,J,L)-                    
     &                 CMPA(I,IHEM)*UTG(I,IHEM,J,L)/CSSQ(J,IHEM)        
                  FVGT(I,IHEM,J,L)=FVG(I,IHEM,J,L)                      
                  FUGT(I,IHEM,J,L)=-FUG(I,IHEM,J,L)                     
                  FUG(I,IHEM,J,L)=CMPA(I,IHEM)*FUG(I,IHEM,J,L)/         
     &                 CSSQ(J,IHEM)                                     
                  FVG(I,IHEM,J,L)=CMPA(I,IHEM)*FVG(I,IHEM,J,L)/         
     &                 CSSQ(J,IHEM)                                     
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Tracer transforms depend on the form of advection used.               
!-----------------------------------------------------                  
      IF (LFLUX) THEN                                                   
         AFAC=-1.0                                                      
      ELSE                                                              
         AFAC=1.0                                                       
      ENDIF                                                             
      DO ITRAC=1,NTRAC                                                  
         DO L=1,NLxx(iCol)                                              
            DO J=1,JG                                                   
               DO IHEM=1,NHEM                                           
                  DO I=1,2*MMTyy(iRow)                                  
                     TRANLG(I,IHEM,J,L,ITRAC)=AFAC*                     
     &                    TRANLG(I,IHEM,J,L,ITRAC)-CMPA(I,IHEM)*        
     &                    UTRAG(I,IHEM,J,L,ITRAC)/CSSQ(J,IHEM)          
                     VTRAG(I,IHEM,J,L,ITRAC)=-VTRAG(I,IHEM,J,L,ITRAC)   
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Call single routine to perform all transforms with maximum            
! vector efficiency.                                                    
! *** NOTE : THE INPUT FOURIER FIELDS ARE MODIFIED IF                   
! GLOBAL ***                                                            
!-----------------------------------------------------                  
      CALL HANALV                                                       
      RETURN                                                            
      END                                                               
!=====================================================                  
! Direct Legendre transform for the diabatic part of the timestep.      
! Only transform the required fields: forcing of temperature and        
! tracers and spatial derivatives of the momentum forcing.              
!                                                                       
! Transforms from Fourier to spectral space at the current latitude     
! (pair).  In a global run the input arrays are complete (even+odd)     
! Fourier coefficients at the northern & southern hemisphere rows.      
!                                                                       
! Original version in IGCM2.              Piers Forster,   25.10.96.    
! Optimisation (/CSSQ to *SECSQ).         Mike Blackburn,  22.12.99.    
!=====================================================                  
      SUBROUTINE LTDDIA                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Half transforms. The levels are divided across the columns of PEs;    
! the zonal wavenumbers are divided across the row of PEs; and on       
! ech PE the latitude are complete.                                     
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA/UG,FUG,VG,FVG,TG,TNLG,PLG                           
     &     ,TRAG,TRANLG                                                 
! U_psi, then U (rotational wind)                                       
      COMPLEX :: UG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FUG(2*MMTy,NHEM,JG,NLx)                                
! V_chi, then V (divergent wind)                                        
      COMPLEX :: VG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FVG(2*MMTy,NHEM,JG,NLx)                                
! Temperature anomaly (T')                                              
      COMPLEX :: TG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: TNLG(2*MMTy,NHEM,JG,NLx)                               
! ln p* or p*                                                           
      COMPLEX :: PLG(2*MMTy,NHEM,JG)                                    
! Tracer term                                                           
      COMPLEX :: TRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                         
!                                                                       
      COMPLEX :: TRANLG(2*MMTy,NHEM,JG,NLx,NTRAC)                       
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB/ZG,DG,PJG,PMG,SPG,VPG,EG,UTG,VTG                    
     &     ,UTRAG,VTRAG                                                 
     &     ,CHIG,SFG,FVGT,FUGT                                          
! Vorticity (zeta)                                                      
      COMPLEX :: ZG(2*MMTy,NHEM,JG,NLx)                                 
! Divergence (d)                                                        
      COMPLEX :: DG(2*MMTy,NHEM,JG,NLx)                                 
! (1-mu^2) (d(ln p*)/d mu)                                              
      COMPLEX :: PJG(2*MMTy,NHEM,JG)                                    
! (d(ln p*)/d lambda)                                                   
      COMPLEX :: PMG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: SPG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: VPG(2*MMTy,NHEM,JG)                                    
! (U^2 + V^2)                                                           
      COMPLEX :: EG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: UTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: VTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: UTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: VTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: CHIG(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: SFG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: FVGT(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: FUGT(2*MMTy,NHEM,JG,NLx)                               
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Polynomial used to aid vectorization of Legendre transforms           
!-----------------------------------------------------                  
      COMMON/POLYNO/POLY,CMPA                                           
      REAL :: POLY(NNT,MMTy,2,2)                                        
      COMPLEX :: CMPA(2*MMTy,NHEM)                                      
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
! Loop variables                                                        
      INTEGER :: I, IHEM, ITRAC, J, L                                   
!-----------------------------------------------------                  
! Prepare Fourier arrays:                                               
!  - change sign of terms which contribute negatively to tendency,      
!  - apply (1-mu**2) weighting,                                         
!  - take zonal derivatives,                                            
!  - make copies of effective momentum tendencies.                      
!-----------------------------------------------------                  
      DO L=1,NLxx(iCol)                                                 
         DO J=1,JG                                                      
            DO IHEM=1,NHEM                                              
               DO I=1,2*MMTyy(iRow)                                     
                  FVGT(I,IHEM,J,L)=FVG(I,IHEM,J,L)                      
                  FUGT(I,IHEM,J,L)=-FUG(I,IHEM,J,L)                     
                  FUG(I,IHEM,J,L)=CMPA(I,IHEM)*FUG(I,IHEM,J,L)/         
     &                 CSSQ(J,IHEM)                                     
                  FVG(I,IHEM,J,L)=CMPA(I,IHEM)*FVG(I,IHEM,J,L)/         
     &                 CSSQ(J,IHEM)                                     
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Call single routine to perform all transforms with maximum            
! vector efficiency.                                                    
! *** NOTE : THE INPUT FOURIER FIELDS ARE MODIFIED IF                   
! GLOBAL ***                                                            
!-----------------------------------------------------                  
      CALL DANALV                                                       
      RETURN                                                            
      END                                                               
!=====================================================                  
! Inverse Legendre transform for the adiabatic part of the timestep     
! Transforms from spectral to Fourier space at the current latitude     
! (pair).  In a global run the resulting arrays are complete            
! (i.e. even+odd) Fourier coefficients at the northern & southern       
! hemisphere rows.                                                      
!                                                                       
! Version for RSGUP3.                     Mike Blackburn,  12.01.95     
!=====================================================                  
      SUBROUTINE LTI                                                    
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Half transforms. The levels are divided across the columns of PEs;    
! the zonal wavenumbers are divided across the row of PEs; and on       
! ech PE the latitude are complete.                                     
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA/UG,FUG,VG,FVG,TG,TNLG,PLG                           
     &     ,TRAG,TRANLG                                                 
! U_psi, then U (rotational wind)                                       
      COMPLEX :: UG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FUG(2*MMTy,NHEM,JG,NLx)                                
! V_chi, then V (divergent wind)                                        
      COMPLEX :: VG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FVG(2*MMTy,NHEM,JG,NLx)                                
! Temperature anomaly (T')                                              
      COMPLEX :: TG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: TNLG(2*MMTy,NHEM,JG,NLx)                               
! ln p* or p*                                                           
      COMPLEX :: PLG(2*MMTy,NHEM,JG)                                    
! Tracer term                                                           
      COMPLEX :: TRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                         
!                                                                       
      COMPLEX :: TRANLG(2*MMTy,NHEM,JG,NLx,NTRAC)                       
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB/ZG,DG,PJG,PMG,SPG,VPG,EG,UTG,VTG                    
     &     ,UTRAG,VTRAG                                                 
     &     ,CHIG,SFG,FVGT,FUGT                                          
! Vorticity (zeta)                                                      
      COMPLEX :: ZG(2*MMTy,NHEM,JG,NLx)                                 
! Divergence (d)                                                        
      COMPLEX :: DG(2*MMTy,NHEM,JG,NLx)                                 
! (1-mu^2) (d(ln p*)/d mu)                                              
      COMPLEX :: PJG(2*MMTy,NHEM,JG)                                    
! (d(ln p*)/d lambda)                                                   
      COMPLEX :: PMG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: SPG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: VPG(2*MMTy,NHEM,JG)                                    
! (U^2 + V^2)                                                           
      COMPLEX :: EG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: UTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: VTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: UTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: VTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: CHIG(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: SFG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: FVGT(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: FUGT(2*MMTy,NHEM,JG,NLx)                               
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Polynomial used to aid vectorization of Legendre transforms           
!-----------------------------------------------------                  
      COMMON/POLYNO/POLY,CMPA                                           
      REAL :: POLY(NNT,MMTy,2,2)                                        
      COMPLEX :: CMPA(2*MMTy,NHEM)                                      
!-----------------------------------------------------                  
! Like the variables in SPECTR but the meridional wavenumbers           
! are complete instead of the levels.                                   
!-----------------------------------------------------                  
      COMMON/SPECTR2/Z2,D2,T2,SP2,SPA2,VP2,DTE2,TT2,DT2,ZT2             
     &     ,TRA2,TRAT2                                                  
! Vorticity coefficient                                                 
      COMPLEX :: Z2(NNT,MMTy,NHEM,NLx)                                  
! Divergence coefficient                                                
      COMPLEX :: D2(NNT,MMTy,NHEM,NLx)                                  
! Temperature coefficient                                               
      COMPLEX :: T2(NNT,MMTy,NHEM,NLx)                                  
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP2(NNT,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA2(NNT,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP2(NNT,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE2(NNT,MMTy,NHEM,NLx)                                
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT2(NNT,MMTy,NHEM,NLx)                                 
! Tracer coefficient                                                    
      COMPLEX :: TRA2(NNT,MMTy,NHEM,NLx,NTRAC)                          
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT2(NNT,MMTy,NHEM,NLx,NTRAC)                         
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      COMPLEX :: CEZN                                                   
! Loop variables                                                        
      INTEGER :: I, IHEM, J, L                                          
!-----------------------------------------------------                  
! Preset Fourier arrays.                                                
!-----------------------------------------------------                  
      CHIG(:,:,:,:)=0.                                                  
      SFG(:,:,:,:)=0.                                                   
      UG(:,:,:,:)=0.                                                    
      VG(:,:,:,:)=0.                                                    
      ZG(:,:,:,:)=0.                                                    
      DG(:,:,:,:)=0.                                                    
      TG(:,:,:,:)=0.                                                    
      TRAG(:,:,:,:,:)=0.                                                
!-----------------------------------------------------                  
! Zero the half transforms on one level                                 
!-----------------------------------------------------                  
      PLG(:,:,:)=0.                                                     
      PJG(:,:,:)=0.                                                     
      PMG(:,:,:)=0.                                                     
!-----------------------------------------------------                  
! Remove planetary vorticity in spectral space, so all transforms       
! use relative vorticity.                                               
!-----------------------------------------------------                  
      IF (iRow.EQ.1) THEN                                               
         DO L=1,NLxx(iRow)                                              
            Z2(1,1,1,L)=Z2(1,1,1,L)-EZ                                  
         ENDDO                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Call single routine to perform all transforms with maximum            
! vector efficiency.                                                    
!-----------------------------------------------------                  
      CALL HEXPV                                                        
!-----------------------------------------------------                  
! 1st zonal wavenumber is stored on row 1                               
!-----------------------------------------------------                  
      IF (iRow.EQ.1) THEN                                               
         !-----------------------------------------------------         
         ! Restore planetary vorticity in spectral space.               
         !-----------------------------------------------------         
         DO L=1,NLxx(1)                                                 
            Z2(1,1,1,L)=Z2(1,1,1,L)+EZ                                  
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Convert from relative to absolute vorticity in Fourier space 
         ! (real part of) m=0 coefficient only.                         
         !-----------------------------------------------------         
         DO J=1,JG                                                      
            CEZN=CMPLX(2.0*SI(J,1),0.0)                                 
            DO L=1,NLxx(1)                                              
               ZG(1,1,J,L)=ZG(1,1,J,L)+CEZN                             
               IF (NHEM.EQ.2) THEN                                      
                  ZG(1,NHEM,J,L)=ZG(1,NHEM,J,L)-CEZN                    
               ENDIF                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Sum to give total winds.  CMPA takes x-derivative.                    
!-----------------------------------------------------                  
       DO L=1,NLxx(iCol)                                                
         DO J=1,JG                                                      
            DO IHEM=1,NHEM                                              
               DO I=1,2*MMTyy(iRow)                                     
                  UG(I,IHEM,J,L)=CMPA(I,IHEM)*CHIG(I,IHEM,J,L)          
     &                 -UG(I,IHEM,J,L)                                  
                  VG(I,IHEM,J,L)=CMPA(I,IHEM)* SFG(I,IHEM,J,L)          
     &                 +VG(I,IHEM,J,L)                                  
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Zonal gradient of ln(ps).                                             
!-----------------------------------------------------                  
      IF (iCol.EQ.1) THEN                                               
         DO J=1,JG                                                      
            DO IHEM=1,NHEM                                              
               DO I=1,2*MMTyy(iRow)                                     
                  PMG(I,IHEM,J)=CMPA(I,IHEM)*PLG(I,IHEM,J)              
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! Inverse Legendre transform for the adiabatic part of the timestep     
! Transforms from spectral to Fourier space at the current latitude     
! (pair).  In a global run the resulting arrays are complete            
! (i.e. even+odd) Fourier coefficients at the northern & southern       
! hemisphere rows.                                                      
!                                                                       
! Version for RSGUP3.                     Mike Blackburn,  12.01.95     
!=====================================================                  
      SUBROUTINE LTIDT                                                  
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB/ZG,DG,PJG,PMG,SPG,VPG,EG,UTG,VTG                    
     &     ,UTRAG,VTRAG                                                 
     &     ,CHIG,SFG,FVGT,FUGT                                          
! Vorticity (zeta)                                                      
      COMPLEX :: ZG(2*MMTy,NHEM,JG,NLx)                                 
! Divergence (d)                                                        
      COMPLEX :: DG(2*MMTy,NHEM,JG,NLx)                                 
! (1-mu^2) (d(ln p*)/d mu)                                              
      COMPLEX :: PJG(2*MMTy,NHEM,JG)                                    
! (d(ln p*)/d lambda)                                                   
      COMPLEX :: PMG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: SPG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: VPG(2*MMTy,NHEM,JG)                                    
! (U^2 + V^2)                                                           
      COMPLEX :: EG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: UTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: VTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: UTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: VTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: CHIG(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: SFG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: FVGT(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: FUGT(2*MMTy,NHEM,JG,NLx)                               
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Polynomial used to aid vectorization of Legendre transforms           
!-----------------------------------------------------                  
      COMMON/POLYNO/POLY,CMPA                                           
      REAL :: POLY(NNT,MMTy,2,2)                                        
      COMPLEX :: CMPA(2*MMTy,NHEM)                                      
!-----------------------------------------------------                  
! Like the variables in SPECTR but the meridional wavenumbers           
! are complete instead of the levels.                                   
!-----------------------------------------------------                  
      COMMON/SPECTR2/Z2,D2,T2,SP2,SPA2,VP2,DTE2,TT2,DT2,ZT2             
     &     ,TRA2,TRAT2                                                  
! Vorticity coefficient                                                 
      COMPLEX :: Z2(NNT,MMTy,NHEM,NLx)                                  
! Divergence coefficient                                                
      COMPLEX :: D2(NNT,MMTy,NHEM,NLx)                                  
! Temperature coefficient                                               
      COMPLEX :: T2(NNT,MMTy,NHEM,NLx)                                  
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP2(NNT,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA2(NNT,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP2(NNT,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE2(NNT,MMTy,NHEM,NLx)                                
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT2(NNT,MMTy,NHEM,NLx)                                 
! Tracer coefficient                                                    
      COMPLEX :: TRA2(NNT,MMTy,NHEM,NLx,NTRAC)                          
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT2(NNT,MMTy,NHEM,NLx,NTRAC)                         
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
! Loop variables                                                        
      INTEGER :: I, IHEM, J, L                                          
!-----------------------------------------------------                  
! Preset Fourier arrays.                                                
!-----------------------------------------------------                  
      CHIG(:,:,:,:)=0.                                                  
      SFG(:,:,:,:)=0.                                                   
!-----------------------------------------------------                  
! Calculate the x-derivative, CHIG, and y-derivative of                 
! temperature field                                                     
!-----------------------------------------------------                  
      CALL HEXPVDT                                                      
!-----------------------------------------------------                  
! Sum to give total winds.  CMPA takes x-derivative.                    
!-----------------------------------------------------                  
      DO L=1,NLxx(iCol)                                                 
         DO J=1,JG                                                      
            DO IHEM=1,NHEM                                              
               DO I=1,2*MMTyy(iRow)                                     
                   CHIG(I,IHEM,J,L)=CMPA(I,IHEM)* CHIG(I,IHEM,J,L)      
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! Inverse Legendre transform for the adiabatic part of the timestep     
! Transforms from spectral to Fourier space at the current latitude     
! (pair).  In a global run the resulting arrays are complete            
! (i.e. even+odd) Fourier coefficients at the northern & southern       
! hemisphere rows.                                                      
!                                                                       
! Version for RSGUP3.                     Mike Blackburn,  12.01.95     
!=====================================================                  
      SUBROUTINE LTIDIA                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Half transforms. The levels are divided across the columns of PEs;    
! the zonal wavenumbers are divided across the row of PEs; and on       
! ech PE the latitude are complete.                                     
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA/UG,FUG,VG,FVG,TG,TNLG,PLG                           
     &     ,TRAG,TRANLG                                                 
! U_psi, then U (rotational wind)                                       
      COMPLEX :: UG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FUG(2*MMTy,NHEM,JG,NLx)                                
! V_chi, then V (divergent wind)                                        
      COMPLEX :: VG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: FVG(2*MMTy,NHEM,JG,NLx)                                
! Temperature anomaly (T')                                              
      COMPLEX :: TG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: TNLG(2*MMTy,NHEM,JG,NLx)                               
! ln p* or p*                                                           
      COMPLEX :: PLG(2*MMTy,NHEM,JG)                                    
! Tracer term                                                           
      COMPLEX :: TRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                         
!                                                                       
      COMPLEX :: TRANLG(2*MMTy,NHEM,JG,NLx,NTRAC)                       
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB/ZG,DG,PJG,PMG,SPG,VPG,EG,UTG,VTG                    
     &     ,UTRAG,VTRAG                                                 
     &     ,CHIG,SFG,FVGT,FUGT                                          
! Vorticity (zeta)                                                      
      COMPLEX :: ZG(2*MMTy,NHEM,JG,NLx)                                 
! Divergence (d)                                                        
      COMPLEX :: DG(2*MMTy,NHEM,JG,NLx)                                 
! (1-mu^2) (d(ln p*)/d mu)                                              
      COMPLEX :: PJG(2*MMTy,NHEM,JG)                                    
! (d(ln p*)/d lambda)                                                   
      COMPLEX :: PMG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: SPG(2*MMTy,NHEM,JG)                                    
      COMPLEX :: VPG(2*MMTy,NHEM,JG)                                    
! (U^2 + V^2)                                                           
      COMPLEX :: EG(2*MMTy,NHEM,JG,NLx)                                 
      COMPLEX :: UTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: VTG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: UTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: VTRAG(2*MMTy,NHEM,JG,NLx,NTRAC)                        
      COMPLEX :: CHIG(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: SFG(2*MMTy,NHEM,JG,NLx)                                
      COMPLEX :: FVGT(2*MMTy,NHEM,JG,NLx)                               
      COMPLEX :: FUGT(2*MMTy,NHEM,JG,NLx)                               
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Polynomial used to aid vectorization of Legendre transforms           
!-----------------------------------------------------                  
      COMMON/POLYNO/POLY,CMPA                                           
      REAL :: POLY(NNT,MMTy,2,2)                                        
      COMPLEX :: CMPA(2*MMTy,NHEM)                                      
!-----------------------------------------------------                  
! Like the variables in SPECTR but the meridional wavenumbers           
! are complete instead of the levels.                                   
!-----------------------------------------------------                  
      COMMON/SPECTR2/Z2,D2,T2,SP2,SPA2,VP2,DTE2,TT2,DT2,ZT2             
     &     ,TRA2,TRAT2                                                  
! Vorticity coefficient                                                 
      COMPLEX :: Z2(NNT,MMTy,NHEM,NLx)                                  
! Divergence coefficient                                                
      COMPLEX :: D2(NNT,MMTy,NHEM,NLx)                                  
! Temperature coefficient                                               
      COMPLEX :: T2(NNT,MMTy,NHEM,NLx)                                  
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP2(NNT,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA2(NNT,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP2(NNT,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE2(NNT,MMTy,NHEM,NLx)                                
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT2(NNT,MMTy,NHEM,NLx)                                 
! Tracer coefficient                                                    
      COMPLEX :: TRA2(NNT,MMTy,NHEM,NLx,NTRAC)                          
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT2(NNT,MMTy,NHEM,NLx,NTRAC)                         
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      COMPLEX :: CEZN                                                   
! Loop variables                                                        
      INTEGER :: I, IHEM, J, L                                          
!-----------------------------------------------------                  
! Preset Fourier arrays.                                                
!-----------------------------------------------------                  
      CHIG(:,:,:,:)=0.                                                  
      SFG(:,:,:,:)=0.                                                   
      UG(:,:,:,:)=0.                                                    
      VG(:,:,:,:)=0.                                                    
      TG(:,:,:,:)=0.                                                    
      TRAG(:,:,:,:,:)=0.                                                
!-----------------------------------------------------                  
! Zero the half transforms on one level                                 
!-----------------------------------------------------                  
      PLG(:,:,:)=0.                                                     
!-----------------------------------------------------                  
! Remove planetary vorticity in spectral space, so all transforms       
! use relative vorticity.                                               
!-----------------------------------------------------                  
      IF (iRow.EQ.1) THEN                                               
         DO L=1,NLxx(iRow)                                              
            Z2(1,1,1,L)=Z2(1,1,1,L)-EZ                                  
         ENDDO                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Call single routine to perform all transforms with maximum            
! vector efficiency.                                                    
!-----------------------------------------------------                  
      CALL DEXPV                                                        
!-----------------------------------------------------                  
! Restore planetary vorticity in spectral space.                        
!-----------------------------------------------------                  
      IF (iRow.EQ.1) THEN                                               
         DO L=1,NLxx(1)                                                 
            Z2(1,1,1,L)=Z2(1,1,1,L)+EZ                                  
         ENDDO                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Sum to give total winds.  CMPA takes x-derivative.                    
!-----------------------------------------------------                  
       DO L=1,NLxx(iCol)                                                
         DO J=1,JG                                                      
            DO IHEM=1,NHEM                                              
               DO I=1,2*MMTyy(iRow)                                     
                  UG(I,IHEM,J,L)=CMPA(I,IHEM)*CHIG(I,IHEM,J,L)          
     &                 -UG(I,IHEM,J,L)                                  
                  VG(I,IHEM,J,L)=CMPA(I,IHEM)* SFG(I,IHEM,J,L)          
     &                 +VG(I,IHEM,J,L)                                  
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! MASCOR - Correction of global average surface pressure.               
! No longer requires KOUNTE=1 to work correctly.                        
!                                                                       
! Mike Blackburn  15/05/2000.  Original version for IGCM2.              
!                                                                       
! Purpose.                                                              
! --------                                                              
!                                                                       
!   This subroutine modifies the m=0,n=0 coefficient of ln(ps)          
! to correct the mean surface pressure (total mass) at that at the      
! start of the run, preventing drift in extended integrations.          
!                                                                       
! Interface.                                                            
! ----------                                                            
!                                                                       
! MASCOR is called from the main program MLTRI.                         
! Timestep counters are stored in common BATS.                          
! Mean surface pressures and switches are stored in common STATS.       
! Spectral arrays SP,SPA are stored in common SPECTR.                   
!                                                                       
! Method.                                                               
! -------                                                               
!                                                                       
!   Rescale the global mean surface pressure to its initial value       
! by adding the required constant to the m=0,n=0 ln(ps) coefficient.    
! Use the time-lagged mean pressure to avoid numerical instability.     
!                                                                       
!   It is assumed there are no sources or sinks in the mass             
! continuity equation (i.e. will not work if SPA changes in the diab    
! part of the time step).                                               
! The correction should be applied to the                               
! mass of dry air if precip/evap mass variations were included.         
!=====================================================                  
      SUBROUTINE MASCOR                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
!                                                                       
!-----------------------------------------------------                  
      COMMON/STATS/GMSP0,GMSPMI,LMASCOR,LMASOLD,LMASPRT                 
      LOGICAL :: LMASCOR,LMASOLD,LMASPRT                                
      REAL :: GMSP0,GMSPMI                                              
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: GMSP                                                      
      REAL :: ZSPRAT                                                    
      REAL :: ZLNRAT                                                    
      INTEGER :: KTEMP                                                  
      REAL :: ZSPDIF                                                    
!-----------------------------------------------------                  
! Extract the global mass.                                              
!-----------------------------------------------------                  
      GMSP=1.0+REAL(SPA(1,1,1))/SQRT(2.0)                               
!-----------------------------------------------------                  
! Preserve the mass at (re)start as required.                           
! Note that the timestep counter has already been incremented.          
! Pressure at start of initial run. Otherwise read in from restart file.
!-----------------------------------------------------                  
      IF ((KOUNT.EQ.KSTART+1).AND.GMSP0.EQ.0.0) THEN                    
         GMSP0=GMSP                                                     
         GMSPMI=GMSP                                                    
         WRITE(2,6900) GMSP                                             
      ENDIF                                                             
!-----------------------------------------------------                  
! Compute correction.                                                   
!-----------------------------------------------------                  
      ZSPRAT=GMSPMI/GMSP0                                               
      ZLNRAT=LOG(ZSPRAT)                                                
      SP(1,1,1)=SP(1,1,1)-SQRT(2.)*CMPLX(ZLNRAT,0.)                     
!-----------------------------------------------------                  
! Print diagnostics, correcting for (KITS-1) offset.                    
!-----------------------------------------------------                  
      IF (KITS.GT.0) THEN                                               
         KTEMP=KOUNT+1-KITS                                             
      ELSE                                                              
         KTEMP=KOUNT                                                    
      ENDIF                                                             
      IF (LMASPRT.AND.                                                  
     :    (KTEMP.LE.(KSTART+ITSPD).OR.MOD(KTEMP,ITSPD).EQ.0)) THEN      
         ZSPDIF=GMSPMI-GMSP0                                            
         WRITE(2,6901) GMSPMI,GMSP0,ZSPDIF,ZLNRAT,REAL(SP(1,1,1))       
      ENDIF                                                             
!-----------------------------------------------------                  
! Update the time-lagged mass.                                          
!-----------------------------------------------------                  
      GMSPMI=GMSP                                                       
!-----------------------------------------------------                  
! Formats.                                                              
!-----------------------------------------------------                  
 6900 FORMAT(' GLOBAL MASS RESET TO CURRENT RESTART VALUE =',1PE15.8)   
 6901 FORMAT('  PMEAN=',1PE15.8,' PM0=',1PE15.8                         
     :,' DIFF=',1PE15.8,' LN(PS) CORR=',1PE15.8,' LN(PS)=',1PE15.8)     
      RETURN                                                            
      END                                                               
!=====================================================                  
! This subroutine calculates the inverse of NxN array A.                
!                                                                       
! Arguements:                                                           
!                                                                       
!   A     - Array of dimension (LDA,N). Contains inverse                
!           of A on exit.                                               
!   N     - Number of rows and columns of A.                            
!   LDA   - Leading dimension of A.                                     
!   IWORK - Integer array contains the pivot indices of A.              
!   WORK  - Real array used as workspace for SGETRI.                    
!                                                                       
! This subroutine replaces the Cray specific subroutine                 
! MINV with portable LAPACK subroutines SGETRF and SGETRI.              
!=====================================================                  
      SUBROUTINE MATINV(A,N,LDA,IWORK,WORK)                             
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER N,LDA,IWORK(N),INFO                                       
      REAL A(LDA,N),WORK(N)                                             
                                                                        
      CALL SGETRF(N,N,A,LDA,IWORK,INFO)                                 
      IF (INFO.NE.0) THEN                                               
         IF (MYPE.EQ.0)                                                 
     &        WRITE(*,*) 'Error: SGETRF returned INFO = ',INFO          
         STOP                                                           
      ENDIF                                                             
      CALL SGETRI(N,A,LDA,IWORK,WORK,N,INFO)                            
      IF (INFO.NE.0) THEN                                               
         IF (MYPE.EQ.0)                                                 
     &        WRITE(*,*) 'Error: SGETRI returned INFO = ',INFO          
         STOP                                                           
      ENDIF                                                             
C                                                                       
      END                                                               
!=====================================================                  
! Computes nonlinear tendencies in grid point space                     
! for the present latitude                                              
!=====================================================                  
      SUBROUTINE MGRMLT                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB3/ZG3,DG3,PJG3,PMG3,SPG3,VPG3,EG3,UTG3,VTG3          
     &     ,UTRAG3,VTRAG3                                               
     &     ,CHIG3,SFG3                                                  
! Vorticity (zeta)                                                      
      REAL :: ZG3(MGx,NHEM,JGy,NL)                                      
! Divergence (d)                                                        
      REAL :: DG3(MGx,NHEM,JGy,NL)                                      
! (1-mu^2) (d(ln p*)/d mu)                                              
      REAL :: PJG3(MGx,NHEM,JGy)                                        
! (d(ln p*)/d lambda)                                                   
      REAL :: PMG3(MGx,NHEM,JGy)                                        
      REAL :: SPG3(MGx,NHEM,JGy)                                        
      REAL :: VPG3(MGx,NHEM,JGy)                                        
! (U^2 + V^2)                                                           
      REAL :: EG3(MGx,NHEM,JGy,NL)                                      
      REAL :: UTG3(MGx,NHEM,JGy,NL)                                     
      REAL :: VTG3(MGx,NHEM,JGy,NL)                                     
      REAL :: UTRAG3(MGx,NHEM,JGy,NL,NTRAC)                             
      REAL :: VTRAG3(MGx,NHEM,JGy,NL,NTRAC)                             
      REAL :: CHIG3(MGx,NHEM,JGy,NL)                                    
      REAL :: SFG3(MGx,NHEM,JGy,NL)                                     
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: SDOTP(MGx,NL-1)                                           
      REAL :: VGPG                                                      
      REAL :: SUMD(MGx)                                                 
      REAL :: TPTA(MGx),TPTB(MGx)                                       
      INTEGER :: K                                                      
      REAL :: FAC, FAC2                                                 
      REAL :: TSUM                                                      
      REAL :: SUMD1                                                     
! Loop variables                                                        
      INTEGER :: I, IHEM, ITRAC, J, L, LL                               
!-----------------------------------------------------                  
! Zero VPG3                                                             
!-----------------------------------------------------                  
      VPG3(:,:,:)=0.0                                                   
!-----------------------------------------------------                  
! Loop over latitudes and hemispheres                                   
!-----------------------------------------------------                  
      DO J=1,JGyy(iRow)                                                 
         DO IHEM=1,NHEM                                                 
            !-----------------------------------------------------      
            ! Zero arrays                                               
            !-----------------------------------------------------      
            DO I=1,MGxx(iCol)                                           
               SUMD(I)=0.0                                              
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Change from Ln(PSTAR) to PSTAR                            
            !-----------------------------------------------------      
            DO I=1,MGxx(iCol)                                           
               SPG3(I,IHEM,J)=EXP(PLG3(I,IHEM,J))-1.0                   
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Set VPG3 and SDOTP                                        
            !-----------------------------------------------------      
            DO L=1,NL-1                                                 
               DO I=1,MGxx(iCol)                                        
                  SUMD(I)=SUMD(I)+DSIGMA(L)*DG3(I,IHEM,J,L)             
                  VPG3(I,IHEM,J)=VPG3(I,IHEM,J)+DSIGMA(L)*              
     &                 SECSQ(JJ1(iRow)+J,IHEM)*                         
     &                 (UG3(I,IHEM,J,L)*PMG3(I,IHEM,J)+                 
     &                 VG3(I,IHEM,J,L)*PJG3(I,IHEM,J))                  
                  SDOTP(I,L)=SUMD(I)+VPG3(I,IHEM,J)                     
               ENDDO                                                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Set VPG3 and SDOTP at NL                                  
            !-----------------------------------------------------      
            DO I=1,MGxx(iCol)                                           
               SUMD(I)=SUMD(I)+DSIGMA(NL)*DG3(I,IHEM,J,NL)              
               VPG3(I,IHEM,J)=VPG3(I,IHEM,J)+DSIGMA(NL)*                
     &              SECSQ(JJ1(iRow)+J,IHEM)*                            
     &              (UG3(I,IHEM,J,NL)*PMG3(I,IHEM,J)+                   
     &              VG3(I,IHEM,J,NL)*PJG3(I,IHEM,J))                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Set SDOTP                                                 
            !-----------------------------------------------------      
            DO L=1,NL-1                                                 
               DO I=1,MGxx(iCol)                                        
                  SDOTP(I,L)=SIGMAH(L)*(SUMD(I)+VPG3(I,IHEM,J))-        
     &                 SDOTP(I,L)                                       
               ENDDO                                                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Zero SUMD                                                 
            !-----------------------------------------------------      
            DO I=1,MGxx(iCol)                                           
               SUMD(I)=0.0                                              
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Loop over layers                                          
            !-----------------------------------------------------      
            DO L=1,NL                                                   
               !-----------------------------------------------------   
               ! Zero TPTA and TPTB                                     
               !-----------------------------------------------------   
               DO I=1,MGxx(iCol)                                        
                  TPTA(I)=0.0                                           
                  TPTB(I)=0.0                                           
               ENDDO                                                    
               !-----------------------------------------------------   
               !                                                        
               !-----------------------------------------------------   
               DO LL=1,L                                                
                  DO I=1,MGxx(iCol)                                     
                     VGPG=SECSQ(JJ1(iRow)+J,IHEM)*(UG3(I,IHEM,J,LL)*    
     &                    PMG3(I,IHEM,J)+                               
     &                    VG3(I,IHEM,J,LL)*PJG3(I,IHEM,J))              
                     TPTA(I)=TPTA(I)+C(LL,L)*VGPG                       
                     TPTB(I)=TPTB(I)+C(LL,L)*(VGPG+DG3(I,IHEM,J,LL))    
                  ENDDO                                                 
               ENDDO                                                    
               !-----------------------------------------------------   
               !                                                        
               !-----------------------------------------------------   
               DO I=1,MGxx(iCol)                                        
                  UTG3(I,IHEM,J,L)=UG3(I,IHEM,J,L)*TG3(I,IHEM,J,L)      
                  VTG3(I,IHEM,J,L)=VG3(I,IHEM,J,L)*TG3(I,IHEM,J,L)      
                  EG3(I,IHEM,J,L)=UG3(I,IHEM,J,L)*UG3(I,IHEM,J,L)+      
     &                 VG3(I,IHEM,J,L)*VG3(I,IHEM,J,L)                  
               ENDDO                                                    
               !-----------------------------------------------------   
               ! UTRAC and VTRAC tracer terms                           
               !-----------------------------------------------------   
               DO ITRAC=1,NTRAC                                         
                  DO I=1,MGxx(iCol)                                     
                     UTRAG3(I,IHEM,J,L,ITRAC)=UG3(I,IHEM,J,L)*          
     &                    TRAG3(I,IHEM,J,L,ITRAC)                       
                     VTRAG3(I,IHEM,J,L,ITRAC)=VG3(I,IHEM,J,L)*          
     &                    TRAG3(I,IHEM,J,L,ITRAC)                       
                  ENDDO                                                 
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Treat top and bottom differently                       
               !-----------------------------------------------------   
               IF (L.GT.1.AND.L.LT.NL) THEN                             
                  DO I=1,MGxx(iCol)                                     
                     TSUM=SECSQ(JJ1(iRow)+J,IHEM)*(UG3(I,IHEM,J,L)*     
     &                    PMG3(I,IHEM,J)+                               
     &                    VG3(I,IHEM,J,L)*PJG3(I,IHEM,J))               
                     SUMD(I)=SUMD(I)+TSUM*DSIGMA(L)                     
                     TNLG3(I,IHEM,J,L)=TG3(I,IHEM,J,L)*DG3(I,IHEM,J,L)+ 
     1                 AKAP*TG3(I,IHEM,J,L)*(TSUM-TPTB(I))+             
     2                    TKP(L)*(TSUM-TPTA(I))-RDSIG(L)*               
     3                 (SDOTP(I,L)*(TG3(I,IHEM,J,L+1)-TG3(I,IHEM,J,L))+ 
     4                    SDOTP(I,L-1)*(TG3(I,IHEM,J,L)-                
     5                    TG3(I,IHEM,J,L-1))+VPG3(I,IHEM,J)*            
     6                    (T01S2(L)*SIGMAH(L)+T01S2(L-1)*SIGMAH(L-1))-  
     7                    SUMD(I)*(T01S2(L-1)+T01S2(L))+                
     8                    TSUM*DSIGMA(L)*T01S2(L-1))                    
                     FVG3(I,IHEM,J,L)=-UG3(I,IHEM,J,L)*ZG3(I,IHEM,J,L)- 
     1                    PJG3(I,IHEM,J)*TG3(I,IHEM,J,L)-               
     2                    RDSIG(L)*(SDOTP(I,L)*                         
     3                    (VG3(I,IHEM,J,L+1)-VG3(I,IHEM,J,L))+          
     4                    SDOTP(I,L-1)*(VG3(I,IHEM,J,L)-                
     5                    VG3(I,IHEM,J,L-1)))                           
                     FUG3(I,IHEM,J,L)=VG3(I,IHEM,J,L)*ZG3(I,IHEM,J,L)-  
     1                    PMG3(I,IHEM,J)*TG3(I,IHEM,J,L)-RDSIG(L)*      
     2                    (SDOTP(I,L)*(UG3(I,IHEM,J,L+1)-               
     3                    UG3(I,IHEM,J,L))+SDOTP(I,L-1)*                
     4                    (UG3(I,IHEM,J,L)-UG3(I,IHEM,J,L-1)))          
                  ENDDO                                                 
                  !---------------------------------------------------  
                  ! TRANLG tracer term                                  
                  !---------------------------------------------------  
                  IF (.NOT.LFLUX) THEN                                  
                     DO ITRAC=1,NTRAC                                   
                        DO I=1,MGxx(iCol)                               
                           TRANLG3(I,IHEM,J,L,ITRAC)=                   
     1                          TRAG3(I,IHEM,J,L,ITRAC)*DG3(I,IHEM,J,L) 
     2                          -RDSIG(L)*(SDOTP(I,L)*                  
     3                          (TRAG3(I,IHEM,J,L+1,ITRAC)-             
     4                          TRAG3(I,IHEM,J,L,ITRAC))+SDOTP(I,L-1)*( 
     5                          TRAG3(I,IHEM,J,L,ITRAC)-                
     6                          TRAG3(I,IHEM,J,L-1,ITRAC)))             
                        ENDDO                                           
                     ENDDO                                              
                  ELSE                                                  
                     DO ITRAC=1,NTRAC                                   
                        DO I=1,MGxx(iCol)                               
                           TRANLG3(I,IHEM,J,L,ITRAC)=RDSIG(L)*(         
     1                          SDOTP(I,L)*(TRAG3(I,IHEM,J,L+1,ITRAC)+  
     2                          TRAG3(I,IHEM,J,L,ITRAC))-SDOTP(I,L-1)*( 
     3                          TRAG3(I,IHEM,J,L,ITRAC)+                
     4                          TRAG3(I,IHEM,J,L-1,ITRAC)))             
                        ENDDO                                           
                     ENDDO                                              
                  ENDIF                                                 
               ELSE                                                     
                  !---------------------------------------------------  
                  ! Top and bottom layers                               
                  !---------------------------------------------------  
                  FAC=1.0                                               
                  FAC2=1.                                               
                  K=L                                                   
                  IF (L.EQ.NL) THEN                                     
                     K=L-1                                              
                     FAC=0.0                                            
                     FAC2=-1.                                           
                  ENDIF                                                 
                  DO I=1,MGxx(iCol)                                     
                     TSUM=SECSQ(JJ1(iRow)+J,IHEM)*(UG3(I,IHEM,J,L)*     
     &                    PMG3(I,IHEM,J)+                               
     &                    VG3(I,IHEM,J,L)*PJG3(I,IHEM,J))               
                     SUMD(I)=SUMD(I)+TSUM*DSIGMA(L)*FAC                 
                     TNLG3(I,IHEM,J,L)=TG3(I,IHEM,J,L)*DG3(I,IHEM,J,L)+ 
     1                    AKAP*TG3(I,IHEM,J,L)*(TSUM-TPTB(I))+          
     2                    TKP(L)*(TSUM-TPTA(I))-RDSIG(L)*               
     3                    (SDOTP(I,K)*(TG3(I,IHEM,J,K+1)-               
     4                    TG3(I,IHEM,J,K))+T01S2(K)*                    
     5                    (SIGMAH(K)*VPG3(I,IHEM,J)-SUMD(I)))           
                     FVG3(I,IHEM,J,L)=-UG3(I,IHEM,J,L)*ZG3(I,IHEM,J,L)- 
     &                    PJG3(I,IHEM,J)*TG3(I,IHEM,J,L)-RDSIG(L)*      
     &                    (SDOTP(I,K)*(VG3(I,IHEM,J,K+1)-               
     &                    VG3(I,IHEM,J,K)))                             
                     FUG3(I,IHEM,J,L)=VG3(I,IHEM,J,L)*ZG3(I,IHEM,J,L)-  
     &                    PMG3(I,IHEM,J)*TG3(I,IHEM,J,L)-               
     &                    RDSIG(L)*(SDOTP(I,K)*                         
     &                    (UG3(I,IHEM,J,K+1)-UG3(I,IHEM,J,K)))          
                  ENDDO                                                 
                  !---------------------------------------------------  
                  ! TRANLG tracer term                                  
                  !---------------------------------------------------  
                  IF (.NOT.LFLUX) THEN                                  
                     DO ITRAC=1,NTRAC                                   
                        DO I=1,MGxx(iCol)                               
                           TRANLG3(I,IHEM,J,L,ITRAC)=                   
     1                          TRAG3(I,IHEM,J,L,ITRAC)*DG3(I,IHEM,J,L) 
     2                          -RDSIG(L)*SDOTP(I,K)*                   
     3                          (TRAG3(I,IHEM,J,K+1,ITRAC)-             
     4                          TRAG3(I,IHEM,J,K,ITRAC))                
                        ENDDO                                           
                     ENDDO                                              
                  ELSE                                                  
                     DO ITRAC=1,NTRAC                                   
                        DO I=1,MGxx(iCol)                               
                           TRANLG3(I,IHEM,J,L,ITRAC)=RDSIG(L)*FAC2*     
     1                          SDOTP(I,K)*(TRAG3(I,IHEM,J,K+1,ITRAC)+  
     2                          TRAG3(I,IHEM,J,K,ITRAC))                
                       ENDDO                                            
                     ENDDO                                              
                  ENDIF                                                 
               ENDIF                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! Adds white noise perturbation to ln(surface pressure)                 
! balanced initial state at T=0.                                        
!=====================================================                  
      SUBROUTINE NOISE                                                  
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Switches counters and constants controlling type and frequency of     
! model output                                                          
!-----------------------------------------------------                  
      COMMON/OUTCON/RNTAPE,DAY,SQR2,RSQR2,EAM1,EAM2,TOUT1               
     &     ,TOUT2,LSPO,LGPO                                             
     &     ,RNTAPO                                                      
     &     ,LSHIST,LMINIH                                               
      COMMON/OUTCONINT/NCOEFF,NLAT,INLAT,INSPC,KOUNTP                   
     &                 ,KOUNTE,KOUNTH,KOUNTR,KOUTP                      
     &                 ,KOUTE,KOUTH,KOUTR                               
! Identyfying run number                                                
      REAL :: RNTAPE                                                    
! Maximum wavenumber of printed spectral coeffecients                   
      INTEGER :: NCOEFF                                                 
! Number of latitudes in printed grid point fields                      
      INTEGER :: NLAT                                                   
      INTEGER :: INLAT                                                  
!                                                                       
      INTEGER :: INSPC                                                  
! Number of timsteps between successive writing of spectral             
! and gridded fields                                                    
      INTEGER :: KOUNTP                                                 
      INTEGER :: KOUNTE                                                 
! Number of timsteps between successive writing of history              
! records, containing spectral fields at the present and                
! previous time levels.                                                 
      INTEGER :: KOUNTH                                                 
! Number of timsteps between successive writing of restart              
! records, containing the spectral fields at the present and            
! previous time levels                                                  
      INTEGER :: KOUNTR                                                 
      INTEGER :: KOUTP                                                  
      INTEGER :: KOUTE                                                  
      INTEGER :: KOUTH                                                  
      INTEGER :: KOUTR                                                  
      REAL :: DAY                                                       
      REAL :: SQR2                                                      
      REAL :: RSQR2                                                     
      REAL :: EAM1                                                      
      REAL :: EAM2                                                      
      REAL :: TOUT1                                                     
      REAL :: TOUT2                                                     
      LOGICAL :: LSPO(NL)                                               
      LOGICAL :: LGPO(NL)                                               
      REAL :: RNTAPO                                                    
      LOGICAL :: LSHIST                                                 
! Reduce the history file to single precision if true                   
      LOGICAL :: LMINIH                                                 
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! External function                                                     
!-----------------------------------------------------                  
      REAL, EXTERNAL :: RANF                                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: EPS                                                       
      REAL :: SCALE                                                     
      INTEGER :: IDUM                                                   
      COMPLEX, ALLOCATABLE :: RanNumbers(:,:,:)                         
      COMPLEX :: HoldVar(NNTx, MMTy, NHEM)                              
      INTEGER, PARAMETER :: TagRef=700                                  
      INTEGER :: iCol2, iRow2                                           
      INTEGER :: IN1                                                    
      REAL :: RAN1, RAN2                                                
! Loop variables                                                        
      INTEGER :: IHEM, IM, IN, iPe                                      
  200 FORMAT(' WHITE NOISE SURFACE PRESSURE PERTURBATION AT T=0'/)      
!-----------------------------------------------------                  
! Generate all the random numbers on PE 0                               
!-----------------------------------------------------                  
      IF (MYPE.EQ.0) THEN                                               
         WRITE (2,200)                                                  
         ALLOCATE(RanNumbers(NNT,MMT,NHEM))                             
         EPS=1.E-4                                                      
         SCALE=EPS/SQRT(2.0)                                            
         IDUM=-1                                                        
         !-----------------------------------------------------         
         ! Don't add random number for first zonal wavenumber           
         !-----------------------------------------------------         
         RanNumbers(1:IDM,1,:)=0.0                                      
         !-----------------------------------------------------         
         ! Generate random number for the other zonal wavenumbers       
         !-----------------------------------------------------         
         DO IHEM=1,NHEM                                                 
            DO IM=1,MMT                                                 
               IF (IM.EQ.1) THEN                                        
                  IN1=IDM                                               
               ELSE                                                     
                  IN1=1                                                 
               ENDIF                                                    
               DO IN=IN1,NNT                                            
                  CALL RANDOM_NUMBER(RAN1)                              
                  CALL RANDOM_NUMBER(RAN2)                              
                  RanNumbers(IN,IM,IHEM)=SCALE*                         
     &                 CMPLX(RAN1-0.5,RAN2-0.5)                         
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Add the randon numbers intended for PE 0                     
         !-----------------------------------------------------         
         DO IHEM=1,NHEM                                                 
            DO IM=1,MMTyy(iRow)                                         
               DO IN=1,NNTxx(iCol)                                      
                  SP(IN,IM,IHEM)=SP(IN,IM,IHEM)+RanNumbers(IN,IM,IHEM)  
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Send random number to other PEs                              
         !-----------------------------------------------------         
         IF (NPE.GT.1) THEN                                             
            DO iPe=1,NPE-1                                              
               !-----------------------------------------------------   
               ! Store data for PE iPe in HoldVar                       
               !-----------------------------------------------------   
               iCol2=GRID(1,iPe)                                        
               iRow2=GRID(2,iPe)                                        
               DO IHEM=1,NHEM                                           
                  DO IM=1,MMTyy(iRow2)                                  
                     DO IN=1,NNTxx(iCol2)                               
                        HoldVar(IN,IM,IHEM)=                            
     &                    RanNumbers(NNT1(iCol2)+IN,MMT1(iRow2)+IM,IHEM)
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Send random number to other PEs                        
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=NNTx*MMTy*NHEM                                 
               iTag=TagRef+iPe                                          
               CALL MPI_BSEND(HoldVar, nBuffSize, ParaCom, iRecv,       
     &              iTag, MPI_COMM_WORLD, ierr)                         
            ENDDO                                                       
         ENDIF                                                          
         DEALLOCATE(RanNumbers)                                         
      ELSE                                                              
         !-----------------------------------------------------         
         ! Receive data from PE 0                                       
         !-----------------------------------------------------         
         iSend=0                                                        
         nBuffSize=NNTx*MMTy*NHEM                                       
         iTag=TagRef+MyPe                                               
         CALL MPI_RECV(HoldVar, nBuffSize, ParaCom, iSend,              
     &        iTag, MPI_COMM_WORLD, istat, ierr)                        
         !-----------------------------------------------------         
         ! Add random numbers to surface pressure                       
         !-----------------------------------------------------         
         DO IHEM=1,NHEM                                                 
            DO IM=1,MMTyy(iRow)                                         
               DO IN=1,NNTxx(iCol)                                      
                  SP(IN,IM,IHEM)=SP(IN,IM,IHEM)+HoldVar(IN,IM,IHEM)     
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! The old log of surface pressure is set the new surface pressure for   
! every zonal wavenumber except the first.                              
!-----------------------------------------------------                  
      DO IHEM=1,NHEM                                                    
         DO IM=1,MMTyy(iRow)                                            
            IF ( (iRow.EQ.1).AND.(IM.EQ.1) ) THEN                       
               IN1=MAX(1,IDM-NNT1(iCol))                                
            ELSE                                                        
               IN1=1                                                    
            ENDIF                                                       
            DO IN=IN1,NNTxx(iCol)                                       
               SPMI(IN,IM,IHEM)=SP(IN,IM,IHEM)                          
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
C                                                                       
      RETURN                                                            
      END                                                               
!=====================================================                  
! This subroutine takes in a profile from climatology (o3clim) and      
! interpolates to the model vertical grid (nl.gt.15), computing the     
! model profile by constraining its column to agree with climatology.   
! The model profile is passed back in o3mod.                            
! N.B. The o3clim passed to the routine must be on the pressure levels  
! defined in the routine (ocliml).                                      
!=====================================================                  
      SUBROUTINE O3INTERP(o3clim,o3mod,ps)                              
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Passed variables                                                      
!-----------------------------------------------------                  
      REAL :: o3clim(15)                                                
      REAL :: o3mod(nl)                                                 
      REAL :: ps                                                        
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: rhoo3                                                     
      REAL :: GR                                                        
      REAL :: col                                                       
      REAL :: sumrdp                                                    
      real :: ocliml(15),halflev(14),deltap(15),ohdcol(15)              
      real :: dpmod(nl),modcol(nl)                                      
      real :: pressu                                                    
      REAL :: fraction                                                  
      logical :: ifirst                                                 
! Loop variables                                                        
      INTEGER :: L, LO                                                  
!-----------------------------------------------------                  
! Store data                                                            
!-----------------------------------------------------                  
      save                                                              
      data ocliml/1.0,3.0,10.0,30.0,50.0,70.0,100.0,150.0,200.0,        
     &              250.0,300.0,400.0,500.0,700.0,850.0/                
      data ifirst/.true./                                               
!-----------------------------------------------------                  
! rhoo3 is density of O3 at STP, GR is g*density of O3 at STP           
!-----------------------------------------------------                  
      rhoo3=2.14                                                        
      GR=GA*rhoo3                                                       
!-----------------------------------------------------                  
! First time through                                                    
!-----------------------------------------------------                  
      if (ifirst) then                                                  
         ifirst=.false.                                                 
         !-----------------------------------------------------         
         ! Halvlev is in mb                                             
         !-----------------------------------------------------         
         do l=1,14                                                      
            halflev(l)=(ocliml(l+1)+ocliml(l))/2.0                      
         enddo                                                          
         !-----------------------------------------------------         
         ! Top                                                          
         !-----------------------------------------------------         
         deltap(1)=halflev(1)                                           
         !-----------------------------------------------------         
         ! In between                                                   
         !-----------------------------------------------------         
         do l=2,14                                                      
            deltap(l)=halflev(l)-halflev(l-1) ! In between              
         enddo                                                          
         !-----------------------------------------------------         
         ! Bottom                                                       
         !-----------------------------------------------------         
         deltap(15)=1.0e3-halflev(14)                                   
         !-----------------------------------------------------         
         ! Convert all to Pa                                            
         !-----------------------------------------------------         
         do l=1,15                                                      
            deltap(l)=deltap(l)*100.0                                   
         enddo                                                          
      endif                                                             
!-----------------------------------------------------                  
! Compute overhead column of climatology for this particular            
! profile.                                                              
!-----------------------------------------------------                  
      col=0.0                                                           
      do l=1,15                                                         
         col=col+o3clim(l)*deltap(l)                                    
         ohdcol(l)=col*1.0e5/(GR)      ! In DU                          
      enddo                                                             
!-----------------------------------------------------                  
! model delta pressure in Pa                                            
!-----------------------------------------------------                  
      do l=1,nl                                                         
         dpmod(l)=dsigma(l)*ps                                          
      enddo                                                             
!-----------------------------------------------------                  
! MMR*dp summed down column                                             
!-----------------------------------------------------                  
      sumrdp=0.0                                                        
       DO l=1,nl                                                        
         IF (l.LT.NL)THEN                                               
            pressu=sigmah(l)*ps/100.0 ! in mb                           
         ELSE                                                           
            pressu=ps/100.0                                             
         ENDIF                                                          
         DO lo=1,14                                                     
            if (pressu.lt.halflev(1)) then                              
               !-----------------------------------------------------   
               ! Model column (DU) forced from climatology              
               !-----------------------------------------------------   
               modcol(l)=(pressu/halflev(1))*ohdcol(1)                  
               o3mod(l)=(modcol(l)*GR/1.0e5-sumrdp)/dpmod(l)            
               goto 271                                                 
            elseif (pressu.eq.halflev(lo)) then                         
               modcol(l)=ohdcol(lo)                                     
               o3mod(l)=(modcol(l)*GR/1.0e5-sumrdp)/dpmod(l)            
               goto 271                                                 
            elseif (pressu.gt.halflev(lo).and.pressu.lt.halflev(lo+1))  
     &              then                                                
               fraction=(pressu-halflev(lo))/(halflev(lo+1)-halflev(lo))
               modcol(l)=ohdcol(lo)+fraction*(ohdcol(lo+1)-ohdcol(lo))  
               o3mod(l)=(modcol(l)*GR/1.0e5-sumrdp)/dpmod(l)            
               goto 271                                                 
            elseif (pressu.ge.halflev(14)) then                         
               fraction=(pressu-halflev(14))/(1.0e3-halflev(14))        
               modcol(l)=ohdcol(14)+fraction*(ohdcol(15)-ohdcol(14))    
               o3mod(l)=(modcol(l)*GR/1.0e5-sumrdp)/dpmod(l)            
               goto 271                                                 
            endif                                                       
         ENDDO                                                          
 271     sumrdp=sumrdp+o3mod(l)*dpmod(l)                                
      ENDDO                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! Calculate potential temperature and Ertel potential vorticity         
! on model levels.                                                      
!=====================================================                  
      SUBROUTINE PVCR(JH,TFIELD,TTYPE)                                  
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Vertical derivative arrays for interpolation                          
!-----------------------------------------------------                  
      COMMON/COMGRM/DUDLSG,DVDLSG,DTDLSG                                
      REAL :: DUDLSG(MGx,NHEM,NL)                                       
      REAL :: DVDLSG(MGx,NHEM,NL)                                       
      REAL :: DTDLSG(MGx,NHEM,NL)                                       
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB3/ZG3,DG3,PJG3,PMG3,SPG3,VPG3,EG3,UTG3,VTG3          
     &     ,UTRAG3,VTRAG3                                               
     &     ,CHIG3,SFG3                                                  
! Vorticity (zeta)                                                      
      REAL :: ZG3(MGx,NHEM,JGy,NL)                                      
! Divergence (d)                                                        
      REAL :: DG3(MGx,NHEM,JGy,NL)                                      
! (1-mu^2) (d(ln p*)/d mu)                                              
      REAL :: PJG3(MGx,NHEM,JGy)                                        
! (d(ln p*)/d lambda)                                                   
      REAL :: PMG3(MGx,NHEM,JGy)                                        
      REAL :: SPG3(MGx,NHEM,JGy)                                        
      REAL :: VPG3(MGx,NHEM,JGy)                                        
! (U^2 + V^2)                                                           
      REAL :: EG3(MGx,NHEM,JGy,NL)                                      
      REAL :: UTG3(MGx,NHEM,JGy,NL)                                     
      REAL :: VTG3(MGx,NHEM,JGy,NL)                                     
      REAL :: UTRAG3(MGx,NHEM,JGy,NL,NTRAC)                             
      REAL :: VTRAG3(MGx,NHEM,JGy,NL,NTRAC)                             
      REAL :: CHIG3(MGx,NHEM,JGy,NL)                                    
      REAL :: SFG3(MGx,NHEM,JGy,NL)                                     
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed arguments                                                      
!-----------------------------------------------------                  
! Index for latitudes                                                   
      INTEGER :: JH                                                     
      REAL :: TFIELD(MGx,NHEM,NL)                                       
      INTEGER :: TTYPE                                                  
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: PSG(MGx,NHEM,NL)                                          
      REAL :: PMKG                                                      
      REAL :: PM1KG                                                     
      REAL :: AKT                                                       
      REAL :: TXP                                                       
      REAL :: TYP                                                       
! Loop variables                                                        
      INTEGER :: I, IHEM, L                                             
!-----------------------------------------------------                  
! Calculate pressure on model levels.                                   
!-----------------------------------------------------                  
      DO L=1,NL                                                         
         DO IHEM=1,NHEM                                                 
            DO I=1,MGxx(iCol)                                           
               PSG(I,IHEM,L)=SIGMA(L)*PLG3(I,IHEM,JH)                   
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Calculate potential temperature and PV on model levels.               
! Generalised formula using array of model level pressures is used      
! to calculate potential temperature.  But note that the reference      
! pressure for adiabatic processes is assumed to be P0, the non-        
! dimensionalising pressure.                                            
!                                                                       
! TFIELD is the tracer field to be initialised.                         
! TTYPE=1   : Initialise tracer as potential temperature.               
! TTYPE=2   : Initialise tracer as Ertel PV.                            
!                                                                       
! Remember that at this point:-                                         
! CHIGG3 contains the x-derivative of T.                                
! SFGG3  contains the y-derivative of T.                                
!-----------------------------------------------------                  
      DO IHEM=1,NHEM                                                    
         DO L=1,NL                                                      
            DO I=1,MGxx(iCol)                                           
               PMKG=PSG(I,IHEM,L)**(-AKAP)                              
               PM1KG=PMKG/PSG(I,IHEM,L)                                 
               AKT=AKAP*TG3(I,IHEM,JH,L)                                
               TXP=CHIG3(I,IHEM,JH,L)-AKT*PMG3(I,IHEM,JH)               
               TYP=SFG3(I,IHEM,JH,L)-AKT*PJG3(I,IHEM,JH)                
               IF(TTYPE.EQ.1) THEN                                      
                  TFIELD(I,IHEM,L)=TG3(I,IHEM,JH,L)*PMKG                
               ELSE                                                     
                  TFIELD(I,IHEM,L)=PM1KG*(ZG3(I,IHEM,JH,L)*             
     &                 (-DTDLSG(I,IHEM,L)+AKT)+SECSQ(JJ1(iRow)+JH,IHEM)*
     &                 (-DUDLSG(I,IHEM,L)*TYP+DVDLSG(I,IHEM,L)*TXP))    
               ENDIF                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! RADIATION SCHEME. MORCRETTE PMF 13/5/97                               
! It passes the pressure of the                                         
! full sigma levels and the surface                                     
! to the Radiation scheme                                               
! (So the morcrette code has one more level than NL)                    
! water vapour values come from QG and QSTAR                            
! and temperatures from TG and TSTAR                                    
!                                                                       
! Dingmin's changes to morcrette                                        
! combine bottom levels                                                 
! and set bottom level heating rate to zero as                          
! this is now the ground and is not a layer                             
!=====================================================                  
      SUBROUTINE RADN(JH,TROPHT)                                        
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Grid variables for DGRMLT loop                                        
!-----------------------------------------------------                  
      COMMON/GRIDPC/ASSBL,ASHBL,ASLBL,ARRCR,ARRLR                       
     &     ,TTVD,QTVD,TYBL,TXBL,TTRD,TTLR,TTCR,UTVD,VTVD                
     &     ,QTLR,QTCR,SSBL,SHBL,SLBL,RRCR,RRLR                          
     &     ,ARFLUX,ASFLD,ACLD,RFLUX,SFLD,CLD                            
      REAL :: ASSBL(MGx,NHEM,JGy)                                       
      REAL :: ASHBL(MGx,NHEM,JGy)                                       
      REAL :: ASLBL(MGx,NHEM,JGy)                                       
      REAL :: ARRCR(MGx,NHEM,JGy)                                       
      REAL :: ARRLR(MGx,NHEM,JGy)                                       
      REAL :: TTVD(MGx,NHEM,NL)                                         
      REAL :: QTVD(MGx,NHEM,NL)                                         
      REAL :: TYBL(MGx,NHEM)                                            
      REAL :: TXBL(MGx,NHEM)                                            
      REAL :: TTRD(MGx,NHEM,NL)                                         
      REAL :: TTLR(MGx,NHEM,NL)                                         
      REAL :: TTCR(MGx,NHEM,NL)                                         
      REAL :: UTVD(MGx,NHEM,NL)                                         
      REAL :: VTVD(MGx,NHEM,NL)                                         
      REAL :: QTLR(MGx,NHEM,NL)                                         
      REAL :: QTCR(MGx,NHEM,NL)                                         
      REAL :: SSBL(MGx,NHEM,JGy)                                        
      REAL :: SHBL(MGx,NHEM,JGy)                                        
      REAL :: SLBL(MGx,NHEM,JGy)                                        
      REAL :: RRCR(MGx,NHEM,JGy)                                        
      REAL :: RRLR(MGx,NHEM,JGy)                                        
      REAL :: arflux(MGx,NHEM,JGy,6)                                    
      REAL :: asfld(MGx,NHEM,JGy,6)                                     
      REAL :: acld(MGx,NHEM,JGy,4)                                      
      REAL :: rflux(MGx,NHEM,JGy,6)                                     
      REAL :: sfld(MGx,NHEM,JGy,6)                                      
      REAL :: cld(MGx,NHEM,JGy,4)                                       
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Radiation variables                                                   
!-----------------------------------------------------                  
      COMMON/CPIERS/ICFLAG,CFRAC,PNET,SNET,RRFLUX,TTSW,TTLW             
      INTEGER :: ICFLAG(MGx,NHEM,5,2)                                   
      REAL :: CFRAC(MGx,NHEM,5)                                         
      REAL :: PNET(MGx,NHEM,JGy)                                        
      REAL :: SNET(MGx,NHEM,JGy)                                        
      REAL :: RRFLUX(MGx,NHEM,JGy,6)                                    
      REAL :: TTSW(MGx,NHEM,NL)                                         
      REAL :: TTLW(MGx,NHEM,NL)                                         
!-----------------------------------------------------                  
! Orogrophy                                                             
!-----------------------------------------------------                  
      COMMON/GSG/GSG,TOPVAR,RLEVBZMO,FXZMO,ofrac,runoff,iareanum        
! Topog variance, GW diags for Orog, Non orog waves                     
      REAL :: TOPVAR(MGx,NHEM,JGy)                                      
      REAL :: RLEVBZMO(JGy,NHEM,NL)                                     
      REAL :: FXZMO(JGy,NHEM,NL)                                        
         REAL :: GSG(MGx,NHEM,JGy)                                      
         REAL :: ofrac(MGx,NHEM,JGy)                                    
         INTEGER :: runoff(50,2,33)                                     
         INTEGER :: iareanum                                            
!-----------------------------------------------------                  
! Radiation variables                                                   
!-----------------------------------------------------                  
      COMMON /RADHT/HTNET,TAVE                                          
      REAL :: HTNET(NHEM,JGy,MGx,NL)                                    
      COMPLEX :: TAVE(NNTx,MMTy,NHEM,NL)                                
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Common block for gas abundances                                       
!-----------------------------------------------------                  
      COMMON/MORCGAS/VMRCO2,ZCARDI,VMRCH4,RCH4,VMRN2O,RN2O,             
     &               VMRCFC11,RCFC11,VMRCFC12,RCFC12,                   
     &               VMRH2OS,RH2OS,NEXOTIC                              
      LOGICAL :: NEXOTIC                                                
      REAL :: ZCARDI,RCH4,RN2O,RCFC11,RCFC12,RH2OS                      
      REAL :: VMRCO2,VMRCH4,VMRN2O,VMRCFC11,VMRCFC12,VMRH2OS            
!-----------------------------------------------------                  
! Passed arguments                                                      
!-----------------------------------------------------                  
! Index for latitudes                                                   
      INTEGER :: JH                                                     
! tropopause height                                                     
      REAL :: tropht(MGx,NHEM,JGy)                                      
!-----------------------------------------------------                  
! Local variables.                                                      
!-----------------------------------------------------                  
      REAL :: RHSCL                                                     
      REAL :: CHRF                                                      
      INTEGER :: NTSTEP                                                 
! If =1, first time reading o3 and h2o (2 months' worth).               
      INTEGER :: ifirst                                                 
      INTEGER :: MTH1                                                   
! Fraction through month                                                
      REAL :: AMFRAC                                                    
! Current month counter. = month. number +1 'cos have 13 months         
! (dec in twice)                                                        
      INTEGER :: CMTH                                                   
      REAL :: adum                                                      
! Month names. 13 to enable wrap-around.                                
      CHARACTER*3 :: MN(13)                                             
! =1 when in process of month change                                    
!      integer ichange                                                  
! Filenames of o3 and h2o climatology read in. 1=current,               
! 2=next.                                                               
      CHARACTER*31 :: o3file1, h2ofile1, o3file2, h2ofile2              
! Levels for *clim variables                                            
      INTEGER, PARAMETER :: ClimLevs=15                                 
! Store the data read in from file                                      
      REAL :: HoldClim(ClimLevs)                                        
! Climatology ozone and water values. '1' is current month, '2' is      
! the next month. (SMR 30-05-97)                                        
! 11-03-97 i.e. nl=15 read in from climatology. Interpolated to         
! nl.gt.15 in subroutine interpo3.                                      
      REAL :: o3clim1(ClimLevs,MGx,nhem,JGy)                            
      REAL :: h2oclim1(ClimLevs,MGx,nhem,JGy)                           
      REAL :: o3clim2(ClimLevs,MGx,nhem,JGy)                            
      REAL :: h2oclim2(ClimLevs,MGx,nhem,JGy)                           
! Ozone interpolated to model levels. Current month.                    
      REAL :: o3mod1(nl,MGx,nhem,JGy)                                   
! Water interpolated to model levels. Current month.                    
      REAL :: h2omod1(nl,MGx,nhem,JGy)                                  
! Ozone interpolated to model levels. Next month.                       
      REAL :: o3mod2(nl,MGx,nhem,JGy)                                   
! Water interpolated to model levels. Next month.                       
      REAL :: h2omod2(nl,MGx,nhem,JGy)                                  
! Dummy profile for skipping to the right part of the                   
! climatology                                                           
      REAL :: rdummy(15)                                                
! sfc pressure (used in interpolation from climatology                  
! to model).                                                            
      REAL :: ps                                                        
      INTEGER :: MTH2                                                   
! Morcrette profiles (have extra level).                                
      REAL :: PR(NL+1),T(NL+1),h2o(nl+1)                                
      REAL :: o3(nl+1),htlw(nl+1),htsw(nl+1)                            
      REAL :: ALAT1                                                     
! Cloud fraction,lwp deep,high,mid,low                                  
      REAL :: cf(4,2)                                                   
! Cloud positions (deep,h,m,l;bottom,top)                               
      INTEGER :: ic(4,2)                                                
      REAL :: SWALB                                                     
      REAL :: ALON                                                      
! Array to hold fluxes at top and bottom of atmosphere                  
! 1st index - flux 1=SW, 2=LW                                           
! 2nd index - Direction 1=DN, 2=UP                                      
! 3rd index - Where 1=TOP, 2=SURFACE                                    
      REAL :: fluxes(2,2,2)                                             
      REAL :: fhtot,fhcorr                                              
      REAL :: A                                                         
      REAL :: B                                                         
      REAL :: HTNETO                                                    
      REAL :: ACCON                                                     
      REAL :: BCCON                                                     
      REAL :: CH4BETA                                                   
! Loop variables                                                        
      INTEGER :: I, IHEM, J, K, L, LD                                   
!-----------------------------------------------------                  
! To avoid reading in the climatology data more than once               
! save the variable that store the information                          
!-----------------------------------------------------                  
      SAVE                                                              
!-----------------------------------------------------                  
! Set starting values                                                   
!-----------------------------------------------------                  
      data mn/'dec','jan','feb','mar','apr','may','jun','jul',          
     &          'aug','sep','oct','nov','dec'/                          
      DATA IFIRST/1/                                                    
!-----------------------------------------------------                  
! Set some variables.                                                   
! CHRF is factor to non-dimensionalise heating rates.                   
!-----------------------------------------------------                  
      RHSCL=288.0*GASCON/GA                                             
      CHRF=86400.*WW*CT                                                 
!-----------------------------------------------------                  
! ntstep is the number of timesteps to skip.                            
!                                                                       
! N.B. No point any more frequent as Morcrette doing diurnal            
! averages anyway.                                                      
!-----------------------------------------------------                  
      ntstep=itspd                                                      
!-----------------------------------------------------                  
! This sets the non-dimensional gridpoint temperature tendency (TTRD)   
! to get to TTRD from K/day divide the K/day heating rate by            
!     (86400*WW*CT)                                                     
!     TTRD(non-dim)=HTRT(k/day)/(86400*WW*CT)                           
!                                                                       
! Reads in bogus ozone and water vapour on first call                   
! other data such as albedo and number of levels is in the include file 
!-----------------------------------------------------                  
! Read in required ozone and water vapour from climatology              
! First time through (read in two months of o3 and h2o if               
! lperpet .false. or just one if lperpet .true.).                       
!-----------------------------------------------------                  
      IF (ifirst.eq.1) then                                             
         !-----------------------------------------------------         
         ! First time around need to read in climatology data           
         !-----------------------------------------------------         
         IF (JH.EQ.1) THEN                                              
            CALL CALNDR(DOY,MTH1,AMFRAC)                                
            IF (MyPe.EQ.0) THEN                                         
               CMTH=MTH1+1                                              
               !-----------------------------------------------------   
               ! Wrap-around to January.                                
               !-----------------------------------------------------   
               IF(cmth.eq.13) cmth=1                                    
               !-----------------------------------------------------   
               ! Make the filenames and open the files:                 
               ! N.B. Read in from global climatology.                  
               !-----------------------------------------------------   
               write(o3file1,'(a,i3.3,a,i3.3,a)')                       
     &              'ozone_'//mn(cmth)//'_jg',jg,'_mg',mg,'_nhem2.dat'  
               write(h2ofile1,'(a,i3.3,a,i3.3,a)')                      
     &              'water_'//mn(cmth)//'_jg',jg,'_mg',mg,'_nhem2.dat'  
               write(o3file2,'(a,i3.3,a,i3.3,a)')                       
     &              'ozone_'//mn(cmth+1)//'_jg',jg,'_mg',mg,'_nhem2.dat'
               write(h2ofile2,'(a,i3.3,a,i3.3,a)')                      
     &              'water_'//mn(cmth+1)//'_jg',jg,'_mg',mg,'_nhem2.dat'
            ENDIF                                                       
            !-----------------------------------------------------      
            ! Read in climatology data                                  
            !-----------------------------------------------------      
            CALL READCLIM(o3file1,ClimLevs,.FALSE.,o3clim1)             
            CALL READCLIM(h2ofile1,ClimLevs,.FALSE.,h2oclim1)           
            CALL READCLIM(o3file2,ClimLevs,.FALSE.,o3clim2)             
            CALL READCLIM(h2ofile2,ClimLevs,.FALSE.,h2oclim2)           
            !-----------------------------------------------------      
            ! Wont need to read in the data next time                   
            !-----------------------------------------------------      
            ifirst=0                                                    
            !-----------------------------------------------------      
            ! Loop over all grid points                                 
            !-----------------------------------------------------      
            DO J=1,JGyy(iRow)                                           
               DO ihem=1,nhem                                           
                  DO i=1,MGxx(iCol)                                     
                     !------------------------------------------------- 
                     ! Interpolate to model vertical grid.              
                     !------------------------------------------------- 
C O3 climatology interpolation fix: ps variable                         
                     ps=PLG3(I,IHEM,J)*P0                               
                     call interpf(h2oclim1(1,i,ihem,J),                 
     &                 h2omod1(1,i,ihem,J),ps)                          
                     call O3INTERP(o3clim1(1,i,ihem,J),                 
     &                    o3mod1(1,i,ihem,J),ps)                        
                     call interpf(h2oclim2(1,i,ihem,J),                 
     &                    h2omod2(1,i,ihem,J),ps)                       
                     call O3INTERP(o3clim2(1,i,ihem,J),                 
     &                    o3mod2(1,i,ihem,J),ps)                        
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDIF                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! There's new climatology data for each month, see if this is a         
! new month.                                                            
!-----------------------------------------------------                  
      IF (JH.EQ.1) THEN                                                 
         !-----------------------------------------------------         
         ! Find out which month we're in now                            
         !-----------------------------------------------------         
         CALL CALNDR(DOY,MTH2,AMFRAC)                                   
         !-----------------------------------------------------         
         ! Check if a new month a climatology data is needed            
         !-----------------------------------------------------         
         IF ( (.not. lperpet).AND.(MTH1.NE.MTH2) ) THEN                 
            !-----------------------------------------------------      
            ! Copy 'next' to 'current'.                                 
            !-----------------------------------------------------      
            do J=1,JGyy(iRow)                                           
               do ihem=1,nhem                                           
                  do i=1,MGxx(iCol)                                     
                     do l=1,nl                                          
                        o3mod1(l,i,ihem,j)=o3mod2(l,i,ihem,j)           
                        h2omod1(l,i,ihem,j)=h2omod2(l,i,ihem,j)         
                     enddo                                              
                  enddo                                                 
               enddo                                                    
            enddo                                                       
            !-----------------------------------------------------      
            ! Move on month and wrap-around to January.                 
            !-----------------------------------------------------      
            IF (MyPe.EQ.0) THEN                                         
               CMTH=MTH2+1                                              
               IF (cmth.eq.13) cmth=1                                   
               !-----------------------------------------------------   
               ! And fetch new 'next':                                  
               !-----------------------------------------------------   
               write(o3file2,'(a,i3.3,a,i3.3,a)')                       
     &              'ozone_'//mn(cmth+1)//'_jg',jg,'_mg',mg,'_nhem2.dat'
               write(h2ofile2,'(a,i3.3,a,i3.3,a)')                      
     &              'water_'//mn(cmth+1)//'_jg',jg,'_mg',mg,'_nhem2.dat'
            ENDIF                                                       
            !-----------------------------------------------------      
            ! Read in the climatology                                   
            !-----------------------------------------------------      
            CALL READCLIM(o3file2,ClimLevs,.FALSE.,o3clim2)             
            CALL READCLIM(h2ofile2,ClimLevs,.FALSE.,h2oclim2)           
            !-----------------------------------------------------      
            ! Loop over all grid points                                 
            !-----------------------------------------------------      
            DO J=1,JGyy(iRow)                                           
               do ihem=1,nhem                                           
                  do i=1,MGxx(iCol)                                     
C O3 climatology interpolation fix: ps variable                         
                     ps=PLG3(I,IHEM,J)*P0                               
                     call interpf(h2oclim2(1,i,ihem,J),                 
     &                    h2omod2(1,i,ihem,J),ps)                       
                     call O3INTERP(o3clim2(1,i,ihem,J),                 
     &                    o3mod2(1,i,ihem,J),ps)                        
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Stop loop repeating                                       
            !-----------------------------------------------------      
            MTH1=MTH2                                                   
         ENDIF                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Now start the radiation bit.                                          
!-----------------------------------------------------                  
      DO ihem=1,nhem                                                    
         !-----------------------------------------------------         
         ! Calculates heating rates every ntstep time steps             
         !-----------------------------------------------------         
         IF (mod(kount,ntstep).eq.1) then                               
            !-----------------------------------------------------      
            ! Radn scheme                                               
            !-----------------------------------------------------      
            DO i=1,MGxx(iCol)                                           
               !---------------------------------------------------     
               ! First set SURFACE VALUES.                              
               ! Pressure, units of Pa                                  
               ! T, units of K                                          
               ! water, units of mmr                                    
               !---------------------------------------------------     
               PR(1)=PLG3(I,IHEM,JH)*P0                                 
               T(1)=TSTAR(I,IHEM,JH)*CT                                 
               IF (lclim) then                                          
                  !------------------------------------------------     
                  ! Using h2o from climatology.                         
                  !------------------------------------------------     
                  h2o(1)=AMFRAC*h2omod2(NL,i,ihem,JH)+                  
     &                 (1.0-AMFRAC)*h2omod1(NL,i,ihem,JH)               
               ELSE                                                     
                  !------------------------------------------------     
                  ! Using h2o from model.                               
                  !------------------------------------------------     
                  h2o(1)=QSTAR(I,IHEM,JH)                               
               ENDIF                                                    
               o3(1)=AMFRAC*o3mod2(NL,i,ihem,JH)+                       
     &              (1.0-AMFRAC)*o3mod1(NL,i,ihem,JH)                   
               !---------------------------------------------------     
               ! Now set rest of column.                                
               ! Start of loop over column. Reverse index (Morc goes    
               ! bottom up).                                            
               !---------------------------------------------------     
               DO LD=1,NL                                               
                  L=NL+2-LD                                             
                  PR(L)=SIGMA(LD)*PR(1) ! Pressure                      
                  T(L)=TG3(I,IHEM,JH,ld)*CT ! Temperature               
                  !------------------------------------------------     
                  ! Water vapour, including dealing with minimum        
                  ! values.                                             
                  !------------------------------------------------     
                  IF (lclim) then                                       
                     !---------------------------------------------     
                     ! Use h2o from climatology.                        
                     !---------------------------------------------     
                     h2o(l)=AMFRAC*h2omod2(ld,i,ihem,JH)+               
     &                    (1.0-AMFRAC)*h2omod1(ld,i,ihem,JH)            
                     !---------------------------------------------     
                     ! FIX WATER VAPOUR FOR LEVELS ABOVE CLIMATOLOGICAL 
                     ! TROPOPAUSE TO VALUE OF RH2OS. M. Bourqui Nov.    
                     ! 2001- fix to MMR- MMJ Jul 08                     
                     !---------------------------------------------     
                     H2O(L)=MAX(RH2OS,H2O(L))                           
                     !---------------------------------------------     
                     ! Fix water vapour for levels above climatological 
                     ! tropopause to be 6e-6 ppmv.                      
                     !---------------------------------------------     
                     IF (PR(L).LE.TROPHT(i,ihem,JH))THEN                
                        !---------------------------------------------  
                        ! Add methane oxidation term onto minimum water 
                        ! From the calculations of Stefan Fueglistahler 
                        !---------------------------------------------  
                        accon=1.5-0.9*(CS(JJ1(iRow)+JH,IHEM)**3)        
                        bccon=0.9+0.2*(CS(JJ1(iRow)+JH,IHEM)**3)        
                        CH4BETA =0.46*(1.0+                             
     &                       tanh((accon-log10(pr(l)/1.0e2))*bccon))    
                        H2O(L)=RH2OS+VMRCH4*CH4BETA*2.*18./29.          
                     ENDIF      ! end of strat. water vapour fix        
                  ELSE                                                  
                     !---------------------------------------------     
                     ! Use h2o from model                               
                     !---------------------------------------------     
                     H2O(L)=MAX(RH2OS,TRAG3(I,IHEM,JH,LD,1))            
                     !---------------------------------------------     
                     ! Fix water vapour for levels above climatological 
                     ! tropopause                                       
                     !---------------------------------------------     
                     IF(PR(L).LE.TROPHT(i,ihem,JH))THEN                 
                        !---------------------------------------------  
                        ! Add methane oxidation term onto minimum water 
                        ! From the calculations of Stefan Fueglistahler 
                        !---------------------------------------------  
                        accon=1.5-0.9*(CS(JJ1(iRow)+JH,IHEM)**3)        
                        bccon=0.9+0.2*(CS(JJ1(iRow)+JH,IHEM)**3)        
                        CH4BETA = 0.46*(1.0+                            
     &                       tanh((accon-log10(pr(l)/1.0e2))*bccon))    
                        H2O(L)=RH2OS+VMRCH4*CH4BETA*2.*18./29.          
                     ENDIF      ! end of strat. water vapour fix        
                  ENDIF                                                 
                  !------------------------------------------------     
                  ! Ozone                                               
                  !------------------------------------------------     
                  o3(l)=AMFRAC*o3mod2(ld,i,ihem,JH)+                    
     &                 (1.0-AMFRAC)*o3mod1(ld,i,ihem,JH)                
               ENDDO            ! End of loop over column.              
               !---------------------------------------------------     
               ! And alat1                                              
               !---------------------------------------------------     
               alat1=alat(JJ1(iRow)+JH,IHEM)                            
               !---------------------------------------------------     
               ! 9-7-97 Piers' clouds.                                  
               ! Deep and shallow                                       
               !---------------------------------------------------     
               cf(1,1)=max(cfrac(I,IHEM,5),cfrac(I,IHEM,4))             
               !---------------------------------------------------     
               ! High cloud fraction                                    
               !---------------------------------------------------     
               cf(2,1)=cfrac(I,IHEM,3)                                  
               !---------------------------------------------------     
               ! Mid cloud fraction                                     
               !---------------------------------------------------     
               cf(3,1)=cfrac(I,IHEM,2)                                  
               !---------------------------------------------------     
               ! Low cloud fraction                                     
               !---------------------------------------------------     
               cf(4,1)=cfrac(I,IHEM,1)                                  
               !---------------------------------------------------     
               ! Cloud levels reversed.                                 
               ! Choose lowest level for bottom (bottom deep/shallow)   
               !---------------------------------------------------     
               ic(1,1)=max(icflag(I,IHEM,4,1),icflag(I,IHEM,5,1))       
               !---------------------------------------------------     
               ! Choose hightest level for top (top deep/shallow)       
               !---------------------------------------------------     
               ic(1,2)=min(icflag(I,IHEM,4,2),icflag(I,IHEM,5,2))       
               !---------------------------------------------------     
               ! Reverse                                                
               !---------------------------------------------------     
               ic(1,1)=NL-ic(1,1)+2                                     
               ic(1,2)=NL-ic(1,2)+2                                     
               !---------------------------------------------------     
               ! Makes sure bottom cloud isn't in bottom level          
               !---------------------------------------------------     
               if (ic(1,1).eq.2) ic(1,1)=3                              
               if (ic(1,2).eq.2) ic(1,1)=3                              
               !---------------------------------------------------     
               ! High pos, one height needed                            
               !---------------------------------------------------     
               ic(2,1)=NL-icflag(I,IHEM,3,1)+2                          
               !---------------------------------------------------     
               ! Mid pos                                                
               !---------------------------------------------------     
               ic(3,1)=NL-icflag(I,IHEM,2,1)+2                          
               !---------------------------------------------------     
               ! Low pos                                                
               !---------------------------------------------------     
               ic(4,1)=NL-icflag(I,IHEM,1,1)+2                          
                                                                        
               SWALB=SALB(I,IHEM,JH)                                    
               !---------------------------------------------------     
               ! Cloud cf and ic passed. Fluxes returned which is net   
               ! flux at TOA in profile.                                
               ! Call radiation scheme                                  
               !---------------------------------------------------     
               alon=REAL(II1(iCol)+I-1)/REAL(mg)*360.0                  
               call morcigcm(pr,t,h2o,o3,alat1,htlw,htsw,DOY,cf,ic,     
     &              fluxes,swalb,alon)                                  
               !---------------------------------------------------     
               ! Store net flux in PNET                                 
               !---------------------------------------------------     
               PNET(I,IHEM,JH)=fluxes(1,1,1)-fluxes(1,2,1)+             
     &              fluxes(2,1,1)-fluxes(2,2,1)                         
               SNET(I,IHEM,JH)=fluxes(1,1,2)-fluxes(1,2,2)+             
     &              fluxes(2,1,2)-fluxes(2,2,2)                         
               rrflux(I,IHEM,JH,1)=fluxes(1,1,2)                        
               rrflux(I,IHEM,JH,2)=fluxes(1,2,2)                        
               rrflux(I,IHEM,JH,3)=fluxes(2,1,2)                        
               rrflux(I,IHEM,JH,4)=fluxes(2,2,2)                        
               rrflux(I,IHEM,JH,5)=fluxes(1,1,1)-fluxes(1,2,1)          
               rrflux(I,IHEM,JH,6)=fluxes(2,2,1)                        
               fhtot=0.0                                                
               DO l=nl,1,-1                                             
                  !------------------------------------------------     
                  ! Bottom heating rate is zero in morecrette           
                  !------------------------------------------------     
                  LD=NL+1-L                                             
                  HTNETO=HTNET(IHem,JH,I,LD)                            
                  htnet(ihem,jh,i,ld)=htlw(l+1)+htsw(l+1)               
                  !------------------------------------------------     
                  ! Sets this heating rate                              
                  !------------------------------------------------     
                  TTRD(I,IHEM,LD)=(HTNETO                               
     &                 +HTNET(IHEM,JH,I,LD))/(CHRF*2.)                  
                 fhtot=fhtot+htnet(ihem,jh,i,ld)*1.0*                   
     :           dsigma(LD)*pr(1)*(GASCON/AKAP/GA)/86400.               
               ENDDO                                                    
               fhcorr=PNET(I,IHEM,JH)-SNET(I,IHEM,JH)-fhtot             
               fluxes(2,2,1)=fluxes(2,2,1)+fhcorr                       
               rrflux(I,IHEM,JH,6)=rrflux(I,IHEM,JH,6)+fhcorr           
               PNET(I,IHEM,JH)=PNET(I,IHEM,JH)-fhcorr                   
            ENDDO                                                       
         ELSE                                                           
            !-----------------------------------------------------      
            ! Doesn't do rad scheme (simply uses old heating rates)     
            !-----------------------------------------------------      
            DO i=1,MGxx(iCol)                                           
               DO LD=1,NL                                               
                  TTRD(I,IHEM,LD)=htnet(ihem,jh,i,ld)/CHRF              
               ENDDO                                                    
            ENDDO                                                       
         ENDIF                                                          
      ENDDO                     ! end of loop over hemispheres          
                                                                        
      IF (LSHORT.AND.(KOUNT.eq.1)) then                                 
         DO l=1,nl                                                      
            DO IHEM=1,NHEM                                              
               DO i=1,MGxx(iCol)                                        
                  ttrd(I,IHEM,l)=ttrd(I,IHEM,l)*2.                      
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
                                                                        
      RETURN                                                            
      END                                                               
!=====================================================                  
! Read climatology data                                                 
!=====================================================                  
      SUBROUTINE READCLIM(FileName,ClimLevs,lTropWrite,ClimArray)       
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variables                                                      
!-----------------------------------------------------                  
! Name of file                                                          
      CHARACTER*(31), INTENT(IN) :: FileName                            
! Number of levels in file                                              
      INTEGER, INTENT(IN) :: ClimLevs                                   
! Require the tropopause height to be written for first call from       
! subroutine SFCT for TROPHT data.                                      
      LOGICAL :: lTropWrite                                             
! Output array                                                          
      REAL :: ClimArray(ClimLevs,MGx,NHEM,JGy)                          
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: RDUMMY(ClimLevs)                                          
      REAL, ALLOCATABLE :: HoldClim(:,:,:,:)                            
      INTEGER, PARAMETER :: TagRef=800                                  
      INTEGER :: iCol2, iRow2                                           
! Loop variables                                                        
      INTEGER :: I, IHEM, iPe, J, L                                     
!-----------------------------------------------------                  
! Open file                                                             
!-----------------------------------------------------                  
      IF (NPE.EQ.1) THEN                                                
         OPEN(UNIT=47,FILE='climdata/'//TRIM(FileName),                 
     &        FORM='FORMATTED',STATUS='OLD')                            
         !-----------------------------------------------------         
         ! Loop over hemisphere and longitudes                          
         !-----------------------------------------------------         
         DO J=1,JG                                                      
            DO IHEM=1,NHEM                                              
               DO I=1,MG                                                
                  READ(47,*)(ClimArray(L,I,IHEM,J),L=1,ClimLevs)        
               ENDDO                                                    
               !---------------------------------------------------     
               ! For one hemisphere runs, the climatology data for      
               ! other hemisphere is not needed                         
               !---------------------------------------------------     
               IF (NHEM.EQ.1) THEN                                      
                  READ(47,*) (RDUMMY(L),L=1,ClimLevs)                   
               ENDIF                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Close file                                                   
         !-----------------------------------------------------         
         CLOSE(47)                                                      
         !-----------------------------------------------------         
         ! Write tropopause height to file                              
         !-----------------------------------------------------         
         IF (lTropWrite) THEN                                           
            WRITE(2,*)'CLIMATOLOGICAL TROPOPAUSE HEIGHT'                
            DO IHEM=1,NHEM                                              
               WRITE(2,*)(ClimArray(5,1,IHEM,J),J=1,JG)                 
            ENDDO                                                       
         ENDIF                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! Need to read in the data on PE 0 before passing to           
         ! other PEs.                                                   
         !-----------------------------------------------------         
         IF (MyPe.eq.0) THEN                                            
            OPEN(UNIT=47,FILE='climdata/'//TRIM(FileName),              
     &           FORM='FORMATTED',STATUS='OLD')                         
            ALLOCATE(HoldClim(ClimLevs,MG,NHEM,JG))                     
            DO J=1,JG                                                   
               DO IHEM=1,NHEM                                           
                  DO I=1,MG                                             
                     READ(47,*) (HoldClim(L,I,IHEM,J),L=1,ClimLevs)     
                  ENDDO                                                 
                  !---------------------------------------------------  
                  ! For one hemisphere runs, the climatology data for   
                  ! other hemisphere is not needed                      
                  !---------------------------------------------------  
                  IF (NHEM.EQ.1) THEN                                   
                     READ(47,*) (RDUMMY(L),L=1,ClimLevs)                
                  ENDIF                                                 
               ENDDO                                                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Close file                                                
            !-----------------------------------------------------      
            CLOSE(47)                                                   
            !-----------------------------------------------------      
            ! Write tropopause height to file                           
            !-----------------------------------------------------      
            IF (lTropWrite) THEN                                        
               WRITE(2,*)'CLIMATOLOGICAL TROPOPAUSE HEIGHT'             
               DO IHEM=1,NHEM                                           
                  WRITE(2,*)(HoldClim(5,1,IHEM,J),J=1,JG)               
               ENDDO                                                    
            ENDIF                                                       
            !-----------------------------------------------------      
            ! Need to send data                                         
            !-----------------------------------------------------      
            DO iPe=NPE-1,0,-1                                           
               iCol2=GRID(1,iPe)                                        
               iRow2=GRID(2,iPe)                                        
               DO J=1,JGyy(iRow2)                                       
                  DO IHEM=1,NHEM                                        
                     DO I=1,MGxx(iCol2)                                 
                        DO L=1,ClimLevs                                 
                           ClimArray(L,I,IHEM,J)=                       
     &                       HoldClim(L,II1(iCol2)+I,IHEM,JJ1(iRow2)+J) 
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Need to send data                                      
               !-----------------------------------------------------   
               IF (iPe.GT.0) THEN                                       
                  iRecv=iPe                                             
                  nBuffSize=ClimLevs*JGy*NHEM*MGx                       
                  iTag=TagRef+iPe                                       
                  CALL MPI_BSEND(ClimArray, nBuffSize, ParaReal, iRecv, 
     &                 iTag, MPI_COMM_WORLD, ierr)                      
               ENDIF                                                    
            ENDDO                                                       
            DEALLOCATE(HoldClim)                                        
         ENDIF                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         IF (MyPe.GT.0) THEN                                            
            !-----------------------------------------------------      
            ! Receive data from PE 0                                    
            !-----------------------------------------------------      
            iSend=0                                                     
            nBuffSize=ClimLevs*JGy*NHEM*MGx                             
            iTag=TagRef+MyPe                                            
            CALL MPI_RECV(ClimArray, nBuffSize, ParaReal, iSend,        
     &           iTag, MPI_COMM_WORLD, istat, ierr)                     
         ENDIF                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Routine to read in the spectral data. Using a binary code             
! for iType to decide which variables are needed:                       
!  - +1: Z, D, T, TRA (if used) and SP are wanted                       
!  - +2: ZMI, DMI, TRAMI (if used) and SPMI are wanted                  
!  - +4: TTRES is wanted                                                
!  - +32: HTNET is wanted (HTNET is a grid-space variable)              
! e.g. For version above 1, to write out RKOUNT, RNTAPE, DAY, DOY,      
! Z, D, T, TRA, SP, RNTAPE, ZMI, DMI, TMI, TRAMI, SPMI, RNTAPE,         
! HTNET and RNTAPE iType needs to be 35 (1+2+32).                       
!=====================================================                  
      SUBROUTINE READCOEF(ID, iType, lRestart, DAYNEAR, RKOUNT,         
     &     RM1TAPE, DAY)                                                
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Restoration temperature field and constants which determine it,       
! also contains timescales                                              
!-----------------------------------------------------                  
      COMMON/RESTIJ/TTRES                                               
     + ,DTNS,DTEP,DTTRP,FAC,DDAMP,TFRC,YRLEN,TRS                        
     +  ,ALR,ZTROP,TGR                                                  
      COMPLEX :: TTRES(NNTx,MMTy,NHEM,NL)                               
      REAL :: DTNS                                                      
      REAL :: DTEP                                                      
      REAL :: DTTRP                                                     
      REAL :: FAC(NL)                                                   
      REAL :: DDAMP(NL)                                                 
      REAL :: TFRC(NL)                                                  
      REAL :: YRLEN                                                     
      REAL :: TRS(NL)                                                   
      REAL :: ALR                                                       
      REAL :: ZTROP                                                     
      REAL :: TGR                                                       
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Radiation variables                                                   
!-----------------------------------------------------                  
      COMMON /RADHT/HTNET,TAVE                                          
      REAL :: HTNET(NHEM,JGy,MGx,NL)                                    
      COMPLEX :: TAVE(NNTx,MMTy,NHEM,NL)                                
!-----------------------------------------------------                  
!                                                                       
!-----------------------------------------------------                  
      COMMON/STATS/GMSP0,GMSPMI,LMASCOR,LMASOLD,LMASPRT                 
      LOGICAL :: LMASCOR,LMASOLD,LMASPRT                                
      REAL :: GMSP0,GMSPMI                                              
!-----------------------------------------------------                  
! Passed variables                                                      
!-----------------------------------------------------                  
! Unit file number                                                      
      INTEGER :: ID                                                     
! The list of variables to write out is deterimed by choice of iType    
      INTEGER :: iType                                                  
! For restart the input data is binary                                  
      LOGICAL :: lRestart                                               
! Time of day that is expected                                          
      REAL :: DAYNEAR                                                   
! Starting count for input data                                         
      REAL :: RKOUNT                                                    
! Run number                                                            
      REAL :: RM1TAPE                                                   
! Time of day for input data                                            
      REAL :: DAY                                                       
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: RM2TAPE, RM3TAPE                                          
      INTEGER :: nLevels                                                
      COMPLEX, ALLOCATABLE :: AllSpec(:,:,:)                            
      COMPLEX, ALLOCATABLE :: HoldVar(:,:,:,:)                          
      REAL, ALLOCATABLE :: HtnetFull(:,:,:,:)                           
      LOGICAL :: lAllocHtnet                                            
      INTEGER :: LL                                                     
      REAL :: RK, RREC                                                  
      REAL :: RMT1, RMT2                                                
      REAL :: RDAY                                                      
! Loop variables                                                        
      INTEGER :: IHEM, IM, IN, KK, L                                    
!-----------------------------------------------------                  
! Format statments                                                      
!-----------------------------------------------------                  
 2010 FORMAT(/' ***ABORT*** THE HISTORY RECORDS READ FROM CHANNEL '     
     & ,I3,/' ARE NOT IN CORRECT FORMAT ')                              
 2020 FORMAT(/' ***ABORT*** THE RUN NUMBER IN THE HISTORY RECORD'/      
     & ' IS NOT THE SAME AS RNTAPE ENTERED IN NAMELIST')                
 2031 FORMAT(/' ***ABORT*** CANNOT FIND THE CORRECT RESTART RECORD.'/   
     & ' LOOKING FOR DAY',F10.2/,' BUT THE NEAREST RECORD FOUND',       
     & ' IS FOR DAY',F10.2)                                             
C                                                                       
 2110 FORMAT(/' ***ABORT: MASS RESTART RECORD HAS WRONG LENGTH')        
!-----------------------------------------------------                  
! Check the type makes sense                                            
!-----------------------------------------------------                  
      IF ( (iType.NE.1).AND.(iType.NE.3).AND.(iType.NE.4)               
     &     .AND.(iType.NE.8).AND.(iType.NE.35) ) THEN                   
         !-----------------------------------------------------         
         ! This type is not currently allowed                           
         !-----------------------------------------------------         
         IF (MyPe.EQ.0) WRITE(6,*) 'ERROR: this type is not currently ',
     &        'allowed: iType=', iType                                  
         CALL ABORT                                                     
      ENDIF                                                             
!-----------------------------------------------------                  
! Work out the number of levels                                         
!-----------------------------------------------------                  
      nLevels=0                                                         
      IF (MOD(iType,2).EQ.1) THEN                                       
         nLevels=nLevels+(3+NTRACO)*NL+1                                
      ENDIF                                                             
      IF (MOD(iType,4).GE.2) THEN                                       
         nLevels=nLevels+(3+NTRACO)*NL+1                                
      ENDIF                                                             
      IF (MOD(iType,8).GE.4) THEN                                       
         nLevels=nLevels+NL                                             
      ENDIF                                                             
!-----------------------------------------------------                  
! Data is read in through PE 0                                          
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         !-----------------------------------------------------         
         ! Allocate size of spectral arrays                             
         !-----------------------------------------------------         
         ALLOCATE(AllSpec(NWJ2,NHEM,nLevels))                           
         !-----------------------------------------------------         
         ! Read in the data                                             
         !-----------------------------------------------------         
         lAllocHtnet=.FALSE.                                            
 180     CONTINUE                                                       
         IF (lRestart) THEN                                             
            IF (iType.EQ.3) THEN                                        
               READ(ID,END=1001) RKOUNT,RM1TAPE,DAY,DOY,                
     &              AllSpec(:,:,1:nLevels/2),RM2TAPE,                   
     &              AllSpec(:,:,nLevels/2+1:nLevels),RM3TAPE            
            ELSE IF (iType.EQ.35) THEN                                  
               IF (.NOT.lAllocHtnet) THEN                               
                  ALLOCATE(HtnetFull(NHEM,JG,MG,NL))                    
                  lAllocHtnet=.TRUE.                                    
               ENDIF                                                    
               READ(ID,END=1001) RKOUNT,RM1TAPE,DAY,DOY,                
     &              AllSpec(:,:,1:nLevels/2),RM2TAPE,                   
     &              AllSpec(:,:,nLevels/2+1:nLevels),HtnetFull,         
     &              RM3TAPE                                             
            ELSE                                                        
               READ(ID,END=1001) RKOUNT,RM1TAPE,DAY,DOY,AllSpec,        
     &              RM2TAPE                                             
            ENDIF                                                       
            !-----------------------------------------------------      
            ! Extra read if restart record contains mass correction.    
            !-----------------------------------------------------      
            IF (LMASOLD) THEN                                           
               READ(ID,END=1001) RK,RMT1,RDAY,RREC,GMSP0,GMSPMI,RMT2    
               !-----------------------------------------------------   
               ! Check this data makes sense                            
               !-----------------------------------------------------   
               IF (RMT2.NE.RMT1) THEN                                   
                  WRITE(2,2110)                                         
                  CALL ABORT                                            
               ENDIF                                                    
            ENDIF                                                       
         ELSE                                                           
            IF (iType.EQ.3) THEN                                        
               READ(ID, '(4E20.12)',END=1001) RKOUNT,RM1TAPE,DAY,DOY,   
     &              AllSpec(:,:,1:nLevels/2),RM2TAPE,                   
     &              AllSpec(:,:,nLevels/2+1:nLevels),RM3TAPE            
            ELSE IF (iType.EQ.35) THEN                                  
               IF (.NOT.lAllocHtnet) THEN                               
                  ALLOCATE(HtnetFull(NHEM,JG,MG,NL))                    
                  lAllocHtnet=.TRUE.                                    
               ENDIF                                                    
               READ(ID, '(4E20.12)',END=1001) RKOUNT,RM1TAPE,DAY,DOY,   
     &              AllSpec(:,:,1:nLevels/2),RM2TAPE,                   
     &              AllSpec(:,:,nLevels/2+1:nLevels),HtnetFull,RM3TAPE  
            ELSE                                                        
               READ(ID, '(4E20.12)',END=1001) RKOUNT,RM1TAPE,DAY,       
     &              AllSpec,RM2TAPE                                     
            ENDIF                                                       
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Check the RM?TAPE variables are consistent                   
         !-----------------------------------------------------         
         IF (ABS(RM1TAPE-RM2TAPE) .GT. 1.0E-03) THEN                    
            WRITE(2,2010) ID                                            
            CALL ABORT                                                  
         ENDIF                                                          
         IF (MOD(iType,4).GE.2) THEN                                    
            IF (ABS(RM2TAPE-RM3TAPE) .GT. 1.0E-03) THEN                 
               WRITE(2,2020)                                            
               CALL ABORT                                               
            ENDIF                                                       
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Check we have right time of day                              
         !-----------------------------------------------------         
         IF (ABS(DAY-BEGDAY) .GT. 1.0E-02) THEN                         
            IF (ABS(DAY-BEGDAY) .LT. ABS(DAYNEAR-BEGDAY)) THEN          
               DAYNEAR = DAY                                            
            ENDIF                                                       
            GOTO 180                                                    
         ENDIF                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Share the coefficients across the other PEs                           
!-----------------------------------------------------                  
      ALLOCATE(HoldVar(NNTx,MMTy,NHEM,nLevels))                         
      HoldVar(:,:,:,:)=(0.0, 0.0)                                       
      CALL SHARECOEF(.TRUE.,(NHEM*nLevels),AllSpec,HoldVar)             
      IF (MyPe.EQ.0) DEALLOCATE(AllSpec)                                
!-----------------------------------------------------                  
! Write the spectral data into spectral arrays                          
!-----------------------------------------------------                  
      LL=0                                                              
      IF (MOD(iType,2).EQ.1) THEN                                       
         Z(1:NNTxx(iCol),1:MMTyy(iRow),:,:)=                            
     &        HoldVar(1:NNTxx(iCol),1:MMTyy(iRow),:,LL+1:LL+NL)         
         LL=LL+NL                                                       
         D(1:NNTxx(iCol),1:MMTyy(iRow),:,:)=                            
     &        HoldVar(1:NNTxx(iCol),1:MMTyy(iRow),:,LL+1:LL+NL)         
         LL=LL+NL                                                       
         T(1:NNTxx(iCol),1:MMTyy(iRow),:,:)=                            
     &        HoldVar(1:NNTxx(iCol),1:MMTyy(iRow),:,LL+1:LL+NL)         
         LL=LL+NL                                                       
         IF (NTRACO.GT.0) THEN                                          
            DO KK=1,NTRACO                                              
               TRA(1:NNTxx(iCol),1:MMTyy(iRow),:,:,KK)=                 
     &              HoldVar(1:NNTxx(iCol),1:MMTyy(iRow),:,LL+1:LL+NL)   
               LL=LL+NL                                                 
            ENDDO                                                       
         ENDIF                                                          
         SP(1:NNTxx(iCol),1:MMTyy(iRow),:)=                             
     &        HoldVar(1:NNTxx(iCol),1:MMTyy(iRow),:,LL+1)               
         LL=LL+1                                                        
      ENDIF                                                             
      IF (MOD(iType,4).GE.2) THEN                                       
         ZMI(1:NNTxx(iCol),1:MMTyy(iRow),:,:)=                          
     &        HoldVar(1:NNTxx(iCol),1:MMTyy(iRow),:,LL+1:LL+NL)         
         LL=LL+NL                                                       
         DMI(1:NNTxx(iCol),1:MMTyy(iRow),:,:)=                          
     &        HoldVar(1:NNTxx(iCol),1:MMTyy(iRow),:,LL+1:LL+NL)         
         LL=LL+NL                                                       
         TMI(1:NNTxx(iCol),1:MMTyy(iRow),:,:)=                          
     &        HoldVar(1:NNTxx(iCol),1:MMTyy(iRow),:,LL+1:LL+NL)         
         LL=LL+NL                                                       
         IF (NTRACO.GT.0) THEN                                          
            DO KK=1,NTRACO                                              
               TRAMI(1:NNTxx(iCol),1:MMTyy(iRow),:,:,KK)=               
     &              HoldVar(1:NNTxx(iCol),1:MMTyy(iRow),:,LL+1:LL+NL)   
               LL=LL+NL                                                 
            ENDDO                                                       
         ENDIF                                                          
         SPMI(1:NNTxx(iCol),1:MMTyy(iRow),:)=                           
     &        HoldVar(1:NNTxx(iCol),1:MMTyy(iRow),:,LL+1)               
      ENDIF                                                             
      IF (MOD(iType,8).GE.4) THEN                                       
         TTRES(1:NNTxx(iCol),1:MMTyy(iRow),:,:)=                        
     &        HoldVar(1:NNTxx(iCol),1:MMTyy(iRow),:,1:NL)               
      ENDIF                                                             
      IF (MOD(iType,64).GE.32) THEN                                     
         !-----------------------------------------------------         
         ! Share HTNET across the PEs                                   
         !-----------------------------------------------------         
         CALL SHAREGRID2(.TRUE.,NL,HtnetFull,HTNET)                     
         IF (MyPe.EQ.0) DEALLOCATE(HtnetFull)                           
      ENDIF                                                             
!-----------------------------------------------------                  
! Variables that need passing to all PEs                                
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         CALL MPI_BARRIER(MPI_COMM_WORLD,ierr)                          
         iSend=0                                                        
         nBuffSize=1                                                    
         CALL MPI_BCAST(RKOUNT,nBuffSize,ParaReal,iSend,                
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(DAY,nBuffSize,ParaReal,iSend,                   
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(DOY,nBuffSize,ParaReal,iSend,                   
     &        MPI_COMM_WORLD,ierr)                                      
         CALL MPI_BCAST(DAYNEAR,nBuffSize,ParaReal,iSend,               
     &              MPI_COMM_WORLD,ierr)                                
      ENDIF                                                             
!-----------------------------------------------------                  
! Deallocate arrays                                                     
!-----------------------------------------------------                  
      DEALLOCATE(HoldVar)                                               
!-----------------------------------------------------                  
! Leave routine                                                         
!-----------------------------------------------------                  
      RETURN                                                            
!-----------------------------------------------------                  
! Error if we get to hear - reached end of file before reading in       
! all the data                                                          
!-----------------------------------------------------                  
 1001 WRITE(2,2031) BEGDAY,DAYNEAR                                      
      CALL ABORT                                                        
      END                                                               
!=====================================================                  
! Set up restoration state from the KOUNT=0 zonally averaged            
! state and write this to FT13 for future use.                          
! This is only done when KOUNT=0 and DAMP.GT.0.0.                       
!=====================================================                  
      SUBROUTINE SETRES                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Restoration fields and timescale                                      
!-----------------------------------------------------                  
      COMMON/RESTOR/ZRES,DRES,TRES,SPRES,DAMP                           
      REAL :: ZRES(IDM,NHEM,NL)                                         
      REAL :: DRES(IDM,NHEM,NL)                                         
      REAL :: TRES(IDM,NHEM,NL)                                         
      REAL :: SPRES(IDM,NHEM)                                           
      REAL :: DAMP                                                      
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: HoldVar(NNTx,NHEM,3*NL+1)                                 
      REAL, ALLOCATABLE :: AllRes(:,:,:)                                
      INTEGER, PARAMETER :: TagRef=900                                  
      INTEGER :: iCol2                                                  
! Loop variables                                                        
      INTEGER :: IHEM, J, L                                             
!-----------------------------------------------------                  
! Format statment                                                       
!-----------------------------------------------------                  
 2200 FORMAT(/' RESTORATION RECORD WRITTEN TO CHANNEL ',I3)             
!-----------------------------------------------------                  
! Only continue with code if DAMP > 0.                                  
!-----------------------------------------------------                  
      IF (DAMP.LE.0) RETURN                                             
!-----------------------------------------------------                  
! First zonal wavenumber is only stored on iRow=1                       
!-----------------------------------------------------                  
      IF (iRow.EQ.1) THEN                                               
         DO IHEM=1,NHEM                                                 
            DO J=mTwin1(1,1)+1,mTwin1(2,1)                              
               SPRES(J,IHEM)=SP(J,1,IHEM)                               
            ENDDO                                                       
            DO L=1,NL                                                   
               DO J=mTwin1(1,1)+1,mTwin1(2,1)                           
                  ZRES(J,IHEM,L)=Z(J,1,IHEM,L)                          
                  DRES(J,IHEM,L)=D(J,1,IHEM,L)                          
                  TRES(J,IHEM,L)=T(J,1,IHEM,L)                          
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
                                                                        
         !-----------------------------------------------------         
         ! Store all the data that needs passing to PE 0                
         !-----------------------------------------------------         
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO J=mTwin1(1,1)+1,mTwin1(2,1)                           
                  HoldVar(J,IHEM,L)=ZRES(J,IHEM,L)                      
                  HoldVar(J,IHEM,NL+L)=DRES(J,IHEM,L)                   
                  HoldVar(J,IHEM,2*NL+L)=TRES(J,IHEM,L)                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO IHEM=1,NHEM                                                 
            DO J=mTwin1(1,1)+1,mTwin1(2,1)                              
               HoldVar(J,IHEM,3*NL+1)=SPRES(J,IHEM)                     
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Send ZRES, etc to PE 0                                       
         !-----------------------------------------------------         
         IF (MYPE.GT.0) THEN                                            
            iRecv=0                                                     
            nBuffSize=NNTx*NHEM*(3*NL+1)                                
            iTag=TagRef+MyPe                                            
            CALL MPI_BSEND(HoldVar,nBuffSize,ParaReal,iRecv,iTag,       
     &           MPI_COMM_WORLD,ierr)                                   
         ELSE                                                           
            !-----------------------------------------------------      
            ! Loop through PEs on row 1                                 
            !-----------------------------------------------------      
            ALLOCATE(AllRes(NNT,NHEM,3*NL+1))                           
            DO iCol2=1,Px                                               
               !-----------------------------------------------------   
               ! Receive the data                                       
               !-----------------------------------------------------   
               IF (iCol2.GT.1) THEN                                     
                  iSend=PeGrid(iCol2,1)                                 
                  nBuffSize=NNTx*NHEM*(3*NL+1)                          
                  iTag=TagRef+iSend                                     
                  CALL MPI_RECV(HoldVar,nBuffSize,ParaReal,iSend,       
     &                 iTag,MPI_COMM_WORLD,istat,ierr)                  
               ENDIF                                                    
               !-----------------------------------------------------   
               ! Write data into array                                  
               !-----------------------------------------------------   
               DO L=1,3*NL+1                                            
                  DO IHEM=1,NHEM                                        
                     DO J=mTwin1(1,1)+1,mTwin1(2,1)                     
                        AllRes(NNT1(iCol2)+J,IHEM,L)=HoldVar(J,IHEM,L)  
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Write data to restoration file                            
            !-----------------------------------------------------      
            WRITE(13) AllRes                                            
            DEALLOCATE(AllRes)                                          
            WRITE(2,2200)13                                             
         ENDIF                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Subroutine to give annual cycle of TRES if wanted                     
!=====================================================                  
      SUBROUTINE SETTEE                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Switches counters and constants controlling type and frequency of     
! model output                                                          
!-----------------------------------------------------                  
      COMMON/OUTCON/RNTAPE,DAY,SQR2,RSQR2,EAM1,EAM2,TOUT1               
     &     ,TOUT2,LSPO,LGPO                                             
     &     ,RNTAPO                                                      
     &     ,LSHIST,LMINIH                                               
      COMMON/OUTCONINT/NCOEFF,NLAT,INLAT,INSPC,KOUNTP                   
     &                 ,KOUNTE,KOUNTH,KOUNTR,KOUTP                      
     &                 ,KOUTE,KOUTH,KOUTR                               
! Identyfying run number                                                
      REAL :: RNTAPE                                                    
! Maximum wavenumber of printed spectral coeffecients                   
      INTEGER :: NCOEFF                                                 
! Number of latitudes in printed grid point fields                      
      INTEGER :: NLAT                                                   
      INTEGER :: INLAT                                                  
!                                                                       
      INTEGER :: INSPC                                                  
! Number of timsteps between successive writing of spectral             
! and gridded fields                                                    
      INTEGER :: KOUNTP                                                 
      INTEGER :: KOUNTE                                                 
! Number of timsteps between successive writing of history              
! records, containing spectral fields at the present and                
! previous time levels.                                                 
      INTEGER :: KOUNTH                                                 
! Number of timsteps between successive writing of restart              
! records, containing the spectral fields at the present and            
! previous time levels                                                  
      INTEGER :: KOUNTR                                                 
      INTEGER :: KOUTP                                                  
      INTEGER :: KOUTE                                                  
      INTEGER :: KOUTH                                                  
      INTEGER :: KOUTR                                                  
      REAL :: DAY                                                       
      REAL :: SQR2                                                      
      REAL :: RSQR2                                                     
      REAL :: EAM1                                                      
      REAL :: EAM2                                                      
      REAL :: TOUT1                                                     
      REAL :: TOUT2                                                     
      LOGICAL :: LSPO(NL)                                               
      LOGICAL :: LGPO(NL)                                               
      REAL :: RNTAPO                                                    
      LOGICAL :: LSHIST                                                 
! Reduce the history file to single precision if true                   
      LOGICAL :: LMINIH                                                 
!-----------------------------------------------------                  
! Restoration temperature field and constants which determine it,       
! also contains timescales                                              
!-----------------------------------------------------                  
      COMMON/RESTIJ/TTRES                                               
     + ,DTNS,DTEP,DTTRP,FAC,DDAMP,TFRC,YRLEN,TRS                        
     +  ,ALR,ZTROP,TGR                                                  
      COMPLEX :: TTRES(NNTx,MMTy,NHEM,NL)                               
      REAL :: DTNS                                                      
      REAL :: DTEP                                                      
      REAL :: DTTRP                                                     
      REAL :: FAC(NL)                                                   
      REAL :: DDAMP(NL)                                                 
      REAL :: TFRC(NL)                                                  
      REAL :: YRLEN                                                     
      REAL :: TRS(NL)                                                   
      REAL :: ALR                                                       
      REAL :: ZTROP                                                     
      REAL :: TGR                                                       
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: YPHS                                                      
! Loop variables                                                        
      INTEGER :: L                                                      
!-----------------------------------------------------                  
! If YRLEN is zero then no seasonal cycle.                              
!-----------------------------------------------------                  
      IF (NINT(YRLEN) .EQ. 0) THEN                                      
         YPHS = (1./SQRT(6.))                                           
      ELSE                                                              
         YPHS=(1./SQRT(6.))*SIN(PI2*DAY/YRLEN)                          
      ENDIF                                                             
!-----------------------------------------------------                  
! NOT SURE THIS IS RIGHT. Seems consistent with old code                
! (commented out below), but I don't understand why it should           
! depend on the number of hemispheres. Marc July 2009.                  
!      IADB=NWJ2+1                                                      
!      DO 10 L=1,NL                                                     
!        IAD=IADB+(L-1)*IGA                                             
!        TTRES(IAD)=FAC(L)*DTNS*YPHS                                    
!10    CONTINUE                                                         
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         IF (NHEM.EQ.1) THEN                                            
            DO L=2,NL                                                   
               TTRES(1,1,1,L)=FAC(L)*DTNS*YPHS                          
            ENDDO                                                       
         ELSE                                                           
            DO L=1,NL                                                   
               TTRES(1,1,NHEM,L)=FAC(L)*DTNS*YPHS                       
            ENDDO                                                       
         ENDIF                                                          
      ENDIF                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! This subroutine sets up restoration temperature field.                
! The temperature at SIGMA = 1 is TGR, entered in Kelvin.               
! a lapse rate of ALR k/m is assumed under the tropopause and           
! zero above. The actual profile tends to this away from the            
! tropopause, with smooth interpolation depending on DTTRP              
! at the model tropopause. The height of                                
! the tropopause is given as ZTROP m.                                   
!=====================================================                  
      SUBROUTINE SETZT                                                  
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Restoration temperature field and constants which determine it,       
! also contains timescales                                              
!-----------------------------------------------------                  
      COMMON/RESTIJ/TTRES                                               
     + ,DTNS,DTEP,DTTRP,FAC,DDAMP,TFRC,YRLEN,TRS                        
     +  ,ALR,ZTROP,TGR                                                  
      COMPLEX :: TTRES(NNTx,MMTy,NHEM,NL)                               
      REAL :: DTNS                                                      
      REAL :: DTEP                                                      
      REAL :: DTTRP                                                     
      REAL :: FAC(NL)                                                   
      REAL :: DDAMP(NL)                                                 
      REAL :: TFRC(NL)                                                  
      REAL :: YRLEN                                                     
      REAL :: TRS(NL)                                                   
      REAL :: ALR                                                       
      REAL :: ZTROP                                                     
      REAL :: TGR                                                       
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: SIGPREV                                                   
      REAL :: TPREV                                                     
      REAL :: ZPREV                                                     
      REAL :: ZP, TP                                                    
      REAL :: TPM                                                       
      REAL :: ZPP, TPP                                                  
! Loop variables                                                        
      INTEGER :: IHEM, IM, IN, L                                        
!-----------------------------------------------------                  
! Zero TTRES array                                                      
!-----------------------------------------------------                  
      DO L=1,NL                                                         
         DO IHEM=1,NHEM                                                 
            DO IM=1,MMTyy(iRow)                                         
               DO IN=1,NNTxx(iCol)                                      
                  TTRES(IN,IM,IHEM,L)=(0.0,0.0)                         
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Calculate TRS                                                         
!-----------------------------------------------------                  
      DTTRP=DTTRP*CT                                                    
      SIGPREV=1.                                                        
      TPREV=TGR                                                         
      ZPREV=0.                                                          
      DO L=NL,1,-1                                                      
         ZP=ZPREV+(GASCON*TPREV/GA)*LOG(SIGPREV/SIGMA(L))               
         TP=TGR-ZTROP*ALR                                               
         TP=TP+SQRT((.5*ALR*(ZP-ZTROP))**2+DTTRP**2)                    
         TP=TP-.5*ALR*(ZP-ZTROP)                                        
         TPM=.5*(TPREV+TP)                                              
         ZPP=ZPREV+(GASCON*TPM/GA)*LOG(SIGPREV/SIGMA(L))                
         TPP=TGR-ZTROP*ALR                                              
         TPP=TPP+SQRT((.5*ALR*(ZPP-ZTROP))**2+DTTRP**2)                 
         TPP=TPP-.5*ALR*(ZPP-ZTROP)                                     
         TRS(L)=TPP                                                     
         ZPREV=ZPREV+(.5*(TPP+TPREV)*GASCON/GA)*LOG(SIGPREV/SIGMA(L))   
         TPREV=TPP                                                      
         SIGPREV=SIGMA(L)                                               
      ENDDO                                                             
                                                                        
      IF (MYPE.EQ.0) THEN                                               
         WRITE(2,2000)                                                  
         WRITE(2,2010) TRS                                              
      ENDIF                                                             
 2000 FORMAT(/' RESTORATION TEMPERATURE STRATIFICATION IN K ')          
 2010 FORMAT(10F7.2)                                                    
C                                                                       
      DO L=1,NL                                                         
         TRS(L)=TRS(L)/CT                                               
      ENDDO                                                             
!-----------------------------------------------------                  
! Now the latitudinal variation in TTRES is set up                      
! (this being in terms of a deviation from T0 which                     
! is usually constant with height)                                      
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         DO L=1,NL                                                      
            TTRES(1,1,1,L)=SQRT(2.)*(TRS(L)-T0(L))                      
            TTRES(2,1,1,L)=-2./3.*SQRT(0.4)*DTEP*FAC(L)                 
         ENDDO                                                          
         IF (NHEM.EQ.2) THEN                                            
            DO L=1,NL                                                   
               TTRES(1,1,NHEM,L)=(1./SQRT(6.))*DTNS*FAC(L)              
            ENDDO                                                       
         ENDIF                                                          
      ENDIF                                                             
      DTTRP=DTTRP/CT                                                    
      END                                                               
!=====================================================                  
! Subroutine to find new T and Q based on DOY and climatology           
!=====================================================                  
      SUBROUTINE SFCT(IFIRST, TROPHT)                                   
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Orogrophy                                                             
!-----------------------------------------------------                  
      COMMON/GSG/GSG,TOPVAR,RLEVBZMO,FXZMO,ofrac,runoff,iareanum        
! Topog variance, GW diags for Orog, Non orog waves                     
      REAL :: TOPVAR(MGx,NHEM,JGy)                                      
      REAL :: RLEVBZMO(JGy,NHEM,NL)                                     
      REAL :: FXZMO(JGy,NHEM,NL)                                        
         REAL :: GSG(MGx,NHEM,JGy)                                      
         REAL :: ofrac(MGx,NHEM,JGy)                                    
         INTEGER :: runoff(50,2,33)                                     
         INTEGER :: iareanum                                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Subroutine arguments                                                  
!-----------------------------------------------------                  
      INTEGER :: IFIRST                                                 
      REAL :: tropht(MGx,nhem,JGy)                                      
!-----------------------------------------------------                  
! External function                                                     
!-----------------------------------------------------                  
      REAL, EXTERNAL :: PQSAT                                           
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: TSTAR1(MGx,NHEM,JGy),TSTAR2(MGx,NHEM,JGy)                 
      integer :: cmth                                                   
      character*3 :: mn(13)                                             
      character*31 :: sstfile                                           
      integer :: mnlen(13)                                              
      INTEGER :: MTH1                                                   
      REAL :: RHSCL                                                     
      REAL :: AMFRAC                                                    
      REAL :: ESCON                                                     
      INTEGER :: MTH2                                                   
      REAL :: ClimData(5,MGx,NHEM,JGy)                                  
      REAL :: SST, SSQ, SSP, SHDUM                                      
! Loop variables                                                        
      INTEGER :: I, IHEM, J, K                                          
!-----------------------------------------------------                  
! Set and store some values                                             
!-----------------------------------------------------                  
      DATA MTH1/0/                                                      
      SAVE MTH1, TSTAR1, TSTAR2                                         
      data mn/'dec','jan','feb','mar','apr','may','jun','jul','aug'     
     &        ,'sep','oct','nov','dec'/                                 
C OASIS size parameters                                                 
      INTEGER, PARAMETER :: IMB = 128                                   
      INTEGER, PARAMETER :: JM = 64                                     
      INTEGER, PARAMETER :: IMJM = IMB*JM                               
!-----------------------------------------------------                  
! Set constant and determine the current month                          
!-----------------------------------------------------                  
      RHSCL=288.0*GASCON/GA                                             
      CALL CALNDR(DOY,MTH2,AMFRAC)                                      
!-----------------------------------------------------                  
! First time around need to read in climatology data                    
!-----------------------------------------------------                  
      IF (IFIRST.EQ.1) THEN                                             
         IF (MyPe.EQ.0) THEN                                            
            cmth=mth2+1                                                 
            !-----------------------------------------------------      
            ! Wrap-around to January.                                   
            !-----------------------------------------------------      
            IF (cmth.eq.13) cmth=1                                      
            !-----------------------------------------------------      
            ! Form the file name                                        
            !-----------------------------------------------------      
            write(sstfile,'(a,i3.3,a,i3.3,a)')                          
     &           'tqpap_'//mn(cmth)//'_jg',jg,'_mg',mg,'_nhem2.dat'     
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Open file and read in the data                               
         !-----------------------------------------------------         
         CALL READCLIM(sstfile,5,.FALSE.,ClimData)                      
         !-----------------------------------------------------         
         ! Loop over all grid points                                    
         !-----------------------------------------------------         
         DO J=1,JGyy(iRow)                                              
            DO IHEM=1,NHEM                                              
               DO I=1,MGxx(iCol)                                        
                  tstar2(I,IHEM,J)=ClimData(1,I,IHEM,J)/ct              
                  tropht(I,IHEM,J)=ClimData(5,I,IHEM,J)                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Wont need to read in the data next time                      
         !-----------------------------------------------------         
         MTH1=CMTH                                                      
      ENDIF  ! IFIRST                                                   
!-----------------------------------------------------                  
! Check if a new month a climatology data is needed                     
!-----------------------------------------------------                  
      IF (MTH1.NE.MTH2) THEN                                            
         !-----------------------------------------------------         
         ! Transfer tstar2 to tstar1                                    
         !-----------------------------------------------------         
         DO J=1,JGyy(iRow)                                              
            DO IHEM=1,NHEM                                              
               DO I=1,MGxx(iCol)                                        
                  tstar1(I,IHEM,J)=tstar2(I,IHEM,J)                     
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Move on month and wrap-around to January.                    
         !-----------------------------------------------------         
         IF (MyPe.EQ.0) THEN                                            
            cmth=mth2+1                                                 
            !-----------------------------------------------------      
            ! Wrap-around to January.                                   
            !-----------------------------------------------------      
            IF (cmth.eq.13) cmth=1                                      
            !-----------------------------------------------------      
            ! Form the file name                                        
            !-----------------------------------------------------      
            write(sstfile,'(a,i3.3,a,i3.3,a)')                          
     &           'tqpap_'//mn(cmth+1)//'_jg',jg,'_mg',mg,'_nhem2.dat'   
         ENDIF                                                          
         !-----------------------------------------------------         
         ! Open file and read in the data                               
         !-----------------------------------------------------         
         CALL READCLIM(sstfile,5,.TRUE.,ClimData)                       
         !-----------------------------------------------------         
         ! Loop over all grid points                                    
         !-----------------------------------------------------         
         DO J=1,JGyy(iRow)                                              
            DO IHEM=1,NHEM                                              
               DO I=1,MGxx(iCol)                                        
                  tstar2(I,IHEM,J)=ClimData(1,I,IHEM,J)/ct              
                  tropht(I,IHEM,J)=ClimData(5,I,IHEM,J)                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Set the old month to current month                           
         !-----------------------------------------------------         
         MTH1=MTH2                                                      
      ENDIF                     ! MTH2.NE.MTH1                          
!-----------------------------------------------------                  
! Set TSTAR, QSTAR and SQSTAR                                           
!-----------------------------------------------------                  
      IF (.NOT.LSL) then                                                
         DO J=1,JGyy(iRow)                                              
            DO IHEM=1,NHEM                                              
               DO I=1,MGxx(iCol)                                        
                  IF (gsg(I,IHEM,j).gt.0.) then                         
                     tstar(I,IHEM,j)=tstar2(I,IHEM,j)*amfrac+           
     &                    tstar1(I,IHEM,j)*(1.0-amfrac)                 
                     IF (LPERPET) THEN                                  
                        escon=1./exp(-gsg(I,IHEM,j)/RHSCL)              
                     ELSE                                               
                        escon=1./plg3(I,IHEM,J)                         
                     ENDIF                                              
                     sqstar(I,IHEM,j)=escon*pqsat(tstar(I,IHEM,j))      
                     qstar(I,IHEM,j)=sqstar(I,IHEM,j)*0.75              
                  endif                                                 
               ENDDO                                                    
            enddo                                                       
         enddo                                                          
      endif                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! Share coefficients between processor PE 0 and the other               
! processors                                                            
!=====================================================                  
      SUBROUTINE SHARECOEF(lShare,nLevels,LargeArray,SmallArray)        
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variables                                                      
!-----------------------------------------------------                  
! lShare=TRUE: data is being shared from PE 0 to other PEs              
! lShare=FALSE: all the data to sent to PE 0                            
      LOGICAL :: lShare                                                 
! Number of levels                                                      
      INTEGER :: nLevels                                                
! Large array containing all the coefficients stored in the             
! original way                                                          
      COMPLEX :: LargeArray(NWJ2, nLevels)                              
! Typically smaller array with coefficients stored for use              
! in model and shared across PEs.                                       
      COMPLEX :: SmallArray(NNTx,MMTy,nLevels)                          
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      COMPLEX :: SmallArray2(NNTx,MMTy,nLevels)                         
      INTEGER, PARAMETER :: TagRef=1000                                 
      INTEGER :: iCol2, iRow2                                           
! Loop variables                                                        
      INTEGER :: I, IM, IN, iPe, L                                      
!-----------------------------------------------------                  
! Determine if data is leaving PE 0 or coming to it                     
!-----------------------------------------------------                  
      IF (lShare) THEN                                                  
         !-----------------------------------------------------         
         ! Data will be sent from PE 0. Determine if this is PE 0       
         !-----------------------------------------------------         
         IF (MyPe.EQ.0) THEN                                            
            !-----------------------------------------------------      
            ! Loop through the PEs                                      
            !-----------------------------------------------------      
            DO iPe=NPE-1,0,-1                                           
               iCol2=GRID(1,iPe)                                        
               iRow2=GRID(2,iPe)                                        
               !-----------------------------------------------------   
               ! Rearrange the zonal wavenumbers                        
               !-----------------------------------------------------   
               DO L=1,nLevels                                           
                  DO IM=1,MMTyy(iRow2)                                  
                     DO IN=1,NNTxx(iCol2)                               
                        SmallArray(IN,IM,L)=                            
     &                       LargeArray(CoefConv(IN,IM,iPe),L)          
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Pass to PE iPe                                         
               !-----------------------------------------------------   
               IF (iPe.GT.0) THEN                                       
                  iRecv=iPe                                             
                  nBuffSize=NNTx*MMTy*nLevels                           
                  iTag=TagRef+iPe                                       
                  CALL MPI_BSEND(SmallArray, nBuffSize, ParaCom, iRecv, 
     &                 iTag, MPI_COMM_WORLD, ierr)                      
               ENDIF                                                    
            ENDDO                                                       
         ENDIF                                                          
         IF (NPE.GT.1) CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)           
         IF (MyPe.GT.0) THEN                                            
            !-----------------------------------------------------      
            ! Receive data from PE 0                                    
            !-----------------------------------------------------      
            iSend=0                                                     
            nBuffSize=NNTx*MMTy*nLevels                                 
            iTag=TagRef+MyPe                                            
            CALL MPI_RECV(SmallArray, nBuffSize, ParaCom, iSend,        
     &           iTag, MPI_COMM_WORLD, istat, ierr)                     
         ENDIF                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! Data will be sent to PE 0. Determine if this is PE 0         
         !-----------------------------------------------------         
         IF (MyPe.GT.0) THEN                                            
            !-----------------------------------------------------      
            ! Send data to PE 0                                         
            !-----------------------------------------------------      
            iRecv=0                                                     
            nBuffSize=NNTx*MMTy*nLevels                                 
            iTag=TagRef+MyPe                                            
            CALL MPI_BSEND(SmallArray, nBuffSize, ParaCom, iRecv,       
     &           iTag, MPI_COMM_WORLD, ierr)                            
         ENDIF                                                          
         IF (NPE.GT.1) CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)           
         IF (MyPe.EQ.0) THEN                                            
            !-----------------------------------------------------      
            ! Loop through the PEs                                      
            !-----------------------------------------------------      
            DO iPe=0,NPE-1                                              
               IF (iPe.EQ.0) THEN                                       
                  !---------------------------------------------------  
                  ! This data is already contained on PE 0              
                  !---------------------------------------------------  
                  DO L=1,nLevels                                        
                     DO IM=1,MMTyy(iRow)                                
                        DO IN=1,NNTxx(iCol)                             
                           LargeArray(CoefConv(IN,IM,iPe),L)=           
     &                          SmallArray(IN,IM,L)                     
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ELSE                                                     
                  !---------------------------------------------------  
                  ! Receive data from the other PEs                     
                  !---------------------------------------------------  
                  iSend=iPe                                             
                  nBuffSize=NNTx*MMTy*nLevels                           
                  iTag=TagRef+iPe                                       
                  CALL MPI_RECV(SmallArray2, nBuffSize, ParaCom, iSend, 
     &                 iTag, MPI_COMM_WORLD, istat, ierr)               
                  !---------------------------------------------------  
                  ! Rearrange the zonal wavenumbers                     
                  !---------------------------------------------------  
                  iCol2=GRID(1,iPe)                                     
                  iRow2=GRID(2,iPe)                                     
                  DO L=1,nLevels                                        
                     DO IM=1,MMTyy(iRow2)                               
                        DO IN=1,NNTxx(iCol2)                            
                           LargeArray(CoefConv(IN,IM,iPe),L)=           
     &                          SmallArray2(IN,IM,L)                    
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDIF                                                    
            ENDDO                                                       
         ENDIF                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Share grid value between processor PE 0 and the other                 
! processors                                                            
!=====================================================                  
      SUBROUTINE SHAREGRID(lShare,xDim,nLevels,LargeArray,SmallArray)   
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variables                                                      
!-----------------------------------------------------                  
! lShare=TRUE: data is being shared from PE 0 to other PEs              
! lShare=FALSE: all the data to sent to PE 0                            
      LOGICAL :: lShare                                                 
! Length of x-axis for LargeArray (should be MG or MGPP)                
      INTEGER :: xDim                                                   
! Number of levels                                                      
      INTEGER :: nLevels                                                
! Large array containing all the coefficients stored in the             
! original way                                                          
      REAL :: LargeArray(xDim, NHEM, JG, nLevels)                       
! Typically smaller array with coefficients stored for use              
! in model and shared across PEs.                                       
      REAL :: SmallArray(MGx, NHEM, JGy, nLevels)                       
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: SmallArray2(MGx, NHEM, JGy, nLevels)                      
      INTEGER, PARAMETER :: TagRef=1100                                 
      INTEGER :: iCol2, iRow2                                           
! Loop variables                                                        
      INTEGER :: I, IHEM, iPe, J, L                                     
!-----------------------------------------------------                  
! No data need passing for one processor                                
!-----------------------------------------------------                  
      IF (NPE.EQ.1) THEN                                                
         IF (lShare) THEN                                               
            DO L=1,nLevels                                              
               DO J=1,JG                                                
                  DO IHEM=1,NHEM                                        
                     DO I=1,MGx                                         
                        SmallArray(I,IHEM,J,L)=LargeArray(I,IHEM,J,L)   
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ELSE                                                           
            DO L=1,nLevels                                              
               DO J=1,JG                                                
                  DO IHEM=1,NHEM                                        
                     DO I=1,MGx                                         
                        LargeArray(I,IHEM,J,L)=SmallArray(I,IHEM,J,L)   
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDIF                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! Determine if data is leaving PE 0 or coming to it            
         !-----------------------------------------------------         
         IF (lShare) THEN                                               
            !-----------------------------------------------------      
            ! Data will be sent from PE 0. Determine if this is PE 0    
            !-----------------------------------------------------      
            IF (MyPe.EQ.0) THEN                                         
               !-----------------------------------------------------   
               ! Loop through the PEs                                   
               !-----------------------------------------------------   
               DO iPe=NPE-1,0,-1                                        
                  iCol2=GRID(1,iPe)                                     
                  iRow2=GRID(2,iPe)                                     
                  !---------------------------------------------------  
                  ! Store the values needed for PE iPe                  
                  !---------------------------------------------------  
                  DO L=1,nLevels                                        
                     DO J=1,JGyy(iRow2)                                 
                        DO IHEM=1,NHEM                                  
                           DO I=1,MGxx(iCol2)                           
                              SmallArray(I,IHEM,J,L)=                   
     &                             LargeArray(II1(iCol2)+I,IHEM,        
     &                             JJ1(iRow2)+J,L)                      
                           ENDDO                                        
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
                  !---------------------------------------------------  
                  ! Pass to PE iPe                                      
                  !---------------------------------------------------  
                  IF (iPe.GT.0) THEN                                    
                     iRecv=iPe                                          
                     nBuffSize=MGx*NHEM*JGy*nLevels                     
                     iTag=TagRef+iPe                                    
                     CALL MPI_BSEND(SmallArray, nBuffSize, ParaReal,    
     &                    iRecv, iTag, MPI_COMM_WORLD, ierr)            
                  ENDIF                                                 
               ENDDO                                                    
            ENDIF                                                       
            CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                      
            IF (MyPe.GT.0) THEN                                         
               !-----------------------------------------------------   
               ! Receive data from PE 0                                 
               !-----------------------------------------------------   
               iSend=0                                                  
               nBuffSize=MGx*NHEM*JGy*nLevels                           
               iTag=TagRef+MyPe                                         
               CALL MPI_RECV(SmallArray, nBuffSize, ParaReal, iSend,    
     &              iTag, MPI_COMM_WORLD, istat, ierr)                  
            ENDIF                                                       
         ELSE                                                           
            !-----------------------------------------------------      
            ! Data will be sent to PE 0. Determine if this is PE 0      
            !-----------------------------------------------------      
            IF (MyPe.GT.0) THEN                                         
               !-----------------------------------------------------   
               ! Send data to PE 0                                      
               !-----------------------------------------------------   
               iRecv=0                                                  
               nBuffSize=MGx*NHEM*JGy*nLevels                           
               iTag=TagRef+MyPe                                         
               CALL MPI_BSEND(SmallArray, nBuffSize, ParaReal, iRecv,   
     &              iTag, MPI_COMM_WORLD, ierr)                         
            ENDIF                                                       
            CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                      
            IF (MyPe.EQ.0) THEN                                         
               !-----------------------------------------------------   
               ! Loop through the PEs                                   
               !-----------------------------------------------------   
               DO iPe=0,NPE-1                                           
                  IF (iPe.EQ.0) THEN                                    
                     !------------------------------------------------  
                     ! Data already stored on PE 0                      
                     !------------------------------------------------  
                     DO L=1,nLevels                                     
                        DO J=1,JGyy(iRow)                               
                           DO IHEM=1,NHEM                               
                              DO I=1,MGxx(iCol)                         
                                 LargeArray(II1(iCol)+I,IHEM,           
     &                                JJ1(iRow)+J,L)=                   
     &                                SmallArray(I,IHEM,J,L)            
                              ENDDO                                     
                           ENDDO                                        
                        ENDDO                                           
                     ENDDO                                              
                  ELSE                                                  
                     !------------------------------------------------  
                     ! Receive data from the other PEs                  
                     !------------------------------------------------  
                     iSend=iPe                                          
                     nBuffSize=MGx*NHEM*JGy*nLevels                     
                     iTag=TagRef+iPe                                    
                     CALL MPI_RECV(SmallArray2, nBuffSize, ParaReal,    
     &                    iSend, iTag, MPI_COMM_WORLD, istat, ierr)     
                     !------------------------------------------------  
                     ! Write values into large array                    
                     !------------------------------------------------  
                     iCol2=GRID(1,iPe)                                  
                     iRow2=GRID(2,iPe)                                  
                     DO L=1,nLevels                                     
                        DO J=1,JGyy(iRow2)                              
                           DO IHEM=1,NHEM                               
                              DO I=1,MGxx(iCol2)                        
                                 LargeArray(II1(iCol2)+I,IHEM,          
     &                                JJ1(iRow2)+J,L)=                  
     &                                SmallArray2(I,IHEM,J,L)           
                              ENDDO                                     
                           ENDDO                                        
                        ENDDO                                           
                     ENDDO                                              
                  ENDIF                                                 
               ENDDO                                                    
            ENDIF                                                       
         ENDIF                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Like subroutine SHAREGRID but the order of the dimensions are         
! different.                                                            
!=====================================================                  
      SUBROUTINE SHAREGRID2(lShare,nLevels,LargeArray,SmallArray)       
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variables                                                      
!-----------------------------------------------------                  
! lShare=TRUE: data is being shared from PE 0 to other PEs              
! lShare=FALSE: all the data to sent to PE 0                            
      LOGICAL :: lShare                                                 
! Number of levels                                                      
      INTEGER :: nLevels                                                
! Large array containing all the coefficients stored in the             
! original way                                                          
      REAL :: LargeArray(NHEM, JG, MG, nLevels)                         
! Typically smaller array with coefficients stored for use              
! in model and shared across PEs.                                       
      REAL :: SmallArray(NHEM, JGy, MGx, nLevels)                       
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=1200                                 
      INTEGER :: iCol2, iRow2                                           
      REAL :: SmallArray2(NHEM, JGy, MGx, nLevels)                      
! Loop variables                                                        
      INTEGER :: I, IHEM, iPe, J, L                                     
!-----------------------------------------------------                  
! No data need passing for one processor                                
!-----------------------------------------------------                  
      IF (NPE.EQ.1) THEN                                                
         IF (lShare) THEN                                               
            DO L=1,nLevels                                              
               DO J=1,JG                                                
                  DO IHEM=1,NHEM                                        
                     DO I=1,MGx                                         
                        SmallArray(IHEM,J,I,L)=LargeArray(IHEM,J,I,L)   
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ELSE                                                           
            DO L=1,nLevels                                              
               DO J=1,JG                                                
                  DO IHEM=1,NHEM                                        
                     DO I=1,MGx                                         
                        LargeArray(IHEM,J,I,L)=SmallArray(IHEM,J,I,L)   
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDIF                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! Determine if data is leaving PE 0 or coming to it            
         !-----------------------------------------------------         
         IF (lShare) THEN                                               
            !-----------------------------------------------------      
            ! Data will be sent from PE 0. Determine if this is PE 0    
            !-----------------------------------------------------      
            IF (MyPe.EQ.0) THEN                                         
               !-----------------------------------------------------   
               ! Loop through the PEs                                   
               !-----------------------------------------------------   
               DO iPe=NPE-1,0,-1                                        
                  !---------------------------------------------------  
                  ! Store the values needed for PE iPe                  
                  !---------------------------------------------------  
                  iCol2=GRID(1,iPe)                                     
                  iRow2=GRID(2,iPe)                                     
                  DO L=1,nLevels                                        
                     DO J=1,JGyy(iRow2)                                 
                        DO IHEM=1,NHEM                                  
                           DO I=1,MGxx(iCol2)                           
                              SmallArray(IHEM,J,I,L)=                   
     &                             LargeArray(IHEM,JJ1(iRow2)+J,        
     &                             II1(iCol2)+I,L)                      
                           ENDDO                                        
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
                  !---------------------------------------------------  
                  ! Pass to PE iPe                                      
                  !---------------------------------------------------  
                  IF (iPe.GT.0) THEN                                    
                     iRecv=iPe                                          
                     nBuffSize=MGx*NHEM*JGy*nLevels                     
                     iTag=TagRef+iPe                                    
                     CALL MPI_BSEND(SmallArray, nBuffSize, ParaReal,    
     &                    iRecv, iTag, MPI_COMM_WORLD, ierr)            
                  ENDIF                                                 
               ENDDO                                                    
            ENDIF                                                       
            CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                      
            IF (MyPe.GT.0) THEN                                         
               !-----------------------------------------------------   
               ! Receive data from PE 0                                 
               !-----------------------------------------------------   
               iSend=0                                                  
               nBuffSize=MGx*NHEM*JGy*nLevels                           
               iTag=TagRef+MyPe                                         
               CALL MPI_RECV(SmallArray, nBuffSize, ParaReal, iSend,    
     &              iTag, MPI_COMM_WORLD, istat, ierr)                  
            ENDIF                                                       
         ELSE                                                           
            !-----------------------------------------------------      
            ! Data will be sent to PE 0. Determine if this is PE 0      
            !-----------------------------------------------------      
            IF (MyPe.GT.0) THEN                                         
               !-----------------------------------------------------   
               ! Send data to PE 0                                      
               !-----------------------------------------------------   
               iRecv=0                                                  
               nBuffSize=MGx*NHEM*JGy*nLevels                           
               iTag=TagRef+MyPe                                         
               CALL MPI_BSEND(SmallArray, nBuffSize, ParaReal, iRecv,   
     &              iTag, MPI_COMM_WORLD, ierr)                         
            ENDIF                                                       
            CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                      
            IF (MyPe.EQ.0) THEN                                         
               !-----------------------------------------------------   
               ! Loop through the PEs                                   
               !-----------------------------------------------------   
               DO iPe=0,NPE-1                                           
                  IF (iPe.EQ.0) THEN                                    
                     !------------------------------------------------- 
                     ! Data already stored on PE 0                      
                     !------------------------------------------------- 
                     DO L=1,nLevels                                     
                        DO J=1,JGyy(iRow)                               
                           DO IHEM=1,NHEM                               
                              DO I=1,MGxx(iCol)                         
                                 LargeArray(IHEM,JJ1(iRow)+J,           
     &                                II1(iCol)+I,L)=                   
     &                                SmallArray(IHEM,J,I,L)            
                              ENDDO                                     
                           ENDDO                                        
                        ENDDO                                           
                     ENDDO                                              
                  ELSE                                                  
                     !------------------------------------------------- 
                     ! Receive data from the other PEs                  
                     !------------------------------------------------- 
                     iSend=iPe                                          
                     nBuffSize=MGx*NHEM*JGy*nLevels                     
                     iTag=TagRef+iPe                                    
                     CALL MPI_RECV(SmallArray2, nBuffSize, ParaReal,    
     &                    iSend, iTag, MPI_COMM_WORLD, istat, ierr)     
                     !------------------------------------------------- 
                     ! Write values into large array                    
                     !------------------------------------------------- 
                     iCol2=GRID(1,iPe)                                  
                     iRow2=GRID(2,iPe)                                  
                     DO L=1,nLevels                                     
                        DO J=1,JGyy(iRow2)                              
                           DO IHEM=1,NHEM                               
                              DO I=1,MGxx(iCol2)                        
                                 LargeArray(IHEM,JJ1(iRow2)+J,          
     &                                II1(iCol2)+I,L)=                  
     &                                SmallArray2(IHEM,J,I,L)           
                              ENDDO                                     
                           ENDDO                                        
                        ENDDO                                           
                     ENDDO                                              
                  ENDIF                                                 
               ENDDO                                                    
            ENDIF                                                       
         ENDIF                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Like subroutine SHAREGRID but the order of the dimensions are         
! different and it only retrieves data from remote PEs for PE 0         
! (in SHAREGRID this occurs when lShare is false).                      
!=====================================================                  
      SUBROUTINE SHAREGRID3(nLevels,SmallArray,LargeArray)              
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variables                                                      
!-----------------------------------------------------                  
! Number of levels                                                      
      INTEGER :: nLevels                                                
! Typically smaller array with coefficients stored for use              
! in model and shared across PEs.                                       
      REAL :: SmallArray(JGy, NHEM, nLevels)                            
! Large array containing all the coefficients stored in the             
! original way                                                          
      REAL :: LargeArray(JG, NHEM, nLevels)                             
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=1300                                 
      INTEGER :: iRow2                                                  
      REAL :: SmallArray2(JGy, NHEM, nLevels)                           
      INTEGER :: JPOS                                                   
! Loop variables                                                        
      INTEGER :: IHEM, iPe, J, L                                        
!-----------------------------------------------------                  
! No data need passing for one processor                                
!-----------------------------------------------------                  
      IF (NPE.EQ.1) THEN                                                
         DO L=1,nLevels                                                 
            DO IHEM=1,NHEM                                              
               DO J=1,JG                                                
                  LargeArray(J,IHEM,L)=SmallArray(J,IHEM,L)             
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! Data will be sent to PE 0. Determine if this is PE 0         
         !-----------------------------------------------------         
         IF (MyPe.GT.0) THEN                                            
            !-----------------------------------------------------      
            ! Send data to PE 0                                         
            !-----------------------------------------------------      
            iRecv=0                                                     
            nBuffSize=NHEM*JGy*nLevels                                  
            iTag=TagRef+MyPe                                            
            CALL MPI_BSEND(SmallArray, nBuffSize, ParaReal, iRecv,      
     &           iTag, MPI_COMM_WORLD, ierr)                            
         ENDIF                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         IF (MyPe.EQ.0) THEN                                            
            !-----------------------------------------------------      
            ! Loop through the PEs                                      
            !-----------------------------------------------------      
            LargeArray(:,:,:)=0.0                                       
            DO iPe=0,NPE-1                                              
                IF (iPe.EQ.0) THEN                                      
                  !---------------------------------------------------  
                  ! Data already stored on PE 0                         
                  !---------------------------------------------------  
                   DO L=1,nLevels                                       
                      DO IHEM=1,NHEM                                    
                         DO J=1,JGyy(iRow)                              
                            LargeArray(JJ1(iRow)+J,IHEM,L)=             
     &                           SmallArray(J,IHEM,L)                   
                         ENDDO                                          
                      ENDDO                                             
                   ENDDO                                                
                ELSE                                                    
                  !---------------------------------------------------  
                  ! Receive data from the other PEs                     
                  !---------------------------------------------------  
                  iSend=iPe                                             
                  nBuffSize=NHEM*JGy*nLevels                            
                  iTag=TagRef+iPe                                       
                  CALL MPI_RECV(SmallArray2, nBuffSize, ParaReal,       
     &                 iSend, iTag, MPI_COMM_WORLD, istat, ierr)        
                  !---------------------------------------------------  
                  ! Write values into large array                       
                  !---------------------------------------------------  
                  iRow2=GRID(2,iPe)                                     
                  JPOS=JJ1(iRow2)                                       
                  DO L=1,nLevels                                        
                     DO IHEM=1,NHEM                                     
                        DO J=1,JGyy(iRow2)                              
                           LargeArray(JPOS+J,IHEM,L)=                   
     &                          LargeArray(JPOS+J,IHEM,L)+              
     &                          SmallArray2(J,IHEM,L)                   
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDIF                                                    
            ENDDO                                                       
         ENDIF                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Perform del**2 or del**(-2) operation on a spectral field.            
! The input spectral array is assumed to use the jagged triangular      
! truncation of the Reading baroclinic spectral models.  This is        
! also used in the diagnostics program for fields from both Reading     
! models and the UGCM.                                                  
!=====================================================                  
      SUBROUTINE SPDEL2(Z,FILT,MMTy,NNTx,NN,MM,NHEM,NL,IPAR,ITYPE,      
     &     MMTyy,NNTxx,nWave)                                           
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Input arguments:                                                      
!-----------------------------------------------------                  
! Z      - Complex array containing input spectral field.               
!          The truncation is assumed to be jagged triangular.           
!          i.e. symmetric (even) coefficients are included up           
!          to total wavenumber (NN-1), while anti-symmetric             
!          (odd) coefficients are included up to wavenumber NN.         
!          This gives equal numbers of even and odd coefficients        
!          in the truncated series.  Ordering is of increasing          
!          total wavenumber within increasing zonal wavenumber.         
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! FILT   - Real array, unset, to receive filter coefficients.           
      REAL :: FILT(0:NN)                                                
! NWJ2   - First dimension of Z: number of even or odd coeffs           
!          at a single level in the jagged triangular truncation        
!      INTEGER :: NWJ2                                                  
      INTEGER :: NNTx                                                   
      INTEGER :: MMTy                                                   
! NN     - Highest total wavenumber of input truncation.                
      INTEGER :: NN                                                     
! MM     - Highest zonal wavenumber of input truncation.                
      INTEGER :: MM                                                     
! NHEM   - Symmetry in latitude: { 1 = hemispheric, only even or        
!                                {     odd coefficients included        
!                                { 2 = global, both even and odd        
!                                {     coefficients included.           
      INTEGER :: NHEM                                                   
! NL     - Number of levels in vertical.                                
      INTEGER :: NL                                                     
! IPAR   - Parity of field: { IPAR=even for even hem symmetry,          
!                           { IPAR=odd  for  odd hem symmetry.          
      INTEGER :: IPAR                                                   
!                                                                       
      INTEGER :: MMTyy                                                  
      INTEGER :: NNTxx                                                  
      INTEGER :: nWave(NNTx,MMTy)                                       
! ITYPE  - Type of operation required: { +2, del**(+2),                 
!                                      { -2, del**(-2).                 
      INTEGER :: ITYPE                                                  
!-----------------------------------------------------                  
! Output arguments:                                                     
!-----------------------------------------------------                  
! Z      - Filtered spectral field.  Ordering of spectral               
!          coefficients is unchanged.                                   
! FILT   - Real array of filter coefficients.                           
! Other arguments unchanged.                                            
!                                                                       
! Method:                                                               
!   This routine can perform the following operations:                  
! ITYPE=+2 : Del**(+2), in which the non-dimensional spectral           
!            coefficient Z(n,m) is multiplied by -[n*(n+1)].            
! ITYPE=-2 : Del**(-2), in which the non-dimensional spectral           
!            coefficient Z(n,m) is divided by -[n*(n+1)].               
!                                                                       
! Author:                                                               
!    Original version.                    Mike Blackburn,  25.11.94     
!    FILT is now a dummy array, (0:NN).   Mike Blackburn,  04.09.96     
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER :: NOF                                                    
      INTEGER :: N                                                      
! Loop variables                                                        
      INTEGER :: IHEM, IM, IN, L                                        
!-----------------------------------------------------                  
! Set up filter coefficients.                                           
! Note that FILT(n)=-n*(n+1) (or its inverse).                          
!-----------------------------------------------------                  
      IF (ITYPE.EQ.2) THEN                                              
         DO N=0,NN                                                      
            FILT(N)=-REAL(N*(N+1))                                      
         ENDDO                                                          
      ELSE IF (ITYPE.EQ.-2) THEN                                        
         DO N=0,NN                                                      
            FILT(N)=-1./REAL(N*(N+1))                                   
         ENDDO                                                          
      ELSE                                                              
         PRINT *,' ***SPDEL2: INVALID VALUE OF ITYPE SUPPLIED = '       
     &          ,ITYPE,' : MUST BE +-2 FOR DEL**(+-2) OPERATION'        
         CALL ABORT                                                     
      ENDIF                                                             
!-----------------------------------------------------                  
! Del**(+-2) operation.                                                 
! Counting for total wavenumber in inner loop is for even coeffs,       
! NOF increases total wavenumber for odd coeffs.                        
!-----------------------------------------------------                  
      DO IHEM=1,NHEM                                                    
         NOF=1-MOD(IHEM+IPAR,2)                                         
         DO L=1,NL                                                      
            DO IM=1,MMTyy                                               
               DO IN=1,NNTxx                                            
                  N=nWave(IN,IM)                                        
                  Z(IN,IM,IHEM,L)=Z(IN,IM,IHEM,L)*FILT(N+NOF)           
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! Controls diagnostic output from model run.                            
! Outputs spectral coefficients.                                        
!=====================================================                  
      SUBROUTINE SPOP                                                   
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Switches counters and constants controlling type and frequency of     
! model output                                                          
!-----------------------------------------------------                  
      COMMON/OUTCON/RNTAPE,DAY,SQR2,RSQR2,EAM1,EAM2,TOUT1               
     &     ,TOUT2,LSPO,LGPO                                             
     &     ,RNTAPO                                                      
     &     ,LSHIST,LMINIH                                               
      COMMON/OUTCONINT/NCOEFF,NLAT,INLAT,INSPC,KOUNTP                   
     &                 ,KOUNTE,KOUNTH,KOUNTR,KOUTP                      
     &                 ,KOUTE,KOUTH,KOUTR                               
! Identyfying run number                                                
      REAL :: RNTAPE                                                    
! Maximum wavenumber of printed spectral coeffecients                   
      INTEGER :: NCOEFF                                                 
! Number of latitudes in printed grid point fields                      
      INTEGER :: NLAT                                                   
      INTEGER :: INLAT                                                  
!                                                                       
      INTEGER :: INSPC                                                  
! Number of timsteps between successive writing of spectral             
! and gridded fields                                                    
      INTEGER :: KOUNTP                                                 
      INTEGER :: KOUNTE                                                 
! Number of timsteps between successive writing of history              
! records, containing spectral fields at the present and                
! previous time levels.                                                 
      INTEGER :: KOUNTH                                                 
! Number of timsteps between successive writing of restart              
! records, containing the spectral fields at the present and            
! previous time levels                                                  
      INTEGER :: KOUNTR                                                 
      INTEGER :: KOUTP                                                  
      INTEGER :: KOUTE                                                  
      INTEGER :: KOUTH                                                  
      INTEGER :: KOUTR                                                  
      REAL :: DAY                                                       
      REAL :: SQR2                                                      
      REAL :: RSQR2                                                     
      REAL :: EAM1                                                      
      REAL :: EAM2                                                      
      REAL :: TOUT1                                                     
      REAL :: TOUT2                                                     
      LOGICAL :: LSPO(NL)                                               
      LOGICAL :: LGPO(NL)                                               
      REAL :: RNTAPO                                                    
      LOGICAL :: LSHIST                                                 
! Reduce the history file to single precision if true                   
      LOGICAL :: LMINIH                                                 
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      CHARACTER*(80) :: DESCRIPTION                                     
! Loop variables                                                        
      INTEGER :: ITRAC, L                                               
!-----------------------------------------------------                  
! Format statements                                                     
!-----------------------------------------------------                  
 200  FORMAT(/' NUMBER OF TIME STEPS COMPLETED =',I5)                   
 202  FORMAT(' SPECTRAL COEFFICIENTS (COEFF ; AMPLITUDE ; PHASE)')      
 204  FORMAT(' VORTICITY AT LEVEL',I2)                                  
 206  FORMAT(' DIVERGENCE AT LEVEL',I2)                                 
 208  FORMAT(' PERTURBATION TEMPERATURE AT LEVEL',I2)                   
 211  FORMAT(' LOG(SURFACE PRESSURE)')                                  
!-----------------------------------------------------                  
! If NCOEFF=0 leave routine                                             
!-----------------------------------------------------                  
      IF (NCOEFF.EQ.0) RETURN                                           
!-----------------------------------------------------                  
! Spectral coeficients are wanted                                       
!-----------------------------------------------------                  
      WRITE (2,200) KOUNT                                               
      WRITE (2,202)                                                     
!-----------------------------------------------------                  
! Absolute vorticity                                                    
!-----------------------------------------------------                  
      WRITE(DESCRIPTION,'(a18)') 'VORTICITY AT LEVEL'                   
      CALL WRSPA(DESCRIPTION,Z,1,NL)                                    
!-----------------------------------------------------                  
! Divergence                                                            
!-----------------------------------------------------                  
      WRITE(DESCRIPTION,'(a19)') 'DIVERGENCE AT LEVEL'                  
      CALL WRSPA(DESCRIPTION,D,2,NL)                                    
!-----------------------------------------------------                  
! Temperature                                                           
!-----------------------------------------------------                  
      WRITE(DESCRIPTION,'(a33)') 'PERTURBATION TEMPERATURE AT LEVEL'    
      CALL WRSPA(DESCRIPTION,T,2,NL)                                    
!-----------------------------------------------------                  
! Tracers                                                               
!-----------------------------------------------------                  
      DO ITRAC=1,NTRAC                                                  
         WRITE(DESCRIPTION,'(a13,i4,a9)') 'TRACER FIELD ',              
     &        ITRAC, ' AT LEVEL'                                        
         CALL WRSPA(DESCRIPTION,TRA(1,1,1,1,ITRAC),2,NL)                
      ENDDO                                                             
!-----------------------------------------------------                  
! Log (Surface Pressure)                                                
!-----------------------------------------------------                  
      WRITE(DESCRIPTION,'(a21)') 'LOG(SURFACE PRESSURE)'                
      CALL WRSPA(DESCRIPTION,SP,2,1)                                    
                                                                        
      RETURN                                                            
      END                                                               
!=====================================================                  
! Perform soil moisture / snow depth update for timestep                
! have to do this after the CONVEC and LSCRN subs                       
!=====================================================                  
      SUBROUTINE SURFM(JH)                                              
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Grid variables for DGRMLT loop                                        
!-----------------------------------------------------                  
      COMMON/GRIDPC/ASSBL,ASHBL,ASLBL,ARRCR,ARRLR                       
     &     ,TTVD,QTVD,TYBL,TXBL,TTRD,TTLR,TTCR,UTVD,VTVD                
     &     ,QTLR,QTCR,SSBL,SHBL,SLBL,RRCR,RRLR                          
     &     ,ARFLUX,ASFLD,ACLD,RFLUX,SFLD,CLD                            
      REAL :: ASSBL(MGx,NHEM,JGy)                                       
      REAL :: ASHBL(MGx,NHEM,JGy)                                       
      REAL :: ASLBL(MGx,NHEM,JGy)                                       
      REAL :: ARRCR(MGx,NHEM,JGy)                                       
      REAL :: ARRLR(MGx,NHEM,JGy)                                       
      REAL :: TTVD(MGx,NHEM,NL)                                         
      REAL :: QTVD(MGx,NHEM,NL)                                         
      REAL :: TYBL(MGx,NHEM)                                            
      REAL :: TXBL(MGx,NHEM)                                            
      REAL :: TTRD(MGx,NHEM,NL)                                         
      REAL :: TTLR(MGx,NHEM,NL)                                         
      REAL :: TTCR(MGx,NHEM,NL)                                         
      REAL :: UTVD(MGx,NHEM,NL)                                         
      REAL :: VTVD(MGx,NHEM,NL)                                         
      REAL :: QTLR(MGx,NHEM,NL)                                         
      REAL :: QTCR(MGx,NHEM,NL)                                         
      REAL :: SSBL(MGx,NHEM,JGy)                                        
      REAL :: SHBL(MGx,NHEM,JGy)                                        
      REAL :: SLBL(MGx,NHEM,JGy)                                        
      REAL :: RRCR(MGx,NHEM,JGy)                                        
      REAL :: RRLR(MGx,NHEM,JGy)                                        
      REAL :: arflux(MGx,NHEM,JGy,6)                                    
      REAL :: asfld(MGx,NHEM,JGy,6)                                     
      REAL :: acld(MGx,NHEM,JGy,4)                                      
      REAL :: rflux(MGx,NHEM,JGy,6)                                     
      REAL :: sfld(MGx,NHEM,JGy,6)                                      
      REAL :: cld(MGx,NHEM,JGy,4)                                       
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Radiation variables                                                   
!-----------------------------------------------------                  
      COMMON/CPIERS/ICFLAG,CFRAC,PNET,SNET,RRFLUX,TTSW,TTLW             
      INTEGER :: ICFLAG(MGx,NHEM,5,2)                                   
      REAL :: CFRAC(MGx,NHEM,5)                                         
      REAL :: PNET(MGx,NHEM,JGy)                                        
      REAL :: SNET(MGx,NHEM,JGy)                                        
      REAL :: RRFLUX(MGx,NHEM,JGy,6)                                    
      REAL :: TTSW(MGx,NHEM,NL)                                         
      REAL :: TTLW(MGx,NHEM,NL)                                         
!-----------------------------------------------------                  
! Orogrophy                                                             
!-----------------------------------------------------                  
      COMMON/GSG/GSG,TOPVAR,RLEVBZMO,FXZMO,ofrac,runoff,iareanum        
! Topog variance, GW diags for Orog, Non orog waves                     
      REAL :: TOPVAR(MGx,NHEM,JGy)                                      
      REAL :: RLEVBZMO(JGy,NHEM,NL)                                     
      REAL :: FXZMO(JGy,NHEM,NL)                                        
         REAL :: GSG(MGx,NHEM,JGy)                                      
         REAL :: ofrac(MGx,NHEM,JGy)                                    
         INTEGER :: runoff(50,2,33)                                     
         INTEGER :: iareanum                                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
C OASIS size parameters                                                 
      INTEGER, PARAMETER :: IMB = 128                                   
      INTEGER, PARAMETER :: JM = 64                                     
      INTEGER, PARAMETER :: IMJM = IMB*JM                               
      common/oasisa2o/swlw(imjm),heatlat(imjm),water(imjm)              
     $   ,sensible(imjm),taux(imjm),tauy(imjm),exwater(imjm)            
     $   ,exwat(MGx,NHEM,JGy),sensible2(imjm),sensible3(imjm)           
     $   ,oceaniceflux(MGx,NHEM),exiwat(MGx,NHEM,JGy),exiwater(imjm)    
     $   ,exiwatn(MGx,NHEM,JGy),exiwatern(imjm)                         
     $   ,tstaroo(MGx,NHEM,JGy),tstarf(MGx,NHEM,JGy)                    
     $   ,qstarf(MGx,NHEM,JGy),sqstarf(MGx,NHEM,JGy)                    
     $   ,swlw2d(MGx,NHEM,JGy),heatlat2d(MGx,NHEM,JGy)                  
     $   ,water2d(MGx,NHEM,JGy)                                         
     $   ,sensible2d(MGx,NHEM,JGy),taux2d(MGx,NHEM,JGy)                 
     $   ,tauy2d(MGx,NHEM,JGy)                                          
     $   ,SHBLC(MGx,NHEM,JGy),SLBLC(MGx,NHEM,JGy)                       
     $   ,exwater2d(MGx,NHEM,JGy),sensible2_2d(MGx,NHEM,JGy)            
     $   ,sensible3_2d(MGx,NHEM,JGy)                                    
     $   ,exiwater2d(MGx,NHEM,JGy),exiwatern2d(MGx,NHEM,JGy)            
      real swlw,heatlat,water,sensible,taux,tauy,shblc,slblc,exwater    
     $,exwat,sensible2,sensible3,oceaniceflux,exiwat,exiwater           
     $,exiwatn,exiwatern,tstaroo,tstarf,qstarf,sqstarf                  
     $,swlw2d,heatlat2d,water2d,sensible2d,taux2d,tauy2d                
     $,exwater2d,sensible2_2d,sensible3_2d,exiwater2d,exiwatern2d       
                                                                        
      common/wat/water2(imjm), water2_2d(MGx,NHEM,JGy)                  
C Arrays wathbl, watlsl, watsbl are used for balancing water in IGCM4   
C ashbl, aslbl, assbl were used in IGCM3 FORTE but these are already    
C in use in elsewhere in IGCM4 for surface flux diagnostics             
     $   ,wathbl(MGx,NHEM,JGy),watlbl(MGx,NHEM,JGy),watsbl(MGx,NHEM,JGy)
     $   ,sicecov(MGx,NHEM,JGy)                                         
      real water2,water2_2d,wathbl,watlbl,watsbl,sicecov                
                                                                        
!-----------------------------------------------------                  
! Passed arguments                                                      
!-----------------------------------------------------                  
! Index for latitudes                                                   
      INTEGER :: JH                                                     
!-----------------------------------------------------                  
! Local variables.                                                      
!-----------------------------------------------------                  
      REAL :: SNOLOOK(24)                                               
      REAL, PARAMETER :: tzc=273.15                                     
      REAL :: TP1                                                       
      REAL :: TZ                                                        
      REAL :: TM2                                                       
      REAL :: TM3                                                       
      REAL :: FRN                                                       
      REAL :: FFL                                                       
      REAL :: TSCUR                                                     
      REAL :: CTQUSE                                                    
      REAL :: DWATER                                                    
      REAL :: SHSLOOK(24)                                               
! Loop variables                                                        
      INTEGER :: I, IHEM                                                
!-----------------------------------------------------                  
! Snow covered lookup table                                             
!-----------------------------------------------------                  
                                                                        
C Snow-covered albedo as a function of land surface                     
                                                                        
       DATA SNOLOOK /0.8, 0.8, 0.8,                                     
     & 0.7, 0.7, 0.7, 0.7, 0.7, 0.7, 0.7,                               
     & 0.8, 0.8, 0.8, 0.8,                                              
     & 0.8, 0.8, 0.8, 0.8, 0.8, 0.8, 0.8,                               
     & 0.8, 0.8, 0.8/                                                   
                                                                        
C The height at which albedo = (snow-free+SNOLOOK)/2                    
C should be dependent on land surface type (approximately done here)    
C Low values balance high IGCM snow density                             
                                                                        
       DATA SHSLOOK /0.05, 0.05, 0.05,                                  
     & 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,                               
     & 0.1, 0.1, 0.1, 0.1,                                              
     & 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,                               
     & 0.05, 0.05, 0.05/                                                
                                                                        
      tp1=(tzc+1.)/ct                                                   
      tz=tzc/ct                                                         
      tm2=(tzc-1.9)/ct    ! Also done in T42.accum                      
      tm3=(tzc-3.)/ct                                                   
                                                                        
      FRN=SDW/(1000.*RADEA)                                             
      FFL=AKAP*DELT                                                     
                                                                        
      if (LSL) then                                                     
         do ihem=1,nhem                                                 
            do i=1,MGxx(iCol)                                           
               if (gsg(I,IHEM,jh).gt.0.) then                           
                  TSCUR=TSTAR(I,IHEM,JH)   ! bug fix RF 10/4/2000       
                  if (tscur.lt.tz) then                                 
                     ctquse=ctqi                                        
                  else                                                  
                     ctquse=ctq                                         
                  endif                                                 
                  !---------------------------------------------------  
                  ! amount of water change in time DELT                 
                  !---------------------------------------------------  
                  dwater=(rrcr(I,IHEM,JH)+rrlr(I,IHEM,JH)-              
     $                   wathbl(I,IHEM,JH))*FRN                         
                                                                        
                  if ((hsnow(I,IHEM,jh).gt.0.).or.                      
     &                 ((tstar(I,IHEM,jh).lt.tz).and.(dwater.gt.0.)))   
     &                 then                                             
                     hsnow(I,IHEM,jh)=hsnow(I,IHEM,jh)+dwater/sdsn      
                  else                                                  
                     smstar(I,IHEM,jh)=smstar(I,IHEM,jh)+dwater/sdw     
                  endif                                                 
                                                                        
C Soil bucket model: excess goes ito runoff                             
                  if (smstar(I,IHEM,jh).gt.ssmc)then                    
                    exwat(I,IHEM,jh)=                                   
     $              (smstar(I,IHEM,jh)-ssmc)*1000.*RADEA                
                    smstar(I,IHEM,jh)=ssmc                              
                  endif                                                 
                  if(smstar(I,IHEM,jh).lt.0.)smstar(I,IHEM,jh)=0.       
                  if (hsnow(I,IHEM,jh).le.0.) then                      
                     if(ofrac(i,ihem,jh).lt.0.5)                        
     &               salb(I,IHEM,jh)=sbal(I,IHEM,jh)                    
                     hsnow(i,ihem,jh) = 0.0                             
                  else                                                  
                     !------------------------------------------------  
                     ! Snow-covered albedo depends on vegetation. This  
                     ! includes a bug fix from the old surface.upd,     
                     ! where SBAL replaces SALB.                        
                     !------------------------------------------------  
                     if(ofrac(i,ihem,jh).lt.0.5)                        
     &               salb(I,IHEM,jh)=sbal(I,IHEM,jh)+                   
                                                                        
     &                    (snolook(NINT(SVEGE(I,IHEM,JH)))-             
     &                    sbal(I,IHEM,jh))*hsnow(I,IHEM,jh)/            
     &                    (hsnow(I,IHEM,jh)+                            
     &                    shslook(NINT(SVEGE(I,IHEM,JH)))/radea)        
                  endif                                                 
               endif                                                    
            enddo                                                       
         enddo                                                          
      endif                                                             
      end                                                               
!=====================================================                  
! Relative vorticity is obtained from temperature and surface pressure  
!=====================================================                  
      SUBROUTINE TBAL                                                   
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Constants and arrays needed for balancing                             
!-----------------------------------------------------                  
      COMMON/BALAN/BFILT,RGT0,RG,TMEAN                                  
     +            ,EP1,EP2,KBAL,SRGT0,LTBAL                             
     &            ,PMNRE                                                
! Maximum zonal wavenumbers used for balancing                          
      INTEGER, PARAMETER :: MFTBAL=9                                    
! Number of zonal wavenumbers used for balancing                        
      INTEGER, PARAMETER :: NFTBAL=1+MFTBAL/MOCT                        
      REAL :: BFILT(NL)                                                 
      REAL :: RGT0(NL)                                                  
      REAL :: RG(NL,NL)                                                 
      REAL :: TMEAN(NL)                                                 
      REAL :: EP1(IDM,NFTBAL,NHEM)                                      
      REAL :: EP2(IDM,NFTBAL,NHEM)                                      
      INTEGER :: KBAL                                                   
      REAL :: SRGT0                                                     
      LOGICAL :: LTBAL                                                  
      REAL :: PMNRE(IDM,IDM,3)                                          
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      COMPLEX, ALLOCATABLE :: ERR(:,:,:,:)                              
      COMPLEX, ALLOCATABLE :: ZTBAL(:,:,:,:)                            
      COMPLEX, ALLOCATABLE :: HOLDVAR(:,:,:,:,:)                        
      COMPLEX :: VPS, GSI1, TA, SRGT                                    
      INTEGER :: J2                                                     
! J2H =J2 if IHEM=1 and =J2-1 if IHEM=2                                 
      INTEGER :: J2H                                                    
      INTEGER :: iSwitch                                                
      INTEGER :: ImLast(Py)                                             
      INTEGER :: IDM2(MMTy,Px,Py)                                       
      INTEGER :: MP, NP                                                 
      COMPLEX :: SRGT2(NL)                                              
      INTEGER, PARAMETER :: TagRef=1500                                 
      INTEGER :: iPe                                                    
      INTEGER :: iCol2, iRow2                                           
      INTEGER :: IN1                                                    
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, IN, J, K, L, M                            
!-----------------------------------------------------                  
! NOTE: This scheme does not converge if wavenumbers M are              
! present for which there are only a small number of modes in the       
! truncation. For a small number of iterations the problem is not       
! serious but it may be removed altogether by limiting the range of     
! the 20 and 80 loops. For a given M, 7 modes are sufficient and 3      
! are insufficient. The loop terminator MFTBAL is set in INITAL.        
!                                                                       
! Find the number of zonal wavenumbers below MFTBAL for each            
! row of PEs                                                            
!-----------------------------------------------------                  
      DO iRow2=1,Py                                                     
         MP=MMT1(iRow2)*MOCT                                            
         IF (MP.GE.MFTBAL) THEN                                         
            ImLast(iRow2)=0                                             
         ELSE                                                           
            IM=0                                                        
            DO WHILE ( (MP.LT.MFTBAL).AND.(IM.LT.MMTyy(iRow2)) )        
               MP=MP+MOCT                                               
               IM=IM+1                                                  
            ENDDO                                                       
            ImLast(iRow2)=IM                                            
         ENDIF                                                          
         IF (ImLast(iRow2).GT.MMTyy(iRow2)) THEN                        
            WRITE(6,*) 'ERROR: not expecting to use more than half ',   
     &           'the zonal wavenumber for balancing, as yet'           
            CALL ABORT                                                  
         ENDIF                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Work out how many meridional wavenumbers are needed from each         
! zonal wavenumber on each PE                                           
!-----------------------------------------------------                  
      DO iRow2=1,Py                                                     
         DO IM=1,ImLast(iRow2)                                          
            MP=(MMT1(iRow2)+IM-1)*MOCT                                  
            DO iCol2=1,Px                                               
               IDM2(IM,iCol2,iRow2)=MAX(0,(NN-MP+1)/2-NNT1(iCol2))      
               IF (IDM2(IM,iCol2,iRow2).GT.NNTxx(iCol2)) THEN           
                  IDM2(IM,iCol2,iRow2)=NNTxx(iCol2)                     
               ENDIF                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Calculate ERR and store the needed vorticity in ZTBAL.                
! Allocate the size of arrays needed.                                   
!-----------------------------------------------------                  
      IF (IDM2(1,iCol,iRow).GT.0) THEN                                  
         ALLOCATE(HoldVar(NNTx,NL,NHEM,ImLast(iRow),2))                 
      ENDIF                                                             
!-----------------------------------------------------                  
! Loop through zonal wavenumbers below MFTBAL                           
!-----------------------------------------------------                  
      DO IM=1,ImLast(iRow)                                              
         MP=mWave(1,IM)                                                 
         DO IHEM=1,NHEM                                                 
            IN=1                                                        
            DO IN=1,IDM2(IM,iCol,iRow)                                  
               NP=nWave(IN,IM,iCol)                                     
               !-----------------------------------------------------   
               ! Only skip code below when NP=0 & IHEM=1                
               !-----------------------------------------------------   
               IF ((NP+IHEM).GT.1) THEN                                 
                  VPS=VP(IN,IM,IHEM)                                    
                  GSI1=GS(IN,IM,IHEM)                                   
                  DO L=1,NL                                             
                     TA=(0.0,0.0)                                       
                     SRGT=(0.0,0.0)                                     
                     DO M=1,NL                                          
                        TA=TA+G(M,L)*TT(IN,IM,IHEM,M)                   
                        SRGT=SRGT+G(M,L)*T(IN,IM,IHEM,M)                
                     ENDDO                                              
                     HoldVar(IN,L,IHEM,IM,2)=-DT(IN,IM,IHEM,L)-         
     &                    SQ(NP+IHEM)*                                  
     &                    (SRGT+GSI1+T0(L)*SP(IN,IM,IHEM)+              
     &                    DELT*(TA-T0(L)*VPS))                          
                  ENDDO                                                 
               ENDIF                                                    
               !-----------------------------------------------------   
               ! Store vorticity                                        
               !-----------------------------------------------------   
               DO L=1,NL                                                
                  HoldVar(IN,L,IHEM,IM,1)=Z(IN,IM,IHEM,L)               
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Only PE 0 carries out the balancing                                   
!-----------------------------------------------------                  
      IF (MyPe.GT.0) THEN                                               
         IF (IDM2(1,iCol,iRow).GT.0) THEN                               
            !-----------------------------------------------------      
            ! All the data needs passing to PE 0                        
            !-----------------------------------------------------      
            iTag=TagRef+2*MyPe                                          
            iRecv=0                                                     
            nBuffSize=NNTx*NL*NHEM*ImLast(iRow)*2                       
            CALL MPI_BSEND(HoldVar,nBuffSize,ParaCom,                   
     &           iRecv,iTag,MPI_COMM_WORLD,ierr)                        
            !-----------------------------------------------------      
            ! After PE 0 has calculated the balancing the data          
            ! is sent back                                              
            !-----------------------------------------------------      
            iSend=0                                                     
            nBuffSize=NNTx*NL*NHEM*ImLast(iRow)                         
            iTag=TagRef+2*MyPe+1                                        
            CALL MPI_RECV(HoldVar, nBuffSize, ParaCom, iSend,           
     &           iTag, MPI_COMM_WORLD, istat, ierr)                     
            !-----------------------------------------------------      
            ! Update vorticity with the balancing                       
            !-----------------------------------------------------      
            DO L=1,NL                                                   
               DO IHEM=1,NHEM                                           
                  DO IM=1,ImLast(iRow)                                  
                     DO IN=1,IDM2(IM,iCol,iRow)                         
                        Z(IN,IM,IHEM,L)=HoldVar(IN,L,IHEM,IM,1)         
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
            DEALLOCATE(HoldVar)                                         
         ENDIF                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! Loop across PEs receiving the required data                  
         !-----------------------------------------------------         
         ALLOCATE(ERR(IDM,NL,NHEM,NFTBAL))                              
         ALLOCATE(ZTBAL(IDM,NL,NHEM,NFTBAL))                            
         ERR(:,:,:,:)=(0.0, 0.0)                                        
         ZTBAL(:,:,:,:)=(0.0, 0.0)                                      
         !-----------------------------------------------------         
         ! Loop across PEs receiving the required data                  
         !-----------------------------------------------------         
         DO iPe=0,NPE-1                                                 
            iCol2=GRID(1,iPe)                                           
            iRow2=GRID(2,iPe)                                           
            IF ( (iPe.GT.0).AND.(IDM2(1,iCol2,iRow2).GT.0) ) THEN       
               !-----------------------------------------------------   
               ! Receive the data                                       
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=NNTx*NL*NHEM*ImLast(iRow2)*2                   
               iTag=TagRef+2*iPe                                        
               CALL MPI_RECV(HoldVar, nBuffSize, ParaCom, iSend,        
     &              iTag, MPI_COMM_WORLD, istat, ierr)                  
            ENDIF                                                       
            !-----------------------------------------------------      
            ! Write to ERR and ZTBAL                                    
            !-----------------------------------------------------      
            DO IM=1,ImLast(iRow2)                                       
               DO IHEM=1,NHEM                                           
                  IF ((IHEM.EQ.1).AND.(iCol2.EQ.1)) THEN                
                     IN1=2                                              
                  ELSE                                                  
                     IN1=1                                              
                  ENDIF                                                 
                  DO L=1,NL                                             
                     DO IN=1,IDM2(IM,iCol2,iRow2)                       
                        ZTBAL(NNT1(iCol2)+IN,L,IHEM,MMT1(iRow2)+IM)=    
     &                       HoldVar(IN,L,IHEM,IM,1)                    
                     ENDDO                                              
                     DO IN=IN1,IDM2(IM,iCol2,iRow2)                     
                        ERR(NNT1(iCol2)+IN-2+IHEM,L,IHEM,               
     &                       MMT1(iRow2)+IM)=HoldVar(IN,L,IHEM,IM,2)    
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Loop over the zonal wavenumbers below MFTBAL                 
         !-----------------------------------------------------         
         DO IHEM=1,NHEM                                                 
            DO MP=0,MFTBAL,MOCT                                         
               IM=(MP/MOCT)+1                                           
               J2=(NN-MP+1)/2                                           
               J2H=J2+1-IHEM                                            
               !-----------------------------------------------------   
               ! Read the balance data file if IHEM=1 & MP>0 or         
               ! IHEM=2 & MP=0.                                         
               !-----------------------------------------------------   
               iSwitch=MP*(2-IHEM)+(1-MP)*(IHEM-1)                      
               IF (iSwitch.GT.0) THEN                                   
                  !---------------------------------------------------  
                  ! Modification due to PMNRE and ERR.                  
                  !---------------------------------------------------  
                  DO IN=1,J2H                                           
                     DO L=1,NL                                          
                        DO K=1,J2                                       
                           ZTBAL(IN,L,IHEM,IM)=ZTBAL(IN,L,IHEM,IM)+     
     &                          PMNRE(K,IN,IM)*ERR(IN,L,IHEM,IM)        
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ELSE                                                     
                  !---------------------------------------------------  
                  ! Modification due to EP1 and EP2.                    
                  !---------------------------------------------------  
                  DO L=1,NL                                             
                     SRGT=(0.0,0.0)                                     
                     DO IN=J2+IHEM-2,1,-1                               
                        SRGT=(ERR(IN,L,IHEM,IM)-EP2(IN+2-IHEM,IM,IHEM)* 
     &                       SRGT)/EP1(IN,IM,IHEM)                      
                        ZTBAL(IN,L,IHEM,IM)=ZTBAL(IN,L,IHEM,IM)+SRGT    
                     ENDDO                                              
                  ENDDO                                                 
               ENDIF                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Send information to the other PEs                            
         !-----------------------------------------------------         
         DO iPe=1,NPE-1                                                 
            iCol2=GRID(1,iPe)                                           
            iRow2=GRID(2,iPe)                                           
            !-----------------------------------------------------      
            ! Store data in HoldVar                                     
            !-----------------------------------------------------      
            DO IM=1,ImLast(iRow2)                                       
               DO IHEM=1,NHEM                                           
                  DO L=1,NL                                             
                     DO IN=1,IDM2(IM,iCol2,iRow2)                       
                        HoldVar(IN,L,IHEM,IM,1)=                        
     &                       ZTBAL(NNT1(iCol2)+IN,L,IHEM,MMT1(iRow2)+IM)
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Pass the data to PE iPe                                   
            !-----------------------------------------------------      
            iTag=TagRef+2*iPe+1                                         
            iRecv=iPe                                                   
            nBuffSize=NNTx*NL*NHEM*ImLast(iRow2)                        
            CALL MPI_BSEND(HoldVar,nBuffSize,ParaCom,                   
     &           iRecv,iTag,MPI_COMM_WORLD,ierr)                        
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Store the data wanted on this PE                             
         !-----------------------------------------------------         
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,ImLast(iRow)                                     
                  DO IN=1,IDM2(IM,iCol,iRow)                            
                     Z(IN,IM,IHEM,L)=ZTBAL(IN,L,IHEM,IM)                
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DEALLOCATE(ZTBAL,ERR,HoldVar)                                  
      ENDIF                                                             
!-----------------------------------------------------                  
! Copy current values into MI variables                                 
!-----------------------------------------------------                  
      IF(KOUNT.EQ.0)THEN                                                
         DO IHEM=1,NHEM                                                 
            DO IM=1,MMTyy(iRow)                                         
               DO IN=1,NNTxx(iCol)                                      
                  SPMI(IN,IM,IHEM)=SP(IN,IM,IHEM)                       
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMTyy(iRow)                                      
                  DO IN=1,NNTxx(iCol)                                   
                     ZMI(IN,IM,IHEM,L)=Z(IN,IM,IHEM,L)                  
                     DMI(IN,IM,IHEM,L)=D(IN,IM,IHEM,L)                  
                     TMI(IN,IM,IHEM,L)=T(IN,IM,IHEM,L)                  
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! Transposing the spectral arrays so that the meriodonal wavenumbers    
! are complete, so that this dimension is read to be converted into     
! latitudes, using LTI and HEXPV, to form the half transforms.          
! Instead the levels are split across the columns of PEs.               
!=====================================================                  
      SUBROUTINE TRANS1                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Like the variables in SPECTR but the meridional wavenumbers           
! are complete instead of the levels.                                   
!-----------------------------------------------------                  
      COMMON/SPECTR2/Z2,D2,T2,SP2,SPA2,VP2,DTE2,TT2,DT2,ZT2             
     &     ,TRA2,TRAT2                                                  
! Vorticity coefficient                                                 
      COMPLEX :: Z2(NNT,MMTy,NHEM,NLx)                                  
! Divergence coefficient                                                
      COMPLEX :: D2(NNT,MMTy,NHEM,NLx)                                  
! Temperature coefficient                                               
      COMPLEX :: T2(NNT,MMTy,NHEM,NLx)                                  
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP2(NNT,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA2(NNT,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP2(NNT,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE2(NNT,MMTy,NHEM,NLx)                                
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT2(NNT,MMTy,NHEM,NLx)                                 
! Tracer coefficient                                                    
      COMPLEX :: TRA2(NNT,MMTy,NHEM,NLx,NTRAC)                          
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT2(NNT,MMTy,NHEM,NLx,NTRAC)                         
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=2000                                 
      COMPLEX :: HoldVar1(NNTx,MMTy,NHEM,NLx,3+NTRAC)                   
      COMPLEX :: HoldVar2(NNTx,MMTy,NHEM)                               
      INTEGER :: iCol2, iPe                                             
      INTEGER :: IMMAX, INMAX, LMAX                                     
      INTEGER :: IN1, IN2                                               
      INTEGER :: L1, L2                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, IN, ITRAC, L                              
!-----------------------------------------------------                  
! Transpose                                                             
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to send them data              
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Store multi-layer variables in HoldVar1                
               !-----------------------------------------------------   
               INMAX=NNTxx(iCol)                                        
               IMMAX=MMTyy(iRow)                                        
               LMAX=NLxx(iCol2)                                         
               L1=KK1(iCol2)+1                                          
               L2=KK1(iCol2)+NLxx(iCol2)                                
               HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,1)=                    
     &              Z(1:INMAX,1:IMMAX,:,L1:L2)                          
               HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,2)=                    
     &              D(1:INMAX,1:IMMAX,:,L1:L2)                          
               HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,3)=                    
     &              T(1:INMAX,1:IMMAX,:,L1:L2)                          
               DO ITRAC=1,NTRAC                                         
                  HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,3+ITRAC)=           
     &                 TRA(1:INMAX,1:IMMAX,:,L1:L2,ITRAC)               
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=(3+NTRAC)*NNTx*MMTy*NHEM*NLx                   
               iTag=TagRef+100*iCol+10*iCol2+iRow                       
               CALL MPI_BSEND(HoldVar1,nBuffSize,ParaCom,iRecv,iTag,    
     &              MPI_COMM_WORLD,ierr)                                
            ENDIF                                                       
         ENDDO                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to receive data                
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Receive message                                        
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=(3+NTRAC)*NNTx*MMTy*NHEM*NLx                   
               iTag=TagRef+100*iCol2+10*iCol+iRow                       
               CALL MPI_RECV(HoldVar1,nBuffSize,ParaCom,iSend,iTag,     
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write to Z2 etc                                        
               !-----------------------------------------------------   
               INMAX=NNTxx(iCol2)                                       
               IMMAX=MMTyy(iRow)                                        
               LMAX=NLxx(iCol)                                          
               IN1=NNT1(iCol2)+1                                        
               IN2=NNT1(iCol2)+INMAX                                    
               L1=KK1(iCol)+1                                           
               L2=KK1(iCol)+NLxx(iCol)                                  
               Z2(IN1:IN2,1:IMMAX,:,1:LMAX)=                            
     &              HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,1)                
               D2(IN1:IN2,1:IMMAX,:,1:LMAX)=                            
     &              HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,2)                
               T2(IN1:IN2,1:IMMAX,:,1:LMAX)=                            
     &              HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,3)                
               DO ITRAC=1,NTRAC                                         
                  TRA2(IN1:IN2,1:IMMAX,:,1:LMAX,ITRAC)=                 
     &                 HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,3+ITRAC)       
               ENDDO                                                    
            ENDIF                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Data is already contained on this PE                         
         !-----------------------------------------------------         
         INMAX=NNTxx(iCol)                                              
         IMMAX=MMTyy(iRow)                                              
         LMAX=NLxx(iCol)                                                
         IN1=NNT1(iCol)+1                                               
         IN2=NNT1(iCol)+NNTxx(iCol)                                     
         L1=KK1(iCol)+1                                                 
         L2=KK1(iCol)+NLxx(iCol)                                        
         Z2(IN1:IN2,1:IMMAX,:,1:LMAX)=Z(1:INMAX,1:IMMAX,:,L1:L2)        
         D2(IN1:IN2,1:IMMAX,:,1:LMAX)=D(1:INMAX,1:IMMAX,:,L1:L2)        
         T2(IN1:IN2,1:IMMAX,:,1:LMAX)=T(1:INMAX,1:IMMAX,:,L1:L2)        
         DO ITRAC=1,NTRAC                                               
            TRA2(IN1:IN2,1:IMMAX,:,1:LMAX,ITRAC)=                       
     &           TRA(1:INMAX,1:IMMAX,:,L1:L2,ITRAC)                     
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Single layer variables will only stored on first             
         ! column of PEs                                                
         !-----------------------------------------------------         
         IF (iCol.GT.1) THEN                                            
            INMAX=NNTxx(iCol)                                           
            IMMAX=MMTyy(iRow)                                           
            HoldVar2(1:INMAX,1:IMMAX,:)=SP(1:INMAX,1:IMMAX,:)           
            !-----------------------------------------------------      
            ! Send message                                              
            !-----------------------------------------------------      
            iRecv=PeGrid(1,iRow)                                        
            nBuffSize=NNTx*MMTy*NHEM                                    
            iTag=TagRef+1010+100*iCol+iRow                              
            CALL MPI_BSEND(HoldVar2,nBuffSize,ParaCom,iRecv,iTag,       
     &           MPI_COMM_WORLD,ierr)                                   
         ENDIF                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         IF (iCol.EQ.1) THEN                                            
            !-----------------------------------------------------      
            ! Loop through the other PEs on this row                    
            !-----------------------------------------------------      
            DO iCol2=2,Px                                               
               iPe=PeGrid(iCol2,iRow)                                   
               !-----------------------------------------------------   
               ! Receive messages                                       
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=NNTx*MMTy*NHEM                                 
               iTag=TagRef+1010+100*iCol2+iRow                          
               CALL MPI_RECV(HoldVar2,nBuffSize,ParaCom,iSend,iTag,     
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write data to SP2                                      
               !-----------------------------------------------------   
               INMAX=NNTxx(iCol2)                                       
               IMMAX=MMTyy(iRow)                                        
               IN1=NNT1(iCol2)+1                                        
               IN2=NNT1(iCol2)+INMAX                                    
               SP2(IN1:IN2,1:IMMAX,:)=HoldVar2(1:INMAX,1:IMMAX,:)       
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Store the data on this array                              
            !-----------------------------------------------------      
            INMAX=NNTxx(iCol)                                           
            IMMAX=MMTyy(iRow)                                           
            IN1=NNT1(iCol)+1                                            
            IN2=NNT1(iCol)+NNTxx(iCol)                                  
            SP2(IN1:IN2,1:IMMAX,:)=SP(1:INMAX,1:IMMAX,:)                
         ENDIF                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! One processor option, just copy one array into another       
         !-----------------------------------------------------         
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMT                                              
                  DO IN=1,NNT                                           
                     Z2(IN,IM,IHEM,L)=Z(IN,IM,IHEM,L)                   
                     D2(IN,IM,IHEM,L)=D(IN,IM,IHEM,L)                   
                     T2(IN,IM,IHEM,L)=T(IN,IM,IHEM,L)                   
                     DO ITRAC=1,NTRAC                                   
                        TRA2(IN,IM,IHEM,L,ITRAC)=                       
     &                       TRA(IN,IM,IHEM,L,ITRAC)                    
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO IHEM=1,NHEM                                                 
            DO IM=1,MMT                                                 
               DO IN=1,NNT                                              
                  SP2(IN,IM,IHEM)=SP(IN,IM,IHEM)                        
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Transposing the half transforms so that the zonal wavenumbers         
! are complete instead of the latitudes, so that the zonal              
! wavenumber dimension is ready to be converted to longitudes           
! using FFT991 to produce grid space variables. Instead the             
! latitudes become split across the rows of PEs.                        
!=====================================================                  
      SUBROUTINE TRANS2(UG,VG,ZG,DG,TG,TRAG,PLG,PJG,PMG)                
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Similar to variables in GRIDP but in grid space, and without          
! equivalent variables for CHIG, SFG, FVGT and FUGT. The levels         
! are divided across the columns of PEs; the latitudes are divided      
! across the row of PEs; and the zonal wavenumbers/longitudes are       
! complete.                                                             
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA2/UG2,FUG2,VG2,FVG2,TG2,TNLG2,PLG2                   
     &     ,TRAG2,TRANLG2                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FUG2(MGPP,NHEM,JGy,NLx)                                   
! V_chi, then V (divergent wind)                                        
      REAL :: VG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FVG2(MGPP,NHEM,JGy,NLx)                                   
! Temperature anomaly (T')                                              
      REAL :: TG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: TNLG2(MGPP,NHEM,JGy,NLx)                                  
! ln p* or p*                                                           
      REAL :: PLG2(MGPP,NHEM,JGy)                                       
! Tracer term                                                           
      REAL :: TRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                            
      REAL :: TRANLG2(MGPP,NHEM,JGy,NLx,NTRAC)                          
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB2/ZG2,DG2,PJG2,PMG2,SPG2,VPG2,EG2,UTG2,VTG2          
     &     ,UTRAG2,VTRAG2                                               
     &     ,CHIG2,SFG2                                                  
! Vorticity (zeta)                                                      
      REAL :: ZG2(MGPP,NHEM,JGy,NLx)                                    
! Divergence (d)                                                        
      REAL :: DG2(MGPP,NHEM,JGy,NLx)                                    
! (1-mu^2) (d(ln p*)/d mu)                                              
      REAL :: PJG2(MGPP,NHEM,JGy)                                       
! (d(ln p*)/d lambda)                                                   
      REAL :: PMG2(MGPP,NHEM,JGy)                                       
      REAL :: SPG2(MGPP,NHEM,JGy)                                       
      REAL :: VPG2(MGPP,NHEM,JGy)                                       
! (U^2 + V^2)                                                           
      REAL :: EG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: UTG2(MGPP,NHEM,JGy,NLx)                                   
      REAL :: VTG2(MGPP,NHEM,JGy,NLx)                                   
      REAL :: UTRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                           
      REAL :: VTRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                           
      REAL :: CHIG2(MGPP,NHEM,JGy,NLx)                                  
      REAL :: SFG2(MGPP,NHEM,JGy,NLx)                                   
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variables - these are the variables in common block            
! GRIDP but defined as REALs, instead of COMPLEXs.                      
!-----------------------------------------------------                  
      REAL :: UG(4*MMTy,NHEM,JG,NLx)                                    
      REAL :: VG(4*MMTy,NHEM,JG,NLx)                                    
      REAL :: ZG(4*MMTy,NHEM,JG,NLx)                                    
      REAL :: DG(4*MMTy,NHEM,JG,NLx)                                    
      REAL :: TG(4*MMTy,NHEM,JG,NLx)                                    
      REAL :: TRAG(4*MMTy,NHEM,JG,NLx,NTRAC)                            
      REAL :: PLG(4*MMTy,NHEM,JG)                                       
      REAL :: PJG(4*MMTy,NHEM,JG)                                       
      REAL :: PMG(4*MMTy,NHEM,JG)                                       
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=4000                                 
      REAL :: HoldVar1(4*MMTy,NHEM,JGy,NLx,5+NTRAC)                     
      REAL :: HoldVar2(4*MMTy,NHEM,JGy,3)                               
      INTEGER :: iPe                                                    
      INTEGER :: iRow2                                                  
      INTEGER :: IMMAX, JMAX, LMAX                                      
      INTEGER :: IM1, IM2, IM3, IM4                                     
      INTEGER :: J1, J2                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, ITRAC, J, L                               
!-----------------------------------------------------                  
! Transpose                                                             
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         !-----------------------------------------------------         
         ! Zero arrays, before writing to them                          
         !-----------------------------------------------------         
         UG2(:,:,:,:)=0.0                                               
         VG2(:,:,:,:)=0.0                                               
         ZG2(:,:,:,:)=0.0                                               
         DG2(:,:,:,:)=0.0                                               
         TG2(:,:,:,:)=0.0                                               
         TRAG2(:,:,:,:,:)=0.0                                           
         PLG2(:,:,:)=0.0                                                
         PJG2(:,:,:)=0.0                                                
         PMG2(:,:,:)=0.0                                                
         !-----------------------------------------------------         
         ! Loop over PEs on the same column to send them data           
         !-----------------------------------------------------         
         DO iRow2=1,Py                                                  
            iPe=PeGrid(iCol,iRow2)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Store multi-layer variables in HoldVar1                
               !-----------------------------------------------------   
               IMMAX=4*MMTyy(iRow)                                      
               JMAX=JGyy(iRow2)                                         
               LMAX=NLxx(iCol)                                          
               J1=JJ1(iRow2)+1                                          
               J2=JJ1(iRow2)+JGyy(iRow2)                                
               HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,1)=                     
     &              UG(1:IMMAX,:,J1:J2,1:LMAX)                          
               HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,2)=                     
     &              VG(1:IMMAX,:,J1:J2,1:LMAX)                          
               HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,3)=                     
     &              ZG(1:IMMAX,:,J1:J2,1:LMAX)                          
               HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,4)=                     
     &              DG(1:IMMAX,:,J1:J2,1:LMAX)                          
               HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,5)=                     
     &              TG(1:IMMAX,:,J1:J2,1:LMAX)                          
               DO ITRAC=1,NTRAC                                         
                  HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,5+ITRAC)=            
     &                 TRAG(1:IMMAX,:,J1:J2,1:LMAX,ITRAC)               
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=(5+NTRAC)*4*MMTy*NHEM*JGy*NLx                  
               iTag=TagRef+100*iRow+10*iRow2+iCol                       
               CALL MPI_BSEND(HoldVar1,nBuffSize,ParaReal,iRecv,iTag,   
     &              MPI_COMM_WORLD,ierr)                                
            ENDIF                                                       
         ENDDO                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to receive data                
         !-----------------------------------------------------         
         DO iRow2=1,Py                                                  
            iPe=PeGrid(iCol,iRow2)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Receive message                                        
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=(5+NTRAC)*4*MMTy*NHEM*JGy*NLx                  
               iTag=TagRef+100*iRow2+10*iRow+iCol                       
               CALL MPI_RECV(HoldVar1,nBuffSize,ParaReal,iSend,iTag,    
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write to UG2 etc                                       
               !-----------------------------------------------------   
               IMMAX=2*MMTyy(iRow2)                                     
               IM1=2*MMT1(iRow2)                                        
               IM3=4*MMT-2*(MMT1(iRow2)+MMTyy(iRow2))                   
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           UG2(IM1+I,IHEM,J,L)=HoldVar1(I,IHEM,J,L,1)   
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           UG2(IM3+I,IHEM,J,L)=                         
     &                          HoldVar1(IMMAX+I,IHEM,J,L,1)            
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           VG2(IM1+I,IHEM,J,L)=HoldVar1(I,IHEM,J,L,2)   
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           VG2(IM3+I,IHEM,J,L)=                         
     &                          HoldVar1(IMMAX+I,IHEM,J,L,2)            
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           ZG2(IM1+I,IHEM,J,L)=HoldVar1(I,IHEM,J,L,3)   
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           ZG2(IM3+I,IHEM,J,L)=                         
     &                          HoldVar1(IMMAX+I,IHEM,J,L,3)            
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           DG2(IM1+I,IHEM,J,L)=HoldVar1(I,IHEM,J,L,4)   
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           DG2(IM3+I,IHEM,J,L)=                         
     &                          HoldVar1(IMMAX+I,IHEM,J,L,4)            
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           TG2(IM1+I,IHEM,J,L)=HoldVar1(I,IHEM,J,L,5)   
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           TG2(IM3+I,IHEM,J,L)=                         
     &                          HoldVar1(IMMAX+I,IHEM,J,L,5)            
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO ITRAC=1,NTRAC                                         
                  DO L=1,NLxx(iCol)                                     
                     DO J=1,JGyy(iRow)                                  
                        DO IHEM=1,NHEM                                  
                           DO I=1,IMMAX                                 
                              TRAG2(IM1+I,IHEM,J,L,ITRAC)=              
     &                             HoldVar1(I,IHEM,J,L,5+ITRAC)         
                           ENDDO                                        
                           DO I=1,IMMAX                                 
                              TRAG2(IM3+I,IHEM,J,L,ITRAC)=              
     &                             HoldVar1(IMMAX+I,IHEM,J,L,5+ITRAC)   
                           ENDDO                                        
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDIF                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Data is already contained on this PE                         
         !-----------------------------------------------------         
         IMMAX=2*MMTyy(iRow)                                            
         IM1=2*MMT1(iRow)                                               
         IM3=4*MMT-2*(MMT1(iRow)+MMTyy(iRow))                           
         J1=JJ1(iRow)                                                   
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     UG2(IM1+I,IHEM,J,L)=UG(I,IHEM,J1+J,L)              
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     UG2(IM3+I,IHEM,J,L)=UG(IMMAX+I,IHEM,J1+J,L)        
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     VG2(IM1+I,IHEM,J,L)=VG(I,IHEM,J1+J,L)              
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     VG2(IM3+I,IHEM,J,L)=VG(IMMAX+I,IHEM,J1+J,L)        
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     ZG2(IM1+I,IHEM,J,L)=ZG(I,IHEM,J1+J,L)              
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     ZG2(IM3+I,IHEM,J,L)=ZG(IMMAX+I,IHEM,J1+J,L)        
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     DG2(IM1+I,IHEM,J,L)=DG(I,IHEM,J1+J,L)              
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     DG2(IM3+I,IHEM,J,L)=DG(IMMAX+I,IHEM,J1+J,L)        
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     TG2(IM1+I,IHEM,J,L)=TG(I,IHEM,J1+J,L)              
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     TG2(IM3+I,IHEM,J,L)=TG(IMMAX+I,IHEM,J1+J,L)        
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO ITRAC=1,NTRAC                                               
            DO L=1,NLxx(iCol)                                           
               DO J=1,JGyy(iRow)                                        
                  DO IHEM=1,NHEM                                        
                     DO I=1,IMMAX                                       
                        TRAG2(IM1+I,IHEM,J,L,ITRAC)=                    
     &                       TRAG(I,IHEM,J1+J,L,ITRAC)                  
                     ENDDO                                              
                     DO I=1,IMMAX                                       
                        TRAG2(IM3+I,IHEM,J,L,ITRAC)=                    
     &                       TRAG(IMMAX+I,IHEM,J1+J,L,ITRAC)            
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Single layer variables are only stored when iCol=1           
         !-----------------------------------------------------         
         IF (iCol.EQ.1) THEN                                            
            !-----------------------------------------------------      
            ! Loop over PEs on the same row to send data                
            !-----------------------------------------------------      
            DO iRow2=1,Py                                               
               iPe=PeGrid(iCol,iRow2)                                   
               IF (iPe.NE.MyPe) THEN                                    
                  !-----------------------------------------------------
                  ! Single layer variables are only stored on first     
                  ! column.                                             
                  !-----------------------------------------------------
                  IMMAX=4*MMTyy(iRow)                                   
                  JMAX=JGyy(iRow2)                                      
                  J1=JJ1(iRow2)+1                                       
                  J2=JJ1(iRow2)+JGyy(iRow2)                             
                  HoldVar2(1:IMMAX,:,1:JMAX,1)=PLG(1:IMMAX,:,J1:J2)     
                  HoldVar2(1:IMMAX,:,1:JMAX,2)=PJG(1:IMMAX,:,J1:J2)     
                  HoldVar2(1:IMMAX,:,1:JMAX,3)=PMG(1:IMMAX,:,J1:J2)     
                  !---------------------------------------------------- 
                  ! Send message                                        
                  !---------------------------------------------------- 
                  iRecv=iPe                                             
                  nBuffSize=12*MMTy*NHEM*JGy                            
                  iTag=TagRef+1001+100*iRow+10*iRow2                    
                  CALL MPI_BSEND(HoldVar2,nBuffSize,ParaReal,iRecv,iTag,
     &                 MPI_COMM_WORLD,ierr)                             
               ENDIF                                                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Loop over PEs on the same row to receive data             
            !-----------------------------------------------------      
            DO iRow2=1,Py                                               
               iPe=PeGrid(iCol,iRow2)                                   
               IF (iPe.NE.MyPe) THEN                                    
                  !---------------------------------------------------- 
                  ! Receive message                                     
                  !---------------------------------------------------- 
                  iSend=iPe                                             
                  nBuffSize=12*MMTy*NHEM*JGy                            
                  iTag=TagRef+1001+100*iRow2+10*iRow                    
                  CALL MPI_RECV(HoldVar2,nBuffSize,ParaReal,iSend,iTag, 
     &                 MPI_COMM_WORLD,istat,ierr)                       
                  !---------------------------------------------------- 
                  ! Write into array                                    
                  !---------------------------------------------------- 
                  IMMAX=2*MMTyy(iRow2)                                  
                  IM1=2*MMT1(iRow2)                                     
                  IM3=4*MMT-2*(MMT1(iRow2)+MMTyy(iRow2))                
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           PLG2(IM1+I,IHEM,J)=HoldVar2(I,IHEM,J,1)      
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           PLG2(IM3+I,IHEM,J)=HoldVar2(IMMAX+I,IHEM,J,1)
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           PJG2(IM1+I,IHEM,J)=HoldVar2(I,IHEM,J,2)      
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           PJG2(IM3+I,IHEM,J)=HoldVar2(IMMAX+I,IHEM,J,2)
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           PMG2(IM1+I,IHEM,J)=HoldVar2(I,IHEM,J,3)      
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           PMG2(IM3+I,IHEM,J)=HoldVar2(IMMAX+I,IHEM,J,3)
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDIF                                                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Data is already stored on this PE                         
            !-----------------------------------------------------      
            IMMAX=2*MMTyy(iRow)                                         
            IM1=2*MMT1(iRow)                                            
            IM3=4*MMT-2*(MMT1(iRow)+MMTyy(iRow))                        
            J1=JJ1(iRow)                                                
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     PLG2(IM1+I,IHEM,J)=PLG(I,IHEM,J1+J)                
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     PLG2(IM3+I,IHEM,J)=PLG(IMMAX+I,IHEM,J1+J)          
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     PJG2(IM1+I,IHEM,J)=PJG(I,IHEM,J1+J)                
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     PJG2(IM3+I,IHEM,J)=PJG(IMMAX+I,IHEM,J1+J)          
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     PMG2(IM1+I,IHEM,J)=PMG(I,IHEM,J1+J)                
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     PMG2(IM3+I,IHEM,J)=PMG(IMMAX+I,IHEM,J1+J)          
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDIF                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! One processor option, just copy one array into another       
         !-----------------------------------------------------         
         UG2(:,:,:,:)=0.0                                               
         VG2(:,:,:,:)=0.0                                               
         ZG2(:,:,:,:)=0.0                                               
         DG2(:,:,:,:)=0.0                                               
         TG2(:,:,:,:)=0.0                                               
         TRAG2(:,:,:,:,:)=0.0                                           
         DO L=1,NL                                                      
            DO J=1,JG                                                   
               DO IHEM=1,NHEM                                           
                  DO I=1,4*MMT                                          
                     UG2(I,IHEM,J,L)=UG(I,IHEM,J,L)                     
                     VG2(I,IHEM,J,L)=VG(I,IHEM,J,L)                     
                     ZG2(I,IHEM,J,L)=ZG(I,IHEM,J,L)                     
                     DG2(I,IHEM,J,L)=DG(I,IHEM,J,L)                     
                     TG2(I,IHEM,J,L)=TG(I,IHEM,J,L)                     
                     DO ITRAC=1,NTRAC                                   
                        TRAG2(I,IHEM,J,L,ITRAC)=TRAG(I,IHEM,J,L,ITRAC)  
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         PLG2(:,:,:)=0.0                                                
         PJG2(:,:,:)=0.0                                                
         PMG2(:,:,:)=0.0                                                
         DO J=1,JG                                                      
            DO IHEM=1,NHEM                                              
               DO I=1,4*MMT                                             
                  PLG2(I,IHEM,J) =PLG(I,IHEM,J)                         
                  PJG2(I,IHEM,J) =PJG(I,IHEM,J)                         
                  PMG2(I,IHEM,J) =PMG(I,IHEM,J)                         
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Transposing the half transforms so that the zonal wavenumbers         
! are complete instead of the latitudes, so that the zonal              
! wavenumber dimension is ready to be converted to longitudes           
! using FFT991 to produce grid space variables. Instead the             
! latitudes become split across the rows of PEs.                        
!=====================================================                  
      SUBROUTINE TRANS2A(CHIG,SFG)                                      
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB2/ZG2,DG2,PJG2,PMG2,SPG2,VPG2,EG2,UTG2,VTG2          
     &     ,UTRAG2,VTRAG2                                               
     &     ,CHIG2,SFG2                                                  
! Vorticity (zeta)                                                      
      REAL :: ZG2(MGPP,NHEM,JGy,NLx)                                    
! Divergence (d)                                                        
      REAL :: DG2(MGPP,NHEM,JGy,NLx)                                    
! (1-mu^2) (d(ln p*)/d mu)                                              
      REAL :: PJG2(MGPP,NHEM,JGy)                                       
! (d(ln p*)/d lambda)                                                   
      REAL :: PMG2(MGPP,NHEM,JGy)                                       
      REAL :: SPG2(MGPP,NHEM,JGy)                                       
      REAL :: VPG2(MGPP,NHEM,JGy)                                       
! (U^2 + V^2)                                                           
      REAL :: EG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: UTG2(MGPP,NHEM,JGy,NLx)                                   
      REAL :: VTG2(MGPP,NHEM,JGy,NLx)                                   
      REAL :: UTRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                           
      REAL :: VTRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                           
      REAL :: CHIG2(MGPP,NHEM,JGy,NLx)                                  
      REAL :: SFG2(MGPP,NHEM,JGy,NLx)                                   
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variables - these are the variables in common block            
! GRIDP but defined as REALs, instead of COMPLEXs.                      
!-----------------------------------------------------                  
      REAL :: CHIG(4*MMTy,NHEM,JG,NLx)                                  
      REAL :: SFG(4*MMTy,NHEM,JG,NLx)                                   
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=6000                                 
      REAL :: HoldVar1(4*MMTy,NHEM,JGy,NLx,2)                           
      INTEGER :: iPe                                                    
      INTEGER :: iRow2                                                  
      INTEGER :: IMMAX, JMAX, LMAX                                      
      INTEGER :: IM1, IM2, IM3, IM4                                     
      INTEGER :: J1, J2                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, ITRAC, J, L                               
!-----------------------------------------------------                  
! Transpose                                                             
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         !-----------------------------------------------------         
         ! Zero arrays, before writing to them                          
         !-----------------------------------------------------         
         CHIG2(:,:,:,:)=0.0                                             
         SFG2(:,:,:,:)=0.0                                              
         !-----------------------------------------------------         
         ! Loop over PEs on the same column to send them data           
         !-----------------------------------------------------         
         DO iRow2=1,Py                                                  
            iPe=PeGrid(iCol,iRow2)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Store multi-layer variables in HoldVar1                
               !-----------------------------------------------------   
               IMMAX=4*MMTyy(iRow)                                      
               JMAX=JGyy(iRow2)                                         
               LMAX=NLxx(iCol)                                          
               J1=JJ1(iRow2)+1                                          
               J2=JJ1(iRow2)+JGyy(iRow2)                                
               HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,1)=                     
     &              CHIG(1:IMMAX,:,J1:J2,1:LMAX)                        
               HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,2)=                     
     &              SFG(1:IMMAX,:,J1:J2,1:LMAX)                         
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=8*MMTy*NHEM*JGy*NLx                            
               iTag=TagRef+100*iRow+10*iRow2+iCol                       
               CALL MPI_BSEND(HoldVar1,nBuffSize,ParaReal,iRecv,iTag,   
     &              MPI_COMM_WORLD,ierr)                                
            ENDIF                                                       
         ENDDO                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to receive data                
         !-----------------------------------------------------         
         DO iRow2=1,Py                                                  
            iPe=PeGrid(iCol,iRow2)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Receive message                                        
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=8*MMTy*NHEM*JGy*NLx                            
               iTag=TagRef+100*iRow2+10*iRow+iCol                       
               CALL MPI_RECV(HoldVar1,nBuffSize,ParaReal,iSend,iTag,    
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write to CHIG2 and SFG2                                
               !-----------------------------------------------------   
               IMMAX=2*MMTyy(iRow2)                                     
               IM1=2*MMT1(iRow2)                                        
               IM3=4*MMT-2*(MMT1(iRow2)+MMTyy(iRow2))                   
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           CHIG2(IM1+I,IHEM,J,L)=HoldVar1(I,IHEM,J,L,1) 
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           CHIG2(IM3+I,IHEM,J,L)=                       
     &                          HoldVar1(IMMAX+I,IHEM,J,L,1)            
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           SFG2(IM1+I,IHEM,J,L)=HoldVar1(I,IHEM,J,L,2)  
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           SFG2(IM3+I,IHEM,J,L)=                        
     &                          HoldVar1(IMMAX+I,IHEM,J,L,2)            
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDIF                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Data is already contained on this PE                         
         !-----------------------------------------------------         
         IMMAX=2*MMTyy(iRow)                                            
         IM1=2*MMT1(iRow)                                               
         IM3=4*MMT-2*(MMT1(iRow)+MMTyy(iRow))                           
         J1=JJ1(iRow)                                                   
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     CHIG2(IM1+I,IHEM,J,L)=CHIG(I,IHEM,J1+J,L)          
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     CHIG2(IM3+I,IHEM,J,L)=CHIG(IMMAX+I,IHEM,J1+J,L)    
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     SFG2(IM1+I,IHEM,J,L)=SFG(I,IHEM,J1+J,L)            
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     SFG2(IM3+I,IHEM,J,L)=SFG(IMMAX+I,IHEM,J1+J,L)      
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! One processor option, just copy one array into another       
         !-----------------------------------------------------         
         CHIG2(:,:,:,:)=0.0                                             
         SFG2(:,:,:,:)=0.0                                              
         DO L=1,NL                                                      
            DO J=1,JG                                                   
               DO IHEM=1,NHEM                                           
                  DO I=1,4*MMT                                          
                     CHIG2(I,IHEM,J,L)=CHIG(I,IHEM,J,L)                 
                     SFG2(I,IHEM,J,L)=SFG(I,IHEM,J,L)                   
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Transposing the half transforms so that the zonal wavenumbers         
! are complete instead of the latitudes, so that the zonal              
! wavenumber dimension is ready to be converted to longitudes           
! using FFT991 to produce grid space variables. Instead the             
! latitudes become split across the rows of PEs.                        
!=====================================================                  
      SUBROUTINE TRANS2B(UG,VG,TG,TRAG,PLG)                             
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Similar to variables in GRIDP but in grid space, and without          
! equivalent variables for CHIG, SFG, FVGT and FUGT. The levels         
! are divided across the columns of PEs; the latitudes are divided      
! across the row of PEs; and the zonal wavenumbers/longitudes are       
! complete.                                                             
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA2/UG2,FUG2,VG2,FVG2,TG2,TNLG2,PLG2                   
     &     ,TRAG2,TRANLG2                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FUG2(MGPP,NHEM,JGy,NLx)                                   
! V_chi, then V (divergent wind)                                        
      REAL :: VG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FVG2(MGPP,NHEM,JGy,NLx)                                   
! Temperature anomaly (T')                                              
      REAL :: TG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: TNLG2(MGPP,NHEM,JGy,NLx)                                  
! ln p* or p*                                                           
      REAL :: PLG2(MGPP,NHEM,JGy)                                       
! Tracer term                                                           
      REAL :: TRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                            
      REAL :: TRANLG2(MGPP,NHEM,JGy,NLx,NTRAC)                          
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variables - these are the variables in common block            
! GRIDP but defined as REALs, instead of COMPLEXs.                      
!-----------------------------------------------------                  
      REAL :: UG(4*MMTy,NHEM,JG,NLx)                                    
      REAL :: VG(4*MMTy,NHEM,JG,NLx)                                    
      REAL :: TG(4*MMTy,NHEM,JG,NLx)                                    
      REAL :: TRAG(4*MMTy,NHEM,JG,NLx,NTRAC)                            
      REAL :: PLG(4*MMTy,NHEM,JG)                                       
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=7000                                 
      REAL :: HoldVar1(4*MMTy,NHEM,JGy,NLx,3+NTRAC)                     
      REAL :: HoldVar2(4*MMTy,NHEM,JGy)                                 
      INTEGER :: iPe                                                    
      INTEGER :: iRow2                                                  
      INTEGER :: IMMAX, JMAX, LMAX                                      
      INTEGER :: IM1, IM2, IM3, IM4                                     
      INTEGER :: J1, J2                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, ITRAC, J, L                               
!-----------------------------------------------------                  
! Transpose                                                             
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         !-----------------------------------------------------         
         ! Zero arrays, before writing to them                          
         !-----------------------------------------------------         
         UG2(:,:,:,:)=0.0                                               
         VG2(:,:,:,:)=0.0                                               
         TG2(:,:,:,:)=0.0                                               
         TRAG2(:,:,:,:,:)=0.0                                           
         PLG2(:,:,:)=0.0                                                
         !-----------------------------------------------------         
         ! Loop over PEs on the same column to send them data           
         !-----------------------------------------------------         
         DO iRow2=1,Py                                                  
            iPe=PeGrid(iCol,iRow2)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Store multi-layer variables in HoldVar1                
               !-----------------------------------------------------   
               IMMAX=4*MMTyy(iRow)                                      
               JMAX=JGyy(iRow2)                                         
               LMAX=NLxx(iCol)                                          
               J1=JJ1(iRow2)+1                                          
               J2=JJ1(iRow2)+JGyy(iRow2)                                
               HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,1)=                     
     &              UG(1:IMMAX,:,J1:J2,1:LMAX)                          
               HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,2)=                     
     &              VG(1:IMMAX,:,J1:J2,1:LMAX)                          
               HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,3)=                     
     &              TG(1:IMMAX,:,J1:J2,1:LMAX)                          
               DO ITRAC=1,NTRAC                                         
                  HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,3+ITRAC)=            
     &                 TRAG(1:IMMAX,:,J1:J2,1:LMAX,ITRAC)               
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=(3+NTRAC)*4*MMTy*NHEM*JGy*NLx                  
               iTag=TagRef+100*iRow+10*iRow2+iCol                       
               CALL MPI_BSEND(HoldVar1,nBuffSize,ParaReal,iRecv,iTag,   
     &              MPI_COMM_WORLD,ierr)                                
            ENDIF                                                       
         ENDDO                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to receive data                
         !-----------------------------------------------------         
         DO iRow2=1,Py                                                  
            iPe=PeGrid(iCol,iRow2)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Receive message                                        
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=(3+NTRAC)*4*MMTy*NHEM*JGy*NLx                  
               iTag=TagRef+100*iRow2+10*iRow+iCol                       
               CALL MPI_RECV(HoldVar1,nBuffSize,ParaReal,iSend,iTag,    
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write to UG2 etc                                       
               !-----------------------------------------------------   
               IMMAX=2*MMTyy(iRow2)                                     
               IM1=2*MMT1(iRow2)                                        
               IM3=4*MMT-2*(MMT1(iRow2)+MMTyy(iRow2))                   
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           UG2(IM1+I,IHEM,J,L)=HoldVar1(I,IHEM,J,L,1)   
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           UG2(IM3+I,IHEM,J,L)=                         
     &                          HoldVar1(IMMAX+I,IHEM,J,L,1)            
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           VG2(IM1+I,IHEM,J,L)=HoldVar1(I,IHEM,J,L,2)   
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           VG2(IM3+I,IHEM,J,L)=                         
     &                          HoldVar1(IMMAX+I,IHEM,J,L,2)            
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           TG2(IM1+I,IHEM,J,L)=HoldVar1(I,IHEM,J,L,3)   
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           TG2(IM3+I,IHEM,J,L)=                         
     &                          HoldVar1(IMMAX+I,IHEM,J,L,3)            
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO ITRAC=1,NTRAC                                         
                  DO L=1,NLxx(iCol)                                     
                     DO J=1,JGyy(iRow)                                  
                        DO IHEM=1,NHEM                                  
                           DO I=1,IMMAX                                 
                              TRAG2(IM1+I,IHEM,J,L,ITRAC)=              
     &                             HoldVar1(I,IHEM,J,L,3+ITRAC)         
                           ENDDO                                        
                           DO I=1,IMMAX                                 
                              TRAG2(IM3+I,IHEM,J,L,ITRAC)=              
     &                             HoldVar1(IMMAX+I,IHEM,J,L,3+ITRAC)   
                           ENDDO                                        
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDIF                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Data is already contained on this PE                         
         !-----------------------------------------------------         
         IMMAX=2*MMTyy(iRow)                                            
         IM1=2*MMT1(iRow)                                               
         IM3=4*MMT-2*(MMT1(iRow)+MMTyy(iRow))                           
         J1=JJ1(iRow)                                                   
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     UG2(IM1+I,IHEM,J,L)=UG(I,IHEM,J1+J,L)              
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     UG2(IM3+I,IHEM,J,L)=UG(IMMAX+I,IHEM,J1+J,L)        
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     VG2(IM1+I,IHEM,J,L)=VG(I,IHEM,J1+J,L)              
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     VG2(IM3+I,IHEM,J,L)=VG(IMMAX+I,IHEM,J1+J,L)        
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     TG2(IM1+I,IHEM,J,L)=TG(I,IHEM,J1+J,L)              
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     TG2(IM3+I,IHEM,J,L)=TG(IMMAX+I,IHEM,J1+J,L)        
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO ITRAC=1,NTRAC                                               
            DO L=1,NLxx(iCol)                                           
               DO J=1,JGyy(iRow)                                        
                  DO IHEM=1,NHEM                                        
                     DO I=1,IMMAX                                       
                        TRAG2(IM1+I,IHEM,J,L,ITRAC)=                    
     &                       TRAG(I,IHEM,J1+J,L,ITRAC)                  
                     ENDDO                                              
                     DO I=1,IMMAX                                       
                        TRAG2(IM3+I,IHEM,J,L,ITRAC)=                    
     &                       TRAG(IMMAX+I,IHEM,J1+J,L,ITRAC)            
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Single layer variables are only stored when iCol=1           
         !-----------------------------------------------------         
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         IF (iCol.EQ.1) THEN                                            
            !-----------------------------------------------------      
            ! Loop over PEs on the same row to send data                
            !-----------------------------------------------------      
            DO iRow2=1,Py                                               
               iPe=PeGrid(iCol,iRow2)                                   
               IF (iPe.NE.MyPe) THEN                                    
                  !-----------------------------------------------------
                  ! Single layer variables are only stored on first     
                  ! column.                                             
                  !-----------------------------------------------------
                  IMMAX=4*MMTyy(iRow)                                   
                  JMAX=JGyy(iRow2)                                      
                  J1=JJ1(iRow2)+1                                       
                  J2=JJ1(iRow2)+JGyy(iRow2)                             
                  HoldVar2(1:IMMAX,:,1:JMAX)=PLG(1:IMMAX,:,J1:J2)       
                  !---------------------------------------------------- 
                  ! Send message                                        
                  !---------------------------------------------------- 
                  iRecv=iPe                                             
                  nBuffSize=4*MMTy*NHEM*JGy                             
                  iTag=TagRef+1001+100*iRow+10*iRow2                    
                  CALL MPI_BSEND(HoldVar2,nBuffSize,ParaReal,iRecv,iTag,
     &                 MPI_COMM_WORLD,ierr)                             
               ENDIF                                                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Loop over PEs on the same row to receive data             
            !-----------------------------------------------------      
            DO iRow2=1,Py                                               
               iPe=PeGrid(iCol,iRow2)                                   
               IF (iPe.NE.MyPe) THEN                                    
                  !---------------------------------------------------- 
                  ! Receive message                                     
                  !---------------------------------------------------- 
                  iSend=iPe                                             
                  nBuffSize=4*MMTy*NHEM*JGy                             
                  iTag=TagRef+1001+100*iRow2+10*iRow                    
                  CALL MPI_RECV(HoldVar2,nBuffSize,ParaReal,iSend,iTag, 
     &                 MPI_COMM_WORLD,istat,ierr)                       
                  !---------------------------------------------------- 
                  ! Write into array                                    
                  !---------------------------------------------------- 
                  IMMAX=2*MMTyy(iRow2)                                  
                  IM1=2*MMT1(iRow2)                                     
                  IM3=4*MMT-2*(MMT1(iRow2)+MMTyy(iRow2))                
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           PLG2(IM1+I,IHEM,J)=HoldVar2(I,IHEM,J)        
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           PLG2(IM3+I,IHEM,J)=HoldVar2(IMMAX+I,IHEM,J)  
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDIF                                                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Data is already stored on this PE                         
            !-----------------------------------------------------      
            IMMAX=2*MMTyy(iRow)                                         
            IM1=2*MMT1(iRow)                                            
            IM3=4*MMT-2*(MMT1(iRow)+MMTyy(iRow))                        
            J1=JJ1(iRow)                                                
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     PLG2(IM1+I,IHEM,J)=PLG(I,IHEM,J1+J)                
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     PLG2(IM3+I,IHEM,J)=PLG(IMMAX+I,IHEM,J1+J)          
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDIF                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! One processor option, just copy one array into another       
         !-----------------------------------------------------         
         UG2(:,:,:,:)=0.0                                               
         VG2(:,:,:,:)=0.0                                               
         TG2(:,:,:,:)=0.0                                               
         TRAG2(:,:,:,:,:)=0.0                                           
         DO L=1,NL                                                      
            DO J=1,JG                                                   
               DO IHEM=1,NHEM                                           
                  DO I=1,4*MMT                                          
                     UG2(I,IHEM,J,L)=UG(I,IHEM,J,L)                     
                     VG2(I,IHEM,J,L)=VG(I,IHEM,J,L)                     
                     TG2(I,IHEM,J,L)=TG(I,IHEM,J,L)                     
                     DO ITRAC=1,NTRAC                                   
                        TRAG2(I,IHEM,J,L,ITRAC)=TRAG(I,IHEM,J,L,ITRAC)  
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         PLG2(:,:,:)=0.0                                                
         DO J=1,JG                                                      
            DO IHEM=1,NHEM                                              
               DO I=1,4*MMT                                             
                  PLG2(I,IHEM,J) =PLG(I,IHEM,J)                         
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Transposing the grid space variables so that the levels               
! are complete instead of the longitudes, because complete              
! levels are needed in MGRMLT. Instead the longitude are                
! split across the columns of PEs.                                      
!=====================================================                  
      SUBROUTINE TRANS3                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Similar to variables in GRIDP but in grid space, and without          
! equivalent variables for CHIG, SFG, FVGT and FUGT. The levels         
! are divided across the columns of PEs; the latitudes are divided      
! across the row of PEs; and the zonal wavenumbers/longitudes are       
! complete.                                                             
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA2/UG2,FUG2,VG2,FVG2,TG2,TNLG2,PLG2                   
     &     ,TRAG2,TRANLG2                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FUG2(MGPP,NHEM,JGy,NLx)                                   
! V_chi, then V (divergent wind)                                        
      REAL :: VG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FVG2(MGPP,NHEM,JGy,NLx)                                   
! Temperature anomaly (T')                                              
      REAL :: TG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: TNLG2(MGPP,NHEM,JGy,NLx)                                  
! ln p* or p*                                                           
      REAL :: PLG2(MGPP,NHEM,JGy)                                       
! Tracer term                                                           
      REAL :: TRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                            
      REAL :: TRANLG2(MGPP,NHEM,JGy,NLx,NTRAC)                          
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB2/ZG2,DG2,PJG2,PMG2,SPG2,VPG2,EG2,UTG2,VTG2          
     &     ,UTRAG2,VTRAG2                                               
     &     ,CHIG2,SFG2                                                  
! Vorticity (zeta)                                                      
      REAL :: ZG2(MGPP,NHEM,JGy,NLx)                                    
! Divergence (d)                                                        
      REAL :: DG2(MGPP,NHEM,JGy,NLx)                                    
! (1-mu^2) (d(ln p*)/d mu)                                              
      REAL :: PJG2(MGPP,NHEM,JGy)                                       
! (d(ln p*)/d lambda)                                                   
      REAL :: PMG2(MGPP,NHEM,JGy)                                       
      REAL :: SPG2(MGPP,NHEM,JGy)                                       
      REAL :: VPG2(MGPP,NHEM,JGy)                                       
! (U^2 + V^2)                                                           
      REAL :: EG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: UTG2(MGPP,NHEM,JGy,NLx)                                   
      REAL :: VTG2(MGPP,NHEM,JGy,NLx)                                   
      REAL :: UTRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                           
      REAL :: VTRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                           
      REAL :: CHIG2(MGPP,NHEM,JGy,NLx)                                  
      REAL :: SFG2(MGPP,NHEM,JGy,NLx)                                   
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB3/ZG3,DG3,PJG3,PMG3,SPG3,VPG3,EG3,UTG3,VTG3          
     &     ,UTRAG3,VTRAG3                                               
     &     ,CHIG3,SFG3                                                  
! Vorticity (zeta)                                                      
      REAL :: ZG3(MGx,NHEM,JGy,NL)                                      
! Divergence (d)                                                        
      REAL :: DG3(MGx,NHEM,JGy,NL)                                      
! (1-mu^2) (d(ln p*)/d mu)                                              
      REAL :: PJG3(MGx,NHEM,JGy)                                        
! (d(ln p*)/d lambda)                                                   
      REAL :: PMG3(MGx,NHEM,JGy)                                        
      REAL :: SPG3(MGx,NHEM,JGy)                                        
      REAL :: VPG3(MGx,NHEM,JGy)                                        
! (U^2 + V^2)                                                           
      REAL :: EG3(MGx,NHEM,JGy,NL)                                      
      REAL :: UTG3(MGx,NHEM,JGy,NL)                                     
      REAL :: VTG3(MGx,NHEM,JGy,NL)                                     
      REAL :: UTRAG3(MGx,NHEM,JGy,NL,NTRAC)                             
      REAL :: VTRAG3(MGx,NHEM,JGy,NL,NTRAC)                             
      REAL :: CHIG3(MGx,NHEM,JGy,NL)                                    
      REAL :: SFG3(MGx,NHEM,JGy,NL)                                     
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=9000                                 
      REAL :: HoldVar1(MGx,NHEM,JGy,NLx,5+NTRAC)                        
      REAL :: HoldVar2(MGx,NHEM,JGy,3)                                  
      INTEGER :: iPe                                                    
      INTEGER :: iCol2                                                  
      INTEGER :: IMAX, JMAX, LMAX                                       
      INTEGER :: I1, I2                                                 
      INTEGER :: L1, L2                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, ITRAC, J, L                                   
!-----------------------------------------------------                  
! Transpose                                                             
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to send them data              
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Store multi-layer data in HoldVar1                     
               !-----------------------------------------------------   
               IMAX=MGxx(iCol2)                                         
               JMAX=JGyy(iRow)                                          
               LMAX=NLxx(iCol)                                          
               I1=II1(iCol2)+1                                          
               I2=II1(iCol2)+IMAX                                       
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,1)=                      
     &              UG2(I1:I2,:,1:JMAX,1:LMAX)                          
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,2)=                      
     &              VG2(I1:I2,:,1:JMAX,1:LMAX)                          
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,3)=                      
     &              ZG2(I1:I2,:,1:JMAX,1:LMAX)                          
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,4)=                      
     &              DG2(I1:I2,:,1:JMAX,1:LMAX)                          
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,5)=                      
     &              TG2(I1:I2,:,1:JMAX,1:LMAX)                          
               DO ITRAC=1,NTRAC                                         
                  HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,5+ITRAC)=             
     &                 TRAG2(I1:I2,:,1:JMAX,1:LMAX,ITRAC)               
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=(5+NTRAC)*MGx*NHEM*JGy*NLx                     
               iTag=TagRef+100*iCol+10*iCol2+iRow                       
               CALL MPI_BSEND(HoldVar1,nBuffSize,ParaReal,iRecv,iTag,   
     &              MPI_COMM_WORLD,ierr)                                
            ENDIF                                                       
         ENDDO                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to receive data                
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Receive message                                        
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=(5+NTRAC)*MGx*NHEM*JGy*NLx                     
               iTag=TagRef+100*iCol2+10*iCol+iRow                       
               CALL MPI_RECV(HoldVar1,nBuffSize,ParaReal,iSend,iTag,    
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write to UG3 etc                                       
               !-----------------------------------------------------   
               IMAX=MGxx(iCol)                                          
               JMAX=JGyy(iRow)                                          
               LMAX=NLxx(iCol2)                                         
               L1=KK1(iCol2)+1                                          
               L2=KK1(iCol2)+LMAX                                       
               UG3(1:IMAX,:,1:JMAX,L1:L2)=                              
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,1)                  
               VG3(1:IMAX,:,1:JMAX,L1:L2)=                              
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,2)                  
               ZG3(1:IMAX,:,1:JMAX,L1:L2)=                              
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,3)                  
               DG3(1:IMAX,:,1:JMAX,L1:L2)=                              
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,4)                  
               TG3(1:IMAX,:,1:JMAX,L1:L2)=                              
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,5)                  
               DO ITRAC=1,NTRAC                                         
                  TRAG3(1:IMAX,:,1:JMAX,L1:L2,ITRAC)=                   
     &                 HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,5+ITRAC)         
               ENDDO                                                    
            ENDIF                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Data is already contained on this PE                         
         !-----------------------------------------------------         
         IMAX=MGxx(iCol)                                                
         JMAX=JGyy(iRow)                                                
         LMAX=NLxx(iCol)                                                
         I1=II1(iCol)+1                                                 
         I2=II1(iCol)+IMAX                                              
         L1=KK1(iCol)+1                                                 
         L2=KK1(iCol)+NLxx(iCol)                                        
         UG3(1:IMAX,:,1:JMAX,L1:L2)=UG2(I1:I2,:,1:JMAX,1:LMAX)          
         VG3(1:IMAX,:,1:JMAX,L1:L2)=VG2(I1:I2,:,1:JMAX,1:LMAX)          
         DG3(1:IMAX,:,1:JMAX,L1:L2)=DG2(I1:I2,:,1:JMAX,1:LMAX)          
         ZG3(1:IMAX,:,1:JMAX,L1:L2)=ZG2(I1:I2,:,1:JMAX,1:LMAX)          
         TG3(1:IMAX,:,1:JMAX,L1:L2)=TG2(I1:I2,:,1:JMAX,1:LMAX)          
         DO ITRAC=1,NTRAC                                               
            TRAG3(1:IMAX,:,1:JMAX,L1:L2,ITRAC)=                         
     &           TRAG2(I1:I2,:,1:JMAX,1:LMAX,ITRAC)                     
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Single layer variables are only stored on first              
         ! column.                                                      
         !-----------------------------------------------------         
         IF (iCol.EQ.1) THEN                                            
            DO iCol2=2,Px                                               
               iPe=PeGrid(iCol2,iRow)                                   
               !-----------------------------------------------------   
               ! Store single layer data in HoldVar2                    
               !-----------------------------------------------------   
               IMAX=MGxx(iCol2)                                         
               JMAX=JGyy(iRow)                                          
               I1=II1(iCol2)+1                                          
               I2=II1(iCol2)+IMAX                                       
               HoldVar2(1:IMAX,:,1:JMAX,1)=PLG2(I1:I2,:,1:JMAX)         
               HoldVar2(1:IMAX,:,1:JMAX,2)=PJG2(I1:I2,:,1:JMAX)         
               HoldVar2(1:IMAX,:,1:JMAX,3)=PMG2(I1:I2,:,1:JMAX)         
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=3*MGx*NHEM*JGy                                 
               iTag=TagRef+1100+10*iCol2+iRow                           
               CALL MPI_BSEND(HoldVar2,nBuffSize,ParaReal,iRecv,iTag,   
     &              MPI_COMM_WORLD,ierr)                                
            ENDDO                                                       
         ENDIF                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         IF (iCol.GT.1) THEN                                            
            !---------------------------------------------------        
            ! Receive message                                           
            !---------------------------------------------------        
            iSend=PeGrid(1,iRow)                                        
            nBuffSize=3*MGx*NHEM*JGy                                    
            iTag=TagRef+1100+10*iCol+iRow                               
            CALL MPI_RECV(HoldVar2,nBuffSize,ParaReal,iSend,iTag,       
     &           MPI_COMM_WORLD,istat,ierr)                             
            !---------------------------------------------------        
            ! Write to PLG3 etc                                         
            !---------------------------------------------------        
            IMAX=MGxx(iCol)                                             
            JMAX=JGyy(iRow)                                             
            PLG3(1:IMAX,:,1:JMAX)=HoldVar2(1:IMAX,:,1:JMAX,1)           
            PJG3(1:IMAX,:,1:JMAX)=HoldVar2(1:IMAX,:,1:JMAX,2)           
            PMG3(1:IMAX,:,1:JMAX)=HoldVar2(1:IMAX,:,1:JMAX,3)           
         ELSE                                                           
            !-----------------------------------------------------      
            ! Data for PE on column 1 is already contained on this PE   
            !-----------------------------------------------------      
            IMAX=MGxx(iCol)                                             
            JMAX=JGyy(iRow)                                             
            I1=II1(iCol)+1                                              
            I2=II1(iCol)+MGxx(iCol)                                     
            PLG3(1:IMAX,:,1:JMAX)=PLG2(I1:I2,:,1:JMAX)                  
            PJG3(1:IMAX,:,1:JMAX)=PJG2(I1:I2,:,1:JMAX)                  
            PMG3(1:IMAX,:,1:JMAX)=PMG2(I1:I2,:,1:JMAX)                  
         ENDIF                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! One processor option, just copy one array into another       
         !-----------------------------------------------------         
         DO L=1,NL                                                      
            DO J=1,JGy                                                  
               DO IHEM=1,NHEM                                           
                  DO I=1,MG                                             
                     UG3(I,IHEM,J,L)=UG2(I,IHEM,J,L)                    
                     VG3(I,IHEM,J,L)=VG2(I,IHEM,J,L)                    
                     ZG3(I,IHEM,J,L)=ZG2(I,IHEM,J,L)                    
                     DG3(I,IHEM,J,L)=DG2(I,IHEM,J,L)                    
                     TG3(I,IHEM,J,L)=TG2(I,IHEM,J,L)                    
                     DO ITRAC=1,NTRAC                                   
                        TRAG3(I,IHEM,J,L,ITRAC)=TRAG2(I,IHEM,J,L,ITRAC) 
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO J=1,JGy                                                     
            DO IHEM=1,NHEM                                              
               DO I=1,MG                                                
                  PLG3(I,IHEM,J) =PLG2(I,IHEM,J)                        
                  PJG3(I,IHEM,J) =PJG2(I,IHEM,J)                        
                  PMG3(I,IHEM,J) =PMG2(I,IHEM,J)                        
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Transposing the grid space variables so that the levels               
! are complete instead of the longitudes, because complete              
! levels are needed in MGRMLT. Instead the longitude are                
! split across the columns of PEs.                                      
!=====================================================                  
      SUBROUTINE TRANS3A                                                
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB2/ZG2,DG2,PJG2,PMG2,SPG2,VPG2,EG2,UTG2,VTG2          
     &     ,UTRAG2,VTRAG2                                               
     &     ,CHIG2,SFG2                                                  
! Vorticity (zeta)                                                      
      REAL :: ZG2(MGPP,NHEM,JGy,NLx)                                    
! Divergence (d)                                                        
      REAL :: DG2(MGPP,NHEM,JGy,NLx)                                    
! (1-mu^2) (d(ln p*)/d mu)                                              
      REAL :: PJG2(MGPP,NHEM,JGy)                                       
! (d(ln p*)/d lambda)                                                   
      REAL :: PMG2(MGPP,NHEM,JGy)                                       
      REAL :: SPG2(MGPP,NHEM,JGy)                                       
      REAL :: VPG2(MGPP,NHEM,JGy)                                       
! (U^2 + V^2)                                                           
      REAL :: EG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: UTG2(MGPP,NHEM,JGy,NLx)                                   
      REAL :: VTG2(MGPP,NHEM,JGy,NLx)                                   
      REAL :: UTRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                           
      REAL :: VTRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                           
      REAL :: CHIG2(MGPP,NHEM,JGy,NLx)                                  
      REAL :: SFG2(MGPP,NHEM,JGy,NLx)                                   
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB3/ZG3,DG3,PJG3,PMG3,SPG3,VPG3,EG3,UTG3,VTG3          
     &     ,UTRAG3,VTRAG3                                               
     &     ,CHIG3,SFG3                                                  
! Vorticity (zeta)                                                      
      REAL :: ZG3(MGx,NHEM,JGy,NL)                                      
! Divergence (d)                                                        
      REAL :: DG3(MGx,NHEM,JGy,NL)                                      
! (1-mu^2) (d(ln p*)/d mu)                                              
      REAL :: PJG3(MGx,NHEM,JGy)                                        
! (d(ln p*)/d lambda)                                                   
      REAL :: PMG3(MGx,NHEM,JGy)                                        
      REAL :: SPG3(MGx,NHEM,JGy)                                        
      REAL :: VPG3(MGx,NHEM,JGy)                                        
! (U^2 + V^2)                                                           
      REAL :: EG3(MGx,NHEM,JGy,NL)                                      
      REAL :: UTG3(MGx,NHEM,JGy,NL)                                     
      REAL :: VTG3(MGx,NHEM,JGy,NL)                                     
      REAL :: UTRAG3(MGx,NHEM,JGy,NL,NTRAC)                             
      REAL :: VTRAG3(MGx,NHEM,JGy,NL,NTRAC)                             
      REAL :: CHIG3(MGx,NHEM,JGy,NL)                                    
      REAL :: SFG3(MGx,NHEM,JGy,NL)                                     
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=11000                                
      REAL :: HoldVar1(MGx,NHEM,JGy,NLx,2)                              
      INTEGER :: iPe                                                    
      INTEGER :: iCol2                                                  
      INTEGER :: IMAX, JMAX, LMAX                                       
      INTEGER :: I1, I2                                                 
      INTEGER :: L1, L2                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, ITRAC, J, L                                   
!-----------------------------------------------------                  
! Transpose                                                             
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to send them data              
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Store multi-layer data in HoldVar1                     
               !-----------------------------------------------------   
               IMAX=MGxx(iCol2)                                         
               JMAX=JGyy(iRow)                                          
               LMAX=NLxx(iCol)                                          
               I1=II1(iCol2)+1                                          
               I2=II1(iCol2)+IMAX                                       
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,1)=                      
     &              CHIG2(I1:I2,:,1:JMAX,1:LMAX)                        
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,2)=                      
     &              SFG2(I1:I2,:,1:JMAX,1:LMAX)                         
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=2*MGx*NHEM*JGy*NLx                             
               iTag=TagRef+100*iCol+10*iCol2+iRow                       
               CALL MPI_BSEND(HoldVar1,nBuffSize,ParaReal,iRecv,iTag,   
     &              MPI_COMM_WORLD,ierr)                                
            ENDIF                                                       
         ENDDO                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to receive data                
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Receive message                                        
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=2*MGx*NHEM*JGy*NLx                             
               iTag=TagRef+100*iCol2+10*iCol+iRow                       
               CALL MPI_RECV(HoldVar1,nBuffSize,ParaReal,iSend,iTag,    
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write to CHIG3 etc                                     
               !-----------------------------------------------------   
               IMAX=MGxx(iCol)                                          
               JMAX=JGyy(iRow)                                          
               LMAX=NLxx(iCol2)                                         
               L1=KK1(iCol2)+1                                          
               L2=KK1(iCol2)+LMAX                                       
               CHIG3(1:IMAX,:,1:JMAX,L1:L2)=                            
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,1)                  
               SFG3(1:IMAX,:,1:JMAX,L1:L2)=                             
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,2)                  
            ENDIF                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Data is already contained on this PE                         
         !-----------------------------------------------------         
         IMAX=MGxx(iCol)                                                
         JMAX=JGyy(iRow)                                                
         LMAX=NLxx(iCol)                                                
         I1=II1(iCol)+1                                                 
         I2=II1(iCol)+MGxx(iCol)                                        
         L1=KK1(iCol)+1                                                 
         L2=KK1(iCol)+NLxx(iCol)                                        
         CHIG3(1:IMAX,:,1:JMAX,L1:L2)=CHIG2(I1:I2,:,1:JMAX,1:LMAX)      
         SFG3(1:IMAX,:,1:JMAX,L1:L2)=SFG2(I1:I2,:,1:JMAX,1:LMAX)        
      ELSE                                                              
         !-----------------------------------------------------         
         ! One processor option, just copy one array into another       
         !-----------------------------------------------------         
         DO L=1,NL                                                      
            DO J=1,JG                                                   
               DO IHEM=1,NHEM                                           
                  DO I=1,MG                                             
                     CHIG3(I,IHEM,J,L)=CHIG2(I,IHEM,J,L)                
                     SFG3(I,IHEM,J,L)=SFG2(I,IHEM,J,L)                  
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Transposing the grid space variables so that the levels               
! are complete instead of the longitudes, because complete              
! levels are needed in MGRMLT. Instead the longitude are                
! split across the columns of PEs.                                      
!=====================================================                  
      SUBROUTINE TRANS3B                                                
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Similar to variables in GRIDP but in grid space, and without          
! equivalent variables for CHIG, SFG, FVGT and FUGT. The levels         
! are divided across the columns of PEs; the latitudes are divided      
! across the row of PEs; and the zonal wavenumbers/longitudes are       
! complete.                                                             
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA2/UG2,FUG2,VG2,FVG2,TG2,TNLG2,PLG2                   
     &     ,TRAG2,TRANLG2                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FUG2(MGPP,NHEM,JGy,NLx)                                   
! V_chi, then V (divergent wind)                                        
      REAL :: VG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FVG2(MGPP,NHEM,JGy,NLx)                                   
! Temperature anomaly (T')                                              
      REAL :: TG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: TNLG2(MGPP,NHEM,JGy,NLx)                                  
! ln p* or p*                                                           
      REAL :: PLG2(MGPP,NHEM,JGy)                                       
! Tracer term                                                           
      REAL :: TRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                            
      REAL :: TRANLG2(MGPP,NHEM,JGy,NLx,NTRAC)                          
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=12000                                
      REAL :: HoldVar1(MGx,NHEM,JGy,NLx,3+NTRAC)                        
      REAL :: HoldVar2(MGx,NHEM,JGy)                                    
      INTEGER :: iPe                                                    
      INTEGER :: iCol2                                                  
      INTEGER :: IMAX, JMAX, LMAX                                       
      INTEGER :: I1, I2                                                 
      INTEGER :: L1, L2                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, ITRAC, J, L                                   
!-----------------------------------------------------                  
! Transpose                                                             
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to send them data              
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Store multi-layer data in HoldVar1                     
               !-----------------------------------------------------   
               IMAX=MGxx(iCol2)                                         
               JMAX=JGyy(iRow)                                          
               LMAX=NLxx(iCol)                                          
               I1=II1(iCol2)+1                                          
               I2=II1(iCol2)+IMAX                                       
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,1)=                      
     &              UG2(I1:I2,:,1:JMAX,1:LMAX)                          
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,2)=                      
     &              VG2(I1:I2,:,1:JMAX,1:LMAX)                          
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,3)=                      
     &              TG2(I1:I2,:,1:JMAX,1:LMAX)                          
               DO ITRAC=1,NTRAC                                         
                  HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,3+ITRAC)=             
     &                 TRAG2(I1:I2,:,1:JMAX,1:LMAX,ITRAC)               
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=(3+NTRAC)*MGx*NHEM*JGy*NLx                     
               iTag=TagRef+100*iCol+10*iCol2+iRow                       
               CALL MPI_BSEND(HoldVar1,nBuffSize,ParaReal,iRecv,iTag,   
     &              MPI_COMM_WORLD,ierr)                                
            ENDIF                                                       
         ENDDO                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to receive data                
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Receive message                                        
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=(3+NTRAC)*MGx*NHEM*JGy*NLx                     
               iTag=TagRef+100*iCol2+10*iCol+iRow                       
               CALL MPI_RECV(HoldVar1,nBuffSize,ParaReal,iSend,iTag,    
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write to UG3 etc                                       
               !-----------------------------------------------------   
               IMAX=MGxx(iCol)                                          
               JMAX=JGyy(iRow)                                          
               LMAX=NLxx(iCol2)                                         
               L1=KK1(iCol2)+1                                          
               L2=KK1(iCol2)+LMAX                                       
               UG3(1:IMAX,:,1:JMAX,L1:L2)=                              
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,1)                  
               VG3(1:IMAX,:,1:JMAX,L1:L2)=                              
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,2)                  
               TG3(1:IMAX,:,1:JMAX,L1:L2)=                              
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,3)                  
               DO ITRAC=1,NTRAC                                         
                  TRAG3(1:IMAX,:,1:JMAX,L1:L2,ITRAC)=                   
     &                 HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,3+ITRAC)         
               ENDDO                                                    
            ENDIF                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Data is already contained on this PE                         
         !-----------------------------------------------------         
         IMAX=MGxx(iCol)                                                
         JMAX=JGyy(iRow)                                                
         LMAX=NLxx(iCol)                                                
         I1=II1(iCol)+1                                                 
         I2=II1(iCol)+MGxx(iCol)                                        
         L1=KK1(iCol)+1                                                 
         L2=KK1(iCol)+NLxx(iCol)                                        
         UG3(1:IMAX,:,1:JMAX,L1:L2)=UG2(I1:I2,:,1:JMAX,1:LMAX)          
         VG3(1:IMAX,:,1:JMAX,L1:L2)=VG2(I1:I2,:,1:JMAX,1:LMAX)          
         TG3(1:IMAX,:,1:JMAX,L1:L2)=TG2(I1:I2,:,1:JMAX,1:LMAX)          
         DO ITRAC=1,NTRAC                                               
            TRAG3(1:IMAX,:,1:JMAX,L1:L2,ITRAC)=                         
     &           TRAG2(I1:I2,:,1:JMAX,1:LMAX,ITRAC)                     
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Single layer variables are only stored on first              
         ! column.                                                      
         !-----------------------------------------------------         
         IF (iCol.EQ.1) THEN                                            
            DO iCol2=2,Px                                               
               iPe=PeGrid(iCol2,iRow)                                   
               !-----------------------------------------------------   
               ! Store single layer data in HoldVar2                    
               !-----------------------------------------------------   
               IMAX=MGxx(iCol2)                                         
               JMAX=JGyy(iRow)                                          
               I1=II1(iCol2)+1                                          
               I2=II1(iCol2)+IMAX                                       
               HoldVar2(1:IMAX,:,1:JMAX)=PLG2(I1:I2,:,1:JMAX)           
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=MGx*NHEM*JGy                                   
               iTag=TagRef+1100+10*iCol2+iRow                           
               CALL MPI_BSEND(HoldVar2,nBuffSize,ParaReal,iRecv,iTag,   
     &              MPI_COMM_WORLD,ierr)                                
            ENDDO                                                       
         ENDIF                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         IF (iCol.GT.1) THEN                                            
            !---------------------------------------------------        
            ! Receive message                                           
            !---------------------------------------------------        
            iSend=PeGrid(1,iRow)                                        
            nBuffSize=MGx*NHEM*JGy                                      
            iTag=TagRef+1100+10*iCol+iRow                               
            CALL MPI_RECV(HoldVar2,nBuffSize,ParaReal,iSend,iTag,       
     &           MPI_COMM_WORLD,istat,ierr)                             
            !---------------------------------------------------        
            ! Write to PLG3 etc                                         
            !---------------------------------------------------        
            IMAX=MGxx(iCol)                                             
            JMAX=JGyy(iRow)                                             
            PLG3(1:IMAX,:,1:JMAX)=HoldVar2(1:IMAX,:,1:JMAX)             
         ELSE                                                           
            !-----------------------------------------------------      
            ! Data for PE on column 1 is already contained on this PE   
            !-----------------------------------------------------      
            IMAX=MGxx(iCol)                                             
            JMAX=JGyy(iRow)                                             
            I1=II1(iCol)+1                                              
            I2=II1(iCol)+IMAX                                           
            PLG3(1:IMAX,:,1:JMAX)=PLG2(I1:I2,:,1:JMAX)                  
         ENDIF                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! One processor option, just copy one array into another       
         !-----------------------------------------------------         
         DO L=1,NL                                                      
            DO J=1,JGy                                                  
               DO IHEM=1,NHEM                                           
                  DO I=1,MG                                             
                     UG3(I,IHEM,J,L)=UG2(I,IHEM,J,L)                    
                     VG3(I,IHEM,J,L)=VG2(I,IHEM,J,L)                    
                     TG3(I,IHEM,J,L)=TG2(I,IHEM,J,L)                    
                     DO ITRAC=1,NTRAC                                   
                        TRAG3(I,IHEM,J,L,ITRAC)=TRAG2(I,IHEM,J,L,ITRAC) 
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO J=1,JGy                                                     
            DO IHEM=1,NHEM                                              
               DO I=1,MG                                                
                  PLG3(I,IHEM,J) =PLG2(I,IHEM,J)                        
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Transposing the grid space variables so that the longitudes           
! are complete (so that this dimension and can be converted to          
! zonal wavenumber in using FFT991). Instead the levels are             
! split across the columns of PEs.                                      
!=====================================================                  
      SUBROUTINE TRANS4                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Similar to variables in GRIDP but in grid space, and without          
! equivalent variables for CHIG, SFG, FVGT and FUGT. The levels         
! are divided across the columns of PEs; the latitudes are divided      
! across the row of PEs; and the zonal wavenumbers/longitudes are       
! complete.                                                             
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA2/UG2,FUG2,VG2,FVG2,TG2,TNLG2,PLG2                   
     &     ,TRAG2,TRANLG2                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FUG2(MGPP,NHEM,JGy,NLx)                                   
! V_chi, then V (divergent wind)                                        
      REAL :: VG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FVG2(MGPP,NHEM,JGy,NLx)                                   
! Temperature anomaly (T')                                              
      REAL :: TG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: TNLG2(MGPP,NHEM,JGy,NLx)                                  
! ln p* or p*                                                           
      REAL :: PLG2(MGPP,NHEM,JGy)                                       
! Tracer term                                                           
      REAL :: TRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                            
      REAL :: TRANLG2(MGPP,NHEM,JGy,NLx,NTRAC)                          
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB2/ZG2,DG2,PJG2,PMG2,SPG2,VPG2,EG2,UTG2,VTG2          
     &     ,UTRAG2,VTRAG2                                               
     &     ,CHIG2,SFG2                                                  
! Vorticity (zeta)                                                      
      REAL :: ZG2(MGPP,NHEM,JGy,NLx)                                    
! Divergence (d)                                                        
      REAL :: DG2(MGPP,NHEM,JGy,NLx)                                    
! (1-mu^2) (d(ln p*)/d mu)                                              
      REAL :: PJG2(MGPP,NHEM,JGy)                                       
! (d(ln p*)/d lambda)                                                   
      REAL :: PMG2(MGPP,NHEM,JGy)                                       
      REAL :: SPG2(MGPP,NHEM,JGy)                                       
      REAL :: VPG2(MGPP,NHEM,JGy)                                       
! (U^2 + V^2)                                                           
      REAL :: EG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: UTG2(MGPP,NHEM,JGy,NLx)                                   
      REAL :: VTG2(MGPP,NHEM,JGy,NLx)                                   
      REAL :: UTRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                           
      REAL :: VTRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                           
      REAL :: CHIG2(MGPP,NHEM,JGy,NLx)                                  
      REAL :: SFG2(MGPP,NHEM,JGy,NLx)                                   
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB3/ZG3,DG3,PJG3,PMG3,SPG3,VPG3,EG3,UTG3,VTG3          
     &     ,UTRAG3,VTRAG3                                               
     &     ,CHIG3,SFG3                                                  
! Vorticity (zeta)                                                      
      REAL :: ZG3(MGx,NHEM,JGy,NL)                                      
! Divergence (d)                                                        
      REAL :: DG3(MGx,NHEM,JGy,NL)                                      
! (1-mu^2) (d(ln p*)/d mu)                                              
      REAL :: PJG3(MGx,NHEM,JGy)                                        
! (d(ln p*)/d lambda)                                                   
      REAL :: PMG3(MGx,NHEM,JGy)                                        
      REAL :: SPG3(MGx,NHEM,JGy)                                        
      REAL :: VPG3(MGx,NHEM,JGy)                                        
! (U^2 + V^2)                                                           
      REAL :: EG3(MGx,NHEM,JGy,NL)                                      
      REAL :: UTG3(MGx,NHEM,JGy,NL)                                     
      REAL :: VTG3(MGx,NHEM,JGy,NL)                                     
      REAL :: UTRAG3(MGx,NHEM,JGy,NL,NTRAC)                             
      REAL :: VTRAG3(MGx,NHEM,JGy,NL,NTRAC)                             
      REAL :: CHIG3(MGx,NHEM,JGy,NL)                                    
      REAL :: SFG3(MGx,NHEM,JGy,NL)                                     
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=14000                                
      REAL :: HoldVar1(MGx,NHEM,JGy,NLx,6+3*NTRAC)                      
      REAL :: HoldVar2(MGx,NHEM,JGy,2)                                  
      INTEGER :: iPe                                                    
      INTEGER :: iCol2                                                  
      INTEGER :: IMAX, JMAX, LMAX                                       
      INTEGER :: I1, I2                                                 
      INTEGER :: L1, L2                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, ITRAC, J, L                                   
!-----------------------------------------------------                  
! Transpose                                                             
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to send them data              
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Store multi-layer data in HoldVar1                     
               !-----------------------------------------------------   
               IMAX=MGxx(iCol)                                          
               JMAX=JGyy(iRow)                                          
               LMAX=NLxx(iCol2)                                         
               L1=KK1(iCol2)+1                                          
               L2=KK1(iCol2)+NLxx(iCol2)                                
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,1)=                      
     &              EG3(1:IMAX,:,1:JMAX,L1:L2)                          
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,2)=                      
     &              TNLG3(1:IMAX,:,1:JMAX,L1:L2)                        
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,3)=                      
     &              FUG3(1:IMAX,:,1:JMAX,L1:L2)                         
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,4)=                      
     &              FVG3(1:IMAX,:,1:JMAX,L1:L2)                         
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,5)=                      
     &              UTG3(1:IMAX,:,1:JMAX,L1:L2)                         
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,6)=                      
     &              VTG3(1:IMAX,:,1:JMAX,L1:L2)                         
               DO ITRAC=1,NTRAC                                         
                  HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,4+3*ITRAC)=           
     &                 TRANLG3(1:IMAX,:,1:JMAX,L1:L2,ITRAC)             
                  HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,5+3*ITRAC)=           
     &                 UTRAG3(1:IMAX,:,1:JMAX,L1:L2,ITRAC)              
                  HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,6+3*ITRAC)=           
     &                 VTRAG3(1:IMAX,:,1:JMAX,L1:L2,ITRAC)              
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=(6+3*NTRAC)*MGx*NHEM*JGy*NLx                   
               iTag=TagRef+100*iCol+10*iCol2+iRow                       
               CALL MPI_BSEND(HoldVar1,nBuffSize,ParaReal,iRecv,iTag,   
     &              MPI_COMM_WORLD,ierr)                                
            ENDIF                                                       
         ENDDO                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to receive data                
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Receive message                                        
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=(6+3*NTRAC)*MGx*NHEM*JGy*NLx                   
               iTag=TagRef+100*iCol2+10*iCol+iRow                       
               CALL MPI_RECV(HoldVar1,nBuffSize,ParaReal,iSend,iTag,    
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write to EG2 etc                                       
               !-----------------------------------------------------   
               IMAX=MGxx(iCol2)                                         
               JMAX=JGyy(iRow)                                          
               LMAX=NLxx(iCol)                                          
               I1=II1(iCol2)+1                                          
               I2=II1(iCol2)+MGxx(iCol2)                                
               EG2(I1:I2,:,1:JMAX,1:LMAX)=                              
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,1)                  
               TNLG2(I1:I2,:,1:JMAX,1:LMAX)=                            
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,2)                  
               FUG2(I1:I2,:,1:JMAX,1:LMAX)=                             
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,3)                  
               FVG2(I1:I2,:,1:JMAX,1:LMAX)=                             
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,4)                  
               UTG2(I1:I2,:,1:JMAX,1:LMAX)=                             
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,5)                  
               VTG2(I1:I2,:,1:JMAX,1:LMAX)=                             
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,6)                  
               DO ITRAC=1,NTRAC                                         
                  TRANLG2(I1:I2,:,1:JMAX,1:LMAX,ITRAC)=                 
     &                 HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,4+3*ITRAC)       
                  UTRAG2(I1:I2,:,1:JMAX,1:LMAX,ITRAC)=                  
     &                 HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,5+3*ITRAC)       
                  VTRAG2(I1:I2,:,1:JMAX,1:LMAX,ITRAC)=                  
     &                 HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,6+3*ITRAC)       
               ENDDO                                                    
            ENDIF                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Data is already contained on this PE                         
         !-----------------------------------------------------         
         IMAX=MGxx(iCol)                                                
         JMAX=JGyy(iRow)                                                
         LMAX=NLxx(iCol)                                                
         I1=II1(iCol)+1                                                 
         I2=II1(iCol)+MGxx(iCol)                                        
         L1=KK1(iCol)+1                                                 
         L2=KK1(iCol)+NLxx(iCol)                                        
         EG2(I1:I2,:,1:JMAX,1:LMAX)=EG3(1:IMAX,:,1:JMAX,L1:L2)          
         TNLG2(I1:I2,:,1:JMAX,1:LMAX)=TNLG3(1:IMAX,:,1:JMAX,L1:L2)      
         FUG2(I1:I2,:,1:JMAX,1:LMAX)=FUG3(1:IMAX,:,1:JMAX,L1:L2)        
         FVG2(I1:I2,:,1:JMAX,1:LMAX)=FVG3(1:IMAX,:,1:JMAX,L1:L2)        
         UTG2(I1:I2,:,1:JMAX,1:LMAX)=UTG3(1:IMAX,:,1:JMAX,L1:L2)        
         VTG2(I1:I2,:,1:JMAX,1:LMAX)=VTG3(1:IMAX,:,1:JMAX,L1:L2)        
         DO ITRAC=1,NTRAC                                               
            TRANLG2(I1:I2,:,1:JMAX,1:LMAX,ITRAC)=                       
     &           TRANLG3(1:IMAX,:,1:JMAX,L1:L2,ITRAC)                   
            UTRAG2(I1:I2,:,1:JMAX,1:LMAX,ITRAC)=                        
     &           UTRAG3(1:IMAX,:,1:JMAX,L1:L2,ITRAC)                    
            VTRAG2(I1:I2,:,1:JMAX,1:LMAX,ITRAC)=                        
     &           VTRAG3(1:IMAX,:,1:JMAX,L1:L2,ITRAC)                    
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Store single-layer variables will only be stored on          
         ! first column after this transpose.                           
         !-----------------------------------------------------         
         IF (iCol.GT.1) THEN                                            
            !----------------------------------------------------       
            ! Store single-layer variables in HoldVar2                  
            !----------------------------------------------------       
            IMAX=MGxx(iCol)                                             
            JMAX=JGyy(iRow)                                             
            HoldVar2(1:IMAX,:,1:JMAX,1)=SPG3(1:IMAX,:,1:JMAX)           
            HoldVar2(1:IMAX,:,1:JMAX,2)=VPG3(1:IMAX,:,1:JMAX)           
            !----------------------------------------------------       
            ! Send message to PEs on column 1                           
            !----------------------------------------------------       
            iRecv=PeGrid(1,iRow)                                        
            nBuffSize=2*MGx*NHEM*JGy                                    
            iTag=TagRef+1010+100*iCol+iRow                              
            CALL MPI_BSEND(HoldVar2,nBuffSize,ParaReal,iRecv,iTag,      
     &           MPI_COMM_WORLD,ierr)                                   
         ENDIF                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         IF (iCol.EQ.1) THEN                                            
            !---------------------------------------------------        
            ! Loop through the other PEs on this row                    
            !---------------------------------------------------        
            DO iCol2=2,Px                                               
               iPe=PeGrid(iCol2,iRow)                                   
               !---------------------------------------------------     
               ! Receive message                                        
               !---------------------------------------------------     
               iSend=iPe                                                
               nBuffSize=2*MGx*NHEM*JGy                                 
               iTag=TagRef+1010+100*iCol2+iRow                          
               CALL MPI_RECV(HoldVar2,nBuffSize,ParaReal,iSend,iTag,    
     &              MPI_COMM_WORLD,istat,ierr)                          
               !---------------------------------------------------     
               ! Write to SPG2 etc                                      
               !---------------------------------------------------     
               IMAX=MGxx(iCol2)                                         
               JMAX=JGyy(iRow)                                          
               LMAX=NLxx(iCol)                                          
               I1=II1(iCol2)+1                                          
               I2=II1(iCol2)+MGxx(iCol2)                                
               SPG2(I1:I2,:,1:JMAX)=HoldVar2(1:IMAX,:,1:JMAX,1)         
               VPG2(I1:I2,:,1:JMAX)=HoldVar2(1:IMAX,:,1:JMAX,2)         
            ENDDO                                                       
            !---------------------------------------------------        
            ! Data is already contained on this PE                      
            !---------------------------------------------------        
            IMAX=MGxx(iCol)                                             
            JMAX=JGyy(iRow)                                             
            I1=II1(iCol)+1                                              
            I2=II1(iCol)+MGxx(iCol)                                     
            SPG2(I1:I2,:,1:JMAX)=SPG3(1:IMAX,:,1:JMAX)                  
            VPG2(I1:I2,:,1:JMAX)=VPG3(1:IMAX,:,1:JMAX)                  
         ENDIF                                                          
      ELSE                                                              
         !---------------------------------------------------           
         ! One processor option                                         
         !---------------------------------------------------           
         DO L=1,NL                                                      
            DO J=1,JGy                                                  
               DO IHEM=1,NHEM                                           
                  DO I=1,MG                                             
                     EG2(I,IHEM,J,L)  =EG3(I,IHEM,J,L)                  
                     TNLG2(I,IHEM,J,L)=TNLG3(I,IHEM,J,L)                
                     FUG2(I,IHEM,J,L) =FUG3(I,IHEM,J,L)                 
                     FVG2(I,IHEM,J,L) =FVG3(I,IHEM,J,L)                 
                     UTG2(I,IHEM,J,L) =UTG3(I,IHEM,J,L)                 
                     VTG2(I,IHEM,J,L) =VTG3(I,IHEM,J,L)                 
                     DO ITRAC=1,NTRAC                                   
                        TRANLG2(I,IHEM,J,L,ITRAC)=                      
     &                       TRANLG3(I,IHEM,J,L,ITRAC)                  
                        UTRAG2(I,IHEM,J,L,ITRAC)=                       
     &                       UTRAG3(I,IHEM,J,L,ITRAC)                   
                        VTRAG2(I,IHEM,J,L,ITRAC)=                       
     &                       VTRAG3(I,IHEM,J,L,ITRAC)                   
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO J=1,JGy                                                     
            DO IHEM=1,NHEM                                              
               DO I=1,MG                                                
                  SPG2(I,IHEM,J)   =SPG3(I,IHEM,J)                      
                  VPG2(I,IHEM,J)   =VPG3(I,IHEM,J)                      
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Transposing the grid space variables so that the longitudes           
! are complete (so that this dimension and can be converted to          
! zonal wavenumber in using FFT991). Instead the levels are             
! split across the columns of PEs.                                      
!=====================================================                  
      SUBROUTINE TRANS4A                                                
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Similar to variables in GRIDP but in grid space, and without          
! equivalent variables for CHIG, SFG, FVGT and FUGT. The levels         
! are divided across the columns of PEs; the latitudes are divided      
! across the row of PEs; and the zonal wavenumbers/longitudes are       
! complete.                                                             
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA2/UG2,FUG2,VG2,FVG2,TG2,TNLG2,PLG2                   
     &     ,TRAG2,TRANLG2                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FUG2(MGPP,NHEM,JGy,NLx)                                   
! V_chi, then V (divergent wind)                                        
      REAL :: VG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FVG2(MGPP,NHEM,JGy,NLx)                                   
! Temperature anomaly (T')                                              
      REAL :: TG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: TNLG2(MGPP,NHEM,JGy,NLx)                                  
! ln p* or p*                                                           
      REAL :: PLG2(MGPP,NHEM,JGy)                                       
! Tracer term                                                           
      REAL :: TRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                            
      REAL :: TRANLG2(MGPP,NHEM,JGy,NLx,NTRAC)                          
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=16000                                
      REAL :: HoldVar1(MGx,NHEM,JGy,NLx,NTRAC)                          
      INTEGER :: iPe                                                    
      INTEGER :: iCol2                                                  
      INTEGER :: IMAX, JMAX, LMAX                                       
      INTEGER :: I1, I2                                                 
      INTEGER :: L1, L2                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, ITRAC, J, L                                   
!-----------------------------------------------------                  
! Transpose                                                             
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to send them data              
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Store multi-layer data in HoldVar1                     
               !-----------------------------------------------------   
               IMAX=MGxx(iCol)                                          
               JMAX=JGyy(iRow)                                          
               LMAX=NLxx(iCol2)                                         
               L1=KK1(iCol2)+1                                          
               L2=KK1(iCol2)+NLxx(iCol2)                                
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,:)=                      
     &              TRANLG3(1:IMAX,:,1:JMAX,L1:L2,:)                    
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=NTRAC*MGx*NHEM*JGy*NLx                         
               iTag=TagRef+100*iCol+10*iCol2+iRow                       
               CALL MPI_BSEND(HoldVar1,nBuffSize,ParaReal,iRecv,iTag,   
     &              MPI_COMM_WORLD,ierr)                                
            ENDIF                                                       
         ENDDO                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to receive data                
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Receive message                                        
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=NTRAC*MGx*NHEM*JGy*NLx                         
               iTag=TagRef+100*iCol2+10*iCol+iRow                       
               CALL MPI_RECV(HoldVar1,nBuffSize,ParaReal,iSend,iTag,    
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write to EG2 etc                                       
               !-----------------------------------------------------   
               IMAX=MGxx(iCol2)                                         
               JMAX=JGyy(iRow)                                          
               LMAX=NLxx(iCol)                                          
               I1=II1(iCol2)+1                                          
               I2=II1(iCol2)+MGxx(iCol2)                                
               TRANLG2(I1:I2,:,1:JMAX,1:LMAX,:)=                        
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,:)                  
            ENDIF                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Data is already contained on this PE                         
         !-----------------------------------------------------         
         IMAX=MGxx(iCol)                                                
         JMAX=JGyy(iRow)                                                
         LMAX=NLxx(iCol)                                                
         I1=II1(iCol)+1                                                 
         I2=II1(iCol)+MGxx(iCol)                                        
         L1=KK1(iCol)+1                                                 
         L2=KK1(iCol)+NLxx(iCol)                                        
         TRANLG2(I1:I2,:,1:JMAX,1:LMAX,:)=                              
     &        TRANLG3(1:IMAX,:,1:JMAX,L1:L2,:)                          
      ELSE                                                              
         !---------------------------------------------------           
         ! One processor option                                         
         !---------------------------------------------------           
         DO L=1,NL                                                      
            DO J=1,JGy                                                  
               DO IHEM=1,NHEM                                           
                  DO I=1,MG                                             
                     DO ITRAC=1,NTRAC                                   
                        TRANLG2(I,IHEM,J,L,ITRAC)=                      
     &                       TRANLG3(I,IHEM,J,L,ITRAC)                  
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Transposing the grid space variables so that the longitudes           
! are complete (so that this dimension and can be converted to          
! zonal wavenumber in using FFT991). Instead the levels are             
! split across the columns of PEs.                                      
!=====================================================                  
      SUBROUTINE TRANS4B                                                
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Similar to variables in GRIDP but in grid space, and without          
! equivalent variables for CHIG, SFG, FVGT and FUGT. The levels         
! are divided across the columns of PEs; the latitudes are divided      
! across the row of PEs; and the zonal wavenumbers/longitudes are       
! complete.                                                             
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA2/UG2,FUG2,VG2,FVG2,TG2,TNLG2,PLG2                   
     &     ,TRAG2,TRANLG2                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FUG2(MGPP,NHEM,JGy,NLx)                                   
! V_chi, then V (divergent wind)                                        
      REAL :: VG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FVG2(MGPP,NHEM,JGy,NLx)                                   
! Temperature anomaly (T')                                              
      REAL :: TG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: TNLG2(MGPP,NHEM,JGy,NLx)                                  
! ln p* or p*                                                           
      REAL :: PLG2(MGPP,NHEM,JGy)                                       
! Tracer term                                                           
      REAL :: TRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                            
      REAL :: TRANLG2(MGPP,NHEM,JGy,NLx,NTRAC)                          
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=17000                                
      REAL :: HoldVar1(MGx,NHEM,JGy,NLx,3+NTRAC)                        
      INTEGER :: iPe                                                    
      INTEGER :: iCol2                                                  
      INTEGER :: IMAX, JMAX, LMAX                                       
      INTEGER :: I1, I2                                                 
      INTEGER :: L1, L2                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, ITRAC, J, L                                   
!-----------------------------------------------------                  
! Transpose                                                             
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to send them data              
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Store multi-layer data in HoldVar1                     
               !-----------------------------------------------------   
               IMAX=MGxx(iCol)                                          
               JMAX=JGyy(iRow)                                          
               LMAX=NLxx(iCol2)                                         
               L1=KK1(iCol2)+1                                          
               L2=KK1(iCol2)+NLxx(iCol2)                                
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,1)=                      
     &              TNLG3(1:IMAX,:,1:JMAX,L1:L2)                        
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,2)=                      
     &              FUG3(1:IMAX,:,1:JMAX,L1:L2)                         
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,3)=                      
     &              FVG3(1:IMAX,:,1:JMAX,L1:L2)                         
               HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,4:3+NTRAC)=              
     &              TRANLG3(1:IMAX,:,1:JMAX,L1:L2,:)                    
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=(3+NTRAC)*MGx*NHEM*JGy*NLx                     
               iTag=TagRef+100*iCol+10*iCol2+iRow                       
               CALL MPI_BSEND(HoldVar1,nBuffSize,ParaReal,iRecv,iTag,   
     &              MPI_COMM_WORLD,ierr)                                
            ENDIF                                                       
         ENDDO                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to receive data                
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Receive message                                        
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=(6+3*NTRAC)*MGx*NHEM*JGy*NLx                   
               iTag=TagRef+100*iCol2+10*iCol+iRow                       
               CALL MPI_RECV(HoldVar1,nBuffSize,ParaReal,iSend,iTag,    
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write to EG2 etc                                       
               !-----------------------------------------------------   
               IMAX=MGxx(iCol2)                                         
               JMAX=JGyy(iRow)                                          
               LMAX=NLxx(iCol)                                          
               I1=II1(iCol2)+1                                          
               I2=II1(iCol2)+MGxx(iCol2)                                
               TNLG2(I1:I2,:,1:JMAX,1:LMAX)=                            
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,1)                  
               FUG2(I1:I2,:,1:JMAX,1:LMAX)=                             
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,2)                  
               FVG2(I1:I2,:,1:JMAX,1:LMAX)=                             
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,3)                  
               TRANLG2(I1:I2,:,1:JMAX,1:LMAX,:)=                        
     &              HoldVar1(1:IMAX,:,1:JMAX,1:LMAX,4:3+NTRAC)          
            ENDIF                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Data is already contained on this PE                         
         !-----------------------------------------------------         
         IMAX=MGxx(iCol)                                                
         JMAX=JGyy(iRow)                                                
         LMAX=NLxx(iCol)                                                
         I1=II1(iCol)+1                                                 
         I2=II1(iCol)+MGxx(iCol)                                        
         L1=KK1(iCol)+1                                                 
         L2=KK1(iCol)+NLxx(iCol)                                        
         TNLG2(I1:I2,:,1:JMAX,1:LMAX)=TNLG3(1:IMAX,:,1:JMAX,L1:L2)      
         FUG2(I1:I2,:,1:JMAX,1:LMAX)=FUG3(1:IMAX,:,1:JMAX,L1:L2)        
         FVG2(I1:I2,:,1:JMAX,1:LMAX)=FVG3(1:IMAX,:,1:JMAX,L1:L2)        
         TRANLG2(I1:I2,:,1:JMAX,1:LMAX,:)=                              
     &        TRANLG3(1:IMAX,:,1:JMAX,L1:L2,:)                          
      ELSE                                                              
         !---------------------------------------------------           
         ! One processor option                                         
         !---------------------------------------------------           
         DO L=1,NL                                                      
            DO J=1,JGy                                                  
               DO IHEM=1,NHEM                                           
                  DO I=1,MG                                             
                     TNLG2(I,IHEM,J,L)=TNLG3(I,IHEM,J,L)                
                     FUG2(I,IHEM,J,L) =FUG3(I,IHEM,J,L)                 
                     FVG2(I,IHEM,J,L) =FVG3(I,IHEM,J,L)                 
                     DO ITRAC=1,NTRAC                                   
                        TRANLG2(I,IHEM,J,L,ITRAC)=                      
     &                       TRANLG3(I,IHEM,J,L,ITRAC)                  
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Transposing the half transforms so that the latitudes are             
! complete (so that this dimension is ready to be transformed           
! to meriodonal wavenumbers in the source terms for the spectral        
! coefficients (e.g. ZT) in both LTD and HANALV). Instead the           
! zonal wavenumbers are split across the rows of PEs.                   
!=====================================================                  
      SUBROUTINE TRANS5(SPG,VPG,EG,TNLG,TRANLG,FUG,FVG,                 
     &     UTG,UTRAG,VTG,VTRAG)                                         
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Similar to variables in GRIDP but in grid space, and without          
! equivalent variables for CHIG, SFG, FVGT and FUGT. The levels         
! are divided across the columns of PEs; the latitudes are divided      
! across the row of PEs; and the zonal wavenumbers/longitudes are       
! complete.                                                             
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA2/UG2,FUG2,VG2,FVG2,TG2,TNLG2,PLG2                   
     &     ,TRAG2,TRANLG2                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FUG2(MGPP,NHEM,JGy,NLx)                                   
! V_chi, then V (divergent wind)                                        
      REAL :: VG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FVG2(MGPP,NHEM,JGy,NLx)                                   
! Temperature anomaly (T')                                              
      REAL :: TG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: TNLG2(MGPP,NHEM,JGy,NLx)                                  
! ln p* or p*                                                           
      REAL :: PLG2(MGPP,NHEM,JGy)                                       
! Tracer term                                                           
      REAL :: TRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                            
      REAL :: TRANLG2(MGPP,NHEM,JGy,NLx,NTRAC)                          
!-----------------------------------------------------                  
! Half transforms needed in MGRMLT loop                                 
!-----------------------------------------------------                  
      COMMON/GRIDPB2/ZG2,DG2,PJG2,PMG2,SPG2,VPG2,EG2,UTG2,VTG2          
     &     ,UTRAG2,VTRAG2                                               
     &     ,CHIG2,SFG2                                                  
! Vorticity (zeta)                                                      
      REAL :: ZG2(MGPP,NHEM,JGy,NLx)                                    
! Divergence (d)                                                        
      REAL :: DG2(MGPP,NHEM,JGy,NLx)                                    
! (1-mu^2) (d(ln p*)/d mu)                                              
      REAL :: PJG2(MGPP,NHEM,JGy)                                       
! (d(ln p*)/d lambda)                                                   
      REAL :: PMG2(MGPP,NHEM,JGy)                                       
      REAL :: SPG2(MGPP,NHEM,JGy)                                       
      REAL :: VPG2(MGPP,NHEM,JGy)                                       
! (U^2 + V^2)                                                           
      REAL :: EG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: UTG2(MGPP,NHEM,JGy,NLx)                                   
      REAL :: VTG2(MGPP,NHEM,JGy,NLx)                                   
      REAL :: UTRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                           
      REAL :: VTRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                           
      REAL :: CHIG2(MGPP,NHEM,JGy,NLx)                                  
      REAL :: SFG2(MGPP,NHEM,JGy,NLx)                                   
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variables - these are the variables in common block            
! GRIDP but defined as REALs, instead of COMPLEXs.                      
!-----------------------------------------------------                  
      REAL :: SPG(4*MMTy,NHEM,JG)                                       
      REAL :: VPG(4*MMTy,NHEM,JG)                                       
      REAL :: EG(4*MMTy,NHEM,JG,NLx)                                    
      REAL :: TNLG(4*MMTy,NHEM,JG,NLx)                                  
      REAL :: FUG(4*MMTy,NHEM,JG,NLx)                                   
      REAL :: FVG(4*MMTy,NHEM,JG,NLx)                                   
      REAL :: UTG(4*MMTy,NHEM,JG,NLx)                                   
      REAL :: VTG(4*MMTy,NHEM,JG,NLx)                                   
      REAL :: FVGT(4*MMTy,NHEM,JG,NLx)                                  
      REAL :: FUGT(4*MMTy,NHEM,JG,NLx)                                  
      REAL :: TRANLG(4*MMTy,NHEM,JG,NLx,NTRAC)                          
      REAL :: UTRAG(4*MMTy,NHEM,JG,NLx,NTRAC)                           
      REAL :: VTRAG(4*MMTy,NHEM,JG,NLx,NTRAC)                           
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=18000                                
      REAL :: HoldVar1(4*MMTy,NHEM,JGy,NLx,6+3*NTRAC)                   
      REAL :: HoldVar2(4*MMTy,NHEM,JGy,2)                               
      INTEGER :: iPe                                                    
      INTEGER :: iRow2                                                  
      INTEGER :: IMMAX, JMAX, LMAX                                      
      INTEGER :: IM1, IM2, IM3, IM4                                     
      INTEGER :: J1, J2                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, ITRAC, J, K, L                            
!-----------------------------------------------------                  
! Transpose                                                             
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         DO iRow2=1,Py                                                  
            iPe=PeGrid(iCol,iRow2)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Store multi-layer variables in HoldVar1                
               !-----------------------------------------------------   
               IMMAX=2*MMTyy(iRow2)                                     
               IM1=2*MMT1(iRow2)                                        
               IM3=4*MMT-2*(MMT1(iRow2)+MMTyy(iRow2))                   
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           HoldVar1(I,IHEM,J,L,1)=EG2(IM1+I,IHEM,J,L)   
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           HoldVar1(IMMAX+I,IHEM,J,L,1)=                
     &                          EG2(IM3+I,IHEM,J,L)                     
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           HoldVar1(I,IHEM,J,L,2)=TNLG2(IM1+I,IHEM,J,L) 
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           HoldVar1(IMMAX+I,IHEM,J,L,2)=                
     &                          TNLG2(IM3+I,IHEM,J,L)                   
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           HoldVar1(I,IHEM,J,L,3)=FUG2(IM1+I,IHEM,J,L)  
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           HoldVar1(IMMAX+I,IHEM,J,L,3)=                
     &                          FUG2(IM3+I,IHEM,J,L)                    
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           HoldVar1(I,IHEM,J,L,4)=FVG2(IM1+I,IHEM,J,L)  
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           HoldVar1(IMMAX+I,IHEM,J,L,4)=                
     &                          FVG2(IM3+I,IHEM,J,L)                    
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           HoldVar1(I,IHEM,J,L,5)=UTG2(IM1+I,IHEM,J,L)  
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           HoldVar1(IMMAX+I,IHEM,J,L,5)=                
     &                          UTG2(IM3+I,IHEM,J,L)                    
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           HoldVar1(I,IHEM,J,L,6)=VTG2(IM1+I,IHEM,J,L)  
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           HoldVar1(IMMAX+I,IHEM,J,L,6)=                
     &                          VTG2(IM3+I,IHEM,J,L)                    
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO ITRAC=1,NTRAC                                         
                  DO L=1,NLxx(iCol)                                     
                     DO J=1,JGyy(iRow)                                  
                        DO IHEM=1,NHEM                                  
                           DO I=1,IMMAX                                 
                              HoldVar1(I,IHEM,J,L,6+ITRAC)=             
     &                             TRANLG2(IM1+I,IHEM,J,L,ITRAC)        
                           ENDDO                                        
                           DO I=1,IMMAX                                 
                              HoldVar1(IMMAX+I,IHEM,J,L,6+ITRAC)=       
     &                             TRANLG2(IM3+I,IHEM,J,L,ITRAC)        
                           ENDDO                                        
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO ITRAC=1,NTRAC                                         
                  DO L=1,NLxx(iCol)                                     
                     DO J=1,JGyy(iRow)                                  
                        DO IHEM=1,NHEM                                  
                           DO I=1,IMMAX                                 
                              HoldVar1(I,IHEM,J,L,6+NTRAC+ITRAC)=       
     &                             UTRAG2(IM1+I,IHEM,J,L,ITRAC)         
                           ENDDO                                        
                           DO I=1,IMMAX                                 
                              HoldVar1(IMMAX+I,IHEM,J,L,6+NTRAC+ITRAC)= 
     &                             UTRAG2(IM3+I,IHEM,J,L,ITRAC)         
                           ENDDO                                        
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO ITRAC=1,NTRAC                                         
                  DO L=1,NLxx(iCol)                                     
                     DO J=1,JGyy(iRow)                                  
                        DO IHEM=1,NHEM                                  
                           DO I=1,IMMAX                                 
                              HoldVar1(I,IHEM,J,L,6+2*NTRAC+ITRAC)=     
     &                             VTRAG2(IM1+I,IHEM,J,L,ITRAC)         
                           ENDDO                                        
                           DO I=1,IMMAX                                 
                              HoldVar1(IMMAX+I,IHEM,J,L,6+2*NTRAC+ITRAC)
     &                          =VTRAG2(IM3+I,IHEM,J,L,ITRAC)           
                           ENDDO                                        
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=(6+3*NTRAC)*4*MMTy*NHEM*JGy*NLx                
               iTag=TagRef+100*iRow+10*iRow2+iCol                       
               CALL MPI_BSEND(HoldVar1,nBuffSize,ParaReal,iRecv,iTag,   
     &              MPI_COMM_WORLD,ierr)                                
            ENDIF                                                       
         ENDDO                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         !-----------------------------------------------------         
         ! Loop over PEs on this column                                 
         !-----------------------------------------------------         
         DO iRow2=1,Py                                                  
            iPe=PeGrid(iCol,iRow2)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Receive message                                        
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=(6+3*NTRAC)*4*MMTy*NHEM*JGy*NLx                
               iTag=TagRef+100*iRow2+10*iRow+iCol                       
               CALL MPI_RECV(HoldVar1,nBuffSize,ParaReal,iSend,iTag,    
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write to EG etc                                        
               !-----------------------------------------------------   
               IMMAX=4*MMTyy(iRow)                                      
               JMAX=JGyy(iRow2)                                         
               LMAX=NLxx(iCol)                                          
               J1=JJ1(iRow2)+1                                          
               J2=JJ1(iRow2)+JGyy(iRow2)                                
               EG(1:IMMAX,:,J1:J2,1:LMAX)=                              
     &              HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,1)                 
               TNLG(1:IMMAX,:,J1:J2,1:LMAX)=                            
     &              HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,2)                 
               FUG(1:IMMAX,:,J1:J2,1:LMAX)=                             
     &              HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,3)                 
               FVG(1:IMMAX,:,J1:J2,1:LMAX)=                             
     &              HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,4)                 
               UTG(1:IMMAX,:,J1:J2,1:LMAX)=                             
     &              HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,5)                 
               VTG(1:IMMAX,:,J1:J2,1:LMAX)=                             
     &              HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,6)                 
               TRANLG(1:IMMAX,:,J1:J2,1:LMAX,:)=                        
     &              HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,7:6+NTRAC)         
               UTRAG(1:IMMAX,:,J1:J2,1:LMAX,:)=                         
     &              HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,7+NTRAC:6+2*NTRAC) 
               VTRAG(1:IMMAX,:,J1:J2,1:LMAX,:)=                         
     &            HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,7+2*NTRAC:6+3*NTRAC) 
            ENDIF                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Data is already contained on this PE                         
         !-----------------------------------------------------         
         IMMAX=2*MMTyy(iRow)                                            
         IM1=2*MMT1(iRow)                                               
         IM3=4*MMT-2*(MMT1(iRow)+MMTyy(iRow))                           
         J1=JJ1(iRow)                                                   
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     EG(I,IHEM,J1+J,L)=EG2(IM1+I,IHEM,J,L)              
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     EG(IMMAX+I,IHEM,J1+J,L)=EG2(IM3+I,IHEM,J,L)        
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     TNLG(I,IHEM,J1+J,L)=TNLG2(IM1+I,IHEM,J,L)          
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     TNLG(IMMAX+I,IHEM,J1+J,L)=TNLG2(IM3+I,IHEM,J,L)    
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     FUG(I,IHEM,J1+J,L)=FUG2(IM1+I,IHEM,J,L)            
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     FUG(IMMAX+I,IHEM,J1+J,L)=FUG2(IM3+I,IHEM,J,L)      
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     FVG(I,IHEM,J1+J,L)=FVG2(IM1+I,IHEM,J,L)            
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     FVG(IMMAX+I,IHEM,J1+J,L)=FVG2(IM3+I,IHEM,J,L)      
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     UTG(I,IHEM,J1+J,L)=UTG2(IM1+I,IHEM,J,L)            
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     UTG(IMMAX+I,IHEM,J1+J,L)=UTG2(IM3+I,IHEM,J,L)      
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     VTG(I,IHEM,J1+J,L)=VTG2(IM1+I,IHEM,J,L)            
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     VTG(IMMAX+I,IHEM,J1+J,L)=VTG2(IM3+I,IHEM,J,L)      
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO ITRAC=1,NTRAC                                               
            DO L=1,NLxx(iCol)                                           
               DO J=1,JGyy(iRow)                                        
                  DO IHEM=1,NHEM                                        
                     DO I=1,IMMAX                                       
                        TRANLG(I,IHEM,J1+J,L,ITRAC)=                    
     &                       TRANLG2(IM1+I,IHEM,J,L,ITRAC)              
                     ENDDO                                              
                     DO I=1,IMMAX                                       
                        TRANLG(IMMAX+I,IHEM,J1+J,L,ITRAC)=              
     &                       TRANLG2(IM3+I,IHEM,J,L,ITRAC)              
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO ITRAC=1,NTRAC                                               
            DO L=1,NLxx(iCol)                                           
               DO J=1,JGyy(iRow)                                        
                  DO IHEM=1,NHEM                                        
                     DO I=1,IMMAX                                       
                        UTRAG(I,IHEM,J1+J,L,ITRAC)=                     
     &                       UTRAG2(IM1+I,IHEM,J,L,ITRAC)               
                     ENDDO                                              
                     DO I=1,IMMAX                                       
                        UTRAG(IMMAX+I,IHEM,J1+J,L,ITRAC)=               
     &                       UTRAG2(IM3+I,IHEM,J,L,ITRAC)               
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO ITRAC=1,NTRAC                                               
            DO L=1,NLxx(iCol)                                           
               DO J=1,JGyy(iRow)                                        
                  DO IHEM=1,NHEM                                        
                     DO I=1,IMMAX                                       
                        VTRAG(I,IHEM,J1+J,L,ITRAC)=                     
     &                       VTRAG2(IM1+I,IHEM,J,L,ITRAC)               
                     ENDDO                                              
                     DO I=1,IMMAX                                       
                        VTRAG(IMMAX+I,IHEM,J1+J,L,ITRAC)=               
     &                       VTRAG2(IM3+I,IHEM,J,L,ITRAC)               
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Single layer variables are only stored on first              
         ! column.                                                      
         !-----------------------------------------------------         
         IF (iCol.EQ.1) THEN                                            
            DO iRow2=1,Py                                               
               iPe=PeGrid(1,iRow2)                                      
               IF (iPe.NE.MyPe) THEN                                    
                  IMMAX=2*MMTyy(iRow2)                                  
                  IM1=2*MMT1(iRow2)                                     
                  IM3=4*MMT-2*(MMT1(iRow2)+MMTyy(iRow2))                
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           HoldVar2(I,IHEM,J,1)=SPG2(IM1+I,IHEM,J)      
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           HoldVar2(IMMAX+I,IHEM,J,1)=SPG2(IM3+I,IHEM,J)
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           HoldVar2(I,IHEM,J,2)=VPG2(IM1+I,IHEM,J)      
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           HoldVar2(IMMAX+I,IHEM,J,2)=VPG2(IM3+I,IHEM,J)
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
                  !---------------------------------------------------- 
                  ! Send message                                        
                  !---------------------------------------------------- 
                  iRecv=iPe                                             
                  nBuffSize=8*MMTy*NHEM*JGy                             
                  iTag=TagRef+1001+100*iRow+10*iRow2                    
                  CALL MPI_BSEND(HoldVar2,nBuffSize,ParaReal,iRecv      
     &                 ,iTag,MPI_COMM_WORLD,ierr)                       
               ENDIF                                                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Receive data                                              
            !-----------------------------------------------------      
            DO iRow2=1,Py                                               
               iPe=PeGrid(1,iRow2)                                      
               IF (iPe.NE.MyPe) THEN                                    
                  iSend=iPe                                             
                  nBuffSize=8*MMTy*NHEM*JGy                             
                  iTag=TagRef+1001+100*iRow2+10*iRow                    
                  CALL MPI_RECV(HoldVar2,nBuffSize,ParaReal,iSend,iTag  
     &                 ,MPI_COMM_WORLD,istat,ierr)                      
                  !---------------------------------------------------- 
                  ! Write into arrays                                   
                  !---------------------------------------------------- 
                  IMMAX=4*MMTyy(iRow)                                   
                  JMAX=JGyy(iRow2)                                      
                  J1=JJ1(iRow2)+1                                       
                  J2=JJ1(iRow2)+JGyy(iRow2)                             
                  SPG(1:IMMAX,:,J1:J2)=HoldVar2(1:IMMAX,:,1:JMAX,1)     
                  VPG(1:IMMAX,:,J1:J2)=HoldVar2(1:IMMAX,:,1:JMAX,2)     
               ENDIF                                                    
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Data is already stored on this PE                         
            !-----------------------------------------------------      
            IMMAX=2*MMTyy(iRow)                                         
            IM1=2*MMT1(iRow)                                            
            IM3=4*MMT-2*(MMT1(iRow)+MMTyy(iRow))                        
            J1=JJ1(iRow)                                                
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     SPG(I,IHEM,J1+J)=SPG2(IM1+I,IHEM,J)                
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     SPG(IMMAX+I,IHEM,J1+J)=SPG2(IM3+I,IHEM,J)          
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     VPG(I,IHEM,J1+J)=VPG2(IM1+I,IHEM,J)                
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     VPG(IMMAX+I,IHEM,J1+J)=VPG2(IM3+I,IHEM,J)          
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDIF                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! One processor option                                         
         !-----------------------------------------------------         
         EG(:,:,:,:)=0.0                                                
         TNLG(:,:,:,:)=0.0                                              
         FUG(:,:,:,:)=0.0                                               
         FVG(:,:,:,:)=0.0                                               
         UTG(:,:,:,:)=0.0                                               
         VTG(:,:,:,:)=0.0                                               
         TRANLG(:,:,:,:,:)=0.0                                          
         UTRAG(:,:,:,:,:)=0.0                                           
         VTRAG(:,:,:,:,:)=0.0                                           
         DO L=1,NL                                                      
            DO J=1,JG                                                   
               DO IHEM=1,NHEM                                           
                  DO I=1,4*MMTy                                         
                     EG(I,IHEM,J,L)  =EG2(I,IHEM,J,L)                   
                     TNLG(I,IHEM,J,L)=TNLG2(I,IHEM,J,L)                 
                     FUG(I,IHEM,J,L) =FUG2(I,IHEM,J,L)                  
                     FVG(I,IHEM,J,L) =FVG2(I,IHEM,J,L)                  
                     UTG(I,IHEM,J,L) =UTG2(I,IHEM,J,L)                  
                     VTG(I,IHEM,J,L) =VTG2(I,IHEM,J,L)                  
                     DO ITRAC=1,NTRAC                                   
                        TRANLG(I,IHEM,J,L,ITRAC)=                       
     &                       TRANLG2(I,IHEM,J,L,ITRAC)                  
                        UTRAG(I,IHEM,J,L,ITRAC)=                        
     &                       UTRAG2(I,IHEM,J,L,ITRAC)                   
                        VTRAG(I,IHEM,J,L,ITRAC)=                        
     &                       VTRAG2(I,IHEM,J,L,ITRAC)                   
                     ENDDO                                              
                 ENDDO                                                  
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         SPG(:,:,:)=0.0                                                 
         VPG(:,:,:)=0.0                                                 
         DO J=1,JG                                                      
            DO IHEM=1,NHEM                                              
               DO I=1,4*MMTy                                            
                  SPG(I,IHEM,J)=SPG2(I,IHEM,J)                          
                  VPG(I,IHEM,J)=VPG2(I,IHEM,J)                          
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Transposing the half transforms so that the latitudes are             
! complete (so that this dimension is ready to be transformed           
! to meriodonal wavenumbers in the source terms for the spectral        
! coefficients (e.g. ZT) in both LTD and HANALV). Instead the           
! zonal wavenumbers are split across the rows of PEs.                   
!=====================================================                  
      SUBROUTINE TRANS5A(TRANLG)                                        
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Similar to variables in GRIDP but in grid space, and without          
! equivalent variables for CHIG, SFG, FVGT and FUGT. The levels         
! are divided across the columns of PEs; the latitudes are divided      
! across the row of PEs; and the zonal wavenumbers/longitudes are       
! complete.                                                             
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA2/UG2,FUG2,VG2,FVG2,TG2,TNLG2,PLG2                   
     &     ,TRAG2,TRANLG2                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FUG2(MGPP,NHEM,JGy,NLx)                                   
! V_chi, then V (divergent wind)                                        
      REAL :: VG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FVG2(MGPP,NHEM,JGy,NLx)                                   
! Temperature anomaly (T')                                              
      REAL :: TG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: TNLG2(MGPP,NHEM,JGy,NLx)                                  
! ln p* or p*                                                           
      REAL :: PLG2(MGPP,NHEM,JGy)                                       
! Tracer term                                                           
      REAL :: TRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                            
      REAL :: TRANLG2(MGPP,NHEM,JGy,NLx,NTRAC)                          
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variables - these are the variables in common block            
! GRIDP but defined as REALs, instead of COMPLEXs.                      
!-----------------------------------------------------                  
      REAL :: TRANLG(4*MMTy,NHEM,JG,NLx,NTRAC)                          
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=20000                                
      REAL :: HoldVar1(4*MMTy,NHEM,JGy,NLx,NTRAC)                       
      INTEGER :: iPe                                                    
      INTEGER :: iRow2                                                  
      INTEGER :: IMMAX, JMAX, LMAX                                      
      INTEGER :: IM1, IM2, IM3, IM4                                     
      INTEGER :: J1, J2                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, ITRAC, J, K, L                            
!-----------------------------------------------------                  
! Transpose                                                             
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         DO iRow2=1,Py                                                  
            iPe=PeGrid(iCol,iRow2)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Store multi-layer variables in HoldVar1                
               !-----------------------------------------------------   
               IMMAX=2*MMTyy(iRow2)                                     
               IM1=2*MMT1(iRow2)                                        
               IM3=4*MMT-2*(MMT1(iRow2)+MMTyy(iRow2))                   
               DO ITRAC=1,NTRAC                                         
                  DO L=1,NLxx(iCol)                                     
                     DO J=1,JGyy(iRow)                                  
                        DO IHEM=1,NHEM                                  
                           DO I=1,IMMAX                                 
                              HoldVar1(I,IHEM,J,L,ITRAC)=               
     &                             TRANLG2(IM1+I,IHEM,J,L,ITRAC)        
                           ENDDO                                        
                           DO I=1,IMMAX                                 
                              HoldVar1(IMMAX+I,IHEM,J,L,ITRAC)=         
     &                             TRANLG2(IM3+I,IHEM,J,L,ITRAC)        
                           ENDDO                                        
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=NTRAC*4*MMTy*NHEM*JGy*NLx                      
               iTag=TagRef+100*iRow+10*iRow2+iCol                       
               CALL MPI_BSEND(HoldVar1,nBuffSize,ParaReal,iRecv,iTag,   
     &              MPI_COMM_WORLD,ierr)                                
            ENDIF                                                       
         ENDDO                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         !-----------------------------------------------------         
         ! Loop over PEs on this column                                 
         !-----------------------------------------------------         
         DO iRow2=1,Py                                                  
            iPe=PeGrid(iCol,iRow2)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Receive message                                        
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=NTRAC*4*MMTy*NHEM*JGy*NLx                      
               iTag=TagRef+100*iRow2+10*iRow+iCol                       
               CALL MPI_RECV(HoldVar1,nBuffSize,ParaReal,iSend,iTag,    
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write to TRANLG                                        
               !-----------------------------------------------------   
               IMMAX=4*MMTyy(iRow)                                      
               JMAX=JGyy(iRow2)                                         
               LMAX=NLxx(iCol)                                          
               J1=JJ1(iRow2)+1                                          
               J2=JJ1(iRow2)+JGyy(iRow2)                                
               TRANLG(1:IMMAX,:,J1:J2,1:LMAX,:)=                        
     &              HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,:)                 
            ENDIF                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Data is already contained on this PE                         
         !-----------------------------------------------------         
         IMMAX=2*MMTyy(iRow)                                            
         IM1=2*MMT1(iRow)                                               
         IM3=4*MMT-2*(MMT1(iRow)+MMTyy(iRow))                           
         J1=JJ1(iRow)                                                   
         DO ITRAC=1,NTRAC                                               
            DO L=1,NLxx(iCol)                                           
               DO J=1,JGyy(iRow)                                        
                  DO IHEM=1,NHEM                                        
                     DO I=1,IMMAX                                       
                        TRANLG(I,IHEM,J1+J,L,ITRAC)=                    
     &                       TRANLG2(IM1+I,IHEM,J,L,ITRAC)              
                     ENDDO                                              
                     DO I=1,IMMAX                                       
                        TRANLG(IMMAX+I,IHEM,J1+J,L,ITRAC)=              
     &                       TRANLG2(IM3+I,IHEM,J,L,ITRAC)              
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! One processor option                                         
         !-----------------------------------------------------         
         TRANLG(:,:,:,:,:)=0.0                                          
         DO L=1,NL                                                      
            DO J=1,JG                                                   
               DO IHEM=1,NHEM                                           
                  DO I=1,4*MMTy                                         
                     DO ITRAC=1,NTRAC                                   
                        TRANLG(I,IHEM,J,L,ITRAC)=                       
     &                       TRANLG2(I,IHEM,J,L,ITRAC)                  
                     ENDDO                                              
                 ENDDO                                                  
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Transposing the half transforms so that the latitudes are             
! complete (so that this dimension is ready to be transformed           
! to meriodonal wavenumbers in the source terms for the spectral        
! coefficients (e.g. ZT) in both LTD and HANALV). Instead the           
! zonal wavenumbers are split across the rows of PEs.                   
!=====================================================                  
      SUBROUTINE TRANS5B(TNLG,TRANLG,FUG,FVG)                           
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Similar to variables in GRIDP but in grid space, and without          
! equivalent variables for CHIG, SFG, FVGT and FUGT. The levels         
! are divided across the columns of PEs; the latitudes are divided      
! across the row of PEs; and the zonal wavenumbers/longitudes are       
! complete.                                                             
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA2/UG2,FUG2,VG2,FVG2,TG2,TNLG2,PLG2                   
     &     ,TRAG2,TRANLG2                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FUG2(MGPP,NHEM,JGy,NLx)                                   
! V_chi, then V (divergent wind)                                        
      REAL :: VG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FVG2(MGPP,NHEM,JGy,NLx)                                   
! Temperature anomaly (T')                                              
      REAL :: TG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: TNLG2(MGPP,NHEM,JGy,NLx)                                  
! ln p* or p*                                                           
      REAL :: PLG2(MGPP,NHEM,JGy)                                       
! Tracer term                                                           
      REAL :: TRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                            
      REAL :: TRANLG2(MGPP,NHEM,JGy,NLx,NTRAC)                          
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variables - these are the variables in common block            
! GRIDP but defined as REALs, instead of COMPLEXs.                      
!-----------------------------------------------------                  
      REAL :: TNLG(4*MMTy,NHEM,JG,NLx)                                  
      REAL :: FUG(4*MMTy,NHEM,JG,NLx)                                   
      REAL :: FVG(4*MMTy,NHEM,JG,NLx)                                   
      REAL :: TRANLG(4*MMTy,NHEM,JG,NLx,NTRAC)                          
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=21000                                
      REAL :: HoldVar1(4*MMTy,NHEM,JGy,NLx,3+NTRAC)                     
      INTEGER :: iPe                                                    
      INTEGER :: iRow2                                                  
      INTEGER :: IMMAX, JMAX, LMAX                                      
      INTEGER :: IM1, IM2, IM3, IM4                                     
      INTEGER :: J1, J2                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, ITRAC, J, K, L                            
!-----------------------------------------------------                  
! Transpose                                                             
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         DO iRow2=1,Py                                                  
            iPe=PeGrid(iCol,iRow2)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Store multi-layer variables in HoldVar1                
               !-----------------------------------------------------   
               IMMAX=2*MMTyy(iRow2)                                     
               IM1=2*MMT1(iRow2)                                        
               IM3=4*MMT-2*(MMT1(iRow2)+MMTyy(iRow2))                   
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           HoldVar1(I,IHEM,J,L,1)=TNLG2(IM1+I,IHEM,J,L) 
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           HoldVar1(IMMAX+I,IHEM,J,L,1)=                
     &                          TNLG2(IM3+I,IHEM,J,L)                   
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           HoldVar1(I,IHEM,J,L,2)=FUG2(IM1+I,IHEM,J,L)  
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           HoldVar1(IMMAX+I,IHEM,J,L,2)=                
     &                          FUG2(IM3+I,IHEM,J,L)                    
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO L=1,NLxx(iCol)                                        
                  DO J=1,JGyy(iRow)                                     
                     DO IHEM=1,NHEM                                     
                        DO I=1,IMMAX                                    
                           HoldVar1(I,IHEM,J,L,3)=FVG2(IM1+I,IHEM,J,L)  
                        ENDDO                                           
                        DO I=1,IMMAX                                    
                           HoldVar1(IMMAX+I,IHEM,J,L,3)=                
     &                          FVG2(IM3+I,IHEM,J,L)                    
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               DO ITRAC=1,NTRAC                                         
                  DO L=1,NLxx(iCol)                                     
                     DO J=1,JGyy(iRow)                                  
                        DO IHEM=1,NHEM                                  
                           DO I=1,IMMAX                                 
                              HoldVar1(I,IHEM,J,L,3+ITRAC)=             
     &                             TRANLG2(IM1+I,IHEM,J,L,ITRAC)        
                           ENDDO                                        
                           DO I=1,IMMAX                                 
                              HoldVar1(IMMAX+I,IHEM,J,L,3+ITRAC)=       
     &                             TRANLG2(IM3+I,IHEM,J,L,ITRAC)        
                           ENDDO                                        
                        ENDDO                                           
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=(3+NTRAC)*4*MMTy*NHEM*JGy*NLx                  
               iTag=TagRef+100*iRow+10*iRow2+iCol                       
               CALL MPI_BSEND(HoldVar1,nBuffSize,ParaReal,iRecv,iTag,   
     &              MPI_COMM_WORLD,ierr)                                
            ENDIF                                                       
         ENDDO                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         !-----------------------------------------------------         
         ! Loop over PEs on this column                                 
         !-----------------------------------------------------         
         DO iRow2=1,Py                                                  
            iPe=PeGrid(iCol,iRow2)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Receive message                                        
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=(3+NTRAC)*4*MMTy*NHEM*JGy*NLx                  
               iTag=TagRef+100*iRow2+10*iRow+iCol                       
               CALL MPI_RECV(HoldVar1,nBuffSize,ParaReal,iSend,iTag,    
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write to EG etc                                        
               !-----------------------------------------------------   
               IMMAX=4*MMTyy(iRow)                                      
               JMAX=JGyy(iRow2)                                         
               LMAX=NLxx(iCol)                                          
               J1=JJ1(iRow2)+1                                          
               J2=JJ1(iRow2)+JGyy(iRow2)                                
               TNLG(1:IMMAX,:,J1:J2,1:LMAX)=                            
     &              HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,1)                 
               FUG(1:IMMAX,:,J1:J2,1:LMAX)=                             
     &              HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,2)                 
               FVG(1:IMMAX,:,J1:J2,1:LMAX)=                             
     &              HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,3)                 
               TRANLG(1:IMMAX,:,J1:J2,1:LMAX,:)=                        
     &              HoldVar1(1:IMMAX,:,1:JMAX,1:LMAX,4:3+NTRAC)         
            ENDIF                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Data is already contained on this PE                         
         !-----------------------------------------------------         
         IMMAX=2*MMTyy(iRow)                                            
         IM1=2*MMT1(iRow)                                               
         IM3=4*MMT-2*(MMT1(iRow)+MMTyy(iRow))                           
         J1=JJ1(iRow)                                                   
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     TNLG(I,IHEM,J1+J,L)=TNLG2(IM1+I,IHEM,J,L)          
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     TNLG(IMMAX+I,IHEM,J1+J,L)=TNLG2(IM3+I,IHEM,J,L)    
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     FUG(I,IHEM,J1+J,L)=FUG2(IM1+I,IHEM,J,L)            
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     FUG(IMMAX+I,IHEM,J1+J,L)=FUG2(IM3+I,IHEM,J,L)      
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NLxx(iCol)                                              
            DO J=1,JGyy(iRow)                                           
               DO IHEM=1,NHEM                                           
                  DO I=1,IMMAX                                          
                     FVG(I,IHEM,J1+J,L)=FVG2(IM1+I,IHEM,J,L)            
                  ENDDO                                                 
                  DO I=1,IMMAX                                          
                     FVG(IMMAX+I,IHEM,J1+J,L)=FVG2(IM3+I,IHEM,J,L)      
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO ITRAC=1,NTRAC                                               
            DO L=1,NLxx(iCol)                                           
               DO J=1,JGyy(iRow)                                        
                  DO IHEM=1,NHEM                                        
                     DO I=1,IMMAX                                       
                        TRANLG(I,IHEM,J1+J,L,ITRAC)=                    
     &                       TRANLG2(IM1+I,IHEM,J,L,ITRAC)              
                     ENDDO                                              
                     DO I=1,IMMAX                                       
                        TRANLG(IMMAX+I,IHEM,J1+J,L,ITRAC)=              
     &                       TRANLG2(IM3+I,IHEM,J,L,ITRAC)              
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! One processor option                                         
         !-----------------------------------------------------         
         TNLG(:,:,:,:)=0.0                                              
         FUG(:,:,:,:)=0.0                                               
         FVG(:,:,:,:)=0.0                                               
         TRANLG(:,:,:,:,:)=0.0                                          
         DO L=1,NL                                                      
            DO J=1,JG                                                   
               DO IHEM=1,NHEM                                           
                  DO I=1,4*MMTy                                         
                     TNLG(I,IHEM,J,L)=TNLG2(I,IHEM,J,L)                 
                     FUG(I,IHEM,J,L) =FUG2(I,IHEM,J,L)                  
                     FVG(I,IHEM,J,L) =FVG2(I,IHEM,J,L)                  
                     DO ITRAC=1,NTRAC                                   
                        TRANLG(I,IHEM,J,L,ITRAC)=                       
     &                       TRANLG2(I,IHEM,J,L,ITRAC)                  
                     ENDDO                                              
                 ENDDO                                                  
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Transposing the source terms for the spectral coefficients so         
! that the levels are complete. Instead the meriodonal wavenumbers      
! are split across the columns of PEs.                                  
!=====================================================                  
      SUBROUTINE TRANS6                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Like the variables in SPECTR but the meridional wavenumbers           
! are complete instead of the levels.                                   
!-----------------------------------------------------                  
      COMMON/SPECTR2/Z2,D2,T2,SP2,SPA2,VP2,DTE2,TT2,DT2,ZT2             
     &     ,TRA2,TRAT2                                                  
! Vorticity coefficient                                                 
      COMPLEX :: Z2(NNT,MMTy,NHEM,NLx)                                  
! Divergence coefficient                                                
      COMPLEX :: D2(NNT,MMTy,NHEM,NLx)                                  
! Temperature coefficient                                               
      COMPLEX :: T2(NNT,MMTy,NHEM,NLx)                                  
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP2(NNT,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA2(NNT,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP2(NNT,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE2(NNT,MMTy,NHEM,NLx)                                
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT2(NNT,MMTy,NHEM,NLx)                                 
! Tracer coefficient                                                    
      COMPLEX :: TRA2(NNT,MMTy,NHEM,NLx,NTRAC)                          
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT2(NNT,MMTy,NHEM,NLx,NTRAC)                         
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=22000                                
      COMPLEX :: HoldVar1(NNTx,MMTy,NHEM,NLx,4+NTRAC)                   
      COMPLEX :: HoldVar2(NNTx,MMTy,NHEM,2)                             
      INTEGER :: iCol2, iPe                                             
      INTEGER :: IMMAX, INMAX, LMAX                                     
      INTEGER :: IN1, IN2                                               
      INTEGER :: L1, L2                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, IN, ITRAC, L                              
!-----------------------------------------------------                  
! Transpose                                                             
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to send them data              
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Store multi-layer variables in HoldVar1                
               !-----------------------------------------------------   
               INMAX=NNTxx(iCol2)                                       
               IMMAX=MMTyy(iRow)                                        
               LMAX=NLxx(iCol)                                          
               IN1=NNT1(iCol2)+1                                        
               IN2=NNT1(iCol2)+NNTxx(iCol2)                             
               HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,1)=                    
     &              ZT2(IN1:IN2,1:IMMAX,:,1:LMAX)                       
               HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,2)=                    
     &              DT2(IN1:IN2,1:IMMAX,:,1:LMAX)                       
               HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,3)=                    
     &              TT2(IN1:IN2,1:IMMAX,:,1:LMAX)                       
               HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,4)=                    
     &              DTE2(IN1:IN2,1:IMMAX,:,1:LMAX)                      
               DO ITRAC=1,NTRAC                                         
                  HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,4+ITRAC)=           
     &                 TRAT2(IN1:IN2,1:IMMAX,:,1:LMAX,ITRAC)            
               ENDDO                                                    
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=(4+NTRAC)*NNTx*MMTy*NHEM*NLx                   
               iTag=TagRef+100*iCol+10*iCol2+iRow                       
               CALL MPI_BSEND(HoldVar1,nBuffSize,ParaCom,iRecv,iTag,    
     &              MPI_COMM_WORLD,ierr)                                
            ENDIF                                                       
         ENDDO                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to receive data                
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Receive message                                        
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=(4+NTRAC)*NNTx*MMTy*NHEM*NLx                   
               iTag=TagRef+100*iCol2+10*iCol+iRow                       
               CALL MPI_RECV(HoldVar1,nBuffSize,ParaCom,iSend,iTag,     
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write to ZT etc                                        
               !-----------------------------------------------------   
               INMAX=NNTxx(iCol)                                        
               IMMAX=MMTyy(iRow)                                        
               LMAX=NLxx(iCol2)                                         
               L1=KK1(iCol2)+1                                          
               L2=KK1(iCol2)+NLxx(iCol2)                                
               ZT(1:INMAX,1:IMMAX,:,L1:L2)=                             
     &              HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,1)                
               DT(1:INMAX,1:IMMAX,:,L1:L2)=                             
     &              HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,2)                
               TT(1:INMAX,1:IMMAX,:,L1:L2)=                             
     &              HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,3)                
               DTE(1:INMAX,1:IMMAX,:,L1:L2)=                            
     &              HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,4)                
               DO ITRAC=1,NTRAC                                         
                  TRAT(1:INMAX,1:IMMAX,:,L1:L2,ITRAC)=                  
     &                 HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,4+ITRAC)       
               ENDDO                                                    
            ENDIF                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Data is already contained on this PE                         
         !-----------------------------------------------------         
         INMAX=NNTxx(iCol)                                              
         IMMAX=MMTyy(iRow)                                              
         LMAX=NLxx(iCol)                                                
         IN1=NNT1(iCol)+1                                               
         IN2=NNT1(iCol)+NNTxx(iCol)                                     
         L1=KK1(iCol)+1                                                 
         L2=KK1(iCol)+NLxx(iCol)                                        
         ZT(1:INMAX,1:IMMAX,:,L1:L2)=ZT2(IN1:IN2,1:IMMAX,:,1:LMAX)      
         DT(1:INMAX,1:IMMAX,:,L1:L2)=DT2(IN1:IN2,1:IMMAX,:,1:LMAX)      
         TT(1:INMAX,1:IMMAX,:,L1:L2)=TT2(IN1:IN2,1:IMMAX,:,1:LMAX)      
         DTE(1:INMAX,1:IMMAX,:,L1:L2)=DTE2(IN1:IN2,1:IMMAX,:,1:LMAX)    
         TRAT(1:INMAX,1:IMMAX,:,L1:L2,:)=                               
     &        TRAT2(IN1:IN2,1:IMMAX,:,1:LMAX,:)                         
         !-----------------------------------------------------         
         ! Single layer variables are only stored on first              
         ! column.                                                      
         !-----------------------------------------------------         
         IF (iCol.EQ.1) THEN                                            
            DO iCol2=2,Px                                               
               iPe=PeGrid(iCol2,iRow)                                   
               INMAX=NNTxx(iCol2)                                       
               IMMAX=MMTyy(iRow)                                        
               IN1=NNT1(iCol2)+1                                        
               IN2=NNT1(iCol2)+NNTxx(iCol2)                             
               HoldVar2(1:INMAX,1:IMMAX,:,1)=SPA2(IN1:IN2,1:IMMAX,:)    
               HoldVar2(1:INMAX,1:IMMAX,:,2)=VP2(IN1:IN2,1:IMMAX,:)     
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=2*NNTx*MMTy*NHEM                               
               iTag=TagRef+1100+10*iCol2+iRow                           
               CALL MPI_BSEND(HoldVar2,nBuffSize,ParaCom,iRecv,iTag,    
     &              MPI_COMM_WORLD,ierr)                                
            ENDDO                                                       
            !-----------------------------------------------------      
            ! Data is already on this array                             
            !-----------------------------------------------------      
            INMAX=NNTxx(iCol)                                           
            IMMAX=MMTyy(iRow)                                           
            SPA(1:INMAX,1:IMMAX,:)=SPA2(1:INMAX,1:IMMAX,:)              
            VP(1:INMAX,1:IMMAX,:)=VP2(1:INMAX,1:IMMAX,:)                
         ENDIF                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         IF (iCol.GT.1) THEN                                            
            !----------------------------------------------------       
            ! Receive messages                                          
            !----------------------------------------------------       
            iSend=PeGrid(1,iRow)                                        
            nBuffSize=2*NNTx*MMTy*NHEM                                  
            iTag=TagRef+1100+10*iCol+iRow                               
            CALL MPI_RECV(HoldVar2,nBuffSize,ParaCom,iSend,iTag,        
     &           MPI_COMM_WORLD,istat,ierr)                             
            !----------------------------------------------------       
            ! Write data to SPA and VP                                  
            !----------------------------------------------------       
            INMAX=NNTxx(iCol)                                           
            IMMAX=MMTyy(iRow)                                           
            SPA(1:INMAX,1:IMMAX,:)=HoldVar2(1:INMAX,1:IMMAX,:,1)        
            VP(1:INMAX,1:IMMAX,:)=HoldVar2(1:INMAX,1:IMMAX,:,2)         
         ENDIF                                                          
      ELSE                                                              
         !-----------------------------------------------------         
         ! One processor option                                         
         !-----------------------------------------------------         
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMT                                              
                  DO IN=1,NNT                                           
                     ZT(IN,IM,IHEM,L)=ZT2(IN,IM,IHEM,L)                 
                     DT(IN,IM,IHEM,L)=DT2(IN,IM,IHEM,L)                 
                     TT(IN,IM,IHEM,L)=TT2(IN,IM,IHEM,L)                 
                     DTE(IN,IM,IHEM,L)=DTE2(IN,IM,IHEM,L)               
                     DO ITRAC=1,NTRAC                                   
                        TRAT(IN,IM,IHEM,L,ITRAC)=                       
     &                       TRAT2(IN,IM,IHEM,L,ITRAC)                  
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO IHEM=1,NHEM                                                 
            DO IM=1,MMT                                                 
               DO IN=1,NNT                                              
                  SPA(IN,IM,IHEM)=SPA2(IN,IM,IHEM)                      
                  VP(IN,IM,IHEM)=VP2(IN,IM,IHEM)                        
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Transposing the source terms for the spectral coefficients so         
! that the levels are complete. Instead the meriodonal wavenumbers      
! are split across the columns of PEs.                                  
!=====================================================                  
      SUBROUTINE TRANS6A                                                
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Like the variables in SPECTR but the meridional wavenumbers           
! are complete instead of the levels.                                   
!-----------------------------------------------------                  
      COMMON/SPECTR2/Z2,D2,T2,SP2,SPA2,VP2,DTE2,TT2,DT2,ZT2             
     &     ,TRA2,TRAT2                                                  
! Vorticity coefficient                                                 
      COMPLEX :: Z2(NNT,MMTy,NHEM,NLx)                                  
! Divergence coefficient                                                
      COMPLEX :: D2(NNT,MMTy,NHEM,NLx)                                  
! Temperature coefficient                                               
      COMPLEX :: T2(NNT,MMTy,NHEM,NLx)                                  
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP2(NNT,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA2(NNT,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP2(NNT,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE2(NNT,MMTy,NHEM,NLx)                                
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT2(NNT,MMTy,NHEM,NLx)                                 
! Tracer coefficient                                                    
      COMPLEX :: TRA2(NNT,MMTy,NHEM,NLx,NTRAC)                          
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT2(NNT,MMTy,NHEM,NLx,NTRAC)                         
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=24000                                
      COMPLEX :: HoldVar1(NNTx,MMTy,NHEM,NLx,NTRAC)                     
      INTEGER :: iCol2, iPe                                             
      INTEGER :: IMMAX, INMAX, LMAX                                     
      INTEGER :: IN1, IN2                                               
      INTEGER :: L1, L2                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, IN, ITRAC, L                              
!-----------------------------------------------------                  
! Transpose                                                             
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to send them data              
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Store multi-layer variables in HoldVar1                
               !-----------------------------------------------------   
               INMAX=NNTxx(iCol2)                                       
               IMMAX=MMTyy(iRow)                                        
               LMAX=NLxx(iCol)                                          
               IN1=NNT1(iCol2)+1                                        
               IN2=NNT1(iCol2)+NNTxx(iCol2)                             
               HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,:)=                    
     &              TRAT2(IN1:IN2,1:IMMAX,:,1:LMAX,:)                   
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=NTRAC*NNTx*MMTy*NHEM*NLx                       
               iTag=TagRef+100*iCol+10*iCol2+iRow                       
               CALL MPI_BSEND(HoldVar1,nBuffSize,ParaCom,iRecv,iTag,    
     &              MPI_COMM_WORLD,ierr)                                
            ENDIF                                                       
         ENDDO                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to receive data                
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Receive message                                        
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=NTRAC*NNTx*MMTy*NHEM*NLx                       
               iTag=TagRef+100*iCol2+10*iCol+iRow                       
               CALL MPI_RECV(HoldVar1,nBuffSize,ParaCom,iSend,iTag,     
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write to TRAT                                          
               !-----------------------------------------------------   
               INMAX=NNTxx(iCol)                                        
               IMMAX=MMTyy(iRow)                                        
               LMAX=NLxx(iCol2)                                         
               L1=KK1(iCol2)+1                                          
               L2=KK1(iCol2)+NLxx(iCol2)                                
               TRAT(1:INMAX,1:IMMAX,:,L1:L2,:)=                         
     &              HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,:)                
            ENDIF                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Data is already contained on this PE                         
         !-----------------------------------------------------         
         INMAX=NNTxx(iCol)                                              
         IMMAX=MMTyy(iRow)                                              
         LMAX=NLxx(iCol)                                                
         IN1=NNT1(iCol)+1                                               
         IN2=NNT1(iCol)+NNTxx(iCol)                                     
         L1=KK1(iCol)+1                                                 
         L2=KK1(iCol)+NLxx(iCol)                                        
         TRAT(1:INMAX,1:IMMAX,:,L1:L2,:)=                               
     &        TRAT2(IN1:IN2,1:IMMAX,:,1:LMAX,:)                         
      ELSE                                                              
         !-----------------------------------------------------         
         ! One processor option                                         
         !-----------------------------------------------------         
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMT                                              
                  DO IN=1,NNT                                           
                     DO ITRAC=1,NTRAC                                   
                        TRAT(IN,IM,IHEM,L,ITRAC)=                       
     &                       TRAT2(IN,IM,IHEM,L,ITRAC)                  
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Transposing the source terms for the spectral coefficients so         
! that the levels are complete. Instead the meriodonal wavenumbers      
! are split across the columns of PEs.                                  
!=====================================================                  
      SUBROUTINE TRANS6B                                                
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Like the variables in SPECTR but the meridional wavenumbers           
! are complete instead of the levels.                                   
!-----------------------------------------------------                  
      COMMON/SPECTR2/Z2,D2,T2,SP2,SPA2,VP2,DTE2,TT2,DT2,ZT2             
     &     ,TRA2,TRAT2                                                  
! Vorticity coefficient                                                 
      COMPLEX :: Z2(NNT,MMTy,NHEM,NLx)                                  
! Divergence coefficient                                                
      COMPLEX :: D2(NNT,MMTy,NHEM,NLx)                                  
! Temperature coefficient                                               
      COMPLEX :: T2(NNT,MMTy,NHEM,NLx)                                  
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP2(NNT,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA2(NNT,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP2(NNT,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE2(NNT,MMTy,NHEM,NLx)                                
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT2(NNT,MMTy,NHEM,NLx)                                 
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT2(NNT,MMTy,NHEM,NLx)                                 
! Tracer coefficient                                                    
      COMPLEX :: TRA2(NNT,MMTy,NHEM,NLx,NTRAC)                          
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT2(NNT,MMTy,NHEM,NLx,NTRAC)                         
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=25000                                
      COMPLEX :: HoldVar1(NNTx,MMTy,NHEM,NLx,3+NTRAC)                   
      INTEGER :: iCol2, iPe                                             
      INTEGER :: IMMAX, INMAX, LMAX                                     
      INTEGER :: IN1, IN2                                               
      INTEGER :: L1, L2                                                 
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, IN, ITRAC, L                              
!-----------------------------------------------------                  
! Transpose                                                             
!-----------------------------------------------------                  
      IF (NPE.GT.1) THEN                                                
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to send them data              
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Store multi-layer variables in HoldVar1                
               !-----------------------------------------------------   
               INMAX=NNTxx(iCol2)                                       
               IMMAX=MMTyy(iRow)                                        
               LMAX=NLxx(iCol)                                          
               IN1=NNT1(iCol2)+1                                        
               IN2=NNT1(iCol2)+NNTxx(iCol2)                             
               HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,1)=                    
     &              ZT2(IN1:IN2,1:IMMAX,:,1:LMAX)                       
               HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,2)=                    
     &              DT2(IN1:IN2,1:IMMAX,:,1:LMAX)                       
               HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,3)=                    
     &              TT2(IN1:IN2,1:IMMAX,:,1:LMAX)                       
               HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,4:3+NTRAC)=            
     &              TRAT2(IN1:IN2,1:IMMAX,:,1:LMAX,:)                   
               !-----------------------------------------------------   
               ! Send message                                           
               !-----------------------------------------------------   
               iRecv=iPe                                                
               nBuffSize=(3+NTRAC)*NNTx*MMTy*NHEM*NLx                   
               iTag=TagRef+100*iCol+10*iCol2+iRow                       
               CALL MPI_BSEND(HoldVar1,nBuffSize,ParaCom,iRecv,iTag,    
     &              MPI_COMM_WORLD,ierr)                                
            ENDIF                                                       
         ENDDO                                                          
         CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)                         
         !-----------------------------------------------------         
         ! Loop over PEs on the same row to receive data                
         !-----------------------------------------------------         
         DO iCol2=1,Px                                                  
            iPe=PeGrid(iCol2,iRow)                                      
            IF (iPe.NE.MyPe) THEN                                       
               !-----------------------------------------------------   
               ! Receive message                                        
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=(3+NTRAC)*NNTx*MMTy*NHEM*NLx                   
               iTag=TagRef+100*iCol2+10*iCol+iRow                       
               CALL MPI_RECV(HoldVar1,nBuffSize,ParaCom,iSend,iTag,     
     &              MPI_COMM_WORLD,istat,ierr)                          
               !-----------------------------------------------------   
               ! Write to ZT etc                                        
               !-----------------------------------------------------   
               INMAX=NNTxx(iCol)                                        
               IMMAX=MMTyy(iRow)                                        
               LMAX=NLxx(iCol2)                                         
               L1=KK1(iCol2)+1                                          
               L2=KK1(iCol2)+NLxx(iCol2)                                
               ZT(1:INMAX,1:IMMAX,:,L1:L2)=                             
     &              HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,1)                
               DT(1:INMAX,1:IMMAX,:,L1:L2)=                             
     &              HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,2)                
               TT(1:INMAX,1:IMMAX,:,L1:L2)=                             
     &              HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,3)                
               TRAT(1:INMAX,1:IMMAX,:,L1:L2,:)=                         
     &              HoldVar1(1:INMAX,1:IMMAX,:,1:LMAX,4:3+NTRAC)        
            ENDIF                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Data is already contained on this PE                         
         !-----------------------------------------------------         
         INMAX=NNTxx(iCol)                                              
         IMMAX=MMTyy(iRow)                                              
         LMAX=NLxx(iCol)                                                
         IN1=NNT1(iCol)+1                                               
         IN2=NNT1(iCol)+NNTxx(iCol)                                     
         L1=KK1(iCol)+1                                                 
         L2=KK1(iCol)+NLxx(iCol)                                        
         ZT(1:INMAX,1:IMMAX,:,L1:L2)=ZT2(IN1:IN2,1:IMMAX,:,1:LMAX)      
         DT(1:INMAX,1:IMMAX,:,L1:L2)=DT2(IN1:IN2,1:IMMAX,:,1:LMAX)      
         TT(1:INMAX,1:IMMAX,:,L1:L2)=TT2(IN1:IN2,1:IMMAX,:,1:LMAX)      
         TRAT(1:INMAX,1:IMMAX,:,L1:L2,:)=                               
     &        TRAT2(IN1:IN2,1:IMMAX,:,1:LMAX,:)                         
      ELSE                                                              
         !-----------------------------------------------------         
         ! One processor option                                         
         !-----------------------------------------------------         
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMT                                              
                  DO IN=1,NNT                                           
                     ZT(IN,IM,IHEM,L)=ZT2(IN,IM,IHEM,L)                 
                     DT(IN,IM,IHEM,L)=DT2(IN,IM,IHEM,L)                 
                     TT(IN,IM,IHEM,L)=TT2(IN,IM,IHEM,L)                 
                     DO ITRAC=1,NTRAC                                   
                        TRAT(IN,IM,IHEM,L,ITRAC)=                       
     &                       TRAT2(IN,IM,IHEM,L,ITRAC)                  
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Takes an adiabatic timestep in spectral space                         
! either a centred semi-implicit timestep for KOUNT>KITS                
! or an initial short timestep for KOUNT<KITS                           
!                                                                       
! A time filter used for centred timestep.                              
!=====================================================                  
      SUBROUTINE TSTEP                                                  
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Restoration fields and timescale                                      
!-----------------------------------------------------                  
      COMMON/RESTOR/ZRES,DRES,TRES,SPRES,DAMP                           
      REAL :: ZRES(IDM,NHEM,NL)                                         
      REAL :: DRES(IDM,NHEM,NL)                                         
      REAL :: TRES(IDM,NHEM,NL)                                         
      REAL :: SPRES(IDM,NHEM)                                           
      REAL :: DAMP                                                      
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: WA(NL)                                                    
      COMPLEX :: D1(NL)                                                 
      INTEGER IWA(NL)                                                   
      REAL :: TBM1(NL,NL)                                               
!-----------------------------------------------------                  
! RMPA and RMPB are REAL versions of the complex arrays TMPA and        
! TMPB.                                                                 
!-----------------------------------------------------                  
      COMPLEX :: TMPA(NL),TMPB(NL)                                      
      REAL :: RMPA(2,NL),RMPB(2,NL)                                     
      EQUIVALENCE (TMPA(1),RMPA(1,1)),(TMPB(1),RMPB(1,1))               
!-----------------------------------------------------                  
! TTR, DTR, TMIR D1R are REAL versions of the COMPLEX arrays TT         
! DT, TMI D1 respectively                                               
!-----------------------------------------------------                  
      REAL :: TTR(2*NNTx,MMTy,NHEM,NL),DTR(2*NNTx,MMTy,NHEM,NL)         
      REAL :: TMIR(2*NNTx,MMTy,NHEM,NL),D1R(2,NL)                       
      EQUIVALENCE (TTR(1,1,1,1),TT(1,1,1,1)),(DTR(1,1,1,1),DT(1,1,1,1)) 
      EQUIVALENCE (TMIR(1,1,1,1),TMI(1,1,1,1))                          
     +,(D1R(1,1),D1(1))                                                 
      COMPLEX :: ZPAV,TPAV,DPAV,DTI,SPAV                                
      COMPLEX :: TRAPAV                                                 
      COMPLEX :: SPPA,VPS,GSI1                                          
      INTEGER :: I1, IBM1                                               
      REAL :: RCN                                                       
      INTEGER :: N                                                      
! Loop variables                                                        
      INTEGER :: IHEM, IL, IM, IN, iZon, KK, L, M, MP, nT               
      INTEGER :: NVAL                                                   
!-----------------------------------------------------                  
! Determine if it should be the short initial timestep or               
! centred semi-implicit timstep.                                        
!-----------------------------------------------------                  
      IF (KOUNT.GT.KITS) THEN                                           
         !-----------------------------------------------------         
         ! Ordinary centred semi-implicit timestep                      
         !-----------------------------------------------------         
         DO IHEM=1,NHEM                                                 
            DO IM=1,MMTyy(iRow)                                         
               DO IN=1,NNTxx(iCol)                                      
                  N=nWave(IN,IM,iCol)                                   
                  IF ((N+IHEM).GT.1) THEN                               
                     SPPA=SPMI(IN,IM,IHEM)                              
                     VPS=VP(IN,IM,IHEM)                                 
                     RCN=RSQ(N+IHEM)                                    
                     GSI1=GS(IN,IM,IHEM)                                
                     !---------------------------------------------     
                     ! Multiplies the temperature source term           
                     ! matrix, TTR/TT, and matrix G (matrix G           
                     ! relates temperature to geopotential).            
                     !---------------------------------------------     
                     CALL SGEMM('N','N',2,NL,NL,1.0,                    
     &                    TTR(2*IN-1,IM,IHEM,1),(2*NNTx*MMTy*NHEM),     
     &                    G,NL,0.0,RMPA,2)                              
                     !---------------------------------------------     
                     ! Multiplies temperature matrix, TMIR/TMI, by G.   
                     !---------------------------------------------     
                     CALL SGEMM('N','N',2,NL,NL,1.0,                    
     &                    TMIR(2*IN-1,IM,IHEM,1),(2*NNTx*MMTy*NHEM),    
     &                    G,NL,0.0,RMPB,2)                              
                     !---------------------------------------------     
                     ! The RHS of equation 17 in Hoskins and Simmons    
                     ! (1975) (H & S) - source term for calculating     
                     ! divergence.                                      
                     !---------------------------------------------     
                     DO L=1,NL                                          
                        D1(L)=RCN*DMI(IN,IM,IHEM,L)+DELT*(TMPB(L)+      
     &                       GSI1+T0(L)*SPPA+RCN*DT(IN,IM,IHEM,L)+      
     &                       DELT*(TMPA(L)-T0(L)*VPS))                  
                     ENDDO                                              
                     !---------------------------------------------     
                     ! Calculates the source term for divergence,       
                     ! DTR/DT, by multiplying the source term matrix    
                     ! above, D1R/D1, by an inversion of H & S's        
                     ! matrix A_n, BM1(:,:,IN).                         
                     !---------------------------------------------     
                     CALL SGEMM('N','N',2,NL,NL,1.0,D1R,2,              
     &                    BM1(1,1,N+IHEM-1),NL,0.0,                     
     &                    DTR(2*IN-1,IM,IHEM,1),(2*NNTx*MMTy*NHEM))     
                     !---------------------------------------------     
                     ! Calculates ln(pressure) (RHS of eq 15 in         
                     ! H & S)                                           
                     !---------------------------------------------     
                     DO L=1,NL                                          
                        VP(IN,IM,IHEM)=VP(IN,IM,IHEM)+                  
     &                       DT(IN,IM,IHEM,L)*DSIGMA(L)                 
                     ENDDO                                              
                     !---------------------------------------------     
                     ! Multiplies divergence source term matrix,        
                     ! DTR/DT, by matrix tau (RHS of eq 14 in H & S)    
                     !---------------------------------------------     
                     CALL SGEMM('N','N',2,NL,NL,1.0,                    
     &                    DTR(2*IN-1,IM,IHEM,1),(2*NNTx*MMTy*NHEM),     
     &                    TAU,NL,0.0,RMPA,2)                            
                     !---------------------------------------------     
                     ! Calculates source term for temperature, TT       
                     !---------------------------------------------     
                     DO L=1,NL                                          
                        TT(IN,IM,IHEM,L)=TT(IN,IM,IHEM,L)-TMPA(L)       
                     ENDDO                                              
                  ENDIF                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Calculate diabatic vorticity, divergence and temperature.    
         !-----------------------------------------------------         
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMTyy(iRow)                                      
                  DO IN=1,NNTxx(iCol)                                   
                     ZPAV=ZMI(IN,IM,IHEM,L)                             
                     ZMI(IN,IM,IHEM,L)=PNU21*Z(IN,IM,IHEM,L)+PNU*ZPAV   
                     Z(IN,IM,IHEM,L)=ZPAV+DELT2*ZT(IN,IM,IHEM,L)        
                     TPAV=TMI(IN,IM,IHEM,L)                             
                     TMI(IN,IM,IHEM,L)=PNU21*T(IN,IM,IHEM,L)+PNU*TPAV   
                     T(IN,IM,IHEM,L)=TPAV+DELT2*TT(IN,IM,IHEM,L)        
                     DPAV=DMI(IN,IM,IHEM,L)                             
                     DTI=DT(IN,IM,IHEM,L)                               
                     DMI(IN,IM,IHEM,L)=PNU21*D(IN,IM,IHEM,L)+PNU*DPAV   
                     D(IN,IM,IHEM,L)=DTI+DTI-DPAV                       
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Calculate tracers                                            
         !-----------------------------------------------------         
         DO KK=1,NTRAC                                                  
            DO L=1,NL                                                   
               DO IHEM=1,NHEM                                           
                  DO IM=1,MMTyy(iRow)                                   
                     DO IN=1,NNTxx(iCol)                                
                        TRAPAV=TRAMI(IN,IM,IHEM,L,KK)                   
                        TRAMI(IN,IM,IHEM,L,KK)=PNU21*                   
     &                       TRA(IN,IM,IHEM,L,KK)+PNU*TRAPAV            
                        TRA(IN,IM,IHEM,L,KK)=TRAPAV+DELT2*              
     &                       TRAT(IN,IM,IHEM,L,KK)                      
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Calculate surface pressure                                   
         !-----------------------------------------------------         
         DO IHEM=1,NHEM                                                 
            DO IM=1,MMTyy(iRow)                                         
               DO IN=1,NNTxx(iCol)                                      
                  SPAV=SPMI(IN,IM,IHEM)                                 
                  SPMI(IN,IM,IHEM)=PNU21*SP(IN,IM,IHEM)+PNU*SPAV        
                  SP(IN,IM,IHEM)=SPAV-DELT2*VP(IN,IM,IHEM)              
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         RETURN                                                         
      ELSE                                                              
         !-----------------------------------------------------         
         ! Initial short timestep                                       
         !-----------------------------------------------------         
         DO IHEM=1,NHEM                                                 
            DO IM=1,MMTyy(iRow)                                         
               DO IN=1,NNTxx(iCol)                                      
                  N=nWave(IN,IM,iCol)                                   
                  IF (N+IHEM.GT.1) THEN                                 
                     SPPA=SPMI(IN,IM,IHEM)                              
                     VPS=VP(IN,IM,IHEM)                                 
                     RCN=RSQ(N+IHEM)                                    
                     GSI1=GS(IN,IM,IHEM)                                
                     DO L=1,NL                                          
                        DO M=1,NL                                       
                           TBM1(M,L)=AQ(M,L)                            
                        ENDDO                                           
                     ENDDO                                              
                     DO L=1,NL                                          
                        TBM1(L,L)=TBM1(L,L)+RCN                         
                     ENDDO                                              
                     !---------------------------------------------     
                     ! Multiplies the temperature source term,          
                     ! matrix, TTR/TT, and matrix G (matrix G relates   
                     ! temperature to geopotential).                    
                     !---------------------------------------------     
                     CALL SGEMM('N','N',2,NL,NL,1.0,                    
     &                    TTR(2*IN-1,IM,IHEM,1),(2*NNTx*MMTy*NHEM),     
     &                    G,NL,0.0,RMPA,2)                              
                     !---------------------------------------------     
                     ! Multiplies temperature matrix, TMIR/TMI, by G.   
                     !---------------------------------------------     
                     CALL SGEMM('N','N',2,NL,NL,1.0,                    
     &                    TMIR(2*IN-1,IM,IHEM,1),(2*NNTx*MMTy*NHEM),    
     &                    G,NL,0.0,RMPB,2)                              
                     !---------------------------------------------     
                     ! The RHS of equation 17 in Hoskins and Simmons    
                     ! (1975) (H & S) - source term for calculating     
                     ! divergence.                                      
                     !---------------------------------------------     
                     DO L=1,NL                                          
                        D1(L)=RCN*DMI(IN,IM,IHEM,L)+DELT*(TMPB(L)+      
     &                       GSI1+T0(L)*SPPA+RCN*DT(IN,IM,IHEM,L)+      
     &                       DELT*(TMPA(L)-T0(L)*VPS))                  
                     ENDDO                                              
                     !---------------------------------------------     
                     ! Invert the matrix on the LHS of eq 17 in         
                     ! H & S (1975)                                     
                     !---------------------------------------------     
                     CALL MATINV(TBM1,NL,NL,IWA,WA)                     
                     !---------------------------------------------     
                     ! Calculates the source term for divergence,       
                     ! DTR/DT, by multiplying the source term matrix    
                     ! above, D1R/D1, by an inversion of H & S's        
                     ! matrix A_n, BM1(:,:,IN).                         
                     !---------------------------------------------     
                     CALL SGEMM('N','N',2,NL,NL,1.0,D1R,2,              
     &                    TBM1,NL,0.0,DTR(2*IN-1,IM,IHEM,1),            
     &                    (2*NNTx*MMTy*NHEM))                           
                     !---------------------------------------------     
                     ! Calculates ln(pressure) (RHS of eq 15 in         
                     ! H & S)                                           
                     !---------------------------------------------     
                     DO L=1,NL                                          
                        VP(IN,IM,IHEM)=VP(IN,IM,IHEM)+                  
     &                       DT(IN,IM,IHEM,L)*DSIGMA(L)                 
                     ENDDO                                              
                     !---------------------------------------------     
                     ! Multiplies divergence source term matrix,        
                     ! DTR/DT, by matrix tau (RHS of eq 14 in H & S)    
                     !---------------------------------------------     
                     CALL SGEMM('N','N',2,NL,NL,1.0,                    
     &                    DTR(2*IN-1,IM,IHEM,1),(2*NNTx*MMTy*NHEM),     
     &                    TAU,NL,0.0,RMPA,2)                            
                     !---------------------------------------------     
                     ! Calculates source term for temperature, TT       
                     !---------------------------------------------     
                     DO L=1,NL                                          
                        TT(IN,IM,IHEM,L)=TT(IN,IM,IHEM,L)-TMPA(L)       
                     ENDDO                                              
                  ENDIF                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMTyy(iRow)                                      
                  DO IN=1,NNTxx(iCol)                                   
                     Z(IN,IM,IHEM,L)=ZMI(IN,IM,IHEM,L)+                 
     &                    DELT2*ZT(IN,IM,IHEM,L)                        
                     T(IN,IM,IHEM,L)=TMI(IN,IM,IHEM,L)+                 
     &                    DELT2*TT(IN,IM,IHEM,L)                        
                     D(IN,IM,IHEM,L)=DT(IN,IM,IHEM,L)+DT(IN,IM,IHEM,L)- 
     &                    DMI(IN,IM,IHEM,L)                             
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO KK=1,NTRAC                                                  
            DO L=1,NL                                                   
               DO IHEM=1,NHEM                                           
                  DO IM=1,MMTyy(iRow)                                   
                     DO IN=1,NNTxx(iCol)                                
                        TRA(IN,IM,IHEM,L,KK)=TRAMI(IN,IM,IHEM,L,KK)+    
     &                       DELT2*TRAT(IN,IM,IHEM,L,KK)                
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO IHEM=1,NHEM                                                 
            DO IM=1,MMTyy(iRow)                                         
               DO IN=1,NNTxx(iCol)                                      
                  SP(IN,IM,IHEM)=SPMI(IN,IM,IHEM)-DELT2*VP(IN,IM,IHEM)  
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         DO L=1,NL                                                      
            DO M=1,NL                                                   
               AQ(M,L)=AQ(M,L)*4.0                                      
            ENDDO                                                       
         ENDDO                                                          
         RETURN                                                         
      ENDIF                                                             
      END                                                               
!=====================================================                  
! VERTICAL DIFFUSION OF MOMENTUM, HEAT AND MOISTURE.                    
! CONSTANT DIFFUSION COEFFICIENTS SET IN INITAL.                        
! ASSUMES ZERO FLUX AT UPPER AND LOWER BOUNDARIES.                      
! SURFACE FLUX DEALT WITH SEPARATELY IN BLAYER.                         
!=====================================================                  
      SUBROUTINE VDIFF(JH)                                              
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Grid variables for DGRMLT loop                                        
!-----------------------------------------------------                  
      COMMON/GRIDPC/ASSBL,ASHBL,ASLBL,ARRCR,ARRLR                       
     &     ,TTVD,QTVD,TYBL,TXBL,TTRD,TTLR,TTCR,UTVD,VTVD                
     &     ,QTLR,QTCR,SSBL,SHBL,SLBL,RRCR,RRLR                          
     &     ,ARFLUX,ASFLD,ACLD,RFLUX,SFLD,CLD                            
      REAL :: ASSBL(MGx,NHEM,JGy)                                       
      REAL :: ASHBL(MGx,NHEM,JGy)                                       
      REAL :: ASLBL(MGx,NHEM,JGy)                                       
      REAL :: ARRCR(MGx,NHEM,JGy)                                       
      REAL :: ARRLR(MGx,NHEM,JGy)                                       
      REAL :: TTVD(MGx,NHEM,NL)                                         
      REAL :: QTVD(MGx,NHEM,NL)                                         
      REAL :: TYBL(MGx,NHEM)                                            
      REAL :: TXBL(MGx,NHEM)                                            
      REAL :: TTRD(MGx,NHEM,NL)                                         
      REAL :: TTLR(MGx,NHEM,NL)                                         
      REAL :: TTCR(MGx,NHEM,NL)                                         
      REAL :: UTVD(MGx,NHEM,NL)                                         
      REAL :: VTVD(MGx,NHEM,NL)                                         
      REAL :: QTLR(MGx,NHEM,NL)                                         
      REAL :: QTCR(MGx,NHEM,NL)                                         
      REAL :: SSBL(MGx,NHEM,JGy)                                        
      REAL :: SHBL(MGx,NHEM,JGy)                                        
      REAL :: SLBL(MGx,NHEM,JGy)                                        
      REAL :: RRCR(MGx,NHEM,JGy)                                        
      REAL :: RRLR(MGx,NHEM,JGy)                                        
      REAL :: arflux(MGx,NHEM,JGy,6)                                    
      REAL :: asfld(MGx,NHEM,JGy,6)                                     
      REAL :: acld(MGx,NHEM,JGy,4)                                      
      REAL :: rflux(MGx,NHEM,JGy,6)                                     
      REAL :: sfld(MGx,NHEM,JGy,6)                                      
      REAL :: cld(MGx,NHEM,JGy,4)                                       
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed argument                                                       
!-----------------------------------------------------                  
! Latitude index                                                        
      INTEGER :: JH                                                     
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL BVP(MGx),BQP(MGx),BTP(MGx),BVM(MGx),BQM(MGx),BTM(MGx)        
      REAL FA(NL)                                                       
      REAL :: TLPH                                                      
      REAL :: FTSQ                                                      
! Loop variables                                                        
      INTEGER :: I, IHEM, L                                             
!-----------------------------------------------------                  
!                                                                       
!-----------------------------------------------------                  
      DO L=1,NL                                                         
         FA(L)=1.0/DSIGMA(L)                                            
      ENDDO                                                             
!-----------------------------------------------------                  
!                                                                       
!-----------------------------------------------------                  
      DO IHEM=1,NHEM                                                    
         DO I=1,MGxx(iCol)                                              
            TLPH=TG3(I,IHEM,JH,2)+TG3(I,IHEM,JH,1)                      
            FTSQ=FB(1)/(TLPH*TLPH)                                      
            BVP(I)=AKVV*FTSQ                                            
            BQP(I)=AKQV*FTSQ                                            
            BTP(I)=AKTV*FTSQ                                            
            UTVD(I,IHEM,1)=FA(1)*BVP(I)*(UG3(I,IHEM,JH,2)-              
     &           UG3(I,IHEM,JH,1))                                      
            VTVD(I,IHEM,1)=FA(1)*BVP(I)*(VG3(I,IHEM,JH,2)-              
     &           VG3(I,IHEM,JH,1))                                      
            QTVD(I,IHEM,1)=FA(1)*BQP(I)*(TRAG3(I,IHEM,JH,2,1)-          
     &           TRAG3(I,IHEM,JH,1,1))                                  
            TTVD(I,IHEM,1)=FA(1)*BTP(I)*(SK(1)*TG3(I,IHEM,JH,2)-        
     &           TG3(I,IHEM,JH,1))                                      
         ENDDO                                                          
         DO L=2,NL-1                                                    
            DO I=1,MGxx(iCol)                                           
               BVM(I)=BVP(I)                                            
               BQM(I)=BQP(I)                                            
               BTM(I)=BTP(I)                                            
               TLPH=TG3(I,IHEM,JH,L+1)+TG3(I,IHEM,JH,L)                 
               FTSQ=FB(L)/(TLPH*TLPH)                                   
               BVP(I)=AKVV*FTSQ                                         
               BQP(I)=AKQV*FTSQ                                         
               BTP(I)=AKTV*FTSQ                                         
               IF (L.EQ.(NL-1)) THEN                                    
                  if (LBL.and.(.not.LOLDBL)) THEN                       
                     BVP(I)=BVP(I)*5.                                   
                  endif                                                 
               ENDIF                                                    
               UTVD(i,ihem,l)=FA(L)*( BVP(I)*(UG3(I,IHEM,JH,L+1)-       
     &              UG3(I,IHEM,JH,L))-BVM(I)*                           
     &              (UG3(I,IHEM,JH,L)-UG3(I,IHEM,JH,L-1)))              
               VTVD(I,IHEM,L)=FA(L)*( BVP(I)*(VG3(I,IHEM,JH,L+1)-       
     &              VG3(I,IHEM,JH,L))-BVM(I)*                           
     &              (VG3(I,IHEM,JH,L)-VG3(I,IHEM,JH,L-1)))              
               QTVD(I,IHEM,L)=FA(L)*( BQP(I)*(TRAG3(I,IHEM,JH,L+1,1)-   
     &              TRAG3(I,IHEM,JH,L,1))-BQM(I)*                       
     &              (TRAG3(I,IHEM,JH,L,1)-TRAG3(I,IHEM,JH,L-1,1)))      
               TTVD(I,IHEM,L)=FA(L)*( BTP(I)*(SK(L)*TG3(I,IHEM,JH,L+1)- 
     &              TG3(I,IHEM,JH,L))-BTM(I)*                           
     &              (TG3(I,IHEM,JH,L)-TG3(I,IHEM,JH,L-1)/SK(L-1)))      
            ENDDO                                                       
         ENDDO                                                          
         DO I=1,MGxx(iCol)                                              
            UTVD(I,IHEM,NL)=-FA(NL)*BVP(I)*(UG3(I,IHEM,JH,NL)-          
     &           UG3(I,IHEM,JH,NL-1))                                   
            VTVD(I,IHEM,NL)=-FA(NL)*BVP(I)*(VG3(I,IHEM,JH,NL)-          
     &           VG3(I,IHEM,JH,NL-1))                                   
            QTVD(I,IHEM,NL)=-FA(NL)*BQP(I)*(TRAG3(I,IHEM,JH,NL,1)-      
     &           TRAG3(I,IHEM,JH,NL-1,1))                               
            TTVD(I,IHEM,NL)=-FA(NL)*BTP(I)*(TG3(I,IHEM,JH,NL)-          
     &           TG3(I,IHEM,JH,NL-1)/SK(NL-1))                          
         ENDDO                                                          
         DO L=1,NL                                                      
            DO I=1,MGxx(iCol)                                           
               UG3(I,IHEM,JH,L)=UG3(I,IHEM,JH,L)+DELT2C*UTVD(I,IHEM,L)  
               VG3(I,IHEM,JH,L)=VG3(I,IHEM,JH,L)+DELT2C*VTVD(I,IHEM,L)  
               TRAG3(I,IHEM,JH,L,1)=TRAG3(I,IHEM,JH,L,1)+               
     &              DELT2C*QTVD(I,IHEM,L)                               
               TG3(I,IHEM,JH,L)=TG3(I,IHEM,JH,L)+DELT2C*TTVD(I,IHEM,L)  
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! Write coefficients to file. Using a binary code for iType to          
! decide which variables are needed:                                    
!  - +1: Z, D, T, TRA (if used) and SP are wanted                       
!  - +2: ZMI, DMI, TRAMI (if used) and SPMI are wanted                  
!  - +4: TTRES is wanted                                                
!  - +8: TAVE is wanted                                                 
!  - +16: TT is wanted                                                  
!  - +32: HTNET is wanted (HTNET is a grid-space variable)              
! e.g. For version above 1, to write out RKOUNT, RNTAPE, DAY, DOY,      
! Z, D, T, TRA, TAVE, SP, RNTAPE iType needs to be 9 (1+8).             
!=====================================================                  
      SUBROUTINE WRITECOEF(UnitId, iType, RKOUNT, DOY)                  
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Restoration temperature field and constants which determine it,       
! also contains timescales                                              
!-----------------------------------------------------                  
      COMMON/RESTIJ/TTRES                                               
     + ,DTNS,DTEP,DTTRP,FAC,DDAMP,TFRC,YRLEN,TRS                        
     +  ,ALR,ZTROP,TGR                                                  
      COMPLEX :: TTRES(NNTx,MMTy,NHEM,NL)                               
      REAL :: DTNS                                                      
      REAL :: DTEP                                                      
      REAL :: DTTRP                                                     
      REAL :: FAC(NL)                                                   
      REAL :: DDAMP(NL)                                                 
      REAL :: TFRC(NL)                                                  
      REAL :: YRLEN                                                     
      REAL :: TRS(NL)                                                   
      REAL :: ALR                                                       
      REAL :: ZTROP                                                     
      REAL :: TGR                                                       
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Switches counters and constants controlling type and frequency of     
! model output                                                          
!-----------------------------------------------------                  
      COMMON/OUTCON/RNTAPE,DAY,SQR2,RSQR2,EAM1,EAM2,TOUT1               
     &     ,TOUT2,LSPO,LGPO                                             
     &     ,RNTAPO                                                      
     &     ,LSHIST,LMINIH                                               
      COMMON/OUTCONINT/NCOEFF,NLAT,INLAT,INSPC,KOUNTP                   
     &                 ,KOUNTE,KOUNTH,KOUNTR,KOUTP                      
     &                 ,KOUTE,KOUTH,KOUTR                               
! Identyfying run number                                                
      REAL :: RNTAPE                                                    
! Maximum wavenumber of printed spectral coeffecients                   
      INTEGER :: NCOEFF                                                 
! Number of latitudes in printed grid point fields                      
      INTEGER :: NLAT                                                   
      INTEGER :: INLAT                                                  
!                                                                       
      INTEGER :: INSPC                                                  
! Number of timsteps between successive writing of spectral             
! and gridded fields                                                    
      INTEGER :: KOUNTP                                                 
      INTEGER :: KOUNTE                                                 
! Number of timsteps between successive writing of history              
! records, containing spectral fields at the present and                
! previous time levels.                                                 
      INTEGER :: KOUNTH                                                 
! Number of timsteps between successive writing of restart              
! records, containing the spectral fields at the present and            
! previous time levels                                                  
      INTEGER :: KOUNTR                                                 
      INTEGER :: KOUTP                                                  
      INTEGER :: KOUTE                                                  
      INTEGER :: KOUTH                                                  
      INTEGER :: KOUTR                                                  
      REAL :: DAY                                                       
      REAL :: SQR2                                                      
      REAL :: RSQR2                                                     
      REAL :: EAM1                                                      
      REAL :: EAM2                                                      
      REAL :: TOUT1                                                     
      REAL :: TOUT2                                                     
      LOGICAL :: LSPO(NL)                                               
      LOGICAL :: LGPO(NL)                                               
      REAL :: RNTAPO                                                    
      LOGICAL :: LSHIST                                                 
! Reduce the history file to single precision if true                   
      LOGICAL :: LMINIH                                                 
!-----------------------------------------------------                  
! Radiation variables                                                   
!-----------------------------------------------------                  
      COMMON /RADHT/HTNET,TAVE                                          
      REAL :: HTNET(NHEM,JGy,MGx,NL)                                    
      COMPLEX :: TAVE(NNTx,MMTy,NHEM,NL)                                
!-----------------------------------------------------                  
! Passed variables                                                      
!-----------------------------------------------------                  
! Unit number of file                                                   
      INTEGER :: UnitId                                                 
! The list of variables to write out is deterimed by choice of iType    
      INTEGER :: iType                                                  
      REAL :: RKOUNT                                                    
      REAL :: DOY                                                       
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      COMPLEX, ALLOCATABLE :: StoreArray(:,:,:,:)                       
      COMPLEX, ALLOCATABLE :: WriteArray(:,:,:)                         
      REAL (KIND=4), ALLOCATABLE :: WriteArraySP(:)                     
      REAL, ALLOCATABLE :: HtnetFull(:,:,:,:)                           
      INTEGER :: nLevels                                                
! Current starting point for levels in StoreArray                       
      INTEGER :: LL                                                     
      INTEGER :: iPos                                                   
! Loop variables                                                        
      INTEGER :: I, IHEM, IM, IN, ITRAC, L                              
!-----------------------------------------------------                  
! Check the type makes sense                                            
!-----------------------------------------------------                  
      IF ( (iType.NE.1).AND.(iType.NE.3).AND.(iType.NE.4)               
     &     .AND.(iType.NE.9).AND.(iType.NE.16).AND.(iType.NE.35) ) THEN 
         !-----------------------------------------------------         
         ! This type is not currently allowed                           
         !-----------------------------------------------------         
         IF (MyPe.EQ.0) WRITE(6,*) 'ERROR: this type is not currently ',
     &        'allowed: iType=', iType                                  
         CALL ABORT                                                     
      ENDIF                                                             
!-----------------------------------------------------                  
! Work out how many levels are needed                                   
!-----------------------------------------------------                  
      nLevels=0                                                         
      IF (MOD(iType,2).EQ.1) THEN                                       
         nLevels=nLevels+(3+NTRAC)*NL+1                                 
      ENDIF                                                             
      IF (MOD(iType,4).GE.2) THEN                                       
         nLevels=nLevels+(3+NTRAC)*NL+1                                 
      ENDIF                                                             
      IF (MOD(iType,8).GE.4) THEN                                       
         nLevels=nLevels+NL                                             
      ENDIF                                                             
      IF (MOD(iType,32).GE.16) THEN                                     
         nLevels=nLevels+NL                                             
      ENDIF                                                             
      ALLOCATE(StoreArray(NNTx,MMTy,NHEM,nLevels))                      
!-----------------------------------------------------                  
! Storing Z, D, and T                                                   
!-----------------------------------------------------                  
      LL=0                                                              
      IF (MOD(iType,2).EQ.1) THEN                                       
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMTyy(iRow)                                      
                  DO IN=1,NNTxx(iCol)                                   
                     StoreArray(IN,IM,IHEM,LL+L)=Z(IN,IM,IHEM,L)        
                     StoreArray(IN,IM,IHEM,NL+LL+L)=D(IN,IM,IHEM,L)     
                     StoreArray(IN,IM,IHEM,2*NL+LL+L)=T(IN,IM,IHEM,L)   
                     DO ITRAC=1,NTRAC                                   
                        StoreArray(IN,IM,IHEM,(2+ITRAC)*NL+LL+L)=       
     &                       TRA(IN,IM,IHEM,L,ITRAC)                    
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         LL=LL+(3+NTRAC)*NL                                             
      ENDIF                                                             
!-----------------------------------------------------                  
! Writing TAVE over entry for T. TAVE is temperature averaged over      
! a number of timesteps                                                 
!-----------------------------------------------------                  
      IF (MOD(iType,16).GE.8) THEN                                      
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMTyy(iRow)                                      
                  DO IN=1,NNTxx(iCol)                                   
                     StoreArray(IN,IM,IHEM,2*NL+L)=TAVE(IN,IM,IHEM,L)   
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
      ENDIF                                                             
!-----------------------------------------------------                  
! Storing SP                                                            
!-----------------------------------------------------                  
      IF (MOD(iType,2).EQ.1) THEN                                       
         DO IHEM=1,NHEM                                                 
            DO IM=1,MMTyy(iRow)                                         
               DO IN=1,NNTxx(iCol)                                      
                  StoreArray(IN,IM,IHEM,LL+1)=SP(IN,IM,IHEM)            
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         LL=LL+1                                                        
      ENDIF                                                             
!-----------------------------------------------------                  
! Storing ZMI, DMI, TMI and SPMI                                        
!-----------------------------------------------------                  
      IF (MOD(iType,4).GE.2) THEN                                       
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMTyy(iRow)                                      
                  DO IN=1,NNTxx(iCol)                                   
                     StoreArray(IN,IM,IHEM,LL+L)=                       
     &                    ZMI(IN,IM,IHEM,L)                             
                     StoreArray(IN,IM,IHEM,NL+LL+L)=                    
     &                    DMI(IN,IM,IHEM,L)                             
                     StoreArray(IN,IM,IHEM,2*NL+LL+L)=                  
     &                    TMI(IN,IM,IHEM,L)                             
                     DO ITRAC=1,NTRAC                                   
                        StoreArray(IN,IM,IHEM,(2+ITRAC)*NL+LL+L)=       
     &                       TRAMI(IN,IM,IHEM,L,ITRAC)                  
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         LL=LL+(3+NTRAC)*NL                                             
         DO IHEM=1,NHEM                                                 
            DO IM=1,MMTyy(iRow)                                         
               DO IN=1,NNTxx(iCol)                                      
                  StoreArray(IN,IM,IHEM,LL+1)=SPMI(IN,IM,IHEM)          
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         LL=LL+1                                                        
      ENDIF                                                             
!-----------------------------------------------------                  
! Storing TTRES                                                         
!-----------------------------------------------------                  
      IF (MOD(iType,8).GE.4) THEN                                       
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMTyy(iRow)                                      
                  DO IN=1,NNTxx(iCol)                                   
                     StoreArray(IN,IM,IHEM,LL+L)=TTRES(IN,IM,IHEM,L)    
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         LL=LL+NL                                                       
      ENDIF                                                             
!-----------------------------------------------------                  
! Storing TT                                                            
!-----------------------------------------------------                  
      IF (MOD(iType,32).GE.16) THEN                                     
         DO L=1,NL                                                      
            DO IHEM=1,NHEM                                              
               DO IM=1,MMTyy(iRow)                                      
                  DO IN=1,NNTxx(iCol)                                   
                     StoreArray(IN,IM,IHEM,LL+L)=TT(IN,IM,IHEM,L)       
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         LL=LL+NL                                                       
      ENDIF                                                             
!-----------------------------------------------------                  
! Get all the data to PE 0                                              
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) ALLOCATE(WriteArray(NWJ2,NHEM,nLevels))            
      CALL SHARECOEF(.FALSE.,(NHEM*nLevels),WriteArray,StoreArray)      
      DEALLOCATE(StoreArray)                                            
      !-----------------------------------------------------            
      ! Write data to file                                              
      !-----------------------------------------------------            
      IF (iType.EQ.35) THEN                                             
         !-----------------------------------------------------         
         ! Need to add in HTNET which is a grid-space variable          
         !-----------------------------------------------------         
         IF (MyPe.EQ.0) ALLOCATE(HtnetFull(NHEM,JG,MG,NL))              
         CALL SHAREGRID2(.FALSE.,NL,HtnetFull,htnet)                    
         IF (MyPe.EQ.0) THEN                                            
            WRITE(UnitId) RKOUNT,RNTAPE,DAY,DOY,                        
     &           WriteArray(:,:,1:nLevels/2),RNTAPE,                    
     &           WriteArray(:,:,nLevels/2+1:nLevels),HtnetFull,RNTAPE   
            DEALLOCATE(HtnetFull)                                       
         ENDIF                                                          
      ELSE IF (MyPe.EQ.0) THEN                                          
         IF (iType.EQ.3) THEN                                           
            WRITE(UnitId) RKOUNT,RNTAPE,DAY,DOY,                        
     &           WriteArray(:,:,1:nLevels/2),RNTAPE,                    
     &           WriteArray(:,:,nLevels/2+1:nLevels),RNTAPE             
         ELSE IF (iType.EQ.16) THEN                                     
            IF ( (UnitId == 9).AND.(LMINIH) ) THEN                      
               ALLOCATE(WriteArraySP(2*NWJ2*NHEM*nLevels+6))            
               WriteArraySP(1)=RKOUNT                                   
               WriteArraySP(2)=RNTAPE                                   
               WriteArraySP(3)=DAY                                      
               WriteArraySP(4)=DOY                                      
               WriteArraySP(5)=JG+2                                     
               DO L=1,nLevels                                           
                  DO IHEM=1,NHEM                                        
                     DO I=1,NWJ2                                        
                        iPos=2*( ((L-1)*NHEM+(IHEM-1))*NWJ2+I )+4       
                        WriteArraySP(iPos)=REAL(WriteArray(I,IHEM,L))   
                        WriteArraySP(iPos+1)=AIMAG(WriteArray(I,IHEM,L))
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               WriteArraySP(2*NWJ2*NHEM*nLevels+6)=RNTAPE               
               WRITE(UnitId) WriteArraySP                               
               DEALLOCATE(WriteArraySP)                                 
            ELSE                                                        
               WRITE(UnitId) RKOUNT,RNTAPE,DAY,DOY,REAL(JG+2),          
     &              WriteArray(:,:,:),RNTAPE                            
            ENDIF                                                       
         ELSE                                                           
            IF ( (UnitId == 9).AND.(LMINIH) ) THEN                      
               !-----------------------------------------------------   
               ! Write as single predision                              
               !-----------------------------------------------------   
               ALLOCATE(WriteArraySP(2*NWJ2*NHEM*nLevels+5))            
               WriteArraySP(1)=RKOUNT                                   
               WriteArraySP(2)=RNTAPE                                   
               WriteArraySP(3)=DAY                                      
               WriteArraySP(4)=DOY                                      
               DO L=1,nLevels                                           
                  DO IHEM=1,NHEM                                        
                     DO I=1,NWJ2                                        
                        iPos=2*( ((L-1)*NHEM+(IHEM-1))*NWJ2+I )+3       
                        WriteArraySP(iPos)=                             
     &                       REAL(WriteArray(I,IHEM,L),KIND=4)          
                        WriteArraySP(iPos+1)=REAL(                      
     &                       AIMAG(WriteArray(I,IHEM,L)),KIND=4)        
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
               WriteArraySP(2*NWJ2*NHEM*nLevels+5)=RNTAPE               
               WRITE(UnitId) WriteArraySP                               
               DEALLOCATE(WriteArraySP)                                 
            ELSE                                                        
               !-----------------------------------------------------   
               ! Write as the normal predision                          
               !-----------------------------------------------------   
               WRITE(UnitId) RKOUNT,RNTAPE,DAY,DOY,                     
     &              WriteArray(:,:,:),RNTAPE                            
            ENDIF                                                       
         ENDIF                                                          
      ENDIF                                                             
      IF (MyPe.EQ.0)DEALLOCATE(WriteArray)                              
      END                                                               
!=====================================================                  
! Write the grid-point diagnostics to file                              
!=====================================================                  
      SUBROUTINE WRITEDIAG(UnitId, RKOUNT, DOY)                         
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Stores diagnostic data                                                
!-----------------------------------------------------                  
      COMMON/PTENDZ/UTVDZ,VTVDZ,TTVDZ,QTVDZ,TTCRZ,QTCRZ                 
     &     ,TTLRZ,QTLRZ,TTRDZ,CTCRZ,CTLRZ                               
     &     ,UTBLZ,VTBLZ,TTBLZ,QTBLZ                                     
     &     ,AUTVDZ,AVTVDZ,ATTVDZ,AQTVDZ,ATTCRZ,AQTCRZ                   
     &     ,ATTLRZ,AQTLRZ,ATTRDZ,ACTCRZ,ACTLRZ                          
     &     ,AUTBLZ,AVTBLZ,ATTBLZ,AQTBLZ                                 
      REAL :: UTVDZ(JGy,NHEM,NL)                                        
      REAL :: VTVDZ(JGy,NHEM,NL)                                        
      REAL :: TTVDZ(JGy,NHEM,NL)                                        
      REAL :: QTVDZ(JGy,NHEM,NL)                                        
      REAL :: TTCRZ(JGy,NHEM,NL)                                        
      REAL :: QTCRZ(JGy,NHEM,NL)                                        
      REAL :: TTLRZ(JGy,NHEM,NL)                                        
      REAL :: QTLRZ(JGy,NHEM,NL)                                        
      REAL :: TTRDZ(JGy,NHEM,NL)                                        
      REAL :: CTCRZ(JGy,NHEM,NL)                                        
      REAL :: CTLRZ(JGy,NHEM,NL)                                        
      REAL :: UTBLZ(JGy,NHEM)                                           
      REAL :: VTBLZ(JGy,NHEM)                                           
      REAL :: TTBLZ(JGy,NHEM)                                           
      REAL :: QTBLZ(JGy,NHEM)                                           
! Variables below are for storing averages of the variables above       
      REAL :: AUTVDZ(JGy,NHEM,NL)                                       
      REAL :: AVTVDZ(JGy,NHEM,NL)                                       
      REAL :: ATTVDZ(JGy,NHEM,NL)                                       
      REAL :: AQTVDZ(JGy,NHEM,NL)                                       
      REAL :: ATTCRZ(JGy,NHEM,NL)                                       
      REAL :: AQTCRZ(JGy,NHEM,NL)                                       
      REAL :: ATTLRZ(JGy,NHEM,NL)                                       
      REAL :: AQTLRZ(JGy,NHEM,NL)                                       
      REAL :: ATTRDZ(JGy,NHEM,NL)                                       
      REAL :: ACTCRZ(JGy,NHEM,NL)                                       
      REAL :: ACTLRZ(JGy,NHEM,NL)                                       
      REAL :: AUTBLZ(JGy,NHEM)                                          
      REAL :: AVTBLZ(JGy,NHEM)                                          
      REAL :: ATTBLZ(JGy,NHEM)                                          
      REAL :: AQTBLZ(JGy,NHEM)                                          
!-----------------------------------------------------                  
! Switches counters and constants controlling type and frequency of     
! model output                                                          
!-----------------------------------------------------                  
      COMMON/OUTCON/RNTAPE,DAY,SQR2,RSQR2,EAM1,EAM2,TOUT1               
     &     ,TOUT2,LSPO,LGPO                                             
     &     ,RNTAPO                                                      
     &     ,LSHIST,LMINIH                                               
      COMMON/OUTCONINT/NCOEFF,NLAT,INLAT,INSPC,KOUNTP                   
     &                 ,KOUNTE,KOUNTH,KOUNTR,KOUTP                      
     &                 ,KOUTE,KOUTH,KOUTR                               
! Identyfying run number                                                
      REAL :: RNTAPE                                                    
! Maximum wavenumber of printed spectral coeffecients                   
      INTEGER :: NCOEFF                                                 
! Number of latitudes in printed grid point fields                      
      INTEGER :: NLAT                                                   
      INTEGER :: INLAT                                                  
!                                                                       
      INTEGER :: INSPC                                                  
! Number of timsteps between successive writing of spectral             
! and gridded fields                                                    
      INTEGER :: KOUNTP                                                 
      INTEGER :: KOUNTE                                                 
! Number of timsteps between successive writing of history              
! records, containing spectral fields at the present and                
! previous time levels.                                                 
      INTEGER :: KOUNTH                                                 
! Number of timsteps between successive writing of restart              
! records, containing the spectral fields at the present and            
! previous time levels                                                  
      INTEGER :: KOUNTR                                                 
      INTEGER :: KOUTP                                                  
      INTEGER :: KOUTE                                                  
      INTEGER :: KOUTH                                                  
      INTEGER :: KOUTR                                                  
      REAL :: DAY                                                       
      REAL :: SQR2                                                      
      REAL :: RSQR2                                                     
      REAL :: EAM1                                                      
      REAL :: EAM2                                                      
      REAL :: TOUT1                                                     
      REAL :: TOUT2                                                     
      LOGICAL :: LSPO(NL)                                               
      LOGICAL :: LGPO(NL)                                               
      REAL :: RNTAPO                                                    
      LOGICAL :: LSHIST                                                 
! Reduce the history file to single precision if true                   
      LOGICAL :: LMINIH                                                 
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variables                                                      
!-----------------------------------------------------                  
! Unit number of file                                                   
      INTEGER :: UnitId                                                 
      REAL :: RKOUNT                                                    
      REAL :: DOY                                                       
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: nLevels=22*NL+8                             
      REAL :: StoreArray(JGy,NHEM,nLevels)                              
      INTEGER :: LL                                                     
      REAL, ALLOCATABLE :: WriteArray(:,:,:)                            
      REAL (KIND=4), ALLOCATABLE :: WriteArraySP(:)                     
! Loop variables                                                        
      INTEGER :: I, J, L                                                
!-----------------------------------------------------                  
! Store all the data needed in one array                                
!-----------------------------------------------------                  
      LL=0                                                              
      StoreArray(:,:,LL+1:LL+NL)=AUTVDZ(:,:,:)                          
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=AVTVDZ(:,:,:)                          
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=ATTVDZ(:,:,:)                          
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=AQTVDZ(:,:,:)                          
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=ATTCRZ(:,:,:)                          
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=AQTCRZ(:,:,:)                          
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=ATTLRZ(:,:,:)                          
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=AQTLRZ(:,:,:)                          
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=ATTRDZ(:,:,:)                          
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=ACTCRZ(:,:,:)                          
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=ACTLRZ(:,:,:)                          
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1)=AUTBLZ(:,:)                                  
      LL=LL+1                                                           
      StoreArray(:,:,LL+1)=AVTBLZ(:,:)                                  
      LL=LL+1                                                           
      StoreArray(:,:,LL+1)=ATTBLZ(:,:)                                  
      LL=LL+1                                                           
      StoreArray(:,:,LL+1)=AQTBLZ(:,:)                                  
      LL=LL+1                                                           
      StoreArray(:,:,LL+1:LL+NL)=UTVDZ(:,:,:)                           
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=VTVDZ(:,:,:)                           
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=TTVDZ(:,:,:)                           
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=QTVDZ(:,:,:)                           
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=TTCRZ(:,:,:)                           
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=QTCRZ(:,:,:)                           
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=TTLRZ(:,:,:)                           
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=QTLRZ(:,:,:)                           
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=TTRDZ(:,:,:)                           
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=CTCRZ(:,:,:)                           
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1:LL+NL)=CTLRZ(:,:,:)                           
      LL=LL+NL                                                          
      StoreArray(:,:,LL+1)=UTBLZ(:,:)                                   
      LL=LL+1                                                           
      StoreArray(:,:,LL+1)=VTBLZ(:,:)                                   
      LL=LL+1                                                           
      StoreArray(:,:,LL+1)=TTBLZ(:,:)                                   
      LL=LL+1                                                           
      StoreArray(:,:,LL+1)=QTBLZ(:,:)                                   
!-----------------------------------------------------                  
! Send all the data to PE 0                                             
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) ALLOCATE(WriteArray(JG,NHEM,nLevels))              
      CALL SHAREGRID3(nLevels,StoreArray,WriteArray)                    
!-----------------------------------------------------                  
! Write the data to file                                                
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         IF ( (UnitId == 9).AND.(LMINIH) ) THEN                         
            !-----------------------------------------------------      
            ! Write as single precision                                 
            !-----------------------------------------------------      
            ALLOCATE(WriteArraySP(6+JG*NHEM*nLevels))                   
            WriteArraySP(1)=RKOUNT                                      
            WriteArraySP(2)=RNTAPE                                      
            WriteArraySP(3)=DAY                                         
            WriteArraySP(4)=DOY                                         
            WriteArraySP(5)=JG+1                                        
            I=6                                                         
            DO L=1,nLevels                                              
               DO J=1,JG                                                
                  WriteArraySp(I)=WriteArray(J,1,L)                     
                  I=I+1                                                 
               ENDDO                                                    
               !-----------------------------------------------------   
               ! The j-index for the 2nd hemisphere needs switching     
               !-----------------------------------------------------   
               IF (NHEM.EQ.2) THEN                                      
                  DO J=1,JG                                             
                     WriteArraySp(I)=WriteArray(JG+1-J,NHEM,L)          
                     I=I+1                                              
                  ENDDO                                                 
               ENDIF                                                    
            ENDDO                                                       
            WriteArraySP(I)=RNTAPE                                      
            !-----------------------------------------------------      
            ! Write to file                                             
            !-----------------------------------------------------      
            WRITE(UnitId) WriteArraySp                                  
            DEALLOCATE(WriteArraySp)                                    
         ELSE                                                           
            !-----------------------------------------------------      
            ! Write as normal precision                                 
            !-----------------------------------------------------      
            IF (NHEM.EQ.1) THEN                                         
               WRITE(UnitId) RKOUNT,RNTAPE,DAY,DOY,REAL(JG+1),          
     &              WriteArray,RNTAPE                                   
            ELSE                                                        
               !-----------------------------------------------------   
               ! The j-index for the 2nd hemisphere needs switching     
               !-----------------------------------------------------   
               WRITE(UnitId) RKOUNT,RNTAPE,DAY,DOY,REAL(JG+1),          
     &              ( (WriteArray(J,1,L),J=1,JG),                       
     &              (WriteArray(J,NHEM,L),J=JG,1,-1),L=1,nLevels ),     
     &              RNTAPE                                              
            ENDIF                                                       
         ENDIF                                                          
         DEALLOCATE(WriteArray)                                         
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Write grid-point variables to file. Using a binary code for iType to  
! decide which variables are needed:                                    
!   - +1: TSTAR, TDEEP, ... and SNET                                    
!   - +2: ASSBL, ASHBL, ... and ACLD                                    
!   - +4: SSBL, SHBL, ... and CLD                                       
!   - +8: TNLG                                                          
!=====================================================                  
      SUBROUTINE WRITEGRID(UnitId, iType, RKOUNT, RNTAPE, DAY, DOY)     
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Radiation variables                                                   
!-----------------------------------------------------                  
      COMMON/CPIERS/ICFLAG,CFRAC,PNET,SNET,RRFLUX,TTSW,TTLW             
      INTEGER :: ICFLAG(MGx,NHEM,5,2)                                   
      REAL :: CFRAC(MGx,NHEM,5)                                         
      REAL :: PNET(MGx,NHEM,JGy)                                        
      REAL :: SNET(MGx,NHEM,JGy)                                        
      REAL :: RRFLUX(MGx,NHEM,JGy,6)                                    
      REAL :: TTSW(MGx,NHEM,NL)                                         
      REAL :: TTLW(MGx,NHEM,NL)                                         
!-----------------------------------------------------                  
! Almost the same as GRIDP2 variables but levels are complete           
! instead of the longitudes.                                            
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA3/UG3,FUG3,VG3,FVG3,TG3,TNLG3,PLG3                   
     &     ,TRAG3,TRANLG3                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FUG3(MGx,NHEM,JGy,NL)                                     
! V_chi, then V (divergent wind)                                        
      REAL :: VG3(MGx,NHEM,JGy,NL)                                      
      REAL :: FVG3(MGx,NHEM,JGy,NL)                                     
! Temperature anomaly (T')                                              
      REAL :: TG3(MGx,NHEM,JGy,NL)                                      
      REAL :: TNLG3(MGx,NHEM,JGy,NL)                                    
! ln p* or p*                                                           
      REAL :: PLG3(MGx,NHEM,JGy)                                        
! Tracer term                                                           
      REAL :: TRAG3(MGx,NHEM,JGy,NL,NTRAC)                              
      REAL :: TRANLG3(MGx,NHEM,JGy,NL,NTRAC)                            
!-----------------------------------------------------                  
! Grid variables for DGRMLT loop                                        
!-----------------------------------------------------                  
      COMMON/GRIDPC/ASSBL,ASHBL,ASLBL,ARRCR,ARRLR                       
     &     ,TTVD,QTVD,TYBL,TXBL,TTRD,TTLR,TTCR,UTVD,VTVD                
     &     ,QTLR,QTCR,SSBL,SHBL,SLBL,RRCR,RRLR                          
     &     ,ARFLUX,ASFLD,ACLD,RFLUX,SFLD,CLD                            
      REAL :: ASSBL(MGx,NHEM,JGy)                                       
      REAL :: ASHBL(MGx,NHEM,JGy)                                       
      REAL :: ASLBL(MGx,NHEM,JGy)                                       
      REAL :: ARRCR(MGx,NHEM,JGy)                                       
      REAL :: ARRLR(MGx,NHEM,JGy)                                       
      REAL :: TTVD(MGx,NHEM,NL)                                         
      REAL :: QTVD(MGx,NHEM,NL)                                         
      REAL :: TYBL(MGx,NHEM)                                            
      REAL :: TXBL(MGx,NHEM)                                            
      REAL :: TTRD(MGx,NHEM,NL)                                         
      REAL :: TTLR(MGx,NHEM,NL)                                         
      REAL :: TTCR(MGx,NHEM,NL)                                         
      REAL :: UTVD(MGx,NHEM,NL)                                         
      REAL :: VTVD(MGx,NHEM,NL)                                         
      REAL :: QTLR(MGx,NHEM,NL)                                         
      REAL :: QTCR(MGx,NHEM,NL)                                         
      REAL :: SSBL(MGx,NHEM,JGy)                                        
      REAL :: SHBL(MGx,NHEM,JGy)                                        
      REAL :: SLBL(MGx,NHEM,JGy)                                        
      REAL :: RRCR(MGx,NHEM,JGy)                                        
      REAL :: RRLR(MGx,NHEM,JGy)                                        
      REAL :: arflux(MGx,NHEM,JGy,6)                                    
      REAL :: asfld(MGx,NHEM,JGy,6)                                     
      REAL :: acld(MGx,NHEM,JGy,4)                                      
      REAL :: rflux(MGx,NHEM,JGy,6)                                     
      REAL :: sfld(MGx,NHEM,JGy,6)                                      
      REAL :: cld(MGx,NHEM,JGy,4)                                       
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
C OASIS size parameters                                                 
      INTEGER, PARAMETER :: IMB = 128                                   
      INTEGER, PARAMETER :: JM = 64                                     
      INTEGER, PARAMETER :: IMJM = IMB*JM                               
      common/oasisa2o/swlw(imjm),heatlat(imjm),water(imjm)              
     $   ,sensible(imjm),taux(imjm),tauy(imjm),exwater(imjm)            
     $   ,exwat(MGx,NHEM,JGy),sensible2(imjm),sensible3(imjm)           
     $   ,oceaniceflux(MGx,NHEM),exiwat(MGx,NHEM,JGy),exiwater(imjm)    
     $   ,exiwatn(MGx,NHEM,JGy),exiwatern(imjm)                         
     $   ,tstaroo(MGx,NHEM,JGy),tstarf(MGx,NHEM,JGy)                    
     $   ,qstarf(MGx,NHEM,JGy),sqstarf(MGx,NHEM,JGy)                    
     $   ,swlw2d(MGx,NHEM,JGy),heatlat2d(MGx,NHEM,JGy)                  
     $   ,water2d(MGx,NHEM,JGy)                                         
     $   ,sensible2d(MGx,NHEM,JGy),taux2d(MGx,NHEM,JGy)                 
     $   ,tauy2d(MGx,NHEM,JGy)                                          
     $   ,SHBLC(MGx,NHEM,JGy),SLBLC(MGx,NHEM,JGy)                       
     $   ,exwater2d(MGx,NHEM,JGy),sensible2_2d(MGx,NHEM,JGy)            
     $   ,sensible3_2d(MGx,NHEM,JGy)                                    
     $   ,exiwater2d(MGx,NHEM,JGy),exiwatern2d(MGx,NHEM,JGy)            
      real swlw,heatlat,water,sensible,taux,tauy,shblc,slblc,exwater    
     $,exwat,sensible2,sensible3,oceaniceflux,exiwat,exiwater           
     $,exiwatn,exiwatern,tstaroo,tstarf,qstarf,sqstarf                  
     $,swlw2d,heatlat2d,water2d,sensible2d,taux2d,tauy2d                
     $,exwater2d,sensible2_2d,sensible3_2d,exiwater2d,exiwatern2d       
                                                                        
      common/wat/water2(imjm), water2_2d(MGx,NHEM,JGy)                  
C Arrays wathbl, watlsl, watsbl are used for balancing water in IGCM4   
C ashbl, aslbl, assbl were used in IGCM3 FORTE but these are already    
C in use in elsewhere in IGCM4 for surface flux diagnostics             
     $   ,wathbl(MGx,NHEM,JGy),watlbl(MGx,NHEM,JGy),watsbl(MGx,NHEM,JGy)
     $   ,sicecov(MGx,NHEM,JGy)                                         
      real water2,water2_2d,wathbl,watlbl,watsbl,sicecov                
                                                                        
      common/oasiso2a/zsst(imjm),zscur(imjm),mscur(imjm)                
     $   ,zsst_2d(MGx,NHEM,JGy)                                         
     $   ,zscur_2d(MGx,NHEM,JGy),mscur_2d(MGx,NHEM,JGy)                 
      real zsst,zsst_2d,zscur,zscur_2d,mscur,mscur_2d                   
!-----------------------------------------------------                  
! Passed variables                                                      
!-----------------------------------------------------                  
! Unit number of file                                                   
      INTEGER :: UnitId                                                 
! The list of variables to write out is deterimed by choice of iType    
      INTEGER :: iType                                                  
      REAL :: RKOUNT                                                    
      REAL :: RNTAPE                                                    
      REAL :: DAY                                                       
      REAL :: DOY                                                       
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
! The number of 2D fields                                               
      INTEGER, PARAMETER :: N2DFLD=21                                   
! Stores all the data that needs passing to PE 0                        
      REAL, ALLOCATABLE :: StoreArray(:,:,:,:)                          
! On PE 0 the data from the StoreArray arrays from all PEs is fitted    
! into this.                                                            
      REAL, ALLOCATABLE :: WriteArray(:,:,:,:)                          
! The number of levels in StoreArray and WriteArray                     
      INTEGER :: nLevels                                                
! Index for levels                                                      
      INTEGER :: LL                                                     
! Loop variables                                                        
      INTEGER :: J                                                      
!-----------------------------------------------------                  
! Check the type makes sense                                            
!-----------------------------------------------------                  
      IF ( (iType.NE.1).AND.(iType.NE.10).AND.(iType.NE.14)             
     &     .AND.(iType.NE.16) ) THEN                                    
         !-----------------------------------------------------         
         ! This type is not currently allowed                           
         !-----------------------------------------------------         
         IF (MyPe.EQ.0) WRITE(6,*) 'ERROR: this type is not currently ',
     &        'allowed: iType=', iType                                  
         CALL ABORT                                                     
      ENDIF                                                             
!-----------------------------------------------------                  
! Work out how many levels are needed                                   
!-----------------------------------------------------                  
      nLevels=0                                                         
      IF (MOD(iType,2).EQ.1) THEN                                       
C         nLevels=nLevels+11                                            
         nLevels=nLevels+17                                             
      ENDIF                                                             
      IF (MOD(iType,4).GE.2) THEN                                       
         nLevels=nLevels+N2DFLD                                         
      ENDIF                                                             
      IF (MOD(iType,8).GE.4) THEN                                       
         nLevels=nLevels+N2DFLD                                         
      ENDIF                                                             
      IF (MOD(iType,16).GE.8) THEN                                      
         nLevels=nLevels+NL                                             
      ENDIF                                                             
      ALLOCATE(StoreArray(MGx,NHEM,JGy,nLevels))                        
!-----------------------------------------------------                  
! Storing surface variables: TSTAR, TDEEP, ... and SNET                 
!-----------------------------------------------------                  
      LL=0                                                              
      IF (MOD(iType,2).EQ.1) THEN                                       
         StoreArray(:,:,:,LL+1)=TSTAR(:,:,:)                            
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=TDEEP(:,:,:)                            
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=SMSTAR(:,:,:)                           
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=QSTAR(:,:,:)                            
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=HSNOW(:,:,:)                            
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=SQSTAR(:,:,:)                           
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=SALB(:,:,:)                             
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=SBAL(:,:,:)                             
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=TSTARO(:,:,:)                           
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=TDEEPO(:,:,:)                           
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=SNET(:,:,:)                             
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=TSTAROO(:,:,:)                          
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=TSTARF(:,:,:)                           
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=zsst_2d(:,:,:)                          
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=SICECOV(:,:,:)                          
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=zscur_2d(:,:,:)                         
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=mscur_2d(:,:,:)                         
         LL=LL+1                                                        
      ENDIF                                                             
!-----------------------------------------------------                  
! Storing grid variables from DGRMLT: ASSBL, ASHBL, ... and ACLD        
!-----------------------------------------------------                  
      IF (MOD(iType,4).GE.2) THEN                                       
         StoreArray(:,:,:,LL+1)=ASSBL(:,:,:)                            
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=ASHBL(:,:,:)                            
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=ASLBL(:,:,:)                            
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=ARRCR(:,:,:)                            
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=ARRLR(:,:,:)                            
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1:LL+6)=ARFLUX(:,:,:,:)                    
         LL=LL+6                                                        
         StoreArray(:,:,:,LL+1:LL+6)=ASFLD(:,:,:,:)                     
         LL=LL+6                                                        
         StoreArray(:,:,:,LL+1:LL+4)=ACLD(:,:,:,:)                      
         LL=LL+4                                                        
      ENDIF                                                             
!-----------------------------------------------------                  
! Storing grid variables from DGRMLT: SSBL, SHBL, ... and CLD           
!-----------------------------------------------------                  
      IF (MOD(iType,8).GE.4) THEN                                       
         StoreArray(:,:,:,LL+1)=SSBL(:,:,:)                             
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=SHBL(:,:,:)                             
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=SLBL(:,:,:)                             
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=RRCR(:,:,:)                             
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1)=RRLR(:,:,:)                             
         LL=LL+1                                                        
         StoreArray(:,:,:,LL+1:LL+6)=RFLUX(:,:,:,:)                     
         LL=LL+6                                                        
         StoreArray(:,:,:,LL+1:LL+6)=SFLD(:,:,:,:)                      
         LL=LL+6                                                        
         StoreArray(:,:,:,LL+1:LL+4)=CLD(:,:,:,:)                       
         LL=LL+4                                                        
      ENDIF                                                             
!-----------------------------------------------------                  
! Storing TNLG                                                          
!-----------------------------------------------------                  
      IF (MOD(iType,16).GE.8) THEN                                      
         StoreArray(:,:,:,LL+1:LL+NL)=TNLG3(1:MGx,:,:,:)                
         LL=LL+NL                                                       
      ENDIF                                                             
!-----------------------------------------------------                  
! Get all the data to PE 0                                              
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) ALLOCATE(WriteArray(MGPP,NHEM,JG,nLevels))         
      CALL SHAREGRID(.FALSE.,MGPP,nLevels,WriteArray,StoreArray)        
      DEALLOCATE(StoreArray)                                            
      IF (MyPe.EQ.0) THEN                                               
         !-----------------------------------------------------         
         ! Wrap around                                                  
         !-----------------------------------------------------         
         WriteArray(MG+1,:,:,:)=WriteArray(1,:,:,:)                     
         WriteArray(MGPP,:,:,:)=WriteArray(2,:,:,:)                     
      ENDIF                                                             
!-----------------------------------------------------                  
! Write data to file                                                    
!-----------------------------------------------------                  
      IF (MyPe.EQ.0) THEN                                               
         IF (iType.EQ.1) THEN                                           
            WRITE(UnitId) -999.999                                      
            WRITE(UnitId) RKOUNT,RNTAPE,DAY,DOY,WriteArray,RNTAPE       
         ELSE                                                           
            !-----------------------------------------------------      
            ! Loop through latitudes                                    
            !-----------------------------------------------------      
            DO J=1,JG                                                   
               !-----------------------------------------------------   
               ! Write data to file as single precision                 
               !-----------------------------------------------------   
               WRITE(UnitId) REAL(RKOUNT,KIND=4),REAL(RNTAPE,KIND=4),   
     &              REAL(DAY,KIND=4),REAL(DOY,KIND=4),REAL(J,KIND=4),   
     &              REAL(WriteArray(:,:,J,1:nLevels-NL),KIND=4),        
     &              REAL(RNTAPE,KIND=4),                                
     &              REAL(WriteArray(:,:,J,nLevels-NL+1:nLevels),KIND=4),
     &              REAL(RNTAPE,KIND=4)                                 
            ENDDO                                                       
         ENDIF                                                          
         DEALLOCATE(WriteArray)                                         
      ENDIF                                                             
      END                                                               
!=====================================================                  
! Prints spectral coefficients                                          
!=====================================================                  
      SUBROUTINE WRSPS(DESCRIPTION,A,IA,nLevs)                          
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Switches counters and constants controlling type and frequency of     
! model output                                                          
!-----------------------------------------------------                  
      COMMON/OUTCON/RNTAPE,DAY,SQR2,RSQR2,EAM1,EAM2,TOUT1               
     &     ,TOUT2,LSPO,LGPO                                             
     &     ,RNTAPO                                                      
     &     ,LSHIST,LMINIH                                               
      COMMON/OUTCONINT/NCOEFF,NLAT,INLAT,INSPC,KOUNTP                   
     &                 ,KOUNTE,KOUNTH,KOUNTR,KOUTP                      
     &                 ,KOUTE,KOUTH,KOUTR                               
! Identyfying run number                                                
      REAL :: RNTAPE                                                    
! Maximum wavenumber of printed spectral coeffecients                   
      INTEGER :: NCOEFF                                                 
! Number of latitudes in printed grid point fields                      
      INTEGER :: NLAT                                                   
      INTEGER :: INLAT                                                  
!                                                                       
      INTEGER :: INSPC                                                  
! Number of timsteps between successive writing of spectral             
! and gridded fields                                                    
      INTEGER :: KOUNTP                                                 
      INTEGER :: KOUNTE                                                 
! Number of timsteps between successive writing of history              
! records, containing spectral fields at the present and                
! previous time levels.                                                 
      INTEGER :: KOUNTH                                                 
! Number of timsteps between successive writing of restart              
! records, containing the spectral fields at the present and            
! previous time levels                                                  
      INTEGER :: KOUNTR                                                 
      INTEGER :: KOUTP                                                  
      INTEGER :: KOUTE                                                  
      INTEGER :: KOUTH                                                  
      INTEGER :: KOUTR                                                  
      REAL :: DAY                                                       
      REAL :: SQR2                                                      
      REAL :: RSQR2                                                     
      REAL :: EAM1                                                      
      REAL :: EAM2                                                      
      REAL :: TOUT1                                                     
      REAL :: TOUT2                                                     
      LOGICAL :: LSPO(NL)                                               
      LOGICAL :: LGPO(NL)                                               
      REAL :: RNTAPO                                                    
      LOGICAL :: LSHIST                                                 
! Reduce the history file to single precision if true                   
      LOGICAL :: LMINIH                                                 
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variables                                                      
!-----------------------------------------------------                  
      CHARACTER*(80) :: DESCRIPTION                                     
      COMPLEX :: A(NNTx,MMTy,NHEM,nLevs)                                
      COMPLEX :: B(NWJ2,NHEM,nLevs)                                     
      INTEGER :: IA                                                     
      INTEGER :: nLevs                                                  
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      COMPLEX, ALLOCATABLE :: HoldVar(:,:,:,:)                          
      CHARACTER COEFF(NWJ2,2)*8                                         
! External function                                                     
      COMPLEX :: POLAR                                                  
      INTEGER, PARAMETER :: MJPm=NWJ2+NWJ2                              
      INTEGER :: FP(NWJ2*NHEM)                                          
      INTEGER :: IP(NWJ2*NHEM)                                          
      INTEGER :: I, IBEG, IG                                            
      INTEGER, PARAMETER :: TagRef=26100                                
      INTEGER :: iPe                                                    
! Loop variables                                                        
      INTEGER :: IHEM, IM, IN, JP, L, MP                                
!-----------------------------------------------------                  
! Store COEFF, FP and IP, so that they only need setting once           
!-----------------------------------------------------                  
      SAVE COEFF,FP,IP                                                  
      DATA COEFF/MJPm*' (  ,  )'/                                       
      IG=0                                                              
      I=0                                                               
      DO MP=0,MM-1,MOCT                                                 
         !-----------------------------------------------------         
         ! Only print if MP<NCOEFF                                      
         !-----------------------------------------------------         
         IF (MP.GE.NCOEFF) RETURN                                       
                                                                        
         IBEG=IG                                                        
         DO JP=MP,NCOEFF-1,MH                                           
            IG=IG+1                                                     
            I=I+1                                                       
            IP(I)=IG                                                    
            IF (MP.EQ.0) THEN                                           
               FP(I)=1                                                  
            ELSE                                                        
               FP(I)=2                                                  
            ENDIF                                                       
            WRITE(COEFF(I,1)(3:4),'(I2)')MP                             
            WRITE(COEFF(I,2)(3:4),'(I2)')MP                             
            WRITE(COEFF(I,1)(6:7),'(I2)')JP+1                           
            WRITE(COEFF(I,2)(6:7),'(I2)')JP                             
         ENDDO                                                          
         IG=IBEG+(NN-MP+1)/MH                                           
      ENDDO                                                             
      RETURN                                                            
!-----------------------------------------------------                  
! By calling WRSPA the computation begins here and initialisation       
! above isn't needed.                                                   
!-----------------------------------------------------                  
      ENTRY WRSPA(DESCRIPTION,A,IA,nLevs)                               
!-----------------------------------------------------                  
! Only interested in the data stored in IHEM=1                          
!-----------------------------------------------------                  
      ALLOCATE(HoldVar(NNTx,MMTy,NHEM,nLevs))                           
      HoldVar(:,:,:,:)=A(:,:,:,:)                                       
!-----------------------------------------------------                  
! Put coefficient together                                              
!-----------------------------------------------------                  
      IF (MyPe.GT.0) THEN                                               
         !-----------------------------------------------------         
         ! Send coefficients to PE 0                                    
         !-----------------------------------------------------         
         iRecv=MyPe                                                     
         nBuffSize=NNTx*MMTy*NHEM*nLevs                                 
         iTag=TagRef+MyPe                                               
         CALL MPI_BSEND(HoldVar, nBuffSize, ParaCom, iRecv,             
     &        iTag, MPI_COMM_WORLD, ierr)                               
      ELSE                                                              
         !-----------------------------------------------------         
         ! Loop through PE that aren't PE 0                             
         !-----------------------------------------------------         
         DO iPe=1,NPE-1                                                 
            !-----------------------------------------------------      
            ! Receive data                                              
            !-----------------------------------------------------      
            iSend=iPe                                                   
            nBuffSize=NNTx*MMTy*NHEM*nLevs                              
            iTag=TagRef+iPe                                             
            CALL MPI_RECV(HoldVar, nBuffSize, ParaCom, iSend,           
     &           iTag, MPI_COMM_WORLD, istat, ierr)                     
            !-----------------------------------------------------      
            ! Write data into array                                     
            !-----------------------------------------------------      
            DO L=1,nLevs                                                
               DO IHEM=1,NHEM                                           
                  DO IM=1,MMTyy(iRow)                                   
                     DO IN=1,NNTxx(iCol)                                
                        B(CoefConv(IN,IM,iPe),IHEM,L)=                  
     &                       HoldVar(IN,IM,IHEM,L)                      
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Write coefficients on PE 0 into array B                      
         !-----------------------------------------------------         
         DO L=1,nLevs                                                   
            DO IHEM=1,NHEM                                              
               DO IM=1,MMTyy(iRow)                                      
                  DO IN=1,NNTxx(iCol)                                   
                     B(CoefConv(IN,IM,0),IHEM,L)=HoldVar(IN,IM,IHEM,L)  
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Write coefficients to output file                            
         !-----------------------------------------------------         
         IF (NHEM.EQ.1) THEN                                            
            DO L=1,nLevs                                                
               IF (LSPO(L)) THEN                                        
                  IF (nLevs.GT.1) THEN                                  
                     WRITE(2, *) TRIM(DESCRIPTION),L                    
                  ELSE                                                  
                     WRITE(2, *) TRIM(DESCRIPTION)                      
                  ENDIF                                                 
                  WRITE(2,1000)(COEFF(I,IA),POLAR(B(IP(I),1,L)*FP(I)),  
     &                 I=1,INSPC)                                       
               ENDIF                                                    
            ENDDO                                                       
         ELSE                                                           
            DO L=1,nLevs                                                
               IF (LSPO(L)) THEN                                        
                  IF (nLevs.GT.1) THEN                                  
                     WRITE(2, *) TRIM(DESCRIPTION),L                    
                  ELSE                                                  
                     WRITE(2, *) TRIM(DESCRIPTION)                      
                  ENDIF                                                 
                  WRITE(2,1000)(COEFF(I,2),POLAR(B(IP(I),3-IA,L)        
     &                 *FP(I)),COEFF(I,1),                              
     &                 POLAR(B(IP(I),IA,L)*FP(I)),I=1,INSPC)            
               ENDIF                                                    
            ENDDO                                                       
         ENDIF                                                          
      ENDIF                                                             
 1000 FORMAT(3(A8,1X,1PE8.2,0PF7.1))                                    
      RETURN                                                            
      END                                                               
!=====================================================                  
! This subroutine gives quick look sigma-latitude x-sections            
! of [U], [T] and [V*T*].                                               
!=====================================================                  
      SUBROUTINE XSECT(ISKIP)                                           
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Basic planetary parameters for run plus information about             
! vertical grid structure                                               
!-----------------------------------------------------                  
      COMMON        SQ,RSQ,SIGMAH,SIGMA,T01S2,T0,ALPHA,DSIGMA           
     &     ,RDSIG,TKP,C,AKAP,GA,GASCON,RADEA,WW,PFAC                    
     &     ,P0,RD,RV,CPD,CLATNT                                         
      COMMON/BLANKCOM/EZ,AIOCT                                          
      COMMON/BLANKLOG/LRSTRT,LSHORT,LSTRETCH,LBALAN,LRESTIJ,LNOISE      
     &     ,LFLUX                                                       
     &     ,LCLIM, LPERPET, L22L, LOROG ,LCSFCT                         
! SQ(n+1) =n (n+1)                                                      
      REAL :: SQ(NNP)                                                   
! RSQ(n+1) = 1/(n(n+1)) for n>0 and RSQ(1)=0 for n=0                    
      REAL :: RSQ(NNP)                                                  
! Sigma at half levels (r+1/2) for r=1 to NL-1                          
      REAL :: SIGMAH(NL-1)                                              
! Sigma at full levels for r=1 to NL                                    
      REAL :: SIGMA(NL)                                                 
! (Mean of temperature at r+1) - (Mean of temperature at r)             
      REAL :: T01S2(NL-1)                                               
! Mean of temperature at r for r=1 to NL                                
      REAL :: T0(NL)                                                    
! ln( (sigma at r+1/2) / (sigma at r-1/2) ) (no longer used at r=1)     
      REAL :: ALPHA(NL)                                                 
! (sigma at r+1/2) - (sigma at r-1/2) for r=1 to NL                     
      REAL :: DSIGMA(NL)                                                
! RDSIG(r) = 1 / (2*DSIGMA(r)) for r=1 to NL                            
      REAL :: RDSIG(NL)                                                 
! Kappa * (mean Temperature at r) for r=1 to NL                         
      REAL :: TKP(NL)                                                   
! NL by NL matrix, C_rs=(DSIGMA(s)/DSIGMA(r)) * g_sr                    
      REAL :: C(NL,NL)                                                  
! Kappa=R/C_p for dry air                                               
      REAL :: AKAP                                                      
! Gravitational acceleration                                            
      REAL :: GA                                                        
! R, gas constant for dry air                                           
      REAL :: GASCON                                                    
! Earth's radius                                                        
      REAL :: RADEA                                                     
! Earth's rotation rate                                                 
      REAL :: WW                                                        
! 0.5*(a*Omega)^2 p_0/g, used in MSPECT                                 
      REAL :: PFAC                                                      
! sqrt(8/3)                                                             
      COMPLEX :: EZ                                                     
! i*MOCT, where i = sqrt(-1)                                            
      COMPLEX :: AIOCT                                                  
! True: restart run from stored spectral data;                          
! false: initial run with bal'                                          
      LOGICAL :: LRSTRT                                                 
! True: restart with short timesteps;                                   
! false: restart without short timesteps                                
      LOGICAL :: LSHORT                                                 
! Strecth vertical coordinate if true                                   
      LOGICAL :: LSTRETCH                                               
! Carry out balancing of vorticity from temperate perturbation or       
! vice versa if true                                                    
      LOGICAL :: LBALAN                                                 
! Use restoration fields if true                                        
      LOGICAL :: LRESTIJ                                                
! Add white noise to surface pressure if true                           
      LOGICAL :: LNOISE                                                 
      LOGICAL :: LFLUX                                                  
      REAL :: P0                                                        
      REAL :: RD                                                        
      REAL :: RV                                                        
      REAL :: CPD                                                       
      REAL :: CLATNT                                                    
      LOGICAL :: LCLIM                                                  
      LOGICAL :: LPERPET                                                
      LOGICAL :: L22L                                                   
      LOGICAL :: LOROG                                                  
      LOGICAL :: LCSFCT                                                 
!-----------------------------------------------------                  
! Constant arrays and variables associated with time and vertical       
! differencing. Also counters.                                          
!-----------------------------------------------------                  
      COMMON/BATS/  BM1,AK,AQ,G,TAU,BEGDAY,PNU21                        
     &     ,DELT,DELT2,CV,CG,CT,PNU,PNU2                                
     &     ,CQ,CTRA,RGG,BEGDOY,DOY                                      
      COMMON/BATSINT/  KOUNT,KITS,KTOTAL,KRUN,ITSPD                     
     &     ,KSTART,NTRACO,KOLOUR                                        
! Contains inverse of A matrix for calculating divergence               
      REAL :: BM1(NL,NL,NN)                                             
! Non-dimensional dissipation factor for wavenumber n                   
      REAL :: AK(NNP)                                                   
      REAL :: AQ(NL,NL)                                                 
! Defines vertical scheme                                               
      REAL :: G(NL,NL)                                                  
! Used for temperature tendency                                         
      REAL :: TAU(NL,NL)                                                
! Number of timesteps completed                                         
      INTEGER :: KOUNT                                                  
! Number of short timesteps                                             
      INTEGER :: KITS                                                   
! Total number of timestep that need completing                         
      INTEGER :: KTOTAL                                                 
! The last timestep                                                     
      INTEGER :: KRUN                                                   
! Start of day                                                          
      REAL :: BEGDAY                                                    
      INTEGER :: ITSPD                                                  
      REAL :: PNU21                                                     
! Size of timestep                                                      
      REAL :: DELT                                                      
! 2*DELT                                                                
      REAL :: DELT2                                                     
      REAL :: CV                                                        
      REAL :: CG                                                        
      REAL :: CT                                                        
      REAL :: PNU                                                       
      REAL :: PNU2                                                      
      INTEGER :: KSTART                                                 
      REAL :: CQ                                                        
      INTEGER :: NTRACO                                                 
      REAL :: CTRA(NTRAC)                                               
      INTEGER :: KOLOUR(NTRAC)                                          
      REAL :: RGG(NL,NL)                                                
      REAL :: BEGDOY,DOY                                                
!-----------------------------------------------------                  
! Similar to variables in GRIDP but in grid space, and without          
! equivalent variables for CHIG, SFG, FVGT and FUGT. The levels         
! are divided across the columns of PEs; the latitudes are divided      
! across the row of PEs; and the zonal wavenumbers/longitudes are       
! complete.                                                             
! Common block GRIDPA contains the variables just needed in both        
! MGRMLT and DGRMLT (for version > 1) loops.                            
!-----------------------------------------------------                  
      COMMON/GRIDPA2/UG2,FUG2,VG2,FVG2,TG2,TNLG2,PLG2                   
     &     ,TRAG2,TRANLG2                                               
! U_psi, then U (rotational wind)                                       
      REAL :: UG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FUG2(MGPP,NHEM,JGy,NLx)                                   
! V_chi, then V (divergent wind)                                        
      REAL :: VG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: FVG2(MGPP,NHEM,JGy,NLx)                                   
! Temperature anomaly (T')                                              
      REAL :: TG2(MGPP,NHEM,JGy,NLx)                                    
      REAL :: TNLG2(MGPP,NHEM,JGy,NLx)                                  
! ln p* or p*                                                           
      REAL :: PLG2(MGPP,NHEM,JGy)                                       
! Tracer term                                                           
      REAL :: TRAG2(MGPP,NHEM,JGy,NLx,NTRAC)                            
      REAL :: TRANLG2(MGPP,NHEM,JGy,NLx,NTRAC)                          
!-----------------------------------------------------                  
! Legendre polynomials and information about gaussian latitudes         
!-----------------------------------------------------                  
      COMMON/LEGAU/ ALP,DALP,RLP,RDLP,SI,CS,                            
     &     CSSQ,SECSQ,ALAT,AW                                           
! Legendre function, P^m_n                                              
      REAL :: ALP(NNT,MMTy,2,JG)                                        
! (1 - mu^2)(d(P^m_n)/d mu)                                             
      REAL :: DALP(NNT,MMTy,2,JG)                                       
      REAL :: RLP(NNT,MMTy,2,JG)                                        
      REAL :: RDLP(NNT,MMTy,2,JG)                                       
! mu = sin (latitude)                                                   
      REAL :: SI(JG,NHEM)                                               
! (1 - mu^2) = cos (latitude)                                           
      REAL :: CS(JG,NHEM)                                               
! (cos(mu))^2                                                           
      REAL :: CSSQ(JG,NHEM)                                             
! 1 / (cos(mu))^2                                                       
      REAL :: SECSQ(JG,NHEM)                                            
      REAL :: AW(JG,NHEM)                                               
! Latitude in degrees (needed by XSECT)                                 
      REAL :: ALAT(JG,NHEM)                                             
!-----------------------------------------------------                  
! Switches counters and constants controlling type and frequency of     
! model output                                                          
!-----------------------------------------------------                  
      COMMON/OUTCON/RNTAPE,DAY,SQR2,RSQR2,EAM1,EAM2,TOUT1               
     &     ,TOUT2,LSPO,LGPO                                             
     &     ,RNTAPO                                                      
     &     ,LSHIST,LMINIH                                               
      COMMON/OUTCONINT/NCOEFF,NLAT,INLAT,INSPC,KOUNTP                   
     &                 ,KOUNTE,KOUNTH,KOUNTR,KOUTP                      
     &                 ,KOUTE,KOUTH,KOUTR                               
! Identyfying run number                                                
      REAL :: RNTAPE                                                    
! Maximum wavenumber of printed spectral coeffecients                   
      INTEGER :: NCOEFF                                                 
! Number of latitudes in printed grid point fields                      
      INTEGER :: NLAT                                                   
      INTEGER :: INLAT                                                  
!                                                                       
      INTEGER :: INSPC                                                  
! Number of timsteps between successive writing of spectral             
! and gridded fields                                                    
      INTEGER :: KOUNTP                                                 
      INTEGER :: KOUNTE                                                 
! Number of timsteps between successive writing of history              
! records, containing spectral fields at the present and                
! previous time levels.                                                 
      INTEGER :: KOUNTH                                                 
! Number of timsteps between successive writing of restart              
! records, containing the spectral fields at the present and            
! previous time levels                                                  
      INTEGER :: KOUNTR                                                 
      INTEGER :: KOUTP                                                  
      INTEGER :: KOUTE                                                  
      INTEGER :: KOUTH                                                  
      INTEGER :: KOUTR                                                  
      REAL :: DAY                                                       
      REAL :: SQR2                                                      
      REAL :: RSQR2                                                     
      REAL :: EAM1                                                      
      REAL :: EAM2                                                      
      REAL :: TOUT1                                                     
      REAL :: TOUT2                                                     
      LOGICAL :: LSPO(NL)                                               
      LOGICAL :: LGPO(NL)                                               
      REAL :: RNTAPO                                                    
      LOGICAL :: LSHIST                                                 
! Reduce the history file to single precision if true                   
      LOGICAL :: LMINIH                                                 
!-----------------------------------------------------                  
! Restoration fields and timescale                                      
!-----------------------------------------------------                  
      COMMON/RESTOR/ZRES,DRES,TRES,SPRES,DAMP                           
      REAL :: ZRES(IDM,NHEM,NL)                                         
      REAL :: DRES(IDM,NHEM,NL)                                         
      REAL :: TRES(IDM,NHEM,NL)                                         
      REAL :: SPRES(IDM,NHEM)                                           
      REAL :: DAMP                                                      
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variable                                                       
!-----------------------------------------------------                  
      INTEGER, INTENT(IN) :: ISKIP                                      
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: TagRef=26200                                
      INTEGER :: SLICE(JGy,NHEM,NLx,3)                                  
      INTEGER :: SliceFull(JG,NHEM,NL,3)                                
      INTEGER :: HOLD(JGy,NHEM,NLx)                                     
      REAL :: SEC                                                       
      REAL :: TB, UB, VB                                                
      REAL :: VT                                                        
      REAL :: RMG                                                       
      INTEGER :: ISLT                                                   
      INTEGER :: IX, J0, JJ                                             
      INTEGER :: J                                                      
      INTEGER :: iCol2, iRow2                                           
! Loop variables                                                        
      INTEGER :: I, IHEM, iPe, J2, L                                    
!-----------------------------------------------------                  
! Set ISLT, The default being to have 16 I5 integers per level.         
!-----------------------------------------------------                  
      IF (ISKIP.EQ.0) THEN                                              
         RETURN                                                         
      ELSE IF (ISKIP.GT.0) THEN                                         
         ISLT = ISKIP                                                   
      ELSE                                                              
         ISLT = (NHEM*JG-1)/18 + 1                                      
      ENDIF                                                             
!-----------------------------------------------------                  
! Output is wanted. Read grid point fields from stream 24               
!-----------------------------------------------------                  
      RMG=1./REAL(MG)                                                   
      IF (MyPe.EQ.0) WRITE (2,104) DAY,DOY                              
  104 FORMAT(/' CROSS SECTIONS FOR DAY, DOY',2F6.1)                     
!-----------------------------------------------------                  
! Loop through latitudes                                                
!-----------------------------------------------------                  
      DO J=1,JGyy(iRow)                                                 
         !-----------------------------------------------------         
         ! Loop over latitudes to read gridpoint data, dimensionalise   
         ! and calculate zonal means                                    
         !-----------------------------------------------------         
         SEC=ALAT(JJ1(iRow)+J,1)/57.29578                               
         SEC=10.*CV/COS(SEC)                                            
         !-----------------------------------------------------         
         ! Loop for [U] and [T] sections                                
         !-----------------------------------------------------         
         DO L=1,NLxx(iCol)                                              
            DO IHEM=1,NHEM                                              
               UB=0.                                                    
               DO I=1,MG                                                
                  UB=UB+UG2(I,IHEM,J,L)                                 
               ENDDO                                                    
               UB=UB*RMG                                                
               TB=0.                                                    
               DO I=1,MG                                                
                  TB=TB+TG2(I,IHEM,J,L)                                 
               ENDDO                                                    
               TB=TB*RMG+T0(L)                                          
                                                                        
               SLICE(J,IHEM,L,1)=NINT(UB*SEC)                           
               SLICE(J,IHEM,L,2)=NINT(TB*CT)                            
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Loop for temperature flux section                            
         !-----------------------------------------------------         
         DO L=1,NLxx(iCol)                                              
            DO IHEM=1,NHEM                                              
               VT=0.                                                    
               DO I=1,MG                                                
                  VT=VT+TG2(I,IHEM,J,L)*VG2(I,IHEM,J,L)                 
               ENDDO                                                    
               VT=RMG*VT                                                
               TB=0.                                                    
               DO I=1,MG                                                
                  TB=TB+TG2(I,IHEM,J,L)                                 
               ENDDO                                                    
               TB=TB*RMG                                                
               VB=0.                                                    
               DO I=1,MG                                                
                  VB=VB+VG2(I,IHEM,J,L)                                 
               ENDDO                                                    
               VB=VB*RMG                                                
               SLICE(J,IHEM,L,3)=NINT((VT-VB*TB)*SEC*CT)                
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
!-----------------------------------------------------                  
! Pass data to PE 0                                                     
!-----------------------------------------------------                  
      IF (MyPe.GT.0) THEN                                               
         !-----------------------------------------------------         
         ! Send arrays to PE 0                                          
         !-----------------------------------------------------         
         iRecv=0                                                        
         nBuffSize=JGy*NHEM*NLx*3                                       
         iTag=TagRef+MyPe                                               
         CALL MPI_BSEND(SLICE, nBuffSize, ParaInt, iRecv,               
     &        iTag, MPI_COMM_WORLD, ierr)                               
      ELSE                                                              
         DO iPe=0,NPE-1                                                 
            IF (iPe.GT.0) THEN                                          
               !-----------------------------------------------------   
               ! Receive arrays                                         
               !-----------------------------------------------------   
               iSend=iPe                                                
               nBuffSize=JGy*NHEM*NLx*3                                 
               iTag=TagRef+iPe                                          
               CALL MPI_RECV(SLICE, nBuffSize, ParaInt, iSend,          
     &              iTag, MPI_COMM_WORLD, istat, ierr)                  
            ENDIF                                                       
            !-----------------------------------------------------      
            ! Write them into full size arrays                          
            !-----------------------------------------------------      
            iCol2=GRID(1,iPe)                                           
            iRow2=GRID(2,iPe)                                           
            DO I=1,3                                                    
               DO L=1,NLxx(iCol2)                                       
                  DO IHEM=1,NHEM                                        
                     DO J=1,JGyy(iRow2)                                 
                        SliceFull(JJ1(iRow2)+J,IHEM,KK1(iCol2)+L,I)=    
     &                       SLICE(J,IHEM,L,I)                          
                     ENDDO                                              
                  ENDDO                                                 
               ENDDO                                                    
            ENDDO                                                       
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Printing sections - separate loop for each field.            
         ! First the zonal wind                                         
         !-----------------------------------------------------         
         WRITE (2,100)                                                  
 100     FORMAT(' ZONAL WIND IN 0.1 m/s')                               
         DO L=1,NL                                                      
            WRITE(2,101)((SliceFull(J+(IHEM-1)*(JG-2*J+1),IHEM,L,1),    
     &           J=1,JG,ISLT),IHEM=1,NHEM)                              
         ENDDO                                                          
 101     FORMAT(18I4)                                                   
         !-----------------------------------------------------         
         ! Secondly the temperature                                     
         !-----------------------------------------------------         
         WRITE(2,102)                                                   
 102     FORMAT(' TEMPERATURE IN K')                                    
         DO L=1,NL                                                      
            WRITE(2,101)((SliceFull(J+(IHEM-1)*(JG-2*J+1),IHEM,L,2),    
     &           J=1,JG,ISLT),IHEM=1,NHEM)                              
         ENDDO                                                          
         !-----------------------------------------------------         
         ! Thirdly temperature flux                                     
         !-----------------------------------------------------         
         WRITE(2,103)                                                   
 103     FORMAT(' POLEWARD TEMPERATURE FLUX IN 0.1Km/s')                
         DO L=1,NL                                                      
            WRITE(2,101)((SliceFull(J+(IHEM-1)*(JG-2*J+1),IHEM,L,3),    
     &           J=1,JG,ISLT),IHEM=1,NHEM)                              
         ENDDO                                                          
      ENDIF                                                             
      RETURN                                                            
      END                                                               
!=====================================================                  
! Function used in CUBM and CUDIF                                       
!=====================================================                  
      FUNCTION FTSL(T,Q,P)                                              
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Variables/arrays for the parametrizations                             
!-----------------------------------------------------                  
       COMMON/PHYS/LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                     
     &     ,CD,DRAG,BLVAD,BLA,BLRH,BLVB                                 
     &     ,AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                  
     &     ,CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2               
     &     ,NLCR,NCUTOP,CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP             
     &     ,NAVRD,NAVWT,DELT2C                                          
     &     ,FRAD,SKAP,SK,FWS,CLR,FB                                     
     &     ,TSTAR,QSTAR                                                 
     &     ,TTDC,QTDC,TTMC,QTMC,TC,QC                                   
     &     ,CTCR,CTLR                                                   
     &     ,LSL,LOC,LNOICE,LOLDBL,LCOND,LNNSK                           
     &     ,ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw       
     &     ,ssmc,sdsnd,sasnow,saice,shsstar,shsmax                      
     &     ,SHCO,SHCI,ITSLL,ITSLO                                       
     &     ,TSTARO,TDEEPO,smstar                                        
     &     ,tdeep,hsnow,sqstar                                          
     &     ,SALB,SBAL,BLCD,SVEGE                                        
      LOGICAL :: LBL,LVD,LCR,LLR,LRD,LCUBM,LCBADJ                       
      REAL :: CD,DRAG,BLVAD,BLA,BLRH                                    
      REAL :: BLVB(MGx,NHEM)                                            
      REAL :: AKVV,AKTV,AKQV,ESCONA,ESCONB,EPSIQ,CTQ,CCC                
      REAL :: CCR,RCON,DTBUOY,TSLA,TSLB,TSLC,TSLD,CUT1,CUT2             
      INTEGER :: NLCR,NCUTOP                                            
      REAL :: CURHM,AKTC,AKQC,CUBMT,CBADJT,CBADJP                       
      INTEGER :: NAVRD,NAVWT                                            
      REAL :: DELT2C                                                    
      REAL :: TSTAR(MGx,NHEM,JGy)                                       
      REAL :: QSTAR(MGx,NHEM,JGy)                                       
      REAL :: FRAD(JGy,NHEM)                                            
      REAL :: SKAP(NL)                                                  
      REAL :: SK(NL-1),FWS(NL),CLR(NL),FB(NL-1)                         
      REAL :: TTDC(NL),QTDC(NL),TTMC(NL),QTMC(NL),TC(NL),QC(NL)         
      REAL :: CTCR(NL,NHEM),CTLR(NL,NHEM)                               
      LOGICAL :: LSL,LOC, LNOICE,LOLDBL,LCOND,LNNSK                     
      REAL :: ctqi,sdsn,shcs,shcsp,shcsn,skse,sksn,slhf,sd1,sd2,sdw     
      REAL :: ssmc,sdsnd,sasnow,saice,shsstar,shsmax                    
      REAL :: SHCO,SHCI                                                 
      INTEGER :: ITSLL,ITSLO                                            
      REAL :: TSTARO(MGx,NHEM,JGy)                                      
      REAL :: TDEEPO(MGx,NHEM,JGy)                                      
      REAL :: smstar(MGx,NHEM,JGy)                                      
      REAL :: tdeep(MGx,NHEM,JGy)                                       
      REAL :: hsnow(MGx,NHEM,JGy)                                       
      REAL :: sqstar(MGx,NHEM,JGy)                                      
      REAL :: SALB(MGx,NHEM,JGy)                                        
      REAL :: SBAL(MGx,NHEM,JGy)                                        
      REAL :: BLCD(MGx,NHEM)                                            
      REAL :: SVEGE(MGx,NHEM,JGy)                                       
!-----------------------------------------------------                  
! Passed variables                                                      
!-----------------------------------------------------                  
      REAL :: FTSL                                                      
      REAL :: T                                                         
      REAL :: Q                                                         
      REAL :: P                                                         
!-----------------------------------------------------                  
! Define function                                                       
!-----------------------------------------------------                  
      FTSL=TSLA+TSLB/(TSLC*LOG(T)-LOG(Q*P)+TSLD)                        
      END FUNCTION                                                      
!=====================================================                  
! Function that takes the complex number Z and return a complex         
! number that contains the magnitude of Z in the real part and          
! the angle in radians of the arctangent of the complex number.         
!=====================================================                  
      FUNCTION POLAR(Z)                                                 
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Passed variable                                                       
!-----------------------------------------------------                  
      COMPLEX :: POLAR                                                  
      COMPLEX :: Z                                                      
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! One radian                                                            
      REAL, PARAMETER :: RAD=180./PI                                    
!-----------------------------------------------------                  
! Define the function. (Atan2 is a standard Fortran function, which     
! calculates the arctangent of the complex number (X+iY).)              
!-----------------------------------------------------                  
      POLAR=CMPLX(ABS(Z),ATAN2(AIMAG(Z),REAL(Z)+1.0E-20)*RAD)           
      RETURN                                                            
      END FUNCTION                                                      
!=====================================================                  
! Exponential function involving tendency of temperature                
!=====================================================                  
      REAL FUNCTION PQSAT(T)                                            
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Used for PQSAT function                                               
!-----------------------------------------------------                  
      INTEGER, PARAMETER :: NQSTAB=50000                                
      COMMON /QSTABS/ PQSVAL                                            
      REAL :: PQSVAL(NQSTAB)                                            
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      REAL :: T                                                         
!-----------------------------------------------------                  
! Carry out calculation                                                 
!-----------------------------------------------------                  
      PQSAT=PQSVAL(int(T*1e5))                                          
      END                                                               
!=====================================================                  
! Marc 14 Jul 2010                                                      
! The new version of the code creates random number with the            
! routine, random_number, which should be available on any              
! platform - unlike RANF. But we need to keep the old NOISE             
! subroutine to compare results with the old version of the             
! code. Also need the subroutines NEW2OLD and OLD2NEW for               
! this.                                                                 
!                                                                       
! Will need to replace the call to NOISE with call to                   
! NOISEold to use the old method of generating random                   
! numbers.                                                              
!=====================================================                  
C**********************************************************             
C             SUBROUTINE NOISE                                          
C**********************************************************             
      SUBROUTINE NOISEold                                               
C                                                                       
C     Adds white noise perturbation to ln(surface pressure)             
C     balanced initial state at T=0.                                    
C                                                                       
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Spectral coefficients. The meridional wavenumbers are divided         
! across the columns of PEs; the zonal wavenumbers are divided          
! across the row of PEs; the levels are complete.                       
!-----------------------------------------------------                  
      COMMON/SPECTR/Z,D,T,SP,GS,SPA,VP,DTE,TT                           
     &     ,DT,ZT                                                       
     &     ,TRA,TRAT                                                    
     &     ,ZMI,DMI,TMI,SPMI                                            
     &     ,TRAMI                                                       
! Vorticity coefficient                                                 
      COMPLEX :: Z(NNTx,MMTy,NHEM,NL)                                   
! Divergence coefficient                                                
      COMPLEX :: D(NNTx,MMTy,NHEM,NL)                                   
! Temperature coefficient                                               
      COMPLEX :: T(NNTx,MMTy,NHEM,NL)                                   
! Coeffecient for the log of surface pressure                           
      COMPLEX :: SP(NNTx,MMTy,NHEM)                                     
! Coefficient for geopotential                                          
      COMPLEX :: GS(NNTx,MMTy,NHEM)                                     
! Coefficient for surface pressure                                      
      COMPLEX :: SPA(NNTx,MMTy,NHEM)                                    
! Non-linear source term for log of surface pressure equation           
      COMPLEX :: VP(NNTx,MMTy,NHEM)                                     
! Energy term                                                           
      COMPLEX :: DTE(NNTx,MMTy,NHEM,NL)                                 
! Coefficient of source term for temperature equation                   
      COMPLEX :: TT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for divergence equation                    
      COMPLEX :: DT(NNTx,MMTy,NHEM,NL)                                  
! Coefficient of source term for vorticity equation                     
      COMPLEX :: ZT(NNTx,MMTy,NHEM,NL)                                  
! Tracer coefficient                                                    
      COMPLEX :: TRA(NNTx,MMTy,NHEM,NL,NTRAC)                           
! Coefficient of source term for tracers                                
      COMPLEX :: TRAT(NNTx,MMTy,NHEM,NL,NTRAC)                          
! Vorticity coefficient at previous timestep                            
      COMPLEX :: ZMI(NNTx,MMTy,NHEM,NL)                                 
! Divergence coefficient at previous timestep                           
      COMPLEX :: DMI(NNTx,MMTy,NHEM,NL)                                 
! Temperature coefficient at previous timestep                          
      COMPLEX :: TMI(NNTx,MMTy,NHEM,NL)                                 
! Coeffecient for the log of surface pressure at prev' timestep         
      COMPLEX :: SPMI(NNTx,MMTy,NHEM)                                   
! Tracer coefficient at previous timestep                               
      COMPLEX :: TRAMI(NNTx,MMTy,NHEM,NL,NTRAC)                         
!-----------------------------------------------------                  
! Switches counters and constants controlling type and frequency of     
! model output                                                          
!-----------------------------------------------------                  
      COMMON/OUTCON/RNTAPE,DAY,SQR2,RSQR2,EAM1,EAM2,TOUT1               
     &     ,TOUT2,LSPO,LGPO                                             
     &     ,RNTAPO                                                      
     &     ,LSHIST,LMINIH                                               
      COMMON/OUTCONINT/NCOEFF,NLAT,INLAT,INSPC,KOUNTP                   
     &                 ,KOUNTE,KOUNTH,KOUNTR,KOUTP                      
     &                 ,KOUTE,KOUTH,KOUTR                               
! Identyfying run number                                                
      REAL :: RNTAPE                                                    
! Maximum wavenumber of printed spectral coeffecients                   
      INTEGER :: NCOEFF                                                 
! Number of latitudes in printed grid point fields                      
      INTEGER :: NLAT                                                   
      INTEGER :: INLAT                                                  
!                                                                       
      INTEGER :: INSPC                                                  
! Number of timsteps between successive writing of spectral             
! and gridded fields                                                    
      INTEGER :: KOUNTP                                                 
      INTEGER :: KOUNTE                                                 
! Number of timsteps between successive writing of history              
! records, containing spectral fields at the present and                
! previous time levels.                                                 
      INTEGER :: KOUNTH                                                 
! Number of timsteps between successive writing of restart              
! records, containing the spectral fields at the present and            
! previous time levels                                                  
      INTEGER :: KOUNTR                                                 
      INTEGER :: KOUTP                                                  
      INTEGER :: KOUTE                                                  
      INTEGER :: KOUTH                                                  
      INTEGER :: KOUTR                                                  
      REAL :: DAY                                                       
      REAL :: SQR2                                                      
      REAL :: RSQR2                                                     
      REAL :: EAM1                                                      
      REAL :: EAM2                                                      
      REAL :: TOUT1                                                     
      REAL :: TOUT2                                                     
      LOGICAL :: LSPO(NL)                                               
      LOGICAL :: LGPO(NL)                                               
      REAL :: RNTAPO                                                    
      LOGICAL :: LSHIST                                                 
! Reduce the history file to single precision if true                   
      LOGICAL :: LMINIH                                                 
      COMPLEX :: SPold(NWJ2*NHEM), SPMIold(NWJ2*NHEM)                   
      EXTERNAL RANF                                                     
  200 FORMAT(' WHITE NOISE SURFACE PRESSURE PERTURBATION AT T=0'/)      
! store new format surface presssure in the old format - marc           
      call new2old(nhem,sp,spold)                                       
      call new2old(nhem,spmi,spmiold)                                   
C                                                                       
C     Eps sets magnitude of the noise                                   
C                                                                       
      EPS=1.E-4                                                         
      WRITE (2,200)                                                     
      SCALE=EPS/SQRT(2.0)                                               
      IBAS=IDM+1                                                        
      IEND=NWJ2                                                         
      IDUM=-1                                                           
      DO 800 IHEM=1,NHEM                                                
         DO 10 I=IBAS,IEND                                              
            ZR1=RANF(IDUM)-0.5                                          
            ZR2=RANF(IDUM)-0.5                                          
            SPold(I)=SPold(I)+SCALE*CMPLX(ZR1,ZR2)                      
            SPMIold(I)=SPold(I)                                         
10       CONTINUE                                                       
         IBAS=IBAS+NWJ2                                                 
         IEND=IEND+NWJ2                                                 
800   CONTINUE                                                          
C                                                                       
! store old format surface presssure in the new format - marc           
      call old2new(nhem,spold,sp)                                       
      call old2new(nhem,spmiold,spmi)                                   
!                                                                       
      RETURN                                                            
      END                                                               
!=====================================================                  
! Convert from the old coefficient formation to the new one             
!=====================================================                  
      SUBROUTINE OLD2NEW(nLevels, InArray, OutArray)                    
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variable                                                       
!-----------------------------------------------------                  
! Number of levels                                                      
      INTEGER, INTENT(IN) :: nLevels                                    
! In array                                                              
      COMPLEX, INTENT(IN) :: InArray(NWJ2,nLevels)                      
! Out array                                                             
      COMPLEX, INTENT(OUT) :: OutArray(NNT,MMT,nLevels)                 
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER :: IM, IN, L                                              
!-----------------------------------------------------                  
! Loop through the levels                                               
!-----------------------------------------------------                  
      DO L=1,nLevels                                                    
         DO IM=1,MMT                                                    
            DO IN=1,NNT                                                 
               OutArray(IN,IM,L)=InArray(CoefConv(IN,IM,0),L)           
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
      END                                                               
!=====================================================                  
! Convert from the old coefficient formation to the new one             
!=====================================================                  
      SUBROUTINE NEW2OLD(nLevels, InArray, OutArray)                    
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variable                                                       
!-----------------------------------------------------                  
! Number of levels                                                      
      INTEGER, INTENT(IN) :: nLevels                                    
! In array                                                              
      COMPLEX, INTENT(IN) :: InArray(NNT,MMT,nLevels)                   
! Out array                                                             
      COMPLEX, INTENT(OUT) :: OutArray(NWJ2,nLevels)                    
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER :: IM, IN, L                                              
!-----------------------------------------------------                  
! Loop through the levels                                               
!-----------------------------------------------------                  
      DO L=1,nLevels                                                    
         DO IM=1,MMT                                                    
            DO IN=1,NNT                                                 
               OutArray(CoefConv(IN,IM,0),L)=InArray(IN,IM,L)           
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
      END                                                               
!=====================================================                  
! Convert from the old coefficient formation to the new one             
!=====================================================                  
      SUBROUTINE NEW2OLD2(nLevels, InArray, OutArray)                   
      IMPLICIT NONE                                                     
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Include the mpi header file                                           
!-----------------------------------------------------                  
       INCLUDE 'mpif.h'                                                 
!-----------------------------------------------------                  
! Data types to be sent through MPI routines                            
!-----------------------------------------------------                  
! MPI variables must be double precision                                
!      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(15, 307)         
      INTEGER, PARAMETER :: Type = SELECTED_REAL_KIND(6, 30)            
! MPI type for the real numbers in this program                         
      INTEGER(KIND=Type), PARAMETER :: ParaReal=MPI_REAL8               
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaInt=MPI_INTEGER              
! MPI type for the integer numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_COMPLEX16           
      INTEGER(KIND=Type), PARAMETER :: ParaCom=MPI_DOUBLE_COMPLEX       
! MPI type for the logical numbers in this program                      
!      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_INTEGER8            
      INTEGER(KIND=Type), PARAMETER :: ParaLog=MPI_LOGICAL              
! MPI type for summing over all PEs                                     
      INTEGER(KIND=Type), PARAMETER :: ParaSum=MPI_SUM                  
! MPI type for taking the maximum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMax=MPI_MAX                  
! MPI type for taking the minimum value over all PEs                    
      INTEGER(KIND=Type), PARAMETER :: ParaMin=MPI_MIN                  
!-----------------------------------------------------                  
! Variables used in MPI routines                                        
!-----------------------------------------------------                  
! Size of buffer                                                        
      INTEGER(KIND=Type) :: nBuffSize                                   
! PE which sent the information                                         
      INTEGER(KIND=Type) :: iSend                                       
! PE which will want the information                                    
      INTEGER(KIND=Type) :: iRecv                                       
! Tag for sending messages                                              
      INTEGER(KIND=Type) :: iTag                                        
! Record errors                                                         
      INTEGER(KIND=Type) :: ierr                                        
      INTEGER(KIND=Type) :: istat(MPI_STATUS_SIZE)                      
!-----------------------------------------------------                  
! The actual size of these variables on this PE                         
!-----------------------------------------------------                  
      COMMON/MPIVARS/MyPe,iCol,iRow,GRID,PeGrid,                        
     &     MMTyy,NNTxx,NLxx,MGxx,JGyy,                                  
     &     II1,JJ1,KK1,MMT1,NNT1,                                       
     &     mTwin1,mTwin2,mWave,nWave,CoefConv                           
! Processor element (PE) number                                         
      INTEGER :: MyPe                                                   
! Identifies the column this PE is on                                   
      INTEGER :: iCol                                                   
! Identifies the row this PE is on                                      
      INTEGER :: iRow                                                   
! Store column and row for each PE                                      
      INTEGER :: GRID(2,0:NPE-1)                                        
! Provies the PE for given column and row                               
      INTEGER :: PeGrid(Px,Py)                                          
! The number of twinned zonal wavenumbers per row of PEs                
      INTEGER :: MMTyy(Py)                                              
! The number of meridional wavenumbers per column of PEs                
      INTEGER :: NNTxx(Px)                                              
! The number of levels per column of PEs                                
      INTEGER :: NLxx(Px)                                               
! The number of longitude per column of PEs                             
      INTEGER :: MGxx(Px)                                               
! The number of latitude per row of PEs                                 
      INTEGER :: JGyy(Py)                                               
! Starting position for longitude for each column of PEs                
      INTEGER :: II1(Px)                                                
! Starting position for latitude for each row of PEs                    
      INTEGER :: JJ1(Py)                                                
! Starting position for levels for each column of PEs                   
      INTEGER :: KK1(Px)                                                
! Starting position for twinned zonal wavenumbers for each row of PEs   
      INTEGER :: MMT1(Py)                                               
! Starting position for meridional wavenumber of each column of PEs     
      INTEGER :: NNT1(Px)                                               
! mTwin1(0,:) = 2 (pair) or =1 (single); mTwin1(1,:) starting position  
! - of 1st zonal wavenumber in pair; mTwin1(2,:) end position of 1st    
! zonal wavenumber and mTwin(3,:) end position of 2nd zonal wavenumber  
      INTEGER :: mTwin1(0:3,MMTy)                                       
! mTwin2 is the same as mTwin1 except the twinned zonal wavenumbers are 
! not split across PEs                                                  
      INTEGER :: mTwin2(0:3,MMTy)                                       
! The zonal wavenumber values                                           
      INTEGER :: mWave(2,MMTy)                                          
! The meridional wavenumber values                                      
      INTEGER :: nWave(NNTx,MMTy,Px)                                    
! Used to convert between the format for reading and writing            
! coefficients and that for storage while running the code              
      INTEGER :: CoefConv(NNTx,MMTy,0:NPE-1)                            
!-----------------------------------------------------                  
! Passed variable                                                       
!-----------------------------------------------------                  
! Number of levels                                                      
      INTEGER, INTENT(IN) :: nLevels                                    
! In array                                                              
      REAL, INTENT(IN) :: InArray(NNT,MMT,nLevels)                      
! Out array                                                             
      REAL, INTENT(OUT) :: OutArray(NWJ2,nLevels)                       
!-----------------------------------------------------                  
! Local variables                                                       
!-----------------------------------------------------                  
      INTEGER :: IM, IN, L                                              
!-----------------------------------------------------                  
! Loop through the levels                                               
!-----------------------------------------------------                  
      DO L=1,nLevels                                                    
         DO IM=1,MMT                                                    
            DO IN=1,NNT                                                 
               OutArray(CoefConv(IN,IM,0),L)=InArray(IN,IM,L)           
            ENDDO                                                       
         ENDDO                                                          
      ENDDO                                                             
      END                                                               
      SUBROUTINE alterwater(exwaflx,exiwaflx,exinwaflx,waflx,waflx2)    
c----------------------------------------------------                   
c rssmi: 14-11-01                                                       
c deals with the lack of run-off in the coupled model                   
c reads in a preprepared drainage basin mask and passes                 
c excess water from land to those basins. Careful to                    
c ensure that the drainage mask fits your other land                    
c masks...                                                              
c----------------------------------------------------                   
C Alterwater only be run on one processor MYPE=0 because it uses        
C the RUNOFF array which is global, not local to processors             
                                                                        
!-----------------------------------------------------                  
! Determines model resolution                                           
!-----------------------------------------------------                  
! Determines domain of integration:                                     
!   =1, hemisheric, with flow symmetric about equator                   
!   =2, full global domain                                              
      INTEGER, PARAMETER :: NHEM=2                                      
! Highest zonal wavenumber retained in the spectral series,             
! the jagged triangular truncation on retains m.le.MM-1                 
      INTEGER, PARAMETER :: MM=42                                       
! Symmetry imposed in longitude. Only retaining m=0, MOCT, 2*MOCT,      
! 3*MOCT, ...                                                           
      INTEGER, PARAMETER :: MOCT=1                                      
! Number of levels in the vertical                                      
      INTEGER, PARAMETER :: NL=20                                       
! Number of longitudes in the transform grid (assumed even).            
      INTEGER, PARAMETER :: MG=128                                      
! Number of Gaussian latitudes in the transform grid between            
! pole and equator                                                      
      INTEGER, PARAMETER :: JG=32                                       
! The number of tracers                                                 
      INTEGER, PARAMETER :: NTRAC=1                                     
! Highest total wavenumber retained in the spectral series              
      INTEGER, PARAMETER :: NN=MM                                       
! Number of either even or odd spectral cofficients retained            
! in spectral series (equation overstimates for odd MM & MOCT=1)        
      INTEGER, PARAMETER :: NWJ2=462                                    
! Optimum number of Fast Fourier Transforms to be performed             
! in parallel                                                           
      INTEGER, PARAMETER :: NCRAY=64                                    
!-----------------------------------------------------                  
! Multi-processor variables                                             
!-----------------------------------------------------                  
! Number of processor elements (PEs)                                    
      INTEGER, PARAMETER :: npe=32                                      
!                                                                       
      INTEGER, PARAMETER :: NLEVRF=3                                    
!-----------------------------------------------------                  
! Sets basic constants, especially those needed for array dimensions    
!-----------------------------------------------------                  
! Used in counting the no. of odd or even coefficients                  
      INTEGER, PARAMETER :: MH=2                                        
! Value of pi                                                           
      REAL, PARAMETER :: PI=3.14159265359                               
! Value of 2*pi                                                         
      REAL, PARAMETER :: PI2=2.0*PI                                     
      INTEGER, PARAMETER :: NNP=NN+1                                    
! Real dimension of a single level of the grid-point/half transform     
! arrays, for a single latitude or for just odd of even values          
      INTEGER, PARAMETER :: MGPP=MG+2                                   
! No. of odd or even coefficients for m=0 at single level               
      INTEGER, PARAMETER :: IDM=NNP/2                                   
!-----------------------------------------------------                  
! Core variables which are reduced in size because of running on more   
! than one processor                                                    
!-----------------------------------------------------                  
! Zonal wavenumbers are twinned, zonal wavenumbers with many            
! meridional wavenumbers are paired with those with few.                
      INTEGER, PARAMETER :: MMT=((MM+MOCT-1)/MOCT+1)/2                  
! Number of meridional wavenumber for `twinned' zonal wavenumbers       
      INTEGER, PARAMETER :: NNT=(MM/2)+((MOCT+1)/2)                     
! Number of PEs to divide longitude domain                              
      INTEGER, PARAMETER :: Px=8                                        
! Number of PEs to divide the latitude domain                           
      INTEGER, PARAMETER :: Py=(NPE+Px-1)/Px                            
! Number of twinned zonal wavenumber when separated between PEs         
      INTEGER, PARAMETER :: MMTy=(MMT+Py-1)/Py                          
! Number of meridional wavenumbers when separated between PEs           
      INTEGER, PARAMETER :: NNTx=(NNT+Px-1)/Px                          
! Number of layers when separated between PEs                           
      INTEGER, PARAMETER :: NLx=(NL+Px-1)/Px                            
! Number of longitudes when separated between PEs                       
      INTEGER, PARAMETER :: MGx=(MG+Px-1)/Px                            
! Number of latitudes when separated between PEs                        
      INTEGER, PARAMETER :: JGy=(JG+Py-1)/Py                            
!-----------------------------------------------------                  
! Orogrophy                                                             
!-----------------------------------------------------                  
      COMMON/GSG/GSG,TOPVAR,RLEVBZMO,FXZMO,ofrac,runoff,iareanum        
! Topog variance, GW diags for Orog, Non orog waves                     
      REAL :: TOPVAR(MGx,NHEM,JGy)                                      
      REAL :: RLEVBZMO(JGy,NHEM,NL)                                     
      REAL :: FXZMO(JGy,NHEM,NL)                                        
         REAL :: GSG(MGx,NHEM,JGy)                                      
         REAL :: ofrac(MGx,NHEM,JGy)                                    
         INTEGER :: runoff(50,2,33)                                     
         INTEGER :: iareanum                                            
C OASIS size parameters                                                 
      INTEGER, PARAMETER :: IMB = 128                                   
      INTEGER, PARAMETER :: JM = 64                                     
      INTEGER, PARAMETER :: IMJM = IMB*JM                               
      common/oasiso2a/zsst(imjm),zscur(imjm),mscur(imjm)                
     $   ,zsst_2d(MGx,NHEM,JGy)                                         
     $   ,zscur_2d(MGx,NHEM,JGy),mscur_2d(MGx,NHEM,JGy)                 
      real zsst,zsst_2d,zscur,zscur_2d,mscur,mscur_2d                   
                                                                        
      integer ymax,iarea,ncoast                                         
      parameter(ymax=jm)                                                
                                                                        
C NOTE: WAFLX2 IS THE FULL 2D VERSION OF OFRAC which is needed below    
                                                                        
      real waflx(mg,nhem,jg), exwaflx(mg,nhem,jg), exiwaflx(mg,nhem,jg) 
      real waflx2(mg,nhem,jg), exinwaflx(mg,nhem,jg)                    
                                                                        
      real wagrid(mg,ymax),exgrid(mg,ymax), exigrid(mg,ymax)            
      real exgrid2(mg,ymax), exingrid(mg,ymax)                          
      real exgrid3(mg,ymax)                                             
      real exgrid4(mg,ymax)                                             
      real exgrid5(mg,ymax)                                             
      real exgrid9(mg,ymax)                                             
      real newagrid(mg,ymax)                                            
      real newagrid2(mg,ymax)                                           
                                                                        
      integer iland(2), jland(2), icoast, jcoast                        
      real totalrain, yagrid                                            
      real ddxx(ymax),ddyy(ymax),rlat(ymax)                             
      data ddyy/3.42217100000001, 2.82076499999999,                     
     1  2.80203100000000, 2.79667400000000, 2.79441100000001,           
     1  2.79322999999999, 2.79255999999999, 2.79211800000000,           
     1  2.79182700000000, 2.79162800000000, 2.79147200000000,           
     1  2.79135000000000, 2.79127300000000, 2.79119100000000,           
     1  2.79114700000000, 2.79108799999999, 2.79106400000001,           
     1  2.79103000000000, 2.79100400000000, 2.79098399999999,           
     1  2.79096700000000, 2.79095000000000, 2.79093800000000,           
     1  2.79092500000000, 2.79092000000000, 2.79090900000000,           
     1  2.79090600000000, 2.79089900000000, 2.79089530000000,           
     1  2.79089230000000, 2.79089080000000, 2.79088960000000,           
     1  2.79088960000000, 2.79089080000000, 2.79089230000000,           
     1  2.79089530000000, 2.79089900000000, 2.79090600000000,           
     1  2.79090900000000, 2.79092000000000, 2.79092500000000,           
     1  2.79093800000000, 2.79095000000000, 2.79096700000000,           
     1  2.79098399999999, 2.79100400000000, 2.79103000000000,           
     1  2.79106400000001, 2.79108799999999, 2.79114700000000,           
     1  2.79119100000000, 2.79127300000000, 2.79135000000000,           
     1  2.79147200000000, 2.79162800000000, 2.79182700000000,           
     1  2.79211800000000, 2.79255999999999, 2.79322999999999,           
     1  2.79441100000001, 2.79667400000000, 2.80203100000000,           
     1  2.82076499999999, 3.42217100000001/                             
      data rlat/ -87.8638460000000,                                     
     1  -85.0965120000000, -82.3129040000000, -79.5256120000000,        
     1  -76.7368930000000, -73.9475170000000, -71.1577450000000,        
     1  -68.3677600000000, -65.5776060000000, -62.7873540000000,        
     1  -59.9970170000000, -57.2066270000000, -54.4161990000000,        
     1  -51.6257320000000, -48.8352360000000, -46.0447200000000,        
     1  -43.2541920000000, -40.4636460000000, -37.6730840000000,        
     1  -34.8825190000000, -32.0919420000000, -29.3013570000000,        
     1  -26.5107670000000, -23.7201710000000, -20.9295770000000,        
     1  -18.1389690000000, -15.3483650000000, -12.5577550000000,        
     1  -9.76714520000000, -6.97653390000000, -4.18592020000000,        
     1  -1.39530680000000,  1.39530680000000,  4.18592020000000,        
     1   6.97653390000000,  9.76714520000000,  12.5577550000000,        
     1   15.3483650000000,  18.1389690000000,  20.9295770000000,        
     1   23.7201710000000,  26.5107670000000,  29.3013570000000,        
     1   32.0919420000000,  34.8825190000000,  37.6730840000000,        
     1   40.4636460000000,  43.2541920000000,  46.0447200000000,        
     1   48.8352360000000,  51.6257320000000,  54.4161990000000,        
     1   57.2066270000000,  59.9970170000000,  62.7873540000000,        
     1   65.5776060000000,  68.3677600000000,  71.1577450000000,        
     1   73.9475170000000,  76.7368930000000,  79.5256120000000,        
     1   82.3129040000000,  85.0965120000000,  87.8638460000000/        
      do i=1,ymax                                                       
      ddxx(i)=ddyy(i)*cos(rlat(i)*3.1415927/180.)                       
      end do                                                            
                                                                        
c      reformat (ex)waflx onto same grid and initialize the 'new' array 
                                                                        
      do i=1,mg                                                         
      do j=1,ymax                                                       
c the next 3 lines are the correct ones: waflx2 is OFRAC                
      if(j.gt.JG)ofr=waflx2(i,1,ymax+1-j)                               
      if(j.le.JG)ofr=waflx2(i,2,j)                                      
           exgrid3(i,j)=ofr                                             
      end do                                                            
      end do                                                            
                                                                        
      do i=1,mg                                                         
      do j=1,ymax                                                       
C runoff and snow excess > 55S & < 66N                                  
      if(j.gt.JG)ofr=exwaflx(i,1,ymax+1-j)                              
      if(j.le.JG)ofr=exwaflx(i,2,j)                                     
         exgrid(i,ymax+1-j)=ofr                                         
C snow excess < 58S                                                     
      if(j.gt.JG)ofr=exiwaflx(i,1,ymax+1-j)                             
      if(j.le.JG)ofr=exiwaflx(i,2,j)                                    
         exigrid(i,ymax+1-j)=ofr                                        
C snow excess > 66N                                                     
      if(j.gt.JG)ofr=exinwaflx(i,1,ymax+1-j)                            
      if(j.le.JG)ofr=exinwaflx(i,2,j)                                   
         exingrid(i,ymax+1-j)=ofr                                       
C surface water balance P minus E correction                            
      if(j.gt.JG)ofr=waflx(i,1,ymax+1-j)                                
      if(j.le.JG)ofr=waflx(i,2,j)                                       
C Do not Multiply by OFRAC since land coast P-E counted in runoff       
         newagrid2(i,ymax+1-j)=ofr                                      
      end do                                                            
      end do                                                            
                                                                        
      do iarea=1,iareanum-5                                             
      totalrain=0.                                                      
                                                                        
c      read in the land box perimeter                                   
      iland(1)=runoff(1,1,iarea)                                        
      iland(2)=runoff(1,2,iarea)                                        
      jland(1)=runoff(2,1,iarea)                                        
      jland(2)=runoff(2,2,iarea)                                        
      ncoast=runoff(3,1,iarea)                                          
                                                                        
c add up area-weighted excess runoff over land in box IAREA             
                                                                        
         do i=iland(1),iland(2)                                         
         do j=jland(1),jland(2)                                         
             totalrain=totalrain+exgrid(i,j)*ddxx(j)                    
C     1   *(1.-exgrid3(icoast,ymax+1-jcoast))                           
C MMJ 9 Sep Fix                                                         
     1   *(1.-exgrid3(i,ymax+1-j))                                      
         end do                                                         
         end do                                                         
                                                                        
c calculate the total (2D) sum of coastal points                        
c total is area-weighted sum of gridpoints and not ocean fractions      
c since flux to ocean only depends on total incoming runoff             
C SUM fluxes not volumes                                                
         yagrid=0                                                       
         do n=1,ncoast                                                  
            icoast=runoff(3+n,1,iarea)                                  
            jcoast=runoff(3+n,2,iarea)                                  
!            yagrid=yagrid+ddxx(jcoast)*exgrid3(icoast,ymax+1-jcoast)   
            yagrid=yagrid+ddxx(jcoast)*ddxx(jcoast)                     
         end do                                                         
                                                                        
c      divide runoff total by the (2D) area of coastal points           
         totalrain=totalrain/yagrid                                     
c        totalrain=totalrain/real(ncoast)                               
                                                                        
c      and add it to the dumps for that coastal point                   
         do n=1,ncoast                                                  
            icoast=runoff(3+n,1,iarea)                                  
            jcoast=runoff(3+n,2,iarea)                                  
      newagrid2(icoast,jcoast)=newagrid2(icoast,jcoast)                 
     $                           +totalrain*ddxx(jcoast)                
         end do                                                         
                                                                        
      end do                                                            
                                                                        
c  distribute Antarctic runoff over Southern Ocean                      
c  distribute snowdiff (icebergs) over southern ocean                   
C  Antarctic runoff is different continent in runoff array              
c     (iceberg fw flux is much larger than runoff in reality)           
                                                                        
      totaliberg=0.                                                     
                                                                        
      do iarea=iareanum-4,iareanum                                      
                                                                        
c      read in the land box perimeter                                   
      iland(1)=runoff(1,1,iarea)                                        
      iland(2)=runoff(1,2,iarea)                                        
      jland(1)=runoff(2,1,iarea)                                        
      jland(2)=runoff(2,2,iarea)                                        
                                                                        
c add up area-weighted excess runoff over land in box IAREA             
                                                                        
         do i=iland(1),iland(2)                                         
         do j=jland(1),jland(2)                                         
             totaliberg=totaliberg+exgrid(i,j)*ddxx(j)                  
C     1   *(1.-exgrid3(icoast,ymax+1-jcoast))                           
C MMJ 9 Sep Fix                                                         
     1   *(1.-exgrid3(i,ymax+1-j))                                      
         end do                                                         
         end do                                                         
      end do                                                            
                                                                        
C add up excess snowfall over SHSMAX south of 55S                       
                                                                        
      do i=1,mg                                                         
      do j=ymax/2+1,ymax                                                
          totaliberg=totaliberg+exigrid(i,j)*ddxx(j)                    
C     1   *(1.-exgrid3(icoast,ymax+1-jcoast))                           
C MMJ 9 Sep Fix                                                         
     1   *(1.-exgrid3(i,ymax+1-j))                                      
      end do                                                            
      end do                                                            
                                                                        
c calculate the total (2D) area of dumps in S Ocean                     
      yagrid=0                                                          
      do i=1,mg                                                         
      do j=53,ymax                                                      
          yagrid=yagrid+ddxx(j)*exgrid3(i,ymax+1-j)                     
!         yagrid=yagrid+ddxx(j)                                         
      end do                                                            
      end do                                                            
                                                                        
c      divide that total by the (2D) area of coastal dumps              
      totaliberg=totaliberg/yagrid                                      
                                                                        
c      and add it to the dumps for that area                            
      do i=1,mg                                                         
      do j=53,ymax                                                      
      if(exgrid3(i,ymax+1-j).gt.0.)then                                 
      newagrid2(i,j)=newagrid2(i,j)+totaliberg                          
      endif                                                             
      end do                                                            
      end do                                                            
                                                                        
c---Also distribute NH iceberg calving over whole arctic                
C  Antarctic runoff done earlier                                        
                                                                        
      totaliberg=0.                                                     
                                                                        
c Snowfall to iceberg flux added up in Arctic                           
C exingrid should only contain values north of 66N                      
                                                                        
      do i=1,mg                                                         
      do j=1,ymax/2                                                     
          totaliberg=totaliberg+exingrid(i,j)*ddxx(j)                   
C     1   *(1.-exgrid3(icoast,ymax+1-jcoast))                           
C MMJ 9 Sep Fix                                                         
     1   *(1.-exgrid3(i,ymax+1-j))                                      
      end do                                                            
      end do                                                            
                                                                        
c calculate the total (2D) area of dumps in Arctic circle               
      yagrid=0                                                          
      do i=1,mg                                                         
      do j=1,8                                                          
         yagrid=yagrid+ddxx(j)*exgrid3(i,ymax+1-j)                      
!         yagrid=yagrid+ddxx(j)                                         
      end do                                                            
      end do                                                            
                                                                        
c      divide that total by the (2D) area of coastal dumps              
      totaliberg=totaliberg/yagrid                                      
                                                                        
c      and add it to the dumps for that area                            
      do i=1,mg                                                         
      do j=1,8                                                          
      if(exgrid3(i,ymax+1-j).gt.0.)then                                 
      newagrid2(i,j)=newagrid2(i,j)+totaliberg                          
      endif                                                             
      end do                                                            
      end do                                                            
                                                                        
c      now, give the field back                                         
                                                                        
      do j=ymax,1,-1                                                    
      do i=1,mg                                                         
         if(exgrid3(i,ymax+1-j).eq.0.)then                              
         exgrid4(i,j)=0.                                                
         else                                                           
C Normalise flux by ocean fraction                                      
         exgrid4(i,j)=newagrid2(i,j)/exgrid3(i,ymax+1-j)                
         endif                                                          
                                                                        
         if(j.gt.32)waflx(i,2,65-j)=exgrid4(i,j)                        
         if(j.le.32)waflx(i,1,j)=exgrid4(i,j)                           
         ofr=exgrid4(i,ymax+1-j)                                        
      end do                                                            
      end do                                                            
                                                                        
      sum1=0.                                                           
      sum4b=0.                                                          
      do i=1,mg                                                         
      do j=1,ymax                                                       
      sum1=sum1+exgrid(i,j)*ddxx(j)                                     
      sum4b=sum4b+exgrid4(i,j)*ddxx(j)*exgrid3(i,ymax+1-j)              
      end do                                                            
      end do                                                            
      write(6,*) " SUM1 ",sum1                                          
      write(6,*) " SUM4b ",sum4b                                        
                                                                        
      return                                                            
      end                                                               
                                                                        
C PMF 13-8-97 Modified to provide interface to IGCM and include his     
C dignostic clouds - similar ro morcigcm_clouds.f                       
C      PROGRAM NEWRAD                                                   
C                                                                       
       SUBROUTINE MORCIGCM(PR,T,H2O,O3,alat,HTLW,HTSW,DOY,cf,ic,        
     $     rfluxes,SWALB,alon)                                          
C                                                                       
C modified by Piers Forster for plug and play IGCM fun                  
C 29.10.96 and again 1-7-97, For clouds                                 
                                                                        
C P  Pressure on Full levels(Pa) level 1 is ground                      
C T Temperature on Full levels(K) level 1 is ground                     
C H2O water vapour mmr on full levels level 1 is ground                 
C O3 ozone mmr on full levels level 1 is ground                         
C DOY Julian Day wanted (real)                                          
C NLEV number of levles                                                 
C ALAT Latitude in degrees                                              
c alon longitude in degrees                                             
C HTLW Output: lomgwave Heating Rate full levels etc (K/day)            
C HTSW Output: shortwave heating rate (K/DAY)                           
C RFLUXES  Array to hold fluxes at top and bottom of atmosphere         
C     1st index - flux 1=SW, 2=LW                                       
C     2nd index - Direction 1=DN, 2=UP                                  
C     3rd index - Where 1=TOP, 2=SURFACE                                
C SWALB SW albedo                                                       
C                 -----------                                           
C  Driving program for the extended atmospheric radiation scheme        
C   of Morcrette.                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRSW* SWITCHES RELATED TO THE RADIATION SCHEME.         
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      IMPLICIT REAL (A-H,O-Z)                                           
C      IMPLICIT UNDEFINED (Z)                                           
C      IMPLICIT LOGICAL (L)                                             
C      IMPLICIT UNDEFINED (L)                                           
      IMPLICIT INTEGER (I-K,M-N)                                        
C      IMPLICIT UNDEFINED (I-K,M-N)                                     
C      IMPLICIT REAL (P)                                                
C      IMPLICIT UNDEFINED (A-H,O,Q-Y)                                   
C                                                                       
C everything except the parameters is defined                           
C in the common blocks                                                  
C                                                                       
      COMMON/COM1/                                                      
C     I NRADFR, NRPART, NRINT, NRADPFR, NRADPLA,                        
C     I NRLSTI,                                                         
     R XLAT,                                                            
     R RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR, ZCCO2,              
C     R PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3,                 
     L LOH, LOM, LOL, LORAD                                             
      COMMON/COM2/                                                      
     I ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,                  
     I         JLOOP, KRLEV,                                            
     I        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     I        ITH, ITM, ITL,                                            
     I        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     I        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     I        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     I        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      COMMON/COM3/                                                      
     I IND4, KM12, KXD, KXU, IG, KBS, KDD,                              
     I        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     I        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     I        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     I        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     I        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
      COMMON/COM4/                                                      
     R           COZQC,COZQS,COZHC,COZHS,                               
     R CSUPSAT,                                                         
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
      COMMON/COM5/                                                      
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NUMVC,                                                           
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      COMMON/COM6/                                                      
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD,       
     R DTIME,    TWODT,    EPS,                                         
     R SINLON,     COSLON,                                              
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2,                          
     I NFAX,                                                            
     R CRAE,                                                            
     R CDISSEM                                                          
      COMMON/COM7/                                                      
     R     CCRH,                                                        
     L LO,LO1,LO2,LODIA,LODIAP,                                         
     R TAUA, PIZA, CGA, CAER, CAEROS                                    
      COMMON/COM7A/                                                     
     R  RT1,WG1,AT,BT,CNTNU,O1H,O2H,PIALF0                              
     S ,OCT,TREF,XP,TSTAND,GA,GB,CE,ETY,PTY                             
      COMMON/COM7B/                                                     
     S TINTP,TSTP,                                                      
     S  TBO3,AO3,BO3,TB,ALP,BET,ULG,ULGO,PLG,AP,BP,TO3B,O3A,O3B,        
     R          CDAY,CCO2,CH2O,DIFF,X10E                                
       COMMON/COM7C/                                                    
     S  ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW,                       
     R  APAD,BPAD,D,CRAY,SUN,                                           
     R   TTT,                                                           
     R   TM1                                                            
      COMMON/COM8/                                                      
     R       TSM1M,                                                     
     R       QM1,                                                       
     R       SNM1M,                                                     
     R       ALBM,                                                      
     R       AVVELM,                                                    
     R   SLMM,                                                          
     R       APHM1,                                                     
     R       APM1                                                       
      COMMON/COM9/                                                      
     L LOLAND,                                                          
     R   ZTI,                                                           
     *       ZCTI,                                                      
     *       ZCPH,                                                      
     *       ZWV,                                                       
     *       ZSW,                                                       
     *       ZALTE,                                                     
     *       ZCALTE,                                                    
     *       ZALSO                                                      
      COMMON/COM10/                                                     
     *       ZCALSO,                                                    
     R   ZOZQ,                                                          
     *       ZOZH,                                                      
     R   ZTRSOL,                                                        
     *       ZEMTER,                                                    
     R   TRSOLM,                                                        
     *       EMTERM,                                                    
     R   AMU0,                                                          
     *       RDAYL                                                      
      COMMON/COM11/                                                     
     R   CLC,                                                           
     *       TI,                                                        
     *       DP,                                                        
     *       AADS,                                                      
     *       AEQ1,                                                      
     *       AEQ2,                                                      
     *       AEQ3,                                                      
     *       AEQ4,                                                      
     *       AEQ5,                                                      
     *       SSW                                                        
      COMMON/COM12/                                                     
     *       WV,                                                        
     *       QOF,                                                       
     *       QCF,                                                       
     *       CLWA,                                                      
     *       APRE,                                                      
     *       SMU0,                                                      
     *       ALTE,                                                      
     *       ALSO                                                       
      COMMON/COM13/                                                     
     R   ZDPO,                                                          
     *       ZQCFO,                                                     
     *       ZAEQSO,                                                    
     *       ZQOFO,                                                     
     *       ZDPN,                                                      
     *       ZQCFN,                                                     
     *       ZAEQSN,                                                    
     *       ZQOFN                                                      
      COMMON/COM14/                                                     
     R   FLT,                                                           
     *       FLS,                                                       
     R   DIAG,                                                          
     *       DIA1,                                                      
     *       ZDIAG,                                                     
     *       ZDIA1,                                                     
     *       ZDIA,                                                      
     *       ZDIAF,                                                     
     *       ZDIAT,                                                     
     R PGG                                                              
      COMMON/COM15/                                                     
     #     PREF,                                                        
     #     PRE1,                                                        
     #     PRE2,                                                        
     #     PRMUZ,                                                       
     #     PTO1,                                                        
     #     PTR1,                                                        
     #     PTR2,                                                        
     #     PW,                                                          
     R     PALBSN                                                       
      COMMON/COM16/                                                     
     R ZABSCU,                                                          
     S     ZABSLY,                                                      
     S     ZADJD ,                                                      
     S     ZADJU ,                                                      
     S     ZBGND ,                                                      
     S     ZBINT ,                                                      
     S     ZBSUR ,                                                      
     S     ZBSUIN,                                                      
     S     ZBTOP ,                                                      
     S     PCLDLW,                                                      
     S     ZCLOUD,                                                      
     S     ZCNTRB,                                                      
     S     ZCUMP ,                                                      
     S     ZCUMPT,                                                      
     S     ZDBDT                                                        
      COMMON/COM17/                                                     
     S     ZDBSL ,                                                      
     S     ZDISTD,                                                      
     S     ZDISTU,                                                      
     S     ZDNF  ,                                                      
     S     PDT0  ,                                                      
     S     ZDUC                                                         
      COMMON/COM18/                                                     
     S     ZEFFT ,                                                      
     S     ZEMU  ,                                                      
     S     ZEPHIO,                                                      
     S     ZEPSC2,                                                      
     S     ZEPSC3,                                                      
     S     ZEPSH1,                                                      
     S     ZEPSH2,                                                      
     S     ZEPSH3                                                       
      COMMON/COM19/                                                     
     S     ZEPSH4,                                                      
     S     ZEPSH5,                                                      
     S     ZEPSH6,                                                      
     S     ZEPSIO,                                                      
     S     ZFACTC,                                                      
     S     ZZFD  ,                                                      
     S     ZFDD  ,                                                      
     S     ZFDN                                                         
      COMMON/COM20/                                                     
     S     PFLUX , PFLUC,                                               
     S     ZFLWP , PCTS                                                 
     S     ZZFU  ,                                                      
     S     ZFUP  ,                                                      
     S     ZGLAYD,                                                      
     S     ZGLAYU,                                                      
     S     ZOMEGA,                                                      
     S     ITX   ,                                                      
     S     IXDIA ,                                                      
     S     IXT   ,                                                      
     S     IXTSU ,                                                      
     S     IXTTP ,                                                      
     S     IMX   ,                                                      
     S     IMXP                                                         
      COMMON/COM21/                                                     
     R ZOZ   ,                                                          
     S     PPMB  ,                                                      
     S     ZRES  , PB                                                   
     S     ZSSIG ,                                                      
     S     PTAU  ,                                                      
     S     PTAVE ,                                                      
     S     ZTAVIC,                                                      
     S     ZTI1  ,                                                      
     S     PTL                                                          
      COMMON/COM22/                                                     
     S     ZTT   ,                                                      
     S     ZTT1  ,                                                      
     S     ZTT2  ,                                                      
     S     ZUAER ,                                                      
     S     ZUPF  ,                                                      
     S     ZUU   ,                                                      
     S     ZXD                                                          
      COMMON/COM23/                                                     
     S     ZXN   ,                                                      
     S     ZXOZ  ,                                                      
     S     ZXWV  ,                                                      
     S     ZZZ   ,                                                      
     S     ZRES2 ,                                                      
     S     ZTI2  ,                                                      
     S     ZBLAY ,                                                      
     S     ZBLEV ,                                                      
     S     ZCTS  ,                                                      
     S     ZHFG                                                         
      COMMON/COM24/                                                     
     I KT,                                                              
     R    PUU   ,                                                       
     S        PTT   ,                                                   
     S        ZZ                                                        
     R  PCLFR  ,                                                        
     S      PT     ,                                                    
     S      PQOF   ,                                                    
     S      PQLWP  ,                                                    
     S      PEMIS  ,                                                    
     S      PFLT                                                        
      COMMON/COM25/                                                     
     R  ZALBSU,                                                         
     S      ZCG   ,                                                     
     S      ZCLDLW,                                                     
     S      ZCLDSW,                                                     
     S      ZCOOLR,                                                     
     S      ZDT0  ,                                                     
     S      ZEMD  ,                                                     
     S      ZFLUX ,                                                     
     S      ZPMB  ,                                                     
     S      ZTAU  ,                                                     
     S      ZTAVE ,                                                     
     S      ZTL   ,                                                     
     S      ZFSDWN                                                      
      COMMON/COM26/                                                     
     S      ZFSUP ,                                                     
     S      ZHEATR,                                                     
     R   PDP   ,                                                        
     S       PWV   ,                                                    
     S       PFLS  ,                                                    
     S       PPSOL ,                                                    
     S       PRMU0 ,                                                    
     S       PAER  ,                                                    
     S       PALBS ,                                                    
     S       PCG   ,                                                    
     S       PCLDSW,                                                    
     S       POMEGA,                                                    
     S       POZ                                                        
      COMMON/COM27/                                                     
     R   PFDOWN,                                                        
     S       PFUP  ,                                                    
     R   ZAKI  ,                                                        
     S       ZCGAZ ,                                                    
     S       ZCNEB ,                                                    
     S       ZC1I  ,                                                    
     S       ZDSC  ,                                                    
     S       ZDSH                                                       
      COMMON/COM28/                                                     
     S       ZDSIG ,                                                    
     S       ZFACT ,                                                    
     S       ZFACT1 ,                                                   
     S       ZFACT2 ,                                                   
     S       ZFACT3 ,                                                   
     S       ZFACT4 ,                                                   
     S       ZFACT5 ,                                                   
     S       ZFACT6 ,                                                   
     S       ZFD   ,                                                    
     S       ZFRAC ,                                                    
     S       ZFU   ,                                                    
     S       ZG    ,                                                    
     S       ZGG   ,                                                    
     S       ZPIZAZ                                                     
      COMMON/COM29/                                                     
     S       ZRAYL ,                                                    
     S       ZRAY1 ,                                                    
     S       ZRAY2 ,                                                    
     S       ZREF  ,                                                    
     S       ZREFZ ,                                                    
     S       ZRE1  ,                                                    
     S       ZRE2  ,                                                    
     S       ZRJ   ,                                                    
     S       ZRK   ,                                                    
     S       ZRL   ,                                                    
     S       ZRMU  ,                                                    
     S       ZRMUE ,                                                    
     S       ZRMUZ                                                      
      COMMON/COM30/                                                     
     S       ZRNEB ,                                                    
     S       ZRUEF ,                                                    
     S       ZR1   ,                                                    
     S       ZR2   ,                                                    
     S       ZR21  ,                                                    
     S       ZR22  ,                                                    
     S       ZR23  ,                                                    
     S       ZS    ,                                                    
     S       ZSEC  ,                                                    
     S       ZSS1  ,                                                    
     S       ZTAUAZ                                                     
      COMMON/COM31/                                                     
     S       ZTOT  ,                                                    
     S       ZTO1  ,                                                    
     S       ZTR   ,                                                    
     S       ZTRA1 ,                                                    
     S       ZTRA2 ,                                                    
     S       ZTR1                                                       
      COMMON/COM32/                                                     
     S       ZTR2  ,                                                    
     S       ZUD   ,                                                    
     S       ZUM   ,                                                    
     S       ZU1D  ,                                                    
     S       ZU2D  ,                                                    
     S       ZW    ,                                                    
     S       ZN175 ,                                                    
     S       ZN190 ,                                                    
     S       ZO175 ,                                                    
     S       ZO190 ,                                                    
     S       ZP75  ,                                                    
     S       ZP90  ,                                                    
     S       ZSIGN ,                                                    
     S       ZSIGO                                                      
      COMMON/COM33/                                                     
     R   PTR,                                                           
     S       PU,                                                        
     R   ZRH,                                                           
     *       ZCH,                                                       
     *       ZCM,                                                       
     *       ZCL,                                                       
     *       ZCT,                                                       
     *       ZCC,                                                       
     I NTOPCM                                                           
      COMMON/COM34/                                                     
     *        NBASECM,                                                  
     R   ARPRCM,                                                        
     I ITOPH,                                                           
     *        ITOPM,                                                    
     *        ITOPL,                                                    
     L LOC,                                                             
     R   ZQS,                                                           
     *       ZCLC,                                                      
     *       ZQLWP                                                      
      COMMON/COM35/                                                     
     R   VERVEL,                                                        
     *       ZTHETA,                                                    
     *       ZETA,                                                      
     *       ZDTHMIN,                                                   
     I INVB,                                                            
     R   ZCTLWP,                                                        
     *       ZQLOC,                                                     
     *       ZQMEC,                                                     
     *       ZQHIC,                                                     
     R   ZFSOT,                                                         
     #       ZFTET,                                                     
     #       ZFLT                                                       
      COMMON/COM36/                                                     
     #       ZFSO ,                                                     
     #       ZFTE ,                                                     
     #       ZFLB ,                                                     
     #       ZDTDT,                                                     
     #       HEATO,                                                     
     #       HEATE,                                                     
     #       HEATR,                                                     
     #       ZI0                                                        
C                                                                       
      INTEGER                                                           
C     I NRADFR,   NRPART,   NRINT,    NRLSTI,  NRADPFR,  NRADPLA,       
     I NRLST,    NRLEV,    NRLEVP1, NLEV, NLP2, NLEVP1,                 
     I NLON, IRLEV, NTRA, NLAT, ITIM, NUA, NINT, NRPARTI                
C                                                                       
C   NRLEV .... number of vertical levels                                
C   NRLST .... number of points on a latitude line                      
C   NUA   .... number of absorber amount                                
C   NINT  .... number of spectral intervals                             
C   NTRA  .... number of transmission functions                         
C   NLAT  .... number of latitude points                                
C   ITIM  .... number of month under consideration                      
C                                                                       
C      PARAMETER (NRLEV=122,NUA=34,NINT=6,NTRA=34, NLAT=1,              
      PARAMETER (NRLEV=21,NUA=34,NINT=6,NTRA=34, NLAT=1,                
cccm      PARAMETER (NRLEV=37,NUA=18,NINT=6,NTRA=19, NLAT=1,            
cccm     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=10,NLEV=NRLEV,        
     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=7,NLEV=NRLEV,             
     #           NLP2=NRLST, NLEVP1=NRLEVP1, NLON=NRLST,                
     #           NRPARTI=1)                                             
C                                                                       
      REAL YCLOCK, XLAT(NLAT), Z0, DZ, ALBMPAR, SNPAR,                  
     #     TSUPAR, CTFREEZ, DELTAC, DELTAO, DELTAH                      
C                                                                       
C   YTIME  .... time of the year in radians                             
C   YCLOCK .... time of the day in radians                              
C   XLAT   .... latitudes in radians                                    
C   Z0     .... zero point of height grid                               
C   DZ     .... Interval between height levels                          
C   ALBMPAR.... sw surface albedo as function of longitude:             
C           yearly averaged sea-ice: 0.55                               
C                           water  : 0.07                               
C                           land   : 0.07.lt.ALBM.lt.0.80               
C                                    depending on snow                  
C   SNPAR  .... snow depth in meter                                     
C   TSUPAR .... surface temperature (Kelvin)                            
C   CTFREEZ.... freezing temperature of sea-water (Kelvin)              
C   DELTAC .... Doppler parameter for line shapes of CO2                
C   DELTAO .... Doppler parameter for line shapes of O3                 
C                                                                       
      PARAMETER (Z0=0., CTFREEZ=271.26,                                 
C     PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
     #           DZ=0.289, ALBMPAR=0.2, SNPAR=0.,                       
cccm  PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
cccm     #           DZ=0.15, ALBMPAR=0.3, SNPAR=0.,                    
CZ     #           DELTAC=0., DELTAO=0.)                                
     #           DELTAC=6.0E-04, DELTAO=1.E-02, DELTAH=5.0E-04)         
C      PARAMETER (YCLOCK=0., Z0=0., TSUPAR=273., CTFREEZ=271.26,        
C     #           DZ=0.46875, ALBMPAR=0.07, SNPAR=0.)                   
C      PARAMETER (YCLOCK=0., Z0=2.608, TSUPAR=273., CTFREEZ=271.26,     
C     #           DZ=0.2878, ALBMPAR=0.07, SNPAR=0.)                    
C                                                                       
      LOGICAL LDIUR, LOLPAR                                             
C                                                                       
C   LDIUR .... true: sun at time, false: diurnally averaged             
C   LOLPAR.... true: land surface, false: else                          
C                                                                       
C      PARAMETER (LDIUR=.TRUE., LOLPAR=.TRUE.)                          
       PARAMETER (LOLPAR=.TRUE.)                                        
C                                                                       
      INTEGER  C2TEMP                                                   
      CHARACTER*64  MIDTMP                                              
cccm      PARAMETER (C2TEMP=10,                                         
cccm     #    MIDTMP='/home/atm/ugamp/midrad/radfildat.dir/MIDTMP.DAT') 
ccc1      PARAMETER (C2TEMP=10, MIDTMP='tprfil2.dat')                   
C                                                                       
      INTEGER  IRLONI, IRLST, IRLVP1, JPNLEV, JPNLVP1                   
C                                                                       
      PARAMETER (IRLONI=NRLST, IRLST=NRLST,                             
     #           IRLVP1=NRLEVP1, JPNLEV=NRLEV,                          
     #           JPNLVP1=NRLEVP1, IRLEV=NRLEV)                          
C                                                                       
      INTEGER NIPD, NIPD2, NTR, NG1, NG1P1                              
      PARAMETER(NIPD=8,NIPD2=16,NTR=11,NG1=2,NG1P1=3)                   
C                                                                       
C    definition of the functions                                        
C                                                                       
      REAL       CVMGT, SIGMA2                                          
      INTEGER    CVMGI                                                  
C                                                                       
C    definitons of the subroutine cloud                                 
C                                                                       
      REAL RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR                  
C                                                                       
C    missing definitions in morcrette                                   
C                                                                       
C      REAL   PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3            
      REAL ZCCO2                                                        
                                                                        
      LOGICAL LOH, LOM, LOL, LORAD                                      
C                                                                       
      INTEGER ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,           
     #        IZQS, IZCLC, IZQLWP, JLOOP, KRLEV,                        
     #        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     #        ITH, ITM, ITL,                                            
     #        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     #        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     #        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     #        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      INTEGER IND4, KM12, KXD, KXU, IG, KBS, KDD,                       
     #        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     #        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     #        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     #        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     #        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD1* SPECTRAL DISTRIBUTION OF AEROSOLS AND OZONE.     
C                     (TRIANGULAR *T10* TRUNCATIONFOR AEROSOLS).        
C                     (TRIANGULAR *T5* TRUNCATIONFOR OZONE).            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R           COZQC(21),COZQS(15),COZHC(21),COZHS(15)                
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD2* CONSTANTS INTERNAL TO THE RADIATION SUBROUTINES. 
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CSUPSAT                                                          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMCON* BASIC UNIVERSAL CONSTANTS AND DERIVED CONSTANTS.  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
C                                                                       
C                                                                       
C       --------------------------------------------------------------- 
C      PARAMETER(                                                       
C     I JPM=106,JPN=106,JPK=106,JPGL=160,JPNLON=320,                    
C     I JPT1MAX=400,          JPT2MAX=50,                               
C     I JPT3MAX=300,          JPT4MAX=50,                               
C     I JPHASH=2048,          JPNTRCRS=20,                              
C     I JPTASKS=4,     JPLOCKS=50,    JPEVENT=4,                        
C     I JPLTMT=JPTASKS,                                                 
C     I JPNUMW=352,            JPNUMP=352,                              
C     I JPTABLE=1024,                                                   
C     I JPMAXB =60,                                                     
C     I JPMAXPB=32,                                                     
C     I JPG3FLD=43,         JPG3XF=99,         JPG3CMP=17,              
C     I JPMP1=JPM+1,        JPNP1=JPN+1,       JPKP1=JPK+1,             
C     I JP2MP1=JPMP1*2,     JP4MP1=JPMP1*4,                             
C     I JPED=JPN/2+1,       JPOD=JPNP1/2,                               
C     I IPSP0=JPM+JPN-JPK,  IPSP1=JPNP1*JPMP1, IPSP2=IPSP0*(IPSP0+1)/2, 
C     I JPSP=IPSP1-IPSP2,   JP2SP=2*JPSP,                               
C     I JPHGL=JPGL/2,       JPNLP2=JPNLON+2,                            
C     I                     JPNVCLV=4*JPNLVP1)                          
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C     J. K. GIBSON     E.C.M.W.F.     22/01/82.                         
C     MODIFIED M.J     E.C.M.W.F.     13/12/82.                         
C                                                                       
      INTEGER                                                           
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      LOGICAL                                                           
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD        
      REAL                                                              
     R DTIME,    TWODT,    EPS                                          
C                                                                       
C      INTEGER                                                          
C     I NWTIME(JPNUMW),     NROW(JPTASKS),                              
C     I NPTIME(JPNUMP)                                                  
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMGAU* - QUANTITIES RELATED TO THE GAUSSIAN GRID.      
C                                                                       
      REAL                                                              
     R SINLON(NLP2),     COSLON(NLP2)                                   
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMTRU* -QUANTITIES RELATED TO THE SPECTRAL TRUNCATION. 
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSIM* - QUANTITIES NEEDED FOR THE SEMI-IMPLICIT SCHEME
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMHDI* - COEFFICIENTS FOR HORIZONTAL DIFFUSION.        
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMFFT* - QUANTITIES NEEDED FOR THE FAST *FOURIER       
C                         TRANSFORMS.                                   
C                                                                       
      INTEGER                                                           
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2                           
C      REAL                                                             
C     R TRIG(JPNLON)                                                    
      INTEGER                                                           
     I NFAX(10)                                                         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSTA* - ARRAYS AND PARAMETERS FOR STATISTICS.         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMDIA* - QUANTITIES RELATED TO GLOBAL PHYSICAL         
C                         DIAGNOSTICS.                                  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPSW* SWITCHES RELATED TO THE PARAMETERISATIONS OF      
C                     DIABATIC PROCESSES.                               
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMDSW* SWITCHES RELATED TO THE DYNAMICS                  
C                     AND THE GENERAL CONTROL OF THE FORECAST.          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH1* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE RADIATION SUBROUTINES.                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CRAE,                                                            
     R CDISSEM                                                          
C                                                                       
C      REAL       COSRAD(128),        SINRAD(128)                       
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH2* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE PHYSICAL SUBROUTINES (EXCEPT RADIATION    
C                     ONES).                                            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL       CCRH(JPNLEV)                                           
C                                                                       
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMDIZ* VARIABLES AND POINTERS FOR ZONAL MEAN DIAGNOSTICS 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMMSK* VARIABLES AND POINTERS FOR AREA MEAN DIAGNOSTICS. 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMSDS* - START/RESTART BLOCK.                          
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMTASK* - TASK RELATED VARIABLES.                      
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*         *COMRAD* - RADIATION POINTERS.                               
C                                                                       
      LOGICAL LO,LO1,LO2,LODIA,LODIAP                                   
C---------------------------------------------------------------------  
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C     COMMON *YOMAER* RADIATIVE CHARACTERISTICS OF THE AEROSOLS         
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
      REAL TAUA(2,5), PIZA(2,5), CGA(2,5), CAER(5,5), CAEROS            
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C     COMMON *YOMLW*                                                    
C                                                                       
C*    COEFFICIENTS FOR THE LONGWAVE RADIATION SUBROUTINES               
C                                                                       
C NINT  :  NUMBER OF SPECTRAL INTERVALS                                 
C NIPD  :  NUMBER OF BANDS DESCRIBED BY PADE APPROXIMANTS               
C NTR   :  NUMBER OF REFERENCE TEMPERATURES                             
C NTRA  :  NUMBER OF TRANSMISSION FUNCTIONS                             
C NUA   :  NUMBER OF ABSORBER AMOUNTS                                   
C NG1   :  ORDER OF THE GAUSSIAN QUADRATURE                             
C                                                                       
C RT1   :  ROOTS OF THE GAUSSIAN POLYNOMIAL                             
C WG1   :  WEIGHTS OF THE GAUSSIAN POLYNOMIAL                           
C AT, BT:  COEFFICIENTS OF THE TEMPERATURE DEPENDENCE                   
C CNTNU :  H2O CONTINUUM ABSORPTION COEFFICIENTS                        
C O1H,O2H,PIALF0,OCT: OZONE LONGWAVE ABSORPTION COEFFICIENTS            
C TREF, TSTAND      : REFERENCE TEMPERATURES                            
C XP    :  COEFFICIENTS OF POLYNOMIAL DEVELOPMENTS OF PLANCK FUNCTION   
C GA,GB :  PADE APPROXIMANTS OF H2O, CO2 TRANSMISSION FUNCTIONS         
C ETY,PTY :  PARAMETERS OF H2O CONTINUUM                                
C TINTP :  REFERENCE TEMPERATURES OF PRE-COMPUTED TRANSMISSIVITIES      
C CE    :  EMS COEFFS OF H2O TRANSMISSION FUNCTIONS                     
C                                                                       
      REAL  RT1(2),WG1(2),AT(12,3),BT(12,3),CNTNU(4),O1H                
     S ,OCT(4),TREF,XP(6,6),TSTAND,GA(11,24,3),GB(11,24,3)              
     S ,PTY(11,20),ETY(11,20),TINTP(11),TSTP,O2H,PIALF0                 
C                                                                       
      REAL  TB(40,31,2),ALP(40,31,2),BET(40,31,2),                      
     S ULG(40),PLG(31),AP(40,31,2),BP(40,31,2),TBO3(38,31),TO3B(38,31)  
     S,ULGO(38),AO3(38,31),BO3(38,31),O3A(38,31),O3B(38,31)             
C     ----------------------------------------------------------------- 
C*COMDECK YOMRDU                                                        
C     ----------------------------------------------------------------- 
C                                                                       
C*    CONTROL VARIABLES FOR RADIATION: INTERNAL SWITCH AND INDICES      
C                                                                       
C ZEELOG:  SECURITY THRESHOLD FOR ABSORBER AMOUNT IN LAPLACE TRANSFORM  
C ZEPSC :  SECURITY THRESHOLD FOR CLOUD COVER                           
C ZEPSCO:  SECURITY THRESHOLD FOR OZONE AMOUNT                          
C ZEPSCQ:  SECURITY THRESHOLD FOR WATER VAPOR                           
C ZEPSCT:  SECURITY THRESHOLD FOR SHORTWAVE OPTICAL THICKNESS           
C ZEPSCW:  SECURITY THRESHOLD FOR CLOUD LIQUID WATER PATH               
C                                                                       
      REAL          CDAY,CCO2,CH2O,DIFF,X10E                            
     S             ,ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW            
C     ----------------------------------------------------------------- 
C*COMDECK YOMSW                                                         
C     ----------------------------------------------------------------- 
C                                                                       
C*    COEFFICIENTS FOR THE SHORTWAVE RADIATION SUBROUTINES              
C                                                                       
C APAD  :  PADE APPROXIMANTS NUMERATOR                                  
C BPAD  :  PADE APPROXIMANTS DENOMINATOR                                
C D     :  TRANSMISSION LIMIT FOR INFINITE ABSORBER AMOUNT              
C CRAY  :  RAYLEIGH SCATTERING COEFFICIENTS                             
C SUN   :  SOLAR FRACTION IN SPECTRAL INTERVALS                         
C                                                                       
      REAL  APAD(2,3,7),BPAD(2,3,7),D(2,3),CRAY(2,6),SUN(2)             
C     ----------------------------------------------------------------- 
                                                                        
C                                                                       
C  -------------------------------------------------------------------- 
C                                                                       
      REAL   TTT(1,NLAT,NRLEV,12)                                       
C                                                                       
C        0.1  "RADINT POINTERS"                                         
C             -----------------                                         
C                                                                       
      REAL   TM1(NLP2,NLEV),                                            
     *       TSM1M(NLP2),                                               
     *       QM1(NLP2,NLEV),                                            
     *       SNM1M(NLP2),                                               
     *       ALBM(NLP2),                                                
     *       AVVELM(NLP2)                                               
      REAL   SLMM(NLP2),                                                
     *       APHM1(NLP2,NLEVP1),                                        
     *       APM1(NLP2,NLEV)                                            
C                                                                       
      LOGICAL LOLAND(NLP2)                                              
C                                                                       
C     VARIABLES FOR FULL GRID AND TRANSFORMS.                           
C                                                                       
      REAL   ZTI(NLP2,NRLEVP1),                                         
     *       ZCTI(NLP2,NRLEVP1),                                        
     *       ZCPH(NLP2,NRLEVP1),                                        
     *       ZWV(NLP2,NRLEV),                                           
     *       ZSW(NLP2,NRLEV),                                           
     *       ZALTE(NLP2),                                               
     *       ZCALTE(NLP2),                                              
     *       ZALSO(NLP2),                                               
     *       ZCALSO(NLP2)                                               
C                                                                       
      REAL   ZOZQ(NLP2),                                                
     #       ZOZH(NLP2)                                                 
C                                                                       
      REAL   ZTRSOL(NLP2,NLEVP1),                                       
     *       ZEMTER(NLP2,NLEVP1)                                        
C                                                                       
C     FINAL RESULTS ARE STORED ON:                                      
C                                                                       
      REAL   TRSOLM(NLP2,NLEVP1),                                       
     *       EMTERM(NLP2,NLEVP1)                                        
C                                                                       
      REAL   AMU0(NLP2),                                                
     *       RDAYL(NLP2)                                                
C                                                                       
C     VARIABLES WITHIN THE SPLITTED LATITUDE LOOP.                      
C                                                                       
      REAL   CLC(NRLST,NRLEV),                                          
     *       TI(NRLST,NRLEVP1),                                         
     *       DP(NRLST,NRLEV),                                           
     *       AADS(NRLST,NRLEV),                                         
     *       AEQ1(NRLST,NRLEV),                                         
     *       AEQ2(NRLST,NRLEV),                                         
     *       AEQ3(NRLST,NRLEV),                                         
     *       AEQ4(NRLST,NRLEV),                                         
     *       AEQ5(NRLST,NRLEV),                                         
     *       SSW(NRLST,NRLEV),                                          
     *       WV(NRLST,NRLEV),                                           
     *       QOF(NRLST,NRLEV),                                          
     *       QCF(NRLST,NRLEV),                                          
     *       CLWA(NRLST,NRLEV),                                         
     *       APRE(NRLST),                                               
     *       SMU0(NRLST),                                               
     *       ALTE(NRLST),                                               
     *       ALSO(NRLST)                                                
      REAL   ZDPO(NRLST),                                               
     *       ZQCFO(NRLST),                                              
     *       ZAEQSO(NRLST),                                             
     *       ZQOFO(NRLST),                                              
     *       ZDPN(NRLST),                                               
     *       ZQCFN(NRLST),                                              
     *       ZAEQSN(NRLST),                                             
     *       ZQOFN(NRLST)                                               
      REAL   FLT(NRLST,NRLEVP1),                                        
     *       FLS(NRLST,NRLEVP1)                                         
C                                                                       
C     DIAGNOSTIC VARIABLES.                                             
C                                                                       
      REAL   DIAG(NLEVP1,4),                                            
     *       DIA1(7),                                                   
     *       ZDIAG(NLEVP1,10),                                          
     *       ZDIA1(21),                                                 
     *       ZDIA(NLON),                                                
     *       ZDIAF(NLON,7),                                             
     *       ZDIAT(NLEVP1)                                              
C                                                                       
C                                                                       
C                                                                       
      REAL PGG(IRLST),                                                  
     #     PREF(IRLST),                                                 
     #     PRE1(IRLST),                                                 
     #     PRE2(IRLST),                                                 
     #     PRMUZ(IRLST),                                                
     #     PTO1(IRLST),                                                 
     #     PTR1(IRLST),                                                 
     #     PTR2(IRLST),                                                 
     #     PW(IRLST)                                                    
C                                                                       
C                                                                       
C  what about the 1-dimensional PALBS???                                
C                                                                       
      REAL     PALBSN  (IRLST)                                          
C                                                                       
C*       0.2   "LOCAL" ARRAYS                                           
C              --------------                                           
      REAL ZABSCU(IRLST ,NUA, 3*NRLEV+1),                               
     S     ZABSLY(IRLST ,NUA, 3*NRLEV+1) ,                              
     S     ZADJD (IRLST , IRLVP1)      ,                                
     S     ZADJU (IRLST , IRLVP1)       ,                               
     S     ZBGND (IRLST)                ,                               
     S     ZBINT (IRLST , IRLVP1)       ,                               
     S     ZBSUR (IRLST , NINT)         ,                               
     S     ZBSUIN(IRLST)                ,                               
     S     ZBTOP (IRLST , NINT)         ,                               
     S     PCLDLW(IRLST , IRLEV)        ,                               
     S     ZCLOUD(IRLST)                ,                               
     S     ZCNTRB(IRLST , IRLVP1,IRLVP1),                               
     S     ZCUMP (IRLST)                ,                               
     S     ZCUMPT(IRLST)                ,                               
     S     ZDBDT (IRLST , NINT , IRLEV) ,                               
     S     ZDBSL (IRLST , NINT , 2*NRLEV),                              
     S     ZDISTD(IRLST , IRLVP1)       ,                               
     S     ZDISTU(IRLST , IRLVP1)                                       
      REAL                                                              
     S     ZDNF  (IRLST , IRLVP1,IRLVP1),                               
     S     PDT0  (IRLST)                                                
      REAL                                                              
     S     ZDUC  (IRLST , 3*NRLEV+1)       ,                            
     S     ZEFFT (IRLST)                ,                               
     S     ZEMD  (IRLST , IRLEV), ZEMU  (IRLST , IRLEV),                
     S     ZEPSC2(IRLST),  ZEPSC3(IRLST),                               
     S     ZEPSH1(IRLST),  ZEPSH2(IRLST),                               
     S     ZEPSH3(IRLST),  ZEPSH4(IRLST),                               
     S     ZEPSH5(IRLST),  ZEPSH6(IRLST),                               
     S     ZEPHIO(IRLST),  ZFACTC(IRLST),                               
     S     ZZFD(IRLST), ZFDD(IRLST), ZFDN(IRLST , IRLVP1)               
      REAL                                                              
     S     PFLUX (IRLST , 2 , IRLVP1)   ,                               
     S     PFLUC (IRLST , 2 , IRLVP1)   ,                               
     S     ZFLWP (IRLST)                ,                               
     S     PCTS  (IRLST , IRLEV)        ,                               
     S     ZZFU   (IRLST)               ,                               
     S     ZFUP  (IRLST , IRLVP1)       ,                               
     S     ZGLAYD(IRLST)                ,                               
     S     ZGLAYU(IRLST)                ,                               
     S     ZOMEGA(IRLST , 2 , IRLEV)                                    
C                                                                       
      INTEGER                                                           
     S     ITX   (IRLST)                ,                               
     S     IXDIA (IRLST , IRLVP1)       ,                               
     S     IXT   (IRLST , IRLVP1,IRLVP1),                               
     S     IXTSU (IRLST , IRLVP1)       ,                               
     S     IXTTP (IRLST , IRLVP1)       ,                               
     S     IMX   (IRLST)                ,                               
     S     IMXP  (IRLST)                                                
C                                                                       
      REAL ZOZ   (IRLST , IRLEV)        ,                               
     S     PPMB  (IRLST , IRLVP1)       ,                               
     S     ZRES  (IRLST)                ,                               
     S     PB    (IRLST, 6 , IRLVP1)    ,                               
     S     ZSSIG (IRLST , 3*NRLEV+1)    ,                               
     S     PTAU  (IRLST , 2 , IRLEV)    ,                               
     S     PTAVE (IRLST , IRLEV)        ,                               
     S     ZTAVIC(IRLST)                ,                               
     S     ZTI1  (IRLST)                ,                               
     S     PTL   (IRLST , IRLVP1)       ,                               
     S     ZTT   (IRLST , NTRA)         ,                               
     S     ZTT1  (IRLST , NTRA)         ,                               
     S     ZTT2  (IRLST , NTRA)         ,                               
     S     ZUAER (IRLST , NINT)         ,                               
     S     ZUPF  (IRLST , IRLVP1,IRLVP1),                               
     S     ZUU   (IRLST , NUA)          ,                               
     S     ZXD   (IRLST)                                                
      REAL                                                              
     S     ZXN   (IRLST)                ,                               
     S     ZXOZ  (IRLST)                ,                               
     S     ZXWV  (IRLST)                ,                               
     S     ZZZ   (IRLST)                ,                               
     S     ZRES2 (IRLST)                ,                               
     S     ZTI2  (IRLST)                ,                               
     S     ZBLAY (IRLST,IRLEV)          ,                               
     S     ZBLEV (IRLST,IRLVP1)         ,                               
     S     ZCTS  (IRLST)                ,                               
     S     ZHFG  (IRLST)                                                
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C                                                                       
      INTEGER KT    (IRLST)                                             
      REAL    PUU   (IRLST , NUA  )     ,                               
     S        PTT   (IRLST , NTRA )     ,                               
     S        ZZ    (IRLST)                                             
C                                                                       
C     -------------------------------------------------------- ---------
C                                                                       
      REAL  PCLFR  (IRLST , IRLEV)       ,                              
     S      PT     (IRLST , IRLVP1)       ,                             
     S      PQOF   (IRLST , IRLEV)          ,                           
     S      PQLWP  (IRLST , IRLEV)          ,                           
     S      PEMIS  (IRLST)                                              
      REAL                                                              
     S      PFLT   (IRLST , IRLVP1)                                     
C                                                                       
      REAL  ZALBSU(IRLST , 2 )           ,                              
     S      ZCG   (IRLST , 2 , IRLEV)    ,                              
     S      ZCLDLW(IRLST , IRLEV)        ,                              
     S      ZCLDSW(IRLST , IRLEV)        ,                              
     S      ZCOOLR(IRLST , IRLEV)        ,                              
     S      ZDT0  (IRLST)                ,                              
     S      ZFLUX (IRLST , 2 , IRLVP1)   ,                              
     S      ZPMB  (IRLST , IRLVP1)       ,                              
     S      ZTAU  (IRLST , 2 , IRLEV)    ,                              
     S      ZTAVE (IRLST , IRLEV)        ,                              
     S      ZTL   (IRLST , IRLVP1)       ,                              
     S      ZFSDWN(IRLST , IRLVP1)       ,                              
     S      ZFSUP (IRLST , IRLVP1)       ,                              
     S      ZHEATR(IRLST , IRLEV )                                      
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C                                                                       
      REAL   PDP   (IRLST , IRLEV)     ,                                
     S       PWV   (IRLST , IRLEV)     ,                                
     S       PFLS  (IRLST , IRLVP1)    ,                                
     S       PPSOL (IRLST        )     ,                                
     S       PRMU0 (IRLST        )     ,                                
     S       PAER  (IRLST,IRLEV,5)     ,                                
     S       PALBS (IRLST , 2 )        ,                                
     S       PCG   (IRLST , 2 , IRLEV) ,                                
     S       PCLDSW(IRLST , IRLEV)     ,                                
     S       POMEGA(IRLST , 2 , IRLEV) ,                                
     S       POZ   (IRLST , IRLEV)                                      
C                                                                       
      REAL   PFDOWN(IRLST , IRLVP1)    ,                                
     S       PFUP  (IRLST , IRLVP1)                                     
C                                                                       
      REAL   ZAKI  (IRLST , 2 )        ,                                
     S       ZCGAZ (IRLST , IRLEV)     ,                                
     S       ZCNEB (IRLST , IRLEV)     ,                                
     S       ZC1I  (IRLST , IRLVP1)    ,                                
     S       ZDSC  (IRLST , IRLEV)     ,                                
     S       ZDSH  (IRLST , IRLEV)     ,                                
     S       ZDSIG (IRLST , IRLEV)     ,                                
     S       ZFACT (IRLST)             ,                                
     S       ZFACT1 (IRLST), ZFACT2 (IRLST),                            
     S       ZFACT3 (IRLST), ZFACT4 (IRLST),                            
     S       ZFACT5 (IRLST), ZFACT6 (IRLST),                            
     S       ZFD   (IRLST , IRLVP1)    ,                                
     S       ZFRAC (IRLST)             ,                                
     S       ZFU   (IRLST , IRLVP1)    ,                                
     S       ZG    (IRLST)             ,                                
     S       ZGG   (IRLST)             ,                                
     S       ZPIZAZ(IRLST , IRLEV)     ,                                
     S       ZRAYL (IRLST),  ZRAY1 (IRLST , IRLVP1)                     
      REAL                                                              
     S       ZRAY2 (IRLST , IRLVP1)    ,                                
     S       ZREF  (IRLST)             ,                                
     S       ZREFZ (IRLST , 2 , IRLVP1),                                
     S       ZRE1  (IRLST)             ,                                
     S       ZRE2  (IRLST)             ,                                
     S       ZRJ   (IRLST , 6 , IRLVP1),                                
     S       ZRK   (IRLST , 6 , IRLVP1),                                
     S       ZRL   (IRLST , 8)         ,                                
     S       ZRMU  (IRLST)             ,                                
     S       ZRMUE (IRLST , IRLVP1)    ,                                
     S       ZRMUZ (IRLST)                                              
      REAL                                                              
     S       ZRNEB (IRLST)             ,                                
     S       ZRUEF (IRLST , 8)         ,                                
     S       ZR1   (IRLST)             ,                                
     S       ZR2   (IRLST)             ,                                
     S       ZR21  (IRLST)             ,                                
     S       ZR22  (IRLST)             ,                                
     S       ZR23  (IRLST)             ,                                
     S       ZS    (IRLST)             ,                                
     S       ZSEC  (IRLST)             ,                                
     S       ZSS1  (IRLST)             ,                                
     S       ZTAUAZ(IRLST , IRLEV)     ,                                
     S       ZTOT  (IRLST , IRLVP1)    ,                                
     S       ZTO1  (IRLST)             ,                                
     S       ZTR   (IRLST , 2 , IRLVP1),                                
     S       ZTRA1 (IRLST , IRLVP1)    ,                                
     S       ZTRA2 (IRLST , IRLVP1)                                     
      REAL                                                              
     S       ZTR1  (IRLST)             ,                                
     S       ZTR2  (IRLST)             ,                                
     S       ZUD   (IRLST , 3 , IRLVP1),                                
     S       ZUM   (IRLST , IRLVP1)    ,                                
     S       ZU1D  (IRLST)             ,                                
     S       ZU2D  (IRLST)             ,                                
     S       ZW    (IRLST)             ,                                
     S       ZN175 (IRLST)             ,                                
     S       ZN190 (IRLST)             ,                                
     S       ZO175 (IRLST)             ,                                
     S       ZO190 (IRLST)             ,                                
     S       ZP75  (IRLST)             ,                                
     S       ZP90  (IRLST)             ,                                
     S       ZSIGN (IRLST)             ,                                
     S       ZSIGO (IRLST)                                              
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
      REAL   PTR    (IRLST)     ,                                       
     S       PU     (IRLST)                                             
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
C   CLOUD-POINTERS                                                      
C   --------------                                                      
C                                                                       
      REAL   ZRH(NLP2,NLEV),                                            
     *       ZCH(NLP2),                                                 
     *       ZCM(NLP2),                                                 
     *       ZCL(NLP2),                                                 
     *       ZCT(NLP2),                                                 
     *       ZCC(NLP2)                                                  
      INTEGER NTOPCM(NLP2),                                             
     *        NBASECM(NLP2)                                             
      REAL   ARPRCM(NLP2)                                               
      INTEGER ITOPH(NLP2),                                              
     *        ITOPM(NLP2),                                              
     *        ITOPL(NLP2)                                               
      LOGICAL LOC(NLP2)                                                 
      REAL   ZQS(NLP2,NLEV),                                            
     *       ZCLC(NLP2,NLEV),                                           
     *       ZQLWP(NLP2,NLEV)                                           
C                                                                       
      REAL   VERVEL(NLP2,NLEV),                                         
     *       ZTHETA(NLP2,NLEV),                                         
     *       ZETA(NLP2,NLEV),                                           
     *       ZDTHMIN(NLP2)                                              
      INTEGER INVB(NLP2)                                                
      REAL   ZCTLWP(NLP2),                                              
     *       ZQLOC(NLP2),                                               
     *       ZQMEC(NLP2),                                               
     *       ZQHIC(NLP2)                                                
C                                                                       
C  definitions of "radheat"                                             
C                                                                       
      REAL   ZFSOT(NLON),                                               
     #       ZFTET(NLON),                                               
     #       ZFLT (NLON),                                               
     #       ZFSO (NLON),                                               
     #       ZFTE (NLON),                                               
     #       ZFLB (NLON),                                               
     #       ZDTDT(NLON),                                               
     #       HEATO(NLON,NLEV),                                          
     #       HEATE(NLON,NLEV),                                          
     #       HEATR(NLON,NLEV),                                          
     #       ZI0  (NLON)                                                
C                                                                       
C PIERS 13-8-97                                                         
C                                                                       
         REAL PIERSLWP                                                  
C                                                                       
C Piece of code needed to replace the Nikos radiation scheme with       
C the Morcrette radiation scheme in the igcm3.1.                        
C See morcrette3_1_1.readme for details.                                
C                                                                       
C                                                                       
c      program lh                                                       
c                                                                       
c      parameter(NL=23)                                                 
c      real P(NL),t(NL),h2o(NL),o3(NL),htlw(NL),htsw(NL)                
c      real swalb,doy                                                   
c       REAL CF(4,2)     !cloud fractions,lwps                          
c       INTEGER IC(4,2)   ! cloud positions                             
c       REAL RFLUXES(2,2,2)                                             
c      open(11,file='mls23.dat')                                        
c      read(11,*) NLEV                                                  
c      DOY=80                                                           
c      swalb=0.07                                                       
c      alat=40.0                                                        
c       doy=float(i)                                                    
c      do i=nlev,1,-1                                                   
c         read(11,*) P(i),t(i),h2o(i),adum,o3(i)                        
c         print *,i,p(i),t(i),h2o(i),o3(i)                              
c      enddo                                                            
c                                                                       
c       ic(1,1)=3                                                       
c      ic(1,2)=17                                                       
c       ic(3,1)=6                                                       
c       ic(2,2)=6                                                       
c       cf(1,1)=0.3                                                     
c       cf(1,2)=45.0                                                    
c        cf(3,1)=0.8                                                    
c       cf(4,2)=20.0                                                    
c      alon=0.0                                                         
c      call morcigcm(p,t,h2o,o3,alat,htlw,htsw,doy,cf,ic,               
c     $     rfluxes,swalb,alon)                                         
c      enddo                                                            
c      do i=nl,1,-1                                                     
c         print '(i6,4F14.4)',i,p(i),t(i),htsw(i),htlw(i)               
c      enddo                                                            
c      print *,RFLUXES(1,1,1), RFLUXES(1,1,2), RFLUXES(1,2,1),          
c     $     RFLUXES(1,2,2)                                              
c      print *,RFLUXES(2,1,1), RFLUXES(2,1,2), RFLUXES(2,2,1),          
c     $     RFLUXES(2,2,2)                                              
c                                                                       
c                                                                       
c      end                                                              
c       INCLUDE 'zparb.upd'                                             
C                                                                       
C Piers'  stuff (defines arrays)                                        
       REAL PR(NRLEV),T(NRLEV),H2O(NRLEV),O3(NRLEV),alat                
       real htlw(nrlev),htsw(nrlev),swalb,DOY,alon                      
       REAL CF(4,2)     !cloud fractions,lwps                           
       INTEGER IC(4,2)   ! cloud positions                              
       REAL RFLUXES(2,2,2)                                              
C                                                                       
C                                                                       
C          1.     DEFINITIONS                                           
C                 -----------                                           
C                                                                       
C definitions for temperature profile                                   
                                                                        
      REAL ZTEMP(NLEVP1), ZPRES(NLEVP1), ZHEIG(NLEVP1)                  
                                                                        
C definitions for read in                                               
                                                                        
c      REAL DUMMY, T(NLEV), H2O(NLEV), O3(NLEV)                         
c     *                ,ZTM1(NLON,NLEV)                                 
       REAL DUMMY,ZTM1(NLON,NLEV),zzztm1(NLEV),zzzqm1(nlev),            
     *  zzzo3(nlev)                                                     
                                                                        
C channel number                                                        
                                                                        
      INTEGER C3EQUIL, IBRUEHL                                          
                                                                        
      CHARACTER*40 EQUILIB,IN_FILE                                      
                                                                        
      PARAMETER (C3EQUIL=14)                                            
                                                                        
C     COMMON BLOCK FOR GAS ABUNDANCES                                   
!-----------------------------------------------------                  
! Common block for gas abundances                                       
!-----------------------------------------------------                  
      COMMON/MORCGAS/VMRCO2,ZCARDI,VMRCH4,RCH4,VMRN2O,RN2O,             
     &               VMRCFC11,RCFC11,VMRCFC12,RCFC12,                   
     &               VMRH2OS,RH2OS,NEXOTIC                              
      LOGICAL :: NEXOTIC                                                
      REAL :: ZCARDI,RCH4,RN2O,RCFC11,RCFC12,RH2OS                      
      REAL :: VMRCO2,VMRCH4,VMRN2O,VMRCFC11,VMRCFC12,VMRH2OS            
C                                                                       
C local arrays                                                          
C                                                                       
C      DIMENSION ZCDIS(5)                                               
      DIMENSION ZALP(66),ZFOZQ(11),ZFOZH(11),ZFAES(21),ZFAEL(21),       
     #          ZFAEU(21),ZFAED(21), ZCDIS(5), ZCDEC(5), ZCEQT(5)       
C                                                                       
      INTEGER  ILAT, ILHT, ILST, IDIAL, IDIAS                           
c      INTEGER MMNTH(12), JDAY                                          
c      DATA MMNTH/15,45,74,105,135,166,196,227,258,288,319,349/         
c      REAL YTIME                                                       
C                                                                       
      ITASK=3                                                           
C                                                                       
C                                                                       
C define all constants                                                  
C                                                                       
      CALL INICON                                                       
C                                                                       
C define the latitude points                                            
C                                                                       
C Set up latitude grid                                                  
                                                                        
      XLAT(1)=3.141579*alat/180.0                                       
C calculate day                                                         
C                                                                       
C  loop over latitudes                                                  
C                                                                       
      DO 1111 ILAT=1,NLAT                                               
C                                                                       
C      WRITE(6,*) '***CALCULATE LATITUDE NUMBER ',ILAT,'***'            
C                                                                       
C Set up height grid (in terms of pressure)                             
C                                                                       
      IF (NLP2 .NE. NLON) WRITE(6,*) ' problems with vertical velocity' 
C      OPEN (11,FILE=IN_FILE,STATUS='OLD',FORM='FORMATTED')             
C                                                                       
C remark: Temp, vmrH2O, vmrO3 and vmrCO2 where changed to use           
C            bruehl.dat (IBRUEHL=1)                                     
C    important: see ZCARDI for changed vmrCO2, independent of IBRUEHL   
C                                                                       
      IBRUEHL=1                                                         
C                                                                       
C      READ(11,*) TSUPAR                                                
      DO 36 ILST=1,NLON                                                 
       DO 35 ILHT=1,NRLEV                                               
          JKL = NRLEV + 1 - ILHT                                        
          APM1(ILST,ILHT)=PR(JKL)                                       
          ZZZTM1(ILHT)=T(JKL)                                           
          ZZZQM1(ILHT)=H2O(JKL)                                         
          ZZZO3(ILHT)=O3(JKL)                                           
c          IF (JKL .NE. NRLEV) THEN                                     
c             APHM1(ILST,JKL+1) = (APM1(ILST,JKL+1)+APM1(ILST,JKL))/2.  
c          END IF                                                       
 35    CONTINUE                                                         
       DO 29 JKL=1,NRLEV                                                
          IF (JKL .NE. NRLEV) THEN                                      
             APHM1(ILST,JKL+1) = (APM1(ILST,JKL+1)+APM1(ILST,JKL))/2.   
          END IF                                                        
 29    CONTINUE                                                         
                                                                        
C                                                                       
        TSUPAR = ZZZTM1(NRLEV)                                          
        APHM1(ILST,1) = 0.                                              
C        APHM1(ILST,NRLEVP1)=101325.                                    
        APHM1(ILST,NRLEVP1)=APM1(ILST,NRLEV)                            
c        print *, 'surface press ',aphm1(ilst,nrlevp1)                  
C                                                                       
C                                                                       
C define moisture and vertical velocity  ==>  clouds                    
C                                                                       
       DO 37 ILHT=1,NRLEV                                               
c        print *, 'input ',ilht,apm1(ilst,ilht),zzztm1(ilht),           
c     $   zzzqm1(ilht),zzzo3(ilht),aphm1(1,ilht)                        
        VERVEL(ILST,ILHT)=0.                                            
        QM1(ILST,ILHT)=ZZZQM1(ILHT)                                     
 37    CONTINUE                                                         
                                                                        
 36   CONTINUE                                                          
C                                                                       
C      REWIND(11)                                                       
C      REWIND(12)                                                       
C                                                                       
C define maximal relative humidity                                      
C                                                                       
      DO 38 JK=1,NLEV                                                   
         CCRH(JK) = 1.                                                  
 38   CONTINUE                                                          
C                                                                       
      DO 33 J=1,NLEV                                                    
            TM1(1,J) = ZZZTM1(J)                                        
 33   CONTINUE                                                          
C                                                                       
      DO 39 JK=1,NLP2                                                   
        ALBM(JK) = SWALB                                                
        SNM1M(JK) = SNPAR                                               
        TSM1M(JK) = TSUPAR                                              
        LOLAND(JK) = LOLPAR                                             
        COSLON(JK) = 1.                                                 
        SINLON(JK) = 0.                                                 
 39   CONTINUE                                                          
      DATA ZALBICE/0.55/                                                
      DATA ZALBSEA/0.07/                                                
      DATA ZALBSNO/0.80/                                                
      DATA ZSNOWAL/0.01/                                                
C                                                                       
C     ZCARDI is the concentration of CO2 in mass mix. ratio             
C                                                                       
C                                                                       
      DATA ZEMISS/0.996/                                                
      DATA ZSUPSAT/0.01/                                                
      DATA ZCDIS/+1.000110,+0.034221,+0.001280,+0.000719,+0.000077/     
      DATA CRAE/+0.1277E-02/                                            
      DATA ZCDEC/+0.006918,-0.399912,+0.070257,-0.006758,+0.000907/     
      DATA ZCEQT/+0.000075,+0.001868,-0.032077,-0.014615,-0.040849/     
C                                                                       
C*    PHYSICAL CONSTANTS.                                               
C     -------- ----------                                               
C                                                                       
C          *ZQWSSAT* AND *ZQSNCR* ARE THE INVERSES OF CRITICAL VALUES   
C     FOR SOIL WATER AND SNOW DEPTH (SEE *VDIFF*). IN THE CODE ALSO     
C     APPEARS *ZCRH* FOR A MAXIMUM ALLOWED ATMOSPHERIC RELATIVE         
C     HUMIDITY.                                                         
C                                                                       
C      CQWSSAT = 1. / (0.75*0.02)                                       
C      ZQWSSAT=CQWSSAT                                                  
C      ZQSNCR=CQSNCR                                                    
C                                                                       
C*    SECURITY PARAMETER.                                               
C     -------- ----------                                               
C                                                                       
C     *ZEPSEC* AVOIDS 0/0 IN THE DIAGNOSTIC OF TOTAL CLOUD COVER.       
C     *ZEPCLC* IS A SECURITY TO AVOID ZERO OR ONE CLOUD COVERS AND      
C     *ZEPH2O* IS A SECURITY TO AVOID WATER VAPOUR CONTENT IN A LAYER   
C    *         TO BE MORE THEN THE RESPECTIVE VALUE AT SATURATION.      
C     *ZEPALB* IS A SECURITY TO AVOID ZERO ALBEDOS.                     
C                                                                       
      ZEPSEC=1.E-12                                                     
      ZEPCLC=1.E-12                                                     
      ZEPH2O=1.E-12                                                     
      ZEPALB=1.E-12                                                     
C                                                                       
C Setup SW code                                                         
C     Solar constant SOLC is now uniquely defined in INICON.            
C     M. Bourqui 15 nov 2001.                                           
                                                                        
      LDIUR=.FALSE.   !Diurnally averaged if false                      
C      LDIUR=.TRUE.   !Diurnally averaged if false                      
C                                                                       
       YCLOCK=3.14159      !time of day in radians                      
C       YCLOCK= (DOY-FLOAT(INT(DOY))*2.*3.14159                         
c       print *,doy,int(doy)                                            
C                                                                       
      CALL SOLANG(LDIUR,DOY,YCLOCK,ALAT,ALON,AMU0,RDAYL(1),CDISSEM)     
C                                                                       
                                                                        
      ZSCT=SOLC*CDISSEM   ! Solar const * earth-sun distance            
      PSOL=ZSCT*RDAYL(1)     ! * fractional day lengthC                 
C     OLD AEROSOL MODEL USED (NO DISTRIBUTION).                         
C                                                                       
      CAEROS=0.1462E-16                                                 
C                                                                       
C  ----------------------------------------------------------           
C                                                                       
C*         3.     FULL GRID COMPUTATIONS.                               
C                 ---- ---- -------------                               
C                                                                       
  300 CONTINUE                                                          
C                                                                       
C*         3.2     SATURATION SPECIFIC HUMIDITY AT ALL LAYERS.          
C                                                                       
  320 CONTINUE                                                          
      DO 322 JK=1,NLEV                                                  
      ZCRH=CCRH(JK)                                                     
      DO 321 JL=1,NLON                                                  
      LO=(TM1(JL,JK)-TMELT).GT.0.                                       
      ZCVM3=CVMGT(C3LES,C3IES,LO)                                       
      ZCVM4=CVMGT(C4LES,C4IES,LO)                                       
      ZQS(JL,JK)=ZCRH*C2ES*EXP(ZCVM3*(TM1(JL,JK)-TMELT)/(TM1(JL,JK)     
     *           -ZCVM4))/APM1(JL,JK)                                   
      ZQS(JL,JK)=ZQS(JL,JK)/(1.-VTMPC1*ZQS(JL,JK))                      
  321 CONTINUE                                                          
  322 CONTINUE                                                          
c      print *,'2.2'                                                    
C                                                                       
C*          3.3    EXTRAPOLATION AT THE TOP.                            
C                                                                       
  330 CONTINUE                                                          
C      DO 331 JL=1,NLON                                                 
C      ZTI(JL,1)=TM1(JL,1)-APM1(JL,1)*(TM1(JL,1)-TM1(JL,2))             
C     *                              /(APM1(JL,1)-APM1(JL,2))           
  331 CONTINUE                                                          
c      print *,'3.3',nlev                                               
                                                                        
C                                                                       
C*         3.4     VERTICAL INTERPOLATION.                              
C                                                                       
  340 CONTINUE                                                          
      DO 342 JK=2,NLEV                                                  
      DO 341 JL=1,NLON                                                  
c       print *,' vert ',jk,jl,aphm1(jl,jk)                             
      ZTI(JL,JK)=(TM1(JL,JK-1)*APM1(JL,JK-1)*(APM1(JL,JK)-APHM1(JL,JK)) 
     *           +TM1(JL,JK)*APM1(JL,JK)*(APHM1(JL,JK)-APM1(JL,JK-1)))  
     *           *(1./(APHM1(JL,JK)*(APM1(JL,JK)-APM1(JL,JK-1))))       
  341 CONTINUE                                                          
  342 CONTINUE                                                          
C                                                                       
c      print *,'3.4'                                                    
                                                                        
C*         3.5     SURFACE VALUES AND TOP BOUNDARY TEMPERATURE.         
C                                                                       
  350 CONTINUE                                                          
      DO 351 JL=1,NLON                                                  
      ZTI(JL,NLEVP1)=TSM1M(JL)                                          
      ZTI(JL,1)=TM1(JL,1)-APM1(JL,1)*(TM1(JL,1)-ZTI(JL,2))              
     *                              /(APM1(JL,1)-APHM1(JL,2))           
  351 CONTINUE                                                          
C                                                                       
c      print *,'3.5'                                                    
                                                                        
C*         3.6     MOISTURE AND CLOUDS.                                 
C                                                                       
  360 CONTINUE                                                          
C   Prescribe cloud amout and height                                    
      DO 361 JL=1,NLON                                                  
       ZCC(JL)=MIN(CF(1,1),1.0)                                         
       ZCH(JL)=MIN(CF(2,1),1.0)                                         
       ZCM(JL)=MIN(CF(3,1),1.0)                                         
       ZCL(JL)=MIN(CF(4,1),1.0)                                         
C Correction of the levels M.B. and PMF 27.06.01                        
       NTOP=NLEV+1-IC(1,2)                                              
       NBASE=NLEV+1-IC(1,1)                                             
       ITCC=NLEV+1-IC(1,1)                                              
       ITH=NLEV+1-IC(2,1)                                               
       ITM=NLEV+1-IC(3,1)                                               
       ITL=NLEV+1-IC(4,1)                                               
C***************************************                                
       DO JK=NRLEV,1,-1                                                 
        ZCLC(JL,JK)=0.0                                                 
        ZQLWP(JL,JK)=0.0                                                
       ENDDO                                                            
c Piers diagnostic cloud 1-7-97                                         
      IF (NBASE.GT.0.and.ZCC(JL).gt.1.0E-5)THEN                         
C cloud water ~< 15-20 mg/m3 in tropics from Su et al JGR 2011          
        DO ITCC=NTOP,NBASE                                              
          ZQLWP(JL,ITCC)=ZCC(JL)*5.0E-5                                 
          ZCLC(JL,ITCC)=ZCC(JL)                                         
        ENDDO                                                           
C deep conv cloudbase is 4x rest of cloud from Slingo et al 1987        
        if(apm1(jl,ntop)/apm1(jl,nrlev).lt.0.7)THEN                     
          ZQLWP(JL,NBASE)=ZQLWP(JL,NBASE)*4.0                           
          ZCLC(JL,NBASE)=ZCLC(JL,NBASE)*4.0                             
        ENDIF                                                           
      ENDIF                                                             
C LARGE SCALE CLOUDS parameterised by supersaturation.                  
C Note clouds occur at one level in IGCM so tuning to match             
C observations takes account of this esp for mid-level cloud            
                                                                        
      ZQLWP(JL,ITL)=ZQLWP(JL,ITL)+0.05*ZQS(JL,ITL)*ZCL(JL)              
      ZQLWP(JL,ITM)=ZQLWP(JL,ITM)+0.05*ZQS(JL,ITM)*ZCM(JL)              
      ZQLWP(JL,ITH)=ZQLWP(JL,ITH)+0.05*ZQS(JL,ITH)*ZCH(JL)              
      ZCLC(JL,ITL)=ZCLC(JL,ITL)+ZCL(JL)                                 
      ZCLC(JL,ITM)=ZCLC(JL,ITM)+ZCM(JL)                                 
      ZCLC(JL,ITH)=ZCLC(JL,ITH)+ZCH(JL)                                 
                                                                        
 361  CONTINUE                                                          
C                                                                       
      DO 363 JK=IRLEV,1,-1                                              
      DO 362 JL=1,NLON                                                  
      ZSW(JL,JK)=ZQS(JL,JK)                                             
c      IF (ZCLC(JL,JK).GT.0.0) THEN                                     
c       print '(a10,i5,2E13.3)','cloud ',jk,zclc(jl,jk),zqlwp(jl,jk)    
c      ENDIF                                                            
      ZCLC(JL,JK)=MIN(MAX(ZCLC(JL,JK),ZEPCLC),1.-ZEPCLC)                
      ZWV(JL,JK)=QM1(JL,JK)                                             
c zqlwp now mmr not g/m-2                                               
      LO=ZCLC(JL,JK).GT.ZEPCLC                                          
C      ZQLWP(JL,JK)=CVMGT(ZQLWP(JL,JK)/ZCLC(JL,JK),0.0,LO)              
C piers 10-6-97  as KPS changed ZQLWP i do too, LWP to g/m2             
C No extra tuning                                                       
C      ZQLWP(JL,JK)=CVMGT(ZQLWP(JL,JK),0.0,LO)/4.0                      
       if (ZCLC(JL,JK).le.ZEPCLC) then                                  
          ZQLWP(JL,JK)=0.0                                              
       endif                                                            
 362  CONTINUE                                                          
 363  CONTINUE                                                          
c         print *,' did clouds'                                         
C                                                                       
C*         3.7     INPUT DIAGNOSTICS FOR TEMPERATURE AND CLOUDS.        
C                                                                       
  370 CONTINUE                                                          
C                                                                       
C*         3.8     ALBEDO.                                              
C                                                                       
  380 CONTINUE                                                          
C                                                                       
      DO 381 JL=1,NLON                                                  
      LO1=TSM1M(JL)-CTFREEZ.GT.0.0                                      
      ZALTE(JL)=1.0-ZEMISS                                              
      ZALTE(JL)=MAX(ZALTE(JL),ZEPALB)                                   
      ZALSO(JL)=ALBM(JL)                                                
C     Bug fix: wrong lines of code removed. M. Bourqui 15 nov 2001      
  381 CONTINUE                                                          
C                                                                       
C     -------------------------------------------------                 
C                                                                       
c        print *,' 3.8'                                                 
C*         5.      SPLITTED LOOP OVER LATITUDE.                         
C*                 -------- ---- ---- ---------                         
C                                                                       
  500 CONTINUE                                                          
C                                                                       
      ISHIFT=0                                                          
      DO 551 JLOOP=1,NRPARTI                                            
C remark: change ishift if you really want splitted loop                
C                                                                       
C                                                                       
C*         5.1     INPUT: ALBEDO,ZENITH ANGLE, CO2, O3 AND AEROSOLS.    
C                                                                       
  510 CONTINUE                                                          
C                                                                       
      DO 511 JL=1,IRLONI                                                
      APRE(JL)=APHM1(JL+ISHIFT,NLEVP1)                                  
c      print *, ' apre ',apre(jl),nlevp1                                
      SMU0(JL)=AMU0(JL+ISHIFT)                                          
      ALSO(JL)=ZALSO(JL+ISHIFT)                                         
      ALTE(JL)=ZALTE(JL+ISHIFT)                                         
  511 CONTINUE                                                          
      DO 513 JK=1,IRLVP1                                                
      DO 512 JL=1,IRLONI                                                
      TI(JL,JK)=ZTI(JL+ISHIFT,JK)                                       
  512 CONTINUE                                                          
  513 CONTINUE                                                          
      DO 514 JL=1,IRLONI                                                
      ZDPO(JL)=APHM1(JL+ISHIFT,1)                                       
      ZQCFO(JL)=ZCARDI*ZDPO(JL)                                         
      ZAEQSO(JL)=CAEROS*APHM1(JL+ISHIFT,1)**3                           
c      ZQOFO(JL)=ZOZQ(JL+ISHIFT)*SQRT(ZDPO(JL)**3)/(SQRT(ZDPO(JL)**3)   
c     *          +ZOZH(JL+ISHIFT))                                      
  514 CONTINUE                                                          
      DO 517 JK=1,IRLEV                                                 
      DO 515 JL=1,IRLONI                                                
      ZDPN(JL)=APHM1(JL+ISHIFT,JK+1)                                    
      ZQCFN(JL)=ZCARDI*ZDPN(JL)                                         
      ZAEQSN(JL)=CAEROS*APHM1(JL+ISHIFT,JK+1)**3                        
c      ZQOFN(JL)=ZOZQ(JL+ISHIFT)*SQRT(ZDPN(JL)**3)/(SQRT(ZDPN(JL)**3)   
c     *          +ZOZH(JL+ISHIFT))                                      
      DP(JL,JK)=ZDPN(JL)-ZDPO(JL)                                       
      QCF(JL,JK)=ZQCFN(JL)-ZQCFO(JL)                                    
      AEQ1(JL,JK)=ZAEQSN(JL)-ZAEQSO(JL)                                 
      AADS(JL,JK)=0.001089*AEQ1(JL,JK)/DP(JL,JK)                        
c      QOF(JL,JK)=ZQOFN(JL)-ZQOFO(JL)                                   
  515 CONTINUE                                                          
      DO 516 JL=1,IRLONI                                                
      ZDPO(JL)=ZDPN(JL)                                                 
      ZQCFO(JL)=ZQCFN(JL)                                               
c      ZQOFO(JL)=ZQOFN(JL)                                              
      ZAEQSO(JL)=ZAEQSN(JL)                                             
  516 CONTINUE                                                          
  517 CONTINUE                                                          
c       print *,'5.1'                                                   
C                                                                       
C*         5.2     INPUT: MOISTURE AND CLOUDS.                          
C                                                                       
  520 CONTINUE                                                          
C                                                                       
      DO 522 JK=1,IRLEV                                                 
      DO 521 JL=1,IRLONI                                                
      SSW(JL,JK)=ZSW(JL+ISHIFT,JK)                                      
      WV(JL,JK)=ZWV(JL+ISHIFT,JK)                                       
      CLC(JL,JK)=ZCLC(JL+ISHIFT,JK)                                     
C      CLWA(JL,JK)=ZQLWP(JL+ISHIFT,JK)*ZCLC(JL+ISHIFT,JK)               
C Piers LWP g/m2                                                        
      CLWA(JL,JK)=ZQLWP(JL+ISHIFT,JK)                                   
  521 CONTINUE                                                          
  522 CONTINUE                                                          
c        print *,'5.2'                                                  
C                                                                       
C*         5.3     CALL TO *RADIA*.                                     
C                                                                       
  530 CONTINUE                                                          
C                                                                       
      IF (CAEROS.LT.1.E-14) THEN                                        
         DO 532 JK = 1 , IRLEV                                          
         DO 531 JL = 1 , IRLONI                                         
         AEQ1(JL,JK) = CAEROS                                           
         AEQ2(JL,JK) = CAEROS                                           
         AEQ3(JL,JK) = CAEROS                                           
         AEQ4(JL,JK) = CAEROS                                           
         AEQ5(JL,JK) = CAEROS                                           
 531     CONTINUE                                                       
 532     CONTINUE                                                       
      END IF                                                            
C                                                                       
      KRLEV2 = 2 * NRLEV                                                
      KNGL = 3 * NRLEV                                                  
      KNGLP1 = KNGL + 1                                                 
C                                                                       
C  Changes in case of IBRUEHL=1                                         
C                                                                       
c      print *, 'got hre'                                               
C  QOF is the mmr*DP                                                    
C                                                                       
      IF (IBRUEHL .EQ. 1) THEN                                          
         DO 533 JK=1,NLEV                                               
c           WV(1,JK) = 0.                                               
            WV(1,JK) = ZZZQM1(JK)                                       
C            CLC(1,JK) = 0.                                             
C            CLWA(1,JK) = 0.                                            
C            QOF(1,JK) = 0.                                             
C            QOF(1,JK) = O3(JK) * 48./28.96 * DP(1,JK)                  
             QOF(1,JK) = ZZZO3(JK)*DP(1,JK)                             
 533     CONTINUE                                                       
      END IF                                                            
C                                                                       
      DO 538 JK=1,NLEV                                                  
        DO 538 JL=1,NLON                                                
          ZTM1(JL,JK)=ZZZTM1(JK)                                        
 538  CONTINUE                                                          
C                                                                       
c      print *,' before call'                                           
c      print *,ztm1,pierslwp                                            
      CALL RADLSW (IRLONI, IRLST, IRLEV, IRLVP1, KRLEV2, KNGL, KNGLP1,  
     S             STBO, zsct, ZCARDI, ITASK, ZTM1,PIERSLWP)            
c      print *,' after call'                                            
C                                                                       
C                                                                       
C*         5.4     STORAGE OF THE OUTPUT.                               
C                                                                       
  540 CONTINUE                                                          
  550 CONTINUE                                                          
  551 CONTINUE                                                          
C***                                                                    
C*         6.2    STORAGE OF TRANSMISSIVITY AND EMISSIVITY.             
C                                                                       
  620 CONTINUE                                                          
C                                                                       
C                                                                       
C*         6.3    OUTPUT DIAGNOSTICS FOR HEATING RATES AND FLUXES.      
C                                                                       
  630 CONTINUE                                                          
C                                                                       
C write out damping rate at the various height levels                   
                                                                        
c         WRITE(C3EQUIL,*) '   '                                        
c         IF (LDIUR) THEN                                               
c           WRITE(C3EQUIL,188) JDAY, XLAT(ILAT),                        
c     #      YCLOCK*13750.987                                           
 188       FORMAT(' jday  =', I4, 6X, ' external',                      
     #            ' latitude =', F8.2, 10X,                             
     #            ' time in seconds (GMT) =', F12.2)                    
c         ELSE                                                          
c            WRITE(C3EQUIL,1881) JDAY, XLAT(ILAT)                       
 1881       FORMAT(' jday  =', I4, 6X, ' external',                     
     #            ' latitude =', F8.2, 10X,                             
     #            ' diurnally averaged ')                               
c         END IF                                                        
                                                                        
c         WRITE(C3EQUIL,*) '   '                                        
c         WRITE(C3EQUIL,*) '   '                                        
c         DO 181 JK=1,NLEVP1                                            
c            JKL = NLEVP1 + 1 - JK                                      
c            WRITE(C3EQUIL,191) ZFLUX(1,1,JKL), -ZFLUX(1,2,JKL),        
c     #               -FLT(1,JK),PPMB(1,JKL)*100., PTL(1,JKL),          
c     #               ZFSDWN(1,JKL), ZFSUP(1,JKL), FLS(1,JK)            
 191        FORMAT(8E15.4)                                              
 181     CONTINUE                                                       
c         WRITE(C3EQUIL,192)                                            
 192     FORMAT(6X,                                                     
     #     ' flxupLW ',6X,' flxdwLW ',6x,' netflxL ',6X,                
     #     ' half pr ',6X,' half te ',6X,' flxdwSW ',6X,                
     #     ' flxupSW ',6X,' netflxS ')                                  
c         WRITE(C3EQUIL,*) '   '                                        
c         WRITE(C3EQUIL,*) '   '                                        
c         WRITE(C3EQUIL,*) '   '                                        
c         DO 180 JL=1,NLON                                              
c           DO 180 JK=1,NLEV                                            
c            JKL = NLEV + 1 - JK                                        
c            PKASCH = -ZCOOLR(JL,JKL) + ZHEATR(JL,JKL)                  
cccm            DUMMY = PQOF(JL,JK)/DP(1,JK)*28.96/48.                  
c            DUMMY = PQOF(JL,JK)/DP(1,JK)                               
c            WRITE(C3EQUIL,190) JL, ILAT, JK,                           
c     >             ZHEATR(JL,JKL), WV(JL,JK),                          
c     >             DUMMY, PCLDLW(JL,JK),                               
c     >             -ZCOOLR(JL,JKL), PKASCH, APM1(JL,JK),               
c     >             TM1(JL,JK)                                          
c            WRITE(8,*) -ZCOOLR(JL,JKL),APM1(JL,JK)                     
c            WRITE(9,*)  ZHEATR(JL,JKL),APM1(JL,JK)                     
c 180     CONTINUE                                                      
 190     FORMAT(I3,1x,I4,1x,I4,1x,8E14.4)                               
c         WRITE(C3EQUIL,189)                                            
 189     FORMAT('Long  Lati  H ',4X,                                    
     #     'solar h ',6X,'moistur ',6X,' O3conc ',6X,                   
     #     'cldfrac ',6x,' netlwh ',6X,' nethrt ',6X,                   
     #     'pressur ',6X,'tempera ')                                    
c         WRITE(C3EQUIL,*) '   '                                        
C                                                                       
C  write out temperature and ozone column amounts                       
C                                                                       
C      OPEN (77,FILE='MIDTO3B6.BIN',FORM='UNFORMATTED',STATUS='NEW')    
C      DO 997 JL=1,NLON                                                 
C       ZCOLUM = 0.                                                     
C       DO 998 JK=1,NLEV                                                
CC         ZCOLUM=ZCOLUM+PQOF(JL,JK)*48./29./G*                         
CC     #          (APHM1(JL,JK+1)-APHM1(JL,JK))                         
C         ZCOLUM=ZCOLUM+PQOF(JL,JK)/G*100.*8314.3*273.16/101325./48.    
C         WRITE(77) ZCOLUM, APHM1(JL,JK), TM1(JL,JK)                    
CC         WRITE(6,*) 'JL,JK,ZCOLUM,APHM1:',JL,JK,ZCOLUM,APHM1(JL,JK)   
C 998   CONTINUE                                                        
C 997  CONTINUE                                                         
C      CLOSE (77)                                                       
C                                                                       
C  end loop over latitudes                                              
C                                                                       
 1111 CONTINUE                                                          
C                                                                       
      DO JKL=1,NLEV                                                     
        HTLW(JKL)=-ZCOOLR(1,JKL)                                        
        HTSW(JKL)=ZHEATR(1,JKL)                                         
c        print *,'zmorc ',jkl,htsw(jkl),htlw(jkl)                       
      ENDDO                                                             
c level 1 surface nrlevp1=top                                           
Crfluxes                                                                
C     1st index - flux 1=SW, 2=LW                                       
C     2nd index - Direction 1=DN, 2=UP                                  
C     3rd index - Where 1=TOP, 2=SURFACE                                
      rfluxes(1,1,1)=zfsdwn(1,nrlevp1)                                  
      rfluxes(1,1,2)=zfsdwn(1,1)                                        
      rfluxes(1,2,1)=zfsup(1,nrlevp1)                                   
      rfluxes(1,2,2)=zfsup(1,1)                                         
      rfluxes(2,1,1)=-zflux(1,2,nrlevp1)                                
      rfluxes(2,1,2)=-zflux(1,2,1)                                      
      rfluxes(2,2,1)=zflux(1,1,nrlevp1)                                 
      rfluxes(2,2,2)=zflux(1,1,1)                                       
                                                                        
ccc1      CLOSE (C2TEMP)                                                
C                                                                       
C     ---------------------------------------------------               
      END                                                               
C                                                                       
      SUBROUTINE DEDD ( KDLON, KRLST, KFLEV, ITASK )                    
C                                                                       
C**** *DEDD* - COMPUTES REFLECTIVITY, TRANSMISSIVITY OF A CLOUDY LAYER  
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C           COMPUTES THE REFLECTIVITY AND TRANSMISSIVITY OF A CLOUDY    
C     LAYER USING THE DELTA-EDDINGTON'S APPROXIMATION.                  
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C          *DEDD* IS CALLED BY *SW*.                                    
C                                                                       
C                                                                       
C        IMPLICIT ARGUMENTS                                             
C        --------------------                                           
C                                                                       
C     ==== INPUTS ===                                                   
C PGG    : (NDLON)             ; ASSYMETRY FACTOR                       
C PREF   : (NDLON)             ; REFLECTIVITY OF THE UNDERLYING LAYER   
C PRMUZ  : (NDLON)             ; COSINE OF SOLAR ZENITH ANGLE           
C PTO1   : (NDLON)             ; OPTICAL THICKNESS                      
C PW     : (NDLON)             ; SINGLE SCATTERING ALBEDO               
C     ==== OUTPUTS ===                                                  
C PRE1   : (NDLON)             ; LAYER REFLECTIVITY ASSUMING NO         
C                              ; REFLECTION FROM UNDERLYING LAYER       
C PTR1   : (NDLON)             ; LAYER TRANSMISSIVITY ASSUMING NO       
C                              ; REFLECTION FROM UNDERLYING LAYER       
C PRE2   : (NDLON)             ; LAYER REFLECTIVITY ASSUMING            
C                              ; REFLECTION FROM UNDERLYING LAYER       
C PTR2   : (NDLON)             ; LAYER TRANSMISSIVITY ASSUMING          
C                              ; REFLECTION FROM UNDERLYING LAYER       
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C          STANDARD DELTA-EDDINGTON LAYER CALCULATIONS.                 
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          NONE                                                         
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION AND          
C        ECMWF RESEARCH DEPARTMENT DOCUMENTATION                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C        JEAN-JACQUES MORCRETTE  *ECMWF*                                
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C        ORIGINAL : 88-12-15                                            
C     -----------------------------------------------------------       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRSW* SWITCHES RELATED TO THE RADIATION SCHEME.         
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      IMPLICIT REAL (A-H,O-Z)                                           
C      IMPLICIT UNDEFINED (Z)                                           
C      IMPLICIT LOGICAL (L)                                             
C      IMPLICIT UNDEFINED (L)                                           
      IMPLICIT INTEGER (I-K,M-N)                                        
C      IMPLICIT UNDEFINED (I-K,M-N)                                     
C      IMPLICIT REAL (P)                                                
C      IMPLICIT UNDEFINED (A-H,O,Q-Y)                                   
C                                                                       
C everything except the parameters is defined                           
C in the common blocks                                                  
C                                                                       
      COMMON/COM1/                                                      
C     I NRADFR, NRPART, NRINT, NRADPFR, NRADPLA,                        
C     I NRLSTI,                                                         
     R XLAT,                                                            
     R RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR, ZCCO2,              
C     R PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3,                 
     L LOH, LOM, LOL, LORAD                                             
      COMMON/COM2/                                                      
     I ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,                  
     I         JLOOP, KRLEV,                                            
     I        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     I        ITH, ITM, ITL,                                            
     I        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     I        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     I        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     I        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      COMMON/COM3/                                                      
     I IND4, KM12, KXD, KXU, IG, KBS, KDD,                              
     I        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     I        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     I        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     I        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     I        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
      COMMON/COM4/                                                      
     R           COZQC,COZQS,COZHC,COZHS,                               
     R CSUPSAT,                                                         
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
      COMMON/COM5/                                                      
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NUMVC,                                                           
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      COMMON/COM6/                                                      
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD,       
     R DTIME,    TWODT,    EPS,                                         
     R SINLON,     COSLON,                                              
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2,                          
     I NFAX,                                                            
     R CRAE,                                                            
     R CDISSEM                                                          
      COMMON/COM7/                                                      
     R     CCRH,                                                        
     L LO,LO1,LO2,LODIA,LODIAP,                                         
     R TAUA, PIZA, CGA, CAER, CAEROS                                    
      COMMON/COM7A/                                                     
     R  RT1,WG1,AT,BT,CNTNU,O1H,O2H,PIALF0                              
     S ,OCT,TREF,XP,TSTAND,GA,GB,CE,ETY,PTY                             
      COMMON/COM7B/                                                     
     S TINTP,TSTP,                                                      
     S  TBO3,AO3,BO3,TB,ALP,BET,ULG,ULGO,PLG,AP,BP,TO3B,O3A,O3B,        
     R          CDAY,CCO2,CH2O,DIFF,X10E                                
       COMMON/COM7C/                                                    
     S  ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW,                       
     R  APAD,BPAD,D,CRAY,SUN,                                           
     R   TTT,                                                           
     R   TM1                                                            
      COMMON/COM8/                                                      
     R       TSM1M,                                                     
     R       QM1,                                                       
     R       SNM1M,                                                     
     R       ALBM,                                                      
     R       AVVELM,                                                    
     R   SLMM,                                                          
     R       APHM1,                                                     
     R       APM1                                                       
      COMMON/COM9/                                                      
     L LOLAND,                                                          
     R   ZTI,                                                           
     *       ZCTI,                                                      
     *       ZCPH,                                                      
     *       ZWV,                                                       
     *       ZSW,                                                       
     *       ZALTE,                                                     
     *       ZCALTE,                                                    
     *       ZALSO                                                      
      COMMON/COM10/                                                     
     *       ZCALSO,                                                    
     R   ZOZQ,                                                          
     *       ZOZH,                                                      
     R   ZTRSOL,                                                        
     *       ZEMTER,                                                    
     R   TRSOLM,                                                        
     *       EMTERM,                                                    
     R   AMU0,                                                          
     *       RDAYL                                                      
      COMMON/COM11/                                                     
     R   CLC,                                                           
     *       TI,                                                        
     *       DP,                                                        
     *       AADS,                                                      
     *       AEQ1,                                                      
     *       AEQ2,                                                      
     *       AEQ3,                                                      
     *       AEQ4,                                                      
     *       AEQ5,                                                      
     *       SSW                                                        
      COMMON/COM12/                                                     
     *       WV,                                                        
     *       QOF,                                                       
     *       QCF,                                                       
     *       CLWA,                                                      
     *       APRE,                                                      
     *       SMU0,                                                      
     *       ALTE,                                                      
     *       ALSO                                                       
      COMMON/COM13/                                                     
     R   ZDPO,                                                          
     *       ZQCFO,                                                     
     *       ZAEQSO,                                                    
     *       ZQOFO,                                                     
     *       ZDPN,                                                      
     *       ZQCFN,                                                     
     *       ZAEQSN,                                                    
     *       ZQOFN                                                      
      COMMON/COM14/                                                     
     R   FLT,                                                           
     *       FLS,                                                       
     R   DIAG,                                                          
     *       DIA1,                                                      
     *       ZDIAG,                                                     
     *       ZDIA1,                                                     
     *       ZDIA,                                                      
     *       ZDIAF,                                                     
     *       ZDIAT,                                                     
     R PGG                                                              
      COMMON/COM15/                                                     
     #     PREF,                                                        
     #     PRE1,                                                        
     #     PRE2,                                                        
     #     PRMUZ,                                                       
     #     PTO1,                                                        
     #     PTR1,                                                        
     #     PTR2,                                                        
     #     PW,                                                          
     R     PALBSN                                                       
      COMMON/COM16/                                                     
     R ZABSCU,                                                          
     S     ZABSLY,                                                      
     S     ZADJD ,                                                      
     S     ZADJU ,                                                      
     S     ZBGND ,                                                      
     S     ZBINT ,                                                      
     S     ZBSUR ,                                                      
     S     ZBSUIN,                                                      
     S     ZBTOP ,                                                      
     S     PCLDLW,                                                      
     S     ZCLOUD,                                                      
     S     ZCNTRB,                                                      
     S     ZCUMP ,                                                      
     S     ZCUMPT,                                                      
     S     ZDBDT                                                        
      COMMON/COM17/                                                     
     S     ZDBSL ,                                                      
     S     ZDISTD,                                                      
     S     ZDISTU,                                                      
     S     ZDNF  ,                                                      
     S     PDT0  ,                                                      
     S     ZDUC                                                         
      COMMON/COM18/                                                     
     S     ZEFFT ,                                                      
     S     ZEMU  ,                                                      
     S     ZEPHIO,                                                      
     S     ZEPSC2,                                                      
     S     ZEPSC3,                                                      
     S     ZEPSH1,                                                      
     S     ZEPSH2,                                                      
     S     ZEPSH3                                                       
      COMMON/COM19/                                                     
     S     ZEPSH4,                                                      
     S     ZEPSH5,                                                      
     S     ZEPSH6,                                                      
     S     ZEPSIO,                                                      
     S     ZFACTC,                                                      
     S     ZZFD  ,                                                      
     S     ZFDD  ,                                                      
     S     ZFDN                                                         
      COMMON/COM20/                                                     
     S     PFLUX , PFLUC,                                               
     S     ZFLWP , PCTS                                                 
     S     ZZFU  ,                                                      
     S     ZFUP  ,                                                      
     S     ZGLAYD,                                                      
     S     ZGLAYU,                                                      
     S     ZOMEGA,                                                      
     S     ITX   ,                                                      
     S     IXDIA ,                                                      
     S     IXT   ,                                                      
     S     IXTSU ,                                                      
     S     IXTTP ,                                                      
     S     IMX   ,                                                      
     S     IMXP                                                         
      COMMON/COM21/                                                     
     R ZOZ   ,                                                          
     S     PPMB  ,                                                      
     S     ZRES  , PB                                                   
     S     ZSSIG ,                                                      
     S     PTAU  ,                                                      
     S     PTAVE ,                                                      
     S     ZTAVIC,                                                      
     S     ZTI1  ,                                                      
     S     PTL                                                          
      COMMON/COM22/                                                     
     S     ZTT   ,                                                      
     S     ZTT1  ,                                                      
     S     ZTT2  ,                                                      
     S     ZUAER ,                                                      
     S     ZUPF  ,                                                      
     S     ZUU   ,                                                      
     S     ZXD                                                          
      COMMON/COM23/                                                     
     S     ZXN   ,                                                      
     S     ZXOZ  ,                                                      
     S     ZXWV  ,                                                      
     S     ZZZ   ,                                                      
     S     ZRES2 ,                                                      
     S     ZTI2  ,                                                      
     S     ZBLAY ,                                                      
     S     ZBLEV ,                                                      
     S     ZCTS  ,                                                      
     S     ZHFG                                                         
      COMMON/COM24/                                                     
     I KT,                                                              
     R    PUU   ,                                                       
     S        PTT   ,                                                   
     S        ZZ                                                        
     R  PCLFR  ,                                                        
     S      PT     ,                                                    
     S      PQOF   ,                                                    
     S      PQLWP  ,                                                    
     S      PEMIS  ,                                                    
     S      PFLT                                                        
      COMMON/COM25/                                                     
     R  ZALBSU,                                                         
     S      ZCG   ,                                                     
     S      ZCLDLW,                                                     
     S      ZCLDSW,                                                     
     S      ZCOOLR,                                                     
     S      ZDT0  ,                                                     
     S      ZEMD  ,                                                     
     S      ZFLUX ,                                                     
     S      ZPMB  ,                                                     
     S      ZTAU  ,                                                     
     S      ZTAVE ,                                                     
     S      ZTL   ,                                                     
     S      ZFSDWN                                                      
      COMMON/COM26/                                                     
     S      ZFSUP ,                                                     
     S      ZHEATR,                                                     
     R   PDP   ,                                                        
     S       PWV   ,                                                    
     S       PFLS  ,                                                    
     S       PPSOL ,                                                    
     S       PRMU0 ,                                                    
     S       PAER  ,                                                    
     S       PALBS ,                                                    
     S       PCG   ,                                                    
     S       PCLDSW,                                                    
     S       POMEGA,                                                    
     S       POZ                                                        
      COMMON/COM27/                                                     
     R   PFDOWN,                                                        
     S       PFUP  ,                                                    
     R   ZAKI  ,                                                        
     S       ZCGAZ ,                                                    
     S       ZCNEB ,                                                    
     S       ZC1I  ,                                                    
     S       ZDSC  ,                                                    
     S       ZDSH                                                       
      COMMON/COM28/                                                     
     S       ZDSIG ,                                                    
     S       ZFACT ,                                                    
     S       ZFACT1 ,                                                   
     S       ZFACT2 ,                                                   
     S       ZFACT3 ,                                                   
     S       ZFACT4 ,                                                   
     S       ZFACT5 ,                                                   
     S       ZFACT6 ,                                                   
     S       ZFD   ,                                                    
     S       ZFRAC ,                                                    
     S       ZFU   ,                                                    
     S       ZG    ,                                                    
     S       ZGG   ,                                                    
     S       ZPIZAZ                                                     
      COMMON/COM29/                                                     
     S       ZRAYL ,                                                    
     S       ZRAY1 ,                                                    
     S       ZRAY2 ,                                                    
     S       ZREF  ,                                                    
     S       ZREFZ ,                                                    
     S       ZRE1  ,                                                    
     S       ZRE2  ,                                                    
     S       ZRJ   ,                                                    
     S       ZRK   ,                                                    
     S       ZRL   ,                                                    
     S       ZRMU  ,                                                    
     S       ZRMUE ,                                                    
     S       ZRMUZ                                                      
      COMMON/COM30/                                                     
     S       ZRNEB ,                                                    
     S       ZRUEF ,                                                    
     S       ZR1   ,                                                    
     S       ZR2   ,                                                    
     S       ZR21  ,                                                    
     S       ZR22  ,                                                    
     S       ZR23  ,                                                    
     S       ZS    ,                                                    
     S       ZSEC  ,                                                    
     S       ZSS1  ,                                                    
     S       ZTAUAZ                                                     
      COMMON/COM31/                                                     
     S       ZTOT  ,                                                    
     S       ZTO1  ,                                                    
     S       ZTR   ,                                                    
     S       ZTRA1 ,                                                    
     S       ZTRA2 ,                                                    
     S       ZTR1                                                       
      COMMON/COM32/                                                     
     S       ZTR2  ,                                                    
     S       ZUD   ,                                                    
     S       ZUM   ,                                                    
     S       ZU1D  ,                                                    
     S       ZU2D  ,                                                    
     S       ZW    ,                                                    
     S       ZN175 ,                                                    
     S       ZN190 ,                                                    
     S       ZO175 ,                                                    
     S       ZO190 ,                                                    
     S       ZP75  ,                                                    
     S       ZP90  ,                                                    
     S       ZSIGN ,                                                    
     S       ZSIGO                                                      
      COMMON/COM33/                                                     
     R   PTR,                                                           
     S       PU,                                                        
     R   ZRH,                                                           
     *       ZCH,                                                       
     *       ZCM,                                                       
     *       ZCL,                                                       
     *       ZCT,                                                       
     *       ZCC,                                                       
     I NTOPCM                                                           
      COMMON/COM34/                                                     
     *        NBASECM,                                                  
     R   ARPRCM,                                                        
     I ITOPH,                                                           
     *        ITOPM,                                                    
     *        ITOPL,                                                    
     L LOC,                                                             
     R   ZQS,                                                           
     *       ZCLC,                                                      
     *       ZQLWP                                                      
      COMMON/COM35/                                                     
     R   VERVEL,                                                        
     *       ZTHETA,                                                    
     *       ZETA,                                                      
     *       ZDTHMIN,                                                   
     I INVB,                                                            
     R   ZCTLWP,                                                        
     *       ZQLOC,                                                     
     *       ZQMEC,                                                     
     *       ZQHIC,                                                     
     R   ZFSOT,                                                         
     #       ZFTET,                                                     
     #       ZFLT                                                       
      COMMON/COM36/                                                     
     #       ZFSO ,                                                     
     #       ZFTE ,                                                     
     #       ZFLB ,                                                     
     #       ZDTDT,                                                     
     #       HEATO,                                                     
     #       HEATE,                                                     
     #       HEATR,                                                     
     #       ZI0                                                        
C                                                                       
      INTEGER                                                           
C     I NRADFR,   NRPART,   NRINT,    NRLSTI,  NRADPFR,  NRADPLA,       
     I NRLST,    NRLEV,    NRLEVP1, NLEV, NLP2, NLEVP1,                 
     I NLON, IRLEV, NTRA, NLAT, ITIM, NUA, NINT, NRPARTI                
C                                                                       
C   NRLEV .... number of vertical levels                                
C   NRLST .... number of points on a latitude line                      
C   NUA   .... number of absorber amount                                
C   NINT  .... number of spectral intervals                             
C   NTRA  .... number of transmission functions                         
C   NLAT  .... number of latitude points                                
C   ITIM  .... number of month under consideration                      
C                                                                       
C      PARAMETER (NRLEV=122,NUA=34,NINT=6,NTRA=34, NLAT=1,              
      PARAMETER (NRLEV=21,NUA=34,NINT=6,NTRA=34, NLAT=1,                
cccm      PARAMETER (NRLEV=37,NUA=18,NINT=6,NTRA=19, NLAT=1,            
cccm     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=10,NLEV=NRLEV,        
     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=7,NLEV=NRLEV,             
     #           NLP2=NRLST, NLEVP1=NRLEVP1, NLON=NRLST,                
     #           NRPARTI=1)                                             
C                                                                       
      REAL YCLOCK, XLAT(NLAT), Z0, DZ, ALBMPAR, SNPAR,                  
     #     TSUPAR, CTFREEZ, DELTAC, DELTAO, DELTAH                      
C                                                                       
C   YTIME  .... time of the year in radians                             
C   YCLOCK .... time of the day in radians                              
C   XLAT   .... latitudes in radians                                    
C   Z0     .... zero point of height grid                               
C   DZ     .... Interval between height levels                          
C   ALBMPAR.... sw surface albedo as function of longitude:             
C           yearly averaged sea-ice: 0.55                               
C                           water  : 0.07                               
C                           land   : 0.07.lt.ALBM.lt.0.80               
C                                    depending on snow                  
C   SNPAR  .... snow depth in meter                                     
C   TSUPAR .... surface temperature (Kelvin)                            
C   CTFREEZ.... freezing temperature of sea-water (Kelvin)              
C   DELTAC .... Doppler parameter for line shapes of CO2                
C   DELTAO .... Doppler parameter for line shapes of O3                 
C                                                                       
      PARAMETER (Z0=0., CTFREEZ=271.26,                                 
C     PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
     #           DZ=0.289, ALBMPAR=0.2, SNPAR=0.,                       
cccm  PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
cccm     #           DZ=0.15, ALBMPAR=0.3, SNPAR=0.,                    
CZ     #           DELTAC=0., DELTAO=0.)                                
     #           DELTAC=6.0E-04, DELTAO=1.E-02, DELTAH=5.0E-04)         
C      PARAMETER (YCLOCK=0., Z0=0., TSUPAR=273., CTFREEZ=271.26,        
C     #           DZ=0.46875, ALBMPAR=0.07, SNPAR=0.)                   
C      PARAMETER (YCLOCK=0., Z0=2.608, TSUPAR=273., CTFREEZ=271.26,     
C     #           DZ=0.2878, ALBMPAR=0.07, SNPAR=0.)                    
C                                                                       
      LOGICAL LDIUR, LOLPAR                                             
C                                                                       
C   LDIUR .... true: sun at time, false: diurnally averaged             
C   LOLPAR.... true: land surface, false: else                          
C                                                                       
C      PARAMETER (LDIUR=.TRUE., LOLPAR=.TRUE.)                          
       PARAMETER (LOLPAR=.TRUE.)                                        
C                                                                       
      INTEGER  C2TEMP                                                   
      CHARACTER*64  MIDTMP                                              
cccm      PARAMETER (C2TEMP=10,                                         
cccm     #    MIDTMP='/home/atm/ugamp/midrad/radfildat.dir/MIDTMP.DAT') 
ccc1      PARAMETER (C2TEMP=10, MIDTMP='tprfil2.dat')                   
C                                                                       
      INTEGER  IRLONI, IRLST, IRLVP1, JPNLEV, JPNLVP1                   
C                                                                       
      PARAMETER (IRLONI=NRLST, IRLST=NRLST,                             
     #           IRLVP1=NRLEVP1, JPNLEV=NRLEV,                          
     #           JPNLVP1=NRLEVP1, IRLEV=NRLEV)                          
C                                                                       
      INTEGER NIPD, NIPD2, NTR, NG1, NG1P1                              
      PARAMETER(NIPD=8,NIPD2=16,NTR=11,NG1=2,NG1P1=3)                   
C                                                                       
C    definition of the functions                                        
C                                                                       
      REAL       CVMGT, SIGMA2                                          
      INTEGER    CVMGI                                                  
C                                                                       
C    definitons of the subroutine cloud                                 
C                                                                       
      REAL RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR                  
C                                                                       
C    missing definitions in morcrette                                   
C                                                                       
C      REAL   PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3            
      REAL ZCCO2                                                        
                                                                        
      LOGICAL LOH, LOM, LOL, LORAD                                      
C                                                                       
      INTEGER ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,           
     #        IZQS, IZCLC, IZQLWP, JLOOP, KRLEV,                        
     #        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     #        ITH, ITM, ITL,                                            
     #        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     #        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     #        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     #        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      INTEGER IND4, KM12, KXD, KXU, IG, KBS, KDD,                       
     #        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     #        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     #        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     #        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     #        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD1* SPECTRAL DISTRIBUTION OF AEROSOLS AND OZONE.     
C                     (TRIANGULAR *T10* TRUNCATIONFOR AEROSOLS).        
C                     (TRIANGULAR *T5* TRUNCATIONFOR OZONE).            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R           COZQC(21),COZQS(15),COZHC(21),COZHS(15)                
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD2* CONSTANTS INTERNAL TO THE RADIATION SUBROUTINES. 
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CSUPSAT                                                          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMCON* BASIC UNIVERSAL CONSTANTS AND DERIVED CONSTANTS.  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
C                                                                       
C                                                                       
C       --------------------------------------------------------------- 
C      PARAMETER(                                                       
C     I JPM=106,JPN=106,JPK=106,JPGL=160,JPNLON=320,                    
C     I JPT1MAX=400,          JPT2MAX=50,                               
C     I JPT3MAX=300,          JPT4MAX=50,                               
C     I JPHASH=2048,          JPNTRCRS=20,                              
C     I JPTASKS=4,     JPLOCKS=50,    JPEVENT=4,                        
C     I JPLTMT=JPTASKS,                                                 
C     I JPNUMW=352,            JPNUMP=352,                              
C     I JPTABLE=1024,                                                   
C     I JPMAXB =60,                                                     
C     I JPMAXPB=32,                                                     
C     I JPG3FLD=43,         JPG3XF=99,         JPG3CMP=17,              
C     I JPMP1=JPM+1,        JPNP1=JPN+1,       JPKP1=JPK+1,             
C     I JP2MP1=JPMP1*2,     JP4MP1=JPMP1*4,                             
C     I JPED=JPN/2+1,       JPOD=JPNP1/2,                               
C     I IPSP0=JPM+JPN-JPK,  IPSP1=JPNP1*JPMP1, IPSP2=IPSP0*(IPSP0+1)/2, 
C     I JPSP=IPSP1-IPSP2,   JP2SP=2*JPSP,                               
C     I JPHGL=JPGL/2,       JPNLP2=JPNLON+2,                            
C     I                     JPNVCLV=4*JPNLVP1)                          
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C     J. K. GIBSON     E.C.M.W.F.     22/01/82.                         
C     MODIFIED M.J     E.C.M.W.F.     13/12/82.                         
C                                                                       
      INTEGER                                                           
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      LOGICAL                                                           
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD        
      REAL                                                              
     R DTIME,    TWODT,    EPS                                          
C                                                                       
C      INTEGER                                                          
C     I NWTIME(JPNUMW),     NROW(JPTASKS),                              
C     I NPTIME(JPNUMP)                                                  
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMGAU* - QUANTITIES RELATED TO THE GAUSSIAN GRID.      
C                                                                       
      REAL                                                              
     R SINLON(NLP2),     COSLON(NLP2)                                   
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMTRU* -QUANTITIES RELATED TO THE SPECTRAL TRUNCATION. 
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSIM* - QUANTITIES NEEDED FOR THE SEMI-IMPLICIT SCHEME
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMHDI* - COEFFICIENTS FOR HORIZONTAL DIFFUSION.        
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMFFT* - QUANTITIES NEEDED FOR THE FAST *FOURIER       
C                         TRANSFORMS.                                   
C                                                                       
      INTEGER                                                           
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2                           
C      REAL                                                             
C     R TRIG(JPNLON)                                                    
      INTEGER                                                           
     I NFAX(10)                                                         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSTA* - ARRAYS AND PARAMETERS FOR STATISTICS.         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMDIA* - QUANTITIES RELATED TO GLOBAL PHYSICAL         
C                         DIAGNOSTICS.                                  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPSW* SWITCHES RELATED TO THE PARAMETERISATIONS OF      
C                     DIABATIC PROCESSES.                               
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMDSW* SWITCHES RELATED TO THE DYNAMICS                  
C                     AND THE GENERAL CONTROL OF THE FORECAST.          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH1* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE RADIATION SUBROUTINES.                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CRAE,                                                            
     R CDISSEM                                                          
C                                                                       
C      REAL       COSRAD(128),        SINRAD(128)                       
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH2* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE PHYSICAL SUBROUTINES (EXCEPT RADIATION    
C                     ONES).                                            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL       CCRH(JPNLEV)                                           
C                                                                       
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMDIZ* VARIABLES AND POINTERS FOR ZONAL MEAN DIAGNOSTICS 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMMSK* VARIABLES AND POINTERS FOR AREA MEAN DIAGNOSTICS. 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMSDS* - START/RESTART BLOCK.                          
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMTASK* - TASK RELATED VARIABLES.                      
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*         *COMRAD* - RADIATION POINTERS.                               
C                                                                       
      LOGICAL LO,LO1,LO2,LODIA,LODIAP                                   
C---------------------------------------------------------------------  
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C     COMMON *YOMAER* RADIATIVE CHARACTERISTICS OF THE AEROSOLS         
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
      REAL TAUA(2,5), PIZA(2,5), CGA(2,5), CAER(5,5), CAEROS            
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C     COMMON *YOMLW*                                                    
C                                                                       
C*    COEFFICIENTS FOR THE LONGWAVE RADIATION SUBROUTINES               
C                                                                       
C NINT  :  NUMBER OF SPECTRAL INTERVALS                                 
C NIPD  :  NUMBER OF BANDS DESCRIBED BY PADE APPROXIMANTS               
C NTR   :  NUMBER OF REFERENCE TEMPERATURES                             
C NTRA  :  NUMBER OF TRANSMISSION FUNCTIONS                             
C NUA   :  NUMBER OF ABSORBER AMOUNTS                                   
C NG1   :  ORDER OF THE GAUSSIAN QUADRATURE                             
C                                                                       
C RT1   :  ROOTS OF THE GAUSSIAN POLYNOMIAL                             
C WG1   :  WEIGHTS OF THE GAUSSIAN POLYNOMIAL                           
C AT, BT:  COEFFICIENTS OF THE TEMPERATURE DEPENDENCE                   
C CNTNU :  H2O CONTINUUM ABSORPTION COEFFICIENTS                        
C O1H,O2H,PIALF0,OCT: OZONE LONGWAVE ABSORPTION COEFFICIENTS            
C TREF, TSTAND      : REFERENCE TEMPERATURES                            
C XP    :  COEFFICIENTS OF POLYNOMIAL DEVELOPMENTS OF PLANCK FUNCTION   
C GA,GB :  PADE APPROXIMANTS OF H2O, CO2 TRANSMISSION FUNCTIONS         
C ETY,PTY :  PARAMETERS OF H2O CONTINUUM                                
C TINTP :  REFERENCE TEMPERATURES OF PRE-COMPUTED TRANSMISSIVITIES      
C CE    :  EMS COEFFS OF H2O TRANSMISSION FUNCTIONS                     
C                                                                       
      REAL  RT1(2),WG1(2),AT(12,3),BT(12,3),CNTNU(4),O1H                
     S ,OCT(4),TREF,XP(6,6),TSTAND,GA(11,24,3),GB(11,24,3)              
     S ,PTY(11,20),ETY(11,20),TINTP(11),TSTP,O2H,PIALF0                 
C                                                                       
      REAL  TB(40,31,2),ALP(40,31,2),BET(40,31,2),                      
     S ULG(40),PLG(31),AP(40,31,2),BP(40,31,2),TBO3(38,31),TO3B(38,31)  
     S,ULGO(38),AO3(38,31),BO3(38,31),O3A(38,31),O3B(38,31)             
C     ----------------------------------------------------------------- 
C*COMDECK YOMRDU                                                        
C     ----------------------------------------------------------------- 
C                                                                       
C*    CONTROL VARIABLES FOR RADIATION: INTERNAL SWITCH AND INDICES      
C                                                                       
C ZEELOG:  SECURITY THRESHOLD FOR ABSORBER AMOUNT IN LAPLACE TRANSFORM  
C ZEPSC :  SECURITY THRESHOLD FOR CLOUD COVER                           
C ZEPSCO:  SECURITY THRESHOLD FOR OZONE AMOUNT                          
C ZEPSCQ:  SECURITY THRESHOLD FOR WATER VAPOR                           
C ZEPSCT:  SECURITY THRESHOLD FOR SHORTWAVE OPTICAL THICKNESS           
C ZEPSCW:  SECURITY THRESHOLD FOR CLOUD LIQUID WATER PATH               
C                                                                       
      REAL          CDAY,CCO2,CH2O,DIFF,X10E                            
     S             ,ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW            
C     ----------------------------------------------------------------- 
C*COMDECK YOMSW                                                         
C     ----------------------------------------------------------------- 
C                                                                       
C*    COEFFICIENTS FOR THE SHORTWAVE RADIATION SUBROUTINES              
C                                                                       
C APAD  :  PADE APPROXIMANTS NUMERATOR                                  
C BPAD  :  PADE APPROXIMANTS DENOMINATOR                                
C D     :  TRANSMISSION LIMIT FOR INFINITE ABSORBER AMOUNT              
C CRAY  :  RAYLEIGH SCATTERING COEFFICIENTS                             
C SUN   :  SOLAR FRACTION IN SPECTRAL INTERVALS                         
C                                                                       
      REAL  APAD(2,3,7),BPAD(2,3,7),D(2,3),CRAY(2,6),SUN(2)             
C     ----------------------------------------------------------------- 
                                                                        
C                                                                       
C  -------------------------------------------------------------------- 
C                                                                       
      REAL   TTT(1,NLAT,NRLEV,12)                                       
C                                                                       
C        0.1  "RADINT POINTERS"                                         
C             -----------------                                         
C                                                                       
      REAL   TM1(NLP2,NLEV),                                            
     *       TSM1M(NLP2),                                               
     *       QM1(NLP2,NLEV),                                            
     *       SNM1M(NLP2),                                               
     *       ALBM(NLP2),                                                
     *       AVVELM(NLP2)                                               
      REAL   SLMM(NLP2),                                                
     *       APHM1(NLP2,NLEVP1),                                        
     *       APM1(NLP2,NLEV)                                            
C                                                                       
      LOGICAL LOLAND(NLP2)                                              
C                                                                       
C     VARIABLES FOR FULL GRID AND TRANSFORMS.                           
C                                                                       
      REAL   ZTI(NLP2,NRLEVP1),                                         
     *       ZCTI(NLP2,NRLEVP1),                                        
     *       ZCPH(NLP2,NRLEVP1),                                        
     *       ZWV(NLP2,NRLEV),                                           
     *       ZSW(NLP2,NRLEV),                                           
     *       ZALTE(NLP2),                                               
     *       ZCALTE(NLP2),                                              
     *       ZALSO(NLP2),                                               
     *       ZCALSO(NLP2)                                               
C                                                                       
      REAL   ZOZQ(NLP2),                                                
     #       ZOZH(NLP2)                                                 
C                                                                       
      REAL   ZTRSOL(NLP2,NLEVP1),                                       
     *       ZEMTER(NLP2,NLEVP1)                                        
C                                                                       
C     FINAL RESULTS ARE STORED ON:                                      
C                                                                       
      REAL   TRSOLM(NLP2,NLEVP1),                                       
     *       EMTERM(NLP2,NLEVP1)                                        
C                                                                       
      REAL   AMU0(NLP2),                                                
     *       RDAYL(NLP2)                                                
C                                                                       
C     VARIABLES WITHIN THE SPLITTED LATITUDE LOOP.                      
C                                                                       
      REAL   CLC(NRLST,NRLEV),                                          
     *       TI(NRLST,NRLEVP1),                                         
     *       DP(NRLST,NRLEV),                                           
     *       AADS(NRLST,NRLEV),                                         
     *       AEQ1(NRLST,NRLEV),                                         
     *       AEQ2(NRLST,NRLEV),                                         
     *       AEQ3(NRLST,NRLEV),                                         
     *       AEQ4(NRLST,NRLEV),                                         
     *       AEQ5(NRLST,NRLEV),                                         
     *       SSW(NRLST,NRLEV),                                          
     *       WV(NRLST,NRLEV),                                           
     *       QOF(NRLST,NRLEV),                                          
     *       QCF(NRLST,NRLEV),                                          
     *       CLWA(NRLST,NRLEV),                                         
     *       APRE(NRLST),                                               
     *       SMU0(NRLST),                                               
     *       ALTE(NRLST),                                               
     *       ALSO(NRLST)                                                
      REAL   ZDPO(NRLST),                                               
     *       ZQCFO(NRLST),                                              
     *       ZAEQSO(NRLST),                                             
     *       ZQOFO(NRLST),                                              
     *       ZDPN(NRLST),                                               
     *       ZQCFN(NRLST),                                              
     *       ZAEQSN(NRLST),                                             
     *       ZQOFN(NRLST)                                               
      REAL   FLT(NRLST,NRLEVP1),                                        
     *       FLS(NRLST,NRLEVP1)                                         
C                                                                       
C     DIAGNOSTIC VARIABLES.                                             
C                                                                       
      REAL   DIAG(NLEVP1,4),                                            
     *       DIA1(7),                                                   
     *       ZDIAG(NLEVP1,10),                                          
     *       ZDIA1(21),                                                 
     *       ZDIA(NLON),                                                
     *       ZDIAF(NLON,7),                                             
     *       ZDIAT(NLEVP1)                                              
C                                                                       
C                                                                       
C                                                                       
      REAL PGG(IRLST),                                                  
     #     PREF(IRLST),                                                 
     #     PRE1(IRLST),                                                 
     #     PRE2(IRLST),                                                 
     #     PRMUZ(IRLST),                                                
     #     PTO1(IRLST),                                                 
     #     PTR1(IRLST),                                                 
     #     PTR2(IRLST),                                                 
     #     PW(IRLST)                                                    
C                                                                       
C                                                                       
C  what about the 1-dimensional PALBS???                                
C                                                                       
      REAL     PALBSN  (IRLST)                                          
C                                                                       
C*       0.2   "LOCAL" ARRAYS                                           
C              --------------                                           
      REAL ZABSCU(IRLST ,NUA, 3*NRLEV+1),                               
     S     ZABSLY(IRLST ,NUA, 3*NRLEV+1) ,                              
     S     ZADJD (IRLST , IRLVP1)      ,                                
     S     ZADJU (IRLST , IRLVP1)       ,                               
     S     ZBGND (IRLST)                ,                               
     S     ZBINT (IRLST , IRLVP1)       ,                               
     S     ZBSUR (IRLST , NINT)         ,                               
     S     ZBSUIN(IRLST)                ,                               
     S     ZBTOP (IRLST , NINT)         ,                               
     S     PCLDLW(IRLST , IRLEV)        ,                               
     S     ZCLOUD(IRLST)                ,                               
     S     ZCNTRB(IRLST , IRLVP1,IRLVP1),                               
     S     ZCUMP (IRLST)                ,                               
     S     ZCUMPT(IRLST)                ,                               
     S     ZDBDT (IRLST , NINT , IRLEV) ,                               
     S     ZDBSL (IRLST , NINT , 2*NRLEV),                              
     S     ZDISTD(IRLST , IRLVP1)       ,                               
     S     ZDISTU(IRLST , IRLVP1)                                       
      REAL                                                              
     S     ZDNF  (IRLST , IRLVP1,IRLVP1),                               
     S     PDT0  (IRLST)                                                
      REAL                                                              
     S     ZDUC  (IRLST , 3*NRLEV+1)       ,                            
     S     ZEFFT (IRLST)                ,                               
     S     ZEMD  (IRLST , IRLEV), ZEMU  (IRLST , IRLEV),                
     S     ZEPSC2(IRLST),  ZEPSC3(IRLST),                               
     S     ZEPSH1(IRLST),  ZEPSH2(IRLST),                               
     S     ZEPSH3(IRLST),  ZEPSH4(IRLST),                               
     S     ZEPSH5(IRLST),  ZEPSH6(IRLST),                               
     S     ZEPHIO(IRLST),  ZFACTC(IRLST),                               
     S     ZZFD(IRLST), ZFDD(IRLST), ZFDN(IRLST , IRLVP1)               
      REAL                                                              
     S     PFLUX (IRLST , 2 , IRLVP1)   ,                               
     S     PFLUC (IRLST , 2 , IRLVP1)   ,                               
     S     ZFLWP (IRLST)                ,                               
     S     PCTS  (IRLST , IRLEV)        ,                               
     S     ZZFU   (IRLST)               ,                               
     S     ZFUP  (IRLST , IRLVP1)       ,                               
     S     ZGLAYD(IRLST)                ,                               
     S     ZGLAYU(IRLST)                ,                               
     S     ZOMEGA(IRLST , 2 , IRLEV)                                    
C                                                                       
      INTEGER                                                           
     S     ITX   (IRLST)                ,                               
     S     IXDIA (IRLST , IRLVP1)       ,                               
     S     IXT   (IRLST , IRLVP1,IRLVP1),                               
     S     IXTSU (IRLST , IRLVP1)       ,                               
     S     IXTTP (IRLST , IRLVP1)       ,                               
     S     IMX   (IRLST)                ,                               
     S     IMXP  (IRLST)                                                
C                                                                       
      REAL ZOZ   (IRLST , IRLEV)        ,                               
     S     PPMB  (IRLST , IRLVP1)       ,                               
     S     ZRES  (IRLST)                ,                               
     S     PB    (IRLST, 6 , IRLVP1)    ,                               
     S     ZSSIG (IRLST , 3*NRLEV+1)    ,                               
     S     PTAU  (IRLST , 2 , IRLEV)    ,                               
     S     PTAVE (IRLST , IRLEV)        ,                               
     S     ZTAVIC(IRLST)                ,                               
     S     ZTI1  (IRLST)                ,                               
     S     PTL   (IRLST , IRLVP1)       ,                               
     S     ZTT   (IRLST , NTRA)         ,                               
     S     ZTT1  (IRLST , NTRA)         ,                               
     S     ZTT2  (IRLST , NTRA)         ,                               
     S     ZUAER (IRLST , NINT)         ,                               
     S     ZUPF  (IRLST , IRLVP1,IRLVP1),                               
     S     ZUU   (IRLST , NUA)          ,                               
     S     ZXD   (IRLST)                                                
      REAL                                                              
     S     ZXN   (IRLST)                ,                               
     S     ZXOZ  (IRLST)                ,                               
     S     ZXWV  (IRLST)                ,                               
     S     ZZZ   (IRLST)                ,                               
     S     ZRES2 (IRLST)                ,                               
     S     ZTI2  (IRLST)                ,                               
     S     ZBLAY (IRLST,IRLEV)          ,                               
     S     ZBLEV (IRLST,IRLVP1)         ,                               
     S     ZCTS  (IRLST)                ,                               
     S     ZHFG  (IRLST)                                                
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C                                                                       
      INTEGER KT    (IRLST)                                             
      REAL    PUU   (IRLST , NUA  )     ,                               
     S        PTT   (IRLST , NTRA )     ,                               
     S        ZZ    (IRLST)                                             
C                                                                       
C     -------------------------------------------------------- ---------
C                                                                       
      REAL  PCLFR  (IRLST , IRLEV)       ,                              
     S      PT     (IRLST , IRLVP1)       ,                             
     S      PQOF   (IRLST , IRLEV)          ,                           
     S      PQLWP  (IRLST , IRLEV)          ,                           
     S      PEMIS  (IRLST)                                              
      REAL                                                              
     S      PFLT   (IRLST , IRLVP1)                                     
C                                                                       
      REAL  ZALBSU(IRLST , 2 )           ,                              
     S      ZCG   (IRLST , 2 , IRLEV)    ,                              
     S      ZCLDLW(IRLST , IRLEV)        ,                              
     S      ZCLDSW(IRLST , IRLEV)        ,                              
     S      ZCOOLR(IRLST , IRLEV)        ,                              
     S      ZDT0  (IRLST)                ,                              
     S      ZFLUX (IRLST , 2 , IRLVP1)   ,                              
     S      ZPMB  (IRLST , IRLVP1)       ,                              
     S      ZTAU  (IRLST , 2 , IRLEV)    ,                              
     S      ZTAVE (IRLST , IRLEV)        ,                              
     S      ZTL   (IRLST , IRLVP1)       ,                              
     S      ZFSDWN(IRLST , IRLVP1)       ,                              
     S      ZFSUP (IRLST , IRLVP1)       ,                              
     S      ZHEATR(IRLST , IRLEV )                                      
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C                                                                       
      REAL   PDP   (IRLST , IRLEV)     ,                                
     S       PWV   (IRLST , IRLEV)     ,                                
     S       PFLS  (IRLST , IRLVP1)    ,                                
     S       PPSOL (IRLST        )     ,                                
     S       PRMU0 (IRLST        )     ,                                
     S       PAER  (IRLST,IRLEV,5)     ,                                
     S       PALBS (IRLST , 2 )        ,                                
     S       PCG   (IRLST , 2 , IRLEV) ,                                
     S       PCLDSW(IRLST , IRLEV)     ,                                
     S       POMEGA(IRLST , 2 , IRLEV) ,                                
     S       POZ   (IRLST , IRLEV)                                      
C                                                                       
      REAL   PFDOWN(IRLST , IRLVP1)    ,                                
     S       PFUP  (IRLST , IRLVP1)                                     
C                                                                       
      REAL   ZAKI  (IRLST , 2 )        ,                                
     S       ZCGAZ (IRLST , IRLEV)     ,                                
     S       ZCNEB (IRLST , IRLEV)     ,                                
     S       ZC1I  (IRLST , IRLVP1)    ,                                
     S       ZDSC  (IRLST , IRLEV)     ,                                
     S       ZDSH  (IRLST , IRLEV)     ,                                
     S       ZDSIG (IRLST , IRLEV)     ,                                
     S       ZFACT (IRLST)             ,                                
     S       ZFACT1 (IRLST), ZFACT2 (IRLST),                            
     S       ZFACT3 (IRLST), ZFACT4 (IRLST),                            
     S       ZFACT5 (IRLST), ZFACT6 (IRLST),                            
     S       ZFD   (IRLST , IRLVP1)    ,                                
     S       ZFRAC (IRLST)             ,                                
     S       ZFU   (IRLST , IRLVP1)    ,                                
     S       ZG    (IRLST)             ,                                
     S       ZGG   (IRLST)             ,                                
     S       ZPIZAZ(IRLST , IRLEV)     ,                                
     S       ZRAYL (IRLST),  ZRAY1 (IRLST , IRLVP1)                     
      REAL                                                              
     S       ZRAY2 (IRLST , IRLVP1)    ,                                
     S       ZREF  (IRLST)             ,                                
     S       ZREFZ (IRLST , 2 , IRLVP1),                                
     S       ZRE1  (IRLST)             ,                                
     S       ZRE2  (IRLST)             ,                                
     S       ZRJ   (IRLST , 6 , IRLVP1),                                
     S       ZRK   (IRLST , 6 , IRLVP1),                                
     S       ZRL   (IRLST , 8)         ,                                
     S       ZRMU  (IRLST)             ,                                
     S       ZRMUE (IRLST , IRLVP1)    ,                                
     S       ZRMUZ (IRLST)                                              
      REAL                                                              
     S       ZRNEB (IRLST)             ,                                
     S       ZRUEF (IRLST , 8)         ,                                
     S       ZR1   (IRLST)             ,                                
     S       ZR2   (IRLST)             ,                                
     S       ZR21  (IRLST)             ,                                
     S       ZR22  (IRLST)             ,                                
     S       ZR23  (IRLST)             ,                                
     S       ZS    (IRLST)             ,                                
     S       ZSEC  (IRLST)             ,                                
     S       ZSS1  (IRLST)             ,                                
     S       ZTAUAZ(IRLST , IRLEV)     ,                                
     S       ZTOT  (IRLST , IRLVP1)    ,                                
     S       ZTO1  (IRLST)             ,                                
     S       ZTR   (IRLST , 2 , IRLVP1),                                
     S       ZTRA1 (IRLST , IRLVP1)    ,                                
     S       ZTRA2 (IRLST , IRLVP1)                                     
      REAL                                                              
     S       ZTR1  (IRLST)             ,                                
     S       ZTR2  (IRLST)             ,                                
     S       ZUD   (IRLST , 3 , IRLVP1),                                
     S       ZUM   (IRLST , IRLVP1)    ,                                
     S       ZU1D  (IRLST)             ,                                
     S       ZU2D  (IRLST)             ,                                
     S       ZW    (IRLST)             ,                                
     S       ZN175 (IRLST)             ,                                
     S       ZN190 (IRLST)             ,                                
     S       ZO175 (IRLST)             ,                                
     S       ZO190 (IRLST)             ,                                
     S       ZP75  (IRLST)             ,                                
     S       ZP90  (IRLST)             ,                                
     S       ZSIGN (IRLST)             ,                                
     S       ZSIGO (IRLST)                                              
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
      REAL   PTR    (IRLST)     ,                                       
     S       PU     (IRLST)                                             
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
C   CLOUD-POINTERS                                                      
C   --------------                                                      
C                                                                       
      REAL   ZRH(NLP2,NLEV),                                            
     *       ZCH(NLP2),                                                 
     *       ZCM(NLP2),                                                 
     *       ZCL(NLP2),                                                 
     *       ZCT(NLP2),                                                 
     *       ZCC(NLP2)                                                  
      INTEGER NTOPCM(NLP2),                                             
     *        NBASECM(NLP2)                                             
      REAL   ARPRCM(NLP2)                                               
      INTEGER ITOPH(NLP2),                                              
     *        ITOPM(NLP2),                                              
     *        ITOPL(NLP2)                                               
      LOGICAL LOC(NLP2)                                                 
      REAL   ZQS(NLP2,NLEV),                                            
     *       ZCLC(NLP2,NLEV),                                           
     *       ZQLWP(NLP2,NLEV)                                           
C                                                                       
      REAL   VERVEL(NLP2,NLEV),                                         
     *       ZTHETA(NLP2,NLEV),                                         
     *       ZETA(NLP2,NLEV),                                           
     *       ZDTHMIN(NLP2)                                              
      INTEGER INVB(NLP2)                                                
      REAL   ZCTLWP(NLP2),                                              
     *       ZQLOC(NLP2),                                               
     *       ZQMEC(NLP2),                                               
     *       ZQHIC(NLP2)                                                
C                                                                       
C  definitions of "radheat"                                             
C                                                                       
      REAL   ZFSOT(NLON),                                               
     #       ZFTET(NLON),                                               
     #       ZFLT (NLON),                                               
     #       ZFSO (NLON),                                               
     #       ZFTE (NLON),                                               
     #       ZFLB (NLON),                                               
     #       ZDTDT(NLON),                                               
     #       HEATO(NLON,NLEV),                                          
     #       HEATE(NLON,NLEV),                                          
     #       HEATR(NLON,NLEV),                                          
     #       ZI0  (NLON)                                                
C                                                                       
C PIERS 13-8-97                                                         
C                                                                       
         REAL PIERSLWP                                                  
C                                                                       
C Piece of code needed to replace the Nikos radiation scheme with       
C the Morcrette radiation scheme in the igcm3.1.                        
C See morcrette3_1_1.readme for details.                                
C                                                                       
C                                                                       
c      program lh                                                       
c                                                                       
c      parameter(NL=23)                                                 
c      real P(NL),t(NL),h2o(NL),o3(NL),htlw(NL),htsw(NL)                
c      real swalb,doy                                                   
c       REAL CF(4,2)     !cloud fractions,lwps                          
c       INTEGER IC(4,2)   ! cloud positions                             
c       REAL RFLUXES(2,2,2)                                             
c      open(11,file='mls23.dat')                                        
c      read(11,*) NLEV                                                  
c      DOY=80                                                           
c      swalb=0.07                                                       
c      alat=40.0                                                        
c       doy=float(i)                                                    
c      do i=nlev,1,-1                                                   
c         read(11,*) P(i),t(i),h2o(i),adum,o3(i)                        
c         print *,i,p(i),t(i),h2o(i),o3(i)                              
c      enddo                                                            
c                                                                       
c       ic(1,1)=3                                                       
c      ic(1,2)=17                                                       
c       ic(3,1)=6                                                       
c       ic(2,2)=6                                                       
c       cf(1,1)=0.3                                                     
c       cf(1,2)=45.0                                                    
c        cf(3,1)=0.8                                                    
c       cf(4,2)=20.0                                                    
c      alon=0.0                                                         
c      call morcigcm(p,t,h2o,o3,alat,htlw,htsw,doy,cf,ic,               
c     $     rfluxes,swalb,alon)                                         
c      enddo                                                            
c      do i=nl,1,-1                                                     
c         print '(i6,4F14.4)',i,p(i),t(i),htsw(i),htlw(i)               
c      enddo                                                            
c      print *,RFLUXES(1,1,1), RFLUXES(1,1,2), RFLUXES(1,2,1),          
c     $     RFLUXES(1,2,2)                                              
c      print *,RFLUXES(2,1,1), RFLUXES(2,1,2), RFLUXES(2,2,1),          
c     $     RFLUXES(2,2,2)                                              
c                                                                       
c                                                                       
c      end                                                              
c      INCLUDE 'zparb.upd'                                              
C                                                                       
C     -------------------------------------------------------------     
C                                                                       
      DO 99 JL=1,NRLST                                                  
         PGG  (JL) = ZGG  (JL)                                          
         PREF (JL) = ZREF (JL)                                          
         PRE1 (JL) = ZRE1 (JL)                                          
         PRE2 (JL) = ZRE2 (JL)                                          
         PRMUZ(JL) = ZRMUZ(JL)                                          
         PTO1 (JL) = ZTO1 (JL)                                          
         PTR1 (JL) = ZTR1 (JL)                                          
         PTR2 (JL) = ZTR2 (JL)                                          
         PW   (JL) = ZW   (JL)                                          
 99   CONTINUE                                                          
C                                                                       
C*         1.      DELTA-EDDINGTON CALCULATIONS                         
C                                                                       
 100  CONTINUE                                                          
C                                                                       
      DO 131 JL   =   1 , KDLON                                         
C                                                                       
C*         1.1     SET UP THE DELTA-MODIFIED PARAMETERS                 
C                                                                       
 110  CONTINUE                                                          
C                                                                       
         ZFF = PGG(JL)*PGG(JL)                                          
         ZGP = PGG(JL)/(1.+PGG(JL))                                     
         ZTOP = (1.- PW(JL) * ZFF) * PTO1(JL)                           
         ZWCP = (1-ZFF)* PW(JL) /(1.- PW(JL) * ZFF)                     
         ZDT = 2./3.                                                    
         ZX1 = 1.-ZWCP*ZGP                                              
         ZWM = 1.-ZWCP                                                  
         ZRM2 =  PRMUZ(JL) * PRMUZ(JL)                                  
         ZZRK = SQRT(3.*ZWM*ZX1)                                        
         ZX2 = 4.*(1.-ZZRK*ZZRK*ZRM2)                                   
         ZRP = SQRT(3.*ZWM/ZX1)                                         
         ZALPHA = 3.*ZWCP*ZRM2*(1.+ZGP*ZWM)/ZX2                         
         ZBETA = 3.*ZWCP* PRMUZ(JL) *(1.+3.*ZGP*ZRM2*ZWM)/ZX2           
         ZEXMUO = EXP(-ZTOP/ PRMUZ(JL) )                                
         ZEXKP = EXP(ZZRK*ZTOP)                                         
         ZEXKM = 1./ZEXKP                                               
         ZXP2P = 1.+ZDT*ZRP                                             
         ZXM2P = 1.-ZDT*ZRP                                             
         ZAP2B = ZALPHA+ZDT*ZBETA                                       
         ZAM2B = ZALPHA-ZDT*ZBETA                                       
C                                                                       
C*         1.2     WITHOUT REFLECTION FROM THE UNDERLYING LAYER         
C                                                                       
 120  CONTINUE                                                          
C                                                                       
         ZA11 = ZXP2P                                                   
         ZA12 = ZXM2P                                                   
         ZA13 = ZAP2B                                                   
         ZA22 = ZXP2P*ZEXKP                                             
         ZA21 = ZXM2P*ZEXKM                                             
         ZA23 = ZAM2B*ZEXMUO                                            
         ZDENA = ZA11 * ZA22 - ZA21 * ZA12                              
         ZC1A = (ZA22*ZA13-ZA12*ZA23)/ZDENA                             
         ZC2A = (ZA11*ZA23-ZA21*ZA13)/ZDENA                             
         ZRI0A = ZC1A+ZC2A-ZALPHA                                       
         ZRI1A = ZRP*(ZC1A-ZC2A)-ZBETA                                  
         PRE1(JL) = (ZRI0A-ZDT*ZRI1A)/ PRMUZ(JL)                        
         ZRI0B = ZC1A*ZEXKM+ZC2A*ZEXKP-ZALPHA*ZEXMUO                    
         ZRI1B = ZRP*(ZC1A*ZEXKM-ZC2A*ZEXKP)-ZBETA*ZEXMUO               
         PTR1(JL) = ZEXMUO+(ZRI0B+ZDT*ZRI1B)/ PRMUZ(JL)                 
C                                                                       
C*         1.3     WITH REFLECTION FROM THE UNDERLYING LAYER            
C                                                                       
 130  CONTINUE                                                          
C                                                                       
         ZB21 = ZA21- PREF(JL) *ZXP2P*ZEXKM                             
         ZB22 = ZA22- PREF(JL) *ZXM2P*ZEXKP                             
         ZB23 = ZA23- PREF(JL) *ZEXMUO*(ZAP2B - PRMUZ(JL) )             
         ZDENB = ZA11 * ZB22 - ZB21 * ZA12                              
         ZC1B = (ZB22*ZA13-ZA12*ZB23)/ZDENB                             
         ZC2B = (ZA11*ZB23-ZB21*ZA13)/ZDENB                             
         ZRI0C = ZC1B+ZC2B-ZALPHA                                       
         ZRI1C = ZRP*(ZC1B-ZC2B)-ZBETA                                  
         PRE2(JL) = (ZRI0C-ZDT*ZRI1C) / PRMUZ(JL)                       
         ZRI0D = ZC1B*ZEXKM + ZC2B*ZEXKP - ZALPHA*ZEXMUO                
         ZRI1D = ZRP * (ZC1B*ZEXKM - ZC2B*ZEXKP) - ZBETA*ZEXMUO         
         PTR2(JL) = ZEXMUO + (ZRI0D + ZDT*ZRI1D) / PRMUZ(JL)            
C                                                                       
 131  CONTINUE                                                          
      DO 132 JL=1,NRLST                                                 
         ZGG  (JL) = PGG  (JL)                                          
         ZREF (JL) = PREF (JL)                                          
         ZRE1 (JL) = PRE1 (JL)                                          
         ZRE2 (JL) = PRE2 (JL)                                          
         ZRMUZ(JL) = PRMUZ(JL)                                          
         ZTO1 (JL) = PTO1 (JL)                                          
         ZTR1 (JL) = PTR1 (JL)                                          
         ZTR2 (JL) = PTR2 (JL)                                          
         ZW   (JL) = PW   (JL)                                          
 132     CONTINUE                                                       
      RETURN                                                            
      END                                                               
C                                                                       
      SUBROUTINE LW (KDLON,KRLST,KFLEV,KFLVP1,KFLEV2,KNGL,              
     S                      KNGLP1, PSIG,PSCT,PCCO2,ITASK )             
C                                                                       
C**** *LW*   - COMPUTES LONGWAVE RADIATION QUANTITIES                   
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C           COMPUTES LONGWAVE FLUXES                                    
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C        EXPLICIT ARGUMENTS                                             
C        --------------------                                           
C                                                                       
C        IMPLICIT ARGUMENTS                                             
C        --------------------                                           
C                                                                       
C     ==== INPUTS ===                                                   
C PEMIS  : (KDLON)             ; SURFACE EMISSIVITY                     
C PCCO2  :                     ; CONCENTRATION IN CO2 (PA/PA)           
C PQOF   : (KDLON,KFLEV)       ; CONCENTRATION IN OZONE (PA/PA)         
C PTAVE  : (KDLON,KFLEV)       ; TEMPERATURE                            
C PTL    : (KDLON,0:KFLEV)     ; HALF LEVEL TEMPERATURE                 
C PPMB   : (KDLON,0:KFLEV)     ; HALF LEVEL PRESSURE                    
C PWV    : (KDLSUR,KFLEV)      ; SPECIFIC HUMIDITY PA/PA                
C PCLDLW : (KDLON,KFLEV)       ; CLOUD FRACTIONAL COVER                 
C PAER   : (KDLON,KFLEV,5)     ; OPTICAL THICKNESS OF THE AEROSOLS      
C     ==== OUTPUTS ===                                                  
C PFLUX(KDLON,2,KFLEV)         ; RADIATIVE FLUXES                       
C                     1  ==>  UPWARD   FLUX TOTAL                       
C                     2  ==>  DOWNWARD FLUX TOTAL                       
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C          1. COMPUTES THE PRESSURE AND TEMPERATURE WEIGHTED AMOUNTS OF 
C     ABSORBERS, AND THE VARIOUS RELEVANT PLANCK FUNCTIONS              
C          2. PERFORMS THE VERTICAL INTEGRATION DISTINGUISHING THE CON- 
C     TRIBUTIONS OF THE ADJACENT AND DISTANT LAYERS AND THOSE FROM THE  
C     BOUNDARIES.                                                       
C          3. COMPUTES THE CLEAR-SKY DOWNWARD AND UPWARD EMISSIVITIES.  
C          4. INTRODUCES THE EFFECTS OF THE CLOUDS ON THE FLUXES.       
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          *LWTT*                                                       
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION              
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C        JEAN-JACQUES MORCRETTE  *ECMWF*                                
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C        ORIGINAL : 88-12-15                                            
C------------------------------------------------------------------     
C      IMPLICIT LOGICAL (L)                                             
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRSW* SWITCHES RELATED TO THE RADIATION SCHEME.         
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      IMPLICIT REAL (A-H,O-Z)                                           
C      IMPLICIT UNDEFINED (Z)                                           
C      IMPLICIT LOGICAL (L)                                             
C      IMPLICIT UNDEFINED (L)                                           
      IMPLICIT INTEGER (I-K,M-N)                                        
C      IMPLICIT UNDEFINED (I-K,M-N)                                     
C      IMPLICIT REAL (P)                                                
C      IMPLICIT UNDEFINED (A-H,O,Q-Y)                                   
C                                                                       
C everything except the parameters is defined                           
C in the common blocks                                                  
C                                                                       
      COMMON/COM1/                                                      
C     I NRADFR, NRPART, NRINT, NRADPFR, NRADPLA,                        
C     I NRLSTI,                                                         
     R XLAT,                                                            
     R RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR, ZCCO2,              
C     R PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3,                 
     L LOH, LOM, LOL, LORAD                                             
      COMMON/COM2/                                                      
     I ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,                  
     I         JLOOP, KRLEV,                                            
     I        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     I        ITH, ITM, ITL,                                            
     I        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     I        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     I        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     I        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      COMMON/COM3/                                                      
     I IND4, KM12, KXD, KXU, IG, KBS, KDD,                              
     I        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     I        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     I        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     I        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     I        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
      COMMON/COM4/                                                      
     R           COZQC,COZQS,COZHC,COZHS,                               
     R CSUPSAT,                                                         
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
      COMMON/COM5/                                                      
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NUMVC,                                                           
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      COMMON/COM6/                                                      
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD,       
     R DTIME,    TWODT,    EPS,                                         
     R SINLON,     COSLON,                                              
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2,                          
     I NFAX,                                                            
     R CRAE,                                                            
     R CDISSEM                                                          
      COMMON/COM7/                                                      
     R     CCRH,                                                        
     L LO,LO1,LO2,LODIA,LODIAP,                                         
     R TAUA, PIZA, CGA, CAER, CAEROS                                    
      COMMON/COM7A/                                                     
     R  RT1,WG1,AT,BT,CNTNU,O1H,O2H,PIALF0                              
     S ,OCT,TREF,XP,TSTAND,GA,GB,CE,ETY,PTY                             
      COMMON/COM7B/                                                     
     S TINTP,TSTP,                                                      
     S  TBO3,AO3,BO3,TB,ALP,BET,ULG,ULGO,PLG,AP,BP,TO3B,O3A,O3B,        
     R          CDAY,CCO2,CH2O,DIFF,X10E                                
       COMMON/COM7C/                                                    
     S  ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW,                       
     R  APAD,BPAD,D,CRAY,SUN,                                           
     R   TTT,                                                           
     R   TM1                                                            
      COMMON/COM8/                                                      
     R       TSM1M,                                                     
     R       QM1,                                                       
     R       SNM1M,                                                     
     R       ALBM,                                                      
     R       AVVELM,                                                    
     R   SLMM,                                                          
     R       APHM1,                                                     
     R       APM1                                                       
      COMMON/COM9/                                                      
     L LOLAND,                                                          
     R   ZTI,                                                           
     *       ZCTI,                                                      
     *       ZCPH,                                                      
     *       ZWV,                                                       
     *       ZSW,                                                       
     *       ZALTE,                                                     
     *       ZCALTE,                                                    
     *       ZALSO                                                      
      COMMON/COM10/                                                     
     *       ZCALSO,                                                    
     R   ZOZQ,                                                          
     *       ZOZH,                                                      
     R   ZTRSOL,                                                        
     *       ZEMTER,                                                    
     R   TRSOLM,                                                        
     *       EMTERM,                                                    
     R   AMU0,                                                          
     *       RDAYL                                                      
      COMMON/COM11/                                                     
     R   CLC,                                                           
     *       TI,                                                        
     *       DP,                                                        
     *       AADS,                                                      
     *       AEQ1,                                                      
     *       AEQ2,                                                      
     *       AEQ3,                                                      
     *       AEQ4,                                                      
     *       AEQ5,                                                      
     *       SSW                                                        
      COMMON/COM12/                                                     
     *       WV,                                                        
     *       QOF,                                                       
     *       QCF,                                                       
     *       CLWA,                                                      
     *       APRE,                                                      
     *       SMU0,                                                      
     *       ALTE,                                                      
     *       ALSO                                                       
      COMMON/COM13/                                                     
     R   ZDPO,                                                          
     *       ZQCFO,                                                     
     *       ZAEQSO,                                                    
     *       ZQOFO,                                                     
     *       ZDPN,                                                      
     *       ZQCFN,                                                     
     *       ZAEQSN,                                                    
     *       ZQOFN                                                      
      COMMON/COM14/                                                     
     R   FLT,                                                           
     *       FLS,                                                       
     R   DIAG,                                                          
     *       DIA1,                                                      
     *       ZDIAG,                                                     
     *       ZDIA1,                                                     
     *       ZDIA,                                                      
     *       ZDIAF,                                                     
     *       ZDIAT,                                                     
     R PGG                                                              
      COMMON/COM15/                                                     
     #     PREF,                                                        
     #     PRE1,                                                        
     #     PRE2,                                                        
     #     PRMUZ,                                                       
     #     PTO1,                                                        
     #     PTR1,                                                        
     #     PTR2,                                                        
     #     PW,                                                          
     R     PALBSN                                                       
      COMMON/COM16/                                                     
     R ZABSCU,                                                          
     S     ZABSLY,                                                      
     S     ZADJD ,                                                      
     S     ZADJU ,                                                      
     S     ZBGND ,                                                      
     S     ZBINT ,                                                      
     S     ZBSUR ,                                                      
     S     ZBSUIN,                                                      
     S     ZBTOP ,                                                      
     S     PCLDLW,                                                      
     S     ZCLOUD,                                                      
     S     ZCNTRB,                                                      
     S     ZCUMP ,                                                      
     S     ZCUMPT,                                                      
     S     ZDBDT                                                        
      COMMON/COM17/                                                     
     S     ZDBSL ,                                                      
     S     ZDISTD,                                                      
     S     ZDISTU,                                                      
     S     ZDNF  ,                                                      
     S     PDT0  ,                                                      
     S     ZDUC                                                         
      COMMON/COM18/                                                     
     S     ZEFFT ,                                                      
     S     ZEMU  ,                                                      
     S     ZEPHIO,                                                      
     S     ZEPSC2,                                                      
     S     ZEPSC3,                                                      
     S     ZEPSH1,                                                      
     S     ZEPSH2,                                                      
     S     ZEPSH3                                                       
      COMMON/COM19/                                                     
     S     ZEPSH4,                                                      
     S     ZEPSH5,                                                      
     S     ZEPSH6,                                                      
     S     ZEPSIO,                                                      
     S     ZFACTC,                                                      
     S     ZZFD  ,                                                      
     S     ZFDD  ,                                                      
     S     ZFDN                                                         
      COMMON/COM20/                                                     
     S     PFLUX , PFLUC,                                               
     S     ZFLWP , PCTS                                                 
     S     ZZFU  ,                                                      
     S     ZFUP  ,                                                      
     S     ZGLAYD,                                                      
     S     ZGLAYU,                                                      
     S     ZOMEGA,                                                      
     S     ITX   ,                                                      
     S     IXDIA ,                                                      
     S     IXT   ,                                                      
     S     IXTSU ,                                                      
     S     IXTTP ,                                                      
     S     IMX   ,                                                      
     S     IMXP                                                         
      COMMON/COM21/                                                     
     R ZOZ   ,                                                          
     S     PPMB  ,                                                      
     S     ZRES  , PB                                                   
     S     ZSSIG ,                                                      
     S     PTAU  ,                                                      
     S     PTAVE ,                                                      
     S     ZTAVIC,                                                      
     S     ZTI1  ,                                                      
     S     PTL                                                          
      COMMON/COM22/                                                     
     S     ZTT   ,                                                      
     S     ZTT1  ,                                                      
     S     ZTT2  ,                                                      
     S     ZUAER ,                                                      
     S     ZUPF  ,                                                      
     S     ZUU   ,                                                      
     S     ZXD                                                          
      COMMON/COM23/                                                     
     S     ZXN   ,                                                      
     S     ZXOZ  ,                                                      
     S     ZXWV  ,                                                      
     S     ZZZ   ,                                                      
     S     ZRES2 ,                                                      
     S     ZTI2  ,                                                      
     S     ZBLAY ,                                                      
     S     ZBLEV ,                                                      
     S     ZCTS  ,                                                      
     S     ZHFG                                                         
      COMMON/COM24/                                                     
     I KT,                                                              
     R    PUU   ,                                                       
     S        PTT   ,                                                   
     S        ZZ                                                        
     R  PCLFR  ,                                                        
     S      PT     ,                                                    
     S      PQOF   ,                                                    
     S      PQLWP  ,                                                    
     S      PEMIS  ,                                                    
     S      PFLT                                                        
      COMMON/COM25/                                                     
     R  ZALBSU,                                                         
     S      ZCG   ,                                                     
     S      ZCLDLW,                                                     
     S      ZCLDSW,                                                     
     S      ZCOOLR,                                                     
     S      ZDT0  ,                                                     
     S      ZEMD  ,                                                     
     S      ZFLUX ,                                                     
     S      ZPMB  ,                                                     
     S      ZTAU  ,                                                     
     S      ZTAVE ,                                                     
     S      ZTL   ,                                                     
     S      ZFSDWN                                                      
      COMMON/COM26/                                                     
     S      ZFSUP ,                                                     
     S      ZHEATR,                                                     
     R   PDP   ,                                                        
     S       PWV   ,                                                    
     S       PFLS  ,                                                    
     S       PPSOL ,                                                    
     S       PRMU0 ,                                                    
     S       PAER  ,                                                    
     S       PALBS ,                                                    
     S       PCG   ,                                                    
     S       PCLDSW,                                                    
     S       POMEGA,                                                    
     S       POZ                                                        
      COMMON/COM27/                                                     
     R   PFDOWN,                                                        
     S       PFUP  ,                                                    
     R   ZAKI  ,                                                        
     S       ZCGAZ ,                                                    
     S       ZCNEB ,                                                    
     S       ZC1I  ,                                                    
     S       ZDSC  ,                                                    
     S       ZDSH                                                       
      COMMON/COM28/                                                     
     S       ZDSIG ,                                                    
     S       ZFACT ,                                                    
     S       ZFACT1 ,                                                   
     S       ZFACT2 ,                                                   
     S       ZFACT3 ,                                                   
     S       ZFACT4 ,                                                   
     S       ZFACT5 ,                                                   
     S       ZFACT6 ,                                                   
     S       ZFD   ,                                                    
     S       ZFRAC ,                                                    
     S       ZFU   ,                                                    
     S       ZG    ,                                                    
     S       ZGG   ,                                                    
     S       ZPIZAZ                                                     
      COMMON/COM29/                                                     
     S       ZRAYL ,                                                    
     S       ZRAY1 ,                                                    
     S       ZRAY2 ,                                                    
     S       ZREF  ,                                                    
     S       ZREFZ ,                                                    
     S       ZRE1  ,                                                    
     S       ZRE2  ,                                                    
     S       ZRJ   ,                                                    
     S       ZRK   ,                                                    
     S       ZRL   ,                                                    
     S       ZRMU  ,                                                    
     S       ZRMUE ,                                                    
     S       ZRMUZ                                                      
      COMMON/COM30/                                                     
     S       ZRNEB ,                                                    
     S       ZRUEF ,                                                    
     S       ZR1   ,                                                    
     S       ZR2   ,                                                    
     S       ZR21  ,                                                    
     S       ZR22  ,                                                    
     S       ZR23  ,                                                    
     S       ZS    ,                                                    
     S       ZSEC  ,                                                    
     S       ZSS1  ,                                                    
     S       ZTAUAZ                                                     
      COMMON/COM31/                                                     
     S       ZTOT  ,                                                    
     S       ZTO1  ,                                                    
     S       ZTR   ,                                                    
     S       ZTRA1 ,                                                    
     S       ZTRA2 ,                                                    
     S       ZTR1                                                       
      COMMON/COM32/                                                     
     S       ZTR2  ,                                                    
     S       ZUD   ,                                                    
     S       ZUM   ,                                                    
     S       ZU1D  ,                                                    
     S       ZU2D  ,                                                    
     S       ZW    ,                                                    
     S       ZN175 ,                                                    
     S       ZN190 ,                                                    
     S       ZO175 ,                                                    
     S       ZO190 ,                                                    
     S       ZP75  ,                                                    
     S       ZP90  ,                                                    
     S       ZSIGN ,                                                    
     S       ZSIGO                                                      
      COMMON/COM33/                                                     
     R   PTR,                                                           
     S       PU,                                                        
     R   ZRH,                                                           
     *       ZCH,                                                       
     *       ZCM,                                                       
     *       ZCL,                                                       
     *       ZCT,                                                       
     *       ZCC,                                                       
     I NTOPCM                                                           
      COMMON/COM34/                                                     
     *        NBASECM,                                                  
     R   ARPRCM,                                                        
     I ITOPH,                                                           
     *        ITOPM,                                                    
     *        ITOPL,                                                    
     L LOC,                                                             
     R   ZQS,                                                           
     *       ZCLC,                                                      
     *       ZQLWP                                                      
      COMMON/COM35/                                                     
     R   VERVEL,                                                        
     *       ZTHETA,                                                    
     *       ZETA,                                                      
     *       ZDTHMIN,                                                   
     I INVB,                                                            
     R   ZCTLWP,                                                        
     *       ZQLOC,                                                     
     *       ZQMEC,                                                     
     *       ZQHIC,                                                     
     R   ZFSOT,                                                         
     #       ZFTET,                                                     
     #       ZFLT                                                       
      COMMON/COM36/                                                     
     #       ZFSO ,                                                     
     #       ZFTE ,                                                     
     #       ZFLB ,                                                     
     #       ZDTDT,                                                     
     #       HEATO,                                                     
     #       HEATE,                                                     
     #       HEATR,                                                     
     #       ZI0                                                        
C                                                                       
      INTEGER                                                           
C     I NRADFR,   NRPART,   NRINT,    NRLSTI,  NRADPFR,  NRADPLA,       
     I NRLST,    NRLEV,    NRLEVP1, NLEV, NLP2, NLEVP1,                 
     I NLON, IRLEV, NTRA, NLAT, ITIM, NUA, NINT, NRPARTI                
C                                                                       
C   NRLEV .... number of vertical levels                                
C   NRLST .... number of points on a latitude line                      
C   NUA   .... number of absorber amount                                
C   NINT  .... number of spectral intervals                             
C   NTRA  .... number of transmission functions                         
C   NLAT  .... number of latitude points                                
C   ITIM  .... number of month under consideration                      
C                                                                       
C      PARAMETER (NRLEV=122,NUA=34,NINT=6,NTRA=34, NLAT=1,              
      PARAMETER (NRLEV=21,NUA=34,NINT=6,NTRA=34, NLAT=1,                
cccm      PARAMETER (NRLEV=37,NUA=18,NINT=6,NTRA=19, NLAT=1,            
cccm     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=10,NLEV=NRLEV,        
     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=7,NLEV=NRLEV,             
     #           NLP2=NRLST, NLEVP1=NRLEVP1, NLON=NRLST,                
     #           NRPARTI=1)                                             
C                                                                       
      REAL YCLOCK, XLAT(NLAT), Z0, DZ, ALBMPAR, SNPAR,                  
     #     TSUPAR, CTFREEZ, DELTAC, DELTAO, DELTAH                      
C                                                                       
C   YTIME  .... time of the year in radians                             
C   YCLOCK .... time of the day in radians                              
C   XLAT   .... latitudes in radians                                    
C   Z0     .... zero point of height grid                               
C   DZ     .... Interval between height levels                          
C   ALBMPAR.... sw surface albedo as function of longitude:             
C           yearly averaged sea-ice: 0.55                               
C                           water  : 0.07                               
C                           land   : 0.07.lt.ALBM.lt.0.80               
C                                    depending on snow                  
C   SNPAR  .... snow depth in meter                                     
C   TSUPAR .... surface temperature (Kelvin)                            
C   CTFREEZ.... freezing temperature of sea-water (Kelvin)              
C   DELTAC .... Doppler parameter for line shapes of CO2                
C   DELTAO .... Doppler parameter for line shapes of O3                 
C                                                                       
      PARAMETER (Z0=0., CTFREEZ=271.26,                                 
C     PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
     #           DZ=0.289, ALBMPAR=0.2, SNPAR=0.,                       
cccm  PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
cccm     #           DZ=0.15, ALBMPAR=0.3, SNPAR=0.,                    
CZ     #           DELTAC=0., DELTAO=0.)                                
     #           DELTAC=6.0E-04, DELTAO=1.E-02, DELTAH=5.0E-04)         
C      PARAMETER (YCLOCK=0., Z0=0., TSUPAR=273., CTFREEZ=271.26,        
C     #           DZ=0.46875, ALBMPAR=0.07, SNPAR=0.)                   
C      PARAMETER (YCLOCK=0., Z0=2.608, TSUPAR=273., CTFREEZ=271.26,     
C     #           DZ=0.2878, ALBMPAR=0.07, SNPAR=0.)                    
C                                                                       
      LOGICAL LDIUR, LOLPAR                                             
C                                                                       
C   LDIUR .... true: sun at time, false: diurnally averaged             
C   LOLPAR.... true: land surface, false: else                          
C                                                                       
C      PARAMETER (LDIUR=.TRUE., LOLPAR=.TRUE.)                          
       PARAMETER (LOLPAR=.TRUE.)                                        
C                                                                       
      INTEGER  C2TEMP                                                   
      CHARACTER*64  MIDTMP                                              
cccm      PARAMETER (C2TEMP=10,                                         
cccm     #    MIDTMP='/home/atm/ugamp/midrad/radfildat.dir/MIDTMP.DAT') 
ccc1      PARAMETER (C2TEMP=10, MIDTMP='tprfil2.dat')                   
C                                                                       
      INTEGER  IRLONI, IRLST, IRLVP1, JPNLEV, JPNLVP1                   
C                                                                       
      PARAMETER (IRLONI=NRLST, IRLST=NRLST,                             
     #           IRLVP1=NRLEVP1, JPNLEV=NRLEV,                          
     #           JPNLVP1=NRLEVP1, IRLEV=NRLEV)                          
C                                                                       
      INTEGER NIPD, NIPD2, NTR, NG1, NG1P1                              
      PARAMETER(NIPD=8,NIPD2=16,NTR=11,NG1=2,NG1P1=3)                   
C                                                                       
C    definition of the functions                                        
C                                                                       
      REAL       CVMGT, SIGMA2                                          
      INTEGER    CVMGI                                                  
C                                                                       
C    definitons of the subroutine cloud                                 
C                                                                       
      REAL RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR                  
C                                                                       
C    missing definitions in morcrette                                   
C                                                                       
C      REAL   PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3            
      REAL ZCCO2                                                        
                                                                        
      LOGICAL LOH, LOM, LOL, LORAD                                      
C                                                                       
      INTEGER ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,           
     #        IZQS, IZCLC, IZQLWP, JLOOP, KRLEV,                        
     #        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     #        ITH, ITM, ITL,                                            
     #        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     #        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     #        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     #        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      INTEGER IND4, KM12, KXD, KXU, IG, KBS, KDD,                       
     #        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     #        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     #        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     #        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     #        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD1* SPECTRAL DISTRIBUTION OF AEROSOLS AND OZONE.     
C                     (TRIANGULAR *T10* TRUNCATIONFOR AEROSOLS).        
C                     (TRIANGULAR *T5* TRUNCATIONFOR OZONE).            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R           COZQC(21),COZQS(15),COZHC(21),COZHS(15)                
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD2* CONSTANTS INTERNAL TO THE RADIATION SUBROUTINES. 
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CSUPSAT                                                          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMCON* BASIC UNIVERSAL CONSTANTS AND DERIVED CONSTANTS.  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
C                                                                       
C                                                                       
C       --------------------------------------------------------------- 
C      PARAMETER(                                                       
C     I JPM=106,JPN=106,JPK=106,JPGL=160,JPNLON=320,                    
C     I JPT1MAX=400,          JPT2MAX=50,                               
C     I JPT3MAX=300,          JPT4MAX=50,                               
C     I JPHASH=2048,          JPNTRCRS=20,                              
C     I JPTASKS=4,     JPLOCKS=50,    JPEVENT=4,                        
C     I JPLTMT=JPTASKS,                                                 
C     I JPNUMW=352,            JPNUMP=352,                              
C     I JPTABLE=1024,                                                   
C     I JPMAXB =60,                                                     
C     I JPMAXPB=32,                                                     
C     I JPG3FLD=43,         JPG3XF=99,         JPG3CMP=17,              
C     I JPMP1=JPM+1,        JPNP1=JPN+1,       JPKP1=JPK+1,             
C     I JP2MP1=JPMP1*2,     JP4MP1=JPMP1*4,                             
C     I JPED=JPN/2+1,       JPOD=JPNP1/2,                               
C     I IPSP0=JPM+JPN-JPK,  IPSP1=JPNP1*JPMP1, IPSP2=IPSP0*(IPSP0+1)/2, 
C     I JPSP=IPSP1-IPSP2,   JP2SP=2*JPSP,                               
C     I JPHGL=JPGL/2,       JPNLP2=JPNLON+2,                            
C     I                     JPNVCLV=4*JPNLVP1)                          
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C     J. K. GIBSON     E.C.M.W.F.     22/01/82.                         
C     MODIFIED M.J     E.C.M.W.F.     13/12/82.                         
C                                                                       
      INTEGER                                                           
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      LOGICAL                                                           
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD        
      REAL                                                              
     R DTIME,    TWODT,    EPS                                          
C                                                                       
C      INTEGER                                                          
C     I NWTIME(JPNUMW),     NROW(JPTASKS),                              
C     I NPTIME(JPNUMP)                                                  
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMGAU* - QUANTITIES RELATED TO THE GAUSSIAN GRID.      
C                                                                       
      REAL                                                              
     R SINLON(NLP2),     COSLON(NLP2)                                   
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMTRU* -QUANTITIES RELATED TO THE SPECTRAL TRUNCATION. 
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSIM* - QUANTITIES NEEDED FOR THE SEMI-IMPLICIT SCHEME
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMHDI* - COEFFICIENTS FOR HORIZONTAL DIFFUSION.        
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMFFT* - QUANTITIES NEEDED FOR THE FAST *FOURIER       
C                         TRANSFORMS.                                   
C                                                                       
      INTEGER                                                           
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2                           
C      REAL                                                             
C     R TRIG(JPNLON)                                                    
      INTEGER                                                           
     I NFAX(10)                                                         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSTA* - ARRAYS AND PARAMETERS FOR STATISTICS.         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMDIA* - QUANTITIES RELATED TO GLOBAL PHYSICAL         
C                         DIAGNOSTICS.                                  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPSW* SWITCHES RELATED TO THE PARAMETERISATIONS OF      
C                     DIABATIC PROCESSES.                               
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMDSW* SWITCHES RELATED TO THE DYNAMICS                  
C                     AND THE GENERAL CONTROL OF THE FORECAST.          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH1* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE RADIATION SUBROUTINES.                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CRAE,                                                            
     R CDISSEM                                                          
C                                                                       
C      REAL       COSRAD(128),        SINRAD(128)                       
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH2* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE PHYSICAL SUBROUTINES (EXCEPT RADIATION    
C                     ONES).                                            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL       CCRH(JPNLEV)                                           
C                                                                       
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMDIZ* VARIABLES AND POINTERS FOR ZONAL MEAN DIAGNOSTICS 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMMSK* VARIABLES AND POINTERS FOR AREA MEAN DIAGNOSTICS. 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMSDS* - START/RESTART BLOCK.                          
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMTASK* - TASK RELATED VARIABLES.                      
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*         *COMRAD* - RADIATION POINTERS.                               
C                                                                       
      LOGICAL LO,LO1,LO2,LODIA,LODIAP                                   
C---------------------------------------------------------------------  
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C     COMMON *YOMAER* RADIATIVE CHARACTERISTICS OF THE AEROSOLS         
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
      REAL TAUA(2,5), PIZA(2,5), CGA(2,5), CAER(5,5), CAEROS            
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C     COMMON *YOMLW*                                                    
C                                                                       
C*    COEFFICIENTS FOR THE LONGWAVE RADIATION SUBROUTINES               
C                                                                       
C NINT  :  NUMBER OF SPECTRAL INTERVALS                                 
C NIPD  :  NUMBER OF BANDS DESCRIBED BY PADE APPROXIMANTS               
C NTR   :  NUMBER OF REFERENCE TEMPERATURES                             
C NTRA  :  NUMBER OF TRANSMISSION FUNCTIONS                             
C NUA   :  NUMBER OF ABSORBER AMOUNTS                                   
C NG1   :  ORDER OF THE GAUSSIAN QUADRATURE                             
C                                                                       
C RT1   :  ROOTS OF THE GAUSSIAN POLYNOMIAL                             
C WG1   :  WEIGHTS OF THE GAUSSIAN POLYNOMIAL                           
C AT, BT:  COEFFICIENTS OF THE TEMPERATURE DEPENDENCE                   
C CNTNU :  H2O CONTINUUM ABSORPTION COEFFICIENTS                        
C O1H,O2H,PIALF0,OCT: OZONE LONGWAVE ABSORPTION COEFFICIENTS            
C TREF, TSTAND      : REFERENCE TEMPERATURES                            
C XP    :  COEFFICIENTS OF POLYNOMIAL DEVELOPMENTS OF PLANCK FUNCTION   
C GA,GB :  PADE APPROXIMANTS OF H2O, CO2 TRANSMISSION FUNCTIONS         
C ETY,PTY :  PARAMETERS OF H2O CONTINUUM                                
C TINTP :  REFERENCE TEMPERATURES OF PRE-COMPUTED TRANSMISSIVITIES      
C CE    :  EMS COEFFS OF H2O TRANSMISSION FUNCTIONS                     
C                                                                       
      REAL  RT1(2),WG1(2),AT(12,3),BT(12,3),CNTNU(4),O1H                
     S ,OCT(4),TREF,XP(6,6),TSTAND,GA(11,24,3),GB(11,24,3)              
     S ,PTY(11,20),ETY(11,20),TINTP(11),TSTP,O2H,PIALF0                 
C                                                                       
      REAL  TB(40,31,2),ALP(40,31,2),BET(40,31,2),                      
     S ULG(40),PLG(31),AP(40,31,2),BP(40,31,2),TBO3(38,31),TO3B(38,31)  
     S,ULGO(38),AO3(38,31),BO3(38,31),O3A(38,31),O3B(38,31)             
C     ----------------------------------------------------------------- 
C*COMDECK YOMRDU                                                        
C     ----------------------------------------------------------------- 
C                                                                       
C*    CONTROL VARIABLES FOR RADIATION: INTERNAL SWITCH AND INDICES      
C                                                                       
C ZEELOG:  SECURITY THRESHOLD FOR ABSORBER AMOUNT IN LAPLACE TRANSFORM  
C ZEPSC :  SECURITY THRESHOLD FOR CLOUD COVER                           
C ZEPSCO:  SECURITY THRESHOLD FOR OZONE AMOUNT                          
C ZEPSCQ:  SECURITY THRESHOLD FOR WATER VAPOR                           
C ZEPSCT:  SECURITY THRESHOLD FOR SHORTWAVE OPTICAL THICKNESS           
C ZEPSCW:  SECURITY THRESHOLD FOR CLOUD LIQUID WATER PATH               
C                                                                       
      REAL          CDAY,CCO2,CH2O,DIFF,X10E                            
     S             ,ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW            
C     ----------------------------------------------------------------- 
C*COMDECK YOMSW                                                         
C     ----------------------------------------------------------------- 
C                                                                       
C*    COEFFICIENTS FOR THE SHORTWAVE RADIATION SUBROUTINES              
C                                                                       
C APAD  :  PADE APPROXIMANTS NUMERATOR                                  
C BPAD  :  PADE APPROXIMANTS DENOMINATOR                                
C D     :  TRANSMISSION LIMIT FOR INFINITE ABSORBER AMOUNT              
C CRAY  :  RAYLEIGH SCATTERING COEFFICIENTS                             
C SUN   :  SOLAR FRACTION IN SPECTRAL INTERVALS                         
C                                                                       
      REAL  APAD(2,3,7),BPAD(2,3,7),D(2,3),CRAY(2,6),SUN(2)             
C     ----------------------------------------------------------------- 
                                                                        
C                                                                       
C  -------------------------------------------------------------------- 
C                                                                       
      REAL   TTT(1,NLAT,NRLEV,12)                                       
C                                                                       
C        0.1  "RADINT POINTERS"                                         
C             -----------------                                         
C                                                                       
      REAL   TM1(NLP2,NLEV),                                            
     *       TSM1M(NLP2),                                               
     *       QM1(NLP2,NLEV),                                            
     *       SNM1M(NLP2),                                               
     *       ALBM(NLP2),                                                
     *       AVVELM(NLP2)                                               
      REAL   SLMM(NLP2),                                                
     *       APHM1(NLP2,NLEVP1),                                        
     *       APM1(NLP2,NLEV)                                            
C                                                                       
      LOGICAL LOLAND(NLP2)                                              
C                                                                       
C     VARIABLES FOR FULL GRID AND TRANSFORMS.                           
C                                                                       
      REAL   ZTI(NLP2,NRLEVP1),                                         
     *       ZCTI(NLP2,NRLEVP1),                                        
     *       ZCPH(NLP2,NRLEVP1),                                        
     *       ZWV(NLP2,NRLEV),                                           
     *       ZSW(NLP2,NRLEV),                                           
     *       ZALTE(NLP2),                                               
     *       ZCALTE(NLP2),                                              
     *       ZALSO(NLP2),                                               
     *       ZCALSO(NLP2)                                               
C                                                                       
      REAL   ZOZQ(NLP2),                                                
     #       ZOZH(NLP2)                                                 
C                                                                       
      REAL   ZTRSOL(NLP2,NLEVP1),                                       
     *       ZEMTER(NLP2,NLEVP1)                                        
C                                                                       
C     FINAL RESULTS ARE STORED ON:                                      
C                                                                       
      REAL   TRSOLM(NLP2,NLEVP1),                                       
     *       EMTERM(NLP2,NLEVP1)                                        
C                                                                       
      REAL   AMU0(NLP2),                                                
     *       RDAYL(NLP2)                                                
C                                                                       
C     VARIABLES WITHIN THE SPLITTED LATITUDE LOOP.                      
C                                                                       
      REAL   CLC(NRLST,NRLEV),                                          
     *       TI(NRLST,NRLEVP1),                                         
     *       DP(NRLST,NRLEV),                                           
     *       AADS(NRLST,NRLEV),                                         
     *       AEQ1(NRLST,NRLEV),                                         
     *       AEQ2(NRLST,NRLEV),                                         
     *       AEQ3(NRLST,NRLEV),                                         
     *       AEQ4(NRLST,NRLEV),                                         
     *       AEQ5(NRLST,NRLEV),                                         
     *       SSW(NRLST,NRLEV),                                          
     *       WV(NRLST,NRLEV),                                           
     *       QOF(NRLST,NRLEV),                                          
     *       QCF(NRLST,NRLEV),                                          
     *       CLWA(NRLST,NRLEV),                                         
     *       APRE(NRLST),                                               
     *       SMU0(NRLST),                                               
     *       ALTE(NRLST),                                               
     *       ALSO(NRLST)                                                
      REAL   ZDPO(NRLST),                                               
     *       ZQCFO(NRLST),                                              
     *       ZAEQSO(NRLST),                                             
     *       ZQOFO(NRLST),                                              
     *       ZDPN(NRLST),                                               
     *       ZQCFN(NRLST),                                              
     *       ZAEQSN(NRLST),                                             
     *       ZQOFN(NRLST)                                               
      REAL   FLT(NRLST,NRLEVP1),                                        
     *       FLS(NRLST,NRLEVP1)                                         
C                                                                       
C     DIAGNOSTIC VARIABLES.                                             
C                                                                       
      REAL   DIAG(NLEVP1,4),                                            
     *       DIA1(7),                                                   
     *       ZDIAG(NLEVP1,10),                                          
     *       ZDIA1(21),                                                 
     *       ZDIA(NLON),                                                
     *       ZDIAF(NLON,7),                                             
     *       ZDIAT(NLEVP1)                                              
C                                                                       
C                                                                       
C                                                                       
      REAL PGG(IRLST),                                                  
     #     PREF(IRLST),                                                 
     #     PRE1(IRLST),                                                 
     #     PRE2(IRLST),                                                 
     #     PRMUZ(IRLST),                                                
     #     PTO1(IRLST),                                                 
     #     PTR1(IRLST),                                                 
     #     PTR2(IRLST),                                                 
     #     PW(IRLST)                                                    
C                                                                       
C                                                                       
C  what about the 1-dimensional PALBS???                                
C                                                                       
      REAL     PALBSN  (IRLST)                                          
C                                                                       
C*       0.2   "LOCAL" ARRAYS                                           
C              --------------                                           
      REAL ZABSCU(IRLST ,NUA, 3*NRLEV+1),                               
     S     ZABSLY(IRLST ,NUA, 3*NRLEV+1) ,                              
     S     ZADJD (IRLST , IRLVP1)      ,                                
     S     ZADJU (IRLST , IRLVP1)       ,                               
     S     ZBGND (IRLST)                ,                               
     S     ZBINT (IRLST , IRLVP1)       ,                               
     S     ZBSUR (IRLST , NINT)         ,                               
     S     ZBSUIN(IRLST)                ,                               
     S     ZBTOP (IRLST , NINT)         ,                               
     S     PCLDLW(IRLST , IRLEV)        ,                               
     S     ZCLOUD(IRLST)                ,                               
     S     ZCNTRB(IRLST , IRLVP1,IRLVP1),                               
     S     ZCUMP (IRLST)                ,                               
     S     ZCUMPT(IRLST)                ,                               
     S     ZDBDT (IRLST , NINT , IRLEV) ,                               
     S     ZDBSL (IRLST , NINT , 2*NRLEV),                              
     S     ZDISTD(IRLST , IRLVP1)       ,                               
     S     ZDISTU(IRLST , IRLVP1)                                       
      REAL                                                              
     S     ZDNF  (IRLST , IRLVP1,IRLVP1),                               
     S     PDT0  (IRLST)                                                
      REAL                                                              
     S     ZDUC  (IRLST , 3*NRLEV+1)       ,                            
     S     ZEFFT (IRLST)                ,                               
     S     ZEMD  (IRLST , IRLEV), ZEMU  (IRLST , IRLEV),                
     S     ZEPSC2(IRLST),  ZEPSC3(IRLST),                               
     S     ZEPSH1(IRLST),  ZEPSH2(IRLST),                               
     S     ZEPSH3(IRLST),  ZEPSH4(IRLST),                               
     S     ZEPSH5(IRLST),  ZEPSH6(IRLST),                               
     S     ZEPHIO(IRLST),  ZFACTC(IRLST),                               
     S     ZZFD(IRLST), ZFDD(IRLST), ZFDN(IRLST , IRLVP1)               
      REAL                                                              
     S     PFLUX (IRLST , 2 , IRLVP1)   ,                               
     S     PFLUC (IRLST , 2 , IRLVP1)   ,                               
     S     ZFLWP (IRLST)                ,                               
     S     PCTS  (IRLST , IRLEV)        ,                               
     S     ZZFU   (IRLST)               ,                               
     S     ZFUP  (IRLST , IRLVP1)       ,                               
     S     ZGLAYD(IRLST)                ,                               
     S     ZGLAYU(IRLST)                ,                               
     S     ZOMEGA(IRLST , 2 , IRLEV)                                    
C                                                                       
      INTEGER                                                           
     S     ITX   (IRLST)                ,                               
     S     IXDIA (IRLST , IRLVP1)       ,                               
     S     IXT   (IRLST , IRLVP1,IRLVP1),                               
     S     IXTSU (IRLST , IRLVP1)       ,                               
     S     IXTTP (IRLST , IRLVP1)       ,                               
     S     IMX   (IRLST)                ,                               
     S     IMXP  (IRLST)                                                
C                                                                       
      REAL ZOZ   (IRLST , IRLEV)        ,                               
     S     PPMB  (IRLST , IRLVP1)       ,                               
     S     ZRES  (IRLST)                ,                               
     S     PB    (IRLST, 6 , IRLVP1)    ,                               
     S     ZSSIG (IRLST , 3*NRLEV+1)    ,                               
     S     PTAU  (IRLST , 2 , IRLEV)    ,                               
     S     PTAVE (IRLST , IRLEV)        ,                               
     S     ZTAVIC(IRLST)                ,                               
     S     ZTI1  (IRLST)                ,                               
     S     PTL   (IRLST , IRLVP1)       ,                               
     S     ZTT   (IRLST , NTRA)         ,                               
     S     ZTT1  (IRLST , NTRA)         ,                               
     S     ZTT2  (IRLST , NTRA)         ,                               
     S     ZUAER (IRLST , NINT)         ,                               
     S     ZUPF  (IRLST , IRLVP1,IRLVP1),                               
     S     ZUU   (IRLST , NUA)          ,                               
     S     ZXD   (IRLST)                                                
      REAL                                                              
     S     ZXN   (IRLST)                ,                               
     S     ZXOZ  (IRLST)                ,                               
     S     ZXWV  (IRLST)                ,                               
     S     ZZZ   (IRLST)                ,                               
     S     ZRES2 (IRLST)                ,                               
     S     ZTI2  (IRLST)                ,                               
     S     ZBLAY (IRLST,IRLEV)          ,                               
     S     ZBLEV (IRLST,IRLVP1)         ,                               
     S     ZCTS  (IRLST)                ,                               
     S     ZHFG  (IRLST)                                                
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C                                                                       
      INTEGER KT    (IRLST)                                             
      REAL    PUU   (IRLST , NUA  )     ,                               
     S        PTT   (IRLST , NTRA )     ,                               
     S        ZZ    (IRLST)                                             
C                                                                       
C     -------------------------------------------------------- ---------
C                                                                       
      REAL  PCLFR  (IRLST , IRLEV)       ,                              
     S      PT     (IRLST , IRLVP1)       ,                             
     S      PQOF   (IRLST , IRLEV)          ,                           
     S      PQLWP  (IRLST , IRLEV)          ,                           
     S      PEMIS  (IRLST)                                              
      REAL                                                              
     S      PFLT   (IRLST , IRLVP1)                                     
C                                                                       
      REAL  ZALBSU(IRLST , 2 )           ,                              
     S      ZCG   (IRLST , 2 , IRLEV)    ,                              
     S      ZCLDLW(IRLST , IRLEV)        ,                              
     S      ZCLDSW(IRLST , IRLEV)        ,                              
     S      ZCOOLR(IRLST , IRLEV)        ,                              
     S      ZDT0  (IRLST)                ,                              
     S      ZFLUX (IRLST , 2 , IRLVP1)   ,                              
     S      ZPMB  (IRLST , IRLVP1)       ,                              
     S      ZTAU  (IRLST , 2 , IRLEV)    ,                              
     S      ZTAVE (IRLST , IRLEV)        ,                              
     S      ZTL   (IRLST , IRLVP1)       ,                              
     S      ZFSDWN(IRLST , IRLVP1)       ,                              
     S      ZFSUP (IRLST , IRLVP1)       ,                              
     S      ZHEATR(IRLST , IRLEV )                                      
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C                                                                       
      REAL   PDP   (IRLST , IRLEV)     ,                                
     S       PWV   (IRLST , IRLEV)     ,                                
     S       PFLS  (IRLST , IRLVP1)    ,                                
     S       PPSOL (IRLST        )     ,                                
     S       PRMU0 (IRLST        )     ,                                
     S       PAER  (IRLST,IRLEV,5)     ,                                
     S       PALBS (IRLST , 2 )        ,                                
     S       PCG   (IRLST , 2 , IRLEV) ,                                
     S       PCLDSW(IRLST , IRLEV)     ,                                
     S       POMEGA(IRLST , 2 , IRLEV) ,                                
     S       POZ   (IRLST , IRLEV)                                      
C                                                                       
      REAL   PFDOWN(IRLST , IRLVP1)    ,                                
     S       PFUP  (IRLST , IRLVP1)                                     
C                                                                       
      REAL   ZAKI  (IRLST , 2 )        ,                                
     S       ZCGAZ (IRLST , IRLEV)     ,                                
     S       ZCNEB (IRLST , IRLEV)     ,                                
     S       ZC1I  (IRLST , IRLVP1)    ,                                
     S       ZDSC  (IRLST , IRLEV)     ,                                
     S       ZDSH  (IRLST , IRLEV)     ,                                
     S       ZDSIG (IRLST , IRLEV)     ,                                
     S       ZFACT (IRLST)             ,                                
     S       ZFACT1 (IRLST), ZFACT2 (IRLST),                            
     S       ZFACT3 (IRLST), ZFACT4 (IRLST),                            
     S       ZFACT5 (IRLST), ZFACT6 (IRLST),                            
     S       ZFD   (IRLST , IRLVP1)    ,                                
     S       ZFRAC (IRLST)             ,                                
     S       ZFU   (IRLST , IRLVP1)    ,                                
     S       ZG    (IRLST)             ,                                
     S       ZGG   (IRLST)             ,                                
     S       ZPIZAZ(IRLST , IRLEV)     ,                                
     S       ZRAYL (IRLST),  ZRAY1 (IRLST , IRLVP1)                     
      REAL                                                              
     S       ZRAY2 (IRLST , IRLVP1)    ,                                
     S       ZREF  (IRLST)             ,                                
     S       ZREFZ (IRLST , 2 , IRLVP1),                                
     S       ZRE1  (IRLST)             ,                                
     S       ZRE2  (IRLST)             ,                                
     S       ZRJ   (IRLST , 6 , IRLVP1),                                
     S       ZRK   (IRLST , 6 , IRLVP1),                                
     S       ZRL   (IRLST , 8)         ,                                
     S       ZRMU  (IRLST)             ,                                
     S       ZRMUE (IRLST , IRLVP1)    ,                                
     S       ZRMUZ (IRLST)                                              
      REAL                                                              
     S       ZRNEB (IRLST)             ,                                
     S       ZRUEF (IRLST , 8)         ,                                
     S       ZR1   (IRLST)             ,                                
     S       ZR2   (IRLST)             ,                                
     S       ZR21  (IRLST)             ,                                
     S       ZR22  (IRLST)             ,                                
     S       ZR23  (IRLST)             ,                                
     S       ZS    (IRLST)             ,                                
     S       ZSEC  (IRLST)             ,                                
     S       ZSS1  (IRLST)             ,                                
     S       ZTAUAZ(IRLST , IRLEV)     ,                                
     S       ZTOT  (IRLST , IRLVP1)    ,                                
     S       ZTO1  (IRLST)             ,                                
     S       ZTR   (IRLST , 2 , IRLVP1),                                
     S       ZTRA1 (IRLST , IRLVP1)    ,                                
     S       ZTRA2 (IRLST , IRLVP1)                                     
      REAL                                                              
     S       ZTR1  (IRLST)             ,                                
     S       ZTR2  (IRLST)             ,                                
     S       ZUD   (IRLST , 3 , IRLVP1),                                
     S       ZUM   (IRLST , IRLVP1)    ,                                
     S       ZU1D  (IRLST)             ,                                
     S       ZU2D  (IRLST)             ,                                
     S       ZW    (IRLST)             ,                                
     S       ZN175 (IRLST)             ,                                
     S       ZN190 (IRLST)             ,                                
     S       ZO175 (IRLST)             ,                                
     S       ZO190 (IRLST)             ,                                
     S       ZP75  (IRLST)             ,                                
     S       ZP90  (IRLST)             ,                                
     S       ZSIGN (IRLST)             ,                                
     S       ZSIGO (IRLST)                                              
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
      REAL   PTR    (IRLST)     ,                                       
     S       PU     (IRLST)                                             
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
C   CLOUD-POINTERS                                                      
C   --------------                                                      
C                                                                       
      REAL   ZRH(NLP2,NLEV),                                            
     *       ZCH(NLP2),                                                 
     *       ZCM(NLP2),                                                 
     *       ZCL(NLP2),                                                 
     *       ZCT(NLP2),                                                 
     *       ZCC(NLP2)                                                  
      INTEGER NTOPCM(NLP2),                                             
     *        NBASECM(NLP2)                                             
      REAL   ARPRCM(NLP2)                                               
      INTEGER ITOPH(NLP2),                                              
     *        ITOPM(NLP2),                                              
     *        ITOPL(NLP2)                                               
      LOGICAL LOC(NLP2)                                                 
      REAL   ZQS(NLP2,NLEV),                                            
     *       ZCLC(NLP2,NLEV),                                           
     *       ZQLWP(NLP2,NLEV)                                           
C                                                                       
      REAL   VERVEL(NLP2,NLEV),                                         
     *       ZTHETA(NLP2,NLEV),                                         
     *       ZETA(NLP2,NLEV),                                           
     *       ZDTHMIN(NLP2)                                              
      INTEGER INVB(NLP2)                                                
      REAL   ZCTLWP(NLP2),                                              
     *       ZQLOC(NLP2),                                               
     *       ZQMEC(NLP2),                                               
     *       ZQHIC(NLP2)                                                
C                                                                       
C  definitions of "radheat"                                             
C                                                                       
      REAL   ZFSOT(NLON),                                               
     #       ZFTET(NLON),                                               
     #       ZFLT (NLON),                                               
     #       ZFSO (NLON),                                               
     #       ZFTE (NLON),                                               
     #       ZFLB (NLON),                                               
     #       ZDTDT(NLON),                                               
     #       HEATO(NLON,NLEV),                                          
     #       HEATE(NLON,NLEV),                                          
     #       HEATR(NLON,NLEV),                                          
     #       ZI0  (NLON)                                                
C                                                                       
C PIERS 13-8-97                                                         
C                                                                       
         REAL PIERSLWP                                                  
C                                                                       
C Piece of code needed to replace the Nikos radiation scheme with       
C the Morcrette radiation scheme in the igcm3.1.                        
C See morcrette3_1_1.readme for details.                                
C                                                                       
C                                                                       
c      program lh                                                       
c                                                                       
c      parameter(NL=23)                                                 
c      real P(NL),t(NL),h2o(NL),o3(NL),htlw(NL),htsw(NL)                
c      real swalb,doy                                                   
c       REAL CF(4,2)     !cloud fractions,lwps                          
c       INTEGER IC(4,2)   ! cloud positions                             
c       REAL RFLUXES(2,2,2)                                             
c      open(11,file='mls23.dat')                                        
c      read(11,*) NLEV                                                  
c      DOY=80                                                           
c      swalb=0.07                                                       
c      alat=40.0                                                        
c       doy=float(i)                                                    
c      do i=nlev,1,-1                                                   
c         read(11,*) P(i),t(i),h2o(i),adum,o3(i)                        
c         print *,i,p(i),t(i),h2o(i),o3(i)                              
c      enddo                                                            
c                                                                       
c       ic(1,1)=3                                                       
c      ic(1,2)=17                                                       
c       ic(3,1)=6                                                       
c       ic(2,2)=6                                                       
c       cf(1,1)=0.3                                                     
c       cf(1,2)=45.0                                                    
c        cf(3,1)=0.8                                                    
c       cf(4,2)=20.0                                                    
c      alon=0.0                                                         
c      call morcigcm(p,t,h2o,o3,alat,htlw,htsw,doy,cf,ic,               
c     $     rfluxes,swalb,alon)                                         
c      enddo                                                            
c      do i=nl,1,-1                                                     
c         print '(i6,4F14.4)',i,p(i),t(i),htsw(i),htlw(i)               
c      enddo                                                            
c      print *,RFLUXES(1,1,1), RFLUXES(1,1,2), RFLUXES(1,2,1),          
c     $     RFLUXES(1,2,2)                                              
c      print *,RFLUXES(2,1,1), RFLUXES(2,1,2), RFLUXES(2,2,1),          
c     $     RFLUXES(2,2,2)                                              
c                                                                       
c                                                                       
c      end                                                              
c      INCLUDE 'zparb.upd'                                              
                                                                        
C__________________________________________________________________     
C                                                                       
      REAL ZPHM6(NLON),ZPSM6(NLON),ZPHN6(NLON),                         
     *     ZPSN6(NLON)                                                  
      REAL ZEPSH7(NLON),ZEPSH8(NLON),ZEPSH9(NLON)                       
      REAL ZFACTC1(NLON),ZFACTC2(NLON),                                 
     *     ZFACTC3(NLON), ZFACTC4(NLON),ZFACTC5(NLON),                  
     *     ZFN10(NLON)                                                  
      INTEGER IXTP(NLON),IXTB(NLON),IXTS(NLON)                          
      REAL PGA(NLON,12,2,NLEV),PGB(NLON,12,2,NLEV),                     
     *     PPTY(NLON,12,NLEV),PETY(NLON,12,NLEV),                       
     *     PGATOP(NLON,12,2), PGBTOP(NLON,12,2),                        
     *     PGASUR(NLON,12,2), PGBSUR(NLON,12,2),                        
     *     PTYTOP(NLON,12),    ETYTOP(NLON,12),                         
     *     PTYSUR(NLON,12),    ETYSUR(NLON,12)                          
C     COMMON BLOCK FOR GAS ABUNDANCES                                   
!-----------------------------------------------------                  
! Common block for gas abundances                                       
!-----------------------------------------------------                  
      COMMON/MORCGAS/VMRCO2,ZCARDI,VMRCH4,RCH4,VMRN2O,RN2O,             
     &               VMRCFC11,RCFC11,VMRCFC12,RCFC12,                   
     &               VMRH2OS,RH2OS,NEXOTIC                              
      LOGICAL :: NEXOTIC                                                
      REAL :: ZCARDI,RCH4,RN2O,RCFC11,RCFC12,RH2OS                      
      REAL :: VMRCO2,VMRCH4,VMRN2O,VMRCFC11,VMRCFC12,VMRH2OS            
C                                                                       
C  *******************************************************************  
C                                                                       
C     Concentration of the various trace gases                          
C     now entered in INIPHYS, Namelist INMORCGAS.                       
C     M. Bourqui Nov. 2001.                                             
C                                                                       
C                                                                       
      DO 97 JL=1,NRLST                                                  
         DO 98 JK=1,NRLEV                                               
C            PCLD  (JL,JK) = PCLFR (JL,JK)                              
C            PQLW  (JL,JK) = PQLWP (JL,JK)                              
            PCLDLW(JL,JK) = ZCLDLW(JL,JK)                               
            DO 99 II=1,2                                                
               PFLUX (JL,II,JK) = ZFLUX (JL,II,JK)                      
               PTAU  (JL,II,JK) = ZTAU  (JL,II,JK)                      
 99         CONTINUE                                                    
            PPMB  (JL,JK) = ZPMB  (JL,JK)                               
            PTAVE (JL,JK) = ZTAVE (JL,JK)                               
            PTL   (JL,JK) = ZTL   (JL,JK)                               
 98      CONTINUE                                                       
         DO 96 II=1,2                                                   
            PFLUX (JL,II,NRLEV+1) = ZFLUX (JL,II,NRLEV+1)               
 96      CONTINUE                                                       
         PDT0(JL) = ZDT0(JL)                                            
         PPMB(JL,NRLEV+1) = ZPMB(JL,NRLEV+1)                            
         PTL (JL,NRLEV+1) = ZTL (JL,NRLEV+1)                            
 97   CONTINUE                                                          
      ZCCO2 = PCCO2                                                     
C                                                                       
C*         1.     COMPUTES ABSORBER AMOUNTS                             
C                 -------------------------                             
C                                                                       
C*         1.0    INITIALIZATION                                        
C                 --------------                                        
C                                                                       
 100  CONTINUE                                                          
                                                                        
      IMAXC = 0                                                         
                                                                        
! changing - marc 30/3/10                                               
c      DO 101 JL = 1 , (KFLEV+1)*KDLON                                  
c        ZADJD(JL,1) = 0.                                               
c        ZADJU(JL,1) = 0.                                               
c        ZBINT(JL,1) = 0.                                               
c        ZDISTD(JL,1)= 0.                                               
c        ZDISTU(JL,1)= 0.                                               
 101  CONTINUE                                                          
      ZADJD(:,:)=0.0                                                    
      ZADJU(:,:)=0.0                                                    
      ZBINT(:,:)=0.0                                                    
      ZDISTD(:,:)=0.0                                                   
      ZDISTU(:,:)=0.0                                                   
!                                                                       
      DO 114 JK=1,KFLEV                                                 
        DO 113 JL = 1 , KDLON                                           
          PCTS(JL,JK)=0.                                                
 113    CONTINUE                                                        
 114  CONTINUE                                                          
      DO 103 JL = 1 , KDLON                                             
        IXTP(JL)    = 1                                                 
        ZBSUIN(JL) = 0.                                                 
        IMX(JL)    = 0                                                  
        IMXP(JL)   = 0                                                  
 103  CONTINUE                                                          
C                                                                       
C*         1.1    SEARCH THE LAYER INDEX OF THE HIGHEST CLOUD           
C                 -------------------------------------------           
C                                                                       
 110  CONTINUE                                                          
                                                                        
      DO 112 JK = 1 , KFLEV                                             
        DO 111 JL = 1 , KDLON                                           
          IF (PCLDLW(JL,JK).GT.ZEPSC) THEN                              
            IMXP(JL)=JK                                                 
           ELSE                                                         
            IMXP(JL)=IMX(JL)                                            
          ENDIF                                                         
          IMAXC=MAX(IMXP(JL),IMAXC)                                     
          IMX(JL)=IMXP(JL)                                              
 111    CONTINUE                                                        
 112  CONTINUE                                                          
C                                                                       
C                                                                       
C*         1.2    PRESSURE OVER GAUSS SUB-LEVELS                        
C                 ------------------------------                        
C                                                                       
 120  CONTINUE                                                          
                                                                        
      DO 121 JL = 1 , KDLON                                             
        ZSSIG(JL, 1 ) = 1.0                                             
 121  CONTINUE                                                          
                                                                        
      DO 126 JK = 1 , KFLEV                                             
        JKJ=(JK-1)*NG1P1+1                                              
        JKJR = JKJ                                                      
        JKJP = JKJ + NG1P1                                              
        DO 123 JL = 1 , KDLON                                           
          ZSSIG(JL,JKJP)=PPMB(JL,JK+1)/PPMB(JL,1)                       
 123    CONTINUE                                                        
        DO 125 IG1=1,NG1                                                
          JKJ=JKJ+1                                                     
          DO 124 JL = 1 , KDLON                                         
            ZSSIG(JL,JKJ)= (ZSSIG(JL,JKJR)+ZSSIG(JL,JKJP))*0.5          
     *     + RT1(IG1) * (ZSSIG(JL,JKJP) - ZSSIG(JL,JKJR)) * 0.5         
 124      CONTINUE                                                      
 125    CONTINUE                                                        
 126  CONTINUE                                                          
C                                                                       
C                                                                       
C*         1.3    PRESSURE-WEIGHTED TEMPERATURE AND TRANSMISSIVITY INDEX
C                 ------------------------------------------------------
C                                                                       
 130  CONTINUE                                                          
                                                                        
      DO 131 JK = 1 , KFLEV                                             
        DO 132 JL = 1 , KDLON                                           
          ZDST1 = (PTAVE(JL,JK)-TINTP(1)) / TSTP                        
          IXTX = MAX( 1, MIN( MXIXT, INT( ZDST1 + 1. ) ) )              
          ZDSTX = (PTAVE(JL,JK)-TINTP(IXTX))/TSTP                       
          IF (ZDSTX.LT.0.5) THEN                                        
            IXTP(JL) = IXTX                                             
           ELSE                                                         
            IXTP(JL) = IXTX+1                                           
          ENDIF                                                         
 132    CONTINUE                                                        
        DO 133 JG=1,12                                                  
          DO 134 JL = 1 , KDLON                                         
            INDT=IXTP(JL)                                               
            PGA(JL,JG,1,JK)=GA(INDT,2*JG,1)                             
            PGA(JL,JG,2,JK)=GA(INDT,2*JG,2)                             
            PGB(JL,JG,1,JK)=GB(INDT,2*JG,1)                             
            PGB(JL,JG,2,JK)=GB(INDT,2*JG,2)                             
 134      CONTINUE                                                      
 133    CONTINUE                                                        
        DO 1331 JG=1,10                                                 
          DO 1341 JL = 1 , KDLON                                        
            INDT=IXTP(JL)                                               
            PPTY(JL,JG,JK)=PTY(INDT,2*JG)                               
            PETY(JL,JG,JK)=ETY(INDT,2*JG)                               
 1341     CONTINUE                                                      
 1331   CONTINUE                                                        
 131  CONTINUE                                                          
                                                                        
      DO 135 JL=1,KDLON                                                 
        ZDST1 = (PTL(JL,NLEV+1)-TINTP(1)) / TSTP                        
        IXTX = MAX( 1, MIN( MXIXT, INT( ZDST1 + 1. ) ) )                
        ZDSTX = (PTL(JL,KFLEV+1)-TINTP(IXTX))/TSTP                      
        IF (ZDSTX.LT.0.5) THEN                                          
          IXTB(JL)=IXTX                                                 
        ELSE                                                            
          IXTB(JL)=IXTX+1                                               
        ENDIF                                                           
        ZDST1 = (PTL(JL,1)-TINTP(1)) / TSTP                             
        IXTX = MAX( 1, MIN( MXIXT, INT( ZDST1 + 1. ) ) )                
        ZDSTX = (PTL(JL,1)-TINTP(IXTX))/TSTP                            
        IF (ZDSTX.LT.0.5) THEN                                          
          IXTS(JL)=IXTX                                                 
        ELSE                                                            
          IXTS(JL)=IXTX+1                                               
        END IF                                                          
 135  CONTINUE                                                          
                                                                        
      DO 136 JG=1, 12                                                   
        DO 137 JL=1,KDLON                                               
          INDSU=IXTS(JL)                                                
          PGASUR(JL,JG,1)=GA(INDSU,2*JG-1,1)                            
          PGBSUR(JL,JG,1)=GB(INDSU,2*JG-1,1)                            
          PGASUR(JL,JG,2)=GA(INDSU,2*JG-1,2)                            
          PGBSUR(JL,JG,2)=GB(INDSU,2*JG-1,2)                            
          INDTP=IXTB(JL)                                                
          PGATOP(JL,JG,1)=GA(INDTP,2*JG-1,1)                            
          PGBTOP(JL,JG,1)=GB(INDTP,2*JG-1,1)                            
          PGATOP(JL,JG,2)=GA(INDTP,2*JG-1,2)                            
          PGBTOP(JL,JG,2)=GB(INDTP,2*JG-1,2)                            
 137    CONTINUE                                                        
 136  CONTINUE                                                          
                                                                        
      DO 139 JG=1, 10                                                   
        DO 1391 JL=1,KDLON                                              
          INDSU=IXTS(JL)                                                
          PTYSUR(JL,JG)=PTY(INDSU,2*JG-1)                               
          ETYSUR(JL,JG)=ETY(INDSU,2*JG-1)                               
          INDTP=IXTB(JL)                                                
          PTYTOP(JL,JG)=PTY(INDTP,2*JG-1)                               
          ETYTOP(JL,JG)=ETY(INDTP,2*JG-1)                               
 1391   CONTINUE                                                        
 139  CONTINUE                                                          
C                                                                       
C*         1.4    PRESSURE THICKNESS AND MEAN PRESSURE OF SUB-LAYERS    
C                 --------------------------------------------------    
C                                                                       
 140  CONTINUE                                                          
                                                                        
      DO 142 JKI=1,3*KFLEV                                              
        JKIP1=JKI+1                                                     
        DO 141 JL = 1 , KDLON                                           
          ZABSLY(JL,5,JKI)=(ZSSIG(JL,JKI)+ZSSIG(JL,JKIP1))*0.5          
          ZABSLY(JL,3,JKI)=(ZSSIG(JL,JKI)-ZSSIG(JL,JKIP1))              
     *                     *PPSOL(JL)/(10.*G)                           
 141    CONTINUE                                                        
 142  CONTINUE                                                          
                                                                        
      DO 146 JK = 1 , KFLEV                                             
        JKP1=JK+1                                                       
        JKL = KFLEV+1 - JK                                              
        DO 143 JL = 1 , KDLON                                           
          ZXWV(JL) = MAX(PWV(JL,JKL) , ZEPSCQ )                         
          ZXOZ(JL) = MAX(PQOF(JL,JKL) / PDP(JL,JKL) , ZEPSCO )          
 143    CONTINUE                                                        
        JKJ=(JK-1)*NG1P1+1                                              
        JKJPN=JKJ+NG1                                                   
                                                                        
        DO 145 JKK=JKJ,JKJPN                                            
          DO 144 JL = 1 , KDLON                                         
            ZDPM = ZABSLY(JL,3,JKK)                                     
            ZUPM = ZABSLY(JL,5,JKK) * PPSOL(JL) * ZDPM / 101325.        
            ZDUC(JL,JKK)=ZDPM                                           
                                                                        
            ZABSLY(JL,12,JKK)=ZXOZ(JL)*ZDPM*208.333                     
            ZABSLY(JL,8,JKK) =PCCO2   *ZDPM*227.2159                    
            ZABSLY(JL,13,JKK)=ZXOZ(JL)*ZUPM*208.333 *1013.25            
            ZABSLY(JL,7,JKK) =PCCO2   *ZUPM*227.2159*1013.25            
            ZABSLY(JL,32,JKK)=ZXOZ(JL)*ZDPM*208.333 *PTAVE(JL,JK)       
            ZABSLY(JL,33,JKK)=PCCO2   *ZDPM*227.2159*PTAVE(JL,JK)       
                                                                        
            ZU6=ZXWV(JL) * ZUPM                                         
            ZFPPW= 1.6078 *ZXWV(JL)/(1.+0.608*ZXWV(JL))                 
            ZABSLY(JL,6,JKK)=ZU6                                        
     #            * (1. + DELTAH/ZABSLY(JL,5,JKK))                      
            ZABSLY(JL,11,JKK)=ZU6*ZFPPW                                 
            ZABSLY(JL,10,JKK)=ZU6*(1.-ZFPPW)                            
            ZABSLY(JL,9,JKK) = PCCO2 * ZUPM                             
     #            * (1. + DELTAC/ZABSLY(JL,5,JKK))                      
 144      CONTINUE                                                      
 145    CONTINUE                                                        
 146  CONTINUE                                                          
C                                                                       
C*         1.5    CUMULATIVE ABSORBER AMOUNTS FROM TOP OF ATMOSPHERE    
C                 --------------------------------------------------    
C                                                                       
 150  CONTINUE                                                          
                                                                        
! changing - marc 30/3/10                                               
c      DO 151 JL = 1 , NUA*KDLON                                        
c        ZABSCU(JL,1,3*KFLEV+1)=0.                                      
c 151  CONTINUE                                                         
      ZABSCU(:,1,3*KFLEV+1)=0.                                          
                                                                        
      DO 159 JK = 1 , KFLEV                                             
        JJ=(JK-1)*NG1P1+1                                               
        JJPN=JJ+NG1                                                     
        JKL=KFLEV+1-JK                                                  
C                                                                       
C*         1.5.1  CUMULATIVE AEROSOL AMOUNTS FROM TOP OF ATMOSPHERE     
C                 --------------------------------------------------    
C                                                                       
 1510   CONTINUE                                                        
                                                                        
        JAE1=3*KFLEV+1-JJ                                               
        JAE2=3*KFLEV+1-(JJ+1)                                           
        JAE3=3*KFLEV+1-JJPN                                             
        DO 1512 IAE=1,5                                                 
          DO 1511 JL = 1 , KDLON                                        
            ZUAER(JL,IAE) = (CAER(IAE,1)*PAER(JL,JKL,1)                 
     *      +CAER(IAE,2)*PAER(JL,JKL,2)+CAER(IAE,3)*PAER(JL,JKL,3)      
     *      +CAER(IAE,4)*PAER(JL,JKL,4)+CAER(IAE,5)*PAER(JL,JKL,5))     
     *      /(ZDUC(JL,JAE1)+ZDUC(JL,JAE2)+ZDUC(JL,JAE3))                
 1511     CONTINUE                                                      
 1512   CONTINUE                                                        
C                                                                       
C*         1.5.2  INTRODUCES TEMPERATURE EFFECTS ON ABSORBER AMOUNTS    
C                                                                       
 1520   CONTINUE                                                        
                                                                        
        DO 1521 JL = 1 , KDLON                                          
          ZTAVIC(JL)=PTAVE(JL,JKL)                                      
C                                                                       
C- TEMPERATURE DEPENDENCE FOR WATER VAPOR CONTINUUM (FROM CKD MODEL)    
C                                                                       
          ZFACTC(JL)  = 296./ZTAVIC(JL)                                 
          ZZZZ         = (1.0-ZTAVIC(JL)/296.)                          
          ZFACTC1(JL) = EXP(3.3338*ZZZZ)                                
          ZFACTC2(JL) = EXP(5.2775*ZZZZ)                                
          ZFACTC3(JL) = EXP(5.9022*ZZZZ)                                
          ZFACTC4(JL) = EXP(6.2938*ZZZZ)                                
          ZFACTC5(JL) = EXP(7.0252*ZZZZ)                                
                                                                        
          ZTX=ZTAVIC(JL)-TREF                                           
          ZTX2=ZTX*ZTX                                                  
          ZZABSLY = ZABSLY(JL,6,JAE1)                                   
     *             +ZABSLY(JL,6,JAE2)                                   
     *             +ZABSLY(JL,6,JAE3)                                   
          ZUP=MIN(MAX( 0.5*X10E*LOG( ZZABSLY ) +5., 0.), 6.0)           
          ZCAH1=AT(1,1)+ZUP*(AT(1,2)+ZUP*(AT(1,3)))                     
          ZCBH1=BT(1,1)+ZUP*(BT(1,2)+ZUP*(BT(1,3)))                     
          ZEPSH1(JL)=EXP( ZCAH1 * ZTX + ZCBH1 * ZTX2 )                  
          ZCAH2=AT(2,1)+ZUP*(AT(2,2)+ZUP*(AT(2,3)))                     
          ZCBH2=BT(2,1)+ZUP*(BT(2,2)+ZUP*(BT(2,3)))                     
          ZEPSH2(JL)=EXP( ZCAH2 * ZTX + ZCBH2 * ZTX2 )                  
          ZCAH3=AT(3,1)+ZUP*(AT(3,2)+ZUP*(AT(3,3)))                     
          ZCBH3=BT(3,1)+ZUP*(BT(3,2)+ZUP*(BT(3,3)))                     
          ZEPSH3(JL)=EXP( ZCAH3 * ZTX + ZCBH3 * ZTX2 )                  
          ZCAH4=AT(4,1)+ZUP*(AT(4,2)+ZUP*(AT(4,3)))                     
          ZCBH4=BT(4,1)+ZUP*(BT(4,2)+ZUP*(BT(4,3)))                     
          ZEPSH4(JL)=EXP( ZCAH4 * ZTX + ZCBH4 * ZTX2 )                  
          ZCAH5=AT(5,1)+ZUP*(AT(5,2)+ZUP*(AT(5,3)))                     
          ZCBH5=BT(5,1)+ZUP*(BT(5,2)+ZUP*(BT(5,3)))                     
          ZEPSH5(JL)=EXP( ZCAH5 * ZTX + ZCBH5 * ZTX2 )                  
          ZCAH6=AT(6,1)+ZUP*(AT(6,2)+ZUP*(AT(6,3)))                     
          ZCBH6=BT(6,1)+ZUP*(BT(6,2)+ZUP*(BT(6,3)))                     
          ZEPSH6(JL)=EXP( ZCAH6 * ZTX + ZCBH6 * ZTX2 )                  
C                                                                       
C*--1250-1450 CM-1 H2O                                                  
C                                                                       
          ZCAH7=AT(7,1)+ZUP*(AT(7,2)+ZUP*(AT(7,3)))                     
          ZCBH7=BT(7,1)+ZUP*(BT(7,2)+ZUP*(BT(7,3)))                     
          ZEPSH7(JL)=EXP( ZCAH7 * ZTX + ZCBH7 * ZTX2 )                  
C                                                                       
C*--800-970 CM-1 H2O                                                    
C                                                                       
          ZCAH8=AT(8,1)+ZUP*(AT(8,2)+ZUP*(AT(8,3)))                     
          ZCBH8=BT(8,1)+ZUP*(BT(8,2)+ZUP*(BT(8,3)))                     
          ZEPSH8(JL)=EXP( ZCAH8 * ZTX + ZCBH8 * ZTX2 )                  
C                                                                       
C*--650-800 CM-1 H2O                                                    
C                                                                       
          ZCAH9=AT(9,1)+ZUP*(AT(9,2)+ZUP*(AT(9,3)))                     
          ZCBH9=BT(9,1)+ZUP*(BT(9,2)+ZUP*(BT(9,3)))                     
          ZEPSH9(JL)=EXP( ZCAH9 * ZTX + ZCBH9 * ZTX2 )                  
                                                                        
          ZPHM6(JL)=EXP(-5.81E-4 * ZTX - 1.13E-6 * ZTX2)                
          ZPSM6(JL)=EXP(-5.57E-4 * ZTX - 3.30E-6 * ZTX2)                
          ZPHN6(JL)=EXP(-3.46E-5 * ZTX + 2.05E-6 * ZTX2)                
          ZPSN6(JL)=EXP( 3.70E-3 * ZTX - 2.30E-6 * ZTX2)                
 1521   CONTINUE                                                        
                                                                        
        DO 1522 JL = 1 , KDLON                                          
          ZTAVIC(JL)=PTAVE(JL,JKL)                                      
          ZTX=ZTAVIC(JL)-TREF                                           
          ZTX2=ZTX*ZTX                                                  
          ZZABSLY= ZABSLY(JL,9,JAE1)                                    
     *            +ZABSLY(JL,9,JAE2)                                    
     *            +ZABSLY(JL,9,JAE3)                                    
          ZALUP = X10E * LOG( ZZABSLY )                                 
          ZUP   = MAX( 0.00 , 5.00 + 0.50 * ZALUP )                     
          ZEPSC2(JL) = (ZTAVIC(JL)/TREF) ** ZUP                         
          ZCAC8=AT(11,1)+ZUP*(AT(11,2)+ZUP*(AT(11,3)))                  
          ZCBC8=BT(11,1)+ZUP*(BT(11,2)+ZUP*(BT(11,3)))                  
          ZEPSC3(JL) = EXP( ZCAC8  * ZTX + ZCBC8  * ZTX2 )              
          ZEPHIO(JL) = EXP( OCT(1) * ZTX + OCT(2) * ZTX2)               
C          ZEPSIO(JL) = EXP( 2.* (OCT(3) * ZTX+OCT(4)*ZTX2))            
 1522   CONTINUE                                                        
                                                                        
        DO 1524 JKK=JJ,JJPN                                             
          JC=3*KFLEV+1-JKK                                              
          JCP1=JC+1                                                     
          DO 1523 JL = 1 , KDLON                                        
            ZABSCU(JL,10,JC)=ZABSCU(JL,10,JCP1)                         
     *                 +ZABSLY(JL,10,JC)  * ZFACTC(JL)                  
            ZABSCU(JL,25,JC)=ZABSCU(JL,25,JCP1)                         
     *                 +ZABSLY(JL,11,JC)  * ZFACTC2(JL)                 
            ZABSCU(JL,26,JC)=ZABSCU(JL,26,JCP1)                         
     *                 +ZABSLY(JL,11,JC)  * ZFACTC3(JL)                 
            ZABSCU(JL,27,JC)=ZABSCU(JL,27,JCP1)                         
     *                 +ZABSLY(JL,11,JC)  * ZFACTC4(JL)                 
            ZABSCU(JL,28,JC)=ZABSCU(JL,28,JCP1)                         
     *                 +ZABSLY(JL,11,JC)  * ZFACTC5(JL)                 
            ZABSCU(JL,11,JC)=ZABSCU(JL,11,JCP1)                         
     *                 +ZABSLY(JL,11,JC)  * ZFACTC1(JL)                 
c                                                                       
            ZABSCU(JL,12,JC)=ZABSCU(JL,12,JCP1)+ZABSLY(JL,12,JC)        
            ZABSCU(JL,13,JC)=ZABSCU(JL,13,JCP1)+ZABSLY(JL,13,JC)        
            ZABSCU(JL,32,JC)=ZABSCU(JL,32,JCP1)+ZABSLY(JL,32,JC)        
            ZABSCU(JL, 7,JC)=ZABSCU(JL, 7,JCP1)+ZABSLY(JL, 7,JC)        
            ZABSCU(JL,34,JC)=ZABSCU(JL,34,JCP1)+ZABSLY(JL, 8,JC)        
            ZABSCU(JL,33,JC)=ZABSCU(JL,33,JCP1)+ZABSLY(JL,33,JC)        
c                                                                       
            ZABSCU(JL,1,JC)=ZABSCU(JL,1,JCP1)                           
     *                +ZABSLY(JL,6,JC)  * ZEPSH1(JL)                    
            ZABSCU(JL,2,JC)=ZABSCU(JL,2,JCP1)                           
     *                +ZABSLY(JL,6,JC)  * ZEPSH2(JL)                    
            ZABSCU(JL,3,JC)=ZABSCU(JL,3,JCP1)                           
     *                +ZABSLY(JL,6,JC)  * ZEPSH5(JL)                    
            ZABSCU(JL,4,JC)=ZABSCU(JL,4,JCP1)                           
     *                +ZABSLY(JL,6,JC)  * ZEPSH3(JL)                    
            ZABSCU(JL,5,JC)=ZABSCU(JL,5,JCP1)                           
     *                +ZABSLY(JL,6,JC)  * ZEPSH4(JL)                    
                                                                        
            ZABSCU(JL,29,JC)=ZABSCU(JL,29,JCP1)                         
     *                +ZABSLY(JL,6,JC)  * ZEPSH7(JL)                    
            ZABSCU(JL,30,JC)=ZABSCU(JL,30,JCP1)                         
     *                +ZABSLY(JL,6,JC)  * ZEPSH8(JL)                    
            ZABSCU(JL,31,JC)=ZABSCU(JL,31,JCP1)                         
     *                +ZABSLY(JL,6,JC)  * ZEPSH9(JL)                    
                                                                        
            ZABSCU(JL,6,JC)=ZABSCU(JL,6,JCP1)                           
     *                +ZABSLY(JL,6,JC)  * ZEPSH6(JL)                    
                                                                        
            ZABSCU(JL,14,JC)=ZABSCU(JL,14,JCP1)                         
     *                +ZUAER(JL,1)    * ZDUC(JL,JC)*DIFF                
            ZABSCU(JL,15,JC)=ZABSCU(JL,15,JCP1)                         
     *                +ZUAER(JL,2)    * ZDUC(JL,JC)*DIFF                
            ZABSCU(JL,16,JC)=ZABSCU(JL,16,JCP1)                         
     *                +ZUAER(JL,3)    * ZDUC(JL,JC)*DIFF                
            ZABSCU(JL,17,JC)=ZABSCU(JL,17,JCP1)                         
     *                +ZUAER(JL,4)    * ZDUC(JL,JC)*DIFF                
            ZABSCU(JL,18,JC)=ZABSCU(JL,18,JCP1)                         
     *                +ZUAER(JL,5)    * ZDUC(JL,JC)*DIFF                
                                                                        
           ZABSCU(JL,19,JC)=ZABSCU(JL,19,JCP1)                          
     *         +ZABSLY(JL,8,JC)*RCH4/PCCO2/227.2159*ZPHM6(JL)*DIFF      
            ZABSCU(JL,20,JC)=ZABSCU(JL,20,JCP1)                         
     *               +ZABSLY(JL,9,JC)*RCH4/PCCO2*ZPSM6(JL)*DIFF         
            ZABSCU(JL,21,JC)=ZABSCU(JL,21,JCP1)                         
     *         +ZABSLY(JL,8,JC)*RN2O/PCCO2/227.2159*ZPHN6(JL)*DIFF      
            ZABSCU(JL,22,JC)=ZABSCU(JL,22,JCP1)                         
     *               +ZABSLY(JL,9,JC)*RN2O/PCCO2*ZPSN6(JL)*DIFF         
                                                                        
            ZABSCU(JL,23,JC)=ZABSCU(JL,23,JCP1)                         
     *               +ZABSLY(JL,8,JC)*RCFC11/PCCO2/227.2159  *DIFF      
           ZABSCU(JL,24,JC)=ZABSCU(JL,24,JCP1)                          
     *               +ZABSLY(JL,8,JC)*RCFC12/PCCO2/227.2159  *DIFF      
                                                                        
            ZABSCU(JL,8,JC)=ZABSCU(JL,8,JCP1)                           
     *               +ZABSLY(JL,9,JC)*ZEPSC3(JL)*DIFF                   
            ZABSCU(JL,9,JC)=ZABSCU(JL,9,JCP1)                           
     *               +ZABSLY(JL,9,JC)*ZEPSC3(JL)*DIFF                   
 1523     CONTINUE                                                      
 1524   CONTINUE                                                        
 159  CONTINUE                                                          
C                                                                       
C*         1.6     PLANCK FUNCTIONS AND GRADIENTS                       
C                  ------------------------------                       
C                                                                       
 160  CONTINUE                                                          
                                                                        
      DO 168 JNU=1,NINT                                                 
C                                                                       
C*         1.6.1   LEVELS FROM SURFACE TO KFLEV                         
C                  ----------------------------                         
C                                                                       
 1610   CONTINUE                                                        
                                                                        
        DO 1612 JK = 1 , KFLEV                                          
          DO 1611 JL = 1 , KDLON                                        
            ZTI(JL,JK)=(PTL(JL,JK)-TSTAND)/TSTAND                       
            ZRES(JL) = XP(1,JNU)+ZTI(JL,JK)*(XP(2,JNU)+ZTI(JL,JK)       
     *      *(XP(3,JNU)                                                 
     *     +ZTI(JL,JK)*(XP(4,JNU)+ZTI(JL,JK)*(XP(5,JNU)+ZTI(JL,JK)      
     *      *(XP(6,JNU)                                                 
     *       )))))                                                      
            ZBINT(JL,JK)=ZBINT(JL,JK)+ZRES(JL)                          
            PB(JL,JNU,JK)= ZRES(JL)                                     
            ZBLEV(JL,JK) = ZRES(JL)                                     
            ZTI2(JL)=(PTAVE(JL,JK)-TSTAND)/TSTAND                       
            ZRES2(JL)=XP(1,JNU)+ZTI2(JL)*(XP(2,JNU)+ZTI2(JL)*(XP(3,JNU) 
     *     +ZTI2(JL)*(XP(4,JNU)+ZTI2(JL)*(XP(5,JNU)+ZTI2(JL)*(XP(6,JNU) 
     *       )))))                                                      
            ZBLAY(JL,JK) = ZRES2(JL)                                    
 1611     CONTINUE                                                      
 1612   CONTINUE                                                        
C                                                                       
C*         1.6.2   TOP OF THE ATMOSPHERE AND SURFACE                    
C                  ---------------------------------                    
C                                                                       
 1620   CONTINUE                                                        
                                                                        
        DO 1621 JL = 1 , KDLON                                          
          ZTI(JL,JK)=(PTL(JL,KFLEV+1)-TSTAND)/TSTAND                    
          ZTI2(JL) = (PTL(JL,1) + PDT0(JL) - TSTAND) / TSTAND           
          ZRES(JL) = XP(1,JNU)+ZTI(JL,JK)*(XP(2,JNU)+ZTI(JL,JK)         
     *     *(XP(3,JNU)                                                  
     *     +ZTI(JL,JK)*(XP(4,JNU)+ZTI(JL,JK)*(XP(5,JNU)+ZTI(JL,JK)      
     *     *(XP(6,JNU)                                                  
     *        )))))                                                     
          ZRES2(JL) = XP(1,JNU)+ZTI2(JL)*(XP(2,JNU)+ZTI2(JL)*(XP(3,JNU) 
     *     +ZTI2(JL)*(XP(4,JNU)+ZTI2(JL)*(XP(5,JNU)+ZTI2(JL)*(XP(6,JNU) 
     *        )))))                                                     
          ZBINT(JL,KFLEV+1) = ZBINT(JL,KFLEV+1)+ZRES(JL)                
          PB(JL,JNU,KFLEV+1)= ZRES(JL)                                  
          ZBLEV(JL,KFLEV+1) = ZRES(JL)                                  
          ZBTOP(JL,JNU) = ZRES(JL)                                      
          ZBSUR(JL,JNU) = ZRES2(JL)                                     
          ZBSUIN(JL) = ZBSUIN(JL) + ZRES2(JL)                           
 1621   CONTINUE                                                        
C                                                                       
C*         1.6.3   GRADIENTS IN SUB-LAYERS                              
C                  -----------------------                              
C                                                                       
 1630   CONTINUE                                                        
                                                                        
        DO 1632 JK = 1 , KFLEV                                          
          JK2 = 2 * JK                                                  
          JK1 = JK2 - 1                                                 
          DO 1631 JL = 1 , KDLON                                        
            ZDBSL(JL,JNU,JK1) = ZBLAY(JL,JK  ) - ZBLEV(JL,JK)           
            ZDBSL(JL,JNU,JK2) = ZBLEV(JL,JK+1) - ZBLAY(JL,JK)           
 1631     CONTINUE                                                      
 1632   CONTINUE                                                        
 168  CONTINUE                                                          
C                                                                       
C*         1.7     INITIALIZE TRANSMISSION FUNCTIONS                    
C                  ---------------------------------                    
C                                                                       
 170  CONTINUE                                                          
                                                                        
! changing - marc 30/3/10                                               
c      DO 171 JL = 1 , NTRA*KDLON                                       
c        ZTT (JL,1) = 1.0                                               
c        ZTT1(JL,1) = 1.0                                               
c        ZTT2(JL,1) = 1.0                                               
c 171  CONTINUE                                                         
      ZTT (:,:) = 1.0                                                   
      ZTT1(:,:) = 1.0                                                   
      ZTT2(:,:) = 1.0                                                   
C                                                                       
C     --------------------------------------------------------          
C                                                                       
C*         2.      VERTICAL INTEGRATION                                 
C                  --------------------                                 
C                                                                       
 200  CONTINUE                                                          
C                                                                       
C*         2.1     CONTRIBUTION FROM ADJACENT LAYERS                    
C                  ---------------------------------                    
C                                                                       
 210  CONTINUE                                                          
                                                                        
      DO 215 JK = 1 , KFLEV                                             
C                                                                       
C*         2.1.1   DOWNWARD LAYERS                                      
C                  ---------------                                      
C                                                                       
 2110   CONTINUE                                                        
                                                                        
        KM12 = 2 * (JK - 1)                                             
        KND = (JK - 1) * NG1P1 + 1                                      
        KNU = JK * NG1P1 + 1                                            
                                                                        
        DO 2111 JL = 1 , KDLON                                          
          ZGLAYD(JL) = 0.                                               
          ZGLAYU(JL) = 0.                                               
 2111   CONTINUE                                                        
                                                                        
        DO 213 IG = 1 , NG1                                             
          KBS = KM12 + IG                                               
          KDD = KND + IG                                                
! changing - marc 30/3/10                                               
c          DO 2112 JL = 1 , NUAER*KDLON                                 
c            ZUU(JL,1) = ZABSCU(JL,1,KND) - ZABSCU(JL,1,KDD)            
c 2112     CONTINUE                                                     
          DO I=1,NUAER                                                  
             DO JL=1,KDLON                                              
                ZUU(JL,I) = ZABSCU(JL,I,KND) - ZABSCU(JL,I,KDD)         
             ENDDO                                                      
          ENDDO                                                         
C                                                                       
          CALL LWTT0(KDLON,KRLST,KFLEV,ITASK,                           
     *               PGA(1,1,1,JK),PGB(1,1,1,JK),                       
     *               PPTY(1,1,JK), PETY(1,1,JK))                        
                                                                        
          DO 2114 JL = 1 , KDLON                                        
            ZWTR=ZDBSL(JL,1,KBS)*ZTT(JL,1)          *ZTT(JL,10)         
     *          +ZDBSL(JL,2,KBS)*ZTT(JL,2)*ZTT(JL,7)*ZTT(JL,11)         
     *          +ZDBSL(JL,3,KBS)*ZTT(JL,4)*ZTT(JL,8)*ZTT(JL,12)         
     *          +ZDBSL(JL,4,KBS)*ZTT(JL,5)*ZTT(JL,9)*ZTT(JL,13)         
     *          +ZDBSL(JL,5,KBS)*ZTT(JL,3)          *ZTT(JL,14)         
     *          +ZDBSL(JL,6,KBS)*ZTT(JL,6)          *ZTT(JL,15)         
            ZGLAYD(JL)=ZGLAYD(JL)+ZWTR*WG1(IG)                          
 2114     CONTINUE                                                      
C                                                                       
C*         2.1.2   DOWNWARD LAYERS                                      
C                  ---------------                                      
C                                                                       
 2120     CONTINUE                                                      
                                                                        
! changing - marc 30/3/10                                               
c          DO 2121 JL = 1 , NUAER*KDLON                                 
c            ZUU(JL,1) = ZABSCU(JL,1,KDD) - ZABSCU(JL,1,KNU)            
c 2121     CONTINUE                                                     
         DO I=1,NUAER                                                   
            DO JL=1,KDLON                                               
               ZUU(JL,I) = ZABSCU(JL,I,KDD) - ZABSCU(JL,I,KNU)          
            ENDDO                                                       
         ENDDO                                                          
                                                                        
          CALL LWTT0(KDLON,KRLST,KFLEV,ITASK,                           
     *               PGA(1,1,1,JK),PGB(1,1,1,JK),                       
     *               PPTY(1,1,JK), PETY(1,1,JK))                        
                                                                        
          DO 2123 JL = 1 , KDLON                                        
            ZWTR=ZDBSL(JL,1,KBS)*ZTT(JL,1)          *ZTT(JL,10)         
     *          +ZDBSL(JL,2,KBS)*ZTT(JL,2)*ZTT(JL,7)*ZTT(JL,11)         
     *          +ZDBSL(JL,3,KBS)*ZTT(JL,4)*ZTT(JL,8)*ZTT(JL,12)         
     *          +ZDBSL(JL,4,KBS)*ZTT(JL,5)*ZTT(JL,9)*ZTT(JL,13)         
     *          +ZDBSL(JL,5,KBS)*ZTT(JL,3)          *ZTT(JL,14)         
     *          +ZDBSL(JL,6,KBS)*ZTT(JL,6)          *ZTT(JL,15)         
            ZGLAYU(JL)=ZGLAYU(JL)+ZWTR*WG1(IG)                          
 2123     CONTINUE                                                      
 213    CONTINUE                                                        
                                                                        
        DO 214 JL = 1 , KDLON                                           
          ZADJD(JL,JK) = ZGLAYD(JL)                                     
          ZCNTRB(JL,JK,JK+1  ) = ZGLAYD(JL)                             
          ZADJU(JL,JK+1) = ZGLAYU(JL)                                   
          ZCNTRB(JL,JK+1,JK  ) = ZGLAYU(JL)                             
          ZCNTRB(JL,JK  ,JK  ) = 0.0                                    
 214    CONTINUE                                                        
 215  CONTINUE                                                          
C                                                                       
C                                                                       
      DO 218 JK = 1 , KFLEV                                             
        JK2 = 2 * JK                                                    
        JK1 = JK2 - 1                                                   
! changing - marc 8/3/10                                                
!        DO 216 JL = 1 , NINT*KDLON                                     
!          ZDBDT(JL,1,JK) = ZDBSL(JL,1,JK1) + ZDBSL(JL,1,JK2)           
! 216    CONTINUE                                                       
        DO I=1,NINT                                                     
           DO JL=1,KDLON                                                
               ZDBDT(JL,I,JK) = ZDBSL(JL,I,JK1) + ZDBSL(JL,I,JK2)       
            ENDDO                                                       
         ENDDO                                                          
                                                                        
 218  CONTINUE                                                          
C                                                                       
C*         2.2     CONTRIBUTION FROM DISTANT LAYERS                     
C                  ---------------------------------                    
C                                                                       
 220  CONTINUE                                                          
C                                                                       
C*         2.2.1   DISTANT AND ABOVE LAYERS                             
C                  ------------------------                             
C                                                                       
 2210 CONTINUE                                                          
C                                                                       
C*         2.2.2   FIRST UPPER LEVEL                                    
C                  -----------------                                    
C                                                                       
 2220 CONTINUE                                                          
                                                                        
      DO 225 JK = 1 , KFLEV-1                                           
        JKP1=JK+1                                                       
        KN = (JK-1)*NG1P1+1                                             
        KD1= KN+NG1P1                                                   
                                                                        
! changing - marc 30/3/10                                               
c        DO 2222 JL = 1 , NUAER*KDLON                                   
c          ZUU(JL,1) = ZABSCU(JL,1,KN) - ZABSCU(JL,1,KD1)               
c 2222   CONTINUE                                                       
          DO I=1,NUAER                                                  
             DO JL=1,KDLON                                              
               ZUU(JL,I) = ZABSCU(JL,I,KN) - ZABSCU(JL,I,KD1)           
             ENDDO                                                      
          ENDDO                                                         
                                                                        
        CALL LWTT0(KDLON,KRLST,KFLEV,ITASK,                             
     *             PGA(1,1,1,JK),PGB(1,1,1,JK),                         
     *             PPTY(1,1,JK), PETY(1,1,JK))                          
                                                                        
! changing - marc 30/3/10                                               
c        DO 2224 JL = 1 , NTRAER*KDLON                                  
c          ZTT1(JL,1)=ZTT(JL,1)                                         
c 2224   CONTINUE                                                       
        DO I=1,NTRAER                                                   
           DO JL=1,KDLON                                                
              ZTT1(JL,I)=ZTT(JL,I)                                      
           ENDDO                                                        
        ENDDO                                                           
C                                                                       
C*         2.2.3   HIGHER UP                                            
C                  ---------                                            
C                                                                       
 2230   CONTINUE                                                        
                                                                        
        DO 224 JKJ=JKP1,KFLEV                                           
          KJP1=JKJ+1                                                    
          KD2= JKJ  *NG1P1+1                                            
                                                                        
! changing - marc 30/3/10                                               
c          DO 2232 JL = 1 , NUAER*KDLON                                 
c            ZUU(JL,1) = ZABSCU(JL,1,KN) - ZABSCU(JL,1,KD2)             
c 2232     CONTINUE                                                     
          DO I=1,NUAER                                                  
             DO JL=1,KDLON                                              
                ZUU(JL,I) = ZABSCU(JL,I,KN) - ZABSCU(JL,I,KD2)          
             ENDDO                                                      
          ENDDO                                                         
                                                                        
          CALL LWTT0(KDLON,KRLST,KFLEV,ITASK,                           
     *               PGA(1,1,1,JKJ),PGB(1,1,1,JKJ),                     
     *               PPTY(1,1,JKJ), PETY(1,1,JKJ))                      
! changing - marc 30/3/10                                               
c          DO 2234 JL = 1 , NTRAER*KDLON                                
c            ZTT2(JL,1)=(ZTT1(JL,1)+ZTT(JL,1))*0.5                      
c            ZTT1(JL,1)=ZTT(JL,1)                                       
c 2234     CONTINUE                                                     
          DO I=1,NTRAER                                                 
             DO JL=1,KDLON                                              
                ZTT2(JL,I)=(ZTT1(JL,I)+ZTT(JL,I))*0.5                   
                ZTT1(JL,I)=ZTT(JL,I)                                    
             ENDDO                                                      
          ENDDO                                                         
                                                                        
          DO 2236 JL = 1 , KDLON                                        
            ZWW=ZDBDT(JL,1,JKJ)*ZTT2(JL,1)           *ZTT2(JL,10)       
     *        + ZDBDT(JL,2,JKJ)*ZTT2(JL,2)*ZTT2(JL,7)*ZTT2(JL,11)       
     *        + ZDBDT(JL,3,JKJ)*ZTT2(JL,4)*ZTT2(JL,8)*ZTT2(JL,12)       
     *        + ZDBDT(JL,4,JKJ)*ZTT2(JL,5)*ZTT2(JL,9)*ZTT2(JL,13)       
     *        + ZDBDT(JL,5,JKJ)*ZTT2(JL,3)           *ZTT2(JL,14)       
     *        + ZDBDT(JL,6,JKJ)*ZTT2(JL,6)           *ZTT2(JL,15)       
            ZGLAYD(JL)=ZWW                                              
            ZDZXDG=ZGLAYD(JL)                                           
            ZDISTD(JL,JK)=ZDISTD(JL,JK)+ZDZXDG                          
            ZCNTRB(JL,JK,KJP1)=ZDZXDG                                   
 2236     CONTINUE                                                      
 224    CONTINUE                                                        
 225  CONTINUE                                                          
C                                                                       
C*         2.2.4   DISTANT AND BELOW LAYERS                             
C                  ------------------------                             
C                                                                       
 2240 CONTINUE                                                          
C                                                                       
C*         2.2.5   FIRST LOWER LEVEL                                    
C                  -----------------                                    
C                                                                       
 2250 CONTINUE                                                          
                                                                        
      DO 228 JK=3,KFLEV+1                                               
        KM1= JK-1                                                       
        KJ=JK-2                                                         
        KN = KM1*NG1P1+1                                                
        KU1= KN-NG1P1                                                   
                                                                        
! changing - marc 30/3/10                                               
c        DO 2252 JL = 1 , NUAER*KDLON                                   
c          ZUU(JL,1) = ZABSCU(JL,1,KU1) - ZABSCU(JL,1,KN)               
c 2252   CONTINUE                                                       
        DO I=1,NUAER                                                    
           DO JL=1,KDLON                                                
              ZUU(JL,I) = ZABSCU(JL,I,KU1) - ZABSCU(JL,I,KN)            
           ENDDO                                                        
        ENDDO                                                           
                                                                        
        CALL LWTT0(KDLON,KRLST,KFLEV,ITASK,                             
     *             PGA(1,1,1,KJ),PGB(1,1,1,KJ),                         
     *             PPTY(1,1,KJ), PETY(1,1,KJ))                          
                                                                        
                                                                        
! changing - marc 30/3/10                                               
c        DO 2254 JL = 1 , NTRAER*KDLON                                  
c          ZTT1(JL,1)=ZTT(JL,1)                                         
c 2254   CONTINUE                                                       
        DO I=1,NTRAER                                                   
           DO JL=1,KDLON                                                
              ZTT1(JL,I)=ZTT(JL,I)                                      
           ENDDO                                                        
        ENDDO                                                           
C                                                                       
C                                                                       
C*         2.2.6   DOWN BELOW                                           
C                  ----------                                           
C                                                                       
 2260   CONTINUE                                                        
                                                                        
        DO 227 JLK=1,JK-2                                               
          JKL=KM1-JLK                                                   
          KU2=(JKL-1)*NG1P1+1                                           
                                                                        
! changing - marc 30/3/10                                               
c          DO 2262 JL = 1 , NUAER*KDLON                                 
c            ZUU(JL,1) = ZABSCU(JL,1,KU2) - ZABSCU(JL,1,KN)             
c 2262     CONTINUE                                                     
        DO I=1,NUAER                                                    
           DO JL=1,KDLON                                                
              ZUU(JL,I) = ZABSCU(JL,I,KU2) - ZABSCU(JL,I,KN)            
           ENDDO                                                        
        ENDDO                                                           
                                                                        
          CALL LWTT0(KDLON,KRLST,KFLEV,ITASK,                           
     *               PGA(1,1,1,JKL),PGB(1,1,1,JKL),                     
     *               PPTY(1,1,JKL), PETY(1,1,JKL))                      
                                                                        
! changing - marc 30/3/10                                               
c          DO 2264 JL = 1 , NTRAER*KDLON                                
c            ZTT2(JL,1)=(ZTT1(JL,1)+ZTT(JL,1))*0.5                      
c            ZTT1(JL,1)=ZTT(JL,1)                                       
c 2264     CONTINUE                                                     
          DO I=1,NTRAER                                                 
             DO JL=1,KDLON                                              
                ZTT2(JL,I)=(ZTT1(JL,I)+ZTT(JL,I))*0.5                   
                ZTT1(JL,I)=ZTT(JL,I)                                    
             ENDDO                                                      
          ENDDO                                                         
                                                                        
          DO 2266 JL = 1 , KDLON                                        
            ZWW=ZDBDT(JL,1,JKL)*ZTT2(JL,1)           *ZTT2(JL,10)       
     *        + ZDBDT(JL,2,JKL)*ZTT2(JL,2)*ZTT2(JL,7)*ZTT2(JL,11)       
     *        + ZDBDT(JL,3,JKL)*ZTT2(JL,4)*ZTT2(JL,8)*ZTT2(JL,12)       
     *        + ZDBDT(JL,4,JKL)*ZTT2(JL,5)*ZTT2(JL,9)*ZTT2(JL,13)       
     *        + ZDBDT(JL,5,JKL)*ZTT2(JL,3)           *ZTT2(JL,14)       
     *        + ZDBDT(JL,6,JKL)*ZTT2(JL,6)           *ZTT2(JL,15)       
            ZGLAYU(JL)=ZWW                                              
            ZDZXMG=ZGLAYU(JL)                                           
            ZDISTU(JL,JK)=ZDISTU(JL,JK)+ZDZXMG                          
            ZCNTRB(JL,JK,JKL)=ZDZXMG                                    
 2266     CONTINUE                                                      
 227    CONTINUE                                                        
 228  CONTINUE                                                          
C                                                                       
C*         2.3     EXCHANGE WITH TOP OF THE ATMOSPHERE                  
C                  -----------------------------------                  
C                                                                       
 230  CONTINUE                                                          
                                                                        
      DO 235 JK = 1 , KFLEV                                             
        KN=(JK-1)*NG1P1+1                                               
                                                                        
! changing - marc 30/3/10                                               
c        DO 231 JL = 1 , NUAER*KDLON                                    
c          ZUU(JL,1) = ZABSCU(JL,1,KN)                                  
c 231    CONTINUE                                                       
        DO I=1,NUAER                                                    
           DO JL=1,KDLON                                                
              ZUU(JL,I) = ZABSCU(JL,I,KN)                               
           ENDDO                                                        
        ENDDO                                                           
                                                                        
        CALL LWTT1(KDLON,KRLST,KFLEV,ITASK,PGATOP,PGBTOP,               
     *             PTYTOP,ETYTOP)                                       
                                                                        
        DO 234 JL = 1 , KDLON                                           
          ZCNTOP=ZBTOP(JL,1)*ZTT(JL,1)          *ZTT(JL,10)             
     *         + ZBTOP(JL,2)*ZTT(JL,2)*ZTT(JL,7)*ZTT(JL,11)             
     *         + ZBTOP(JL,3)*ZTT(JL,4)*ZTT(JL,8)*ZTT(JL,12)             
     *         + ZBTOP(JL,4)*ZTT(JL,5)*ZTT(JL,9)*ZTT(JL,13)             
     *         + ZBTOP(JL,5)*ZTT(JL,3)          *ZTT(JL,14)             
     *         + ZBTOP(JL,6)*ZTT(JL,6)          *ZTT(JL,15)             
          ZFD(JL,JK)=ZCNTOP-ZBINT(JL,JK)-ZDISTD(JL,JK)-ZADJD(JL,JK)     
          ZFDN(JL,JK)=ZFD(JL,JK)                                        
          PFLUC(JL,2,JK)=ZFD(JL,JK)                                     
 234    CONTINUE                                                        
 235  CONTINUE                                                          
                                                                        
      JK = KFLEV+1                                                      
                                                                        
      DO 236 JL = 1 , KDLON                                             
        ZCNTOP= ZBTOP(JL,1)                                             
     *        + ZBTOP(JL,2)                                             
     *        + ZBTOP(JL,3)                                             
     *        + ZBTOP(JL,4)                                             
     *        + ZBTOP(JL,5)                                             
     *        + ZBTOP(JL,6)                                             
        ZFD(JL,JK)=ZCNTOP-ZBINT(JL,JK)-ZDISTD(JL,JK)-ZADJD(JL,JK)       
        ZFDN(JL,JK)=ZFD(JL,JK)                                          
        PFLUC(JL,2,JK)=ZFD(JL,JK)                                       
 236  CONTINUE                                                          
C                                                                       
C                                                                       
C*         2.4     COOLING-TO-SPACE OF LAYERS ABOVE 0.0001 HPA          
C                  ---------------------------------------              
C                                                                       
 240  CONTINUE                                                          
C                                                                       
C                                                                       
C*         2.4.1   INITIALIZATION                                       
C                  --------------                                       
C                                                                       
 2410 CONTINUE                                                          
C                                                                       
      JLIM = KFLEV                                                      
C                                                                       
      DO 2412 JK = KFLEV,1,-1                                           
      IF(PPMB(1,JK).LT.0.0001) THEN                                     
         JLIM=JK                                                        
      ENDIF                                                             
 2412 CONTINUE                                                          
      KLIM=JLIM                                                         
C                                                                       
! changing - marc 30/3/10                                               
c      DO 2414 JL = 1 , NTRAER*KDLON                                    
c        ZTT1(JL,1)=1.0                                                 
c 2414 CONTINUE                                                         
      DO I=1,NTRAER                                                     
         DO JL=1,KDLON                                                  
            ZTT1(JL,I)=1.0                                              
         ENDDO                                                          
      ENDDO                                                             
C                                                                       
C*         2.4.2   LOOP OVER LAYERS ABOVE 0.0001 HPA                    
C                  -----------------------------                        
C                                                                       
 2420 CONTINUE                                                          
C                                                                       
      DO 2427 JSTRA = KFLEV,JLIM,-1                                     
      JSTRU=(JSTRA-1)*NG1P1+1                                           
C                                                                       
! changing - marc 30/3/10                                               
c      DO 2422 JL = 1 , NUAER*KDLON                                     
c        ZUU(JL,1) = ZABSCU(JL,1,JSTRU)                                 
c 2422 CONTINUE                                                         
        DO I=1,NUAER                                                    
           DO JL=1,KDLON                                                
              ZUU(JL,I) = ZABSCU(JL,I,JSTRU)                            
           ENDDO                                                        
        ENDDO                                                           
C                                                                       
        CALL LWTT1(KDLON,KRLST,KFLEV,ITASK,PGATOP,PGBTOP,               
     *             PTYTOP,ETYTOP)                                       
c                                                                       
      DO 2424 JL = 1, KDLON                                             
      ZCTSTR =                                                          
     1 (PB(JL,1,JSTRA)+PB(JL,1,JSTRA+1))                                
     1     *(ZTT1(JL,1)           *ZTT1(JL,10)                          
     1     - ZTT (JL,1)           *ZTT (JL,10))                         
     2+(PB(JL,2,JSTRA)+PB(JL,2,JSTRA+1))                                
     2     *(ZTT1(JL,2)*ZTT1(JL,7)*ZTT1(JL,11)                          
     2     - ZTT (JL,2)*ZTT (JL,7)*ZTT (JL,11))                         
     3+(PB(JL,3,JSTRA)+PB(JL,3,JSTRA+1))                                
     3     *(ZTT1(JL,4)*ZTT1(JL,8)*ZTT1(JL,12)                          
     3     - ZTT (JL,4)*ZTT (JL,8)*ZTT (JL,12))                         
     4+(PB(JL,4,JSTRA)+PB(JL,4,JSTRA+1))                                
     4     *(ZTT1(JL,5)*ZTT1(JL,9)*ZTT1(JL,13)                          
     4     - ZTT (JL,5)*ZTT (JL,9)*ZTT (JL,13))                         
     5+(PB(JL,5,JSTRA)+PB(JL,5,JSTRA+1))                                
     5     *(ZTT1(JL,3)           *ZTT1(JL,14)                          
     5     - ZTT (JL,3)           *ZTT (JL,14))                         
     6+(PB(JL,6,JSTRA)+PB(JL,6,JSTRA+1))                                
     6     *(ZTT1(JL,6)           *ZTT1(JL,15)                          
     6     - ZTT (JL,6)           *ZTT (JL,15))                         
      PCTS(JL,JSTRA)=ZCTSTR*0.5                                         
 2424 CONTINUE                                                          
! changing - marc 30/3/10                                               
c      DO 2426 JL = 1 , NTRAER*KDLON                                    
c        ZTT1(JL,1)=ZTT(JL,1)                                           
c 2426 CONTINUE                                                         
      DO I=1,NTRAER                                                     
         DO JL=1,KDLON                                                  
            ZTT1(JL,I)=ZTT(JL,I)                                        
         ENDDO                                                          
      ENDDO                                                             
                                                                        
 2427 CONTINUE                                                          
C                                                                       
C*         2.5     EXCHANGE WITH LOWER LIMIT                            
C                  -------------------------                            
C                                                                       
 250  CONTINUE                                                          
                                                                        
      DO 251 JL = 1 , KDLON                                             
        ZBGND(JL)=ZBSUIN(JL)*PEMIS(JL)-(1.-PEMIS(JL))                   
     *           *PFLUC(JL,2,1)-ZBINT(JL,1)                             
 251  CONTINUE                                                          
                                                                        
! JK wasn't set before, I'm guessing what is needed - marc 30/3/10      
      DO JK = 2 , KFLEV+1                                               
!                                                                       
      DO 252 JL = 1 , KDLON                                             
        ZCNSOL = ZBSUR(JL,1)                                            
     *          +ZBSUR(JL,2)                                            
     *          +ZBSUR(JL,3)                                            
     *          +ZBSUR(JL,4)                                            
     *          +ZBSUR(JL,5)                                            
     *          +ZBSUR(JL,6)                                            
        ZCNSOL=ZCNSOL*ZBGND(JL)/ZBSUIN(JL)                              
        ZFU(JL,JK)=ZCNSOL+ZBINT(JL,1)-ZDISTU(JL,1)-ZADJU(JL,1)          
        ZFUP(JL,1)=ZFU(JL,JK)                                           
        PFLUC(JL,1,1)=ZFU(JL,JK)                                        
 252  CONTINUE                                                          
      enddo                                                             
                                                                        
      DO 255 JK = 2 , KFLEV+1                                           
        KN=(JK-1)*NG1P1+1                                               
                                                                        
! changing -marc                                                        
c        DO 2521 JL = 1 , NUAER*KDLON                                   
c          ZUU(JL,1) = ZABSCU(JL,1,1) - ZABSCU(JL,1,KN)                 
c2521    CONTINUE                                                       
        DO I=1,NUAER                                                    
           DO JL=1,KDLON                                                
              ZUU(JL,I) = ZABSCU(JL,I,1) - ZABSCU(JL,I,KN)              
           ENDDO                                                        
        ENDDO                                                           
                                                                        
        CALL LWTT1(KDLON,KRLST,KFLEV,ITASK,PGASUR,PGBSUR,               
     *             PTYSUR,ETYSUR)                                       
                                                                        
        DO 254 JL = 1 , KDLON                                           
          ZCNSOL=ZBSUR(JL,1)*ZTT(JL,1)          *ZTT(JL,10)             
     *          +ZBSUR(JL,2)*ZTT(JL,2)*ZTT(JL,7)*ZTT(JL,11)             
     *          +ZBSUR(JL,3)*ZTT(JL,4)*ZTT(JL,8)*ZTT(JL,12)             
     *          +ZBSUR(JL,4)*ZTT(JL,5)*ZTT(JL,9)*ZTT(JL,13)             
     *          +ZBSUR(JL,5)*ZTT(JL,3)          *ZTT(JL,14)             
     *          +ZBSUR(JL,6)*ZTT(JL,6)          *ZTT(JL,15)             
          ZCNSOL  =ZCNSOL*ZBGND(JL)/ZBSUIN(JL)                          
          ZFU(JL,JK)=ZCNSOL+ZBINT(JL,JK)-ZDISTU(JL,JK)-ZADJU(JL,JK)     
          ZFUP(JL,JK)=ZFU(JL,JK)                                        
          PFLUC(JL,1,JK)=ZFU(JL,JK)                                     
          ZCNTRB(JL,JK,JK) = 0.0                                        
 254    CONTINUE                                                        
 255  CONTINUE                                                          
C                                                                       
C*         2.6     HEATING-FROM-GROUND OF UPPERMOST LAYER               
C                  --------------------------------------               
C                                                                       
 260  CONTINUE                                                          
C                                                                       
C*         2.7     CLEAR-SKY FLUXES                                     
C                  ----------------                                     
C                                                                       
 270  CONTINUE                                                          
C                                                                       
      DO 271 JL = 1 , KDLON                                             
      ZFN10(JL) = ZFUP(JL,JLIM) + ZFDN(JL,JLIM)                         
 271  CONTINUE                                                          
      DO 273 JK = JLIM+1,KFLEV+1                                        
      DO 272 JL = 1 , KDLON                                             
      ZFN10(JL) = ZFN10(JL) + PCTS(JL,JK-1)                             
      ZFUP(JL,JK) = ZFN10(JL)                                           
      ZFDN(JL,JK) = 0.                                                  
 272  CONTINUE                                                          
 273  CONTINUE                                                          
      DO 275 JK = 1 , KFLEV+1                                           
      DO 274 JL = 1 , KDLON                                             
      PFLUC(JL,1,JK) = ZFUP(JL,JK)                                      
      PFLUC(JL,2,JK) = ZFDN(JL,JK)                                      
 274  CONTINUE                                                          
 275  CONTINUE                                                          
c      IF (IMP.LT.4) THEN                                               
c        WRITE(NOUT,884) (PFLUX(1,1,JK),JK = 1 , KFLEV+1)               
c        WRITE(NOUT,884) (PFLUX(1,2,JK),JK = 1 , KFLEV+1)               
c      END IF                                                           
C                                                                       
C     --------------------------------------------------------------    
C                                                                       
C*         3.      EFFECTIVE DOWNWARD AND UPWARD CLEAR-SKY EMISSIVITIES 
C                  ---------------------------------------------------- 
C                                                                       
 300  CONTINUE                                                          
                                                                        
      DO 302 JKL = 1 , KFLEV                                            
        JK = KFLEV+1 - JKL                                              
        DO 301 JL = 1 , KDLON                                           
          ZDFNET = PFLUC(JL,1,JK+1) + PFLUC(JL,2,JK+1)                  
     *            -PFLUC(JL,1,JK  ) - PFLUC(JL,2,JK  )                  
          ZEMD(JL,JK)= (-PFLUC(JL,2,JK)+PFLUC(JL,2,JK+1))               
     *             /(PSIG*PTAVE(JL,JK)**4.+PFLUC(JL,2,JK+1))            
          ZEMU(JL,JKL)=(PFLUC(JL,1,JKL+1)-PFLUC(JL,1,JKL))              
     *             /(PSIG*PTAVE(JL,JKL)**4.-PFLUC(JL,1,JKL))            
 301    CONTINUE                                                        
 302  CONTINUE                                                          
C                                                                       
C     ----------------------------------------------------------        
C                                                                       
C*         4.      EFFECT OF CLOUDINESS ON LONGWAVE FLUXES              
C                  ---------------------------------------              
C                                                                       
      DO 4004 JK = 1 , KFLEV+1                                          
        DO 4003 JL = 1 , KDLON                                          
          PFLUX(JL,1,JK) = ZFUP(JL,JK)                                  
          PFLUX(JL,2,JK) = ZFDN(JL,JK)                                  
 4003   CONTINUE                                                        
 4004 CONTINUE                                                          
C                                                                       
c      IF (IMP.LT.4) THEN                                               
c        PRINT 886,IMAXC                                                
c      END IF                                                           
                                                                        
      IF (IMAXC.GT.0) THEN                                              
        IMXP1 = IMAXC + 1                                               
        IMXM1 = IMAXC - 1                                               
C                                                                       
C*         4.0     INITIALIZE TO CLEAR-SKY FLUXES                       
C                  ------------------------------                       
C                                                                       
 400    CONTINUE                                                        
c                                                                       
        DO 403 JK1=1,KFLEV+1                                            
          DO 402 JK2=1,KFLEV+1                                          
            DO 401 JL = 1 , KDLON                                       
              ZUPF(JL,JK2,JK1)=ZFUP(JL,JK1)                             
              ZDNF(JL,JK2,JK1)=ZFDN(JL,JK1)                             
 401        CONTINUE                                                    
 402      CONTINUE                                                      
 403    CONTINUE                                                        
C                                                                       
C*         4.1     FLUXES FOR ONE OVERCAST UNITY EMISSIVITY CLOUD       
C                  ----------------------------------------------       
C                                                                       
 410  CONTINUE                                                          
                                                                        
        DO 413 JKC = 1 , IMAXC                                          
          JCLOUD=JKC                                                    
          JKCP1=JCLOUD+1                                                
C                                                                       
C*         4.1.1   ABOVE THE CLOUD                                      
C                  ---------------                                      
C                                                                       
 4110     CONTINUE                                                      
                                                                        
          DO 4115 JK=JKCP1,KFLEV+1                                      
            JKM1=JK-1                                                   
            DO 4111 JL = 1 , KDLON                                      
              ZFU(JL,JK)=0.                                             
 4111       CONTINUE                                                    
            IF (JK .GT. JKCP1) THEN                                     
              DO 4113 JKJ=JKCP1,JKM1                                    
                DO 4112 JL = 1 , KDLON                                  
                  ZFU(JL,JK) = ZFU(JL,JK) + ZCNTRB(JL,JK,JKJ)           
 4112           CONTINUE                                                
 4113         CONTINUE                                                  
            ENDIF                                                       
                                                                        
            DO 4114 JL = 1 , KDLON                                      
              ZUPF(JL,JKCP1,JK)=ZBINT(JL,JK)-ZFU(JL,JK)                 
 4114       CONTINUE                                                    
 4115     CONTINUE                                                      
C                                                                       
C*         4.1.2   FLUXES FOR ONE OVERCAST UNITY EMISSIVITY CLOUD       
C                  ----------------------------------------------       
C                                                                       
 4120     CONTINUE                                                      
                                                                        
          DO 4125 JK=1,JCLOUD                                           
            JKP1=JK+1                                                   
            DO 4121 JL = 1 , KDLON                                      
              ZFD(JL,JK)=0.                                             
 4121       CONTINUE                                                    
                                                                        
            IF (JK .LT. JCLOUD) THEN                                    
              DO 4123 JKJ=JKP1,JCLOUD                                   
                DO 4122 JL = 1 , KDLON                                  
                  ZFD(JL,JK) = ZFD(JL,JK) + ZCNTRB(JL,JK,JKJ)           
 4122           CONTINUE                                                
 4123         CONTINUE                                                  
            ENDIF                                                       
            DO 4124 JL = 1 , KDLON                                      
              ZDNF(JL,JKCP1,JK)=-ZBINT(JL,JK)-ZFD(JL,JK)                
 4124       CONTINUE                                                    
 4125     CONTINUE                                                      
 413    CONTINUE                                                        
c        IF (IMP.LT.2) THEN                                             
c          WRITE(NOUT,884) ((ZUPF(1,JK1,JK),JK=1,KFLEV+1),JK1=1,IMXP1)  
c          WRITE(NOUT,884) ((ZDNF(1,JK1,JK),JK=1,KFLEV+1),JK1=1,IMXP1)  
c        END IF                                                         
C                                                                       
C*         4.2     FLUXES FOR PARTIAL/MULTIPLE LAYERED CLOUDINESS       
C                  ----------------------------------------------       
C                                                                       
 420    CONTINUE                                                        
C                                                                       
C                                                                       
C*         4.2.1   DOWNWARD FLUXES                                      
C                  ---------------                                      
C                                                                       
 4210   CONTINUE                                                        
                                                                        
        DO 4212 JK= IMXP1 , KFLEV+1                                     
          DO 4211 JL = 1 , KDLON                                        
            PFLUX(JL,2,JK)=ZDNF(JL,1,JK)                                
 4211     CONTINUE                                                      
 4212   CONTINUE                                                        
                                                                        
        IF(IMXM1.GE.1) THEN                                             
          DO 4217 JK=1,IMXM1                                            
            JKP1=JK+1                                                   
            DO 4213 JL = 1 , KDLON                                      
              ZCLOUD(JL)=1.                                             
              ZFD(JL,JK)=ZDNF(JL,JKP1,JK)*PCLDLW(JL,JK)                 
 4213       CONTINUE                                                    
            DO 4215 JKJ=JK,IMXM1                                        
              JKJ1=JKJ+1                                                
              JKJ2=JKJ+2                                                
              DO 4214 JL = 1 , KDLON                                    
                ZCLOUD(JL)=ZCLOUD(JL)*(1.-PCLDLW(JL,JKJ))               
                ZCCLD=ZCLOUD(JL)*PCLDLW(JL,JKJ1)                        
                ZFD(JL,JK)=ZFD(JL,JK)+ZDNF(JL,JKJ2,JK)*ZCCLD            
 4214         CONTINUE                                                  
 4215       CONTINUE                                                    
            JKJ=IMAXC                                                   
            DO 4216 JL = 1 , KDLON                                      
              ZCLOUD(JL)=ZCLOUD(JL)*(1.-PCLDLW(JL,JKJ))                 
              ZFD(JL,JK)=ZFD(JL,JK)+ZDNF(JL,1,JK)*ZCLOUD(JL)            
              PFLUX(JL,2,JK)=ZFD(JL,JK)                                 
 4216       CONTINUE                                                    
 4217     CONTINUE                                                      
        ENDIF                                                           
C                                                                       
C*         4.2.2   UPWARD FLUX AT THE SURFACE                           
C                  --------------------------                           
C                                                                       
 4220   CONTINUE                                                        
                                                                        
        JK=IMAXC                                                        
        JKP1=JK+1                                                       
        DO 4221 JL = 1 , KDLON                                          
          ZFD(JL,JK)=ZDNF(JL,JKP1,JK)*PCLDLW(JL,JK)                     
     *           +ZDNF(JL,1,JK)*(1.-PCLDLW(JL,JK))                      
          PFLUX(JL,2,JK)=ZFD(JL,JK)                                     
          PFLUX(JL,1,1)=PEMIS(JL)*ZBSUIN(JL)-(1.-PEMIS(JL))             
     *                 *PFLUX(JL,2,1)                                   
 4221   CONTINUE                                                        
C                                                                       
C*         4.2.3   UPWARD FLUXES                                        
C                  -------------                                        
C                                                                       
 4230   CONTINUE                                                        
                                                                        
        DO 4235 JK = 2 , KFLEV+1                                        
          JK1=MIN(JK,IMXP1)                                             
          JK2=JK1-1                                                     
          JK2M1=JK2-1                                                   
          DO 4231 JL = 1 , KDLON                                        
            ZCLOUD(JL)=1.                                               
            ZFU(JL,JK)=ZUPF(JL,JK1,JK)*PCLDLW(JL,JK2)                   
 4231     CONTINUE                                                      
                                                                        
          IF(JK2M1.GE.1) THEN                                           
            DO 4233 JKJ=1,JK2M1                                         
              JKIJ=JK1-JKJ                                              
              DO 4232 JL = 1 , KDLON                                    
                ZCLOUD(JL)=ZCLOUD(JL)*(1.-PCLDLW(JL,JKIJ))              
                ZCCLD=ZCLOUD(JL)*PCLDLW(JL,JKIJ-1)                      
                ZFU(JL,JK)=ZFU(JL,JK)+ZUPF(JL,JKIJ,JK)*ZCCLD            
 4232         CONTINUE                                                  
 4233       CONTINUE                                                    
          ENDIF                                                         
                                                                        
          JKJ=JK2                                                       
          JKIJ=1                                                        
          DO 4234 JL = 1 , KDLON                                        
            ZCLOUD(JL)=ZCLOUD(JL)*(1.-PCLDLW(JL,JKIJ))                  
            ZFU(JL,JK)=ZFU(JL,JK)+ZUPF(JL,1,JK)*ZCLOUD(JL)              
            PFLUX(JL,1,JK)=ZFU(JL,JK)                                   
 4234     CONTINUE                                                      
 4235   CONTINUE                                                        
      ENDIF                                                             
C                                                                       
C*         4.3     END OF CLOUD EFFECT COMPUTATIONS                     
C                                                                       
 430  CONTINUE                                                          
                                                                        
C                                                                       
      DO 431 JL = 1 , KDLON                                             
        ZFN10(JL) = PFLUX(JL,1,KLIM) + PFLUX(JL,2,KLIM)                 
 431  CONTINUE                                                          
      DO 433 JK = KLIM+1 , KFLEV+1                                      
        DO 432 JL = 1 , KDLON                                           
          ZFN10(JL) = ZFN10(JL) + PCTS(JL,JK-1)                         
          PFLUX(JL,1,JK) = ZFN10(JL)                                    
          PFLUX(JL,2,JK) = 0.0                                          
 432    CONTINUE                                                        
 433  CONTINUE                                                          
C                                                                       
c      DO 431 JL = 1 , KDLON                                            
c        PFLUX(JL,1,KFLEV+1) = PFLUX(JL,1,KFLEV)                        
c 431  CONTINUE                                                         
                                                                        
c      IF (IMP.LT.4) THEN                                               
c        WRITE(NOUT,884) (PFLUX(1,1,JK),JK = 1 , KFLEV+1)               
c        WRITE(NOUT,884) (PFLUX(1,2,JK),JK = 1 , KFLEV+1)               
c      END IF                                                           
C                                                                       
      DO 501 JL=1,NRLST                                                 
         DO 502 JK=1,NRLEV                                              
C            PCLFR (JL,JK) = PCLD  (JL,JK)                              
C            PQLWP (JL,JK) = PQLW  (JL,JK)                              
            ZCLDLW(JL,JK) = PCLDLW(JL,JK)                               
            DO 503 II=1,2                                               
               ZFLUX (JL,II,JK) = PFLUX (JL,II,JK)                      
               ZTAU  (JL,II,JK) = PTAU  (JL,II,JK)                      
 503        CONTINUE                                                    
            ZPMB  (JL,JK) = PPMB  (JL,JK)                               
            ZTAVE (JL,JK) = PTAVE (JL,JK)                               
            ZTL   (JL,JK) = PTL   (JL,JK)                               
 502     CONTINUE                                                       
         DO 504 II=1,2                                                  
            ZFLUX (JL,II,NRLEV+1) = PFLUX (JL,II,NRLEV+1)               
 504      CONTINUE                                                      
         ZDT0(JL) = PDT0(JL)                                            
         ZPMB(JL,NRLEV+1) = PPMB(JL,NRLEV+1)                            
         ZTL (JL,NRLEV+1) = PTL (JL,NRLEV+1)                            
 501  CONTINUE                                                          
C                                                                       
      RETURN                                                            
C                                                                       
C-----------------------------------------------------------------      
C                       FORMATS                                         
C                       -------                                         
C                                                                       
 884  FORMAT (1X,F7.2,18F6.1,F7.2)                                      
 886  FORMAT (1X,20I5)                                                  
 888  FORMAT (1X,13E9.2)                                                
 890  FORMAT (1X,2I4,13F9.2)                                            
      END                                                               
C__________________________________________________________________     
C                                                                       
C ii) Replace the old LWTT with the new improved version.               
C                                                                       
      SUBROUTINE LWTT0(KDLON,KRLST,KFLEV,ITASK,PGGA,PGGB,PPPTY,PPETY)   
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRSW* SWITCHES RELATED TO THE RADIATION SCHEME.         
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      IMPLICIT REAL (A-H,O-Z)                                           
C      IMPLICIT UNDEFINED (Z)                                           
C      IMPLICIT LOGICAL (L)                                             
C      IMPLICIT UNDEFINED (L)                                           
      IMPLICIT INTEGER (I-K,M-N)                                        
C      IMPLICIT UNDEFINED (I-K,M-N)                                     
C      IMPLICIT REAL (P)                                                
C      IMPLICIT UNDEFINED (A-H,O,Q-Y)                                   
C                                                                       
C everything except the parameters is defined                           
C in the common blocks                                                  
C                                                                       
      COMMON/COM1/                                                      
C     I NRADFR, NRPART, NRINT, NRADPFR, NRADPLA,                        
C     I NRLSTI,                                                         
     R XLAT,                                                            
     R RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR, ZCCO2,              
C     R PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3,                 
     L LOH, LOM, LOL, LORAD                                             
      COMMON/COM2/                                                      
     I ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,                  
     I         JLOOP, KRLEV,                                            
     I        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     I        ITH, ITM, ITL,                                            
     I        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     I        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     I        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     I        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      COMMON/COM3/                                                      
     I IND4, KM12, KXD, KXU, IG, KBS, KDD,                              
     I        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     I        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     I        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     I        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     I        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
      COMMON/COM4/                                                      
     R           COZQC,COZQS,COZHC,COZHS,                               
     R CSUPSAT,                                                         
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
      COMMON/COM5/                                                      
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NUMVC,                                                           
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      COMMON/COM6/                                                      
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD,       
     R DTIME,    TWODT,    EPS,                                         
     R SINLON,     COSLON,                                              
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2,                          
     I NFAX,                                                            
     R CRAE,                                                            
     R CDISSEM                                                          
      COMMON/COM7/                                                      
     R     CCRH,                                                        
     L LO,LO1,LO2,LODIA,LODIAP,                                         
     R TAUA, PIZA, CGA, CAER, CAEROS                                    
      COMMON/COM7A/                                                     
     R  RT1,WG1,AT,BT,CNTNU,O1H,O2H,PIALF0                              
     S ,OCT,TREF,XP,TSTAND,GA,GB,CE,ETY,PTY                             
      COMMON/COM7B/                                                     
     S TINTP,TSTP,                                                      
     S  TBO3,AO3,BO3,TB,ALP,BET,ULG,ULGO,PLG,AP,BP,TO3B,O3A,O3B,        
     R          CDAY,CCO2,CH2O,DIFF,X10E                                
       COMMON/COM7C/                                                    
     S  ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW,                       
     R  APAD,BPAD,D,CRAY,SUN,                                           
     R   TTT,                                                           
     R   TM1                                                            
      COMMON/COM8/                                                      
     R       TSM1M,                                                     
     R       QM1,                                                       
     R       SNM1M,                                                     
     R       ALBM,                                                      
     R       AVVELM,                                                    
     R   SLMM,                                                          
     R       APHM1,                                                     
     R       APM1                                                       
      COMMON/COM9/                                                      
     L LOLAND,                                                          
     R   ZTI,                                                           
     *       ZCTI,                                                      
     *       ZCPH,                                                      
     *       ZWV,                                                       
     *       ZSW,                                                       
     *       ZALTE,                                                     
     *       ZCALTE,                                                    
     *       ZALSO                                                      
      COMMON/COM10/                                                     
     *       ZCALSO,                                                    
     R   ZOZQ,                                                          
     *       ZOZH,                                                      
     R   ZTRSOL,                                                        
     *       ZEMTER,                                                    
     R   TRSOLM,                                                        
     *       EMTERM,                                                    
     R   AMU0,                                                          
     *       RDAYL                                                      
      COMMON/COM11/                                                     
     R   CLC,                                                           
     *       TI,                                                        
     *       DP,                                                        
     *       AADS,                                                      
     *       AEQ1,                                                      
     *       AEQ2,                                                      
     *       AEQ3,                                                      
     *       AEQ4,                                                      
     *       AEQ5,                                                      
     *       SSW                                                        
      COMMON/COM12/                                                     
     *       WV,                                                        
     *       QOF,                                                       
     *       QCF,                                                       
     *       CLWA,                                                      
     *       APRE,                                                      
     *       SMU0,                                                      
     *       ALTE,                                                      
     *       ALSO                                                       
      COMMON/COM13/                                                     
     R   ZDPO,                                                          
     *       ZQCFO,                                                     
     *       ZAEQSO,                                                    
     *       ZQOFO,                                                     
     *       ZDPN,                                                      
     *       ZQCFN,                                                     
     *       ZAEQSN,                                                    
     *       ZQOFN                                                      
      COMMON/COM14/                                                     
     R   FLT,                                                           
     *       FLS,                                                       
     R   DIAG,                                                          
     *       DIA1,                                                      
     *       ZDIAG,                                                     
     *       ZDIA1,                                                     
     *       ZDIA,                                                      
     *       ZDIAF,                                                     
     *       ZDIAT,                                                     
     R PGG                                                              
      COMMON/COM15/                                                     
     #     PREF,                                                        
     #     PRE1,                                                        
     #     PRE2,                                                        
     #     PRMUZ,                                                       
     #     PTO1,                                                        
     #     PTR1,                                                        
     #     PTR2,                                                        
     #     PW,                                                          
     R     PALBSN                                                       
      COMMON/COM16/                                                     
     R ZABSCU,                                                          
     S     ZABSLY,                                                      
     S     ZADJD ,                                                      
     S     ZADJU ,                                                      
     S     ZBGND ,                                                      
     S     ZBINT ,                                                      
     S     ZBSUR ,                                                      
     S     ZBSUIN,                                                      
     S     ZBTOP ,                                                      
     S     PCLDLW,                                                      
     S     ZCLOUD,                                                      
     S     ZCNTRB,                                                      
     S     ZCUMP ,                                                      
     S     ZCUMPT,                                                      
     S     ZDBDT                                                        
      COMMON/COM17/                                                     
     S     ZDBSL ,                                                      
     S     ZDISTD,                                                      
     S     ZDISTU,                                                      
     S     ZDNF  ,                                                      
     S     PDT0  ,                                                      
     S     ZDUC                                                         
      COMMON/COM18/                                                     
     S     ZEFFT ,                                                      
     S     ZEMU  ,                                                      
     S     ZEPHIO,                                                      
     S     ZEPSC2,                                                      
     S     ZEPSC3,                                                      
     S     ZEPSH1,                                                      
     S     ZEPSH2,                                                      
     S     ZEPSH3                                                       
      COMMON/COM19/                                                     
     S     ZEPSH4,                                                      
     S     ZEPSH5,                                                      
     S     ZEPSH6,                                                      
     S     ZEPSIO,                                                      
     S     ZFACTC,                                                      
     S     ZZFD  ,                                                      
     S     ZFDD  ,                                                      
     S     ZFDN                                                         
      COMMON/COM20/                                                     
     S     PFLUX , PFLUC,                                               
     S     ZFLWP , PCTS                                                 
     S     ZZFU  ,                                                      
     S     ZFUP  ,                                                      
     S     ZGLAYD,                                                      
     S     ZGLAYU,                                                      
     S     ZOMEGA,                                                      
     S     ITX   ,                                                      
     S     IXDIA ,                                                      
     S     IXT   ,                                                      
     S     IXTSU ,                                                      
     S     IXTTP ,                                                      
     S     IMX   ,                                                      
     S     IMXP                                                         
      COMMON/COM21/                                                     
     R ZOZ   ,                                                          
     S     PPMB  ,                                                      
     S     ZRES  , PB                                                   
     S     ZSSIG ,                                                      
     S     PTAU  ,                                                      
     S     PTAVE ,                                                      
     S     ZTAVIC,                                                      
     S     ZTI1  ,                                                      
     S     PTL                                                          
      COMMON/COM22/                                                     
     S     ZTT   ,                                                      
     S     ZTT1  ,                                                      
     S     ZTT2  ,                                                      
     S     ZUAER ,                                                      
     S     ZUPF  ,                                                      
     S     ZUU   ,                                                      
     S     ZXD                                                          
      COMMON/COM23/                                                     
     S     ZXN   ,                                                      
     S     ZXOZ  ,                                                      
     S     ZXWV  ,                                                      
     S     ZZZ   ,                                                      
     S     ZRES2 ,                                                      
     S     ZTI2  ,                                                      
     S     ZBLAY ,                                                      
     S     ZBLEV ,                                                      
     S     ZCTS  ,                                                      
     S     ZHFG                                                         
      COMMON/COM24/                                                     
     I KT,                                                              
     R    PUU   ,                                                       
     S        PTT   ,                                                   
     S        ZZ                                                        
     R  PCLFR  ,                                                        
     S      PT     ,                                                    
     S      PQOF   ,                                                    
     S      PQLWP  ,                                                    
     S      PEMIS  ,                                                    
     S      PFLT                                                        
      COMMON/COM25/                                                     
     R  ZALBSU,                                                         
     S      ZCG   ,                                                     
     S      ZCLDLW,                                                     
     S      ZCLDSW,                                                     
     S      ZCOOLR,                                                     
     S      ZDT0  ,                                                     
     S      ZEMD  ,                                                     
     S      ZFLUX ,                                                     
     S      ZPMB  ,                                                     
     S      ZTAU  ,                                                     
     S      ZTAVE ,                                                     
     S      ZTL   ,                                                     
     S      ZFSDWN                                                      
      COMMON/COM26/                                                     
     S      ZFSUP ,                                                     
     S      ZHEATR,                                                     
     R   PDP   ,                                                        
     S       PWV   ,                                                    
     S       PFLS  ,                                                    
     S       PPSOL ,                                                    
     S       PRMU0 ,                                                    
     S       PAER  ,                                                    
     S       PALBS ,                                                    
     S       PCG   ,                                                    
     S       PCLDSW,                                                    
     S       POMEGA,                                                    
     S       POZ                                                        
      COMMON/COM27/                                                     
     R   PFDOWN,                                                        
     S       PFUP  ,                                                    
     R   ZAKI  ,                                                        
     S       ZCGAZ ,                                                    
     S       ZCNEB ,                                                    
     S       ZC1I  ,                                                    
     S       ZDSC  ,                                                    
     S       ZDSH                                                       
      COMMON/COM28/                                                     
     S       ZDSIG ,                                                    
     S       ZFACT ,                                                    
     S       ZFACT1 ,                                                   
     S       ZFACT2 ,                                                   
     S       ZFACT3 ,                                                   
     S       ZFACT4 ,                                                   
     S       ZFACT5 ,                                                   
     S       ZFACT6 ,                                                   
     S       ZFD   ,                                                    
     S       ZFRAC ,                                                    
     S       ZFU   ,                                                    
     S       ZG    ,                                                    
     S       ZGG   ,                                                    
     S       ZPIZAZ                                                     
      COMMON/COM29/                                                     
     S       ZRAYL ,                                                    
     S       ZRAY1 ,                                                    
     S       ZRAY2 ,                                                    
     S       ZREF  ,                                                    
     S       ZREFZ ,                                                    
     S       ZRE1  ,                                                    
     S       ZRE2  ,                                                    
     S       ZRJ   ,                                                    
     S       ZRK   ,                                                    
     S       ZRL   ,                                                    
     S       ZRMU  ,                                                    
     S       ZRMUE ,                                                    
     S       ZRMUZ                                                      
      COMMON/COM30/                                                     
     S       ZRNEB ,                                                    
     S       ZRUEF ,                                                    
     S       ZR1   ,                                                    
     S       ZR2   ,                                                    
     S       ZR21  ,                                                    
     S       ZR22  ,                                                    
     S       ZR23  ,                                                    
     S       ZS    ,                                                    
     S       ZSEC  ,                                                    
     S       ZSS1  ,                                                    
     S       ZTAUAZ                                                     
      COMMON/COM31/                                                     
     S       ZTOT  ,                                                    
     S       ZTO1  ,                                                    
     S       ZTR   ,                                                    
     S       ZTRA1 ,                                                    
     S       ZTRA2 ,                                                    
     S       ZTR1                                                       
      COMMON/COM32/                                                     
     S       ZTR2  ,                                                    
     S       ZUD   ,                                                    
     S       ZUM   ,                                                    
     S       ZU1D  ,                                                    
     S       ZU2D  ,                                                    
     S       ZW    ,                                                    
     S       ZN175 ,                                                    
     S       ZN190 ,                                                    
     S       ZO175 ,                                                    
     S       ZO190 ,                                                    
     S       ZP75  ,                                                    
     S       ZP90  ,                                                    
     S       ZSIGN ,                                                    
     S       ZSIGO                                                      
      COMMON/COM33/                                                     
     R   PTR,                                                           
     S       PU,                                                        
     R   ZRH,                                                           
     *       ZCH,                                                       
     *       ZCM,                                                       
     *       ZCL,                                                       
     *       ZCT,                                                       
     *       ZCC,                                                       
     I NTOPCM                                                           
      COMMON/COM34/                                                     
     *        NBASECM,                                                  
     R   ARPRCM,                                                        
     I ITOPH,                                                           
     *        ITOPM,                                                    
     *        ITOPL,                                                    
     L LOC,                                                             
     R   ZQS,                                                           
     *       ZCLC,                                                      
     *       ZQLWP                                                      
      COMMON/COM35/                                                     
     R   VERVEL,                                                        
     *       ZTHETA,                                                    
     *       ZETA,                                                      
     *       ZDTHMIN,                                                   
     I INVB,                                                            
     R   ZCTLWP,                                                        
     *       ZQLOC,                                                     
     *       ZQMEC,                                                     
     *       ZQHIC,                                                     
     R   ZFSOT,                                                         
     #       ZFTET,                                                     
     #       ZFLT                                                       
      COMMON/COM36/                                                     
     #       ZFSO ,                                                     
     #       ZFTE ,                                                     
     #       ZFLB ,                                                     
     #       ZDTDT,                                                     
     #       HEATO,                                                     
     #       HEATE,                                                     
     #       HEATR,                                                     
     #       ZI0                                                        
C                                                                       
      INTEGER                                                           
C     I NRADFR,   NRPART,   NRINT,    NRLSTI,  NRADPFR,  NRADPLA,       
     I NRLST,    NRLEV,    NRLEVP1, NLEV, NLP2, NLEVP1,                 
     I NLON, IRLEV, NTRA, NLAT, ITIM, NUA, NINT, NRPARTI                
C                                                                       
C   NRLEV .... number of vertical levels                                
C   NRLST .... number of points on a latitude line                      
C   NUA   .... number of absorber amount                                
C   NINT  .... number of spectral intervals                             
C   NTRA  .... number of transmission functions                         
C   NLAT  .... number of latitude points                                
C   ITIM  .... number of month under consideration                      
C                                                                       
C      PARAMETER (NRLEV=122,NUA=34,NINT=6,NTRA=34, NLAT=1,              
      PARAMETER (NRLEV=21,NUA=34,NINT=6,NTRA=34, NLAT=1,                
cccm      PARAMETER (NRLEV=37,NUA=18,NINT=6,NTRA=19, NLAT=1,            
cccm     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=10,NLEV=NRLEV,        
     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=7,NLEV=NRLEV,             
     #           NLP2=NRLST, NLEVP1=NRLEVP1, NLON=NRLST,                
     #           NRPARTI=1)                                             
C                                                                       
      REAL YCLOCK, XLAT(NLAT), Z0, DZ, ALBMPAR, SNPAR,                  
     #     TSUPAR, CTFREEZ, DELTAC, DELTAO, DELTAH                      
C                                                                       
C   YTIME  .... time of the year in radians                             
C   YCLOCK .... time of the day in radians                              
C   XLAT   .... latitudes in radians                                    
C   Z0     .... zero point of height grid                               
C   DZ     .... Interval between height levels                          
C   ALBMPAR.... sw surface albedo as function of longitude:             
C           yearly averaged sea-ice: 0.55                               
C                           water  : 0.07                               
C                           land   : 0.07.lt.ALBM.lt.0.80               
C                                    depending on snow                  
C   SNPAR  .... snow depth in meter                                     
C   TSUPAR .... surface temperature (Kelvin)                            
C   CTFREEZ.... freezing temperature of sea-water (Kelvin)              
C   DELTAC .... Doppler parameter for line shapes of CO2                
C   DELTAO .... Doppler parameter for line shapes of O3                 
C                                                                       
      PARAMETER (Z0=0., CTFREEZ=271.26,                                 
C     PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
     #           DZ=0.289, ALBMPAR=0.2, SNPAR=0.,                       
cccm  PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
cccm     #           DZ=0.15, ALBMPAR=0.3, SNPAR=0.,                    
CZ     #           DELTAC=0., DELTAO=0.)                                
     #           DELTAC=6.0E-04, DELTAO=1.E-02, DELTAH=5.0E-04)         
C      PARAMETER (YCLOCK=0., Z0=0., TSUPAR=273., CTFREEZ=271.26,        
C     #           DZ=0.46875, ALBMPAR=0.07, SNPAR=0.)                   
C      PARAMETER (YCLOCK=0., Z0=2.608, TSUPAR=273., CTFREEZ=271.26,     
C     #           DZ=0.2878, ALBMPAR=0.07, SNPAR=0.)                    
C                                                                       
      LOGICAL LDIUR, LOLPAR                                             
C                                                                       
C   LDIUR .... true: sun at time, false: diurnally averaged             
C   LOLPAR.... true: land surface, false: else                          
C                                                                       
C      PARAMETER (LDIUR=.TRUE., LOLPAR=.TRUE.)                          
       PARAMETER (LOLPAR=.TRUE.)                                        
C                                                                       
      INTEGER  C2TEMP                                                   
      CHARACTER*64  MIDTMP                                              
cccm      PARAMETER (C2TEMP=10,                                         
cccm     #    MIDTMP='/home/atm/ugamp/midrad/radfildat.dir/MIDTMP.DAT') 
ccc1      PARAMETER (C2TEMP=10, MIDTMP='tprfil2.dat')                   
C                                                                       
      INTEGER  IRLONI, IRLST, IRLVP1, JPNLEV, JPNLVP1                   
C                                                                       
      PARAMETER (IRLONI=NRLST, IRLST=NRLST,                             
     #           IRLVP1=NRLEVP1, JPNLEV=NRLEV,                          
     #           JPNLVP1=NRLEVP1, IRLEV=NRLEV)                          
C                                                                       
      INTEGER NIPD, NIPD2, NTR, NG1, NG1P1                              
      PARAMETER(NIPD=8,NIPD2=16,NTR=11,NG1=2,NG1P1=3)                   
C                                                                       
C    definition of the functions                                        
C                                                                       
      REAL       CVMGT, SIGMA2                                          
      INTEGER    CVMGI                                                  
C                                                                       
C    definitons of the subroutine cloud                                 
C                                                                       
      REAL RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR                  
C                                                                       
C    missing definitions in morcrette                                   
C                                                                       
C      REAL   PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3            
      REAL ZCCO2                                                        
                                                                        
      LOGICAL LOH, LOM, LOL, LORAD                                      
C                                                                       
      INTEGER ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,           
     #        IZQS, IZCLC, IZQLWP, JLOOP, KRLEV,                        
     #        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     #        ITH, ITM, ITL,                                            
     #        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     #        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     #        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     #        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      INTEGER IND4, KM12, KXD, KXU, IG, KBS, KDD,                       
     #        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     #        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     #        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     #        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     #        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD1* SPECTRAL DISTRIBUTION OF AEROSOLS AND OZONE.     
C                     (TRIANGULAR *T10* TRUNCATIONFOR AEROSOLS).        
C                     (TRIANGULAR *T5* TRUNCATIONFOR OZONE).            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R           COZQC(21),COZQS(15),COZHC(21),COZHS(15)                
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD2* CONSTANTS INTERNAL TO THE RADIATION SUBROUTINES. 
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CSUPSAT                                                          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMCON* BASIC UNIVERSAL CONSTANTS AND DERIVED CONSTANTS.  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
C                                                                       
C                                                                       
C       --------------------------------------------------------------- 
C      PARAMETER(                                                       
C     I JPM=106,JPN=106,JPK=106,JPGL=160,JPNLON=320,                    
C     I JPT1MAX=400,          JPT2MAX=50,                               
C     I JPT3MAX=300,          JPT4MAX=50,                               
C     I JPHASH=2048,          JPNTRCRS=20,                              
C     I JPTASKS=4,     JPLOCKS=50,    JPEVENT=4,                        
C     I JPLTMT=JPTASKS,                                                 
C     I JPNUMW=352,            JPNUMP=352,                              
C     I JPTABLE=1024,                                                   
C     I JPMAXB =60,                                                     
C     I JPMAXPB=32,                                                     
C     I JPG3FLD=43,         JPG3XF=99,         JPG3CMP=17,              
C     I JPMP1=JPM+1,        JPNP1=JPN+1,       JPKP1=JPK+1,             
C     I JP2MP1=JPMP1*2,     JP4MP1=JPMP1*4,                             
C     I JPED=JPN/2+1,       JPOD=JPNP1/2,                               
C     I IPSP0=JPM+JPN-JPK,  IPSP1=JPNP1*JPMP1, IPSP2=IPSP0*(IPSP0+1)/2, 
C     I JPSP=IPSP1-IPSP2,   JP2SP=2*JPSP,                               
C     I JPHGL=JPGL/2,       JPNLP2=JPNLON+2,                            
C     I                     JPNVCLV=4*JPNLVP1)                          
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C     J. K. GIBSON     E.C.M.W.F.     22/01/82.                         
C     MODIFIED M.J     E.C.M.W.F.     13/12/82.                         
C                                                                       
      INTEGER                                                           
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      LOGICAL                                                           
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD        
      REAL                                                              
     R DTIME,    TWODT,    EPS                                          
C                                                                       
C      INTEGER                                                          
C     I NWTIME(JPNUMW),     NROW(JPTASKS),                              
C     I NPTIME(JPNUMP)                                                  
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMGAU* - QUANTITIES RELATED TO THE GAUSSIAN GRID.      
C                                                                       
      REAL                                                              
     R SINLON(NLP2),     COSLON(NLP2)                                   
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMTRU* -QUANTITIES RELATED TO THE SPECTRAL TRUNCATION. 
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSIM* - QUANTITIES NEEDED FOR THE SEMI-IMPLICIT SCHEME
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMHDI* - COEFFICIENTS FOR HORIZONTAL DIFFUSION.        
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMFFT* - QUANTITIES NEEDED FOR THE FAST *FOURIER       
C                         TRANSFORMS.                                   
C                                                                       
      INTEGER                                                           
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2                           
C      REAL                                                             
C     R TRIG(JPNLON)                                                    
      INTEGER                                                           
     I NFAX(10)                                                         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSTA* - ARRAYS AND PARAMETERS FOR STATISTICS.         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMDIA* - QUANTITIES RELATED TO GLOBAL PHYSICAL         
C                         DIAGNOSTICS.                                  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPSW* SWITCHES RELATED TO THE PARAMETERISATIONS OF      
C                     DIABATIC PROCESSES.                               
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMDSW* SWITCHES RELATED TO THE DYNAMICS                  
C                     AND THE GENERAL CONTROL OF THE FORECAST.          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH1* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE RADIATION SUBROUTINES.                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CRAE,                                                            
     R CDISSEM                                                          
C                                                                       
C      REAL       COSRAD(128),        SINRAD(128)                       
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH2* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE PHYSICAL SUBROUTINES (EXCEPT RADIATION    
C                     ONES).                                            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL       CCRH(JPNLEV)                                           
C                                                                       
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMDIZ* VARIABLES AND POINTERS FOR ZONAL MEAN DIAGNOSTICS 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMMSK* VARIABLES AND POINTERS FOR AREA MEAN DIAGNOSTICS. 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMSDS* - START/RESTART BLOCK.                          
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMTASK* - TASK RELATED VARIABLES.                      
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*         *COMRAD* - RADIATION POINTERS.                               
C                                                                       
      LOGICAL LO,LO1,LO2,LODIA,LODIAP                                   
C---------------------------------------------------------------------  
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C     COMMON *YOMAER* RADIATIVE CHARACTERISTICS OF THE AEROSOLS         
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
      REAL TAUA(2,5), PIZA(2,5), CGA(2,5), CAER(5,5), CAEROS            
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C     COMMON *YOMLW*                                                    
C                                                                       
C*    COEFFICIENTS FOR THE LONGWAVE RADIATION SUBROUTINES               
C                                                                       
C NINT  :  NUMBER OF SPECTRAL INTERVALS                                 
C NIPD  :  NUMBER OF BANDS DESCRIBED BY PADE APPROXIMANTS               
C NTR   :  NUMBER OF REFERENCE TEMPERATURES                             
C NTRA  :  NUMBER OF TRANSMISSION FUNCTIONS                             
C NUA   :  NUMBER OF ABSORBER AMOUNTS                                   
C NG1   :  ORDER OF THE GAUSSIAN QUADRATURE                             
C                                                                       
C RT1   :  ROOTS OF THE GAUSSIAN POLYNOMIAL                             
C WG1   :  WEIGHTS OF THE GAUSSIAN POLYNOMIAL                           
C AT, BT:  COEFFICIENTS OF THE TEMPERATURE DEPENDENCE                   
C CNTNU :  H2O CONTINUUM ABSORPTION COEFFICIENTS                        
C O1H,O2H,PIALF0,OCT: OZONE LONGWAVE ABSORPTION COEFFICIENTS            
C TREF, TSTAND      : REFERENCE TEMPERATURES                            
C XP    :  COEFFICIENTS OF POLYNOMIAL DEVELOPMENTS OF PLANCK FUNCTION   
C GA,GB :  PADE APPROXIMANTS OF H2O, CO2 TRANSMISSION FUNCTIONS         
C ETY,PTY :  PARAMETERS OF H2O CONTINUUM                                
C TINTP :  REFERENCE TEMPERATURES OF PRE-COMPUTED TRANSMISSIVITIES      
C CE    :  EMS COEFFS OF H2O TRANSMISSION FUNCTIONS                     
C                                                                       
      REAL  RT1(2),WG1(2),AT(12,3),BT(12,3),CNTNU(4),O1H                
     S ,OCT(4),TREF,XP(6,6),TSTAND,GA(11,24,3),GB(11,24,3)              
     S ,PTY(11,20),ETY(11,20),TINTP(11),TSTP,O2H,PIALF0                 
C                                                                       
      REAL  TB(40,31,2),ALP(40,31,2),BET(40,31,2),                      
     S ULG(40),PLG(31),AP(40,31,2),BP(40,31,2),TBO3(38,31),TO3B(38,31)  
     S,ULGO(38),AO3(38,31),BO3(38,31),O3A(38,31),O3B(38,31)             
C     ----------------------------------------------------------------- 
C*COMDECK YOMRDU                                                        
C     ----------------------------------------------------------------- 
C                                                                       
C*    CONTROL VARIABLES FOR RADIATION: INTERNAL SWITCH AND INDICES      
C                                                                       
C ZEELOG:  SECURITY THRESHOLD FOR ABSORBER AMOUNT IN LAPLACE TRANSFORM  
C ZEPSC :  SECURITY THRESHOLD FOR CLOUD COVER                           
C ZEPSCO:  SECURITY THRESHOLD FOR OZONE AMOUNT                          
C ZEPSCQ:  SECURITY THRESHOLD FOR WATER VAPOR                           
C ZEPSCT:  SECURITY THRESHOLD FOR SHORTWAVE OPTICAL THICKNESS           
C ZEPSCW:  SECURITY THRESHOLD FOR CLOUD LIQUID WATER PATH               
C                                                                       
      REAL          CDAY,CCO2,CH2O,DIFF,X10E                            
     S             ,ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW            
C     ----------------------------------------------------------------- 
C*COMDECK YOMSW                                                         
C     ----------------------------------------------------------------- 
C                                                                       
C*    COEFFICIENTS FOR THE SHORTWAVE RADIATION SUBROUTINES              
C                                                                       
C APAD  :  PADE APPROXIMANTS NUMERATOR                                  
C BPAD  :  PADE APPROXIMANTS DENOMINATOR                                
C D     :  TRANSMISSION LIMIT FOR INFINITE ABSORBER AMOUNT              
C CRAY  :  RAYLEIGH SCATTERING COEFFICIENTS                             
C SUN   :  SOLAR FRACTION IN SPECTRAL INTERVALS                         
C                                                                       
      REAL  APAD(2,3,7),BPAD(2,3,7),D(2,3),CRAY(2,6),SUN(2)             
C     ----------------------------------------------------------------- 
                                                                        
C                                                                       
C  -------------------------------------------------------------------- 
C                                                                       
      REAL   TTT(1,NLAT,NRLEV,12)                                       
C                                                                       
C        0.1  "RADINT POINTERS"                                         
C             -----------------                                         
C                                                                       
      REAL   TM1(NLP2,NLEV),                                            
     *       TSM1M(NLP2),                                               
     *       QM1(NLP2,NLEV),                                            
     *       SNM1M(NLP2),                                               
     *       ALBM(NLP2),                                                
     *       AVVELM(NLP2)                                               
      REAL   SLMM(NLP2),                                                
     *       APHM1(NLP2,NLEVP1),                                        
     *       APM1(NLP2,NLEV)                                            
C                                                                       
      LOGICAL LOLAND(NLP2)                                              
C                                                                       
C     VARIABLES FOR FULL GRID AND TRANSFORMS.                           
C                                                                       
      REAL   ZTI(NLP2,NRLEVP1),                                         
     *       ZCTI(NLP2,NRLEVP1),                                        
     *       ZCPH(NLP2,NRLEVP1),                                        
     *       ZWV(NLP2,NRLEV),                                           
     *       ZSW(NLP2,NRLEV),                                           
     *       ZALTE(NLP2),                                               
     *       ZCALTE(NLP2),                                              
     *       ZALSO(NLP2),                                               
     *       ZCALSO(NLP2)                                               
C                                                                       
      REAL   ZOZQ(NLP2),                                                
     #       ZOZH(NLP2)                                                 
C                                                                       
      REAL   ZTRSOL(NLP2,NLEVP1),                                       
     *       ZEMTER(NLP2,NLEVP1)                                        
C                                                                       
C     FINAL RESULTS ARE STORED ON:                                      
C                                                                       
      REAL   TRSOLM(NLP2,NLEVP1),                                       
     *       EMTERM(NLP2,NLEVP1)                                        
C                                                                       
      REAL   AMU0(NLP2),                                                
     *       RDAYL(NLP2)                                                
C                                                                       
C     VARIABLES WITHIN THE SPLITTED LATITUDE LOOP.                      
C                                                                       
      REAL   CLC(NRLST,NRLEV),                                          
     *       TI(NRLST,NRLEVP1),                                         
     *       DP(NRLST,NRLEV),                                           
     *       AADS(NRLST,NRLEV),                                         
     *       AEQ1(NRLST,NRLEV),                                         
     *       AEQ2(NRLST,NRLEV),                                         
     *       AEQ3(NRLST,NRLEV),                                         
     *       AEQ4(NRLST,NRLEV),                                         
     *       AEQ5(NRLST,NRLEV),                                         
     *       SSW(NRLST,NRLEV),                                          
     *       WV(NRLST,NRLEV),                                           
     *       QOF(NRLST,NRLEV),                                          
     *       QCF(NRLST,NRLEV),                                          
     *       CLWA(NRLST,NRLEV),                                         
     *       APRE(NRLST),                                               
     *       SMU0(NRLST),                                               
     *       ALTE(NRLST),                                               
     *       ALSO(NRLST)                                                
      REAL   ZDPO(NRLST),                                               
     *       ZQCFO(NRLST),                                              
     *       ZAEQSO(NRLST),                                             
     *       ZQOFO(NRLST),                                              
     *       ZDPN(NRLST),                                               
     *       ZQCFN(NRLST),                                              
     *       ZAEQSN(NRLST),                                             
     *       ZQOFN(NRLST)                                               
      REAL   FLT(NRLST,NRLEVP1),                                        
     *       FLS(NRLST,NRLEVP1)                                         
C                                                                       
C     DIAGNOSTIC VARIABLES.                                             
C                                                                       
      REAL   DIAG(NLEVP1,4),                                            
     *       DIA1(7),                                                   
     *       ZDIAG(NLEVP1,10),                                          
     *       ZDIA1(21),                                                 
     *       ZDIA(NLON),                                                
     *       ZDIAF(NLON,7),                                             
     *       ZDIAT(NLEVP1)                                              
C                                                                       
C                                                                       
C                                                                       
      REAL PGG(IRLST),                                                  
     #     PREF(IRLST),                                                 
     #     PRE1(IRLST),                                                 
     #     PRE2(IRLST),                                                 
     #     PRMUZ(IRLST),                                                
     #     PTO1(IRLST),                                                 
     #     PTR1(IRLST),                                                 
     #     PTR2(IRLST),                                                 
     #     PW(IRLST)                                                    
C                                                                       
C                                                                       
C  what about the 1-dimensional PALBS???                                
C                                                                       
      REAL     PALBSN  (IRLST)                                          
C                                                                       
C*       0.2   "LOCAL" ARRAYS                                           
C              --------------                                           
      REAL ZABSCU(IRLST ,NUA, 3*NRLEV+1),                               
     S     ZABSLY(IRLST ,NUA, 3*NRLEV+1) ,                              
     S     ZADJD (IRLST , IRLVP1)      ,                                
     S     ZADJU (IRLST , IRLVP1)       ,                               
     S     ZBGND (IRLST)                ,                               
     S     ZBINT (IRLST , IRLVP1)       ,                               
     S     ZBSUR (IRLST , NINT)         ,                               
     S     ZBSUIN(IRLST)                ,                               
     S     ZBTOP (IRLST , NINT)         ,                               
     S     PCLDLW(IRLST , IRLEV)        ,                               
     S     ZCLOUD(IRLST)                ,                               
     S     ZCNTRB(IRLST , IRLVP1,IRLVP1),                               
     S     ZCUMP (IRLST)                ,                               
     S     ZCUMPT(IRLST)                ,                               
     S     ZDBDT (IRLST , NINT , IRLEV) ,                               
     S     ZDBSL (IRLST , NINT , 2*NRLEV),                              
     S     ZDISTD(IRLST , IRLVP1)       ,                               
     S     ZDISTU(IRLST , IRLVP1)                                       
      REAL                                                              
     S     ZDNF  (IRLST , IRLVP1,IRLVP1),                               
     S     PDT0  (IRLST)                                                
      REAL                                                              
     S     ZDUC  (IRLST , 3*NRLEV+1)       ,                            
     S     ZEFFT (IRLST)                ,                               
     S     ZEMD  (IRLST , IRLEV), ZEMU  (IRLST , IRLEV),                
     S     ZEPSC2(IRLST),  ZEPSC3(IRLST),                               
     S     ZEPSH1(IRLST),  ZEPSH2(IRLST),                               
     S     ZEPSH3(IRLST),  ZEPSH4(IRLST),                               
     S     ZEPSH5(IRLST),  ZEPSH6(IRLST),                               
     S     ZEPHIO(IRLST),  ZFACTC(IRLST),                               
     S     ZZFD(IRLST), ZFDD(IRLST), ZFDN(IRLST , IRLVP1)               
      REAL                                                              
     S     PFLUX (IRLST , 2 , IRLVP1)   ,                               
     S     PFLUC (IRLST , 2 , IRLVP1)   ,                               
     S     ZFLWP (IRLST)                ,                               
     S     PCTS  (IRLST , IRLEV)        ,                               
     S     ZZFU   (IRLST)               ,                               
     S     ZFUP  (IRLST , IRLVP1)       ,                               
     S     ZGLAYD(IRLST)                ,                               
     S     ZGLAYU(IRLST)                ,                               
     S     ZOMEGA(IRLST , 2 , IRLEV)                                    
C                                                                       
      INTEGER                                                           
     S     ITX   (IRLST)                ,                               
     S     IXDIA (IRLST , IRLVP1)       ,                               
     S     IXT   (IRLST , IRLVP1,IRLVP1),                               
     S     IXTSU (IRLST , IRLVP1)       ,                               
     S     IXTTP (IRLST , IRLVP1)       ,                               
     S     IMX   (IRLST)                ,                               
     S     IMXP  (IRLST)                                                
C                                                                       
      REAL ZOZ   (IRLST , IRLEV)        ,                               
     S     PPMB  (IRLST , IRLVP1)       ,                               
     S     ZRES  (IRLST)                ,                               
     S     PB    (IRLST, 6 , IRLVP1)    ,                               
     S     ZSSIG (IRLST , 3*NRLEV+1)    ,                               
     S     PTAU  (IRLST , 2 , IRLEV)    ,                               
     S     PTAVE (IRLST , IRLEV)        ,                               
     S     ZTAVIC(IRLST)                ,                               
     S     ZTI1  (IRLST)                ,                               
     S     PTL   (IRLST , IRLVP1)       ,                               
     S     ZTT   (IRLST , NTRA)         ,                               
     S     ZTT1  (IRLST , NTRA)         ,                               
     S     ZTT2  (IRLST , NTRA)         ,                               
     S     ZUAER (IRLST , NINT)         ,                               
     S     ZUPF  (IRLST , IRLVP1,IRLVP1),                               
     S     ZUU   (IRLST , NUA)          ,                               
     S     ZXD   (IRLST)                                                
      REAL                                                              
     S     ZXN   (IRLST)                ,                               
     S     ZXOZ  (IRLST)                ,                               
     S     ZXWV  (IRLST)                ,                               
     S     ZZZ   (IRLST)                ,                               
     S     ZRES2 (IRLST)                ,                               
     S     ZTI2  (IRLST)                ,                               
     S     ZBLAY (IRLST,IRLEV)          ,                               
     S     ZBLEV (IRLST,IRLVP1)         ,                               
     S     ZCTS  (IRLST)                ,                               
     S     ZHFG  (IRLST)                                                
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C                                                                       
      INTEGER KT    (IRLST)                                             
      REAL    PUU   (IRLST , NUA  )     ,                               
     S        PTT   (IRLST , NTRA )     ,                               
     S        ZZ    (IRLST)                                             
C                                                                       
C     -------------------------------------------------------- ---------
C                                                                       
      REAL  PCLFR  (IRLST , IRLEV)       ,                              
     S      PT     (IRLST , IRLVP1)       ,                             
     S      PQOF   (IRLST , IRLEV)          ,                           
     S      PQLWP  (IRLST , IRLEV)          ,                           
     S      PEMIS  (IRLST)                                              
      REAL                                                              
     S      PFLT   (IRLST , IRLVP1)                                     
C                                                                       
      REAL  ZALBSU(IRLST , 2 )           ,                              
     S      ZCG   (IRLST , 2 , IRLEV)    ,                              
     S      ZCLDLW(IRLST , IRLEV)        ,                              
     S      ZCLDSW(IRLST , IRLEV)        ,                              
     S      ZCOOLR(IRLST , IRLEV)        ,                              
     S      ZDT0  (IRLST)                ,                              
     S      ZFLUX (IRLST , 2 , IRLVP1)   ,                              
     S      ZPMB  (IRLST , IRLVP1)       ,                              
     S      ZTAU  (IRLST , 2 , IRLEV)    ,                              
     S      ZTAVE (IRLST , IRLEV)        ,                              
     S      ZTL   (IRLST , IRLVP1)       ,                              
     S      ZFSDWN(IRLST , IRLVP1)       ,                              
     S      ZFSUP (IRLST , IRLVP1)       ,                              
     S      ZHEATR(IRLST , IRLEV )                                      
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C                                                                       
      REAL   PDP   (IRLST , IRLEV)     ,                                
     S       PWV   (IRLST , IRLEV)     ,                                
     S       PFLS  (IRLST , IRLVP1)    ,                                
     S       PPSOL (IRLST        )     ,                                
     S       PRMU0 (IRLST        )     ,                                
     S       PAER  (IRLST,IRLEV,5)     ,                                
     S       PALBS (IRLST , 2 )        ,                                
     S       PCG   (IRLST , 2 , IRLEV) ,                                
     S       PCLDSW(IRLST , IRLEV)     ,                                
     S       POMEGA(IRLST , 2 , IRLEV) ,                                
     S       POZ   (IRLST , IRLEV)                                      
C                                                                       
      REAL   PFDOWN(IRLST , IRLVP1)    ,                                
     S       PFUP  (IRLST , IRLVP1)                                     
C                                                                       
      REAL   ZAKI  (IRLST , 2 )        ,                                
     S       ZCGAZ (IRLST , IRLEV)     ,                                
     S       ZCNEB (IRLST , IRLEV)     ,                                
     S       ZC1I  (IRLST , IRLVP1)    ,                                
     S       ZDSC  (IRLST , IRLEV)     ,                                
     S       ZDSH  (IRLST , IRLEV)     ,                                
     S       ZDSIG (IRLST , IRLEV)     ,                                
     S       ZFACT (IRLST)             ,                                
     S       ZFACT1 (IRLST), ZFACT2 (IRLST),                            
     S       ZFACT3 (IRLST), ZFACT4 (IRLST),                            
     S       ZFACT5 (IRLST), ZFACT6 (IRLST),                            
     S       ZFD   (IRLST , IRLVP1)    ,                                
     S       ZFRAC (IRLST)             ,                                
     S       ZFU   (IRLST , IRLVP1)    ,                                
     S       ZG    (IRLST)             ,                                
     S       ZGG   (IRLST)             ,                                
     S       ZPIZAZ(IRLST , IRLEV)     ,                                
     S       ZRAYL (IRLST),  ZRAY1 (IRLST , IRLVP1)                     
      REAL                                                              
     S       ZRAY2 (IRLST , IRLVP1)    ,                                
     S       ZREF  (IRLST)             ,                                
     S       ZREFZ (IRLST , 2 , IRLVP1),                                
     S       ZRE1  (IRLST)             ,                                
     S       ZRE2  (IRLST)             ,                                
     S       ZRJ   (IRLST , 6 , IRLVP1),                                
     S       ZRK   (IRLST , 6 , IRLVP1),                                
     S       ZRL   (IRLST , 8)         ,                                
     S       ZRMU  (IRLST)             ,                                
     S       ZRMUE (IRLST , IRLVP1)    ,                                
     S       ZRMUZ (IRLST)                                              
      REAL                                                              
     S       ZRNEB (IRLST)             ,                                
     S       ZRUEF (IRLST , 8)         ,                                
     S       ZR1   (IRLST)             ,                                
     S       ZR2   (IRLST)             ,                                
     S       ZR21  (IRLST)             ,                                
     S       ZR22  (IRLST)             ,                                
     S       ZR23  (IRLST)             ,                                
     S       ZS    (IRLST)             ,                                
     S       ZSEC  (IRLST)             ,                                
     S       ZSS1  (IRLST)             ,                                
     S       ZTAUAZ(IRLST , IRLEV)     ,                                
     S       ZTOT  (IRLST , IRLVP1)    ,                                
     S       ZTO1  (IRLST)             ,                                
     S       ZTR   (IRLST , 2 , IRLVP1),                                
     S       ZTRA1 (IRLST , IRLVP1)    ,                                
     S       ZTRA2 (IRLST , IRLVP1)                                     
      REAL                                                              
     S       ZTR1  (IRLST)             ,                                
     S       ZTR2  (IRLST)             ,                                
     S       ZUD   (IRLST , 3 , IRLVP1),                                
     S       ZUM   (IRLST , IRLVP1)    ,                                
     S       ZU1D  (IRLST)             ,                                
     S       ZU2D  (IRLST)             ,                                
     S       ZW    (IRLST)             ,                                
     S       ZN175 (IRLST)             ,                                
     S       ZN190 (IRLST)             ,                                
     S       ZO175 (IRLST)             ,                                
     S       ZO190 (IRLST)             ,                                
     S       ZP75  (IRLST)             ,                                
     S       ZP90  (IRLST)             ,                                
     S       ZSIGN (IRLST)             ,                                
     S       ZSIGO (IRLST)                                              
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
      REAL   PTR    (IRLST)     ,                                       
     S       PU     (IRLST)                                             
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
C   CLOUD-POINTERS                                                      
C   --------------                                                      
C                                                                       
      REAL   ZRH(NLP2,NLEV),                                            
     *       ZCH(NLP2),                                                 
     *       ZCM(NLP2),                                                 
     *       ZCL(NLP2),                                                 
     *       ZCT(NLP2),                                                 
     *       ZCC(NLP2)                                                  
      INTEGER NTOPCM(NLP2),                                             
     *        NBASECM(NLP2)                                             
      REAL   ARPRCM(NLP2)                                               
      INTEGER ITOPH(NLP2),                                              
     *        ITOPM(NLP2),                                              
     *        ITOPL(NLP2)                                               
      LOGICAL LOC(NLP2)                                                 
      REAL   ZQS(NLP2,NLEV),                                            
     *       ZCLC(NLP2,NLEV),                                           
     *       ZQLWP(NLP2,NLEV)                                           
C                                                                       
      REAL   VERVEL(NLP2,NLEV),                                         
     *       ZTHETA(NLP2,NLEV),                                         
     *       ZETA(NLP2,NLEV),                                           
     *       ZDTHMIN(NLP2)                                              
      INTEGER INVB(NLP2)                                                
      REAL   ZCTLWP(NLP2),                                              
     *       ZQLOC(NLP2),                                               
     *       ZQMEC(NLP2),                                               
     *       ZQHIC(NLP2)                                                
C                                                                       
C  definitions of "radheat"                                             
C                                                                       
      REAL   ZFSOT(NLON),                                               
     #       ZFTET(NLON),                                               
     #       ZFLT (NLON),                                               
     #       ZFSO (NLON),                                               
     #       ZFTE (NLON),                                               
     #       ZFLB (NLON),                                               
     #       ZDTDT(NLON),                                               
     #       HEATO(NLON,NLEV),                                          
     #       HEATE(NLON,NLEV),                                          
     #       HEATR(NLON,NLEV),                                          
     #       ZI0  (NLON)                                                
C                                                                       
C PIERS 13-8-97                                                         
C                                                                       
         REAL PIERSLWP                                                  
C                                                                       
C Piece of code needed to replace the Nikos radiation scheme with       
C the Morcrette radiation scheme in the igcm3.1.                        
C See morcrette3_1_1.readme for details.                                
C                                                                       
C                                                                       
c      program lh                                                       
c                                                                       
c      parameter(NL=23)                                                 
c      real P(NL),t(NL),h2o(NL),o3(NL),htlw(NL),htsw(NL)                
c      real swalb,doy                                                   
c       REAL CF(4,2)     !cloud fractions,lwps                          
c       INTEGER IC(4,2)   ! cloud positions                             
c       REAL RFLUXES(2,2,2)                                             
c      open(11,file='mls23.dat')                                        
c      read(11,*) NLEV                                                  
c      DOY=80                                                           
c      swalb=0.07                                                       
c      alat=40.0                                                        
c       doy=float(i)                                                    
c      do i=nlev,1,-1                                                   
c         read(11,*) P(i),t(i),h2o(i),adum,o3(i)                        
c         print *,i,p(i),t(i),h2o(i),o3(i)                              
c      enddo                                                            
c                                                                       
c       ic(1,1)=3                                                       
c      ic(1,2)=17                                                       
c       ic(3,1)=6                                                       
c       ic(2,2)=6                                                       
c       cf(1,1)=0.3                                                     
c       cf(1,2)=45.0                                                    
c        cf(3,1)=0.8                                                    
c       cf(4,2)=20.0                                                    
c      alon=0.0                                                         
c      call morcigcm(p,t,h2o,o3,alat,htlw,htsw,doy,cf,ic,               
c     $     rfluxes,swalb,alon)                                         
c      enddo                                                            
c      do i=nl,1,-1                                                     
c         print '(i6,4F14.4)',i,p(i),t(i),htsw(i),htlw(i)               
c      enddo                                                            
c      print *,RFLUXES(1,1,1), RFLUXES(1,1,2), RFLUXES(1,2,1),          
c     $     RFLUXES(1,2,2)                                              
c      print *,RFLUXES(2,1,1), RFLUXES(2,1,2), RFLUXES(2,2,1),          
c     $     RFLUXES(2,2,2)                                              
c                                                                       
c                                                                       
c      end                                                              
c      INCLUDE 'zparb.upd'                                              
C     COMMON BLOCK FOR GAS ABUNDANCES                                   
!-----------------------------------------------------                  
! Common block for gas abundances                                       
!-----------------------------------------------------                  
      COMMON/MORCGAS/VMRCO2,ZCARDI,VMRCH4,RCH4,VMRN2O,RN2O,             
     &               VMRCFC11,RCFC11,VMRCFC12,RCFC12,                   
     &               VMRH2OS,RH2OS,NEXOTIC                              
      LOGICAL :: NEXOTIC                                                
      REAL :: ZCARDI,RCH4,RN2O,RCFC11,RCFC12,RH2OS                      
      REAL :: VMRCO2,VMRCH4,VMRN2O,VMRCFC11,VMRCFC12,VMRH2OS            
      REAL PGGA(KDLON,12,2),PGGB(KDLON,12,2),                           
     *                 PPPTY(KDLON,12), PPETY(KDLON,12)                 
                                                                        
      DO 97 JL=1,NRLST                                                  
         DO 98 II=1,NUA                                                 
            PUU(JL,II) = ZUU(JL,II)                                     
 98      CONTINUE                                                       
         DO 99 JJ=1,NTRA                                                
            PTT(JL,JJ) = ZTT(JL,JJ)                                     
 99      CONTINUE                                                       
 97       CONTINUE                                                      
C                                                                       
C                                                                       
C*         1.     HORNER'S ALGORITHM FOR H2O AND CO2 TRANSMISSION       
C                 -----------------------------------------             
C                                                                       
 100  CONTINUE                                                          
                                                                        
! changing - marc 30/3/10                                               
!      DO 101 JL = 1 , 6*KDLON                                          
!        ZZ(JL) = SQRT (PUU(JL,1))                                      
!        ZXN(JL) = PGGA(JL,1,1) + ZZ(JL)*(PGGA(JL,1,2))                 
!        ZXD(JL) = PGGB(JL,1,1) + ZZ(JL)*(PGGB(JL,1,2)+ZZ(JL))          
!        PTT(JL,1) = ZXN(JL) / ZXD(JL)                                  
! 101  CONTINUE                                                         
      DO I=1,6                                                          
         DO JL=1,KDLON                                                  
            ZZ(JL) = SQRT (PUU(JL,i))                                   
            ZXN(JL) = PGGA(JL,i,1) + ZZ(JL)*(PGGA(JL,i,2))              
            ZXD(JL) = PGGB(JL,i,1) + ZZ(JL)*(PGGB(JL,i,2)+ZZ(JL))       
            PTT(JL,i) = ZXN(JL) / ZXD(JL)                               
         ENDDO                                                          
      ENDDO                                                             
                                                                        
      DO 1003 JL = 1 , KDLON                                            
C                                                                       
        ZZ(JL) = SQRT (PUU(JL,8))                                       
        ZXN(JL) = PGGA(JL,8,1) + ZZ(JL)*(PGGA(JL,8,2))                  
        ZXD(JL) = PGGB(JL,8,1) + ZZ(JL)*(PGGB(JL,8,2)+ZZ(JL))           
        PTT(JL,8) = ZXN(JL)/ZXD(JL)                                     
C                                                                       
C  1250-1450 CM-1 H2O                                                   
C                                                                       
        ZZ(JL) = SQRT (PUU(JL,29))                                      
        ZXN(JL)= PGGA(JL,9,1) + ZZ(JL)*(PGGA(JL,9,2) )                  
        ZXD(JL)= PGGB(JL,9,1) + ZZ(JL)*(PGGB(JL,9,2) + ZZ(JL))          
        PTT(JL,16) = ZXN(JL)/ZXD(JL)                                    
C                                                                       
C  800-970 CM-1 H2O                                                     
C                                                                       
        ZZ(JL) = SQRT (PUU(JL,30))                                      
        ZXN(JL)= PGGA(JL,10,1) + ZZ(JL)*(PGGA(JL,10,2) )                
        ZXD(JL)= PGGB(JL,10,1) + ZZ(JL)*(PGGB(JL,10,2) + ZZ(JL))        
        PTT(JL,17) = ZXN(JL)/ZXD(JL)                                    
C                                                                       
C  650-800 CM-1 H2O                                                     
C                                                                       
        ZZ(JL) = SQRT (PUU(JL,31))                                      
        ZXN(JL)= PGGA(JL,11,1) + ZZ(JL)*(PGGA(JL,11,2) )                
        ZXD(JL)= PGGB(JL,11,1) + ZZ(JL)*(PGGB(JL,11,2) + ZZ(JL))        
        PTT(JL,18) = ZXN(JL)/ZXD(JL)                                    
1003  CONTINUE                                                          
C     --------------------------------------------------------          
C                                                                       
C*         1.1    2-D INTERPOLATION BETWEEN LBL CO2 TRANSMISSION        
C                 -----------------------------------------------       
C                                                                       
      DO 1011 JL = 1 , KDLON                                            
        CGU  = LOG(PUU(JL,34))                                          
        CGP  = LOG(PUU(JL,7)/PUU(JL,34))                                
        CGT  = PUU(JL,33)/PUU(JL,34)                                    
        DTX  = CGT-TREF                                                 
        JU   =INT((CGU+13.)*2.)+1                                       
        JP   =INT((CGP+8.)*2.)+1                                        
        XIU  = (CGU - ULG(JU))*2.0                                      
        XIP  = (CGP - PLG(JP))*2.0                                      
        JP1  = JP+1                                                     
        JU1  = JU+1                                                     
        XUP0 = XIU * XIP                                                
        XUP1 = 1.0 - XIP -XIU + XUP0                                    
        XUP2 = XIU - XUP0                                               
        XUP3 = XIP - XUP0                                               
        TBUP = XUP1 * TB (  JU ,   JP ,   2)                            
     S       + XUP2 * TB (  JU1,   JP ,   2)                            
     S       + XUP3 * TB (  JU ,   JP1,   2)                            
     S       + XUP0 * TB (  JU1,   JP1,   2)                            
        ALUP = XUP1 * ALP(  JU ,   JP,    2)                            
     S       + XUP2 * ALP(  JU1,   JP,    2)                            
     S       + XUP3 * ALP(  JU ,   JP1,   2)                            
     S       + XUP0 * ALP(  JU1,   JP1,   2)                            
        BTUP = XUP1 * BET(  JU ,   JP,    2)                            
     S       + XUP2 * BET(  JU1,   JP,    2)                            
     S       + XUP3 * BET(  JU ,   JP1,   2)                            
     S       + XUP0 * BET(  JU1,   JP1,   2)                            
        TTRR=1.0-EXP(-TBUP/1000.)                                       
        PTT(JL,7)=TTRR - (1.-TTRR)*DTX*(ALUP + BTUP*DTX)                
c        print*,'ptt7',ptt(jl,7)                                        
 1011 CONTINUE                                                          
C                                                                       
C*         1.2    2-D INTERPOLATION BETWEEN LBL O3 TRANSMISSION         
C                 -----------------------------------------------       
C                                                                       
      DO 1021 JL = 1 , KDLON                                            
        CGU  = MAX( LOG(PUU(JL,12)), -17.5)                             
        CGP  = MAX( LOG(PUU(JL,13)/PUU(JL,12)), -8.0)                   
        CGT  = PUU(JL,32)/PUU(JL,12)                                    
        DTX  = CGT-TREF                                                 
        JU   = INT((CGU+17.5)*2.)+1.                                    
        JP   = INT((CGP+8.)*2.)+1.                                      
        XIU  = (CGU - ULGO(JU))*2.0                                     
        XIP  = (CGP - PLG (JP))*2.0                                     
        JU1  = JU+1                                                     
        JP1  = JP+1                                                     
        XUP0 = XIU * XIP                                                
        XUP1 = 1.0 - XIP -XIU + XUP0                                    
        XUP2 = XIU - XUP0                                               
        XUP3 = XIP - XUP0                                               
                                                                        
        TBUP = XUP1 * TBO3( JU,   JP )                                  
     S       + XUP2 * TBO3( JU1,  JP )                                  
     S       + XUP3 * TBO3( JU,   JP1)                                  
     S       + XUP0 * TBO3( JU1,  JP1)                                  
        ALUP = XUP1 * AO3(  JU,   JP )                                  
     S       + XUP2 * AO3(  JU1,  JP )                                  
     S       + XUP3 * AO3(  JU,   JP1)                                  
     S       + XUP0 * AO3(  JU1,  JP1)                                  
        BTUP = XUP1 * BO3(  JU,   JP )                                  
     S       + XUP2 * BO3(  JU1,  JP )                                  
     S       + XUP3 * BO3(  JU,   JP1)                                  
     S       + XUP0 * BO3(  JU1,  JP1)                                  
        TBUP2= XUP1 * TO3B( JU,   JP )                                  
     S       + XUP2 * TO3B( JU1,  JP )                                  
     S       + XUP3 * TO3B( JU,   JP1)                                  
     S       + XUP0 * TO3B( JU1,  JP1)                                  
        ALUP2= XUP1 * O3A(  JU,   JP )                                  
     S       + XUP2 * O3A(  JU1,  JP )                                  
     S       + XUP3 * O3A(  JU,   JP1)                                  
     S       + XUP0 * O3A(  JU1,  JP1)                                  
        BTUP2= XUP1 * O3B(  JU,   JP )                                  
     S       + XUP2 * O3B(  JU1,  JP )                                  
     S       + XUP3 * O3B(  JU,   JP1)                                  
     S       + XUP0 * O3B(  JU1,  JP1)                                  
        TTRR=1.0-EXP(-TBUP/100.)                                        
        PTT(JL,13)=TTRR - (1.-TTRR)*DTX*(ALUP + BTUP*DTX)               
        TTRR2=1.0-EXP(-TBUP2/100.)                                      
        PTT(JL,16)=TTRR2 - (1.-TTRR2)*DTX*(ALUP2 + BTUP2*DTX)           
 1021 CONTINUE                                                          
C     -------------------------------------------------------           
C                                                                       
 200  CONTINUE                                                          
                                                                        
      DO 201 JL = 1 , KDLON                                             
        PTT(JL, 9) = PTT(JL, 8)                                         
        PTT(JL, 7) = PTT(JL, 7)*PTT(JL,16)                              
C                                                                       
C-  CONTINUUM ABSORPTION: E- AND P-TYPE                                 
C                                                                       
        ZPU   = PUU(JL,10)                                              
        ZPU2  = ZPU*ZPU                                                 
        ZPU10 = PPPTY(JL,1)*(ZPU+0.3*ZPU2)/(1.+9.*ZPU)                  
        ZPU11 = 0.7*PPPTY(JL,2)*(ZPU+0.34*ZPU2)/(1.+2.*ZPU)             
        ZPU12A= PPPTY(JL,3)*(ZPU+0.066*ZPU2)/(1.+0.27*ZPU)              
        ZPU12B= PPPTY(JL,4)*(ZPU+0.057*ZPU2)/(1.+0.095*ZPU)             
        ZPU13 = 0.017 * ZPU                                             
        ZPU14 = 0.041 * ZPU                                             
        ZPU12 = 0.017 * ZPU                                             
        ZPU16 = PPPTY(JL,8)*(ZPU+0.19*ZPU2)/(1.+0.85*ZPU)               
        ZPU17 = PPPTY(JL,9)*(ZPU+0.035*ZPU2)/(1.+0.71*ZPU)              
                                                                        
        ZEUB  = PUU(JL,11)                                              
        ZEUB2 = ZEUB*ZEUB                                               
        ZEUC  = PUU(JL,25)                                              
        ZEUD  = PUU(JL,26)                                              
        ZEUE  = PUU(JL,27)                                              
        ZEUF  = PUU(JL,28)                                              
                                                                        
        ZEU10 = PPETY(JL,1)*(ZEUB+5.*ZEUB2)/(1.+150.*ZEUB)              
        ZEU11 = PPETY(JL,2)*(ZEUB+15.5*ZEUB2)/(1.+42.*ZEUB)             
        ZEU12A= PPETY(JL,3)*(1.+7.*ZEUC)/(1.+17.5*ZEUC)*ZEUC            
        ZEU12B= PPETY(JL,4)*(1.+4.2*ZEUD)/(1.+9.5*ZEUD)*ZEUD            
        ZEU13 = PPETY(JL,5)*(1.+3.8*ZEUE)/(1.+7.1*ZEUE)*ZEUE            
        ZEU14 = 6.5 * ZEUC                                              
        ZEU12 = 7.0 * ZEUD                                              
        ZEU16 = PPETY(JL,8)*(ZEUB+4.3*ZEUB2)/(1.+11.8*ZEUB)             
        ZEU17 = PPETY(JL,9)*(1.+1.*ZEUF)/(1.+9.*ZEUF)*ZEUF              
C                                                                       
C                                                                       
C     ---------------------------------------------------------         
C                                                                       
C*         2     OZONE AND AEROSOL TRANSMISSION FUNCTIONS               
C                 -----------------------------------------             
C                                                                       
C-  OZONE ABSORPTION                                                    
C                                                                       
c        ZX = MAX(1.0E-15, PUU(JL,12))                                  
c        ZY = MAX(1.0E-15, PUU(JL,13))                                  
c        ZUXY = 4. * ZX * ZX / (PIALF0 * ZY)                            
c        ZSQ1 = SQRT(1. + O1H * ZUXY ) - 1.                             
c        ZSQ2 = SQRT(1. + O2H * ZUXY ) - 1.                             
c        ZVXY = PIALF0 * ZY / (2. * ZX)                                 
c        ZAERCN = PUU(JL,17) + ZEU12 + ZPU12                            
c        ZTO11 = EXP( - ZVXY * ZSQ1 - ZAERCN )                          
c        ZTO2 = EXP( - ZVXY * ZSQ2 - ZAERCN )                           
C                                                                       
C-- TRACE GASES (CH4, N2O, CFC-11, CFC-12)                              
C                                                                       
C* CH4 IN INTERVAL 800-970 + 1110-1250 CM-1                             
C                                                                       
      IF (NEXOTIC) THEN                                                 
                                                                        
      ZXCH4 = MAX(1.0D-12, PUU(JL,19))                                  
      ZYCH4 = MAX(1.0D-12, PUU(JL,20))                                  
      ZUXY = 4. * ZXCH4*ZXCH4/(0.103*ZYCH4)                             
      ZSQH41 = SQRT(1. + 33.7 * ZUXY) - 1.                              
      ZVXY = 0.103 * ZYCH4 / (2. * ZXCH4)                               
      ZODH41 = ZVXY * ZSQH41                                            
C                                                                       
C* N2O IN INTERVAL 800-970 + 1110-1250 CM-1                             
C                                                                       
      ZXN2O = MAX(1.0D-12, PUU(JL,21))                                  
      ZYN2O = MAX(1.0D-12, PUU(JL,22))                                  
      ZUXY = 4. * ZXN2O*ZXN2O/(0.416*ZYN2O)                             
      ZSQN21 = SQRT(1. + 21.3 * ZUXY) - 1.                              
      ZVXY = 0.416 * ZYN2O / (2. * ZXN2O)                               
      ZODN21 = ZVXY * ZSQN21                                            
C                                                                       
C* CH4 IN INTERVAL 1250-1450 + 1880-2820 CM-1                           
C                                                                       
      ZUXY = 4. * ZXCH4*ZXCH4/(0.113*ZYCH4)                             
      ZSQH42 = SQRT(1. + 400. * ZUXY) - 1.                              
      ZVXY = 0.113 * ZYCH4 / (2. * ZXCH4)                               
      ZODH42 = ZVXY * ZSQH42                                            
C                                                                       
C* N2O IN INTERVAL 1250-1450 + 1880-2820 CM-1                           
C                                                                       
      ZUXY = 4. * ZXN2O*ZXN2O/(0.197*ZYN2O)                             
      ZSQN22 = SQRT(1. + 2000. * ZUXY) - 1.                             
      ZVXY = 0.416 * ZYN2O / (2. * ZXN2O)                               
      ZODN22 = ZVXY * ZSQN22                                            
C                                                                       
C* CFC-11 IN INTERVAL 800-970 + 1110-1250 CM-1                          
C                                                                       
      ZA11 = 2. * PUU(JL,23) * 4.404E+05                                
      ZTTF11 = 1. - ZA11 * 0.003225                                     
C                                                                       
C* CFC-12 IN INTERVAL 800-970 + 1110-1250 CM-1                          
C                                                                       
      ZA12 = 2. * PUU(JL,24) * 6.7435E+05                               
      ZTTF12 = 1. - ZA12 * 0.003225                                     
      ELSE                                                              
         ZODH41=0.                                                      
         ZODN21=0.                                                      
         ZODH42=0.                                                      
         ZODN22=0.                                                      
         ZTTF11=1.                                                      
         ZTTF12=1.                                                      
      END IF                                                            
C                                                                       
      TC1=EXP(-ZEU12A-ZPU12A)                                           
      TC2=EXP(-ZEU12B-ZPU12B)                                           
      PTT2=PTT(JL, 2)                                                   
      PTTB=PTT(JL,18)                                                   
      PTT(JL, 2) = 0.5 *(PTTB*TC2 + (2.*PTT2-PTTB)*TC1)                 
C                                                                       
      TC1=EXP(-ZEU13-ZPU13)                                             
      TC2=EXP(-ZEU14-ZPU14)                                             
      PTT4=PTT(JL, 4)                                                   
      PTTB=PTT(JL,17)                                                   
      PTT(JL, 4) =  PTT4*TC2 + 0.54839*PTTB*(TC1-TC2)                   
C                                                                       
      TC1=EXP(-ZEU16-ZPU16)                                             
      TC2=EXP(-ZEU17-ZPU17)                                             
      PTT6=PTT(JL, 6)                                                   
      PTTB=PTT(JL,16)                                                   
      PTT(JL, 6) =  PTT6*TC2 + 0.17544*PTTB*(TC1-TC2)                   
C                                                                       
      ZUU11 = - PUU(JL,15)                                              
      ZUU12 = - PUU(JL,16) - ZODH41 - ZODN21                            
      PTT(JL,10) = EXP( - PUU(JL,14) -ZEU10 -ZPU10 )                    
      PTT(JL,11) = EXP( ZUU11 )                                         
      PTT(JL,12) = EXP( ZUU12 ) * ZTTF11 * ZTTF12                       
      PTT(JL,14) = EXP( -PUU(JL,14) - ZEU11 - ZPU11 )                   
      PTT(JL,15) = EXP ( - PUU(JL,14) - ZODH42 - ZODN22 )               
C                                                                       
c        TC1=EXP(-ZEU12A-ZPU12A)                                        
c        TC2=EXP(-ZEU12B-ZPU12B)                                        
c        PTT2=PTT(JL, 2)                                                
c        PTTB=PTT(JL,18)                                                
c        PTT(JL, 2) = 0.5 *(PTTB*TC2 + (2.*PTT2-PTTB)*TC1)              
c                                                                       
c        TC1=EXP(-ZEU13-ZPU13)                                          
c        TC2=EXP(-ZEU14-ZPU14)                                          
c        PTT4=PTT(JL, 4)                                                
c        PTTB=PTT(JL,17)                                                
c        PTT(JL, 4) =  PTT4*TC2 + 0.54839*PTTB*(TC1-TC2)                
c                                                                       
c        TC1=EXP(-ZEU16-ZPU16)                                          
c        TC2=EXP(-ZEU17-ZPU17)                                          
c        PTT6=PTT(JL, 6)                                                
c        PTTB=PTT(JL,16)                                                
c        PTT(JL, 6) =  PTT6*TC2 + 0.17544*PTTB*(TC1-TC2)                
c                                                                       
c        ZUU11 = - PUU(JL,15)                                           
c        ZUU12 = - PUU(JL,16)                                           
c        PTT(JL,10) = EXP( - PUU(JL,14) -ZEU10 -ZPU10 )                 
c        PTT(JL,11) = EXP( ZUU11 )                                      
c        PTT(JL,12) = EXP( ZUU12 )                                      
c        PTT(JL,14) = EXP( -PUU(JL,14) - ZEU11 - ZPU11 )                
c        PTT(JL,15) = EXP ( - PUU(JL,14) )                              
 201  CONTINUE                                                          
      DO 301 JL=1,NRLST                                                 
         DO 302 II=1,NUA                                                
           ZUU(JL,II) = PUU(JL,II)                                      
 302     CONTINUE                                                       
         DO 303 JJ=1,NTRA                                               
           ZTT(JL,JJ) = 1.0                                             
 303     CONTINUE                                                       
         IF(ITASK.EQ.1)THEN                                             
           ZTT(JL,1)  = PTT(JL,1)                                       
           ZTT(JL,2)  = PTT(JL,2)                                       
           ZTT(JL,3)  = PTT(JL,3)                                       
           ZTT(JL,4)  = PTT(JL,4)                                       
           ZTT(JL,5)  = PTT(JL,5)                                       
           ZTT(JL,6)  = PTT(JL,6)                                       
         ELSE IF(ITASK.EQ.2)THEN                                        
           ZTT(JL,1)  = PTT(JL,1)                                       
           ZTT(JL,2)  = PTT(JL,2)                                       
           ZTT(JL,3)  = PTT(JL,3)                                       
           ZTT(JL,4)  = PTT(JL,4)                                       
           ZTT(JL,5)  = PTT(JL,5)                                       
           ZTT(JL,6)  = PTT(JL,6)                                       
           ZTT(JL,10) = PTT(JL,10)                                      
           ZTT(JL,11) = PTT(JL,11)                                      
           ZTT(JL,12) = PTT(JL,12)                                      
           ZTT(JL,14) = PTT(JL,14)                                      
           ZTT(JL,15) = PTT(JL,15)                                      
         ELSE IF(ITASK.EQ.3)THEN                                        
           DO 3031 JJ=1,NTRA                                            
             ZTT(JL,JJ) = PTT(JL,JJ)                                    
c        print*,'lwtt0 ptt(jj)=',ptt(jl,jj) , jj                        
 3031       CONTINUE                                                    
         ENDIF                                                          
 301  CONTINUE                                                          
      RETURN                                                            
      END                                                               
C________________________________________________                       
C ii) Replace the old LWTT1 with the new improved version.              
C                                                                       
      SUBROUTINE LWTT1(KDLON,KRLST,KFLEV,ITASK,PGGA,PGGB,PPPTY,PPETY)   
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRSW* SWITCHES RELATED TO THE RADIATION SCHEME.         
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      IMPLICIT REAL (A-H,O-Z)                                           
C      IMPLICIT UNDEFINED (Z)                                           
C      IMPLICIT LOGICAL (L)                                             
C      IMPLICIT UNDEFINED (L)                                           
      IMPLICIT INTEGER (I-K,M-N)                                        
C      IMPLICIT UNDEFINED (I-K,M-N)                                     
C      IMPLICIT REAL (P)                                                
C      IMPLICIT UNDEFINED (A-H,O,Q-Y)                                   
C                                                                       
C everything except the parameters is defined                           
C in the common blocks                                                  
C                                                                       
      COMMON/COM1/                                                      
C     I NRADFR, NRPART, NRINT, NRADPFR, NRADPLA,                        
C     I NRLSTI,                                                         
     R XLAT,                                                            
     R RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR, ZCCO2,              
C     R PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3,                 
     L LOH, LOM, LOL, LORAD                                             
      COMMON/COM2/                                                      
     I ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,                  
     I         JLOOP, KRLEV,                                            
     I        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     I        ITH, ITM, ITL,                                            
     I        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     I        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     I        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     I        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      COMMON/COM3/                                                      
     I IND4, KM12, KXD, KXU, IG, KBS, KDD,                              
     I        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     I        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     I        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     I        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     I        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
      COMMON/COM4/                                                      
     R           COZQC,COZQS,COZHC,COZHS,                               
     R CSUPSAT,                                                         
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
      COMMON/COM5/                                                      
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NUMVC,                                                           
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      COMMON/COM6/                                                      
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD,       
     R DTIME,    TWODT,    EPS,                                         
     R SINLON,     COSLON,                                              
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2,                          
     I NFAX,                                                            
     R CRAE,                                                            
     R CDISSEM                                                          
      COMMON/COM7/                                                      
     R     CCRH,                                                        
     L LO,LO1,LO2,LODIA,LODIAP,                                         
     R TAUA, PIZA, CGA, CAER, CAEROS                                    
      COMMON/COM7A/                                                     
     R  RT1,WG1,AT,BT,CNTNU,O1H,O2H,PIALF0                              
     S ,OCT,TREF,XP,TSTAND,GA,GB,CE,ETY,PTY                             
      COMMON/COM7B/                                                     
     S TINTP,TSTP,                                                      
     S  TBO3,AO3,BO3,TB,ALP,BET,ULG,ULGO,PLG,AP,BP,TO3B,O3A,O3B,        
     R          CDAY,CCO2,CH2O,DIFF,X10E                                
       COMMON/COM7C/                                                    
     S  ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW,                       
     R  APAD,BPAD,D,CRAY,SUN,                                           
     R   TTT,                                                           
     R   TM1                                                            
      COMMON/COM8/                                                      
     R       TSM1M,                                                     
     R       QM1,                                                       
     R       SNM1M,                                                     
     R       ALBM,                                                      
     R       AVVELM,                                                    
     R   SLMM,                                                          
     R       APHM1,                                                     
     R       APM1                                                       
      COMMON/COM9/                                                      
     L LOLAND,                                                          
     R   ZTI,                                                           
     *       ZCTI,                                                      
     *       ZCPH,                                                      
     *       ZWV,                                                       
     *       ZSW,                                                       
     *       ZALTE,                                                     
     *       ZCALTE,                                                    
     *       ZALSO                                                      
      COMMON/COM10/                                                     
     *       ZCALSO,                                                    
     R   ZOZQ,                                                          
     *       ZOZH,                                                      
     R   ZTRSOL,                                                        
     *       ZEMTER,                                                    
     R   TRSOLM,                                                        
     *       EMTERM,                                                    
     R   AMU0,                                                          
     *       RDAYL                                                      
      COMMON/COM11/                                                     
     R   CLC,                                                           
     *       TI,                                                        
     *       DP,                                                        
     *       AADS,                                                      
     *       AEQ1,                                                      
     *       AEQ2,                                                      
     *       AEQ3,                                                      
     *       AEQ4,                                                      
     *       AEQ5,                                                      
     *       SSW                                                        
      COMMON/COM12/                                                     
     *       WV,                                                        
     *       QOF,                                                       
     *       QCF,                                                       
     *       CLWA,                                                      
     *       APRE,                                                      
     *       SMU0,                                                      
     *       ALTE,                                                      
     *       ALSO                                                       
      COMMON/COM13/                                                     
     R   ZDPO,                                                          
     *       ZQCFO,                                                     
     *       ZAEQSO,                                                    
     *       ZQOFO,                                                     
     *       ZDPN,                                                      
     *       ZQCFN,                                                     
     *       ZAEQSN,                                                    
     *       ZQOFN                                                      
      COMMON/COM14/                                                     
     R   FLT,                                                           
     *       FLS,                                                       
     R   DIAG,                                                          
     *       DIA1,                                                      
     *       ZDIAG,                                                     
     *       ZDIA1,                                                     
     *       ZDIA,                                                      
     *       ZDIAF,                                                     
     *       ZDIAT,                                                     
     R PGG                                                              
      COMMON/COM15/                                                     
     #     PREF,                                                        
     #     PRE1,                                                        
     #     PRE2,                                                        
     #     PRMUZ,                                                       
     #     PTO1,                                                        
     #     PTR1,                                                        
     #     PTR2,                                                        
     #     PW,                                                          
     R     PALBSN                                                       
      COMMON/COM16/                                                     
     R ZABSCU,                                                          
     S     ZABSLY,                                                      
     S     ZADJD ,                                                      
     S     ZADJU ,                                                      
     S     ZBGND ,                                                      
     S     ZBINT ,                                                      
     S     ZBSUR ,                                                      
     S     ZBSUIN,                                                      
     S     ZBTOP ,                                                      
     S     PCLDLW,                                                      
     S     ZCLOUD,                                                      
     S     ZCNTRB,                                                      
     S     ZCUMP ,                                                      
     S     ZCUMPT,                                                      
     S     ZDBDT                                                        
      COMMON/COM17/                                                     
     S     ZDBSL ,                                                      
     S     ZDISTD,                                                      
     S     ZDISTU,                                                      
     S     ZDNF  ,                                                      
     S     PDT0  ,                                                      
     S     ZDUC                                                         
      COMMON/COM18/                                                     
     S     ZEFFT ,                                                      
     S     ZEMU  ,                                                      
     S     ZEPHIO,                                                      
     S     ZEPSC2,                                                      
     S     ZEPSC3,                                                      
     S     ZEPSH1,                                                      
     S     ZEPSH2,                                                      
     S     ZEPSH3                                                       
      COMMON/COM19/                                                     
     S     ZEPSH4,                                                      
     S     ZEPSH5,                                                      
     S     ZEPSH6,                                                      
     S     ZEPSIO,                                                      
     S     ZFACTC,                                                      
     S     ZZFD  ,                                                      
     S     ZFDD  ,                                                      
     S     ZFDN                                                         
      COMMON/COM20/                                                     
     S     PFLUX , PFLUC,                                               
     S     ZFLWP , PCTS                                                 
     S     ZZFU  ,                                                      
     S     ZFUP  ,                                                      
     S     ZGLAYD,                                                      
     S     ZGLAYU,                                                      
     S     ZOMEGA,                                                      
     S     ITX   ,                                                      
     S     IXDIA ,                                                      
     S     IXT   ,                                                      
     S     IXTSU ,                                                      
     S     IXTTP ,                                                      
     S     IMX   ,                                                      
     S     IMXP                                                         
      COMMON/COM21/                                                     
     R ZOZ   ,                                                          
     S     PPMB  ,                                                      
     S     ZRES  , PB                                                   
     S     ZSSIG ,                                                      
     S     PTAU  ,                                                      
     S     PTAVE ,                                                      
     S     ZTAVIC,                                                      
     S     ZTI1  ,                                                      
     S     PTL                                                          
      COMMON/COM22/                                                     
     S     ZTT   ,                                                      
     S     ZTT1  ,                                                      
     S     ZTT2  ,                                                      
     S     ZUAER ,                                                      
     S     ZUPF  ,                                                      
     S     ZUU   ,                                                      
     S     ZXD                                                          
      COMMON/COM23/                                                     
     S     ZXN   ,                                                      
     S     ZXOZ  ,                                                      
     S     ZXWV  ,                                                      
     S     ZZZ   ,                                                      
     S     ZRES2 ,                                                      
     S     ZTI2  ,                                                      
     S     ZBLAY ,                                                      
     S     ZBLEV ,                                                      
     S     ZCTS  ,                                                      
     S     ZHFG                                                         
      COMMON/COM24/                                                     
     I KT,                                                              
     R    PUU   ,                                                       
     S        PTT   ,                                                   
     S        ZZ                                                        
     R  PCLFR  ,                                                        
     S      PT     ,                                                    
     S      PQOF   ,                                                    
     S      PQLWP  ,                                                    
     S      PEMIS  ,                                                    
     S      PFLT                                                        
      COMMON/COM25/                                                     
     R  ZALBSU,                                                         
     S      ZCG   ,                                                     
     S      ZCLDLW,                                                     
     S      ZCLDSW,                                                     
     S      ZCOOLR,                                                     
     S      ZDT0  ,                                                     
     S      ZEMD  ,                                                     
     S      ZFLUX ,                                                     
     S      ZPMB  ,                                                     
     S      ZTAU  ,                                                     
     S      ZTAVE ,                                                     
     S      ZTL   ,                                                     
     S      ZFSDWN                                                      
      COMMON/COM26/                                                     
     S      ZFSUP ,                                                     
     S      ZHEATR,                                                     
     R   PDP   ,                                                        
     S       PWV   ,                                                    
     S       PFLS  ,                                                    
     S       PPSOL ,                                                    
     S       PRMU0 ,                                                    
     S       PAER  ,                                                    
     S       PALBS ,                                                    
     S       PCG   ,                                                    
     S       PCLDSW,                                                    
     S       POMEGA,                                                    
     S       POZ                                                        
      COMMON/COM27/                                                     
     R   PFDOWN,                                                        
     S       PFUP  ,                                                    
     R   ZAKI  ,                                                        
     S       ZCGAZ ,                                                    
     S       ZCNEB ,                                                    
     S       ZC1I  ,                                                    
     S       ZDSC  ,                                                    
     S       ZDSH                                                       
      COMMON/COM28/                                                     
     S       ZDSIG ,                                                    
     S       ZFACT ,                                                    
     S       ZFACT1 ,                                                   
     S       ZFACT2 ,                                                   
     S       ZFACT3 ,                                                   
     S       ZFACT4 ,                                                   
     S       ZFACT5 ,                                                   
     S       ZFACT6 ,                                                   
     S       ZFD   ,                                                    
     S       ZFRAC ,                                                    
     S       ZFU   ,                                                    
     S       ZG    ,                                                    
     S       ZGG   ,                                                    
     S       ZPIZAZ                                                     
      COMMON/COM29/                                                     
     S       ZRAYL ,                                                    
     S       ZRAY1 ,                                                    
     S       ZRAY2 ,                                                    
     S       ZREF  ,                                                    
     S       ZREFZ ,                                                    
     S       ZRE1  ,                                                    
     S       ZRE2  ,                                                    
     S       ZRJ   ,                                                    
     S       ZRK   ,                                                    
     S       ZRL   ,                                                    
     S       ZRMU  ,                                                    
     S       ZRMUE ,                                                    
     S       ZRMUZ                                                      
      COMMON/COM30/                                                     
     S       ZRNEB ,                                                    
     S       ZRUEF ,                                                    
     S       ZR1   ,                                                    
     S       ZR2   ,                                                    
     S       ZR21  ,                                                    
     S       ZR22  ,                                                    
     S       ZR23  ,                                                    
     S       ZS    ,                                                    
     S       ZSEC  ,                                                    
     S       ZSS1  ,                                                    
     S       ZTAUAZ                                                     
      COMMON/COM31/                                                     
     S       ZTOT  ,                                                    
     S       ZTO1  ,                                                    
     S       ZTR   ,                                                    
     S       ZTRA1 ,                                                    
     S       ZTRA2 ,                                                    
     S       ZTR1                                                       
      COMMON/COM32/                                                     
     S       ZTR2  ,                                                    
     S       ZUD   ,                                                    
     S       ZUM   ,                                                    
     S       ZU1D  ,                                                    
     S       ZU2D  ,                                                    
     S       ZW    ,                                                    
     S       ZN175 ,                                                    
     S       ZN190 ,                                                    
     S       ZO175 ,                                                    
     S       ZO190 ,                                                    
     S       ZP75  ,                                                    
     S       ZP90  ,                                                    
     S       ZSIGN ,                                                    
     S       ZSIGO                                                      
      COMMON/COM33/                                                     
     R   PTR,                                                           
     S       PU,                                                        
     R   ZRH,                                                           
     *       ZCH,                                                       
     *       ZCM,                                                       
     *       ZCL,                                                       
     *       ZCT,                                                       
     *       ZCC,                                                       
     I NTOPCM                                                           
      COMMON/COM34/                                                     
     *        NBASECM,                                                  
     R   ARPRCM,                                                        
     I ITOPH,                                                           
     *        ITOPM,                                                    
     *        ITOPL,                                                    
     L LOC,                                                             
     R   ZQS,                                                           
     *       ZCLC,                                                      
     *       ZQLWP                                                      
      COMMON/COM35/                                                     
     R   VERVEL,                                                        
     *       ZTHETA,                                                    
     *       ZETA,                                                      
     *       ZDTHMIN,                                                   
     I INVB,                                                            
     R   ZCTLWP,                                                        
     *       ZQLOC,                                                     
     *       ZQMEC,                                                     
     *       ZQHIC,                                                     
     R   ZFSOT,                                                         
     #       ZFTET,                                                     
     #       ZFLT                                                       
      COMMON/COM36/                                                     
     #       ZFSO ,                                                     
     #       ZFTE ,                                                     
     #       ZFLB ,                                                     
     #       ZDTDT,                                                     
     #       HEATO,                                                     
     #       HEATE,                                                     
     #       HEATR,                                                     
     #       ZI0                                                        
C                                                                       
      INTEGER                                                           
C     I NRADFR,   NRPART,   NRINT,    NRLSTI,  NRADPFR,  NRADPLA,       
     I NRLST,    NRLEV,    NRLEVP1, NLEV, NLP2, NLEVP1,                 
     I NLON, IRLEV, NTRA, NLAT, ITIM, NUA, NINT, NRPARTI                
C                                                                       
C   NRLEV .... number of vertical levels                                
C   NRLST .... number of points on a latitude line                      
C   NUA   .... number of absorber amount                                
C   NINT  .... number of spectral intervals                             
C   NTRA  .... number of transmission functions                         
C   NLAT  .... number of latitude points                                
C   ITIM  .... number of month under consideration                      
C                                                                       
C      PARAMETER (NRLEV=122,NUA=34,NINT=6,NTRA=34, NLAT=1,              
      PARAMETER (NRLEV=21,NUA=34,NINT=6,NTRA=34, NLAT=1,                
cccm      PARAMETER (NRLEV=37,NUA=18,NINT=6,NTRA=19, NLAT=1,            
cccm     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=10,NLEV=NRLEV,        
     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=7,NLEV=NRLEV,             
     #           NLP2=NRLST, NLEVP1=NRLEVP1, NLON=NRLST,                
     #           NRPARTI=1)                                             
C                                                                       
      REAL YCLOCK, XLAT(NLAT), Z0, DZ, ALBMPAR, SNPAR,                  
     #     TSUPAR, CTFREEZ, DELTAC, DELTAO, DELTAH                      
C                                                                       
C   YTIME  .... time of the year in radians                             
C   YCLOCK .... time of the day in radians                              
C   XLAT   .... latitudes in radians                                    
C   Z0     .... zero point of height grid                               
C   DZ     .... Interval between height levels                          
C   ALBMPAR.... sw surface albedo as function of longitude:             
C           yearly averaged sea-ice: 0.55                               
C                           water  : 0.07                               
C                           land   : 0.07.lt.ALBM.lt.0.80               
C                                    depending on snow                  
C   SNPAR  .... snow depth in meter                                     
C   TSUPAR .... surface temperature (Kelvin)                            
C   CTFREEZ.... freezing temperature of sea-water (Kelvin)              
C   DELTAC .... Doppler parameter for line shapes of CO2                
C   DELTAO .... Doppler parameter for line shapes of O3                 
C                                                                       
      PARAMETER (Z0=0., CTFREEZ=271.26,                                 
C     PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
     #           DZ=0.289, ALBMPAR=0.2, SNPAR=0.,                       
cccm  PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
cccm     #           DZ=0.15, ALBMPAR=0.3, SNPAR=0.,                    
CZ     #           DELTAC=0., DELTAO=0.)                                
     #           DELTAC=6.0E-04, DELTAO=1.E-02, DELTAH=5.0E-04)         
C      PARAMETER (YCLOCK=0., Z0=0., TSUPAR=273., CTFREEZ=271.26,        
C     #           DZ=0.46875, ALBMPAR=0.07, SNPAR=0.)                   
C      PARAMETER (YCLOCK=0., Z0=2.608, TSUPAR=273., CTFREEZ=271.26,     
C     #           DZ=0.2878, ALBMPAR=0.07, SNPAR=0.)                    
C                                                                       
      LOGICAL LDIUR, LOLPAR                                             
C                                                                       
C   LDIUR .... true: sun at time, false: diurnally averaged             
C   LOLPAR.... true: land surface, false: else                          
C                                                                       
C      PARAMETER (LDIUR=.TRUE., LOLPAR=.TRUE.)                          
       PARAMETER (LOLPAR=.TRUE.)                                        
C                                                                       
      INTEGER  C2TEMP                                                   
      CHARACTER*64  MIDTMP                                              
cccm      PARAMETER (C2TEMP=10,                                         
cccm     #    MIDTMP='/home/atm/ugamp/midrad/radfildat.dir/MIDTMP.DAT') 
ccc1      PARAMETER (C2TEMP=10, MIDTMP='tprfil2.dat')                   
C                                                                       
      INTEGER  IRLONI, IRLST, IRLVP1, JPNLEV, JPNLVP1                   
C                                                                       
      PARAMETER (IRLONI=NRLST, IRLST=NRLST,                             
     #           IRLVP1=NRLEVP1, JPNLEV=NRLEV,                          
     #           JPNLVP1=NRLEVP1, IRLEV=NRLEV)                          
C                                                                       
      INTEGER NIPD, NIPD2, NTR, NG1, NG1P1                              
      PARAMETER(NIPD=8,NIPD2=16,NTR=11,NG1=2,NG1P1=3)                   
C                                                                       
C    definition of the functions                                        
C                                                                       
      REAL       CVMGT, SIGMA2                                          
      INTEGER    CVMGI                                                  
C                                                                       
C    definitons of the subroutine cloud                                 
C                                                                       
      REAL RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR                  
C                                                                       
C    missing definitions in morcrette                                   
C                                                                       
C      REAL   PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3            
      REAL ZCCO2                                                        
                                                                        
      LOGICAL LOH, LOM, LOL, LORAD                                      
C                                                                       
      INTEGER ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,           
     #        IZQS, IZCLC, IZQLWP, JLOOP, KRLEV,                        
     #        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     #        ITH, ITM, ITL,                                            
     #        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     #        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     #        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     #        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      INTEGER IND4, KM12, KXD, KXU, IG, KBS, KDD,                       
     #        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     #        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     #        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     #        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     #        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD1* SPECTRAL DISTRIBUTION OF AEROSOLS AND OZONE.     
C                     (TRIANGULAR *T10* TRUNCATIONFOR AEROSOLS).        
C                     (TRIANGULAR *T5* TRUNCATIONFOR OZONE).            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R           COZQC(21),COZQS(15),COZHC(21),COZHS(15)                
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD2* CONSTANTS INTERNAL TO THE RADIATION SUBROUTINES. 
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CSUPSAT                                                          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMCON* BASIC UNIVERSAL CONSTANTS AND DERIVED CONSTANTS.  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
C                                                                       
C                                                                       
C       --------------------------------------------------------------- 
C      PARAMETER(                                                       
C     I JPM=106,JPN=106,JPK=106,JPGL=160,JPNLON=320,                    
C     I JPT1MAX=400,          JPT2MAX=50,                               
C     I JPT3MAX=300,          JPT4MAX=50,                               
C     I JPHASH=2048,          JPNTRCRS=20,                              
C     I JPTASKS=4,     JPLOCKS=50,    JPEVENT=4,                        
C     I JPLTMT=JPTASKS,                                                 
C     I JPNUMW=352,            JPNUMP=352,                              
C     I JPTABLE=1024,                                                   
C     I JPMAXB =60,                                                     
C     I JPMAXPB=32,                                                     
C     I JPG3FLD=43,         JPG3XF=99,         JPG3CMP=17,              
C     I JPMP1=JPM+1,        JPNP1=JPN+1,       JPKP1=JPK+1,             
C     I JP2MP1=JPMP1*2,     JP4MP1=JPMP1*4,                             
C     I JPED=JPN/2+1,       JPOD=JPNP1/2,                               
C     I IPSP0=JPM+JPN-JPK,  IPSP1=JPNP1*JPMP1, IPSP2=IPSP0*(IPSP0+1)/2, 
C     I JPSP=IPSP1-IPSP2,   JP2SP=2*JPSP,                               
C     I JPHGL=JPGL/2,       JPNLP2=JPNLON+2,                            
C     I                     JPNVCLV=4*JPNLVP1)                          
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C     J. K. GIBSON     E.C.M.W.F.     22/01/82.                         
C     MODIFIED M.J     E.C.M.W.F.     13/12/82.                         
C                                                                       
      INTEGER                                                           
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      LOGICAL                                                           
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD        
      REAL                                                              
     R DTIME,    TWODT,    EPS                                          
C                                                                       
C      INTEGER                                                          
C     I NWTIME(JPNUMW),     NROW(JPTASKS),                              
C     I NPTIME(JPNUMP)                                                  
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMGAU* - QUANTITIES RELATED TO THE GAUSSIAN GRID.      
C                                                                       
      REAL                                                              
     R SINLON(NLP2),     COSLON(NLP2)                                   
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMTRU* -QUANTITIES RELATED TO THE SPECTRAL TRUNCATION. 
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSIM* - QUANTITIES NEEDED FOR THE SEMI-IMPLICIT SCHEME
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMHDI* - COEFFICIENTS FOR HORIZONTAL DIFFUSION.        
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMFFT* - QUANTITIES NEEDED FOR THE FAST *FOURIER       
C                         TRANSFORMS.                                   
C                                                                       
      INTEGER                                                           
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2                           
C      REAL                                                             
C     R TRIG(JPNLON)                                                    
      INTEGER                                                           
     I NFAX(10)                                                         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSTA* - ARRAYS AND PARAMETERS FOR STATISTICS.         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMDIA* - QUANTITIES RELATED TO GLOBAL PHYSICAL         
C                         DIAGNOSTICS.                                  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPSW* SWITCHES RELATED TO THE PARAMETERISATIONS OF      
C                     DIABATIC PROCESSES.                               
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMDSW* SWITCHES RELATED TO THE DYNAMICS                  
C                     AND THE GENERAL CONTROL OF THE FORECAST.          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH1* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE RADIATION SUBROUTINES.                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CRAE,                                                            
     R CDISSEM                                                          
C                                                                       
C      REAL       COSRAD(128),        SINRAD(128)                       
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH2* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE PHYSICAL SUBROUTINES (EXCEPT RADIATION    
C                     ONES).                                            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL       CCRH(JPNLEV)                                           
C                                                                       
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMDIZ* VARIABLES AND POINTERS FOR ZONAL MEAN DIAGNOSTICS 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMMSK* VARIABLES AND POINTERS FOR AREA MEAN DIAGNOSTICS. 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMSDS* - START/RESTART BLOCK.                          
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMTASK* - TASK RELATED VARIABLES.                      
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*         *COMRAD* - RADIATION POINTERS.                               
C                                                                       
      LOGICAL LO,LO1,LO2,LODIA,LODIAP                                   
C---------------------------------------------------------------------  
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C     COMMON *YOMAER* RADIATIVE CHARACTERISTICS OF THE AEROSOLS         
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
      REAL TAUA(2,5), PIZA(2,5), CGA(2,5), CAER(5,5), CAEROS            
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C     COMMON *YOMLW*                                                    
C                                                                       
C*    COEFFICIENTS FOR THE LONGWAVE RADIATION SUBROUTINES               
C                                                                       
C NINT  :  NUMBER OF SPECTRAL INTERVALS                                 
C NIPD  :  NUMBER OF BANDS DESCRIBED BY PADE APPROXIMANTS               
C NTR   :  NUMBER OF REFERENCE TEMPERATURES                             
C NTRA  :  NUMBER OF TRANSMISSION FUNCTIONS                             
C NUA   :  NUMBER OF ABSORBER AMOUNTS                                   
C NG1   :  ORDER OF THE GAUSSIAN QUADRATURE                             
C                                                                       
C RT1   :  ROOTS OF THE GAUSSIAN POLYNOMIAL                             
C WG1   :  WEIGHTS OF THE GAUSSIAN POLYNOMIAL                           
C AT, BT:  COEFFICIENTS OF THE TEMPERATURE DEPENDENCE                   
C CNTNU :  H2O CONTINUUM ABSORPTION COEFFICIENTS                        
C O1H,O2H,PIALF0,OCT: OZONE LONGWAVE ABSORPTION COEFFICIENTS            
C TREF, TSTAND      : REFERENCE TEMPERATURES                            
C XP    :  COEFFICIENTS OF POLYNOMIAL DEVELOPMENTS OF PLANCK FUNCTION   
C GA,GB :  PADE APPROXIMANTS OF H2O, CO2 TRANSMISSION FUNCTIONS         
C ETY,PTY :  PARAMETERS OF H2O CONTINUUM                                
C TINTP :  REFERENCE TEMPERATURES OF PRE-COMPUTED TRANSMISSIVITIES      
C CE    :  EMS COEFFS OF H2O TRANSMISSION FUNCTIONS                     
C                                                                       
      REAL  RT1(2),WG1(2),AT(12,3),BT(12,3),CNTNU(4),O1H                
     S ,OCT(4),TREF,XP(6,6),TSTAND,GA(11,24,3),GB(11,24,3)              
     S ,PTY(11,20),ETY(11,20),TINTP(11),TSTP,O2H,PIALF0                 
C                                                                       
      REAL  TB(40,31,2),ALP(40,31,2),BET(40,31,2),                      
     S ULG(40),PLG(31),AP(40,31,2),BP(40,31,2),TBO3(38,31),TO3B(38,31)  
     S,ULGO(38),AO3(38,31),BO3(38,31),O3A(38,31),O3B(38,31)             
C     ----------------------------------------------------------------- 
C*COMDECK YOMRDU                                                        
C     ----------------------------------------------------------------- 
C                                                                       
C*    CONTROL VARIABLES FOR RADIATION: INTERNAL SWITCH AND INDICES      
C                                                                       
C ZEELOG:  SECURITY THRESHOLD FOR ABSORBER AMOUNT IN LAPLACE TRANSFORM  
C ZEPSC :  SECURITY THRESHOLD FOR CLOUD COVER                           
C ZEPSCO:  SECURITY THRESHOLD FOR OZONE AMOUNT                          
C ZEPSCQ:  SECURITY THRESHOLD FOR WATER VAPOR                           
C ZEPSCT:  SECURITY THRESHOLD FOR SHORTWAVE OPTICAL THICKNESS           
C ZEPSCW:  SECURITY THRESHOLD FOR CLOUD LIQUID WATER PATH               
C                                                                       
      REAL          CDAY,CCO2,CH2O,DIFF,X10E                            
     S             ,ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW            
C     ----------------------------------------------------------------- 
C*COMDECK YOMSW                                                         
C     ----------------------------------------------------------------- 
C                                                                       
C*    COEFFICIENTS FOR THE SHORTWAVE RADIATION SUBROUTINES              
C                                                                       
C APAD  :  PADE APPROXIMANTS NUMERATOR                                  
C BPAD  :  PADE APPROXIMANTS DENOMINATOR                                
C D     :  TRANSMISSION LIMIT FOR INFINITE ABSORBER AMOUNT              
C CRAY  :  RAYLEIGH SCATTERING COEFFICIENTS                             
C SUN   :  SOLAR FRACTION IN SPECTRAL INTERVALS                         
C                                                                       
      REAL  APAD(2,3,7),BPAD(2,3,7),D(2,3),CRAY(2,6),SUN(2)             
C     ----------------------------------------------------------------- 
                                                                        
C                                                                       
C  -------------------------------------------------------------------- 
C                                                                       
      REAL   TTT(1,NLAT,NRLEV,12)                                       
C                                                                       
C        0.1  "RADINT POINTERS"                                         
C             -----------------                                         
C                                                                       
      REAL   TM1(NLP2,NLEV),                                            
     *       TSM1M(NLP2),                                               
     *       QM1(NLP2,NLEV),                                            
     *       SNM1M(NLP2),                                               
     *       ALBM(NLP2),                                                
     *       AVVELM(NLP2)                                               
      REAL   SLMM(NLP2),                                                
     *       APHM1(NLP2,NLEVP1),                                        
     *       APM1(NLP2,NLEV)                                            
C                                                                       
      LOGICAL LOLAND(NLP2)                                              
C                                                                       
C     VARIABLES FOR FULL GRID AND TRANSFORMS.                           
C                                                                       
      REAL   ZTI(NLP2,NRLEVP1),                                         
     *       ZCTI(NLP2,NRLEVP1),                                        
     *       ZCPH(NLP2,NRLEVP1),                                        
     *       ZWV(NLP2,NRLEV),                                           
     *       ZSW(NLP2,NRLEV),                                           
     *       ZALTE(NLP2),                                               
     *       ZCALTE(NLP2),                                              
     *       ZALSO(NLP2),                                               
     *       ZCALSO(NLP2)                                               
C                                                                       
      REAL   ZOZQ(NLP2),                                                
     #       ZOZH(NLP2)                                                 
C                                                                       
      REAL   ZTRSOL(NLP2,NLEVP1),                                       
     *       ZEMTER(NLP2,NLEVP1)                                        
C                                                                       
C     FINAL RESULTS ARE STORED ON:                                      
C                                                                       
      REAL   TRSOLM(NLP2,NLEVP1),                                       
     *       EMTERM(NLP2,NLEVP1)                                        
C                                                                       
      REAL   AMU0(NLP2),                                                
     *       RDAYL(NLP2)                                                
C                                                                       
C     VARIABLES WITHIN THE SPLITTED LATITUDE LOOP.                      
C                                                                       
      REAL   CLC(NRLST,NRLEV),                                          
     *       TI(NRLST,NRLEVP1),                                         
     *       DP(NRLST,NRLEV),                                           
     *       AADS(NRLST,NRLEV),                                         
     *       AEQ1(NRLST,NRLEV),                                         
     *       AEQ2(NRLST,NRLEV),                                         
     *       AEQ3(NRLST,NRLEV),                                         
     *       AEQ4(NRLST,NRLEV),                                         
     *       AEQ5(NRLST,NRLEV),                                         
     *       SSW(NRLST,NRLEV),                                          
     *       WV(NRLST,NRLEV),                                           
     *       QOF(NRLST,NRLEV),                                          
     *       QCF(NRLST,NRLEV),                                          
     *       CLWA(NRLST,NRLEV),                                         
     *       APRE(NRLST),                                               
     *       SMU0(NRLST),                                               
     *       ALTE(NRLST),                                               
     *       ALSO(NRLST)                                                
      REAL   ZDPO(NRLST),                                               
     *       ZQCFO(NRLST),                                              
     *       ZAEQSO(NRLST),                                             
     *       ZQOFO(NRLST),                                              
     *       ZDPN(NRLST),                                               
     *       ZQCFN(NRLST),                                              
     *       ZAEQSN(NRLST),                                             
     *       ZQOFN(NRLST)                                               
      REAL   FLT(NRLST,NRLEVP1),                                        
     *       FLS(NRLST,NRLEVP1)                                         
C                                                                       
C     DIAGNOSTIC VARIABLES.                                             
C                                                                       
      REAL   DIAG(NLEVP1,4),                                            
     *       DIA1(7),                                                   
     *       ZDIAG(NLEVP1,10),                                          
     *       ZDIA1(21),                                                 
     *       ZDIA(NLON),                                                
     *       ZDIAF(NLON,7),                                             
     *       ZDIAT(NLEVP1)                                              
C                                                                       
C                                                                       
C                                                                       
      REAL PGG(IRLST),                                                  
     #     PREF(IRLST),                                                 
     #     PRE1(IRLST),                                                 
     #     PRE2(IRLST),                                                 
     #     PRMUZ(IRLST),                                                
     #     PTO1(IRLST),                                                 
     #     PTR1(IRLST),                                                 
     #     PTR2(IRLST),                                                 
     #     PW(IRLST)                                                    
C                                                                       
C                                                                       
C  what about the 1-dimensional PALBS???                                
C                                                                       
      REAL     PALBSN  (IRLST)                                          
C                                                                       
C*       0.2   "LOCAL" ARRAYS                                           
C              --------------                                           
      REAL ZABSCU(IRLST ,NUA, 3*NRLEV+1),                               
     S     ZABSLY(IRLST ,NUA, 3*NRLEV+1) ,                              
     S     ZADJD (IRLST , IRLVP1)      ,                                
     S     ZADJU (IRLST , IRLVP1)       ,                               
     S     ZBGND (IRLST)                ,                               
     S     ZBINT (IRLST , IRLVP1)       ,                               
     S     ZBSUR (IRLST , NINT)         ,                               
     S     ZBSUIN(IRLST)                ,                               
     S     ZBTOP (IRLST , NINT)         ,                               
     S     PCLDLW(IRLST , IRLEV)        ,                               
     S     ZCLOUD(IRLST)                ,                               
     S     ZCNTRB(IRLST , IRLVP1,IRLVP1),                               
     S     ZCUMP (IRLST)                ,                               
     S     ZCUMPT(IRLST)                ,                               
     S     ZDBDT (IRLST , NINT , IRLEV) ,                               
     S     ZDBSL (IRLST , NINT , 2*NRLEV),                              
     S     ZDISTD(IRLST , IRLVP1)       ,                               
     S     ZDISTU(IRLST , IRLVP1)                                       
      REAL                                                              
     S     ZDNF  (IRLST , IRLVP1,IRLVP1),                               
     S     PDT0  (IRLST)                                                
      REAL                                                              
     S     ZDUC  (IRLST , 3*NRLEV+1)       ,                            
     S     ZEFFT (IRLST)                ,                               
     S     ZEMD  (IRLST , IRLEV), ZEMU  (IRLST , IRLEV),                
     S     ZEPSC2(IRLST),  ZEPSC3(IRLST),                               
     S     ZEPSH1(IRLST),  ZEPSH2(IRLST),                               
     S     ZEPSH3(IRLST),  ZEPSH4(IRLST),                               
     S     ZEPSH5(IRLST),  ZEPSH6(IRLST),                               
     S     ZEPHIO(IRLST),  ZFACTC(IRLST),                               
     S     ZZFD(IRLST), ZFDD(IRLST), ZFDN(IRLST , IRLVP1)               
      REAL                                                              
     S     PFLUX (IRLST , 2 , IRLVP1)   ,                               
     S     PFLUC (IRLST , 2 , IRLVP1)   ,                               
     S     ZFLWP (IRLST)                ,                               
     S     PCTS  (IRLST , IRLEV)        ,                               
     S     ZZFU   (IRLST)               ,                               
     S     ZFUP  (IRLST , IRLVP1)       ,                               
     S     ZGLAYD(IRLST)                ,                               
     S     ZGLAYU(IRLST)                ,                               
     S     ZOMEGA(IRLST , 2 , IRLEV)                                    
C                                                                       
      INTEGER                                                           
     S     ITX   (IRLST)                ,                               
     S     IXDIA (IRLST , IRLVP1)       ,                               
     S     IXT   (IRLST , IRLVP1,IRLVP1),                               
     S     IXTSU (IRLST , IRLVP1)       ,                               
     S     IXTTP (IRLST , IRLVP1)       ,                               
     S     IMX   (IRLST)                ,                               
     S     IMXP  (IRLST)                                                
C                                                                       
      REAL ZOZ   (IRLST , IRLEV)        ,                               
     S     PPMB  (IRLST , IRLVP1)       ,                               
     S     ZRES  (IRLST)                ,                               
     S     PB    (IRLST, 6 , IRLVP1)    ,                               
     S     ZSSIG (IRLST , 3*NRLEV+1)    ,                               
     S     PTAU  (IRLST , 2 , IRLEV)    ,                               
     S     PTAVE (IRLST , IRLEV)        ,                               
     S     ZTAVIC(IRLST)                ,                               
     S     ZTI1  (IRLST)                ,                               
     S     PTL   (IRLST , IRLVP1)       ,                               
     S     ZTT   (IRLST , NTRA)         ,                               
     S     ZTT1  (IRLST , NTRA)         ,                               
     S     ZTT2  (IRLST , NTRA)         ,                               
     S     ZUAER (IRLST , NINT)         ,                               
     S     ZUPF  (IRLST , IRLVP1,IRLVP1),                               
     S     ZUU   (IRLST , NUA)          ,                               
     S     ZXD   (IRLST)                                                
      REAL                                                              
     S     ZXN   (IRLST)                ,                               
     S     ZXOZ  (IRLST)                ,                               
     S     ZXWV  (IRLST)                ,                               
     S     ZZZ   (IRLST)                ,                               
     S     ZRES2 (IRLST)                ,                               
     S     ZTI2  (IRLST)                ,                               
     S     ZBLAY (IRLST,IRLEV)          ,                               
     S     ZBLEV (IRLST,IRLVP1)         ,                               
     S     ZCTS  (IRLST)                ,                               
     S     ZHFG  (IRLST)                                                
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C                                                                       
      INTEGER KT    (IRLST)                                             
      REAL    PUU   (IRLST , NUA  )     ,                               
     S        PTT   (IRLST , NTRA )     ,                               
     S        ZZ    (IRLST)                                             
C                                                                       
C     -------------------------------------------------------- ---------
C                                                                       
      REAL  PCLFR  (IRLST , IRLEV)       ,                              
     S      PT     (IRLST , IRLVP1)       ,                             
     S      PQOF   (IRLST , IRLEV)          ,                           
     S      PQLWP  (IRLST , IRLEV)          ,                           
     S      PEMIS  (IRLST)                                              
      REAL                                                              
     S      PFLT   (IRLST , IRLVP1)                                     
C                                                                       
      REAL  ZALBSU(IRLST , 2 )           ,                              
     S      ZCG   (IRLST , 2 , IRLEV)    ,                              
     S      ZCLDLW(IRLST , IRLEV)        ,                              
     S      ZCLDSW(IRLST , IRLEV)        ,                              
     S      ZCOOLR(IRLST , IRLEV)        ,                              
     S      ZDT0  (IRLST)                ,                              
     S      ZFLUX (IRLST , 2 , IRLVP1)   ,                              
     S      ZPMB  (IRLST , IRLVP1)       ,                              
     S      ZTAU  (IRLST , 2 , IRLEV)    ,                              
     S      ZTAVE (IRLST , IRLEV)        ,                              
     S      ZTL   (IRLST , IRLVP1)       ,                              
     S      ZFSDWN(IRLST , IRLVP1)       ,                              
     S      ZFSUP (IRLST , IRLVP1)       ,                              
     S      ZHEATR(IRLST , IRLEV )                                      
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C                                                                       
      REAL   PDP   (IRLST , IRLEV)     ,                                
     S       PWV   (IRLST , IRLEV)     ,                                
     S       PFLS  (IRLST , IRLVP1)    ,                                
     S       PPSOL (IRLST        )     ,                                
     S       PRMU0 (IRLST        )     ,                                
     S       PAER  (IRLST,IRLEV,5)     ,                                
     S       PALBS (IRLST , 2 )        ,                                
     S       PCG   (IRLST , 2 , IRLEV) ,                                
     S       PCLDSW(IRLST , IRLEV)     ,                                
     S       POMEGA(IRLST , 2 , IRLEV) ,                                
     S       POZ   (IRLST , IRLEV)                                      
C                                                                       
      REAL   PFDOWN(IRLST , IRLVP1)    ,                                
     S       PFUP  (IRLST , IRLVP1)                                     
C                                                                       
      REAL   ZAKI  (IRLST , 2 )        ,                                
     S       ZCGAZ (IRLST , IRLEV)     ,                                
     S       ZCNEB (IRLST , IRLEV)     ,                                
     S       ZC1I  (IRLST , IRLVP1)    ,                                
     S       ZDSC  (IRLST , IRLEV)     ,                                
     S       ZDSH  (IRLST , IRLEV)     ,                                
     S       ZDSIG (IRLST , IRLEV)     ,                                
     S       ZFACT (IRLST)             ,                                
     S       ZFACT1 (IRLST), ZFACT2 (IRLST),                            
     S       ZFACT3 (IRLST), ZFACT4 (IRLST),                            
     S       ZFACT5 (IRLST), ZFACT6 (IRLST),                            
     S       ZFD   (IRLST , IRLVP1)    ,                                
     S       ZFRAC (IRLST)             ,                                
     S       ZFU   (IRLST , IRLVP1)    ,                                
     S       ZG    (IRLST)             ,                                
     S       ZGG   (IRLST)             ,                                
     S       ZPIZAZ(IRLST , IRLEV)     ,                                
     S       ZRAYL (IRLST),  ZRAY1 (IRLST , IRLVP1)                     
      REAL                                                              
     S       ZRAY2 (IRLST , IRLVP1)    ,                                
     S       ZREF  (IRLST)             ,                                
     S       ZREFZ (IRLST , 2 , IRLVP1),                                
     S       ZRE1  (IRLST)             ,                                
     S       ZRE2  (IRLST)             ,                                
     S       ZRJ   (IRLST , 6 , IRLVP1),                                
     S       ZRK   (IRLST , 6 , IRLVP1),                                
     S       ZRL   (IRLST , 8)         ,                                
     S       ZRMU  (IRLST)             ,                                
     S       ZRMUE (IRLST , IRLVP1)    ,                                
     S       ZRMUZ (IRLST)                                              
      REAL                                                              
     S       ZRNEB (IRLST)             ,                                
     S       ZRUEF (IRLST , 8)         ,                                
     S       ZR1   (IRLST)             ,                                
     S       ZR2   (IRLST)             ,                                
     S       ZR21  (IRLST)             ,                                
     S       ZR22  (IRLST)             ,                                
     S       ZR23  (IRLST)             ,                                
     S       ZS    (IRLST)             ,                                
     S       ZSEC  (IRLST)             ,                                
     S       ZSS1  (IRLST)             ,                                
     S       ZTAUAZ(IRLST , IRLEV)     ,                                
     S       ZTOT  (IRLST , IRLVP1)    ,                                
     S       ZTO1  (IRLST)             ,                                
     S       ZTR   (IRLST , 2 , IRLVP1),                                
     S       ZTRA1 (IRLST , IRLVP1)    ,                                
     S       ZTRA2 (IRLST , IRLVP1)                                     
      REAL                                                              
     S       ZTR1  (IRLST)             ,                                
     S       ZTR2  (IRLST)             ,                                
     S       ZUD   (IRLST , 3 , IRLVP1),                                
     S       ZUM   (IRLST , IRLVP1)    ,                                
     S       ZU1D  (IRLST)             ,                                
     S       ZU2D  (IRLST)             ,                                
     S       ZW    (IRLST)             ,                                
     S       ZN175 (IRLST)             ,                                
     S       ZN190 (IRLST)             ,                                
     S       ZO175 (IRLST)             ,                                
     S       ZO190 (IRLST)             ,                                
     S       ZP75  (IRLST)             ,                                
     S       ZP90  (IRLST)             ,                                
     S       ZSIGN (IRLST)             ,                                
     S       ZSIGO (IRLST)                                              
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
      REAL   PTR    (IRLST)     ,                                       
     S       PU     (IRLST)                                             
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
C   CLOUD-POINTERS                                                      
C   --------------                                                      
C                                                                       
      REAL   ZRH(NLP2,NLEV),                                            
     *       ZCH(NLP2),                                                 
     *       ZCM(NLP2),                                                 
     *       ZCL(NLP2),                                                 
     *       ZCT(NLP2),                                                 
     *       ZCC(NLP2)                                                  
      INTEGER NTOPCM(NLP2),                                             
     *        NBASECM(NLP2)                                             
      REAL   ARPRCM(NLP2)                                               
      INTEGER ITOPH(NLP2),                                              
     *        ITOPM(NLP2),                                              
     *        ITOPL(NLP2)                                               
      LOGICAL LOC(NLP2)                                                 
      REAL   ZQS(NLP2,NLEV),                                            
     *       ZCLC(NLP2,NLEV),                                           
     *       ZQLWP(NLP2,NLEV)                                           
C                                                                       
      REAL   VERVEL(NLP2,NLEV),                                         
     *       ZTHETA(NLP2,NLEV),                                         
     *       ZETA(NLP2,NLEV),                                           
     *       ZDTHMIN(NLP2)                                              
      INTEGER INVB(NLP2)                                                
      REAL   ZCTLWP(NLP2),                                              
     *       ZQLOC(NLP2),                                               
     *       ZQMEC(NLP2),                                               
     *       ZQHIC(NLP2)                                                
C                                                                       
C  definitions of "radheat"                                             
C                                                                       
      REAL   ZFSOT(NLON),                                               
     #       ZFTET(NLON),                                               
     #       ZFLT (NLON),                                               
     #       ZFSO (NLON),                                               
     #       ZFTE (NLON),                                               
     #       ZFLB (NLON),                                               
     #       ZDTDT(NLON),                                               
     #       HEATO(NLON,NLEV),                                          
     #       HEATE(NLON,NLEV),                                          
     #       HEATR(NLON,NLEV),                                          
     #       ZI0  (NLON)                                                
C                                                                       
C PIERS 13-8-97                                                         
C                                                                       
         REAL PIERSLWP                                                  
C                                                                       
C Piece of code needed to replace the Nikos radiation scheme with       
C the Morcrette radiation scheme in the igcm3.1.                        
C See morcrette3_1_1.readme for details.                                
C                                                                       
C                                                                       
c      program lh                                                       
c                                                                       
c      parameter(NL=23)                                                 
c      real P(NL),t(NL),h2o(NL),o3(NL),htlw(NL),htsw(NL)                
c      real swalb,doy                                                   
c       REAL CF(4,2)     !cloud fractions,lwps                          
c       INTEGER IC(4,2)   ! cloud positions                             
c       REAL RFLUXES(2,2,2)                                             
c      open(11,file='mls23.dat')                                        
c      read(11,*) NLEV                                                  
c      DOY=80                                                           
c      swalb=0.07                                                       
c      alat=40.0                                                        
c       doy=float(i)                                                    
c      do i=nlev,1,-1                                                   
c         read(11,*) P(i),t(i),h2o(i),adum,o3(i)                        
c         print *,i,p(i),t(i),h2o(i),o3(i)                              
c      enddo                                                            
c                                                                       
c       ic(1,1)=3                                                       
c      ic(1,2)=17                                                       
c       ic(3,1)=6                                                       
c       ic(2,2)=6                                                       
c       cf(1,1)=0.3                                                     
c       cf(1,2)=45.0                                                    
c        cf(3,1)=0.8                                                    
c       cf(4,2)=20.0                                                    
c      alon=0.0                                                         
c      call morcigcm(p,t,h2o,o3,alat,htlw,htsw,doy,cf,ic,               
c     $     rfluxes,swalb,alon)                                         
c      enddo                                                            
c      do i=nl,1,-1                                                     
c         print '(i6,4F14.4)',i,p(i),t(i),htsw(i),htlw(i)               
c      enddo                                                            
c      print *,RFLUXES(1,1,1), RFLUXES(1,1,2), RFLUXES(1,2,1),          
c     $     RFLUXES(1,2,2)                                              
c      print *,RFLUXES(2,1,1), RFLUXES(2,1,2), RFLUXES(2,2,1),          
c     $     RFLUXES(2,2,2)                                              
c                                                                       
c                                                                       
c      end                                                              
c      INCLUDE 'zparb.upd'                                              
C     COMMON BLOCK FOR GAS ABUNDANCES                                   
!-----------------------------------------------------                  
! Common block for gas abundances                                       
!-----------------------------------------------------                  
      COMMON/MORCGAS/VMRCO2,ZCARDI,VMRCH4,RCH4,VMRN2O,RN2O,             
     &               VMRCFC11,RCFC11,VMRCFC12,RCFC12,                   
     &               VMRH2OS,RH2OS,NEXOTIC                              
      LOGICAL :: NEXOTIC                                                
      REAL :: ZCARDI,RCH4,RN2O,RCFC11,RCFC12,RH2OS                      
      REAL :: VMRCO2,VMRCH4,VMRN2O,VMRCFC11,VMRCFC12,VMRH2OS            
      REAL PGGA(KDLON,12,2),PGGB(KDLON,12,2),                           
     *                 PPPTY(KDLON,12), PPETY(KDLON,12)                 
                                                                        
      DO 97 JL=1,NRLST                                                  
         DO 98 II=1,NUA                                                 
            PUU(JL,II) = ZUU(JL,II)                                     
 98      CONTINUE                                                       
         DO 99 JJ=1,NTRA                                                
            PTT(JL,JJ) = ZTT(JL,JJ)                                     
 99      CONTINUE                                                       
 97   CONTINUE                                                          
C                                                                       
C*         1.     HORNER'S ALGORITHM FOR H2O AND CO2 TRANSMISSION       
C                 -----------------------------------------------       
C                                                                       
 100  CONTINUE                                                          
                                                                        
! changing - marc 30/3/10                                               
c      DO 101 JL = 1 , 6*KDLON                                          
c        ZZ(JL) = SQRT (PUU(JL,1))                                      
c        ZXN(JL) = PGGA(JL,1,1) + ZZ(JL)*(PGGA(JL,1,2))                 
c        ZXD(JL) = PGGB(JL,1,1) + ZZ(JL)*(PGGB(JL,1,2)+ZZ(JL))          
c        PTT(JL,1) = ZXN(JL) / ZXD(JL)                                  
c 101  CONTINUE                                                         
      DO I=1,6                                                          
         DO JL=1,KDLON                                                  
            ZZ(JL) = SQRT (PUU(JL,i))                                   
            ZXN(JL) = PGGA(JL,i,1) + ZZ(JL)*(PGGA(JL,i,2))              
            ZXD(JL) = PGGB(JL,i,1) + ZZ(JL)*(PGGB(JL,i,2)+ZZ(JL))       
            PTT(JL,i) = ZXN(JL) / ZXD(JL)                               
         ENDDO                                                          
      ENDDO                                                             
                                                                        
      DO 1003 JL = 1 , KDLON                                            
C                                                                       
        ZZ(JL) = SQRT (PUU(JL,8))                                       
        ZXN(JL) = PGGA(JL,8,1) + ZZ(JL)*(PGGA(JL,8,2))                  
        ZXD(JL) = PGGB(JL,8,1) + ZZ(JL)*(PGGB(JL,8,2)+ZZ(JL))           
        PTT(JL,8) = ZXN(JL)/ZXD(JL)                                     
C                                                                       
C  1250-1450 CM-1 H2O                                                   
C                                                                       
        ZZ(JL) = SQRT (PUU(JL,29))                                      
        ZXN(JL)= PGGA(JL,9,1) + ZZ(JL)*(PGGA(JL,9,2) )                  
        ZXD(JL)= PGGB(JL,9,1) + ZZ(JL)*(PGGB(JL,9,2) + ZZ(JL))          
        PTT(JL,16) = ZXN(JL)/ZXD(JL)                                    
C                                                                       
C  800-970 CM-1 H2O                                                     
C                                                                       
        ZZ(JL) = SQRT (PUU(JL,30))                                      
        ZXN(JL)= PGGA(JL,10,1) + ZZ(JL)*(PGGA(JL,10,2) )                
        ZXD(JL)= PGGB(JL,10,1) + ZZ(JL)*(PGGB(JL,10,2) + ZZ(JL))        
        PTT(JL,17) = ZXN(JL)/ZXD(JL)                                    
C                                                                       
C  650-800 CM-1 H2O                                                     
C                                                                       
        ZZ(JL) = SQRT (PUU(JL,31))                                      
        ZXN(JL)= PGGA(JL,11,1) + ZZ(JL)*(PGGA(JL,11,2) )                
        ZXD(JL)= PGGB(JL,11,1) + ZZ(JL)*(PGGB(JL,11,2) + ZZ(JL))        
        PTT(JL,18) = ZXN(JL)/ZXD(JL)                                    
1003  CONTINUE                                                          
C     -----------------------------------------------------------       
C                                                                       
C*         1.1    2-D INTERPOLATION BETWEEN LBL CO2 TRANSMISSION        
C                 -----------------------------------------------       
C                                                                       
      DO 1011 JL = 1 , KDLON                                            
        CGU  = LOG(PUU(JL,34))                                          
        CGP  = LOG(PUU(JL,7)/PUU(JL,34))                                
        CGT  = PUU(JL,33)/PUU(JL,34)                                    
        DTX  = CGT-TREF                                                 
        JU   =INT((CGU+13.)*2.)+1                                       
        JP   =INT((CGP+8.)*2.)+1                                        
        XIU  = (CGU - ULG(JU))*2.0                                      
        XIP  = (CGP - PLG(JP))*2.0                                      
        JP1  = JP+1                                                     
        JU1  = JU+1                                                     
        XUP0 = XIU * XIP                                                
        XUP1 = 1.0 - XIP -XIU + XUP0                                    
        XUP2 = XIU - XUP0                                               
        XUP3 = XIP - XUP0                                               
        TBUP = XUP1 * TB (  JU ,   JP ,   1)                            
     S       + XUP2 * TB (  JU1,   JP ,   1)                            
     S       + XUP3 * TB (  JU ,   JP1,   1)                            
     S       + XUP0 * TB (  JU1,   JP1,   1)                            
        ALUP = XUP1 * ALP(  JU ,   JP,    1)                            
     S       + XUP2 * ALP(  JU1,   JP,    1)                            
     S       + XUP3 * ALP(  JU ,   JP1,   1)                            
     S       + XUP0 * ALP(  JU1,   JP1,   1)                            
        BTUP = XUP1 * BET(  JU ,   JP,    1)                            
     S       + XUP2 * BET(  JU1,   JP,    1)                            
     S       + XUP3 * BET(  JU ,   JP1,   1)                            
     S       + XUP0 * BET(  JU1,   JP1,   1)                            
        TTRR=1.0-EXP(-TBUP/1000.)                                       
        PTT(JL,7)=TTRR - (1.-TTRR)*DTX*(ALUP + BTUP*DTX)                
 1011 CONTINUE                                                          
C                                                                       
C*         1.2    2-D INTERPOLATION BETWEEN LBL O3 TRANSMISSION         
C                 -----------------------------------------------       
C                                                                       
      DO 1021 JL = 1 , KDLON                                            
        CGU  = MAX( LOG(PUU(JL,12)), -17.5)                             
        CGP  = MAX( LOG(PUU(JL,13)/PUU(JL,12)), -8.0)                   
        CGT  = PUU(JL,32)/PUU(JL,12)                                    
        DTX  = CGT-TREF                                                 
        JU   = INT((CGU+17.5)*2.)+1.                                    
        JP   = INT((CGP+8.)*2.)+1.                                      
        XIU  = (CGU - ULGO(JU))*2.0                                     
        XIP  = (CGP - PLG (JP))*2.0                                     
        JU1  = JU+1                                                     
        JP1  = JP+1                                                     
        XUP0 = XIU * XIP                                                
        XUP1 = 1.0 - XIP -XIU + XUP0                                    
        XUP2 = XIU - XUP0                                               
        XUP3 = XIP - XUP0                                               
        TBUP = XUP1 * TBO3( JU,   JP )                                  
     S       + XUP2 * TBO3( JU1,  JP )                                  
     S       + XUP3 * TBO3( JU,   JP1)                                  
     S       + XUP0 * TBO3( JU1,  JP1)                                  
        ALUP = XUP1 * AO3(  JU,   JP )                                  
     S       + XUP2 * AO3(  JU1,  JP )                                  
     S       + XUP3 * AO3(  JU,   JP1)                                  
     S       + XUP0 * AO3(  JU1,  JP1)                                  
        BTUP = XUP1 * BO3(  JU,   JP )                                  
     S       + XUP2 * BO3(  JU1,  JP )                                  
     S       + XUP3 * BO3(  JU,   JP1)                                  
     S       + XUP0 * BO3(  JU1,  JP1)                                  
        TBUP2= XUP1 * TO3B( JU,   JP )                                  
     S       + XUP2 * TO3B( JU1,  JP )                                  
     S       + XUP3 * TO3B( JU,   JP1)                                  
     S       + XUP0 * TO3B( JU1,  JP1)                                  
        ALUP2= XUP1 * O3A(  JU,   JP )                                  
     S       + XUP2 * O3A(  JU1,  JP )                                  
     S       + XUP3 * O3A(  JU,   JP1)                                  
     S       + XUP0 * O3A(  JU1,  JP1)                                  
        BTUP2= XUP1 * O3B(  JU,   JP )                                  
     S       + XUP2 * O3B(  JU1,  JP )                                  
     S       + XUP3 * O3B(  JU,   JP1)                                  
     S       + XUP0 * O3B(  JU1,  JP1)                                  
        TTRR=1.0-EXP(-TBUP/100.)                                        
        PTT(JL,13)=TTRR - (1.-TTRR)*DTX*(ALUP + BTUP*DTX)               
        TTRR2=1.0-EXP(-TBUP2/100.)                                      
        PTT(JL,16)=TTRR2 - (1.-TTRR2)*DTX*(ALUP2 + BTUP2*DTX)           
 1021 CONTINUE                                                          
C     -----------------------------------------------------             
C                                                                       
 200  CONTINUE                                                          
                                                                        
      DO 201 JL = 1 , KDLON                                             
        PTT(JL, 9) = PTT(JL, 8)                                         
        PTT(JL, 7) = PTT(JL, 7)*PTT(JL,16)                              
C                                                                       
C-  CONTINUUM ABSORPTION: E- AND P-TYPE                                 
C                                                                       
        ZPU   = PUU(JL,10)                                              
        ZPU2  = ZPU*ZPU                                                 
        ZPU10 = PPPTY(JL,1)*(ZPU+0.3*ZPU2)/(1.+9.*ZPU)                  
        ZPU11 = 0.7*PPPTY(JL,2)*(ZPU+0.34*ZPU2)/(1.+2.*ZPU)             
        ZPU12A= PPPTY(JL,3)*(ZPU+0.066*ZPU2)/(1.+0.27*ZPU)              
        ZPU12B= PPPTY(JL,4)*(ZPU+0.057*ZPU2)/(1.+0.095*ZPU)             
        ZPU13 = 0.017 * ZPU                                             
        ZPU14 = 0.041 * ZPU                                             
        ZPU12 = 0.017 * ZPU                                             
        ZPU16 = PPPTY(JL,8)*(ZPU+0.19*ZPU2)/(1.+0.85*ZPU)               
        ZPU17 = PPPTY(JL,9)*(ZPU+0.035*ZPU2)/(1.+0.71*ZPU)              
                                                                        
        ZEUB  = PUU(JL,11)                                              
        ZEUB2 = ZEUB*ZEUB                                               
        ZEUC  = PUU(JL,25)                                              
        ZEUD  = PUU(JL,26)                                              
        ZEUE  = PUU(JL,27)                                              
        ZEUF  = PUU(JL,28)                                              
                                                                        
        ZEU10 = PPETY(JL,1)*(ZEUB+5.*ZEUB2)/(1.+150.*ZEUB)              
        ZEU11 = PPETY(JL,2)*(ZEUB+15.5*ZEUB2)/(1.+42.*ZEUB)             
        ZEU12A= PPETY(JL,3)*(1.+7.*ZEUC)/(1.+17.5*ZEUC)*ZEUC            
        ZEU12B= PPETY(JL,4)*(1.+4.2*ZEUD)/(1.+9.5*ZEUD)*ZEUD            
        ZEU13 = PPETY(JL,5)*(1.+3.8*ZEUE)/(1.+7.1*ZEUE)*ZEUE            
        ZEU14 = 6.5 * ZEUC                                              
        ZEU12 = 7.0 * ZEUD                                              
        ZEU16 = PPETY(JL,8)*(ZEUB+4.3*ZEUB2)/(1.+11.8*ZEUB)             
        ZEU17 = PPETY(JL,9)*(1.+1.*ZEUF)/(1.+9.*ZEUF)*ZEUF              
C                                                                       
C                                                                       
C     ---------------------------------------------------------         
C                                                                       
C*         2     OZONE AND AEROSOL TRANSMISSION FUNCTIONS               
C                 -----------------------------------------             
C                                                                       
C-  OZONE ABSORPTION                                                    
C                                                                       
c        ZX = MAX(1.0E-15, PUU(JL,12))                                  
c        ZY = MAX(1.0E-15, PUU(JL,13))                                  
c        ZUXY = 4. * ZX * ZX / (PIALF0 * ZY)                            
c        ZSQ1 = SQRT(1. + O1H * ZUXY ) - 1.                             
c        ZSQ2 = SQRT(1. + O2H * ZUXY ) - 1.                             
c        ZVXY = PIALF0 * ZY / (2. * ZX)                                 
c        ZAERCN = PUU(JL,17) + ZEU12 + ZPU12                            
c        ZTO11 = EXP( - ZVXY * ZSQ1 - ZAERCN )                          
c        ZTO2 = EXP( - ZVXY * ZSQ2 - ZAERCN )                           
C                                                                       
C-- TRACE GASES (CH4, N2O, CFC-11, CFC-12)                              
C                                                                       
C* CH4 IN INTERVAL 800-970 + 1110-1250 CM-1                             
C                                                                       
      IF (NEXOTIC) THEN                                                 
      ZXCH4 = MAX(1.0D-12, PUU(JL,19))                                  
      ZYCH4 = MAX(1.0D-12, PUU(JL,20))                                  
      ZUXY = 4. * ZXCH4*ZXCH4/(0.103*ZYCH4)                             
      ZSQH41 = SQRT(1. + 33.7 * ZUXY) - 1.                              
      ZVXY = 0.103 * ZYCH4 / (2. * ZXCH4)                               
      ZODH41 = ZVXY * ZSQH41                                            
C                                                                       
C* N2O IN INTERVAL 800-970 + 1110-1250 CM-1                             
C                                                                       
      ZXN2O = MAX(1.0D-12, PUU(JL,21))                                  
      ZYN2O = MAX(1.0D-12, PUU(JL,22))                                  
      ZUXY = 4. * ZXN2O*ZXN2O/(0.416*ZYN2O)                             
      ZSQN21 = SQRT(1. + 21.3 * ZUXY) - 1.                              
      ZVXY = 0.416 * ZYN2O / (2. * ZXN2O)                               
      ZODN21 = ZVXY * ZSQN21                                            
C                                                                       
C* CH4 IN INTERVAL 1250-1450 + 1880-2820 CM-1                           
C                                                                       
      ZUXY = 4. * ZXCH4*ZXCH4/(0.113*ZYCH4)                             
      ZSQH42 = SQRT(1. + 400. * ZUXY) - 1.                              
      ZVXY = 0.113 * ZYCH4 / (2. * ZXCH4)                               
      ZODH42 = ZVXY * ZSQH42                                            
C                                                                       
C* N2O IN INTERVAL 1250-1450 + 1880-2820 CM-1                           
C                                                                       
      ZUXY = 4. * ZXN2O*ZXN2O/(0.197*ZYN2O)                             
      ZSQN22 = SQRT(1. + 2000. * ZUXY) - 1.                             
      ZVXY = 0.416 * ZYN2O / (2. * ZXN2O)                               
      ZODN22 = ZVXY * ZSQN22                                            
C                                                                       
C* CFC-11 IN INTERVAL 800-970 + 1110-1250 CM-1                          
C                                                                       
      ZA11 = 2. * PUU(JL,23) * 4.404E+05                                
      ZTTF11 = 1. - ZA11 * 0.003225                                     
C                                                                       
C* CFC-12 IN INTERVAL 800-970 + 1110-1250 CM-1                          
C                                                                       
      ZA12 = 2. * PUU(JL,24) * 6.7435E+05                               
      ZTTF12 = 1. - ZA12 * 0.003225                                     
      ELSE                                                              
         ZODH41=0.                                                      
         ZODN21=0.                                                      
         ZODH42=0.                                                      
         ZODN22=0.                                                      
         ZTTF11=1.                                                      
         ZTTF12=1.                                                      
      END IF                                                            
C                                                                       
      TC1=EXP(-ZEU12A-ZPU12A)                                           
      TC2=EXP(-ZEU12B-ZPU12B)                                           
      PTT2=PTT(JL, 2)                                                   
      PTTB=PTT(JL,18)                                                   
      PTT(JL, 2) = 0.5 *(PTTB*TC2 + (2.*PTT2-PTTB)*TC1)                 
C                                                                       
      TC1=EXP(-ZEU13-ZPU13)                                             
      TC2=EXP(-ZEU14-ZPU14)                                             
      PTT4=PTT(JL, 4)                                                   
      PTTB=PTT(JL,17)                                                   
      PTT(JL, 4) =  PTT4*TC2 + 0.54839*PTTB*(TC1-TC2)                   
C                                                                       
      TC1=EXP(-ZEU16-ZPU16)                                             
      TC2=EXP(-ZEU17-ZPU17)                                             
      PTT6=PTT(JL, 6)                                                   
      PTTB=PTT(JL,16)                                                   
      PTT(JL, 6) =  PTT6*TC2 + 0.17544*PTTB*(TC1-TC2)                   
C                                                                       
      ZUU11 = - PUU(JL,15)                                              
      ZUU12 = - PUU(JL,16) - ZODH41 - ZODN21                            
      PTT(JL,10) = EXP( - PUU(JL,14) -ZEU10 -ZPU10 )                    
      PTT(JL,11) = EXP( ZUU11 )                                         
      PTT(JL,12) = EXP( ZUU12 ) * ZTTF11 * ZTTF12                       
      PTT(JL,14) = EXP( -PUU(JL,14) - ZEU11 - ZPU11 )                   
      PTT(JL,15) = EXP ( - PUU(JL,14) - ZODH42 - ZODN22 )               
c                                                                       
c        TC1=EXP(-ZEU12A-ZPU12A)                                        
c        TC2=EXP(-ZEU12B-ZPU12B)                                        
c        PTT2=PTT(JL, 2)                                                
c        PTTB=PTT(JL,18)                                                
c        PTT(JL, 2) = 0.5 *(PTTB*TC2 + (2.*PTT2-PTTB)*TC1)              
c                                                                       
c        TC1=EXP(-ZEU13-ZPU13)                                          
c        TC2=EXP(-ZEU14-ZPU14)                                          
c        PTT4=PTT(JL, 4)                                                
c        PTTB=PTT(JL,17)                                                
c        PTT(JL, 4) =  PTT4*TC2 + 0.54839*PTTB*(TC1-TC2)                
c                                                                       
c        TC1=EXP(-ZEU16-ZPU16)                                          
c        TC2=EXP(-ZEU17-ZPU17)                                          
c        PTT6=PTT(JL, 6)                                                
c        PTTB=PTT(JL,16)                                                
c        PTT(JL, 6) =  PTT6*TC2 + 0.17544*PTTB*(TC1-TC2)                
c                                                                       
c        ZUU11 = - PUU(JL,15)                                           
c        ZUU12 = - PUU(JL,16)                                           
c        PTT(JL,10) = EXP( - PUU(JL,14) -ZEU10 -ZPU10 )                 
c        PTT(JL,11) = EXP( ZUU11 )                                      
c        PTT(JL,12) = EXP( ZUU12 )                                      
c        PTT(JL,14) = EXP( -PUU(JL,14) - ZEU11 - ZPU11 )                
c        PTT(JL,15) = EXP ( - PUU(JL,14) )                              
c        print*,'lwtt1 ptt(jj)=',ptt(jl,jj),jj                          
 201  CONTINUE                                                          
      DO 301 JL=1,NRLST                                                 
         DO 302 II=1,NUA                                                
           ZUU(JL,II) = PUU(JL,II)                                      
 302     CONTINUE                                                       
         DO 303 JJ=1,NTRA                                               
           ZTT(JL,JJ) = 1.0                                             
 303     CONTINUE                                                       
         IF(ITASK.EQ.1)THEN                                             
           ZTT(JL,1)  = PTT(JL,1)                                       
           ZTT(JL,2)  = PTT(JL,2)                                       
           ZTT(JL,3)  = PTT(JL,3)                                       
           ZTT(JL,4)  = PTT(JL,4)                                       
           ZTT(JL,5)  = PTT(JL,5)                                       
           ZTT(JL,6)  = PTT(JL,6)                                       
         ELSE IF(ITASK.EQ.2)THEN                                        
           ZTT(JL,1)  = PTT(JL,1)                                       
           ZTT(JL,2)  = PTT(JL,2)                                       
           ZTT(JL,3)  = PTT(JL,3)                                       
           ZTT(JL,4)  = PTT(JL,4)                                       
           ZTT(JL,5)  = PTT(JL,5)                                       
           ZTT(JL,6)  = PTT(JL,6)                                       
           ZTT(JL,10) = PTT(JL,10)                                      
           ZTT(JL,11) = PTT(JL,11)                                      
           ZTT(JL,12) = PTT(JL,12)                                      
           ZTT(JL,14) = PTT(JL,14)                                      
           ZTT(JL,15) = PTT(JL,15)                                      
         ELSE IF(ITASK.EQ.3)THEN                                        
           DO 3031 JJ=1,NTRA                                            
             ZTT(JL,JJ) = PTT(JL,JJ)                                    
 3031       CONTINUE                                                    
         ENDIF                                                          
 301  CONTINUE                                                          
      RETURN                                                            
      END                                                               
C                                                                       
      SUBROUTINE RADLSW ( KDLON, KRLST, KFLEV, KFLVP1, KFLEV2,          
     S            KNGL, KNGLP1, PSIG,  PSCT, PCARDI, ITASK, ZTM1        
     S ,PIERSLWP)                                                       
C                                                                       
C**** *RADLSW* - RADIATION INTERFACE                                    
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C           CONTROLS RADIATION TRANSFER COMPUTATIONS                    
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C          *RADLSW* IS CALLED FROM *RADINT*                             
C                                                                       
C        EXPLICIT ARGUMENTS                                             
C        --------------------                                           
C                                                                       
C        IMPLICIT ARGUMENTS                                             
C        --------------------                                           
C                                                                       
C     ==== INPUTS ===                                                   
C     ==== OUTPUTS ===                                                  
C FLS(KDLON,KFLEV)            ; NET SHORTWAVE RADIATIVE FLUXES          
C FLT(KDLON,KFLEV)            ; NET LONGWAVE RADIATIVE FLUXES           
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C        SEE DOCUMENTATION                                              
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C        ECMWF MODEL DOCUMENTATION                                      
C                                                                       
C     AUTHORS.                                                          
C     --------                                                          
C        JEAN-JACQUES MORCRETTE  *ECMWF*                                
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C        ORIGINAL : 88-02-04                                            
C------------------------------------------------------------           
C      IMPLICIT LOGICAL (L)                                             
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRSW* SWITCHES RELATED TO THE RADIATION SCHEME.         
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      IMPLICIT REAL (A-H,O-Z)                                           
C      IMPLICIT UNDEFINED (Z)                                           
C      IMPLICIT LOGICAL (L)                                             
C      IMPLICIT UNDEFINED (L)                                           
      IMPLICIT INTEGER (I-K,M-N)                                        
C      IMPLICIT UNDEFINED (I-K,M-N)                                     
C      IMPLICIT REAL (P)                                                
C      IMPLICIT UNDEFINED (A-H,O,Q-Y)                                   
C                                                                       
C everything except the parameters is defined                           
C in the common blocks                                                  
C                                                                       
      COMMON/COM1/                                                      
C     I NRADFR, NRPART, NRINT, NRADPFR, NRADPLA,                        
C     I NRLSTI,                                                         
     R XLAT,                                                            
     R RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR, ZCCO2,              
C     R PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3,                 
     L LOH, LOM, LOL, LORAD                                             
      COMMON/COM2/                                                      
     I ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,                  
     I         JLOOP, KRLEV,                                            
     I        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     I        ITH, ITM, ITL,                                            
     I        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     I        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     I        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     I        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      COMMON/COM3/                                                      
     I IND4, KM12, KXD, KXU, IG, KBS, KDD,                              
     I        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     I        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     I        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     I        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     I        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
      COMMON/COM4/                                                      
     R           COZQC,COZQS,COZHC,COZHS,                               
     R CSUPSAT,                                                         
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
      COMMON/COM5/                                                      
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NUMVC,                                                           
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      COMMON/COM6/                                                      
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD,       
     R DTIME,    TWODT,    EPS,                                         
     R SINLON,     COSLON,                                              
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2,                          
     I NFAX,                                                            
     R CRAE,                                                            
     R CDISSEM                                                          
      COMMON/COM7/                                                      
     R     CCRH,                                                        
     L LO,LO1,LO2,LODIA,LODIAP,                                         
     R TAUA, PIZA, CGA, CAER, CAEROS                                    
      COMMON/COM7A/                                                     
     R  RT1,WG1,AT,BT,CNTNU,O1H,O2H,PIALF0                              
     S ,OCT,TREF,XP,TSTAND,GA,GB,CE,ETY,PTY                             
      COMMON/COM7B/                                                     
     S TINTP,TSTP,                                                      
     S  TBO3,AO3,BO3,TB,ALP,BET,ULG,ULGO,PLG,AP,BP,TO3B,O3A,O3B,        
     R          CDAY,CCO2,CH2O,DIFF,X10E                                
       COMMON/COM7C/                                                    
     S  ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW,                       
     R  APAD,BPAD,D,CRAY,SUN,                                           
     R   TTT,                                                           
     R   TM1                                                            
      COMMON/COM8/                                                      
     R       TSM1M,                                                     
     R       QM1,                                                       
     R       SNM1M,                                                     
     R       ALBM,                                                      
     R       AVVELM,                                                    
     R   SLMM,                                                          
     R       APHM1,                                                     
     R       APM1                                                       
      COMMON/COM9/                                                      
     L LOLAND,                                                          
     R   ZTI,                                                           
     *       ZCTI,                                                      
     *       ZCPH,                                                      
     *       ZWV,                                                       
     *       ZSW,                                                       
     *       ZALTE,                                                     
     *       ZCALTE,                                                    
     *       ZALSO                                                      
      COMMON/COM10/                                                     
     *       ZCALSO,                                                    
     R   ZOZQ,                                                          
     *       ZOZH,                                                      
     R   ZTRSOL,                                                        
     *       ZEMTER,                                                    
     R   TRSOLM,                                                        
     *       EMTERM,                                                    
     R   AMU0,                                                          
     *       RDAYL                                                      
      COMMON/COM11/                                                     
     R   CLC,                                                           
     *       TI,                                                        
     *       DP,                                                        
     *       AADS,                                                      
     *       AEQ1,                                                      
     *       AEQ2,                                                      
     *       AEQ3,                                                      
     *       AEQ4,                                                      
     *       AEQ5,                                                      
     *       SSW                                                        
      COMMON/COM12/                                                     
     *       WV,                                                        
     *       QOF,                                                       
     *       QCF,                                                       
     *       CLWA,                                                      
     *       APRE,                                                      
     *       SMU0,                                                      
     *       ALTE,                                                      
     *       ALSO                                                       
      COMMON/COM13/                                                     
     R   ZDPO,                                                          
     *       ZQCFO,                                                     
     *       ZAEQSO,                                                    
     *       ZQOFO,                                                     
     *       ZDPN,                                                      
     *       ZQCFN,                                                     
     *       ZAEQSN,                                                    
     *       ZQOFN                                                      
      COMMON/COM14/                                                     
     R   FLT,                                                           
     *       FLS,                                                       
     R   DIAG,                                                          
     *       DIA1,                                                      
     *       ZDIAG,                                                     
     *       ZDIA1,                                                     
     *       ZDIA,                                                      
     *       ZDIAF,                                                     
     *       ZDIAT,                                                     
     R PGG                                                              
      COMMON/COM15/                                                     
     #     PREF,                                                        
     #     PRE1,                                                        
     #     PRE2,                                                        
     #     PRMUZ,                                                       
     #     PTO1,                                                        
     #     PTR1,                                                        
     #     PTR2,                                                        
     #     PW,                                                          
     R     PALBSN                                                       
      COMMON/COM16/                                                     
     R ZABSCU,                                                          
     S     ZABSLY,                                                      
     S     ZADJD ,                                                      
     S     ZADJU ,                                                      
     S     ZBGND ,                                                      
     S     ZBINT ,                                                      
     S     ZBSUR ,                                                      
     S     ZBSUIN,                                                      
     S     ZBTOP ,                                                      
     S     PCLDLW,                                                      
     S     ZCLOUD,                                                      
     S     ZCNTRB,                                                      
     S     ZCUMP ,                                                      
     S     ZCUMPT,                                                      
     S     ZDBDT                                                        
      COMMON/COM17/                                                     
     S     ZDBSL ,                                                      
     S     ZDISTD,                                                      
     S     ZDISTU,                                                      
     S     ZDNF  ,                                                      
     S     PDT0  ,                                                      
     S     ZDUC                                                         
      COMMON/COM18/                                                     
     S     ZEFFT ,                                                      
     S     ZEMU  ,                                                      
     S     ZEPHIO,                                                      
     S     ZEPSC2,                                                      
     S     ZEPSC3,                                                      
     S     ZEPSH1,                                                      
     S     ZEPSH2,                                                      
     S     ZEPSH3                                                       
      COMMON/COM19/                                                     
     S     ZEPSH4,                                                      
     S     ZEPSH5,                                                      
     S     ZEPSH6,                                                      
     S     ZEPSIO,                                                      
     S     ZFACTC,                                                      
     S     ZZFD  ,                                                      
     S     ZFDD  ,                                                      
     S     ZFDN                                                         
      COMMON/COM20/                                                     
     S     PFLUX , PFLUC,                                               
     S     ZFLWP , PCTS                                                 
     S     ZZFU  ,                                                      
     S     ZFUP  ,                                                      
     S     ZGLAYD,                                                      
     S     ZGLAYU,                                                      
     S     ZOMEGA,                                                      
     S     ITX   ,                                                      
     S     IXDIA ,                                                      
     S     IXT   ,                                                      
     S     IXTSU ,                                                      
     S     IXTTP ,                                                      
     S     IMX   ,                                                      
     S     IMXP                                                         
      COMMON/COM21/                                                     
     R ZOZ   ,                                                          
     S     PPMB  ,                                                      
     S     ZRES  , PB                                                   
     S     ZSSIG ,                                                      
     S     PTAU  ,                                                      
     S     PTAVE ,                                                      
     S     ZTAVIC,                                                      
     S     ZTI1  ,                                                      
     S     PTL                                                          
      COMMON/COM22/                                                     
     S     ZTT   ,                                                      
     S     ZTT1  ,                                                      
     S     ZTT2  ,                                                      
     S     ZUAER ,                                                      
     S     ZUPF  ,                                                      
     S     ZUU   ,                                                      
     S     ZXD                                                          
      COMMON/COM23/                                                     
     S     ZXN   ,                                                      
     S     ZXOZ  ,                                                      
     S     ZXWV  ,                                                      
     S     ZZZ   ,                                                      
     S     ZRES2 ,                                                      
     S     ZTI2  ,                                                      
     S     ZBLAY ,                                                      
     S     ZBLEV ,                                                      
     S     ZCTS  ,                                                      
     S     ZHFG                                                         
      COMMON/COM24/                                                     
     I KT,                                                              
     R    PUU   ,                                                       
     S        PTT   ,                                                   
     S        ZZ                                                        
     R  PCLFR  ,                                                        
     S      PT     ,                                                    
     S      PQOF   ,                                                    
     S      PQLWP  ,                                                    
     S      PEMIS  ,                                                    
     S      PFLT                                                        
      COMMON/COM25/                                                     
     R  ZALBSU,                                                         
     S      ZCG   ,                                                     
     S      ZCLDLW,                                                     
     S      ZCLDSW,                                                     
     S      ZCOOLR,                                                     
     S      ZDT0  ,                                                     
     S      ZEMD  ,                                                     
     S      ZFLUX ,                                                     
     S      ZPMB  ,                                                     
     S      ZTAU  ,                                                     
     S      ZTAVE ,                                                     
     S      ZTL   ,                                                     
     S      ZFSDWN                                                      
      COMMON/COM26/                                                     
     S      ZFSUP ,                                                     
     S      ZHEATR,                                                     
     R   PDP   ,                                                        
     S       PWV   ,                                                    
     S       PFLS  ,                                                    
     S       PPSOL ,                                                    
     S       PRMU0 ,                                                    
     S       PAER  ,                                                    
     S       PALBS ,                                                    
     S       PCG   ,                                                    
     S       PCLDSW,                                                    
     S       POMEGA,                                                    
     S       POZ                                                        
      COMMON/COM27/                                                     
     R   PFDOWN,                                                        
     S       PFUP  ,                                                    
     R   ZAKI  ,                                                        
     S       ZCGAZ ,                                                    
     S       ZCNEB ,                                                    
     S       ZC1I  ,                                                    
     S       ZDSC  ,                                                    
     S       ZDSH                                                       
      COMMON/COM28/                                                     
     S       ZDSIG ,                                                    
     S       ZFACT ,                                                    
     S       ZFACT1 ,                                                   
     S       ZFACT2 ,                                                   
     S       ZFACT3 ,                                                   
     S       ZFACT4 ,                                                   
     S       ZFACT5 ,                                                   
     S       ZFACT6 ,                                                   
     S       ZFD   ,                                                    
     S       ZFRAC ,                                                    
     S       ZFU   ,                                                    
     S       ZG    ,                                                    
     S       ZGG   ,                                                    
     S       ZPIZAZ                                                     
      COMMON/COM29/                                                     
     S       ZRAYL ,                                                    
     S       ZRAY1 ,                                                    
     S       ZRAY2 ,                                                    
     S       ZREF  ,                                                    
     S       ZREFZ ,                                                    
     S       ZRE1  ,                                                    
     S       ZRE2  ,                                                    
     S       ZRJ   ,                                                    
     S       ZRK   ,                                                    
     S       ZRL   ,                                                    
     S       ZRMU  ,                                                    
     S       ZRMUE ,                                                    
     S       ZRMUZ                                                      
      COMMON/COM30/                                                     
     S       ZRNEB ,                                                    
     S       ZRUEF ,                                                    
     S       ZR1   ,                                                    
     S       ZR2   ,                                                    
     S       ZR21  ,                                                    
     S       ZR22  ,                                                    
     S       ZR23  ,                                                    
     S       ZS    ,                                                    
     S       ZSEC  ,                                                    
     S       ZSS1  ,                                                    
     S       ZTAUAZ                                                     
      COMMON/COM31/                                                     
     S       ZTOT  ,                                                    
     S       ZTO1  ,                                                    
     S       ZTR   ,                                                    
     S       ZTRA1 ,                                                    
     S       ZTRA2 ,                                                    
     S       ZTR1                                                       
      COMMON/COM32/                                                     
     S       ZTR2  ,                                                    
     S       ZUD   ,                                                    
     S       ZUM   ,                                                    
     S       ZU1D  ,                                                    
     S       ZU2D  ,                                                    
     S       ZW    ,                                                    
     S       ZN175 ,                                                    
     S       ZN190 ,                                                    
     S       ZO175 ,                                                    
     S       ZO190 ,                                                    
     S       ZP75  ,                                                    
     S       ZP90  ,                                                    
     S       ZSIGN ,                                                    
     S       ZSIGO                                                      
      COMMON/COM33/                                                     
     R   PTR,                                                           
     S       PU,                                                        
     R   ZRH,                                                           
     *       ZCH,                                                       
     *       ZCM,                                                       
     *       ZCL,                                                       
     *       ZCT,                                                       
     *       ZCC,                                                       
     I NTOPCM                                                           
      COMMON/COM34/                                                     
     *        NBASECM,                                                  
     R   ARPRCM,                                                        
     I ITOPH,                                                           
     *        ITOPM,                                                    
     *        ITOPL,                                                    
     L LOC,                                                             
     R   ZQS,                                                           
     *       ZCLC,                                                      
     *       ZQLWP                                                      
      COMMON/COM35/                                                     
     R   VERVEL,                                                        
     *       ZTHETA,                                                    
     *       ZETA,                                                      
     *       ZDTHMIN,                                                   
     I INVB,                                                            
     R   ZCTLWP,                                                        
     *       ZQLOC,                                                     
     *       ZQMEC,                                                     
     *       ZQHIC,                                                     
     R   ZFSOT,                                                         
     #       ZFTET,                                                     
     #       ZFLT                                                       
      COMMON/COM36/                                                     
     #       ZFSO ,                                                     
     #       ZFTE ,                                                     
     #       ZFLB ,                                                     
     #       ZDTDT,                                                     
     #       HEATO,                                                     
     #       HEATE,                                                     
     #       HEATR,                                                     
     #       ZI0                                                        
C                                                                       
      INTEGER                                                           
C     I NRADFR,   NRPART,   NRINT,    NRLSTI,  NRADPFR,  NRADPLA,       
     I NRLST,    NRLEV,    NRLEVP1, NLEV, NLP2, NLEVP1,                 
     I NLON, IRLEV, NTRA, NLAT, ITIM, NUA, NINT, NRPARTI                
C                                                                       
C   NRLEV .... number of vertical levels                                
C   NRLST .... number of points on a latitude line                      
C   NUA   .... number of absorber amount                                
C   NINT  .... number of spectral intervals                             
C   NTRA  .... number of transmission functions                         
C   NLAT  .... number of latitude points                                
C   ITIM  .... number of month under consideration                      
C                                                                       
C      PARAMETER (NRLEV=122,NUA=34,NINT=6,NTRA=34, NLAT=1,              
      PARAMETER (NRLEV=21,NUA=34,NINT=6,NTRA=34, NLAT=1,                
cccm      PARAMETER (NRLEV=37,NUA=18,NINT=6,NTRA=19, NLAT=1,            
cccm     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=10,NLEV=NRLEV,        
     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=7,NLEV=NRLEV,             
     #           NLP2=NRLST, NLEVP1=NRLEVP1, NLON=NRLST,                
     #           NRPARTI=1)                                             
C                                                                       
      REAL YCLOCK, XLAT(NLAT), Z0, DZ, ALBMPAR, SNPAR,                  
     #     TSUPAR, CTFREEZ, DELTAC, DELTAO, DELTAH                      
C                                                                       
C   YTIME  .... time of the year in radians                             
C   YCLOCK .... time of the day in radians                              
C   XLAT   .... latitudes in radians                                    
C   Z0     .... zero point of height grid                               
C   DZ     .... Interval between height levels                          
C   ALBMPAR.... sw surface albedo as function of longitude:             
C           yearly averaged sea-ice: 0.55                               
C                           water  : 0.07                               
C                           land   : 0.07.lt.ALBM.lt.0.80               
C                                    depending on snow                  
C   SNPAR  .... snow depth in meter                                     
C   TSUPAR .... surface temperature (Kelvin)                            
C   CTFREEZ.... freezing temperature of sea-water (Kelvin)              
C   DELTAC .... Doppler parameter for line shapes of CO2                
C   DELTAO .... Doppler parameter for line shapes of O3                 
C                                                                       
      PARAMETER (Z0=0., CTFREEZ=271.26,                                 
C     PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
     #           DZ=0.289, ALBMPAR=0.2, SNPAR=0.,                       
cccm  PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
cccm     #           DZ=0.15, ALBMPAR=0.3, SNPAR=0.,                    
CZ     #           DELTAC=0., DELTAO=0.)                                
     #           DELTAC=6.0E-04, DELTAO=1.E-02, DELTAH=5.0E-04)         
C      PARAMETER (YCLOCK=0., Z0=0., TSUPAR=273., CTFREEZ=271.26,        
C     #           DZ=0.46875, ALBMPAR=0.07, SNPAR=0.)                   
C      PARAMETER (YCLOCK=0., Z0=2.608, TSUPAR=273., CTFREEZ=271.26,     
C     #           DZ=0.2878, ALBMPAR=0.07, SNPAR=0.)                    
C                                                                       
      LOGICAL LDIUR, LOLPAR                                             
C                                                                       
C   LDIUR .... true: sun at time, false: diurnally averaged             
C   LOLPAR.... true: land surface, false: else                          
C                                                                       
C      PARAMETER (LDIUR=.TRUE., LOLPAR=.TRUE.)                          
       PARAMETER (LOLPAR=.TRUE.)                                        
C                                                                       
      INTEGER  C2TEMP                                                   
      CHARACTER*64  MIDTMP                                              
cccm      PARAMETER (C2TEMP=10,                                         
cccm     #    MIDTMP='/home/atm/ugamp/midrad/radfildat.dir/MIDTMP.DAT') 
ccc1      PARAMETER (C2TEMP=10, MIDTMP='tprfil2.dat')                   
C                                                                       
      INTEGER  IRLONI, IRLST, IRLVP1, JPNLEV, JPNLVP1                   
C                                                                       
      PARAMETER (IRLONI=NRLST, IRLST=NRLST,                             
     #           IRLVP1=NRLEVP1, JPNLEV=NRLEV,                          
     #           JPNLVP1=NRLEVP1, IRLEV=NRLEV)                          
C                                                                       
      INTEGER NIPD, NIPD2, NTR, NG1, NG1P1                              
      PARAMETER(NIPD=8,NIPD2=16,NTR=11,NG1=2,NG1P1=3)                   
C                                                                       
C    definition of the functions                                        
C                                                                       
      REAL       CVMGT, SIGMA2                                          
      INTEGER    CVMGI                                                  
C                                                                       
C    definitons of the subroutine cloud                                 
C                                                                       
      REAL RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR                  
C                                                                       
C    missing definitions in morcrette                                   
C                                                                       
C      REAL   PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3            
      REAL ZCCO2                                                        
                                                                        
      LOGICAL LOH, LOM, LOL, LORAD                                      
C                                                                       
      INTEGER ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,           
     #        IZQS, IZCLC, IZQLWP, JLOOP, KRLEV,                        
     #        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     #        ITH, ITM, ITL,                                            
     #        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     #        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     #        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     #        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      INTEGER IND4, KM12, KXD, KXU, IG, KBS, KDD,                       
     #        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     #        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     #        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     #        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     #        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD1* SPECTRAL DISTRIBUTION OF AEROSOLS AND OZONE.     
C                     (TRIANGULAR *T10* TRUNCATIONFOR AEROSOLS).        
C                     (TRIANGULAR *T5* TRUNCATIONFOR OZONE).            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R           COZQC(21),COZQS(15),COZHC(21),COZHS(15)                
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD2* CONSTANTS INTERNAL TO THE RADIATION SUBROUTINES. 
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CSUPSAT                                                          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMCON* BASIC UNIVERSAL CONSTANTS AND DERIVED CONSTANTS.  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
C                                                                       
C                                                                       
C       --------------------------------------------------------------- 
C      PARAMETER(                                                       
C     I JPM=106,JPN=106,JPK=106,JPGL=160,JPNLON=320,                    
C     I JPT1MAX=400,          JPT2MAX=50,                               
C     I JPT3MAX=300,          JPT4MAX=50,                               
C     I JPHASH=2048,          JPNTRCRS=20,                              
C     I JPTASKS=4,     JPLOCKS=50,    JPEVENT=4,                        
C     I JPLTMT=JPTASKS,                                                 
C     I JPNUMW=352,            JPNUMP=352,                              
C     I JPTABLE=1024,                                                   
C     I JPMAXB =60,                                                     
C     I JPMAXPB=32,                                                     
C     I JPG3FLD=43,         JPG3XF=99,         JPG3CMP=17,              
C     I JPMP1=JPM+1,        JPNP1=JPN+1,       JPKP1=JPK+1,             
C     I JP2MP1=JPMP1*2,     JP4MP1=JPMP1*4,                             
C     I JPED=JPN/2+1,       JPOD=JPNP1/2,                               
C     I IPSP0=JPM+JPN-JPK,  IPSP1=JPNP1*JPMP1, IPSP2=IPSP0*(IPSP0+1)/2, 
C     I JPSP=IPSP1-IPSP2,   JP2SP=2*JPSP,                               
C     I JPHGL=JPGL/2,       JPNLP2=JPNLON+2,                            
C     I                     JPNVCLV=4*JPNLVP1)                          
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C     J. K. GIBSON     E.C.M.W.F.     22/01/82.                         
C     MODIFIED M.J     E.C.M.W.F.     13/12/82.                         
C                                                                       
      INTEGER                                                           
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      LOGICAL                                                           
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD        
      REAL                                                              
     R DTIME,    TWODT,    EPS                                          
C                                                                       
C      INTEGER                                                          
C     I NWTIME(JPNUMW),     NROW(JPTASKS),                              
C     I NPTIME(JPNUMP)                                                  
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMGAU* - QUANTITIES RELATED TO THE GAUSSIAN GRID.      
C                                                                       
      REAL                                                              
     R SINLON(NLP2),     COSLON(NLP2)                                   
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMTRU* -QUANTITIES RELATED TO THE SPECTRAL TRUNCATION. 
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSIM* - QUANTITIES NEEDED FOR THE SEMI-IMPLICIT SCHEME
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMHDI* - COEFFICIENTS FOR HORIZONTAL DIFFUSION.        
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMFFT* - QUANTITIES NEEDED FOR THE FAST *FOURIER       
C                         TRANSFORMS.                                   
C                                                                       
      INTEGER                                                           
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2                           
C      REAL                                                             
C     R TRIG(JPNLON)                                                    
      INTEGER                                                           
     I NFAX(10)                                                         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSTA* - ARRAYS AND PARAMETERS FOR STATISTICS.         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMDIA* - QUANTITIES RELATED TO GLOBAL PHYSICAL         
C                         DIAGNOSTICS.                                  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPSW* SWITCHES RELATED TO THE PARAMETERISATIONS OF      
C                     DIABATIC PROCESSES.                               
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMDSW* SWITCHES RELATED TO THE DYNAMICS                  
C                     AND THE GENERAL CONTROL OF THE FORECAST.          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH1* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE RADIATION SUBROUTINES.                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CRAE,                                                            
     R CDISSEM                                                          
C                                                                       
C      REAL       COSRAD(128),        SINRAD(128)                       
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH2* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE PHYSICAL SUBROUTINES (EXCEPT RADIATION    
C                     ONES).                                            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL       CCRH(JPNLEV)                                           
C                                                                       
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMDIZ* VARIABLES AND POINTERS FOR ZONAL MEAN DIAGNOSTICS 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMMSK* VARIABLES AND POINTERS FOR AREA MEAN DIAGNOSTICS. 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMSDS* - START/RESTART BLOCK.                          
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMTASK* - TASK RELATED VARIABLES.                      
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*         *COMRAD* - RADIATION POINTERS.                               
C                                                                       
      LOGICAL LO,LO1,LO2,LODIA,LODIAP                                   
C---------------------------------------------------------------------  
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C     COMMON *YOMAER* RADIATIVE CHARACTERISTICS OF THE AEROSOLS         
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
      REAL TAUA(2,5), PIZA(2,5), CGA(2,5), CAER(5,5), CAEROS            
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C     COMMON *YOMLW*                                                    
C                                                                       
C*    COEFFICIENTS FOR THE LONGWAVE RADIATION SUBROUTINES               
C                                                                       
C NINT  :  NUMBER OF SPECTRAL INTERVALS                                 
C NIPD  :  NUMBER OF BANDS DESCRIBED BY PADE APPROXIMANTS               
C NTR   :  NUMBER OF REFERENCE TEMPERATURES                             
C NTRA  :  NUMBER OF TRANSMISSION FUNCTIONS                             
C NUA   :  NUMBER OF ABSORBER AMOUNTS                                   
C NG1   :  ORDER OF THE GAUSSIAN QUADRATURE                             
C                                                                       
C RT1   :  ROOTS OF THE GAUSSIAN POLYNOMIAL                             
C WG1   :  WEIGHTS OF THE GAUSSIAN POLYNOMIAL                           
C AT, BT:  COEFFICIENTS OF THE TEMPERATURE DEPENDENCE                   
C CNTNU :  H2O CONTINUUM ABSORPTION COEFFICIENTS                        
C O1H,O2H,PIALF0,OCT: OZONE LONGWAVE ABSORPTION COEFFICIENTS            
C TREF, TSTAND      : REFERENCE TEMPERATURES                            
C XP    :  COEFFICIENTS OF POLYNOMIAL DEVELOPMENTS OF PLANCK FUNCTION   
C GA,GB :  PADE APPROXIMANTS OF H2O, CO2 TRANSMISSION FUNCTIONS         
C ETY,PTY :  PARAMETERS OF H2O CONTINUUM                                
C TINTP :  REFERENCE TEMPERATURES OF PRE-COMPUTED TRANSMISSIVITIES      
C CE    :  EMS COEFFS OF H2O TRANSMISSION FUNCTIONS                     
C                                                                       
      REAL  RT1(2),WG1(2),AT(12,3),BT(12,3),CNTNU(4),O1H                
     S ,OCT(4),TREF,XP(6,6),TSTAND,GA(11,24,3),GB(11,24,3)              
     S ,PTY(11,20),ETY(11,20),TINTP(11),TSTP,O2H,PIALF0                 
C                                                                       
      REAL  TB(40,31,2),ALP(40,31,2),BET(40,31,2),                      
     S ULG(40),PLG(31),AP(40,31,2),BP(40,31,2),TBO3(38,31),TO3B(38,31)  
     S,ULGO(38),AO3(38,31),BO3(38,31),O3A(38,31),O3B(38,31)             
C     ----------------------------------------------------------------- 
C*COMDECK YOMRDU                                                        
C     ----------------------------------------------------------------- 
C                                                                       
C*    CONTROL VARIABLES FOR RADIATION: INTERNAL SWITCH AND INDICES      
C                                                                       
C ZEELOG:  SECURITY THRESHOLD FOR ABSORBER AMOUNT IN LAPLACE TRANSFORM  
C ZEPSC :  SECURITY THRESHOLD FOR CLOUD COVER                           
C ZEPSCO:  SECURITY THRESHOLD FOR OZONE AMOUNT                          
C ZEPSCQ:  SECURITY THRESHOLD FOR WATER VAPOR                           
C ZEPSCT:  SECURITY THRESHOLD FOR SHORTWAVE OPTICAL THICKNESS           
C ZEPSCW:  SECURITY THRESHOLD FOR CLOUD LIQUID WATER PATH               
C                                                                       
      REAL          CDAY,CCO2,CH2O,DIFF,X10E                            
     S             ,ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW            
C     ----------------------------------------------------------------- 
C*COMDECK YOMSW                                                         
C     ----------------------------------------------------------------- 
C                                                                       
C*    COEFFICIENTS FOR THE SHORTWAVE RADIATION SUBROUTINES              
C                                                                       
C APAD  :  PADE APPROXIMANTS NUMERATOR                                  
C BPAD  :  PADE APPROXIMANTS DENOMINATOR                                
C D     :  TRANSMISSION LIMIT FOR INFINITE ABSORBER AMOUNT              
C CRAY  :  RAYLEIGH SCATTERING COEFFICIENTS                             
C SUN   :  SOLAR FRACTION IN SPECTRAL INTERVALS                         
C                                                                       
      REAL  APAD(2,3,7),BPAD(2,3,7),D(2,3),CRAY(2,6),SUN(2)             
C     ----------------------------------------------------------------- 
                                                                        
C                                                                       
C  -------------------------------------------------------------------- 
C                                                                       
      REAL   TTT(1,NLAT,NRLEV,12)                                       
C                                                                       
C        0.1  "RADINT POINTERS"                                         
C             -----------------                                         
C                                                                       
      REAL   TM1(NLP2,NLEV),                                            
     *       TSM1M(NLP2),                                               
     *       QM1(NLP2,NLEV),                                            
     *       SNM1M(NLP2),                                               
     *       ALBM(NLP2),                                                
     *       AVVELM(NLP2)                                               
      REAL   SLMM(NLP2),                                                
     *       APHM1(NLP2,NLEVP1),                                        
     *       APM1(NLP2,NLEV)                                            
C                                                                       
      LOGICAL LOLAND(NLP2)                                              
C                                                                       
C     VARIABLES FOR FULL GRID AND TRANSFORMS.                           
C                                                                       
      REAL   ZTI(NLP2,NRLEVP1),                                         
     *       ZCTI(NLP2,NRLEVP1),                                        
     *       ZCPH(NLP2,NRLEVP1),                                        
     *       ZWV(NLP2,NRLEV),                                           
     *       ZSW(NLP2,NRLEV),                                           
     *       ZALTE(NLP2),                                               
     *       ZCALTE(NLP2),                                              
     *       ZALSO(NLP2),                                               
     *       ZCALSO(NLP2)                                               
C                                                                       
      REAL   ZOZQ(NLP2),                                                
     #       ZOZH(NLP2)                                                 
C                                                                       
      REAL   ZTRSOL(NLP2,NLEVP1),                                       
     *       ZEMTER(NLP2,NLEVP1)                                        
C                                                                       
C     FINAL RESULTS ARE STORED ON:                                      
C                                                                       
      REAL   TRSOLM(NLP2,NLEVP1),                                       
     *       EMTERM(NLP2,NLEVP1)                                        
C                                                                       
      REAL   AMU0(NLP2),                                                
     *       RDAYL(NLP2)                                                
C                                                                       
C     VARIABLES WITHIN THE SPLITTED LATITUDE LOOP.                      
C                                                                       
      REAL   CLC(NRLST,NRLEV),                                          
     *       TI(NRLST,NRLEVP1),                                         
     *       DP(NRLST,NRLEV),                                           
     *       AADS(NRLST,NRLEV),                                         
     *       AEQ1(NRLST,NRLEV),                                         
     *       AEQ2(NRLST,NRLEV),                                         
     *       AEQ3(NRLST,NRLEV),                                         
     *       AEQ4(NRLST,NRLEV),                                         
     *       AEQ5(NRLST,NRLEV),                                         
     *       SSW(NRLST,NRLEV),                                          
     *       WV(NRLST,NRLEV),                                           
     *       QOF(NRLST,NRLEV),                                          
     *       QCF(NRLST,NRLEV),                                          
     *       CLWA(NRLST,NRLEV),                                         
     *       APRE(NRLST),                                               
     *       SMU0(NRLST),                                               
     *       ALTE(NRLST),                                               
     *       ALSO(NRLST)                                                
      REAL   ZDPO(NRLST),                                               
     *       ZQCFO(NRLST),                                              
     *       ZAEQSO(NRLST),                                             
     *       ZQOFO(NRLST),                                              
     *       ZDPN(NRLST),                                               
     *       ZQCFN(NRLST),                                              
     *       ZAEQSN(NRLST),                                             
     *       ZQOFN(NRLST)                                               
      REAL   FLT(NRLST,NRLEVP1),                                        
     *       FLS(NRLST,NRLEVP1)                                         
C                                                                       
C     DIAGNOSTIC VARIABLES.                                             
C                                                                       
      REAL   DIAG(NLEVP1,4),                                            
     *       DIA1(7),                                                   
     *       ZDIAG(NLEVP1,10),                                          
     *       ZDIA1(21),                                                 
     *       ZDIA(NLON),                                                
     *       ZDIAF(NLON,7),                                             
     *       ZDIAT(NLEVP1)                                              
C                                                                       
C                                                                       
C                                                                       
      REAL PGG(IRLST),                                                  
     #     PREF(IRLST),                                                 
     #     PRE1(IRLST),                                                 
     #     PRE2(IRLST),                                                 
     #     PRMUZ(IRLST),                                                
     #     PTO1(IRLST),                                                 
     #     PTR1(IRLST),                                                 
     #     PTR2(IRLST),                                                 
     #     PW(IRLST)                                                    
C                                                                       
C                                                                       
C  what about the 1-dimensional PALBS???                                
C                                                                       
      REAL     PALBSN  (IRLST)                                          
C                                                                       
C*       0.2   "LOCAL" ARRAYS                                           
C              --------------                                           
      REAL ZABSCU(IRLST ,NUA, 3*NRLEV+1),                               
     S     ZABSLY(IRLST ,NUA, 3*NRLEV+1) ,                              
     S     ZADJD (IRLST , IRLVP1)      ,                                
     S     ZADJU (IRLST , IRLVP1)       ,                               
     S     ZBGND (IRLST)                ,                               
     S     ZBINT (IRLST , IRLVP1)       ,                               
     S     ZBSUR (IRLST , NINT)         ,                               
     S     ZBSUIN(IRLST)                ,                               
     S     ZBTOP (IRLST , NINT)         ,                               
     S     PCLDLW(IRLST , IRLEV)        ,                               
     S     ZCLOUD(IRLST)                ,                               
     S     ZCNTRB(IRLST , IRLVP1,IRLVP1),                               
     S     ZCUMP (IRLST)                ,                               
     S     ZCUMPT(IRLST)                ,                               
     S     ZDBDT (IRLST , NINT , IRLEV) ,                               
     S     ZDBSL (IRLST , NINT , 2*NRLEV),                              
     S     ZDISTD(IRLST , IRLVP1)       ,                               
     S     ZDISTU(IRLST , IRLVP1)                                       
      REAL                                                              
     S     ZDNF  (IRLST , IRLVP1,IRLVP1),                               
     S     PDT0  (IRLST)                                                
      REAL                                                              
     S     ZDUC  (IRLST , 3*NRLEV+1)       ,                            
     S     ZEFFT (IRLST)                ,                               
     S     ZEMD  (IRLST , IRLEV), ZEMU  (IRLST , IRLEV),                
     S     ZEPSC2(IRLST),  ZEPSC3(IRLST),                               
     S     ZEPSH1(IRLST),  ZEPSH2(IRLST),                               
     S     ZEPSH3(IRLST),  ZEPSH4(IRLST),                               
     S     ZEPSH5(IRLST),  ZEPSH6(IRLST),                               
     S     ZEPHIO(IRLST),  ZFACTC(IRLST),                               
     S     ZZFD(IRLST), ZFDD(IRLST), ZFDN(IRLST , IRLVP1)               
      REAL                                                              
     S     PFLUX (IRLST , 2 , IRLVP1)   ,                               
     S     PFLUC (IRLST , 2 , IRLVP1)   ,                               
     S     ZFLWP (IRLST)                ,                               
     S     PCTS  (IRLST , IRLEV)        ,                               
     S     ZZFU   (IRLST)               ,                               
     S     ZFUP  (IRLST , IRLVP1)       ,                               
     S     ZGLAYD(IRLST)                ,                               
     S     ZGLAYU(IRLST)                ,                               
     S     ZOMEGA(IRLST , 2 , IRLEV)                                    
C                                                                       
      INTEGER                                                           
     S     ITX   (IRLST)                ,                               
     S     IXDIA (IRLST , IRLVP1)       ,                               
     S     IXT   (IRLST , IRLVP1,IRLVP1),                               
     S     IXTSU (IRLST , IRLVP1)       ,                               
     S     IXTTP (IRLST , IRLVP1)       ,                               
     S     IMX   (IRLST)                ,                               
     S     IMXP  (IRLST)                                                
C                                                                       
      REAL ZOZ   (IRLST , IRLEV)        ,                               
     S     PPMB  (IRLST , IRLVP1)       ,                               
     S     ZRES  (IRLST)                ,                               
     S     PB    (IRLST, 6 , IRLVP1)    ,                               
     S     ZSSIG (IRLST , 3*NRLEV+1)    ,                               
     S     PTAU  (IRLST , 2 , IRLEV)    ,                               
     S     PTAVE (IRLST , IRLEV)        ,                               
     S     ZTAVIC(IRLST)                ,                               
     S     ZTI1  (IRLST)                ,                               
     S     PTL   (IRLST , IRLVP1)       ,                               
     S     ZTT   (IRLST , NTRA)         ,                               
     S     ZTT1  (IRLST , NTRA)         ,                               
     S     ZTT2  (IRLST , NTRA)         ,                               
     S     ZUAER (IRLST , NINT)         ,                               
     S     ZUPF  (IRLST , IRLVP1,IRLVP1),                               
     S     ZUU   (IRLST , NUA)          ,                               
     S     ZXD   (IRLST)                                                
      REAL                                                              
     S     ZXN   (IRLST)                ,                               
     S     ZXOZ  (IRLST)                ,                               
     S     ZXWV  (IRLST)                ,                               
     S     ZZZ   (IRLST)                ,                               
     S     ZRES2 (IRLST)                ,                               
     S     ZTI2  (IRLST)                ,                               
     S     ZBLAY (IRLST,IRLEV)          ,                               
     S     ZBLEV (IRLST,IRLVP1)         ,                               
     S     ZCTS  (IRLST)                ,                               
     S     ZHFG  (IRLST)                                                
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C                                                                       
      INTEGER KT    (IRLST)                                             
      REAL    PUU   (IRLST , NUA  )     ,                               
     S        PTT   (IRLST , NTRA )     ,                               
     S        ZZ    (IRLST)                                             
C                                                                       
C     -------------------------------------------------------- ---------
C                                                                       
      REAL  PCLFR  (IRLST , IRLEV)       ,                              
     S      PT     (IRLST , IRLVP1)       ,                             
     S      PQOF   (IRLST , IRLEV)          ,                           
     S      PQLWP  (IRLST , IRLEV)          ,                           
     S      PEMIS  (IRLST)                                              
      REAL                                                              
     S      PFLT   (IRLST , IRLVP1)                                     
C                                                                       
      REAL  ZALBSU(IRLST , 2 )           ,                              
     S      ZCG   (IRLST , 2 , IRLEV)    ,                              
     S      ZCLDLW(IRLST , IRLEV)        ,                              
     S      ZCLDSW(IRLST , IRLEV)        ,                              
     S      ZCOOLR(IRLST , IRLEV)        ,                              
     S      ZDT0  (IRLST)                ,                              
     S      ZFLUX (IRLST , 2 , IRLVP1)   ,                              
     S      ZPMB  (IRLST , IRLVP1)       ,                              
     S      ZTAU  (IRLST , 2 , IRLEV)    ,                              
     S      ZTAVE (IRLST , IRLEV)        ,                              
     S      ZTL   (IRLST , IRLVP1)       ,                              
     S      ZFSDWN(IRLST , IRLVP1)       ,                              
     S      ZFSUP (IRLST , IRLVP1)       ,                              
     S      ZHEATR(IRLST , IRLEV )                                      
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C                                                                       
      REAL   PDP   (IRLST , IRLEV)     ,                                
     S       PWV   (IRLST , IRLEV)     ,                                
     S       PFLS  (IRLST , IRLVP1)    ,                                
     S       PPSOL (IRLST        )     ,                                
     S       PRMU0 (IRLST        )     ,                                
     S       PAER  (IRLST,IRLEV,5)     ,                                
     S       PALBS (IRLST , 2 )        ,                                
     S       PCG   (IRLST , 2 , IRLEV) ,                                
     S       PCLDSW(IRLST , IRLEV)     ,                                
     S       POMEGA(IRLST , 2 , IRLEV) ,                                
     S       POZ   (IRLST , IRLEV)                                      
C                                                                       
      REAL   PFDOWN(IRLST , IRLVP1)    ,                                
     S       PFUP  (IRLST , IRLVP1)                                     
C                                                                       
      REAL   ZAKI  (IRLST , 2 )        ,                                
     S       ZCGAZ (IRLST , IRLEV)     ,                                
     S       ZCNEB (IRLST , IRLEV)     ,                                
     S       ZC1I  (IRLST , IRLVP1)    ,                                
     S       ZDSC  (IRLST , IRLEV)     ,                                
     S       ZDSH  (IRLST , IRLEV)     ,                                
     S       ZDSIG (IRLST , IRLEV)     ,                                
     S       ZFACT (IRLST)             ,                                
     S       ZFACT1 (IRLST), ZFACT2 (IRLST),                            
     S       ZFACT3 (IRLST), ZFACT4 (IRLST),                            
     S       ZFACT5 (IRLST), ZFACT6 (IRLST),                            
     S       ZFD   (IRLST , IRLVP1)    ,                                
     S       ZFRAC (IRLST)             ,                                
     S       ZFU   (IRLST , IRLVP1)    ,                                
     S       ZG    (IRLST)             ,                                
     S       ZGG   (IRLST)             ,                                
     S       ZPIZAZ(IRLST , IRLEV)     ,                                
     S       ZRAYL (IRLST),  ZRAY1 (IRLST , IRLVP1)                     
      REAL                                                              
     S       ZRAY2 (IRLST , IRLVP1)    ,                                
     S       ZREF  (IRLST)             ,                                
     S       ZREFZ (IRLST , 2 , IRLVP1),                                
     S       ZRE1  (IRLST)             ,                                
     S       ZRE2  (IRLST)             ,                                
     S       ZRJ   (IRLST , 6 , IRLVP1),                                
     S       ZRK   (IRLST , 6 , IRLVP1),                                
     S       ZRL   (IRLST , 8)         ,                                
     S       ZRMU  (IRLST)             ,                                
     S       ZRMUE (IRLST , IRLVP1)    ,                                
     S       ZRMUZ (IRLST)                                              
      REAL                                                              
     S       ZRNEB (IRLST)             ,                                
     S       ZRUEF (IRLST , 8)         ,                                
     S       ZR1   (IRLST)             ,                                
     S       ZR2   (IRLST)             ,                                
     S       ZR21  (IRLST)             ,                                
     S       ZR22  (IRLST)             ,                                
     S       ZR23  (IRLST)             ,                                
     S       ZS    (IRLST)             ,                                
     S       ZSEC  (IRLST)             ,                                
     S       ZSS1  (IRLST)             ,                                
     S       ZTAUAZ(IRLST , IRLEV)     ,                                
     S       ZTOT  (IRLST , IRLVP1)    ,                                
     S       ZTO1  (IRLST)             ,                                
     S       ZTR   (IRLST , 2 , IRLVP1),                                
     S       ZTRA1 (IRLST , IRLVP1)    ,                                
     S       ZTRA2 (IRLST , IRLVP1)                                     
      REAL                                                              
     S       ZTR1  (IRLST)             ,                                
     S       ZTR2  (IRLST)             ,                                
     S       ZUD   (IRLST , 3 , IRLVP1),                                
     S       ZUM   (IRLST , IRLVP1)    ,                                
     S       ZU1D  (IRLST)             ,                                
     S       ZU2D  (IRLST)             ,                                
     S       ZW    (IRLST)             ,                                
     S       ZN175 (IRLST)             ,                                
     S       ZN190 (IRLST)             ,                                
     S       ZO175 (IRLST)             ,                                
     S       ZO190 (IRLST)             ,                                
     S       ZP75  (IRLST)             ,                                
     S       ZP90  (IRLST)             ,                                
     S       ZSIGN (IRLST)             ,                                
     S       ZSIGO (IRLST)                                              
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
      REAL   PTR    (IRLST)     ,                                       
     S       PU     (IRLST)                                             
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
C   CLOUD-POINTERS                                                      
C   --------------                                                      
C                                                                       
      REAL   ZRH(NLP2,NLEV),                                            
     *       ZCH(NLP2),                                                 
     *       ZCM(NLP2),                                                 
     *       ZCL(NLP2),                                                 
     *       ZCT(NLP2),                                                 
     *       ZCC(NLP2)                                                  
      INTEGER NTOPCM(NLP2),                                             
     *        NBASECM(NLP2)                                             
      REAL   ARPRCM(NLP2)                                               
      INTEGER ITOPH(NLP2),                                              
     *        ITOPM(NLP2),                                              
     *        ITOPL(NLP2)                                               
      LOGICAL LOC(NLP2)                                                 
      REAL   ZQS(NLP2,NLEV),                                            
     *       ZCLC(NLP2,NLEV),                                           
     *       ZQLWP(NLP2,NLEV)                                           
C                                                                       
      REAL   VERVEL(NLP2,NLEV),                                         
     *       ZTHETA(NLP2,NLEV),                                         
     *       ZETA(NLP2,NLEV),                                           
     *       ZDTHMIN(NLP2)                                              
      INTEGER INVB(NLP2)                                                
      REAL   ZCTLWP(NLP2),                                              
     *       ZQLOC(NLP2),                                               
     *       ZQMEC(NLP2),                                               
     *       ZQHIC(NLP2)                                                
C                                                                       
C  definitions of "radheat"                                             
C                                                                       
      REAL   ZFSOT(NLON),                                               
     #       ZFTET(NLON),                                               
     #       ZFLT (NLON),                                               
     #       ZFSO (NLON),                                               
     #       ZFTE (NLON),                                               
     #       ZFLB (NLON),                                               
     #       ZDTDT(NLON),                                               
     #       HEATO(NLON,NLEV),                                          
     #       HEATE(NLON,NLEV),                                          
     #       HEATR(NLON,NLEV),                                          
     #       ZI0  (NLON)                                                
C                                                                       
C PIERS 13-8-97                                                         
C                                                                       
         REAL PIERSLWP                                                  
C                                                                       
C Piece of code needed to replace the Nikos radiation scheme with       
C the Morcrette radiation scheme in the igcm3.1.                        
C See morcrette3_1_1.readme for details.                                
C                                                                       
C                                                                       
c      program lh                                                       
c                                                                       
c      parameter(NL=23)                                                 
c      real P(NL),t(NL),h2o(NL),o3(NL),htlw(NL),htsw(NL)                
c      real swalb,doy                                                   
c       REAL CF(4,2)     !cloud fractions,lwps                          
c       INTEGER IC(4,2)   ! cloud positions                             
c       REAL RFLUXES(2,2,2)                                             
c      open(11,file='mls23.dat')                                        
c      read(11,*) NLEV                                                  
c      DOY=80                                                           
c      swalb=0.07                                                       
c      alat=40.0                                                        
c       doy=float(i)                                                    
c      do i=nlev,1,-1                                                   
c         read(11,*) P(i),t(i),h2o(i),adum,o3(i)                        
c         print *,i,p(i),t(i),h2o(i),o3(i)                              
c      enddo                                                            
c                                                                       
c       ic(1,1)=3                                                       
c      ic(1,2)=17                                                       
c       ic(3,1)=6                                                       
c       ic(2,2)=6                                                       
c       cf(1,1)=0.3                                                     
c       cf(1,2)=45.0                                                    
c        cf(3,1)=0.8                                                    
c       cf(4,2)=20.0                                                    
c      alon=0.0                                                         
c      call morcigcm(p,t,h2o,o3,alat,htlw,htsw,doy,cf,ic,               
c     $     rfluxes,swalb,alon)                                         
c      enddo                                                            
c      do i=nl,1,-1                                                     
c         print '(i6,4F14.4)',i,p(i),t(i),htsw(i),htlw(i)               
c      enddo                                                            
c      print *,RFLUXES(1,1,1), RFLUXES(1,1,2), RFLUXES(1,2,1),          
c     $     RFLUXES(1,2,2)                                              
c      print *,RFLUXES(2,1,1), RFLUXES(2,1,2), RFLUXES(2,2,1),          
c     $     RFLUXES(2,2,2)                                              
c                                                                       
c                                                                       
c      end                                                              
c      INCLUDE 'zparb.upd'                                              
      REAL ZTM1(NLON,NLEV)                                              
      integer ipass                                                     
      data ipass/1/                                                     
C                                                                       
C----------------------------------------------------------------       
C                                                                       
C     ----------------------------------------------------------        
C                                                                       
C                                                                       
CDIR$ VFUNCTION ALOG,EXP                                                
C     ----------------------                                            
C                                                                       
C    -----------------------------------------------------------        
C     ----------------------------------------------------------        
C                                                                       
C*       0.3   SET-UP RADIATION ROUTINE COEFFICIENTS                    
C              -------------------------------------                    
C                                                                       
  30  CONTINUE                                                          
                                                                        
      IF (IPASS.eq.1) THEN                                              
       IPASS=0                                                          
      CALL SUAER                                                        
      CALL SULW                                                         
      CALL SUSW                                                         
       ENDIF                                                            
C                                                                       
      IMP=2                                                             
ccc1      NOUT=6                                                        
cc      NOUT=2                                                          
cc      open(2,FILE='termout.dat',STATUS='unknown')                     
      IABS=5                                                            
      IAER=0                                                            
      LORAD = .TRUE.                                                    
C                                                                       
      ZEPSC  = 1.E-04                                                   
      ZEPSCO = 1.E-10                                                   
      ZEPSCQ = 1.E-07                                                   
      ZEPSCT = 1.E-04                                                   
      ZEPSCW = 1.E-12                                                   
      ZEELOG = 1.E-10                                                   
C                                                                       
      CDAY   = DAYL * G / CPD                                           
      CH2O   = 5.3669274E-03                                            
      CCO2   = 5.8269497E-03                                            
      DIFF   = 1.66                                                     
      X10E   = 0.4342945                                                
      CCLWMR = 0.2                                                      
C                                                                       
C     ---------------------------------------------------------         
C                                                                       
C*         1.     SET-UP INPUT QUANTITIES FOR RADIATION                 
C                 -------------------------------------                 
C                                                                       
 100  CONTINUE                                                          
      DO 101 JL=1,NRLST                                                 
       DO 102 JK=1,NRLEV                                                
        PCLFR(JL,JK) = CLC (JL,JK)                                      
        PT   (JL,JK) = TI  (JL,JK)                                      
        PDP  (JL,JK) = DP  (JL,JK)                                      
        PWV  (JL,JK) = WV  (JL,JK)                                      
        PQOF (JL,JK) = QOF (JL,JK)                                      
C        PQCF (JL,JK) = QCF (JL,JK)                                     
        PQLWP(JL,JK) = CLWA(JL,JK)                                      
        PAER (JL,JK,1) = AEQ1(JL,JK)                                    
        PAER (JL,JK,2) = AEQ2(JL,JK)                                    
        PAER (JL,JK,3) = AEQ3(JL,JK)                                    
        PAER (JL,JK,4) = AEQ4(JL,JK)                                    
        PAER (JL,JK,5) = AEQ5(JL,JK)                                    
 102   CONTINUE                                                         
       PT (JL,NRLEV+1) = TI(JL,NRLEV+1)                                 
       PPSOL (JL) = APRE(JL)                                            
c       print *, ' ppsol ', ppsol(jl)                                   
       PRMU0 (JL) = SMU0(JL)                                            
       PEMIS (JL) = ALTE(JL)                                            
       PALBSN(JL) = ALSO(JL)                                            
 101  CONTINUE                                                          
C                                                                       
C                                                                       
C*         1.1    INITIALIZE VARIOUS FIELDS                             
C                 -------------------------                             
C                                                                       
 110  CONTINUE                                                          
C                                                                       
      DO 112 JK = 1 , KFLEV                                             
      DO 111 JL = 1 , KDLON                                             
      ZCOOLR(JL,JK) = 0.                                                
      ZHEATR(JL,JK) = 0.                                                
 111  CONTINUE                                                          
 112  CONTINUE                                                          
C                                                                       
      DO 113 JL = 1 , KDLON                                             
         ZALBSU(JL,1)=PALBSN(JL)                                        
         ZALBSU(JL,2)=PALBSN(JL)                                        
         ZFSUP(JL,KFLEV+1) = 0.                                         
         ZFSDWN(JL,KFLEV+1) = ZEPSCO                                    
         ZFLUX(JL,1,KFLEV+1) = 0.                                       
         ZFLUX(JL,2,KFLEV+1) = 0.                                       
         PEMIS(JL) = 1. - PEMIS(JL)                                     
         ZPMB(JL,1) = PPSOL(JL)/100.                                    
c         print *,' zpmb ',zpmb(jl,1)                                   
         ZTL(JL,KFLEV+1) = PT(JL,1)                                     
         ZDT0(JL) = 0.                                                  
 113  CONTINUE                                                          
C                                                                       
      DO 115 JK = 1 , KFLEV                                             
         JKP1 = JK + 1                                                  
         JKL = KFLEV+ 1 - JK                                            
         JKLP1 = JKL + 1                                                
         DO 114 JL = 1 , KDLON                                          
         ZPMB(JL,JK+1) = ZPMB(JL,JK) - PDP(JL,JKL) / 100.               
         ZTL(JL,JK) = PT(JL,JKLP1)                                      
C         ZTAVE(JL,JK) = 0.5*(PT(JL,JKL)+PT(JL,JKLP1))                  
         ZTAVE(JL,JK) = ZTM1(JL,JKL)                                    
         ZOZ(JL,JK)   = PQOF(JL,JKL) * 46.6968 / G                      
         ZCLDSW(JL,JK) = PCLFR(JL,JKL)                                  
         ZCLDLW(JL,JK) = PCLFR(JL,JKL)                                  
                                                                        
         ZTAU(JL,1,JK) = ZEPSCW                                         
         ZTAU(JL,2,JK) = ZEPSCW                                         
         ZOMEGA(JL,1,JK) = 0.9994                                       
         ZOMEGA(JL,2,JK) = 0.9963                                       
         ZCG(JL,1,JK) = 0.865                                           
         ZCG(JL,2,JK) = 0.910                                           
         ZCOOLR(JL,JK) = 0.                                             
         ZHEATR(JL,JK) = 0.                                             
         ZFSUP(JL,JK) = 0.                                              
         ZFSDWN(JL,JK) = 0.                                             
         ZFLUX(JL,1,JK) = 0.                                            
         ZFLUX(JL,2,JK) = 0.                                            
 114     CONTINUE                                                       
 115  CONTINUE                                                          
C                                                                       
C     --------------------------------------------------------          
C                                                                       
C*         2.     CLOUD AND AEROSOL PARAMETERS                          
C                 ----------------------------                          
C                                                                       
 200  CONTINUE                                                          
C                                                                       
      PIERSLWP=0.0                                                      
      DO 202 JK = 2 , KFLEV-1                                           
         JKL = KFLEV + 1 - JK                                           
         DO 201 JL = 1 , KDLON                                          
         LO1 = PCLFR(JL,JKL).GT.ZEPSC                                   
         ZLWGKG = CVMGT(PQLWP(JL,JKL)*1000./PCLFR(JL,JKL),0.D0,LO1)     
         ZFCCA = MIN( PCLFR(JL,JKL) , PCLFR(JL,JKL-1) )                 
         ZFCCB = MIN( PCLFR(JL,JKL) , PCLFR(JL,JKL+1) )                 
         ZFCC  = MAX( ZFCCA , ZFCCB )                                   
         ZFCC = 0.0                                                     
         ZLWGKG = ZFCC * CCLWMR + (PCLFR(JL,JKL) - ZFCC) * ZLWGKG       
         ZFLWP(JL) = CVMGT( ZLWGKG*PDP(JL,JKL)/(G*PCLFR(JL,JKL)) ,      
     S                    ZEPSCW , LO1 )                                
         PIERSLWP=PIERSLWP+ZFLWP(JL)                                    
         ZCLDSW(JL,JK) = PCLFR(JL,JKL)                                  
         ZCLDLW(JL,JK) = PCLFR(JL,JKL)*(1. - EXP(-0.158 * ZFLWP(JL)))   
         ZRADEF = 15.0                                                  
         ZTAUEQ = 1.5 * ZFLWP(JL) / ZRADEF                              
         ZTAU(JL,1,JK) = ZTAUEQ                                         
         ZTAU(JL,2,JK) = ZTAUEQ                                         
         ZOMEGA(JL,1,JK) = 0.9999 - 5.0E-04*EXP(-0.5 * ZTAUEQ)          
         ZOMEGA(JL,2,JK) = 0.9988 - 2.5E-03*EXP(-0.05 * ZTAUEQ)         
         ZCG(JL,1,JK)=0.865                                             
         ZCG(JL,2,JK)=0.910                                             
 201     CONTINUE                                                       
 202  CONTINUE                                                          
C                                                                       
      DO 203 JL = 1 , KDLON                                             
      ZPMB(JL,KFLEV+1)=0.0                                              
      ZTAVE(JL,KFLEV)=ZTL(JL,KFLEV+1)                                   
 203  CONTINUE                                                          
C                                                                       
      NUAER = NUA                                                       
      NTRAER = NTRA                                                     
C                                                                       
      IF (IAER.EQ.0) THEN                                               
C         NUAER = 13                                                    
C         NUAER = 13                                                    
C         NTRAER = 14                                                   
         DO 206 JK = 1 , KFLEV                                          
         DO 205 JAE = 1 , 5                                             
         DO 204 JL = 1 , KDLON                                          
         PAER(JL,JK,JAE)=1.E-15                                         
 204     CONTINUE                                                       
 205     CONTINUE                                                       
 206     CONTINUE                                                       
      END IF                                                            
C                                                                       
C                                                                       
C      END IF                                                           
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C*         3.     CALL LONGWAVE RADIATION CODE                          
C                 ----------------------------                          
C                                                                       
 300  CONTINUE                                                          
c      print *,' hi ',kflev+1,ZPMB(1,KFLEV+1),zpmb(1,1)                 
c      DO JK=KFLEV,1,-1                                                 
c       JKL=KFLEV+1-JK                                                  
c        print *,jk,zpmb(1,jk),pdp(1,jkl),ztave(1,jk),                  
c     $      qof(1,jkl)/pdp(1,jkl),wv(1,jkl),pqlwp(1,jkl)               
c      enddo                                                            
c zflux is as tave and zpmb with level 1 the surface                    
C                                                                       
      CALL LW ( KDLON, KRLST, KFLEV, KFLVP1, KFLEV2, KNGL, KNGLP1,      
     S          PSIG, PSCT, PCARDI, ITASK                         )     
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C*         4.     CALL SHORTWAVE RADIATION CODE                         
C                 -----------------------------                         
C                                                                       
 400  CONTINUE                                                          
C                                                                       
      ZZRMUZ=0.                                                         
      DO 401 JL = 1 , KDLON                                             
         ZZRMUZ = MAX(ZZRMUZ, PRMU0(JL))                                
 401  CONTINUE                                                          
C                                                                       
      IF (ZZRMUZ.GT.0.) THEN                                            
C                                                                       
         CALL SW ( KDLON, KRLST, KFLEV, KFLVP1, PSIG, PSCT, PCARDI,     
     S             ITASK )                                              
C                                                                       
      END IF                                                            
C                                                                       
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*         5.     FILL UP THE MODEL NET LW AND SW RADIATIVE FLUXES      
C                 ------------------------------------------------      
C                                                                       
 500  CONTINUE                                                          
C                                                                       
      DO 501 JL = 1 , KDLON                                             
       PEMIS(JL) = 1. - PEMIS(JL)                                       
 501  CONTINUE                                                          
C                                                                       
      DO 503 JKL = 1 , KFLEV+1                                          
         JK = KFLVP1 + 1 - JKL                                          
         DO 502 JL = 1 , KDLON                                          
            FLS(JL,JKL) = ZFSDWN(JL,JK) - ZFSUP(JL,JK)                  
            FLT(JL,JKL) = - ZFLUX(JL,1,JK) - ZFLUX(JL,2,JK)             
 502     CONTINUE                                                       
 503  CONTINUE                                                          
C                                                                       
C                                                                       
C*         5.1     HEATING/COOLING RATES (K/DAY)                        
C                  -----------------------------                        
 510  CONTINUE                                                          
C                                                                       
      DO 512 JK = 1 , KFLEV                                             
      JKL = KFLEV+1 - JK                                                
      DO 511 JL = 1 , KDLON                                             
      ZDFNET = ZFLUX(JL,1,JK+1) + ZFLUX(JL,2,JK+1)                      
     S              -ZFLUX(JL,1,JK  ) - ZFLUX(JL,2,JK  )                
      ZCOOLR(JL,JK) = CDAY * ZDFNET / PDP(JL,JKL)                       
      ZDFNET = ZFSUP(JL,JK  ) - ZFSDWN(JL,JK  )                         
     S        -ZFSUP(JL,JK+1) + ZFSDWN(JL,JK+1)                         
      ZHEATR(JL,JK) = CDAY * ZDFNET / PDP(JL,JKL)                       
 511  CONTINUE                                                          
 512  CONTINUE                                                          
CCC  Combine the lowerest two layer as one layer to correspond          
CCC  the radiative convective model. (leave the lowerest level          
CCC  heating rate undefined---set to zero)   D.LI 8/8 1991              
      DO 518 JL = 1, KDLON                                              
      ZDFNET = ZFLUX(JL,1,3) + ZFLUX(JL,2,3)                            
     S        -ZFLUX(JL,1,1) - ZFLUX(JL,2,1)                            
      ZCOOLR(JL,2) = CDAY*ZDFNET/(PDP(JL,KFLEV-1)+PDP(JL,KFLEV))        
      ZCOOLR(JL,1)=0.0D0                                                
      ZDFNET = ZFSUP(JL,1) - ZFSDWN(JL,1)                               
     S        -ZFSUP(JL,3) + ZFSDWN(JL,3)                               
      ZHEATR(JL,2) = CDAY*ZDFNET/(PDP(JL,KFLEV-1)+PDP(JL,KFLEV))        
      ZHEATR(JL,1)=0.0D0                                                
 518  CONTINUE                                                          
CCC                                                                     
C                                                                       
c      IF (IMP.LT.4) THEN                                               
c       JL = 1                                                          
c       WRITE(NOUT,891) (ZFLUX(JL,1,JK),JK = 1 , KFLVP1)                
c       WRITE(NOUT,891) (ZFLUX(JL,2,JK),JK = 1 , KFLVP1)                
c       WRITE(NOUT,889) (ZCOOLR(JL,JK),JK = 1 , KFLEV)                  
c      WRITE(NOUT,891) (ZFSUP(JL,JK),JK = 1 , KFLVP1)                   
c      WRITE(NOUT,891) (ZFSDWN(JL,JK),JK = 1 , KFLVP1)                  
c      WRITE(NOUT,889) (ZHEATR(JL,JK),JK = 1 , KFLEV)                   
C      END IF                                                           
C                                                                       
C-----------------------------------------------------------------      
C                       FORMATS                                         
C                       -------                                         
C                                                                       
C                                                                       
 883  FORMAT(2X,16E8.2)                                                 
 884  FORMAT(4X,18F7.4)                                                 
 885  FORMAT(4X,18F7.1)                                                 
 887  FORMAT(1X,10E12.6)                                                
 889  FORMAT(4X,18F7.3)                                                 
 891  FORMAT(1X,18F7.1)                                                 
C                                                                       
C     ------------------------------------------------------            
C                                                                       
      RETURN                                                            
      END                                                               
C                                                                       
      SUBROUTINE SW ( KDLON,KRLST,KFLEV,KFLVP1,PSIG,PSCT,PCARDI,ITASK ) 
C                                                                       
C**** *SW* - COMPUTES THE SHORTWAVE RADIATION FLUXES.                   
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C           COMPUTES THE SHORTWAVE RADIATION FLUXES IN TWO SPECTRAL     
C     INTERVALS FOLLOWING FOUQUART AND BONNEL (1980).                   
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C          *SW* IS CALLED BY *RADLSW*                                   
C                                                                       
C                                                                       
C        IMPLICIT ARGUMENTS                                             
C        --------------------                                           
C                                                                       
C     ==== INPUTS ===                                                   
C     ==== OUTPUTS ===                                                  
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C          1. COMPUTES ABSORBER AMOUNTS WITH TEMPERATURE AND PRESSURE   
C     SCALING.                                                          
C          2. COMPUTES UPWARD AND DOWNWARD FLUXES IN THE 0.25-0.68      
C     MICRON SPECTRAL INTERVAL.                                         
C          3. COMPUTES UPWARD AND DOWNWARD FLUXES IN THE 0.68-4.0       
C     MICRON SPECTRAL INTERVAL.                                         
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          *DEDD*, *SWTT*                                               
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C        SEE RADIATION'S PART OF THE ECMWF RESEARCH DEPARTMENT          
C        DOCUMENTATION, AND FOUQUART AND BONNEL (1980)                  
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C        JEAN-JACQUES MORCRETTE  *ECMWF*                                
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C        ORIGINAL : 88-12-15                                            
C     ----------------------------------------------------------------  
C                                                                       
C      IMPLICIT LOGICAL (L)                                             
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRSW* SWITCHES RELATED TO THE RADIATION SCHEME.         
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      IMPLICIT REAL (A-H,O-Z)                                           
C      IMPLICIT UNDEFINED (Z)                                           
C      IMPLICIT LOGICAL (L)                                             
C      IMPLICIT UNDEFINED (L)                                           
      IMPLICIT INTEGER (I-K,M-N)                                        
C      IMPLICIT UNDEFINED (I-K,M-N)                                     
C      IMPLICIT REAL (P)                                                
C      IMPLICIT UNDEFINED (A-H,O,Q-Y)                                   
C                                                                       
C everything except the parameters is defined                           
C in the common blocks                                                  
C                                                                       
      COMMON/COM1/                                                      
C     I NRADFR, NRPART, NRINT, NRADPFR, NRADPLA,                        
C     I NRLSTI,                                                         
     R XLAT,                                                            
     R RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR, ZCCO2,              
C     R PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3,                 
     L LOH, LOM, LOL, LORAD                                             
      COMMON/COM2/                                                      
     I ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,                  
     I         JLOOP, KRLEV,                                            
     I        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     I        ITH, ITM, ITL,                                            
     I        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     I        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     I        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     I        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      COMMON/COM3/                                                      
     I IND4, KM12, KXD, KXU, IG, KBS, KDD,                              
     I        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     I        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     I        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     I        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     I        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
      COMMON/COM4/                                                      
     R           COZQC,COZQS,COZHC,COZHS,                               
     R CSUPSAT,                                                         
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
      COMMON/COM5/                                                      
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NUMVC,                                                           
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      COMMON/COM6/                                                      
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD,       
     R DTIME,    TWODT,    EPS,                                         
     R SINLON,     COSLON,                                              
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2,                          
     I NFAX,                                                            
     R CRAE,                                                            
     R CDISSEM                                                          
      COMMON/COM7/                                                      
     R     CCRH,                                                        
     L LO,LO1,LO2,LODIA,LODIAP,                                         
     R TAUA, PIZA, CGA, CAER, CAEROS                                    
      COMMON/COM7A/                                                     
     R  RT1,WG1,AT,BT,CNTNU,O1H,O2H,PIALF0                              
     S ,OCT,TREF,XP,TSTAND,GA,GB,CE,ETY,PTY                             
      COMMON/COM7B/                                                     
     S TINTP,TSTP,                                                      
     S  TBO3,AO3,BO3,TB,ALP,BET,ULG,ULGO,PLG,AP,BP,TO3B,O3A,O3B,        
     R          CDAY,CCO2,CH2O,DIFF,X10E                                
       COMMON/COM7C/                                                    
     S  ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW,                       
     R  APAD,BPAD,D,CRAY,SUN,                                           
     R   TTT,                                                           
     R   TM1                                                            
      COMMON/COM8/                                                      
     R       TSM1M,                                                     
     R       QM1,                                                       
     R       SNM1M,                                                     
     R       ALBM,                                                      
     R       AVVELM,                                                    
     R   SLMM,                                                          
     R       APHM1,                                                     
     R       APM1                                                       
      COMMON/COM9/                                                      
     L LOLAND,                                                          
     R   ZTI,                                                           
     *       ZCTI,                                                      
     *       ZCPH,                                                      
     *       ZWV,                                                       
     *       ZSW,                                                       
     *       ZALTE,                                                     
     *       ZCALTE,                                                    
     *       ZALSO                                                      
      COMMON/COM10/                                                     
     *       ZCALSO,                                                    
     R   ZOZQ,                                                          
     *       ZOZH,                                                      
     R   ZTRSOL,                                                        
     *       ZEMTER,                                                    
     R   TRSOLM,                                                        
     *       EMTERM,                                                    
     R   AMU0,                                                          
     *       RDAYL                                                      
      COMMON/COM11/                                                     
     R   CLC,                                                           
     *       TI,                                                        
     *       DP,                                                        
     *       AADS,                                                      
     *       AEQ1,                                                      
     *       AEQ2,                                                      
     *       AEQ3,                                                      
     *       AEQ4,                                                      
     *       AEQ5,                                                      
     *       SSW                                                        
      COMMON/COM12/                                                     
     *       WV,                                                        
     *       QOF,                                                       
     *       QCF,                                                       
     *       CLWA,                                                      
     *       APRE,                                                      
     *       SMU0,                                                      
     *       ALTE,                                                      
     *       ALSO                                                       
      COMMON/COM13/                                                     
     R   ZDPO,                                                          
     *       ZQCFO,                                                     
     *       ZAEQSO,                                                    
     *       ZQOFO,                                                     
     *       ZDPN,                                                      
     *       ZQCFN,                                                     
     *       ZAEQSN,                                                    
     *       ZQOFN                                                      
      COMMON/COM14/                                                     
     R   FLT,                                                           
     *       FLS,                                                       
     R   DIAG,                                                          
     *       DIA1,                                                      
     *       ZDIAG,                                                     
     *       ZDIA1,                                                     
     *       ZDIA,                                                      
     *       ZDIAF,                                                     
     *       ZDIAT,                                                     
     R PGG                                                              
      COMMON/COM15/                                                     
     #     PREF,                                                        
     #     PRE1,                                                        
     #     PRE2,                                                        
     #     PRMUZ,                                                       
     #     PTO1,                                                        
     #     PTR1,                                                        
     #     PTR2,                                                        
     #     PW,                                                          
     R     PALBSN                                                       
      COMMON/COM16/                                                     
     R ZABSCU,                                                          
     S     ZABSLY,                                                      
     S     ZADJD ,                                                      
     S     ZADJU ,                                                      
     S     ZBGND ,                                                      
     S     ZBINT ,                                                      
     S     ZBSUR ,                                                      
     S     ZBSUIN,                                                      
     S     ZBTOP ,                                                      
     S     PCLDLW,                                                      
     S     ZCLOUD,                                                      
     S     ZCNTRB,                                                      
     S     ZCUMP ,                                                      
     S     ZCUMPT,                                                      
     S     ZDBDT                                                        
      COMMON/COM17/                                                     
     S     ZDBSL ,                                                      
     S     ZDISTD,                                                      
     S     ZDISTU,                                                      
     S     ZDNF  ,                                                      
     S     PDT0  ,                                                      
     S     ZDUC                                                         
      COMMON/COM18/                                                     
     S     ZEFFT ,                                                      
     S     ZEMU  ,                                                      
     S     ZEPHIO,                                                      
     S     ZEPSC2,                                                      
     S     ZEPSC3,                                                      
     S     ZEPSH1,                                                      
     S     ZEPSH2,                                                      
     S     ZEPSH3                                                       
      COMMON/COM19/                                                     
     S     ZEPSH4,                                                      
     S     ZEPSH5,                                                      
     S     ZEPSH6,                                                      
     S     ZEPSIO,                                                      
     S     ZFACTC,                                                      
     S     ZZFD  ,                                                      
     S     ZFDD  ,                                                      
     S     ZFDN                                                         
      COMMON/COM20/                                                     
     S     PFLUX , PFLUC,                                               
     S     ZFLWP , PCTS                                                 
     S     ZZFU  ,                                                      
     S     ZFUP  ,                                                      
     S     ZGLAYD,                                                      
     S     ZGLAYU,                                                      
     S     ZOMEGA,                                                      
     S     ITX   ,                                                      
     S     IXDIA ,                                                      
     S     IXT   ,                                                      
     S     IXTSU ,                                                      
     S     IXTTP ,                                                      
     S     IMX   ,                                                      
     S     IMXP                                                         
      COMMON/COM21/                                                     
     R ZOZ   ,                                                          
     S     PPMB  ,                                                      
     S     ZRES  , PB                                                   
     S     ZSSIG ,                                                      
     S     PTAU  ,                                                      
     S     PTAVE ,                                                      
     S     ZTAVIC,                                                      
     S     ZTI1  ,                                                      
     S     PTL                                                          
      COMMON/COM22/                                                     
     S     ZTT   ,                                                      
     S     ZTT1  ,                                                      
     S     ZTT2  ,                                                      
     S     ZUAER ,                                                      
     S     ZUPF  ,                                                      
     S     ZUU   ,                                                      
     S     ZXD                                                          
      COMMON/COM23/                                                     
     S     ZXN   ,                                                      
     S     ZXOZ  ,                                                      
     S     ZXWV  ,                                                      
     S     ZZZ   ,                                                      
     S     ZRES2 ,                                                      
     S     ZTI2  ,                                                      
     S     ZBLAY ,                                                      
     S     ZBLEV ,                                                      
     S     ZCTS  ,                                                      
     S     ZHFG                                                         
      COMMON/COM24/                                                     
     I KT,                                                              
     R    PUU   ,                                                       
     S        PTT   ,                                                   
     S        ZZ                                                        
     R  PCLFR  ,                                                        
     S      PT     ,                                                    
     S      PQOF   ,                                                    
     S      PQLWP  ,                                                    
     S      PEMIS  ,                                                    
     S      PFLT                                                        
      COMMON/COM25/                                                     
     R  ZALBSU,                                                         
     S      ZCG   ,                                                     
     S      ZCLDLW,                                                     
     S      ZCLDSW,                                                     
     S      ZCOOLR,                                                     
     S      ZDT0  ,                                                     
     S      ZEMD  ,                                                     
     S      ZFLUX ,                                                     
     S      ZPMB  ,                                                     
     S      ZTAU  ,                                                     
     S      ZTAVE ,                                                     
     S      ZTL   ,                                                     
     S      ZFSDWN                                                      
      COMMON/COM26/                                                     
     S      ZFSUP ,                                                     
     S      ZHEATR,                                                     
     R   PDP   ,                                                        
     S       PWV   ,                                                    
     S       PFLS  ,                                                    
     S       PPSOL ,                                                    
     S       PRMU0 ,                                                    
     S       PAER  ,                                                    
     S       PALBS ,                                                    
     S       PCG   ,                                                    
     S       PCLDSW,                                                    
     S       POMEGA,                                                    
     S       POZ                                                        
      COMMON/COM27/                                                     
     R   PFDOWN,                                                        
     S       PFUP  ,                                                    
     R   ZAKI  ,                                                        
     S       ZCGAZ ,                                                    
     S       ZCNEB ,                                                    
     S       ZC1I  ,                                                    
     S       ZDSC  ,                                                    
     S       ZDSH                                                       
      COMMON/COM28/                                                     
     S       ZDSIG ,                                                    
     S       ZFACT ,                                                    
     S       ZFACT1 ,                                                   
     S       ZFACT2 ,                                                   
     S       ZFACT3 ,                                                   
     S       ZFACT4 ,                                                   
     S       ZFACT5 ,                                                   
     S       ZFACT6 ,                                                   
     S       ZFD   ,                                                    
     S       ZFRAC ,                                                    
     S       ZFU   ,                                                    
     S       ZG    ,                                                    
     S       ZGG   ,                                                    
     S       ZPIZAZ                                                     
      COMMON/COM29/                                                     
     S       ZRAYL ,                                                    
     S       ZRAY1 ,                                                    
     S       ZRAY2 ,                                                    
     S       ZREF  ,                                                    
     S       ZREFZ ,                                                    
     S       ZRE1  ,                                                    
     S       ZRE2  ,                                                    
     S       ZRJ   ,                                                    
     S       ZRK   ,                                                    
     S       ZRL   ,                                                    
     S       ZRMU  ,                                                    
     S       ZRMUE ,                                                    
     S       ZRMUZ                                                      
      COMMON/COM30/                                                     
     S       ZRNEB ,                                                    
     S       ZRUEF ,                                                    
     S       ZR1   ,                                                    
     S       ZR2   ,                                                    
     S       ZR21  ,                                                    
     S       ZR22  ,                                                    
     S       ZR23  ,                                                    
     S       ZS    ,                                                    
     S       ZSEC  ,                                                    
     S       ZSS1  ,                                                    
     S       ZTAUAZ                                                     
      COMMON/COM31/                                                     
     S       ZTOT  ,                                                    
     S       ZTO1  ,                                                    
     S       ZTR   ,                                                    
     S       ZTRA1 ,                                                    
     S       ZTRA2 ,                                                    
     S       ZTR1                                                       
      COMMON/COM32/                                                     
     S       ZTR2  ,                                                    
     S       ZUD   ,                                                    
     S       ZUM   ,                                                    
     S       ZU1D  ,                                                    
     S       ZU2D  ,                                                    
     S       ZW    ,                                                    
     S       ZN175 ,                                                    
     S       ZN190 ,                                                    
     S       ZO175 ,                                                    
     S       ZO190 ,                                                    
     S       ZP75  ,                                                    
     S       ZP90  ,                                                    
     S       ZSIGN ,                                                    
     S       ZSIGO                                                      
      COMMON/COM33/                                                     
     R   PTR,                                                           
     S       PU,                                                        
     R   ZRH,                                                           
     *       ZCH,                                                       
     *       ZCM,                                                       
     *       ZCL,                                                       
     *       ZCT,                                                       
     *       ZCC,                                                       
     I NTOPCM                                                           
      COMMON/COM34/                                                     
     *        NBASECM,                                                  
     R   ARPRCM,                                                        
     I ITOPH,                                                           
     *        ITOPM,                                                    
     *        ITOPL,                                                    
     L LOC,                                                             
     R   ZQS,                                                           
     *       ZCLC,                                                      
     *       ZQLWP                                                      
      COMMON/COM35/                                                     
     R   VERVEL,                                                        
     *       ZTHETA,                                                    
     *       ZETA,                                                      
     *       ZDTHMIN,                                                   
     I INVB,                                                            
     R   ZCTLWP,                                                        
     *       ZQLOC,                                                     
     *       ZQMEC,                                                     
     *       ZQHIC,                                                     
     R   ZFSOT,                                                         
     #       ZFTET,                                                     
     #       ZFLT                                                       
      COMMON/COM36/                                                     
     #       ZFSO ,                                                     
     #       ZFTE ,                                                     
     #       ZFLB ,                                                     
     #       ZDTDT,                                                     
     #       HEATO,                                                     
     #       HEATE,                                                     
     #       HEATR,                                                     
     #       ZI0                                                        
C                                                                       
      INTEGER                                                           
C     I NRADFR,   NRPART,   NRINT,    NRLSTI,  NRADPFR,  NRADPLA,       
     I NRLST,    NRLEV,    NRLEVP1, NLEV, NLP2, NLEVP1,                 
     I NLON, IRLEV, NTRA, NLAT, ITIM, NUA, NINT, NRPARTI                
C                                                                       
C   NRLEV .... number of vertical levels                                
C   NRLST .... number of points on a latitude line                      
C   NUA   .... number of absorber amount                                
C   NINT  .... number of spectral intervals                             
C   NTRA  .... number of transmission functions                         
C   NLAT  .... number of latitude points                                
C   ITIM  .... number of month under consideration                      
C                                                                       
C      PARAMETER (NRLEV=122,NUA=34,NINT=6,NTRA=34, NLAT=1,              
      PARAMETER (NRLEV=21,NUA=34,NINT=6,NTRA=34, NLAT=1,                
cccm      PARAMETER (NRLEV=37,NUA=18,NINT=6,NTRA=19, NLAT=1,            
cccm     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=10,NLEV=NRLEV,        
     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=7,NLEV=NRLEV,             
     #           NLP2=NRLST, NLEVP1=NRLEVP1, NLON=NRLST,                
     #           NRPARTI=1)                                             
C                                                                       
      REAL YCLOCK, XLAT(NLAT), Z0, DZ, ALBMPAR, SNPAR,                  
     #     TSUPAR, CTFREEZ, DELTAC, DELTAO, DELTAH                      
C                                                                       
C   YTIME  .... time of the year in radians                             
C   YCLOCK .... time of the day in radians                              
C   XLAT   .... latitudes in radians                                    
C   Z0     .... zero point of height grid                               
C   DZ     .... Interval between height levels                          
C   ALBMPAR.... sw surface albedo as function of longitude:             
C           yearly averaged sea-ice: 0.55                               
C                           water  : 0.07                               
C                           land   : 0.07.lt.ALBM.lt.0.80               
C                                    depending on snow                  
C   SNPAR  .... snow depth in meter                                     
C   TSUPAR .... surface temperature (Kelvin)                            
C   CTFREEZ.... freezing temperature of sea-water (Kelvin)              
C   DELTAC .... Doppler parameter for line shapes of CO2                
C   DELTAO .... Doppler parameter for line shapes of O3                 
C                                                                       
      PARAMETER (Z0=0., CTFREEZ=271.26,                                 
C     PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
     #           DZ=0.289, ALBMPAR=0.2, SNPAR=0.,                       
cccm  PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
cccm     #           DZ=0.15, ALBMPAR=0.3, SNPAR=0.,                    
CZ     #           DELTAC=0., DELTAO=0.)                                
     #           DELTAC=6.0E-04, DELTAO=1.E-02, DELTAH=5.0E-04)         
C      PARAMETER (YCLOCK=0., Z0=0., TSUPAR=273., CTFREEZ=271.26,        
C     #           DZ=0.46875, ALBMPAR=0.07, SNPAR=0.)                   
C      PARAMETER (YCLOCK=0., Z0=2.608, TSUPAR=273., CTFREEZ=271.26,     
C     #           DZ=0.2878, ALBMPAR=0.07, SNPAR=0.)                    
C                                                                       
      LOGICAL LDIUR, LOLPAR                                             
C                                                                       
C   LDIUR .... true: sun at time, false: diurnally averaged             
C   LOLPAR.... true: land surface, false: else                          
C                                                                       
C      PARAMETER (LDIUR=.TRUE., LOLPAR=.TRUE.)                          
       PARAMETER (LOLPAR=.TRUE.)                                        
C                                                                       
      INTEGER  C2TEMP                                                   
      CHARACTER*64  MIDTMP                                              
cccm      PARAMETER (C2TEMP=10,                                         
cccm     #    MIDTMP='/home/atm/ugamp/midrad/radfildat.dir/MIDTMP.DAT') 
ccc1      PARAMETER (C2TEMP=10, MIDTMP='tprfil2.dat')                   
C                                                                       
      INTEGER  IRLONI, IRLST, IRLVP1, JPNLEV, JPNLVP1                   
C                                                                       
      PARAMETER (IRLONI=NRLST, IRLST=NRLST,                             
     #           IRLVP1=NRLEVP1, JPNLEV=NRLEV,                          
     #           JPNLVP1=NRLEVP1, IRLEV=NRLEV)                          
C                                                                       
      INTEGER NIPD, NIPD2, NTR, NG1, NG1P1                              
      PARAMETER(NIPD=8,NIPD2=16,NTR=11,NG1=2,NG1P1=3)                   
C                                                                       
C    definition of the functions                                        
C                                                                       
      REAL       CVMGT, SIGMA2                                          
      INTEGER    CVMGI                                                  
C                                                                       
C    definitons of the subroutine cloud                                 
C                                                                       
      REAL RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR                  
C                                                                       
C    missing definitions in morcrette                                   
C                                                                       
C      REAL   PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3            
      REAL ZCCO2                                                        
                                                                        
      LOGICAL LOH, LOM, LOL, LORAD                                      
C                                                                       
      INTEGER ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,           
     #        IZQS, IZCLC, IZQLWP, JLOOP, KRLEV,                        
     #        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     #        ITH, ITM, ITL,                                            
     #        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     #        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     #        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     #        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      INTEGER IND4, KM12, KXD, KXU, IG, KBS, KDD,                       
     #        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     #        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     #        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     #        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     #        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD1* SPECTRAL DISTRIBUTION OF AEROSOLS AND OZONE.     
C                     (TRIANGULAR *T10* TRUNCATIONFOR AEROSOLS).        
C                     (TRIANGULAR *T5* TRUNCATIONFOR OZONE).            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R           COZQC(21),COZQS(15),COZHC(21),COZHS(15)                
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD2* CONSTANTS INTERNAL TO THE RADIATION SUBROUTINES. 
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CSUPSAT                                                          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMCON* BASIC UNIVERSAL CONSTANTS AND DERIVED CONSTANTS.  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
C                                                                       
C                                                                       
C       --------------------------------------------------------------- 
C      PARAMETER(                                                       
C     I JPM=106,JPN=106,JPK=106,JPGL=160,JPNLON=320,                    
C     I JPT1MAX=400,          JPT2MAX=50,                               
C     I JPT3MAX=300,          JPT4MAX=50,                               
C     I JPHASH=2048,          JPNTRCRS=20,                              
C     I JPTASKS=4,     JPLOCKS=50,    JPEVENT=4,                        
C     I JPLTMT=JPTASKS,                                                 
C     I JPNUMW=352,            JPNUMP=352,                              
C     I JPTABLE=1024,                                                   
C     I JPMAXB =60,                                                     
C     I JPMAXPB=32,                                                     
C     I JPG3FLD=43,         JPG3XF=99,         JPG3CMP=17,              
C     I JPMP1=JPM+1,        JPNP1=JPN+1,       JPKP1=JPK+1,             
C     I JP2MP1=JPMP1*2,     JP4MP1=JPMP1*4,                             
C     I JPED=JPN/2+1,       JPOD=JPNP1/2,                               
C     I IPSP0=JPM+JPN-JPK,  IPSP1=JPNP1*JPMP1, IPSP2=IPSP0*(IPSP0+1)/2, 
C     I JPSP=IPSP1-IPSP2,   JP2SP=2*JPSP,                               
C     I JPHGL=JPGL/2,       JPNLP2=JPNLON+2,                            
C     I                     JPNVCLV=4*JPNLVP1)                          
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C     J. K. GIBSON     E.C.M.W.F.     22/01/82.                         
C     MODIFIED M.J     E.C.M.W.F.     13/12/82.                         
C                                                                       
      INTEGER                                                           
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      LOGICAL                                                           
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD        
      REAL                                                              
     R DTIME,    TWODT,    EPS                                          
C                                                                       
C      INTEGER                                                          
C     I NWTIME(JPNUMW),     NROW(JPTASKS),                              
C     I NPTIME(JPNUMP)                                                  
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMGAU* - QUANTITIES RELATED TO THE GAUSSIAN GRID.      
C                                                                       
      REAL                                                              
     R SINLON(NLP2),     COSLON(NLP2)                                   
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMTRU* -QUANTITIES RELATED TO THE SPECTRAL TRUNCATION. 
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSIM* - QUANTITIES NEEDED FOR THE SEMI-IMPLICIT SCHEME
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMHDI* - COEFFICIENTS FOR HORIZONTAL DIFFUSION.        
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMFFT* - QUANTITIES NEEDED FOR THE FAST *FOURIER       
C                         TRANSFORMS.                                   
C                                                                       
      INTEGER                                                           
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2                           
C      REAL                                                             
C     R TRIG(JPNLON)                                                    
      INTEGER                                                           
     I NFAX(10)                                                         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSTA* - ARRAYS AND PARAMETERS FOR STATISTICS.         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMDIA* - QUANTITIES RELATED TO GLOBAL PHYSICAL         
C                         DIAGNOSTICS.                                  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPSW* SWITCHES RELATED TO THE PARAMETERISATIONS OF      
C                     DIABATIC PROCESSES.                               
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMDSW* SWITCHES RELATED TO THE DYNAMICS                  
C                     AND THE GENERAL CONTROL OF THE FORECAST.          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH1* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE RADIATION SUBROUTINES.                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CRAE,                                                            
     R CDISSEM                                                          
C                                                                       
C      REAL       COSRAD(128),        SINRAD(128)                       
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH2* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE PHYSICAL SUBROUTINES (EXCEPT RADIATION    
C                     ONES).                                            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL       CCRH(JPNLEV)                                           
C                                                                       
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMDIZ* VARIABLES AND POINTERS FOR ZONAL MEAN DIAGNOSTICS 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMMSK* VARIABLES AND POINTERS FOR AREA MEAN DIAGNOSTICS. 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMSDS* - START/RESTART BLOCK.                          
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMTASK* - TASK RELATED VARIABLES.                      
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*         *COMRAD* - RADIATION POINTERS.                               
C                                                                       
      LOGICAL LO,LO1,LO2,LODIA,LODIAP                                   
C---------------------------------------------------------------------  
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C     COMMON *YOMAER* RADIATIVE CHARACTERISTICS OF THE AEROSOLS         
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
      REAL TAUA(2,5), PIZA(2,5), CGA(2,5), CAER(5,5), CAEROS            
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C     COMMON *YOMLW*                                                    
C                                                                       
C*    COEFFICIENTS FOR THE LONGWAVE RADIATION SUBROUTINES               
C                                                                       
C NINT  :  NUMBER OF SPECTRAL INTERVALS                                 
C NIPD  :  NUMBER OF BANDS DESCRIBED BY PADE APPROXIMANTS               
C NTR   :  NUMBER OF REFERENCE TEMPERATURES                             
C NTRA  :  NUMBER OF TRANSMISSION FUNCTIONS                             
C NUA   :  NUMBER OF ABSORBER AMOUNTS                                   
C NG1   :  ORDER OF THE GAUSSIAN QUADRATURE                             
C                                                                       
C RT1   :  ROOTS OF THE GAUSSIAN POLYNOMIAL                             
C WG1   :  WEIGHTS OF THE GAUSSIAN POLYNOMIAL                           
C AT, BT:  COEFFICIENTS OF THE TEMPERATURE DEPENDENCE                   
C CNTNU :  H2O CONTINUUM ABSORPTION COEFFICIENTS                        
C O1H,O2H,PIALF0,OCT: OZONE LONGWAVE ABSORPTION COEFFICIENTS            
C TREF, TSTAND      : REFERENCE TEMPERATURES                            
C XP    :  COEFFICIENTS OF POLYNOMIAL DEVELOPMENTS OF PLANCK FUNCTION   
C GA,GB :  PADE APPROXIMANTS OF H2O, CO2 TRANSMISSION FUNCTIONS         
C ETY,PTY :  PARAMETERS OF H2O CONTINUUM                                
C TINTP :  REFERENCE TEMPERATURES OF PRE-COMPUTED TRANSMISSIVITIES      
C CE    :  EMS COEFFS OF H2O TRANSMISSION FUNCTIONS                     
C                                                                       
      REAL  RT1(2),WG1(2),AT(12,3),BT(12,3),CNTNU(4),O1H                
     S ,OCT(4),TREF,XP(6,6),TSTAND,GA(11,24,3),GB(11,24,3)              
     S ,PTY(11,20),ETY(11,20),TINTP(11),TSTP,O2H,PIALF0                 
C                                                                       
      REAL  TB(40,31,2),ALP(40,31,2),BET(40,31,2),                      
     S ULG(40),PLG(31),AP(40,31,2),BP(40,31,2),TBO3(38,31),TO3B(38,31)  
     S,ULGO(38),AO3(38,31),BO3(38,31),O3A(38,31),O3B(38,31)             
C     ----------------------------------------------------------------- 
C*COMDECK YOMRDU                                                        
C     ----------------------------------------------------------------- 
C                                                                       
C*    CONTROL VARIABLES FOR RADIATION: INTERNAL SWITCH AND INDICES      
C                                                                       
C ZEELOG:  SECURITY THRESHOLD FOR ABSORBER AMOUNT IN LAPLACE TRANSFORM  
C ZEPSC :  SECURITY THRESHOLD FOR CLOUD COVER                           
C ZEPSCO:  SECURITY THRESHOLD FOR OZONE AMOUNT                          
C ZEPSCQ:  SECURITY THRESHOLD FOR WATER VAPOR                           
C ZEPSCT:  SECURITY THRESHOLD FOR SHORTWAVE OPTICAL THICKNESS           
C ZEPSCW:  SECURITY THRESHOLD FOR CLOUD LIQUID WATER PATH               
C                                                                       
      REAL          CDAY,CCO2,CH2O,DIFF,X10E                            
     S             ,ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW            
C     ----------------------------------------------------------------- 
C*COMDECK YOMSW                                                         
C     ----------------------------------------------------------------- 
C                                                                       
C*    COEFFICIENTS FOR THE SHORTWAVE RADIATION SUBROUTINES              
C                                                                       
C APAD  :  PADE APPROXIMANTS NUMERATOR                                  
C BPAD  :  PADE APPROXIMANTS DENOMINATOR                                
C D     :  TRANSMISSION LIMIT FOR INFINITE ABSORBER AMOUNT              
C CRAY  :  RAYLEIGH SCATTERING COEFFICIENTS                             
C SUN   :  SOLAR FRACTION IN SPECTRAL INTERVALS                         
C                                                                       
      REAL  APAD(2,3,7),BPAD(2,3,7),D(2,3),CRAY(2,6),SUN(2)             
C     ----------------------------------------------------------------- 
                                                                        
C                                                                       
C  -------------------------------------------------------------------- 
C                                                                       
      REAL   TTT(1,NLAT,NRLEV,12)                                       
C                                                                       
C        0.1  "RADINT POINTERS"                                         
C             -----------------                                         
C                                                                       
      REAL   TM1(NLP2,NLEV),                                            
     *       TSM1M(NLP2),                                               
     *       QM1(NLP2,NLEV),                                            
     *       SNM1M(NLP2),                                               
     *       ALBM(NLP2),                                                
     *       AVVELM(NLP2)                                               
      REAL   SLMM(NLP2),                                                
     *       APHM1(NLP2,NLEVP1),                                        
     *       APM1(NLP2,NLEV)                                            
C                                                                       
      LOGICAL LOLAND(NLP2)                                              
C                                                                       
C     VARIABLES FOR FULL GRID AND TRANSFORMS.                           
C                                                                       
      REAL   ZTI(NLP2,NRLEVP1),                                         
     *       ZCTI(NLP2,NRLEVP1),                                        
     *       ZCPH(NLP2,NRLEVP1),                                        
     *       ZWV(NLP2,NRLEV),                                           
     *       ZSW(NLP2,NRLEV),                                           
     *       ZALTE(NLP2),                                               
     *       ZCALTE(NLP2),                                              
     *       ZALSO(NLP2),                                               
     *       ZCALSO(NLP2)                                               
C                                                                       
      REAL   ZOZQ(NLP2),                                                
     #       ZOZH(NLP2)                                                 
C                                                                       
      REAL   ZTRSOL(NLP2,NLEVP1),                                       
     *       ZEMTER(NLP2,NLEVP1)                                        
C                                                                       
C     FINAL RESULTS ARE STORED ON:                                      
C                                                                       
      REAL   TRSOLM(NLP2,NLEVP1),                                       
     *       EMTERM(NLP2,NLEVP1)                                        
C                                                                       
      REAL   AMU0(NLP2),                                                
     *       RDAYL(NLP2)                                                
C                                                                       
C     VARIABLES WITHIN THE SPLITTED LATITUDE LOOP.                      
C                                                                       
      REAL   CLC(NRLST,NRLEV),                                          
     *       TI(NRLST,NRLEVP1),                                         
     *       DP(NRLST,NRLEV),                                           
     *       AADS(NRLST,NRLEV),                                         
     *       AEQ1(NRLST,NRLEV),                                         
     *       AEQ2(NRLST,NRLEV),                                         
     *       AEQ3(NRLST,NRLEV),                                         
     *       AEQ4(NRLST,NRLEV),                                         
     *       AEQ5(NRLST,NRLEV),                                         
     *       SSW(NRLST,NRLEV),                                          
     *       WV(NRLST,NRLEV),                                           
     *       QOF(NRLST,NRLEV),                                          
     *       QCF(NRLST,NRLEV),                                          
     *       CLWA(NRLST,NRLEV),                                         
     *       APRE(NRLST),                                               
     *       SMU0(NRLST),                                               
     *       ALTE(NRLST),                                               
     *       ALSO(NRLST)                                                
      REAL   ZDPO(NRLST),                                               
     *       ZQCFO(NRLST),                                              
     *       ZAEQSO(NRLST),                                             
     *       ZQOFO(NRLST),                                              
     *       ZDPN(NRLST),                                               
     *       ZQCFN(NRLST),                                              
     *       ZAEQSN(NRLST),                                             
     *       ZQOFN(NRLST)                                               
      REAL   FLT(NRLST,NRLEVP1),                                        
     *       FLS(NRLST,NRLEVP1)                                         
C                                                                       
C     DIAGNOSTIC VARIABLES.                                             
C                                                                       
      REAL   DIAG(NLEVP1,4),                                            
     *       DIA1(7),                                                   
     *       ZDIAG(NLEVP1,10),                                          
     *       ZDIA1(21),                                                 
     *       ZDIA(NLON),                                                
     *       ZDIAF(NLON,7),                                             
     *       ZDIAT(NLEVP1)                                              
C                                                                       
C                                                                       
C                                                                       
      REAL PGG(IRLST),                                                  
     #     PREF(IRLST),                                                 
     #     PRE1(IRLST),                                                 
     #     PRE2(IRLST),                                                 
     #     PRMUZ(IRLST),                                                
     #     PTO1(IRLST),                                                 
     #     PTR1(IRLST),                                                 
     #     PTR2(IRLST),                                                 
     #     PW(IRLST)                                                    
C                                                                       
C                                                                       
C  what about the 1-dimensional PALBS???                                
C                                                                       
      REAL     PALBSN  (IRLST)                                          
C                                                                       
C*       0.2   "LOCAL" ARRAYS                                           
C              --------------                                           
      REAL ZABSCU(IRLST ,NUA, 3*NRLEV+1),                               
     S     ZABSLY(IRLST ,NUA, 3*NRLEV+1) ,                              
     S     ZADJD (IRLST , IRLVP1)      ,                                
     S     ZADJU (IRLST , IRLVP1)       ,                               
     S     ZBGND (IRLST)                ,                               
     S     ZBINT (IRLST , IRLVP1)       ,                               
     S     ZBSUR (IRLST , NINT)         ,                               
     S     ZBSUIN(IRLST)                ,                               
     S     ZBTOP (IRLST , NINT)         ,                               
     S     PCLDLW(IRLST , IRLEV)        ,                               
     S     ZCLOUD(IRLST)                ,                               
     S     ZCNTRB(IRLST , IRLVP1,IRLVP1),                               
     S     ZCUMP (IRLST)                ,                               
     S     ZCUMPT(IRLST)                ,                               
     S     ZDBDT (IRLST , NINT , IRLEV) ,                               
     S     ZDBSL (IRLST , NINT , 2*NRLEV),                              
     S     ZDISTD(IRLST , IRLVP1)       ,                               
     S     ZDISTU(IRLST , IRLVP1)                                       
      REAL                                                              
     S     ZDNF  (IRLST , IRLVP1,IRLVP1),                               
     S     PDT0  (IRLST)                                                
      REAL                                                              
     S     ZDUC  (IRLST , 3*NRLEV+1)       ,                            
     S     ZEFFT (IRLST)                ,                               
     S     ZEMD  (IRLST , IRLEV), ZEMU  (IRLST , IRLEV),                
     S     ZEPSC2(IRLST),  ZEPSC3(IRLST),                               
     S     ZEPSH1(IRLST),  ZEPSH2(IRLST),                               
     S     ZEPSH3(IRLST),  ZEPSH4(IRLST),                               
     S     ZEPSH5(IRLST),  ZEPSH6(IRLST),                               
     S     ZEPHIO(IRLST),  ZFACTC(IRLST),                               
     S     ZZFD(IRLST), ZFDD(IRLST), ZFDN(IRLST , IRLVP1)               
      REAL                                                              
     S     PFLUX (IRLST , 2 , IRLVP1)   ,                               
     S     PFLUC (IRLST , 2 , IRLVP1)   ,                               
     S     ZFLWP (IRLST)                ,                               
     S     PCTS  (IRLST , IRLEV)        ,                               
     S     ZZFU   (IRLST)               ,                               
     S     ZFUP  (IRLST , IRLVP1)       ,                               
     S     ZGLAYD(IRLST)                ,                               
     S     ZGLAYU(IRLST)                ,                               
     S     ZOMEGA(IRLST , 2 , IRLEV)                                    
C                                                                       
      INTEGER                                                           
     S     ITX   (IRLST)                ,                               
     S     IXDIA (IRLST , IRLVP1)       ,                               
     S     IXT   (IRLST , IRLVP1,IRLVP1),                               
     S     IXTSU (IRLST , IRLVP1)       ,                               
     S     IXTTP (IRLST , IRLVP1)       ,                               
     S     IMX   (IRLST)                ,                               
     S     IMXP  (IRLST)                                                
C                                                                       
      REAL ZOZ   (IRLST , IRLEV)        ,                               
     S     PPMB  (IRLST , IRLVP1)       ,                               
     S     ZRES  (IRLST)                ,                               
     S     PB    (IRLST, 6 , IRLVP1)    ,                               
     S     ZSSIG (IRLST , 3*NRLEV+1)    ,                               
     S     PTAU  (IRLST , 2 , IRLEV)    ,                               
     S     PTAVE (IRLST , IRLEV)        ,                               
     S     ZTAVIC(IRLST)                ,                               
     S     ZTI1  (IRLST)                ,                               
     S     PTL   (IRLST , IRLVP1)       ,                               
     S     ZTT   (IRLST , NTRA)         ,                               
     S     ZTT1  (IRLST , NTRA)         ,                               
     S     ZTT2  (IRLST , NTRA)         ,                               
     S     ZUAER (IRLST , NINT)         ,                               
     S     ZUPF  (IRLST , IRLVP1,IRLVP1),                               
     S     ZUU   (IRLST , NUA)          ,                               
     S     ZXD   (IRLST)                                                
      REAL                                                              
     S     ZXN   (IRLST)                ,                               
     S     ZXOZ  (IRLST)                ,                               
     S     ZXWV  (IRLST)                ,                               
     S     ZZZ   (IRLST)                ,                               
     S     ZRES2 (IRLST)                ,                               
     S     ZTI2  (IRLST)                ,                               
     S     ZBLAY (IRLST,IRLEV)          ,                               
     S     ZBLEV (IRLST,IRLVP1)         ,                               
     S     ZCTS  (IRLST)                ,                               
     S     ZHFG  (IRLST)                                                
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C                                                                       
      INTEGER KT    (IRLST)                                             
      REAL    PUU   (IRLST , NUA  )     ,                               
     S        PTT   (IRLST , NTRA )     ,                               
     S        ZZ    (IRLST)                                             
C                                                                       
C     -------------------------------------------------------- ---------
C                                                                       
      REAL  PCLFR  (IRLST , IRLEV)       ,                              
     S      PT     (IRLST , IRLVP1)       ,                             
     S      PQOF   (IRLST , IRLEV)          ,                           
     S      PQLWP  (IRLST , IRLEV)          ,                           
     S      PEMIS  (IRLST)                                              
      REAL                                                              
     S      PFLT   (IRLST , IRLVP1)                                     
C                                                                       
      REAL  ZALBSU(IRLST , 2 )           ,                              
     S      ZCG   (IRLST , 2 , IRLEV)    ,                              
     S      ZCLDLW(IRLST , IRLEV)        ,                              
     S      ZCLDSW(IRLST , IRLEV)        ,                              
     S      ZCOOLR(IRLST , IRLEV)        ,                              
     S      ZDT0  (IRLST)                ,                              
     S      ZFLUX (IRLST , 2 , IRLVP1)   ,                              
     S      ZPMB  (IRLST , IRLVP1)       ,                              
     S      ZTAU  (IRLST , 2 , IRLEV)    ,                              
     S      ZTAVE (IRLST , IRLEV)        ,                              
     S      ZTL   (IRLST , IRLVP1)       ,                              
     S      ZFSDWN(IRLST , IRLVP1)       ,                              
     S      ZFSUP (IRLST , IRLVP1)       ,                              
     S      ZHEATR(IRLST , IRLEV )                                      
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C                                                                       
      REAL   PDP   (IRLST , IRLEV)     ,                                
     S       PWV   (IRLST , IRLEV)     ,                                
     S       PFLS  (IRLST , IRLVP1)    ,                                
     S       PPSOL (IRLST        )     ,                                
     S       PRMU0 (IRLST        )     ,                                
     S       PAER  (IRLST,IRLEV,5)     ,                                
     S       PALBS (IRLST , 2 )        ,                                
     S       PCG   (IRLST , 2 , IRLEV) ,                                
     S       PCLDSW(IRLST , IRLEV)     ,                                
     S       POMEGA(IRLST , 2 , IRLEV) ,                                
     S       POZ   (IRLST , IRLEV)                                      
C                                                                       
      REAL   PFDOWN(IRLST , IRLVP1)    ,                                
     S       PFUP  (IRLST , IRLVP1)                                     
C                                                                       
      REAL   ZAKI  (IRLST , 2 )        ,                                
     S       ZCGAZ (IRLST , IRLEV)     ,                                
     S       ZCNEB (IRLST , IRLEV)     ,                                
     S       ZC1I  (IRLST , IRLVP1)    ,                                
     S       ZDSC  (IRLST , IRLEV)     ,                                
     S       ZDSH  (IRLST , IRLEV)     ,                                
     S       ZDSIG (IRLST , IRLEV)     ,                                
     S       ZFACT (IRLST)             ,                                
     S       ZFACT1 (IRLST), ZFACT2 (IRLST),                            
     S       ZFACT3 (IRLST), ZFACT4 (IRLST),                            
     S       ZFACT5 (IRLST), ZFACT6 (IRLST),                            
     S       ZFD   (IRLST , IRLVP1)    ,                                
     S       ZFRAC (IRLST)             ,                                
     S       ZFU   (IRLST , IRLVP1)    ,                                
     S       ZG    (IRLST)             ,                                
     S       ZGG   (IRLST)             ,                                
     S       ZPIZAZ(IRLST , IRLEV)     ,                                
     S       ZRAYL (IRLST),  ZRAY1 (IRLST , IRLVP1)                     
      REAL                                                              
     S       ZRAY2 (IRLST , IRLVP1)    ,                                
     S       ZREF  (IRLST)             ,                                
     S       ZREFZ (IRLST , 2 , IRLVP1),                                
     S       ZRE1  (IRLST)             ,                                
     S       ZRE2  (IRLST)             ,                                
     S       ZRJ   (IRLST , 6 , IRLVP1),                                
     S       ZRK   (IRLST , 6 , IRLVP1),                                
     S       ZRL   (IRLST , 8)         ,                                
     S       ZRMU  (IRLST)             ,                                
     S       ZRMUE (IRLST , IRLVP1)    ,                                
     S       ZRMUZ (IRLST)                                              
      REAL                                                              
     S       ZRNEB (IRLST)             ,                                
     S       ZRUEF (IRLST , 8)         ,                                
     S       ZR1   (IRLST)             ,                                
     S       ZR2   (IRLST)             ,                                
     S       ZR21  (IRLST)             ,                                
     S       ZR22  (IRLST)             ,                                
     S       ZR23  (IRLST)             ,                                
     S       ZS    (IRLST)             ,                                
     S       ZSEC  (IRLST)             ,                                
     S       ZSS1  (IRLST)             ,                                
     S       ZTAUAZ(IRLST , IRLEV)     ,                                
     S       ZTOT  (IRLST , IRLVP1)    ,                                
     S       ZTO1  (IRLST)             ,                                
     S       ZTR   (IRLST , 2 , IRLVP1),                                
     S       ZTRA1 (IRLST , IRLVP1)    ,                                
     S       ZTRA2 (IRLST , IRLVP1)                                     
      REAL                                                              
     S       ZTR1  (IRLST)             ,                                
     S       ZTR2  (IRLST)             ,                                
     S       ZUD   (IRLST , 3 , IRLVP1),                                
     S       ZUM   (IRLST , IRLVP1)    ,                                
     S       ZU1D  (IRLST)             ,                                
     S       ZU2D  (IRLST)             ,                                
     S       ZW    (IRLST)             ,                                
     S       ZN175 (IRLST)             ,                                
     S       ZN190 (IRLST)             ,                                
     S       ZO175 (IRLST)             ,                                
     S       ZO190 (IRLST)             ,                                
     S       ZP75  (IRLST)             ,                                
     S       ZP90  (IRLST)             ,                                
     S       ZSIGN (IRLST)             ,                                
     S       ZSIGO (IRLST)                                              
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
      REAL   PTR    (IRLST)     ,                                       
     S       PU     (IRLST)                                             
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
C   CLOUD-POINTERS                                                      
C   --------------                                                      
C                                                                       
      REAL   ZRH(NLP2,NLEV),                                            
     *       ZCH(NLP2),                                                 
     *       ZCM(NLP2),                                                 
     *       ZCL(NLP2),                                                 
     *       ZCT(NLP2),                                                 
     *       ZCC(NLP2)                                                  
      INTEGER NTOPCM(NLP2),                                             
     *        NBASECM(NLP2)                                             
      REAL   ARPRCM(NLP2)                                               
      INTEGER ITOPH(NLP2),                                              
     *        ITOPM(NLP2),                                              
     *        ITOPL(NLP2)                                               
      LOGICAL LOC(NLP2)                                                 
      REAL   ZQS(NLP2,NLEV),                                            
     *       ZCLC(NLP2,NLEV),                                           
     *       ZQLWP(NLP2,NLEV)                                           
C                                                                       
      REAL   VERVEL(NLP2,NLEV),                                         
     *       ZTHETA(NLP2,NLEV),                                         
     *       ZETA(NLP2,NLEV),                                           
     *       ZDTHMIN(NLP2)                                              
      INTEGER INVB(NLP2)                                                
      REAL   ZCTLWP(NLP2),                                              
     *       ZQLOC(NLP2),                                               
     *       ZQMEC(NLP2),                                               
     *       ZQHIC(NLP2)                                                
C                                                                       
C  definitions of "radheat"                                             
C                                                                       
      REAL   ZFSOT(NLON),                                               
     #       ZFTET(NLON),                                               
     #       ZFLT (NLON),                                               
     #       ZFSO (NLON),                                               
     #       ZFTE (NLON),                                               
     #       ZFLB (NLON),                                               
     #       ZDTDT(NLON),                                               
     #       HEATO(NLON,NLEV),                                          
     #       HEATE(NLON,NLEV),                                          
     #       HEATR(NLON,NLEV),                                          
     #       ZI0  (NLON)                                                
C                                                                       
C PIERS 13-8-97                                                         
C                                                                       
         REAL PIERSLWP                                                  
C                                                                       
C Piece of code needed to replace the Nikos radiation scheme with       
C the Morcrette radiation scheme in the igcm3.1.                        
C See morcrette3_1_1.readme for details.                                
C                                                                       
C                                                                       
c      program lh                                                       
c                                                                       
c      parameter(NL=23)                                                 
c      real P(NL),t(NL),h2o(NL),o3(NL),htlw(NL),htsw(NL)                
c      real swalb,doy                                                   
c       REAL CF(4,2)     !cloud fractions,lwps                          
c       INTEGER IC(4,2)   ! cloud positions                             
c       REAL RFLUXES(2,2,2)                                             
c      open(11,file='mls23.dat')                                        
c      read(11,*) NLEV                                                  
c      DOY=80                                                           
c      swalb=0.07                                                       
c      alat=40.0                                                        
c       doy=float(i)                                                    
c      do i=nlev,1,-1                                                   
c         read(11,*) P(i),t(i),h2o(i),adum,o3(i)                        
c         print *,i,p(i),t(i),h2o(i),o3(i)                              
c      enddo                                                            
c                                                                       
c       ic(1,1)=3                                                       
c      ic(1,2)=17                                                       
c       ic(3,1)=6                                                       
c       ic(2,2)=6                                                       
c       cf(1,1)=0.3                                                     
c       cf(1,2)=45.0                                                    
c        cf(3,1)=0.8                                                    
c       cf(4,2)=20.0                                                    
c      alon=0.0                                                         
c      call morcigcm(p,t,h2o,o3,alat,htlw,htsw,doy,cf,ic,               
c     $     rfluxes,swalb,alon)                                         
c      enddo                                                            
c      do i=nl,1,-1                                                     
c         print '(i6,4F14.4)',i,p(i),t(i),htsw(i),htlw(i)               
c      enddo                                                            
c      print *,RFLUXES(1,1,1), RFLUXES(1,1,2), RFLUXES(1,2,1),          
c     $     RFLUXES(1,2,2)                                              
c      print *,RFLUXES(2,1,1), RFLUXES(2,1,2), RFLUXES(2,2,1),          
c     $     RFLUXES(2,2,2)                                              
c                                                                       
c                                                                       
c      end                                                              
c      INCLUDE 'zparb.upd'                                              
      REAL XL2, XL3, ALOS                                               
      REAL FOXDWN(IRLST, IRLVP1)                                        
C     ----------------------------------------------------------------  
C                                                                       
CDIR$ VFUNCTION ALOG, EXP                                               
C     -----------------------                                           
C                                                                       
C                                                                       
C     ----------------------------------------------------------------  
C                                                                       
C      WRITE(6,*) ' ***SUBROUTINE SW*** '                               
C                                                                       
      DO 96 JL=1,NRLST                                                  
         DO 97 JK=1,NLEV                                                
            PCLDSW(JL,JK) = ZCLDSW(JL,JK)                               
            POZ   (JL,JK) = ZOZ   (JL,JK)                               
            PPMB  (JL,JK) = ZPMB  (JL,JK)                               
            PTAVE (JL,JK) = ZTAVE (JL,JK)                               
            PFDOWN(JL,JK) = ZFSDWN(JL,JK)                               
            PFUP  (JL,JK) = ZFSUP (JL,JK)                               
            DO 98 II=1,2                                                
               PCG   (JL,II,JK) = ZCG   (JL,II,JK)                      
               POMEGA(JL,II,JK) = ZOMEGA(JL,II,JK)                      
               PTAU  (JL,II,JK) = ZTAU  (JL,II,JK)                      
 98         CONTINUE                                                    
 97      CONTINUE                                                       
         DO 99 II=1,2                                                   
            PALBS(JL,II) = ZALBSU(JL,II)                                
 99      CONTINUE                                                       
         PPMB  (JL,NRLEV+1) = ZPMB  (JL,NRLEV+1)                        
         PFDOWN(JL,NRLEV+1) = ZFSDWN(JL,NRLEV+1)                        
         PFUP  (JL,NRLEV+1) = ZFSUP (JL,NRLEV+1)                        
 96   CONTINUE                                                          
C                                                                       
C*         1.     COMPUTES AMOUNTS OF ABSORBERS                         
C                 -----------------------------                         
C                                                                       
 100  CONTINUE                                                          
C                                                                       
C                                                                       
C*         1.1    INITIALIZES QUANTITIES                                
C                 ----------------------                                
C                                                                       
 110  CONTINUE                                                          
C                                                                       
      DO 111 JL = 1 , KDLON                                             
         ZC1I(JL,KFLEV+1)=0.                                            
         ZUD(JL,1,KFLEV+1)=0.                                           
         ZUD(JL,2,KFLEV+1)=0.                                           
         ZUD(JL,3,KFLEV+1)=0.                                           
         ZFACT(JL)= PRMU0(JL) * PSCT * RDAYL(JL)                        
         ZRMU(JL)=SQRT(1224.* PRMU0(JL) * PRMU0(JL) + 1.) / 35.         
         ZSEC(JL)=1./ZRMU(JL)                                           
 111  CONTINUE                                                          
C                                                                       
C                                                                       
C*         1.2    OZONE FOR DOWNWARD LOOKING PATH                       
C                 -------------------------------                       
C                                                                       
 120  CONTINUE                                                          
C                                                                       
      DO 122 JK = 1 , KFLEV                                             
         JKL = KFLEV+1 - JK                                             
         JKLP1 = JKL + 1                                                
         DO 121 JL = 1 , KDLON                                          
            ZUD(JL,3,JKL) = ZUD(JL,3,JKLP1) + POZ(JL,JKL) * ZSEC(JL)    
 121     CONTINUE                                                       
 122  CONTINUE                                                          
C                                                                       
c          SCHUMANN-RUNGE and HERZBERG CONTINUUM (0.12 - 0.25 MICRON)   
c          ----------------------------------------------------------   
c                                                                       
      ALOS=2.687E19                                                     
      DO 50 JL = 1 , KDLON                                              
        FOXDWN(JL,1)=0.0                                                
        IF (PRMU0(JL).GT.0.0001) THEN                                   
          XL3=0.0                                                       
          DO 55 JK=1,KFLEV                                              
            XL3=XL3+POZ(JL,JK)*ZSEC(JL)*ALOS                            
   55     CONTINUE                                                      
C                                                                       
C set XL2O as mesopause value if too small                              
          XL2O=4.442E19*1.E-4*ZSEC(JL)                                  
C                                                                       
          DO 60 JK=1,KFLEV                                              
            XL3=XL3-POZ(JL,JK)*ZSEC(JL)*ALOS                            
C            XL2=4.442E19*PPMB(JL,JK+1)*ZSEC(JL)*100.0                  
            XL2=MAX(4.442E19*PPMB(JL,JK+1)*ZSEC(JL)*100.0, XL2O)        
C                                                                       
            FOXDWN(JL,JK+1) =                                           
     *      (  0.913 *EXP(-5.500E-24*XL2-6.215E-18*XL3)                 
     *    + 0.6308 *EXP(-1.342E-26*XL2-1.656E-18*XL3)                   
     *    + 0.4*2.30E-3*EXP(-3.159E-20*XL2-1.857E-06*XL2**0.33837)      
     *    + 0.4*3.00E-3*EXP(-2.261E-24*XL2-1.917E-14*XL2**0.78903)      
     *    + 0.3*3.50E-3*EXP(-5.399E-24*XL2-2.466E-17*XL2**0.92105)      
     *    + 0.3*2.80E-3*EXP(-3.406E-24*XL2-7.787E-18*XL2**0.9269 )      
     *    + 0.1*0.1466 *EXP(-2.915E-25*XL2**1.06135                     
     *                   - 1.866E-07*XL2**0.2895) )/SOLC                
C                                                                       
   60     CONTINUE                                                      
        ELSE                                                            
          DO 65 JK=KFLEV,1,-1                                           
            FOXDWN(JL,JK)=0.0                                           
   65     CONTINUE                                                      
        ENDIF                                                           
   50 CONTINUE                                                          
C                                                                       
C*         1.3    OZONE FOR UPWARD LOOKING PATH AND OTHER ABSORBERS     
C                 -------------------------------------------------     
C                                                                       
 130  CONTINUE                                                          
C                                                                       
      DO 131 JL = 1 , KDLON                                             
         ZUM(JL,1) = ZUD(JL,3,1)                                        
         ZU1D(JL) = 0.                                                  
         ZU2D(JL) = 0.                                                  
         ZPSIG = PPSOL(JL) / 101325.                                    
         ZP75(JL) = PPSOL(JL) * ZPSIG ** 0.75                           
         ZP90(JL) = PPSOL(JL) * ZPSIG ** 0.90                           
         ZO175(JL) = 1.0                                                
         ZO190(JL) = 1.0                                                
         ZSIGO(JL) = 1.0                                                
 131  CONTINUE                                                          
C                                                                       
      DO 133 JK = 1 , KFLEV                                             
         JKP1 = JK + 1                                                  
         JKL = KFLEV+1 - JK                                             
         DO 132 JL = 1 , KDLON                                          
            ZUM(JL,JKP1) = ZUM(JL,JK) + POZ(JL,JK) * 1.66               
            ZRT = 273.15 / PTAVE(JL,JK)                                 
            ZWH2O = MAX(PWV(JL,JKL) , ZEPSCQ )                          
            ZSIGN(JL) = 100. * PPMB(JL,JKP1) / PPSOL(JL)                
            ZDSIG(JL,JK) = ZSIGO(JL) - ZSIGN(JL)                        
            ZN175(JL) = ZSIGN(JL) ** 1.75                               
            ZN190(JL) = ZSIGN(JL) ** 1.90                               
            ZDSCO2 = ZO175(JL) - ZN175(JL)                              
            ZDSH2O = ZO190(JL) - ZN190(JL)                              
            ZUD(JL,1,JK) = ZP90(JL) * ZDSH2O*CH2O*ZWH2O  * ZRT **0.45   
            ZUD(JL,2,JK) = ZP75(JL) * ZDSCO2*CCO2*PCARDI * ZRT **0.375  
            ZU1D(JL) = ZU1D(JL) + ZUD(JL,1,JK)                          
            ZU2D(JL) = ZU2D(JL) + ZUD(JL,2,JK)                          
            ZSIGO(JL) = ZSIGN(JL)                                       
            ZO175(JL) = ZN175(JL)                                       
            ZO190(JL) = ZN190(JL)                                       
 132     CONTINUE                                                       
 133  CONTINUE                                                          
C                                                                       
C*         1.4    COMPUTES CLEAR-SKY GREY ABSORPTION COEFFICIENTS       
C                 -----------------------------------------------       
C                                                                       
 140  CONTINUE                                                          
C                                                                       
      DO 141 JL = 1 , KDLON                                             
         ZU1D(JL) = ZU1D(JL) * ZSEC(JL)                                 
         ZU2D(JL) = ZU2D(JL) * ZSEC(JL)                                 
         ZW(JL) = ZU1D(JL)                                              
 141  CONTINUE                                                          
C                                                                       
      CALL SWTT ( 2, 1, KDLON,KRLST,KFLEV,ITASK )                       
C                                                                       
      DO 142 JL = 1 , KDLON                                             
         ZAKI(JL,1) = -LOG( ZR1 (JL)) / ZU1D(JL)                        
         ZW(JL) = ZU2D(JL)                                              
 142  CONTINUE                                                          
C                                                                       
      CALL SWTT ( 2, 2, KDLON,KRLST,KFLEV,ITASK )                       
C                                                                       
      DO 143 JL = 1 , KDLON                                             
         ZAKI(JL,2) = -LOG( ZR1 (JL)) / ZU2D(JL)                        
 143  CONTINUE                                                          
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*         2.     FIRST SPECTRAL INTERVAL (0.25-0.68 MICRON)            
C                 ----------------------- ------------------            
C                                                                       
 200  CONTINUE                                                          
C                                                                       
      INU = 1                                                           
C                                                                       
C*         2.1    OPTICAL THICKNESS FOR RAYLEIGH SCATTERING             
C                 -----------------------------------------             
C                                                                       
 210  CONTINUE                                                          
C                                                                       
      DO 211 JL = 1 , KDLON                                             
          ZRAYL(JL) = CRAY(INU,1) + ZRMU(JL) * (CRAY(INU,2) + ZRMU(JL)  
     S            * (CRAY(INU,3) + ZRMU(JL) * (CRAY(INU,4) + ZRMU(JL)   
     S            * (CRAY(INU,5) + ZRMU(JL) *   CRAY(INU,6)       ))))  
 211  CONTINUE                                                          
C                                                                       
C*         2.2    OPTICAL PARAMETERS FOR AEROSOLS AND RAYLEIGH          
C                 --------------------------------------------          
C                                                                       
 220  CONTINUE                                                          
C                                                                       
      DO 225 JK = 1 , KFLEV                                             
         DO 221 JL = 1 , KDLON                                          
            ZCGAZ(JL,JK) = 0.                                           
            ZPIZAZ(JL,JK) =  0.                                         
            ZTAUAZ(JL,JK) = 0.                                          
 221     CONTINUE                                                       
         DO 223 JAE=1,5                                                 
            DO 222 JL = 1 , KDLON                                       
               ZTAUAZ(JL,JK)=ZTAUAZ(JL,JK)                              
     S                      +PAER(JL,JK,JAE)*TAUA(INU,JAE)              
               ZPIZAZ(JL,JK)=ZPIZAZ(JL,JK)+PAER(JL,JK,JAE)              
     S                   * TAUA(INU,JAE)*PIZA(INU,JAE)                  
               ZCGAZ(JL,JK) =  ZCGAZ(JL,JK) +PAER(JL,JK,JAE)            
     S                   * TAUA(INU,JAE)*PIZA(INU,JAE)*CGA(INU,JAE)     
 222        CONTINUE                                                    
 223     CONTINUE                                                       
C                                                                       
         DO 224 JL = 1 , KDLON                                          
            ZCGAZ(JL,JK) = CVMGT( 0.D0, ZCGAZ(JL,JK) / ZPIZAZ(JL,JK),   
     S                            IAER.EQ.0)                            
             ZPIZAZ(JL,JK) = CVMGT( 1.D0, ZPIZAZ(JL,JK) / ZTAUAZ(JL,JK),
     S                            IAER.EQ.0)                            
            ZTRAY = ZRAYL(JL) * ZDSIG(JL,JK)                            
            ZRATIO = ZTRAY / (ZTRAY + ZTAUAZ(JL,JK))                    
            ZGAR = ZCGAZ(JL,JK)                                         
            ZFF = ZGAR * ZGAR                                           
            ZTAUAZ(JL,JK)=ZTRAY+ZTAUAZ(JL,JK)*(1.-ZPIZAZ(JL,JK)*ZFF)    
            ZCGAZ(JL,JK) = ZGAR * (1. - ZRATIO) / (1. + ZGAR)           
            ZPIZAZ(JL,JK) =ZRATIO+(1.-ZRATIO)*ZPIZAZ(JL,JK)*(1.-ZFF)    
     S                    / (1. - ZPIZAZ(JL,JK) * ZFF)                  
 224     CONTINUE                                                       
 225  CONTINUE                                                          
C                                                                       
C*         2.3    TOTAL EFFECTIVE CLOUDINESS ABOVE A GIVEN LEVEL        
C                 ----------------------------------------------        
C                                                                       
 230  CONTINUE                                                          
C                                                                       
      DO 231 JL = 1 , KDLON                                             
         ZR23(JL) = 0.                                                  
         ZC1I(JL,KFLEV+1) = 0.                                          
 231  CONTINUE                                                          
C                                                                       
      DO 233 JK = 1 , KFLEV                                             
         JKL = KFLEV+1 - JK                                             
         JKLP1 = JKL + 1                                                
         DO 232 JL = 1 , KDLON                                          
            ZFAOA = 1.-ZPIZAZ(JL,JKL)*ZCGAZ(JL,JKL)*ZCGAZ(JL,JKL)       
            ZFAOC = 1. - POMEGA(JL,INU,JKL) * PCG(JL,INU,JKL)           
     S                                       * PCG(JL,INU,JKL)          
            ZCORAE = ZFAOA * ZTAUAZ(JL,JKL) * ZSEC(JL)                  
            ZCORCD = ZFAOC * PTAU(JL,INU,JKL) * ZSEC(JL)                
            ZR21(JL) = EXP(-ZCORAE   )                                  
            ZR22(JL) = EXP(-ZCORCD   )                                  
            ZSS1(JL) = PCLDSW(JL,JKL)*(1.0-ZR21(JL)*ZR22(JL))           
     S               + (1.0-PCLDSW(JL,JKL))*(1.0-ZR21(JL))              
            ZC1I(JL,JKL) = 1.0-(1.0-ZSS1(JL))*(1.0-ZC1I(JL,JKLP1))      
 232     CONTINUE                                                       
 233  CONTINUE                                                          
C                                                                       
C*         2.4    REFLECTIVITY/TRANSMISSIVITY FOR PURE SCATTERING       
C                 -----------------------------------------------       
C                                                                       
 240  CONTINUE                                                          
C                                                                       
      DO 241 JL = 1 , KDLON                                             
         ZREFZ(JL,2,1) = PALBS(JL,INU)                                  
         ZREFZ(JL,1,1) = PALBS(JL,INU)                                  
 241  CONTINUE                                                          
C                                                                       
      DO 246 JK = 2 , KFLEV+1                                           
         JKM1 = JK-1                                                    
         DO 242 JL = 1 , KDLON                                          
            ZRNEB(JL)= PCLDSW(JL,JKM1)                                  
             ZRE1(JL)=0.                                                
            ZTR1(JL)=0.                                                 
            ZRE2(JL)=0.                                                 
            ZTR2(JL)=0.                                                 
C                                                                       
C*         2.4.1  EQUIVALENT ZENITH ANGLE                               
C                 -----------------------                               
C                                                                       
 2410 CONTINUE                                                          
C                                                                       
            ZMUE = (1.-ZC1I(JL,JK)) * ZSEC(JL)                          
     S            + ZC1I(JL,JK) * 1.66                                  
            ZRMUE(JL,JK) = 1./ZMUE                                      
C                                                                       
C*         2.4.2  REFLECT./TRANSMISSIVITY DUE TO RAYLEIGH AND AEROSOLS  
C                 ----------------------------------------------------  
C                                                                       
 2420 CONTINUE                                                          
C                                                                       
            ZGAP = ZCGAZ(JL,JKM1)                                       
            ZBMU0 = 0.5 - 0.75 * ZGAP / ZMUE                            
            ZWW = ZPIZAZ(JL,JKM1)                                       
            ZTO = ZTAUAZ(JL,JKM1)                                       
            ZDEN = 1. + (1. - ZWW + ZBMU0 * ZWW) * ZTO * ZMUE           
     S           + (1-ZWW) * (1. - ZWW +2.*ZBMU0*ZWW)*ZTO*ZTO*ZMUE*ZMUE 
            ZRAY1(JL,JKM1) = ZBMU0 * ZWW * ZTO * ZMUE / ZDEN            
            ZTRA1(JL,JKM1) = 1. / ZDEN                                  
C                                                                       
            ZMU1 = 0.5                                                  
            ZBMU1 = 0.5 - 0.75 * ZGAP * ZMU1                            
            ZDEN1= 1. + (1. - ZWW + ZBMU1 * ZWW) * ZTO / ZMU1           
     S           + (1-ZWW) * (1. - ZWW +2.*ZBMU1*ZWW)*ZTO*ZTO/ZMU1/ZMU1 
            ZRAY2(JL,JKM1) = ZBMU1 * ZWW * ZTO / ZMU1 / ZDEN1           
            ZTRA2(JL,JKM1) = 1. / ZDEN1                                 
C                                                                       
C*         2.4.3  EFFECT OF CLOUD LAYER                                 
C                 ---------------------                                 
C                                                                       
 2430 CONTINUE                                                          
C                                                                       
            PTAU(JL,INU,JKM1) = MAX( PTAU(JL,INU,JKM1) , ZEPSCT )       
            ZW(JL) = POMEGA(JL,INU,JKM1)                                
            ZTO1(JL) = PTAU(JL,INU,JKM1)/ZW(JL)                         
     S            + ZTAUAZ(JL,JKM1)/ZPIZAZ(JL,JKM1)                     
            ZR21(JL) = PTAU(JL,INU,JKM1) + ZTAUAZ(JL,JKM1)              
            ZR22(JL) = PTAU(JL,INU,JKM1) / ZR21(JL)                     
            ZGG(JL) = ZR22(JL) * PCG(JL,INU,JKM1)                       
     S              + (1. - ZR22(JL)) * ZCGAZ(JL,JKM1)                  
            ZW(JL) = ZR21(JL) / ZTO1(JL)                                
            ZREF(JL) = ZREFZ(JL,1,JKM1)                                 
            ZRMUZ(JL) = ZRMUE(JL,JK)                                    
 242     CONTINUE                                                       
C                                                                       
         CALL DEDD ( KDLON,KRLST,KFLEV,ITASK )                          
C                                                                       
         DO 245 JL = 1 , KDLON                                          
C                                                                       
             ZREFZ(JL,1,JK) = (1.-ZRNEB(JL)) * (ZRAY1(JL,JKM1)          
     S                     + ZREFZ(JL,1,JKM1) * ZTRA1(JL,JKM1)          
     S                     * ZTRA2(JL,JKM1)                             
     S                     /(1.-ZRAY2(JL,JKM1)*ZREFZ(JL,1,JKM1)))       
     S                     + ZRNEB(JL) * ZRE2(JL)                       
C                                                                       
            ZTR(JL,1,JKM1) = ZRNEB(JL) * ZTR2(JL) + (ZTRA1(JL,JKM1)     
     S                     /(1.-ZRAY2(JL,JKM1)*ZREFZ(JL,1,JKM1)))       
     S                     * (1.-ZRNEB(JL))                             
C                                                                       
 245     CONTINUE                                                       
 246  CONTINUE                                                          
C                                                                       
C*         2.5    REFLECT./TRANSMISSIVITY BETWEEN SURFACE AND LEVEL     
C                 -------------------------------------------------     
C                                                                       
 250  CONTINUE                                                          
C                                                                       
      JAJ = 2                                                           
      DO 251 JL = 1 , KDLON                                             
         ZRJ(JL,JAJ,KFLEV+1) = 1.                                       
         ZRK(JL,JAJ,KFLEV+1) = ZREFZ(JL, 1,KFLEV+1)                     
 251  CONTINUE                                                          
C                                                                       
      DO 253 JK = 1 , KFLEV                                             
         JKL = KFLEV+1 - JK                                             
         JKLP1 = JKL + 1                                                
         DO 252 JL = 1 , KDLON                                          
            ZRE11= ZRJ(JL,JAJ,JKLP1) * ZTR(JL, 1,JKL)                   
            ZRJ(JL,JAJ,JKL) = ZRE11                                     
            ZRK(JL,JAJ,JKL) = ZRE11 * ZREFZ(JL, 1,JKL)                  
 252     CONTINUE                                                       
 253  CONTINUE                                                          
C                                                                       
C*         2.6    OZONE ABSORPTION AND FLUXES                           
C                 ---------------------------                           
C                                                                       
 260  CONTINUE                                                          
C                                                                       
      DO 264 JK = 1 , KFLEV+1                                           
         JKL = KFLEV+1 - JK + 1                                         
         DO 262 JL = 1 , KDLON                                          
            ZW(JL) = ZUD(JL,3,JKL)                                      
 262     CONTINUE                                                       
C                                                                       
         CALL SWTT ( INU, 3, KDLON,KRLST,KFLEV,ITASK )                  
C                                                                       
         DO 263 JL = 1 , KDLON                                          
             ZFD(JL,JKL) = ZR1(JL) * ZRJ(JL,JAJ,JKL) * SUN(INU)         
 263     CONTINUE                                                       
 264  CONTINUE                                                          
C                                                                       
      DO 265 JL = 1 , KDLON                                             
         ZFU(JL,1) = PALBS(JL,INU) * ZFD(JL,1)                          
 265  CONTINUE                                                          
C                                                                       
      DO 268 JK = 1 , KFLEV+1                                           
         DO 266 JL = 1 , KDLON                                          
            ZW(JL) = ZUM(JL,JK)                                         
 266     CONTINUE                                                       
C                                                                       
         CALL SWTT ( INU, 3, KDLON,KRLST,KFLEV,ITASK )                  
C                                                                       
         DO 267 JL = 1 , KDLON                                          
            ZFU(JL,JK) = ZR1(JL) * ZRK(JL,JAJ,JK) * SUN(INU)            
 267     CONTINUE                                                       
 268  CONTINUE                                                          
C                                                                       
C                                                                       
      IWHERE=268                                                        
C     PRINT 878,IWHERE                                                  
 878  FORMAT(1X,' LOOP ',I4,' SUCCESSFUL IN SW    INDEX=',I3)           
C     ------------------------------------------------------------------
C                                                                       
C*         3.     SECOND SPECTRAL INTERVAL (0.68-4.00 MICRON)           
C                 ----------------------- -------------------           
C                                                                       
 300  CONTINUE                                                          
C                                                                       
      INU = 2                                                           
C                                                                       
C*         3.1    OPTICAL THICKNESS FOR RAYLEIGH SCATTERING             
C                 -----------------------------------------             
C                                                                       
 310  CONTINUE                                                          
C                                                                       
      DO 311 JL = 1 , KDLON                                             
         ZRMUM1 = 1. - ZRMU(JL)                                         
         ZRAYL(JL) = CRAY(INU,1) + ZRMUM1   * (CRAY(INU,2) + ZRMUM1     
     S            * (CRAY(INU,3) + ZRMUM1   * (CRAY(INU,4) + ZRMUM1     
     S            * (CRAY(INU,5) + ZRMUM1   *  CRAY(INU,6)       ))))   
 311  CONTINUE                                                          
C                                                                       
C*         3.2    OPTICAL PARAMETERS FOR AEROSOLS AND RAYLEIGH          
C                 --------------------------------------------          
C                                                                       
 320  CONTINUE                                                          
C                                                                       
      DO 325 JK = 1 , KFLEV                                             
         DO 321 JL = 1 , KDLON                                          
            ZCGAZ(JL,JK) = 0.                                           
            ZPIZAZ(JL,JK) = 0.                                          
            ZTAUAZ(JL,JK) = 0.                                          
 321     CONTINUE                                                       
         DO 323 JAE=1,5                                                 
            DO 322 JL = 1 , KDLON                                       
               ZTAUAZ(JL,JK) = ZTAUAZ(JL,JK) + PAER(JL,JK,JAE)          
     S                       * TAUA(INU,JAE)                            
               ZPIZAZ(JL,JK) = ZPIZAZ(JL,JK) + PAER(JL,JK,JAE)          
     S                       * TAUA(INU,JAE) * PIZA(INU,JAE)            
               ZCGAZ(JL,JK) =  ZCGAZ(JL,JK) + PAER(JL,JK,JAE)           
     S                   * TAUA(INU,JAE)*PIZA(INU,JAE)*CGA(INU,JAE)     
 322        CONTINUE                                                    
 323     CONTINUE                                                       
         DO 324 JL = 1 , KDLON                                          
            ZCGAZ(JL,JK) = CVMGT( 0.D0, ZCGAZ(JL,JK) / ZPIZAZ(JL,JK),   
     S                            IAER.EQ.0)                            
            ZPIZAZ(JL,JK) = CVMGT( 1.D0, ZPIZAZ(JL,JK) / ZTAUAZ(JL,JK), 
     S                             IAER.EQ.0)                           
            ZTRAY = ZRAYL(JL) * ZDSIG(JL,JK)                            
            ZRATIO = ZTRAY / (ZTRAY + ZTAUAZ(JL,JK))                    
            ZGAR = ZCGAZ(JL,JK)                                         
            ZFF = ZGAR * ZGAR                                           
            ZTAUAZ(JL,JK)=ZTRAY+ZTAUAZ(JL,JK)*(1.-ZPIZAZ(JL,JK)*ZFF)    
            ZCGAZ(JL,JK) = ZGAR * (1. - ZRATIO) / (1. + ZGAR)           
            ZPIZAZ(JL,JK) = ZRATIO+(1. - ZRATIO)*ZPIZAZ(JL,JK)*(1.-ZFF) 
     S                    / (1. - ZPIZAZ(JL,JK) * ZFF)                  
 324     CONTINUE                                                       
 325  CONTINUE                                                          
C                                                                       
C*         3.3    TOTAL EFFECTIVE CLOUDINESS ABOVE A GIVEN LEVEL        
C                 ----------------------------------------------        
C                                                                       
 330  CONTINUE                                                          
C                                                                       
      DO 331 JL = 1 , KDLON                                             
         ZR23(JL) = 0.                                                  
         ZC1I(JL,KFLEV+1) = 0.                                          
 331  CONTINUE                                                          
      DO 333 JK = 1 , KFLEV                                             
         JKL = KFLEV+1 - JK                                             
         JKLP1 = JKL + 1                                                
         DO 332 JL = 1 , KDLON                                          
            ZFAOA = 1.-ZPIZAZ(JL,JKL)*ZCGAZ(JL,JKL)*ZCGAZ(JL,JKL)       
            ZFAOC = 1. - POMEGA(JL,INU,JKL) * PCG(JL,INU,JKL)           
     S                                       * PCG(JL,INU,JKL)          
            ZCORAE = ZFAOA * ZTAUAZ(JL,JKL) * ZSEC(JL)                  
            ZCORCD = ZFAOC * PTAU(JL,INU,JKL) * ZSEC(JL)                
            ZR21(JL) = EXP(-ZCORAE   )                                  
            ZR22(JL) = EXP(-ZCORCD   )                                  
            ZSS1(JL) = PCLDSW(JL,JKL)*(1.0-ZR21(JL)*ZR22(JL))           
     S               + (1.0-PCLDSW(JL,JKL))*(1.0-ZR21(JL))              
            ZC1I(JL,JKL) = 1.0-(1.0-ZSS1(JL))*(1.0-ZC1I(JL,JKLP1))      
 332     CONTINUE                                                       
 333  CONTINUE                                                          
C                                                                       
C*         3.4    REFLECTIVITY/TRANSMISSIVITY FOR PURE SCATTERING       
C                 -----------------------------------------------       
C                                                                       
 340  CONTINUE                                                          
C                                                                       
      DO 341 JL = 1 , KDLON                                             
         ZREFZ(JL,2,1) = PALBS(JL,INU)                                  
         ZREFZ(JL,1,1) = PALBS(JL,INU)                                  
 341  CONTINUE                                                          
C                                                                       
      DO 346 JK = 2 , KFLEV+1                                           
         JKM1 = JK - 1                                                  
         DO 342 JL = 1 , KDLON                                          
            ZRNEB(JL) = PCLDSW(JL,JKM1)                                 
            ZRE1(JL)=0.                                                 
            ZTR1(JL)=0.                                                 
            ZRE2(JL)=0.                                                 
            ZTR2(JL)=0.                                                 
C                                                                       
C*         3.4.1  EQUIVALENT ZENITH ANGLE                               
C                 -----------------------                               
C                                                                       
 3410 CONTINUE                                                          
C                                                                       
            ZMUE = (1.-ZC1I(JL,JK)) * ZSEC(JL)                          
     S           + ZC1I(JL,JK) * 1.66                                   
            ZRMUE(JL,JK) = 1./ZMUE                                      
C                                                                       
C*         3.4.2  REFLECT./TRANSMISSIVITY DUE TO RAYLEIGH AND AEROSOLS  
C                 ----------------------------------------------------  
C                                                                       
 3420 CONTINUE                                                          
C                                                                       
            ZGAP = ZCGAZ(JL,JKM1)                                       
            ZBMU0 = 0.5 - 0.75 * ZGAP / ZMUE                            
            ZWW = ZPIZAZ(JL,JKM1)                                       
            ZTO = ZTAUAZ(JL,JKM1)                                       
            ZDEN = 1. + (1. - ZWW + ZBMU0 * ZWW) * ZTO * ZMUE           
     S           + (1-ZWW)*(1.-ZWW+2.*ZBMU0*ZWW)*ZTO*ZTO*ZMUE*ZMUE      
            ZRAY1(JL,JKM1) = ZBMU0 * ZWW * ZTO * ZMUE / ZDEN            
            ZTRA1(JL,JKM1) = 1. / ZDEN                                  
C                                                                       
            ZMU1 = 0.5                                                  
            ZBMU1 = 0.5 - 0.75 * ZGAP * ZMU1                            
            ZDEN1= 1. + (1. - ZWW + ZBMU1 * ZWW) * ZTO / ZMU1           
     S           + (1.-ZWW)*(1.-ZWW+2.*ZBMU1*ZWW)*ZTO*ZTO/ZMU1/ZMU1     
            ZRAY2(JL,JKM1) = ZBMU1 * ZWW * ZTO / ZMU1 / ZDEN1           
            ZTRA2(JL,JKM1) = 1. / ZDEN1                                 
C                                                                       
C*         3.4.3  EFFECT OF CLOUD LAYER                                 
C                 ---------------------                                 
C                                                                       
 3430 CONTINUE                                                          
C                                                                       
            PTAU(JL,INU,JKM1) = MAX( PTAU(JL,INU,JKM1) , ZEPSCT )       
            ZW(JL) = POMEGA(JL,INU,JKM1)                                
            ZTO1(JL) = PTAU(JL,INU,JKM1)/ZW(JL)                         
     S               + ZTAUAZ(JL,JKM1)/ZPIZAZ(JL,JKM1)                  
            ZR21(JL) = PTAU(JL,INU,JKM1) + ZTAUAZ(JL,JKM1)              
            ZR22(JL) = PTAU(JL,INU,JKM1) / ZR21(JL)                     
            ZGG(JL) = ZR22(JL) * PCG(JL,INU,JKM1)                       
     S              + (1. - ZR22(JL)) * ZCGAZ(JL,JKM1)                  
             ZW(JL) = ZR21(JL) / ZTO1(JL)                               
            ZREF(JL)=ZREFZ(JL,1,JKM1)                                   
            ZRMUZ(JL)=ZRMUE(JL,JK)                                      
 342     CONTINUE                                                       
C                                                                       
      CALL DEDD ( KDLON,KRLST,KFLEV,ITASK )                             
C                                                                       
         DO 345 JL = 1 , KDLON                                          
C                                                                       
            ZREFZ(JL,2,JK) = (1.-ZRNEB(JL)) * (ZRAY1(JL,JKM1)           
     S                     + ZREFZ(JL,2,JKM1) * ZTRA1(JL,JKM1)          
     S                     * ZTRA2(JL,JKM1) )                           
     S                     + ZRNEB(JL) * ZRE1(JL)                       
C                                                                       
            ZTR(JL,2,JKM1) = ZRNEB(JL) * ZTR1(JL)                       
     S                     + ZTRA1(JL,JKM1) * (1.-ZRNEB(JL))            
C                                                                       
            ZREFZ(JL,1,JK) = (1.-ZRNEB(JL)) * (ZRAY1(JL,JKM1)           
     S                     + ZREFZ(JL,1,JKM1) * ZTRA1(JL,JKM1)          
     S                     * ZTRA2(JL,JKM1)                             
     S                     / (1.-ZRAY2(JL,JKM1)*ZREFZ(JL,1,JKM1)))      
     S                     + ZRNEB(JL)*ZRE2(JL)                         
C                                                                       
            ZTR(JL,1,JKM1) = ZRNEB(JL) * ZTR2(JL) + (ZTRA1(JL,JKM1)     
     S                     / (1.-ZRAY2(JL,JKM1) * ZREFZ(JL,1,JKM1)))    
     S                     * (1.-ZRNEB(JL))                             
C                                                                       
 345     CONTINUE                                                       
 346  CONTINUE                                                          
      IWHERE=346                                                        
C     PRINT 878,IWHERE                                                  
C                                                                       
C*         3.5    REFLECT./TRANSMISSIVITY BETWEEN SURFACE AND LEVEL     
C                 -------------------------------------------------     
C                                                                       
 350  CONTINUE                                                          
C                                                                       
      DO 354 JABS = 1 , 2                                               
         DO 351 JL = 1 , KDLON                                          
            ZRJ(JL,JABS,KFLEV+1) = 1.                                   
            ZRK(JL,JABS,KFLEV+1) = ZREFZ(JL,JABS,KFLEV+1)               
 351     CONTINUE                                                       
C                                                                       
         DO 353 JK = 1 , KFLEV                                          
            JKL = KFLEV+1 - JK                                          
            JKLP1 = JKL + 1                                             
            DO 352 JL = 1 , KDLON                                       
               ZRE11 = ZRJ(JL,JABS,JKLP1) * ZTR(JL,JABS,JKL)            
               ZRJ(JL,JABS,JKL) = ZRE11                                 
               ZRK(JL,JABS,JKL) = ZRE11 * ZREFZ(JL,JABS,JKL)            
 352        CONTINUE                                                    
 353     CONTINUE                                                       
 354  CONTINUE                                                          
      IWHERE=354                                                        
C     PRINT 878,IWHERE                                                  
C                                                                       
C*         3.6    REFLECT./TRANSMISSIVITY WITH GREY ABSORPTION          
C                 --------------------------------------------          
C                                                                       
 360  CONTINUE                                                          
C                                                                       
      JN = 2                                                            
C                                                                       
      DO 369 JABS=1,2                                                   
C                                                                       
         DO 361 JL = 1 , KDLON                                          
            ZREFZ(JL,2,1) = PALBS(JL,INU)                               
            ZREFZ(JL,1,1) = PALBS(JL,INU)                               
 361     CONTINUE                                                       
      IWHERE=361                                                        
C     PRINT 878,IWHERE,JABS                                             
C                                                                       
      DO 364 JK = 2 , KFLEV+1                                           
         JKM1 = JK - 1                                                  
         DO 362 JL = 1 , KDLON                                          
            ZRNEB(JL) = PCLDSW(JL,JKM1)                                 
            ZAA = ZUD(JL,JABS,JKM1)                                     
            ZRKI = ZAKI(JL,JABS)                                        
            ZS(JL) = EXP(-ZRKI * ZAA * 1.66)                            
            ZG(JL) = EXP(-ZRKI * ZAA / ZRMUE(JL,JK))                    
            ZTR1(JL) = 0.                                               
            ZRE1(JL) = 0.                                               
            ZTR2(JL) = 0.                                               
            ZRE2(JL) = 0.                                               
C                                                                       
C*         3.6.1  INTRODUCING CLOUD EFFECTS                             
C                 -------------------------                             
C                                                                       
 3610 CONTINUE                                                          
C                                                                       
            PTAU(JL,INU,JKM1) = MAX( PTAU(JL,INU,JKM1) , ZEPSCT )       
            ZW(JL)= POMEGA(JL,INU,JKM1)                                 
            ZTO1(JL) = PTAU(JL,INU,JKM1) / ZW(JL)                       
     S               + ZTAUAZ(JL,JKM1) / ZPIZAZ(JL,JKM1)                
     S               + ZAA * ZRKI                                       
            ZR21(JL) = PTAU(JL,INU,JKM1) + ZTAUAZ(JL,JKM1)              
            ZR22(JL) = PTAU(JL,INU,JKM1) / ZR21(JL)                     
            ZGG(JL) = ZR22(JL) * PCG(JL,INU,JKM1)                       
     S              + (1. - ZR22(JL)) * ZCGAZ(JL,JKM1)                  
            ZW(JL) = ZR21(JL) / ZTO1(JL)                                
            ZREF(JL) = ZREFZ(JL,1,JKM1)                                 
            ZRMUZ(JL) = ZRMUE(JL,JK)                                    
 362     CONTINUE                                                       
      IWHERE=362                                                        
C     PRINT 878,IWHERE,JK                                               
C                                                                       
         CALL DEDD ( KDLON,KRLST,KFLEV,ITASK )                          
      IWHERE=3621                                                       
C     PRINT 878,IWHERE,JK                                               
C                                                                       
         DO 363 JL = 1 , KDLON                                          
C                                                                       
            ZREFZ(JL,2,JK) = (1.-ZRNEB(JL)) * (ZRAY1(JL,JKM1)           
     S                     + ZREFZ(JL,2,JKM1) * ZTRA1(JL,JKM1)          
     S                     * ZTRA2(JL,JKM1) ) * ZG(JL) * ZS(JL)         
     S                     + ZRNEB(JL) * ZRE1(JL)                       
C                                                                       
            ZTR(JL,2,JKM1)=ZRNEB(JL)*ZTR1(JL)                           
     S                    + (ZTRA1(JL,JKM1)) * ZG(JL) * (1.-ZRNEB(JL))  
C                                                                       
            ZREFZ(JL,1,JK)=(1.-ZRNEB(JL))*(ZRAY1(JL,JKM1)               
     S                  +ZREFZ(JL,1,JKM1)*ZTRA1(JL,JKM1)*ZTRA2(JL,JKM1) 
     S             /(1.-ZRAY2(JL,JKM1)*ZREFZ(JL,1,JKM1)))*ZG(JL)*ZS(JL) 
     S             + ZRNEB(JL) * ZRE2(JL)                               
C                                                                       
            ZTR(JL,1,JKM1)= ZRNEB(JL) * ZTR2(JL)                        
     S                    + (ZTRA1(JL,JKM1)/(1.-ZRAY2(JL,JKM1)          
     S                    * ZREFZ(JL,1,JKM1)))                          
     S                    * ZG(JL) * (1. -ZRNEB(JL))                    
C                                                                       
C                                                                       
 363        CONTINUE                                                    
      IWHERE=363                                                        
C     PRINT 878,IWHERE,JK                                               
 364     CONTINUE                                                       
      IWHERE=364                                                        
C     PRINT 878,IWHERE                                                  
C                                                                       
C*         3.6.2  REFLECT./TRANSMISSIVITY BETWEEN SURFACE AND LEVEL     
C                 -------------------------------------------------     
C                                                                       
 3620 CONTINUE                                                          
C                                                                       
         DO 368 KREF=1,2                                                
C                                                                       
            JN = JN + 1                                                 
C                                                                       
            DO 365 JL = 1 , KDLON                                       
               ZRJ(JL,JN,KFLEV+1) = 1.                                  
               ZRK(JL,JN,KFLEV+1) = ZREFZ(JL,KREF,KFLEV+1)              
 365        CONTINUE                                                    
C                                                                       
            DO 367 JK = 1 , KFLEV                                       
               JKL = KFLEV+1 - JK                                       
               JKLP1 = JKL + 1                                          
               DO 366 JL = 1 , KDLON                                    
                  ZRE11 = ZRJ(JL,JN,JKLP1) * ZTR(JL,KREF,JKL)           
                  ZRJ(JL,JN,JKL) = ZRE11                                
                  ZRK(JL,JN,JKL) = ZRE11 * ZREFZ(JL,KREF,JKL)           
 366           CONTINUE                                                 
 367        CONTINUE                                                    
 368     CONTINUE                                                       
 369  CONTINUE                                                          
C                                                                       
C*         3.7    UPWARD (ZRK) AND DOWNWARD (ZRJ) PSEUDO-FLUXES         
C                 ---------------------------------------------         
C                                                                       
 370  CONTINUE                                                          
C                                                                       
      DO 374 JK = 1 , KFLEV+1                                           
         DO 373 JAJ = 1 , 5 , 2                                         
            JAJP = JAJ + 1                                              
            DO 372 JL = 1 , KDLON                                       
               ZRJ(JL,JAJ,JK)=        ZRJ(JL,JAJ,JK) - ZRJ(JL,JAJP,JK)  
               ZRK(JL,JAJ,JK)=        ZRK(JL,JAJ,JK) - ZRK(JL,JAJP,JK)  
               ZRJ(JL,JAJ,JK)= MAX( ZRJ(JL,JAJ,JK) , ZEELOG )           
               ZRK(JL,JAJ,JK)= MAX( ZRK(JL,JAJ,JK) , ZEELOG )           
 372        CONTINUE                                                    
 373     CONTINUE                                                       
 374  CONTINUE                                                          
C                                                                       
      DO 377 JK = 1 , KFLEV+1                                           
         DO 376 JAJ = 2 , 6 , 2                                         
            DO 375 JL = 1 , KDLON                                       
               ZRJ(JL,JAJ,JK)= MAX( ZRJ(JL,JAJ,JK) , ZEELOG )           
               ZRK(JL,JAJ,JK)= MAX( ZRK(JL,JAJ,JK) , ZEELOG )           
 375        CONTINUE                                                    
 376     CONTINUE                                                       
 377  CONTINUE                                                          
      IWHERE=377                                                        
C     PRINT 878,IWHERE                                                  
C                                                                       
C*         3.8    EFFECTIVE ABSORBER AMOUNTS BY INVERSE LAPLACE         
C                 ---------------------------------------------         
C                                                                       
 380  CONTINUE                                                          
C                                                                       
      DO 387 JK = 1 , KFLEV+1                                           
         JKKI = 1                                                       
         DO 385 JAJ = 1 , 2                                             
            DO 384 JN = 1 , 2                                           
               JN2J = JN + 2 * JAJ                                      
               JKKP4 = JKKI + 4                                         
C                                                                       
C*         3.8.1  EFFECTIVE ABSORBER AMOUNTS                            
C                 ---------------------------------------------         
C                                                                       
 3810 CONTINUE                                                          
C                                                                       
C                                                                       
               DO 3811 JL = 1 , KDLON                                   
                  ZW(JL) = LOG( ZRJ(JL,JN,JK) / ZRJ(JL,JN2J,JK))        
     S                   / ZAKI(JL,JAJ)                                 
 3811          CONTINUE                                                 
C                                                                       
C*         3.8.2  TRANSMISSION FUNCTION                                 
C                 ---------------------                                 
C                                                                       
 3820 CONTINUE                                                          
C                                                                       
                CALL SWTT ( INU, JAJ, KDLON,KRLST,KFLEV,ITASK )         
C                                                                       
                DO 3821 JL = 1 , KDLON                                  
                   ZRL(JL,JKKI) = ZR1(JL)                               
                   ZRUEF(JL,JKKI) = ZW(JL)                              
                   ZW(JL) = LOG( ZRK(JL,JN,JK) / ZRK(JL,JN2J,JK))       
     S                    / ZAKI(JL,JAJ)                                
 3821           CONTINUE                                                
C                                                                       
                CALL SWTT ( INU, JAJ, KDLON,KRLST,KFLEV,ITASK )         
C                                                                       
                DO 383 JL = 1 , KDLON                                   
                   ZRL(JL,JKKP4) = ZR1(JL)                              
                   ZRUEF(JL,JKKP4) = ZW(JL)                             
 383            CONTINUE                                                
C                                                                       
                JKKI=JKKI+1                                             
 384         CONTINUE                                                   
 385      CONTINUE                                                      
      IWHERE=385                                                        
C     PRINT 878,IWHERE                                                  
C                                                                       
C*         3.8.3  UPWARD AND DOWNWARD FLUXES WITH H2O AND UMG ABSORPTION
C                 ------------------------------------------------------
C                                                                       
 3830 CONTINUE                                                          
C                                                                       
          DO 386 JL = 1 , KDLON                                         
             PFDOWN(JL,JK) = ZRJ(JL,1,JK) * ZRL(JL,1) * ZRL(JL,3)       
     S                    + ZRJ(JL,2,JK) * ZRL(JL,2) * ZRL(JL,4)        
             PFUP(JL,JK)   = ZRK(JL,1,JK) * ZRL(JL,5) * ZRL(JL,7)       
     S                    + ZRK(JL,2,JK) * ZRL(JL,6) * ZRL(JL,8)        
 386      CONTINUE                                                      
 387  CONTINUE                                                          
      IWHERE=387                                                        
C     PRINT 878,IWHERE                                                  
C                                                                       
C*         3.9    INTRODUCTION OF OZONE ABSORPTION                      
C                 --------------------------------                      
C                                                                       
 390  CONTINUE                                                          
C                                                                       
      JABS=3                                                            
      DO 395 JK = 1 , KFLEV+1                                           
         DO 392 JL = 1 , KDLON                                          
            ZW(JL) = ZUD(JL,JABS,JK)                                    
 392     CONTINUE                                                       
C                                                                       
         CALL SWTT ( INU, JABS, KDLON,KRLST,KFLEV,ITASK )               
C                                                                       
         DO 393 JL = 1 , KDLON                                          
            PFDOWN(JL,JK) = ZR1(JL) * PFDOWN(JL,JK) * SUN(INU)          
            ZW(JL) = ZUM(JL,JK)                                         
 393     CONTINUE                                                       
C                                                                       
         CALL SWTT ( INU, JABS, KDLON,KRLST,KFLEV,ITASK )               
C                                                                       
         DO 394 JL = 1 , KDLON                                          
            PFUP(JL,JK) = ZR1(JL) * PFUP(JL,JK) * SUN(INU)              
 394     CONTINUE                                                       
 395  CONTINUE                                                          
      IWHERE=395                                                        
C     PRINT 878,IWHERE                                                  
C                                                                       
C     ------------------------------------------------------------      
C                                                                       
C*         4.     NET TOTAL SHORTWAVE FLUXES                            
C                 --------------------------                            
C                                                                       
C 400  CONTINUE                                                         
CZ      DO 402 JK = 1 , KFLEV+1                                         
C         DO 401 JL = 1 , KDLON                                         
C            PFUP(JL,JK)   = (PFUP(JL,JK)   + ZFU(JL,JK)) * ZFACT(JL)   
C            PFDOWN(JL,JK) = (PFDOWN(JL,JK) + ZFD(JL,JK)) * ZFACT(JL)   
C 401     CONTINUE                                                      
C 402  CONTINUE                                                         
C      IWHERE=402                                                       
C     PRINT 878,IWHERE                                                  
C                                                                       
C                                                                       
C                                                                       
C*         4.     NET TOTAL SHORTWAVE FLUXES                            
C                 --------------------------                            
C                                                                       
 400  CONTINUE                                                          
      DO 402 JK = 1 , KFLEV+1                                           
         DO 401 JL = 1 , KDLON                                          
c            IF(ITASK.EQ.1)THEN                                         
c              PFUP(JL,JK)   = ZFU(JL,JK) * ZFACT(JL)                   
c              PFDOWN(JL,JK) = ZFD(JL,JK) * ZFACT(JL)                   
c            ELSE IF(ITASK.EQ.2)THEN                                    
c              PFUP(JL,JK)   = PFUP(JL,JK) * ZFACT(JL)                  
c              PFDOWN(JL,JK) = PFDOWN(JL,JK) * ZFACT(JL)                
c            ELSE IF(ITASK.EQ.3)THEN                                    
              PFUP(JL,JK)   = (PFUP(JL,JK)   + ZFU(JL,JK)) * ZFACT(JL)  
              PFDOWN(JL,JK) = (PFDOWN(JL,JK) + ZFD(JL,JK)               
     *                      + FOXDWN(JL,JK)) * ZFACT(JL)                
c            ENDIF                                                      
 401     CONTINUE                                                       
 402  CONTINUE                                                          
      IWHERE=402                                                        
C     PRINT 878,IWHERE                                                  
C                                                                       
      DO 501 JL=1,NRLST                                                 
         DO 502 JK=1,NLEV                                               
            ZCLDSW(JL,JK) = PCLDSW(JL,JK)                               
            ZOZ   (JL,JK) = POZ   (JL,JK)                               
            ZPMB  (JL,JK) = PPMB  (JL,JK)                               
            ZTAVE (JL,JK) = PTAVE (JL,JK)                               
            ZFSDWN(JL,JK) = PFDOWN(JL,JK)                               
            ZFSUP (JL,JK) = PFUP  (JL,JK)                               
            DO 503 II=1,2                                               
               ZCG   (JL,II,JK) = PCG   (JL,II,JK)                      
               ZOMEGA(JL,II,JK) = POMEGA(JL,II,JK)                      
               ZTAU  (JL,II,JK) = PTAU  (JL,II,JK)                      
 503        CONTINUE                                                    
 502     CONTINUE                                                       
         DO 504 II=1,2                                                  
            ZALBSU(JL,II) = PALBS(JL,II)                                
 504     CONTINUE                                                       
         ZPMB  (JL,NRLEV+1) = PPMB  (JL,NRLEV+1)                        
         ZFSDWN(JL,NRLEV+1) = PFDOWN(JL,NRLEV+1)                        
         ZFSUP (JL,NRLEV+1) = PFUP  (JL,NRLEV+1)                        
 501  CONTINUE                                                          
C                                                                       
      RETURN                                                            
      END                                                               
C                                                                       
      SUBROUTINE SWTT ( KNU, KA, KDLON, KRLST, KFLEV, ITASK)            
C                                                                       
C**** *SWTT* - COMPUTES THE SHORTWAVE TRANSMISSION FUNCTIONS            
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C           THIS ROUTINE COMPUTES THE TRANSMISSION FUNCTIONS FOR ALL THE
C     ABSORBERS (H2O, UNIFORMLY MIXED GASES, AND O3) IN THE TWO SPECTRAL
C     INTERVALS.                                                        
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C          *SWTT* IS CALLED FROM *SW*.                                  
C                                                                       
C        EXPLICIT ARGUMENTS                                             
C        --------------------                                           
C KNU    :                     ; INDEX OF THE SPECTRAL INTERVAL         
C KA     :                     ; INDEX OF THE ABSORBER                  
C                                                                       
C        IMPLICIT ARGUMENTS                                             
C        --------------------                                           
C                                                                       
C     ==== INPUTS ===                                                   
C PU     : (KDLON)             ; ABSORBER AMOUNT                        
C     ==== OUTPUTS ===                                                  
C PTR    : (KDLON)             ; TRANSMISSION FUNCTION                  
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C                                                                       
C          TRANSMISSION FUNCTION ARE COMPUTED USING PADE APPROXIMANTS   
C     AND HORNER'S ALGORITHM.                                           
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C          NONE                                                         
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C                                                                       
C        SEE RADIATION'S PART OF THE MODEL'S DOCUMENTATION              
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C        JEAN-JACQUES MORCRETTE  *ECMWF*                                
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C        ORIGINAL : 88-12-15                                            
C------------------------------------------------------------------     
C      IMPLICIT LOGICAL (L)                                             
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRSW* SWITCHES RELATED TO THE RADIATION SCHEME.         
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      IMPLICIT REAL (A-H,O-Z)                                           
C      IMPLICIT UNDEFINED (Z)                                           
C      IMPLICIT LOGICAL (L)                                             
C      IMPLICIT UNDEFINED (L)                                           
      IMPLICIT INTEGER (I-K,M-N)                                        
C      IMPLICIT UNDEFINED (I-K,M-N)                                     
C      IMPLICIT REAL (P)                                                
C      IMPLICIT UNDEFINED (A-H,O,Q-Y)                                   
C                                                                       
C everything except the parameters is defined                           
C in the common blocks                                                  
C                                                                       
      COMMON/COM1/                                                      
C     I NRADFR, NRPART, NRINT, NRADPFR, NRADPLA,                        
C     I NRLSTI,                                                         
     R XLAT,                                                            
     R RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR, ZCCO2,              
C     R PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3,                 
     L LOH, LOM, LOL, LORAD                                             
      COMMON/COM2/                                                      
     I ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,                  
     I         JLOOP, KRLEV,                                            
     I        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     I        ITH, ITM, ITL,                                            
     I        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     I        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     I        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     I        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      COMMON/COM3/                                                      
     I IND4, KM12, KXD, KXU, IG, KBS, KDD,                              
     I        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     I        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     I        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     I        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     I        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
      COMMON/COM4/                                                      
     R           COZQC,COZQS,COZHC,COZHS,                               
     R CSUPSAT,                                                         
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
      COMMON/COM5/                                                      
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NUMVC,                                                           
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      COMMON/COM6/                                                      
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD,       
     R DTIME,    TWODT,    EPS,                                         
     R SINLON,     COSLON,                                              
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2,                          
     I NFAX,                                                            
     R CRAE,                                                            
     R CDISSEM                                                          
      COMMON/COM7/                                                      
     R     CCRH,                                                        
     L LO,LO1,LO2,LODIA,LODIAP,                                         
     R TAUA, PIZA, CGA, CAER, CAEROS                                    
      COMMON/COM7A/                                                     
     R  RT1,WG1,AT,BT,CNTNU,O1H,O2H,PIALF0                              
     S ,OCT,TREF,XP,TSTAND,GA,GB,CE,ETY,PTY                             
      COMMON/COM7B/                                                     
     S TINTP,TSTP,                                                      
     S  TBO3,AO3,BO3,TB,ALP,BET,ULG,ULGO,PLG,AP,BP,TO3B,O3A,O3B,        
     R          CDAY,CCO2,CH2O,DIFF,X10E                                
       COMMON/COM7C/                                                    
     S  ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW,                       
     R  APAD,BPAD,D,CRAY,SUN,                                           
     R   TTT,                                                           
     R   TM1                                                            
      COMMON/COM8/                                                      
     R       TSM1M,                                                     
     R       QM1,                                                       
     R       SNM1M,                                                     
     R       ALBM,                                                      
     R       AVVELM,                                                    
     R   SLMM,                                                          
     R       APHM1,                                                     
     R       APM1                                                       
      COMMON/COM9/                                                      
     L LOLAND,                                                          
     R   ZTI,                                                           
     *       ZCTI,                                                      
     *       ZCPH,                                                      
     *       ZWV,                                                       
     *       ZSW,                                                       
     *       ZALTE,                                                     
     *       ZCALTE,                                                    
     *       ZALSO                                                      
      COMMON/COM10/                                                     
     *       ZCALSO,                                                    
     R   ZOZQ,                                                          
     *       ZOZH,                                                      
     R   ZTRSOL,                                                        
     *       ZEMTER,                                                    
     R   TRSOLM,                                                        
     *       EMTERM,                                                    
     R   AMU0,                                                          
     *       RDAYL                                                      
      COMMON/COM11/                                                     
     R   CLC,                                                           
     *       TI,                                                        
     *       DP,                                                        
     *       AADS,                                                      
     *       AEQ1,                                                      
     *       AEQ2,                                                      
     *       AEQ3,                                                      
     *       AEQ4,                                                      
     *       AEQ5,                                                      
     *       SSW                                                        
      COMMON/COM12/                                                     
     *       WV,                                                        
     *       QOF,                                                       
     *       QCF,                                                       
     *       CLWA,                                                      
     *       APRE,                                                      
     *       SMU0,                                                      
     *       ALTE,                                                      
     *       ALSO                                                       
      COMMON/COM13/                                                     
     R   ZDPO,                                                          
     *       ZQCFO,                                                     
     *       ZAEQSO,                                                    
     *       ZQOFO,                                                     
     *       ZDPN,                                                      
     *       ZQCFN,                                                     
     *       ZAEQSN,                                                    
     *       ZQOFN                                                      
      COMMON/COM14/                                                     
     R   FLT,                                                           
     *       FLS,                                                       
     R   DIAG,                                                          
     *       DIA1,                                                      
     *       ZDIAG,                                                     
     *       ZDIA1,                                                     
     *       ZDIA,                                                      
     *       ZDIAF,                                                     
     *       ZDIAT,                                                     
     R PGG                                                              
      COMMON/COM15/                                                     
     #     PREF,                                                        
     #     PRE1,                                                        
     #     PRE2,                                                        
     #     PRMUZ,                                                       
     #     PTO1,                                                        
     #     PTR1,                                                        
     #     PTR2,                                                        
     #     PW,                                                          
     R     PALBSN                                                       
      COMMON/COM16/                                                     
     R ZABSCU,                                                          
     S     ZABSLY,                                                      
     S     ZADJD ,                                                      
     S     ZADJU ,                                                      
     S     ZBGND ,                                                      
     S     ZBINT ,                                                      
     S     ZBSUR ,                                                      
     S     ZBSUIN,                                                      
     S     ZBTOP ,                                                      
     S     PCLDLW,                                                      
     S     ZCLOUD,                                                      
     S     ZCNTRB,                                                      
     S     ZCUMP ,                                                      
     S     ZCUMPT,                                                      
     S     ZDBDT                                                        
      COMMON/COM17/                                                     
     S     ZDBSL ,                                                      
     S     ZDISTD,                                                      
     S     ZDISTU,                                                      
     S     ZDNF  ,                                                      
     S     PDT0  ,                                                      
     S     ZDUC                                                         
      COMMON/COM18/                                                     
     S     ZEFFT ,                                                      
     S     ZEMU  ,                                                      
     S     ZEPHIO,                                                      
     S     ZEPSC2,                                                      
     S     ZEPSC3,                                                      
     S     ZEPSH1,                                                      
     S     ZEPSH2,                                                      
     S     ZEPSH3                                                       
      COMMON/COM19/                                                     
     S     ZEPSH4,                                                      
     S     ZEPSH5,                                                      
     S     ZEPSH6,                                                      
     S     ZEPSIO,                                                      
     S     ZFACTC,                                                      
     S     ZZFD  ,                                                      
     S     ZFDD  ,                                                      
     S     ZFDN                                                         
      COMMON/COM20/                                                     
     S     PFLUX , PFLUC,                                               
     S     ZFLWP , PCTS                                                 
     S     ZZFU  ,                                                      
     S     ZFUP  ,                                                      
     S     ZGLAYD,                                                      
     S     ZGLAYU,                                                      
     S     ZOMEGA,                                                      
     S     ITX   ,                                                      
     S     IXDIA ,                                                      
     S     IXT   ,                                                      
     S     IXTSU ,                                                      
     S     IXTTP ,                                                      
     S     IMX   ,                                                      
     S     IMXP                                                         
      COMMON/COM21/                                                     
     R ZOZ   ,                                                          
     S     PPMB  ,                                                      
     S     ZRES  , PB                                                   
     S     ZSSIG ,                                                      
     S     PTAU  ,                                                      
     S     PTAVE ,                                                      
     S     ZTAVIC,                                                      
     S     ZTI1  ,                                                      
     S     PTL                                                          
      COMMON/COM22/                                                     
     S     ZTT   ,                                                      
     S     ZTT1  ,                                                      
     S     ZTT2  ,                                                      
     S     ZUAER ,                                                      
     S     ZUPF  ,                                                      
     S     ZUU   ,                                                      
     S     ZXD                                                          
      COMMON/COM23/                                                     
     S     ZXN   ,                                                      
     S     ZXOZ  ,                                                      
     S     ZXWV  ,                                                      
     S     ZZZ   ,                                                      
     S     ZRES2 ,                                                      
     S     ZTI2  ,                                                      
     S     ZBLAY ,                                                      
     S     ZBLEV ,                                                      
     S     ZCTS  ,                                                      
     S     ZHFG                                                         
      COMMON/COM24/                                                     
     I KT,                                                              
     R    PUU   ,                                                       
     S        PTT   ,                                                   
     S        ZZ                                                        
     R  PCLFR  ,                                                        
     S      PT     ,                                                    
     S      PQOF   ,                                                    
     S      PQLWP  ,                                                    
     S      PEMIS  ,                                                    
     S      PFLT                                                        
      COMMON/COM25/                                                     
     R  ZALBSU,                                                         
     S      ZCG   ,                                                     
     S      ZCLDLW,                                                     
     S      ZCLDSW,                                                     
     S      ZCOOLR,                                                     
     S      ZDT0  ,                                                     
     S      ZEMD  ,                                                     
     S      ZFLUX ,                                                     
     S      ZPMB  ,                                                     
     S      ZTAU  ,                                                     
     S      ZTAVE ,                                                     
     S      ZTL   ,                                                     
     S      ZFSDWN                                                      
      COMMON/COM26/                                                     
     S      ZFSUP ,                                                     
     S      ZHEATR,                                                     
     R   PDP   ,                                                        
     S       PWV   ,                                                    
     S       PFLS  ,                                                    
     S       PPSOL ,                                                    
     S       PRMU0 ,                                                    
     S       PAER  ,                                                    
     S       PALBS ,                                                    
     S       PCG   ,                                                    
     S       PCLDSW,                                                    
     S       POMEGA,                                                    
     S       POZ                                                        
      COMMON/COM27/                                                     
     R   PFDOWN,                                                        
     S       PFUP  ,                                                    
     R   ZAKI  ,                                                        
     S       ZCGAZ ,                                                    
     S       ZCNEB ,                                                    
     S       ZC1I  ,                                                    
     S       ZDSC  ,                                                    
     S       ZDSH                                                       
      COMMON/COM28/                                                     
     S       ZDSIG ,                                                    
     S       ZFACT ,                                                    
     S       ZFACT1 ,                                                   
     S       ZFACT2 ,                                                   
     S       ZFACT3 ,                                                   
     S       ZFACT4 ,                                                   
     S       ZFACT5 ,                                                   
     S       ZFACT6 ,                                                   
     S       ZFD   ,                                                    
     S       ZFRAC ,                                                    
     S       ZFU   ,                                                    
     S       ZG    ,                                                    
     S       ZGG   ,                                                    
     S       ZPIZAZ                                                     
      COMMON/COM29/                                                     
     S       ZRAYL ,                                                    
     S       ZRAY1 ,                                                    
     S       ZRAY2 ,                                                    
     S       ZREF  ,                                                    
     S       ZREFZ ,                                                    
     S       ZRE1  ,                                                    
     S       ZRE2  ,                                                    
     S       ZRJ   ,                                                    
     S       ZRK   ,                                                    
     S       ZRL   ,                                                    
     S       ZRMU  ,                                                    
     S       ZRMUE ,                                                    
     S       ZRMUZ                                                      
      COMMON/COM30/                                                     
     S       ZRNEB ,                                                    
     S       ZRUEF ,                                                    
     S       ZR1   ,                                                    
     S       ZR2   ,                                                    
     S       ZR21  ,                                                    
     S       ZR22  ,                                                    
     S       ZR23  ,                                                    
     S       ZS    ,                                                    
     S       ZSEC  ,                                                    
     S       ZSS1  ,                                                    
     S       ZTAUAZ                                                     
      COMMON/COM31/                                                     
     S       ZTOT  ,                                                    
     S       ZTO1  ,                                                    
     S       ZTR   ,                                                    
     S       ZTRA1 ,                                                    
     S       ZTRA2 ,                                                    
     S       ZTR1                                                       
      COMMON/COM32/                                                     
     S       ZTR2  ,                                                    
     S       ZUD   ,                                                    
     S       ZUM   ,                                                    
     S       ZU1D  ,                                                    
     S       ZU2D  ,                                                    
     S       ZW    ,                                                    
     S       ZN175 ,                                                    
     S       ZN190 ,                                                    
     S       ZO175 ,                                                    
     S       ZO190 ,                                                    
     S       ZP75  ,                                                    
     S       ZP90  ,                                                    
     S       ZSIGN ,                                                    
     S       ZSIGO                                                      
      COMMON/COM33/                                                     
     R   PTR,                                                           
     S       PU,                                                        
     R   ZRH,                                                           
     *       ZCH,                                                       
     *       ZCM,                                                       
     *       ZCL,                                                       
     *       ZCT,                                                       
     *       ZCC,                                                       
     I NTOPCM                                                           
      COMMON/COM34/                                                     
     *        NBASECM,                                                  
     R   ARPRCM,                                                        
     I ITOPH,                                                           
     *        ITOPM,                                                    
     *        ITOPL,                                                    
     L LOC,                                                             
     R   ZQS,                                                           
     *       ZCLC,                                                      
     *       ZQLWP                                                      
      COMMON/COM35/                                                     
     R   VERVEL,                                                        
     *       ZTHETA,                                                    
     *       ZETA,                                                      
     *       ZDTHMIN,                                                   
     I INVB,                                                            
     R   ZCTLWP,                                                        
     *       ZQLOC,                                                     
     *       ZQMEC,                                                     
     *       ZQHIC,                                                     
     R   ZFSOT,                                                         
     #       ZFTET,                                                     
     #       ZFLT                                                       
      COMMON/COM36/                                                     
     #       ZFSO ,                                                     
     #       ZFTE ,                                                     
     #       ZFLB ,                                                     
     #       ZDTDT,                                                     
     #       HEATO,                                                     
     #       HEATE,                                                     
     #       HEATR,                                                     
     #       ZI0                                                        
C                                                                       
      INTEGER                                                           
C     I NRADFR,   NRPART,   NRINT,    NRLSTI,  NRADPFR,  NRADPLA,       
     I NRLST,    NRLEV,    NRLEVP1, NLEV, NLP2, NLEVP1,                 
     I NLON, IRLEV, NTRA, NLAT, ITIM, NUA, NINT, NRPARTI                
C                                                                       
C   NRLEV .... number of vertical levels                                
C   NRLST .... number of points on a latitude line                      
C   NUA   .... number of absorber amount                                
C   NINT  .... number of spectral intervals                             
C   NTRA  .... number of transmission functions                         
C   NLAT  .... number of latitude points                                
C   ITIM  .... number of month under consideration                      
C                                                                       
C      PARAMETER (NRLEV=122,NUA=34,NINT=6,NTRA=34, NLAT=1,              
      PARAMETER (NRLEV=21,NUA=34,NINT=6,NTRA=34, NLAT=1,                
cccm      PARAMETER (NRLEV=37,NUA=18,NINT=6,NTRA=19, NLAT=1,            
cccm     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=10,NLEV=NRLEV,        
     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=7,NLEV=NRLEV,             
     #           NLP2=NRLST, NLEVP1=NRLEVP1, NLON=NRLST,                
     #           NRPARTI=1)                                             
C                                                                       
      REAL YCLOCK, XLAT(NLAT), Z0, DZ, ALBMPAR, SNPAR,                  
     #     TSUPAR, CTFREEZ, DELTAC, DELTAO, DELTAH                      
C                                                                       
C   YTIME  .... time of the year in radians                             
C   YCLOCK .... time of the day in radians                              
C   XLAT   .... latitudes in radians                                    
C   Z0     .... zero point of height grid                               
C   DZ     .... Interval between height levels                          
C   ALBMPAR.... sw surface albedo as function of longitude:             
C           yearly averaged sea-ice: 0.55                               
C                           water  : 0.07                               
C                           land   : 0.07.lt.ALBM.lt.0.80               
C                                    depending on snow                  
C   SNPAR  .... snow depth in meter                                     
C   TSUPAR .... surface temperature (Kelvin)                            
C   CTFREEZ.... freezing temperature of sea-water (Kelvin)              
C   DELTAC .... Doppler parameter for line shapes of CO2                
C   DELTAO .... Doppler parameter for line shapes of O3                 
C                                                                       
      PARAMETER (Z0=0., CTFREEZ=271.26,                                 
C     PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
     #           DZ=0.289, ALBMPAR=0.2, SNPAR=0.,                       
cccm  PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
cccm     #           DZ=0.15, ALBMPAR=0.3, SNPAR=0.,                    
CZ     #           DELTAC=0., DELTAO=0.)                                
     #           DELTAC=6.0E-04, DELTAO=1.E-02, DELTAH=5.0E-04)         
C      PARAMETER (YCLOCK=0., Z0=0., TSUPAR=273., CTFREEZ=271.26,        
C     #           DZ=0.46875, ALBMPAR=0.07, SNPAR=0.)                   
C      PARAMETER (YCLOCK=0., Z0=2.608, TSUPAR=273., CTFREEZ=271.26,     
C     #           DZ=0.2878, ALBMPAR=0.07, SNPAR=0.)                    
C                                                                       
      LOGICAL LDIUR, LOLPAR                                             
C                                                                       
C   LDIUR .... true: sun at time, false: diurnally averaged             
C   LOLPAR.... true: land surface, false: else                          
C                                                                       
C      PARAMETER (LDIUR=.TRUE., LOLPAR=.TRUE.)                          
       PARAMETER (LOLPAR=.TRUE.)                                        
C                                                                       
      INTEGER  C2TEMP                                                   
      CHARACTER*64  MIDTMP                                              
cccm      PARAMETER (C2TEMP=10,                                         
cccm     #    MIDTMP='/home/atm/ugamp/midrad/radfildat.dir/MIDTMP.DAT') 
ccc1      PARAMETER (C2TEMP=10, MIDTMP='tprfil2.dat')                   
C                                                                       
      INTEGER  IRLONI, IRLST, IRLVP1, JPNLEV, JPNLVP1                   
C                                                                       
      PARAMETER (IRLONI=NRLST, IRLST=NRLST,                             
     #           IRLVP1=NRLEVP1, JPNLEV=NRLEV,                          
     #           JPNLVP1=NRLEVP1, IRLEV=NRLEV)                          
C                                                                       
      INTEGER NIPD, NIPD2, NTR, NG1, NG1P1                              
      PARAMETER(NIPD=8,NIPD2=16,NTR=11,NG1=2,NG1P1=3)                   
C                                                                       
C    definition of the functions                                        
C                                                                       
      REAL       CVMGT, SIGMA2                                          
      INTEGER    CVMGI                                                  
C                                                                       
C    definitons of the subroutine cloud                                 
C                                                                       
      REAL RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR                  
C                                                                       
C    missing definitions in morcrette                                   
C                                                                       
C      REAL   PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3            
      REAL ZCCO2                                                        
                                                                        
      LOGICAL LOH, LOM, LOL, LORAD                                      
C                                                                       
      INTEGER ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,           
     #        IZQS, IZCLC, IZQLWP, JLOOP, KRLEV,                        
     #        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     #        ITH, ITM, ITL,                                            
     #        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     #        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     #        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     #        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      INTEGER IND4, KM12, KXD, KXU, IG, KBS, KDD,                       
     #        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     #        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     #        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     #        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     #        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD1* SPECTRAL DISTRIBUTION OF AEROSOLS AND OZONE.     
C                     (TRIANGULAR *T10* TRUNCATIONFOR AEROSOLS).        
C                     (TRIANGULAR *T5* TRUNCATIONFOR OZONE).            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R           COZQC(21),COZQS(15),COZHC(21),COZHS(15)                
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD2* CONSTANTS INTERNAL TO THE RADIATION SUBROUTINES. 
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CSUPSAT                                                          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMCON* BASIC UNIVERSAL CONSTANTS AND DERIVED CONSTANTS.  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
C                                                                       
C                                                                       
C       --------------------------------------------------------------- 
C      PARAMETER(                                                       
C     I JPM=106,JPN=106,JPK=106,JPGL=160,JPNLON=320,                    
C     I JPT1MAX=400,          JPT2MAX=50,                               
C     I JPT3MAX=300,          JPT4MAX=50,                               
C     I JPHASH=2048,          JPNTRCRS=20,                              
C     I JPTASKS=4,     JPLOCKS=50,    JPEVENT=4,                        
C     I JPLTMT=JPTASKS,                                                 
C     I JPNUMW=352,            JPNUMP=352,                              
C     I JPTABLE=1024,                                                   
C     I JPMAXB =60,                                                     
C     I JPMAXPB=32,                                                     
C     I JPG3FLD=43,         JPG3XF=99,         JPG3CMP=17,              
C     I JPMP1=JPM+1,        JPNP1=JPN+1,       JPKP1=JPK+1,             
C     I JP2MP1=JPMP1*2,     JP4MP1=JPMP1*4,                             
C     I JPED=JPN/2+1,       JPOD=JPNP1/2,                               
C     I IPSP0=JPM+JPN-JPK,  IPSP1=JPNP1*JPMP1, IPSP2=IPSP0*(IPSP0+1)/2, 
C     I JPSP=IPSP1-IPSP2,   JP2SP=2*JPSP,                               
C     I JPHGL=JPGL/2,       JPNLP2=JPNLON+2,                            
C     I                     JPNVCLV=4*JPNLVP1)                          
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C     J. K. GIBSON     E.C.M.W.F.     22/01/82.                         
C     MODIFIED M.J     E.C.M.W.F.     13/12/82.                         
C                                                                       
      INTEGER                                                           
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      LOGICAL                                                           
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD        
      REAL                                                              
     R DTIME,    TWODT,    EPS                                          
C                                                                       
C      INTEGER                                                          
C     I NWTIME(JPNUMW),     NROW(JPTASKS),                              
C     I NPTIME(JPNUMP)                                                  
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMGAU* - QUANTITIES RELATED TO THE GAUSSIAN GRID.      
C                                                                       
      REAL                                                              
     R SINLON(NLP2),     COSLON(NLP2)                                   
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMTRU* -QUANTITIES RELATED TO THE SPECTRAL TRUNCATION. 
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSIM* - QUANTITIES NEEDED FOR THE SEMI-IMPLICIT SCHEME
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMHDI* - COEFFICIENTS FOR HORIZONTAL DIFFUSION.        
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMFFT* - QUANTITIES NEEDED FOR THE FAST *FOURIER       
C                         TRANSFORMS.                                   
C                                                                       
      INTEGER                                                           
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2                           
C      REAL                                                             
C     R TRIG(JPNLON)                                                    
      INTEGER                                                           
     I NFAX(10)                                                         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSTA* - ARRAYS AND PARAMETERS FOR STATISTICS.         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMDIA* - QUANTITIES RELATED TO GLOBAL PHYSICAL         
C                         DIAGNOSTICS.                                  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPSW* SWITCHES RELATED TO THE PARAMETERISATIONS OF      
C                     DIABATIC PROCESSES.                               
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMDSW* SWITCHES RELATED TO THE DYNAMICS                  
C                     AND THE GENERAL CONTROL OF THE FORECAST.          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH1* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE RADIATION SUBROUTINES.                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CRAE,                                                            
     R CDISSEM                                                          
C                                                                       
C      REAL       COSRAD(128),        SINRAD(128)                       
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH2* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE PHYSICAL SUBROUTINES (EXCEPT RADIATION    
C                     ONES).                                            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL       CCRH(JPNLEV)                                           
C                                                                       
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMDIZ* VARIABLES AND POINTERS FOR ZONAL MEAN DIAGNOSTICS 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMMSK* VARIABLES AND POINTERS FOR AREA MEAN DIAGNOSTICS. 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMSDS* - START/RESTART BLOCK.                          
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMTASK* - TASK RELATED VARIABLES.                      
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*         *COMRAD* - RADIATION POINTERS.                               
C                                                                       
      LOGICAL LO,LO1,LO2,LODIA,LODIAP                                   
C---------------------------------------------------------------------  
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C     COMMON *YOMAER* RADIATIVE CHARACTERISTICS OF THE AEROSOLS         
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
      REAL TAUA(2,5), PIZA(2,5), CGA(2,5), CAER(5,5), CAEROS            
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C     COMMON *YOMLW*                                                    
C                                                                       
C*    COEFFICIENTS FOR THE LONGWAVE RADIATION SUBROUTINES               
C                                                                       
C NINT  :  NUMBER OF SPECTRAL INTERVALS                                 
C NIPD  :  NUMBER OF BANDS DESCRIBED BY PADE APPROXIMANTS               
C NTR   :  NUMBER OF REFERENCE TEMPERATURES                             
C NTRA  :  NUMBER OF TRANSMISSION FUNCTIONS                             
C NUA   :  NUMBER OF ABSORBER AMOUNTS                                   
C NG1   :  ORDER OF THE GAUSSIAN QUADRATURE                             
C                                                                       
C RT1   :  ROOTS OF THE GAUSSIAN POLYNOMIAL                             
C WG1   :  WEIGHTS OF THE GAUSSIAN POLYNOMIAL                           
C AT, BT:  COEFFICIENTS OF THE TEMPERATURE DEPENDENCE                   
C CNTNU :  H2O CONTINUUM ABSORPTION COEFFICIENTS                        
C O1H,O2H,PIALF0,OCT: OZONE LONGWAVE ABSORPTION COEFFICIENTS            
C TREF, TSTAND      : REFERENCE TEMPERATURES                            
C XP    :  COEFFICIENTS OF POLYNOMIAL DEVELOPMENTS OF PLANCK FUNCTION   
C GA,GB :  PADE APPROXIMANTS OF H2O, CO2 TRANSMISSION FUNCTIONS         
C ETY,PTY :  PARAMETERS OF H2O CONTINUUM                                
C TINTP :  REFERENCE TEMPERATURES OF PRE-COMPUTED TRANSMISSIVITIES      
C CE    :  EMS COEFFS OF H2O TRANSMISSION FUNCTIONS                     
C                                                                       
      REAL  RT1(2),WG1(2),AT(12,3),BT(12,3),CNTNU(4),O1H                
     S ,OCT(4),TREF,XP(6,6),TSTAND,GA(11,24,3),GB(11,24,3)              
     S ,PTY(11,20),ETY(11,20),TINTP(11),TSTP,O2H,PIALF0                 
C                                                                       
      REAL  TB(40,31,2),ALP(40,31,2),BET(40,31,2),                      
     S ULG(40),PLG(31),AP(40,31,2),BP(40,31,2),TBO3(38,31),TO3B(38,31)  
     S,ULGO(38),AO3(38,31),BO3(38,31),O3A(38,31),O3B(38,31)             
C     ----------------------------------------------------------------- 
C*COMDECK YOMRDU                                                        
C     ----------------------------------------------------------------- 
C                                                                       
C*    CONTROL VARIABLES FOR RADIATION: INTERNAL SWITCH AND INDICES      
C                                                                       
C ZEELOG:  SECURITY THRESHOLD FOR ABSORBER AMOUNT IN LAPLACE TRANSFORM  
C ZEPSC :  SECURITY THRESHOLD FOR CLOUD COVER                           
C ZEPSCO:  SECURITY THRESHOLD FOR OZONE AMOUNT                          
C ZEPSCQ:  SECURITY THRESHOLD FOR WATER VAPOR                           
C ZEPSCT:  SECURITY THRESHOLD FOR SHORTWAVE OPTICAL THICKNESS           
C ZEPSCW:  SECURITY THRESHOLD FOR CLOUD LIQUID WATER PATH               
C                                                                       
      REAL          CDAY,CCO2,CH2O,DIFF,X10E                            
     S             ,ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW            
C     ----------------------------------------------------------------- 
C*COMDECK YOMSW                                                         
C     ----------------------------------------------------------------- 
C                                                                       
C*    COEFFICIENTS FOR THE SHORTWAVE RADIATION SUBROUTINES              
C                                                                       
C APAD  :  PADE APPROXIMANTS NUMERATOR                                  
C BPAD  :  PADE APPROXIMANTS DENOMINATOR                                
C D     :  TRANSMISSION LIMIT FOR INFINITE ABSORBER AMOUNT              
C CRAY  :  RAYLEIGH SCATTERING COEFFICIENTS                             
C SUN   :  SOLAR FRACTION IN SPECTRAL INTERVALS                         
C                                                                       
      REAL  APAD(2,3,7),BPAD(2,3,7),D(2,3),CRAY(2,6),SUN(2)             
C     ----------------------------------------------------------------- 
                                                                        
C                                                                       
C  -------------------------------------------------------------------- 
C                                                                       
      REAL   TTT(1,NLAT,NRLEV,12)                                       
C                                                                       
C        0.1  "RADINT POINTERS"                                         
C             -----------------                                         
C                                                                       
      REAL   TM1(NLP2,NLEV),                                            
     *       TSM1M(NLP2),                                               
     *       QM1(NLP2,NLEV),                                            
     *       SNM1M(NLP2),                                               
     *       ALBM(NLP2),                                                
     *       AVVELM(NLP2)                                               
      REAL   SLMM(NLP2),                                                
     *       APHM1(NLP2,NLEVP1),                                        
     *       APM1(NLP2,NLEV)                                            
C                                                                       
      LOGICAL LOLAND(NLP2)                                              
C                                                                       
C     VARIABLES FOR FULL GRID AND TRANSFORMS.                           
C                                                                       
      REAL   ZTI(NLP2,NRLEVP1),                                         
     *       ZCTI(NLP2,NRLEVP1),                                        
     *       ZCPH(NLP2,NRLEVP1),                                        
     *       ZWV(NLP2,NRLEV),                                           
     *       ZSW(NLP2,NRLEV),                                           
     *       ZALTE(NLP2),                                               
     *       ZCALTE(NLP2),                                              
     *       ZALSO(NLP2),                                               
     *       ZCALSO(NLP2)                                               
C                                                                       
      REAL   ZOZQ(NLP2),                                                
     #       ZOZH(NLP2)                                                 
C                                                                       
      REAL   ZTRSOL(NLP2,NLEVP1),                                       
     *       ZEMTER(NLP2,NLEVP1)                                        
C                                                                       
C     FINAL RESULTS ARE STORED ON:                                      
C                                                                       
      REAL   TRSOLM(NLP2,NLEVP1),                                       
     *       EMTERM(NLP2,NLEVP1)                                        
C                                                                       
      REAL   AMU0(NLP2),                                                
     *       RDAYL(NLP2)                                                
C                                                                       
C     VARIABLES WITHIN THE SPLITTED LATITUDE LOOP.                      
C                                                                       
      REAL   CLC(NRLST,NRLEV),                                          
     *       TI(NRLST,NRLEVP1),                                         
     *       DP(NRLST,NRLEV),                                           
     *       AADS(NRLST,NRLEV),                                         
     *       AEQ1(NRLST,NRLEV),                                         
     *       AEQ2(NRLST,NRLEV),                                         
     *       AEQ3(NRLST,NRLEV),                                         
     *       AEQ4(NRLST,NRLEV),                                         
     *       AEQ5(NRLST,NRLEV),                                         
     *       SSW(NRLST,NRLEV),                                          
     *       WV(NRLST,NRLEV),                                           
     *       QOF(NRLST,NRLEV),                                          
     *       QCF(NRLST,NRLEV),                                          
     *       CLWA(NRLST,NRLEV),                                         
     *       APRE(NRLST),                                               
     *       SMU0(NRLST),                                               
     *       ALTE(NRLST),                                               
     *       ALSO(NRLST)                                                
      REAL   ZDPO(NRLST),                                               
     *       ZQCFO(NRLST),                                              
     *       ZAEQSO(NRLST),                                             
     *       ZQOFO(NRLST),                                              
     *       ZDPN(NRLST),                                               
     *       ZQCFN(NRLST),                                              
     *       ZAEQSN(NRLST),                                             
     *       ZQOFN(NRLST)                                               
      REAL   FLT(NRLST,NRLEVP1),                                        
     *       FLS(NRLST,NRLEVP1)                                         
C                                                                       
C     DIAGNOSTIC VARIABLES.                                             
C                                                                       
      REAL   DIAG(NLEVP1,4),                                            
     *       DIA1(7),                                                   
     *       ZDIAG(NLEVP1,10),                                          
     *       ZDIA1(21),                                                 
     *       ZDIA(NLON),                                                
     *       ZDIAF(NLON,7),                                             
     *       ZDIAT(NLEVP1)                                              
C                                                                       
C                                                                       
C                                                                       
      REAL PGG(IRLST),                                                  
     #     PREF(IRLST),                                                 
     #     PRE1(IRLST),                                                 
     #     PRE2(IRLST),                                                 
     #     PRMUZ(IRLST),                                                
     #     PTO1(IRLST),                                                 
     #     PTR1(IRLST),                                                 
     #     PTR2(IRLST),                                                 
     #     PW(IRLST)                                                    
C                                                                       
C                                                                       
C  what about the 1-dimensional PALBS???                                
C                                                                       
      REAL     PALBSN  (IRLST)                                          
C                                                                       
C*       0.2   "LOCAL" ARRAYS                                           
C              --------------                                           
      REAL ZABSCU(IRLST ,NUA, 3*NRLEV+1),                               
     S     ZABSLY(IRLST ,NUA, 3*NRLEV+1) ,                              
     S     ZADJD (IRLST , IRLVP1)      ,                                
     S     ZADJU (IRLST , IRLVP1)       ,                               
     S     ZBGND (IRLST)                ,                               
     S     ZBINT (IRLST , IRLVP1)       ,                               
     S     ZBSUR (IRLST , NINT)         ,                               
     S     ZBSUIN(IRLST)                ,                               
     S     ZBTOP (IRLST , NINT)         ,                               
     S     PCLDLW(IRLST , IRLEV)        ,                               
     S     ZCLOUD(IRLST)                ,                               
     S     ZCNTRB(IRLST , IRLVP1,IRLVP1),                               
     S     ZCUMP (IRLST)                ,                               
     S     ZCUMPT(IRLST)                ,                               
     S     ZDBDT (IRLST , NINT , IRLEV) ,                               
     S     ZDBSL (IRLST , NINT , 2*NRLEV),                              
     S     ZDISTD(IRLST , IRLVP1)       ,                               
     S     ZDISTU(IRLST , IRLVP1)                                       
      REAL                                                              
     S     ZDNF  (IRLST , IRLVP1,IRLVP1),                               
     S     PDT0  (IRLST)                                                
      REAL                                                              
     S     ZDUC  (IRLST , 3*NRLEV+1)       ,                            
     S     ZEFFT (IRLST)                ,                               
     S     ZEMD  (IRLST , IRLEV), ZEMU  (IRLST , IRLEV),                
     S     ZEPSC2(IRLST),  ZEPSC3(IRLST),                               
     S     ZEPSH1(IRLST),  ZEPSH2(IRLST),                               
     S     ZEPSH3(IRLST),  ZEPSH4(IRLST),                               
     S     ZEPSH5(IRLST),  ZEPSH6(IRLST),                               
     S     ZEPHIO(IRLST),  ZFACTC(IRLST),                               
     S     ZZFD(IRLST), ZFDD(IRLST), ZFDN(IRLST , IRLVP1)               
      REAL                                                              
     S     PFLUX (IRLST , 2 , IRLVP1)   ,                               
     S     PFLUC (IRLST , 2 , IRLVP1)   ,                               
     S     ZFLWP (IRLST)                ,                               
     S     PCTS  (IRLST , IRLEV)        ,                               
     S     ZZFU   (IRLST)               ,                               
     S     ZFUP  (IRLST , IRLVP1)       ,                               
     S     ZGLAYD(IRLST)                ,                               
     S     ZGLAYU(IRLST)                ,                               
     S     ZOMEGA(IRLST , 2 , IRLEV)                                    
C                                                                       
      INTEGER                                                           
     S     ITX   (IRLST)                ,                               
     S     IXDIA (IRLST , IRLVP1)       ,                               
     S     IXT   (IRLST , IRLVP1,IRLVP1),                               
     S     IXTSU (IRLST , IRLVP1)       ,                               
     S     IXTTP (IRLST , IRLVP1)       ,                               
     S     IMX   (IRLST)                ,                               
     S     IMXP  (IRLST)                                                
C                                                                       
      REAL ZOZ   (IRLST , IRLEV)        ,                               
     S     PPMB  (IRLST , IRLVP1)       ,                               
     S     ZRES  (IRLST)                ,                               
     S     PB    (IRLST, 6 , IRLVP1)    ,                               
     S     ZSSIG (IRLST , 3*NRLEV+1)    ,                               
     S     PTAU  (IRLST , 2 , IRLEV)    ,                               
     S     PTAVE (IRLST , IRLEV)        ,                               
     S     ZTAVIC(IRLST)                ,                               
     S     ZTI1  (IRLST)                ,                               
     S     PTL   (IRLST , IRLVP1)       ,                               
     S     ZTT   (IRLST , NTRA)         ,                               
     S     ZTT1  (IRLST , NTRA)         ,                               
     S     ZTT2  (IRLST , NTRA)         ,                               
     S     ZUAER (IRLST , NINT)         ,                               
     S     ZUPF  (IRLST , IRLVP1,IRLVP1),                               
     S     ZUU   (IRLST , NUA)          ,                               
     S     ZXD   (IRLST)                                                
      REAL                                                              
     S     ZXN   (IRLST)                ,                               
     S     ZXOZ  (IRLST)                ,                               
     S     ZXWV  (IRLST)                ,                               
     S     ZZZ   (IRLST)                ,                               
     S     ZRES2 (IRLST)                ,                               
     S     ZTI2  (IRLST)                ,                               
     S     ZBLAY (IRLST,IRLEV)          ,                               
     S     ZBLEV (IRLST,IRLVP1)         ,                               
     S     ZCTS  (IRLST)                ,                               
     S     ZHFG  (IRLST)                                                
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C                                                                       
      INTEGER KT    (IRLST)                                             
      REAL    PUU   (IRLST , NUA  )     ,                               
     S        PTT   (IRLST , NTRA )     ,                               
     S        ZZ    (IRLST)                                             
C                                                                       
C     -------------------------------------------------------- ---------
C                                                                       
      REAL  PCLFR  (IRLST , IRLEV)       ,                              
     S      PT     (IRLST , IRLVP1)       ,                             
     S      PQOF   (IRLST , IRLEV)          ,                           
     S      PQLWP  (IRLST , IRLEV)          ,                           
     S      PEMIS  (IRLST)                                              
      REAL                                                              
     S      PFLT   (IRLST , IRLVP1)                                     
C                                                                       
      REAL  ZALBSU(IRLST , 2 )           ,                              
     S      ZCG   (IRLST , 2 , IRLEV)    ,                              
     S      ZCLDLW(IRLST , IRLEV)        ,                              
     S      ZCLDSW(IRLST , IRLEV)        ,                              
     S      ZCOOLR(IRLST , IRLEV)        ,                              
     S      ZDT0  (IRLST)                ,                              
     S      ZFLUX (IRLST , 2 , IRLVP1)   ,                              
     S      ZPMB  (IRLST , IRLVP1)       ,                              
     S      ZTAU  (IRLST , 2 , IRLEV)    ,                              
     S      ZTAVE (IRLST , IRLEV)        ,                              
     S      ZTL   (IRLST , IRLVP1)       ,                              
     S      ZFSDWN(IRLST , IRLVP1)       ,                              
     S      ZFSUP (IRLST , IRLVP1)       ,                              
     S      ZHEATR(IRLST , IRLEV )                                      
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C                                                                       
      REAL   PDP   (IRLST , IRLEV)     ,                                
     S       PWV   (IRLST , IRLEV)     ,                                
     S       PFLS  (IRLST , IRLVP1)    ,                                
     S       PPSOL (IRLST        )     ,                                
     S       PRMU0 (IRLST        )     ,                                
     S       PAER  (IRLST,IRLEV,5)     ,                                
     S       PALBS (IRLST , 2 )        ,                                
     S       PCG   (IRLST , 2 , IRLEV) ,                                
     S       PCLDSW(IRLST , IRLEV)     ,                                
     S       POMEGA(IRLST , 2 , IRLEV) ,                                
     S       POZ   (IRLST , IRLEV)                                      
C                                                                       
      REAL   PFDOWN(IRLST , IRLVP1)    ,                                
     S       PFUP  (IRLST , IRLVP1)                                     
C                                                                       
      REAL   ZAKI  (IRLST , 2 )        ,                                
     S       ZCGAZ (IRLST , IRLEV)     ,                                
     S       ZCNEB (IRLST , IRLEV)     ,                                
     S       ZC1I  (IRLST , IRLVP1)    ,                                
     S       ZDSC  (IRLST , IRLEV)     ,                                
     S       ZDSH  (IRLST , IRLEV)     ,                                
     S       ZDSIG (IRLST , IRLEV)     ,                                
     S       ZFACT (IRLST)             ,                                
     S       ZFACT1 (IRLST), ZFACT2 (IRLST),                            
     S       ZFACT3 (IRLST), ZFACT4 (IRLST),                            
     S       ZFACT5 (IRLST), ZFACT6 (IRLST),                            
     S       ZFD   (IRLST , IRLVP1)    ,                                
     S       ZFRAC (IRLST)             ,                                
     S       ZFU   (IRLST , IRLVP1)    ,                                
     S       ZG    (IRLST)             ,                                
     S       ZGG   (IRLST)             ,                                
     S       ZPIZAZ(IRLST , IRLEV)     ,                                
     S       ZRAYL (IRLST),  ZRAY1 (IRLST , IRLVP1)                     
      REAL                                                              
     S       ZRAY2 (IRLST , IRLVP1)    ,                                
     S       ZREF  (IRLST)             ,                                
     S       ZREFZ (IRLST , 2 , IRLVP1),                                
     S       ZRE1  (IRLST)             ,                                
     S       ZRE2  (IRLST)             ,                                
     S       ZRJ   (IRLST , 6 , IRLVP1),                                
     S       ZRK   (IRLST , 6 , IRLVP1),                                
     S       ZRL   (IRLST , 8)         ,                                
     S       ZRMU  (IRLST)             ,                                
     S       ZRMUE (IRLST , IRLVP1)    ,                                
     S       ZRMUZ (IRLST)                                              
      REAL                                                              
     S       ZRNEB (IRLST)             ,                                
     S       ZRUEF (IRLST , 8)         ,                                
     S       ZR1   (IRLST)             ,                                
     S       ZR2   (IRLST)             ,                                
     S       ZR21  (IRLST)             ,                                
     S       ZR22  (IRLST)             ,                                
     S       ZR23  (IRLST)             ,                                
     S       ZS    (IRLST)             ,                                
     S       ZSEC  (IRLST)             ,                                
     S       ZSS1  (IRLST)             ,                                
     S       ZTAUAZ(IRLST , IRLEV)     ,                                
     S       ZTOT  (IRLST , IRLVP1)    ,                                
     S       ZTO1  (IRLST)             ,                                
     S       ZTR   (IRLST , 2 , IRLVP1),                                
     S       ZTRA1 (IRLST , IRLVP1)    ,                                
     S       ZTRA2 (IRLST , IRLVP1)                                     
      REAL                                                              
     S       ZTR1  (IRLST)             ,                                
     S       ZTR2  (IRLST)             ,                                
     S       ZUD   (IRLST , 3 , IRLVP1),                                
     S       ZUM   (IRLST , IRLVP1)    ,                                
     S       ZU1D  (IRLST)             ,                                
     S       ZU2D  (IRLST)             ,                                
     S       ZW    (IRLST)             ,                                
     S       ZN175 (IRLST)             ,                                
     S       ZN190 (IRLST)             ,                                
     S       ZO175 (IRLST)             ,                                
     S       ZO190 (IRLST)             ,                                
     S       ZP75  (IRLST)             ,                                
     S       ZP90  (IRLST)             ,                                
     S       ZSIGN (IRLST)             ,                                
     S       ZSIGO (IRLST)                                              
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
      REAL   PTR    (IRLST)     ,                                       
     S       PU     (IRLST)                                             
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
C   CLOUD-POINTERS                                                      
C   --------------                                                      
C                                                                       
      REAL   ZRH(NLP2,NLEV),                                            
     *       ZCH(NLP2),                                                 
     *       ZCM(NLP2),                                                 
     *       ZCL(NLP2),                                                 
     *       ZCT(NLP2),                                                 
     *       ZCC(NLP2)                                                  
      INTEGER NTOPCM(NLP2),                                             
     *        NBASECM(NLP2)                                             
      REAL   ARPRCM(NLP2)                                               
      INTEGER ITOPH(NLP2),                                              
     *        ITOPM(NLP2),                                              
     *        ITOPL(NLP2)                                               
      LOGICAL LOC(NLP2)                                                 
      REAL   ZQS(NLP2,NLEV),                                            
     *       ZCLC(NLP2,NLEV),                                           
     *       ZQLWP(NLP2,NLEV)                                           
C                                                                       
      REAL   VERVEL(NLP2,NLEV),                                         
     *       ZTHETA(NLP2,NLEV),                                         
     *       ZETA(NLP2,NLEV),                                           
     *       ZDTHMIN(NLP2)                                              
      INTEGER INVB(NLP2)                                                
      REAL   ZCTLWP(NLP2),                                              
     *       ZQLOC(NLP2),                                               
     *       ZQMEC(NLP2),                                               
     *       ZQHIC(NLP2)                                                
C                                                                       
C  definitions of "radheat"                                             
C                                                                       
      REAL   ZFSOT(NLON),                                               
     #       ZFTET(NLON),                                               
     #       ZFLT (NLON),                                               
     #       ZFSO (NLON),                                               
     #       ZFTE (NLON),                                               
     #       ZFLB (NLON),                                               
     #       ZDTDT(NLON),                                               
     #       HEATO(NLON,NLEV),                                          
     #       HEATE(NLON,NLEV),                                          
     #       HEATR(NLON,NLEV),                                          
     #       ZI0  (NLON)                                                
C                                                                       
C PIERS 13-8-97                                                         
C                                                                       
         REAL PIERSLWP                                                  
C                                                                       
C Piece of code needed to replace the Nikos radiation scheme with       
C the Morcrette radiation scheme in the igcm3.1.                        
C See morcrette3_1_1.readme for details.                                
C                                                                       
C                                                                       
c      program lh                                                       
c                                                                       
c      parameter(NL=23)                                                 
c      real P(NL),t(NL),h2o(NL),o3(NL),htlw(NL),htsw(NL)                
c      real swalb,doy                                                   
c       REAL CF(4,2)     !cloud fractions,lwps                          
c       INTEGER IC(4,2)   ! cloud positions                             
c       REAL RFLUXES(2,2,2)                                             
c      open(11,file='mls23.dat')                                        
c      read(11,*) NLEV                                                  
c      DOY=80                                                           
c      swalb=0.07                                                       
c      alat=40.0                                                        
c       doy=float(i)                                                    
c      do i=nlev,1,-1                                                   
c         read(11,*) P(i),t(i),h2o(i),adum,o3(i)                        
c         print *,i,p(i),t(i),h2o(i),o3(i)                              
c      enddo                                                            
c                                                                       
c       ic(1,1)=3                                                       
c      ic(1,2)=17                                                       
c       ic(3,1)=6                                                       
c       ic(2,2)=6                                                       
c       cf(1,1)=0.3                                                     
c       cf(1,2)=45.0                                                    
c        cf(3,1)=0.8                                                    
c       cf(4,2)=20.0                                                    
c      alon=0.0                                                         
c      call morcigcm(p,t,h2o,o3,alat,htlw,htsw,doy,cf,ic,               
c     $     rfluxes,swalb,alon)                                         
c      enddo                                                            
c      do i=nl,1,-1                                                     
c         print '(i6,4F14.4)',i,p(i),t(i),htsw(i),htlw(i)               
c      enddo                                                            
c      print *,RFLUXES(1,1,1), RFLUXES(1,1,2), RFLUXES(1,2,1),          
c     $     RFLUXES(1,2,2)                                              
c      print *,RFLUXES(2,1,1), RFLUXES(2,1,2), RFLUXES(2,2,1),          
c     $     RFLUXES(2,2,2)                                              
c                                                                       
c                                                                       
c      end                                                              
c      INCLUDE 'zparb.upd'                                              
C                                                                       
C-----------------------------------------------------------------      
C                                                                       
      DO 99 JL=1,NRLST                                                  
         PTR(JL) = ZR1(JL)                                              
         PU (JL) = ZW (JL)                                              
 99   CONTINUE                                                          
C                                                                       
C-----------------------------------------------------------------      
C                                                                       
C*         1.      HORNER'S ALGORITHM TO COMPUTE TRANSMISSION FUNCTION  
C                                                                       
 100  CONTINUE                                                          
C                                                                       
      DO 101 JL = 1 , KDLON                                             
         PTR(JL) = APAD(KNU,KA,1) + PU(JL) * (APAD(KNU,KA,2) + PU(JL)   
     S         * ( APAD(KNU,KA,3) + PU(JL) * (APAD(KNU,KA,4) + PU(JL)   
     S         * ( APAD(KNU,KA,5) + PU(JL) * (APAD(KNU,KA,6) + PU(JL)   
     S         * ( APAD(KNU,KA,7) ))))))                                
C                                                                       
         ZR2(JL) = BPAD(KNU,KA,1) + PU(JL) * (BPAD(KNU,KA,2) + PU(JL)   
     S         * ( BPAD(KNU,KA,3) + PU(JL) * (BPAD(KNU,KA,4) + PU(JL)   
     S         * ( BPAD(KNU,KA,5) + PU(JL) * (BPAD(KNU,KA,6) + PU(JL)   
     S         * ( BPAD(KNU,KA,7) ))))))                                
 101  CONTINUE                                                          
C                                                                       
C*         2.      ADD THE BACKGROUND TRANSMISSION                      
C                                                                       
 200  CONTINUE                                                          
C                                                                       
      DO 201 JL = 1 , KDLON                                             
         PTR(JL) = (PTR(JL) / ZR2(JL)) * (1. - D(KNU,KA)) + D(KNU,KA)   
 201  CONTINUE                                                          
C                                                                       
C                                                                       
      DO 301 JL=1,NRLST                                                 
         ZR1(JL) = PTR(JL)                                              
         ZW (JL) = PU (JL)                                              
 301  CONTINUE                                                          
C                                                                       
      RETURN                                                            
      END                                                               
C                                                                       
      SUBROUTINE SUAER                                                  
C                                                                       
C**** *SUAER*   - INITIALIZE COMMON YOMAER                              
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C           INITIALIZE YOMAER, THE COMMON THAT CONTAINS THE             
C           RADIATIVE CHARACTERISTICS OF THE AEROSOLS                   
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C        *CALL* *SUAER*                                                 
C                                                                       
C        EXPLICIT ARGUMENTS                                             
C        --------------------                                           
C        NONE                                                           
C                                                                       
C        IMPLICIT ARGUMENTS                                             
C        --------------------                                           
C        COMMON YOMAER                                                  
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C        SEE DOCUMENTATION                                              
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C        ECMWF RESEARCH DEPARTMENT DOCUMENTATION                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C        JEAN-JACQUES MORCRETTE *ECMWF*                                 
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C        ORIGINAL : 88-02-15                                            
C     ---------------------------------------------------------------   
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRSW* SWITCHES RELATED TO THE RADIATION SCHEME.         
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      IMPLICIT REAL (A-H,O-Z)                                           
C      IMPLICIT UNDEFINED (Z)                                           
C      IMPLICIT LOGICAL (L)                                             
C      IMPLICIT UNDEFINED (L)                                           
      IMPLICIT INTEGER (I-K,M-N)                                        
C      IMPLICIT UNDEFINED (I-K,M-N)                                     
C      IMPLICIT REAL (P)                                                
C      IMPLICIT UNDEFINED (A-H,O,Q-Y)                                   
C                                                                       
C everything except the parameters is defined                           
C in the common blocks                                                  
C                                                                       
      COMMON/COM1/                                                      
C     I NRADFR, NRPART, NRINT, NRADPFR, NRADPLA,                        
C     I NRLSTI,                                                         
     R XLAT,                                                            
     R RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR, ZCCO2,              
C     R PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3,                 
     L LOH, LOM, LOL, LORAD                                             
      COMMON/COM2/                                                      
     I ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,                  
     I         JLOOP, KRLEV,                                            
     I        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     I        ITH, ITM, ITL,                                            
     I        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     I        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     I        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     I        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      COMMON/COM3/                                                      
     I IND4, KM12, KXD, KXU, IG, KBS, KDD,                              
     I        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     I        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     I        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     I        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     I        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
      COMMON/COM4/                                                      
     R           COZQC,COZQS,COZHC,COZHS,                               
     R CSUPSAT,                                                         
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
      COMMON/COM5/                                                      
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NUMVC,                                                           
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      COMMON/COM6/                                                      
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD,       
     R DTIME,    TWODT,    EPS,                                         
     R SINLON,     COSLON,                                              
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2,                          
     I NFAX,                                                            
     R CRAE,                                                            
     R CDISSEM                                                          
      COMMON/COM7/                                                      
     R     CCRH,                                                        
     L LO,LO1,LO2,LODIA,LODIAP,                                         
     R TAUA, PIZA, CGA, CAER, CAEROS                                    
      COMMON/COM7A/                                                     
     R  RT1,WG1,AT,BT,CNTNU,O1H,O2H,PIALF0                              
     S ,OCT,TREF,XP,TSTAND,GA,GB,CE,ETY,PTY                             
      COMMON/COM7B/                                                     
     S TINTP,TSTP,                                                      
     S  TBO3,AO3,BO3,TB,ALP,BET,ULG,ULGO,PLG,AP,BP,TO3B,O3A,O3B,        
     R          CDAY,CCO2,CH2O,DIFF,X10E                                
       COMMON/COM7C/                                                    
     S  ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW,                       
     R  APAD,BPAD,D,CRAY,SUN,                                           
     R   TTT,                                                           
     R   TM1                                                            
      COMMON/COM8/                                                      
     R       TSM1M,                                                     
     R       QM1,                                                       
     R       SNM1M,                                                     
     R       ALBM,                                                      
     R       AVVELM,                                                    
     R   SLMM,                                                          
     R       APHM1,                                                     
     R       APM1                                                       
      COMMON/COM9/                                                      
     L LOLAND,                                                          
     R   ZTI,                                                           
     *       ZCTI,                                                      
     *       ZCPH,                                                      
     *       ZWV,                                                       
     *       ZSW,                                                       
     *       ZALTE,                                                     
     *       ZCALTE,                                                    
     *       ZALSO                                                      
      COMMON/COM10/                                                     
     *       ZCALSO,                                                    
     R   ZOZQ,                                                          
     *       ZOZH,                                                      
     R   ZTRSOL,                                                        
     *       ZEMTER,                                                    
     R   TRSOLM,                                                        
     *       EMTERM,                                                    
     R   AMU0,                                                          
     *       RDAYL                                                      
      COMMON/COM11/                                                     
     R   CLC,                                                           
     *       TI,                                                        
     *       DP,                                                        
     *       AADS,                                                      
     *       AEQ1,                                                      
     *       AEQ2,                                                      
     *       AEQ3,                                                      
     *       AEQ4,                                                      
     *       AEQ5,                                                      
     *       SSW                                                        
      COMMON/COM12/                                                     
     *       WV,                                                        
     *       QOF,                                                       
     *       QCF,                                                       
     *       CLWA,                                                      
     *       APRE,                                                      
     *       SMU0,                                                      
     *       ALTE,                                                      
     *       ALSO                                                       
      COMMON/COM13/                                                     
     R   ZDPO,                                                          
     *       ZQCFO,                                                     
     *       ZAEQSO,                                                    
     *       ZQOFO,                                                     
     *       ZDPN,                                                      
     *       ZQCFN,                                                     
     *       ZAEQSN,                                                    
     *       ZQOFN                                                      
      COMMON/COM14/                                                     
     R   FLT,                                                           
     *       FLS,                                                       
     R   DIAG,                                                          
     *       DIA1,                                                      
     *       ZDIAG,                                                     
     *       ZDIA1,                                                     
     *       ZDIA,                                                      
     *       ZDIAF,                                                     
     *       ZDIAT,                                                     
     R PGG                                                              
      COMMON/COM15/                                                     
     #     PREF,                                                        
     #     PRE1,                                                        
     #     PRE2,                                                        
     #     PRMUZ,                                                       
     #     PTO1,                                                        
     #     PTR1,                                                        
     #     PTR2,                                                        
     #     PW,                                                          
     R     PALBSN                                                       
      COMMON/COM16/                                                     
     R ZABSCU,                                                          
     S     ZABSLY,                                                      
     S     ZADJD ,                                                      
     S     ZADJU ,                                                      
     S     ZBGND ,                                                      
     S     ZBINT ,                                                      
     S     ZBSUR ,                                                      
     S     ZBSUIN,                                                      
     S     ZBTOP ,                                                      
     S     PCLDLW,                                                      
     S     ZCLOUD,                                                      
     S     ZCNTRB,                                                      
     S     ZCUMP ,                                                      
     S     ZCUMPT,                                                      
     S     ZDBDT                                                        
      COMMON/COM17/                                                     
     S     ZDBSL ,                                                      
     S     ZDISTD,                                                      
     S     ZDISTU,                                                      
     S     ZDNF  ,                                                      
     S     PDT0  ,                                                      
     S     ZDUC                                                         
      COMMON/COM18/                                                     
     S     ZEFFT ,                                                      
     S     ZEMU  ,                                                      
     S     ZEPHIO,                                                      
     S     ZEPSC2,                                                      
     S     ZEPSC3,                                                      
     S     ZEPSH1,                                                      
     S     ZEPSH2,                                                      
     S     ZEPSH3                                                       
      COMMON/COM19/                                                     
     S     ZEPSH4,                                                      
     S     ZEPSH5,                                                      
     S     ZEPSH6,                                                      
     S     ZEPSIO,                                                      
     S     ZFACTC,                                                      
     S     ZZFD  ,                                                      
     S     ZFDD  ,                                                      
     S     ZFDN                                                         
      COMMON/COM20/                                                     
     S     PFLUX , PFLUC,                                               
     S     ZFLWP , PCTS                                                 
     S     ZZFU  ,                                                      
     S     ZFUP  ,                                                      
     S     ZGLAYD,                                                      
     S     ZGLAYU,                                                      
     S     ZOMEGA,                                                      
     S     ITX   ,                                                      
     S     IXDIA ,                                                      
     S     IXT   ,                                                      
     S     IXTSU ,                                                      
     S     IXTTP ,                                                      
     S     IMX   ,                                                      
     S     IMXP                                                         
      COMMON/COM21/                                                     
     R ZOZ   ,                                                          
     S     PPMB  ,                                                      
     S     ZRES  , PB                                                   
     S     ZSSIG ,                                                      
     S     PTAU  ,                                                      
     S     PTAVE ,                                                      
     S     ZTAVIC,                                                      
     S     ZTI1  ,                                                      
     S     PTL                                                          
      COMMON/COM22/                                                     
     S     ZTT   ,                                                      
     S     ZTT1  ,                                                      
     S     ZTT2  ,                                                      
     S     ZUAER ,                                                      
     S     ZUPF  ,                                                      
     S     ZUU   ,                                                      
     S     ZXD                                                          
      COMMON/COM23/                                                     
     S     ZXN   ,                                                      
     S     ZXOZ  ,                                                      
     S     ZXWV  ,                                                      
     S     ZZZ   ,                                                      
     S     ZRES2 ,                                                      
     S     ZTI2  ,                                                      
     S     ZBLAY ,                                                      
     S     ZBLEV ,                                                      
     S     ZCTS  ,                                                      
     S     ZHFG                                                         
      COMMON/COM24/                                                     
     I KT,                                                              
     R    PUU   ,                                                       
     S        PTT   ,                                                   
     S        ZZ                                                        
     R  PCLFR  ,                                                        
     S      PT     ,                                                    
     S      PQOF   ,                                                    
     S      PQLWP  ,                                                    
     S      PEMIS  ,                                                    
     S      PFLT                                                        
      COMMON/COM25/                                                     
     R  ZALBSU,                                                         
     S      ZCG   ,                                                     
     S      ZCLDLW,                                                     
     S      ZCLDSW,                                                     
     S      ZCOOLR,                                                     
     S      ZDT0  ,                                                     
     S      ZEMD  ,                                                     
     S      ZFLUX ,                                                     
     S      ZPMB  ,                                                     
     S      ZTAU  ,                                                     
     S      ZTAVE ,                                                     
     S      ZTL   ,                                                     
     S      ZFSDWN                                                      
      COMMON/COM26/                                                     
     S      ZFSUP ,                                                     
     S      ZHEATR,                                                     
     R   PDP   ,                                                        
     S       PWV   ,                                                    
     S       PFLS  ,                                                    
     S       PPSOL ,                                                    
     S       PRMU0 ,                                                    
     S       PAER  ,                                                    
     S       PALBS ,                                                    
     S       PCG   ,                                                    
     S       PCLDSW,                                                    
     S       POMEGA,                                                    
     S       POZ                                                        
      COMMON/COM27/                                                     
     R   PFDOWN,                                                        
     S       PFUP  ,                                                    
     R   ZAKI  ,                                                        
     S       ZCGAZ ,                                                    
     S       ZCNEB ,                                                    
     S       ZC1I  ,                                                    
     S       ZDSC  ,                                                    
     S       ZDSH                                                       
      COMMON/COM28/                                                     
     S       ZDSIG ,                                                    
     S       ZFACT ,                                                    
     S       ZFACT1 ,                                                   
     S       ZFACT2 ,                                                   
     S       ZFACT3 ,                                                   
     S       ZFACT4 ,                                                   
     S       ZFACT5 ,                                                   
     S       ZFACT6 ,                                                   
     S       ZFD   ,                                                    
     S       ZFRAC ,                                                    
     S       ZFU   ,                                                    
     S       ZG    ,                                                    
     S       ZGG   ,                                                    
     S       ZPIZAZ                                                     
      COMMON/COM29/                                                     
     S       ZRAYL ,                                                    
     S       ZRAY1 ,                                                    
     S       ZRAY2 ,                                                    
     S       ZREF  ,                                                    
     S       ZREFZ ,                                                    
     S       ZRE1  ,                                                    
     S       ZRE2  ,                                                    
     S       ZRJ   ,                                                    
     S       ZRK   ,                                                    
     S       ZRL   ,                                                    
     S       ZRMU  ,                                                    
     S       ZRMUE ,                                                    
     S       ZRMUZ                                                      
      COMMON/COM30/                                                     
     S       ZRNEB ,                                                    
     S       ZRUEF ,                                                    
     S       ZR1   ,                                                    
     S       ZR2   ,                                                    
     S       ZR21  ,                                                    
     S       ZR22  ,                                                    
     S       ZR23  ,                                                    
     S       ZS    ,                                                    
     S       ZSEC  ,                                                    
     S       ZSS1  ,                                                    
     S       ZTAUAZ                                                     
      COMMON/COM31/                                                     
     S       ZTOT  ,                                                    
     S       ZTO1  ,                                                    
     S       ZTR   ,                                                    
     S       ZTRA1 ,                                                    
     S       ZTRA2 ,                                                    
     S       ZTR1                                                       
      COMMON/COM32/                                                     
     S       ZTR2  ,                                                    
     S       ZUD   ,                                                    
     S       ZUM   ,                                                    
     S       ZU1D  ,                                                    
     S       ZU2D  ,                                                    
     S       ZW    ,                                                    
     S       ZN175 ,                                                    
     S       ZN190 ,                                                    
     S       ZO175 ,                                                    
     S       ZO190 ,                                                    
     S       ZP75  ,                                                    
     S       ZP90  ,                                                    
     S       ZSIGN ,                                                    
     S       ZSIGO                                                      
      COMMON/COM33/                                                     
     R   PTR,                                                           
     S       PU,                                                        
     R   ZRH,                                                           
     *       ZCH,                                                       
     *       ZCM,                                                       
     *       ZCL,                                                       
     *       ZCT,                                                       
     *       ZCC,                                                       
     I NTOPCM                                                           
      COMMON/COM34/                                                     
     *        NBASECM,                                                  
     R   ARPRCM,                                                        
     I ITOPH,                                                           
     *        ITOPM,                                                    
     *        ITOPL,                                                    
     L LOC,                                                             
     R   ZQS,                                                           
     *       ZCLC,                                                      
     *       ZQLWP                                                      
      COMMON/COM35/                                                     
     R   VERVEL,                                                        
     *       ZTHETA,                                                    
     *       ZETA,                                                      
     *       ZDTHMIN,                                                   
     I INVB,                                                            
     R   ZCTLWP,                                                        
     *       ZQLOC,                                                     
     *       ZQMEC,                                                     
     *       ZQHIC,                                                     
     R   ZFSOT,                                                         
     #       ZFTET,                                                     
     #       ZFLT                                                       
      COMMON/COM36/                                                     
     #       ZFSO ,                                                     
     #       ZFTE ,                                                     
     #       ZFLB ,                                                     
     #       ZDTDT,                                                     
     #       HEATO,                                                     
     #       HEATE,                                                     
     #       HEATR,                                                     
     #       ZI0                                                        
C                                                                       
      INTEGER                                                           
C     I NRADFR,   NRPART,   NRINT,    NRLSTI,  NRADPFR,  NRADPLA,       
     I NRLST,    NRLEV,    NRLEVP1, NLEV, NLP2, NLEVP1,                 
     I NLON, IRLEV, NTRA, NLAT, ITIM, NUA, NINT, NRPARTI                
C                                                                       
C   NRLEV .... number of vertical levels                                
C   NRLST .... number of points on a latitude line                      
C   NUA   .... number of absorber amount                                
C   NINT  .... number of spectral intervals                             
C   NTRA  .... number of transmission functions                         
C   NLAT  .... number of latitude points                                
C   ITIM  .... number of month under consideration                      
C                                                                       
C      PARAMETER (NRLEV=122,NUA=34,NINT=6,NTRA=34, NLAT=1,              
      PARAMETER (NRLEV=21,NUA=34,NINT=6,NTRA=34, NLAT=1,                
cccm      PARAMETER (NRLEV=37,NUA=18,NINT=6,NTRA=19, NLAT=1,            
cccm     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=10,NLEV=NRLEV,        
     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=7,NLEV=NRLEV,             
     #           NLP2=NRLST, NLEVP1=NRLEVP1, NLON=NRLST,                
     #           NRPARTI=1)                                             
C                                                                       
      REAL YCLOCK, XLAT(NLAT), Z0, DZ, ALBMPAR, SNPAR,                  
     #     TSUPAR, CTFREEZ, DELTAC, DELTAO, DELTAH                      
C                                                                       
C   YTIME  .... time of the year in radians                             
C   YCLOCK .... time of the day in radians                              
C   XLAT   .... latitudes in radians                                    
C   Z0     .... zero point of height grid                               
C   DZ     .... Interval between height levels                          
C   ALBMPAR.... sw surface albedo as function of longitude:             
C           yearly averaged sea-ice: 0.55                               
C                           water  : 0.07                               
C                           land   : 0.07.lt.ALBM.lt.0.80               
C                                    depending on snow                  
C   SNPAR  .... snow depth in meter                                     
C   TSUPAR .... surface temperature (Kelvin)                            
C   CTFREEZ.... freezing temperature of sea-water (Kelvin)              
C   DELTAC .... Doppler parameter for line shapes of CO2                
C   DELTAO .... Doppler parameter for line shapes of O3                 
C                                                                       
      PARAMETER (Z0=0., CTFREEZ=271.26,                                 
C     PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
     #           DZ=0.289, ALBMPAR=0.2, SNPAR=0.,                       
cccm  PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
cccm     #           DZ=0.15, ALBMPAR=0.3, SNPAR=0.,                    
CZ     #           DELTAC=0., DELTAO=0.)                                
     #           DELTAC=6.0E-04, DELTAO=1.E-02, DELTAH=5.0E-04)         
C      PARAMETER (YCLOCK=0., Z0=0., TSUPAR=273., CTFREEZ=271.26,        
C     #           DZ=0.46875, ALBMPAR=0.07, SNPAR=0.)                   
C      PARAMETER (YCLOCK=0., Z0=2.608, TSUPAR=273., CTFREEZ=271.26,     
C     #           DZ=0.2878, ALBMPAR=0.07, SNPAR=0.)                    
C                                                                       
      LOGICAL LDIUR, LOLPAR                                             
C                                                                       
C   LDIUR .... true: sun at time, false: diurnally averaged             
C   LOLPAR.... true: land surface, false: else                          
C                                                                       
C      PARAMETER (LDIUR=.TRUE., LOLPAR=.TRUE.)                          
       PARAMETER (LOLPAR=.TRUE.)                                        
C                                                                       
      INTEGER  C2TEMP                                                   
      CHARACTER*64  MIDTMP                                              
cccm      PARAMETER (C2TEMP=10,                                         
cccm     #    MIDTMP='/home/atm/ugamp/midrad/radfildat.dir/MIDTMP.DAT') 
ccc1      PARAMETER (C2TEMP=10, MIDTMP='tprfil2.dat')                   
C                                                                       
      INTEGER  IRLONI, IRLST, IRLVP1, JPNLEV, JPNLVP1                   
C                                                                       
      PARAMETER (IRLONI=NRLST, IRLST=NRLST,                             
     #           IRLVP1=NRLEVP1, JPNLEV=NRLEV,                          
     #           JPNLVP1=NRLEVP1, IRLEV=NRLEV)                          
C                                                                       
      INTEGER NIPD, NIPD2, NTR, NG1, NG1P1                              
      PARAMETER(NIPD=8,NIPD2=16,NTR=11,NG1=2,NG1P1=3)                   
C                                                                       
C    definition of the functions                                        
C                                                                       
      REAL       CVMGT, SIGMA2                                          
      INTEGER    CVMGI                                                  
C                                                                       
C    definitons of the subroutine cloud                                 
C                                                                       
      REAL RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR                  
C                                                                       
C    missing definitions in morcrette                                   
C                                                                       
C      REAL   PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3            
      REAL ZCCO2                                                        
                                                                        
      LOGICAL LOH, LOM, LOL, LORAD                                      
C                                                                       
      INTEGER ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,           
     #        IZQS, IZCLC, IZQLWP, JLOOP, KRLEV,                        
     #        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     #        ITH, ITM, ITL,                                            
     #        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     #        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     #        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     #        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      INTEGER IND4, KM12, KXD, KXU, IG, KBS, KDD,                       
     #        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     #        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     #        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     #        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     #        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD1* SPECTRAL DISTRIBUTION OF AEROSOLS AND OZONE.     
C                     (TRIANGULAR *T10* TRUNCATIONFOR AEROSOLS).        
C                     (TRIANGULAR *T5* TRUNCATIONFOR OZONE).            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R           COZQC(21),COZQS(15),COZHC(21),COZHS(15)                
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD2* CONSTANTS INTERNAL TO THE RADIATION SUBROUTINES. 
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CSUPSAT                                                          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMCON* BASIC UNIVERSAL CONSTANTS AND DERIVED CONSTANTS.  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
C                                                                       
C                                                                       
C       --------------------------------------------------------------- 
C      PARAMETER(                                                       
C     I JPM=106,JPN=106,JPK=106,JPGL=160,JPNLON=320,                    
C     I JPT1MAX=400,          JPT2MAX=50,                               
C     I JPT3MAX=300,          JPT4MAX=50,                               
C     I JPHASH=2048,          JPNTRCRS=20,                              
C     I JPTASKS=4,     JPLOCKS=50,    JPEVENT=4,                        
C     I JPLTMT=JPTASKS,                                                 
C     I JPNUMW=352,            JPNUMP=352,                              
C     I JPTABLE=1024,                                                   
C     I JPMAXB =60,                                                     
C     I JPMAXPB=32,                                                     
C     I JPG3FLD=43,         JPG3XF=99,         JPG3CMP=17,              
C     I JPMP1=JPM+1,        JPNP1=JPN+1,       JPKP1=JPK+1,             
C     I JP2MP1=JPMP1*2,     JP4MP1=JPMP1*4,                             
C     I JPED=JPN/2+1,       JPOD=JPNP1/2,                               
C     I IPSP0=JPM+JPN-JPK,  IPSP1=JPNP1*JPMP1, IPSP2=IPSP0*(IPSP0+1)/2, 
C     I JPSP=IPSP1-IPSP2,   JP2SP=2*JPSP,                               
C     I JPHGL=JPGL/2,       JPNLP2=JPNLON+2,                            
C     I                     JPNVCLV=4*JPNLVP1)                          
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C     J. K. GIBSON     E.C.M.W.F.     22/01/82.                         
C     MODIFIED M.J     E.C.M.W.F.     13/12/82.                         
C                                                                       
      INTEGER                                                           
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      LOGICAL                                                           
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD        
      REAL                                                              
     R DTIME,    TWODT,    EPS                                          
C                                                                       
C      INTEGER                                                          
C     I NWTIME(JPNUMW),     NROW(JPTASKS),                              
C     I NPTIME(JPNUMP)                                                  
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMGAU* - QUANTITIES RELATED TO THE GAUSSIAN GRID.      
C                                                                       
      REAL                                                              
     R SINLON(NLP2),     COSLON(NLP2)                                   
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMTRU* -QUANTITIES RELATED TO THE SPECTRAL TRUNCATION. 
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSIM* - QUANTITIES NEEDED FOR THE SEMI-IMPLICIT SCHEME
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMHDI* - COEFFICIENTS FOR HORIZONTAL DIFFUSION.        
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMFFT* - QUANTITIES NEEDED FOR THE FAST *FOURIER       
C                         TRANSFORMS.                                   
C                                                                       
      INTEGER                                                           
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2                           
C      REAL                                                             
C     R TRIG(JPNLON)                                                    
      INTEGER                                                           
     I NFAX(10)                                                         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSTA* - ARRAYS AND PARAMETERS FOR STATISTICS.         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMDIA* - QUANTITIES RELATED TO GLOBAL PHYSICAL         
C                         DIAGNOSTICS.                                  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPSW* SWITCHES RELATED TO THE PARAMETERISATIONS OF      
C                     DIABATIC PROCESSES.                               
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMDSW* SWITCHES RELATED TO THE DYNAMICS                  
C                     AND THE GENERAL CONTROL OF THE FORECAST.          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH1* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE RADIATION SUBROUTINES.                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CRAE,                                                            
     R CDISSEM                                                          
C                                                                       
C      REAL       COSRAD(128),        SINRAD(128)                       
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH2* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE PHYSICAL SUBROUTINES (EXCEPT RADIATION    
C                     ONES).                                            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL       CCRH(JPNLEV)                                           
C                                                                       
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMDIZ* VARIABLES AND POINTERS FOR ZONAL MEAN DIAGNOSTICS 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMMSK* VARIABLES AND POINTERS FOR AREA MEAN DIAGNOSTICS. 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMSDS* - START/RESTART BLOCK.                          
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMTASK* - TASK RELATED VARIABLES.                      
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*         *COMRAD* - RADIATION POINTERS.                               
C                                                                       
      LOGICAL LO,LO1,LO2,LODIA,LODIAP                                   
C---------------------------------------------------------------------  
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C     COMMON *YOMAER* RADIATIVE CHARACTERISTICS OF THE AEROSOLS         
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
      REAL TAUA(2,5), PIZA(2,5), CGA(2,5), CAER(5,5), CAEROS            
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C     COMMON *YOMLW*                                                    
C                                                                       
C*    COEFFICIENTS FOR THE LONGWAVE RADIATION SUBROUTINES               
C                                                                       
C NINT  :  NUMBER OF SPECTRAL INTERVALS                                 
C NIPD  :  NUMBER OF BANDS DESCRIBED BY PADE APPROXIMANTS               
C NTR   :  NUMBER OF REFERENCE TEMPERATURES                             
C NTRA  :  NUMBER OF TRANSMISSION FUNCTIONS                             
C NUA   :  NUMBER OF ABSORBER AMOUNTS                                   
C NG1   :  ORDER OF THE GAUSSIAN QUADRATURE                             
C                                                                       
C RT1   :  ROOTS OF THE GAUSSIAN POLYNOMIAL                             
C WG1   :  WEIGHTS OF THE GAUSSIAN POLYNOMIAL                           
C AT, BT:  COEFFICIENTS OF THE TEMPERATURE DEPENDENCE                   
C CNTNU :  H2O CONTINUUM ABSORPTION COEFFICIENTS                        
C O1H,O2H,PIALF0,OCT: OZONE LONGWAVE ABSORPTION COEFFICIENTS            
C TREF, TSTAND      : REFERENCE TEMPERATURES                            
C XP    :  COEFFICIENTS OF POLYNOMIAL DEVELOPMENTS OF PLANCK FUNCTION   
C GA,GB :  PADE APPROXIMANTS OF H2O, CO2 TRANSMISSION FUNCTIONS         
C ETY,PTY :  PARAMETERS OF H2O CONTINUUM                                
C TINTP :  REFERENCE TEMPERATURES OF PRE-COMPUTED TRANSMISSIVITIES      
C CE    :  EMS COEFFS OF H2O TRANSMISSION FUNCTIONS                     
C                                                                       
      REAL  RT1(2),WG1(2),AT(12,3),BT(12,3),CNTNU(4),O1H                
     S ,OCT(4),TREF,XP(6,6),TSTAND,GA(11,24,3),GB(11,24,3)              
     S ,PTY(11,20),ETY(11,20),TINTP(11),TSTP,O2H,PIALF0                 
C                                                                       
      REAL  TB(40,31,2),ALP(40,31,2),BET(40,31,2),                      
     S ULG(40),PLG(31),AP(40,31,2),BP(40,31,2),TBO3(38,31),TO3B(38,31)  
     S,ULGO(38),AO3(38,31),BO3(38,31),O3A(38,31),O3B(38,31)             
C     ----------------------------------------------------------------- 
C*COMDECK YOMRDU                                                        
C     ----------------------------------------------------------------- 
C                                                                       
C*    CONTROL VARIABLES FOR RADIATION: INTERNAL SWITCH AND INDICES      
C                                                                       
C ZEELOG:  SECURITY THRESHOLD FOR ABSORBER AMOUNT IN LAPLACE TRANSFORM  
C ZEPSC :  SECURITY THRESHOLD FOR CLOUD COVER                           
C ZEPSCO:  SECURITY THRESHOLD FOR OZONE AMOUNT                          
C ZEPSCQ:  SECURITY THRESHOLD FOR WATER VAPOR                           
C ZEPSCT:  SECURITY THRESHOLD FOR SHORTWAVE OPTICAL THICKNESS           
C ZEPSCW:  SECURITY THRESHOLD FOR CLOUD LIQUID WATER PATH               
C                                                                       
      REAL          CDAY,CCO2,CH2O,DIFF,X10E                            
     S             ,ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW            
C     ----------------------------------------------------------------- 
C*COMDECK YOMSW                                                         
C     ----------------------------------------------------------------- 
C                                                                       
C*    COEFFICIENTS FOR THE SHORTWAVE RADIATION SUBROUTINES              
C                                                                       
C APAD  :  PADE APPROXIMANTS NUMERATOR                                  
C BPAD  :  PADE APPROXIMANTS DENOMINATOR                                
C D     :  TRANSMISSION LIMIT FOR INFINITE ABSORBER AMOUNT              
C CRAY  :  RAYLEIGH SCATTERING COEFFICIENTS                             
C SUN   :  SOLAR FRACTION IN SPECTRAL INTERVALS                         
C                                                                       
      REAL  APAD(2,3,7),BPAD(2,3,7),D(2,3),CRAY(2,6),SUN(2)             
C     ----------------------------------------------------------------- 
                                                                        
C                                                                       
C  -------------------------------------------------------------------- 
C                                                                       
      REAL   TTT(1,NLAT,NRLEV,12)                                       
C                                                                       
C        0.1  "RADINT POINTERS"                                         
C             -----------------                                         
C                                                                       
      REAL   TM1(NLP2,NLEV),                                            
     *       TSM1M(NLP2),                                               
     *       QM1(NLP2,NLEV),                                            
     *       SNM1M(NLP2),                                               
     *       ALBM(NLP2),                                                
     *       AVVELM(NLP2)                                               
      REAL   SLMM(NLP2),                                                
     *       APHM1(NLP2,NLEVP1),                                        
     *       APM1(NLP2,NLEV)                                            
C                                                                       
      LOGICAL LOLAND(NLP2)                                              
C                                                                       
C     VARIABLES FOR FULL GRID AND TRANSFORMS.                           
C                                                                       
      REAL   ZTI(NLP2,NRLEVP1),                                         
     *       ZCTI(NLP2,NRLEVP1),                                        
     *       ZCPH(NLP2,NRLEVP1),                                        
     *       ZWV(NLP2,NRLEV),                                           
     *       ZSW(NLP2,NRLEV),                                           
     *       ZALTE(NLP2),                                               
     *       ZCALTE(NLP2),                                              
     *       ZALSO(NLP2),                                               
     *       ZCALSO(NLP2)                                               
C                                                                       
      REAL   ZOZQ(NLP2),                                                
     #       ZOZH(NLP2)                                                 
C                                                                       
      REAL   ZTRSOL(NLP2,NLEVP1),                                       
     *       ZEMTER(NLP2,NLEVP1)                                        
C                                                                       
C     FINAL RESULTS ARE STORED ON:                                      
C                                                                       
      REAL   TRSOLM(NLP2,NLEVP1),                                       
     *       EMTERM(NLP2,NLEVP1)                                        
C                                                                       
      REAL   AMU0(NLP2),                                                
     *       RDAYL(NLP2)                                                
C                                                                       
C     VARIABLES WITHIN THE SPLITTED LATITUDE LOOP.                      
C                                                                       
      REAL   CLC(NRLST,NRLEV),                                          
     *       TI(NRLST,NRLEVP1),                                         
     *       DP(NRLST,NRLEV),                                           
     *       AADS(NRLST,NRLEV),                                         
     *       AEQ1(NRLST,NRLEV),                                         
     *       AEQ2(NRLST,NRLEV),                                         
     *       AEQ3(NRLST,NRLEV),                                         
     *       AEQ4(NRLST,NRLEV),                                         
     *       AEQ5(NRLST,NRLEV),                                         
     *       SSW(NRLST,NRLEV),                                          
     *       WV(NRLST,NRLEV),                                           
     *       QOF(NRLST,NRLEV),                                          
     *       QCF(NRLST,NRLEV),                                          
     *       CLWA(NRLST,NRLEV),                                         
     *       APRE(NRLST),                                               
     *       SMU0(NRLST),                                               
     *       ALTE(NRLST),                                               
     *       ALSO(NRLST)                                                
      REAL   ZDPO(NRLST),                                               
     *       ZQCFO(NRLST),                                              
     *       ZAEQSO(NRLST),                                             
     *       ZQOFO(NRLST),                                              
     *       ZDPN(NRLST),                                               
     *       ZQCFN(NRLST),                                              
     *       ZAEQSN(NRLST),                                             
     *       ZQOFN(NRLST)                                               
      REAL   FLT(NRLST,NRLEVP1),                                        
     *       FLS(NRLST,NRLEVP1)                                         
C                                                                       
C     DIAGNOSTIC VARIABLES.                                             
C                                                                       
      REAL   DIAG(NLEVP1,4),                                            
     *       DIA1(7),                                                   
     *       ZDIAG(NLEVP1,10),                                          
     *       ZDIA1(21),                                                 
     *       ZDIA(NLON),                                                
     *       ZDIAF(NLON,7),                                             
     *       ZDIAT(NLEVP1)                                              
C                                                                       
C                                                                       
C                                                                       
      REAL PGG(IRLST),                                                  
     #     PREF(IRLST),                                                 
     #     PRE1(IRLST),                                                 
     #     PRE2(IRLST),                                                 
     #     PRMUZ(IRLST),                                                
     #     PTO1(IRLST),                                                 
     #     PTR1(IRLST),                                                 
     #     PTR2(IRLST),                                                 
     #     PW(IRLST)                                                    
C                                                                       
C                                                                       
C  what about the 1-dimensional PALBS???                                
C                                                                       
      REAL     PALBSN  (IRLST)                                          
C                                                                       
C*       0.2   "LOCAL" ARRAYS                                           
C              --------------                                           
      REAL ZABSCU(IRLST ,NUA, 3*NRLEV+1),                               
     S     ZABSLY(IRLST ,NUA, 3*NRLEV+1) ,                              
     S     ZADJD (IRLST , IRLVP1)      ,                                
     S     ZADJU (IRLST , IRLVP1)       ,                               
     S     ZBGND (IRLST)                ,                               
     S     ZBINT (IRLST , IRLVP1)       ,                               
     S     ZBSUR (IRLST , NINT)         ,                               
     S     ZBSUIN(IRLST)                ,                               
     S     ZBTOP (IRLST , NINT)         ,                               
     S     PCLDLW(IRLST , IRLEV)        ,                               
     S     ZCLOUD(IRLST)                ,                               
     S     ZCNTRB(IRLST , IRLVP1,IRLVP1),                               
     S     ZCUMP (IRLST)                ,                               
     S     ZCUMPT(IRLST)                ,                               
     S     ZDBDT (IRLST , NINT , IRLEV) ,                               
     S     ZDBSL (IRLST , NINT , 2*NRLEV),                              
     S     ZDISTD(IRLST , IRLVP1)       ,                               
     S     ZDISTU(IRLST , IRLVP1)                                       
      REAL                                                              
     S     ZDNF  (IRLST , IRLVP1,IRLVP1),                               
     S     PDT0  (IRLST)                                                
      REAL                                                              
     S     ZDUC  (IRLST , 3*NRLEV+1)       ,                            
     S     ZEFFT (IRLST)                ,                               
     S     ZEMD  (IRLST , IRLEV), ZEMU  (IRLST , IRLEV),                
     S     ZEPSC2(IRLST),  ZEPSC3(IRLST),                               
     S     ZEPSH1(IRLST),  ZEPSH2(IRLST),                               
     S     ZEPSH3(IRLST),  ZEPSH4(IRLST),                               
     S     ZEPSH5(IRLST),  ZEPSH6(IRLST),                               
     S     ZEPHIO(IRLST),  ZFACTC(IRLST),                               
     S     ZZFD(IRLST), ZFDD(IRLST), ZFDN(IRLST , IRLVP1)               
      REAL                                                              
     S     PFLUX (IRLST , 2 , IRLVP1)   ,                               
     S     PFLUC (IRLST , 2 , IRLVP1)   ,                               
     S     ZFLWP (IRLST)                ,                               
     S     PCTS  (IRLST , IRLEV)        ,                               
     S     ZZFU   (IRLST)               ,                               
     S     ZFUP  (IRLST , IRLVP1)       ,                               
     S     ZGLAYD(IRLST)                ,                               
     S     ZGLAYU(IRLST)                ,                               
     S     ZOMEGA(IRLST , 2 , IRLEV)                                    
C                                                                       
      INTEGER                                                           
     S     ITX   (IRLST)                ,                               
     S     IXDIA (IRLST , IRLVP1)       ,                               
     S     IXT   (IRLST , IRLVP1,IRLVP1),                               
     S     IXTSU (IRLST , IRLVP1)       ,                               
     S     IXTTP (IRLST , IRLVP1)       ,                               
     S     IMX   (IRLST)                ,                               
     S     IMXP  (IRLST)                                                
C                                                                       
      REAL ZOZ   (IRLST , IRLEV)        ,                               
     S     PPMB  (IRLST , IRLVP1)       ,                               
     S     ZRES  (IRLST)                ,                               
     S     PB    (IRLST, 6 , IRLVP1)    ,                               
     S     ZSSIG (IRLST , 3*NRLEV+1)    ,                               
     S     PTAU  (IRLST , 2 , IRLEV)    ,                               
     S     PTAVE (IRLST , IRLEV)        ,                               
     S     ZTAVIC(IRLST)                ,                               
     S     ZTI1  (IRLST)                ,                               
     S     PTL   (IRLST , IRLVP1)       ,                               
     S     ZTT   (IRLST , NTRA)         ,                               
     S     ZTT1  (IRLST , NTRA)         ,                               
     S     ZTT2  (IRLST , NTRA)         ,                               
     S     ZUAER (IRLST , NINT)         ,                               
     S     ZUPF  (IRLST , IRLVP1,IRLVP1),                               
     S     ZUU   (IRLST , NUA)          ,                               
     S     ZXD   (IRLST)                                                
      REAL                                                              
     S     ZXN   (IRLST)                ,                               
     S     ZXOZ  (IRLST)                ,                               
     S     ZXWV  (IRLST)                ,                               
     S     ZZZ   (IRLST)                ,                               
     S     ZRES2 (IRLST)                ,                               
     S     ZTI2  (IRLST)                ,                               
     S     ZBLAY (IRLST,IRLEV)          ,                               
     S     ZBLEV (IRLST,IRLVP1)         ,                               
     S     ZCTS  (IRLST)                ,                               
     S     ZHFG  (IRLST)                                                
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C                                                                       
      INTEGER KT    (IRLST)                                             
      REAL    PUU   (IRLST , NUA  )     ,                               
     S        PTT   (IRLST , NTRA )     ,                               
     S        ZZ    (IRLST)                                             
C                                                                       
C     -------------------------------------------------------- ---------
C                                                                       
      REAL  PCLFR  (IRLST , IRLEV)       ,                              
     S      PT     (IRLST , IRLVP1)       ,                             
     S      PQOF   (IRLST , IRLEV)          ,                           
     S      PQLWP  (IRLST , IRLEV)          ,                           
     S      PEMIS  (IRLST)                                              
      REAL                                                              
     S      PFLT   (IRLST , IRLVP1)                                     
C                                                                       
      REAL  ZALBSU(IRLST , 2 )           ,                              
     S      ZCG   (IRLST , 2 , IRLEV)    ,                              
     S      ZCLDLW(IRLST , IRLEV)        ,                              
     S      ZCLDSW(IRLST , IRLEV)        ,                              
     S      ZCOOLR(IRLST , IRLEV)        ,                              
     S      ZDT0  (IRLST)                ,                              
     S      ZFLUX (IRLST , 2 , IRLVP1)   ,                              
     S      ZPMB  (IRLST , IRLVP1)       ,                              
     S      ZTAU  (IRLST , 2 , IRLEV)    ,                              
     S      ZTAVE (IRLST , IRLEV)        ,                              
     S      ZTL   (IRLST , IRLVP1)       ,                              
     S      ZFSDWN(IRLST , IRLVP1)       ,                              
     S      ZFSUP (IRLST , IRLVP1)       ,                              
     S      ZHEATR(IRLST , IRLEV )                                      
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C                                                                       
      REAL   PDP   (IRLST , IRLEV)     ,                                
     S       PWV   (IRLST , IRLEV)     ,                                
     S       PFLS  (IRLST , IRLVP1)    ,                                
     S       PPSOL (IRLST        )     ,                                
     S       PRMU0 (IRLST        )     ,                                
     S       PAER  (IRLST,IRLEV,5)     ,                                
     S       PALBS (IRLST , 2 )        ,                                
     S       PCG   (IRLST , 2 , IRLEV) ,                                
     S       PCLDSW(IRLST , IRLEV)     ,                                
     S       POMEGA(IRLST , 2 , IRLEV) ,                                
     S       POZ   (IRLST , IRLEV)                                      
C                                                                       
      REAL   PFDOWN(IRLST , IRLVP1)    ,                                
     S       PFUP  (IRLST , IRLVP1)                                     
C                                                                       
      REAL   ZAKI  (IRLST , 2 )        ,                                
     S       ZCGAZ (IRLST , IRLEV)     ,                                
     S       ZCNEB (IRLST , IRLEV)     ,                                
     S       ZC1I  (IRLST , IRLVP1)    ,                                
     S       ZDSC  (IRLST , IRLEV)     ,                                
     S       ZDSH  (IRLST , IRLEV)     ,                                
     S       ZDSIG (IRLST , IRLEV)     ,                                
     S       ZFACT (IRLST)             ,                                
     S       ZFACT1 (IRLST), ZFACT2 (IRLST),                            
     S       ZFACT3 (IRLST), ZFACT4 (IRLST),                            
     S       ZFACT5 (IRLST), ZFACT6 (IRLST),                            
     S       ZFD   (IRLST , IRLVP1)    ,                                
     S       ZFRAC (IRLST)             ,                                
     S       ZFU   (IRLST , IRLVP1)    ,                                
     S       ZG    (IRLST)             ,                                
     S       ZGG   (IRLST)             ,                                
     S       ZPIZAZ(IRLST , IRLEV)     ,                                
     S       ZRAYL (IRLST),  ZRAY1 (IRLST , IRLVP1)                     
      REAL                                                              
     S       ZRAY2 (IRLST , IRLVP1)    ,                                
     S       ZREF  (IRLST)             ,                                
     S       ZREFZ (IRLST , 2 , IRLVP1),                                
     S       ZRE1  (IRLST)             ,                                
     S       ZRE2  (IRLST)             ,                                
     S       ZRJ   (IRLST , 6 , IRLVP1),                                
     S       ZRK   (IRLST , 6 , IRLVP1),                                
     S       ZRL   (IRLST , 8)         ,                                
     S       ZRMU  (IRLST)             ,                                
     S       ZRMUE (IRLST , IRLVP1)    ,                                
     S       ZRMUZ (IRLST)                                              
      REAL                                                              
     S       ZRNEB (IRLST)             ,                                
     S       ZRUEF (IRLST , 8)         ,                                
     S       ZR1   (IRLST)             ,                                
     S       ZR2   (IRLST)             ,                                
     S       ZR21  (IRLST)             ,                                
     S       ZR22  (IRLST)             ,                                
     S       ZR23  (IRLST)             ,                                
     S       ZS    (IRLST)             ,                                
     S       ZSEC  (IRLST)             ,                                
     S       ZSS1  (IRLST)             ,                                
     S       ZTAUAZ(IRLST , IRLEV)     ,                                
     S       ZTOT  (IRLST , IRLVP1)    ,                                
     S       ZTO1  (IRLST)             ,                                
     S       ZTR   (IRLST , 2 , IRLVP1),                                
     S       ZTRA1 (IRLST , IRLVP1)    ,                                
     S       ZTRA2 (IRLST , IRLVP1)                                     
      REAL                                                              
     S       ZTR1  (IRLST)             ,                                
     S       ZTR2  (IRLST)             ,                                
     S       ZUD   (IRLST , 3 , IRLVP1),                                
     S       ZUM   (IRLST , IRLVP1)    ,                                
     S       ZU1D  (IRLST)             ,                                
     S       ZU2D  (IRLST)             ,                                
     S       ZW    (IRLST)             ,                                
     S       ZN175 (IRLST)             ,                                
     S       ZN190 (IRLST)             ,                                
     S       ZO175 (IRLST)             ,                                
     S       ZO190 (IRLST)             ,                                
     S       ZP75  (IRLST)             ,                                
     S       ZP90  (IRLST)             ,                                
     S       ZSIGN (IRLST)             ,                                
     S       ZSIGO (IRLST)                                              
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
      REAL   PTR    (IRLST)     ,                                       
     S       PU     (IRLST)                                             
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
C   CLOUD-POINTERS                                                      
C   --------------                                                      
C                                                                       
      REAL   ZRH(NLP2,NLEV),                                            
     *       ZCH(NLP2),                                                 
     *       ZCM(NLP2),                                                 
     *       ZCL(NLP2),                                                 
     *       ZCT(NLP2),                                                 
     *       ZCC(NLP2)                                                  
      INTEGER NTOPCM(NLP2),                                             
     *        NBASECM(NLP2)                                             
      REAL   ARPRCM(NLP2)                                               
      INTEGER ITOPH(NLP2),                                              
     *        ITOPM(NLP2),                                              
     *        ITOPL(NLP2)                                               
      LOGICAL LOC(NLP2)                                                 
      REAL   ZQS(NLP2,NLEV),                                            
     *       ZCLC(NLP2,NLEV),                                           
     *       ZQLWP(NLP2,NLEV)                                           
C                                                                       
      REAL   VERVEL(NLP2,NLEV),                                         
     *       ZTHETA(NLP2,NLEV),                                         
     *       ZETA(NLP2,NLEV),                                           
     *       ZDTHMIN(NLP2)                                              
      INTEGER INVB(NLP2)                                                
      REAL   ZCTLWP(NLP2),                                              
     *       ZQLOC(NLP2),                                               
     *       ZQMEC(NLP2),                                               
     *       ZQHIC(NLP2)                                                
C                                                                       
C  definitions of "radheat"                                             
C                                                                       
      REAL   ZFSOT(NLON),                                               
     #       ZFTET(NLON),                                               
     #       ZFLT (NLON),                                               
     #       ZFSO (NLON),                                               
     #       ZFTE (NLON),                                               
     #       ZFLB (NLON),                                               
     #       ZDTDT(NLON),                                               
     #       HEATO(NLON,NLEV),                                          
     #       HEATE(NLON,NLEV),                                          
     #       HEATR(NLON,NLEV),                                          
     #       ZI0  (NLON)                                                
C                                                                       
C PIERS 13-8-97                                                         
C                                                                       
         REAL PIERSLWP                                                  
C                                                                       
C Piece of code needed to replace the Nikos radiation scheme with       
C the Morcrette radiation scheme in the igcm3.1.                        
C See morcrette3_1_1.readme for details.                                
C                                                                       
C                                                                       
c      program lh                                                       
c                                                                       
c      parameter(NL=23)                                                 
c      real P(NL),t(NL),h2o(NL),o3(NL),htlw(NL),htsw(NL)                
c      real swalb,doy                                                   
c       REAL CF(4,2)     !cloud fractions,lwps                          
c       INTEGER IC(4,2)   ! cloud positions                             
c       REAL RFLUXES(2,2,2)                                             
c      open(11,file='mls23.dat')                                        
c      read(11,*) NLEV                                                  
c      DOY=80                                                           
c      swalb=0.07                                                       
c      alat=40.0                                                        
c       doy=float(i)                                                    
c      do i=nlev,1,-1                                                   
c         read(11,*) P(i),t(i),h2o(i),adum,o3(i)                        
c         print *,i,p(i),t(i),h2o(i),o3(i)                              
c      enddo                                                            
c                                                                       
c       ic(1,1)=3                                                       
c      ic(1,2)=17                                                       
c       ic(3,1)=6                                                       
c       ic(2,2)=6                                                       
c       cf(1,1)=0.3                                                     
c       cf(1,2)=45.0                                                    
c        cf(3,1)=0.8                                                    
c       cf(4,2)=20.0                                                    
c      alon=0.0                                                         
c      call morcigcm(p,t,h2o,o3,alat,htlw,htsw,doy,cf,ic,               
c     $     rfluxes,swalb,alon)                                         
c      enddo                                                            
c      do i=nl,1,-1                                                     
c         print '(i6,4F14.4)',i,p(i),t(i),htsw(i),htlw(i)               
c      enddo                                                            
c      print *,RFLUXES(1,1,1), RFLUXES(1,1,2), RFLUXES(1,2,1),          
c     $     RFLUXES(1,2,2)                                              
c      print *,RFLUXES(2,1,1), RFLUXES(2,1,2), RFLUXES(2,2,1),          
c     $     RFLUXES(2,2,2)                                              
c                                                                       
c                                                                       
c      end                                                              
c      INCLUDE 'zparb.upd'                                              
C                                                                       
C      -------------------------------------------------------------    
C                                                                       
C*       1.    SHORTWAVE COEFFICIENTS                                   
C              ----------------------                                   
C                                                                       
      DATA ((TAUA(IN,JA),JA=1,5),IN=1,2) /                              
     S .730719, .912819, .725059, .745405, .682188 ,                    
     S .730719, .912819, .725059, .745405, .682188 /                    
      DATA ((PIZA(IN,JA),JA=1,5),IN=1,2) /                              
     S .872212, .982545, .623143, .944887, .997975 ,                    
     S .872212, .982545, .623143, .944887, .997975 /                    
      DATA ((CGA (IN,JA),JA=1,5),IN=1,2) /                              
     S .647596, .739002, .580845, .662657, .624246 ,                    
     S .647596, .739002, .580845, .662657, .624246 /                    
C      -------------------------------------------------------------    
C                                                                       
C*       2.    LONGWAVE COEFFICIENTS                                    
C              ---------------------                                    
C                                                                       
      DATA CAER / .038520, .037196, .040532, .054934, .038520           
     1          , .12613 , .18313 , .10357 , .064106, .126130           
     2          , .012579, .013649, .018652, .025181, .012579           
     3          , .011890, .016142, .021105, .028908, .011890           
     4          , .013792, .026810, .052203, .066338, .013792 /         
C                                                                       
      RETURN                                                            
      END                                                               
        SUBROUTINE SULW                                                 
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRSW* SWITCHES RELATED TO THE RADIATION SCHEME.         
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      IMPLICIT REAL (A-H,O-Z)                                           
C      IMPLICIT UNDEFINED (Z)                                           
C      IMPLICIT LOGICAL (L)                                             
C      IMPLICIT UNDEFINED (L)                                           
      IMPLICIT INTEGER (I-K,M-N)                                        
C      IMPLICIT UNDEFINED (I-K,M-N)                                     
C      IMPLICIT REAL (P)                                                
C      IMPLICIT UNDEFINED (A-H,O,Q-Y)                                   
C                                                                       
C everything except the parameters is defined                           
C in the common blocks                                                  
C                                                                       
      COMMON/COM1/                                                      
C     I NRADFR, NRPART, NRINT, NRADPFR, NRADPLA,                        
C     I NRLSTI,                                                         
     R XLAT,                                                            
     R RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR, ZCCO2,              
C     R PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3,                 
     L LOH, LOM, LOL, LORAD                                             
      COMMON/COM2/                                                      
     I ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,                  
     I         JLOOP, KRLEV,                                            
     I        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     I        ITH, ITM, ITL,                                            
     I        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     I        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     I        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     I        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      COMMON/COM3/                                                      
     I IND4, KM12, KXD, KXU, IG, KBS, KDD,                              
     I        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     I        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     I        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     I        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     I        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
      COMMON/COM4/                                                      
     R           COZQC,COZQS,COZHC,COZHS,                               
     R CSUPSAT,                                                         
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
      COMMON/COM5/                                                      
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NUMVC,                                                           
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      COMMON/COM6/                                                      
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD,       
     R DTIME,    TWODT,    EPS,                                         
     R SINLON,     COSLON,                                              
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2,                          
     I NFAX,                                                            
     R CRAE,                                                            
     R CDISSEM                                                          
      COMMON/COM7/                                                      
     R     CCRH,                                                        
     L LO,LO1,LO2,LODIA,LODIAP,                                         
     R TAUA, PIZA, CGA, CAER, CAEROS                                    
      COMMON/COM7A/                                                     
     R  RT1,WG1,AT,BT,CNTNU,O1H,O2H,PIALF0                              
     S ,OCT,TREF,XP,TSTAND,GA,GB,CE,ETY,PTY                             
      COMMON/COM7B/                                                     
     S TINTP,TSTP,                                                      
     S  TBO3,AO3,BO3,TB,ALP,BET,ULG,ULGO,PLG,AP,BP,TO3B,O3A,O3B,        
     R          CDAY,CCO2,CH2O,DIFF,X10E                                
       COMMON/COM7C/                                                    
     S  ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW,                       
     R  APAD,BPAD,D,CRAY,SUN,                                           
     R   TTT,                                                           
     R   TM1                                                            
      COMMON/COM8/                                                      
     R       TSM1M,                                                     
     R       QM1,                                                       
     R       SNM1M,                                                     
     R       ALBM,                                                      
     R       AVVELM,                                                    
     R   SLMM,                                                          
     R       APHM1,                                                     
     R       APM1                                                       
      COMMON/COM9/                                                      
     L LOLAND,                                                          
     R   ZTI,                                                           
     *       ZCTI,                                                      
     *       ZCPH,                                                      
     *       ZWV,                                                       
     *       ZSW,                                                       
     *       ZALTE,                                                     
     *       ZCALTE,                                                    
     *       ZALSO                                                      
      COMMON/COM10/                                                     
     *       ZCALSO,                                                    
     R   ZOZQ,                                                          
     *       ZOZH,                                                      
     R   ZTRSOL,                                                        
     *       ZEMTER,                                                    
     R   TRSOLM,                                                        
     *       EMTERM,                                                    
     R   AMU0,                                                          
     *       RDAYL                                                      
      COMMON/COM11/                                                     
     R   CLC,                                                           
     *       TI,                                                        
     *       DP,                                                        
     *       AADS,                                                      
     *       AEQ1,                                                      
     *       AEQ2,                                                      
     *       AEQ3,                                                      
     *       AEQ4,                                                      
     *       AEQ5,                                                      
     *       SSW                                                        
      COMMON/COM12/                                                     
     *       WV,                                                        
     *       QOF,                                                       
     *       QCF,                                                       
     *       CLWA,                                                      
     *       APRE,                                                      
     *       SMU0,                                                      
     *       ALTE,                                                      
     *       ALSO                                                       
      COMMON/COM13/                                                     
     R   ZDPO,                                                          
     *       ZQCFO,                                                     
     *       ZAEQSO,                                                    
     *       ZQOFO,                                                     
     *       ZDPN,                                                      
     *       ZQCFN,                                                     
     *       ZAEQSN,                                                    
     *       ZQOFN                                                      
      COMMON/COM14/                                                     
     R   FLT,                                                           
     *       FLS,                                                       
     R   DIAG,                                                          
     *       DIA1,                                                      
     *       ZDIAG,                                                     
     *       ZDIA1,                                                     
     *       ZDIA,                                                      
     *       ZDIAF,                                                     
     *       ZDIAT,                                                     
     R PGG                                                              
      COMMON/COM15/                                                     
     #     PREF,                                                        
     #     PRE1,                                                        
     #     PRE2,                                                        
     #     PRMUZ,                                                       
     #     PTO1,                                                        
     #     PTR1,                                                        
     #     PTR2,                                                        
     #     PW,                                                          
     R     PALBSN                                                       
      COMMON/COM16/                                                     
     R ZABSCU,                                                          
     S     ZABSLY,                                                      
     S     ZADJD ,                                                      
     S     ZADJU ,                                                      
     S     ZBGND ,                                                      
     S     ZBINT ,                                                      
     S     ZBSUR ,                                                      
     S     ZBSUIN,                                                      
     S     ZBTOP ,                                                      
     S     PCLDLW,                                                      
     S     ZCLOUD,                                                      
     S     ZCNTRB,                                                      
     S     ZCUMP ,                                                      
     S     ZCUMPT,                                                      
     S     ZDBDT                                                        
      COMMON/COM17/                                                     
     S     ZDBSL ,                                                      
     S     ZDISTD,                                                      
     S     ZDISTU,                                                      
     S     ZDNF  ,                                                      
     S     PDT0  ,                                                      
     S     ZDUC                                                         
      COMMON/COM18/                                                     
     S     ZEFFT ,                                                      
     S     ZEMU  ,                                                      
     S     ZEPHIO,                                                      
     S     ZEPSC2,                                                      
     S     ZEPSC3,                                                      
     S     ZEPSH1,                                                      
     S     ZEPSH2,                                                      
     S     ZEPSH3                                                       
      COMMON/COM19/                                                     
     S     ZEPSH4,                                                      
     S     ZEPSH5,                                                      
     S     ZEPSH6,                                                      
     S     ZEPSIO,                                                      
     S     ZFACTC,                                                      
     S     ZZFD  ,                                                      
     S     ZFDD  ,                                                      
     S     ZFDN                                                         
      COMMON/COM20/                                                     
     S     PFLUX , PFLUC,                                               
     S     ZFLWP , PCTS                                                 
     S     ZZFU  ,                                                      
     S     ZFUP  ,                                                      
     S     ZGLAYD,                                                      
     S     ZGLAYU,                                                      
     S     ZOMEGA,                                                      
     S     ITX   ,                                                      
     S     IXDIA ,                                                      
     S     IXT   ,                                                      
     S     IXTSU ,                                                      
     S     IXTTP ,                                                      
     S     IMX   ,                                                      
     S     IMXP                                                         
      COMMON/COM21/                                                     
     R ZOZ   ,                                                          
     S     PPMB  ,                                                      
     S     ZRES  , PB                                                   
     S     ZSSIG ,                                                      
     S     PTAU  ,                                                      
     S     PTAVE ,                                                      
     S     ZTAVIC,                                                      
     S     ZTI1  ,                                                      
     S     PTL                                                          
      COMMON/COM22/                                                     
     S     ZTT   ,                                                      
     S     ZTT1  ,                                                      
     S     ZTT2  ,                                                      
     S     ZUAER ,                                                      
     S     ZUPF  ,                                                      
     S     ZUU   ,                                                      
     S     ZXD                                                          
      COMMON/COM23/                                                     
     S     ZXN   ,                                                      
     S     ZXOZ  ,                                                      
     S     ZXWV  ,                                                      
     S     ZZZ   ,                                                      
     S     ZRES2 ,                                                      
     S     ZTI2  ,                                                      
     S     ZBLAY ,                                                      
     S     ZBLEV ,                                                      
     S     ZCTS  ,                                                      
     S     ZHFG                                                         
      COMMON/COM24/                                                     
     I KT,                                                              
     R    PUU   ,                                                       
     S        PTT   ,                                                   
     S        ZZ                                                        
     R  PCLFR  ,                                                        
     S      PT     ,                                                    
     S      PQOF   ,                                                    
     S      PQLWP  ,                                                    
     S      PEMIS  ,                                                    
     S      PFLT                                                        
      COMMON/COM25/                                                     
     R  ZALBSU,                                                         
     S      ZCG   ,                                                     
     S      ZCLDLW,                                                     
     S      ZCLDSW,                                                     
     S      ZCOOLR,                                                     
     S      ZDT0  ,                                                     
     S      ZEMD  ,                                                     
     S      ZFLUX ,                                                     
     S      ZPMB  ,                                                     
     S      ZTAU  ,                                                     
     S      ZTAVE ,                                                     
     S      ZTL   ,                                                     
     S      ZFSDWN                                                      
      COMMON/COM26/                                                     
     S      ZFSUP ,                                                     
     S      ZHEATR,                                                     
     R   PDP   ,                                                        
     S       PWV   ,                                                    
     S       PFLS  ,                                                    
     S       PPSOL ,                                                    
     S       PRMU0 ,                                                    
     S       PAER  ,                                                    
     S       PALBS ,                                                    
     S       PCG   ,                                                    
     S       PCLDSW,                                                    
     S       POMEGA,                                                    
     S       POZ                                                        
      COMMON/COM27/                                                     
     R   PFDOWN,                                                        
     S       PFUP  ,                                                    
     R   ZAKI  ,                                                        
     S       ZCGAZ ,                                                    
     S       ZCNEB ,                                                    
     S       ZC1I  ,                                                    
     S       ZDSC  ,                                                    
     S       ZDSH                                                       
      COMMON/COM28/                                                     
     S       ZDSIG ,                                                    
     S       ZFACT ,                                                    
     S       ZFACT1 ,                                                   
     S       ZFACT2 ,                                                   
     S       ZFACT3 ,                                                   
     S       ZFACT4 ,                                                   
     S       ZFACT5 ,                                                   
     S       ZFACT6 ,                                                   
     S       ZFD   ,                                                    
     S       ZFRAC ,                                                    
     S       ZFU   ,                                                    
     S       ZG    ,                                                    
     S       ZGG   ,                                                    
     S       ZPIZAZ                                                     
      COMMON/COM29/                                                     
     S       ZRAYL ,                                                    
     S       ZRAY1 ,                                                    
     S       ZRAY2 ,                                                    
     S       ZREF  ,                                                    
     S       ZREFZ ,                                                    
     S       ZRE1  ,                                                    
     S       ZRE2  ,                                                    
     S       ZRJ   ,                                                    
     S       ZRK   ,                                                    
     S       ZRL   ,                                                    
     S       ZRMU  ,                                                    
     S       ZRMUE ,                                                    
     S       ZRMUZ                                                      
      COMMON/COM30/                                                     
     S       ZRNEB ,                                                    
     S       ZRUEF ,                                                    
     S       ZR1   ,                                                    
     S       ZR2   ,                                                    
     S       ZR21  ,                                                    
     S       ZR22  ,                                                    
     S       ZR23  ,                                                    
     S       ZS    ,                                                    
     S       ZSEC  ,                                                    
     S       ZSS1  ,                                                    
     S       ZTAUAZ                                                     
      COMMON/COM31/                                                     
     S       ZTOT  ,                                                    
     S       ZTO1  ,                                                    
     S       ZTR   ,                                                    
     S       ZTRA1 ,                                                    
     S       ZTRA2 ,                                                    
     S       ZTR1                                                       
      COMMON/COM32/                                                     
     S       ZTR2  ,                                                    
     S       ZUD   ,                                                    
     S       ZUM   ,                                                    
     S       ZU1D  ,                                                    
     S       ZU2D  ,                                                    
     S       ZW    ,                                                    
     S       ZN175 ,                                                    
     S       ZN190 ,                                                    
     S       ZO175 ,                                                    
     S       ZO190 ,                                                    
     S       ZP75  ,                                                    
     S       ZP90  ,                                                    
     S       ZSIGN ,                                                    
     S       ZSIGO                                                      
      COMMON/COM33/                                                     
     R   PTR,                                                           
     S       PU,                                                        
     R   ZRH,                                                           
     *       ZCH,                                                       
     *       ZCM,                                                       
     *       ZCL,                                                       
     *       ZCT,                                                       
     *       ZCC,                                                       
     I NTOPCM                                                           
      COMMON/COM34/                                                     
     *        NBASECM,                                                  
     R   ARPRCM,                                                        
     I ITOPH,                                                           
     *        ITOPM,                                                    
     *        ITOPL,                                                    
     L LOC,                                                             
     R   ZQS,                                                           
     *       ZCLC,                                                      
     *       ZQLWP                                                      
      COMMON/COM35/                                                     
     R   VERVEL,                                                        
     *       ZTHETA,                                                    
     *       ZETA,                                                      
     *       ZDTHMIN,                                                   
     I INVB,                                                            
     R   ZCTLWP,                                                        
     *       ZQLOC,                                                     
     *       ZQMEC,                                                     
     *       ZQHIC,                                                     
     R   ZFSOT,                                                         
     #       ZFTET,                                                     
     #       ZFLT                                                       
      COMMON/COM36/                                                     
     #       ZFSO ,                                                     
     #       ZFTE ,                                                     
     #       ZFLB ,                                                     
     #       ZDTDT,                                                     
     #       HEATO,                                                     
     #       HEATE,                                                     
     #       HEATR,                                                     
     #       ZI0                                                        
C                                                                       
      INTEGER                                                           
C     I NRADFR,   NRPART,   NRINT,    NRLSTI,  NRADPFR,  NRADPLA,       
     I NRLST,    NRLEV,    NRLEVP1, NLEV, NLP2, NLEVP1,                 
     I NLON, IRLEV, NTRA, NLAT, ITIM, NUA, NINT, NRPARTI                
C                                                                       
C   NRLEV .... number of vertical levels                                
C   NRLST .... number of points on a latitude line                      
C   NUA   .... number of absorber amount                                
C   NINT  .... number of spectral intervals                             
C   NTRA  .... number of transmission functions                         
C   NLAT  .... number of latitude points                                
C   ITIM  .... number of month under consideration                      
C                                                                       
C      PARAMETER (NRLEV=122,NUA=34,NINT=6,NTRA=34, NLAT=1,              
      PARAMETER (NRLEV=21,NUA=34,NINT=6,NTRA=34, NLAT=1,                
cccm      PARAMETER (NRLEV=37,NUA=18,NINT=6,NTRA=19, NLAT=1,            
cccm     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=10,NLEV=NRLEV,        
     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=7,NLEV=NRLEV,             
     #           NLP2=NRLST, NLEVP1=NRLEVP1, NLON=NRLST,                
     #           NRPARTI=1)                                             
C                                                                       
      REAL YCLOCK, XLAT(NLAT), Z0, DZ, ALBMPAR, SNPAR,                  
     #     TSUPAR, CTFREEZ, DELTAC, DELTAO, DELTAH                      
C                                                                       
C   YTIME  .... time of the year in radians                             
C   YCLOCK .... time of the day in radians                              
C   XLAT   .... latitudes in radians                                    
C   Z0     .... zero point of height grid                               
C   DZ     .... Interval between height levels                          
C   ALBMPAR.... sw surface albedo as function of longitude:             
C           yearly averaged sea-ice: 0.55                               
C                           water  : 0.07                               
C                           land   : 0.07.lt.ALBM.lt.0.80               
C                                    depending on snow                  
C   SNPAR  .... snow depth in meter                                     
C   TSUPAR .... surface temperature (Kelvin)                            
C   CTFREEZ.... freezing temperature of sea-water (Kelvin)              
C   DELTAC .... Doppler parameter for line shapes of CO2                
C   DELTAO .... Doppler parameter for line shapes of O3                 
C                                                                       
      PARAMETER (Z0=0., CTFREEZ=271.26,                                 
C     PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
     #           DZ=0.289, ALBMPAR=0.2, SNPAR=0.,                       
cccm  PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
cccm     #           DZ=0.15, ALBMPAR=0.3, SNPAR=0.,                    
CZ     #           DELTAC=0., DELTAO=0.)                                
     #           DELTAC=6.0E-04, DELTAO=1.E-02, DELTAH=5.0E-04)         
C      PARAMETER (YCLOCK=0., Z0=0., TSUPAR=273., CTFREEZ=271.26,        
C     #           DZ=0.46875, ALBMPAR=0.07, SNPAR=0.)                   
C      PARAMETER (YCLOCK=0., Z0=2.608, TSUPAR=273., CTFREEZ=271.26,     
C     #           DZ=0.2878, ALBMPAR=0.07, SNPAR=0.)                    
C                                                                       
      LOGICAL LDIUR, LOLPAR                                             
C                                                                       
C   LDIUR .... true: sun at time, false: diurnally averaged             
C   LOLPAR.... true: land surface, false: else                          
C                                                                       
C      PARAMETER (LDIUR=.TRUE., LOLPAR=.TRUE.)                          
       PARAMETER (LOLPAR=.TRUE.)                                        
C                                                                       
      INTEGER  C2TEMP                                                   
      CHARACTER*64  MIDTMP                                              
cccm      PARAMETER (C2TEMP=10,                                         
cccm     #    MIDTMP='/home/atm/ugamp/midrad/radfildat.dir/MIDTMP.DAT') 
ccc1      PARAMETER (C2TEMP=10, MIDTMP='tprfil2.dat')                   
C                                                                       
      INTEGER  IRLONI, IRLST, IRLVP1, JPNLEV, JPNLVP1                   
C                                                                       
      PARAMETER (IRLONI=NRLST, IRLST=NRLST,                             
     #           IRLVP1=NRLEVP1, JPNLEV=NRLEV,                          
     #           JPNLVP1=NRLEVP1, IRLEV=NRLEV)                          
C                                                                       
      INTEGER NIPD, NIPD2, NTR, NG1, NG1P1                              
      PARAMETER(NIPD=8,NIPD2=16,NTR=11,NG1=2,NG1P1=3)                   
C                                                                       
C    definition of the functions                                        
C                                                                       
      REAL       CVMGT, SIGMA2                                          
      INTEGER    CVMGI                                                  
C                                                                       
C    definitons of the subroutine cloud                                 
C                                                                       
      REAL RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR                  
C                                                                       
C    missing definitions in morcrette                                   
C                                                                       
C      REAL   PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3            
      REAL ZCCO2                                                        
                                                                        
      LOGICAL LOH, LOM, LOL, LORAD                                      
C                                                                       
      INTEGER ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,           
     #        IZQS, IZCLC, IZQLWP, JLOOP, KRLEV,                        
     #        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     #        ITH, ITM, ITL,                                            
     #        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     #        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     #        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     #        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      INTEGER IND4, KM12, KXD, KXU, IG, KBS, KDD,                       
     #        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     #        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     #        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     #        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     #        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD1* SPECTRAL DISTRIBUTION OF AEROSOLS AND OZONE.     
C                     (TRIANGULAR *T10* TRUNCATIONFOR AEROSOLS).        
C                     (TRIANGULAR *T5* TRUNCATIONFOR OZONE).            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R           COZQC(21),COZQS(15),COZHC(21),COZHS(15)                
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD2* CONSTANTS INTERNAL TO THE RADIATION SUBROUTINES. 
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CSUPSAT                                                          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMCON* BASIC UNIVERSAL CONSTANTS AND DERIVED CONSTANTS.  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
C                                                                       
C                                                                       
C       --------------------------------------------------------------- 
C      PARAMETER(                                                       
C     I JPM=106,JPN=106,JPK=106,JPGL=160,JPNLON=320,                    
C     I JPT1MAX=400,          JPT2MAX=50,                               
C     I JPT3MAX=300,          JPT4MAX=50,                               
C     I JPHASH=2048,          JPNTRCRS=20,                              
C     I JPTASKS=4,     JPLOCKS=50,    JPEVENT=4,                        
C     I JPLTMT=JPTASKS,                                                 
C     I JPNUMW=352,            JPNUMP=352,                              
C     I JPTABLE=1024,                                                   
C     I JPMAXB =60,                                                     
C     I JPMAXPB=32,                                                     
C     I JPG3FLD=43,         JPG3XF=99,         JPG3CMP=17,              
C     I JPMP1=JPM+1,        JPNP1=JPN+1,       JPKP1=JPK+1,             
C     I JP2MP1=JPMP1*2,     JP4MP1=JPMP1*4,                             
C     I JPED=JPN/2+1,       JPOD=JPNP1/2,                               
C     I IPSP0=JPM+JPN-JPK,  IPSP1=JPNP1*JPMP1, IPSP2=IPSP0*(IPSP0+1)/2, 
C     I JPSP=IPSP1-IPSP2,   JP2SP=2*JPSP,                               
C     I JPHGL=JPGL/2,       JPNLP2=JPNLON+2,                            
C     I                     JPNVCLV=4*JPNLVP1)                          
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C     J. K. GIBSON     E.C.M.W.F.     22/01/82.                         
C     MODIFIED M.J     E.C.M.W.F.     13/12/82.                         
C                                                                       
      INTEGER                                                           
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      LOGICAL                                                           
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD        
      REAL                                                              
     R DTIME,    TWODT,    EPS                                          
C                                                                       
C      INTEGER                                                          
C     I NWTIME(JPNUMW),     NROW(JPTASKS),                              
C     I NPTIME(JPNUMP)                                                  
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMGAU* - QUANTITIES RELATED TO THE GAUSSIAN GRID.      
C                                                                       
      REAL                                                              
     R SINLON(NLP2),     COSLON(NLP2)                                   
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMTRU* -QUANTITIES RELATED TO THE SPECTRAL TRUNCATION. 
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSIM* - QUANTITIES NEEDED FOR THE SEMI-IMPLICIT SCHEME
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMHDI* - COEFFICIENTS FOR HORIZONTAL DIFFUSION.        
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMFFT* - QUANTITIES NEEDED FOR THE FAST *FOURIER       
C                         TRANSFORMS.                                   
C                                                                       
      INTEGER                                                           
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2                           
C      REAL                                                             
C     R TRIG(JPNLON)                                                    
      INTEGER                                                           
     I NFAX(10)                                                         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSTA* - ARRAYS AND PARAMETERS FOR STATISTICS.         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMDIA* - QUANTITIES RELATED TO GLOBAL PHYSICAL         
C                         DIAGNOSTICS.                                  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPSW* SWITCHES RELATED TO THE PARAMETERISATIONS OF      
C                     DIABATIC PROCESSES.                               
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMDSW* SWITCHES RELATED TO THE DYNAMICS                  
C                     AND THE GENERAL CONTROL OF THE FORECAST.          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH1* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE RADIATION SUBROUTINES.                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CRAE,                                                            
     R CDISSEM                                                          
C                                                                       
C      REAL       COSRAD(128),        SINRAD(128)                       
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH2* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE PHYSICAL SUBROUTINES (EXCEPT RADIATION    
C                     ONES).                                            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL       CCRH(JPNLEV)                                           
C                                                                       
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMDIZ* VARIABLES AND POINTERS FOR ZONAL MEAN DIAGNOSTICS 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMMSK* VARIABLES AND POINTERS FOR AREA MEAN DIAGNOSTICS. 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMSDS* - START/RESTART BLOCK.                          
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMTASK* - TASK RELATED VARIABLES.                      
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*         *COMRAD* - RADIATION POINTERS.                               
C                                                                       
      LOGICAL LO,LO1,LO2,LODIA,LODIAP                                   
C---------------------------------------------------------------------  
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C     COMMON *YOMAER* RADIATIVE CHARACTERISTICS OF THE AEROSOLS         
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
      REAL TAUA(2,5), PIZA(2,5), CGA(2,5), CAER(5,5), CAEROS            
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C     COMMON *YOMLW*                                                    
C                                                                       
C*    COEFFICIENTS FOR THE LONGWAVE RADIATION SUBROUTINES               
C                                                                       
C NINT  :  NUMBER OF SPECTRAL INTERVALS                                 
C NIPD  :  NUMBER OF BANDS DESCRIBED BY PADE APPROXIMANTS               
C NTR   :  NUMBER OF REFERENCE TEMPERATURES                             
C NTRA  :  NUMBER OF TRANSMISSION FUNCTIONS                             
C NUA   :  NUMBER OF ABSORBER AMOUNTS                                   
C NG1   :  ORDER OF THE GAUSSIAN QUADRATURE                             
C                                                                       
C RT1   :  ROOTS OF THE GAUSSIAN POLYNOMIAL                             
C WG1   :  WEIGHTS OF THE GAUSSIAN POLYNOMIAL                           
C AT, BT:  COEFFICIENTS OF THE TEMPERATURE DEPENDENCE                   
C CNTNU :  H2O CONTINUUM ABSORPTION COEFFICIENTS                        
C O1H,O2H,PIALF0,OCT: OZONE LONGWAVE ABSORPTION COEFFICIENTS            
C TREF, TSTAND      : REFERENCE TEMPERATURES                            
C XP    :  COEFFICIENTS OF POLYNOMIAL DEVELOPMENTS OF PLANCK FUNCTION   
C GA,GB :  PADE APPROXIMANTS OF H2O, CO2 TRANSMISSION FUNCTIONS         
C ETY,PTY :  PARAMETERS OF H2O CONTINUUM                                
C TINTP :  REFERENCE TEMPERATURES OF PRE-COMPUTED TRANSMISSIVITIES      
C CE    :  EMS COEFFS OF H2O TRANSMISSION FUNCTIONS                     
C                                                                       
      REAL  RT1(2),WG1(2),AT(12,3),BT(12,3),CNTNU(4),O1H                
     S ,OCT(4),TREF,XP(6,6),TSTAND,GA(11,24,3),GB(11,24,3)              
     S ,PTY(11,20),ETY(11,20),TINTP(11),TSTP,O2H,PIALF0                 
C                                                                       
      REAL  TB(40,31,2),ALP(40,31,2),BET(40,31,2),                      
     S ULG(40),PLG(31),AP(40,31,2),BP(40,31,2),TBO3(38,31),TO3B(38,31)  
     S,ULGO(38),AO3(38,31),BO3(38,31),O3A(38,31),O3B(38,31)             
C     ----------------------------------------------------------------- 
C*COMDECK YOMRDU                                                        
C     ----------------------------------------------------------------- 
C                                                                       
C*    CONTROL VARIABLES FOR RADIATION: INTERNAL SWITCH AND INDICES      
C                                                                       
C ZEELOG:  SECURITY THRESHOLD FOR ABSORBER AMOUNT IN LAPLACE TRANSFORM  
C ZEPSC :  SECURITY THRESHOLD FOR CLOUD COVER                           
C ZEPSCO:  SECURITY THRESHOLD FOR OZONE AMOUNT                          
C ZEPSCQ:  SECURITY THRESHOLD FOR WATER VAPOR                           
C ZEPSCT:  SECURITY THRESHOLD FOR SHORTWAVE OPTICAL THICKNESS           
C ZEPSCW:  SECURITY THRESHOLD FOR CLOUD LIQUID WATER PATH               
C                                                                       
      REAL          CDAY,CCO2,CH2O,DIFF,X10E                            
     S             ,ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW            
C     ----------------------------------------------------------------- 
C*COMDECK YOMSW                                                         
C     ----------------------------------------------------------------- 
C                                                                       
C*    COEFFICIENTS FOR THE SHORTWAVE RADIATION SUBROUTINES              
C                                                                       
C APAD  :  PADE APPROXIMANTS NUMERATOR                                  
C BPAD  :  PADE APPROXIMANTS DENOMINATOR                                
C D     :  TRANSMISSION LIMIT FOR INFINITE ABSORBER AMOUNT              
C CRAY  :  RAYLEIGH SCATTERING COEFFICIENTS                             
C SUN   :  SOLAR FRACTION IN SPECTRAL INTERVALS                         
C                                                                       
      REAL  APAD(2,3,7),BPAD(2,3,7),D(2,3),CRAY(2,6),SUN(2)             
C     ----------------------------------------------------------------- 
                                                                        
C                                                                       
C  -------------------------------------------------------------------- 
C                                                                       
      REAL   TTT(1,NLAT,NRLEV,12)                                       
C                                                                       
C        0.1  "RADINT POINTERS"                                         
C             -----------------                                         
C                                                                       
      REAL   TM1(NLP2,NLEV),                                            
     *       TSM1M(NLP2),                                               
     *       QM1(NLP2,NLEV),                                            
     *       SNM1M(NLP2),                                               
     *       ALBM(NLP2),                                                
     *       AVVELM(NLP2)                                               
      REAL   SLMM(NLP2),                                                
     *       APHM1(NLP2,NLEVP1),                                        
     *       APM1(NLP2,NLEV)                                            
C                                                                       
      LOGICAL LOLAND(NLP2)                                              
C                                                                       
C     VARIABLES FOR FULL GRID AND TRANSFORMS.                           
C                                                                       
      REAL   ZTI(NLP2,NRLEVP1),                                         
     *       ZCTI(NLP2,NRLEVP1),                                        
     *       ZCPH(NLP2,NRLEVP1),                                        
     *       ZWV(NLP2,NRLEV),                                           
     *       ZSW(NLP2,NRLEV),                                           
     *       ZALTE(NLP2),                                               
     *       ZCALTE(NLP2),                                              
     *       ZALSO(NLP2),                                               
     *       ZCALSO(NLP2)                                               
C                                                                       
      REAL   ZOZQ(NLP2),                                                
     #       ZOZH(NLP2)                                                 
C                                                                       
      REAL   ZTRSOL(NLP2,NLEVP1),                                       
     *       ZEMTER(NLP2,NLEVP1)                                        
C                                                                       
C     FINAL RESULTS ARE STORED ON:                                      
C                                                                       
      REAL   TRSOLM(NLP2,NLEVP1),                                       
     *       EMTERM(NLP2,NLEVP1)                                        
C                                                                       
      REAL   AMU0(NLP2),                                                
     *       RDAYL(NLP2)                                                
C                                                                       
C     VARIABLES WITHIN THE SPLITTED LATITUDE LOOP.                      
C                                                                       
      REAL   CLC(NRLST,NRLEV),                                          
     *       TI(NRLST,NRLEVP1),                                         
     *       DP(NRLST,NRLEV),                                           
     *       AADS(NRLST,NRLEV),                                         
     *       AEQ1(NRLST,NRLEV),                                         
     *       AEQ2(NRLST,NRLEV),                                         
     *       AEQ3(NRLST,NRLEV),                                         
     *       AEQ4(NRLST,NRLEV),                                         
     *       AEQ5(NRLST,NRLEV),                                         
     *       SSW(NRLST,NRLEV),                                          
     *       WV(NRLST,NRLEV),                                           
     *       QOF(NRLST,NRLEV),                                          
     *       QCF(NRLST,NRLEV),                                          
     *       CLWA(NRLST,NRLEV),                                         
     *       APRE(NRLST),                                               
     *       SMU0(NRLST),                                               
     *       ALTE(NRLST),                                               
     *       ALSO(NRLST)                                                
      REAL   ZDPO(NRLST),                                               
     *       ZQCFO(NRLST),                                              
     *       ZAEQSO(NRLST),                                             
     *       ZQOFO(NRLST),                                              
     *       ZDPN(NRLST),                                               
     *       ZQCFN(NRLST),                                              
     *       ZAEQSN(NRLST),                                             
     *       ZQOFN(NRLST)                                               
      REAL   FLT(NRLST,NRLEVP1),                                        
     *       FLS(NRLST,NRLEVP1)                                         
C                                                                       
C     DIAGNOSTIC VARIABLES.                                             
C                                                                       
      REAL   DIAG(NLEVP1,4),                                            
     *       DIA1(7),                                                   
     *       ZDIAG(NLEVP1,10),                                          
     *       ZDIA1(21),                                                 
     *       ZDIA(NLON),                                                
     *       ZDIAF(NLON,7),                                             
     *       ZDIAT(NLEVP1)                                              
C                                                                       
C                                                                       
C                                                                       
      REAL PGG(IRLST),                                                  
     #     PREF(IRLST),                                                 
     #     PRE1(IRLST),                                                 
     #     PRE2(IRLST),                                                 
     #     PRMUZ(IRLST),                                                
     #     PTO1(IRLST),                                                 
     #     PTR1(IRLST),                                                 
     #     PTR2(IRLST),                                                 
     #     PW(IRLST)                                                    
C                                                                       
C                                                                       
C  what about the 1-dimensional PALBS???                                
C                                                                       
      REAL     PALBSN  (IRLST)                                          
C                                                                       
C*       0.2   "LOCAL" ARRAYS                                           
C              --------------                                           
      REAL ZABSCU(IRLST ,NUA, 3*NRLEV+1),                               
     S     ZABSLY(IRLST ,NUA, 3*NRLEV+1) ,                              
     S     ZADJD (IRLST , IRLVP1)      ,                                
     S     ZADJU (IRLST , IRLVP1)       ,                               
     S     ZBGND (IRLST)                ,                               
     S     ZBINT (IRLST , IRLVP1)       ,                               
     S     ZBSUR (IRLST , NINT)         ,                               
     S     ZBSUIN(IRLST)                ,                               
     S     ZBTOP (IRLST , NINT)         ,                               
     S     PCLDLW(IRLST , IRLEV)        ,                               
     S     ZCLOUD(IRLST)                ,                               
     S     ZCNTRB(IRLST , IRLVP1,IRLVP1),                               
     S     ZCUMP (IRLST)                ,                               
     S     ZCUMPT(IRLST)                ,                               
     S     ZDBDT (IRLST , NINT , IRLEV) ,                               
     S     ZDBSL (IRLST , NINT , 2*NRLEV),                              
     S     ZDISTD(IRLST , IRLVP1)       ,                               
     S     ZDISTU(IRLST , IRLVP1)                                       
      REAL                                                              
     S     ZDNF  (IRLST , IRLVP1,IRLVP1),                               
     S     PDT0  (IRLST)                                                
      REAL                                                              
     S     ZDUC  (IRLST , 3*NRLEV+1)       ,                            
     S     ZEFFT (IRLST)                ,                               
     S     ZEMD  (IRLST , IRLEV), ZEMU  (IRLST , IRLEV),                
     S     ZEPSC2(IRLST),  ZEPSC3(IRLST),                               
     S     ZEPSH1(IRLST),  ZEPSH2(IRLST),                               
     S     ZEPSH3(IRLST),  ZEPSH4(IRLST),                               
     S     ZEPSH5(IRLST),  ZEPSH6(IRLST),                               
     S     ZEPHIO(IRLST),  ZFACTC(IRLST),                               
     S     ZZFD(IRLST), ZFDD(IRLST), ZFDN(IRLST , IRLVP1)               
      REAL                                                              
     S     PFLUX (IRLST , 2 , IRLVP1)   ,                               
     S     PFLUC (IRLST , 2 , IRLVP1)   ,                               
     S     ZFLWP (IRLST)                ,                               
     S     PCTS  (IRLST , IRLEV)        ,                               
     S     ZZFU   (IRLST)               ,                               
     S     ZFUP  (IRLST , IRLVP1)       ,                               
     S     ZGLAYD(IRLST)                ,                               
     S     ZGLAYU(IRLST)                ,                               
     S     ZOMEGA(IRLST , 2 , IRLEV)                                    
C                                                                       
      INTEGER                                                           
     S     ITX   (IRLST)                ,                               
     S     IXDIA (IRLST , IRLVP1)       ,                               
     S     IXT   (IRLST , IRLVP1,IRLVP1),                               
     S     IXTSU (IRLST , IRLVP1)       ,                               
     S     IXTTP (IRLST , IRLVP1)       ,                               
     S     IMX   (IRLST)                ,                               
     S     IMXP  (IRLST)                                                
C                                                                       
      REAL ZOZ   (IRLST , IRLEV)        ,                               
     S     PPMB  (IRLST , IRLVP1)       ,                               
     S     ZRES  (IRLST)                ,                               
     S     PB    (IRLST, 6 , IRLVP1)    ,                               
     S     ZSSIG (IRLST , 3*NRLEV+1)    ,                               
     S     PTAU  (IRLST , 2 , IRLEV)    ,                               
     S     PTAVE (IRLST , IRLEV)        ,                               
     S     ZTAVIC(IRLST)                ,                               
     S     ZTI1  (IRLST)                ,                               
     S     PTL   (IRLST , IRLVP1)       ,                               
     S     ZTT   (IRLST , NTRA)         ,                               
     S     ZTT1  (IRLST , NTRA)         ,                               
     S     ZTT2  (IRLST , NTRA)         ,                               
     S     ZUAER (IRLST , NINT)         ,                               
     S     ZUPF  (IRLST , IRLVP1,IRLVP1),                               
     S     ZUU   (IRLST , NUA)          ,                               
     S     ZXD   (IRLST)                                                
      REAL                                                              
     S     ZXN   (IRLST)                ,                               
     S     ZXOZ  (IRLST)                ,                               
     S     ZXWV  (IRLST)                ,                               
     S     ZZZ   (IRLST)                ,                               
     S     ZRES2 (IRLST)                ,                               
     S     ZTI2  (IRLST)                ,                               
     S     ZBLAY (IRLST,IRLEV)          ,                               
     S     ZBLEV (IRLST,IRLVP1)         ,                               
     S     ZCTS  (IRLST)                ,                               
     S     ZHFG  (IRLST)                                                
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C                                                                       
      INTEGER KT    (IRLST)                                             
      REAL    PUU   (IRLST , NUA  )     ,                               
     S        PTT   (IRLST , NTRA )     ,                               
     S        ZZ    (IRLST)                                             
C                                                                       
C     -------------------------------------------------------- ---------
C                                                                       
      REAL  PCLFR  (IRLST , IRLEV)       ,                              
     S      PT     (IRLST , IRLVP1)       ,                             
     S      PQOF   (IRLST , IRLEV)          ,                           
     S      PQLWP  (IRLST , IRLEV)          ,                           
     S      PEMIS  (IRLST)                                              
      REAL                                                              
     S      PFLT   (IRLST , IRLVP1)                                     
C                                                                       
      REAL  ZALBSU(IRLST , 2 )           ,                              
     S      ZCG   (IRLST , 2 , IRLEV)    ,                              
     S      ZCLDLW(IRLST , IRLEV)        ,                              
     S      ZCLDSW(IRLST , IRLEV)        ,                              
     S      ZCOOLR(IRLST , IRLEV)        ,                              
     S      ZDT0  (IRLST)                ,                              
     S      ZFLUX (IRLST , 2 , IRLVP1)   ,                              
     S      ZPMB  (IRLST , IRLVP1)       ,                              
     S      ZTAU  (IRLST , 2 , IRLEV)    ,                              
     S      ZTAVE (IRLST , IRLEV)        ,                              
     S      ZTL   (IRLST , IRLVP1)       ,                              
     S      ZFSDWN(IRLST , IRLVP1)       ,                              
     S      ZFSUP (IRLST , IRLVP1)       ,                              
     S      ZHEATR(IRLST , IRLEV )                                      
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C                                                                       
      REAL   PDP   (IRLST , IRLEV)     ,                                
     S       PWV   (IRLST , IRLEV)     ,                                
     S       PFLS  (IRLST , IRLVP1)    ,                                
     S       PPSOL (IRLST        )     ,                                
     S       PRMU0 (IRLST        )     ,                                
     S       PAER  (IRLST,IRLEV,5)     ,                                
     S       PALBS (IRLST , 2 )        ,                                
     S       PCG   (IRLST , 2 , IRLEV) ,                                
     S       PCLDSW(IRLST , IRLEV)     ,                                
     S       POMEGA(IRLST , 2 , IRLEV) ,                                
     S       POZ   (IRLST , IRLEV)                                      
C                                                                       
      REAL   PFDOWN(IRLST , IRLVP1)    ,                                
     S       PFUP  (IRLST , IRLVP1)                                     
C                                                                       
      REAL   ZAKI  (IRLST , 2 )        ,                                
     S       ZCGAZ (IRLST , IRLEV)     ,                                
     S       ZCNEB (IRLST , IRLEV)     ,                                
     S       ZC1I  (IRLST , IRLVP1)    ,                                
     S       ZDSC  (IRLST , IRLEV)     ,                                
     S       ZDSH  (IRLST , IRLEV)     ,                                
     S       ZDSIG (IRLST , IRLEV)     ,                                
     S       ZFACT (IRLST)             ,                                
     S       ZFACT1 (IRLST), ZFACT2 (IRLST),                            
     S       ZFACT3 (IRLST), ZFACT4 (IRLST),                            
     S       ZFACT5 (IRLST), ZFACT6 (IRLST),                            
     S       ZFD   (IRLST , IRLVP1)    ,                                
     S       ZFRAC (IRLST)             ,                                
     S       ZFU   (IRLST , IRLVP1)    ,                                
     S       ZG    (IRLST)             ,                                
     S       ZGG   (IRLST)             ,                                
     S       ZPIZAZ(IRLST , IRLEV)     ,                                
     S       ZRAYL (IRLST),  ZRAY1 (IRLST , IRLVP1)                     
      REAL                                                              
     S       ZRAY2 (IRLST , IRLVP1)    ,                                
     S       ZREF  (IRLST)             ,                                
     S       ZREFZ (IRLST , 2 , IRLVP1),                                
     S       ZRE1  (IRLST)             ,                                
     S       ZRE2  (IRLST)             ,                                
     S       ZRJ   (IRLST , 6 , IRLVP1),                                
     S       ZRK   (IRLST , 6 , IRLVP1),                                
     S       ZRL   (IRLST , 8)         ,                                
     S       ZRMU  (IRLST)             ,                                
     S       ZRMUE (IRLST , IRLVP1)    ,                                
     S       ZRMUZ (IRLST)                                              
      REAL                                                              
     S       ZRNEB (IRLST)             ,                                
     S       ZRUEF (IRLST , 8)         ,                                
     S       ZR1   (IRLST)             ,                                
     S       ZR2   (IRLST)             ,                                
     S       ZR21  (IRLST)             ,                                
     S       ZR22  (IRLST)             ,                                
     S       ZR23  (IRLST)             ,                                
     S       ZS    (IRLST)             ,                                
     S       ZSEC  (IRLST)             ,                                
     S       ZSS1  (IRLST)             ,                                
     S       ZTAUAZ(IRLST , IRLEV)     ,                                
     S       ZTOT  (IRLST , IRLVP1)    ,                                
     S       ZTO1  (IRLST)             ,                                
     S       ZTR   (IRLST , 2 , IRLVP1),                                
     S       ZTRA1 (IRLST , IRLVP1)    ,                                
     S       ZTRA2 (IRLST , IRLVP1)                                     
      REAL                                                              
     S       ZTR1  (IRLST)             ,                                
     S       ZTR2  (IRLST)             ,                                
     S       ZUD   (IRLST , 3 , IRLVP1),                                
     S       ZUM   (IRLST , IRLVP1)    ,                                
     S       ZU1D  (IRLST)             ,                                
     S       ZU2D  (IRLST)             ,                                
     S       ZW    (IRLST)             ,                                
     S       ZN175 (IRLST)             ,                                
     S       ZN190 (IRLST)             ,                                
     S       ZO175 (IRLST)             ,                                
     S       ZO190 (IRLST)             ,                                
     S       ZP75  (IRLST)             ,                                
     S       ZP90  (IRLST)             ,                                
     S       ZSIGN (IRLST)             ,                                
     S       ZSIGO (IRLST)                                              
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
      REAL   PTR    (IRLST)     ,                                       
     S       PU     (IRLST)                                             
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
C   CLOUD-POINTERS                                                      
C   --------------                                                      
C                                                                       
      REAL   ZRH(NLP2,NLEV),                                            
     *       ZCH(NLP2),                                                 
     *       ZCM(NLP2),                                                 
     *       ZCL(NLP2),                                                 
     *       ZCT(NLP2),                                                 
     *       ZCC(NLP2)                                                  
      INTEGER NTOPCM(NLP2),                                             
     *        NBASECM(NLP2)                                             
      REAL   ARPRCM(NLP2)                                               
      INTEGER ITOPH(NLP2),                                              
     *        ITOPM(NLP2),                                              
     *        ITOPL(NLP2)                                               
      LOGICAL LOC(NLP2)                                                 
      REAL   ZQS(NLP2,NLEV),                                            
     *       ZCLC(NLP2,NLEV),                                           
     *       ZQLWP(NLP2,NLEV)                                           
C                                                                       
      REAL   VERVEL(NLP2,NLEV),                                         
     *       ZTHETA(NLP2,NLEV),                                         
     *       ZETA(NLP2,NLEV),                                           
     *       ZDTHMIN(NLP2)                                              
      INTEGER INVB(NLP2)                                                
      REAL   ZCTLWP(NLP2),                                              
     *       ZQLOC(NLP2),                                               
     *       ZQMEC(NLP2),                                               
     *       ZQHIC(NLP2)                                                
C                                                                       
C  definitions of "radheat"                                             
C                                                                       
      REAL   ZFSOT(NLON),                                               
     #       ZFTET(NLON),                                               
     #       ZFLT (NLON),                                               
     #       ZFSO (NLON),                                               
     #       ZFTE (NLON),                                               
     #       ZFLB (NLON),                                               
     #       ZDTDT(NLON),                                               
     #       HEATO(NLON,NLEV),                                          
     #       HEATE(NLON,NLEV),                                          
     #       HEATR(NLON,NLEV),                                          
     #       ZI0  (NLON)                                                
C                                                                       
C PIERS 13-8-97                                                         
C                                                                       
         REAL PIERSLWP                                                  
C                                                                       
C Piece of code needed to replace the Nikos radiation scheme with       
C the Morcrette radiation scheme in the igcm3.1.                        
C See morcrette3_1_1.readme for details.                                
C                                                                       
C                                                                       
c      program lh                                                       
c                                                                       
c      parameter(NL=23)                                                 
c      real P(NL),t(NL),h2o(NL),o3(NL),htlw(NL),htsw(NL)                
c      real swalb,doy                                                   
c       REAL CF(4,2)     !cloud fractions,lwps                          
c       INTEGER IC(4,2)   ! cloud positions                             
c       REAL RFLUXES(2,2,2)                                             
c      open(11,file='mls23.dat')                                        
c      read(11,*) NLEV                                                  
c      DOY=80                                                           
c      swalb=0.07                                                       
c      alat=40.0                                                        
c       doy=float(i)                                                    
c      do i=nlev,1,-1                                                   
c         read(11,*) P(i),t(i),h2o(i),adum,o3(i)                        
c         print *,i,p(i),t(i),h2o(i),o3(i)                              
c      enddo                                                            
c                                                                       
c       ic(1,1)=3                                                       
c      ic(1,2)=17                                                       
c       ic(3,1)=6                                                       
c       ic(2,2)=6                                                       
c       cf(1,1)=0.3                                                     
c       cf(1,2)=45.0                                                    
c        cf(3,1)=0.8                                                    
c       cf(4,2)=20.0                                                    
c      alon=0.0                                                         
c      call morcigcm(p,t,h2o,o3,alat,htlw,htsw,doy,cf,ic,               
c     $     rfluxes,swalb,alon)                                         
c      enddo                                                            
c      do i=nl,1,-1                                                     
c         print '(i6,4F14.4)',i,p(i),t(i),htsw(i),htlw(i)               
c      enddo                                                            
c      print *,RFLUXES(1,1,1), RFLUXES(1,1,2), RFLUXES(1,2,1),          
c     $     RFLUXES(1,2,2)                                              
c      print *,RFLUXES(2,1,1), RFLUXES(2,1,2), RFLUXES(2,2,1),          
c     $     RFLUXES(2,2,2)                                              
c                                                                       
c                                                                       
c      end                                                              
c        INCLUDE "zparb.upd"                                            
C                                                                       
C Yet again we have to change the hard wired memory allocation for      
C the radiation blocks.                                                 
C                                                                       
C iii) Totally replace SULW                                             
C                                                                       
C      ---------------------------------------------------------        
C                                                                       
C*       1.    SET VALUES.                                              
C              -----------                                              
C                                                                       
C      DATA NINT, NIPD, NIPD2, NTR, NTRA, NUA, NG1, NG1P1               
C     S  /   6  ,    8,    16,  11,   19,  18,   2,     3  /            
C                                                                       
C-- REFERENCE TEMPERATURES FOR THE PLANCK FUNCTIONS ----- TSTAND        
C          TEMPERATURE DEPENDENCE OF THE ABSORPTION ----- TREF          
      DATA TSTAND,TREF / 250.0, 250.0 /                                 
C                                                                       
C-- ROOTS AND WEIGHTS FOR THE 2-POINT GAUSSIAN QUADRATURE               
      DATA (RT1(IG1),IG1=1,2) / -0.577350269, +0.577350269 /            
      DATA (WG1(IG1),IG1=1,2) /  1.0        ,  1.0         /            
C                                                                       
C-- TEMPERATURE STEP AND MAXIMUM INDEX FOR THE TRANSMISSIVITIES         
      DATA TSTP/ 12.5 /,  MXIXT / 10 /                                  
C                                                                       
C-- REFERENCE TEMPERATURE FOR THE TRANSMISSIVITIES                      
      DATA TINTP / 187.5, 200., 212.5, 225., 237.5, 250.,               
     S             262.5, 275., 287.5, 300., 312.5 /                    
C                                                                       
C-- COEFFICIENTS OF THE POLYNOMIALS GIVING THE PLANCK FUNCTIONS         
C                                                                       
c      DATA (( XP(  J,K),J=1,6),       K=1,6) /                         
c     S 0.46430621E+02, 0.12928299E+03, 0.20732648E+03,                 
c     S 0.31398411E+03, 0.18373177E+03,-0.11412303E+03,                 
c     S 0.73604774E+02, 0.27887914E+03, 0.27076947E+03,                 
c     S-0.57322111E+02,-0.64742459E+02, 0.87238280E+02,                 
c     S 0.37050866E+02, 0.20498759E+03, 0.37558029E+03,                 
c     S 0.17401171E+03,-0.13350302E+03,-0.37651795E+02,                 
c     S 0.14930141E+02, 0.89161160E+02, 0.17793062E+03,                 
c     S 0.93433860E+02,-0.70646020E+02,-0.26373150E+02,                 
c     S 0.40386780E+02, 0.10855270E+03, 0.50755010E+02,                 
c     S-0.31496190E+02, 0.12791300E+02, 0.18017770E+01,                 
c     S 0.90811926E+01, 0.75073923E+02, 0.24654438E+03,                 
c     S 0.39332612E+03, 0.29385281E+03, 0.89107921E+02 /                
c                                                                       
C                                                                       
C-- COEFFICIENTS OF THE POLYNOMIALS GIVING THE PLANCK FUNCTIONS         
C                                                                       
c      DATA (( XP(  J,K),J=1,6),       K=1,6) /                         
c     * 0.46440912E+02, 0.12928373E+03, 0.20721063E+03,                 
c     * 0.31465410E+03, 0.18532753E+03,-0.12081117E+03,                 
c     * 0.73615150E+02, 0.27892884E+03, 0.27075955E+03,                 
c     *-0.57109865E+02,-0.63336785E+02, 0.85537973E+02,                 
c     * 0.37048742E+02, 0.20503052E+03, 0.37594173E+03,                 
c     * 0.17258251E+03,-0.13796046E+03,-0.23295511E+02,                 
c     * 0.14928616E+02, 0.89179092E+02, 0.17810427E+03,                 
c     * 0.92742845E+02,-0.72842724E+02,-0.19361244E+02,                 
c     * 0.40395378E+02, 0.10858185E+03, 0.50773924E+02,                 
c     *-0.31496423E+02, 0.12828971E+02, 0.16837921E+01,                 
c     * 0.90587573E+01, 0.74736166E+02, 0.24423578E+03,                 
c     * 0.38511585E+03, 0.26543217E+03, 0.26930073E+02 /                
C                                                                       
C-- COEFFICIENTS OF THE POLYNOMIALS GIVING THE PLANCK FUNCTIONS         
C                                                                       
      DATA (( XP(  J,K),J=1,6),       K=1,6) /                          
     * 0.46448212E+02, 0.12950083E+03, 0.20616074E+03,                  
     * 0.30440182E+03, 0.20088503E+03,-0.19180727E+02,                  
     * 0.73614893E+02, 0.27896120E+03, 0.27077036E+03,                  
     *-0.58840871E+02,-0.63235846E+02, 0.10332900E+03,                  
     * 0.37042681E+02, 0.20484389E+03, 0.37681519E+03,                  
     * 0.18143404E+03,-0.15093530E+03,-0.11119628E+03,                  
     * 0.14925225E+02, 0.89067009E+02, 0.17859760E+03,                  
     * 0.98102682E+02,-0.80224280E+02,-0.72734490E+02,                  
     * 0.40397268E+02, 0.10862521E+03, 0.50512477E+02,                  
     *-0.33484892E+02, 0.16614982E+02, 0.21239268E+02,                  
     * 0.90605881E+01, 0.74792389E+02, 0.24397449E+03,                  
     * 0.38243354E+03, 0.26928532E+03, 0.53743993E+02 /                 
C                                                                       
C                                                                       
C-- COEFFICIENTS OF THE POLYNOMIALS GIVING THE TEMPERATURE DEPENDENCE OF
C     THE ABSORPTION                                                    
C--------H2O                                                            
      DATA (AT(1,IR),IR=1,3)/                                           
     S  2.34995778846E-03, -2.76713384294E-04,  1.16480338097E-06/      
      DATA (BT(1,IR),IR=1,3)/                                           
     S -1.58165952771E-05, -2.39167473702E-05,  1.80148335147E-07/      
      DATA (AT(2,IR),IR=1,3)/                                           
     S  2.52595953359E-02, -6.72079319635E-04,  5.74103783815E-06/      
      DATA (BT(2,IR),IR=1,3)/                                           
     S -8.26196996910E-05, -1.05514645370E-05,  7.02984234993E-08/      
      DATA (AT(3,IR),IR=1,3)/                                           
     S  3.55525229363E-02, -7.94992134865E-04,  6.64848193374E-06/      
      DATA (BT(3,IR),IR=1,3)/                                           
     S -9.71124908025E-05, -2.95218285256E-06,  2.23374952695E-08/      
      DATA (AT(4,IR),IR=1,3)/                                           
     S  5.47044522259E-02, -1.19105877667E-03,  9.74676865100E-06/      
      DATA (BT(4,IR),IR=1,3)/                                           
     S -2.10153035427E-04, -6.93815560155E-06,  5.50381928956E-08/      
      DATA (AT(5,IR),IR=1,3) /                                          
     S  1.88063066950E-02, -1.11250846983E-04,  1.66977895508E-06/      
      DATA (BT(5,IR),IR=1,3) /                                          
     S -5.72683436311E-05, -2.48448308453E-05,  1.88450887138E-07/      
      DATA (AT(6,IR),IR=1,3) /                                          
     S  1.29526952131E-02, -3.81424455035E-04,  3.07118138204E-06/      
      DATA (BT(6,IR),IR=1,3) /                                          
     S -4.20789327775E-05, -1.25055291318E-05,  8.58839668934E-08/      
C--1250-1450                                                            
      DATA (AT(7,IR),IR=1,3)/                                           
     S  1.31749957617E-02, -5.90100320028E-05,  6.25471011857E-07/      
      DATA (BT(7,IR),IR=1,3)/                                           
     S -4.30693119882E-05, -3.14712185336E-07,  3.69886377880E-09/      
C--800-970                                                              
      DATA (AT(8,IR),IR=1,3)/                                           
     S  4.88895361215E-02, -1.10191335476E-03,  9.12442493536E-06/      
      DATA (BT(8,IR),IR=1,3)/                                           
     S -1.07667174073E-04,  2.47171336712E-06, -1.96942567968E-08/      
C--650-800                                                              
      DATA (AT(9,IR),IR=1,3) /                                          
     S  3.53337502532E-02, -9.07920050431E-04,  7.57030599278E-06/      
      DATA (BT(9,IR),IR=1,3) /                                          
     S -1.09062411235E-04, -9.99616246128E-07,  7.90293124935E-09/      
C-------CO2                                                             
      DATA (AT(10,IR),IR=1,3) /                                         
     S 0.250073E-03,0.455875E-03,0.109242E-03 /                         
      DATA (BT(10,IR),IR=1,3) /                                         
     S 0.199846E-05,-.216313E-05,0.175991E-06 /                         
      DATA (AT(11,IR),IR=1,3) /                                         
     S 0.307423E-01,0.110879E-02,-.322172E-03 /                         
      DATA (BT(11,IR),IR=1,3) /                                         
     S-0.108482E-03,0.258096E-05,-.814575E-06 /                         
C                                                                       
C--  COEFFICIENTS FOR OZONE ABSORPTION                                  
C                                                                       
      DATA O1H,O2H,PIALF0,(OCT(IT),IT=1,4) /                            
     S 2230., 100., 2., -.326E-03, -.102E-05, .137E-02, -.535E-05 /     
C                                                                       
C--  COEFFICIENTS FOR H2O E-TYPE CONTINUUM ABSORPTION                   
C                                                                       
C      DATA CNTNU /   70.0, 26.77, 8.45, 5.78  /                        
C                                                                       
C--------------------------------------------------------               
C  PADE APPROXIMANTS FOR THE TRANSMISSIVITIES                           
C  ==========================================                           
C-- WATER VAPOR -- INT.1 -- 0- 350 + 1450- 1880 CM-1 --------           
C                                                                       
C----- INTERVAL = 1 ----- T =    187.50                                 
C                                                                       
      DATA (GA( 1, 1,JC),JC=1,3) /                                      
     S 0.25000224E-02, 0.18701843E-02, 0.00000000E+00/                  
      DATA (GB( 1, 1,JC),JC=1,3) /                                      
     S 0.25000224E-02, 0.61593968E-01, 0.10000000E+01/                  
      DATA (GA( 1, 2,JC),JC=1,3) /                                      
     S 0.34131105E-02, 0.55153039E-03, 0.00000000E+00/                  
      DATA (GB( 1, 2,JC),JC=1,3) /                                      
     S 0.34131105E-02, 0.73127832E-01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 1 ----- T =    200.00                                 
C                                                                       
      DATA (GA( 2, 1,JC),JC=1,3) /                                      
     S 0.26344616E-02, 0.16894582E-02, 0.00000000E+00/                  
      DATA (GB( 2, 1,JC),JC=1,3) /                                      
     S 0.26344616E-02, 0.63633167E-01, 0.10000000E+01/                  
      DATA (GA( 2, 2,JC),JC=1,3) /                                      
     S 0.36120928E-02, 0.44720171E-03, 0.00000000E+00/                  
      DATA (GB( 2, 2,JC),JC=1,3) /                                      
     S 0.36120928E-02, 0.75062171E-01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 1 ----- T =    212.50                                 
C                                                                       
      DATA (GA( 3, 1,JC),JC=1,3) /                                      
     S 0.27743234E-02, 0.15245487E-02, 0.00000000E+00/                  
      DATA (GB( 3, 1,JC),JC=1,3) /                                      
     S 0.27743234E-02, 0.65651733E-01, 0.10000000E+01/                  
      DATA (GA( 3, 2,JC),JC=1,3) /                                      
     S 0.38453121E-02, 0.42382776E-03, 0.00000000E+00/                  
      DATA (GB( 3, 2,JC),JC=1,3) /                                      
     S 0.38453121E-02, 0.77088315E-01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 1 ----- T =    225.00                                 
C                                                                       
      DATA (GA( 4, 1,JC),JC=1,3) /                                      
     S 0.29265376E-02, 0.13719090E-02, 0.00000000E+00/                  
      DATA (GB( 4, 1,JC),JC=1,3) /                                      
     S 0.29265376E-02, 0.67739270E-01, 0.10000000E+01/                  
      DATA (GA( 4, 2,JC),JC=1,3) /                                      
     S 0.41211460E-02, 0.42401401E-03, 0.00000000E+00/                  
      DATA (GB( 4, 2,JC),JC=1,3) /                                      
     S 0.41211460E-02, 0.79241307E-01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 1 ----- T =    237.50                                 
C                                                                       
      DATA (GA( 5, 1,JC),JC=1,3) /                                      
     S 0.30979053E-02, 0.12223308E-02, 0.00000000E+00/                  
      DATA (GB( 5, 1,JC),JC=1,3) /                                      
     S 0.30979053E-02, 0.69968340E-01, 0.10000000E+01/                  
      DATA (GA( 5, 2,JC),JC=1,3) /                                      
     S 0.43966504E-02, 0.67198946E-03, 0.00000000E+00/                  
      DATA (GB( 5, 2,JC),JC=1,3) /                                      
     S 0.43966504E-02, 0.80947310E-01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 1 ----- T =    250.00                                 
C                                                                       
      DATA (GA( 6, 1,JC),JC=1,3) /                                      
     S 0.32813184E-02, 0.11573810E-02, 0.00000000E+00/                  
      DATA (GB( 6, 1,JC),JC=1,3) /                                      
     S 0.32813184E-02, 0.72183750E-01, 0.10000000E+01/                  
      DATA (GA( 6, 2,JC),JC=1,3) /                                      
     S 0.46792452E-02, 0.99691013E-03, 0.00000000E+00/                  
      DATA (GB( 6, 2,JC),JC=1,3) /                                      
     S 0.46792452E-02, 0.82404099E-01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 1 ----- T =    262.50                                 
C                                                                       
      DATA (GA( 7, 1,JC),JC=1,3) /                                      
     S 0.34869261E-02, 0.11110812E-02, 0.00000000E+00/                  
      DATA (GB( 7, 1,JC),JC=1,3) /                                      
     S 0.34869261E-02, 0.74507149E-01, 0.10000000E+01/                  
      DATA (GA( 7, 2,JC),JC=1,3) /                                      
     S 0.49313571E-02, 0.15144806E-02, 0.00000000E+00/                  
      DATA (GB( 7, 2,JC),JC=1,3) /                                      
     S 0.49313571E-02, 0.83301687E-01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 1 ----- T =    275.00                                 
C                                                                       
      DATA (GA( 8, 1,JC),JC=1,3) /                                      
     S 0.37169013E-02, 0.10586459E-02, 0.00000000E+00/                  
      DATA (GB( 8, 1,JC),JC=1,3) /                                      
     S 0.37169013E-02, 0.76930655E-01, 0.10000000E+01/                  
      DATA (GA( 8, 2,JC),JC=1,3) /                                      
     S 0.51606177E-02, 0.20799953E-02, 0.00000000E+00/                  
      DATA (GB( 8, 2,JC),JC=1,3) /                                      
     S 0.51606177E-02, 0.83904484E-01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 1 ----- T =    287.50                                 
C                                                                       
      DATA (GA( 9, 1,JC),JC=1,3) /                                      
     S 0.39482491E-02, 0.11280761E-02, 0.00000000E+00/                  
      DATA (GB( 9, 1,JC),JC=1,3) /                                      
     S 0.39482491E-02, 0.79096624E-01, 0.10000000E+01/                  
      DATA (GA( 9, 2,JC),JC=1,3) /                                      
     S 0.53608496E-02, 0.26755638E-02, 0.00000000E+00/                  
      DATA (GB( 9, 2,JC),JC=1,3) /                                      
     S 0.53608496E-02, 0.84271466E-01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 1 ----- T =    300.00                                 
C                                                                       
      DATA (GA(10, 1,JC),JC=1,3) /                                      
     S 0.41849825E-02, 0.12500965E-02, 0.00000000E+00/                  
      DATA (GB(10, 1,JC),JC=1,3) /                                      
     S 0.41849825E-02, 0.81087000E-01, 0.10000000E+01/                  
      DATA (GA(10, 2,JC),JC=1,3) /                                      
     S 0.55372909E-02, 0.32512851E-02, 0.00000000E+00/                  
      DATA (GB(10, 2,JC),JC=1,3) /                                      
     S 0.55372909E-02, 0.84546756E-01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 1 ----- T =    312.50                                 
C                                                                       
      DATA (GA(11, 1,JC),JC=1,3) /                                      
     S 0.44204714E-02, 0.14195697E-02, 0.00000000E+00/                  
      DATA (GB(11, 1,JC),JC=1,3) /                                      
     S 0.44204714E-02, 0.82840020E-01, 0.10000000E+01/                  
      DATA (GA(11, 2,JC),JC=1,3) /                                      
     S 0.56935299E-02, 0.38054592E-02, 0.00000000E+00/                  
      DATA (GB(11, 2,JC),JC=1,3) /                                      
     S 0.56935299E-02, 0.84847060E-01, 0.10000000E+01/                  
C                                                                       
C-- WATER VAPOR --- 500 - 800 CM-1                                      
C                                                                       
C                                                                       
C----- INTERVAL = 2 ----- T =    187.50                                 
C                                                                       
      DATA (GA( 1, 3,JC),JC=1,3) /                                      
     S 0.15053751E+01, 0.45361026E+00, 0.00000000E+00/                  
      DATA (GB( 1, 3,JC),JC=1,3) /                                      
     S 0.15053751E+01, 0.15789584E+01, 0.10000000E+01/                  
      DATA (GA( 1, 4,JC),JC=1,3) /                                      
     S 0.18920793E+01, 0.53457554E+00, 0.00000000E+00/                  
      DATA (GB( 1, 4,JC),JC=1,3) /                                      
     S 0.18920793E+01, 0.18418304E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =    200.00                                 
C                                                                       
      DATA (GA( 2, 3,JC),JC=1,3) /                                      
     S 0.15977537E+01, 0.48084314E+00, 0.00000000E+00/                  
      DATA (GB( 2, 3,JC),JC=1,3) /                                      
     S 0.15977537E+01, 0.16476468E+01, 0.10000000E+01/                  
      DATA (GA( 2, 4,JC),JC=1,3) /                                      
     S 0.20435568E+01, 0.55893717E+00, 0.00000000E+00/                  
      DATA (GB( 2, 4,JC),JC=1,3) /                                      
     S 0.20435568E+01, 0.19386656E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =    212.50                                 
C                                                                       
      DATA (GA( 3, 3,JC),JC=1,3) /                                      
     S 0.16913237E+01, 0.50454053E+00, 0.00000000E+00/                  
      DATA (GB( 3, 3,JC),JC=1,3) /                                      
     S 0.16913237E+01, 0.17144170E+01, 0.10000000E+01/                  
      DATA (GA( 3, 4,JC),JC=1,3) /                                      
     S 0.21978186E+01, 0.57905459E+00, 0.00000000E+00/                  
      DATA (GB( 3, 4,JC),JC=1,3) /                                      
     S 0.21978186E+01, 0.20334044E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =    225.00                                 
C                                                                       
      DATA (GA( 4, 3,JC),JC=1,3) /                                      
     S 0.17856476E+01, 0.52514194E+00, 0.00000000E+00/                  
      DATA (GB( 4, 3,JC),JC=1,3) /                                      
     S 0.17856476E+01, 0.17792823E+01, 0.10000000E+01/                  
      DATA (GA( 4, 4,JC),JC=1,3) /                                      
     S 0.23533355E+01, 0.59558005E+00, 0.00000000E+00/                  
      DATA (GB( 4, 4,JC),JC=1,3) /                                      
     S 0.23533355E+01, 0.21256517E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =    237.50                                 
C                                                                       
      DATA (GA( 5, 3,JC),JC=1,3) /                                      
     S 0.18802811E+01, 0.54304459E+00, 0.00000000E+00/                  
      DATA (GB( 5, 3,JC),JC=1,3) /                                      
     S 0.18802811E+01, 0.18422412E+01, 0.10000000E+01/                  
      DATA (GA( 5, 4,JC),JC=1,3) /                                      
     S 0.25086500E+01, 0.60908958E+00, 0.00000000E+00/                  
      DATA (GB( 5, 4,JC),JC=1,3) /                                      
     S 0.25086500E+01, 0.22150389E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =    250.00                                 
C                                                                       
      DATA (GA( 6, 3,JC),JC=1,3) /                                      
     S 0.19747954E+01, 0.55860186E+00, 0.00000000E+00/                  
      DATA (GB( 6, 3,JC),JC=1,3) /                                      
     S 0.19747954E+01, 0.19032856E+01, 0.10000000E+01/                  
      DATA (GA( 6, 4,JC),JC=1,3) /                                      
     S 0.26624407E+01, 0.62008317E+00, 0.00000000E+00/                  
      DATA (GB( 6, 4,JC),JC=1,3) /                                      
     S 0.26624407E+01, 0.23012482E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =    262.50                                 
C                                                                       
      DATA (GA( 7, 3,JC),JC=1,3) /                                      
     S 0.20687911E+01, 0.57212480E+00, 0.00000000E+00/                  
      DATA (GB( 7, 3,JC),JC=1,3) /                                      
     S 0.20687911E+01, 0.19624068E+01, 0.10000000E+01/                  
      DATA (GA( 7, 4,JC),JC=1,3) /                                      
     S 0.28135635E+01, 0.62898892E+00, 0.00000000E+00/                  
      DATA (GB( 7, 4,JC),JC=1,3) /                                      
     S 0.28135635E+01, 0.23840281E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =    275.00                                 
C                                                                       
      DATA (GA( 8, 3,JC),JC=1,3) /                                      
     S 0.21619083E+01, 0.58388519E+00, 0.00000000E+00/                  
      DATA (GB( 8, 3,JC),JC=1,3) /                                      
     S 0.21619083E+01, 0.20195993E+01, 0.10000000E+01/                  
      DATA (GA( 8, 4,JC),JC=1,3) /                                      
     S 0.29610693E+01, 0.63616954E+00, 0.00000000E+00/                  
      DATA (GB( 8, 4,JC),JC=1,3) /                                      
     S 0.29610693E+01, 0.24631984E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =    287.50                                 
C                                                                       
      DATA (GA( 9, 3,JC),JC=1,3) /                                      
     S 0.22538305E+01, 0.59411934E+00, 0.00000000E+00/                  
      DATA (GB( 9, 3,JC),JC=1,3) /                                      
     S 0.22538305E+01, 0.20748638E+01, 0.10000000E+01/                  
      DATA (GA( 9, 4,JC),JC=1,3) /                                      
     S 0.31042087E+01, 0.64192944E+00, 0.00000000E+00/                  
      DATA (GB( 9, 4,JC),JC=1,3) /                                      
     S 0.31042087E+01, 0.25386513E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =    300.00                                 
C                                                                       
      DATA (GA(10, 3,JC),JC=1,3) /                                      
     S 0.23442867E+01, 0.60303198E+00, 0.00000000E+00/                  
      DATA (GB(10, 3,JC),JC=1,3) /                                      
     S 0.23442867E+01, 0.21282086E+01, 0.10000000E+01/                  
      DATA (GA(10, 4,JC),JC=1,3) /                                      
     S 0.32424163E+01, 0.64652280E+00, 0.00000000E+00/                  
      DATA (GB(10, 4,JC),JC=1,3) /                                      
     S 0.32424163E+01, 0.26103424E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =    312.50                                 
C                                                                       
      DATA (GA(11, 3,JC),JC=1,3) /                                      
     S 0.24330507E+01, 0.61080006E+00, 0.00000000E+00/                  
      DATA (GB(11, 3,JC),JC=1,3) /                                      
     S 0.24330507E+01, 0.21796505E+01, 0.10000000E+01/                  
      DATA (GA(11, 4,JC),JC=1,3) /                                      
     S 0.33752974E+01, 0.65016031E+00, 0.00000000E+00/                  
      DATA (GB(11, 4,JC),JC=1,3) /                                      
     S 0.33752974E+01, 0.26782849E+01, 0.10000000E+01/                  
C                                                                       
C                                                                       
C- WATER VAPOR - INT. 3 -- 800-970 + 1110-1250 CM-1                     
C                                                                       
C                                                                       
C----- INTERVAL = 3 ----- T =    187.50                                 
C                                                                       
      DATA (GA( 1, 7,JC),JC=1,3) /                                      
     S 0.14334746E+02, 0.11649992E+02, 0.00000000E+00/                  
      DATA (GB( 1, 7,JC),JC=1,3) /                                      
     S 0.14334746E+02, 0.12059903E+02, 0.10000000E+01/                  
      DATA (GA( 1, 8,JC),JC=1,3) /                                      
     S 0.11596027E+02, 0.11051587E+02, 0.00000000E+00/                  
      DATA (GB( 1, 8,JC),JC=1,3) /                                      
     S 0.11596027E+02, 0.11390898E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 3 ----- T =    200.00                                 
C                                                                       
      DATA (GA( 2, 7,JC),JC=1,3) /                                      
     S 0.12924521E+02, 0.11365906E+02, 0.00000000E+00/                  
      DATA (GB( 2, 7,JC),JC=1,3) /                                      
     S 0.12924521E+02, 0.11739611E+02, 0.10000000E+01/                  
      DATA (GA( 2, 8,JC),JC=1,3) /                                      
     S 0.10445365E+02, 0.10682810E+02, 0.00000000E+00/                  
      DATA (GB( 2, 8,JC),JC=1,3) /                                      
     S 0.10445365E+02, 0.10993927E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 3 ----- T =    212.50                                 
C                                                                       
      DATA (GA( 3, 7,JC),JC=1,3) /                                      
     S 0.11774060E+02, 0.11080218E+02, 0.00000000E+00/                  
      DATA (GB( 3, 7,JC),JC=1,3) /                                      
     S 0.11774060E+02, 0.11424740E+02, 0.10000000E+01/                  
      DATA (GA( 3, 8,JC),JC=1,3) /                                      
     S 0.95571840E+01, 0.10328058E+02, 0.00000000E+00/                  
      DATA (GB( 3, 8,JC),JC=1,3) /                                      
     S 0.95571840E+01, 0.10618404E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 3 ----- T =    225.00                                 
C                                                                       
      DATA (GA( 4, 7,JC),JC=1,3) /                                      
     S 0.10839967E+02, 0.10800025E+02, 0.00000000E+00/                  
      DATA (GB( 4, 7,JC),JC=1,3) /                                      
     S 0.10839967E+02, 0.11121318E+02, 0.10000000E+01/                  
      DATA (GA( 4, 8,JC),JC=1,3) /                                      
     S 0.88719975E+01, 0.99930444E+01, 0.00000000E+00/                  
      DATA (GB( 4, 8,JC),JC=1,3) /                                      
     S 0.88719975E+01, 0.10268470E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 3 ----- T =    237.50                                 
C                                                                       
      DATA (GA( 5, 7,JC),JC=1,3) /                                      
     S 0.10082160E+02, 0.10529965E+02, 0.00000000E+00/                  
      DATA (GB( 5, 7,JC),JC=1,3) /                                      
     S 0.10082160E+02, 0.10832955E+02, 0.10000000E+01/                  
      DATA (GA( 5, 8,JC),JC=1,3) /                                      
     S 0.83425443E+01, 0.96805504E+01, 0.00000000E+00/                  
      DATA (GB( 5, 8,JC),JC=1,3) /                                      
     S 0.83425443E+01, 0.99456138E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 3 ----- T =    250.00                                 
C                                                                       
      DATA (GA( 6, 7,JC),JC=1,3) /                                      
     S 0.94663507E+01, 0.10272823E+02, 0.00000000E+00/                  
      DATA (GB( 6, 7,JC),JC=1,3) /                                      
     S 0.94663507E+01, 0.10561525E+02, 0.10000000E+01/                  
      DATA (GA( 6, 8,JC),JC=1,3) /                                      
     S 0.79324712E+01, 0.93914198E+01, 0.00000000E+00/                  
      DATA (GB( 6, 8,JC),JC=1,3) /                                      
     S 0.79324712E+01, 0.96496576E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 3 ----- T =    262.50                                 
C                                                                       
      DATA (GA( 7, 7,JC),JC=1,3) /                                      
     S 0.89643981E+01, 0.10030051E+02, 0.00000000E+00/                  
      DATA (GB( 7, 7,JC),JC=1,3) /                                      
     S 0.89643981E+01, 0.10307714E+02, 0.10000000E+01/                  
      DATA (GA( 7, 8,JC),JC=1,3) /                                      
     S 0.76141974E+01, 0.91252916E+01, 0.00000000E+00/                  
      DATA (GB( 7, 8,JC),JC=1,3) /                                      
     S 0.76141974E+01, 0.93794463E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 3 ----- T =    275.00                                 
C                                                                       
      DATA (GA( 8, 7,JC),JC=1,3) /                                      
     S 0.85537123E+01, 0.98021793E+01, 0.00000000E+00/                  
      DATA (GB( 8, 7,JC),JC=1,3) /                                      
     S 0.85537123E+01, 0.10071426E+02, 0.10000000E+01/                  
      DATA (GA( 8, 8,JC),JC=1,3) /                                      
     S 0.73668729E+01, 0.88811055E+01, 0.00000000E+00/                  
      DATA (GB( 8, 8,JC),JC=1,3) /                                      
     S 0.73668729E+01, 0.91333042E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 3 ----- T =    287.50                                 
C                                                                       
      DATA (GA( 9, 7,JC),JC=1,3) /                                      
     S 0.82163567E+01, 0.95891296E+01, 0.00000000E+00/                  
      DATA (GB( 9, 7,JC),JC=1,3) /                                      
     S 0.82163567E+01, 0.98520772E+01, 0.10000000E+01/                  
      DATA (GA( 9, 8,JC),JC=1,3) /                                      
     S 0.71747025E+01, 0.86574340E+01, 0.00000000E+00/                  
      DATA (GB( 9, 8,JC),JC=1,3) /                                      
     S 0.71747025E+01, 0.89093260E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 3 ----- T =    300.00                                 
C                                                                       
      DATA (GA(10, 7,JC),JC=1,3) /                                      
     S 0.79381525E+01, 0.93904375E+01, 0.00000000E+00/                  
      DATA (GB(10, 7,JC),JC=1,3) /                                      
     S 0.79381525E+01, 0.96487960E+01, 0.10000000E+01/                  
      DATA (GA(10, 8,JC),JC=1,3) /                                      
     S 0.70256573E+01, 0.84526940E+01, 0.00000000E+00/                  
      DATA (GB(10, 8,JC),JC=1,3) /                                      
     S 0.70256573E+01, 0.87055560E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 3 ----- T =    312.50                                 
C                                                                       
      DATA (GA(11, 7,JC),JC=1,3) /                                      
     S 0.77078849E+01, 0.92054098E+01, 0.00000000E+00/                  
      DATA (GB(11, 7,JC),JC=1,3) /                                      
     S 0.77078849E+01, 0.94605633E+01, 0.10000000E+01/                  
      DATA (GA(11, 8,JC),JC=1,3) /                                      
     S 0.69105145E+01, 0.82652769E+01, 0.00000000E+00/                  
      DATA (GB(11, 8,JC),JC=1,3) /                                      
     S 0.69105145E+01, 0.85200926E+01, 0.10000000E+01/                  
C                                                                       
C-- WATER VAPOR --- 970 -1110 CM-1                                      
C                                                                       
C                                                                       
C----- INTERVAL = 4 ----- T =    187.500                                
C                                                                       
      DATA (GA( 1, 9,JC),JC=1,3) /                                      
     S 0.94868790E+02, 0.17437229E+02, 0.00000000E+00/                  
      DATA (GB( 1, 9,JC),JC=1,3) /                                      
     S 0.94868790E+02, 0.18796329E+02, 0.10000000E+01/                  
      DATA (GA( 1,10,JC),JC=1,3) /                                      
     S 0.95419113E+02, 0.17234291E+02, 0.00000000E+00/                  
      DATA (GB( 1,10,JC),JC=1,3) /                                      
     S 0.95419113E+02, 0.18632164E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =    200.000                                
C                                                                       
      DATA (GA( 2, 9,JC),JC=1,3) /                                      
     S 0.95143959E+02, 0.17335575E+02, 0.00000000E+00/                  
      DATA (GB( 2, 9,JC),JC=1,3) /                                      
     S 0.95143959E+02, 0.18714111E+02, 0.10000000E+01/                  
      DATA (GA( 2,10,JC),JC=1,3) /                                      
     S 0.95675995E+02, 0.17129913E+02, 0.00000000E+00/                  
      DATA (GB( 2,10,JC),JC=1,3) /                                      
     S 0.95675995E+02, 0.18547161E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =    212.500                                
C                                                                       
      DATA (GA( 3, 9,JC),JC=1,3) /                                      
     S 0.95363609E+02, 0.17245064E+02, 0.00000000E+00/                  
      DATA (GB( 3, 9,JC),JC=1,3) /                                      
     S 0.95363609E+02, 0.18640270E+02, 0.10000000E+01/                  
      DATA (GA( 3,10,JC),JC=1,3) /                                      
     S 0.95911537E+02, 0.17036016E+02, 0.00000000E+00/                  
      DATA (GB( 3,10,JC),JC=1,3) /                                      
     S 0.95911537E+02, 0.18470697E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =    225.000                                
C                                                                       
      DATA (GA( 4, 9,JC),JC=1,3) /                                      
     S 0.95551239E+02, 0.17163479E+02, 0.00000000E+00/                  
      DATA (GB( 4, 9,JC),JC=1,3) /                                      
     S 0.95551239E+02, 0.18573282E+02, 0.10000000E+01/                  
      DATA (GA( 4,10,JC),JC=1,3) /                                      
     S 0.96128136E+02, 0.16952074E+02, 0.00000000E+00/                  
      DATA (GB( 4,10,JC),JC=1,3) /                                      
     S 0.96128136E+02, 0.18402708E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =    237.500                                
C                                                                       
      DATA (GA( 5, 9,JC),JC=1,3) /                                      
     S 0.95763680E+02, 0.17089018E+02, 0.00000000E+00/                  
      DATA (GB( 5, 9,JC),JC=1,3) /                                      
     S 0.95763680E+02, 0.18513403E+02, 0.10000000E+01/                  
      DATA (GA( 5,10,JC),JC=1,3) /                                      
     S 0.96294296E+02, 0.16877150E+02, 0.00000000E+00/                  
      DATA (GB( 5,10,JC),JC=1,3) /                                      
     S 0.96294296E+02, 0.18341324E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =    250.000                                
C                                                                       
      DATA (GA( 6, 9,JC),JC=1,3) /                                      
     S 0.95913246E+02, 0.17022213E+02, 0.00000000E+00/                  
      DATA (GB( 6, 9,JC),JC=1,3) /                                      
     S 0.95913246E+02, 0.18458538E+02, 0.10000000E+01/                  
      DATA (GA( 6,10,JC),JC=1,3) /                                      
     S 0.96462036E+02, 0.16808561E+02, 0.00000000E+00/                  
      DATA (GB( 6,10,JC),JC=1,3) /                                      
     S 0.96462036E+02, 0.18285398E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =    262.500                                
C                                                                       
      DATA (GA( 7, 9,JC),JC=1,3) /                                      
     S 0.96069473E+02, 0.16961018E+02, 0.00000000E+00/                  
      DATA (GB( 7, 9,JC),JC=1,3) /                                      
     S 0.96069473E+02, 0.18408932E+02, 0.10000000E+01/                  
      DATA (GA( 7,10,JC),JC=1,3) /                                      
     S 0.96610298E+02, 0.16746571E+02, 0.00000000E+00/                  
      DATA (GB( 7,10,JC),JC=1,3) /                                      
     S 0.96610298E+02, 0.18234850E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =    275.000                                
C                                                                       
      DATA (GA( 8, 9,JC),JC=1,3) /                                      
     S 0.96201309E+02, 0.16905493E+02, 0.00000000E+00/                  
      DATA (GB( 8, 9,JC),JC=1,3) /                                      
     S 0.96201309E+02, 0.18363655E+02, 0.10000000E+01/                  
      DATA (GA( 8,10,JC),JC=1,3) /                                      
     S 0.96754684E+02, 0.16690109E+02, 0.00000000E+00/                  
      DATA (GB( 8,10,JC),JC=1,3) /                                      
     S 0.96754684E+02, 0.18189079E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =    287.500                                
C                                                                       
      DATA (GA( 9, 9,JC),JC=1,3) /                                      
     S 0.96342339E+02, 0.16853893E+02, 0.00000000E+00/                  
      DATA (GB( 9, 9,JC),JC=1,3) /                                      
     S 0.96342339E+02, 0.18321957E+02, 0.10000000E+01/                  
      DATA (GA( 9,10,JC),JC=1,3) /                                      
     S 0.96883301E+02, 0.16638573E+02, 0.00000000E+00/                  
      DATA (GB( 9,10,JC),JC=1,3) /                                      
     S 0.96883301E+02, 0.18147303E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =    300.000                                
C                                                                       
      DATA (GA(10, 9,JC),JC=1,3) /                                      
     S 0.96457764E+02, 0.16806755E+02, 0.00000000E+00/                  
      DATA (GB(10, 9,JC),JC=1,3) /                                      
     S 0.96457764E+02, 0.18283590E+02, 0.10000000E+01/                  
      DATA (GA(10,10,JC),JC=1,3) /                                      
     S 0.96989792E+02, 0.16591677E+02, 0.00000000E+00/                  
      DATA (GB(10,10,JC),JC=1,3) /                                      
     S 0.96989792E+02, 0.18108814E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =    312.500                                
C                                                                       
      DATA (GA(11, 9,JC),JC=1,3) /                                      
     S 0.96549957E+02, 0.16763689E+02, 0.00000000E+00/                  
      DATA (GB(11, 9,JC),JC=1,3) /                                      
     S 0.96549957E+02, 0.18248074E+02, 0.10000000E+01/                  
      DATA (GA(11,10,JC),JC=1,3) /                                      
     S 0.97097275E+02, 0.16548479E+02, 0.00000000E+00/                  
      DATA (GB(11,10,JC),JC=1,3) /                                      
     S 0.97097275E+02, 0.18073788E+02, 0.10000000E+01/                  
C                                                                       
C-- WATER VAPOR -- INT.5 --  350- 500 CM-1                              
C                                                                       
C                                                                       
C----- INTERVAL = 5 ----- T =    187.500                                
C                                                                       
      DATA (GA( 1, 5,JC),JC=1,3) /                                      
     S 0.13134047E+00,-0.23649048E-01, 0.00000000E+00/                  
      DATA (GB( 1, 5,JC),JC=1,3) /                                      
     S 0.13134047E+00, 0.54008597E+00, 0.10000000E+01/                  
      DATA (GA( 1, 6,JC),JC=1,3) /                                      
     S 0.14049581E+00,-0.26231026E-01, 0.00000000E+00/                  
      DATA (GB( 1, 6,JC),JC=1,3) /                                      
     S 0.14049581E+00, 0.54143775E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 5 ----- T =    200.000                                
C                                                                       
      DATA (GA( 2, 5,JC),JC=1,3) /                                      
     S 0.13330981E+00,-0.24178915E-01, 0.00000000E+00/                  
      DATA (GB( 2, 5,JC),JC=1,3) /                                      
     S 0.13330981E+00, 0.54094988E+00, 0.10000000E+01/                  
      DATA (GA( 2, 6,JC),JC=1,3) /                                      
     S 0.14229582E+00,-0.26725627E-01, 0.00000000E+00/                  
      DATA (GB( 2, 6,JC),JC=1,3) /                                      
     S 0.14229582E+00, 0.54189402E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 5 ----- T =    212.500                                
C                                                                       
      DATA (GA( 3, 5,JC),JC=1,3) /                                      
     S 0.13504511E+00,-0.24650861E-01, 0.00000000E+00/                  
      DATA (GB( 3, 5,JC),JC=1,3) /                                      
     S 0.13504511E+00, 0.54169583E+00, 0.10000000E+01/                  
      DATA (GA( 3, 6,JC),JC=1,3) /                                      
     S 0.14384151E+00,-0.27153090E-01, 0.00000000E+00/                  
      DATA (GB( 3, 6,JC),JC=1,3) /                                      
     S 0.14384151E+00, 0.54222512E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 5 ----- T =    225.000                                
C                                                                       
      DATA (GA( 4, 5,JC),JC=1,3) /                                      
     S 0.13651314E+00,-0.25044570E-01, 0.00000000E+00/                  
      DATA (GB( 4, 5,JC),JC=1,3) /                                      
     S 0.13651314E+00, 0.54191625E+00, 0.10000000E+01/                  
      DATA (GA( 4, 6,JC),JC=1,3) /                                      
     S 0.14517485E+00,-0.27523417E-01, 0.00000000E+00/                  
      DATA (GB( 4, 6,JC),JC=1,3) /                                      
     S 0.14517485E+00, 0.54246801E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 5 ----- T =    237.500                                
C                                                                       
      DATA (GA( 5, 5,JC),JC=1,3) /                                      
     S 0.13790466E+00,-0.25431205E-01, 0.00000000E+00/                  
      DATA (GB( 5, 5,JC),JC=1,3) /                                      
     S 0.13790466E+00, 0.54262739E+00, 0.10000000E+01/                  
      DATA (GA( 5, 6,JC),JC=1,3) /                                      
     S 0.14633319E+00,-0.27845515E-01, 0.00000000E+00/                  
      DATA (GB( 5, 6,JC),JC=1,3) /                                      
     S 0.14633319E+00, 0.54265642E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 5 ----- T =    250.000                                
C                                                                       
      DATA (GA( 6, 5,JC),JC=1,3) /                                      
     S 0.13909847E+00,-0.25759069E-01, 0.00000000E+00/                  
      DATA (GB( 6, 5,JC),JC=1,3) /                                      
     S 0.13909847E+00, 0.54293746E+00, 0.10000000E+01/                  
      DATA (GA( 6, 6,JC),JC=1,3) /                                      
     S 0.14734478E+00,-0.28128289E-01, 0.00000000E+00/                  
      DATA (GB( 6, 6,JC),JC=1,3) /                                      
     S 0.14734478E+00, 0.54279846E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 5 ----- T =    262.500                                
C                                                                       
      DATA (GA( 7, 5,JC),JC=1,3) /                                      
     S 0.14014673E+00,-0.26045626E-01, 0.00000000E+00/                  
      DATA (GB( 7, 5,JC),JC=1,3) /                                      
     S 0.14014673E+00, 0.54306209E+00, 0.10000000E+01/                  
      DATA (GA( 7, 6,JC),JC=1,3) /                                      
     S 0.14823231E+00,-0.28376250E-01, 0.00000000E+00/                  
      DATA (GB( 7, 6,JC),JC=1,3) /                                      
     S 0.14823231E+00, 0.54290944E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 5 ----- T =    275.000                                
C                                                                       
      DATA (GA( 8, 5,JC),JC=1,3) /                                      
     S 0.14110570E+00,-0.26310772E-01, 0.00000000E+00/                  
      DATA (GB( 8, 5,JC),JC=1,3) /                                      
     S 0.14110570E+00, 0.54324186E+00, 0.10000000E+01/                  
      DATA (GA( 8, 6,JC),JC=1,3) /                                      
     S 0.14901793E+00,-0.28597040E-01, 0.00000000E+00/                  
      DATA (GB( 8, 6,JC),JC=1,3) /                                      
     S 0.14901793E+00, 0.54300690E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 5 ----- T =    287.500                                
C                                                                       
      DATA (GA( 9, 5,JC),JC=1,3) /                                      
     S 0.14198618E+00,-0.26556855E-01, 0.00000000E+00/                  
      DATA (GB( 9, 5,JC),JC=1,3) /                                      
     S 0.14198618E+00, 0.54346353E+00, 0.10000000E+01/                  
      DATA (GA( 9, 6,JC),JC=1,3) /                                      
     S 0.14971229E+00,-0.28792191E-01, 0.00000000E+00/                  
      DATA (GB( 9, 6,JC),JC=1,3) /                                      
     S 0.14971229E+00, 0.54307669E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 5 ----- T =    300.000                                
C                                                                       
      DATA (GA(10, 5,JC),JC=1,3) /                                      
     S 0.14277160E+00,-0.26774861E-01, 0.00000000E+00/                  
      DATA (GB(10, 5,JC),JC=1,3) /                                      
     S 0.14277160E+00, 0.54357564E+00, 0.10000000E+01/                  
      DATA (GA(10, 6,JC),JC=1,3) /                                      
     S 0.15033126E+00,-0.28966904E-01, 0.00000000E+00/                  
      DATA (GB(10, 6,JC),JC=1,3) /                                      
     S 0.15033126E+00, 0.54313493E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 5 ----- T =    312.500                                
C                                                                       
      DATA (GA(11, 5,JC),JC=1,3) /                                      
     S 0.14349319E+00,-0.26976623E-01, 0.00000000E+00/                  
      DATA (GB(11, 5,JC),JC=1,3) /                                      
     S 0.14349319E+00, 0.54369903E+00, 0.10000000E+01/                  
      DATA(GA(11, 6,JC),JC=1,3) /                                       
     S 0.15088280E+00,-0.29121898E-01, 0.00000000E+00/                  
      DATA (GB(11, 6,JC),JC=1,3) /                                      
     S 0.15088280E+00, 0.54318237E+00, 0.10000000E+01/                  
C                                                                       
C-WATER VAPOR-WINGS OF VIBRATION-ROTATION BAND - 1250-1450+1880-2820-   
C                                                                       
C                                                                       
C----- INTERVAL = 6 ----- T =    187.50                                 
C                                                                       
      DATA (GA( 1,11,JC),JC=1,3) /                                      
     S 0.74737163E+00,-0.60590485E-01, 0.00000000E+00/                  
      DATA (GB( 1,11,JC),JC=1,3) /                                      
     S 0.74737163E+00, 0.17888287E+01, 0.10000000E+01/                  
      DATA (GA( 1,12,JC),JC=1,3) /                                      
     S 0.64412124E+00,-0.16340195E-01, 0.00000000E+00/                  
      DATA (GB( 1,12,JC),JC=1,3) /                                      
     S 0.64412124E+00, 0.16133750E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 6 ----- T =    200.00                                 
C                                                                       
      DATA (GA( 2,11,JC),JC=1,3) /                                      
     S 0.67637182E+00,-0.27088422E-01, 0.00000000E+00/                  
      DATA (GB( 2,11,JC),JC=1,3) /                                      
     S 0.67637182E+00, 0.16680907E+01, 0.10000000E+01/                  
      DATA (GA( 2,12,JC),JC=1,3) /                                      
     S 0.55131795E+00, 0.32252772E-01, 0.00000000E+00/                  
      DATA (GB( 2,12,JC),JC=1,3) /                                      
     S 0.55131795E+00, 0.14332603E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 6 ----- T =    212.50                                 
C                                                                       
      DATA (GA( 3,11,JC),JC=1,3) /                                      
     S 0.59891051E+00, 0.12874948E-01, 0.00000000E+00/                  
      DATA (GB( 3,11,JC),JC=1,3) /                                      
     S 0.59891051E+00, 0.15230591E+01, 0.10000000E+01/                  
      DATA (GA( 3,12,JC),JC=1,3) /                                      
     S 0.44063067E+00, 0.91024989E-01, 0.00000000E+00/                  
      DATA (GB( 3,12,JC),JC=1,3) /                                      
     S 0.44063067E+00, 0.12039660E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 6 ----- T =    225.00                                 
C                                                                       
      DATA (GA( 4,11,JC),JC=1,3) /                                      
     S 0.51061614E+00, 0.59883548E-01, 0.00000000E+00/                  
      DATA (GB( 4,11,JC),JC=1,3) /                                      
     S 0.51061614E+00, 0.13461757E+01, 0.10000000E+01/                  
      DATA (GA( 4,12,JC),JC=1,3) /                                      
     S 0.30439727E+00, 0.15980470E+00, 0.00000000E+00/                  
      DATA (GB( 4,12,JC),JC=1,3) /                                      
     S 0.30439727E+00, 0.91241774E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 6 ----- T =    237.50                                 
C                                                                       
      DATA (GA( 5,11,JC),JC=1,3) /                                      
     S 0.40673842E+00, 0.11398935E+00, 0.00000000E+00/                  
      DATA (GB( 5,11,JC),JC=1,3) /                                      
     S 0.40673842E+00, 0.11289965E+01, 0.10000000E+01/                  
      DATA (GA( 5,12,JC),JC=1,3) /                                      
     S 0.15005620E+00, 0.23446409E+00, 0.00000000E+00/                  
      DATA (GB( 5,12,JC),JC=1,3) /                                      
     S 0.15005620E+00, 0.58547602E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 6 ----- T =    250.00                                 
C                                                                       
      DATA (GA( 6,11,JC),JC=1,3) /                                      
     S 0.28375203E+00, 0.17444874E+00, 0.00000000E+00/                  
      DATA (GB( 6,11,JC),JC=1,3) /                                      
     S 0.28375203E+00, 0.86693115E+00, 0.10000000E+01/                  
      DATA (GA( 6,12,JC),JC=1,3) /                                      
     S 0.51658133E-01, 0.29464628E+00, 0.00000000E+00/                  
      DATA (GB( 6,12,JC),JC=1,3) /                                      
     S 0.51658133E-01, 0.40170363E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 6 ----- T =    262.50                                 
C                                                                       
      DATA (GA( 7,11,JC),JC=1,3) /                                      
     S 0.15136005E+00, 0.23761536E+00, 0.00000000E+00/                  
      DATA (GB( 7,11,JC),JC=1,3) /                                      
     S 0.15136005E+00, 0.58911683E+00, 0.10000000E+01/                  
      DATA (GA( 7,12,JC),JC=1,3) /                                      
     S 0.18932679E-01, 0.33516250E+00, 0.00000000E+00/                  
      DATA (GB( 7,12,JC),JC=1,3) /                                      
     S 0.18932679E-01, 0.36796152E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 6 ----- T =    275.00                                 
C                                                                       
      DATA (GA( 8,11,JC),JC=1,3) /                                      
     S 0.61752765E-01, 0.28965454E+00, 0.00000000E+00/                  
      DATA (GB( 8,11,JC),JC=1,3) /                                      
     S 0.61752765E-01, 0.41975793E+00, 0.10000000E+01/                  
      DATA (GA( 8,12,JC),JC=1,3) /                                      
     S 0.79619427E-02, 0.36873141E+00, 0.00000000E+00/                  
      DATA (GB( 8,12,JC),JC=1,3) /                                      
     S 0.79619427E-02, 0.37964009E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 6 ----- T =    287.50                                 
C                                                                       
      DATA (GA( 9,11,JC),JC=1,3) /                                      
     S 0.26192233E-01, 0.32568668E+00, 0.00000000E+00/                  
      DATA (GB( 9,11,JC),JC=1,3) /                                      
     S 0.26192233E-01, 0.37385106E+00, 0.10000000E+01/                  
      DATA (GA( 9,12,JC),JC=1,3) /                                      
     S 0.40764469E-02, 0.40137572E+00, 0.00000000E+00/                  
      DATA (GB( 9,12,JC),JC=1,3) /                                      
     S 0.40764469E-02, 0.40608889E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 6 ----- T =    300.00                                 
C                                                                       
      DATA (GA(10,11,JC),JC=1,3) /                                      
     S 0.12168738E-01, 0.35455148E+00, 0.00000000E+00/                  
      DATA (GB(10,11,JC),JC=1,3) /                                      
     S 0.12168738E-01, 0.37329084E+00, 0.10000000E+01/                  
      DATA (GA(10,12,JC),JC=1,3) /                                      
     S 0.23914061E-02, 0.43615987E+00, 0.00000000E+00/                  
      DATA (GB(10,12,JC),JC=1,3) /                                      
     S 0.23914061E-02, 0.43919208E+00, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 6 ----- T =    312.50                                 
C                                                                       
      DATA (GA(10,11,JC),JC=1,3) /                                      
     S 0.12168738E-01, 0.35455148E+00, 0.00000000E+00/                  
      DATA (GB(10,11,JC),JC=1,3) /                                      
     S 0.12168738E-01, 0.37329084E+00, 0.10000000E+01/                  
      DATA (GA(10,12,JC),JC=1,3) /                                      
     S 0.23914061E-02, 0.43615987E+00, 0.00000000E+00/                  
      DATA (GB(10,12,JC),JC=1,3) /                                      
     S 0.23914061E-02, 0.43919208E+00, 0.10000000E+01/                  
C                                                                       
C-- CO2 -- INT.2 -- 500-800 CM-1 --- FROM ABS225 ------------           
C                                                                       
C-- FIU = 0.8 + MAX(0.35,(7-IU)*0.9)  , X/T,  9                         
C                                                                       
C----- INTERVAL = 2 ----- T =  187.5                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION   1 30 38 45                         
      DATA (GA( 1,13,JC),JC=1,3) /                                      
     S 0.87668459E-01, 0.13845511E+01, 0.00000000E+00/                  
      DATA (GB( 1,13,JC),JC=1,3) /                                      
     S 0.87668459E-01, 0.23203798E+01, 0.10000000E+01/                  
      DATA (GA( 1,14,JC),JC=1,3) /                                      
     S 0.74878820E-01, 0.11718758E+01, 0.00000000E+00/                  
      DATA (GB( 1,14,JC),JC=1,3) /                                      
     S 0.74878820E-01, 0.20206726E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =  200.0                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION   1 30 38 45                         
      DATA (GA( 2,13,JC),JC=1,3) /                                      
     S 0.83754276E-01, 0.13187042E+01, 0.00000000E+00/                  
      DATA (GB( 2,13,JC),JC=1,3) /                                      
     S 0.83754276E-01, 0.22288925E+01, 0.10000000E+01/                  
      DATA (GA( 2,14,JC),JC=1,3) /                                      
     S 0.71650966E-01, 0.11216131E+01, 0.00000000E+00/                  
      DATA (GB( 2,14,JC),JC=1,3) /                                      
     S 0.71650966E-01, 0.19441824E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =  212.5                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION   1 30 38 45                         
      DATA (GA( 3,13,JC),JC=1,3) /                                      
     S 0.80460283E-01, 0.12644396E+01, 0.00000000E+00/                  
      DATA (GB( 3,13,JC),JC=1,3) /                                      
     S 0.80460283E-01, 0.21515593E+01, 0.10000000E+01/                  
      DATA (GA( 3,14,JC),JC=1,3) /                                      
     S 0.68979615E-01, 0.10809473E+01, 0.00000000E+00/                  
      DATA (GB( 3,14,JC),JC=1,3) /                                      
     S 0.68979615E-01, 0.18807257E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =  225.0                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION   1 30 38 45                         
      DATA (GA( 4,13,JC),JC=1,3) /                                      
     S 0.77659686E-01, 0.12191543E+01, 0.00000000E+00/                  
      DATA (GB( 4,13,JC),JC=1,3) /                                      
     S 0.77659686E-01, 0.20855896E+01, 0.10000000E+01/                  
      DATA (GA( 4,14,JC),JC=1,3) /                                      
     S 0.66745345E-01, 0.10476396E+01, 0.00000000E+00/                  
      DATA (GB( 4,14,JC),JC=1,3) /                                      
     S 0.66745345E-01, 0.18275618E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =  237.5                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION   1 30 38 45                         
      DATA (GA( 5,13,JC),JC=1,3) /                                      
     S 0.75257056E-01, 0.11809511E+01, 0.00000000E+00/                  
      DATA (GB( 5,13,JC),JC=1,3) /                                      
     S 0.75257056E-01, 0.20288489E+01, 0.10000000E+01/                  
      DATA (GA( 5,14,JC),JC=1,3) /                                      
     S 0.64857571E-01, 0.10200373E+01, 0.00000000E+00/                  
      DATA (GB( 5,14,JC),JC=1,3) /                                      
     S 0.64857571E-01, 0.17825910E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =  250.0                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION   1 30 38 45                         
      DATA (GA( 6,13,JC),JC=1,3) /                                      
     S 0.73179175E-01, 0.11484154E+01, 0.00000000E+00/                  
      DATA (GB( 6,13,JC),JC=1,3) /                                      
     S 0.73179175E-01, 0.19796791E+01, 0.10000000E+01/                  
      DATA (GA( 6,14,JC),JC=1,3) /                                      
     S 0.63248495E-01, 0.99692726E+00, 0.00000000E+00/                  
      DATA (GB( 6,14,JC),JC=1,3) /                                      
     S 0.63248495E-01, 0.17442308E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =  262.5                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION   1 30 38 45                         
      DATA (GA( 7,13,JC),JC=1,3) /                                      
     S 0.71369063E-01, 0.11204723E+01, 0.00000000E+00/                  
      DATA (GB( 7,13,JC),JC=1,3) /                                      
     S 0.71369063E-01, 0.19367778E+01, 0.10000000E+01/                  
      DATA (GA( 7,14,JC),JC=1,3) /                                      
     S 0.61866970E-01, 0.97740923E+00, 0.00000000E+00/                  
      DATA (GB( 7,14,JC),JC=1,3) /                                      
     S 0.61866970E-01, 0.17112809E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =  275.0                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION   1 30 38 45                         
      DATA (GA( 8,13,JC),JC=1,3) /                                      
     S 0.69781812E-01, 0.10962918E+01, 0.00000000E+00/                  
      DATA (GB( 8,13,JC),JC=1,3) /                                      
     S 0.69781812E-01, 0.18991112E+01, 0.10000000E+01/                  
      DATA (GA( 8,14,JC),JC=1,3) /                                      
     S 0.60673632E-01, 0.96080188E+00, 0.00000000E+00/                  
      DATA (GB( 8,14,JC),JC=1,3) /                                      
     S 0.60673632E-01, 0.16828137E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =  287.5                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION   1 30 38 45                         
      DATA (GA( 9,13,JC),JC=1,3) /                                      
     S 0.68381606E-01, 0.10752229E+01, 0.00000000E+00/                  
      DATA (GB( 9,13,JC),JC=1,3) /                                      
     S 0.68381606E-01, 0.18658501E+01, 0.10000000E+01/                  
      DATA (GA( 9,14,JC),JC=1,3) /                                      
     S 0.59637277E-01, 0.94657562E+00, 0.00000000E+00/                  
      DATA (GB( 9,14,JC),JC=1,3) /                                      
     S 0.59637277E-01, 0.16580908E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =  300.0                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION   1 30 38 45                         
      DATA (GA(10,13,JC),JC=1,3) /                                      
     S 0.67139539E-01, 0.10567474E+01, 0.00000000E+00/                  
      DATA (GB(10,13,JC),JC=1,3) /                                      
     S 0.67139539E-01, 0.18363226E+01, 0.10000000E+01/                  
      DATA (GA(10,14,JC),JC=1,3) /                                      
     S 0.58732178E-01, 0.93430511E+00, 0.00000000E+00/                  
      DATA (GB(10,14,JC),JC=1,3) /                                      
     S 0.58732178E-01, 0.16365014E+01, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 2 ----- T =  312.5                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION   1 30 38 45                         
      DATA (GA(11,13,JC),JC=1,3) /                                      
     S 0.66032012E-01, 0.10404465E+01, 0.00000000E+00/                  
      DATA (GB(11,13,JC),JC=1,3) /                                      
     S 0.66032012E-01, 0.18099779E+01, 0.10000000E+01/                  
      DATA (GA(11,14,JC),JC=1,3) /                                      
     S 0.57936092E-01, 0.92363528E+00, 0.00000000E+00/                  
      DATA (GB(11,14,JC),JC=1,3) /                                      
     S 0.57936092E-01, 0.16175164E+01, 0.10000000E+01/                  
C                                                                       
C                                                                       
C-- CARBON DIOXIDE LINES IN THE WINDOW REGION (800-1250 CM-1)           
C                                                                       
C-- G = 0.0                                                             
C                                                                       
C----- INTERVAL = 4 ----- T =  187.5                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION     1   15   29   45                 
      DATA (GA( 1,15,JC),JC=1,3) /                                      
     S 0.13230067E+02, 0.22042132E+02, 0.00000000E+00/                  
      DATA (GB( 1,15,JC),JC=1,3) /                                      
     S 0.13230067E+02, 0.22051750E+02, 0.10000000E+01/                  
      DATA (GA( 1,16,JC),JC=1,3) /                                      
     S 0.13183816E+02, 0.22169501E+02, 0.00000000E+00/                  
      DATA (GB( 1,16,JC),JC=1,3) /                                      
     S 0.13183816E+02, 0.22178972E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =  200.0                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION     1   15   29   45                 
      DATA (GA( 2,15,JC),JC=1,3) /                                      
     S 0.13213564E+02, 0.22107298E+02, 0.00000000E+00/                  
      DATA (GB( 2,15,JC),JC=1,3) /                                      
     S 0.13213564E+02, 0.22116850E+02, 0.10000000E+01/                  
      DATA (GA( 2,16,JC),JC=1,3) /                                      
     S 0.13189991E+02, 0.22270075E+02, 0.00000000E+00/                  
      DATA (GB( 2,16,JC),JC=1,3) /                                      
     S 0.13189991E+02, 0.22279484E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =  212.5                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION     1   15   29   45                 
      DATA (GA( 3,15,JC),JC=1,3) /                                      
     S 0.13209140E+02, 0.22180915E+02, 0.00000000E+00/                  
      DATA (GB( 3,15,JC),JC=1,3) /                                      
     S 0.13209140E+02, 0.22190410E+02, 0.10000000E+01/                  
      DATA (GA( 3,16,JC),JC=1,3) /                                      
     S 0.13209485E+02, 0.22379193E+02, 0.00000000E+00/                  
      DATA (GB( 3,16,JC),JC=1,3) /                                      
     S 0.13209485E+02, 0.22388551E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =  225.0                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION     1   15   29   45                 
      DATA (GA( 4,15,JC),JC=1,3) /                                      
     S 0.13213894E+02, 0.22259478E+02, 0.00000000E+00/                  
      DATA (GB( 4,15,JC),JC=1,3) /                                      
     S 0.13213894E+02, 0.22268925E+02, 0.10000000E+01/                  
      DATA (GA( 4,16,JC),JC=1,3) /                                      
     S 0.13238789E+02, 0.22492992E+02, 0.00000000E+00/                  
      DATA (GB( 4,16,JC),JC=1,3) /                                      
     S 0.13238789E+02, 0.22502309E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =  237.5                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION     1   15   29   45                 
      DATA (GA( 5,15,JC),JC=1,3) /                                      
     S 0.13225963E+02, 0.22341039E+02, 0.00000000E+00/                  
      DATA (GB( 5,15,JC),JC=1,3) /                                      
     S 0.13225963E+02, 0.22350445E+02, 0.10000000E+01/                  
      DATA (GA( 5,16,JC),JC=1,3) /                                      
     S 0.13275017E+02, 0.22608508E+02, 0.00000000E+00/                  
      DATA (GB( 5,16,JC),JC=1,3) /                                      
     S 0.13275017E+02, 0.22617792E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =  250.0                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION     1   15   29   45                 
      DATA (GA( 6,15,JC),JC=1,3) /                                      
     S 0.13243806E+02, 0.22424247E+02, 0.00000000E+00/                  
      DATA (GB( 6,15,JC),JC=1,3) /                                      
     S 0.13243806E+02, 0.22433617E+02, 0.10000000E+01/                  
      DATA (GA( 6,16,JC),JC=1,3) /                                      
     S 0.13316096E+02, 0.22723843E+02, 0.00000000E+00/                  
      DATA (GB( 6,16,JC),JC=1,3) /                                      
     S 0.13316096E+02, 0.22733099E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =  262.5                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION     1   15   29   45                 
      DATA (GA( 7,15,JC),JC=1,3) /                                      
     S 0.13266104E+02, 0.22508089E+02, 0.00000000E+00/                  
      DATA (GB( 7,15,JC),JC=1,3) /                                      
     S 0.13266104E+02, 0.22517429E+02, 0.10000000E+01/                  
      DATA (GA( 7,16,JC),JC=1,3) /                                      
     S 0.13360555E+02, 0.22837837E+02, 0.00000000E+00/                  
      DATA (GB( 7,16,JC),JC=1,3) /                                      
     S 0.13360555E+02, 0.22847071E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =  275.0                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION     1   15   29   45                 
      DATA (GA( 8,15,JC),JC=1,3) /                                      
     S 0.13291782E+02, 0.22591771E+02, 0.00000000E+00/                  
      DATA (GB( 8,15,JC),JC=1,3) /                                      
     S 0.13291782E+02, 0.22601086E+02, 0.10000000E+01/                  
      DATA (GA( 8,16,JC),JC=1,3) /                                      
     S 0.13407324E+02, 0.22949751E+02, 0.00000000E+00/                  
      DATA (GB( 8,16,JC),JC=1,3) /                                      
     S 0.13407324E+02, 0.22958967E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =  287.5                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION     1   15   29   45                 
      DATA (GA( 9,15,JC),JC=1,3) /                                      
     S 0.13319961E+02, 0.22674661E+02, 0.00000000E+00/                  
      DATA (GB( 9,15,JC),JC=1,3) /                                      
     S 0.13319961E+02, 0.22683956E+02, 0.10000000E+01/                  
      DATA (GA( 9,16,JC),JC=1,3) /                                      
     S 0.13455544E+02, 0.23059032E+02, 0.00000000E+00/                  
      DATA (GB( 9,16,JC),JC=1,3) /                                      
     S 0.13455544E+02, 0.23068234E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =  300.0                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION     1   15   29   45                 
      DATA (GA(10,15,JC),JC=1,3) /                                      
     S 0.13349927E+02, 0.22756246E+02, 0.00000000E+00/                  
      DATA (GB(10,15,JC),JC=1,3) /                                      
     S 0.13349927E+02, 0.22765522E+02, 0.10000000E+01/                  
      DATA (GA(10,16,JC),JC=1,3) /                                      
     S 0.13504450E+02, 0.23165146E+02, 0.00000000E+00/                  
      DATA (GB(10,16,JC),JC=1,3) /                                      
     S 0.13504450E+02, 0.23174336E+02, 0.10000000E+01/                  
C                                                                       
C----- INTERVAL = 4 ----- T =  312.5                                    
C                                                                       
C-- INDICES FOR PADE APPROXIMATION     1   15   29   45                 
      DATA (GA(11,15,JC),JC=1,3) /                                      
     S 0.13381108E+02, 0.22836093E+02, 0.00000000E+00/                  
      DATA (GB(11,15,JC),JC=1,3) /                                      
     S 0.13381108E+02, 0.22845354E+02, 0.10000000E+01/                  
      DATA (GA(11,16,JC),JC=1,3) /                                      
     S 0.13553282E+02, 0.23267456E+02, 0.00000000E+00/                  
      DATA (GB(11,16,JC),JC=1,3) /                                      
     S 0.13553282E+02, 0.23276638E+02, 0.10000000E+01/                  
C                                                                       
C                                                                       
C-- WATER VAPOR --- 650 - 800 CM-1                                      
C                                                                       
C---- INTERVAL = 2B ---- T = 187.5                                      
C                                                                       
      DATA (GA( 1,21,JC),JC=1,3) /                                      
     S 0.19550761E+02,-0.26333148E+00, 0.00000000E+00/                  
      DATA (GB( 1,21,JC),JC=1,3) /                                      
     S 0.19550761E+02, 0.49595891E+01, 0.10000000E+01/                  
      DATA (GA( 1,22,JC),JC=1,3) /                                      
     S 0.20402482E+02,-0.24922589E+00, 0.00000000E+00/                  
      DATA (GB( 1,22,JC),JC=1,3) /                                      
     S 0.20402482E+02, 0.51104394E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 2B ---- T = 200.0                                      
C                                                                       
      DATA (GA( 2,21,JC),JC=1,3) /                                      
     S 0.19830199E+02,-0.25754698E+00, 0.00000000E+00/                  
      DATA (GB( 2,21,JC),JC=1,3) /                                      
     S 0.19830199E+02, 0.50093036E+01, 0.10000000E+01/                  
      DATA (GA( 2,22,JC),JC=1,3) /                                      
     S 0.20696147E+02,-0.24494693E+00, 0.00000000E+00/                  
      DATA (GB( 2,22,JC),JC=1,3) /                                      
     S 0.20696147E+02, 0.51601103E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 2B ---- T = 212.5                                      
C                                                                       
      DATA (GA( 3,21,JC),JC=1,3) /                                      
     S 0.20094853E+02,-0.25394587E+00, 0.00000000E+00/                  
      DATA (GB( 3,21,JC),JC=1,3) /                                      
     S 0.20094853E+02, 0.50564022E+01, 0.10000000E+01/                  
      DATA (GA( 3,22,JC),JC=1,3) /                                      
     S 0.20957105E+02,-0.24158774E+00, 0.00000000E+00/                  
      DATA (GB( 3,22,JC),JC=1,3) /                                      
     S 0.20957105E+02, 0.52035338E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 2B ---- T = 225.0                                      
C                                                                       
      DATA (GA( 4,21,JC),JC=1,3) /                                      
     S 0.20321192E+02,-0.25012572E+00, 0.00000000E+00/                  
      DATA (GB( 4,21,JC),JC=1,3) /                                      
     S 0.20321192E+02, 0.50954623E+01, 0.10000000E+01/                  
      DATA (GA( 4,22,JC),JC=1,3) /                                      
     S 0.21189573E+02,-0.23892446E+00, 0.00000000E+00/                  
      DATA (GB( 4,22,JC),JC=1,3) /                                      
     S 0.21189573E+02, 0.52416609E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 2B ---- T = 237.5                                      
C                                                                       
      DATA (GA( 5,21,JC),JC=1,3) /                                      
     S 0.20525678E+02,-0.24697148E+00, 0.00000000E+00/                  
      DATA (GB( 5,21,JC),JC=1,3) /                                      
     S 0.20525678E+02, 0.51303033E+01, 0.10000000E+01/                  
      DATA (GA( 5,22,JC),JC=1,3) /                                      
     S 0.21397190E+02,-0.23679344E+00, 0.00000000E+00/                  
      DATA (GB( 5,22,JC),JC=1,3) /                                      
     S 0.21397190E+02, 0.52752763E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 2B ---- T = 250.0                                      
C                                                                       
      DATA (GA( 6,21,JC),JC=1,3) /                                      
     S 0.20710965E+02,-0.24434489E+00, 0.00000000E+00/                  
      DATA (GB( 6,21,JC),JC=1,3) /                                      
     S 0.20710965E+02, 0.51615130E+01, 0.10000000E+01/                  
      DATA (GA( 6,22,JC),JC=1,3) /                                      
     S 0.21583088E+02,-0.23507367E+00, 0.00000000E+00/                  
      DATA (GB( 6,22,JC),JC=1,3) /                                      
     S 0.21583088E+02, 0.53050307E+01, 0.10000000E+01/                  
                                                                        
C                                                                       
C---- INTERVAL = 2B ---- T = 262.5                                      
C                                                                       
      DATA (GA( 7,21,JC),JC=1,3) /                                      
     S 0.20879342E+02,-0.24214043E+00, 0.00000000E+00/                  
      DATA (GB( 7,21,JC),JC=1,3) /                                      
     S 0.20879342E+02, 0.51895822E+01, 0.10000000E+01/                  
      DATA (GA( 7,22,JC),JC=1,3) /                                      
     S 0.21767391E+02,-0.23500593E+00, 0.00000000E+00/                  
      DATA (GB( 7,22,JC),JC=1,3) /                                      
     S 0.21767391E+02, 0.53353369E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 2B ---- T = 275.0                                      
C                                                                       
      DATA (GA( 8,21,JC),JC=1,3) /                                      
     S 0.21032783E+02,-0.24027675E+00, 0.00000000E+00/                  
      DATA (GB( 8,21,JC),JC=1,3) /                                      
     S 0.21032783E+02, 0.52149228E+01, 0.10000000E+01/                  
      DATA (GA( 8,22,JC),JC=1,3) /                                      
     S 0.21917785E+02,-0.23387069E+00, 0.00000000E+00/                  
      DATA (GB( 8,22,JC),JC=1,3) /                                      
     S 0.21917785E+02, 0.53589399E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 2B ---- T = 287.5                                      
C                                                                       
      DATA (GA( 9,21,JC),JC=1,3) /                                      
     S 0.21172996E+02,-0.23869043E+00, 0.00000000E+00/                  
      DATA (GB( 9,21,JC),JC=1,3) /                                      
     S 0.21172996E+02, 0.52378823E+01, 0.10000000E+01/                  
      DATA (GA( 9,22,JC),JC=1,3) /                                      
     S 0.22053477E+02,-0.23293447E+00, 0.00000000E+00/                  
      DATA (GB( 9,22,JC),JC=1,3) /                                      
     S 0.22053477E+02, 0.53800578E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 2B ---- T = 300.0                                      
C                                                                       
      DATA (GA(10,21,JC),JC=1,3) /                                      
     S 0.21301462E+02,-0.23733163E+00, 0.00000000E+00/                  
      DATA (GB(10,21,JC),JC=1,3) /                                      
     S 0.21301462E+02, 0.52587551E+01, 0.10000000E+01/                  
      DATA (GA(10,22,JC),JC=1,3) /                                      
     S 0.22176209E+02,-0.23215726E+00, 0.00000000E+00/                  
      DATA (GB(10,22,JC),JC=1,3) /                                      
     S 0.22176209E+02, 0.53990137E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 2B ---- T = 312.5                                      
C                                                                       
      DATA (GA(11,21,JC),JC=1,3) /                                      
     S 0.21419467E+02,-0.23616087E+00, 0.00000000E+00/                  
      DATA (GB(11,21,JC),JC=1,3) /                                      
     S 0.21419467E+02, 0.52777920E+01, 0.10000000E+01/                  
      DATA (GA(11,22,JC),JC=1,3) /                                      
     S 0.22287486E+02,-0.23150796E+00, 0.00000000E+00/                  
      DATA (GB(11,22,JC),JC=1,3) /                                      
     S 0.22287486E+02, 0.54160819E+01, 0.10000000E+01/                  
C                                                                       
C                                                                       
C-- WATER VAPOR --- 800 - 970 CM-1                                      
C                                                                       
C---- INTERVAL = 3A ---- T = 187.5                                      
C                                                                       
      DATA (GA( 1,19,JC),JC=1,3) /                                      
     S 0.76275605E+02, 0.87071385E+01, 0.00000000E+00/                  
      DATA (GB( 1,19,JC),JC=1,3) /                                      
     S 0.76275605E+02, 0.12222527E+02, 0.10000000E+01/                  
      DATA (GA( 1,20,JC),JC=1,3) /                                      
     S 0.76707978E+02, 0.89850885E+01, 0.00000000E+00/                  
      DATA (GB( 1,20,JC),JC=1,3) /                                      
     S 0.76707978E+02, 0.12434732E+02, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 3A ---- T = 200                                        
C                                                                       
      DATA (GA( 2,19,JC),JC=1,3) /                                      
     S 0.76444692E+02, 0.88235285E+01, 0.00000000E+00/                  
      DATA (GB( 2,19,JC),JC=1,3) /                                      
     S 0.76444692E+02, 0.12310814E+02, 0.10000000E+01/                  
      DATA (GA( 2,20,JC),JC=1,3) /                                      
     S 0.76898353E+02, 0.91013883E+01, 0.00000000E+00/                  
      DATA (GB( 2,20,JC),JC=1,3) /                                      
     S 0.76898353E+02, 0.12523769E+02, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 3A ---- T = 212.5                                      
C                                                                       
      DATA (GA( 3,19,JC),JC=1,3) /                                      
     S 0.76601454E+02, 0.89263097E+01, 0.00000000E+00/                  
      DATA (GB( 3,19,JC),JC=1,3) /                                      
     S 0.76601454E+02, 0.12389074E+02, 0.10000000E+01/                  
      DATA (GA( 3,20,JC),JC=1,3) /                                      
     S 0.77725293E+02, 0.91803661E+01, 0.00000000E+00/                  
      DATA (GB( 3,20,JC),JC=1,3) /                                      
     S 0.77725293E+02, 0.12614198E+02, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 3A ---- T = 225                                        
C                                                                       
      DATA (GA( 4,19,JC),JC=1,3) /                                      
     S 0.76746614E+02, 0.90176179E+01, 0.00000000E+00/                  
      DATA (GB( 4,19,JC),JC=1,3) /                                      
     S 0.76746614E+02, 0.12458826E+02, 0.10000000E+01/                  
      DATA (GA( 4,20,JC),JC=1,3) /                                      
     S 0.77890904E+02, 0.92706174E+01, 0.00000000E+00/                  
      DATA (GB( 4,20,JC),JC=1,3) /                                      
     S 0.77890904E+02, 0.12683738E+02, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 3A ---- T = 237.5                                      
C                                                                       
      DATA (GA( 5,19,JC),JC=1,3) /                                      
     S 0.76880972E+02, 0.90991713E+01, 0.00000000E+00/                  
      DATA (GB( 5,19,JC),JC=1,3) /                                      
     S 0.76880972E+02, 0.12521305E+02, 0.10000000E+01/                  
      DATA (GA( 5,20,JC),JC=1,3) /                                      
     S 0.78042609E+02, 0.93508020E+01, 0.00000000E+00/                  
      DATA (GB( 5,20,JC),JC=1,3) /                                      
     S 0.78042609E+02, 0.12745679E+02, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 3A ---- T = 250                                        
C                                                                       
      DATA (GA( 6,19,JC),JC=1,3) /                                      
     S 0.77655425E+02, 0.91491477E+01, 0.00000000E+00/                  
      DATA (GB( 6,19,JC),JC=1,3) /                                      
     S 0.77655425E+02, 0.12589427E+02, 0.10000000E+01/                  
      DATA (GA( 6,20,JC),JC=1,3) /                                      
     S 0.78181610E+02, 0.94223568E+01, 0.00000000E+00/                  
      DATA (GB( 6,20,JC),JC=1,3) /                                      
     S 0.78181610E+02, 0.12801076E+02, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 3A ---- T = 262.5                                      
C                                                                       
      DATA (GA( 7,19,JC),JC=1,3) /                                      
     S 0.77774452E+02, 0.92150200E+01, 0.00000000E+00/                  
      DATA (GB( 7,19,JC),JC=1,3) /                                      
     S 0.77774452E+02, 0.12640126E+02, 0.10000000E+01/                  
      DATA (GA( 7,20,JC),JC=1,3) /                                      
     S 0.78309039E+02, 0.94864655E+01, 0.00000000E+00/                  
      DATA (GB( 7,20,JC),JC=1,3) /                                      
     S 0.78309039E+02, 0.12850804E+02, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 3A ---- T = 275                                        
C                                                                       
      DATA (GA( 8,19,JC),JC=1,3) /                                      
     S 0.77884708E+02, 0.92746449E+01, 0.00000000E+00/                  
      DATA (GB( 8,19,JC),JC=1,3) /                                      
     S 0.77884708E+02, 0.12686105E+02, 0.10000000E+01/                  
      DATA (GA( 8,20,JC),JC=1,3) /                                      
     S 0.78425953E+02, 0.95441121E+01, 0.00000000E+00/                  
      DATA (GB( 8,20,JC),JC=1,3) /                                      
     S 0.78425953E+02, 0.12895595E+02, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 3A ---- T = 287.5                                      
C                                                                       
      DATA (GA( 9,19,JC),JC=1,3) /                                      
     S 0.77986957E+02, 0.93288167E+01, 0.00000000E+00/                  
      DATA (GB( 9,19,JC),JC=1,3) /                                      
     S 0.77986957E+02, 0.12727952E+02, 0.10000000E+01/                  
      DATA (GA( 9,20,JC),JC=1,3) /                                      
     S 0.78533324E+02, 0.95961213E+01, 0.00000000E+00/                  
      DATA (GB( 9,20,JC),JC=1,3) /                                      
     S 0.78533324E+02, 0.12936067E+02, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 3A ---- T = 300                                        
C                                                                       
      DATA (GA(10,19,JC),JC=1,3) /                                      
     S 0.78081895E+02, 0.93782046E+01, 0.00000000E+00/                  
      DATA (GB(10,19,JC),JC=1,3) /                                      
     S 0.78081895E+02, 0.12766161E+02, 0.10000000E+01/                  
      DATA (GA(10,20,JC),JC=1,3) /                                      
     S 0.78632031E+02, 0.96431901E+01, 0.00000000E+00/                  
      DATA (GB(10,20,JC),JC=1,3) /                                      
     S 0.78632031E+02, 0.12972742E+02, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 3A ---- T = 312.5                                      
C                                                                       
      DATA (GA(11,19,JC),JC=1,3) /                                      
     S 0.78170173E+02, 0.94233755E+01, 0.00000000E+00/                  
      DATA (GB(11,19,JC),JC=1,3) /                                      
     S 0.78170173E+02, 0.12801157E+02, 0.10000000E+01/                  
      DATA (GA(11,20,JC),JC=1,3) /                                      
     S 0.79479954E+02, 0.96591215E+01, 0.00000000E+00/                  
      DATA (GB(11,20,JC),JC=1,3) /                                      
     S 0.79479954E+02, 0.13018486E+02, 0.10000000E+01/                  
C                                                                       
C                                                                       
C-- WATER VAPOR --- 1250 - 1450 CM-1                                    
C                                                                       
C---- INTERVAL = 6A ---- T = 187.5                                      
C                                                                       
      DATA (GA( 1,17,JC),JC=1,3) /                                      
     S 0.89351314E+00,-0.11716478E+00, 0.00000000E+00/                  
      DATA (GB( 1,17,JC),JC=1,3) /                                      
     S 0.89351314E+00, 0.23371168E+01, 0.10000000E+01/                  
      DATA (GA( 1,18,JC),JC=1,3) /                                      
     S 0.78489275E+00,-0.93578500E-01, 0.00000000E+00/                  
      DATA (GB( 1,18,JC),JC=1,3) /                                      
     S 0.78489275E+00, 0.21397518E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 6A ---- T = 200                                        
C                                                                       
      DATA (GA( 2,17,JC),JC=1,3) /                                      
     S 0.82462699E+00,-0.10225793E+00, 0.00000000E+00/                  
      DATA (GB( 2,17,JC),JC=1,3) /                                      
     S 0.82462699E+00, 0.22153415E+01, 0.10000000E+01/                  
      DATA (GA( 2,18,JC),JC=1,3) /                                      
     S 0.71552080E+00,-0.78132444E-01, 0.00000000E+00/                  
      DATA (GB( 2,18,JC),JC=1,3) /                                      
     S 0.71552080E+00, 0.20032846E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 6A ---- T = 212.5                                      
C                                                                       
      DATA (GA( 3,17,JC),JC=1,3) /                                      
     S 0.76338653E+00,-0.88787922E-01, 0.00000000E+00/                  
      DATA (GB( 3,17,JC),JC=1,3) /                                      
     S 0.76338653E+00, 0.20996613E+01, 0.10000000E+01/                  
      DATA (GA( 3,18,JC),JC=1,3) /                                      
     S 0.65500758E+00,-0.64411300E-01, 0.00000000E+00/                  
      DATA (GB( 3,18,JC),JC=1,3) /                                      
     S 0.65500758E+00, 0.18774865E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 6A ---- T = 225                                        
C                                                                       
      DATA (GA( 4,17,JC),JC=1,3) /                                      
     S 0.70887760E+00,-0.76600481E-01, 0.00000000E+00/                  
      DATA (GB( 4,17,JC),JC=1,3) /                                      
     S 0.70887760E+00, 0.19909030E+01, 0.10000000E+01/                  
      DATA (GA( 4,18,JC),JC=1,3) /                                      
     S 0.60215560E+00,-0.52213461E-01, 0.00000000E+00/                  
      DATA (GB( 4,18,JC),JC=1,3) /                                      
     S 0.60215560E+00, 0.17624913E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 6A ---- T = 237.5                                      
C                                                                       
      DATA (GA( 5,17,JC),JC=1,3) /                                      
     S 0.66065205E+00,-0.65662686E-01, 0.00000000E+00/                  
      DATA (GB( 5,17,JC),JC=1,3) /                                      
     S 0.66065205E+00, 0.18904910E+01, 0.10000000E+01/                  
      DATA (GA( 5,18,JC),JC=1,3) /                                      
     S 0.55598748E+00,-0.41381040E-01, 0.00000000E+00/                  
      DATA (GB( 5,18,JC),JC=1,3) /                                      
     S 0.55598748E+00, 0.16581940E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 6A ---- T = 250                                        
C                                                                       
      DATA (GA( 6,17,JC),JC=1,3) /                                      
     S 0.61783356E+00,-0.55813085E-01, 0.00000000E+00/                  
      DATA (GB( 6,17,JC),JC=1,3) /                                      
     S 0.61783356E+00, 0.17980026E+01, 0.10000000E+01/                  
      DATA (GA( 6,18,JC),JC=1,3) /                                      
     S 0.51447244E+00,-0.31409439E-01, 0.00000000E+00/                  
      DATA (GB( 6,18,JC),JC=1,3) /                                      
     S 0.51447244E+00, 0.15605436E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 6A ---- T = 262.5                                      
C                                                                       
      DATA (GA( 7,17,JC),JC=1,3) /                                      
     S 0.57976804E+00,-0.46940089E-01, 0.00000000E+00/                  
      DATA (GB( 7,17,JC),JC=1,3) /                                      
     S 0.57976804E+00, 0.17131839E+01, 0.10000000E+01/                  
      DATA (GA( 7,18,JC),JC=1,3) /                                      
     S 0.47868911E+00,-0.22726019E-01, 0.00000000E+00/                  
      DATA (GB( 7,18,JC),JC=1,3) /                                      
     S 0.47868911E+00, 0.14745410E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 6A ---- T = 275                                        
C                                                                       
      DATA (GA( 8,17,JC),JC=1,3) /                                      
     S 0.54587141E+00,-0.38940929E-01, 0.00000000E+00/                  
      DATA (GB( 8,17,JC),JC=1,3) /                                      
     S 0.54587141E+00, 0.16356179E+01, 0.10000000E+01/                  
      DATA (GA( 8,18,JC),JC=1,3) /                                      
     S 0.44731443E+00,-0.15014979E-01, 0.00000000E+00/                  
      DATA (GB( 8,18,JC),JC=1,3) /                                      
     S 0.44731443E+00, 0.13974344E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 6A ---- T = 287.5                                      
C                                                                       
      DATA (GA( 9,17,JC),JC=1,3) /                                      
     S 0.51448094E+00,-0.31366881E-01, 0.00000000E+00/                  
      DATA (GB( 9,17,JC),JC=1,3) /                                      
     S 0.51448094E+00, 0.15612510E+01, 0.10000000E+01/                  
      DATA (GA( 9,18,JC),JC=1,3) /                                      
     S 0.41974527E+00,-0.81628316E-02, 0.00000000E+00/                  
      DATA (GB( 9,18,JC),JC=1,3) /                                      
     S 0.41974527E+00, 0.13283720E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 6A ---- T = 300                                        
C                                                                       
      DATA (GA(10,17,JC),JC=1,3) /                                      
     S 0.48699661E+00,-0.24698543E-01, 0.00000000E+00/                  
      DATA (GB(10,17,JC),JC=1,3) /                                      
     S 0.48699661E+00, 0.14952467E+01, 0.10000000E+01/                  
      DATA (GA(10,18,JC),JC=1,3) /                                      
     S 0.39576456E+00,-0.21710464E-02, 0.00000000E+00/                  
      DATA (GB(10,18,JC),JC=1,3) /                                      
     S 0.39576456E+00, 0.12674953E+01, 0.10000000E+01/                  
C                                                                       
C---- INTERVAL = 6A ---- T = 312.5                                      
C                                                                       
      DATA (GA(11,17,JC),JC=1,3) /                                      
     S 0.46280034E+00,-0.18801305E-01, 0.00000000E+00/                  
      DATA (GB(11,17,JC),JC=1,3) /                                      
     S 0.46280034E+00, 0.14364190E+01, 0.10000000E+01/                  
      DATA (GA(11,18,JC),JC=1,3) /                                      
     S 0.37428169E+00, 0.32669120E-02, 0.00000000E+00/                  
      DATA (GB(11,18,JC),JC=1,3) /                                      
     S 0.37428169E+00, 0.12119726E+01, 0.10000000E+01/                  
C                                                                       
C-----  (CONTINUUM ABSORPTION)                                          
C                                                                       
      DATA (ETY( 1, JC),JC=1,18)/1178.11, 996.24, 285.28, 283.63,       
     S     98.88, 97.84, 39.02, 38.56, 13.01, 12.93, 7.04, 7.02,        
     S     6.46, 6.48, 26.52, 26.95, 12.92, 12.20/                      
      DATA (ETY( 2, JC),JC=1,18)/1108.77, 924.99, 284.92, 283.32,       
     S     98.58, 97.56, 38.85, 38.40, 12.97, 12.89, 7.03, 7.01,        
     S     6.47, 6.49, 26.80, 27.25, 12.19, 11.45/                      
      DATA (ETY( 3, JC),JC=1,18)/1044.99, 864.44, 284.60, 283.03,       
     S     98.32, 97.33, 38.71, 38.27, 12.94, 12.86, 7.02, 7.00,        
     S     6.48, 6.50, 27.05, 27.51, 11.52, 10.77/                      
      DATA (ETY( 4, JC),JC=1,18)/ 987.37, 813.88, 284.32, 282.76,       
     S     98.09, 97.13, 38.59, 38.15, 12.92, 12.83, 7.02, 6.99,        
     S     6.49, 6.50, 27.28, 27.74, 10.92, 10.16/                      
      DATA (ETY( 5, JC),JC=1,18)/ 935.91, 772.35, 284.08, 282.52,       
     S     97.90, 96.95, 38.48, 38.05, 12.89, 12.82, 7.01, 6.99,        
     S     6.49, 6.51, 27.48, 27.95, 10.37, 9.61/                       
      DATA (ETY( 6, JC),JC=1,18)/ 890.38, 738.66, 283.87, 282.33,       
     S     97.72, 96.80, 38.38, 37.96, 12.87, 12.80, 7.01, 6.98,        
     S     6.50, 6.52, 27.67, 28.15, 9.87, 9.10/                        
      DATA (ETY( 7, JC),JC=1,18)/ 850.38, 711.54, 283.69, 282.15,       
     S     97.57, 96.67, 38.30, 37.89, 12.85, 12.78, 6.99, 6.97,        
     S     6.50, 6.53, 27.84, 28.32, 9.42, 8.65/                        
      DATA (ETY( 8, JC),JC=1,18)/ 815.50, 689.77, 283.52, 282.00,       
     S     97.43, 96.55, 38.22, 37.82, 12.83, 12.76, 6.99, 6.97,        
     S     6.51, 6.53, 28.00, 28.49, 9.00, 8.23/                        
      DATA (ETY( 9, JC),JC=1,18)/ 785.25, 672.23, 283.27, 281.87,       
     S     97.30, 96.44, 38.15, 37.75, 12.82, 12.74, 6.99, 6.96,        
     S     6.51, 6.53, 28.15, 28.64, 8.62, 7.85/                        
      DATA (ETY(10, JC),JC=1,18)/ 759.14, 658.02, 283.23, 281.75,       
     S     97.19, 96.35, 38.09, 37.70, 12.81, 12.73, 6.98, 6.96,        
     S     6.52, 6.54, 28.28, 28.78, 8.27, 7.50/                        
      DATA (ETY(11, JC),JC=1,18)/ 736.69, 646.40, 283.09, 281.64,       
     S     97.09, 96.27, 38.03, 37.65, 12.79, 12.72, 6.98, 6.95,        
     S     6.52, 6.54, 28.41, 28.90, 7.94, 7.19/                        
C                                                                       
      DATA (PTY( 1, JC),JC=1,18)/76.577, 68.422, 6.982, 6.864,          
     S     0.586, 0.565, 0.043, 0.042, 0.017, 0.017, 0.017, 0.017,      
     S     0.038, 0.039, 0.684, 0.709, 0.852, 0.800/                    
      DATA (PTY( 2, JC),JC=1,18)/73.332, 70.449, 6.955, 6.842,          
     S     0.580, 0.560, 0.043, 0.041, 0.017, 0.017, 0.017, 0.017,      
     S     0.039, 0.039, 0.700, 0.725, 0.800, 0.748/                    
      DATA (PTY( 3, JC),JC=1,18)/62.687, 67.895, 6.933, 6.823,          
     S     0.575, 0.555, 0.042, 0.041, 0.017, 0.017, 0.017, 0.017,      
     S     0.039, 0.040, 0.714, 0.740, 0.753, 0.701/                    
      DATA (PTY( 4, JC),JC=1,18)/60.501, 65.641, 6.913, 6.807,          
     S     0.571, 0.552, 0.042, 0.041, 0.017, 0.017, 0.017, 0.017,      
     S     0.039, 0.040, 0.727, 0.753, 0.711, 0.660/                    
      DATA (PTY( 5, JC),JC=1,18)/58.705, 63.661, 6.896, 6.793,          
     S     0.567, 0.548, 0.042, 0.040, 0.017, 0.017, 0.017, 0.017,      
     S     0.040, 0.040, 0.739, 0.765, 0.674, 0.623/                    
      DATA (PTY( 6, JC),JC=1,18)/57.245, 61.930, 6.881, 6.781,          
     S     0.563, 0.545, 0.041, 0.040, 0.017, 0.017, 0.017, 0.017,      
     S     0.040, 0.041, 0.749, 0.776, 0.641, 0.590/                    
      DATA (PTY( 7, JC),JC=1,18)/56.066, 60.424, 6.868, 6.771,          
     S     0.560, 0.543, 0.041, 0.040, 0.017, 0.017, 0.017, 0.017,      
     S     0.040, 0.041, 0.759, 0.786, 0.611, 0.561/                    
      DATA (PTY( 8, JC),JC=1,18)/65.313, 55.114, 6.856, 6.762,          
     S     0.558, 0.540, 0.041, 0.039, 0.017, 0.017, 0.017, 0.017,      
     S     0.040, 0.041, 0.768, 0.795, 0.584, 0.534/                    
      DATA (PTY( 9, JC),JC=1,18)/59.119, 54.342, 6.845, 6.754,          
     S     0.555, 0.538, 0.041, 0.039, 0.017, 0.017, 0.017, 0.017,      
     S     0.040, 0.041, 0.776, 0.804, 0.559, 0.510/                    
      DATA (PTY(10, JC),JC=1,18)/57.994, 53.713, 6.836, 6.747,          
     S     0.553, 0.537, 0.040, 0.039, 0.017, 0.017, 0.017, 0.017,      
     S     0.041, 0.041, 0.784, 0.812, 0.537, 0.488/                    
      DATA (PTY(11, JC),JC=1,18)/57.025, 53.194, 6.827, 6.741,          
     S     0.551, 0.535, 0.040, 0.039, 0.017, 0.017, 0.017, 0.017,      
     S     0.041, 0.042, 0.791, 0.819, 0.516, 0.468/                    
C                                                                       
C              -----------                                              
C                                                                       
C     THE COMMON THAT CONTAINS  PLANCK FUNCTION WEIGHTED  FLUX          
C     TRANSMISSIVITIES PRECALCULATED BY THE LINE-BY-LINE MODEL          
C     GENLN2 4.0 ( D. P. EDWARDS 1995 )                                 
C     THE COEFFICIENTS OF TEMPERATURE DEPENDENCE ARE FITTED FROM        
C     LBL CALCULATED TRANSMISSION FOR 200, 250 AND 300 K.               
C     O3 TB IS ALSO IN.                                                 
C                                                                       
C     ------------------------------------------------------------      
C                                                                       
C                                                                       
C     ------------------------------------------------------------      
      DATA (PLG(JP),JP=1,31)/                                           
     S -8.,-7.5,-7.,-6.5,-6.,-5.5,-5.,-4.5,-4.,-3.5,-3.,-2.5,-2.,-1.5,  
     S -1.,-0.5, 0., 0.5, 1., 1.5, 2., 2.5, 3., 3.5,4., 4.5, 5., 5.5,   
     S  6., 6.5, 7./                                                    
C                                                                       
      DATA (ULG(JU),JU=1,40)/                                           
     S -13.,-12.5,-12.,-11.5,-11.,-10.5,-10.,-9.5,-9.,-8.5,-8.,-7.5,-7.,
     S -6.5,-6.,-5.5,-5.,-4.5,-4.,-3.5,-3.,-2.5,-2.,-1.5,-1.,-0.5, 0.,  
     S  0.5, 1., 1.5, 2., 2.5, 3., 3.5, 4., 4.5, 5., 5.5, 6.,6.5/       
C                                                                       
      DATA (ULGO(JU),JU=1,38)/                                          
     S -17.5,-17.,-16.5,-16.,-15.5,-15.,-14.5,-14.,-13.5,-13.,          
     S -12.5,-12.,-11.5,-11.,-10.5,-10.,-9.5,-9.,-8.5,-8.,              
     S -7.5,-7.,-6.5,-6.,-5.5,-5.,-4.5,-4.,-3.5,-3.,-2.5,-2.,           
     S -1.5,-1.,-0.5, 0.,0.5,1.0/                                       
C                                                                       
C----- -1,0E3*(ln(1-TB))    ( CO2 15 micron band )                      
      DATA (TB(I,1,1),I=1,40)/                                          
     S 11718,11236,10761,10297, 9848, 9419, 9017, 8647, 8315, 8020,     
     S  7758, 7517, 7281, 7045, 6805, 6564, 6329, 6104, 5890, 5689,     
     S  5496, 5309, 5127, 4950, 4777, 4607, 4440, 4276, 4115, 3956,     
     S  3801, 3649, 3501, 3358, 3220, 3087, 2958, 2832, 2708, 2586/     
      DATA (TB(I,2,1),I=1,40)/                                          
     S 11718,11236,10761,10297, 9848, 9419, 9017, 8647, 8315, 8020,     
     S  7758, 7516, 7281, 7045, 6804, 6564, 6329, 6104, 5890, 5689,     
     S  5496, 5309, 5127, 4949, 4776, 4606, 4439, 4274, 4112, 3953,     
     S  3796, 3643, 3493, 3349, 3209, 3074, 2943, 2814, 2687, 2562/     
      DATA (TB(I,3,1),I=1,40)/                                          
     S 11718,11236,10761,10297, 9848, 9419, 9017, 8647, 8315, 8020,     
     S  7758, 7516, 7281, 7045, 6804, 6564, 6329, 6103, 5890, 5688,     
     S  5495, 5308, 5126, 4948, 4774, 4604, 4436, 4271, 4108, 3947,     
     S  3789, 3634, 3482, 3336, 3194, 3056, 2922, 2791, 2661, 2533/     
      DATA (TB(I,4,1),I=1,40)/                                          
     S 11718,11236,10761,10297, 9848, 9419, 9017, 8647, 8314, 8020,     
     S  7758, 7516, 7281, 7044, 6804, 6564, 6329, 6103, 5889, 5688,     
     S  5494, 5307, 5125, 4946, 4772, 4601, 4432, 4266, 4101, 3938,     
     S  3778, 3621, 3467, 3318, 3173, 3033, 2896, 2761, 2628, 2495/     
      DATA (TB(I,5,1),I=1,40)/                                          
     S 11718,11236,10761,10297, 9848, 9419, 9017, 8647, 8314, 8020,     
     S  7758, 7516, 7281, 7044, 6804, 6563, 6328, 6102, 5889, 5687,     
     S  5493, 5305, 5122, 4943, 4768, 4596, 4426, 4257, 4091, 3926,     
     S  3763, 3603, 3446, 3294, 3147, 3003, 2862, 2723, 2585, 2448/     
      DATA (TB(I,6,1),I=1,40)/                                          
     S 11718,11236,10761,10297, 9848, 9419, 9017, 8647, 8314, 8020,     
     S  7758, 7516, 7281, 7044, 6803, 6563, 6327, 6101, 5887, 5685,     
     S  5491, 5302, 5119, 4938, 4762, 4588, 4416, 4245, 4075, 3907,     
     S  3742, 3579, 3419, 3263, 3112, 2964, 2819, 2675, 2532, 2389/     
      DATA (TB(I,7,1),I=1,40)/                                          
     S 11718,11236,10761,10297, 9848, 9419, 9016, 8646, 8314, 8020,     
     S  7757, 7515, 7280, 7043, 6802, 6562, 6326, 6100, 5885, 5682,     
     S  5487, 5298, 5113, 4931, 4752, 4576, 4401, 4227, 4054, 3882,     
     S  3713, 3546, 3383, 3224, 3068, 2916, 2766, 2616, 2468, 2319/     
      DATA (TB(I,8,1),I=1,40)/                                          
     S 11718,11236,10761,10297, 9848, 9419, 9016, 8646, 8314, 8019,     
     S  7757, 7514, 7279, 7042, 6801, 6560, 6323, 6097, 5881, 5677,     
     S  5481, 5290, 5103, 4919, 4737, 4557, 4379, 4201, 4024, 3849,     
     S  3675, 3505, 3337, 3174, 3013, 2856, 2700, 2545, 2390, 2235/     
      DATA (TB(I,9,1),I=1,40)/                                          
     S 11718,11236,10761,10297, 9848, 9419, 9016, 8646, 8313, 8018,     
     S  7756, 7513, 7277, 7039, 6798, 6557, 6320, 6092, 5876, 5670,     
     S  5472, 5279, 5089, 4901, 4715, 4531, 4348, 4166, 3985, 3805,     
     S  3627, 3452, 3280, 3111, 2946, 2782, 2620, 2459, 2298, 2138/     
      DATA (TB(I,10,1),I=1,40)/                                         
     S 11718,11235,10761,10297, 9848, 9418, 9015, 8645, 8312, 8017,     
     S  7754, 7511, 7274, 7036, 6794, 6551, 6314, 6084, 5866, 5658,     
     S  5457, 5260, 5066, 4874, 4684, 4495, 4306, 4119, 3933, 3749,     
     S  3566, 3387, 3210, 3036, 2864, 2695, 2526, 2359, 2193, 2029/     
      DATA (TB(I,11,1),I=1,40)/                                         
     S 11718,11235,10761,10297, 9847, 9418, 9015, 8644, 8310, 8015,     
     S  7751, 7507, 7270, 7030, 6787, 6543, 6304, 6072, 5851, 5640,     
     S  5435, 5233, 5034, 4836, 4640, 4445, 4252, 4060, 3869, 3679,     
     S  3492, 3307, 3125, 2945, 2768, 2592, 2418, 2246, 2077, 1911/     
      DATA (TB(I,12,1),I=1,40)/                                         
     S 11718,11235,10760,10296, 9847, 9417, 9013, 8642, 8308, 8011,     
     S  7746, 7501, 7262, 7021, 6776, 6530, 6288, 6054, 5828, 5612,     
     S  5401, 5194, 4988, 4784, 4582, 4382, 4183, 3985, 3789, 3594,     
     S  3402, 3212, 3024, 2839, 2655, 2475, 2296, 2121, 1950, 1785/     
      DATA (TB(I,13,1),I=1,40)/                                         
     S 11718,11235,10760,10296, 9846, 9415, 9011, 8639, 8303, 8005,     
     S  7738, 7491, 7250, 7007, 6759, 6510, 6264, 6024, 5794, 5571,     
     S  5354, 5139, 4927, 4716, 4508, 4302, 4097, 3894, 3693, 3493,     
     S  3296, 3101, 2907, 2717, 2529, 2344, 2163, 1987, 1817, 1655/     
      DATA (TB(I,14,1),I=1,40)/                                         
     S 11718,11235,10759,10294, 9844, 9413, 9007, 8633, 8296, 7995,     
     S  7725, 7475, 7231, 6984, 6732, 6478, 6227, 5981, 5744, 5513,     
     S  5288, 5066, 4847, 4631, 4416, 4204, 3994, 3786, 3580, 3376,     
     S  3174, 2973, 2775, 2580, 2389, 2203, 2022, 1847, 1682, 1526/     
      DATA (TB(I,15,1),I=1,40)/                                         
     S 11718,11234,10758,10293, 9841, 9409, 9001, 8625, 8284, 7980,     
     S  7706, 7450, 7201, 6949, 6691, 6432, 6173, 5920, 5674, 5436,     
     S  5203, 4974, 4748, 4526, 4306, 4088, 3873, 3661, 3450, 3242,     
     S  3035, 2831, 2629, 2432, 2239, 2053, 1875, 1705, 1547, 1400/     
      DATA (TB(I,16,1),I=1,40)/                                         
     S 11718,11233,10757,10290, 9837, 9402, 8992, 8612, 8266, 7956,     
     S  7675, 7413, 7156, 6897, 6632, 6365, 6099, 5837, 5583, 5336,     
     S  5096, 4861, 4629, 4401, 4176, 3954, 3735, 3519, 3305, 3093,     
     S  2882, 2675, 2472, 2274, 2083, 1900, 1727, 1565, 1416, 1281/     
      DATA (TB(I,17,1),I=1,40)/                                         
     S 11717,11232,10754,10286, 9831, 9393, 8978, 8592, 8239, 7920,     
     S  7630, 7357, 7091, 6823, 6550, 6275, 6000, 5731, 5468, 5215,     
     S  4968, 4727, 4491, 4258, 4030, 3805, 3582, 3363, 3145, 2930,     
     S  2718, 2509, 2306, 2111, 1924, 1747, 1583, 1431, 1294, 1171/     
      DATA (TB(I,18,1),I=1,40)/                                         
     S 11717,11231,10751,10280, 9821, 9379, 8957, 8563, 8200, 7869,     
     S  7565, 7281, 7003, 6725, 6444, 6161, 5878, 5601, 5333, 5073,     
     S  4821, 4576, 4336, 4100, 3869, 3641, 3416, 3194, 2974, 2757,     
     S  2544, 2337, 2137, 1946, 1766, 1599, 1445, 1305, 1181, 1071/     
      DATA (TB(I,19,1),I=1,40)/                                         
     S 11716,11228,10746,10272, 9808, 9359, 8929, 8524, 8147, 7800,     
     S  7480, 7181, 6891, 6603, 6314, 6024, 5736, 5453, 5179, 4914,     
     S  4658, 4410, 4167, 3930, 3696, 3466, 3239, 3015, 2794, 2577,     
     S  2366, 2163, 1968, 1785, 1614, 1458, 1317, 1191, 1080,  981/     
      DATA (TB(I,20,1),I=1,40)/                                         
     S 11715,11225,10740,10261, 9792, 9335, 8895, 8476, 8082, 7716,     
     S  7377, 7060, 6757, 6460, 6164, 5869, 5577, 5290, 5012, 4744,     
     S  4485, 4234, 3990, 3751, 3516, 3284, 3055, 2830, 2609, 2395,     
     S  2188, 1990, 1803, 1630, 1471, 1328, 1201, 1089,  989,  899/     
      DATA (TB(I,21,1),I=1,40)/                                         
     S 11714,11221,10733,10250, 9774, 9308, 8856, 8422, 8010, 7623,     
     S  7262, 6926, 6608, 6302, 6000, 5702, 5408, 5119, 4838, 4567,     
     S  4306, 4053, 3808, 3567, 3330, 3096, 2867, 2642, 2424, 2214,     
     S  2013, 1823, 1647, 1486, 1341, 1212, 1098,  997,  907,  823/     
      DATA (TB(I,22,1),I=1,40)/                                         
     S 11712,11217,10726,10238, 9756, 9282, 8819, 8370, 7939, 7530,     
     S  7147, 6790, 6455, 6137, 5830, 5530, 5235, 4946, 4664, 4391,     
     S  4128, 3873, 3624, 3381, 3142, 2907, 2678, 2456, 2242, 2038,     
     S  1846, 1666, 1502, 1355, 1224, 1109, 1007,  915,  831,  752/     
      DATA (TB(I,23,1),I=1,40)/                                         
     S 11710,11213,10719,10228, 9741, 9259, 8786, 8324, 7877, 7449,     
     S  7043, 6663, 6308, 5976, 5661, 5358, 5063, 4775, 4494, 4220,     
     S  3954, 3695, 3443, 3196, 2955, 2720, 2493, 2275, 2067, 1871,     
     S  1689, 1522, 1372, 1239, 1122, 1019,  926,  840,  761,  687/     
      DATA (TB(I,24,1),I=1,40)/                                         
     S 11708,11211,10714,10220, 9729, 9242, 8762, 8290, 7829, 7383,     
     S  6957, 6554, 6177, 5827, 5500, 5191, 4896, 4609, 4329, 4055,     
     S  3786, 3522, 3265, 3014, 2772, 2538, 2315, 2102, 1902, 1716,     
     S  1546, 1393, 1258, 1139, 1034,  939,  852,  772,  697,  629/     
      DATA (TB(I,25,1),I=1,40)/                                         
     S 11707,11209,10711,10215, 9721, 9230, 8744, 8265, 7794, 7335,     
     S  6892, 6469, 6070, 5699, 5355, 5036, 4736, 4449, 4170, 3895,     
     S  3623, 3355, 3093, 2839, 2595, 2363, 2144, 1938, 1748, 1574,     
     S  1418, 1280, 1159, 1053,  956,  868,  786,  710,  640,  576/     
      DATA (TB(I,26,1),I=1,40)/                                         
     S 11707,11208,10709,10212, 9716, 9223, 8733, 8248, 7770, 7301,     
     S  6845, 6406, 5989, 5597, 5234, 4900, 4590, 4297, 4015, 3739,     
     S  3465, 3194, 2928, 2672, 2427, 2198, 1984, 1787, 1608, 1448,     
     S  1307, 1183, 1075,  977,  887,  803,  726,  654,  589,  531/     
      DATA (TB(I,27,1),I=1,40)/                                         
     S 11707,11208,10709,10211, 9714, 9218, 8726, 8238, 7755, 7280,     
     S  6815, 6364, 5932, 5522, 5140, 4787, 4460, 4156, 3866, 3586,     
     S  3309, 3037, 2771, 2514, 2270, 2044, 1836, 1649, 1482, 1337,     
     S  1211, 1101, 1001,  910,  825,  745,  672,  605,  546,  493/     
      DATA (TB(I,28,1),I=1,40)/                                         
     S 11707,11208,10709,10210, 9713, 9216, 8722, 8232, 7746, 7266,     
     S  6795, 6336, 5893, 5470, 5071, 4699, 4353, 4031, 3727, 3438,     
     S  3157, 2884, 2619, 2364, 2124, 1903, 1703, 1527, 1374, 1242,     
     S  1129, 1029,  936,  850,  769,  693,  625,  564,  510,  461/     
      DATA (TB(I,29,1),I=1,40)/                                         
     S 11708,11209,10710,10211, 9713, 9216, 8721, 8229, 7741, 7258,     
     S  6783, 6318, 5867, 5434, 5021, 4632, 4267, 3925, 3603, 3299,     
     S  3010, 2735, 2472, 2222, 1989, 1777, 1588, 1423, 1283, 1163,     
     S  1059,  964,  877,  795,  718,  648,  585,  530,  481,  435/     
      DATA (TB(I,30,1),I=1,40)/                                         
     S 11710,11211,10711,10212, 9714, 9217, 8721, 8228, 7738, 7253,     
     S  6775, 6306, 5848, 5407, 4983, 4581, 4199, 3838, 3496, 3174,     
     S  2873, 2592, 2330, 2087, 1864, 1665, 1490, 1340, 1211, 1099,     
     S   998,  907,  823,  745,  673,  609,  553,  503,  457,  412/     
      DATA (TB(I,31,1),I=1,40)/                                         
     S 11713,11214,10714,10215, 9717, 9219, 8723, 8228, 7737, 7250,     
     S  6769, 6296, 5833, 5384, 4952, 4538, 4143, 3766, 3407, 3067,     
     S  2751, 2461, 2198, 1962, 1752, 1569, 1412, 1277, 1157, 1048,     
     S   948,  857,  774,  700,  635,  578,  528,  482,  438,  393/     
C----- -1,0E3*(ln(1-TDB))                                               
      DATA (TB(I,1,2),I=1,40)/                                          
     S 11682,11199,10724,10261, 9812, 9383, 8980, 8610, 8278, 7984,     
     S  7722, 7480, 7245, 7009, 6769, 6529, 6295, 6070, 5857, 5655,     
     S  5463, 5276, 5094, 4917, 4744, 4575, 4408, 4244, 4083, 3924,     
     S  3769, 3617, 3470, 3328, 3190, 3058, 2929, 2803, 2679, 2557/     
      DATA (TB(I,2,2),I=1,40)/                                          
     S 11682,11199,10724,10261, 9812, 9383, 8980, 8610, 8278, 7984,     
     S  7722, 7480, 7245, 7009, 6769, 6529, 6294, 6069, 5856, 5655,     
     S  5462, 5276, 5094, 4916, 4743, 4573, 4407, 4242, 4080, 3921,     
     S  3764, 3611, 3463, 3318, 3179, 3044, 2913, 2785, 2658, 2533/     
      DATA (TB(I,3,2),I=1,40)/                                          
     S 11682,11199,10724,10261, 9812, 9383, 8980, 8610, 8278, 7984,     
     S  7722, 7480, 7245, 7009, 6769, 6529, 6294, 6069, 5856, 5655,     
     S  5462, 5275, 5093, 4915, 4742, 4571, 4404, 4239, 4076, 3915,     
     S  3757, 3602, 3452, 3305, 3164, 3026, 2893, 2761, 2632, 2504/     
      DATA (TB(I,4,2),I=1,40)/                                          
     S 11682,11199,10724,10261, 9812, 9383, 8980, 8610, 8278, 7984,     
     S  7721, 7480, 7245, 7008, 6769, 6529, 6294, 6069, 5856, 5654,     
     S  5461, 5274, 5092, 4913, 4739, 4568, 4400, 4233, 4069, 3906,     
     S  3746, 3590, 3436, 3288, 3143, 3003, 2866, 2731, 2598, 2466/     
      DATA (TB(I,5,2),I=1,40)/                                          
     S 11682,11199,10724,10261, 9812, 9383, 8980, 8610, 8278, 7983,     
     S  7721, 7480, 7245, 7008, 6768, 6528, 6293, 6068, 5855, 5653,     
     S  5460, 5272, 5089, 4910, 4735, 4563, 4393, 4225, 4059, 3894,     
     S  3731, 3572, 3415, 3264, 3116, 2973, 2832, 2693, 2555, 2418/     
      DATA (TB(I,6,2),I=1,40)/                                          
     S 11682,11199,10724,10261, 9812, 9383, 8980, 8610, 8278, 7983,     
     S  7721, 7479, 7244, 7008, 6768, 6528, 6293, 6067, 5853, 5651,     
     S  5457, 5269, 5086, 4906, 4729, 4555, 4383, 4213, 4043, 3875,     
     S  3710, 3547, 3388, 3233, 3081, 2934, 2789, 2645, 2502, 2359/     
      DATA (TB(I,7,2),I=1,40)/                                          
     S 11682,11199,10724,10261, 9812, 9383, 8980, 8610, 8277, 7983,     
     S  7721, 7479, 7244, 7007, 6767, 6527, 6291, 6065, 5851, 5648,     
     S  5454, 5265, 5080, 4898, 4719, 4543, 4368, 4194, 4022, 3850,     
     S  3681, 3515, 3352, 3193, 3037, 2885, 2735, 2586, 2437, 2288/     
      DATA (TB(I,8,2),I=1,40)/                                          
     S 11682,11199,10724,10261, 9812, 9382, 8980, 8610, 8277, 7982,     
     S  7720, 7478, 7243, 7006, 6765, 6525, 6289, 6062, 5848, 5644,     
     S  5448, 5257, 5070, 4886, 4705, 4525, 4346, 4168, 3992, 3817,     
     S  3643, 3473, 3306, 3142, 2982, 2825, 2669, 2514, 2359, 2204/     
      DATA (TB(I,9,2),I=1,40)/                                          
     S 11682,11199,10724,10260, 9811, 9382, 8979, 8609, 8276, 7982,     
     S  7719, 7476, 7241, 7004, 6763, 6522, 6285, 6058, 5842, 5636,     
     S  5439, 5245, 5055, 4868, 4682, 4498, 4315, 4133, 3952, 3772,     
     S  3595, 3420, 3248, 3080, 2914, 2751, 2589, 2428, 2267, 2107/     
      DATA (TB(I,10,2),I=1,40)/                                         
     S 11682,11199,10724,10260, 9811, 9382, 8979, 8608, 8275, 7980,     
     S  7717, 7474, 7238, 7000, 6758, 6517, 6279, 6050, 5832, 5625,     
     S  5424, 5227, 5033, 4841, 4651, 4462, 4274, 4087, 3901, 3716,     
     S  3534, 3355, 3178, 3004, 2833, 2663, 2495, 2328, 2162, 1998/     
      DATA (TB(I,11,2),I=1,40)/                                         
     S 11682,11199,10724,10260, 9811, 9381, 8978, 8607, 8274, 7978,     
     S  7714, 7470, 7233, 6994, 6752, 6508, 6269, 6038, 5818, 5606,     
     S  5401, 5200, 5001, 4803, 4607, 4413, 4219, 4027, 3836, 3646,     
     S  3459, 3275, 3093, 2913, 2736, 2560, 2386, 2214, 2045, 1879/     
      DATA (TB(I,12,2),I=1,40)/                                         
     S 11682,11199,10724,10260, 9810, 9380, 8977, 8605, 8271, 7974,     
     S  7709, 7464, 7226, 6985, 6741, 6495, 6253, 6019, 5794, 5578,     
     S  5368, 5160, 4955, 4751, 4549, 4349, 4150, 3952, 3756, 3561,     
     S  3369, 3179, 2992, 2807, 2623, 2442, 2264, 2089, 1918, 1753/     
      DATA (TB(I,13,2),I=1,40)/                                         
     S 11682,11199,10723,10259, 9809, 9379, 8974, 8602, 8266, 7968,     
     S  7701, 7454, 7214, 6971, 6723, 6475, 6229, 5990, 5760, 5537,     
     S  5320, 5105, 4893, 4683, 4475, 4269, 4064, 3861, 3659, 3460,     
     S  3263, 3068, 2875, 2684, 2496, 2311, 2131, 1955, 1785, 1623/     
      DATA (TB(I,14,2),I=1,40)/                                         
     S 11682,11198,10723,10258, 9807, 9376, 8971, 8597, 8259, 7958,     
     S  7689, 7439, 7194, 6948, 6696, 6443, 6192, 5947, 5710, 5479,     
     S  5254, 5033, 4814, 4597, 4383, 4171, 3961, 3752, 3546, 3342,     
     S  3140, 2940, 2743, 2548, 2356, 2170, 1989, 1815, 1649, 1493/     
      DATA (TB(I,15,2),I=1,40)/                                         
     S 11681,11198,10722,10256, 9805, 9372, 8965, 8588, 8248, 7943,     
     S  7669, 7414, 7165, 6913, 6656, 6396, 6138, 5886, 5640, 5401,     
     S  5169, 4940, 4714, 4492, 4272, 4054, 3840, 3627, 3417, 3208,     
     S  3002, 2798, 2596, 2399, 2206, 2020, 1842, 1672, 1514, 1367/     
      DATA (TB(I,16,2),I=1,40)/                                         
     S 11681,11197,10720,10253, 9800, 9366, 8955, 8575, 8230, 7919,     
     S  7638, 7376, 7120, 6861, 6597, 6330, 6064, 5802, 5548, 5302,     
     S  5062, 4826, 4595, 4367, 4142, 3921, 3702, 3485, 3271, 3059,     
     S  2849, 2642, 2438, 2241, 2050, 1867, 1694, 1532, 1383, 1248/     
      DATA (TB(I,17,2),I=1,40)/                                         
     S 11681,11196,10718,10249, 9794, 9356, 8941, 8555, 8202, 7883,     
     S  7593, 7321, 7055, 6787, 6515, 6240, 5965, 5696, 5434, 5180,     
     S  4934, 4693, 4457, 4224, 3996, 3771, 3549, 3329, 3111, 2896,     
     S  2684, 2476, 2273, 2078, 1891, 1714, 1549, 1398, 1261, 1138/     
      DATA (TB(I,18,2),I=1,40)/                                         
     S 11680,11194,10714,10243, 9784, 9342, 8921, 8526, 8163, 7832,     
     S  7529, 7244, 6967, 6689, 6408, 6125, 5843, 5566, 5298, 5038,     
     S  4787, 4541, 4301, 4066, 3835, 3607, 3382, 3160, 2940, 2723,     
     S  2510, 2303, 2103, 1913, 1733, 1565, 1411, 1272, 1148, 1038/     
      DATA (TB(I,19,2),I=1,40)/                                         
     S 11680,11191,10709,10235, 9772, 9322, 8893, 8487, 8110, 7763,     
     S  7444, 7144, 6854, 6567, 6278, 5988, 5700, 5418, 5144, 4879,     
     S  4624, 4375, 4133, 3895, 3662, 3432, 3205, 2981, 2760, 2543,     
     S  2332, 2129, 1934, 1751, 1581, 1424, 1283, 1158, 1047,  949/     
      DATA (TB(I,20,2),I=1,40)/                                         
     S 11679,11188,10703,10225, 9755, 9298, 8858, 8439, 8045, 7679,     
     S  7340, 7024, 6720, 6424, 6128, 5833, 5541, 5255, 4977, 4709,     
     S  4450, 4199, 3955, 3716, 3481, 3249, 3021, 2795, 2575, 2360,     
     S  2153, 1956, 1769, 1596, 1437, 1294, 1168, 1055,  956,  866/     
      DATA (TB(I,21,2),I=1,40)/                                         
     S 11677,11185,10696,10213, 9737, 9271, 8819, 8385, 7973, 7586,     
     S  7226, 6890, 6572, 6265, 5964, 5666, 5372, 5084, 4803, 4532,     
     S  4271, 4019, 3773, 3532, 3295, 3062, 2832, 2608, 2390, 2179,     
     S  1979, 1789, 1613, 1452, 1307, 1178, 1065,  964,  874,  790/     
      DATA (TB(I,22,2),I=1,40)/                                         
     S 11675,11180,10689,10201, 9719, 9245, 8782, 8333, 7902, 7494,     
     S  7110, 6753, 6419, 6101, 5794, 5494, 5199, 4910, 4629, 4356,     
     S  4093, 3838, 3590, 3346, 3107, 2873, 2644, 2422, 2208, 2004,     
     S  1811, 1632, 1468, 1321, 1190, 1075,  974,  882,  798,  720/     
      DATA (TB(I,23,2),I=1,40)/                                         
     S 11673,11176,10682,10191, 9704, 9222, 8749, 8287, 7840, 7412,     
     S  7006, 6626, 6271, 5939, 5624, 5321, 5027, 4739, 4458, 4185,     
     S  3918, 3660, 3407, 3161, 2920, 2686, 2459, 2240, 2033, 1837,     
     S  1654, 1488, 1338, 1205, 1089,  985,  893,  807,  728,  655/     
      DATA (TB(I,24,2),I=1,40)/                                         
     S 11671,11173,10677,10183, 9692, 9205, 8725, 8253, 7792, 7346,     
     S  6920, 6517, 6141, 5790, 5463, 5155, 4859, 4573, 4293, 4019,     
     S  3750, 3487, 3229, 2979, 2737, 2503, 2280, 2067, 1867, 1681,     
     S  1511, 1359, 1223, 1105, 1000,  906,  819,  739,  664,  596/     
      DATA (TB(I,25,2),I=1,40)/                                         
     S 11670,11172,10674,10178, 9684, 9193, 8707, 8228, 7757, 7298,     
     S  6855, 6432, 6033, 5662, 5319, 5000, 4700, 4413, 4134, 3859,     
     S  3587, 3319, 3057, 2803, 2560, 2328, 2109, 1903, 1713, 1539,     
     S  1383, 1246, 1125, 1018,  922,  834,  752,  677,  607,  544/     
      DATA (TB(I,26,2),I=1,40)/                                         
     S 11670,11171,10672,10175, 9679, 9186, 8696, 8211, 7733, 7264,     
     S  6808, 6369, 5952, 5560, 5197, 4863, 4553, 4260, 3979, 3702,     
     S  3429, 3158, 2893, 2636, 2392, 2162, 1948, 1751, 1573, 1413,     
     S  1272, 1149, 1040,  943,  853,  769,  692,  621,  556,  499/     
      DATA (TB(I,27,2),I=1,40)/                                         
     S 11670,11170,10672,10173, 9677, 9181, 8689, 8200, 7718, 7242,     
     S  6778, 6327, 5895, 5485, 5103, 4750, 4424, 4119, 3830, 3549,     
     S  3273, 3001, 2735, 2478, 2234, 2008, 1800, 1613, 1447, 1302,     
     S  1176, 1066,  967,  876,  790,  711,  638,  572,  513,  461/     
      DATA (TB(I,28,2),I=1,40)/                                         
     S 11670,11171,10672,10173, 9676, 9179, 8685, 8195, 7709, 7229,     
     S  6758, 6299, 5856, 5433, 5034, 4662, 4316, 3994, 3691, 3401,     
     S  3121, 2848, 2583, 2328, 2088, 1867, 1667, 1491, 1338, 1207,     
     S  1094,  994,  901,  815,  734,  659,  591,  530,  477,  430/     
      DATA (TB(I,29,2),I=1,40)/                                         
     S 11671,11172,10673,10174, 9676, 9179, 8684, 8192, 7704, 7221,     
     S  6746, 6281, 5830, 5397, 4984, 4595, 4230, 3888, 3566, 3262,     
     S  2973, 2698, 2435, 2186, 1953, 1741, 1552, 1388, 1247, 1128,     
     S  1024,  929,  842,  760,  684,  614,  551,  497,  448,  403/     
      DATA (TB(I,30,2),I=1,40)/                                         
     S 11673,11174,10674,10175, 9677, 9180, 8684, 8191, 7701, 7216,     
     S  6738, 6269, 5811, 5369, 4946, 4543, 4162, 3800, 3458, 3137,     
     S  2836, 2555, 2294, 2050, 1828, 1629, 1454, 1304, 1175, 1063,     
     S   963,  872,  788,  710,  639,  575,  519,  470,  425,  381/     
      DATA (TB(I,31,2),I=1,40)/                                         
     S 11676,11177,10677,10178, 9680, 9182, 8686, 8191, 7700, 7213,     
     S  6732, 6259, 5796, 5347, 4914, 4501, 4106, 3729, 3370, 3030,     
     S  2714, 2424, 2161, 1925, 1715, 1533, 1376, 1241, 1121, 1012,     
     S   912,  821,  739,  666,  600,  544,  494,  449,  405,  361/     
C                                                                       
C----- 1.,0E4*ALP(I,J,1) FOR TB                                         
      DATA (ALP(I, 1,1),I=1,40)/                                        
     S   1.,   1.,   3.,   4.,   6.,   9.,  13.,  18.,  24.,  32.,      
     S  39.,  47.,  54.,  59.,  63.,  66.,  67.,  68.,  69.,  70.,      
     S  72.,  74.,  76.,  79.,  81.,  84.,  86.,  88.,  90.,  91.,      
     S  92.,  93.,  93.,  92.,  92.,  92.,  91.,  90.,  89.,  88./      
      DATA (ALP(I, 2,1),I=1,40)/                                        
     S   1.,   1.,   3.,   4.,   6.,   9.,  13.,  18.,  24.,  32.,      
     S  39.,  47.,  54.,  59.,  63.,  66.,  67.,  68.,  69.,  70.,      
     S  72.,  74.,  76.,  79.,  81.,  83.,  86.,  88.,  89.,  91.,      
     S  92.,  92.,  92.,  91.,  91.,  90.,  89.,  89.,  88.,  86./      
      DATA (ALP(I, 3,1),I=1,40)/                                        
     S   1.,   1.,   3.,   4.,   6.,   9.,  13.,  18.,  24.,  32.,      
     S  39.,  47.,  54.,  59.,  63.,  66.,  67.,  68.,  69.,  70.,      
     S  72.,  74.,  76.,  79.,  81.,  83.,  85.,  87.,  89.,  90.,      
     S  91.,  91.,  90.,  90.,  89.,  88.,  88.,  87.,  85.,  83./      
      DATA (ALP(I, 4,1),I=1,40)/                                        
     S   1.,   1.,   3.,   4.,   6.,   9.,  13.,  18.,  24.,  32.,      
     S  39.,  47.,  54.,  59.,  63.,  66.,  67.,  68.,  69.,  70.,      
     S  72.,  74.,  76.,  78.,  81.,  83.,  85.,  87.,  88.,  89.,      
     S  89.,  89.,  89.,  88.,  87.,  86.,  85.,  84.,  82.,  80./      
      DATA (ALP(I, 5,1),I=1,40)/                                        
     S   1.,   1.,   3.,   4.,   6.,   9.,  13.,  18.,  24.,  32.,      
     S  39.,  47.,  54.,  59.,  63.,  66.,  67.,  68.,  69.,  70.,      
     S  72.,  74.,  76.,  78.,  80.,  82.,  84.,  86.,  87.,  87.,      
     S  88.,  87.,  87.,  86.,  85.,  83.,  82.,  81.,  79.,  77./      
      DATA (ALP(I, 6,1),I=1,40)/                                        
     S   1.,   1.,   3.,   4.,   6.,   9.,  13.,  18.,  24.,  32.,      
     S  39.,  47.,  54.,  59.,  63.,  66.,  67.,  68.,  68.,  70.,      
     S  71.,  73.,  75.,  77.,  79.,  81.,  83.,  84.,  85.,  85.,      
     S  85.,  85.,  84.,  83.,  82.,  80.,  79.,  77.,  75.,  72./      
      DATA (ALP(I, 7,1),I=1,40)/                                        
     S   1.,   1.,   3.,   4.,   6.,   9.,  13.,  18.,  24.,  32.,      
     S  39.,  47.,  54.,  59.,  63.,  66.,  67.,  68.,  68.,  69.,      
     S  71.,  73.,  75.,  76.,  78.,  80.,  81.,  82.,  83.,  83.,      
     S  83.,  82.,  81.,  80.,  78.,  77.,  75.,  73.,  71.,  68./      
      DATA (ALP(I, 8,1),I=1,40)/                                        
     S   1.,   1.,   3.,   4.,   6.,   9.,  13.,  18.,  24.,  32.,      
     S  39.,  47.,  53.,  59.,  63.,  65.,  67.,  67.,  68.,  69.,      
     S  70.,  72.,  74.,  75.,  77.,  78.,  79.,  80.,  80.,  80.,      
     S  80.,  79.,  77.,  76.,  74.,  73.,  71.,  69.,  66.,  63./      
      DATA (ALP(I, 9,1),I=1,40)/                                        
     S   1.,   1.,   3.,   4.,   6.,   9.,  13.,  18.,  24.,  32.,      
     S  39.,  47.,  53.,  59.,  63.,  65.,  66.,  67.,  67.,  68.,      
     S  69.,  71.,  72.,  73.,  74.,  75.,  76.,  76.,  77.,  76.,      
     S  76.,  75.,  73.,  72.,  70.,  68.,  66.,  64.,  61.,  58./      
      DATA (ALP(I,10,1),I=1,40)/                                        
     S   1.,   1.,   3.,   4.,   6.,   9.,  13.,  18.,  24.,  31.,      
     S  39.,  46.,  53.,  58.,  62.,  65.,  66.,  66.,  66.,  67.,      
     S  68.,  69.,  70.,  71.,  71.,  72.,  73.,  73.,  73.,  72.,      
     S  72.,  71.,  69.,  67.,  65.,  63.,  61.,  59.,  56.,  53./      
      DATA (ALP(I,11,1),I=1,40)/                                        
     S   1.,   1.,   2.,   4.,   6.,   9.,  13.,  18.,  24.,  31.,      
     S  39.,  46.,  52.,  58.,  61.,  64.,  65.,  65.,  65.,  65.,      
     S  65.,  66.,  67.,  67.,  68.,  68.,  69.,  69.,  68.,  68.,      
     S  67.,  66.,  64.,  62.,  60.,  58.,  56.,  54.,  51.,  49./      
      DATA (ALP(I,12,1),I=1,40)/                                        
     S   1.,   1.,   2.,   4.,   6.,   9.,  13.,  18.,  24.,  31.,      
     S  38.,  45.,  52.,  57.,  60.,  62.,  63.,  63.,  62.,  62.,      
     S  62.,  63.,  63.,  63.,  64.,  64.,  64.,  64.,  64.,  63.,      
     S  62.,  61.,  59.,  57.,  55.,  53.,  51.,  49.,  47.,  44./      
      DATA (ALP(I,13,1),I=1,40)/                                        
     S   1.,   1.,   2.,   4.,   6.,   9.,  13.,  17.,  23.,  30.,      
     S  38.,  44.,  50.,  55.,  58.,  60.,  61.,  60.,  59.,  59.,      
     S  59.,  59.,  59.,  59.,  59.,  59.,  59.,  59.,  59.,  58.,      
     S  57.,  56.,  54.,  52.,  50.,  48.,  47.,  45.,  43.,  41./      
      DATA (ALP(I,14,1),I=1,40)/                                        
     S   1.,   1.,   2.,   4.,   6.,   9.,  12.,  17.,  23.,  29.,      
     S  36.,  43.,  49.,  53.,  56.,  57.,  57.,  56.,  55.,  54.,      
     S  54.,  54.,  54.,  54.,  54.,  54.,  55.,  54.,  54.,  53.,      
     S  52.,  51.,  50.,  48.,  46.,  44.,  43.,  41.,  39.,  38./      
      DATA (ALP(I,15,1),I=1,40)/                                        
     S   0.,   1.,   2.,   4.,   6.,   8.,  12.,  16.,  22.,  28.,      
     S  35.,  41.,  46.,  50.,  52.,  53.,  53.,  52.,  51.,  50.,      
     S  49.,  49.,  49.,  49.,  49.,  50.,  50.,  50.,  49.,  49.,      
     S  48.,  47.,  45.,  44.,  42.,  41.,  39.,  38.,  37.,  36./      
      DATA (ALP(I,16,1),I=1,40)/                                        
     S   0.,   1.,   2.,   3.,   5.,   8.,  11.,  15.,  20.,  26.,      
     S  32.,  37.,  42.,  45.,  47.,  48.,  47.,  46.,  45.,  44.,      
     S  44.,  44.,  44.,  44.,  45.,  45.,  45.,  45.,  45.,  44.,      
     S  44.,  42.,  41.,  40.,  39.,  37.,  36.,  36.,  35.,  34./      
      DATA (ALP(I,17,1),I=1,40)/                                        
     S   0.,   1.,   2.,   3.,   5.,   7.,  10.,  13.,  18.,  23.,      
     S  28.,  33.,  37.,  40.,  41.,  42.,  42.,  41.,  40.,  39.,      
     S  39.,  39.,  40.,  40.,  41.,  41.,  41.,  41.,  41.,  41.,      
     S  40.,  39.,  38.,  37.,  36.,  35.,  34.,  34.,  34.,  34./      
      DATA (ALP(I,18,1),I=1,40)/                                        
     S   0.,   1.,   1.,   2.,   4.,   6.,   8.,  11.,  15.,  19.,      
     S  24.,  28.,  31.,  33.,  35.,  36.,  36.,  36.,  35.,  35.,      
     S  35.,  35.,  36.,  37.,  37.,  38.,  38.,  38.,  38.,  37.,      
     S  37.,  36.,  35.,  34.,  34.,  33.,  33.,  33.,  33.,  33./      
      DATA (ALP(I,19,1),I=1,40)/                                        
     S   0.,   1.,   1.,   2.,   3.,   4.,   6.,   9.,  12.,  15.,      
     S  19.,  22.,  25.,  27.,  29.,  30.,  31.,  31.,  31.,  31.,      
     S  32.,  32.,  33.,  34.,  34.,  35.,  35.,  35.,  35.,  35.,      
     S  34.,  34.,  33.,  33.,  32.,  32.,  32.,  32.,  33.,  33./      
      DATA (ALP(I,20,1),I=1,40)/                                        
     S   0.,   0.,   1.,   1.,   2.,   3.,   4.,   6.,   8.,  11.,      
     S  13.,  16.,  19.,  21.,  23.,  25.,  27.,  28.,  28.,  29.,      
     S  29.,  30.,  30.,  31.,  32.,  32.,  33.,  33.,  33.,  32.,      
     S  32.,  32.,  32.,  31.,  31.,  32.,  32.,  32.,  33.,  33./      
      DATA (ALP(I,21,1),I=1,40)/                                        
     S   0.,   0.,   0.,   1.,   1.,   2.,   3.,   4.,   5.,   7.,      
     S   9.,  11.,  14.,  16.,  19.,  21.,  23.,  25.,  26.,  27.,      
     S  27.,  28.,  29.,  29.,  30.,  30.,  31.,  31.,  31.,  31.,      
     S  31.,  31.,  31.,  31.,  31.,  31.,  32.,  32.,  32.,  32./      
      DATA (ALP(I,22,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   1.,   1.,   1.,   2.,   3.,   4.,      
     S   6.,   8.,  10.,  12.,  15.,  18.,  21.,  23.,  25.,  26.,      
     S  26.,  27.,  27.,  28.,  28.,  29.,  29.,  29.,  29.,  30.,      
     S  30.,  30.,  30.,  30.,  31.,  32.,  32.,  32.,  32.,  31./      
      DATA (ALP(I,23,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   2.,   2.,      
     S   4.,   5.,   7.,   9.,  12.,  15.,  18.,  21.,  23.,  25.,      
     S  26.,  26.,  26.,  27.,  27.,  27.,  27.,  28.,  28.,  28.,      
     S  29.,  29.,  30.,  31.,  31.,  32.,  32.,  31.,  31.,  30./      
      DATA (ALP(I,24,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   2.,      
     S   2.,   4.,   5.,   7.,  10.,  13.,  16.,  19.,  21.,  23.,      
     S  24.,  25.,  25.,  26.,  26.,  26.,  26.,  27.,  27.,  28.,      
     S  28.,  29.,  30.,  31.,  31.,  32.,  31.,  31.,  30.,  29./      
      DATA (ALP(I,25,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,      
     S   2.,   3.,   4.,   6.,   8.,  10.,  13.,  16.,  19.,  21.,      
     S  23.,  24.,  24.,  25.,  25.,  25.,  25.,  25.,  26.,  27.,      
     S  28.,  29.,  30.,  31.,  31.,  31.,  31.,  30.,  29.,  28./      
      DATA (ALP(I,26,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,      
     S   2.,   2.,   4.,   5.,   7.,   9.,  11.,  14.,  16.,  19.,      
     S  21.,  22.,  23.,  24.,  24.,  24.,  24.,  25.,  26.,  27.,      
     S  28.,  29.,  30.,  31.,  31.,  30.,  30.,  29.,  28.,  27./      
      DATA (ALP(I,27,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,      
     S   2.,   2.,   3.,   5.,   6.,   8.,  10.,  12.,  14.,  16.,      
     S  18.,  20.,  21.,  22.,  23.,  23.,  24.,  24.,  25.,  27.,      
     S  28.,  29.,  30.,  30.,  30.,  30.,  29.,  28.,  27.,  26./      
      DATA (ALP(I,28,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,      
     S   1.,   2.,   3.,   4.,   6.,   7.,   8.,  10.,  11.,  13.,      
     S  15.,  17.,  19.,  20.,  21.,  22.,  23.,  24.,  26.,  27.,      
     S  28.,  29.,  29.,  29.,  29.,  28.,  28.,  27.,  26.,  25./      
      DATA (ALP(I,29,1),I=1,40)/                                        
     S  -1.,  -1.,  -1.,  -1.,   0.,   0.,   0.,   0.,   0.,   1.,      
     S   1.,   2.,   3.,   4.,   5.,   6.,   7.,   8.,   9.,  10.,      
     S  12.,  14.,  16.,  18.,  20.,  21.,  23.,  25.,  26.,  27.,      
     S  28.,  28.,  28.,  28.,  28.,  27.,  26.,  25.,  25.,  25./      
      DATA (ALP(I,30,1),I=1,40)/                                        
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   0.,   1.,   2.,   2.,   3.,   4.,   5.,   6.,   6.,   7.,      
     S   9.,  10.,  13.,  15.,  18.,  20.,  23.,  26.,  27.,  28.,      
     S  28.,  28.,  27.,  27.,  26.,  25.,  25.,  24.,  25.,  25./      
      DATA (ALP(I,31,1),I=1,40)/                                        
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S   0.,   0.,   1.,   1.,   2.,   2.,   3.,   4.,   4.,   5.,      
     S   6.,   8.,  10.,  13.,  17.,  20.,  24.,  27.,  28.,  29.,      
     S  28.,  28.,  26.,  25.,  24.,  24.,  23.,  24.,  24.,  25./      
C----- 1.,0E6*BET(I,J,1) FOR TB                                         
      DATA (BET(I, 1,1),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  14.,      
     S  16.,  18.,  19.,  20.,  21.,  24.,  26.,  28.,  29.,  30.,      
     S  29.,  29.,  27.,  26.,  26.,  25.,  26.,  26.,  25.,  23./      
      DATA (BET(I, 2,1),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  14.,      
     S  16.,  18.,  19.,  20.,  21.,  24.,  26.,  28.,  29.,  29.,      
     S  29.,  28.,  27.,  26.,  25.,  25.,  25.,  25.,  24.,  23./      
      DATA (BET(I, 3,1),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  14.,      
     S  16.,  18.,  19.,  20.,  21.,  24.,  26.,  28.,  29.,  29.,      
     S  29.,  28.,  27.,  26.,  25.,  25.,  25.,  24.,  24.,  22./      
      DATA (BET(I, 4,1),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  14.,      
     S  16.,  18.,  19.,  20.,  21.,  23.,  26.,  28.,  29.,  29.,      
     S  29.,  28.,  27.,  25.,  24.,  24.,  24.,  24.,  23.,  21./      
      DATA (BET(I, 5,1),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  14.,      
     S  16.,  18.,  19.,  20.,  21.,  23.,  26.,  27.,  28.,  29.,      
     S  28.,  27.,  26.,  25.,  24.,  23.,  23.,  23.,  22.,  20./      
      DATA (BET(I, 6,1),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  14.,      
     S  16.,  18.,  19.,  20.,  21.,  23.,  25.,  27.,  28.,  28.,      
     S  28.,  27.,  25.,  24.,  23.,  22.,  22.,  22.,  21.,  19./      
      DATA (BET(I, 7,1),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  14.,      
     S  16.,  18.,  19.,  20.,  21.,  23.,  25.,  27.,  27.,  27.,      
     S  27.,  26.,  24.,  23.,  22.,  21.,  21.,  20.,  19.,  17./      
      DATA (BET(I, 8,1),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  14.,      
     S  16.,  18.,  19.,  20.,  21.,  23.,  25.,  26.,  26.,  26.,      
     S  26.,  24.,  23.,  21.,  20.,  20.,  19.,  19.,  17.,  16./      
      DATA (BET(I, 9,1),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  14.,      
     S  16.,  18.,  18.,  19.,  21.,  22.,  24.,  25.,  25.,  25.,      
     S  24.,  23.,  21.,  20.,  19.,  18.,  18.,  17.,  16.,  14./      
      DATA (BET(I,10,1),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   9.,   9.,   9.,  10.,  12.,  14.,      
     S  16.,  18.,  18.,  19.,  20.,  21.,  23.,  24.,  24.,  23.,      
     S  23.,  21.,  20.,  18.,  17.,  17.,  16.,  15.,  14.,  12./      
      DATA (BET(I,11,1),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   9.,   9.,  10.,  11.,  12.,  14.,      
     S  16.,  17.,  18.,  18.,  19.,  20.,  21.,  22.,  22.,  22.,      
     S  21.,  20.,  18.,  17.,  16.,  15.,  14.,  13.,  12.,  10./      
      DATA (BET(I,12,1),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   9.,   9.,  10.,  11.,  12.,  14.,      
     S  16.,  17.,  17.,  17.,  18.,  19.,  20.,  20.,  20.,  20.,      
     S  19.,  18.,  16.,  15.,  14.,  13.,  12.,  11.,  10.,   9./      
      DATA (BET(I,13,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,  -1.,  -1.,  -1.,   0.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   9.,   9.,  10.,  11.,  12.,  14.,      
     S  15.,  16.,  16.,  16.,  17.,  17.,  18.,  18.,  18.,  18.,      
     S  17.,  16.,  14.,  13.,  12.,  11.,  11.,  10.,   9.,   7./      
      DATA (BET(I,14,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,  -1.,  -1.,  -1.,   0.,   0.,   2.,      
     S   3.,   5.,   7.,   8.,   9.,   9.,  10.,  10.,  12.,  13.,      
     S  14.,  15.,  15.,  15.,  15.,  16.,  16.,  16.,  16.,  16.,      
     S  15.,  14.,  13.,  11.,  10.,  10.,   9.,   8.,   7.,   6./      
      DATA (BET(I,15,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   2.,      
     S   4.,   6.,   7.,   9.,   9.,   9.,  10.,  10.,  11.,  12.,      
     S  13.,  13.,  13.,  13.,  13.,  14.,  14.,  14.,  14.,  14.,      
     S  13.,  12.,  11.,  10.,   9.,   8.,   7.,   7.,   6.,   4./      
      DATA (BET(I,16,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   2.,      
     S   4.,   6.,   7.,   9.,   9.,   9.,   9.,   9.,  10.,  11.,      
     S  11.,  12.,  12.,  12.,  12.,  12.,  12.,  12.,  12.,  12.,      
     S  11.,  10.,   9.,   8.,   7.,   7.,   6.,   5.,   4.,   3./      
      DATA (BET(I,17,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   3.,      
     S   4.,   6.,   8.,   9.,   9.,   9.,   8.,   8.,   9.,   9.,      
     S  10.,  10.,  10.,  10.,  10.,  10.,  10.,  11.,  10.,  10.,      
     S   9.,   9.,   8.,   7.,   6.,   6.,   5.,   4.,   3.,   2./      
      DATA (BET(I,18,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   2.,   3.,      
     S   5.,   6.,   7.,   8.,   8.,   8.,   8.,   7.,   7.,   8.,      
     S   8.,   9.,   9.,   9.,   9.,   9.,   9.,   9.,   9.,   8.,      
     S   8.,   7.,   7.,   6.,   5.,   4.,   4.,   3.,   2.,   0./      
      DATA (BET(I,19,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   1.,   1.,   2.,   2.,   3.,      
     S   5.,   6.,   7.,   8.,   8.,   7.,   7.,   6.,   6.,   6.,      
     S   7.,   7.,   7.,   8.,   8.,   8.,   8.,   8.,   8.,   7.,      
     S   7.,   6.,   5.,   5.,   4.,   3.,   2.,   1.,   0.,  -1./      
      DATA (BET(I,20,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   1.,   1.,   1.,   2.,   2.,   3.,      
     S   4.,   5.,   6.,   7.,   7.,   6.,   6.,   5.,   5.,   5.,      
     S   5.,   6.,   6.,   7.,   7.,   7.,   7.,   7.,   6.,   6.,      
     S   6.,   5.,   5.,   4.,   3.,   2.,   1.,   0.,  -1.,  -2./      
      DATA (BET(I,21,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   1.,   1.,   1.,   2.,   2.,   3.,      
     S   4.,   5.,   5.,   6.,   6.,   6.,   5.,   5.,   4.,   4.,      
     S   4.,   5.,   5.,   6.,   6.,   6.,   6.,   6.,   5.,   5.,      
     S   5.,   4.,   4.,   3.,   2.,   1.,   0.,  -1.,  -2.,  -2./      
      DATA (BET(I,22,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   1.,   1.,   1.,   2.,   2.,      
     S   3.,   4.,   4.,   5.,   5.,   5.,   5.,   4.,   4.,   3.,      
     S   4.,   4.,   5.,   5.,   5.,   5.,   5.,   5.,   5.,   4.,      
     S   4.,   4.,   3.,   2.,   1.,   0.,  -1.,  -2.,  -3.,  -3./      
      DATA (BET(I,23,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   1.,   2.,      
     S   2.,   3.,   3.,   4.,   4.,   5.,   5.,   4.,   4.,   4.,      
     S   3.,   3.,   4.,   4.,   5.,   5.,   5.,   4.,   4.,   4.,      
     S   3.,   3.,   2.,   1.,   0.,  -1.,  -2.,  -3.,  -3.,  -4./      
      DATA (BET(I,24,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   1.,      
     S   1.,   2.,   2.,   3.,   3.,   4.,   4.,   4.,   4.,   4.,      
     S   4.,   3.,   3.,   3.,   4.,   4.,   4.,   4.,   4.,   3.,      
     S   3.,   2.,   1.,   0.,  -1.,  -2.,  -3.,  -3.,  -4.,  -4./      
      DATA (BET(I,25,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,      
     S   1.,   1.,   1.,   2.,   2.,   3.,   4.,   4.,   5.,   4.,      
     S   4.,   3.,   3.,   3.,   3.,   3.,   3.,   3.,   3.,   2.,      
     S   2.,   1.,   0.,  -1.,  -2.,  -3.,  -3.,  -4.,  -4.,  -4./      
      DATA (BET(I,26,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   1.,   1.,   2.,   2.,   3.,   4.,   4.,   5.,      
     S   4.,   4.,   3.,   2.,   2.,   2.,   2.,   2.,   2.,   1.,      
     S   1.,   0.,  -1.,  -2.,  -3.,  -3.,  -4.,  -4.,  -4.,  -3./      
      DATA (BET(I,27,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   1.,   1.,   2.,   3.,   4.,   4.,      
     S   4.,   4.,   4.,   3.,   2.,   2.,   1.,   1.,   1.,   0.,      
     S   0.,  -1.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -3.,  -2./      
      DATA (BET(I,28,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   1.,   2.,   2.,   3.,   4.,      
     S   4.,   4.,   4.,   3.,   2.,   1.,   1.,   0.,  -1.,  -1.,      
     S  -2.,  -2.,  -2.,  -3.,  -3.,  -3.,  -4.,  -3.,  -2.,  -2./      
      DATA (BET(I,29,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   1.,   1.,   2.,   2.,   3.,      
     S   4.,   4.,   4.,   3.,   2.,   1.,   0.,  -1.,  -2.,  -3.,      
     S  -3.,  -3.,  -3.,  -3.,  -3.,  -3.,  -3.,  -2.,  -1.,  -1./      
      DATA (BET(I,30,1),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   2.,   2.,      
     S   3.,   3.,   3.,   2.,   2.,   1.,  -1.,  -2.,  -3.,  -4.,      
     S  -5.,  -5.,  -4.,  -4.,  -3.,  -3.,  -2.,  -1.,   0.,   0./      
      DATA (BET(I,31,1),I=1,40)/                                        
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,   0.,   0.,   0.,   0.,   1.,   1.,   1.,      
     S   1.,   1.,   1.,   1.,   0.,  -1.,  -2.,  -3.,  -4.,  -5.,      
     S  -6.,  -6.,  -5.,  -4.,  -3.,  -2.,  -1.,   0.,   1.,   1./      
C----- 1.,0E4*ALP(I,J,2) FOR TDB                                        
      DATA (ALP(I, 1,2),I=1,40)/                                        
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  18.,  25.,  32.,      
     S  40.,  47.,  54.,  59.,  64.,  66.,  68.,  68.,  69.,  70.,      
     S  72.,  74.,  77.,  79.,  81.,  84.,  86.,  88.,  90.,  91.,      
     S  92.,  93.,  93.,  92.,  92.,  92.,  91.,  90.,  89.,  88./      
      DATA (ALP(I, 2,2),I=1,40)/                                        
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  18.,  25.,  32.,      
     S  40.,  47.,  54.,  59.,  63.,  66.,  68.,  68.,  69.,  70.,      
     S  72.,  74.,  77.,  79.,  81.,  84.,  86.,  88.,  90.,  91.,      
     S  92.,  92.,  92.,  91.,  91.,  90.,  89.,  89.,  87.,  86./      
      DATA (ALP(I, 3,2),I=1,40)/                                        
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  18.,  25.,  32.,      
     S  40.,  47.,  54.,  59.,  63.,  66.,  67.,  68.,  69.,  70.,      
     S  72.,  74.,  76.,  79.,  81.,  83.,  86.,  88.,  89.,  90.,      
     S  91.,  91.,  90.,  90.,  89.,  88.,  87.,  86.,  85.,  83./      
      DATA (ALP(I, 4,2),I=1,40)/                                        
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  18.,  25.,  32.,      
     S  40.,  47.,  54.,  59.,  63.,  66.,  67.,  68.,  69.,  70.,      
     S  72.,  74.,  76.,  79.,  81.,  83.,  85.,  87.,  88.,  89.,      
     S  89.,  89.,  89.,  88.,  87.,  86.,  85.,  84.,  82.,  80./      
      DATA (ALP(I, 5,2),I=1,40)/                                        
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  18.,  25.,  32.,      
     S  40.,  47.,  54.,  59.,  63.,  66.,  67.,  68.,  69.,  70.,      
     S  72.,  74.,  76.,  78.,  80.,  82.,  84.,  86.,  87.,  88.,      
     S  88.,  87.,  87.,  86.,  85.,  83.,  82.,  81.,  79.,  76./      
      DATA (ALP(I, 6,2),I=1,40)/                                        
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  18.,  25.,  32.,      
     S  40.,  47.,  54.,  59.,  63.,  66.,  67.,  68.,  69.,  70.,      
     S  71.,  73.,  76.,  78.,  80.,  81.,  83.,  84.,  85.,  86.,      
     S  85.,  85.,  84.,  83.,  82.,  80.,  79.,  77.,  75.,  72./      
      DATA (ALP(I, 7,2),I=1,40)/                                        
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  18.,  25.,  32.,      
     S  39.,  47.,  54.,  59.,  63.,  66.,  67.,  68.,  68.,  69.,      
     S  71.,  73.,  75.,  77.,  78.,  80.,  81.,  82.,  83.,  83.,      
     S  83.,  82.,  81.,  80.,  78.,  77.,  75.,  73.,  71.,  68./      
      DATA (ALP(I, 8,2),I=1,40)/                                        
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  18.,  25.,  32.,      
     S  39.,  47.,  53.,  59.,  63.,  66.,  67.,  67.,  68.,  69.,      
     S  70.,  72.,  74.,  75.,  77.,  78.,  79.,  80.,  80.,  80.,      
     S  80.,  79.,  77.,  76.,  74.,  73.,  71.,  68.,  66.,  63./      
      DATA (ALP(I, 9,2),I=1,40)/                                        
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  18.,  24.,  32.,      
     S  39.,  47.,  53.,  59.,  63.,  65.,  66.,  67.,  67.,  68.,      
     S  69.,  71.,  72.,  73.,  74.,  75.,  76.,  77.,  77.,  76.,      
     S  76.,  75.,  73.,  72.,  70.,  68.,  66.,  64.,  61.,  58./      
      DATA (ALP(I,10,2),I=1,40)/                                        
     S   1.,   1.,   3.,   4.,   6.,   9.,  13.,  18.,  24.,  31.,      
     S  39.,  46.,  53.,  58.,  62.,  65.,  66.,  66.,  66.,  67.,      
     S  68.,  69.,  70.,  71.,  72.,  72.,  73.,  73.,  73.,  72.,      
     S  72.,  71.,  69.,  67.,  65.,  63.,  61.,  58.,  56.,  53./      
      DATA (ALP(I,11,2),I=1,40)/                                        
     S   1.,   1.,   3.,   4.,   6.,   9.,  13.,  18.,  24.,  31.,      
     S  39.,  46.,  53.,  58.,  62.,  64.,  65.,  65.,  65.,  65.,      
     S  66.,  66.,  67.,  67.,  68.,  68.,  69.,  69.,  69.,  68.,      
     S  67.,  66.,  64.,  62.,  60.,  58.,  56.,  53.,  51.,  48./      
      DATA (ALP(I,12,2),I=1,40)/                                        
     S   1.,   1.,   3.,   4.,   6.,   9.,  13.,  18.,  24.,  31.,      
     S  38.,  45.,  52.,  57.,  60.,  62.,  63.,  63.,  63.,  62.,      
     S  63.,  63.,  63.,  63.,  64.,  64.,  64.,  64.,  64.,  63.,      
     S  62.,  61.,  59.,  57.,  55.,  53.,  51.,  49.,  47.,  44./      
      DATA (ALP(I,13,2),I=1,40)/                                        
     S   1.,   1.,   3.,   4.,   6.,   9.,  13.,  18.,  24.,  30.,      
     S  38.,  45.,  51.,  55.,  59.,  60.,  61.,  60.,  59.,  59.,      
     S  59.,  59.,  59.,  59.,  59.,  59.,  59.,  59.,  59.,  58.,      
     S  57.,  56.,  54.,  52.,  50.,  48.,  46.,  45.,  43.,  41./      
      DATA (ALP(I,14,2),I=1,40)/                                        
     S   1.,   1.,   2.,   4.,   6.,   9.,  12.,  17.,  23.,  29.,      
     S  36.,  43.,  49.,  53.,  56.,  57.,  57.,  56.,  55.,  55.,      
     S  54.,  54.,  54.,  54.,  54.,  55.,  55.,  54.,  54.,  53.,      
     S  52.,  51.,  50.,  48.,  46.,  44.,  42.,  41.,  39.,  38./      
      DATA (ALP(I,15,2),I=1,40)/                                        
     S   1.,   1.,   2.,   4.,   6.,   8.,  12.,  16.,  22.,  28.,      
     S  35.,  41.,  46.,  50.,  52.,  53.,  53.,  52.,  51.,  50.,      
     S  49.,  49.,  49.,  49.,  49.,  50.,  50.,  50.,  49.,  49.,      
     S  48.,  47.,  45.,  44.,  42.,  40.,  39.,  38.,  37.,  36./      
      DATA (ALP(I,16,2),I=1,40)/                                        
     S   1.,   1.,   2.,   3.,   5.,   8.,  11.,  15.,  20.,  26.,      
     S  32.,  37.,  42.,  45.,  47.,  48.,  48.,  47.,  45.,  45.,      
     S  44.,  44.,  44.,  45.,  45.,  45.,  45.,  45.,  45.,  44.,      
     S  44.,  42.,  41.,  40.,  39.,  37.,  36.,  35.,  35.,  34./      
      DATA (ALP(I,17,2),I=1,40)/                                        
     S   0.,   1.,   2.,   3.,   5.,   7.,  10.,  14.,  18.,  23.,      
     S  28.,  33.,  37.,  40.,  41.,  42.,  42.,  41.,  40.,  40.,      
     S  39.,  40.,  40.,  40.,  41.,  41.,  41.,  41.,  41.,  41.,      
     S  40.,  39.,  38.,  37.,  36.,  35.,  34.,  34.,  34.,  34./      
      DATA (ALP(I,18,2),I=1,40)/                                        
     S   0.,   1.,   2.,   3.,   4.,   6.,   8.,  11.,  15.,  19.,      
     S  24.,  28.,  31.,  33.,  35.,  36.,  36.,  36.,  35.,  35.,      
     S  35.,  35.,  36.,  37.,  37.,  38.,  38.,  38.,  38.,  37.,      
     S  37.,  36.,  35.,  34.,  34.,  33.,  33.,  33.,  33.,  33./      
      DATA (ALP(I,19,2),I=1,40)/                                        
     S   0.,   1.,   1.,   2.,   3.,   4.,   6.,   9.,  12.,  15.,      
     S  19.,  22.,  25.,  27.,  29.,  30.,  31.,  31.,  31.,  31.,      
     S  32.,  32.,  33.,  34.,  34.,  35.,  35.,  35.,  35.,  35.,      
     S  34.,  34.,  33.,  33.,  32.,  32.,  32.,  32.,  33.,  33./      
      DATA (ALP(I,20,2),I=1,40)/                                        
     S   0.,   0.,   1.,   1.,   2.,   3.,   4.,   6.,   8.,  11.,      
     S  13.,  16.,  19.,  21.,  24.,  25.,  27.,  28.,  28.,  29.,      
     S  29.,  30.,  31.,  31.,  32.,  32.,  33.,  33.,  33.,  33.,      
     S  32.,  32.,  32.,  31.,  31.,  32.,  32.,  32.,  32.,  32./      
      DATA (ALP(I,21,2),I=1,40)/                                        
     S   0.,   0.,   0.,   1.,   1.,   2.,   3.,   4.,   5.,   7.,      
     S   9.,  11.,  14.,  16.,  19.,  21.,  24.,  25.,  26.,  27.,      
     S  28.,  28.,  29.,  29.,  30.,  30.,  31.,  31.,  31.,  31.,      
     S  31.,  31.,  31.,  31.,  31.,  31.,  32.,  32.,  32.,  32./      
      DATA (ALP(I,22,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   1.,   1.,   1.,   2.,   3.,   4.,      
     S   6.,   8.,  10.,  12.,  15.,  18.,  21.,  23.,  25.,  26.,      
     S  26.,  27.,  27.,  28.,  28.,  29.,  29.,  29.,  29.,  30.,      
     S  30.,  30.,  30.,  30.,  31.,  31.,  32.,  32.,  32.,  31./      
      DATA (ALP(I,23,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   2.,   3.,      
     S   4.,   5.,   7.,   9.,  12.,  15.,  18.,  21.,  23.,  25.,      
     S  26.,  26.,  26.,  27.,  27.,  27.,  28.,  28.,  28.,  28.,      
     S  29.,  29.,  30.,  30.,  31.,  32.,  32.,  31.,  31.,  30./      
      DATA (ALP(I,24,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   2.,      
     S   2.,   4.,   5.,   7.,  10.,  13.,  16.,  19.,  21.,  23.,      
     S  25.,  25.,  25.,  26.,  26.,  26.,  26.,  27.,  27.,  28.,      
     S  28.,  29.,  30.,  31.,  31.,  31.,  31.,  31.,  30.,  29./      
      DATA (ALP(I,25,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,      
     S   2.,   3.,   4.,   6.,   8.,  11.,  13.,  16.,  19.,  21.,      
     S  23.,  24.,  25.,  25.,  25.,  25.,  25.,  26.,  26.,  27.,      
     S  28.,  29.,  30.,  31.,  31.,  31.,  31.,  30.,  29.,  28./      
      DATA (ALP(I,26,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,      
     S   2.,   3.,   4.,   5.,   7.,   9.,  11.,  14.,  16.,  19.,      
     S  21.,  22.,  23.,  24.,  24.,  24.,  24.,  25.,  26.,  27.,      
     S  28.,  29.,  30.,  31.,  31.,  30.,  30.,  29.,  28.,  26./      
      DATA (ALP(I,27,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,      
     S   2.,   2.,   3.,   5.,   6.,   8.,  10.,  12.,  14.,  16.,      
     S  18.,  20.,  21.,  22.,  23.,  23.,  24.,  24.,  25.,  27.,      
     S  28.,  29.,  30.,  30.,  30.,  29.,  29.,  28.,  26.,  25./      
      DATA (ALP(I,28,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,      
     S   1.,   2.,   3.,   4.,   6.,   7.,   8.,  10.,  11.,  13.,      
     S  15.,  17.,  19.,  20.,  21.,  22.,  23.,  24.,  26.,  27.,      
     S  28.,  29.,  29.,  29.,  29.,  28.,  27.,  26.,  25.,  25./      
      DATA (ALP(I,29,2),I=1,40)/                                        
     S  -1.,  -1.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,      
     S   1.,   2.,   3.,   4.,   5.,   6.,   7.,   8.,   9.,  10.,      
     S  12.,  14.,  16.,  18.,  20.,  21.,  23.,  25.,  26.,  27.,      
     S  28.,  28.,  28.,  28.,  28.,  27.,  26.,  25.,  25.,  25./      
      DATA (ALP(I,30,2),I=1,40)/                                        
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   1.,   1.,   2.,   3.,   3.,   4.,   5.,   6.,   6.,   7.,      
     S   9.,  10.,  13.,  15.,  18.,  21.,  23.,  26.,  27.,  28.,      
     S  28.,  28.,  27.,  27.,  26.,  25.,  24.,  24.,  24.,  25./      
      DATA (ALP(I,31,2),I=1,40)/                                        
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S   0.,   0.,   1.,   1.,   2.,   2.,   3.,   4.,   4.,   5.,      
     S   6.,   8.,  10.,  13.,  17.,  20.,  24.,  27.,  28.,  29.,      
     S  28.,  28.,  26.,  25.,  24.,  23.,  23.,  23.,  24.,  24./      
C----- 1.,0E6*BET(I,J,2) FOR TDB                                        
      DATA (BET(I, 1,2),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  15.,      
     S  17.,  18.,  19.,  20.,  21.,  24.,  26.,  28.,  29.,  29.,      
     S  29.,  29.,  27.,  26.,  26.,  25.,  26.,  26.,  25.,  23./      
      DATA (BET(I, 2,2),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  15.,      
     S  17.,  18.,  19.,  20.,  21.,  24.,  26.,  28.,  29.,  29.,      
     S  29.,  28.,  27.,  26.,  25.,  25.,  25.,  25.,  24.,  23./      
      DATA (BET(I, 3,2),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  15.,      
     S  17.,  18.,  19.,  20.,  21.,  24.,  26.,  28.,  29.,  29.,      
     S  29.,  28.,  27.,  26.,  25.,  25.,  25.,  24.,  24.,  22./      
      DATA (BET(I, 4,2),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  15.,      
     S  17.,  18.,  19.,  20.,  21.,  24.,  26.,  28.,  29.,  29.,      
     S  29.,  28.,  26.,  25.,  24.,  24.,  24.,  24.,  23.,  21./      
      DATA (BET(I, 5,2),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  15.,      
     S  17.,  18.,  19.,  20.,  21.,  24.,  26.,  27.,  28.,  29.,      
     S  28.,  27.,  26.,  25.,  24.,  23.,  23.,  23.,  22.,  20./      
      DATA (BET(I, 6,2),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  15.,      
     S  17.,  18.,  19.,  20.,  21.,  23.,  26.,  27.,  28.,  28.,      
     S  28.,  26.,  25.,  24.,  23.,  22.,  22.,  22.,  20.,  19./      
      DATA (BET(I, 7,2),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  15.,      
     S  17.,  18.,  19.,  20.,  21.,  23.,  25.,  27.,  27.,  27.,      
     S  27.,  25.,  24.,  23.,  22.,  21.,  21.,  20.,  19.,  17./      
      DATA (BET(I, 8,2),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   8.,   9.,   9.,  10.,  12.,  15.,      
     S  17.,  18.,  19.,  20.,  21.,  23.,  25.,  26.,  26.,  26.,      
     S  26.,  24.,  23.,  21.,  20.,  20.,  19.,  19.,  17.,  15./      
      DATA (BET(I, 9,2),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   9.,   9.,   9.,  11.,  12.,  15.,      
     S  17.,  18.,  19.,  19.,  21.,  22.,  24.,  25.,  25.,  25.,      
     S  24.,  23.,  21.,  20.,  19.,  18.,  18.,  17.,  16.,  14./      
      DATA (BET(I,10,2),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   9.,   9.,  10.,  11.,  12.,  15.,      
     S  16.,  18.,  18.,  19.,  20.,  22.,  23.,  24.,  24.,  23.,      
     S  22.,  21.,  20.,  18.,  17.,  17.,  16.,  15.,  14.,  12./      
      DATA (BET(I,11,2),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   9.,   9.,  10.,  11.,  12.,  14.,      
     S  16.,  17.,  18.,  18.,  19.,  20.,  21.,  22.,  22.,  22.,      
     S  21.,  19.,  18.,  17.,  16.,  15.,  14.,  13.,  12.,  10./      
      DATA (BET(I,12,2),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   9.,   9.,  10.,  11.,  12.,  14.,      
     S  16.,  17.,  17.,  17.,  18.,  19.,  20.,  20.,  20.,  20.,      
     S  19.,  18.,  16.,  15.,  14.,  13.,  12.,  11.,  10.,   9./      
      DATA (BET(I,13,2),I=1,40)/                                        
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,   0.,   0.,   1.,      
     S   3.,   5.,   7.,   8.,   9.,   9.,  10.,  11.,  12.,  14.,      
     S  15.,  16.,  16.,  16.,  17.,  17.,  18.,  18.,  18.,  18.,      
     S  17.,  16.,  14.,  13.,  12.,  11.,  11.,  10.,   9.,   7./      
      DATA (BET(I,14,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,  -1.,  -1.,  -1.,   0.,   0.,   2.,      
     S   3.,   5.,   7.,   8.,   9.,  10.,  10.,  11.,  12.,  13.,      
     S  14.,  15.,  15.,  15.,  15.,  16.,  16.,  16.,  16.,  16.,      
     S  15.,  14.,  13.,  11.,  10.,  10.,   9.,   8.,   7.,   6./      
      DATA (BET(I,15,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   2.,      
     S   4.,   6.,   7.,   9.,   9.,  10.,  10.,  10.,  11.,  12.,      
     S  13.,  13.,  13.,  13.,  13.,  14.,  14.,  14.,  14.,  14.,      
     S  13.,  12.,  11.,  10.,   9.,   8.,   7.,   7.,   6.,   4./      
      DATA (BET(I,16,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   2.,      
     S   4.,   6.,   8.,   9.,   9.,   9.,   9.,   9.,  10.,  11.,      
     S  11.,  12.,  12.,  12.,  12.,  12.,  12.,  12.,  12.,  12.,      
     S  11.,  10.,   9.,   8.,   7.,   7.,   6.,   5.,   4.,   3./      
      DATA (BET(I,17,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   3.,      
     S   4.,   6.,   8.,   9.,   9.,   9.,   9.,   8.,   9.,   9.,      
     S  10.,  10.,  10.,  10.,  10.,  10.,  10.,  11.,  10.,  10.,      
     S   9.,   9.,   8.,   7.,   6.,   6.,   5.,   4.,   3.,   2./      
      DATA (BET(I,18,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   2.,   3.,      
     S   5.,   6.,   7.,   8.,   8.,   8.,   8.,   7.,   7.,   8.,      
     S   8.,   9.,   9.,   9.,   9.,   9.,   9.,   9.,   9.,   8.,      
     S   8.,   7.,   7.,   6.,   5.,   4.,   4.,   3.,   1.,   0./      
      DATA (BET(I,19,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   1.,   1.,   2.,   2.,   3.,      
     S   5.,   6.,   7.,   8.,   8.,   7.,   7.,   6.,   6.,   6.,      
     S   7.,   7.,   7.,   8.,   8.,   8.,   8.,   8.,   7.,   7.,      
     S   7.,   6.,   5.,   5.,   4.,   3.,   2.,   1.,   0.,  -1./      
      DATA (BET(I,20,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   1.,   1.,   1.,   2.,   2.,   3.,      
     S   4.,   5.,   6.,   7.,   7.,   6.,   6.,   5.,   5.,   5.,      
     S   5.,   6.,   6.,   7.,   7.,   7.,   7.,   7.,   6.,   6.,      
     S   6.,   5.,   5.,   4.,   3.,   2.,   1.,   0.,  -1.,  -2./      
      DATA (BET(I,21,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   1.,   1.,   1.,   2.,   2.,   3.,      
     S   4.,   5.,   5.,   6.,   6.,   6.,   5.,   5.,   4.,   4.,      
     S   4.,   5.,   5.,   6.,   6.,   6.,   6.,   6.,   5.,   5.,      
     S   5.,   4.,   4.,   3.,   2.,   1.,   0.,  -1.,  -2.,  -3./      
      DATA (BET(I,22,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   1.,   1.,   1.,   2.,   2.,      
     S   3.,   4.,   4.,   5.,   5.,   5.,   5.,   4.,   4.,   3.,      
     S   4.,   4.,   5.,   5.,   5.,   5.,   5.,   5.,   5.,   4.,      
     S   4.,   3.,   3.,   2.,   1.,   0.,  -1.,  -2.,  -3.,  -3./      
      DATA (BET(I,23,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   1.,   2.,      
     S   2.,   3.,   3.,   4.,   4.,   5.,   5.,   4.,   4.,   4.,      
     S   3.,   3.,   4.,   4.,   5.,   5.,   5.,   4.,   4.,   4.,      
     S   3.,   3.,   2.,   1.,   0.,  -1.,  -2.,  -3.,  -3.,  -4./      
      DATA (BET(I,24,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   1.,      
     S   1.,   2.,   2.,   3.,   3.,   4.,   4.,   5.,   4.,   4.,      
     S   4.,   3.,   3.,   3.,   4.,   4.,   4.,   4.,   4.,   3.,      
     S   2.,   2.,   1.,   0.,  -1.,  -2.,  -3.,  -3.,  -4.,  -4./      
      DATA (BET(I,25,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,      
     S   1.,   1.,   1.,   2.,   2.,   3.,   4.,   4.,   5.,   4.,      
     S   4.,   3.,   3.,   3.,   3.,   3.,   3.,   3.,   3.,   2.,      
     S   2.,   1.,   0.,  -1.,  -2.,  -3.,  -3.,  -4.,  -4.,  -4./      
      DATA (BET(I,26,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   1.,   1.,   2.,   2.,   3.,   4.,   4.,   5.,      
     S   4.,   4.,   3.,   2.,   2.,   2.,   2.,   2.,   2.,   1.,      
     S   1.,   0.,  -1.,  -2.,  -3.,  -3.,  -4.,  -4.,  -4.,  -3./      
      DATA (BET(I,27,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   1.,   1.,   2.,   3.,   4.,   4.,      
     S   4.,   4.,   4.,   3.,   2.,   2.,   1.,   1.,   1.,   0.,      
     S   0.,  -1.,  -2.,  -2.,  -3.,  -4.,  -4.,  -4.,  -3.,  -3./      
      DATA (BET(I,28,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   1.,   2.,   2.,   3.,   4.,      
     S   4.,   4.,   4.,   3.,   2.,   1.,   1.,   0.,  -1.,  -1.,      
     S  -2.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -3.,  -3.,  -2./      
      DATA (BET(I,29,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   1.,   1.,   2.,   2.,   3.,      
     S   4.,   4.,   4.,   3.,   2.,   1.,   0.,  -1.,  -2.,  -3.,      
     S  -3.,  -3.,  -3.,  -3.,  -3.,  -3.,  -3.,  -2.,  -1.,  -1./      
      DATA (BET(I,30,2),I=1,40)/                                        
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   2.,   2.,      
     S   3.,   3.,   3.,   2.,   2.,   1.,  -1.,  -2.,  -3.,  -4.,      
     S  -5.,  -5.,  -4.,  -4.,  -3.,  -3.,  -2.,  -1.,   0.,   0./      
      DATA (BET(I,31,2),I=1,40)/                                        
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,   0.,   0.,   0.,   0.,   1.,   1.,   1.,      
     S   1.,   1.,   1.,   1.,   0.,  -1.,  -2.,  -3.,  -4.,  -5.,      
     S  -6.,  -6.,  -6.,  -4.,  -3.,  -2.,  -1.,   0.,   1.,   0./      
C                                                                       
C-----  -10E+2*LN(1.-TB)    (O3 9.6 micron band)                        
      DATA (TBO3(I, 1),I=1,38)/                                         
     S1487.2,1437.2,1387.2,1337.2,1287.2,1237.2,1187.2,1137.2,1087.2,   
     S1037.2, 987.2, 937.9, 889.8, 840.8, 791.9, 743.8, 696.8, 651.1,   
     S 606.8, 564.8, 525.5, 489.4, 457.0, 428.0, 401.7, 377.3, 353.9,   
     S 331.2, 308.8, 286.8, 265.4, 244.7, 225.2, 207.1, 190.5, 175.6,   
     S 162.2, 150.4/                                                    
      DATA (TBO3(I, 2),I=1,38)/                                         
     S1487.2,1437.2,1387.2,1337.2,1287.2,1237.2,1187.2,1137.2,1087.2,   
     S1037.2, 987.2, 937.9, 889.8, 840.8, 791.9, 743.8, 696.8, 651.1,   
     S 606.8, 564.8, 525.5, 489.4, 457.0, 428.0, 401.7, 377.3, 353.9,   
     S 331.2, 308.8, 286.8, 265.4, 244.7, 225.2, 207.0, 190.5, 175.5,   
     S 162.2, 150.3/                                                    
      DATA (TBO3(I, 3),I=1,38)/                                         
     S1487.2,1437.2,1387.2,1337.2,1287.2,1237.2,1187.2,1137.2,1087.2,   
     S1037.2, 987.2, 937.9, 889.8, 840.8, 791.9, 743.8, 696.8, 651.1,   
     S 606.8, 564.8, 525.5, 489.4, 457.0, 428.0, 401.7, 377.3, 353.9,   
     S 331.2, 308.8, 286.8, 265.4, 244.7, 225.2, 207.0, 190.5, 175.5,   
     S 162.2, 150.3/                                                    
      DATA (TBO3(I, 4),I=1,38)/                                         
     S1487.2,1437.2,1387.2,1337.2,1287.2,1237.2,1187.2,1137.2,1087.2,   
     S1037.2, 987.2, 937.9, 889.8, 840.8, 791.9, 743.8, 696.8, 651.1,   
     S 606.8, 564.8, 525.5, 489.4, 457.0, 428.0, 401.7, 377.3, 353.9,   
     S 331.2, 308.8, 286.8, 265.4, 244.7, 225.1, 207.0, 190.4, 175.4,   
     S 162.1, 150.2/                                                    
      DATA (TBO3(I, 5),I=1,38)/                                         
     S1487.2,1437.2,1387.2,1337.2,1287.2,1237.2,1187.2,1137.2,1087.2,   
     S1037.2, 987.2, 937.9, 889.8, 840.8, 791.9, 743.8, 696.9, 651.1,   
     S 606.8, 564.8, 525.5, 489.4, 457.0, 428.0, 401.7, 377.2, 353.9,   
     S 331.2, 308.8, 286.8, 265.3, 244.6, 225.1, 206.9, 190.3, 175.3,   
     S 161.9, 150.0/                                                    
      DATA (TBO3(I, 6),I=1,38)/                                         
     S1487.2,1437.2,1387.2,1337.2,1287.2,1237.2,1187.2,1137.2,1087.2,   
     S1037.2, 987.2, 937.9, 889.7, 840.8, 791.9, 743.8, 696.8, 651.1,   
     S 606.8, 564.8, 525.5, 489.4, 457.0, 427.9, 401.7, 377.2, 353.9,   
     S 331.1, 308.7, 286.7, 265.3, 244.6, 225.0, 206.8, 190.2, 175.2,   
     S 161.7, 149.7/                                                    
      DATA (TBO3(I, 7),I=1,38)/                                         
     S1487.2,1437.2,1387.2,1337.2,1287.2,1237.2,1187.2,1137.2,1087.2,   
     S1037.2, 987.2, 937.9, 889.8, 840.7, 792.0, 743.8, 696.8, 651.1,   
     S 606.8, 564.8, 525.5, 489.4, 457.0, 427.9, 401.6, 377.2, 353.8,   
     S 331.1, 308.7, 286.7, 265.2, 244.5, 224.9, 206.6, 190.0, 174.9,   
     S 161.3, 149.2/                                                    
      DATA (TBO3(I, 8),I=1,38)/                                         
     S1487.2,1437.2,1387.2,1337.2,1287.2,1237.2,1187.2,1137.2,1087.2,   
     S1037.2, 987.2, 937.9, 889.8, 840.8, 791.9, 743.8, 696.8, 651.1,   
     S 606.8, 564.8, 525.4, 489.4, 456.9, 427.9, 401.6, 377.1, 353.8,   
     S 331.0, 308.6, 286.6, 265.0, 244.3, 224.6, 206.4, 189.6, 174.4,   
     S 160.7, 148.4/                                                    
      DATA (TBO3(I, 9),I=1,38)/                                         
     S1487.4,1437.4,1387.4,1337.4,1287.4,1237.4,1187.4,1137.4,1087.4,   
     S1037.4, 987.4, 937.9, 889.8, 840.8, 792.0, 743.8, 696.9, 651.1,   
     S 606.8, 564.8, 525.4, 489.4, 456.9, 427.8, 401.5, 377.0, 353.7,   
     S 330.9, 308.4, 286.4, 264.8, 244.0, 224.3, 205.9, 189.0, 173.7,   
     S 159.8, 147.2/                                                    
      DATA (TBO3(I,10),I=1,38)/                                         
     S1487.4,1437.4,1387.4,1337.4,1287.4,1237.4,1187.4,1137.4,1087.4,   
     S1037.4, 987.4, 937.9, 889.6, 840.7, 792.0, 743.7, 696.8, 651.1,   
     S 606.8, 564.7, 525.4, 489.3, 456.8, 427.7, 401.4, 376.9, 353.5,   
     S 330.6, 308.2, 286.0, 264.4, 243.5, 223.7, 205.2, 188.1, 172.5,   
     S 158.3, 145.4/                                                    
      DATA (TBO3(I,11),I=1,38)/                                         
     S1487.6,1437.6,1387.6,1337.6,1287.6,1237.6,1187.6,1137.6,1087.6,   
     S1037.6, 987.6, 937.9, 889.7, 840.8, 792.0, 743.8, 696.8, 651.1,   
     S 606.8, 564.7, 525.3, 489.2, 456.7, 427.6, 401.2, 376.7, 353.2,   
     S 330.3, 307.7, 285.5, 263.8, 242.8, 222.8, 204.0, 186.7, 170.8,   
     S 156.2, 142.7/                                                    
      DATA (TBO3(I,12),I=1,38)/                                         
     S1487.2,1437.2,1387.2,1337.2,1287.2,1237.2,1187.2,1137.2,1087.2,   
     S1037.2, 987.2, 937.9, 889.6, 840.8, 791.9, 743.8, 696.8, 651.0,   
     S 606.7, 564.6, 525.2, 489.1, 456.5, 427.3, 400.9, 376.3, 352.7,   
     S 329.7, 307.0, 284.7, 262.8, 241.5, 221.3, 202.2, 184.5, 168.2,   
     S 153.0, 139.0/                                                    
      DATA (TBO3(I,13),I=1,38)/                                         
     S1487.0,1437.0,1387.0,1337.0,1287.0,1237.0,1187.0,1137.0,1087.0,   
     S1037.0, 987.0, 938.1, 889.6, 840.8, 791.8, 743.9, 696.8, 651.0,   
     S 606.7, 564.5, 525.0, 488.9, 456.2, 426.9, 400.4, 375.7, 352.0,   
     S 328.8, 305.9, 283.4, 261.2, 239.6, 219.0, 199.5, 181.3, 164.4,   
     S 148.7, 134.1/                                                    
      DATA (TBO3(I,14),I=1,38)/                                         
     S1487.4,1437.4,1387.4,1337.4,1287.4,1237.4,1187.4,1137.4,1087.4,   
     S1037.4, 987.4, 938.5, 889.7, 840.6, 791.9, 743.8, 696.7, 650.9,   
     S 606.5, 564.3, 524.8, 488.5, 455.7, 426.3, 399.6, 374.6, 350.7,   
     S 327.3, 304.2, 281.2, 258.7, 236.7, 215.6, 195.6, 176.8, 159.2,   
     S 142.9, 127.8/                                                    
      DATA (TBO3(I,15),I=1,38)/                                         
     S1487.8,1437.8,1387.8,1337.8,1287.8,1237.8,1187.8,1137.8,1087.8,   
     S1037.8, 987.8, 938.4, 890.1, 840.7, 791.9, 743.8, 696.7, 650.8,   
     S 606.3, 564.0, 524.4, 487.9, 454.9, 425.3, 398.3, 373.0, 348.8,   
     S 325.0, 301.4, 278.0, 254.9, 232.4, 210.7, 190.1, 170.6, 152.5,   
     S 135.7, 120.0/                                                    
      DATA (TBO3(I,16),I=1,38)/                                         
     S1487.8,1437.8,1387.8,1337.8,1287.8,1237.8,1187.8,1137.8,1087.8,   
     S1037.8, 987.8, 938.9, 890.2, 840.7, 791.9, 743.8, 696.5, 650.6,   
     S 606.0, 563.5, 523.6, 486.9, 453.6, 423.6, 396.2, 370.5, 345.8,   
     S 321.4, 297.3, 273.3, 249.5, 226.3, 204.0, 182.7, 162.8, 144.2,   
     S 126.9, 111.0/                                                    
      DATA (TBO3(I,17),I=1,38)/                                         
     S1488.2,1438.2,1388.2,1338.2,1288.2,1238.2,1188.2,1138.2,1088.2,   
     S1038.2, 988.2, 939.4, 890.4, 841.1, 791.8, 743.7, 696.4, 650.2,   
     S 605.6, 562.8, 522.6, 485.5, 451.7, 421.1, 393.0, 366.7, 341.2,   
     S 316.2, 291.3, 266.5, 242.1, 218.2, 195.3, 173.6, 153.2, 134.3,   
     S 116.9, 101.0/                                                    
      DATA (TBO3(I,18),I=1,38)/                                         
     S1489.0,1439.0,1389.0,1339.0,1289.0,1239.0,1189.0,1139.0,1089.0,   
     S1039.0, 989.0, 938.9, 890.7, 840.6, 791.2, 743.4, 696.2, 649.8,   
     S 604.8, 561.7, 521.0, 483.2, 448.7, 417.2, 388.3, 361.1, 334.7,   
     S 308.8, 283.1, 257.5, 232.4, 208.0, 184.7, 162.6, 142.1, 123.1,   
     S 105.8,  90.2/                                                    
      DATA (TBO3(I,19),I=1,38)/                                         
     S1489.0,1439.0,1389.0,1339.0,1289.0,1239.0,1189.0,1139.0,1089.0,   
     S1039.0, 989.0, 938.9, 890.0, 839.6, 791.3, 743.3, 695.8, 649.1,   
     S 603.7, 560.1, 518.7, 480.1, 444.4, 411.8, 381.7, 353.3, 325.9,   
     S 299.0, 272.5, 246.3, 220.6, 195.9, 172.3, 150.2, 129.8, 111.1,   
     S  94.2,  79.3/                                                    
      DATA (TBO3(I,20),I=1,38)/                                         
     S1489.0,1439.0,1389.0,1339.0,1289.0,1239.0,1189.0,1139.0,1089.0,   
     S1039.0, 989.0, 938.6, 888.9, 839.6, 791.3, 743.2, 695.5, 648.2,   
     S 602.4, 557.9, 515.6, 475.7, 438.7, 404.6, 373.0, 343.3, 314.7,   
     S 286.9, 259.6, 233.0, 207.1, 182.3, 158.8, 136.9, 116.8,  98.7,   
     S  82.7,  68.7/                                                    
      DATA (TBO3(I,21),I=1,38)/                                         
     S1487.2,1437.2,1387.2,1337.2,1287.2,1237.2,1187.2,1137.2,1087.2,   
     S1037.2, 987.2, 937.8, 888.6, 840.0, 791.6, 742.9, 694.5, 647.2,   
     S 600.5, 555.3, 511.7, 470.4, 431.8, 395.9, 362.5, 331.3, 301.6,   
     S 272.9, 245.2, 218.3, 192.4, 167.7, 144.6, 123.2, 103.9,  86.7,   
     S  71.7,  58.9/                                                    
      DATA (TBO3(I,22),I=1,38)/                                         
     S1486.6,1436.6,1386.6,1336.6,1286.6,1236.6,1186.6,1136.6,1086.6,   
     S1036.6, 986.6, 937.6, 889.5, 840.2, 791.5, 742.1, 694.1, 645.8,   
     S 598.5, 552.3, 507.5, 464.7, 424.2, 386.3, 351.0, 318.2, 287.3,   
     S 257.9, 229.8, 202.9, 177.2, 153.0, 130.5, 109.9,  91.6,  75.5,   
     S  61.8,  50.3/                                                    
      DATA (TBO3(I,23),I=1,38)/                                         
     S1485.9,1435.9,1385.9,1335.9,1285.9,1235.9,1185.9,1135.9,1085.9,   
     S1035.9, 985.9, 937.8, 888.5, 840.6, 790.8, 742.4, 693.2, 644.4,   
     S 596.7, 549.5, 503.6, 459.2, 416.8, 376.9, 339.6, 305.0, 272.8,   
     S 242.6, 214.3, 187.5, 162.2, 138.7, 117.1,  97.6,  80.4,  65.7,   
     S  53.3,  43.0/                                                    
      DATA (TBO3(I,24),I=1,38)/                                         
     S1484.1,1434.1,1384.1,1334.1,1284.1,1234.1,1184.1,1134.1,1084.1,   
     S1034.1, 984.1, 937.9, 887.8, 839.6, 791.1, 741.9, 692.4, 643.2,   
     S 594.9, 547.1, 500.2, 454.5, 410.6, 368.7, 329.4, 292.9, 259.2,   
     S 228.1, 199.4, 172.7, 148.1, 125.4, 104.9,  86.7,  70.9,  57.5,   
     S  46.3,  37.1/                                                    
      DATA (TBO3(I,25),I=1,38)/                                         
     S1482.1,1432.1,1382.1,1332.1,1282.1,1232.1,1182.1,1132.1,1082.1,   
     S1032.1, 982.1, 937.4, 888.7, 840.2, 790.9, 741.8, 692.0, 642.4,   
     S 593.7, 545.3, 497.6, 451.0, 405.7, 362.3, 321.1, 282.7, 247.3,   
     S 215.0, 185.7, 159.1, 135.1, 113.4,  94.2,  77.4,  63.0,  50.9,   
     S  40.8,  32.2/                                                    
      DATA (TBO3(I,26),I=1,38)/                                         
     S1483.8,1433.8,1383.8,1333.8,1283.8,1233.8,1183.8,1133.8,1083.8,   
     S1033.8, 983.8, 936.5, 890.0, 840.4, 791.2, 741.8, 691.7, 642.0,   
     S 592.8, 544.0, 495.9, 448.5, 402.3, 357.6, 314.8, 274.7, 237.6,   
     S 204.0, 173.8, 147.1, 123.6, 103.0,  85.2,  69.9,  56.9,  46.0,   
     S  36.6,  28.5/                                                    
      DATA (TBO3(I,27),I=1,38)/                                         
     S1482.3,1432.3,1382.3,1332.3,1282.3,1232.3,1182.3,1132.3,1082.3,   
     S1032.3, 982.3, 935.2, 889.5, 841.6, 792.1, 741.2, 691.5, 641.6,   
     S 592.3, 543.2, 494.7, 446.8, 399.9, 354.2, 310.4, 268.9, 230.3,   
     S 195.2, 164.1, 137.0, 113.9,  94.3,  77.8,  63.9,  52.2,  42.2,   
     S  33.4,  25.5/                                                    
      DATA (TBO3(I,28),I=1,38)/                                         
     S1482.8,1432.8,1382.8,1332.8,1282.8,1232.8,1182.8,1132.8,1082.8,   
     S1032.8, 982.8, 935.7, 889.2, 842.7, 791.9, 742.1, 691.9, 641.4,   
     S 591.7, 542.7, 493.9, 445.7, 398.4, 352.1, 307.5, 264.9, 225.1,   
     S 188.9, 156.8, 129.2, 106.3,  87.5,  72.2,  59.6,  49.0,  39.7,   
     S  31.2,  23.5/                                                    
      DATA (TBO3(I,29),I=1,38)/                                         
     S1483.4,1433.4,1383.4,1333.4,1283.4,1233.4,1183.4,1133.4,1083.4,   
     S1033.4, 983.4, 935.7, 889.2, 843.3, 791.2, 742.1, 691.3, 641.3,   
     S 591.6, 542.3, 493.5, 445.0, 397.3, 350.7, 305.5, 262.3, 221.6,   
     S 184.4, 151.5, 123.5, 100.5,  82.4,  68.0,  56.5,  46.6,  37.7,   
     S  29.4,  21.7/                                                    
      DATA (TBO3(I,30),I=1,38)/                                         
     S1484.9,1434.9,1384.9,1334.9,1284.9,1234.9,1184.9,1134.9,1084.9,   
     S1034.9, 984.9, 938.5, 889.3, 842.1, 791.7, 741.8, 691.4, 641.4,   
     S 591.9, 542.4, 493.4, 444.7, 396.9, 350.1, 304.5, 260.8, 219.7,   
     S 181.9, 148.3, 119.9,  96.9,  79.2,  65.6,  54.7,  45.3,  36.6,   
     S  28.4,  20.8/                                                    
      DATA (TBO3(I,31),I=1,38)/                                         
     S1486.6,1436.6,1386.6,1336.6,1286.6,1236.6,1186.6,1136.6,1086.6,   
     S1036.6, 986.6, 938.5, 890.1, 840.9, 792.3, 741.0, 690.5, 641.6,   
     S 591.7, 542.2, 493.1, 444.4, 396.4, 349.5, 303.7, 259.8, 218.3,   
     S 180.1, 146.2, 117.4,  94.4,  76.9,  63.7,  53.2,  43.9,  35.4,   
     S  27.2,  19.6/                                                    
C----- 1.,0E4*AO3(I,J) FOR TB                                           
      DATA (AO3(I, 1),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   4.,   7.,  11.,      
     S  15.,  21.,  27.,  33.,  39.,  45.,  49.,  53.,  57.,  59.,      
     S  60.,  60.,  60.,  58.,  56.,  54.,  52.,  49./                  
      DATA (AO3(I, 2),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   4.,   7.,  11.,      
     S  15.,  21.,  27.,  33.,  39.,  45.,  49.,  53.,  57.,  59.,      
     S  60.,  60.,  60.,  58.,  56.,  54.,  52.,  49./                  
      DATA (AO3(I, 3),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   4.,   7.,  11.,      
     S  15.,  21.,  27.,  33.,  39.,  45.,  49.,  53.,  57.,  59.,      
     S  60.,  60.,  60.,  58.,  56.,  54.,  52.,  49./                  
      DATA (AO3(I, 4),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   4.,   7.,  11.,      
     S  15.,  21.,  27.,  33.,  39.,  45.,  49.,  53.,  57.,  59.,      
     S  60.,  60.,  60.,  58.,  56.,  54.,  52.,  49./                  
      DATA (AO3(I, 5),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   4.,   7.,  11.,      
     S  15.,  21.,  27.,  33.,  39.,  45.,  49.,  53.,  57.,  59.,      
     S  60.,  60.,  59.,  58.,  56.,  54.,  52.,  49./                  
      DATA (AO3(I, 6),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   4.,   7.,  11.,      
     S  15.,  21.,  27.,  33.,  39.,  45.,  49.,  53.,  57.,  59.,      
     S  60.,  60.,  59.,  58.,  56.,  54.,  51.,  49./                  
      DATA (AO3(I, 7),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   4.,   7.,  11.,      
     S  15.,  21.,  27.,  33.,  39.,  45.,  49.,  53.,  56.,  59.,      
     S  60.,  60.,  59.,  58.,  56.,  53.,  51.,  48./                  
      DATA (AO3(I, 8),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   4.,   7.,  10.,      
     S  15.,  21.,  27.,  33.,  39.,  45.,  49.,  53.,  56.,  59.,      
     S  60.,  60.,  59.,  57.,  55.,  53.,  50.,  47./                  
      DATA (AO3(I, 9),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   4.,   7.,  10.,      
     S  15.,  21.,  27.,  33.,  39.,  44.,  49.,  53.,  56.,  58.,      
     S  59.,  59.,  59.,  57.,  55.,  52.,  49.,  46./                  
      DATA (AO3(I,10),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   4.,   7.,  10.,      
     S  15.,  21.,  27.,  33.,  39.,  44.,  49.,  53.,  56.,  58.,      
     S  59.,  59.,  58.,  56.,  54.,  51.,  48.,  44./                  
      DATA (AO3(I,11),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   4.,   7.,  10.,      
     S  15.,  21.,  27.,  33.,  39.,  44.,  49.,  52.,  55.,  57.,      
     S  58.,  58.,  57.,  55.,  52.,  49.,  46.,  42./                  
      DATA (AO3(I,12),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   4.,   7.,  10.,      
     S  15.,  21.,  27.,  33.,  39.,  44.,  48.,  52.,  55.,  57.,      
     S  57.,  57.,  55.,  53.,  50.,  47.,  44.,  40./                  
      DATA (AO3(I,13),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   4.,   6.,  10.,      
     S  15.,  20.,  26.,  33.,  38.,  43.,  47.,  51.,  54.,  55.,      
     S  56.,  55.,  53.,  51.,  48.,  44.,  41.,  37./                  
      DATA (AO3(I,14),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   4.,   6.,  10.,      
     S  15.,  20.,  26.,  32.,  37.,  42.,  46.,  50.,  52.,  53.,      
     S  53.,  52.,  50.,  48.,  44.,  41.,  37.,  33./                  
      DATA (AO3(I,15),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   4.,   6.,  10.,      
     S  14.,  20.,  25.,  31.,  36.,  41.,  45.,  48.,  49.,  50.,      
     S  50.,  49.,  47.,  44.,  41.,  37.,  34.,  30./                  
      DATA (AO3(I,16),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   3.,   6.,   9.,      
     S  14.,  19.,  24.,  30.,  34.,  39.,  42.,  45.,  46.,  47.,      
     S  46.,  44.,  42.,  39.,  36.,  33.,  30.,  27./                  
      DATA (AO3(I,17),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   3.,   5.,   9.,      
     S  13.,  17.,  22.,  27.,  32.,  36.,  38.,  40.,  42.,  42.,      
     S  41.,  40.,  37.,  35.,  32.,  30.,  27.,  24./                  
      DATA (AO3(I,18),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   3.,   5.,   8.,      
     S  11.,  16.,  20.,  25.,  28.,  32.,  34.,  36.,  36.,  36.,      
     S  36.,  35.,  33.,  31.,  29.,  26.,  24.,  22./                  
      DATA (AO3(I,19),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   2.,   4.,   6.,      
     S  10.,  13.,  17.,  21.,  24.,  27.,  29.,  30.,  31.,  31.,      
     S  31.,  30.,  29.,  27.,  26.,  24.,  22.,  20./                  
      DATA (AO3(I,20),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   2.,   3.,   5.,      
     S   7.,  10.,  13.,  17.,  19.,  22.,  23.,  25.,  25.,  26.,      
     S  26.,  26.,  25.,  24.,  23.,  22.,  20.,  19./                  
      DATA (AO3(I,21),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   2.,   4.,      
     S   5.,   7.,  10.,  12.,  15.,  17.,  18.,  20.,  21.,  22.,      
     S  23.,  23.,  23.,  22.,  22.,  21.,  19.,  18./                  
      DATA (AO3(I,22),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   2.,   2.,      
     S   4.,   5.,   7.,   9.,  11.,  13.,  14.,  16.,  17.,  19.,      
     S  20.,  21.,  21.,  21.,  21.,  20.,  19.,  18./                  
      DATA (AO3(I,23),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   2.,      
     S   2.,   3.,   4.,   6.,   8.,   9.,  11.,  13.,  15.,  17.,      
     S  18.,  19.,  20.,  20.,  20.,  19.,  19.,  18./                  
      DATA (AO3(I,24),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   1.,      
     S   1.,   2.,   3.,   4.,   6.,   7.,   9.,  11.,  13.,  15.,      
     S  16.,  18.,  19.,  20.,  20.,  19.,  19.,  17./                  
      DATA (AO3(I,25),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,      
     S   1.,   2.,   2.,   3.,   4.,   6.,   7.,   9.,  11.,  13.,      
     S  15.,  17.,  18.,  19.,  20.,  19.,  18.,  17./                  
      DATA (AO3(I,26),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   1.,   1.,   2.,   2.,   3.,   5.,   6.,   8.,  10.,  12.,      
     S  14.,  16.,  18.,  19.,  19.,  19.,  18.,  17./                  
      DATA (AO3(I,27),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   1.,   1.,   1.,   2.,   3.,   4.,   5.,   7.,   9.,  11.,      
     S  14.,  16.,  17.,  19.,  19.,  19.,  18.,  17./                  
      DATA (AO3(I,28),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   1.,   1.,   2.,   3.,   4.,   5.,   7.,   9.,  11.,      
     S  13.,  15.,  17.,  18.,  19.,  19.,  18.,  17./                  
      DATA (AO3(I,29),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   1.,   1.,   2.,   2.,   3.,   5.,   6.,   8.,  11.,      
     S  13.,  15.,  17.,  18.,  19.,  19.,  18.,  16./                  
      DATA (AO3(I,30),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   1.,   1.,   2.,   2.,   3.,   4.,   6.,   8.,  11.,      
     S  13.,  16.,  18.,  19.,  19.,  19.,  18.,  16./                  
      DATA (AO3(I,31),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   1.,   1.,   1.,   2.,   2.,   3.,   4.,   6.,   8.,  11.,      
     S  13.,  16.,  18.,  18.,  19.,  18.,  17.,  16./                  
C----- 1.,0E6*BO3(I,J) FOR TB                                           
      DATA (BO3(I, 1),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -2.,  -3.,  -3.,      
     S  -4.,  -4.,  -3.,  -2.,   0.,   2.,   4.,   5.,   6.,   6.,      
     S   5.,   4.,   2.,  -1.,  -3.,  -5.,  -7.,  -8./                  
      DATA (BO3(I, 2),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -2.,  -3.,  -3.,      
     S  -4.,  -4.,  -3.,  -2.,   0.,   2.,   4.,   5.,   6.,   6.,      
     S   5.,   4.,   2.,  -1.,  -3.,  -5.,  -7.,  -8./                  
      DATA (BO3(I, 3),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -2.,  -3.,  -3.,      
     S  -4.,  -4.,  -3.,  -2.,   0.,   2.,   4.,   5.,   6.,   6.,      
     S   5.,   4.,   2.,   0.,  -3.,  -5.,  -7.,  -8./                  
      DATA (BO3(I, 4),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -2.,  -3.,  -3.,      
     S  -4.,  -4.,  -3.,  -2.,   0.,   2.,   4.,   5.,   6.,   6.,      
     S   5.,   4.,   2.,   0.,  -3.,  -5.,  -7.,  -8./                  
      DATA (BO3(I, 5),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -2.,  -3.,  -3.,      
     S  -4.,  -4.,  -3.,  -2.,   0.,   2.,   4.,   5.,   6.,   6.,      
     S   5.,   4.,   2.,   0.,  -3.,  -5.,  -7.,  -8./                  
      DATA (BO3(I, 6),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -2.,  -3.,  -3.,      
     S  -4.,  -4.,  -3.,  -2.,   0.,   2.,   4.,   5.,   6.,   6.,      
     S   6.,   4.,   2.,   0.,  -3.,  -5.,  -7.,  -8./                  
      DATA (BO3(I, 7),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -2.,  -3.,  -3.,      
     S  -4.,  -4.,  -3.,  -2.,   0.,   2.,   4.,   5.,   6.,   6.,      
     S   6.,   4.,   2.,   0.,  -3.,  -5.,  -6.,  -8./                  
      DATA (BO3(I, 8),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -2.,  -2.,  -3.,      
     S  -4.,  -4.,  -3.,  -2.,   0.,   2.,   4.,   5.,   6.,   6.,      
     S   6.,   4.,   2.,   0.,  -3.,  -5.,  -6.,  -8./                  
      DATA (BO3(I, 9),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -2.,  -2.,  -3.,      
     S  -4.,  -4.,  -3.,  -2.,   0.,   2.,   4.,   5.,   6.,   6.,      
     S   6.,   4.,   2.,   0.,  -2.,  -4.,  -6.,  -8./                  
      DATA (BO3(I,10),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -2.,  -2.,  -3.,      
     S  -4.,  -4.,  -3.,  -2.,   0.,   2.,   4.,   5.,   6.,   6.,      
     S   6.,   4.,   2.,   0.,  -2.,  -4.,  -6.,  -7./                  
      DATA (BO3(I,11),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -2.,  -2.,  -3.,      
     S  -3.,  -4.,  -3.,  -2.,   0.,   2.,   4.,   5.,   6.,   6.,      
     S   6.,   4.,   2.,   0.,  -2.,  -4.,  -5.,  -7./                  
      DATA (BO3(I,12),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,  -2.,  -3.,      
     S  -3.,  -4.,  -3.,  -2.,   0.,   2.,   4.,   5.,   6.,   6.,      
     S   6.,   4.,   2.,   0.,  -2.,  -4.,  -5.,  -7./                  
      DATA (BO3(I,13),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -2.,  -2.,  -3.,      
     S  -3.,  -3.,  -3.,  -2.,   0.,   2.,   4.,   5.,   6.,   7.,      
     S   6.,   5.,   3.,   1.,  -1.,  -3.,  -5.,  -6./                  
      DATA (BO3(I,14),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,  -2.,  -3.,      
     S  -3.,  -3.,  -3.,  -1.,   0.,   2.,   4.,   6.,   6.,   7.,      
     S   6.,   5.,   3.,   1.,  -1.,  -3.,  -5.,  -6./                  
      DATA (BO3(I,15),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,  -2.,  -3.,      
     S  -3.,  -3.,  -3.,  -1.,   1.,   3.,   4.,   6.,   7.,   7.,      
     S   6.,   5.,   3.,   1.,  -1.,  -3.,  -5.,  -6./                  
      DATA (BO3(I,16),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,  -2.,  -3.,      
     S  -3.,  -3.,  -2.,  -1.,   1.,   3.,   4.,   6.,   7.,   7.,      
     S   6.,   5.,   3.,   1.,  -1.,  -3.,  -4.,  -5./                  
      DATA (BO3(I,17),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -2.,  -1.,  -2.,      
     S  -2.,  -2.,  -2.,   0.,   1.,   3.,   5.,   6.,   7.,   7.,      
     S   6.,   4.,   2.,   0.,  -1.,  -3.,  -4.,  -5./                  
      DATA (BO3(I,18),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,  -1.,  -2.,      
     S  -2.,  -2.,  -1.,   0.,   2.,   3.,   5.,   6.,   6.,   6.,      
     S   5.,   4.,   2.,   0.,  -2.,  -3.,  -4.,  -5./                  
      DATA (BO3(I,19),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,   1.,   2.,   3.,   4.,   5.,   5.,   5.,      
     S   4.,   3.,   1.,   0.,  -2.,  -3.,  -4.,  -5./                  
      DATA (BO3(I,20),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,   0.,  -1.,      
     S  -1.,  -1.,   0.,   1.,   2.,   3.,   4.,   4.,   5.,   4.,      
     S   4.,   2.,   1.,   0.,  -2.,  -3.,  -4.,  -4./                  
      DATA (BO3(I,21),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,   0.,      
     S   0.,   0.,   0.,   1.,   2.,   2.,   3.,   4.,   4.,   3.,      
     S   3.,   2.,   1.,  -1.,  -2.,  -3.,  -4.,  -4./                  
      DATA (BO3(I,22),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,   0.,   0.,      
     S   0.,   0.,   0.,   1.,   1.,   2.,   2.,   3.,   3.,   3.,      
     S   2.,   1.,   0.,  -1.,  -2.,  -3.,  -4.,  -4./                  
      DATA (BO3(I,23),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   1.,   1.,   1.,   2.,   2.,   2.,      
     S   2.,   1.,   0.,  -1.,  -2.,  -3.,  -3.,  -4./                  
      DATA (BO3(I,24),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,   1.,   1.,      
     S   1.,   1.,   0.,  -1.,  -1.,  -2.,  -3.,  -4./                  
      DATA (BO3(I,25),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,   0.,  -1.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   1.,      
     S   1.,   1.,   0.,   0.,  -1.,  -2.,  -3.,  -4./                  
      DATA (BO3(I,26),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,  -1.,  -2.,  -3.,  -4./                  
      DATA (BO3(I,27),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,   0.,   0.,      
     S   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,  -1.,  -1.,  -2.,  -3.,  -4./                  
      DATA (BO3(I,28),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,   0.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -2.,  -3.,  -4./                  
      DATA (BO3(I,29),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,  -1.,  -1.,      
     S  -1.,  -1.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -2.,  -2.,  -3.,  -4./                  
      DATA (BO3(I,30),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -2.,  -2.,  -3.,  -4./                  
      DATA (BO3(I,31),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   1.,   0.,   0.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -2.,      
     S  -2.,  -2.,  -1.,  -2.,  -2.,  -2.,  -3.,  -4./                  
C                                                                       
C-----  -10E+3*LN(1.-TB)   ( O3 14 micron band )                        
      DATA (TO3B(I, 1),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1234.5,1184.0,1134.2,1084.3,1034.6,   
     S 984.9, 935.6, 886.4, 837.8, 789.9, 742.9, 697.1, 652.9, 610.9,   
     S 571.6, 535.5, 502.9, 473.6, 447.3, 423.3, 401.0, 379.9, 359.8,   
     S 340.6, 321.9/                                                    
      DATA (TO3B(I, 2),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1234.5,1184.0,1134.2,1084.3,1034.6,   
     S 984.9, 935.6, 886.4, 837.8, 789.9, 742.9, 697.1, 652.9, 610.9,   
     S 571.6, 535.5, 502.9, 473.6, 447.3, 423.3, 401.0, 379.9, 359.8,   
     S 340.6, 321.9/                                                    
      DATA (TO3B(I, 3),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1234.5,1184.0,1134.2,1084.3,1034.6,   
     S 984.9, 935.6, 886.4, 837.8, 789.9, 742.9, 697.1, 652.9, 610.9,   
     S 571.6, 535.5, 502.9, 473.6, 447.3, 423.3, 401.0, 379.9, 359.8,   
     S 340.5, 321.9/                                                    
      DATA (TO3B(I, 4),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1234.5,1184.0,1134.2,1084.3,1034.6,   
     S 984.9, 935.6, 886.4, 837.8, 789.9, 742.9, 697.1, 652.9, 610.9,   
     S 571.6, 535.5, 502.9, 473.6, 447.3, 423.3, 401.0, 379.9, 359.8,   
     S 340.5, 321.9/                                                    
      DATA (TO3B(I, 5),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1234.5,1184.0,1134.2,1084.3,1034.6,   
     S 984.9, 935.5, 886.4, 837.8, 789.9, 742.9, 697.1, 652.9, 610.9,   
     S 571.6, 535.5, 502.9, 473.6, 447.3, 423.3, 400.9, 379.8, 359.7,   
     S 340.5, 321.8/                                                    
      DATA (TO3B(I, 6),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1234.5,1184.0,1134.2,1084.3,1034.6,   
     S 984.9, 935.5, 886.4, 837.8, 789.9, 742.9, 697.1, 652.9, 610.9,   
     S 571.6, 535.5, 502.8, 473.6, 447.3, 423.2, 400.9, 379.8, 359.7,   
     S 340.4, 321.7/                                                    
      DATA (TO3B(I, 7),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1234.5,1184.0,1134.2,1084.3,1034.6,   
     S 984.9, 935.5, 886.4, 837.8, 789.9, 742.9, 697.0, 652.9, 610.9,   
     S 571.6, 535.5, 502.8, 473.5, 447.2, 423.2, 400.8, 379.7, 359.6,   
     S 340.2, 321.5/                                                    
      DATA (TO3B(I, 8),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1234.5,1184.0,1134.2,1084.3,1034.6,   
     S 984.9, 935.5, 886.4, 837.8, 789.9, 742.8, 697.0, 652.9, 610.9,   
     S 571.6, 535.4, 502.8, 473.5, 447.1, 423.1, 400.7, 379.6, 359.4,   
     S 340.0, 321.3/                                                    
      DATA (TO3B(I, 9),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1234.5,1184.0,1134.2,1084.3,1034.6,   
     S 984.9, 935.5, 886.4, 837.8, 789.9, 742.8, 697.0, 652.8, 610.8,   
     S 571.5, 535.4, 502.7, 473.4, 447.0, 423.0, 400.5, 379.3, 359.1,   
     S 339.7, 320.8/                                                    
      DATA (TO3B(I,10),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1234.5,1184.0,1134.2,1084.3,1034.6,   
     S 984.9, 935.5, 886.4, 837.8, 789.9, 742.8, 697.0, 652.8, 610.8,   
     S 571.4, 535.3, 502.6, 473.2, 446.8, 422.7, 400.2, 379.0, 358.7,   
     S 339.1, 320.1/                                                    
      DATA (TO3B(I,11),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1234.5,1184.0,1134.2,1084.3,1034.6,   
     S 984.9, 935.5, 886.4, 837.8, 789.8, 742.8, 696.9, 652.8, 610.7,   
     S 571.3, 535.1, 502.4, 473.0, 446.5, 422.3, 399.7, 378.4, 357.9,   
     S 338.2, 318.9/                                                    
      DATA (TO3B(I,12),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1234.5,1184.0,1134.2,1084.3,1034.6,   
     S 984.9, 935.5, 886.4, 837.8, 789.8, 742.7, 696.9, 652.6, 610.6,   
     S 571.2, 534.9, 502.1, 472.6, 446.0, 421.7, 399.0, 377.4, 356.7,   
     S 336.7, 317.1/                                                    
      DATA (TO3B(I,13),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1234.5,1184.0,1134.2,1084.3,1034.4,   
     S 984.9, 935.4, 886.4, 837.7, 789.7, 742.6, 696.8, 652.5, 610.3,   
     S 570.9, 534.5, 501.5, 471.9, 445.2, 420.7, 397.7, 375.9, 354.8,   
     S 334.4, 314.3/                                                    
      DATA (TO3B(I,14),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1234.5,1184.0,1134.2,1084.3,1034.4,   
     S 984.8, 935.4, 886.3, 837.6, 789.6, 742.5, 696.6, 652.2, 610.0,   
     S 570.4, 533.9, 500.7, 470.9, 443.9, 419.0, 395.7, 373.4, 351.9,   
     S 330.8, 310.1/                                                    
      DATA (TO3B(I,15),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1234.5,1184.0,1134.2,1084.3,1034.4,   
     S 984.8, 935.3, 886.2, 837.5, 789.5, 742.3, 696.3, 651.8, 609.4,   
     S 569.6, 532.8, 499.4, 469.2, 441.8, 416.5, 392.6, 369.7, 347.5,   
     S 325.6, 304.0/                                                    
      DATA (TO3B(I,16),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1233.1,1184.0,1133.7,1083.9,1034.4,   
     S 984.7, 935.2, 886.0, 837.3, 789.2, 741.9, 695.8, 651.2, 608.5,   
     S 568.4, 531.3, 497.3, 466.6, 438.6, 412.6, 388.0, 364.3, 341.1,   
     S 318.3, 295.7/                                                    
      DATA (TO3B(I,17),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1233.1,1184.0,1133.7,1083.9,1034.2,   
     S 984.5, 935.0, 885.8, 837.0, 788.8, 741.4, 695.1, 650.2, 607.2,   
     S 566.6, 528.9, 494.3, 462.7, 433.8, 406.9, 381.3, 356.6, 332.4,   
     S 308.5, 284.9/                                                    
      DATA (TO3B(I,18),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1386.3,1334.0,1282.9,1233.1,1184.0,1133.7,1083.9,1034.0,   
     S 984.3, 934.8, 885.5, 836.6, 788.2, 740.6, 694.0, 648.7, 605.2,   
     S 564.0, 525.4, 489.9, 457.2, 427.2, 399.1, 372.4, 346.5, 321.2,   
     S 296.3, 271.8/                                                    
      DATA (TO3B(I,19),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1380.2,1334.0,1282.9,1233.1,1183.2,1133.7,1083.6,1033.8,   
     S 984.1, 934.5, 885.1, 836.0, 787.5, 739.6, 692.6, 646.8, 602.6,   
     S 560.5, 520.8, 484.0, 450.0, 418.5, 389.0, 361.0, 333.9, 307.6,   
     S 281.9, 256.8/                                                    
      DATA (TO3B(I,20),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1380.2,1334.0,1282.9,1233.1,1183.2,1133.2,1083.3,1033.5,   
     S 983.8, 934.0, 884.6, 835.4, 786.6, 738.3, 690.9, 644.4, 599.4,   
     S 556.2, 515.2, 476.8, 441.1, 407.9, 376.9, 347.6, 319.4, 292.2,   
     S 265.9, 240.3/                                                    
      DATA (TO3B(I,21),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1380.2,1334.0,1282.9,1233.1,1183.2,1133.2,1083.0,1033.3,   
     S 983.4, 933.7, 884.1, 834.7, 785.6, 737.0, 689.0, 641.9, 596.0,   
     S 551.6, 509.1, 468.9, 431.3, 396.3, 363.6, 332.9, 303.7, 275.8,   
     S 249.0, 223.2/                                                    
      DATA (TO3B(I,22),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1380.2,1334.0,1282.9,1233.1,1183.2,1133.2,1083.0,1033.1,   
     S 983.2, 933.4, 883.7, 834.1, 784.8, 735.8, 687.4, 639.6, 592.8,   
     S 547.2, 503.2, 461.2, 421.6, 384.6, 350.1, 317.9, 287.7, 259.1,   
     S 231.9, 206.1/                                                    
      DATA (TO3B(I,23),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1380.2,1334.0,1282.9,1233.1,1183.2,1132.7,1083.0,1033.1,   
     S 983.1, 933.2, 883.4, 833.7, 784.2, 734.9, 686.1, 637.7, 590.1,   
     S 543.5, 498.1, 454.5, 413.0, 373.9, 337.5, 303.6, 272.2, 242.8,   
     S 215.3, 189.6/                                                    
      DATA (TO3B(I,24),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1380.2,1334.0,1282.9,1233.1,1183.2,1132.7,1083.0,1032.9,   
     S 983.1, 933.1, 883.3, 833.5, 783.8, 734.3, 685.1, 636.3, 588.1,   
     S 540.6, 494.2, 449.2, 406.0, 364.9, 326.5, 290.9, 258.0, 227.7,   
     S 199.9, 174.2/                                                    
      DATA (TO3B(I,25),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1380.2,1334.0,1282.9,1233.1,1183.2,1132.7,1083.0,1032.9,   
     S 983.0, 933.1, 883.2, 833.3, 783.5, 733.9, 684.5, 635.3, 586.6,   
     S 538.6, 491.3, 445.2, 400.6, 358.0, 317.7, 280.2, 245.7, 214.3,   
     S 185.9, 160.3/                                                    
      DATA (TO3B(I,26),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1380.2,1334.0,1282.9,1233.1,1183.2,1132.7,1083.0,1032.9,   
     S 983.0, 933.0, 883.1, 833.2, 783.4, 733.6, 684.0, 634.7, 585.7,   
     S 537.1, 489.3, 442.4, 396.8, 352.8, 311.0, 271.8, 235.8, 203.1,   
     S 173.9, 148.1/                                                    
      DATA (TO3B(I,27),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1380.2,1334.0,1282.9,1233.1,1183.2,1133.2,1083.0,1033.1,   
     S 983.1, 933.0, 883.1, 833.2, 783.3, 733.5, 683.8, 634.3, 585.1,   
     S 536.2, 488.0, 440.5, 394.1, 349.2, 306.2, 265.7, 228.2, 194.3,   
     S 164.2, 138.0/                                                    
      DATA (TO3B(I,28),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1380.2,1334.0,1282.9,1233.1,1183.2,1133.2,1083.0,1033.1,   
     S 983.1, 933.1, 883.1, 833.2, 783.3, 733.4, 683.7, 634.1, 584.7,   
     S 535.7, 487.1, 439.3, 392.4, 346.8, 302.9, 261.4, 222.8, 187.7,   
     S 156.7, 130.0/                                                    
      DATA (TO3B(I,29),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1380.2,1334.0,1282.9,1233.1,1183.2,1133.2,1083.0,1033.1,   
     S 983.2, 933.2, 883.2, 833.3, 783.3, 733.5, 683.7, 634.0, 584.5,   
     S 535.4, 486.6, 438.5, 391.3, 345.2, 300.8, 258.5, 219.0, 183.1,   
     S 151.2, 124.0/                                                    
      DATA (TO3B(I,30),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1380.2,1334.0,1282.9,1233.1,1183.2,1133.2,1083.3,1033.3,   
     S 983.3, 933.3, 883.3, 833.4, 783.4, 733.6, 683.7, 634.0, 584.5,   
     S 535.2, 486.4, 438.1, 390.6, 344.3, 299.4, 256.6, 216.5, 179.9,   
     S 147.5, 119.8/                                                    
      DATA (TO3B(I,31),I=1,38)/                                         
     S1663.6,1663.6,1663.6,1663.6,1663.6,1663.6,1594.2,1524.9,1484.4,   
     S1433.3,1380.2,1334.0,1282.9,1233.1,1183.2,1133.7,1083.3,1033.5,   
     S 983.5, 933.5, 883.5, 833.6, 783.6, 733.8, 683.9, 634.2, 584.6,   
     S 535.3, 486.3, 438.0, 390.3, 343.8, 298.6, 255.5, 215.0, 177.8,   
     S 144.9, 116.9/                                                    
C----- 1.0E4*O3A(I,J) FOR TB                                            
      DATA (O3A(I, 1),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,   0.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  17.,  23.,  28.,      
     S  34.,  40.,  45.,  49.,  52.,  55.,  57.,  58./                  
      DATA (O3A(I, 2),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,   0.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  17.,  23.,  28.,      
     S  34.,  40.,  44.,  49.,  52.,  55.,  57.,  58./                  
      DATA (O3A(I, 3),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,   0.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  17.,  23.,  28.,      
     S  34.,  40.,  44.,  49.,  52.,  55.,  57.,  58./                  
      DATA (O3A(I, 4),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,   0.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  17.,  23.,  28.,      
     S  34.,  40.,  44.,  49.,  52.,  55.,  57.,  58./                  
      DATA (O3A(I, 5),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,   0.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  17.,  23.,  28.,      
     S  34.,  40.,  44.,  49.,  52.,  55.,  57.,  58./                  
      DATA (O3A(I, 6),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,   0.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  17.,  23.,  28.,      
     S  34.,  40.,  44.,  49.,  52.,  55.,  57.,  58./                  
      DATA (O3A(I, 7),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,   0.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  17.,  23.,  28.,      
     S  34.,  40.,  44.,  48.,  52.,  54.,  56.,  58./                  
      DATA (O3A(I, 8),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,   0.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  17.,  22.,  28.,      
     S  34.,  39.,  44.,  48.,  52.,  54.,  56.,  57./                  
      DATA (O3A(I, 9),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,   0.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  17.,  22.,  28.,      
     S  34.,  39.,  44.,  48.,  51.,  54.,  56.,  57./                  
      DATA (O3A(I,10),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,   0.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  17.,  22.,  28.,      
     S  34.,  39.,  44.,  48.,  51.,  54.,  55.,  56./                  
      DATA (O3A(I,11),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,   0.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   1.,   2.,   3.,   4.,   6.,   9.,  13.,  17.,  22.,  28.,      
     S  34.,  39.,  44.,  47.,  50.,  53.,  54.,  55./                  
      DATA (O3A(I,12),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,   0.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   1.,   2.,   3.,   4.,   6.,   9.,  12.,  17.,  22.,  28.,      
     S  33.,  38.,  43.,  47.,  50.,  52.,  53.,  53./                  
      DATA (O3A(I,13),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,   0.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   1.,   2.,   3.,   4.,   6.,   9.,  12.,  17.,  22.,  27.,      
     S  33.,  38.,  42.,  45.,  48.,  50.,  51.,  51./                  
      DATA (O3A(I,14),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,  -1.,  -1.,  -1.,  -1.,   0.,   0.,   0.,      
     S   1.,   1.,   3.,   4.,   6.,   8.,  12.,  16.,  21.,  26.,      
     S  32.,  36.,  41.,  44.,  46.,  47.,  48.,  47./                  
      DATA (O3A(I,15),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   0.,      
     S   1.,   1.,   2.,   4.,   6.,   8.,  11.,  15.,  20.,  25.,      
     S  30.,  35.,  38.,  41.,  43.,  44.,  44.,  43./                  
      DATA (O3A(I,16),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   0.,      
     S   0.,   1.,   2.,   3.,   5.,   7.,  10.,  14.,  19.,  23.,      
     S  28.,  32.,  35.,  38.,  39.,  39.,  39.,  38./                  
      DATA (O3A(I,17),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -2.,  -1.,  -2.,  -1.,  -1.,  -1.,   0.,   0.,      
     S   0.,   1.,   2.,   3.,   4.,   6.,   9.,  13.,  17.,  21.,      
     S  25.,  28.,  31.,  33.,  34.,  34.,  34.,  33./                  
      DATA (O3A(I,18),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S  -6.,   0.,  -2.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,      
     S   0.,   0.,   1.,   2.,   3.,   5.,   8.,  10.,  14.,  17.,      
     S  21.,  24.,  26.,  28.,  29.,  29.,  28.,  28./                  
      DATA (O3A(I,19),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S  -6.,   0.,  -2.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S   0.,   0.,   0.,   1.,   2.,   4.,   6.,   8.,  11.,  14.,      
     S  17.,  19.,  21.,  23.,  23.,  24.,  24.,  23./                  
      DATA (O3A(I,20),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S  -6.,   0.,   0.,  -1.,  -2.,  -1.,  -2.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,   0.,   0.,   1.,   2.,   4.,   5.,   7.,  10.,      
     S  12.,  14.,  16.,  18.,  19.,  19.,  20.,  20./                  
      DATA (O3A(I,21),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S  -6.,   0.,   0.,  -3.,  -2.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,   0.,   0.,   1.,   2.,   3.,   5.,   6.,      
     S   8.,  10.,  12.,  14.,  15.,  16.,  17.,  17./                  
      DATA (O3A(I,22),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S  -6.,  -4.,   0.,  -1.,  -2.,  -1.,  -2.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,   0.,   0.,   1.,   2.,   3.,   4.,      
     S   6.,   7.,   9.,  11.,  12.,  14.,  15.,  16./                  
      DATA (O3A(I,23),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,  -4.,   0.,  -1.,  -2.,  -1.,  -2.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   0.,   1.,   1.,   2.,      
     S   4.,   5.,   7.,   9.,  10.,  12.,  13.,  14./                  
      DATA (O3A(I,24),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,  -4.,   0.,  -1.,  -2.,  -1.,  -2.,  -1.,  -2.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   0.,   1.,   1.,      
     S   2.,   4.,   5.,   7.,   9.,  10.,  12.,  13./                  
      DATA (O3A(I,25),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,  -4.,   0.,  -1.,  -2.,  -1.,  -2.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   0.,   1.,      
     S   2.,   3.,   4.,   6.,   7.,   9.,  11.,  12./                  
      DATA (O3A(I,26),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,  -4.,   0.,  -1.,  -2.,  -1.,  -2.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   0.,      
     S   1.,   2.,   3.,   4.,   6.,   8.,  10.,  11./                  
      DATA (O3A(I,27),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S  -6.,  -4.,   0.,  -1.,  -2.,  -1.,  -2.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   0.,      
     S   1.,   1.,   2.,   4.,   5.,   7.,   9.,  11./                  
      DATA (O3A(I,28),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S  -6.,   0.,   0.,  -3.,  -2.,  -1.,  -2.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,      
     S   0.,   1.,   2.,   3.,   5.,   6.,   8.,  10./                  
      DATA (O3A(I,29),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S  -6.,   0.,   0.,  -1.,  -2.,  -1.,  -2.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,      
     S   0.,   1.,   2.,   3.,   4.,   6.,   7.,   9./                  
      DATA (O3A(I,30),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S  -6.,   0.,   0.,  -1.,  -2.,  -1.,  -2.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,      
     S   0.,   1.,   1.,   2.,   4.,   5.,   7.,   9./                  
      DATA (O3A(I,31),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S  -6.,   0.,  -2.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,      
     S   0.,   0.,   1.,   2.,   3.,   5.,   6.,   8./                  
C----- 1.0E6*O3B(I,J) FOR TB                                            
      DATA (O3B(I, 1),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,   0.,  -2.,  -1.,  -1.,  -1.,  -2.,  -1.,      
     S  -2.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -5.,  -5.,  -4.,      
     S  -3.,  -1.,   1.,   2.,   3.,   4.,   5.,   6./                  
      DATA (O3B(I, 2),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,   0.,  -2.,  -1.,  -1.,  -1.,  -2.,  -1.,      
     S  -2.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -5.,  -5.,  -4.,      
     S  -3.,  -1.,   1.,   2.,   3.,   4.,   5.,   6./                  
      DATA (O3B(I, 3),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,   0.,  -2.,  -1.,  -1.,  -1.,  -2.,  -1.,      
     S  -2.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -5.,  -5.,  -4.,      
     S  -3.,  -1.,   1.,   2.,   3.,   4.,   5.,   6./                  
      DATA (O3B(I, 4),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,   0.,  -2.,  -1.,  -1.,  -1.,  -2.,  -1.,      
     S  -2.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -5.,  -5.,  -4.,      
     S  -3.,  -1.,   1.,   2.,   3.,   4.,   5.,   6./                  
      DATA (O3B(I, 5),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,   0.,  -2.,  -1.,  -1.,  -1.,  -2.,  -2.,      
     S  -2.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -5.,  -5.,  -4.,      
     S  -3.,  -1.,   1.,   2.,   3.,   4.,   5.,   6./                  
      DATA (O3B(I, 6),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,   0.,  -2.,  -1.,  -1.,  -1.,  -2.,  -2.,      
     S  -2.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -5.,  -5.,  -4.,      
     S  -3.,  -1.,   1.,   2.,   3.,   4.,   5.,   6./                  
      DATA (O3B(I, 7),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,   0.,  -2.,  -1.,  -1.,  -1.,  -2.,  -2.,      
     S  -2.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -5.,  -5.,  -4.,      
     S  -3.,  -1.,   1.,   2.,   3.,   4.,   5.,   6./                  
      DATA (O3B(I, 8),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,   0.,  -2.,  -1.,  -1.,  -1.,  -2.,  -2.,      
     S  -2.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -5.,  -5.,  -4.,      
     S  -3.,  -1.,   1.,   2.,   3.,   4.,   5.,   6./                  
      DATA (O3B(I, 9),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,   0.,  -2.,  -1.,  -1.,  -1.,  -2.,  -2.,      
     S  -2.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -5.,  -5.,  -4.,      
     S  -3.,  -1.,   1.,   2.,   3.,   5.,   5.,   6./                  
      DATA (O3B(I,10),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,   0.,  -2.,  -1.,  -1.,  -1.,  -2.,  -2.,      
     S  -2.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -5.,  -5.,  -4.,      
     S  -3.,  -1.,   1.,   2.,   3.,   5.,   5.,   6./                  
      DATA (O3B(I,11),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,   0.,  -2.,  -1.,  -1.,  -1.,  -1.,  -2.,      
     S  -2.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -5.,  -5.,  -4.,      
     S  -3.,  -1.,   1.,   2.,   4.,   5.,   6.,   6./                  
      DATA (O3B(I,12),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,   0.,  -2.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -2.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -5.,  -5.,  -4.,      
     S  -3.,  -1.,   1.,   2.,   4.,   5.,   6.,   6./                  
      DATA (O3B(I,13),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,   0.,  -2.,  -1.,  -1.,  -1.,  -1.,  -2.,      
     S  -2.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -4.,  -4.,  -4.,      
     S  -2.,  -1.,   1.,   3.,   4.,   5.,   6.,   6./                  
      DATA (O3B(I,14),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,   3.,  -2.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -2.,  -2.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -4.,  -3.,      
     S  -2.,  -1.,   1.,   3.,   4.,   5.,   6.,   6./                  
      DATA (O3B(I,15),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,   3.,  -2.,   0.,  -1.,  -1.,  -1.,  -1.,      
     S  -2.,  -2.,  -2.,  -2.,  -3.,  -3.,  -4.,  -4.,  -4.,  -3.,      
     S  -2.,   0.,   2.,   3.,   4.,   5.,   6.,   6./                  
      DATA (O3B(I,16),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,  -3.,  -2.,  -2.,  -1.,  -1.,  -1.,  -1.,      
     S  -2.,  -2.,  -2.,  -2.,  -2.,  -3.,  -3.,  -3.,  -3.,  -3.,      
     S  -1.,   0.,   2.,   3.,   4.,   5.,   6.,   6./                  
      DATA (O3B(I,17),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   0.,  -4.,  -3.,   0.,  -2.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -2.,  -2.,  -2.,  -2.,  -3.,  -3.,  -3.,  -2.,      
     S  -1.,   1.,   2.,   3.,   4.,   5.,   5.,   5./                  
      DATA (O3B(I,18),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S  12.,   0.,  -4.,  -3.,   2.,   0.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -2.,  -2.,  -2.,  -2.,  -2.,  -1.,      
     S   0.,   1.,   2.,   3.,   4.,   4.,   4.,   4./                  
      DATA (O3B(I,19),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S -12.,   0.,  -4.,  -3.,  -2.,   0.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S   0.,   1.,   2.,   3.,   3.,   3.,   3.,   3./                  
      DATA (O3B(I,20),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S -12.,   0.,   0.,  -3.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,      
     S   0.,   1.,   2.,   2.,   2.,   2.,   2.,   2./                  
      DATA (O3B(I,21),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S -12.,   0.,   0.,   0.,   0.,  -1.,  -1.,  -1.,  -1.,   0.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   0.,      
     S   0.,   1.,   1.,   1.,   2.,   2.,   1.,   1./                  
      DATA (O3B(I,22),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S -12.,   7.,   0.,   3.,   0.,   1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,   0.,      
     S   0.,   0.,   0.,   1.,   1.,   1.,   1.,   0./                  
      DATA (O3B(I,23),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   7.,   0.,   3.,   0.,  -1.,  -1.,   0.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,      
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0./                  
      DATA (O3B(I,24),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   7.,   0.,   3.,   0.,  -1.,  -1.,  -1.,   0.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,   0.,   0.,   0.,   0.,   0.,   0./                  
      DATA (O3B(I,25),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   7.,   0.,   3.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,   0.,  -1./                  
      DATA (O3B(I,26),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S   0.,   7.,   0.,   3.,   0.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1./                  
      DATA (O3B(I,27),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S -12.,   7.,   0.,   3.,   0.,   1.,  -1.,   0.,   0.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1./                  
      DATA (O3B(I,28),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S -12.,   0.,   0.,   0.,   0.,   1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1./                  
      DATA (O3B(I,29),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S -12.,   0.,   0.,  -3.,   0.,  -1.,  -1.,  -1.,   0.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -2.,  -2./                  
      DATA (O3B(I,30),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S -12.,   0.,   0.,  -3.,   0.,  -1.,  -1.,  -1.,   0.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -2.,  -2.,  -2./                  
      DATA (O3B(I,31),I=1,38)/                                          
     S   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,      
     S -12.,   0.,  -4.,  -3.,  -2.,   0.,  -1.,  -1.,   0.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,  -1.,      
     S  -1.,  -1.,  -1.,  -1.,  -1.,  -2.,  -2.,  -2./                  
c                                                                       
      DO 1000 I=1,40                                                    
        DO 1000 J=1,31                                                  
          DO 1000 K=1,2                                                 
            ALP(I,J,K)=ALP(I,J,K)*1.D-04                                
            BET(I,J,K)=BET(I,J,K)*1.D-07                                
 1000 CONTINUE                                                          
      DO 1001 I=1,38                                                    
        DO 1001 J=1,31                                                  
          AO3(I,J)=AO3(I,J)*1.D-04                                      
          BO3(I,J)=BO3(I,J)*1.D-06                                      
          O3A(I,J)=O3A(I,J)*1.D-04                                      
          O3B(I,J)=O3B(I,J)*1.D-06                                      
 1001 CONTINUE                                                          
      RETURN                                                            
      END                                                               
C                                                                       
      SUBROUTINE SUSW                                                   
C                                                                       
C**** *SUSW*   - INITIALIZE COMMON YOMSW                                
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C           INITIALIZE YOMSW, THE COMMON THAT CONTAINS COEFFICIENTS     
C           NEEDED TO RUN THE SHORTWAVE RADIATION SUBROUTINES           
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C        *CALL* *SUSW                                                   
C                                                                       
C        EXPLICIT ARGUMENTS                                             
C        --------------------                                           
C        NONE                                                           
C                                                                       
C        IMPLICIT ARGUMENTS                                             
C        --------------------                                           
C        COMMON YOMSW                                                   
C                                                                       
C     METHOD.                                                           
C     -------                                                           
C        SEE DOCUMENTATION                                              
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C     REFERENCE.                                                        
C     ----------                                                        
C        ECMWF RESEARCH DEPARTMENT DOCUMENTATION                        
C                                                                       
C     AUTHOR.                                                           
C     -------                                                           
C        JEAN-JACQUES MORCRETTE *ECMWF*                                 
C                                                                       
C     MODIFICATIONS.                                                    
C     --------------                                                    
C        ORIGINAL : 88-12-15                                            
C     ------------------------------------------------------------------
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRSW* SWITCHES RELATED TO THE RADIATION SCHEME.         
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      IMPLICIT REAL (A-H,O-Z)                                           
C      IMPLICIT UNDEFINED (Z)                                           
C      IMPLICIT LOGICAL (L)                                             
C      IMPLICIT UNDEFINED (L)                                           
      IMPLICIT INTEGER (I-K,M-N)                                        
C      IMPLICIT UNDEFINED (I-K,M-N)                                     
C      IMPLICIT REAL (P)                                                
C      IMPLICIT UNDEFINED (A-H,O,Q-Y)                                   
C                                                                       
C everything except the parameters is defined                           
C in the common blocks                                                  
C                                                                       
      COMMON/COM1/                                                      
C     I NRADFR, NRPART, NRINT, NRADPFR, NRADPLA,                        
C     I NRLSTI,                                                         
     R XLAT,                                                            
     R RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR, ZCCO2,              
C     R PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3,                 
     L LOH, LOM, LOL, LORAD                                             
      COMMON/COM2/                                                      
     I ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,                  
     I         JLOOP, KRLEV,                                            
     I        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     I        ITH, ITM, ITL,                                            
     I        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     I        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     I        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     I        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      COMMON/COM3/                                                      
     I IND4, KM12, KXD, KXU, IG, KBS, KDD,                              
     I        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     I        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     I        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     I        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     I        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
      COMMON/COM4/                                                      
     R           COZQC,COZQS,COZHC,COZHS,                               
     R CSUPSAT,                                                         
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
      COMMON/COM5/                                                      
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NUMVC,                                                           
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      COMMON/COM6/                                                      
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD,       
     R DTIME,    TWODT,    EPS,                                         
     R SINLON,     COSLON,                                              
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2,                          
     I NFAX,                                                            
     R CRAE,                                                            
     R CDISSEM                                                          
      COMMON/COM7/                                                      
     R     CCRH,                                                        
     L LO,LO1,LO2,LODIA,LODIAP,                                         
     R TAUA, PIZA, CGA, CAER, CAEROS                                    
      COMMON/COM7A/                                                     
     R  RT1,WG1,AT,BT,CNTNU,O1H,O2H,PIALF0                              
     S ,OCT,TREF,XP,TSTAND,GA,GB,CE,ETY,PTY                             
      COMMON/COM7B/                                                     
     S TINTP,TSTP,                                                      
     S  TBO3,AO3,BO3,TB,ALP,BET,ULG,ULGO,PLG,AP,BP,TO3B,O3A,O3B,        
     R          CDAY,CCO2,CH2O,DIFF,X10E                                
       COMMON/COM7C/                                                    
     S  ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW,                       
     R  APAD,BPAD,D,CRAY,SUN,                                           
     R   TTT,                                                           
     R   TM1                                                            
      COMMON/COM8/                                                      
     R       TSM1M,                                                     
     R       QM1,                                                       
     R       SNM1M,                                                     
     R       ALBM,                                                      
     R       AVVELM,                                                    
     R   SLMM,                                                          
     R       APHM1,                                                     
     R       APM1                                                       
      COMMON/COM9/                                                      
     L LOLAND,                                                          
     R   ZTI,                                                           
     *       ZCTI,                                                      
     *       ZCPH,                                                      
     *       ZWV,                                                       
     *       ZSW,                                                       
     *       ZALTE,                                                     
     *       ZCALTE,                                                    
     *       ZALSO                                                      
      COMMON/COM10/                                                     
     *       ZCALSO,                                                    
     R   ZOZQ,                                                          
     *       ZOZH,                                                      
     R   ZTRSOL,                                                        
     *       ZEMTER,                                                    
     R   TRSOLM,                                                        
     *       EMTERM,                                                    
     R   AMU0,                                                          
     *       RDAYL                                                      
      COMMON/COM11/                                                     
     R   CLC,                                                           
     *       TI,                                                        
     *       DP,                                                        
     *       AADS,                                                      
     *       AEQ1,                                                      
     *       AEQ2,                                                      
     *       AEQ3,                                                      
     *       AEQ4,                                                      
     *       AEQ5,                                                      
     *       SSW                                                        
      COMMON/COM12/                                                     
     *       WV,                                                        
     *       QOF,                                                       
     *       QCF,                                                       
     *       CLWA,                                                      
     *       APRE,                                                      
     *       SMU0,                                                      
     *       ALTE,                                                      
     *       ALSO                                                       
      COMMON/COM13/                                                     
     R   ZDPO,                                                          
     *       ZQCFO,                                                     
     *       ZAEQSO,                                                    
     *       ZQOFO,                                                     
     *       ZDPN,                                                      
     *       ZQCFN,                                                     
     *       ZAEQSN,                                                    
     *       ZQOFN                                                      
      COMMON/COM14/                                                     
     R   FLT,                                                           
     *       FLS,                                                       
     R   DIAG,                                                          
     *       DIA1,                                                      
     *       ZDIAG,                                                     
     *       ZDIA1,                                                     
     *       ZDIA,                                                      
     *       ZDIAF,                                                     
     *       ZDIAT,                                                     
     R PGG                                                              
      COMMON/COM15/                                                     
     #     PREF,                                                        
     #     PRE1,                                                        
     #     PRE2,                                                        
     #     PRMUZ,                                                       
     #     PTO1,                                                        
     #     PTR1,                                                        
     #     PTR2,                                                        
     #     PW,                                                          
     R     PALBSN                                                       
      COMMON/COM16/                                                     
     R ZABSCU,                                                          
     S     ZABSLY,                                                      
     S     ZADJD ,                                                      
     S     ZADJU ,                                                      
     S     ZBGND ,                                                      
     S     ZBINT ,                                                      
     S     ZBSUR ,                                                      
     S     ZBSUIN,                                                      
     S     ZBTOP ,                                                      
     S     PCLDLW,                                                      
     S     ZCLOUD,                                                      
     S     ZCNTRB,                                                      
     S     ZCUMP ,                                                      
     S     ZCUMPT,                                                      
     S     ZDBDT                                                        
      COMMON/COM17/                                                     
     S     ZDBSL ,                                                      
     S     ZDISTD,                                                      
     S     ZDISTU,                                                      
     S     ZDNF  ,                                                      
     S     PDT0  ,                                                      
     S     ZDUC                                                         
      COMMON/COM18/                                                     
     S     ZEFFT ,                                                      
     S     ZEMU  ,                                                      
     S     ZEPHIO,                                                      
     S     ZEPSC2,                                                      
     S     ZEPSC3,                                                      
     S     ZEPSH1,                                                      
     S     ZEPSH2,                                                      
     S     ZEPSH3                                                       
      COMMON/COM19/                                                     
     S     ZEPSH4,                                                      
     S     ZEPSH5,                                                      
     S     ZEPSH6,                                                      
     S     ZEPSIO,                                                      
     S     ZFACTC,                                                      
     S     ZZFD  ,                                                      
     S     ZFDD  ,                                                      
     S     ZFDN                                                         
      COMMON/COM20/                                                     
     S     PFLUX , PFLUC,                                               
     S     ZFLWP , PCTS                                                 
     S     ZZFU  ,                                                      
     S     ZFUP  ,                                                      
     S     ZGLAYD,                                                      
     S     ZGLAYU,                                                      
     S     ZOMEGA,                                                      
     S     ITX   ,                                                      
     S     IXDIA ,                                                      
     S     IXT   ,                                                      
     S     IXTSU ,                                                      
     S     IXTTP ,                                                      
     S     IMX   ,                                                      
     S     IMXP                                                         
      COMMON/COM21/                                                     
     R ZOZ   ,                                                          
     S     PPMB  ,                                                      
     S     ZRES  , PB                                                   
     S     ZSSIG ,                                                      
     S     PTAU  ,                                                      
     S     PTAVE ,                                                      
     S     ZTAVIC,                                                      
     S     ZTI1  ,                                                      
     S     PTL                                                          
      COMMON/COM22/                                                     
     S     ZTT   ,                                                      
     S     ZTT1  ,                                                      
     S     ZTT2  ,                                                      
     S     ZUAER ,                                                      
     S     ZUPF  ,                                                      
     S     ZUU   ,                                                      
     S     ZXD                                                          
      COMMON/COM23/                                                     
     S     ZXN   ,                                                      
     S     ZXOZ  ,                                                      
     S     ZXWV  ,                                                      
     S     ZZZ   ,                                                      
     S     ZRES2 ,                                                      
     S     ZTI2  ,                                                      
     S     ZBLAY ,                                                      
     S     ZBLEV ,                                                      
     S     ZCTS  ,                                                      
     S     ZHFG                                                         
      COMMON/COM24/                                                     
     I KT,                                                              
     R    PUU   ,                                                       
     S        PTT   ,                                                   
     S        ZZ                                                        
     R  PCLFR  ,                                                        
     S      PT     ,                                                    
     S      PQOF   ,                                                    
     S      PQLWP  ,                                                    
     S      PEMIS  ,                                                    
     S      PFLT                                                        
      COMMON/COM25/                                                     
     R  ZALBSU,                                                         
     S      ZCG   ,                                                     
     S      ZCLDLW,                                                     
     S      ZCLDSW,                                                     
     S      ZCOOLR,                                                     
     S      ZDT0  ,                                                     
     S      ZEMD  ,                                                     
     S      ZFLUX ,                                                     
     S      ZPMB  ,                                                     
     S      ZTAU  ,                                                     
     S      ZTAVE ,                                                     
     S      ZTL   ,                                                     
     S      ZFSDWN                                                      
      COMMON/COM26/                                                     
     S      ZFSUP ,                                                     
     S      ZHEATR,                                                     
     R   PDP   ,                                                        
     S       PWV   ,                                                    
     S       PFLS  ,                                                    
     S       PPSOL ,                                                    
     S       PRMU0 ,                                                    
     S       PAER  ,                                                    
     S       PALBS ,                                                    
     S       PCG   ,                                                    
     S       PCLDSW,                                                    
     S       POMEGA,                                                    
     S       POZ                                                        
      COMMON/COM27/                                                     
     R   PFDOWN,                                                        
     S       PFUP  ,                                                    
     R   ZAKI  ,                                                        
     S       ZCGAZ ,                                                    
     S       ZCNEB ,                                                    
     S       ZC1I  ,                                                    
     S       ZDSC  ,                                                    
     S       ZDSH                                                       
      COMMON/COM28/                                                     
     S       ZDSIG ,                                                    
     S       ZFACT ,                                                    
     S       ZFACT1 ,                                                   
     S       ZFACT2 ,                                                   
     S       ZFACT3 ,                                                   
     S       ZFACT4 ,                                                   
     S       ZFACT5 ,                                                   
     S       ZFACT6 ,                                                   
     S       ZFD   ,                                                    
     S       ZFRAC ,                                                    
     S       ZFU   ,                                                    
     S       ZG    ,                                                    
     S       ZGG   ,                                                    
     S       ZPIZAZ                                                     
      COMMON/COM29/                                                     
     S       ZRAYL ,                                                    
     S       ZRAY1 ,                                                    
     S       ZRAY2 ,                                                    
     S       ZREF  ,                                                    
     S       ZREFZ ,                                                    
     S       ZRE1  ,                                                    
     S       ZRE2  ,                                                    
     S       ZRJ   ,                                                    
     S       ZRK   ,                                                    
     S       ZRL   ,                                                    
     S       ZRMU  ,                                                    
     S       ZRMUE ,                                                    
     S       ZRMUZ                                                      
      COMMON/COM30/                                                     
     S       ZRNEB ,                                                    
     S       ZRUEF ,                                                    
     S       ZR1   ,                                                    
     S       ZR2   ,                                                    
     S       ZR21  ,                                                    
     S       ZR22  ,                                                    
     S       ZR23  ,                                                    
     S       ZS    ,                                                    
     S       ZSEC  ,                                                    
     S       ZSS1  ,                                                    
     S       ZTAUAZ                                                     
      COMMON/COM31/                                                     
     S       ZTOT  ,                                                    
     S       ZTO1  ,                                                    
     S       ZTR   ,                                                    
     S       ZTRA1 ,                                                    
     S       ZTRA2 ,                                                    
     S       ZTR1                                                       
      COMMON/COM32/                                                     
     S       ZTR2  ,                                                    
     S       ZUD   ,                                                    
     S       ZUM   ,                                                    
     S       ZU1D  ,                                                    
     S       ZU2D  ,                                                    
     S       ZW    ,                                                    
     S       ZN175 ,                                                    
     S       ZN190 ,                                                    
     S       ZO175 ,                                                    
     S       ZO190 ,                                                    
     S       ZP75  ,                                                    
     S       ZP90  ,                                                    
     S       ZSIGN ,                                                    
     S       ZSIGO                                                      
      COMMON/COM33/                                                     
     R   PTR,                                                           
     S       PU,                                                        
     R   ZRH,                                                           
     *       ZCH,                                                       
     *       ZCM,                                                       
     *       ZCL,                                                       
     *       ZCT,                                                       
     *       ZCC,                                                       
     I NTOPCM                                                           
      COMMON/COM34/                                                     
     *        NBASECM,                                                  
     R   ARPRCM,                                                        
     I ITOPH,                                                           
     *        ITOPM,                                                    
     *        ITOPL,                                                    
     L LOC,                                                             
     R   ZQS,                                                           
     *       ZCLC,                                                      
     *       ZQLWP                                                      
      COMMON/COM35/                                                     
     R   VERVEL,                                                        
     *       ZTHETA,                                                    
     *       ZETA,                                                      
     *       ZDTHMIN,                                                   
     I INVB,                                                            
     R   ZCTLWP,                                                        
     *       ZQLOC,                                                     
     *       ZQMEC,                                                     
     *       ZQHIC,                                                     
     R   ZFSOT,                                                         
     #       ZFTET,                                                     
     #       ZFLT                                                       
      COMMON/COM36/                                                     
     #       ZFSO ,                                                     
     #       ZFTE ,                                                     
     #       ZFLB ,                                                     
     #       ZDTDT,                                                     
     #       HEATO,                                                     
     #       HEATE,                                                     
     #       HEATR,                                                     
     #       ZI0                                                        
C                                                                       
      INTEGER                                                           
C     I NRADFR,   NRPART,   NRINT,    NRLSTI,  NRADPFR,  NRADPLA,       
     I NRLST,    NRLEV,    NRLEVP1, NLEV, NLP2, NLEVP1,                 
     I NLON, IRLEV, NTRA, NLAT, ITIM, NUA, NINT, NRPARTI                
C                                                                       
C   NRLEV .... number of vertical levels                                
C   NRLST .... number of points on a latitude line                      
C   NUA   .... number of absorber amount                                
C   NINT  .... number of spectral intervals                             
C   NTRA  .... number of transmission functions                         
C   NLAT  .... number of latitude points                                
C   ITIM  .... number of month under consideration                      
C                                                                       
C      PARAMETER (NRLEV=122,NUA=34,NINT=6,NTRA=34, NLAT=1,              
      PARAMETER (NRLEV=21,NUA=34,NINT=6,NTRA=34, NLAT=1,                
cccm      PARAMETER (NRLEV=37,NUA=18,NINT=6,NTRA=19, NLAT=1,            
cccm     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=10,NLEV=NRLEV,        
     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=7,NLEV=NRLEV,             
     #           NLP2=NRLST, NLEVP1=NRLEVP1, NLON=NRLST,                
     #           NRPARTI=1)                                             
C                                                                       
      REAL YCLOCK, XLAT(NLAT), Z0, DZ, ALBMPAR, SNPAR,                  
     #     TSUPAR, CTFREEZ, DELTAC, DELTAO, DELTAH                      
C                                                                       
C   YTIME  .... time of the year in radians                             
C   YCLOCK .... time of the day in radians                              
C   XLAT   .... latitudes in radians                                    
C   Z0     .... zero point of height grid                               
C   DZ     .... Interval between height levels                          
C   ALBMPAR.... sw surface albedo as function of longitude:             
C           yearly averaged sea-ice: 0.55                               
C                           water  : 0.07                               
C                           land   : 0.07.lt.ALBM.lt.0.80               
C                                    depending on snow                  
C   SNPAR  .... snow depth in meter                                     
C   TSUPAR .... surface temperature (Kelvin)                            
C   CTFREEZ.... freezing temperature of sea-water (Kelvin)              
C   DELTAC .... Doppler parameter for line shapes of CO2                
C   DELTAO .... Doppler parameter for line shapes of O3                 
C                                                                       
      PARAMETER (Z0=0., CTFREEZ=271.26,                                 
C     PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
     #           DZ=0.289, ALBMPAR=0.2, SNPAR=0.,                       
cccm  PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
cccm     #           DZ=0.15, ALBMPAR=0.3, SNPAR=0.,                    
CZ     #           DELTAC=0., DELTAO=0.)                                
     #           DELTAC=6.0E-04, DELTAO=1.E-02, DELTAH=5.0E-04)         
C      PARAMETER (YCLOCK=0., Z0=0., TSUPAR=273., CTFREEZ=271.26,        
C     #           DZ=0.46875, ALBMPAR=0.07, SNPAR=0.)                   
C      PARAMETER (YCLOCK=0., Z0=2.608, TSUPAR=273., CTFREEZ=271.26,     
C     #           DZ=0.2878, ALBMPAR=0.07, SNPAR=0.)                    
C                                                                       
      LOGICAL LDIUR, LOLPAR                                             
C                                                                       
C   LDIUR .... true: sun at time, false: diurnally averaged             
C   LOLPAR.... true: land surface, false: else                          
C                                                                       
C      PARAMETER (LDIUR=.TRUE., LOLPAR=.TRUE.)                          
       PARAMETER (LOLPAR=.TRUE.)                                        
C                                                                       
      INTEGER  C2TEMP                                                   
      CHARACTER*64  MIDTMP                                              
cccm      PARAMETER (C2TEMP=10,                                         
cccm     #    MIDTMP='/home/atm/ugamp/midrad/radfildat.dir/MIDTMP.DAT') 
ccc1      PARAMETER (C2TEMP=10, MIDTMP='tprfil2.dat')                   
C                                                                       
      INTEGER  IRLONI, IRLST, IRLVP1, JPNLEV, JPNLVP1                   
C                                                                       
      PARAMETER (IRLONI=NRLST, IRLST=NRLST,                             
     #           IRLVP1=NRLEVP1, JPNLEV=NRLEV,                          
     #           JPNLVP1=NRLEVP1, IRLEV=NRLEV)                          
C                                                                       
      INTEGER NIPD, NIPD2, NTR, NG1, NG1P1                              
      PARAMETER(NIPD=8,NIPD2=16,NTR=11,NG1=2,NG1P1=3)                   
C                                                                       
C    definition of the functions                                        
C                                                                       
      REAL       CVMGT, SIGMA2                                          
      INTEGER    CVMGI                                                  
C                                                                       
C    definitons of the subroutine cloud                                 
C                                                                       
      REAL RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR                  
C                                                                       
C    missing definitions in morcrette                                   
C                                                                       
C      REAL   PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3            
      REAL ZCCO2                                                        
                                                                        
      LOGICAL LOH, LOM, LOL, LORAD                                      
C                                                                       
      INTEGER ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,           
     #        IZQS, IZCLC, IZQLWP, JLOOP, KRLEV,                        
     #        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     #        ITH, ITM, ITL,                                            
     #        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     #        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     #        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     #        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      INTEGER IND4, KM12, KXD, KXU, IG, KBS, KDD,                       
     #        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     #        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     #        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     #        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     #        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD1* SPECTRAL DISTRIBUTION OF AEROSOLS AND OZONE.     
C                     (TRIANGULAR *T10* TRUNCATIONFOR AEROSOLS).        
C                     (TRIANGULAR *T5* TRUNCATIONFOR OZONE).            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R           COZQC(21),COZQS(15),COZHC(21),COZHS(15)                
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD2* CONSTANTS INTERNAL TO THE RADIATION SUBROUTINES. 
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CSUPSAT                                                          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMCON* BASIC UNIVERSAL CONSTANTS AND DERIVED CONSTANTS.  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
C                                                                       
C                                                                       
C       --------------------------------------------------------------- 
C      PARAMETER(                                                       
C     I JPM=106,JPN=106,JPK=106,JPGL=160,JPNLON=320,                    
C     I JPT1MAX=400,          JPT2MAX=50,                               
C     I JPT3MAX=300,          JPT4MAX=50,                               
C     I JPHASH=2048,          JPNTRCRS=20,                              
C     I JPTASKS=4,     JPLOCKS=50,    JPEVENT=4,                        
C     I JPLTMT=JPTASKS,                                                 
C     I JPNUMW=352,            JPNUMP=352,                              
C     I JPTABLE=1024,                                                   
C     I JPMAXB =60,                                                     
C     I JPMAXPB=32,                                                     
C     I JPG3FLD=43,         JPG3XF=99,         JPG3CMP=17,              
C     I JPMP1=JPM+1,        JPNP1=JPN+1,       JPKP1=JPK+1,             
C     I JP2MP1=JPMP1*2,     JP4MP1=JPMP1*4,                             
C     I JPED=JPN/2+1,       JPOD=JPNP1/2,                               
C     I IPSP0=JPM+JPN-JPK,  IPSP1=JPNP1*JPMP1, IPSP2=IPSP0*(IPSP0+1)/2, 
C     I JPSP=IPSP1-IPSP2,   JP2SP=2*JPSP,                               
C     I JPHGL=JPGL/2,       JPNLP2=JPNLON+2,                            
C     I                     JPNVCLV=4*JPNLVP1)                          
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C     J. K. GIBSON     E.C.M.W.F.     22/01/82.                         
C     MODIFIED M.J     E.C.M.W.F.     13/12/82.                         
C                                                                       
      INTEGER                                                           
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      LOGICAL                                                           
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD        
      REAL                                                              
     R DTIME,    TWODT,    EPS                                          
C                                                                       
C      INTEGER                                                          
C     I NWTIME(JPNUMW),     NROW(JPTASKS),                              
C     I NPTIME(JPNUMP)                                                  
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMGAU* - QUANTITIES RELATED TO THE GAUSSIAN GRID.      
C                                                                       
      REAL                                                              
     R SINLON(NLP2),     COSLON(NLP2)                                   
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMTRU* -QUANTITIES RELATED TO THE SPECTRAL TRUNCATION. 
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSIM* - QUANTITIES NEEDED FOR THE SEMI-IMPLICIT SCHEME
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMHDI* - COEFFICIENTS FOR HORIZONTAL DIFFUSION.        
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMFFT* - QUANTITIES NEEDED FOR THE FAST *FOURIER       
C                         TRANSFORMS.                                   
C                                                                       
      INTEGER                                                           
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2                           
C      REAL                                                             
C     R TRIG(JPNLON)                                                    
      INTEGER                                                           
     I NFAX(10)                                                         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSTA* - ARRAYS AND PARAMETERS FOR STATISTICS.         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMDIA* - QUANTITIES RELATED TO GLOBAL PHYSICAL         
C                         DIAGNOSTICS.                                  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPSW* SWITCHES RELATED TO THE PARAMETERISATIONS OF      
C                     DIABATIC PROCESSES.                               
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMDSW* SWITCHES RELATED TO THE DYNAMICS                  
C                     AND THE GENERAL CONTROL OF THE FORECAST.          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH1* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE RADIATION SUBROUTINES.                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CRAE,                                                            
     R CDISSEM                                                          
C                                                                       
C      REAL       COSRAD(128),        SINRAD(128)                       
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH2* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE PHYSICAL SUBROUTINES (EXCEPT RADIATION    
C                     ONES).                                            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL       CCRH(JPNLEV)                                           
C                                                                       
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMDIZ* VARIABLES AND POINTERS FOR ZONAL MEAN DIAGNOSTICS 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMMSK* VARIABLES AND POINTERS FOR AREA MEAN DIAGNOSTICS. 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMSDS* - START/RESTART BLOCK.                          
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMTASK* - TASK RELATED VARIABLES.                      
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*         *COMRAD* - RADIATION POINTERS.                               
C                                                                       
      LOGICAL LO,LO1,LO2,LODIA,LODIAP                                   
C---------------------------------------------------------------------  
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C     COMMON *YOMAER* RADIATIVE CHARACTERISTICS OF THE AEROSOLS         
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
      REAL TAUA(2,5), PIZA(2,5), CGA(2,5), CAER(5,5), CAEROS            
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C     COMMON *YOMLW*                                                    
C                                                                       
C*    COEFFICIENTS FOR THE LONGWAVE RADIATION SUBROUTINES               
C                                                                       
C NINT  :  NUMBER OF SPECTRAL INTERVALS                                 
C NIPD  :  NUMBER OF BANDS DESCRIBED BY PADE APPROXIMANTS               
C NTR   :  NUMBER OF REFERENCE TEMPERATURES                             
C NTRA  :  NUMBER OF TRANSMISSION FUNCTIONS                             
C NUA   :  NUMBER OF ABSORBER AMOUNTS                                   
C NG1   :  ORDER OF THE GAUSSIAN QUADRATURE                             
C                                                                       
C RT1   :  ROOTS OF THE GAUSSIAN POLYNOMIAL                             
C WG1   :  WEIGHTS OF THE GAUSSIAN POLYNOMIAL                           
C AT, BT:  COEFFICIENTS OF THE TEMPERATURE DEPENDENCE                   
C CNTNU :  H2O CONTINUUM ABSORPTION COEFFICIENTS                        
C O1H,O2H,PIALF0,OCT: OZONE LONGWAVE ABSORPTION COEFFICIENTS            
C TREF, TSTAND      : REFERENCE TEMPERATURES                            
C XP    :  COEFFICIENTS OF POLYNOMIAL DEVELOPMENTS OF PLANCK FUNCTION   
C GA,GB :  PADE APPROXIMANTS OF H2O, CO2 TRANSMISSION FUNCTIONS         
C ETY,PTY :  PARAMETERS OF H2O CONTINUUM                                
C TINTP :  REFERENCE TEMPERATURES OF PRE-COMPUTED TRANSMISSIVITIES      
C CE    :  EMS COEFFS OF H2O TRANSMISSION FUNCTIONS                     
C                                                                       
      REAL  RT1(2),WG1(2),AT(12,3),BT(12,3),CNTNU(4),O1H                
     S ,OCT(4),TREF,XP(6,6),TSTAND,GA(11,24,3),GB(11,24,3)              
     S ,PTY(11,20),ETY(11,20),TINTP(11),TSTP,O2H,PIALF0                 
C                                                                       
      REAL  TB(40,31,2),ALP(40,31,2),BET(40,31,2),                      
     S ULG(40),PLG(31),AP(40,31,2),BP(40,31,2),TBO3(38,31),TO3B(38,31)  
     S,ULGO(38),AO3(38,31),BO3(38,31),O3A(38,31),O3B(38,31)             
C     ----------------------------------------------------------------- 
C*COMDECK YOMRDU                                                        
C     ----------------------------------------------------------------- 
C                                                                       
C*    CONTROL VARIABLES FOR RADIATION: INTERNAL SWITCH AND INDICES      
C                                                                       
C ZEELOG:  SECURITY THRESHOLD FOR ABSORBER AMOUNT IN LAPLACE TRANSFORM  
C ZEPSC :  SECURITY THRESHOLD FOR CLOUD COVER                           
C ZEPSCO:  SECURITY THRESHOLD FOR OZONE AMOUNT                          
C ZEPSCQ:  SECURITY THRESHOLD FOR WATER VAPOR                           
C ZEPSCT:  SECURITY THRESHOLD FOR SHORTWAVE OPTICAL THICKNESS           
C ZEPSCW:  SECURITY THRESHOLD FOR CLOUD LIQUID WATER PATH               
C                                                                       
      REAL          CDAY,CCO2,CH2O,DIFF,X10E                            
     S             ,ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW            
C     ----------------------------------------------------------------- 
C*COMDECK YOMSW                                                         
C     ----------------------------------------------------------------- 
C                                                                       
C*    COEFFICIENTS FOR THE SHORTWAVE RADIATION SUBROUTINES              
C                                                                       
C APAD  :  PADE APPROXIMANTS NUMERATOR                                  
C BPAD  :  PADE APPROXIMANTS DENOMINATOR                                
C D     :  TRANSMISSION LIMIT FOR INFINITE ABSORBER AMOUNT              
C CRAY  :  RAYLEIGH SCATTERING COEFFICIENTS                             
C SUN   :  SOLAR FRACTION IN SPECTRAL INTERVALS                         
C                                                                       
      REAL  APAD(2,3,7),BPAD(2,3,7),D(2,3),CRAY(2,6),SUN(2)             
C     ----------------------------------------------------------------- 
                                                                        
C                                                                       
C  -------------------------------------------------------------------- 
C                                                                       
      REAL   TTT(1,NLAT,NRLEV,12)                                       
C                                                                       
C        0.1  "RADINT POINTERS"                                         
C             -----------------                                         
C                                                                       
      REAL   TM1(NLP2,NLEV),                                            
     *       TSM1M(NLP2),                                               
     *       QM1(NLP2,NLEV),                                            
     *       SNM1M(NLP2),                                               
     *       ALBM(NLP2),                                                
     *       AVVELM(NLP2)                                               
      REAL   SLMM(NLP2),                                                
     *       APHM1(NLP2,NLEVP1),                                        
     *       APM1(NLP2,NLEV)                                            
C                                                                       
      LOGICAL LOLAND(NLP2)                                              
C                                                                       
C     VARIABLES FOR FULL GRID AND TRANSFORMS.                           
C                                                                       
      REAL   ZTI(NLP2,NRLEVP1),                                         
     *       ZCTI(NLP2,NRLEVP1),                                        
     *       ZCPH(NLP2,NRLEVP1),                                        
     *       ZWV(NLP2,NRLEV),                                           
     *       ZSW(NLP2,NRLEV),                                           
     *       ZALTE(NLP2),                                               
     *       ZCALTE(NLP2),                                              
     *       ZALSO(NLP2),                                               
     *       ZCALSO(NLP2)                                               
C                                                                       
      REAL   ZOZQ(NLP2),                                                
     #       ZOZH(NLP2)                                                 
C                                                                       
      REAL   ZTRSOL(NLP2,NLEVP1),                                       
     *       ZEMTER(NLP2,NLEVP1)                                        
C                                                                       
C     FINAL RESULTS ARE STORED ON:                                      
C                                                                       
      REAL   TRSOLM(NLP2,NLEVP1),                                       
     *       EMTERM(NLP2,NLEVP1)                                        
C                                                                       
      REAL   AMU0(NLP2),                                                
     *       RDAYL(NLP2)                                                
C                                                                       
C     VARIABLES WITHIN THE SPLITTED LATITUDE LOOP.                      
C                                                                       
      REAL   CLC(NRLST,NRLEV),                                          
     *       TI(NRLST,NRLEVP1),                                         
     *       DP(NRLST,NRLEV),                                           
     *       AADS(NRLST,NRLEV),                                         
     *       AEQ1(NRLST,NRLEV),                                         
     *       AEQ2(NRLST,NRLEV),                                         
     *       AEQ3(NRLST,NRLEV),                                         
     *       AEQ4(NRLST,NRLEV),                                         
     *       AEQ5(NRLST,NRLEV),                                         
     *       SSW(NRLST,NRLEV),                                          
     *       WV(NRLST,NRLEV),                                           
     *       QOF(NRLST,NRLEV),                                          
     *       QCF(NRLST,NRLEV),                                          
     *       CLWA(NRLST,NRLEV),                                         
     *       APRE(NRLST),                                               
     *       SMU0(NRLST),                                               
     *       ALTE(NRLST),                                               
     *       ALSO(NRLST)                                                
      REAL   ZDPO(NRLST),                                               
     *       ZQCFO(NRLST),                                              
     *       ZAEQSO(NRLST),                                             
     *       ZQOFO(NRLST),                                              
     *       ZDPN(NRLST),                                               
     *       ZQCFN(NRLST),                                              
     *       ZAEQSN(NRLST),                                             
     *       ZQOFN(NRLST)                                               
      REAL   FLT(NRLST,NRLEVP1),                                        
     *       FLS(NRLST,NRLEVP1)                                         
C                                                                       
C     DIAGNOSTIC VARIABLES.                                             
C                                                                       
      REAL   DIAG(NLEVP1,4),                                            
     *       DIA1(7),                                                   
     *       ZDIAG(NLEVP1,10),                                          
     *       ZDIA1(21),                                                 
     *       ZDIA(NLON),                                                
     *       ZDIAF(NLON,7),                                             
     *       ZDIAT(NLEVP1)                                              
C                                                                       
C                                                                       
C                                                                       
      REAL PGG(IRLST),                                                  
     #     PREF(IRLST),                                                 
     #     PRE1(IRLST),                                                 
     #     PRE2(IRLST),                                                 
     #     PRMUZ(IRLST),                                                
     #     PTO1(IRLST),                                                 
     #     PTR1(IRLST),                                                 
     #     PTR2(IRLST),                                                 
     #     PW(IRLST)                                                    
C                                                                       
C                                                                       
C  what about the 1-dimensional PALBS???                                
C                                                                       
      REAL     PALBSN  (IRLST)                                          
C                                                                       
C*       0.2   "LOCAL" ARRAYS                                           
C              --------------                                           
      REAL ZABSCU(IRLST ,NUA, 3*NRLEV+1),                               
     S     ZABSLY(IRLST ,NUA, 3*NRLEV+1) ,                              
     S     ZADJD (IRLST , IRLVP1)      ,                                
     S     ZADJU (IRLST , IRLVP1)       ,                               
     S     ZBGND (IRLST)                ,                               
     S     ZBINT (IRLST , IRLVP1)       ,                               
     S     ZBSUR (IRLST , NINT)         ,                               
     S     ZBSUIN(IRLST)                ,                               
     S     ZBTOP (IRLST , NINT)         ,                               
     S     PCLDLW(IRLST , IRLEV)        ,                               
     S     ZCLOUD(IRLST)                ,                               
     S     ZCNTRB(IRLST , IRLVP1,IRLVP1),                               
     S     ZCUMP (IRLST)                ,                               
     S     ZCUMPT(IRLST)                ,                               
     S     ZDBDT (IRLST , NINT , IRLEV) ,                               
     S     ZDBSL (IRLST , NINT , 2*NRLEV),                              
     S     ZDISTD(IRLST , IRLVP1)       ,                               
     S     ZDISTU(IRLST , IRLVP1)                                       
      REAL                                                              
     S     ZDNF  (IRLST , IRLVP1,IRLVP1),                               
     S     PDT0  (IRLST)                                                
      REAL                                                              
     S     ZDUC  (IRLST , 3*NRLEV+1)       ,                            
     S     ZEFFT (IRLST)                ,                               
     S     ZEMD  (IRLST , IRLEV), ZEMU  (IRLST , IRLEV),                
     S     ZEPSC2(IRLST),  ZEPSC3(IRLST),                               
     S     ZEPSH1(IRLST),  ZEPSH2(IRLST),                               
     S     ZEPSH3(IRLST),  ZEPSH4(IRLST),                               
     S     ZEPSH5(IRLST),  ZEPSH6(IRLST),                               
     S     ZEPHIO(IRLST),  ZFACTC(IRLST),                               
     S     ZZFD(IRLST), ZFDD(IRLST), ZFDN(IRLST , IRLVP1)               
      REAL                                                              
     S     PFLUX (IRLST , 2 , IRLVP1)   ,                               
     S     PFLUC (IRLST , 2 , IRLVP1)   ,                               
     S     ZFLWP (IRLST)                ,                               
     S     PCTS  (IRLST , IRLEV)        ,                               
     S     ZZFU   (IRLST)               ,                               
     S     ZFUP  (IRLST , IRLVP1)       ,                               
     S     ZGLAYD(IRLST)                ,                               
     S     ZGLAYU(IRLST)                ,                               
     S     ZOMEGA(IRLST , 2 , IRLEV)                                    
C                                                                       
      INTEGER                                                           
     S     ITX   (IRLST)                ,                               
     S     IXDIA (IRLST , IRLVP1)       ,                               
     S     IXT   (IRLST , IRLVP1,IRLVP1),                               
     S     IXTSU (IRLST , IRLVP1)       ,                               
     S     IXTTP (IRLST , IRLVP1)       ,                               
     S     IMX   (IRLST)                ,                               
     S     IMXP  (IRLST)                                                
C                                                                       
      REAL ZOZ   (IRLST , IRLEV)        ,                               
     S     PPMB  (IRLST , IRLVP1)       ,                               
     S     ZRES  (IRLST)                ,                               
     S     PB    (IRLST, 6 , IRLVP1)    ,                               
     S     ZSSIG (IRLST , 3*NRLEV+1)    ,                               
     S     PTAU  (IRLST , 2 , IRLEV)    ,                               
     S     PTAVE (IRLST , IRLEV)        ,                               
     S     ZTAVIC(IRLST)                ,                               
     S     ZTI1  (IRLST)                ,                               
     S     PTL   (IRLST , IRLVP1)       ,                               
     S     ZTT   (IRLST , NTRA)         ,                               
     S     ZTT1  (IRLST , NTRA)         ,                               
     S     ZTT2  (IRLST , NTRA)         ,                               
     S     ZUAER (IRLST , NINT)         ,                               
     S     ZUPF  (IRLST , IRLVP1,IRLVP1),                               
     S     ZUU   (IRLST , NUA)          ,                               
     S     ZXD   (IRLST)                                                
      REAL                                                              
     S     ZXN   (IRLST)                ,                               
     S     ZXOZ  (IRLST)                ,                               
     S     ZXWV  (IRLST)                ,                               
     S     ZZZ   (IRLST)                ,                               
     S     ZRES2 (IRLST)                ,                               
     S     ZTI2  (IRLST)                ,                               
     S     ZBLAY (IRLST,IRLEV)          ,                               
     S     ZBLEV (IRLST,IRLVP1)         ,                               
     S     ZCTS  (IRLST)                ,                               
     S     ZHFG  (IRLST)                                                
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C                                                                       
      INTEGER KT    (IRLST)                                             
      REAL    PUU   (IRLST , NUA  )     ,                               
     S        PTT   (IRLST , NTRA )     ,                               
     S        ZZ    (IRLST)                                             
C                                                                       
C     -------------------------------------------------------- ---------
C                                                                       
      REAL  PCLFR  (IRLST , IRLEV)       ,                              
     S      PT     (IRLST , IRLVP1)       ,                             
     S      PQOF   (IRLST , IRLEV)          ,                           
     S      PQLWP  (IRLST , IRLEV)          ,                           
     S      PEMIS  (IRLST)                                              
      REAL                                                              
     S      PFLT   (IRLST , IRLVP1)                                     
C                                                                       
      REAL  ZALBSU(IRLST , 2 )           ,                              
     S      ZCG   (IRLST , 2 , IRLEV)    ,                              
     S      ZCLDLW(IRLST , IRLEV)        ,                              
     S      ZCLDSW(IRLST , IRLEV)        ,                              
     S      ZCOOLR(IRLST , IRLEV)        ,                              
     S      ZDT0  (IRLST)                ,                              
     S      ZFLUX (IRLST , 2 , IRLVP1)   ,                              
     S      ZPMB  (IRLST , IRLVP1)       ,                              
     S      ZTAU  (IRLST , 2 , IRLEV)    ,                              
     S      ZTAVE (IRLST , IRLEV)        ,                              
     S      ZTL   (IRLST , IRLVP1)       ,                              
     S      ZFSDWN(IRLST , IRLVP1)       ,                              
     S      ZFSUP (IRLST , IRLVP1)       ,                              
     S      ZHEATR(IRLST , IRLEV )                                      
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C                                                                       
      REAL   PDP   (IRLST , IRLEV)     ,                                
     S       PWV   (IRLST , IRLEV)     ,                                
     S       PFLS  (IRLST , IRLVP1)    ,                                
     S       PPSOL (IRLST        )     ,                                
     S       PRMU0 (IRLST        )     ,                                
     S       PAER  (IRLST,IRLEV,5)     ,                                
     S       PALBS (IRLST , 2 )        ,                                
     S       PCG   (IRLST , 2 , IRLEV) ,                                
     S       PCLDSW(IRLST , IRLEV)     ,                                
     S       POMEGA(IRLST , 2 , IRLEV) ,                                
     S       POZ   (IRLST , IRLEV)                                      
C                                                                       
      REAL   PFDOWN(IRLST , IRLVP1)    ,                                
     S       PFUP  (IRLST , IRLVP1)                                     
C                                                                       
      REAL   ZAKI  (IRLST , 2 )        ,                                
     S       ZCGAZ (IRLST , IRLEV)     ,                                
     S       ZCNEB (IRLST , IRLEV)     ,                                
     S       ZC1I  (IRLST , IRLVP1)    ,                                
     S       ZDSC  (IRLST , IRLEV)     ,                                
     S       ZDSH  (IRLST , IRLEV)     ,                                
     S       ZDSIG (IRLST , IRLEV)     ,                                
     S       ZFACT (IRLST)             ,                                
     S       ZFACT1 (IRLST), ZFACT2 (IRLST),                            
     S       ZFACT3 (IRLST), ZFACT4 (IRLST),                            
     S       ZFACT5 (IRLST), ZFACT6 (IRLST),                            
     S       ZFD   (IRLST , IRLVP1)    ,                                
     S       ZFRAC (IRLST)             ,                                
     S       ZFU   (IRLST , IRLVP1)    ,                                
     S       ZG    (IRLST)             ,                                
     S       ZGG   (IRLST)             ,                                
     S       ZPIZAZ(IRLST , IRLEV)     ,                                
     S       ZRAYL (IRLST),  ZRAY1 (IRLST , IRLVP1)                     
      REAL                                                              
     S       ZRAY2 (IRLST , IRLVP1)    ,                                
     S       ZREF  (IRLST)             ,                                
     S       ZREFZ (IRLST , 2 , IRLVP1),                                
     S       ZRE1  (IRLST)             ,                                
     S       ZRE2  (IRLST)             ,                                
     S       ZRJ   (IRLST , 6 , IRLVP1),                                
     S       ZRK   (IRLST , 6 , IRLVP1),                                
     S       ZRL   (IRLST , 8)         ,                                
     S       ZRMU  (IRLST)             ,                                
     S       ZRMUE (IRLST , IRLVP1)    ,                                
     S       ZRMUZ (IRLST)                                              
      REAL                                                              
     S       ZRNEB (IRLST)             ,                                
     S       ZRUEF (IRLST , 8)         ,                                
     S       ZR1   (IRLST)             ,                                
     S       ZR2   (IRLST)             ,                                
     S       ZR21  (IRLST)             ,                                
     S       ZR22  (IRLST)             ,                                
     S       ZR23  (IRLST)             ,                                
     S       ZS    (IRLST)             ,                                
     S       ZSEC  (IRLST)             ,                                
     S       ZSS1  (IRLST)             ,                                
     S       ZTAUAZ(IRLST , IRLEV)     ,                                
     S       ZTOT  (IRLST , IRLVP1)    ,                                
     S       ZTO1  (IRLST)             ,                                
     S       ZTR   (IRLST , 2 , IRLVP1),                                
     S       ZTRA1 (IRLST , IRLVP1)    ,                                
     S       ZTRA2 (IRLST , IRLVP1)                                     
      REAL                                                              
     S       ZTR1  (IRLST)             ,                                
     S       ZTR2  (IRLST)             ,                                
     S       ZUD   (IRLST , 3 , IRLVP1),                                
     S       ZUM   (IRLST , IRLVP1)    ,                                
     S       ZU1D  (IRLST)             ,                                
     S       ZU2D  (IRLST)             ,                                
     S       ZW    (IRLST)             ,                                
     S       ZN175 (IRLST)             ,                                
     S       ZN190 (IRLST)             ,                                
     S       ZO175 (IRLST)             ,                                
     S       ZO190 (IRLST)             ,                                
     S       ZP75  (IRLST)             ,                                
     S       ZP90  (IRLST)             ,                                
     S       ZSIGN (IRLST)             ,                                
     S       ZSIGO (IRLST)                                              
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
      REAL   PTR    (IRLST)     ,                                       
     S       PU     (IRLST)                                             
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
C   CLOUD-POINTERS                                                      
C   --------------                                                      
C                                                                       
      REAL   ZRH(NLP2,NLEV),                                            
     *       ZCH(NLP2),                                                 
     *       ZCM(NLP2),                                                 
     *       ZCL(NLP2),                                                 
     *       ZCT(NLP2),                                                 
     *       ZCC(NLP2)                                                  
      INTEGER NTOPCM(NLP2),                                             
     *        NBASECM(NLP2)                                             
      REAL   ARPRCM(NLP2)                                               
      INTEGER ITOPH(NLP2),                                              
     *        ITOPM(NLP2),                                              
     *        ITOPL(NLP2)                                               
      LOGICAL LOC(NLP2)                                                 
      REAL   ZQS(NLP2,NLEV),                                            
     *       ZCLC(NLP2,NLEV),                                           
     *       ZQLWP(NLP2,NLEV)                                           
C                                                                       
      REAL   VERVEL(NLP2,NLEV),                                         
     *       ZTHETA(NLP2,NLEV),                                         
     *       ZETA(NLP2,NLEV),                                           
     *       ZDTHMIN(NLP2)                                              
      INTEGER INVB(NLP2)                                                
      REAL   ZCTLWP(NLP2),                                              
     *       ZQLOC(NLP2),                                               
     *       ZQMEC(NLP2),                                               
     *       ZQHIC(NLP2)                                                
C                                                                       
C  definitions of "radheat"                                             
C                                                                       
      REAL   ZFSOT(NLON),                                               
     #       ZFTET(NLON),                                               
     #       ZFLT (NLON),                                               
     #       ZFSO (NLON),                                               
     #       ZFTE (NLON),                                               
     #       ZFLB (NLON),                                               
     #       ZDTDT(NLON),                                               
     #       HEATO(NLON,NLEV),                                          
     #       HEATE(NLON,NLEV),                                          
     #       HEATR(NLON,NLEV),                                          
     #       ZI0  (NLON)                                                
C                                                                       
C PIERS 13-8-97                                                         
C                                                                       
         REAL PIERSLWP                                                  
C                                                                       
C Piece of code needed to replace the Nikos radiation scheme with       
C the Morcrette radiation scheme in the igcm3.1.                        
C See morcrette3_1_1.readme for details.                                
C                                                                       
C                                                                       
c      program lh                                                       
c                                                                       
c      parameter(NL=23)                                                 
c      real P(NL),t(NL),h2o(NL),o3(NL),htlw(NL),htsw(NL)                
c      real swalb,doy                                                   
c       REAL CF(4,2)     !cloud fractions,lwps                          
c       INTEGER IC(4,2)   ! cloud positions                             
c       REAL RFLUXES(2,2,2)                                             
c      open(11,file='mls23.dat')                                        
c      read(11,*) NLEV                                                  
c      DOY=80                                                           
c      swalb=0.07                                                       
c      alat=40.0                                                        
c       doy=float(i)                                                    
c      do i=nlev,1,-1                                                   
c         read(11,*) P(i),t(i),h2o(i),adum,o3(i)                        
c         print *,i,p(i),t(i),h2o(i),o3(i)                              
c      enddo                                                            
c                                                                       
c       ic(1,1)=3                                                       
c      ic(1,2)=17                                                       
c       ic(3,1)=6                                                       
c       ic(2,2)=6                                                       
c       cf(1,1)=0.3                                                     
c       cf(1,2)=45.0                                                    
c        cf(3,1)=0.8                                                    
c       cf(4,2)=20.0                                                    
c      alon=0.0                                                         
c      call morcigcm(p,t,h2o,o3,alat,htlw,htsw,doy,cf,ic,               
c     $     rfluxes,swalb,alon)                                         
c      enddo                                                            
c      do i=nl,1,-1                                                     
c         print '(i6,4F14.4)',i,p(i),t(i),htsw(i),htlw(i)               
c      enddo                                                            
c      print *,RFLUXES(1,1,1), RFLUXES(1,1,2), RFLUXES(1,2,1),          
c     $     RFLUXES(1,2,2)                                              
c      print *,RFLUXES(2,1,1), RFLUXES(2,1,2), RFLUXES(2,2,1),          
c     $     RFLUXES(2,2,2)                                              
c                                                                       
c                                                                       
c      end                                                              
c      INCLUDE 'zparb.upd'                                              
C                                                                       
C      ---------------------------------------------------------------- 
C                                                                       
C*       1.    SET VALUES.                                              
C              -----------                                              
C                                                                       
c      DATA SUN(1) / 0.441676 /                                         
      DATA SUN(1) / 0.441130 /                                          
      DATA (D(1,K),K = 1,3) / 0.00, 0.00, 0.00 /                        
      DATA ((APAD(1,I,J),I=1,3),J=1,7) /                                
     S 0.000000000E-00, 0.000000000E-00, 0.925887084E-04,               
     S 0.000000000E-00, 0.000000000E-00, 0.129353723E-01,               
     S 0.000000000E-00, 0.000000000E-00, 0.800821928E+00,               
     S 0.000000000E-00, 0.000000000E-00, 0.242715973E+02,               
     S 0.000000000E-00, 0.000000000E-00, 0.878331486E+02,               
     S 0.000000000E-00, 0.000000000E-00, 0.191559725E+02,               
     S 0.000000000E-00, 0.000000000E-00, 0.000000000E+00 /              
C                                                                       
      DATA ((BPAD(1,I,J),I=1,3),J=1,7) /                                
     S 0.000000000E-00, 0.000000000E-00, 0.925887084E-04,               
     S 0.000000000E-00, 0.000000000E-00, 0.131812683E-01,               
     S 0.000000000E-00, 0.000000000E-00, 0.812706117E+00,               
     S 0.000000000E-00, 0.000000000E-00, 0.249863591E+02,               
     S 0.000000000E-00, 0.000000000E-00, 0.931071925E+02,               
     S 0.000000000E-00, 0.000000000E-00, 0.252233437E+02,               
     S 0.000000000E-00, 0.000000000E-00, 0.100000000E+01 /              
C                                                                       
      DATA (CRAY(1,K),K=1,6) /                                          
     S .428937E-01, .890743E+00,-.288555E+01,                           
     S .522744E+01,-.469173E+01, .161645E+01/                           
C                                                                       
c      DATA SUN(2) / 0.558324 /                                         
      DATA SUN(2) / 0.557633 /                                          
      DATA (D(2,K),K=1,3) / 0.317735694, 0.775570952, 0.800000000 /     
      DATA ((APAD(2,I,J),I=1,3),J=1,7) /                                
     S 0.822745535E-02, 0.145321703E-04, 0.410177786E+03,               
     S 0.705825794E+01, 0.175741897E-01, 0.672595424E+02,               
     S 0.348747605E+03, 0.259696276E+01, 0.000000000E-00,               
     S 0.174921268E+04, 0.599852834E+02, 0.000000000E-00,               
     S 0.100138721E+04, 0.203510317E+03, 0.000000000E-00,               
     S 0.518496206E+02, 0.757222990E+02, 0.000000000E-00,               
     S 0.000000000E+00, 0.000000000E+00, 0.000000000E+00 /              
C                                                                       
      DATA ((BPAD(2,I,J),I=1,3),J=1,7) /                                
     S 0.822745535E-02, 0.145321703E-04, 0.410177786E+03,               
     S 0.719686994E+01, 0.176779370E-01, 0.731185438E+02,               
     S 0.381961022E+03, 0.265802733E+01, 0.100000000E+01,               
     S 0.219460901E+04, 0.634292876E+02, 0.000000000E+00,               
     S 0.159321079E+04, 0.228829763E+03, 0.000000000E+00,               
     S 0.138748279E+03, 0.992586506E+02, 0.000000000E+00,               
     S 0.100000000E+01, 0.100000000E+01, 0.000000000E+00 /              
C                                                                       
      DATA (CRAY(2,K),K=1,6) /                                          
     S .697200E-02, .173297E-01,-.850903E-01,                           
     S .248261E+00,-.302031E+00, .129662E+00/                           
C                                                                       
      RETURN                                                            
      END                                                               
C                                                                       
      SUBROUTINE INICON                                                 
C                                                                       
C*** **INICON  PRESET CONSTANTS IN *COMCON*.                            
C                                                                       
C      M.JARRAUD      E.C.M.W.F.     10/12/1982.                        
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C                                                                       
C             PRESET CONSTANTS IN *COMCON*.                             
C                                                                       
C**   INTERFACE.                                                        
C     ----------                                                        
C                                                                       
C             **INICON IS CALLED FROM *SETDYN*.                         
C                                                                       
C     EXTERNALS.                                                        
C     ----------                                                        
C                                                                       
C             NONE.                                                     
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRSW* SWITCHES RELATED TO THE RADIATION SCHEME.         
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      IMPLICIT REAL (A-H,O-Z)                                           
C      IMPLICIT UNDEFINED (Z)                                           
C      IMPLICIT LOGICAL (L)                                             
C      IMPLICIT UNDEFINED (L)                                           
      IMPLICIT INTEGER (I-K,M-N)                                        
C      IMPLICIT UNDEFINED (I-K,M-N)                                     
C      IMPLICIT REAL (P)                                                
C      IMPLICIT UNDEFINED (A-H,O,Q-Y)                                   
C                                                                       
C everything except the parameters is defined                           
C in the common blocks                                                  
C                                                                       
      COMMON/COM1/                                                      
C     I NRADFR, NRPART, NRINT, NRADPFR, NRADPLA,                        
C     I NRLSTI,                                                         
     R XLAT,                                                            
     R RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR, ZCCO2,              
C     R PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3,                 
     L LOH, LOM, LOL, LORAD                                             
      COMMON/COM2/                                                      
     I ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,                  
     I         JLOOP, KRLEV,                                            
     I        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     I        ITH, ITM, ITL,                                            
     I        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     I        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     I        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     I        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      COMMON/COM3/                                                      
     I IND4, KM12, KXD, KXU, IG, KBS, KDD,                              
     I        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     I        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     I        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     I        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     I        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
      COMMON/COM4/                                                      
     R           COZQC,COZQS,COZHC,COZHS,                               
     R CSUPSAT,                                                         
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
      COMMON/COM5/                                                      
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NUMVC,                                                           
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      COMMON/COM6/                                                      
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD,       
     R DTIME,    TWODT,    EPS,                                         
     R SINLON,     COSLON,                                              
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2,                          
     I NFAX,                                                            
     R CRAE,                                                            
     R CDISSEM                                                          
      COMMON/COM7/                                                      
     R     CCRH,                                                        
     L LO,LO1,LO2,LODIA,LODIAP,                                         
     R TAUA, PIZA, CGA, CAER, CAEROS                                    
      COMMON/COM7A/                                                     
     R  RT1,WG1,AT,BT,CNTNU,O1H,O2H,PIALF0                              
     S ,OCT,TREF,XP,TSTAND,GA,GB,CE,ETY,PTY                             
      COMMON/COM7B/                                                     
     S TINTP,TSTP,                                                      
     S  TBO3,AO3,BO3,TB,ALP,BET,ULG,ULGO,PLG,AP,BP,TO3B,O3A,O3B,        
     R          CDAY,CCO2,CH2O,DIFF,X10E                                
       COMMON/COM7C/                                                    
     S  ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW,                       
     R  APAD,BPAD,D,CRAY,SUN,                                           
     R   TTT,                                                           
     R   TM1                                                            
      COMMON/COM8/                                                      
     R       TSM1M,                                                     
     R       QM1,                                                       
     R       SNM1M,                                                     
     R       ALBM,                                                      
     R       AVVELM,                                                    
     R   SLMM,                                                          
     R       APHM1,                                                     
     R       APM1                                                       
      COMMON/COM9/                                                      
     L LOLAND,                                                          
     R   ZTI,                                                           
     *       ZCTI,                                                      
     *       ZCPH,                                                      
     *       ZWV,                                                       
     *       ZSW,                                                       
     *       ZALTE,                                                     
     *       ZCALTE,                                                    
     *       ZALSO                                                      
      COMMON/COM10/                                                     
     *       ZCALSO,                                                    
     R   ZOZQ,                                                          
     *       ZOZH,                                                      
     R   ZTRSOL,                                                        
     *       ZEMTER,                                                    
     R   TRSOLM,                                                        
     *       EMTERM,                                                    
     R   AMU0,                                                          
     *       RDAYL                                                      
      COMMON/COM11/                                                     
     R   CLC,                                                           
     *       TI,                                                        
     *       DP,                                                        
     *       AADS,                                                      
     *       AEQ1,                                                      
     *       AEQ2,                                                      
     *       AEQ3,                                                      
     *       AEQ4,                                                      
     *       AEQ5,                                                      
     *       SSW                                                        
      COMMON/COM12/                                                     
     *       WV,                                                        
     *       QOF,                                                       
     *       QCF,                                                       
     *       CLWA,                                                      
     *       APRE,                                                      
     *       SMU0,                                                      
     *       ALTE,                                                      
     *       ALSO                                                       
      COMMON/COM13/                                                     
     R   ZDPO,                                                          
     *       ZQCFO,                                                     
     *       ZAEQSO,                                                    
     *       ZQOFO,                                                     
     *       ZDPN,                                                      
     *       ZQCFN,                                                     
     *       ZAEQSN,                                                    
     *       ZQOFN                                                      
      COMMON/COM14/                                                     
     R   FLT,                                                           
     *       FLS,                                                       
     R   DIAG,                                                          
     *       DIA1,                                                      
     *       ZDIAG,                                                     
     *       ZDIA1,                                                     
     *       ZDIA,                                                      
     *       ZDIAF,                                                     
     *       ZDIAT,                                                     
     R PGG                                                              
      COMMON/COM15/                                                     
     #     PREF,                                                        
     #     PRE1,                                                        
     #     PRE2,                                                        
     #     PRMUZ,                                                       
     #     PTO1,                                                        
     #     PTR1,                                                        
     #     PTR2,                                                        
     #     PW,                                                          
     R     PALBSN                                                       
      COMMON/COM16/                                                     
     R ZABSCU,                                                          
     S     ZABSLY,                                                      
     S     ZADJD ,                                                      
     S     ZADJU ,                                                      
     S     ZBGND ,                                                      
     S     ZBINT ,                                                      
     S     ZBSUR ,                                                      
     S     ZBSUIN,                                                      
     S     ZBTOP ,                                                      
     S     PCLDLW,                                                      
     S     ZCLOUD,                                                      
     S     ZCNTRB,                                                      
     S     ZCUMP ,                                                      
     S     ZCUMPT,                                                      
     S     ZDBDT                                                        
      COMMON/COM17/                                                     
     S     ZDBSL ,                                                      
     S     ZDISTD,                                                      
     S     ZDISTU,                                                      
     S     ZDNF  ,                                                      
     S     PDT0  ,                                                      
     S     ZDUC                                                         
      COMMON/COM18/                                                     
     S     ZEFFT ,                                                      
     S     ZEMU  ,                                                      
     S     ZEPHIO,                                                      
     S     ZEPSC2,                                                      
     S     ZEPSC3,                                                      
     S     ZEPSH1,                                                      
     S     ZEPSH2,                                                      
     S     ZEPSH3                                                       
      COMMON/COM19/                                                     
     S     ZEPSH4,                                                      
     S     ZEPSH5,                                                      
     S     ZEPSH6,                                                      
     S     ZEPSIO,                                                      
     S     ZFACTC,                                                      
     S     ZZFD  ,                                                      
     S     ZFDD  ,                                                      
     S     ZFDN                                                         
      COMMON/COM20/                                                     
     S     PFLUX , PFLUC,                                               
     S     ZFLWP , PCTS                                                 
     S     ZZFU  ,                                                      
     S     ZFUP  ,                                                      
     S     ZGLAYD,                                                      
     S     ZGLAYU,                                                      
     S     ZOMEGA,                                                      
     S     ITX   ,                                                      
     S     IXDIA ,                                                      
     S     IXT   ,                                                      
     S     IXTSU ,                                                      
     S     IXTTP ,                                                      
     S     IMX   ,                                                      
     S     IMXP                                                         
      COMMON/COM21/                                                     
     R ZOZ   ,                                                          
     S     PPMB  ,                                                      
     S     ZRES  , PB                                                   
     S     ZSSIG ,                                                      
     S     PTAU  ,                                                      
     S     PTAVE ,                                                      
     S     ZTAVIC,                                                      
     S     ZTI1  ,                                                      
     S     PTL                                                          
      COMMON/COM22/                                                     
     S     ZTT   ,                                                      
     S     ZTT1  ,                                                      
     S     ZTT2  ,                                                      
     S     ZUAER ,                                                      
     S     ZUPF  ,                                                      
     S     ZUU   ,                                                      
     S     ZXD                                                          
      COMMON/COM23/                                                     
     S     ZXN   ,                                                      
     S     ZXOZ  ,                                                      
     S     ZXWV  ,                                                      
     S     ZZZ   ,                                                      
     S     ZRES2 ,                                                      
     S     ZTI2  ,                                                      
     S     ZBLAY ,                                                      
     S     ZBLEV ,                                                      
     S     ZCTS  ,                                                      
     S     ZHFG                                                         
      COMMON/COM24/                                                     
     I KT,                                                              
     R    PUU   ,                                                       
     S        PTT   ,                                                   
     S        ZZ                                                        
     R  PCLFR  ,                                                        
     S      PT     ,                                                    
     S      PQOF   ,                                                    
     S      PQLWP  ,                                                    
     S      PEMIS  ,                                                    
     S      PFLT                                                        
      COMMON/COM25/                                                     
     R  ZALBSU,                                                         
     S      ZCG   ,                                                     
     S      ZCLDLW,                                                     
     S      ZCLDSW,                                                     
     S      ZCOOLR,                                                     
     S      ZDT0  ,                                                     
     S      ZEMD  ,                                                     
     S      ZFLUX ,                                                     
     S      ZPMB  ,                                                     
     S      ZTAU  ,                                                     
     S      ZTAVE ,                                                     
     S      ZTL   ,                                                     
     S      ZFSDWN                                                      
      COMMON/COM26/                                                     
     S      ZFSUP ,                                                     
     S      ZHEATR,                                                     
     R   PDP   ,                                                        
     S       PWV   ,                                                    
     S       PFLS  ,                                                    
     S       PPSOL ,                                                    
     S       PRMU0 ,                                                    
     S       PAER  ,                                                    
     S       PALBS ,                                                    
     S       PCG   ,                                                    
     S       PCLDSW,                                                    
     S       POMEGA,                                                    
     S       POZ                                                        
      COMMON/COM27/                                                     
     R   PFDOWN,                                                        
     S       PFUP  ,                                                    
     R   ZAKI  ,                                                        
     S       ZCGAZ ,                                                    
     S       ZCNEB ,                                                    
     S       ZC1I  ,                                                    
     S       ZDSC  ,                                                    
     S       ZDSH                                                       
      COMMON/COM28/                                                     
     S       ZDSIG ,                                                    
     S       ZFACT ,                                                    
     S       ZFACT1 ,                                                   
     S       ZFACT2 ,                                                   
     S       ZFACT3 ,                                                   
     S       ZFACT4 ,                                                   
     S       ZFACT5 ,                                                   
     S       ZFACT6 ,                                                   
     S       ZFD   ,                                                    
     S       ZFRAC ,                                                    
     S       ZFU   ,                                                    
     S       ZG    ,                                                    
     S       ZGG   ,                                                    
     S       ZPIZAZ                                                     
      COMMON/COM29/                                                     
     S       ZRAYL ,                                                    
     S       ZRAY1 ,                                                    
     S       ZRAY2 ,                                                    
     S       ZREF  ,                                                    
     S       ZREFZ ,                                                    
     S       ZRE1  ,                                                    
     S       ZRE2  ,                                                    
     S       ZRJ   ,                                                    
     S       ZRK   ,                                                    
     S       ZRL   ,                                                    
     S       ZRMU  ,                                                    
     S       ZRMUE ,                                                    
     S       ZRMUZ                                                      
      COMMON/COM30/                                                     
     S       ZRNEB ,                                                    
     S       ZRUEF ,                                                    
     S       ZR1   ,                                                    
     S       ZR2   ,                                                    
     S       ZR21  ,                                                    
     S       ZR22  ,                                                    
     S       ZR23  ,                                                    
     S       ZS    ,                                                    
     S       ZSEC  ,                                                    
     S       ZSS1  ,                                                    
     S       ZTAUAZ                                                     
      COMMON/COM31/                                                     
     S       ZTOT  ,                                                    
     S       ZTO1  ,                                                    
     S       ZTR   ,                                                    
     S       ZTRA1 ,                                                    
     S       ZTRA2 ,                                                    
     S       ZTR1                                                       
      COMMON/COM32/                                                     
     S       ZTR2  ,                                                    
     S       ZUD   ,                                                    
     S       ZUM   ,                                                    
     S       ZU1D  ,                                                    
     S       ZU2D  ,                                                    
     S       ZW    ,                                                    
     S       ZN175 ,                                                    
     S       ZN190 ,                                                    
     S       ZO175 ,                                                    
     S       ZO190 ,                                                    
     S       ZP75  ,                                                    
     S       ZP90  ,                                                    
     S       ZSIGN ,                                                    
     S       ZSIGO                                                      
      COMMON/COM33/                                                     
     R   PTR,                                                           
     S       PU,                                                        
     R   ZRH,                                                           
     *       ZCH,                                                       
     *       ZCM,                                                       
     *       ZCL,                                                       
     *       ZCT,                                                       
     *       ZCC,                                                       
     I NTOPCM                                                           
      COMMON/COM34/                                                     
     *        NBASECM,                                                  
     R   ARPRCM,                                                        
     I ITOPH,                                                           
     *        ITOPM,                                                    
     *        ITOPL,                                                    
     L LOC,                                                             
     R   ZQS,                                                           
     *       ZCLC,                                                      
     *       ZQLWP                                                      
      COMMON/COM35/                                                     
     R   VERVEL,                                                        
     *       ZTHETA,                                                    
     *       ZETA,                                                      
     *       ZDTHMIN,                                                   
     I INVB,                                                            
     R   ZCTLWP,                                                        
     *       ZQLOC,                                                     
     *       ZQMEC,                                                     
     *       ZQHIC,                                                     
     R   ZFSOT,                                                         
     #       ZFTET,                                                     
     #       ZFLT                                                       
      COMMON/COM36/                                                     
     #       ZFSO ,                                                     
     #       ZFTE ,                                                     
     #       ZFLB ,                                                     
     #       ZDTDT,                                                     
     #       HEATO,                                                     
     #       HEATE,                                                     
     #       HEATR,                                                     
     #       ZI0                                                        
C                                                                       
      INTEGER                                                           
C     I NRADFR,   NRPART,   NRINT,    NRLSTI,  NRADPFR,  NRADPLA,       
     I NRLST,    NRLEV,    NRLEVP1, NLEV, NLP2, NLEVP1,                 
     I NLON, IRLEV, NTRA, NLAT, ITIM, NUA, NINT, NRPARTI                
C                                                                       
C   NRLEV .... number of vertical levels                                
C   NRLST .... number of points on a latitude line                      
C   NUA   .... number of absorber amount                                
C   NINT  .... number of spectral intervals                             
C   NTRA  .... number of transmission functions                         
C   NLAT  .... number of latitude points                                
C   ITIM  .... number of month under consideration                      
C                                                                       
C      PARAMETER (NRLEV=122,NUA=34,NINT=6,NTRA=34, NLAT=1,              
      PARAMETER (NRLEV=21,NUA=34,NINT=6,NTRA=34, NLAT=1,                
cccm      PARAMETER (NRLEV=37,NUA=18,NINT=6,NTRA=19, NLAT=1,            
cccm     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=10,NLEV=NRLEV,        
     #           NRLEVP1=NRLEV+1,NRLST=1,ITIM=7,NLEV=NRLEV,             
     #           NLP2=NRLST, NLEVP1=NRLEVP1, NLON=NRLST,                
     #           NRPARTI=1)                                             
C                                                                       
      REAL YCLOCK, XLAT(NLAT), Z0, DZ, ALBMPAR, SNPAR,                  
     #     TSUPAR, CTFREEZ, DELTAC, DELTAO, DELTAH                      
C                                                                       
C   YTIME  .... time of the year in radians                             
C   YCLOCK .... time of the day in radians                              
C   XLAT   .... latitudes in radians                                    
C   Z0     .... zero point of height grid                               
C   DZ     .... Interval between height levels                          
C   ALBMPAR.... sw surface albedo as function of longitude:             
C           yearly averaged sea-ice: 0.55                               
C                           water  : 0.07                               
C                           land   : 0.07.lt.ALBM.lt.0.80               
C                                    depending on snow                  
C   SNPAR  .... snow depth in meter                                     
C   TSUPAR .... surface temperature (Kelvin)                            
C   CTFREEZ.... freezing temperature of sea-water (Kelvin)              
C   DELTAC .... Doppler parameter for line shapes of CO2                
C   DELTAO .... Doppler parameter for line shapes of O3                 
C                                                                       
      PARAMETER (Z0=0., CTFREEZ=271.26,                                 
C     PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
     #           DZ=0.289, ALBMPAR=0.2, SNPAR=0.,                       
cccm  PARAMETER (YCLOCK=3.14159, Z0=0., TSUPAR=293.5, CTFREEZ=271.26,   
cccm     #           DZ=0.15, ALBMPAR=0.3, SNPAR=0.,                    
CZ     #           DELTAC=0., DELTAO=0.)                                
     #           DELTAC=6.0E-04, DELTAO=1.E-02, DELTAH=5.0E-04)         
C      PARAMETER (YCLOCK=0., Z0=0., TSUPAR=273., CTFREEZ=271.26,        
C     #           DZ=0.46875, ALBMPAR=0.07, SNPAR=0.)                   
C      PARAMETER (YCLOCK=0., Z0=2.608, TSUPAR=273., CTFREEZ=271.26,     
C     #           DZ=0.2878, ALBMPAR=0.07, SNPAR=0.)                    
C                                                                       
      LOGICAL LDIUR, LOLPAR                                             
C                                                                       
C   LDIUR .... true: sun at time, false: diurnally averaged             
C   LOLPAR.... true: land surface, false: else                          
C                                                                       
C      PARAMETER (LDIUR=.TRUE., LOLPAR=.TRUE.)                          
       PARAMETER (LOLPAR=.TRUE.)                                        
C                                                                       
      INTEGER  C2TEMP                                                   
      CHARACTER*64  MIDTMP                                              
cccm      PARAMETER (C2TEMP=10,                                         
cccm     #    MIDTMP='/home/atm/ugamp/midrad/radfildat.dir/MIDTMP.DAT') 
ccc1      PARAMETER (C2TEMP=10, MIDTMP='tprfil2.dat')                   
C                                                                       
      INTEGER  IRLONI, IRLST, IRLVP1, JPNLEV, JPNLVP1                   
C                                                                       
      PARAMETER (IRLONI=NRLST, IRLST=NRLST,                             
     #           IRLVP1=NRLEVP1, JPNLEV=NRLEV,                          
     #           JPNLVP1=NRLEVP1, IRLEV=NRLEV)                          
C                                                                       
      INTEGER NIPD, NIPD2, NTR, NG1, NG1P1                              
      PARAMETER(NIPD=8,NIPD2=16,NTR=11,NG1=2,NG1P1=3)                   
C                                                                       
C    definition of the functions                                        
C                                                                       
      REAL       CVMGT, SIGMA2                                          
      INTEGER    CVMGI                                                  
C                                                                       
C    definitons of the subroutine cloud                                 
C                                                                       
      REAL RDCPD, CCCC, CFCT, CRHH, CRHM, CRHL, CCLWMR                  
C                                                                       
C    missing definitions in morcrette                                   
C                                                                       
C      REAL   PSIG, PSCT, PCCO2, PCARDI, PTIM1, PTIM2, PTIM3            
      REAL ZCCO2                                                        
                                                                        
      LOGICAL LOH, LOM, LOL, LORAD                                      
C                                                                       
      INTEGER ISHIFT, JK, JKK, JL, JMM, IMM, IMNC, IMNS, JNN,           
     #        IZQS, IZCLC, IZQLWP, JLOOP, KRLEV,                        
     #        NLEV2, NSTEP, NSTART, NLEVM1, NLEV2P1, JB, IBC,           
     #        ITH, ITM, ITL,                                            
     #        IMAXC, JK1, KRLEV2, JK2, IMX1, IMX2, JKJ, JKJR,           
     #        JKJP, IG1, JK3, IXTX, MXIXT, JKIP1,                       
     #        JKI, JKP1, JKL, JKJPN, JI, JJ, JAE3, JJPN, IAE,           
     #        JAE1, JAE2, JC, JCP1, JNU, JA, IND1, IND2, IND3           
      INTEGER IND4, KM12, KXD, KXU, IG, KBS, KDD,                       
     #        NUAER, KMU, KN, KD1, NTRAER, KJP1, KD2, KM1,              
     #        KJ, KU1, JLK, KU2, KTOPM, IMP, NOUT, IMXP1, IMXM1,        
     #        JKC, JCLOUD, JKCP1, JKM1, JKJ1, JKJ2, JK2M1, JKIJ,        
     #        JG, IABS, IAER, JKLP1, JAE, INU, JAJ, IWHERE, JABS,       
     #        JN, KREF, JAJP, JKKI, JN2J, JKKP4, JMN                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD1* SPECTRAL DISTRIBUTION OF AEROSOLS AND OZONE.     
C                     (TRIANGULAR *T10* TRUNCATIONFOR AEROSOLS).        
C                     (TRIANGULAR *T5* TRUNCATIONFOR OZONE).            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R           COZQC(21),COZQS(15),COZHC(21),COZHS(15)                
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMRAD2* CONSTANTS INTERNAL TO THE RADIATION SUBROUTINES. 
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CSUPSAT                                                          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMCON* BASIC UNIVERSAL CONSTANTS AND DERIVED CONSTANTS.  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R API,      A,        OMEGA,    G,                                 
     R CPD,      CPV,      RD,       RV,                                
     R RCPD,     VTMPC1,   VTMPC2,                                      
     R RHOH2O,   ALV,      ALS,      ALF,      CLW,      TMELT,         
     R SOLC,     STBO,     DAYL,     YEARL,                             
     R C1ES,     C2ES,     C3LES,    C3IES,    C4LES,    C4IES,         
     R C5LES,   C5IES                                                   
C                                                                       
C                                                                       
C       --------------------------------------------------------------- 
C      PARAMETER(                                                       
C     I JPM=106,JPN=106,JPK=106,JPGL=160,JPNLON=320,                    
C     I JPT1MAX=400,          JPT2MAX=50,                               
C     I JPT3MAX=300,          JPT4MAX=50,                               
C     I JPHASH=2048,          JPNTRCRS=20,                              
C     I JPTASKS=4,     JPLOCKS=50,    JPEVENT=4,                        
C     I JPLTMT=JPTASKS,                                                 
C     I JPNUMW=352,            JPNUMP=352,                              
C     I JPTABLE=1024,                                                   
C     I JPMAXB =60,                                                     
C     I JPMAXPB=32,                                                     
C     I JPG3FLD=43,         JPG3XF=99,         JPG3CMP=17,              
C     I JPMP1=JPM+1,        JPNP1=JPN+1,       JPKP1=JPK+1,             
C     I JP2MP1=JPMP1*2,     JP4MP1=JPMP1*4,                             
C     I JPED=JPN/2+1,       JPOD=JPNP1/2,                               
C     I IPSP0=JPM+JPN-JPK,  IPSP1=JPNP1*JPMP1, IPSP2=IPSP0*(IPSP0+1)/2, 
C     I JPSP=IPSP1-IPSP2,   JP2SP=2*JPSP,                               
C     I JPHGL=JPGL/2,       JPNLP2=JPNLON+2,                            
C     I                     JPNVCLV=4*JPNLVP1)                          
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C     J. K. GIBSON     E.C.M.W.F.     22/01/82.                         
C     MODIFIED M.J     E.C.M.W.F.     13/12/82.                         
C                                                                       
      INTEGER                                                           
     I NM,       NN,       NK,       NGL,                               
     I NTEMP2,   NMP1,     NNP1,     NKP1,     N2MP1,    N4MP1,         
     I NED,      NOD,      NSP,      N2SP,     NHGL,                    
     I NSCAN,    NRESUM,                                                
     I NFFT,     NFFTW,                                                 
     I NSPACE1,  NSPACE2,                                               
     I NCBASE,   NTBASE,   NTIMST,                                      
     I NWRITE,   NWPTR,                                                 
     I NWLAG,                                                           
     I NPP,      NPPTR,                                                 
     I NPLAG,    NPPDATA,                                               
     I NFRECS,   NGRECS,   NDRECS,                                      
     I NSTOP,    MAXROW                                                 
      LOGICAL                                                           
     L LHIGCM,   LSCYCL,   LHFCLR,   LHFSAV,                            
     L LEND,     LDIFF1,   LINITER,  LDIFFRQ,  LWUNIN,   LWINIT,        
     L LDEBUG,   LSVPCTL,  LSSDFF,   LSSDGF,   LSSDIF,   LRESETP,       
     L LINTPP,   L2BUF,    LOWTIME,  LOPTIME,  LTSTATS,  LOCKLTD        
      REAL                                                              
     R DTIME,    TWODT,    EPS                                          
C                                                                       
C      INTEGER                                                          
C     I NWTIME(JPNUMW),     NROW(JPTASKS),                              
C     I NPTIME(JPNUMP)                                                  
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMGAU* - QUANTITIES RELATED TO THE GAUSSIAN GRID.      
C                                                                       
      REAL                                                              
     R SINLON(NLP2),     COSLON(NLP2)                                   
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMTRU* -QUANTITIES RELATED TO THE SPECTRAL TRUNCATION. 
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSIM* - QUANTITIES NEEDED FOR THE SEMI-IMPLICIT SCHEME
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMHDI* - COEFFICIENTS FOR HORIZONTAL DIFFUSION.        
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMFFT* - QUANTITIES NEEDED FOR THE FAST *FOURIER       
C                         TRANSFORMS.                                   
C                                                                       
      INTEGER                                                           
     I NFFTD1,   NRFFTD1,  NFFTD2,   NRFFTD2,  NFFTD3,   NRFFTD3,       
     I NFFTD4,   NRFFTD4,                                               
     I NFFTI1,   NRFFTI1,  NFFTI2,    NRFFTI2                           
C      REAL                                                             
C     R TRIG(JPNLON)                                                    
      INTEGER                                                           
     I NFAX(10)                                                         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*    *COMMON* *COMSTA* - ARRAYS AND PARAMETERS FOR STATISTICS.         
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMDIA* - QUANTITIES RELATED TO GLOBAL PHYSICAL         
C                         DIAGNOSTICS.                                  
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPSW* SWITCHES RELATED TO THE PARAMETERISATIONS OF      
C                     DIABATIC PROCESSES.                               
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMDSW* SWITCHES RELATED TO THE DYNAMICS                  
C                     AND THE GENERAL CONTROL OF THE FORECAST.          
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH1* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE RADIATION SUBROUTINES.                    
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL                                                              
     R CRAE,                                                            
     R CDISSEM                                                          
C                                                                       
C      REAL       COSRAD(128),        SINRAD(128)                       
C                                                                       
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
C*    COMMON *COMPH2* CONSTANTS TO COMMUNICATE BETWEEN THE MAIN PROGRAM 
C                     AND THE PHYSICAL SUBROUTINES (EXCEPT RADIATION    
C                     ONES).                                            
C                                                                       
C       ----------------------------------------------------------------
C                                                                       
      REAL       CCRH(JPNLEV)                                           
C                                                                       
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMDIZ* VARIABLES AND POINTERS FOR ZONAL MEAN DIAGNOSTICS 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*  *COMMON* *COMMSK* VARIABLES AND POINTERS FOR AREA MEAN DIAGNOSTICS. 
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMSDS* - START/RESTART BLOCK.                          
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*    *COMMON* *COMTASK* - TASK RELATED VARIABLES.                      
C                                                                       
C ----------------------------------------------------------------------
C                                                                       
C*         *COMRAD* - RADIATION POINTERS.                               
C                                                                       
      LOGICAL LO,LO1,LO2,LODIA,LODIAP                                   
C---------------------------------------------------------------------  
C                                                                       
C*    *COMMON* *COMCTL* - CONTROL VARIABLES FOR MODEL HOUSEKEEPING.     
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C     COMMON *YOMAER* RADIATIVE CHARACTERISTICS OF THE AEROSOLS         
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
      REAL TAUA(2,5), PIZA(2,5), CGA(2,5), CAER(5,5), CAEROS            
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C     COMMON *YOMLW*                                                    
C                                                                       
C*    COEFFICIENTS FOR THE LONGWAVE RADIATION SUBROUTINES               
C                                                                       
C NINT  :  NUMBER OF SPECTRAL INTERVALS                                 
C NIPD  :  NUMBER OF BANDS DESCRIBED BY PADE APPROXIMANTS               
C NTR   :  NUMBER OF REFERENCE TEMPERATURES                             
C NTRA  :  NUMBER OF TRANSMISSION FUNCTIONS                             
C NUA   :  NUMBER OF ABSORBER AMOUNTS                                   
C NG1   :  ORDER OF THE GAUSSIAN QUADRATURE                             
C                                                                       
C RT1   :  ROOTS OF THE GAUSSIAN POLYNOMIAL                             
C WG1   :  WEIGHTS OF THE GAUSSIAN POLYNOMIAL                           
C AT, BT:  COEFFICIENTS OF THE TEMPERATURE DEPENDENCE                   
C CNTNU :  H2O CONTINUUM ABSORPTION COEFFICIENTS                        
C O1H,O2H,PIALF0,OCT: OZONE LONGWAVE ABSORPTION COEFFICIENTS            
C TREF, TSTAND      : REFERENCE TEMPERATURES                            
C XP    :  COEFFICIENTS OF POLYNOMIAL DEVELOPMENTS OF PLANCK FUNCTION   
C GA,GB :  PADE APPROXIMANTS OF H2O, CO2 TRANSMISSION FUNCTIONS         
C ETY,PTY :  PARAMETERS OF H2O CONTINUUM                                
C TINTP :  REFERENCE TEMPERATURES OF PRE-COMPUTED TRANSMISSIVITIES      
C CE    :  EMS COEFFS OF H2O TRANSMISSION FUNCTIONS                     
C                                                                       
      REAL  RT1(2),WG1(2),AT(12,3),BT(12,3),CNTNU(4),O1H                
     S ,OCT(4),TREF,XP(6,6),TSTAND,GA(11,24,3),GB(11,24,3)              
     S ,PTY(11,20),ETY(11,20),TINTP(11),TSTP,O2H,PIALF0                 
C                                                                       
      REAL  TB(40,31,2),ALP(40,31,2),BET(40,31,2),                      
     S ULG(40),PLG(31),AP(40,31,2),BP(40,31,2),TBO3(38,31),TO3B(38,31)  
     S,ULGO(38),AO3(38,31),BO3(38,31),O3A(38,31),O3B(38,31)             
C     ----------------------------------------------------------------- 
C*COMDECK YOMRDU                                                        
C     ----------------------------------------------------------------- 
C                                                                       
C*    CONTROL VARIABLES FOR RADIATION: INTERNAL SWITCH AND INDICES      
C                                                                       
C ZEELOG:  SECURITY THRESHOLD FOR ABSORBER AMOUNT IN LAPLACE TRANSFORM  
C ZEPSC :  SECURITY THRESHOLD FOR CLOUD COVER                           
C ZEPSCO:  SECURITY THRESHOLD FOR OZONE AMOUNT                          
C ZEPSCQ:  SECURITY THRESHOLD FOR WATER VAPOR                           
C ZEPSCT:  SECURITY THRESHOLD FOR SHORTWAVE OPTICAL THICKNESS           
C ZEPSCW:  SECURITY THRESHOLD FOR CLOUD LIQUID WATER PATH               
C                                                                       
      REAL          CDAY,CCO2,CH2O,DIFF,X10E                            
     S             ,ZEELOG,ZEPSC,ZEPSCO,ZEPSCQ,ZEPSCT,ZEPSCW            
C     ----------------------------------------------------------------- 
C*COMDECK YOMSW                                                         
C     ----------------------------------------------------------------- 
C                                                                       
C*    COEFFICIENTS FOR THE SHORTWAVE RADIATION SUBROUTINES              
C                                                                       
C APAD  :  PADE APPROXIMANTS NUMERATOR                                  
C BPAD  :  PADE APPROXIMANTS DENOMINATOR                                
C D     :  TRANSMISSION LIMIT FOR INFINITE ABSORBER AMOUNT              
C CRAY  :  RAYLEIGH SCATTERING COEFFICIENTS                             
C SUN   :  SOLAR FRACTION IN SPECTRAL INTERVALS                         
C                                                                       
      REAL  APAD(2,3,7),BPAD(2,3,7),D(2,3),CRAY(2,6),SUN(2)             
C     ----------------------------------------------------------------- 
                                                                        
C                                                                       
C  -------------------------------------------------------------------- 
C                                                                       
      REAL   TTT(1,NLAT,NRLEV,12)                                       
C                                                                       
C        0.1  "RADINT POINTERS"                                         
C             -----------------                                         
C                                                                       
      REAL   TM1(NLP2,NLEV),                                            
     *       TSM1M(NLP2),                                               
     *       QM1(NLP2,NLEV),                                            
     *       SNM1M(NLP2),                                               
     *       ALBM(NLP2),                                                
     *       AVVELM(NLP2)                                               
      REAL   SLMM(NLP2),                                                
     *       APHM1(NLP2,NLEVP1),                                        
     *       APM1(NLP2,NLEV)                                            
C                                                                       
      LOGICAL LOLAND(NLP2)                                              
C                                                                       
C     VARIABLES FOR FULL GRID AND TRANSFORMS.                           
C                                                                       
      REAL   ZTI(NLP2,NRLEVP1),                                         
     *       ZCTI(NLP2,NRLEVP1),                                        
     *       ZCPH(NLP2,NRLEVP1),                                        
     *       ZWV(NLP2,NRLEV),                                           
     *       ZSW(NLP2,NRLEV),                                           
     *       ZALTE(NLP2),                                               
     *       ZCALTE(NLP2),                                              
     *       ZALSO(NLP2),                                               
     *       ZCALSO(NLP2)                                               
C                                                                       
      REAL   ZOZQ(NLP2),                                                
     #       ZOZH(NLP2)                                                 
C                                                                       
      REAL   ZTRSOL(NLP2,NLEVP1),                                       
     *       ZEMTER(NLP2,NLEVP1)                                        
C                                                                       
C     FINAL RESULTS ARE STORED ON:                                      
C                                                                       
      REAL   TRSOLM(NLP2,NLEVP1),                                       
     *       EMTERM(NLP2,NLEVP1)                                        
C                                                                       
      REAL   AMU0(NLP2),                                                
     *       RDAYL(NLP2)                                                
C                                                                       
C     VARIABLES WITHIN THE SPLITTED LATITUDE LOOP.                      
C                                                                       
      REAL   CLC(NRLST,NRLEV),                                          
     *       TI(NRLST,NRLEVP1),                                         
     *       DP(NRLST,NRLEV),                                           
     *       AADS(NRLST,NRLEV),                                         
     *       AEQ1(NRLST,NRLEV),                                         
     *       AEQ2(NRLST,NRLEV),                                         
     *       AEQ3(NRLST,NRLEV),                                         
     *       AEQ4(NRLST,NRLEV),                                         
     *       AEQ5(NRLST,NRLEV),                                         
     *       SSW(NRLST,NRLEV),                                          
     *       WV(NRLST,NRLEV),                                           
     *       QOF(NRLST,NRLEV),                                          
     *       QCF(NRLST,NRLEV),                                          
     *       CLWA(NRLST,NRLEV),                                         
     *       APRE(NRLST),                                               
     *       SMU0(NRLST),                                               
     *       ALTE(NRLST),                                               
     *       ALSO(NRLST)                                                
      REAL   ZDPO(NRLST),                                               
     *       ZQCFO(NRLST),                                              
     *       ZAEQSO(NRLST),                                             
     *       ZQOFO(NRLST),                                              
     *       ZDPN(NRLST),                                               
     *       ZQCFN(NRLST),                                              
     *       ZAEQSN(NRLST),                                             
     *       ZQOFN(NRLST)                                               
      REAL   FLT(NRLST,NRLEVP1),                                        
     *       FLS(NRLST,NRLEVP1)                                         
C                                                                       
C     DIAGNOSTIC VARIABLES.                                             
C                                                                       
      REAL   DIAG(NLEVP1,4),                                            
     *       DIA1(7),                                                   
     *       ZDIAG(NLEVP1,10),                                          
     *       ZDIA1(21),                                                 
     *       ZDIA(NLON),                                                
     *       ZDIAF(NLON,7),                                             
     *       ZDIAT(NLEVP1)                                              
C                                                                       
C                                                                       
C                                                                       
      REAL PGG(IRLST),                                                  
     #     PREF(IRLST),                                                 
     #     PRE1(IRLST),                                                 
     #     PRE2(IRLST),                                                 
     #     PRMUZ(IRLST),                                                
     #     PTO1(IRLST),                                                 
     #     PTR1(IRLST),                                                 
     #     PTR2(IRLST),                                                 
     #     PW(IRLST)                                                    
C                                                                       
C                                                                       
C  what about the 1-dimensional PALBS???                                
C                                                                       
      REAL     PALBSN  (IRLST)                                          
C                                                                       
C*       0.2   "LOCAL" ARRAYS                                           
C              --------------                                           
      REAL ZABSCU(IRLST ,NUA, 3*NRLEV+1),                               
     S     ZABSLY(IRLST ,NUA, 3*NRLEV+1) ,                              
     S     ZADJD (IRLST , IRLVP1)      ,                                
     S     ZADJU (IRLST , IRLVP1)       ,                               
     S     ZBGND (IRLST)                ,                               
     S     ZBINT (IRLST , IRLVP1)       ,                               
     S     ZBSUR (IRLST , NINT)         ,                               
     S     ZBSUIN(IRLST)                ,                               
     S     ZBTOP (IRLST , NINT)         ,                               
     S     PCLDLW(IRLST , IRLEV)        ,                               
     S     ZCLOUD(IRLST)                ,                               
     S     ZCNTRB(IRLST , IRLVP1,IRLVP1),                               
     S     ZCUMP (IRLST)                ,                               
     S     ZCUMPT(IRLST)                ,                               
     S     ZDBDT (IRLST , NINT , IRLEV) ,                               
     S     ZDBSL (IRLST , NINT , 2*NRLEV),                              
     S     ZDISTD(IRLST , IRLVP1)       ,                               
     S     ZDISTU(IRLST , IRLVP1)                                       
      REAL                                                              
     S     ZDNF  (IRLST , IRLVP1,IRLVP1),                               
     S     PDT0  (IRLST)                                                
      REAL                                                              
     S     ZDUC  (IRLST , 3*NRLEV+1)       ,                            
     S     ZEFFT (IRLST)                ,                               
     S     ZEMD  (IRLST , IRLEV), ZEMU  (IRLST , IRLEV),                
     S     ZEPSC2(IRLST),  ZEPSC3(IRLST),                               
     S     ZEPSH1(IRLST),  ZEPSH2(IRLST),                               
     S     ZEPSH3(IRLST),  ZEPSH4(IRLST),                               
     S     ZEPSH5(IRLST),  ZEPSH6(IRLST),                               
     S     ZEPHIO(IRLST),  ZFACTC(IRLST),                               
     S     ZZFD(IRLST), ZFDD(IRLST), ZFDN(IRLST , IRLVP1)               
      REAL                                                              
     S     PFLUX (IRLST , 2 , IRLVP1)   ,                               
     S     PFLUC (IRLST , 2 , IRLVP1)   ,                               
     S     ZFLWP (IRLST)                ,                               
     S     PCTS  (IRLST , IRLEV)        ,                               
     S     ZZFU   (IRLST)               ,                               
     S     ZFUP  (IRLST , IRLVP1)       ,                               
     S     ZGLAYD(IRLST)                ,                               
     S     ZGLAYU(IRLST)                ,                               
     S     ZOMEGA(IRLST , 2 , IRLEV)                                    
C                                                                       
      INTEGER                                                           
     S     ITX   (IRLST)                ,                               
     S     IXDIA (IRLST , IRLVP1)       ,                               
     S     IXT   (IRLST , IRLVP1,IRLVP1),                               
     S     IXTSU (IRLST , IRLVP1)       ,                               
     S     IXTTP (IRLST , IRLVP1)       ,                               
     S     IMX   (IRLST)                ,                               
     S     IMXP  (IRLST)                                                
C                                                                       
      REAL ZOZ   (IRLST , IRLEV)        ,                               
     S     PPMB  (IRLST , IRLVP1)       ,                               
     S     ZRES  (IRLST)                ,                               
     S     PB    (IRLST, 6 , IRLVP1)    ,                               
     S     ZSSIG (IRLST , 3*NRLEV+1)    ,                               
     S     PTAU  (IRLST , 2 , IRLEV)    ,                               
     S     PTAVE (IRLST , IRLEV)        ,                               
     S     ZTAVIC(IRLST)                ,                               
     S     ZTI1  (IRLST)                ,                               
     S     PTL   (IRLST , IRLVP1)       ,                               
     S     ZTT   (IRLST , NTRA)         ,                               
     S     ZTT1  (IRLST , NTRA)         ,                               
     S     ZTT2  (IRLST , NTRA)         ,                               
     S     ZUAER (IRLST , NINT)         ,                               
     S     ZUPF  (IRLST , IRLVP1,IRLVP1),                               
     S     ZUU   (IRLST , NUA)          ,                               
     S     ZXD   (IRLST)                                                
      REAL                                                              
     S     ZXN   (IRLST)                ,                               
     S     ZXOZ  (IRLST)                ,                               
     S     ZXWV  (IRLST)                ,                               
     S     ZZZ   (IRLST)                ,                               
     S     ZRES2 (IRLST)                ,                               
     S     ZTI2  (IRLST)                ,                               
     S     ZBLAY (IRLST,IRLEV)          ,                               
     S     ZBLEV (IRLST,IRLVP1)         ,                               
     S     ZCTS  (IRLST)                ,                               
     S     ZHFG  (IRLST)                                                
C                                                                       
C     ------------------------------------------------------------------
C                                                                       
C                                                                       
      INTEGER KT    (IRLST)                                             
      REAL    PUU   (IRLST , NUA  )     ,                               
     S        PTT   (IRLST , NTRA )     ,                               
     S        ZZ    (IRLST)                                             
C                                                                       
C     -------------------------------------------------------- ---------
C                                                                       
      REAL  PCLFR  (IRLST , IRLEV)       ,                              
     S      PT     (IRLST , IRLVP1)       ,                             
     S      PQOF   (IRLST , IRLEV)          ,                           
     S      PQLWP  (IRLST , IRLEV)          ,                           
     S      PEMIS  (IRLST)                                              
      REAL                                                              
     S      PFLT   (IRLST , IRLVP1)                                     
C                                                                       
      REAL  ZALBSU(IRLST , 2 )           ,                              
     S      ZCG   (IRLST , 2 , IRLEV)    ,                              
     S      ZCLDLW(IRLST , IRLEV)        ,                              
     S      ZCLDSW(IRLST , IRLEV)        ,                              
     S      ZCOOLR(IRLST , IRLEV)        ,                              
     S      ZDT0  (IRLST)                ,                              
     S      ZFLUX (IRLST , 2 , IRLVP1)   ,                              
     S      ZPMB  (IRLST , IRLVP1)       ,                              
     S      ZTAU  (IRLST , 2 , IRLEV)    ,                              
     S      ZTAVE (IRLST , IRLEV)        ,                              
     S      ZTL   (IRLST , IRLVP1)       ,                              
     S      ZFSDWN(IRLST , IRLVP1)       ,                              
     S      ZFSUP (IRLST , IRLVP1)       ,                              
     S      ZHEATR(IRLST , IRLEV )                                      
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C                                                                       
      REAL   PDP   (IRLST , IRLEV)     ,                                
     S       PWV   (IRLST , IRLEV)     ,                                
     S       PFLS  (IRLST , IRLVP1)    ,                                
     S       PPSOL (IRLST        )     ,                                
     S       PRMU0 (IRLST        )     ,                                
     S       PAER  (IRLST,IRLEV,5)     ,                                
     S       PALBS (IRLST , 2 )        ,                                
     S       PCG   (IRLST , 2 , IRLEV) ,                                
     S       PCLDSW(IRLST , IRLEV)     ,                                
     S       POMEGA(IRLST , 2 , IRLEV) ,                                
     S       POZ   (IRLST , IRLEV)                                      
C                                                                       
      REAL   PFDOWN(IRLST , IRLVP1)    ,                                
     S       PFUP  (IRLST , IRLVP1)                                     
C                                                                       
      REAL   ZAKI  (IRLST , 2 )        ,                                
     S       ZCGAZ (IRLST , IRLEV)     ,                                
     S       ZCNEB (IRLST , IRLEV)     ,                                
     S       ZC1I  (IRLST , IRLVP1)    ,                                
     S       ZDSC  (IRLST , IRLEV)     ,                                
     S       ZDSH  (IRLST , IRLEV)     ,                                
     S       ZDSIG (IRLST , IRLEV)     ,                                
     S       ZFACT (IRLST)             ,                                
     S       ZFACT1 (IRLST), ZFACT2 (IRLST),                            
     S       ZFACT3 (IRLST), ZFACT4 (IRLST),                            
     S       ZFACT5 (IRLST), ZFACT6 (IRLST),                            
     S       ZFD   (IRLST , IRLVP1)    ,                                
     S       ZFRAC (IRLST)             ,                                
     S       ZFU   (IRLST , IRLVP1)    ,                                
     S       ZG    (IRLST)             ,                                
     S       ZGG   (IRLST)             ,                                
     S       ZPIZAZ(IRLST , IRLEV)     ,                                
     S       ZRAYL (IRLST),  ZRAY1 (IRLST , IRLVP1)                     
      REAL                                                              
     S       ZRAY2 (IRLST , IRLVP1)    ,                                
     S       ZREF  (IRLST)             ,                                
     S       ZREFZ (IRLST , 2 , IRLVP1),                                
     S       ZRE1  (IRLST)             ,                                
     S       ZRE2  (IRLST)             ,                                
     S       ZRJ   (IRLST , 6 , IRLVP1),                                
     S       ZRK   (IRLST , 6 , IRLVP1),                                
     S       ZRL   (IRLST , 8)         ,                                
     S       ZRMU  (IRLST)             ,                                
     S       ZRMUE (IRLST , IRLVP1)    ,                                
     S       ZRMUZ (IRLST)                                              
      REAL                                                              
     S       ZRNEB (IRLST)             ,                                
     S       ZRUEF (IRLST , 8)         ,                                
     S       ZR1   (IRLST)             ,                                
     S       ZR2   (IRLST)             ,                                
     S       ZR21  (IRLST)             ,                                
     S       ZR22  (IRLST)             ,                                
     S       ZR23  (IRLST)             ,                                
     S       ZS    (IRLST)             ,                                
     S       ZSEC  (IRLST)             ,                                
     S       ZSS1  (IRLST)             ,                                
     S       ZTAUAZ(IRLST , IRLEV)     ,                                
     S       ZTOT  (IRLST , IRLVP1)    ,                                
     S       ZTO1  (IRLST)             ,                                
     S       ZTR   (IRLST , 2 , IRLVP1),                                
     S       ZTRA1 (IRLST , IRLVP1)    ,                                
     S       ZTRA2 (IRLST , IRLVP1)                                     
      REAL                                                              
     S       ZTR1  (IRLST)             ,                                
     S       ZTR2  (IRLST)             ,                                
     S       ZUD   (IRLST , 3 , IRLVP1),                                
     S       ZUM   (IRLST , IRLVP1)    ,                                
     S       ZU1D  (IRLST)             ,                                
     S       ZU2D  (IRLST)             ,                                
     S       ZW    (IRLST)             ,                                
     S       ZN175 (IRLST)             ,                                
     S       ZN190 (IRLST)             ,                                
     S       ZO175 (IRLST)             ,                                
     S       ZO190 (IRLST)             ,                                
     S       ZP75  (IRLST)             ,                                
     S       ZP90  (IRLST)             ,                                
     S       ZSIGN (IRLST)             ,                                
     S       ZSIGO (IRLST)                                              
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
      REAL   PTR    (IRLST)     ,                                       
     S       PU     (IRLST)                                             
C                                                                       
C-----------------------------------------------------------------------
C                                                                       
C   CLOUD-POINTERS                                                      
C   --------------                                                      
C                                                                       
      REAL   ZRH(NLP2,NLEV),                                            
     *       ZCH(NLP2),                                                 
     *       ZCM(NLP2),                                                 
     *       ZCL(NLP2),                                                 
     *       ZCT(NLP2),                                                 
     *       ZCC(NLP2)                                                  
      INTEGER NTOPCM(NLP2),                                             
     *        NBASECM(NLP2)                                             
      REAL   ARPRCM(NLP2)                                               
      INTEGER ITOPH(NLP2),                                              
     *        ITOPM(NLP2),                                              
     *        ITOPL(NLP2)                                               
      LOGICAL LOC(NLP2)                                                 
      REAL   ZQS(NLP2,NLEV),                                            
     *       ZCLC(NLP2,NLEV),                                           
     *       ZQLWP(NLP2,NLEV)                                           
C                                                                       
      REAL   VERVEL(NLP2,NLEV),                                         
     *       ZTHETA(NLP2,NLEV),                                         
     *       ZETA(NLP2,NLEV),                                           
     *       ZDTHMIN(NLP2)                                              
      INTEGER INVB(NLP2)                                                
      REAL   ZCTLWP(NLP2),                                              
     *       ZQLOC(NLP2),                                               
     *       ZQMEC(NLP2),                                               
     *       ZQHIC(NLP2)                                                
C                                                                       
C  definitions of "radheat"                                             
C                                                                       
      REAL   ZFSOT(NLON),                                               
     #       ZFTET(NLON),                                               
     #       ZFLT (NLON),                                               
     #       ZFSO (NLON),                                               
     #       ZFTE (NLON),                                               
     #       ZFLB (NLON),                                               
     #       ZDTDT(NLON),                                               
     #       HEATO(NLON,NLEV),                                          
     #       HEATE(NLON,NLEV),                                          
     #       HEATR(NLON,NLEV),                                          
     #       ZI0  (NLON)                                                
C                                                                       
C PIERS 13-8-97                                                         
C                                                                       
         REAL PIERSLWP                                                  
C                                                                       
C Piece of code needed to replace the Nikos radiation scheme with       
C the Morcrette radiation scheme in the igcm3.1.                        
C See morcrette3_1_1.readme for details.                                
C                                                                       
C                                                                       
c      program lh                                                       
c                                                                       
c      parameter(NL=23)                                                 
c      real P(NL),t(NL),h2o(NL),o3(NL),htlw(NL),htsw(NL)                
c      real swalb,doy                                                   
c       REAL CF(4,2)     !cloud fractions,lwps                          
c       INTEGER IC(4,2)   ! cloud positions                             
c       REAL RFLUXES(2,2,2)                                             
c      open(11,file='mls23.dat')                                        
c      read(11,*) NLEV                                                  
c      DOY=80                                                           
c      swalb=0.07                                                       
c      alat=40.0                                                        
c       doy=float(i)                                                    
c      do i=nlev,1,-1                                                   
c         read(11,*) P(i),t(i),h2o(i),adum,o3(i)                        
c         print *,i,p(i),t(i),h2o(i),o3(i)                              
c      enddo                                                            
c                                                                       
c       ic(1,1)=3                                                       
c      ic(1,2)=17                                                       
c       ic(3,1)=6                                                       
c       ic(2,2)=6                                                       
c       cf(1,1)=0.3                                                     
c       cf(1,2)=45.0                                                    
c        cf(3,1)=0.8                                                    
c       cf(4,2)=20.0                                                    
c      alon=0.0                                                         
c      call morcigcm(p,t,h2o,o3,alat,htlw,htsw,doy,cf,ic,               
c     $     rfluxes,swalb,alon)                                         
c      enddo                                                            
c      do i=nl,1,-1                                                     
c         print '(i6,4F14.4)',i,p(i),t(i),htsw(i),htlw(i)               
c      enddo                                                            
c      print *,RFLUXES(1,1,1), RFLUXES(1,1,2), RFLUXES(1,2,1),          
c     $     RFLUXES(1,2,2)                                              
c      print *,RFLUXES(2,1,1), RFLUXES(2,1,2), RFLUXES(2,2,1),          
c     $     RFLUXES(2,2,2)                                              
c                                                                       
c                                                                       
c      end                                                              
c      INCLUDE 'zparb.upd'                                              
C                                                                       
   10 CONTINUE                                                          
C                                                                       
C     ------------------------------------------------------------      
C                                                                       
C*        1.       PRESET CONSTANTS.                                    
C                  ------ ----------                                    
C                                                                       
  100 CONTINUE                                                          
C                                                                       
      API=2.*ASIN(1.D0)                                                 
      A=6371000.                                                        
      OMEGA=.7292E-4                                                    
      G=9.80665                                                         
      CPD=1005.46                                                       
      CPV=1869.46                                                       
      RD=287.05                                                         
      RV=461.51                                                         
C                                                                       
      RCPD=1./CPD                                                       
      VTMPC1=RV/RD-1.                                                   
      VTMPC2=CPV/CPD-1.                                                 
C                                                                       
      RHOH2O=1000.                                                      
      ALV=2.5008E6                                                      
      ALS=2.8345E6                                                      
      ALF=ALS-ALV                                                       
C                                                                       
      CLW=4186.84                                                       
      TMELT=273.16                                                      
C                                                                       
C Solar constant                                                        
C      SOLC=1376.                                                       
      SOLC=1365.                                                        
      STBO=5.67E-8                                                      
C                                                                       
      DAYL=86400.                                                       
      YEARL=365.2425                                                    
C                                                                       
      C1ES=610.78                                                       
      C2ES=C1ES*RD/RV                                                   
      C3LES=17.269                                                      
      C3IES=21.875                                                      
      C4LES=35.86                                                       
      C4IES=7.66                                                        
      C5LES=C3LES*(TMELT-C4LES)                                         
      C5IES=C3IES*(TMELT-C4IES)                                         
C                                                                       
C     ------------------------------------------------------------      
C                                                                       
      RETURN                                                            
      END                                                               
C***************************************************************        
C                                                                       
C   FUNCTION CVMGT                                                      
C                                                                       
C***************************************************************        
C                                                                       
       REAL FUNCTION CVMGT(Z1, Z2, LO)                                  
C                                                                       
       IMPLICIT NONE                                                    
C                                                                       
       REAL Z1, Z2                                                      
       LOGICAL LO                                                       
C                                                                       
       IF (LO) THEN                                                     
          CVMGT = Z1                                                    
       ELSE                                                             
          CVMGT = Z2                                                    
       END IF                                                           
C                                                                       
       RETURN                                                           
       END                                                              
C***************************************************************        
C                                                                       
C   FUNCTION CVMGI                                                      
C                                                                       
C***************************************************************        
C                                                                       
       INTEGER FUNCTION CVMGI(I1, I2, LO)                               
C                                                                       
       IMPLICIT NONE                                                    
C                                                                       
       INTEGER I1, I2                                                   
       LOGICAL LO                                                       
C                                                                       
       IF (LO) THEN                                                     
          CVMGI = I1                                                    
       ELSE                                                             
          CVMGI = I2                                                    
       END IF                                                           
C                                                                       
       RETURN                                                           
       END                                                              
C                                                                       
C***************************************************************        
C                                                                       
C   FUNCTION CVMGM                                                      
C                                                                       
C***************************************************************        
C                                                                       
       REAL FUNCTION CVMGM(Z1, Z2, Z3)                                  
C                                                                       
       IMPLICIT NONE                                                    
C                                                                       
       REAL Z1, Z2, Z3                                                  
C                                                                       
       IF (Z3 .LT. 0) THEN                                              
          CVMGM = Z1                                                    
       ELSE                                                             
          CVMGM = Z2                                                    
       END IF                                                           
C                                                                       
       RETURN                                                           
       END                                                              
C                                                                       
C**************************************************************         
C                                                                       
C   FUNCTION SIGMA2                                                     
C                                                                       
C***************************************************************        
C                                                                       
       REAL FUNCTION SIGMA2(N, SX, INCX)                                
C                                                                       
       IMPLICIT NONE                                                    
C                                                                       
       INTEGER N, INCX, I                                               
       REAL SX(N*INCX)                                                  
C                                                                       
       SIGMA2 = 0.                                                      
       DO 1 I=1,N                                                       
          SIGMA2 = SIGMA2 + SX(I*INCX)                                  
 1     CONTINUE                                                         
C                                                                       
       END                                                              
                                                                        
      SUBROUTINE SOLANG (LDIUR,DOY,YCLOCK,ALAT,alon,                    
     :     AMU0,RDAYL,CDISSEM)                                          
C Pm.M. deF   27-1-98                                                   
C inputs                                                                
C LDIUR  ! Logical for diurnal average                                  
C DOY    ! Julian day of year                                           
C YCLOCK ! Don't know yet                                               
C ALAT   ! Latitude in degrees                                          
C ALON   ! Longitude in degrees                                         
C outputs                                                               
C AMU0   ! Cosine of solar zenith angle                                 
C RDAYL  ! Fractional day length                                        
C CDISSEM  ! Suns relative distance as a fraction                       
                                                                        
C**** *SOLANG* - FOR SOLAR ZENITH ANGLE AND RELATIVE DAYLENGTH.         
C                                                                       
C     PURPOSE.                                                          
C     --------                                                          
C                                                                       
C          THIS ROUTINE GIVES DIFFERENT RESULTS DEPENDING ON A LOGICAL  
C     SWITCH. IF LDIUR IS TRUE ONE OBTAINS ACTUAL SOLAR ZENITH ANGLES   
C     AND VALUES OF ONE OR ZERO DEPENDING ON THE SIGN OF THE FORMER. IF 
C     LDIUR IS FALSE ONE GETS THE SAME ANSWERS AT ALL POINTS, I.E. MEAN 
C     VALUE OF THE DAYTIME SOLAR ZENITH ANGLE AND RELATIVE LENGTH OF    
C     THE DAY.                                                          
C                                                                       
C   LDIUR .... true: sun at time, false: diurnally averaged             
C                                                                       
C     ----------                                                        
C                                                                       
      REAL DOY,ALAT,RDAYL,AMU0,YCLOCK,ALON,XLON                         
C                                                                       
      REAL XLAT,YTIME,YEARL,API,ZC1YT,ZS1YT,ZC2YT,ZS2YT,CDISSEM         
      INTEGER JDAY                                                      
                                                                        
      REAL ZMU0(128),ZRDAYL(128)                                        
      LOGICAL LDIUR,LO                                                  
      REAL ZCDIS(5),ZCEQT(5),ZCDEC(5)                                   
      DATA ZCDIS/+1.000110,+0.034221,+0.001280,+0.000719,+0.000077/     
      DATA CRAE/+0.1277E-02/                                            
      DATA ZCDEC/+0.006918,-0.399912,+0.070257,-0.006758,+0.000907/     
      DATA ZCEQT/+0.000075,+0.001868,-0.032077,-0.014615,-0.040849/     
                                                                        
                                                                        
C     Bug fix: year length made consistent with the remaining of        
C              code. M. Bourqui 15 nov 2001                             
      YEARL=360.0                                                       
      API=2.0*ASIN(1.0)                                                 
      XLAT=API*alat/180.0                                               
      XLON=API*ALON/180.0                                               
      JDAY=INT(DOY)                                                     
      YTIME = (FLOAT(JDAY-1)+YCLOCK/2./API)/YEARL*2.*API                
                                                                        
C                                                                       
C                                                                       
C                                                                       
C dirrnal cycle part                                                    
C                                                                       
C*    COMPUTATIONAL CONSTANTS.                                          
C     ------------- ----------                                          
C                                                                       
      ZC1YT=COS(YTIME)                                                  
      ZS1YT=SIN(YTIME)                                                  
      ZC2YT=ZC1YT**2-ZS1YT**2                                           
      ZS2YT=2.*ZS1YT*ZC1YT                                              
      CDISSEM=ZCDIS(1)+ZCDIS(2)*ZC1YT+ZCDIS(3)*ZS1YT+ZCDIS(4)*ZC2YT     
     *       +ZCDIS(5)*ZS2YT                                            
C                                                                       
      ZCRAE=CRAE*(CRAE+2.)                                              
C     ------------------------------------------------------------------
C*         2.     SOLAR ANGLE AND OZONE/AEROSOL PARAMETERS COMPUTATIONS.
C                 ----- ----- --- ------------- ---------- -------------
C                                                                       
 200  CONTINUE                                                          
C                                                                       
C*         2.1     INTRODUCE THE LATITUDE DEPENDENCY.                   
      ZSIN = SIN(XLAT)                                                  
      ZSQCST = SQRT(1.0-ZSIN**2)                                        
      ZDECLI=ZCDEC(1)+ZCDEC(2)*ZC1YT+ZCDEC(3)*ZS1YT+ZCDEC(4)*ZC2YT      
     *       +ZCDEC(5)*ZS2YT                                            
      ZEQTIM=ZCEQT(1)+ZCEQT(2)*ZC1YT+ZCEQT(3)*ZS1YT+ZCEQT(4)*ZC2YT      
     *       +ZCEQT(5)*ZS2YT                                            
      ZZEN1=SIN(ZDECLI)                                                 
      ZZEN2=COS(ZDECLI)*COS(YCLOCK+ZEQTIM)                              
      ZZEN3=COS(ZDECLI)*SIN(YCLOCK+ZEQTIM)                              
C                                                                       
      ZTIM1 = ZZEN1 * ZSIN                                              
      ZTIM2 =-ZZEN2 * ZSQCST                                            
      ZTIM3 = ZZEN3 * ZSQCST                                            
C                                                                       
C     ---------------------------------------------------------------   
C                                                                       
C*         2.     COMPUTATIONS IF DIURNAL CYCLE "ON".                   
C                 ------------ -- ------- ----- -----                   
C                                                                       
C***                                                                    
                                                                        
c      COSLON=1.0                                                       
c      SINLON=0.0                                                       
       COSLON=COS(XLON)                                                 
       SINLON=SIN(XLON)                                                 
      IF(LDIUR) THEN                                                    
C***                                                                    
      AMU0=ZTIM1+ZTIM2*COSLON+ZTIM3*SINLON                              
      LO=AMU0.GE.0.                                                     
      AMU0=CVMGT(AMU0,0.0,LO)                                           
      RDAYL=CVMGT(1.0,0.0,LO)                                           
      IF (LO) THEN                                                      
        RDAYL=1.0                                                       
        AMU0=AMU0                                                       
      ELSE                                                              
        RDAYL=0.0                                                       
        AMU0=0.0                                                        
      ENDIF                                                             
C                                                                       
C     ----------------------------------------------------------------- 
C                                                                       
C*         3.     COMPUTATIONS IF DIURNAL CYCLE "OFF".                  
C                 ------------ -- ------- ----- ------                  
      ELSE                                                              
C***                                                                    
      DO 301 JL=1,128                                                   
      ZL=2.*API*(JL-1.)/128.                                            
      ZMU0(JL)=ZTIM1+ZTIM2*COS(ZL)+ZTIM3*SIN(ZL)                        
                                                                        
      LO=ZMU0(JL).GE.0.                                                 
      ZMU0(JL)=CVMGT(ZMU0(JL),0.0,LO)                                   
      ZRDAYL(JL)=CVMGT(1.0,0.0,LO)                                      
  301 CONTINUE                                                          
      ZS1=SIGMA2(128,ZMU0(1),1)                                         
      ZS2=SIGMA2(128,ZRDAYL(1),1)                                       
      IF(ZS2.NE.0.) THEN                                                
      ZS1=ZS1/ZS2                                                       
      ZS2=ZS2/128.                                                      
      END IF                                                            
      AMU0=ZS1                                                          
      RDAYL=ZS2                                                         
C***                                                                    
      END IF                                                            
C                                                                       
      AMU0=CRAE/(SQRT(AMU0**2+ZCRAE)-AMU0)                              
                                                                        
      RETURN                                                            
      END                                                               
c                                                                       
C****                                                                   
C                                                                       
C**** *INICMA*  - Initialize coupled mode communication for atmosphere  
C                 and exchange some initial information with Oasis      
C                                                                       
C     Input:                                                            
C     -----                                                             
C       KASTP  : total number of timesteps in atmospheric model         
C       KEXCH  : frequency of exchange (in time steps)                  
C       KSTEP  : length of timestep (in seconds)                        
C                                                                       
C     -----------------------------------------------------------       
C                                                                       
      SUBROUTINE inicma(kastp,kexch,kstep)                              
c                                                                       
C OASIS size parameters                                                 
      INTEGER, PARAMETER :: IMB = 128                                   
      INTEGER, PARAMETER :: JM = 64                                     
      INTEGER, PARAMETER :: IMJM = IMB*JM                               
c                                                                       
      INTEGER kastp, kexch, kstep                                       
      INTEGER iparal(3)                                                 
      INTEGER ifcpl, idt, info, imxtag, istep, jf                       
c                                                                       
C                                                                       
C -- param_cou.h                                                        
C                                                                       
      INTEGER jpmaxfld                                                  
      PARAMETER(jpmaxfld = 20)                                          
      INTEGER jpflda2o1                                                 
      PARAMETER(jpflda2o1 = 4)                                          
      INTEGER jpflda2o2                                                 
      PARAMETER(jpflda2o2 = 4)                                          
C                                                                       
      INTEGER jpfldo2a                                                  
      PARAMETER(jpfldo2a = 3)                                           
C                                                                       
C                                                                       
C -- inc_cpl.h                                                          
C    **********                                                         
C@                                                                      
C@  Contents : variables describing field restart file names            
C@  --------                                                            
C@                                                                      
C@ -- cl_write/cl_read  : for fields to write/READ                      
C@ -- cl_f_write/cl_f_read  : for fields to write/read                  
C@                                                                      
C     ----------------------------------------------------------------- 
C                                                                       
      CHARACTER*8 cl_writ(jpmaxfld), cl_read(jpmaxfld)                  
      CHARACTER*8 cl_f_writ(jpmaxfld), cl_f_read(jpmaxfld)              
      COMMON / comcpl / cl_writ, cl_read, cl_f_writ, cl_f_read          
C     ----------------------------------------------------------------- 
      CHARACTER*3 cljobnam      ! experiment name                       
      CHARACTER*6 clmodnam      ! model name                            
      CHARACTER*5 cloasis       ! coupler name (Oasis)                  
      INTEGER imess(4)                                                  
      INTEGER getpid            ! system functions                      
      INTEGER nuout                                                     
      PARAMETER (nuout = 6)                                             
c                                                                       
C                                                                       
C     - - - - - - - - - - - - - - - - - - - - - - - - -                 
C                                                                       
C*    =clim.h=  CLIM 1.1 include file                                   
C	        Coupling Library for Interfacing Models                       
C                                                                       
C     - - - - - - - - - - - - - - - - - - - - - - - - -                 
C                                                                       
      INTEGER*4 CLIM_Void                                               
C                                                                       
      INTEGER*4	CLIM_MaxMod,                                            
     *          CLIM_MaxPort,                                           
     *          CLIM_MaxSegments,                                       
     *          CLIM_MaxTag,                                            
     *          CLIM_MaxLink,                                           
     *          CLIM_ParSize,                                           
     *          CLIM_Clength                                            
C                                                                       
      INTEGER*4 CLIM_In,        CLIM_Out,       CLIM_InOut              
C                                                                       
      INTEGER*4 CLIM_Strategy,  CLIM_Serial,                            
     *          CLIM_Apple,     CLIM_Orange,    CLIM_Box,               
     *          CLIM_Segments,  CLIM_Length,    CLIM_Offset,            
     *          CLIM_SizeX,     CLIM_SizeY,     CLIM_LdX                
C                                                                       
      INTEGER*4 CLIM_Integer,   CLIM_Real,      CLIM_Double             
C                                                                       
      INTEGER*4 CLIM_StopPvm,   CLIM_ContPvm                            
C                                                                       
      INTEGER*4	CLIM_MaxCodes,  CLIM_Ok,                                
     *          CLIM_FastExit,  CLIM_BadName,   CLIM_BadPort,           
     *          CLIM_BadType,   CLIM_DoubleDef, CLIM_NotStep,           
     *          CLIM_IncStep,   CLIM_IncSize,   CLIM_NotClim,           
     *          CLIM_TimeOut,                                           
     *          CLIM_Pvm,       CLIM_FirstCall, CLIM_PbRoute,           
     *          CLIM_Group,     CLIM_BadTaskId, CLIM_NoTask,            
     *          CLIM_InitBuff,  CLIM_Pack,      CLIM_Unpack,            
     *          CLIM_Down,      CLIM_PvmExit                            
C                                                                       
C-----Parameter sizes                                                   
C                                                                       
      PARAMETER ( CLIM_Void    = 0  )                                   
      PARAMETER ( CLIM_MaxMod  = 8 )                                    
      PARAMETER ( CLIM_MaxPort = 40 )                                   
      PARAMETER ( CLIM_MaxSegments = 160 )                              
      PARAMETER ( CLIM_MaxLink = CLIM_MaxMod * CLIM_MaxPort )           
      PARAMETER ( CLIM_ParSize = 2 * CLIM_MaxSegments + 2 )             
      PARAMETER ( CLIM_MaxTag  = 16777215 )                             
      PARAMETER ( CLIM_Clength = 32 )                                   
C                                                                       
C-----Ports status                                                      
C                                                                       
      PARAMETER ( CLIM_In      = 1 )                                    
      PARAMETER ( CLIM_Out     = 0 )                                    
      PARAMETER ( CLIM_InOut   = 2 )                                    
C                                                                       
C-----Parallel distribution                                             
C                                                                       
      PARAMETER ( CLIM_Strategy = 1 )                                   
      PARAMETER ( CLIM_Segments = 2 )                                   
      PARAMETER ( CLIM_Serial   = 0 )                                   
      PARAMETER ( CLIM_Apple    = 1 )                                   
      PARAMETER ( CLIM_Box      = 2 )                                   
      PARAMETER ( CLIM_Orange   = 3 )                                   
      PARAMETER ( CLIM_Offset   = 2 )                                   
      PARAMETER ( CLIM_Length   = 3 )                                   
      PARAMETER ( CLIM_SizeX    = 3 )                                   
      PARAMETER ( CLIM_SizeY    = 4 )                                   
      PARAMETER ( CLIM_LdX      = 5 )                                   
C                                                                       
C-----Datatypes                                                         
C                                                                       
      PARAMETER ( CLIM_Integer = 1 )                                    
      PARAMETER ( CLIM_Real    = 4 )                                    
      PARAMETER ( CLIM_Double  = 8 )                                    
C                                                                       
C-----Quit parameters                                                   
C                                                                       
      PARAMETER ( CLIM_ContPvm = 0 )                                    
      PARAMETER ( CLIM_StopPvm = 1 )                                    
C                                                                       
C-----Error Codes                                                       
C                                                                       
      PARAMETER ( CLIM_MaxCodes  = -21 )                                
C                                                                       
      PARAMETER ( CLIM_Ok        = 0 )                                  
      PARAMETER ( CLIM_FastExit  = -1 )                                 
      PARAMETER ( CLIM_BadName   = -2 )                                 
      PARAMETER ( CLIM_BadPort   = -3 )                                 
      PARAMETER ( CLIM_BadType   = -4 )                                 
      PARAMETER ( CLIM_DoubleDef = -5 )                                 
      PARAMETER ( CLIM_NotStep   = -6 )                                 
      PARAMETER ( CLIM_IncStep   = -7 )                                 
      PARAMETER ( CLIM_IncSize   = -8 )                                 
      PARAMETER ( CLIM_NotClim   = -9 )                                 
      PARAMETER ( CLIM_TimeOut   = -10 )                                
      PARAMETER ( CLIM_Pvm       = -11 )                                
      PARAMETER ( CLIM_FirstCall = -12 )                                
      PARAMETER ( CLIM_PbRoute   = -13 )                                
      PARAMETER ( CLIM_Group     = -14 )                                
      PARAMETER ( CLIM_BadTaskId = -15 )                                
      PARAMETER ( CLIM_NoTask    = -16 )                                
      PARAMETER ( CLIM_InitBuff  = -17 )                                
      PARAMETER ( CLIM_Pack      = -18 )                                
      PARAMETER ( CLIM_Unpack    = -19 )                                
      PARAMETER ( CLIM_Down      = -20 )                                
      PARAMETER ( CLIM_PvmExit   = -21 )                                
C                                                                       
C-----End of clim.h                                                     
c                                                                       
c     INCLUDE 'oasis.h' ! contains the name of communication technique. 
c                       ! Here cchan=CLIM only is possible.             
C                                                                       
C -- oasis.h                                                            
C    ******                                                             
C@                                                                      
C@  Contents : choice for the OASIS version: clim or pipe               
C@  --------                                                            
      CHARACTER*8 cchan                                                 
      PARAMETER ( cchan='CLIM' )                                        
C                                                                       
C     --- end of oasis.h                                                
c                                                                       
C     -----------------------------------------------------------       
C                                                                       
C*    1. Initializations                                                
C        ---------------                                                
C                                                                       
      WRITE(nuout,*) ' '                                                
      WRITE(nuout,*) ' '                                                
      WRITE(nuout,*) ' ROUTINE INICMA'                                  
      WRITE(nuout,*) ' **************'                                  
      WRITE(nuout,*) ' '                                                
      WRITE(nuout,*) ' '                                                
c                                                                       
c     Define the model name                                             
c                                                                       
      clmodnam = 'toyatm'    ! as in $NBMODEL in Cpl/Nam/namcouple.tmp  
c                                                                       
c     Define the coupler name                                           
c                                                                       
      cloasis = 'Oasis'     !  always 'Oasis' as in the coupler         
c                                                                       
c                                                                       
c     Define symbolic name for fields exchanged from atmos to coupler,  
c         must be the same as (1) of the field  definition in namcouple:
c                                                                       
      cl_writ(1)='CONSFTOT'                                             
      cl_writ(2)='COSHFTOT'                                             
      cl_writ(3)='COWATFLU'                                             
      cl_writ(4)='CORUNOFF'                                             
      cl_writ(5)='COZOTAUX'                                             
      cl_writ(6)='COMETAUY'                                             
      cl_writ(7)='COZOTAUV'                                             
      cl_writ(8)='COMETAUU'                                             
c                                                                       
c     Define files name for fields exchanged from atmos to coupler,     
c         must be the same as (6) of the field  definition in namcouple:
c                                                                       
c     cl_f_writ(1)='flxatmos'                                           
c     cl_f_writ(2)='flxatmos'                                           
c     cl_f_writ(3)='flxatmos'                                           
c     cl_f_writ(4)='flxatmos'                                           
c     cl_f_writ(5)='flxatmos'                                           
c     cl_f_writ(6)='flxatmos'                                           
c     cl_f_writ(7)='flxatmos'                                           
c     cl_f_writ(8)='flxatmos'                                           
      cl_f_writ(1)='flxocean'                                           
      cl_f_writ(2)='flxocean'                                           
      cl_f_writ(3)='flxocean'                                           
      cl_f_writ(4)='flxocean'                                           
      cl_f_writ(5)='flxocean'                                           
      cl_f_writ(6)='flxocean'                                           
      cl_f_writ(7)='flxocean'                                           
      cl_f_writ(8)='flxocean'                                           
c                                                                       
c                                                                       
c Define symbolic name for fields exchanged from coupler to atmosphere, 
c must be the same as (2) of the field  definition in namcouple:        
c                                                                       
      cl_read(1)='SISUTESU'                                             
      cl_read(2)='SIZONCUR'                                             
      cl_read(3)='SIMERCUR'                                             
c                                                                       
c Define files names for fields exchanged from coupler to atmosphere,   
c must be the same as (7) of the field  definition in namcouple:        
c                                                                       
      cl_f_read(1)='sstatmos'                                           
      cl_f_read(2)='sstatmos'                                           
      cl_f_read(3)='sstatmos'                                           
c                                                                       
c     Define infos to be sent initially to oasis                        
c                                                                       
      imess(1) = kastp  ! total number of timesteps in atmospheric model
      imess(2) = kexch  ! period of exchange (in time steps)            
      imess(3) = kstep  ! length of atmospheric timestep (in seconds)   
      imess(4) = getpid()   ! PID of atmospheric model                  
c                                                                       
c     Initialization and exchange of initial info in the CLIM technique 
c                                                                       
      IF (cchan.eq.'CLIM') THEN                                         
c                                                                       
c     Define the experiment name :                                      
c                                                                       
          cljobnam = 'CLI'      ! as $JOBNAM in namcouple               
c                                                                       
c         Start the coupling                                            
c   (see lib/clim/src/CLIM_Init for the definition of input parameters) 
c                                                                       
          CALL CLIM_Init ( cljobnam, clmodnam, 3, 71,                   
     *                 kastp, kexch, kstep,                             
     *                 5, 3600, 3600, info )                            
c                                                                       
                                                                        
          IF (info.ne.CLIM_Ok) THEN                                     
              WRITE ( nuout, *) ' inicma : pb init clim '               
              WRITE ( nuout, *) ' error code is = ', info               
              CALL halte('STOP in inicma')                              
            ELSE                                                        
              WRITE(nuout,*) 'inicma : init clim ok '                   
          ENDIF                                                         
c                                                                       
c For each coupling field, association of a port to its symbolic name   
c                                                                       
c -Define the parallel decomposition associated to the port of each     
c          field; here no decomposition for all ports.                  
          iparal ( clim_strategy ) = clim_serial                        
          iparal ( clim_length   ) = imjm                               
          iparal ( clim_offset   ) = 0                                  
c                                                                       
c         -Loop on total number of coupler-to-atmosphere fields         
c (see lib/clim/src/CLIM_Define for the definition of input parameters) 
          DO jf=1, jpfldo2a                                             
            CALL CLIM_Define (cl_read(jf), clim_in , clim_double, iparal
     $          , info )                                                
          END DO                                                        
c                                                                       
c         -Loop on total number of atmosphere-to-coupler fields         
c (see lib/clim/src/CLIM_Define for the definition of input parameters) 
          DO jf=1, jpflda2o1+jpflda2o2                                  
            CALL CLIM_Define (cl_writ(jf), clim_out , clim_double,      
     $          iparal, info )                                          
          END DO                                                        
c                                                                       
          WRITE(nuout,*) 'inicma : clim_define ok '                     
c                                                                       
c -Join a pvm group, wait for other programs and broadcast usefull      
c  informations to Oasis and to the ocean (see lib/clim/src/CLIM_Start) 
          CALL CLIM_Start ( imxtag, info )                              
                                                                        
                                                                        
          IF (info.ne.clim_ok) THEN                                     
              WRITE ( nuout, *) 'inicma : pb start clim '               
              WRITE ( nuout, *) ' error code is = ', info               
              CALL halte('stop in inicma')                              
            ELSE                                                        
              WRITE ( nuout, *)  'inicma : start clim ok '              
          ENDIF                                                         
c                                                                       
c         -Get initial information from Oasis                           
c          (see lib/clim/src/CLIM_Stepi)                                
          CALL CLIM_Stepi (cloasis, istep, ifcpl, idt, info)            
          IF (info .NE. clim_ok) THEN                                   
              WRITE ( UNIT = nuout, FMT = *)                            
     $            ' warning : problem in getting step info ',           
     $            'from oasis '                                         
              WRITE (UNIT = nuout, FMT = *)                             
     $            ' =======   error code number = ', info               
            ELSE                                                        
              WRITE (UNIT = nuout, FMT = *)                             
     $            ' got step information from oasis '                   
          ENDIF                                                         
          WRITE ( nuout, *) ' number of tstep in oasis ', istep         
          WRITE ( nuout, *) ' exchange frequency in oasis ', ifcpl      
          WRITE ( nuout, *) ' length of tstep in oasis ', idt           
      ENDIF                                                             
                                                                        
      RETURN                                                            
      END                                                               
      SUBROUTINE fromcpl(kt, imjm, sst, zsc, msc)                       
c ======================================================================
c S. Valcke (02/99) adapted From L.Z.X Li:this subroutine reads the SST 
c and Sea-Ice provided by the coupler with the CLIM (PVM exchange)      
c technique.                                                            
c====================================================================== 
      IMPLICIT none                                                     
      INTEGER imjm, kt                                                  
      REAL sst(imjm)          ! sea-surface-temperature                 
      REAL zsc(imjm)          ! zonal sea surface current               
      REAL msc(imjm)          ! meridional sea surface current          
c                                                                       
      INTEGER nuout             ! listing output unit                   
      PARAMETER (nuout=6)                                               
c                                                                       
      INTEGER nuread, ios, iflag, icpliter                              
      INTEGER info, jf                                                  
c                                                                       
C                                                                       
C     - - - - - - - - - - - - - - - - - - - - - - - - -                 
C                                                                       
C*    =clim.h=  CLIM 1.1 include file                                   
C	        Coupling Library for Interfacing Models                       
C                                                                       
C     - - - - - - - - - - - - - - - - - - - - - - - - -                 
C                                                                       
      INTEGER*4 CLIM_Void                                               
C                                                                       
      INTEGER*4	CLIM_MaxMod,                                            
     *          CLIM_MaxPort,                                           
     *          CLIM_MaxSegments,                                       
     *          CLIM_MaxTag,                                            
     *          CLIM_MaxLink,                                           
     *          CLIM_ParSize,                                           
     *          CLIM_Clength                                            
C                                                                       
      INTEGER*4 CLIM_In,        CLIM_Out,       CLIM_InOut              
C                                                                       
      INTEGER*4 CLIM_Strategy,  CLIM_Serial,                            
     *          CLIM_Apple,     CLIM_Orange,    CLIM_Box,               
     *          CLIM_Segments,  CLIM_Length,    CLIM_Offset,            
     *          CLIM_SizeX,     CLIM_SizeY,     CLIM_LdX                
C                                                                       
      INTEGER*4 CLIM_Integer,   CLIM_Real,      CLIM_Double             
C                                                                       
      INTEGER*4 CLIM_StopPvm,   CLIM_ContPvm                            
C                                                                       
      INTEGER*4	CLIM_MaxCodes,  CLIM_Ok,                                
     *          CLIM_FastExit,  CLIM_BadName,   CLIM_BadPort,           
     *          CLIM_BadType,   CLIM_DoubleDef, CLIM_NotStep,           
     *          CLIM_IncStep,   CLIM_IncSize,   CLIM_NotClim,           
     *          CLIM_TimeOut,                                           
     *          CLIM_Pvm,       CLIM_FirstCall, CLIM_PbRoute,           
     *          CLIM_Group,     CLIM_BadTaskId, CLIM_NoTask,            
     *          CLIM_InitBuff,  CLIM_Pack,      CLIM_Unpack,            
     *          CLIM_Down,      CLIM_PvmExit                            
C                                                                       
C-----Parameter sizes                                                   
C                                                                       
      PARAMETER ( CLIM_Void    = 0  )                                   
      PARAMETER ( CLIM_MaxMod  = 8 )                                    
      PARAMETER ( CLIM_MaxPort = 40 )                                   
      PARAMETER ( CLIM_MaxSegments = 160 )                              
      PARAMETER ( CLIM_MaxLink = CLIM_MaxMod * CLIM_MaxPort )           
      PARAMETER ( CLIM_ParSize = 2 * CLIM_MaxSegments + 2 )             
      PARAMETER ( CLIM_MaxTag  = 16777215 )                             
      PARAMETER ( CLIM_Clength = 32 )                                   
C                                                                       
C-----Ports status                                                      
C                                                                       
      PARAMETER ( CLIM_In      = 1 )                                    
      PARAMETER ( CLIM_Out     = 0 )                                    
      PARAMETER ( CLIM_InOut   = 2 )                                    
C                                                                       
C-----Parallel distribution                                             
C                                                                       
      PARAMETER ( CLIM_Strategy = 1 )                                   
      PARAMETER ( CLIM_Segments = 2 )                                   
      PARAMETER ( CLIM_Serial   = 0 )                                   
      PARAMETER ( CLIM_Apple    = 1 )                                   
      PARAMETER ( CLIM_Box      = 2 )                                   
      PARAMETER ( CLIM_Orange   = 3 )                                   
      PARAMETER ( CLIM_Offset   = 2 )                                   
      PARAMETER ( CLIM_Length   = 3 )                                   
      PARAMETER ( CLIM_SizeX    = 3 )                                   
      PARAMETER ( CLIM_SizeY    = 4 )                                   
      PARAMETER ( CLIM_LdX      = 5 )                                   
C                                                                       
C-----Datatypes                                                         
C                                                                       
      PARAMETER ( CLIM_Integer = 1 )                                    
      PARAMETER ( CLIM_Real    = 4 )                                    
      PARAMETER ( CLIM_Double  = 8 )                                    
C                                                                       
C-----Quit parameters                                                   
C                                                                       
      PARAMETER ( CLIM_ContPvm = 0 )                                    
      PARAMETER ( CLIM_StopPvm = 1 )                                    
C                                                                       
C-----Error Codes                                                       
C                                                                       
      PARAMETER ( CLIM_MaxCodes  = -21 )                                
C                                                                       
      PARAMETER ( CLIM_Ok        = 0 )                                  
      PARAMETER ( CLIM_FastExit  = -1 )                                 
      PARAMETER ( CLIM_BadName   = -2 )                                 
      PARAMETER ( CLIM_BadPort   = -3 )                                 
      PARAMETER ( CLIM_BadType   = -4 )                                 
      PARAMETER ( CLIM_DoubleDef = -5 )                                 
      PARAMETER ( CLIM_NotStep   = -6 )                                 
      PARAMETER ( CLIM_IncStep   = -7 )                                 
      PARAMETER ( CLIM_IncSize   = -8 )                                 
      PARAMETER ( CLIM_NotClim   = -9 )                                 
      PARAMETER ( CLIM_TimeOut   = -10 )                                
      PARAMETER ( CLIM_Pvm       = -11 )                                
      PARAMETER ( CLIM_FirstCall = -12 )                                
      PARAMETER ( CLIM_PbRoute   = -13 )                                
      PARAMETER ( CLIM_Group     = -14 )                                
      PARAMETER ( CLIM_BadTaskId = -15 )                                
      PARAMETER ( CLIM_NoTask    = -16 )                                
      PARAMETER ( CLIM_InitBuff  = -17 )                                
      PARAMETER ( CLIM_Pack      = -18 )                                
      PARAMETER ( CLIM_Unpack    = -19 )                                
      PARAMETER ( CLIM_Down      = -20 )                                
      PARAMETER ( CLIM_PvmExit   = -21 )                                
C                                                                       
C-----End of clim.h                                                     
c                                                                       
C                                                                       
C -- oasis.h                                                            
C    ******                                                             
C@                                                                      
C@  Contents : choice for the OASIS version: clim or pipe               
C@  --------                                                            
      CHARACTER*8 cchan                                                 
      PARAMETER ( cchan='CLIM' )                                        
C                                                                       
C     --- end of oasis.h                                                
C                                                                       
C -- param_cou.h                                                        
C                                                                       
      INTEGER jpmaxfld                                                  
      PARAMETER(jpmaxfld = 20)                                          
      INTEGER jpflda2o1                                                 
      PARAMETER(jpflda2o1 = 4)                                          
      INTEGER jpflda2o2                                                 
      PARAMETER(jpflda2o2 = 4)                                          
C                                                                       
      INTEGER jpfldo2a                                                  
      PARAMETER(jpfldo2a = 3)                                           
C                                                                       
c                                                                       
C                                                                       
C -- inc_cpl.h                                                          
C    **********                                                         
C@                                                                      
C@  Contents : variables describing field restart file names            
C@  --------                                                            
C@                                                                      
C@ -- cl_write/cl_read  : for fields to write/READ                      
C@ -- cl_f_write/cl_f_read  : for fields to write/read                  
C@                                                                      
C     ----------------------------------------------------------------- 
C                                                                       
      CHARACTER*8 cl_writ(jpmaxfld), cl_read(jpmaxfld)                  
      CHARACTER*8 cl_f_writ(jpmaxfld), cl_f_read(jpmaxfld)              
      COMMON / comcpl / cl_writ, cl_read, cl_f_writ, cl_f_read          
C     ----------------------------------------------------------------- 
c                                                                       
c                                                                       
      WRITE (nuout,*) ' '                                               
      WRITE (nuout,*) 'Fromcpl: Reading fields from CPL, kt=',kt        
      WRITE (nuout,*) ' '                                               
c     CALL flush (nuout)                                                
      CALL fish (nuout)                                                 
                                                                        
                                                                        
      IF (cchan.eq.'CLIM') THEN                                         
                                                                        
c                                                                       
c     -Get interpolated oceanic fields from Oasis                       
c                                                                       
          DO jf=1,jpfldo2a                                              
            IF (jf.eq.1) CALL CLIM_Import (cl_read(jf) , kt, sst, info) 
            IF (jf.eq.2) CALL CLIM_Import (cl_read(jf) , kt, zsc, info) 
            IF (jf.eq.3) CALL CLIM_Import (cl_read(jf) , kt, msc, info) 
            IF ( info .NE. CLIM_Ok) THEN                                
                WRITE(nuout,*)'Pb in reading ', cl_read(jf), jf         
                WRITE(nuout,*)'Couplage kt is = ',kt                    
                WRITE(nuout,*)'CLIM error code is = ', info             
                CALL halte('STOP in fromcpl.F')                         
            ENDIF                                                       
          END DO                                                        
                                                                        
      ENDIF                                                             
c                                                                       
      RETURN                                                            
      END                                                               
                                                                        
                                                                        
      SUBROUTINE intocpl(kt, imjm, fsol, fnsol, waflx, runoff, taux,    
     $    tauy, archts, last)                                           
c ======================================================================
c S. Valcke (02/99) adapted From L.Z.X Li:this subroutine provides the  
c atmospheric coupling fields to the coupler with the CLIM (PVM         
c messages) technique.                                                  
c IF last time step, writes output fields to binary files.              
c ======================================================================
      IMPLICIT NONE                                                     
      INTEGER kt, imjm                                                  
c                                                                       
      REAL fsol(imjm)                                                   
      REAL fnsol(imjm)                                                  
      REAL waflx(imjm)                                                  
      REAL runoff(imjm)                                                 
      REAL taux(imjm)                                                   
      REAL tauy(imjm)                                                   
      LOGICAL last,archts                                               
c                                                                       
      INTEGER nuout                                                     
      PARAMETER (nuout = 6)                                             
c                                                                       
C                                                                       
C     - - - - - - - - - - - - - - - - - - - - - - - - -                 
C                                                                       
C*    =clim.h=  CLIM 1.1 include file                                   
C	        Coupling Library for Interfacing Models                       
C                                                                       
C     - - - - - - - - - - - - - - - - - - - - - - - - -                 
C                                                                       
      INTEGER*4 CLIM_Void                                               
C                                                                       
      INTEGER*4	CLIM_MaxMod,                                            
     *          CLIM_MaxPort,                                           
     *          CLIM_MaxSegments,                                       
     *          CLIM_MaxTag,                                            
     *          CLIM_MaxLink,                                           
     *          CLIM_ParSize,                                           
     *          CLIM_Clength                                            
C                                                                       
      INTEGER*4 CLIM_In,        CLIM_Out,       CLIM_InOut              
C                                                                       
      INTEGER*4 CLIM_Strategy,  CLIM_Serial,                            
     *          CLIM_Apple,     CLIM_Orange,    CLIM_Box,               
     *          CLIM_Segments,  CLIM_Length,    CLIM_Offset,            
     *          CLIM_SizeX,     CLIM_SizeY,     CLIM_LdX                
C                                                                       
      INTEGER*4 CLIM_Integer,   CLIM_Real,      CLIM_Double             
C                                                                       
      INTEGER*4 CLIM_StopPvm,   CLIM_ContPvm                            
C                                                                       
      INTEGER*4	CLIM_MaxCodes,  CLIM_Ok,                                
     *          CLIM_FastExit,  CLIM_BadName,   CLIM_BadPort,           
     *          CLIM_BadType,   CLIM_DoubleDef, CLIM_NotStep,           
     *          CLIM_IncStep,   CLIM_IncSize,   CLIM_NotClim,           
     *          CLIM_TimeOut,                                           
     *          CLIM_Pvm,       CLIM_FirstCall, CLIM_PbRoute,           
     *          CLIM_Group,     CLIM_BadTaskId, CLIM_NoTask,            
     *          CLIM_InitBuff,  CLIM_Pack,      CLIM_Unpack,            
     *          CLIM_Down,      CLIM_PvmExit                            
C                                                                       
C-----Parameter sizes                                                   
C                                                                       
      PARAMETER ( CLIM_Void    = 0  )                                   
      PARAMETER ( CLIM_MaxMod  = 8 )                                    
      PARAMETER ( CLIM_MaxPort = 40 )                                   
      PARAMETER ( CLIM_MaxSegments = 160 )                              
      PARAMETER ( CLIM_MaxLink = CLIM_MaxMod * CLIM_MaxPort )           
      PARAMETER ( CLIM_ParSize = 2 * CLIM_MaxSegments + 2 )             
      PARAMETER ( CLIM_MaxTag  = 16777215 )                             
      PARAMETER ( CLIM_Clength = 32 )                                   
C                                                                       
C-----Ports status                                                      
C                                                                       
      PARAMETER ( CLIM_In      = 1 )                                    
      PARAMETER ( CLIM_Out     = 0 )                                    
      PARAMETER ( CLIM_InOut   = 2 )                                    
C                                                                       
C-----Parallel distribution                                             
C                                                                       
      PARAMETER ( CLIM_Strategy = 1 )                                   
      PARAMETER ( CLIM_Segments = 2 )                                   
      PARAMETER ( CLIM_Serial   = 0 )                                   
      PARAMETER ( CLIM_Apple    = 1 )                                   
      PARAMETER ( CLIM_Box      = 2 )                                   
      PARAMETER ( CLIM_Orange   = 3 )                                   
      PARAMETER ( CLIM_Offset   = 2 )                                   
      PARAMETER ( CLIM_Length   = 3 )                                   
      PARAMETER ( CLIM_SizeX    = 3 )                                   
      PARAMETER ( CLIM_SizeY    = 4 )                                   
      PARAMETER ( CLIM_LdX      = 5 )                                   
C                                                                       
C-----Datatypes                                                         
C                                                                       
      PARAMETER ( CLIM_Integer = 1 )                                    
      PARAMETER ( CLIM_Real    = 4 )                                    
      PARAMETER ( CLIM_Double  = 8 )                                    
C                                                                       
C-----Quit parameters                                                   
C                                                                       
      PARAMETER ( CLIM_ContPvm = 0 )                                    
      PARAMETER ( CLIM_StopPvm = 1 )                                    
C                                                                       
C-----Error Codes                                                       
C                                                                       
      PARAMETER ( CLIM_MaxCodes  = -21 )                                
C                                                                       
      PARAMETER ( CLIM_Ok        = 0 )                                  
      PARAMETER ( CLIM_FastExit  = -1 )                                 
      PARAMETER ( CLIM_BadName   = -2 )                                 
      PARAMETER ( CLIM_BadPort   = -3 )                                 
      PARAMETER ( CLIM_BadType   = -4 )                                 
      PARAMETER ( CLIM_DoubleDef = -5 )                                 
      PARAMETER ( CLIM_NotStep   = -6 )                                 
      PARAMETER ( CLIM_IncStep   = -7 )                                 
      PARAMETER ( CLIM_IncSize   = -8 )                                 
      PARAMETER ( CLIM_NotClim   = -9 )                                 
      PARAMETER ( CLIM_TimeOut   = -10 )                                
      PARAMETER ( CLIM_Pvm       = -11 )                                
      PARAMETER ( CLIM_FirstCall = -12 )                                
      PARAMETER ( CLIM_PbRoute   = -13 )                                
      PARAMETER ( CLIM_Group     = -14 )                                
      PARAMETER ( CLIM_BadTaskId = -15 )                                
      PARAMETER ( CLIM_NoTask    = -16 )                                
      PARAMETER ( CLIM_InitBuff  = -17 )                                
      PARAMETER ( CLIM_Pack      = -18 )                                
      PARAMETER ( CLIM_Unpack    = -19 )                                
      PARAMETER ( CLIM_Down      = -20 )                                
      PARAMETER ( CLIM_PvmExit   = -21 )                                
C                                                                       
C-----End of clim.h                                                     
C                                                                       
C -- param_cou.h                                                        
C                                                                       
      INTEGER jpmaxfld                                                  
      PARAMETER(jpmaxfld = 20)                                          
      INTEGER jpflda2o1                                                 
      PARAMETER(jpflda2o1 = 4)                                          
      INTEGER jpflda2o2                                                 
      PARAMETER(jpflda2o2 = 4)                                          
C                                                                       
      INTEGER jpfldo2a                                                  
      PARAMETER(jpfldo2a = 3)                                           
C                                                                       
C                                                                       
C -- inc_cpl.h                                                          
C    **********                                                         
C@                                                                      
C@  Contents : variables describing field restart file names            
C@  --------                                                            
C@                                                                      
C@ -- cl_write/cl_read  : for fields to write/READ                      
C@ -- cl_f_write/cl_f_read  : for fields to write/read                  
C@                                                                      
C     ----------------------------------------------------------------- 
C                                                                       
      CHARACTER*8 cl_writ(jpmaxfld), cl_read(jpmaxfld)                  
      CHARACTER*8 cl_f_writ(jpmaxfld), cl_f_read(jpmaxfld)              
      COMMON / comcpl / cl_writ, cl_read, cl_f_writ, cl_f_read          
C     ----------------------------------------------------------------- 
c                                                                       
      CHARACTER*8 file_name(jpmaxfld)                                   
      INTEGER max_file                                                  
      INTEGER file_unit_max, file_unit(jpmaxfld),                       
     $    file_unit_field(jpmaxfld)                                     
                                                                        
      INTEGER icstep, info, jn, jf, ierror                              
      LOGICAL trouve                                                    
c                                                                       
C                                                                       
C -- oasis.h                                                            
C    ******                                                             
C@                                                                      
C@  Contents : choice for the OASIS version: clim or pipe               
C@  --------                                                            
      CHARACTER*8 cchan                                                 
      PARAMETER ( cchan='CLIM' )                                        
C                                                                       
C     --- end of oasis.h                                                
c                                                                       
      icstep=kt                                                         
c                                                                       
      WRITE(nuout,*) ' '                                                
      WRITE(nuout,*) 'Intocpl: sending fields to CPL, kt= ', kt         
      WRITE(nuout,*) ' '                                                
                                                                        
      IF (last.or.archts) THEN                                          
c                                                                       
c   -WRITE fields to binary files for coupler restart at last time step 
c                                                                       
c         -initialisation and files opening                             
c                                                                       
          max_file=1                                                    
          file_unit_max=99                                              
c         -keeps first file name                                        
          file_name(max_file)=cl_f_writ(max_file)                       
c         -keeps first file unit                                        
          file_unit(max_file)=file_unit_max                             
c         -decrements file unit maximum                                 
          file_unit_max=file_unit_max-1                                 
c         -keeps file unit for field                                    
          file_unit_field(1)=file_unit(max_file)                        
c                                                                       
c         -different files names counter                                
c                                                                       
          DO jf= 2, jpflda2o1 + jpflda2o2                               
            trouve=.false.                                              
            DO jn= 1, max_file                                          
              IF (.not.trouve) THEN                                     
                  IF (cl_f_writ(jf).EQ.file_name(jn)) THEN              
c                 -keep file unit for field                             
                      file_unit_field(jf)=file_unit(jn)                 
                      trouve=.true.                                     
                  END IF                                                
              END IF                                                    
            END DO                                                      
            IF (.not.trouve) then                                       
c           -increment the number of different files                    
                max_file=max_file+1                                     
c           -keep file name                                             
                file_name(max_file)=cl_f_writ(jf)                       
c           -keep file unit for file                                    
                file_unit(max_file)=file_unit_max                       
c           -keep file unit for field                                   
                file_unit_field(jf)=file_unit(max_file)                 
c           -decrement unit maximum number from 99 to 98, ...           
                file_unit_max=file_unit_max-1                           
            END IF                                                      
          END DO                                                        
c                                                                       
          DO jn=1, max_file                                             
           OPEN (file_unit(jn), FILE=file_name(jn), FORM='UNFORMATTED') 
          END DO                                                        
c                                                                       
c         WRITE fields to files                                         
          DO jf=1, jpflda2o1 + jpflda2o2                                
            IF (jf.eq.1)                                                
     $          CALL locwrite(cl_writ(jf),fnsol, imjm,                  
     $          file_unit_field(jf), ierror, nuout)                     
            IF (jf.eq.2)                                                
     $          CALL locwrite(cl_writ(jf),fsol, imjm,                   
     $          file_unit_field(jf), ierror, nuout)                     
            IF (jf.eq.3)                                                
     $          CALL locwrite(cl_writ(jf),waflx, imjm,                  
     $          file_unit_field(jf), ierror, nuout)                     
            IF (jf.eq.4)                                                
     $          CALL locwrite(cl_writ(jf),runoff, imjm,                 
     $          file_unit_field(jf), ierror, nuout)                     
            IF (jf.eq.5)                                                
     $          CALL locwrite(cl_writ(jf),taux, imjm,                   
     $          file_unit_field(jf),ierror, nuout)                      
            IF (jf.eq.6)                                                
     $          CALL locwrite(cl_writ(jf),tauy, imjm,                   
     $          file_unit_field(jf),ierror, nuout)                      
            IF (jf.eq.7)                                                
     $          CALL locwrite(cl_writ(jf),taux, imjm,                   
     $          file_unit_field(jf), ierror, nuout)                     
            IF (jf.eq.8)                                                
     $          CALL locwrite(cl_writ(jf),tauy, imjm,                   
     $          file_unit_field(jf), ierror, nuout)                     
          END DO                                                        
C                                                                       
C         -simulate a FLUSH                                             
C                                                                       
          DO jn=1, max_file                                             
            CLOSE (file_unit(jn))                                       
          END DO                                                        
                                                                        
          end if                                                        
C                                                                       
C                                                                       
          if (last) then                                                
                                                                        
          IF(cchan.eq.'CLIM') THEN                                      
C                                                                       
C         -inform PVM daemon that message exchange is finished          
C                                                                       
              write(6,*)'calling CLIM_Quit',CLIM_ContPvm, info          
              CALL CLIM_Quit (CLIM_ContPvm, info)                       
              IF (info .NE. CLIM_Ok) THEN                               
                  WRITE (6, *)                                          
     $                'An error occured while leaving CLIM. Error = ',  
     $                info                                              
              ENDIF                                                     
          END IF                                                        
          RETURN                                                        
      else                                                              
      write(6,*) " never got to clim quit "                             
      END IF                                                            
C                                                                       
      IF(cchan.eq.'CLIM') THEN                                          
C                                                                       
C     -Give atmospheric fields to Oasis                                 
C                                                                       
          DO jn=1, jpflda2o1 + jpflda2o2                                
C                                                                       
            IF (jn.eq.1) CALL CLIM_Export(cl_writ(jn), kt, fnsol, info) 
            IF (jn.eq.2) CALL CLIM_Export(cl_writ(jn), kt, fsol, info)  
            IF (jn.eq.3) CALL CLIM_Export(cl_writ(jn), kt, waflx, info) 
            IF (jn.eq.4) CALL CLIM_Export(cl_writ(jn), kt, runoff, info)
            IF (jn.eq.5) CALL CLIM_Export(cl_writ(jn), kt, taux, info)  
            IF (jn.eq.6) CALL CLIM_Export(cl_writ(jn), kt, tauy, info)  
            IF (jn.eq.7) CALL CLIM_Export(cl_writ(jn), kt, taux, info)  
            IF (jn.eq.8) CALL CLIM_Export(cl_writ(jn), kt, tauy, info)  
                                                                        
            IF (info .NE. CLIM_Ok) THEN                                 
                WRITE (nuout,*) 'STEP : Pb giving ',cl_writ(jn), ':',jn 
                WRITE (nuout,*) ' at timestep = ', icstep,'kt = ',kt    
                WRITE (nuout,*) 'Clim error code is = ',info            
                CALL halte('STOP in intocpl ')                          
            ENDIF                                                       
          END DO                                                        
      ENDIF                                                             
C                                                                       
      RETURN                                                            
      END                                                               
      SUBROUTINE locread ( cdfldn, pfield, kdimax, knulre, kflgre       
     $                   , kout)                                        
C****                                                                   
C               *****************************                           
C               * OASIS ROUTINE  -  LEVEL 0 *                           
C               * -------------     ------- *                           
C               *****************************                           
C                                                                       
C**** *locread*  - Read binary field on unit knulre                     
C                                                                       
C     Purpose:                                                          
C     -------                                                           
C     Find string cdfldn on unit knulre and read array pfield           
C                                                                       
C**   Interface:                                                        
C     ---------                                                         
C       *CALL*  *locread (cdfldn, pfield, kdimax, knulre, kflgre, kout)*
C                                                                       
C     Input:                                                            
C     -----                                                             
C                cdfldn : character string locator                      
C                kdimax : dimension of field to be read                 
C                knulre : logical unit to be read                       
C                kout   : logical unit to write messages                
C                                                                       
C     Output:                                                           
C     ------                                                            
C                pfield : field array (real 1D)                         
C                kflgre : error status flag                             
C                                                                       
C     Reference:                                                        
C     ---------                                                         
C     See OASIS manual (1995)                                           
C                                                                       
C     History:                                                          
C     -------                                                           
C       Version   Programmer     Date      Description                  
C       -------   ----------     ----      -----------                  
C       2.0       L. Terray      95/09/01  created                      
C                                                                       
C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
C                                                                       
      IMPLICIT CHARACTER(c)                                             
      IMPLICIT LOGICAL(l)                                               
C                                                                       
C*--------------------------- Argument declarations ------------------- 
C                                                                       
      REAL pfield(kdimax)                                               
      CHARACTER*8 cdfldn                                                
C                                                                       
C*--------------------------- Local declarations ---------------------- 
C                                                                       
      CHARACTER*8 clecfl                                                
C                                                                       
C*--------------------------- Poema verses ---------------------------- 
C                                                                       
C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
C                                                                       
C*    1. Initialization                                                 
C        --------------                                                 
C                                                                       
      WRITE (UNIT = kout,FMT = 1001) knulre                             
C                                                                       
C* Formats                                                              
C                                                                       
 1001 FORMAT('Locread : Read binary file connected to unit = ',I3)      
C                                                                       
C     2. Find field in file                                             
C        ------------------                                             
C                                                                       
      REWIND knulre                                                     
 200  CONTINUE                                                          
C* Find string                                                          
      READ (UNIT = knulre, ERR = 200, END = 210) clecfl                 
      IF (clecfl .NE. cdfldn) GO TO  200                                
C* Read associated field                                                
      READ (UNIT = knulre, ERR = 210, END = 210) pfield                 
C* Reading done and ok                                                  
      kflgre = 0                                                        
      GO TO 220                                                         
C* Problem in reading                                                   
 210  kflgre = 1                                                        
 220  CONTINUE                                                          
C                                                                       
C                                                                       
C*    3. End of routine                                                 
C        --------------                                                 
C                                                                       
      WRITE (UNIT = kout,FMT = *) 'Locread : done'                      
      CALL FISH (kout)                                                  
      RETURN                                                            
      END                                                               
      SUBROUTINE locwrite (cdfldn, pfield, kdimax, knulre, kflgre, kout)
      IMPLICIT none                                                     
C****                                                                   
C               *****************************                           
C               * OASIS ROUTINE  -  LEVEL 0 *                           
C               * -------------     ------- *                           
C               *****************************                           
C                                                                       
C**** *locwrite*  - Write binary field on unit knulre                   
C                                                                       
C     Purpose:                                                          
C     -------                                                           
C     Write string cdfldn and array pfield on unit knulre               
C                                                                       
C**   Interface:                                                        
C     ---------                                                         
C      *CALL*  *locwrite (cdfldn, pfield, kdimax, knulre, kflgre, kout)*
C                                                                       
C     Input:                                                            
C     -----                                                             
C                cdfldn : character string locator                      
C                kdimax : dimension of field to be written              
C                knulre : logical unit to be written                    
C                pfield : field array (real 1D)                         
C                kout   : logical unit to write messages                
C                                                                       
C     Output:                                                           
C     ------                                                            
C                kflgre : error status flag                             
C                                                                       
C     Workspace:                                                        
C     ---------                                                         
C     None                                                              
C                                                                       
C     Externals:                                                        
C     ---------                                                         
C     None                                                              
C                                                                       
C     Reference:                                                        
C     ---------                                                         
C     See OASIS manual (1995)                                           
C                                                                       
C     History:                                                          
C     -------                                                           
C       Version   Programmer     Date      Description                  
C       -------   ----------     ----      -----------                  
C       2.0       L. Terray      95/09/01  created                      
C                                                                       
C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
C                                                                       
C                                                                       
C*--------------------------- Argument declarations ------------------- 
C                                                                       
      INTEGER kdimax, knulre, kflgre, kout                              
      REAL pfield(kdimax)                                               
      CHARACTER*8 cdfldn                                                
C                                                                       
C*--------------------------- Poema verses ---------------------------- 
C                                                                       
C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
C                                                                       
C*    1. Initialization                                                 
C        --------------                                                 
C                                                                       
      WRITE (UNIT = kout,FMT = 1001) knulre                             
C                                                                       
C* Formats                                                              
C                                                                       
 1001 FORMAT(5X,' Write binary file connected to unit = ',I3)           
C                                                                       
C     2. Find field in file                                             
C        ------------------                                             
C                                                                       
C* Write string                                                         
      WRITE (UNIT = knulre, ERR = 210) cdfldn                           
C* Write associated field                                               
      WRITE (UNIT = knulre, ERR = 210) pfield                           
C* Writing done and ok                                                  
      kflgre = 0                                                        
      GO TO 220                                                         
C* Problem in Writing                                                   
 210  kflgre = 1                                                        
 220  CONTINUE                                                          
C                                                                       
C                                                                       
C*    3. End of routine                                                 
C        --------------                                                 
C                                                                       
      WRITE (UNIT = kout,FMT = *) 'Locwrite : done'                     
      CALL FISH (kout)                                                  
      RETURN                                                            
      END                                                               
                                                                        
                                                                        
                                                                        
                                                                        
      SUBROUTINE halte (cdtext)                                         
C****                                                                   
C               *****************************                           
C               * OASIS ROUTINE  -  LEVEL C *                           
C               * -------------     ------- *                           
C               *****************************                           
C                                                                       
C**** *halte*  - Abort the program                                      
C                                                                       
C     Purpose:                                                          
C     -------                                                           
C     Print an error message to standard output and abort the coupler   
C                                                                       
C**   Interface:                                                        
C     ---------                                                         
C       *CALL*  *halte (cdtext)*                                        
C                                                                       
C     Input:                                                            
C     -----                                                             
C                cdtext   : character string to be printed              
C                                                                       
C     Output:                                                           
C     ------                                                            
C     None                                                              
C                                                                       
C     Workspace:                                                        
C     ---------                                                         
C     None                                                              
C                                                                       
C     Externals:                                                        
C     ---------                                                         
C     None                                                              
C                                                                       
C     Reference:                                                        
C     ---------                                                         
C     See OASIS 2.2 manual (1997)                                       
C                                                                       
C     History:                                                          
C     -------                                                           
C       Version   Programmer     Date      Description                  
C       -------   ----------     ----      -----------                  
C       2.2       S. Valcke      97/11/18  created                      
C                                                                       
C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
C                                                                       
C*----------------------- Argument declarations ----------------------  
C                                                                       
      CHARACTER*(*) cdtext                                              
C                                                                       
C*--------------------------- Local declarations ---------------------- 
C                                                                       
      CHARACTER cpbase                                                  
      CHARACTER*10 cprpt, cpdots                                        
      CHARACTER*69 cline                                                
      PARAMETER ( cpbase = '-' )                                        
      PARAMETER ( cprpt = '* ===>>> :' )                                
      PARAMETER ( cpdots = '  ------  ' )                               
C                                                                       
C*--------------------------- Poema verses ---------------------------- 
C                                                                       
C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
C                                                                       
C*    1. Print text                                                     
C        ----------                                                     
C                                                                       
          cline = ' '                                                   
          ilen = len(cdtext)                                            
          DO 110 jl = 1, ilen                                           
            cline(jl:jl) = cpbase                                       
 110      CONTINUE                                                      
          nulou=6                                                       
          WRITE(UNIT = nulou,FMT='(/,A,X,A)') cpdots, cline             
          WRITE(UNIT = nulou,FMT='(/,A,X,A,/)') cprpt, cdtext           
          WRITE(UNIT = nulou,FMT='(A,X,A,/)')cpdots, cline              
C                                                                       
C                                                                       
C*    2. FLUSH the coupler output                                       
C        ------------------------                                       
C                                                                       
      CALL FISH (nulou)                                                 
C                                                                       
C                                                                       
C*    3. Abort the coupler                                              
C        -----------------                                              
C                                                                       
      CALL abort                                                        
      RETURN                                                            
      END                                                               
